import { defineConfig, devices } from '@playwright/test'

/**
 * We don't have APPLICATION_HOST_SERVER available to us in the drone step
 * so we need to infer the environment from the git branch
 */
function determineBaseURL() {
  const { DEPLOY_BRANCH = '' } = process.env
  const IS_STAGING = /^(patch\/)|^main$/g.test(DEPLOY_BRANCH)

  const branchRegex = /^.+\//g
  // Main and patch branches always deploy to staging
  // For pull requests use the git branch
  if (IS_STAGING && process.env.CI) {
    process.env.IS_STAGING = `${IS_STAGING}`
    return `https://www.stg.nytimes.com`
  }

  if (DEPLOY_BRANCH) {
    return `https://games-${DEPLOY_BRANCH.replace(
      branchRegex,
      '',
    )}.dev.nytimes.com`
  }

  return 'https://local.nytimes.com'
}

/**
 * See https://playwright.dev/docs/test-configuration.
 */
export default defineConfig({
  testDir: './e2e',
  /* Run tests in files in parallel */
  fullyParallel: true,
  /* Fail the build on CI if you accidentally left test.only in the source code. */
  forbidOnly: !!process.env.CI,
  /* Retry on CI only */
  retries: process.env.CI ? 5 : 0,
  /* Reporter to use. See https://playwright.dev/docs/test-reporters */
  reporter: 'list',
  /* Shared settings for all the projects below. See https://playwright.dev/docs/api/class-testoptions. */
  use: {
    /* Base URL to use in actions like `await page.goto('/')`. */
    baseURL: determineBaseURL(),

    /* Collect trace when retrying the failed test. See https://playwright.dev/docs/trace-viewer */
    trace: 'on-first-retry',
  },

  /* Configure projects for major browsers */
  projects: [
    {
      name: 'chromium',
      grepInvert: /mobile/,
      use: { ...devices['Desktop Chrome'] },
    },

    {
      name: 'webkit',
      grepInvert: /mobile|caching/,
      use: { ...devices['Desktop Safari'] },
    },

    {
      name: 'Mobile Safari',
      grepInvert: /desktop|caching/,
      use: { ...devices['iPhone 12'] },
    },
  ],
})
