const path = require('path')
const url = require('url')
const cp = require('child_process')
const os = require('os')
const process = require('process')

const HttpAgent = require('agentkeepalive/lib/agent')
const HttpsAgent = require('agentkeepalive/lib/https_agent')
const axios = require('axios').default
const moment = require('moment')
const defaultsDeep = require('lodash/defaultsDeep')

const featureFlags = require('./featureFlags')

exports.COMMIT_HASH = cp
  .execSync('git rev-parse HEAD')
  .toString()
  .trim()

/** @see {@link https://nodejs.org/docs/latest-v14.x/api/http.html#http_new_agent_options|Node.js Docs} */
const agentOptions = {
  keepAlive: true,
  maxSockets: 1024,
}

const serviceName =
  process.env.DD_SERVICE || process.env.K_SERVICE || 'phoenix'

const serviceVersion =
  process.env.DD_VERSION ||
  process.env.K_REVISION ||
  process.env.VERSION ||
  'latest'

const serviceVersionShort = `${
  process.env.COMMIT || exports.COMMIT_HASH
}`.slice(0, 8)

const { versions } = process
const { node, v8, openssl } = versions
const userAgent = `${serviceName}/${serviceVersionShort} (${os.arch()}-${os.platform()}) node/${node} (v8/${v8}; OpenSSL/${openssl})`

/** @see {@link https://axios-http.com/docs/req_config|Axios Request Config} */
const axiosConfig = {
  headers: {
    common: {
      connection: 'keep-alive',
      from: 'games-dev@nytimes.com',
      'keep-alive': 'timeout=60',
      'user-agent': userAgent,
    },
  },
  httpAgent: new HttpAgent(agentOptions),
  httpsAgent: new HttpsAgent(agentOptions),
}

/** @see {@link https://axios-http.com/docs/config_defaults|Axios Config Defaults} */
axios.defaults = defaultsDeep({}, axiosConfig, axios.defaults)

/**
 * NOTE: RUNTIME_ENV is set by drone via the 'env' environment variable
 * Its values can be either 'dev', 'stg', or 'prd'.
 */
exports.RUNTIME_ENV = process.env.ENVIRONMENT || 'dev'
exports.IS_PROD = exports.RUNTIME_ENV === 'prd'

// see https://getpino.io/#/docs/api?id=level-string for valid values
exports.LOG_LEVEL = process.env.LOG_LEVEL || 'info'

/**
 * NOTE: IS_DRONE_BUILD is set to true in the drone.yml. This informs our app
 * whether or not to use minified assets, source maps, and debug outputs
 */
exports.IS_DRONE_BUILD = !!process.env.IS_DRONE_BUILD

if (
  process.env.env &&
  !['stg', 'prd', 'dev'].includes(exports.RUNTIME_ENV)
) {
  throw new Error(
    `misconfigured environment '${exports.RUNTIME_ENV}'`,
  )
}

const envify = (values) => values[exports.RUNTIME_ENV]
const DEFAULT_PORT = 3000

exports.APPLICATION_HOST_SERVER =
  process.env.APPLICATION_HOST_SERVER || 'https://www.nytimes.com'
exports.APPLICATION_HOST_CLIENT =
  process.env.APPLICATION_HOST_CLIENT ||
  `${exports.APPLICATION_HOST_SERVER}`.replace(/^http:/, 'https:')

exports.USE_OAUTH =
  !!process.env.USE_OAUTH || exports.RUNTIME_ENV === 'stg'
exports.USE_DIST = !!process.env.USE_DIST
exports.USE_STATIC_API = !!process.env.STATIC_API
exports.USE_LIVE_RELOAD = !!process.env.USE_LIVE_RELOAD
exports.DISABLE_ADS = !!process.env.DISABLE_ADS

// Port number the application will run on. On dev, we allow customizing the port.
exports.PORT =
  exports.RUNTIME_ENV === 'dev'
    ? parseInt(process.env.PORT, 10) || DEFAULT_PORT
    : DEFAULT_PORT
exports.LIVE_RELOAD_PORT = 35729

// For faking users, we use a fancy `bash -c` in package.json
// however, when there is no user specified, we get the string `bash` :(
exports.FAKE_USER =
  process.env.FAKE_USER && process.env.FAKE_USER !== 'bash'
    ? process.env.FAKE_USER
    : undefined

exports.REPO_ROOT = path.resolve(__dirname, '..')
exports.DIST_ROOT = path.resolve(exports.REPO_ROOT, 'dist')
exports.SRC_ROOT = path.resolve(exports.REPO_ROOT, 'src')
exports.PUBLIC_ROOT = path.resolve(exports.SRC_ROOT, 'public')
exports.TOOLS_ROOT = path.resolve(exports.REPO_ROOT, 'tools')
exports.VENDOR_ROOT = path.resolve(exports.REPO_ROOT, 'vendor')
exports.STORYBOOK_ASSETS = path.resolve(
  exports.PUBLIC_ROOT,
  'storybook',
)
exports.STATIC_API_ROOT = path.resolve(
  exports.REPO_ROOT,
  'static-api',
)
exports.NODE_MODULES = path.resolve(exports.REPO_ROOT, 'node_modules')

exports.MANIFEST_PATH = path.resolve(
  exports.DIST_ROOT,
  'manifest.json',
)
exports.SERVER_VIEWS_PATH = path.resolve(
  exports.DIST_ROOT,
  'serverViews.js',
)
exports.PACKAGE_VERSION = process.env.VERSION // drone build in stg and tag in prd

exports.FEATURE_FLAGS = envify(featureFlags)

exports.USE_HTTPS = !!process.env.HTTPS
exports.SSL_KEY_FILE_PATH = process.env.SSL_KEY_FILE_PATH || ''
exports.SSL_CERT_FILE_PATH = process.env.SSL_CERT_FILE_PATH || ''

exports.PUZZLES_ROUTE_BASE = '/puzzles'
exports.CROSSWORDS_ROUTE_BASE = '/crosswords'
exports.GAMES_ROUTE_BASE = '/games'
exports.CDN_URL = '/games-assets/v2'
// Only override assets paths if USE_DIST is true. Default to CDN_URL
exports.ROUTE_DIST = exports.USE_DIST
  ? `${exports.PUZZLES_ROUTE_BASE}/dist`
  : exports.CDN_URL
exports.ROUTE_ASSETS = exports.USE_DIST
  ? `${exports.PUZZLES_ROUTE_BASE}/public`
  : exports.CDN_URL

exports.ROUTE_FIXTURES = `${exports.PUZZLES_ROUTE_BASE}/fixtures`
exports.GOOGLE_STORAGE_API = `https://storage.googleapis.com/games-assets`
exports.ROUTE_STORYBOOK = `${exports.PUZZLES_ROUTE_BASE}/storybook`
exports.WORDLE_ASSET_BASE = `${exports.APPLICATION_HOST_CLIENT}${exports.ROUTE_ASSETS}/assets/wordle`
exports.CONNECTIONS_ASSET_BASE = `${exports.APPLICATION_HOST_CLIENT}${exports.ROUTE_ASSETS}/assets/connections`

exports.ACCOUNT_URL = envify({
  prd: 'https://myaccount.nytimes.com',
  stg: 'https://myaccount.stg.nytimes.com',
  dev: 'https://myaccount.stg.nytimes.com',
})

exports.SAMIZDAT_HOST = envify({
  prd: 'https://samizdat-graphql.nytimes.com',
  stg: 'https://samizdat-graphql-test.stg.nytimes.com',
  dev: 'https://samizdat-graphql-test.stg.nytimes.com',
})

exports.SAMIZDAT_APP_TYPE = 'games-phoenix'
exports.SAMIZDAT_APP_VERSION = '1.0.0'

exports.SAMIZDAT_TOKEN = envify({
  prd: process.env.SAMIZDAT_TOKEN_PRD,
  stg: process.env.SAMIZDAT_TOKEN_STG,
  dev: process.env.SAMIZDAT_TOKEN_STG,
})

exports.SHORTZ_MODE_URL = envify({
  prd: 'https://people.games.prd.nyt.net',
  stg: 'https://people.games.stg.nyt.net',
  dev: 'https://people.games.dev.nyt.net',
})

const SUBSCRIBE_URL = 'https://www.nytimes.com/subscription/games'

exports.SUBSCRIBE_URL = SUBSCRIBE_URL
exports.NATIVE_SCHEME_NEWSREADER = 'nytimes://'
exports.NATIVE_SCHEME_GAMES = 'nytxwd://'
exports.NATIVE_SUBSCRIBE_PATH = 'gamesProducts'
exports.NATIVE_SUBSCRIBE_PATH_ANDROID = 'reader/gamesProducts/'
exports.NATIVE_LOGIN_PATH = 'login'
exports.NATIVE_REGI_PATH = 'createAccount'

exports.SUBSCRIPTION_CAMPAIGNS = {
  daily: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=4QHQJ`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=4QHQ6`,
  },
  archive: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=4QHQR`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=4QHQF`,
  },
  gameplay: {
    // unused
    halfPrice: `${SUBSCRIBE_URL}?campaignId=4R67Y`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=4R67W`,
  },
  congrats: {
    // unused
    halfPrice: `${SUBSCRIBE_URL}?campaignId=4XKYF`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=4XKYJ`,
  },
  spellingBeeCard: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7QXX9`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7QXX9`,
  },
  spellingBeePaywall: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7RJKF`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7RJKF`,
  },
  spellingBeeBanner: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7HHK8`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7HHK8`,
  },
  spellingBeeCutoffStart: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7F9J7`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7F9J7`,
  },
  spellingBeeCutoffStop: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=79XQU`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=79XQU`,
  },
  spellingBeeCutoffWelcomeBack: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7KK8Q`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7KK8Q`,
  },
  spellingBeeHub: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=8XUQX`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=8XUQX`,
  },
  spellingBeeWelcomePastPuzzles: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=8LL7Q`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=8LL7Q`,
  },
  vertexCongratsMomentMidStarterPack: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=989YK`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=989YK`,
  },
  vertexCongratsMomentEndOfStarterPack: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=9K9UL`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=9K9UL`,
  },
  vertexWelcomeMomentDailyPuzzle: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=9RXHL`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=9RXHL`,
  },
  vertexPreviewPageNonsubs: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=9J9RU`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=9J9RU`,
  },
  vertexTodaysPuzzle: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=9KK7K`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=9KK7K`,
  },
  vertexDefault: {
    // unused
    halfPrice: `${SUBSCRIBE_URL}?campaignId=9HURJ`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=9HURJ`,
  },
  letterBoxedBanner: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7UHY9`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7UHY9`,
  },
  letterBoxedCard: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=78UQ9`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=78UQ9`,
  },
  letterBoxedPaywall: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7JXU8`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7JXU8`,
  },
  tilesWelcomeDesktop: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7U798`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7U798`,
  },
  tilesWelcomeMobile: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7Y97X`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7Y97X`,
  },
  tilesCongratsDesktop: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7Y8WW`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7Y8WW`,
  },
  tilesCongratsMobile: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7RYYF`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7RYYF`,
  },
  tilesSettingsDesktop: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7XH8W`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7XH8W`,
  },
  tilesSettingsMobile: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=7JR89`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=7JR89`,
  },
  wordle: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=8YU98`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=8YU98`,
  },
  connections: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=8YQFQ`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=8YQFQ`,
  },
  branding: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=9W9LL`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=9W9LL`,
  },
  default: {
    halfPrice: `${SUBSCRIBE_URL}?campaignId=4QHQL`,
    fullPrice: `${SUBSCRIBE_URL}?campaignId=4QHQ8`,
  },
}

const XWD_SUBMISSION_URL =
  'https://nytimes.com/article/submit-crossword-puzzles-the-new-york-times.html'
exports.XWD_SUBMISSION_URL = XWD_SUBMISSION_URL

const KENKEN_AND_SET_REDIRECT_URL =
  'https://help.nytimes.com/hc/en-us/articles/360029050872-Word-Games-and-Logic-Puzzles#SETANDKENKEN'

const ACROSTIC_AND_VARIETY_REDIRECT_URL =
  'https://help.nytimes.com/hc/en-us/articles/360052406391#Acrosticandvariety'

exports.ACROSTIC_AND_VARIETY_REDIRECT_URL =
  ACROSTIC_AND_VARIETY_REDIRECT_URL

exports.GLOBAL_REDIRECTS = {
  '/': '/crosswords',
  '/puzzles': '/crosswords',
  '/puzzles/': '/crosswords',
  '/crosswords/': '/crosswords',
  '/puzzles/kenken': KENKEN_AND_SET_REDIRECT_URL,
  '/puzzles/set': KENKEN_AND_SET_REDIRECT_URL,
  '/crosswords/game/set': KENKEN_AND_SET_REDIRECT_URL,
  '/crosswords/game/set/': KENKEN_AND_SET_REDIRECT_URL,
  '/crosswords/game/kenken': KENKEN_AND_SET_REDIRECT_URL,
  '/crosswords/game/kenken/': KENKEN_AND_SET_REDIRECT_URL,
  '/crosswords/game/acrostic/': ACROSTIC_AND_VARIETY_REDIRECT_URL,
  '/crosswords/archive/variety': ACROSTIC_AND_VARIETY_REDIRECT_URL,
  '/crosswords/archive/acrostic': ACROSTIC_AND_VARIETY_REDIRECT_URL,
  '/puzzles/acrostic': ACROSTIC_AND_VARIETY_REDIRECT_URL,
  '/crosswords/index.html': '/crosswords',
  '/crosswords/stats': '/puzzles/stats',
  '/crosswords/stats/': '/puzzles/stats',
  '/crosswords/primer':
    '/guides/crosswords/how-to-solve-a-crossword-puzzle',
  '/crosswords/hiddenpuzzle2018':
    '/puzzles/contests/hidden-puzzle-2018', // not widely used internally
  '/crosswords/submissions': XWD_SUBMISSION_URL,
  '/puzzles/submissions/crossword': XWD_SUBMISSION_URL,
  '/crosswords/game/sudoku': '/puzzles/sudoku',
  '/crosswords/game/sudoku/': '/puzzles/sudoku',
  '/crosswords/archive/index.html': '/crosswords/archive', // no longer works
}

exports.GLOBAL_STYLES = [
  { src: 'responsive.css', inline: true },
  'foundation.css',
]

exports.GLOBAL_SCRIPTS = [
  'polyfills.js',
  { src: 'responsive.js', inline: true },
  { src: 'native-bridge.js', inline: true },
  'foundation.js',
]

const GTM_CONTAINER = 'GTM-P528B3'
const GTM_BASE = `https://www.googletagmanager.com/gtm.js?id=${GTM_CONTAINER}`

exports.GTM_URL = `${GTM_BASE}${envify({
  prd: '&gtm_auth=tfAzqo1rYDLgYhmTnSjPqw&gtm_preview=env-130&gtm_cookies_win=x',
  stg: '&gtm_auth=FOuAsPhG-4kWeLk6Kq5AzQ&gtm_preview=env-7&gtm_cookies_win=x',
  dev: '&gtm_auth=WiJyA7zv1sohHCWSZ3mF1Q&gtm_preview=env-8&gtm_cookies_win=x',
})}`

exports.TAGX_HOST = envify({
  prd: 'https://a.nytimes.com',
  stg: 'https://a.dev.nytimes.com',
  dev: 'https://a.dev.nytimes.com',
})

const stagingRegex = /^(patch\/)|^main$/g
const scheme = process.env.HTTPS ? 'https' : 'http'

/**
 * For making API calls that go through the edge server
 */
exports.EDGE_HOST = envify({
  prd: 'https://www.nytimes.com',
  stg: 'https://www.stg.nytimes.com',
  dev: 'https://www.dev.nytimes.com',
})

/**
 * For making API calls to moogle services
 */
exports.MOOGLE_HOST = envify({
  prd: 'https://www.nytimes.com',
  stg: 'https://www.stg.nytimes.com',
  dev: 'https://www.stg.nytimes.com',
})

exports.API_HOST_SERVER =
  process.env.API_HOST_SERVER || 'https://edge.games.nyti.nyt.net'
// since egress traffic to API_HOST_SERVER will be managed via Istio on the server, the following line ensures
// that connections made from the client / browser will be made over https
exports.API_HOST_CLIENT = exports.EDGE_HOST

exports.ABRA_CONFIG_ORIGIN =
  process.env.ABRA_CONFIG_ORIGIN || 'https://a1.nyt.com'
exports.SARTRE_SUBINFO_HOST =
  process.env.SARTRE_SUBINFO_HOST ||
  'https://subinfo-app-prd-lb.spg-prd.awsma.nyt.net'

exports.MOCK_API_HOST = envify({
  prd: '',
  stg: `https://www.nytimes.com/games-assets/fixtures`,
  dev: url.resolve(
    exports.APPLICATION_HOST_CLIENT,
    exports.ROUTE_FIXTURES,
  ),
})

exports.METER_API = envify({
  prd: 'https://meter-svc.nytimes.com',
  stg: 'https://meter-svc.stg.nytimes.com',
  dev: 'https://meter-svc.stg.nytimes.com',
})

exports.UNIVERSAL_ENVIRONMENT = {
  version: exports.PACKAGE_VERSION,
  api: exports.API_HOST_CLIENT,
  tagx: exports.TAGX_HOST,
  gtm: exports.GTM_URL,
  hostname: exports.APPLICATION_HOST_CLIENT,
  edge: exports.EDGE_HOST,
  moogle: exports.MOOGLE_HOST,
  meter: exports.METER_API,
  account: exports.ACCOUNT_URL,
  samizdat: {
    host: exports.SAMIZDAT_HOST,
    appType: exports.SAMIZDAT_APP_TYPE,
    appVersion: exports.SAMIZDAT_APP_VERSION,
    token: exports.SAMIZDAT_TOKEN,
  },
  name: envify({
    prd: 'prod', // for some sad reason, GTM seems to want 'prod'
    stg: 'stg',
    dev: 'dev',
  }),
}

// These values get inlined into the built javascript
// NOTE: these must also be defined in .eslint
exports.JS_STATICS = {
  __DEV__: JSON.stringify(!exports.IS_DRONE_BUILD),
  __PROD__: JSON.stringify(exports.IS_DRONE_BUILD),
  __DISABLE_ADS__: JSON.stringify(exports.DISABLE_ADS),
}

exports.ZENDESK_API = 'https://nytimes.zendesk.com/api/v2'

exports.ANON_ENTITLEMENTS_API =
  process.env.ANON_ENTITLEMENTS_API ||
  'http://edge.games.svc.cluster.local/svc/crosswords/v6/anon/entitlements.json'

exports.MAGNOLIA_URL = envify({
  prd: 'https://mwcm.nyt.com',
  stg: 'https://mwcm.stg.nyt.com',
  dev: 'https://mwcm.stg.nyt.com',
})

exports.MAGNOLIA_DAM_ASSET_PATH = `${exports.MAGNOLIA_URL}/dam/mkt_assets/img/games/`

exports.PUZZLE_DATES = {
  daily: {
    hasMonths: true,
    earliestDate: moment('11-1993', 'MM-YYYY'),
  },
  mini: {
    hasMonths: true,
    earliestDate: moment('08-2014', 'MM-YYYY'),
  },
  bonus: {
    hasMonths: false,
    earliestDate: moment('1997', 'YYYY'),
  },
}

exports.SENTRY_DSN = process.env.SENTRY_DSN
exports.SENTRY_SAMPLE_RATE = process.env.SENTRY_SAMPLE_RATE
exports.CLOUD_TRACE_ENABLED = Boolean(
  parseInt(process.env.CLOUD_TRACE_ENABLED || '0', 10),
)
exports.CLOUD_ERROR_REPORTING_ENABLED = Boolean(
  parseInt(process.env.CLOUD_ERROR_REPORTING_ENABLED || '0', 10),
)
// CLOUD_SERVICE_CONTEXT is used by GCP's APM clients to set
// the client's `serviceContext` configuration values
// see https://github.com/googleapis/nodejs-error-reporting#configuration for an example
exports.CLOUD_SERVICE_CONTEXT = {
  service: serviceName,
  version: serviceVersion,
}
exports.WORDLE_ENABLED = Boolean(
  parseInt(process.env.WORDLE_ENABLED || '0', 10),
)
