import { UserEntitlement } from 'foundation'

import {
  canCheckForAdTest,
  isInVariant,
  isMobileWeb,
} from '../validation'
import * as isPhone from '../isPhone'

describe('validation', () => {
  // Mocked abra methods
  const getVariant = jest.fn(() => '1_Variant')
  const getVariantControl = jest.fn(() => '0_Control')
  const getVariantNull = jest.fn(() => null)
  const reportExposure = jest.fn()

  describe('isInVariant', () => {
    it('should return false if not in "1_Variant" group of the A/B test', () => {
      expect(
        isInVariant({
          getVariant: getVariantControl,
          reportExposure,
        }),
      ).toEqual(false)
      expect(
        isInVariant({
          getVariant: getVariantNull,
          reportExposure,
        }),
      ).toEqual(false)
    })

    it('should return true if user is in the "1_Variant" of the A/B test', () => {
      expect(
        isInVariant({
          getVariant,
          reportExposure,
        }),
      ).toEqual(true)
    })
  })
  describe('isMobileWeb', () => {
    it('should return false if on the Games app platform', () => {
      window.gamesAppPlatform = 'ios'
      expect(isMobileWeb()).toEqual(false)

      window.gamesAppPlatform = 'android'
      expect(isMobileWeb()).toEqual(false)
    })

    it('should return false if on the newsreader app platform', () => {
      window.newsreaderAppPlatform = 'ios'
      expect(isMobileWeb()).toEqual(false)

      window.newsreaderAppPlatform = 'android'
      expect(isMobileWeb()).toEqual(false)
    })

    it('should return false if not on a mobile device', () => {
      window.isMobileDevice = undefined
      expect(isMobileWeb()).toEqual(false)

      window.isMobileDevice = false
      expect(isMobileWeb()).toEqual(false)
    })

    it('should return false if not on a phone', () => {
      jest.spyOn(isPhone, 'default').mockReturnValue(false)

      expect(isMobileWeb()).toEqual(false)
    })

    it('should return true if in a mobile web browser', () => {
      jest.spyOn(isPhone, 'default').mockReturnValue(true)
      window.isMobileDevice = true
      window.gamesAppPlatform = ''
      window.newsreaderAppPlatform = ''

      expect(isMobileWeb()).toEqual(true)
    })
  })

  describe('canCheckForAdTest', () => {
    it('should return false if user is a games sub', () => {
      const crosswordEntitlement: UserEntitlement = 'cr'
      const crosswordSubEntitlement: UserEntitlement = 'sub,cr'

      expect(
        canCheckForAdTest({
          entitlement: crosswordEntitlement,
          isGameFinished: false,
          abraVariantChecks: { getVariant, reportExposure },
        }),
      ).toEqual(false)
      expect(
        canCheckForAdTest({
          entitlement: crosswordSubEntitlement,
          isGameFinished: false,
          abraVariantChecks: { getVariant, reportExposure },
        }),
      ).toEqual(false)
    })

    it('should return false if the game is finished', () => {
      const entitlement: UserEntitlement = 'anon'
      const isGameFinished = true
      expect(
        canCheckForAdTest({
          entitlement,
          isGameFinished,
          abraVariantChecks: { getVariant, reportExposure },
        }),
      ).toEqual(false)
    })

    it('should return false if the user is not in the A/B test variant', () => {
      const entitlement: UserEntitlement = 'anon'
      const isGameFinished = false
      expect(
        canCheckForAdTest({
          entitlement,
          isGameFinished,
          abraVariantChecks: {
            getVariant: getVariantControl,
            reportExposure,
          },
        }),
      ).toEqual(false)
    })

    it('should return true if if the user is not a games sub, game is not finished, and in the A/B test variant', () => {
      const entitlement: UserEntitlement = 'anon'
      const isGameFinished = false
      expect(
        canCheckForAdTest({
          entitlement,
          isGameFinished,
          abraVariantChecks: { getVariant, reportExposure },
        }),
      ).toEqual(true)
    })
  })
})
