import React, { useEffect, useState } from 'react'

import { UserType, trackClick } from 'foundation'
import useAbra from 'shared/abra/useAbra'
import { useUser } from 'user'

import { getLoginLink } from '../../helpers/loginLinks'
import { getLoginOfferLink } from '../../helpers/loginOfferLinks'
import { getSubscribeLink } from '../../helpers/subscribeLinks'
import WelcomeShared from '../../shared/moments/components/Welcome'
import {
  getNthDayOfGame,
  todaysDateAsString,
} from '../components/sharedHelpers'

import { canCheckForAdTest } from './AdMoment/validation'

type WelcomeProps = {
  today: Date
  editor: string
  isGameFinished: boolean
  transitionFromWelcomeToAd: () => void
  transitionFromWelcomeToGame: () => void
}

const createButtonGroups = ({
  userType,
  hasAd,
  today,
  showOfferLoginLink,
  handleLoginOfferExpose,
  transitionFromWelcomeToAd,
  transitionFromWelcomeToGame,
}: {
  userType: UserType
  hasAd: boolean
  showOfferLoginLink: boolean
  handleLoginOfferExpose: Function
  today: Date
  transitionFromWelcomeToAd: () => void
  transitionFromWelcomeToGame: () => void
}) => {
  const { isLoggedIn, hasXwd } = userType

  const dayOfGame = getNthDayOfGame(today)

  const playButton = {
    text: 'Play',
    action: () => {
      hasAd
        ? transitionFromWelcomeToAd()
        : transitionFromWelcomeToGame()

      trackClick({
        name: 'connections',
        label: 'start-game',
        context: dayOfGame,
        moduleName: ['b: 4-8-4'],
      })
    },
  }

  const loginButton = {
    text: 'Log In',
    color: 'secondary',
    action: () => {
      trackClick({
        name: 'connections',
        label: 'log-in',
        context: dayOfGame,
        region: 'welcome',
      })
      handleLoginOfferExpose()
      const loginLink = showOfferLoginLink
        ? getLoginOfferLink(window || undefined)
        : getLoginLink(window || undefined)
      window.location.assign(loginLink)
      return null
    },
  }

  const subscribeButton = {
    text: 'Subscribe',
    color: 'secondary',
    action: () => {
      trackClick({
        name: 'connections',
        label: 'subscribe',
        context: dayOfGame,
        region: 'welcome',
      })
      const subscribeLink = getSubscribeLink('connections')
      window.location.assign(subscribeLink)
      return null
    },
  }

  // Subs only see play button
  if (hasXwd) {
    return [{ buttons: [playButton] }]
  }

  // Regis see play and subscribe buttons
  if (isLoggedIn) {
    return [
      { buttons: [playButton] },
      {
        header: 'Want to access all of our games?',
        desktopButtonOrder: 'vertical',
        buttons: [subscribeButton],
      },
    ]
  }

  // Anons see play, login and subscribe buttons
  return [
    { buttons: [playButton] },
    {
      header: 'Want to access all of our games?',
      desktopButtonOrder: 'vertical',
      buttons: [loginButton, subscribeButton],
    },
  ]
}

const Welcome = ({
  today,
  editor,
  isGameFinished,
  transitionFromWelcomeToAd,
  transitionFromWelcomeToGame,
}: WelcomeProps) => {
  const { getVariant, reportExposure } = useAbra()
  const { userType, hasLoaded } = useUser()
  const hasAd = canCheckForAdTest({
    isGameFinished,
    entitlement: userType.entitlement,
    abraVariantChecks: { getVariant, reportExposure },
  })

  const loginOfferVariant = getVariant(
    'MKT_SubCon_Connections_PLOPRO_12_23',
  )

  const handleLoginOfferExpose = () => {
    if (loginOfferVariant) {
      reportExposure('MKT_SubCon_Connections_PLOPRO_12_23')
    }
  }

  const [momentProps, updateMomentProps] = useState({
    bgColor: '$connections-periwinkle',
    icon: 'connections',
    title: 'Connections',
    buttonGroups: [{}],
    description: {
      text: 'Group words that share a common thread.',
    },
    infoText: (
      <span className="pz-moment__info-editor">By {editor}</span>
    ),
    date: todaysDateAsString(today),
  })

  useEffect(() => {
    const showOfferLoginLink =
      !!loginOfferVariant && loginOfferVariant !== '0_Control'

    if (hasLoaded) {
      updateMomentProps({
        ...momentProps,
        buttonGroups: createButtonGroups({
          userType,
          hasAd,
          showOfferLoginLink,
          handleLoginOfferExpose,
          today,
          transitionFromWelcomeToAd,
          transitionFromWelcomeToGame,
        }),
      })
    }
  }, [hasLoaded, loginOfferVariant])

  return <WelcomeShared game={momentProps} />
}

export default Welcome
