import React, { useEffect, useState } from 'react'
import { Flipper } from 'react-flip-toolkit'
import { useDispatch, useSelector } from 'react-redux'

import { trackClick } from 'foundation'
import { useUser } from 'user'

import ModalManager from '../../../shared/ModalSystem/ModalManager'
import { ANIMATION_MS_PER_GUESS, MAX_MISTAKES } from '../../constants'
import { Level } from '../../levelTypes'
import {
  saveCategory,
  saveGuess,
  saveResetPuzzle,
} from '../../localstorage/localstorage'
import {
  closeModal,
  closeToast,
  openModal,
  openToast,
  removeModal,
  removeToast,
} from '../../store/actions'
import {
  getIsModalClosing,
  getIsToastClosing,
  getOpenModalName,
  getOpenToastName,
} from '../../store/selectors'
import { ConnectionsModal, ConnectionsToast } from '../../store/types'
import {
  AnimationEnum,
  Board,
  Card,
  CardsAnimation,
  Category,
  CongratsTitleByMistakesMade,
  Guess,
  MistakeCount,
} from '../../types'
import ActionButton from '../ActionButton/ActionButton'
import { ButtonSize } from '../ActionButton/types'
import Actions from '../Actions/Actions'
import Cards from '../Cards/Cards'
import { getLevel } from '../Game/helpers'
import Mistakes from '../Mistakes/Mistakes'
import Congrats from '../Modals/Congrats'
import HowToPlay from '../Modals/HowToPlay'
import Settings from '../Modals/Settings'
import ShortzControls from '../ShortzControls/ShortzControls'
import SolvedCategory from '../SolvedCategory/SolvedCategory'
import ToastManager from '../Toast/ToastManager'
import { toastMap } from '../Toast/gameToasts'
import Toolbar from '../Toolbar/Toolbar'
import { cardSort, getNthDayOfGame } from '../sharedHelpers'

import styles from './Board.module.scss'
import * as helpers from './helpers'

const modalMap = {
  [ConnectionsModal.SETTINGS]: Settings,
  [ConnectionsModal.CONGRATS]: Congrats,
  [ConnectionsModal.HOW_TO_PLAY]: HowToPlay,
}

function cardCountOf(cards: number, cardsPerCategory: number) {
  const count = cards / cardsPerCategory
  switch (count) {
    case 3:
      return styles.cardsContainer3
    case 2:
      return styles.cardsContainer2
    case 1:
      return styles.cardsContainer1
    default:
      return styles.cardsContainer4
  }
}

export default ({
  categories,
  guesses,
  today,
  isReady,
  playedBefore,
}: {
  categories: Category[]
  guesses: Guess[]
  today: Date
  isReady: boolean
  playedBefore: boolean
}) => {
  const user = useUser()
  const dispatch = useDispatch()
  const openToastName = useSelector(getOpenToastName)
  const sortedCards = categories
    .filter((c) => !c.solved)
    .reduce<Card[]>((a, b) => a.concat([...b.cards]), [])
    .sort(cardSort)

  const [board, updateBoard] = useState<Board>({
    cards: sortedCards,
    categories,
    guesses,
  })
  const [disabled, disableBoard] = useState<boolean>(false)
  const [animation, setAnimation] = useState<CardsAnimation>(false)
  const [updatedCategories, setUpdatedCategories] =
    useState<Category[]>(categories)

  const [categoryCompleted, setCategoryCompleted] =
    useState<boolean>(false)

  const [autoSolving, setAutoSolving] = useState<{
    inProgress: boolean
    categoryToSolve: number
  }>({ inProgress: false, categoryToSolve: 0 })

  const [recentGuessIncorrect, setRecentGuessIncorrect] =
    useState<boolean>(false)

  const shortzControlsOn =
    !(user.userType.isLoggedIn && !user.shortzHasLoaded) &&
    user.userType.inShortzMode
  const cardsPerCategory = categories[0].cards.length
  const solvedCategories = board.categories.filter((c) => c.solved)
  const incorrectGuessCount = board.guesses.filter(
    (g) => !g.correct,
  ).length

  const puzzleSolved =
    solvedCategories.length === board.categories.length

  const isGameOver =
    incorrectGuessCount >= MAX_MISTAKES || puzzleSolved

  // TODO once localstorage work is complete ensure ranks are correct when game is started
  const congratsRank =
    CongratsTitleByMistakesMade[incorrectGuessCount as MistakeCount]

  const solvedCategoryCount = updatedCategories.filter(
    (c) => c.solved,
  ).length

  useEffect(() => {
    if (isReady && !playedBefore) {
      dispatch(openModal(ConnectionsModal.HOW_TO_PLAY))
    }
  }, [isReady])

  const correctGuessForLevel = (
    solvedLevel: Level,
    selected: Card[],
    guess: Guess,
    autoSolved: boolean,
  ) => {
    const reorderedCards = helpers.reorderCards(
      selected,
      board,
      solvedLevel,
      cardsPerCategory,
    )

    const guesses = autoSolved
      ? board.guesses
      : board.guesses.concat([{ ...guess, correct: true }])

    const updatedCategories = helpers.solveCategory(
      board,
      solvedLevel,
    )

    // set updated categories to state (to be updated after the animation completes)
    setUpdatedCategories(updatedCategories)
    setCategoryCompleted(true)

    const isGameNowSolved = solvedCategoryCount === 3

    if (isGameNowSolved) {
      // show toast and modal when all categories are solved
      if (!autoSolved) {
        dispatch(openToast(ConnectionsToast.REVEAL_MISTAKE_TITLE))
      }
      setTimeout(() => {
        dispatch(openModal(ConnectionsModal.CONGRATS))
      }, ANIMATION_MS_PER_GUESS)
    }

    const cards = isGameNowSolved ? [] : reorderedCards

    saveCategory({
      category: board.categories.find((c) => c.level === solvedLevel),
      cards,
      solvedCategoryCount,
      autoSolved,
    })

    // reorder cards and update guesses
    updateBoard({
      ...board,
      cards,
      guesses,
    })
  }

  useEffect(() => {
    if (autoSolving.inProgress) {
      setTimeout(() => {
        const levelToSolve = getLevel(autoSolving.categoryToSolve)
        const fauxSelectedCards = board.cards.filter(
          (card) => card.level === levelToSolve,
        )

        const fauxGuess: Guess = helpers.createGuess({
          correct: true,
          cards: fauxSelectedCards,
        })

        correctGuessForLevel(
          getLevel(autoSolving.categoryToSolve),
          fauxSelectedCards,
          fauxGuess,
          true,
        )
      }, ANIMATION_MS_PER_GUESS)
    }
  }, [autoSolving])

  const incorrectGuess = (selected: Card[], guess: Guess) => {
    setRecentGuessIncorrect(true)
    setAnimation(AnimationEnum.SHAKE)

    const incompleteBoardWithMistake: Board = {
      ...board,
      guesses: board.guesses.concat([guess]),
    }

    setTimeout(() => {
      setAnimation(false)

      if (helpers.guessIsOneAway(selected, cardsPerCategory)) {
        dispatch(openToast(ConnectionsToast.ONE_AWAY))
      }

      saveGuess(selected)

      if (incorrectGuessCount >= MAX_MISTAKES - 1) {
        // If the game is over, deselect all cards and save the last guess
        // before dispatching the end-game toast and solving the rest of the board
        updateBoard({
          ...board,
          cards: helpers.deselectAllCards(board.cards),
          guesses: board.guesses.concat([guess]),
        })
        dispatch(openToast(ConnectionsToast.REVEAL_MISTAKE_TITLE))
        setAutoSolving({
          inProgress: true,
          categoryToSolve: solvedCategories.length,
        })
      } else {
        updateBoard(incompleteBoardWithMistake)
      }

      disableBoard(false)
    }, ANIMATION_MS_PER_GUESS)
  }

  const handleSubmit = () => {
    disableBoard(true)

    const selectedCards = board.cards.filter((c) => c.selected)
    const guess = helpers.createGuess({
      correct: false,
      cards: selectedCards,
    })

    if (helpers.alreadyGuessed(board, guess)) {
      dispatch(openToast(ConnectionsToast.ALREADY_GUESSED))

      disableBoard(false)
      return
    }

    setAnimation(AnimationEnum.BOUNCE)

    const selectedLevels = [
      ...new Set(selectedCards.map((c) => c.level)),
    ]
    const correctGuess = selectedLevels.length === 1

    // Key for label array:
    // r = result: pass/fail,
    // w = words: all four words with commas between
    // d = difficulty: 1-4 (4 is hardest) or null on a failed attempt
    // m = mistakes: # of mistakes made so far
    const guessTrackingLabel = [
      `r: ${correctGuess ? 'pass' : 'fail'}`,
      `w: ${selectedCards.map((card) => card.content).join(',')}`,
      `d: ${correctGuess ? selectedLevels[0] : null}`, // levels are 0-indexed
      `m: ${
        correctGuess ? incorrectGuessCount : incorrectGuessCount + 1
      }`,
    ]

    // Key for label array:
    // o = order of difficulty: order in which the users submitted correct difficulty guesses
    // m = mistakes: # of mistakes made
    const gameCompleteTrackingLabel = [
      `o: ${board.categories
        .filter((cat) => !!cat.solved)
        .sort((a, b) => {
          if (a.solved && b.solved) {
            return a.solved.order - b.solved.order
          }

          return 0
        })
        .map((cat) => cat.level)
        .join(',')}`,
      `m: ${
        correctGuess ? incorrectGuessCount : incorrectGuessCount + 1
      }`,
    ]

    trackClick({
      name: 'connections',
      label: 'solve-attempt',
      context: getNthDayOfGame(today),
      moduleName: ['b: 4-8-4'],
      moduleLabel: guessTrackingLabel,
    })

    const lastWrongGuess = incorrectGuessCount === 3 && !correctGuess
    const lastRightGuess = solvedCategoryCount === 3 && correctGuess
    const isGameAboutToBeOver = lastWrongGuess || lastRightGuess

    if (isGameAboutToBeOver) {
      trackClick({
        name: 'connections',
        label: '100%-complete',
        context: getNthDayOfGame(today),
        moduleName: ['b: 4-8-4'],
        moduleLabel: gameCompleteTrackingLabel,
      })
    }

    setTimeout(() => {
      correctGuess
        ? correctGuessForLevel(
            selectedLevels[0],
            selectedCards,
            guess,
            false,
          )
        : incorrectGuess(selectedCards, guess)
    }, ANIMATION_MS_PER_GUESS)
  }

  const flipKey = board.cards.reduce(
    (key, card) => (!key ? card.content : `${key}-${card.content}`),
    '',
  )

  return (
    <>
      {openToastName && (
        <ToastManager
          portalId="portal-toast-system"
          toastMap={toastMap}
          getOpenToastName={getOpenToastName}
          getIsClosing={getIsToastClosing}
          onClose={closeToast}
          onRemove={removeToast}
          rankTitle={congratsRank}
        />
      )}
      <Toolbar
        hide={!isReady}
        onButtonClick={(modalName: ConnectionsModal) => {
          dispatch(openModal(modalName))
        }}
        today={today}
      />
      <ModalManager
        modalMap={modalMap}
        portalId="portal-modal-system"
        getOpenModalName={getOpenModalName}
        getIsClosing={getIsModalClosing}
        onClose={closeModal}
        onRemove={removeModal}
        today={today}
        guesses={helpers.convertToSquares(
          board.guesses,
          board.categories,
        )}
        numMistakes={incorrectGuessCount}
        cardsPerCategory={cardsPerCategory}
      />

      <article className={styles.board}>
        <form id="default-choices" className={styles.form}>
          <h2>Create four groups of four!</h2>
          <fieldset className={styles.boardContainer}>
            {!puzzleSolved && (
              <legend className="visually-hidden">
                Select four items that have something in common
              </legend>
            )}
            {solvedCategories.length > 0 && (
              <article aria-live="polite">
                <p className="visually-hidden">
                  {solvedCategories.length} Found
                  {`Categor${
                    solvedCategories.length === 1 ? 'y' : 'ies'
                  }`}
                </p>
                <ol className={styles.solvedCategoriesContainer}>
                  {solvedCategories
                    .sort(helpers.categorySort)
                    .map((c) => (
                      <SolvedCategory
                        category={c}
                        key={`solved-${c.level}`}
                      />
                    ))}
                </ol>
              </article>
            )}
            <Flipper
              flipKey={flipKey}
              decisionData={{ animation }}
              onComplete={() => {
                if (categoryCompleted) {
                  const solvedCategories = updatedCategories.filter(
                    (c) => c.solved,
                  )

                  const solvedCategoryCount =
                    solvedCategories.length - 1

                  const remainingCards = helpers.removeTopRow(
                    board.cards,
                    cardsPerCategory,
                  )
                  setCategoryCompleted(false)
                  updateBoard({
                    ...board,
                    cards: remainingCards,
                    categories: updatedCategories,
                  })

                  if (autoSolving.categoryToSolve === 3) {
                    setAutoSolving({
                      inProgress: false,
                      categoryToSolve: 0,
                    })
                  } else if (autoSolving.inProgress) {
                    setAutoSolving({
                      inProgress: true,
                      categoryToSolve:
                        autoSolving.categoryToSolve + 1,
                    })
                  }
                  disableBoard(false)
                }
              }}
            >
              <div
                className={`${styles.cardsContainer} ${cardCountOf(
                  board.cards.length,
                  cardsPerCategory,
                )}`}
              >
                <Cards
                  board={board}
                  disabled={disabled}
                  updateBoard={(arg) => {
                    if (recentGuessIncorrect) {
                      setRecentGuessIncorrect(false)
                    }

                    return updateBoard(arg)
                  }}
                  animation={animation}
                  cardsPerCategory={cardsPerCategory}
                />
              </div>
            </Flipper>
          </fieldset>
        </form>
        <section>
          <Mistakes
            mistakes={incorrectGuessCount}
            isMistakeIndicatorDisabled={isGameOver}
            puzzleSolved={puzzleSolved}
          />
        </section>
        <section className={styles.boardActionGroup}>
          {puzzleSolved ? (
            <ActionButton
              form="default-choices"
              size={ButtonSize.SMALL}
              onClick={() => {
                dispatch(openModal(ConnectionsModal.CONGRATS))
              }}
            >
              View Results
            </ActionButton>
          ) : (
            <Actions
              board={board}
              updateBoard={updateBoard}
              onSubmit={handleSubmit}
              isRecentIncorrect={recentGuessIncorrect}
              isDisabled={disabled || autoSolving.inProgress}
              cardsPerCategory={cardsPerCategory}
              setAnimation={setAnimation}
              today={today}
            />
          )}
        </section>
        {shortzControlsOn && (
          <ShortzControls
            disabled={disabled || autoSolving.inProgress}
            puzzleSolved={puzzleSolved}
            resetBoard={() => {
              const resetBoard = helpers.resetBoard(board)

              updateBoard(resetBoard)
              saveResetPuzzle(resetBoard.categories)
              disableBoard(false)
            }}
            autoSolve={() => {
              updateBoard({
                ...board,
                cards: helpers.deselectAllCards(board.cards),
              })
              setAutoSolving({
                inProgress: true,
                categoryToSolve: solvedCategories.length,
              })
            }}
          />
        )}
      </article>
    </>
  )
}
