import React from 'react'
import { render, screen, within } from '@testing-library/react'
import userEvent from '@testing-library/user-event'

import {
  getOpenToastName,
  getOpenModalName,
} from '../../../store/selectors'
import Board from '../Board'
import { completedPuzzle } from '../../../__tests__/fixtures/game-data/valid-completed-puzzle'
import { unplayedPuzzle } from '../../../__tests__/fixtures/game-data/unplayed-puzzle'

jest.mock('../../../store/selectors', () => ({
  getOpenModalName: jest.fn((state) => {
    return ''
  }),
  getOpenToastName: jest.fn((state) => null),
}))

const mockDispatch = jest.fn()

jest.mock('react-redux', () => ({
  ...jest.requireActual('react-redux'),
  useSelector: jest.fn((func) => {
    // call through to selector function passed through
    if (func) {
      return func()
    }

    return func
  }),
  useDispatch: () => mockDispatch,
}))

jest.mock('user', () => ({
  useUser: jest.fn(() => ({
    userType: { isLoggedIn: true, hasXwd: false },
  })),
}))

describe('<Board />', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })
  afterEach(() => {
    jest.clearAllMocks()
  })

  const renderBoard = (props: any) => {
    const store = {}

    const board = render(<Board {...props} />)

    return { store, board }
  }

  describe('should render basic board content as expected', () => {
    it('should display header', () => {
      renderBoard(completedPuzzle)

      const header = screen.getByText('Create four groups of four!')
      expect(header).toBeTruthy()
    })

    it('should render toolbar', () => {
      renderBoard({ ...completedPuzzle, isReady: true })

      const toolbarNav = screen.getByRole('navigation')
      const toolbarButton =
        within(toolbarNav).getByLabelText('Settings')

      expect(toolbarNav).toBeTruthy()
      expect(toolbarButton).toBeTruthy()
    })

    it('should render example toast as expected', () => {
      // @ts-ignore
      getOpenToastName.mockImplementation((state) => 'oneAway')
      renderBoard({ ...completedPuzzle, isReady: true })

      const toastText = screen.getByText('One away...')
      expect(toastText).toBeTruthy()

      // quick reset of open toast to not rely on transition waiting
      // @ts-ignore
      getOpenToastName.mockImplementation((state) => null)
    })

    it('should render example modal when expected', () => {
      // @ts-ignore
      getOpenModalName.mockImplementation((state) => 'congrats')
      renderBoard({
        ...completedPuzzle,
        today: new Date(),
        isReady: true,
      })

      const resultsButton = screen.getByText('View Results')
      const rankText = screen.getByText('Perfect!')

      expect(rankText).toBeTruthy()
      expect(resultsButton).toBeTruthy()
    })
  })

  describe('perfect player has never played connections before', () => {
    it('player sees how to play modal and then is able to solve game', async () => {
      // renderBoard({ ...unplayedPuzzle, isReady: true })
      // TODO GAMEPLAY REDUX, figure out how to best handle full gameplay
      // AC - they see how to play modal, they dismiss it,
      //  AC - they perfectly solve the game
      // - below: example of user able to select one category
      // const CARROT = screen.getByLabelText('CARROT')
      // const BASKETBALL = screen.getByLabelText('BASKETBALL')
      // const GOLDFISH = screen.getByLabelText('GOLDFISH')
      // const PUMPKIN = screen.getByLabelText('PUMPKIN')
      // await userEvent.click(CARROT)
      // await userEvent.click(BASKETBALL)
      // await userEvent.click(GOLDFISH)
      // await userEvent.click(PUMPKIN)
      // sanity check one is clicked
      // expect(CARROT).toBeChecked()
      // const submitButton = screen.getByText('Submit')
      // TODO fiture out how to handle animation delay before congrats
      // await userEvent.click(submitButton)
      //  AC -they see congrats modal
    })
  })

  describe('player has made one mistake already and returns to game', () => {
    it('sees already played toast', () => {
      // TODO GAMEPLAY REDUX
      // AC - they make same mistake again
      // AC - see toast about how it's already been guessed
    })
  })

  describe('mistake-only player has never played connections before ', () => {
    it('player plays through to next time modal', () => {
      // TODO GAMEPLAY REDUX
      // AC - they see how to play modal, they dismiss it
      // AC - they guess only mistakes
      // AC - they see each mistake removed from ui
      // AC - they get Next Time modal
    })
  })

  describe('player has solved 3 categories already and returns to game', () => {
    it('player sees congrats on last try', () => {
      // TODO GAMEPLAY REDUX, figure out how to best handle full gameplay
      // AC - they solve 4th and final category
      // AC - see congrats modal
    })
  })

  describe('player has already solved todays puzzle', () => {
    it('should not see how to play', () => {
      // TODO GAMEPLAY REDUX,
      // AC -  should not see how to play
      // AC - should only see the congrats modal which is dismissable
    })
  })

  describe('player has solved 2 categories already', () => {
    it('player plays through and see congrats modal', () => {
      // TODO GAMEPLAY REDUX
      // AC - return to game, guess a wrong guess
      // AC - guess a right guess
      // AC - test that local storage reflects that there are 3 solved categories and one less mistake
    })
  })
})
