import { Level } from '../../levelTypes'
import { Board, Card, Category, Guess } from '../../types'

export const categorySort = (a: Category, b: Category): number => {
  if (a.solved && b.solved) {
    return a.solved.order - b.solved.order
  }

  return -1
}

export const convertToSquares = (
  guesses: Guess[],
  categories: Category[],
): Level[][] => {
  const mapIdToLevel = new Map<string, Level>()
  categories.forEach((category) => {
    category.cards.forEach((card) => {
      mapIdToLevel.set(card.originalPosition.toString(), card.level)
    })
  })

  return guesses.map((g) =>
    g.cards
      .split(',')
      .map((ogPosition) => mapIdToLevel.get(ogPosition) as Level),
  )
}

export const reorderCards = (
  selected: Card[],
  board: Board,
  solvedLevel: Level,
  cardsPerCategory: number,
): Card[] => {
  const cardsToReplace = selected.filter(
    (c) => c.currentPosition >= cardsPerCategory,
  )

  const lowerRowCards = board.cards.reduce<Card[]>((lowerRows, c) => {
    if (c.level === solvedLevel) return lowerRows

    const isInTopRow = c.currentPosition < cardsPerCategory

    if (!isInTopRow) {
      lowerRows.push({ ...c })
    } else {
      lowerRows.push({
        ...c,
        currentPosition: (cardsToReplace.pop() as Card)
          .currentPosition,
      })
    }

    return lowerRows
  }, [])

  return selected
    .map((c, i) => ({ ...c, selected: true, currentPosition: i }))
    .concat(lowerRowCards)
}

export const removeTopRow = (
  cs: Card[],
  cardsPerCategory: number,
): Card[] => {
  return cs.reduce<Card[]>((cards, c) => {
    const newPosition = c.currentPosition - cardsPerCategory
    if (newPosition >= 0) {
      cards.push({
        ...c,
        currentPosition: c.currentPosition - cardsPerCategory,
      })
    }

    return cards
  }, [])
}

export const guessIsOneAway = (
  selected: Card[],
  cardsPerCategory: number,
): boolean => {
  const count: { [key in Level]?: number } = {}
  selected.forEach(({ level }) => {
    const currentCount = count[level]
    currentCount !== undefined
      ? (count[level] = currentCount + 1)
      : (count[level] = 1)
  })
  return Object.values(count).includes(cardsPerCategory - 1)
}

export function solveCategory(
  board: Board,
  solvedLevel: Level,
): Category[] {
  return board.categories.map((c) => {
    return c.level === solvedLevel
      ? {
          ...c,
          solved: {
            order:
              board.categories.filter((c) => c.solved).length + 1,
          },
        }
      : { ...c }
  })
}

export const createGuess = ({
  correct,
  cards,
}: {
  correct: boolean
  cards: Card[]
}): Guess => ({
  correct,
  cards: cards
    .map((c) => c.originalPosition)
    .sort((a, b) => a - b)
    .join(','),
})

export const alreadyGuessed = (
  board: Board,
  guess: { cards: string; correct: boolean },
): boolean => board.guesses.map((g) => g.cards).includes(guess.cards)

export const resetBoard = (board: Board): Board => {
  const newCategories: Category[] = board.categories.map((c) => ({
    ...c,
    solved: false,
  }))

  const sortedCards = newCategories
    .flatMap((cat) => cat.cards)
    .sort((a, b) => a.originalPosition - b.originalPosition)

  return {
    categories: newCategories,
    guesses: [],
    cards: sortedCards.map((c) => ({
      ...c,
      currentPosition: c.originalPosition,
    })),
  }
}

export const deselectAllCards = (cards: Card[]): Card[] =>
  cards.map((card) => ({ ...card, selected: false }))
