import React, { useEffect, useState } from 'react'

import { isMobile } from 'responsive'

import { Card, CardAnimation } from '../../types'
import { useTheme } from '../Theme'

import styles from './Card.module.scss'
import * as helpers from './helpers'

interface CardProps extends Card {
  disabled: boolean
  updateSelected: (s: boolean) => void
  showColor?: boolean
  animation?: CardAnimation
}

function getWindowDimensions() {
  const { innerWidth: width } = window
  return { width }
}

function useWindowDimensions() {
  const [windowDimensions, setWindowDimensions] = useState(
    getWindowDimensions(),
  )

  useEffect(() => {
    function handleResize() {
      setWindowDimensions(getWindowDimensions())
    }

    window.addEventListener('resize', handleResize)
    return () => window.removeEventListener('resize', handleResize)
  }, [])

  return windowDimensions
}

export default ({
  content,
  originalPosition,
  disabled,
  selected,
  level,
  updateSelected,
  showColor = false,
  animation,
  currentPosition,
  ...flipProps
}: CardProps) => {
  const theme = useTheme()
  const { width } = useWindowDimensions()
  const [shrunk, shrink] = useState<boolean>(false)

  const fontSize =
    width < 640
      ? helpers.calculateFontSize(content)
      : styles.labelXXLg

  let colors = theme.GAME.INITIAL
  if (showColor) colors = theme.GAME.LEVELS[level]

  if (selected || shrunk) colors = theme.GAME.SELECTED

  const cardBackgroundAndColor = helpers.animationShakeStyle(
    animation,
    theme,
    colors,
  )

  return (
    <label
      htmlFor={`inner-card-${originalPosition}`}
      className={`${styles.label} ${fontSize}${
        shrunk ? ` ${styles.shrinkCard}` : ''
      }${helpers.animationClass(animation)}${
        disabled ? ` ${styles.disabled}` : ''
      }`}
      style={{
        ...cardBackgroundAndColor,
        // selected cards should float over the top when animated
        zIndex: selected ? '1' : '0',
      }}
      key={`inner-card-${originalPosition}`}
      onPointerDown={() => {
        !disabled && shrink(true)
      }}
      onPointerUp={() => {
        !disabled && shrink(false)
      }}
      onPointerCancel={() => {
        shrink(false)
      }}
      onPointerOut={() => {
        shrink(false)
      }}
      {...flipProps}
    >
      <input
        className="visually-hidden"
        type="checkbox"
        id={`inner-card-${originalPosition}`}
        name={content}
        value={content}
        defaultChecked={selected}
        disabled={disabled}
        onClick={() => updateSelected(!selected)}
      />
      {content}
    </label>
  )
}
