import React from 'react'
import { format } from 'date-fns'

import { navigateBack } from '../../../helpers/navigateBack'
import { ErrorName } from '../../api/types'
import { useTheme } from '../Theme'
import { LAUNCH_DATE } from '../../constants'

import styles from './Error.module.scss'

const ErrorIcon = ({ errorType }: { errorType?: ErrorName }) => {
  const offlineIcon =
    errorType === 'offline' ? styles.offlineErrorIcon : false

  return (
    <span
      role="img"
      aria-hidden
      className={offlineIcon || styles.defaultErrorIcon}
    />
  )
}

const OfflineMessage = () => (
  <div className={styles.errorContent}>
    <h1 className={styles.errorTitle}>Oh no!</h1>
    <h2 className={styles.errorSubtitle}>You&rsquo;re offline.</h2>
    <p className={styles.errorText}>
      Find a connection and come back.
    </p>
  </div>
)

const DefaultMessage = () => (
  <div className={styles.errorContent}>
    <h1 className={styles.errorTitle}>Oops!</h1>
    <h2 className={styles.errorSubtitle}>Something went wrong.</h2>
    <p className={styles.errorText}>Please try again later.</p>
  </div>
)

const MissingData = () => (
  <div className={styles.errorContent}>
    <h1 className={styles.errorTitle}>Oops!</h1>
    <h2 className={styles.errorSubtitle}>
      {`We're missing data for this puzzle.`}
    </h2>
    <p className={styles.errorText}>
      Please test after the data has been added.
    </p>
  </div>
)

const PrecedesLaunch = () => (
  <div className={styles.errorContent}>
    <h1 className={styles.errorTitle}>Oops!</h1>
    <h2 className={styles.errorSubtitle}>
      Date precedes the launch date of{' '}
      {format(LAUNCH_DATE, 'MMM d, yyyy')}.
    </h2>
    <p className={styles.errorText}>Please test a later puzzle.</p>
  </div>
)

interface ErrorProps {
  errorType: ErrorName
  showBackButton?: boolean
}

export default function ({
  errorType,
  showBackButton = false,
}: ErrorProps) {
  // Only show the "Back" button if user was referred internally
  const showCTA =
    showBackButton || document.referrer.includes('nytimes')

  if (!errorType) return null

  const errorOptions = {
    offline: {
      icon: <ErrorIcon errorType="offline" />,
      content: <OfflineMessage />,
    },
    default: {
      icon: <ErrorIcon />,
      content: <DefaultMessage />,
    },
    missingData: {
      icon: <ErrorIcon />,
      content: <MissingData />,
    },
    precedesLaunch: {
      icon: <ErrorIcon />,
      content: <PrecedesLaunch />,
    },
  }

  const { icon, content } =
    errorOptions[errorType] || errorOptions.default

  const theme = useTheme()
  return (
    <div
      className={styles.container}
      style={{
        backgroundColor: theme.ERROR.BACKGROUND,
        color: theme.ERROR.COLOR,
      }}
      role="dialog"
    >
      <div className={styles.errorContainer}>
        {icon}
        {content}
        {showCTA && (
          <button
            className={styles.backButton}
            style={{
              backgroundColor: theme.BUTTONS.CTA.BACKGROUND,
              color: theme.BUTTONS.CTA.COLOR,
            }}
            type="button"
            data-testid="back"
            onClick={() => {
              navigateBack()
            }}
          >
            Back
          </button>
        )}
      </div>
    </div>
  )
}
