import { isBefore, isFuture } from 'date-fns'
import React, { useEffect, useState } from 'react'
import { useSelector } from 'react-redux'

import { useUser } from 'user'

import '../../index.scss'

import useAndroidBack from '../../../shared/hooks/use-android-back'
import { EVENT_ON_NAVIGATE_BACK } from '../../../../shared/foundation/js/hybrid/commands'
import { Moments, momentSystem } from '../../Moments'
import { loadSolution } from '../../api/api'
import { getOpenModalName } from '../../store/selectors'
import { closeModal } from '../../store/actions'
import {
  ErrorName,
  PuzzleResponseError,
  PuzzleResponseSuccess,
} from '../../api/types'
import { LAUNCH_DATE } from '../../constants'
import { saveInitialPuzzle } from '../../localstorage/localstorage'
import { Guess, Puzzle } from '../../types'
import Board from '../Board/Board'
import ErrorComponent from '../Error/Error'
import { ThemeProvider } from '../Theme'
import NavDate from '../NavDate/NavDate'

import * as helpers from './helpers'

class StatusError extends Error {
  constructor(public status?: number, message?: string) {
    super(message)
  }
}

const errorType = (
  shortzControlsOn: boolean,
  hasLoaded: boolean,
  error: StatusError | null,
  today: Date,
): ErrorName => {
  if (!shortzControlsOn || !hasLoaded) return 'default'

  if (error?.status === 404) {
    if (isFuture(today)) return 'missingData'

    if (isBefore(today, LAUNCH_DATE)) return 'precedesLaunch'
  }

  return 'default'
}

const Game = () => {
  const user = useUser()
  const modalName = useSelector(getOpenModalName)
  const [hasLoaded, setHasLoaded] = useState<boolean>(false)
  const [puzzle, setPuzzle] = useState<Puzzle | null>(null)
  const [error, setError] = useState<StatusError | null>(null)
  const [isReady, setIsReady] = useState<boolean>(false)
  const [storedGuesses, setStoredGuesses] = useState<Guess[]>([])
  const [playedBefore, setPlayedBefore] = useState<boolean>(false)
  // we'll have to wire this in later
  const [isGameFinished, _] = useState(false)

  const today = helpers.printDate(user)

  const shortzControlsOn =
    !(user.userType.isLoggedIn && !user.shortzHasLoaded) &&
    user.userType.inShortzMode

  const errorName = errorType(
    shortzControlsOn,
    hasLoaded,
    error,
    today,
  )

  useEffect(() => {
    momentSystem.load('loading')
  }, [])

  useEffect(() => {
    if (!hasLoaded) {
      loadSolution({
        today,
      })
        .then((data) => {
          if (data?.status === 200) {
            // Convert API data
            const convertedAPIData = helpers.convertAPIDataToGameData(
              data as PuzzleResponseSuccess,
            )

            // Save API data to localstorage and get current
            // user progress if there is any
            const { currentGameState, currentProgress } =
              saveInitialPuzzle(convertedAPIData)

            setStoredGuesses(currentProgress.guesses)
            setPlayedBefore(currentProgress.playedBefore)
            setPuzzle(currentGameState)
          } else {
            let status: undefined | number
            if (data?.status !== undefined) status = data?.status
            setError(
              new StatusError(
                status,
                (data as PuzzleResponseError).errors.join(', '),
              ),
            )
          }
          setHasLoaded(true)
        })
        .catch((e) => {
          setError(e)
          setHasLoaded(true)
        })
    } else {
      momentSystem.transition('loading', 'welcome')
    }
  }, [hasLoaded])

  useAndroidBack(
    (event) => {
      type Response = {
        gamesOnNavigateBack?: boolean
      }
      const response: Response = {}

      if (modalName) {
        closeModal(true)
        response[EVENT_ON_NAVIGATE_BACK] = true
      } else {
        // go back to app feed
        response[EVENT_ON_NAVIGATE_BACK] = false
      }
      event.respondWith(response)
    },
    [modalName],
  )

  return (
    <ThemeProvider>
      <NavDate today={today} />
      <Moments
        contentProps={{
          isGameFinished,
          editor: hasLoaded ? puzzle?.editor : 'Wyna Liu',
          today,
          puzzleId: puzzle?.id,
          transitionFromWelcomeToGame: () => {
            momentSystem
              .transition('welcome', 'game')
              .then(() => setIsReady(true))
          },
          transitionFromAdToGame: () => {
            momentSystem
              .transition('ad', 'game')
              .then(() => setIsReady(true))
          },
          transitionFromWelcomeToAd: () => {
            momentSystem.transition('welcome', 'ad')
          },
        }}
      />
      <>
        {(hasLoaded && !error && puzzle && (
          <Board
            categories={puzzle.categories}
            guesses={storedGuesses}
            today={today}
            isReady={isReady}
            playedBefore={playedBefore}
          />
        )) || <ErrorComponent showBackButton errorType={errorName} />}
      </>
    </ThemeProvider>
  )
}

export default Game
