import { format } from 'date-fns'

import { UserType } from 'foundation'

import { PuzzleResponseSuccess } from '../../api/types'
import { completedPuzzle } from '../../__tests__/fixtures/api/completed-puzzle'

import { convertAPIDataToGameData, printDate } from './helpers'

const updateUserType = jest.fn()

export const defaultShortzUserType: UserType = {
  isLoggedIn: false,
  hasXwd: false,
  hasDigi: false,
  hasHd: false,
  entitlement: 'anon',
  regiId: '',
  inShortzMode: true,
  isFreeTrial: false,
}

describe('convertAPIDataToGameData', () => {
  it('converts API response to correct shape for game', () => {
    const convertedGameData = convertAPIDataToGameData(
      completedPuzzle as PuzzleResponseSuccess,
    )

    // basic validation that categories and card transformations are correct
    expect(convertedGameData.categories.length).toEqual(4)
    expect(convertedGameData.categories[0].cards.length).toEqual(4)

    expect(convertedGameData.categories[0].title).toEqual(
      'PLAYFULLY BOTHER',
    )
    expect(convertedGameData.categories[0].solved).toEqual(false)
    expect(convertedGameData.categories[0].level).toEqual('Level1')
    expect(convertedGameData.categories[0].cards[0]).toEqual({
      level: 'Level1',
      selected: false,
      originalPosition: 11,
      currentPosition: 11,
      content: 'JOSH',
    })

    // additional check that more than one card category is  correct
    expect(convertedGameData.categories[1].cards[1]).toEqual({
      level: 'Level2',
      selected: false,
      originalPosition: 4,
      currentPosition: 4,
      content: 'MAX',
    })
  })

  describe('printDate', () => {
    it('returns correct print date from window location if user is in shortz mode and in shortz route', () => {
      const previousDate = `2022-12-12`

      Object.defineProperty(window, 'location', {
        writable: true,
        value: {
          pathname: '/puzzles/shortz',
        },
      })

      Object.defineProperty(window, 'gameData', {
        writable: true,
        value: {
          connectionsShortzDate: previousDate,
        },
      })

      const connectionsPrintDate = printDate({
        updateUserType,
        userType: defaultShortzUserType,
        hasLoaded: true,
        shortzHasLoaded: true,
      })

      expect(connectionsPrintDate).toEqual(
        new Date(`${previousDate}T12:00:00`),
      )
    })

    it('returns correct print data if user is not in shortz mode', () => {
      const today = new Date()
      Object.defineProperty(window, 'location', {
        writable: true,
        value: {
          pathname: '/puzzles/blah',
        },
      })

      const connectionsPrintDate = printDate({
        updateUserType,
        userType: { ...defaultShortzUserType, inShortzMode: false },
        hasLoaded: true,
        shortzHasLoaded: true,
      })

      // print date should not be from url
      expect(format(connectionsPrintDate, 'MMM d, yyyy')).not.toEqual(
        format(new Date('2022-12-12T20:00:00.000Z'), 'MMM d, yyyy'),
      )
      expect(format(connectionsPrintDate, 'MMM d, yyyy')).toEqual(
        format(today, 'MMM d, yyyy'),
      )
    })
  })
})
