import cn from 'classnames'
import React, { useEffect } from 'react'
import { useDispatch } from 'react-redux'

import { trackClick, trackImpression } from 'foundation'

import Button from '../../../shared/Button'
import CountdownTimer from '../../../shared/CountdownTimer'
import ModalWrapper from '../../../shared/ModalSystem/ModalWrapper'
import type { ModalProps } from '../../../shared/ModalSystem/modalTypes'
import { share } from '../../../../shared/utils/share'
import { DEFAULT_CARDS_PER_CATEGORY } from '../../constants'
import { Level } from '../../levelTypes'
import { openToast } from '../../store/actions'
import { ConnectionsToast } from '../../store/types'
import {
  CongratsTitleByMistakesMade,
  MistakeCount,
} from '../../types'
import { useTheme } from '../Theme'
import { getNthDayOfGame } from '../sharedHelpers'

import styles from './Congrats.module.scss'

const emojis: {
  [key in Level]: string
} = {
  Level1: '🟨',
  Level2: '🟩',
  Level3: '🟦',
  Level4: '🟪',
}

const generateShareText = (
  puzzle: string,
  guesses: Level[][] | undefined,
) => {
  const emojiRows = guesses
    ? guesses.map((row) => row.map((guess) => emojis[guess]).join(''))
    : []

  return `Connections \nPuzzle #${puzzle}\n${emojiRows.join('\n')}`
}

const ShareBlocks = ({
  guesses,
  cardsPerCategory,
}: {
  guesses: Level[][]
  cardsPerCategory: number
}) => {
  const theme = useTheme()

  const backgroundColor = (level: Level) =>
    theme.GAME.LEVELS[level].BACKGROUND

  return (
    <dl
      className={styles.shareBlocksContainer}
      aria-label={`${cardsPerCategory} by ${guesses.length} grid of color block emojis representing your guesses`}
    >
      {guesses.map((guess: Level[], i) => {
        const evaluation = guess.every((val) => val === guess[0])
          ? 'correct'
          : 'incorrect'

        const ariaLabelText = `${evaluation}. ${
          evaluation === 'correct'
            ? `Found level ${guess[0] + 1}.`
            : `Guessed items from levels ${guess.join(', ')}`
        }`

        return (
          <div key={`attempt-${i}`} className={styles.shareRow}>
            <dd aria-label={`attempt #${i + 1}`} />
            <dt aria-label={ariaLabelText}>
              {guess.map((level, j) => (
                <span
                  className={styles.shareBlock}
                  style={{
                    backgroundColor: backgroundColor(level),
                  }}
                  aria-hidden="true"
                  key={`item${j}-level${level}`}
                />
              ))}
            </dt>
          </div>
        )
      })}
    </dl>
  )
}

const Share = ({
  guesses,
  trackingPuzzleId,
  displayPuzzleNumber,
}: {
  guesses: Level[][] | undefined
  trackingPuzzleId: number
  displayPuzzleNumber: number
}) => {
  const dispatch = useDispatch()

  const shareMessage = generateShareText(
    displayPuzzleNumber.toString(),
    guesses,
  )

  const handleShareClick = (
    e: React.MouseEvent<HTMLButtonElement>,
  ) => {
    e.preventDefault()
    e.stopPropagation()

    trackClick({
      name: 'connections',
      label: 'share',
      context: trackingPuzzleId,
      moduleName: ['b: 4-8-4'],
      region: 'congrats',
    })

    const onCopy = () => {
      dispatch(openToast(ConnectionsToast.COPY_RESULTS))
    }

    const onFail = () => {
      dispatch(openToast(ConnectionsToast.SHARE_FAIL))
    }

    share({ text: shareMessage }, onCopy, onFail, e)
  }

  return (
    <Button
      className={styles.button}
      text="Share Your Results"
      iconLeft="share-outline"
      onClick={handleShareClick}
    />
  )
}

const PlaySpellingBee = ({
  trackingPuzzleId,
}: {
  trackingPuzzleId: number
}) => (
  <a href="/puzzles/spelling-bee">
    <Button
      className={cn(styles.button, styles.playSB)}
      type="secondary"
      text="Play Spelling Bee"
      iconRight="spelling-bee-wings"
      onClick={(e) => {
        e.stopPropagation()
        trackClick({
          name: 'connections',
          label: 'next-game',
          context: trackingPuzzleId,
          moduleLabel: 'spelling-bee',
        })
      }}
    />
  </a>
)

interface CongratsModalProps extends ModalProps {
  today?: Date
  numMistakes?: MistakeCount
  guesses?: Level[][]
  cardsPerCategory?: number
}

export default ({
  close,
  isClosing,
  onRemove,
  today = new Date(),
  numMistakes,
  guesses,
  cardsPerCategory = DEFAULT_CARDS_PER_CATEGORY,
}: CongratsModalProps) => {
  /**
   * TODO: Consolidate Puzzle ID logic.
   * At the moment, for tracking consistency, we are using 0-indexed puzzle ids,
   * but need to increment by 1 for an accurate display puzzle #
   */
  const trackingPuzzleId = getNthDayOfGame(today)
  const displayPuzzleNumber = trackingPuzzleId + 1

  useEffect(() => {
    trackImpression(
      'congrats-modal',
      'connections',
      `${numMistakes}`,
      `${trackingPuzzleId}`,
    )
  }, [])

  return (
    <ModalWrapper
      bodyClassName={cn('modal-rules-body', 'conn__modal')}
      close={close}
      isClosing={isClosing}
      onRemove={onRemove}
      closeLabel="Back to puzzle"
    >
      <div className={styles.modalContent}>
        <div>
          <h2
            data-testid="conn-modal__title"
            className={styles.modalTitle}
          >
            {`${
              numMistakes !== undefined &&
              CongratsTitleByMistakesMade[numMistakes]
            }!`}
          </h2>
          <h3 className={styles.modalDescription}>
            {`Connections #${displayPuzzleNumber}`}
          </h3>
          <ShareBlocks
            guesses={(guesses && guesses) || []}
            cardsPerCategory={cardsPerCategory}
          />
        </div>
        <div>
          <p className={styles.timerLabel}>Next puzzle in</p>
          <CountdownTimer />
        </div>

        <div className={styles.buttonsContainer}>
          <Share
            guesses={guesses}
            trackingPuzzleId={trackingPuzzleId}
            displayPuzzleNumber={displayPuzzleNumber}
          />
          <PlaySpellingBee trackingPuzzleId={trackingPuzzleId} />
        </div>
        <p className={styles.modalFeedback}>
          Have general feedback?{' '}
          <a
            href="https://iteratehq.com/nytimes/646f92ea79f79000014aff6a"
            onClick={(e) => {
              e.stopPropagation()
            }}
          >
            Let us know!
          </a>
        </p>
      </div>
    </ModalWrapper>
  )
}
