import cn from 'classnames'
import React, { useEffect } from 'react'

import { trackClick, trackImpression } from 'foundation'

import ModalWrapper from '../../../shared/ModalSystem/ModalWrapper'
import type { ModalProps } from '../../../shared/ModalSystem/modalTypes'
import { MAX_MISTAKES } from '../../constants'
import { Level } from '../../levelTypes'
import { savePlayedBefore } from '../../localstorage/localstorage'
import { useTheme } from '../Theme'
import { getNthDayOfGame } from '../sharedHelpers'
import { MistakeCount } from '../../types'

import styles from './HowToPlay.module.scss'

const CategoryDifficultyArrow = () => (
  <span aria-hidden="true" className={styles.helpArrowContainer}>
    <span className={styles.helpArrow} />
  </span>
)

const categoryLabelMapping = {
  [Level.Level1]: 'Straightforward',
  [Level.Level2]: 'Medium',
  [Level.Level3]: 'Difficult',
  [Level.Level4]: 'Tricky',
}

const CategoryDifficultyKey = () => {
  const theme = useTheme()

  const backgroundColor = (level: keyof typeof Level) => {
    return theme.GAME.LEVELS[level].BACKGROUND
  }

  return (
    <ul className={styles.categoryDifficultyKey}>
      <CategoryDifficultyArrow />
      {(Object.keys(Level) as Array<keyof typeof Level>).map(
        (level, i) => {
          return (
            <li
              key={`level-${i + 1}`}
              className={styles.categoryKeyItem}
              aria-label={`Level ${i + 1}: ${
                categoryLabelMapping[level]
              }`}
            >
              <span
                style={{ backgroundColor: backgroundColor(level) }}
                className={cn(styles.categoryKeyColor)}
                aria-hidden="true"
              />
              <span aria-hidden="true">
                {/* Only show level descriptor for first and last levels */}
                {level === Level.Level1 || level === Level.Level4
                  ? categoryLabelMapping[level]
                  : ''}
              </span>
            </li>
          )
        },
      )}
    </ul>
  )
}

interface HowToPlayModalProps extends ModalProps {
  today?: Date
}

export default ({
  close,
  isClosing,
  onRemove,
  today,
}: HowToPlayModalProps) => {
  useEffect(() => {
    trackImpression(
      'help',
      'connections',
      null,
      `${getNthDayOfGame(today)}`,
    )
  }, [])

  return (
    <ModalWrapper
      bodyClassName="modal-rules-body"
      close={() => {
        trackClick({
          name: 'connections',
          label: 'close',
          context: getNthDayOfGame(today),
          moduleName: ['b: 4-8-4'],
          region: 'help',
        })
        savePlayedBefore(true)
        close()
      }}
      isClosing={isClosing}
      onRemove={onRemove}
    >
      <div>
        <h2
          data-testid="conn-modal__title"
          className="modal-system-header"
        >
          How to Play
        </h2>
        <p className="modal-system-description">
          Find groups of four items that share something in common.
        </p>
        <ul className="bullet-list">
          <li className="modal-system-body">
            Select four items and tap &apos;
            <span className={styles.bold}>Submit</span>&apos; to check
            if your guess is correct.
          </li>
          <li className="modal-system-body">
            Find the groups without making {MAX_MISTAKES} mistakes!
          </li>
        </ul>
        <h4 className={styles.modalHeading}>Category Examples</h4>
        <dl className={cn('modal-system-body', styles.categoryList)}>
          <span className={styles.categoryListItem}>
            <dt>FISH:</dt>
            {` `}
            <dd>Bass, Flounder, Salmon, Trout</dd>
          </span>
          <span
            className={cn(
              'modal-system-body',
              styles.categoryListItem,
            )}
          >
            <dt aria-label="FIRE fill in the blank">
              <span aria-hidden="true">FIRE ___:</span>
            </dt>
            {` `}
            <dd>Ant, Drill, Island, Opal</dd>
          </span>
        </dl>
        <p className={cn('modal-system-body', styles.modalMessage)}>
          Categories will always be more specific than
          &quot;5-LETTER-WORDS,&quot; &quot;NAMES&quot; or
          &quot;VERBS.&quot;
        </p>
        <p className={cn('modal-system-body', styles.modalMessage)}>
          Each puzzle has exactly one solution. Watch out for words
          that seem to belong to multiple categories!
        </p>
        <p
          className={cn(
            'modal-system-body',
            styles.modalMessage,
            styles.categoryDifficultyDescription,
          )}
          aria-label="Each group is assigned a level, which will be revealed to you as you solve."
        >
          <span aria-hidden="true">
            Each group is assigned a color, which will be revealed as
            you solve:
          </span>
        </p>
        <CategoryDifficultyKey />
      </div>
    </ModalWrapper>
  )
}
