import React from 'react'
import {
  render,
  screen,
  act,
  fireEvent,
} from '@testing-library/react'

import toastStyles from './Toast.module.scss'
import Toast from './Toast'

describe('<Toast />', () => {
  beforeAll(() => jest.useFakeTimers())
  afterAll(() => jest.useRealTimers())
  const renderToast = (props: any) => {
    const store = {}

    const board = render(<Toast {...props} />)

    return { store, board }
  }

  it('should display Toast', () => {
    const removeFun = jest.fn(() => {})
    renderToast({
      text: 'Hi I am a delicious toast!',
      onRemove: removeFun,
    })

    const header = screen.getByText('Hi I am a delicious toast!')
    expect(header).toBeTruthy()
    expect(header.parentElement?.classList).toContain(
      toastStyles.fadeIn,
    )
  })

  it('should call remove on transition end', async () => {
    // eslint-disable-next-line require-await
    await act(async () => {
      const removeDuration = 4000
      const removeCallback = jest.fn(() => {})
      renderToast({
        text: 'Hi I am a delicious toast!',
        onRemove: removeCallback,
        durationVisible: removeDuration,
      })

      jest.advanceTimersByTime(removeDuration)
      // manually fire native transition end for DOM removal callback in component
      fireEvent.transitionEnd(screen.getByTestId('connection-toast'))

      expect(removeCallback).toBeCalled()
    })
  })

  it('should always set fade style regardless of callback', async () => {
    // eslint-disable-next-line require-await
    await act(async () => {
      const removeDuration = 4000
      const removeCallback = jest.fn(() => {})
      renderToast({
        text: 'Hi I am a delicious toast!',
        onRemove: removeCallback,
        durationVisible: removeDuration,
      })
      const header = screen.getByText('Hi I am a delicious toast!')
      jest.advanceTimersByTime(removeDuration)

      expect(header.parentElement?.classList).toContain(
        toastStyles.fadeOut,
      )
      expect(removeCallback).not.toBeCalled()
    })
  })

  it('should not call remove when toast is set to be always visible', async () => {
    // eslint-disable-next-line require-await
    await act(async () => {
      const removeDuration = Infinity
      const removeCallback = jest.fn(() => {})
      renderToast({
        text: 'Hi I am a delicious toast!',
        onRemove: removeCallback,
        durationVisible: removeDuration,
      })

      jest.advanceTimersByTime(removeDuration)
      // manually fire native transition end for DOM removal callback in component
      fireEvent.transitionEnd(screen.getByTestId('connection-toast'))

      expect(removeCallback).not.toBeCalled()
    })
  })
})
