import { captureSentryError } from 'foundation'

import { getNthDayOfGame } from '../components/sharedHelpers'
import { Level } from '../levelTypes'
import {
  Card,
  Category,
  ConnectionsLSCard,
  ConnectionsLSGameState,
  ConnectionsProgress,
  GameStateProps,
  Guess,
  Puzzle,
} from '../types'

/**
 * GLOBALS
 */
const ls = window.localStorage
const LS_KEY = 'nyt-connections-beta'
const STARTING_NUM_OF_ATTEMPTS = 4
let currentBoardLayout = null
let gameState: ConnectionsLSGameState = {}

/**
 * HELPERS
 */
const isStale = (timestamp: string) => {
  const lastWeek = new Date().getTime() - 604800000
  const lastPlayed =
    Number.parseInt(timestamp.toString().slice(0, 10)) * 1000

  return !lastPlayed || (lastPlayed && lastPlayed <= lastWeek)
}

const generateGuesses = (): Guess[] | undefined => {
  const flattenedCards = gameState.savedBoard?.flat()

  return gameState.history?.map((guess) => {
    // Track each guess word's category
    const groups: { [key: string]: boolean } = {}

    const guesses = guess.map((word: string) => {
      const foundCard = flattenedCards?.find(
        (card: ConnectionsLSCard) => card.text === word,
      )

      if (foundCard) {
        // Set the guess word's category and return its ID
        groups[foundCard.group] = true
        return foundCard.id.toString()
      }

      return ''
    })

    return {
      // An array of cards by ID
      cards: guesses.join(','),
      // If a guess's cards all share the same category, it's correct
      correct: Object.keys(groups).length === 1 || false,
    }
  })
}

/**
 * A helper util to take a flat array of cards
 * and break it into an array of 4 arrays (rows)
 * with 4 cards each
 */
const generateRows = (cards: ConnectionsLSCard[]) => {
  const board: ConnectionsLSCard[][] = []
  let newRow: ConnectionsLSCard[] = []

  cards.forEach((card, i) => {
    if ((i + 1) % 4 === 0) {
      newRow.push(card)
      board.push(newRow)
      newRow = []
    } else {
      newRow.push(card)
    }
  })

  return board
}

/**
 * LOCAL STORAGE UTILS
 */
const initialState: ConnectionsLSGameState = {
  highContrastEnabled: false,
  playedBefore: false,
  dayOfTest: getNthDayOfGame(),
  groupsFound: [],
  savedBoard: null,
  attemptsRemaining: STARTING_NUM_OF_ATTEMPTS,
  history: [],
  loading: false,
  gameStarted: false,
}

/**
 * Utility for clearing up space in local storage if a
 * user maxes out. Should be used in the event of a
 * QuotaExceeded error.
 */
function freeUpStorage() {
  const lsKeys = Object.keys(localStorage)

  // evaluate SBs
  const sbKeys = lsKeys.filter((k) => k.includes('nytsb'))

  sbKeys.forEach((bee) => {
    const { timestamp } = JSON.parse(ls.getItem(bee) || '')

    if (isStale(timestamp)) {
      ls.removeItem(bee)
    }
  })

  // evaluate XWDs
  const xwdKeys = lsKeys.filter((k) => k.includes('lastPlayed'))
  xwdKeys.forEach((xwd) => {
    const timestamp = JSON.parse(ls.getItem(xwd) || '')

    if (isStale(timestamp)) {
      ls.removeItem(xwd)
    }
  })
}

function isLocalStorageFull() {
  let save = ls.getItem(LS_KEY)
  const saveObj = JSON.parse(save || '')

  if (save && saveObj.status) {
    saveObj.status === 'full'
    save = JSON.stringify(saveObj)
    return save
  }

  return false
}

function setGameState(state: ConnectionsLSGameState) {
  gameState = state
  try {
    // only save to local storage if there's room
    if (!isLocalStorageFull()) {
      ls.setItem(LS_KEY, JSON.stringify(state))
    }
  } catch (error) {
    // try to clear out stale (>1 week old) XWD
    // and SB puzzles from local storage and re-try
    try {
      freeUpStorage()
      ls.setItem(LS_KEY, JSON.stringify(state))
    } catch (e) {
      // if gameState is somehow still empty, we'll default to the initial state
      // and bail on saving user's progress to local storage
      if (
        Object.keys(gameState).length === 0 ||
        Object.keys(gameState).length === 1
      ) {
        gameState = initialState
      }
      // We'll add a "status" of "full" to local storage (if we can) to prevent
      // other operations from trying to add to it
      ls.removeItem(LS_KEY)
      try {
        ls.setItem(LS_KEY, JSON.stringify({ status: 'full' }))
      } catch (err) {
        captureSentryError(
          new Error(`Failed to set local storage status: ${err}`),
        )
      }
    }
  }
}

function getSavedState() {
  let savedState = initialState
  try {
    const currentSavedState = ls.getItem(LS_KEY)
    if (currentSavedState) savedState = JSON.parse(currentSavedState)
  } catch (e) {
    // couldn't read saved state
    captureSentryError(new Error(`Couldn't fetch saved state: ${e}`))
  }

  return savedState
}

const getGameState = () => {
  if (Object.keys(gameState).length === 0) {
    gameState = initialState
  }

  return gameState
}

// Updates a given property with a given value.
function updateGameState(
  propertyName: string,
  propertyValue: GameStateProps,
) {
  gameState[propertyName] = propertyValue

  setGameState(gameState)
}

// Complete reset, erases all progress and regenerates
// puzzles. Does not erase playedBefore, dayOfTest, or chainMode.
function resetGameState() {
  const { playedBefore }: ConnectionsLSGameState = getGameState()

  const state = {
    ...initialState,
    playedBefore,
    dayOfTest: getNthDayOfGame(),
  }

  // clear local storage before reset
  localStorage.removeItem(LS_KEY)
  setGameState(state)
}

function resetGame() {
  resetGameState()
  currentBoardLayout = []
}

const convertGameDataToLSData = (
  categories: Category[],
): ConnectionsLSCard[][] => {
  const updatedCards = categories
    // Take each category's cards and flatten them into an array
    .flatMap((cat) =>
      cat.cards.map((card) => ({ ...card, category: cat.title })),
    )
    // Sort cards by ID (original position)
    .sort((a, b) => a.originalPosition - b.originalPosition)
    // Convert into expected card shape
    .map(
      (card): ConnectionsLSCard => ({
        id: card.originalPosition,
        selected: card.selected,
        text: card.content,
        group: card.category,
        solved: false,
      }),
    )

  // Break cards into 4 rows
  return generateRows(updatedCards) as ConnectionsLSCard[][]
}

const convertLSDataToGameData = (
  puzzleData: Puzzle,
  data: ConnectionsLSGameState,
): ConnectionsProgress => {
  // If we don't pass in localstorage data or if localstorage data
  // is empty, defer to API data
  if (!data || !data.savedBoard) {
    return {
      currentGameState: puzzleData,
      currentProgress: { guesses: [], playedBefore: false },
    }
  }

  const newBoard: Category[] = data.savedBoard?.reduce(
    (
      newCategories: Category[],
      oldCards: ConnectionsLSCard[],
      rowIdx: number,
    ) => {
      const updatedCategories = newCategories

      oldCards.forEach((card, cardIdx: number) => {
        const findMatchingCategory = (cat: Category) =>
          cat.title === card.group

        const newCategory = puzzleData.categories.find(
          findMatchingCategory,
        )

        // Category should always exist, but to appease TS, default to level 1
        const categoryLevel: Level = newCategory
          ? newCategory.level
          : Level.Level1

        const newCard = {
          content: card.text,
          originalPosition: card.id,
          level: categoryLevel,
          selected: card.selected,
          currentPosition: oldCards.length * rowIdx + cardIdx,
        }

        const categoryInUpdatedCategories = updatedCategories.find(
          findMatchingCategory,
        )

        // If the card's category already exists, add it to the array
        if (categoryInUpdatedCategories) {
          categoryInUpdatedCategories.cards.push(newCard)
        } else {
          // If not, create a new category and populate the cards array
          updatedCategories.push({
            title: card.group,
            solved: card.solved ? { order: rowIdx } : false,
            level: categoryLevel,
            cards: [newCard],
          })
        }
      })

      return updatedCategories
    },
    [],
  )

  return {
    currentGameState: {
      ...puzzleData,
      categories: newBoard,
    },
    currentProgress: {
      guesses: generateGuesses() || [],
      playedBefore: data.playedBefore || false,
    },
  }
}

function loadTodaysPuzzle(data: Puzzle): ConnectionsProgress {
  // Make sure our default puzzle data is set based on API response
  initialState.savedBoard = convertGameDataToLSData(data.categories)

  const { dayOfTest, savedBoard, attemptsRemaining, groupsFound } =
    getGameState()

  const nthDayOfTest = getNthDayOfGame()

  // check saved dayOfTest against today's puzzle #
  const isNewDay = nthDayOfTest !== dayOfTest

  function isGameDataOutOfSync(savedBoard: ConnectionsLSCard[][]) {
    const { categories } = data

    // get array of arrays of just group names
    const justGroups = savedBoard.map((row: ConnectionsLSCard[]) =>
      row.map((item: ConnectionsLSCard) => item.group),
    )
    const savedGroups: { [key: string]: boolean } = {}
    justGroups.forEach((g: string[]) => {
      // look through array of arrays of group names
      g.forEach((groupName: string) => {
        if (!savedGroups[groupName]) {
          savedGroups[groupName] = true
        }
      })
    })

    const savedGroupsArray = Object.keys(savedGroups)
    for (let i = 0; i < savedGroupsArray.length; i += 1) {
      const savedGroupName = savedGroupsArray[i]
      const realPuzzleGroups = categories.map(
        (cat: Category) => cat.title,
      )

      if (!realPuzzleGroups.includes(savedGroupName)) {
        // we need to reset our puzzle data
        resetGameState()
        return true
      }
      // otherwise we keep going and do nothing
    }

    return false
  }

  // @ts-ignore
  const gameDataOutOfSync = isGameDataOutOfSync(savedBoard)

  // Compare savedBoard with the data we get from fetch; if saved
  // game data is out of sync, we want to reset and clear out the in
  // progress game
  const gameInProgress = !isNewDay && savedBoard && !gameDataOutOfSync
  const updatedGroupsFound: string[] = []

  // if a game is in-progress, update the board with its saved state
  if (gameInProgress) {
    if (savedBoard) {
      currentBoardLayout = savedBoard
      let numSelected = 0

      currentBoardLayout.forEach((row: ConnectionsLSCard[]) => {
        if (row.every((item) => item.solved)) {
          updatedGroupsFound.push(row[0].group)
        }

        row.forEach((item) => {
          if (item.selected) numSelected += 1
        })
      })

      // if groupsFound doesn't match the board, update; we hit this check
      // if someone refreshes mid lose animation
      if (
        groupsFound &&
        groupsFound.length !== updatedGroupsFound.length
      ) {
        updateGameState('groupsFound', updatedGroupsFound)
      }
    }

    if (attemptsRemaining && attemptsRemaining >= 0) {
      updateGameState('attemptsRemaining', attemptsRemaining)
    }

    return convertLSDataToGameData(data, gameState)
  }

  // otherwise, reset
  resetGame()
  return convertLSDataToGameData(data, gameState)
}

/**
 * GAME STATE UPDATES
 */
export const saveInitialPuzzle = (data: Puzzle) => {
  // load saved data
  const savedState: ConnectionsLSGameState = getSavedState()
  const { playedBefore = false, dayOfTest = 0 } = savedState

  const localStorageIsFull =
    savedState.status && savedState.status === 'full'

  // check for an in-progress game
  if (!localStorageIsFull) {
    const gameIsInProgress =
      dayOfTest && dayOfTest === getNthDayOfGame()

    // replace global game state with saved state
    if (gameIsInProgress) {
      setGameState(savedState)
    }
  } else {
    // Try to reset local storage in case more space has become available.
    resetGameState()
  }

  return loadTodaysPuzzle(data)
}

export const saveSelectedCard = ({
  cardContent,
  selected,
}: {
  cardContent?: string
  selected: boolean
}) => {
  const { savedBoard } = gameState

  if (savedBoard) {
    const updatedBoard = savedBoard.map((row: ConnectionsLSCard[]) =>
      row.map((card) => {
        if (card.text === cardContent) {
          card.selected = selected
        }

        return card
      }),
    )

    updateGameState('savedBoard', updatedBoard)
  }
}

export const saveDeselectedCards = () => {
  const { savedBoard } = gameState

  if (savedBoard) {
    const updatedBoard = savedBoard.map((row: ConnectionsLSCard[]) =>
      row.map((card) => ({ ...card, selected: false })),
    )

    updateGameState('savedBoard', updatedBoard)
  }
}

export const saveCardOrder = ({
  cards,
  startIndex = 0,
}: {
  cards: Card[]
  startIndex?: number
}) => {
  const LSCardMap: { [key: number]: ConnectionsLSCard } = {}
  const LSCards = gameState.savedBoard?.flat() || []

  LSCards.forEach((card: ConnectionsLSCard): void => {
    LSCardMap[card.id] = card
  })

  const shuffledLSCards: ConnectionsLSCard[] = cards
    .sort((a, b) => a.currentPosition - b.currentPosition)
    .map((card) => {
      const LSCard = LSCardMap[card.originalPosition]

      return LSCard
    })

  /**
   * Because of rendering and animation issues * waves hands *
   * the last row doesn't send any cards along (and we
   * wouldn't need to shuffle them if we did get them).
   * Instead, pass the existing cards through unchanged.
   */
  const tempCards: ConnectionsLSCard[] =
    startIndex === 12 ? LSCards : LSCards.slice(0, startIndex)

  updateGameState(
    'savedBoard',
    generateRows([...tempCards, ...shuffledLSCards]),
  )
}

export const saveCategory = ({
  category,
  cards,
  solvedCategoryCount,
  autoSolved,
}: {
  category: Category | undefined
  cards: Card[]
  solvedCategoryCount: number
  autoSolved: boolean
}) => {
  if (category) {
    // Only update card order for cards that remain unsolved
    saveCardOrder({ cards, startIndex: solvedCategoryCount * 4 })

    const guess = category.cards.map((card) => card.content)

    const updatedBoard = gameState.savedBoard?.map(
      (row: ConnectionsLSCard[]) =>
        row.map((card) => {
          // If the card belongs to the solved category,
          // deselect it and mark it as solved
          if (card.group === category.title) {
            return { ...card, selected: false, solved: true }
          }

          return card
        }),
    )

    updateGameState('savedBoard', updatedBoard)
    updateGameState('groupsFound', [
      // @ts-ignore
      ...gameState.groupsFound,
      category.title,
    ])
    if (!autoSolved) {
      updateGameState('history', [
        ...(gameState.history as []),
        guess,
      ])
    }
  }
}

export const saveGuess = (cards: Card[]) => {
  const guess = cards.map((card) => card.content)
  const attemptsRemaining = gameState.attemptsRemaining
    ? gameState.attemptsRemaining - 1
    : undefined
  updateGameState('history', [...(gameState.history as []), guess])
  updateGameState('attemptsRemaining', attemptsRemaining)
}

export const savePlayedBefore = (playedBefore: boolean) => {
  updateGameState('playedBefore', playedBefore)
}

export const saveResetPuzzle = (categories: Category[]) => {
  const convertedBoard = convertGameDataToLSData(categories)

  updateGameState('playedBefore', false)
  updateGameState('history', [])
  updateGameState('groupsFound', [])
  updateGameState('savedBoard', convertedBoard)
}
