import React, { useEffect, useState } from 'react'

import { UserType, trackClick } from 'foundation'
import useAbra from 'shared/abra/useAbra'
import { useUser } from 'user'

import { momentSystem } from './index'

import { getLoginLink } from '../../helpers/loginLinks'
import { getLoginOfferLink } from '../../helpers/loginOfferLinks'
import { getSubscribeLink } from '../../helpers/subscribeLinks'
import Welcome from '../../shared/moments/components/Welcome'
import {
  getGameState,
  getNthDayOfTest,
  isGameFinished,
} from '../js/game-state'

import { canCheckForAdTest } from './AdMoment/validation'

function todaysDateAsString() {
  const today = new Date()
  return `${today.toLocaleString('default', {
    month: 'long',
  })} ${today.getDate()}, ${today.getFullYear()}`
}

const createButtonGroups = ({
  userType,
  hasAd,
  showOfferLoginLink,
  handleLoginOfferExpose,
}: {
  userType: UserType
  hasAd: boolean
  showOfferLoginLink: boolean
  handleLoginOfferExpose: Function
}) => {
  const { isLoggedIn, hasXwd } = userType
  const { dayOfTest }: { dayOfTest?: number } = getGameState()
  const nthDayOfTest = getNthDayOfTest()
  const playButton = {
    text: 'Play',
    action: () => {
      momentSystem.transition('welcome', hasAd ? 'ad' : 'game')

      try {
        if (window.dataLayer) {
          window.dataLayer.push({
            event: 'moduleInteraction',
            eventData: {
              trigger: 'module',
              type: 'click',
            },
            module: {
              context: dayOfTest || nthDayOfTest,
              name: ['b: 4-8-4'],
              element: {
                name: 'connections',
                label: 'start-game',
              },
            },
          })
        }
      } catch (e) {
        console.error(e)
      }
    },
  }

  const loginButton = {
    text: 'Log In',
    color: 'secondary',
    action: () => {
      trackClick({
        name: 'connections',
        label: 'log-in',
        context: dayOfTest || nthDayOfTest,
        region: 'welcome',
      })
      handleLoginOfferExpose()
      const loginLink = showOfferLoginLink
        ? getLoginOfferLink(window || undefined)
        : getLoginLink(window || undefined)
      window.location.assign(loginLink)
      return null
    },
  }

  const subscribeButton = {
    text: 'Subscribe',
    color: 'secondary',
    action: () => {
      trackClick({
        name: 'connections',
        label: 'subscribe',
        context: dayOfTest || nthDayOfTest,
        region: 'welcome',
      })
      const subscribeLink = getSubscribeLink('connections')
      window.location.assign(subscribeLink)
      return null
    },
  }

  // Subs only see play button
  if (hasXwd) {
    return [{ buttons: [playButton] }]
  }

  // Regis see play and subscribe buttons
  if (isLoggedIn) {
    return [
      { buttons: [playButton] },
      {
        header: 'Want to access all of our games?',
        desktopButtonOrder: 'vertical',
        buttons: [subscribeButton],
      },
    ]
  }

  // Anons see play, login and subscribe buttons
  return [
    { buttons: [playButton] },
    {
      header: 'Want to access all of our games?',
      desktopButtonOrder: 'vertical',
      buttons: [loginButton, subscribeButton],
    },
  ]
}

const MomentWelcome = () => {
  const { getVariant, reportExposure } = useAbra()
  const { userType, hasLoaded } = useUser()
  const hasAd = canCheckForAdTest({
    entitlement: userType.entitlement,
    isGameFinished: isGameFinished(),
    abraVariantChecks: { getVariant, reportExposure },
  })

  const loginOfferVariant = getVariant(
    'MKT_SubCon_Connections_PLOPRO_12_23',
  )

  const handleLoginOfferExpose = () => {
    if (loginOfferVariant) {
      reportExposure('MKT_SubCon_Connections_PLOPRO_12_23')
    }
  }

  const [momentProps, updateMomentProps] = useState({
    bgColor: '$connections-periwinkle',
    icon: 'connections',
    title: 'Connections',
    buttonGroups: [{}],
    description: {
      text: 'Group words that share a common thread.',
    },
    infoText: (
      <span className="pz-moment__info-editor">By Wyna Liu</span>
    ),
    date: todaysDateAsString(),
  })

  useEffect(() => {

    const showOfferLoginLink =
      !!loginOfferVariant && loginOfferVariant !== '0_Control'

    if (hasLoaded) {
      updateMomentProps({
        ...momentProps,
        buttonGroups: createButtonGroups({
          userType,
          hasAd,
          showOfferLoginLink,
          handleLoginOfferExpose,
        }),
      })
    }
  }, [hasLoaded, loginOfferVariant])

  return <Welcome game={momentProps} />
}

export default MomentWelcome
