import { format } from 'date-fns'
import React, { useEffect, useState } from 'react'

import { env, xhr } from 'foundation'

import { Moments, momentSystem } from '../Moments'
import { getNthDayOfTest } from '../js/game-state'
import startGame from '../js/main'
// @ts-ignore
import gameHtml from '../main.html'
import useSaleBannerHybridSubscribe from '../../shared/hooks/use-sale-banner-hybrid-subscribe'

import ErrorComponent from './Error'

const V1_ENDPOINT = (d: Date) =>
  `${env.edge}/svc/connections/v1/${format(d, 'yyyy-MM-dd')}.json`

export type PuzzleResponseError = {
  status: number
  errors: string[]
  results?: {}[]
}

export type PuzzleResponseSuccess = {
  id: number
  startingGroups: string[][]
  groups: { [key: string]: { level: number; members: string[] } }
  status: 200
}

class StatusError extends Error {
  constructor(public status?: number, message?: string) {
    super(message)
  }
}

export const loadSolution = ({ today }: { today: Date }) => {
  return xhr
    .get<PuzzleResponseSuccess | PuzzleResponseError>(
      V1_ENDPOINT(today),
      {
        withStatus: true,
      },
    )
    .catch((e) => {
      throw e
    })
}

const Game = () => {
  const [hasLoaded, setHasLoaded] = useState<boolean>(false)
  const [error, setError] = useState<StatusError | null>(null)
  useSaleBannerHybridSubscribe()

  useEffect(() => {
    momentSystem.load('welcome')

    if (!hasLoaded) {
      const today = new Date()
      const diffInDays = getNthDayOfTest()
      const fakedData = Array(diffInDays)
        .fill(0)
        .map((_: 0, id: number) => ({ id }))

      loadSolution({ today })
        .then((data) => {
          // if successful then replace the array saved to the window
          if (data.status === 200) {
            const d = data as PuzzleResponseSuccess
            const puzzle = {
              id: d.id,
              groups: d.groups,
              startingGroups: d.startingGroups,
            }
            fakedData.push(puzzle)
            window.gameData = fakedData
            setHasLoaded(true)
            startGame()
          } else {
            let status: undefined | number
            if (data?.status !== undefined) status = data?.status
            setError(
              new StatusError(
                status,
                (data as PuzzleResponseError).errors.join(', '),
              ),
            )
            setHasLoaded(true)
          }
        })
        .catch((e) => {
          setError(e)
          setHasLoaded(true)
        })
    }
  }, [hasLoaded])

  return (
    <>
      <Moments />
      {hasLoaded && !error && (
        <div dangerouslySetInnerHTML={{ __html: gameHtml }} />
      )}
      {hasLoaded && error && (
        <ErrorComponent showBackButton errorType="default" />
      )}
    </>
  )
}

export default Game
