import { STARTING_NUM_OF_ATTEMPTS } from './constants'
import { getGameState, getNthDayOfTest } from './game-state'
import { currPuzz, getLevelFromWord } from './gameManager'

// Analytics object for connections
const Analytics = {
  // Define name of prototype during initialization. Ex:  Analytics.defineGame('spelling-bee')
  defineGame: function defineGame(name, prefix) {
    this.gameName = name
    this.prefix = prefix
    this.eventLabel = prefix
  },

  shuffle: function shuffle() {
    const { dayOfTest } = getGameState()
    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            name: ['b: 4-8-4'],
            element: {
              name: 'connections',
              label: 'shuffle',
            },
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  deselect: function deselect() {
    const { dayOfTest } = getGameState()
    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            name: ['b: 4-8-4'],
            element: {
              name: 'connections',
              label: 'deselect',
            },
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  openHelp: function openHelp() {
    const { dayOfTest } = getGameState()
    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            name: ['b: 4-8-4'],
            element: {
              name: 'connections',
              label: 'help',
            },
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  closeHelp: function closeHelp() {
    const { dayOfTest } = getGameState()
    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            name: ['b: 4-8-4'],
            region: 'help',
            element: {
              name: 'connections',
              label: 'close',
            },
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  congrats: function congrats(mistakesMade) {
    const { dayOfTest } = getGameState()
    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'impression',
          module: {
            name: 'congrats-modal',
            region: 'connections',
            label: mistakesMade,
            context: dayOfTest || getNthDayOfTest(),
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  help: function help() {
    const { dayOfTest } = getGameState()
    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'impression',
          module: {
            name: 'help',
            region: 'connections',
            context: dayOfTest || getNthDayOfTest(),
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  solveAttempt: function solveAttempt(success, level) {
    const { dayOfTest, history, attemptsRemaining } = getGameState()
    const mistakesMade = STARTING_NUM_OF_ATTEMPTS - attemptsRemaining

    // Key for label array:
    // r = result: pass/fail,
    // w = words: all four words with commas between
    // d = difficulty: 1-4 (4 is hardest) or null on a failed attempt
    // m = mistakes: # of mistakes made so far
    const label = [
      `r: ${success ? 'pass' : 'fail'}`,
      `w: ${history[history.length - 1]}`,
      `d: ${success ? level + 1 : null}`, // levels are 0-indexed
      `m: ${mistakesMade}`,
    ]

    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            name: ['b: 4-8-4'],
            element: {
              name: 'connections',
              label: 'solve-attempt',
            },
            label,
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  completeGame: function completeGame() {
    const { dayOfTest, groupsFound, attemptsRemaining } =
      getGameState()
    const mistakesMade = STARTING_NUM_OF_ATTEMPTS - attemptsRemaining

    const { groups } = currPuzz

    // Key for label array:
    // o = order of difficulty: order in which the users submitted correct difficulty guesses
    // m = mistakes: # of mistakes made
    const label = [
      `o: ${groupsFound.map((g) => groups[g].level + 1)}`,
      `m: ${mistakesMade}`,
    ]

    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            name: ['b: 4-8-4'],
            element: {
              name: 'connections',
              label: '100%-complete',
            },
            label,
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  share: function share() {
    const { dayOfTest, attemptsRemaining, history } = getGameState()
    const mistakesMade = STARTING_NUM_OF_ATTEMPTS - attemptsRemaining

    // transform arrays of word guesses into array of each word's level
    // e.g. ['word1', 'word3', ...] => [1, 3, ...]
    const levelsHistory = history.map((entry) =>
      entry.map((w) => getLevelFromWord(w)),
    )

    // filter to only correct guesses
    const correctCategories = levelsHistory
      .filter((l) => l.every((level) => level === l[0]))
      .map((l) => l[0])

    // Key for label array:
    // o = order of difficulty: order in which the users submitted correct difficulty guesses
    // m = mistakes: # of mistakes made
    const label = [
      `o: ${correctCategories.map((c) => c + 1)}`,
      `m: ${mistakesMade}`,
    ]

    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            name: ['b: 4-8-4'],
            region: 'congrats',
            element: {
              name: 'connections',
              label: 'share',
            },
            label,
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },

  nextGame: function nextGame() {
    const { dayOfTest } = getGameState()
    try {
      if (window.dataLayer) {
        window.dataLayer.push({
          event: 'moduleInteraction',
          eventData: {
            trigger: 'module',
            type: 'click',
          },
          module: {
            context: dayOfTest || getNthDayOfTest(),
            label: 'spelling-bee',
            name: ['b: 4-8-4'],
            element: {
              name: 'connections',
              label: 'next-game',
            },
          },
        })
      }
    } catch (e) {
      console.log(e)
    }
  },
}

export default Analytics
