import { STARTING_NUM_OF_ATTEMPTS } from './constants'
import { freeUpStorage } from './freeUpStorage'

const ls = window.localStorage
const KEY = 'nyt-connections-beta'

// starts at 0
const ONE_DAY_EXACTLY_MS = 1000 * 60 * 60 * 24

export function getNthDayOfTest() {
  const startDate = new Date('6/12/2023')
  const todaysDate = new Date()

  const msDiff =
    todaysDate.setHours(0, 0, 0, 0) - startDate.setHours(0, 0, 0, 0)

  let dayDiff = Math.round(msDiff / ONE_DAY_EXACTLY_MS)

  if (dayDiff < 0) {
    dayDiff = Math.abs(dayDiff)
  }

  return dayDiff
}

const initialState = {
  highContrastEnabled: false,
  playedBefore: false,
  dayOfTest: getNthDayOfTest(),
  groupsFound: [],
  savedBoard: null,
  attemptsRemaining: STARTING_NUM_OF_ATTEMPTS,
  history: [],
  loading: false,
  gameStarted: false,
}

let gameState = {}

function isLocalStorageFull() {
  const save = ls.getItem(KEY)
  if (save && save.status) {
    return save.status === 'full'
  }

  return false
}

// Sets the saved game state to a given object.
export function setGameState(state) {
  try {
    gameState = state

    // only save to local storage if there's room
    if (!isLocalStorageFull()) {
      ls.setItem(KEY, JSON.stringify(state))
    }
  } catch (error) {
    // try to clear out stale (>1 week old) XWD
    // and SB puzzles from local storage and re-try
    try {
      freeUpStorage()
      ls.setItem(KEY, JSON.stringify(state))
    } catch (e) {
      // if gameState is somehow still empty, we'll default to the initial state
      // and bail on saving user's progress to local storage
      if (
        Object.keys(gameState).length === 0 ||
        Object.keys(gameState).length === 1
      ) {
        gameState = initialState
      }
      // We'll add a "status" of "full" to local storage (if we can) to prevent
      // other operations from trying to add to it
      ls.removeItem(KEY)
      try {
        ls.setItem(KEY, JSON.stringify({ status: 'full' }))
      } catch (err) {
        window.captureSentryException(
          `Failed to set local storage status: ${err}`,
        )
      }
    }
  }
}

export function getSavedState() {
  let savedState = {}
  try {
    savedState = JSON.parse(ls.getItem(KEY)) || initialState
  } catch (e) {
    // couldn't read saved state
    window.captureSentryException(`Couldn't fetch saved state: ${e}`)
  }

  return savedState
}

export function getGameState() {
  if (Object.keys(gameState).length === 0) {
    gameState = initialState
  }

  return gameState
}

// Updates a given property with a given value.
export function updateGameState(propertyName, propertyValue) {
  gameState[propertyName] = propertyValue
  setGameState(gameState)
}

// Complete reset, erases all progress and regenerates
// puzzles. Does not erase playedBefore, dayOfTest, or chainMode.
export function resetGameState() {
  const { playedBefore } = getGameState()

  const state = {
    ...initialState,
    playedBefore,
    dayOfTest: getNthDayOfTest(),
  }

  // clear local storage before reset
  localStorage.removeItem(KEY)

  setGameState(state)
}

export function isGameFinished() {
  const { savedBoard, groupsFound, attemptsRemaining } =
    getGameState()

  if (
    savedBoard?.length &&
    groupsFound?.length &&
    groupsFound.length === savedBoard.length
  ) {
    return true
  }

  if (attemptsRemaining <= 0) {
    return true
  }

  return false
}
