/* eslint-disable no-plusplus, import/no-mutable-exports */
import {
  getGameState,
  getNthDayOfTest,
  resetGameState,
  updateGameState,
} from './game-state'
import Item from './Item'
import {
  GROUP_SIZE,
  NUM_OF_GROUPS,
  STARTING_NUM_OF_ATTEMPTS,
  titlesByMistakesMade,
} from './constants'
// TODO look into dependency cycles
import {
  addListenersToItemButtons,
  deselectAll,
  grayOutGameButtons,
  hideGameButtons,
  openCongrats,
  removeShakeAnimAfterShakeFinished,
  renderBoard,
  resetGUI,
  showViewResultsButton,
  showToast,
  shuffle,
  setSubmitButtonActive,
  // removeLoading,
  renderSettingToggles,
} from './gui'
import {
  SLIDE_TIME_IN_MS,
  parsePosition,
  slideAnim,
  addPressedListeners,
} from './animations'
import { generateShareText } from './social'
import { share } from './share'
import getFeedbackLink from './feedback'
import { todaysDateAsString } from './utils'
import Analytics from './analytics'
import { closeAllModals } from './modal'
import { destroyCountdownTimer } from './countdown'

// CURRENT GAME STATE:
export let currPuzz = null
export let currPuzzIndex = null

// 2D array of Items
export let currentBoardLayout = null

const items = {}

// array to track which "banners" (solved categories)
// have already been animated
const answerDisplayed = Array(NUM_OF_GROUPS).fill(false)

// shows category atop solved elements
export function showAnswerBanners(groupsFound) {
  let bannersShown = 0
  for (let i = 0; i < groupsFound.length; i++) {
    const groupName = groupsFound[i]
    const { members, level } = currPuzz.groups[groupName]

    const banner = document.createElement('div')
    banner?.classList?.add('answer-banner', `group-${level}`)

    // animate any new banners
    const bannerAlreadyShown = answerDisplayed[i]
    if (!bannerAlreadyShown) {
      banner?.classList?.add('solved-pulse')
      setTimeout(() => {
        banner?.classList?.remove('solved-pulse')
      }, 800)
    }

    const solvedCategoryContents = `<span>${groupName.toUpperCase()}</span><br/>${members.join(
      ', ',
    )}`

    const txt = document.createElement('p')
    txt.innerHTML = solvedCategoryContents
    banner.appendChild(txt)

    const rowID = `row-${bannersShown}`
    document.getElementById(rowID).appendChild(banner)
    document.getElementById(`row-${bannersShown}`).appendChild(banner)
    banner?.classList?.add(`item-row-${bannersShown}`)
    bannersShown++
    answerDisplayed[i] = true
  }
}

export function getGroupNumOfItem(item) {
  return currPuzz.groups[item.group].level
}

// what we've covered up should never be a bigger number than what we've solved for.
// if we're about to cover something up beyond the quantity we've solved for, move
function moveSolvedGroup(whichGroup, timeout) {
  const { groupsFound } = getGameState()

  const newGroupsFound = [...groupsFound, whichGroup]
  updateGameState('groupsFound', newGroupsFound)

  // Start at row of interest. e.g. If this is the first group you've found, you'll be moving the found items to the first row.
  const rowNumToMoveTo =
    newGroupsFound.length - 1 >= 0 ? newGroupsFound.length - 1 : 0

  const { level } = currPuzz.groups[whichGroup]

  // make a shallow copy of group members since we'll be removing items
  const itemsToMove = [...currPuzz.groups[whichGroup].members]
  const rowToMoveTo = Array(GROUP_SIZE) // array of Items
  const displacedItems = []

  // Move through row of interest as it currently is.
  for (
    let j = 0;
    j < currentBoardLayout[rowNumToMoveTo].length;
    j++
  ) {
    const item = currentBoardLayout[rowNumToMoveTo][j]
    if (getGroupNumOfItem(item) === level) {
      // item is already in correct row
      rowToMoveTo[j] = item
      // remove from itemsToMove, no need to move it if it's already in the correct row.
      const index = itemsToMove.indexOf(item.text)
      itemsToMove.splice(index, 1)
    } else {
      rowToMoveTo[j] = null
      displacedItems.push(item)
    }
  }

  let slotNumToMoveTo = 0
  // Move through remaining rows until all itemsToMove have been found
  for (
    let r = newGroupsFound.length;
    r < currentBoardLayout.length;
    r++
  ) {
    for (let c = 0; c < currentBoardLayout[r].length; c++) {
      const item = currentBoardLayout[r][c]
      if (itemsToMove.length > 0 && displacedItems.length > 0) {
        if (getGroupNumOfItem(item) === level) {
          // item is one we want to move

          // move to next available slot of rowToMoveTo
          for (let n = slotNumToMoveTo; n < rowToMoveTo.length; n++) {
            if (rowToMoveTo[n] === null) {
              slotNumToMoveTo = n
              break
            }
          }
          rowToMoveTo[slotNumToMoveTo] = item
          currentBoardLayout[rowNumToMoveTo][slotNumToMoveTo] = item
          const index = itemsToMove.indexOf(item.text)
          itemsToMove.splice(index, 1)
          const displacedItem = displacedItems.pop()
          currentBoardLayout[r][c] = displacedItem

          const itemDiv = document.getElementById(`item-${item.id}`)
          const displacedItemDiv = document.getElementById(
            `item-${displacedItem.id}`,
          )

          setTimeout(() => {
            const itemPos = [
              parsePosition(itemDiv, 'top'),
              parsePosition(itemDiv, 'left'),
            ]

            const displacedItemPos = [
              parsePosition(displacedItemDiv, 'top'),
              parsePosition(displacedItemDiv, 'left'),
            ]

            slideAnim(itemDiv, itemPos, displacedItemPos)
            slideAnim(displacedItemDiv, displacedItemPos, itemPos)
          }, timeout)
        }
      } else {
        break
      }

      updateGameState('savedBoard', currentBoardLayout)
    }
  }

  // printBoard()
}

const staggeredGroupMove = (group, num) =>
  new Promise((resolve) => {
    setTimeout(() => {
      moveSolvedGroup(group)
      setTimeout(() => {
        const { groupsFound } = getGameState()
        renderBoard(groupsFound)
        resolve()
      }, 1000)
    }, 2000 * num)
  })

const staggeredMoveGroups = (groups) => {
  const promises = groups.map((g, i) => staggeredGroupMove(g, i + 1))
  return Promise.all(promises)
}

function getCorrectBoardLayout() {
  let newID = 0
  const correctItems = []

  if (currPuzz) {
    Object.keys(currPuzz.groups).forEach((category) => {
      const correctRow = []
      const words = currPuzz.groups[category].members
      words.forEach((w) => {
        const item = new Item(newID, false, w, category, true)
        correctRow.push(item)
      })
      correctItems.push(correctRow)
      newID++
    })
  }

  return correctItems
}

export async function revealAllGroups() {
  const groups = Object.keys(currPuzz.groups)
  let { groupsFound } = getGameState()
  Object.values(items).forEach((item) => {
    const itemDiv = document.getElementById(`item-${item.id}`)
    itemDiv?.classList?.remove('selected')
    itemDiv?.classList?.add('correct')
    items[item.id].solved = true
    items[item.id].selected = false
  })

  // immediately update saved state to the "correct" layout
  // in case the player refreshes/disconnects
  updateGameState('savedBoard', getCorrectBoardLayout())

  const groupsNotSolved = groups.filter(
    (g) => !groupsFound.includes(g),
  )

  if (groupsNotSolved.length === 0) return

  // Get our updated groups found
  groupsFound = getGameState().groupsFound

  await staggeredMoveGroups(groupsNotSolved).then(() => {
    setTimeout(() => {
      showAnswerBanners(groupsFound)
    }, 500 * (groupsNotSolved.length + 1))
  })
}

// success is true if solved within given # of attempts
export async function finishGame(success, returning = false) {
  if (returning) {
    hideGameButtons()
    showViewResultsButton()
  }

  const gameNewlyLost = !success && !returning
  const gameNewlyWon = success && !returning

  // only show toast when game is newly won/lost
  if (gameNewlyWon || gameNewlyLost) {
    const { attemptsRemaining } = getGameState()
    showToast(
      titlesByMistakesMade[
        STARTING_NUM_OF_ATTEMPTS - attemptsRemaining
      ],
    )
    try {
      Analytics.completeGame()
    } catch (e) {
      console.log(e)
    }
  }

  if (gameNewlyLost) {
    grayOutGameButtons()
    await revealAllGroups()
    setTimeout(openCongrats, 3000)
    return
  }

  if (gameNewlyWon) {
    setTimeout(openCongrats, 2000)
  }
}

// animate: if true, animate number change. if false, just change instantaneously
function updateAttemptsRemaining(newNum, animate, timeout) {
  updateGameState('attemptsRemaining', newNum)
  const bubbleToPop = document.getElementById(`bubble-${newNum}`)

  if (animate && bubbleToPop) {
    setTimeout(() => {
      bubbleToPop?.classList?.add('pop')
      setTimeout(() => {
        bubbleToPop?.classList?.remove('pop')
        bubbleToPop?.classList?.add('lost')
      }, 400)
    }, timeout)
  } else {
    // hide attempts already used
    for (let i = newNum; i < STARTING_NUM_OF_ATTEMPTS; i++) {
      document.getElementById(`bubble-${i}`)?.classList?.add('lost')
    }
  }
}

function bounceCurrentlySelectedItems() {
  const currentlySelectedItems = Array.from(
    document.querySelectorAll('.selected:not(#connections-nav)'),
  )

  // eslint-disable-next-line
  for (let i = 0; i < currentlySelectedItems.length; i++) {
    const curr = currentlySelectedItems[i]
    setTimeout(() => {
      curr?.classList?.add('short-bounce')
    }, 110 * i)

    setTimeout(() => {
      curr?.classList?.remove('short-bounce')
    }, 600)
  }
}

function addToHistory(selected) {
  const { history } = getGameState()
  updateGameState('history', [
    ...history,
    selected.map((i) => i.text),
  ])
}

function getCurrentlySelectedItems() {
  return Array.from(
    document.querySelectorAll('.selected:not(#connections-nav)'),
  ).map((i) => items[i.id.replace('item-', '')])
}

export function exactlyThreeCorrect() {
  const selected = getCurrentlySelectedItems()

  const groupCounts = [0, 0, 0, 0] // groupCounts is how many of each group were submitted. e.g. [0, 3, 1, 0] means 3 of second group and 1 of third group were submitted.
  for (let n = 0; n < 4; n++) {
    const whichGroup = getGroupNumOfItem(selected[n])
    groupCounts[whichGroup]++
  }

  return groupCounts.includes(3)
}

// check if all of the selected elements are in the same group
// (determine if user was correct or not)
// and call functions so that graphics reflect that accordingly
function checkSelectedElements() {
  const { attemptsRemaining, groupsFound } = getGameState()

  const selected = getCurrentlySelectedItems()
  const whichGroup = selected[0].group

  addToHistory(selected)

  bounceCurrentlySelectedItems()
  // eslint-disable-next-line
  for (let k = 1; k < GROUP_SIZE; k++) {
    const groupOfInterest = currPuzz.groups[whichGroup].members.map(
      (t) => t.toLowerCase(),
    )

    const incorrect = !groupOfInterest.includes(
      selected[k].text.toLowerCase(),
    )

    if (incorrect) {
      const newAttemptsRemaining = attemptsRemaining - 1

      // only show "one away" toast if
      // you have more guesses left
      if (newAttemptsRemaining > 0 && exactlyThreeCorrect()) {
        setTimeout(() => {
          showToast('One away...')
        }, 1200)
      }

      for (let n = 0; n < selected.length; n++) {
        const curr = selected[n]
        const itemDiv = document.getElementById(`item-${curr.id}`)

        itemDiv?.classList?.add('inactive')
        setTimeout(() => {
          itemDiv?.classList?.add('invalid-shake')
        }, 1200)
      }

      setTimeout(() => {
        removeShakeAnimAfterShakeFinished(selected)
      }, 2200)

      updateAttemptsRemaining(newAttemptsRemaining, true, 2200)
      try {
        Analytics.solveAttempt(false)
      } catch (e) {
        console.log(e)
      }

      if (newAttemptsRemaining === 0) {
        setTimeout(() => finishGame(false), 2200)
      }

      return false
    }
  }

  // Correct

  // only deselect if correct
  deselectAll()

  for (let i = 0; i < selected.length; i++) {
    selected[i].solved = true
  }

  const newGroupsFound = [...groupsFound, whichGroup]

  // beat 1. swap positions of items, slide animation
  moveSolvedGroup(whichGroup, 1200)

  // beat 2. high energy pulse in correct color
  setTimeout(() => {
    for (let i = 0; i < selected.length; i++) {
      selected[i].solved = true
    }
    renderBoard(newGroupsFound)
  }, 1200 + SLIDE_TIME_IN_MS)

  // beat 3. fusing of row + category reveal is last step.
  setTimeout(() => {
    for (let i = 0; i < selected.length; i++) {
      const itemDiv = document.getElementById(
        `item-${selected[i].id}`,
      )
      itemDiv?.classList?.add('correct')
    }

    if (newGroupsFound.length === NUM_OF_GROUPS) {
      grayOutGameButtons()
      finishGame(true)
    }
  }, 1200 + SLIDE_TIME_IN_MS)

  try {
    const { level } = currPuzz.groups[whichGroup]
    Analytics.solveAttempt(true, level)
  } catch (e) {
    console.log(e)
  }

  return true
}

export function getGroupNameFromLevel(level) {
  const groupName = Object.keys(currPuzz.groups).find(
    (g) => currPuzz.groups[g].level === level,
  )

  return currPuzz.groups[groupName].level
}

function getGroupNameFromWord(word) {
  let result
  const { groups } = currPuzz
  Object.keys(groups).forEach((g) => {
    const { members } = groups[g]
    members.forEach((t) => {
      if (t.toLowerCase() === word.toLowerCase()) {
        result = g
      }
    })
  })
  return result
}

// convert puzzle data to divs & fill out currentBoardLayout
export function createItems() {
  const board = document.getElementById('board')
  board.innerHTML = ''
  let newID = 0
  for (let i = 0; i < NUM_OF_GROUPS; i++) {
    const row = document.createElement('div')
    row?.classList?.add('flex-col')
    row.setAttribute('id', `row-${i}`)
    currentBoardLayout[i] = []
    for (let j = 0; j < GROUP_SIZE; j++) {
      const itemDiv = document.createElement('div')
      itemDiv?.classList?.add('item')
      itemDiv.innerHTML = currPuzz.startingGroups[i][j] /// Hmmm do we need this given image puzzles TODO
      itemDiv.setAttribute('id', `item-${newID.toString()}`)

      // determine an item's group
      const text = currPuzz.startingGroups[i][j]
      const group = getGroupNameFromWord(text)
      const item = new Item(newID, false, text, group)

      row.appendChild(itemDiv)
      currentBoardLayout[i][j] = item
      items[newID] = item
      newID++
    }
    board.appendChild(row)
  }

  addListenersToItemButtons()
  updateGameState('savedBoard', currentBoardLayout)
}

function resetGame() {
  resetGameState()
  currentBoardLayout = []
  createItems()
}

function alreadyGuessed(selected) {
  const { history } = getGameState()
  const group = selected.map((s) => s.textContent)

  for (let i = 0; i < history.length; i++) {
    const entry = history[i]
    if (entry.every((e) => group.includes(e))) {
      return true
    }
  }

  return false
}

export function submitAnswer() {
  const { attemptsRemaining } = getGameState()

  const currentlySelectedItems = Array.from(
    document.querySelectorAll('.selected:not(#connections-nav)'),
  )

  const submitButton = document.getElementById('submit-button')
  if (
    currentlySelectedItems.length === GROUP_SIZE &&
    attemptsRemaining > 0 &&
    !submitButton?.classList?.contains('inactive')
  ) {
    submitButton?.classList?.add('inactive')

    // show toast and bail if group has already been guessed
    if (alreadyGuessed(currentlySelectedItems)) {
      showToast('Already guessed!')
      return
    }

    checkSelectedElements()
  }
}

function showErrorScreen() {
  const cover = document.getElementById('error-cover')
  cover?.classList?.remove('hide')
  // to prevent scrolling behind screen
  document.body.style.overflow = 'hidden'
}

export function loadTodaysPuzzle() {
  const { dayOfTest, savedBoard, attemptsRemaining, groupsFound } =
    getGameState()
  const dataNotLoaded = !window?.gameData || !window?.gameData.length
  if (dataNotLoaded) {
    showErrorScreen()
  }
  const fetchPuzzles = window?.gameData
  resetGUI()
  const puzzles = fetchPuzzles
  const nthDayOfTest = getNthDayOfTest()

  // check saved dayOfTest against today's puzzle #
  const isNewDay = nthDayOfTest !== dayOfTest
  const index = isNewDay ? nthDayOfTest : dayOfTest

  // allows us to loop through puzzle list
  // if we run out (shouldn't happen, but JIC)
  currPuzzIndex = index % puzzles.length
  currPuzz = puzzles[currPuzzIndex]

  if (isNewDay) {
    closeAllModals()
    destroyCountdownTimer()
  }

  function isGameDataOutOfSync(savedBoard) {
    if (savedBoard) {
      const { groups: realPuzzleGroups } = currPuzz

      // get array of arrays of just group names
      const justGroups = savedBoard.map((row) =>
        row.map((item) => item.group),
      )
      const savedGroups = {}
      justGroups.forEach((g) => {
        // look through array of arrays of group names
        g.forEach((groupName) => {
          if (!savedGroups[groupName]) {
            savedGroups[groupName] = true
          }
        })
      })

      const savedGroupsArray = Object.keys(savedGroups)
      for (let i = 0; i < savedGroupsArray.length; i++) {
        const savedGroupName = savedGroupsArray[i]
        if (!realPuzzleGroups[savedGroupName]) {
          // we need to reset our puzzle data
          resetGameState()
          return true
        }
        // otherwise we keep going and do nothing
      }
    }

    return false
  }

  const gameDataOutOfSync = isGameDataOutOfSync(savedBoard)

  // Compare savedBoard with the data we get from fetch; if saved
  // game data is out of sync, we want to reset and clear out the in
  // progress game
  const gameInProgress = !isNewDay && savedBoard && !gameDataOutOfSync

  const updatedGroupsFound = []

  // if a game is in-progress, update the board with its saved state
  if (gameInProgress) {
    if (savedBoard) {
      currentBoardLayout = savedBoard
      let numSelected = 0

      currentBoardLayout.forEach((row) => {
        if (row.every((item) => item.solved)) {
          updatedGroupsFound.push(row[0].group)
        }

        row.forEach((item) => {
          items[item.id] = item
          if (item.selected) numSelected++
        })
      })

      // if groupsFound doesn't match the board, update; we hit this check
      // if someone refreshes mid lose animation
      if (groupsFound.length !== updatedGroupsFound) {
        updateGameState('groupsFound', updatedGroupsFound)
      }

      // if four cards are selected, activate the submit button
      if (numSelected === 4) {
        setSubmitButtonActive(true)
      }
    }

    if (attemptsRemaining >= 0) {
      updateAttemptsRemaining(attemptsRemaining, false, 0)
    }

    renderBoard(
      updatedGroupsFound.length >= groupsFound
        ? updatedGroupsFound
        : groupsFound,
    )

    // did we load an already-complete game?
    const lastRow = savedBoard[NUM_OF_GROUPS - 1]
    const gameAlreadyComplete = lastRow.every((item) => item.solved)
    if (gameAlreadyComplete) {
      finishGame(attemptsRemaining > 0, true)
    }
  } else {
    // otherwise, reset
    resetGame()
    renderBoard([])
  }
}

// event listener to ensure folks coming back to the same
// tab to play get a new day's puzzles (or their in-progress
// game from before)
window?.addEventListener('focus', () => {
  // ensure that the initial data had been set in Game.tsx before loading
  if (window.gameData) {
    loadTodaysPuzzle()
  }
})

export function getLevelFromWord(word) {
  let result
  const { groups } = currPuzz
  Object.keys(groups).forEach((g) => {
    const { members, level } = groups[g]
    if (
      members.map((m) => m.toLowerCase()).includes(word.toLowerCase())
    ) {
      result = level
    }
  })
  return result
}

export function getItemFromDiv(div) {
  const idOfInterest = parseInt(div.id.substring(5))
  return items[idOfInterest]
}

export function printBoard() {
  for (let i = 0; i < currentBoardLayout.length; i++) {
    let rowToPrint = ''
    for (let j = 0; j < currentBoardLayout[i].length; j++) {
      rowToPrint += `${currentBoardLayout[i][j].text} `
    }
    console.log(rowToPrint)
  }
}

function shareClick(evt) {
  const message = generateShareText()
  const onCopy = () => {
    showToast('Copied results to clipboard')
  }

  const onFail = (error) => {
    console.log({ onFailError: error })
    showToast('Share failed')
  }

  share(message, onCopy, onFail, evt)

  try {
    Analytics.share()
  } catch (e) {
    console.log(e)
  }
}

export function addListenersToUIButtons() {
  const submitButton = document.getElementById('submit-button')
  addPressedListeners(submitButton)
  submitButton?.addEventListener('pointerdown', submitAnswer, {
    passive: false,
  })

  const viewResultsButton = document.getElementById('results-button')
  const shareButton = document.getElementById('share-button')
  addPressedListeners(viewResultsButton)
  addPressedListeners(shareButton)

  viewResultsButton?.addEventListener('click', openCongrats, {
    passive: false,
  })
  shareButton?.addEventListener('click', shareClick, {
    passive: false,
  })

  const deselectAllButton = document.getElementById(
    'deselect-all-button',
  )
  addPressedListeners(deselectAllButton)
  deselectAllButton?.addEventListener(
    'pointerdown',
    () => deselectAll(true),
    {
      passive: false,
    },
  )
  deselectAllButton?.addEventListener('click', () => {
    try {
      Analytics.deselect()
    } catch (e) {
      console.log(e)
    }
  })

  const shuffleButton = document.getElementById('shuffle-button')
  addPressedListeners(shuffleButton)
  shuffleButton?.addEventListener('pointerdown', shuffle, {
    passive: false,
  })

  document?.addEventListener('keypress', (event) => {
    if (event.key === 'Enter') {
      submitAnswer()
    }
  })

  const helpButton = document.getElementById('help-button')
  helpButton?.addEventListener('click', () => {
    try {
      Analytics.openHelp()
      Analytics.help()
    } catch (e) {
      console.log(e)
    }
  })

  const nextGameButton = document.getElementById('play-sb')
  nextGameButton?.addEventListener('click', () => {
    try {
      Analytics.nextGame()
    } catch (e) {
      console.log(e)
    }
  })

  const errorFeedbackButton = document.getElementById(
    'error-feedback-button',
  )
  errorFeedbackButton?.addEventListener('click', () => {
    document.location.href =
      'mailto:nytgames@nytimes.com?Subject=Connections%20Feedback'
  })

  const errorBackButton = document.getElementById('error-back-button')
  errorBackButton?.addEventListener('click', () => {
    window.history.back()
  })
}

export function toggleHighContrastMode() {
  const { highContrastEnabled } = getGameState()
  updateGameState('highContrastEnabled', !highContrastEnabled)
  renderSettingToggles(!highContrastEnabled)
}

export function addFeedbackLinks() {
  const bug = document.getElementById('link-feedback-bug')
  const feedback = document.getElementById('link-feedback')

  if (bug) bug.href = getFeedbackLink('Connections', true)

  if (feedback) feedback.href = getFeedbackLink('Connections', false)
}

window.toggleHighContrastMode = toggleHighContrastMode
