/* eslint-disable no-plusplus */
// This file contains variables & functions related to game visuals.
import { addShrinkGrowListeners } from './animations'
import { closeModal, openModal } from './modal'
import {
  currentBoardLayout,
  getGroupNumOfItem,
  getItemFromDiv,
  showAnswerBanners,
} from './gameManager'
import { createEmojiRecordForCongrats } from './social'
import resizeAllItems from './textResizer'
import {
  GROUP_SIZE,
  STARTING_NUM_OF_ATTEMPTS,
  titlesByMistakesMade,
} from './constants'
import { getGameState, updateGameState } from './game-state'
import { createCountdownTimer } from './countdown'
import Analytics from './analytics'

const pulsingItems = []

export function setSubmitButtonActive(bool) {
  const submitButton = document.getElementById('submit-button')
  if (bool) {
    submitButton.classList.remove('inactive')
  } else {
    submitButton.classList.add('inactive')
  }
}

function userPressed(event) {
  const { attemptsRemaining } = getGameState()

  const currentlySelectedItems = Array.from(
    document.querySelectorAll('.selected:not(#connections-nav)'),
  )
  event.preventDefault()

  if (attemptsRemaining > 0) {
    const div = event.target
    const item = getItemFromDiv(div)
    if (div.classList.contains('selected')) {
      div.classList.remove('selected')
      item.selected = false
      const index = currentlySelectedItems.indexOf(item)
      currentlySelectedItems.splice(index, 1)
      updateGameState('savedBoard', currentBoardLayout)
    } else if (
      !div.classList.contains('correct') &&
      currentlySelectedItems.length < GROUP_SIZE
    ) {
      div.classList.add('selected')
      item.selected = true
      currentlySelectedItems.push(item)
      updateGameState('savedBoard', currentBoardLayout)
    }

    if (
      currentlySelectedItems.length === GROUP_SIZE &&
      attemptsRemaining > 0
    ) {
      setSubmitButtonActive(true)
    } else {
      setSubmitButtonActive(false)
    }
  }
}

export function addListenersToItemButtons() {
  const itemDivs = document.getElementsByClassName('item')

  for (let i = 0; i < itemDivs.length; i++) {
    itemDivs[i].addEventListener('pointerdown', userPressed, {
      passive: false,
    })
    addShrinkGrowListeners(itemDivs[i])
  }
}

function removePulseAnimAfterPulseFinished() {
  for (let n = 0; n < pulsingItems.length; n++) {
    const itemDiv = document.getElementById(`item-${pulsingItems[n]}`)
    itemDiv.classList.remove('solved-pulse')
  }

  while (pulsingItems.length > 0) {
    pulsingItems.pop()
  }
}

// render based on currentBoardLayout
export function renderBoard(groupsFound) {
  updateGameState('savedBoard', currentBoardLayout)
  const board = document.getElementById('board')
  board.innerHTML = ''

  for (let i = 0; i < currentBoardLayout.length; i++) {
    const row = document.createElement('div')
    row.setAttribute('id', `row-${i}`)
    for (let j = 0; j < currentBoardLayout[i].length; j++) {
      const item = currentBoardLayout[i][j]
      const itemDiv = document.createElement('div')
      itemDiv.classList.add('item')

      itemDiv.classList.add(`item-row-${i}`)
      itemDiv.classList.add(`item-col-${j}`)

      itemDiv.setAttribute('id', `item-${item.id.toString()}`)

      itemDiv.innerHTML = currentBoardLayout[i][j].text
      row.appendChild(itemDiv)

      if (item.selected) {
        itemDiv.classList.add('selected')
      }

      if (item.solved) {
        itemDiv.classList.add('correct')
      }

      if (pulsingItems.includes(item)) {
        itemDiv.classList.add('solved-pulse')

        const whichGroup = getGroupNumOfItem(item)
        itemDiv.classList.add(`group-${whichGroup}`)
        setTimeout(removePulseAnimAfterPulseFinished, 500)
      }
    }
    board.appendChild(row)
  }

  addListenersToItemButtons()
  resizeAllItems()

  showAnswerBanners(groupsFound)
}

function hideViewResultsButton() {
  const viewResultsContainer =
    document.getElementById('results-buttons')
  viewResultsContainer.classList.remove('active')
}

export function showViewResultsButton() {
  const viewResultsContainer =
    document.getElementById('results-buttons')
  viewResultsContainer.classList.add('active')
}

function showAttemptsRemaining() {
  const attemptsText = document.getElementById(
    'attempts-remaining-text',
  )
  attemptsText.style.display = 'flex'
}

export function showGameButtons() {
  showAttemptsRemaining()
  document.getElementById('tool-buttons').classList.remove('hide')
  const tools = Array.from(
    document.getElementById('tool-buttons').children,
  )

  const attempts = document.getElementById(
    'text-label-attempts-remaining',
  )

  tools.forEach((t) => t.classList.remove('inactive'))
  attempts.classList.remove('inactive')

  const currentlySelectedItems = Array.from(
    document.querySelectorAll('.selected:not(#connections-nav)'),
  )

  if (currentlySelectedItems.length !== 4) {
    setSubmitButtonActive(false)
  }
}

function renderAttemptBubbles() {
  const container = document.getElementById(
    'attempts-remaining-bubbles',
  )
  container.innerHTML = ''
  for (let i = 0; i < STARTING_NUM_OF_ATTEMPTS; i++) {
    const bubble = document.createElement('span')
    bubble.id = `bubble-${i}`
    bubble.classList.add('bubble')
    container.appendChild(bubble)
  }
}

// called at start of new puzz to reset the board, UI elements
export function resetGUI() {
  hideViewResultsButton()
  showGameButtons()
  renderAttemptBubbles()
}

// Adds text to a toast; only shows for 3 seconds
export function showToast(msg, persist = false) {
  const toast = document.getElementById('toast')
  const toastContents = document.getElementById('toast-contents')

  toast.className = 'show'
  toastContents.innerText = msg

  if (!persist) {
    setTimeout(() => {
      toast.className = toast.className.replace('show', '')
    }, 3000)
  }
}

// Custom openModal to show/hide color descriptions
export function openModalConnections(str) {
  const { playedBefore } = getGameState()
  if (playedBefore) {
    document
      .getElementById('color-descriptions')
      .classList.remove('hide')
  }

  return openModal(str)
}

export function hideAttemptsRemaining() {
  const attemptsText = document.getElementById(
    'attempts-remaining-text',
  )
  attemptsText.style.display = 'none'
}

export function hideGameButtons() {
  hideAttemptsRemaining()
  document.getElementById('tool-buttons').classList.add('hide')
}

export function openCongrats() {
  const { attemptsRemaining, dayOfTest } = getGameState()
  const mistakesMade = STARTING_NUM_OF_ATTEMPTS - attemptsRemaining

  openModalConnections('congrats')

  try {
    Analytics.congrats(mistakesMade)
  } catch (e) {
    console.log(e)
  }

  document.getElementById(
    'congrats-title',
  ).innerHTML = `${titlesByMistakesMade[mistakesMade]}!`

  document.getElementById('puzzle-title').innerHTML = `Connections #${
    parseInt(dayOfTest) + 1
  }`
  createEmojiRecordForCongrats()

  // only add a timer if there isn't one already
  if (!document.getElementById('timer')) {
    const countdown = createCountdownTimer()
    const countdownContainer = document.getElementById(
      'countdown-container',
    )
    countdownContainer.appendChild(countdown)
  }

  hideGameButtons()
  showViewResultsButton()
}

export function grayOutGameButtons() {
  const tools = Array.from(
    document.getElementById('tool-buttons').children,
  )

  const attempts = document.getElementById(
    'text-label-attempts-remaining',
  )

  tools.forEach((t) => t.classList.add('inactive'))
  attempts.classList.add('inactive')
}

export function removeShakeAnimAfterShakeFinished(
  itemsToRemoveAnimFor,
) {
  itemsToRemoveAnimFor.forEach((item) => {
    const itemDiv = document.getElementById(`item-${item.id}`)
    itemDiv.classList.remove('invalid-shake')
    itemDiv.classList.remove('inactive')
  })
}

function removeShuffleAnimAfterShuffleFinished(itemsToRemoveAnimFor) {
  itemsToRemoveAnimFor.forEach((itemDiv) => {
    itemDiv.classList.remove('fade')
    itemDiv.classList.remove('fadewhite')
  })
}

export function deselectAll(removeStyling = false) {
  const currentlySelectedItems = Array.from(
    document.querySelectorAll('.selected:not(#connections-nav)'),
  )

  for (let i = 0; i < currentlySelectedItems.length; i++) {
    const itemDiv = currentlySelectedItems[i]
    if (removeStyling) itemDiv.classList.remove('selected')
    getItemFromDiv(itemDiv).selected = false
  }

  // update "selected" prop in local storage
  const { savedBoard } = getGameState()
  if (savedBoard) {
    savedBoard.forEach((row) => {
      row.forEach((item) => {
        item.selected = false
      })
    })

    updateGameState('savedBoard', savedBoard)
  }

  setSubmitButtonActive(false)
}

export function shuffle() {
  try {
    Analytics.shuffle()
  } catch (e) {
    console.log(e)
  }

  window.requestAnimationFrame(() => {
    const items = Array.from(document.getElementsByClassName('item'))
    items.forEach((el) => {
      if (el.classList.contains('selected')) {
        el.classList.add('fadewhite')
      } else {
        el.classList.add('fade')
      }
    })
    setTimeout(() => {
      removeShuffleAnimAfterShuffleFinished(items)
    }, 200)
  })

  const filteredLayout = currentBoardLayout.filter(
    (row) => !row[0].solved,
  )

  for (let i = filteredLayout.length - 1; i >= 0; i--) {
    for (let j = filteredLayout[i].length - 1; j >= 0; j--) {
      const m = Math.floor(filteredLayout.length * Math.random(i + 1))
      const n = Math.floor(
        filteredLayout[i].length * Math.random(j + 1),
      )

      const temp = filteredLayout[i][j]
      filteredLayout[i][j] = filteredLayout[m][n]
      filteredLayout[m][n] = temp
    }
  }
  const { groupsFound } = getGameState()
  renderBoard(groupsFound)
}

export function addModalEventListeners() {
  const modalNames = ['congrats', 'help', 'settings']

  // Remove default shared event listener
  const closeButtons = document.querySelectorAll('.close-x')
  closeButtons.forEach((btn) => {
    const clone = btn.cloneNode(true)
    btn.parentNode.replaceChild(clone, btn)
  })

  modalNames.forEach((name) => {
    const modalID = `${name}-modal`

    const modal = document.getElementById(modalID)

    if (modal) {
      modal.addEventListener('click', (e) => {
        if (
          !e.target.id.includes('close') &&
          e.target.id !== modalID
        ) {
          return
        }

        const modalContent = modal.firstElementChild
        modalContent.classList.add('closing')
        setTimeout(() => {
          modalContent.classList.remove('closing')
          if (name === 'congrats') {
            closeModal('congrats')
            hideGameButtons()
          } else {
            closeModal(name)
            if (name === 'help') {
              try {
                Analytics.closeHelp()
              } catch (err) {
                console.log(err)
              }
            }
          }
        }, 190)
      })
    }
  })
}

function setRootProperties(enabled) {
  const root = document.querySelector(':root')
  const rootStyles = getComputedStyle(root)
  for (let i = 0; i < 4; i++) {
    // eslint-disable-line no-plusplus
    const colors = ['yellow', 'green', 'blue', 'purple']
    root.style.setProperty(
      `--group-${i}-color`,
      rootStyles.getPropertyValue(
        `${enabled ? '--contrast-' : '--'}${colors[i]}`,
      ),
    )
  }
}

export function renderSettingToggles(enabled) {
  const highContrastToggle = document.getElementById(
    'high-contrast-toggle',
  )
  const settingToggle = Array.from(highContrastToggle.children)[0]
  const settingToggleKnob = Array.from(settingToggle.children)[0]

  window.requestAnimationFrame(() => {
    if (enabled) {
      settingToggle.classList.add('toggled')
      settingToggleKnob.classList.add('toggled')
    } else {
      settingToggle.classList.remove('toggled')
      settingToggleKnob.classList.remove('toggled')
    }
    setRootProperties(enabled)
  })
}

export function updateLoadingFill() {
  let value = 25
  setInterval(function interval() {
    if (value < 100) {
      value++
      document.querySelector(
        '#loading-bar-fill',
      ).style.width = `${value}%`
    }
  }, 10)
}
