import React, { useEffect, useState } from 'react'
import propTypes from 'prop-types'
import { compose } from 'redux'
import { withRouter } from 'react-router-dom'

import { withUser } from 'user'

import tabs from '../config'
import ModeToggle, {
  GRID_MODE,
} from '../../shared/ModeToggle/ModeToggle'
import DateSelector from '../../shared/DateSelector/DateSelector'
import TabGroup from '../../../shared/hub/shared/TabGroup/TabGroup'
import TabContents from '../../../shared/hub/shared/TabContents/TabContents'
import useLocalStore from '../../shared/hooks/use-local-store'
import { MODE_KEY, SORT_KEY } from '../../shared/LocalStore'

import View from './View'
import { SORT_DESC } from './ListView'
import Upsell from './Upsell'
import withTodaysDate from './withTodaysDate'
import withStatsAndStreaks from './withStatsAndStreaks'

export const formatDate = (todaysDate, params) => {
  const year =
    typeof params.year === 'undefined'
      ? todaysDate.year()
      : parseInt(params.year, 10)

  const month =
    typeof params.month === 'undefined'
      ? todaysDate.month()
      : parseInt(params.month, 10) - 1

  return { month, year }
}

/**
 * The Archive Viewer
 * needs to track and manipulate
 * the following from the url & local storage:
 *   1) type: what kind of puzzle are we viewing?
 *   2) mode: are we viewing as a grid or list?
 *   3) sort: how are we sorting the list view? (only applies to list view)
 *   4) year: what year are we viewing?
 *   5) month: what month are we viewing (jan if the type doesnt require month granularity)?
 */
export const ArchiveViewer = ({
  match: { params },
  streakDates,
  todaysDate,
  user: { userType, hasLoaded } = { userType: {}, hasLoaded: false },
}) => {
  // Match the current params to a tab, and use its index
  // If there isn't a match or no params are present, default to the first tab
  const tabIndex = tabs.findIndex((tab) => params.type === tab.param)
  const initialTab = tabIndex > 0 ? tabIndex : 0

  // Filter puzzles by date pulled from URL params or todays date (from server)
  const { month, year } = formatDate(todaysDate, params)

  // State
  const [currentTabIndex, setCurrentTabIndex] = useState(initialTab)
  const [mode, setMode] = useLocalStore(MODE_KEY, GRID_MODE)
  const [sort, setSort] = useLocalStore(SORT_KEY, SORT_DESC)

  // @TODO: Should this live here or in ModeToggle? Should it update when hasXwd changes?
  useEffect(() => {
    if (!userType.hasXwd) {
      setMode(GRID_MODE)
    }
  }, [])

  const currentTab = tabs[currentTabIndex]
  const param = !userType.hasXwd ? 'daily' : params.type || 'daily'
  const selectedMonth = currentTab.hasMonths ? month : undefined

  return (
    <div className="archive_viewer">
      <ModeToggle mode={mode} setMode={setMode} />

      <TabGroup
        param={param}
        initialTab={currentTabIndex}
        tabs={tabs}
        onChange={setCurrentTabIndex}
      />
      <TabContents title={currentTab.key}>
        <DateSelector
          earliestDate={currentTab.earliestDate}
          month={selectedMonth}
          todaysDate={todaysDate}
          year={year}
        />
        <div className="archive_viewer-content">
          <View
            streakDates={streakDates}
            mode={mode}
            type={param}
            year={year}
            month={selectedMonth}
            setSort={setSort}
            sort={sort}
          />
        </div>
      </TabContents>
      {hasLoaded && !userType.hasXwd && <Upsell />}
    </div>
  )
}

ArchiveViewer.propTypes = {
  match: propTypes.shape({
    params: propTypes.shape({
      month: propTypes.string,
      type: propTypes.string,
      year: propTypes.string,
    }),
  }),
  streakDates: propTypes.arrayOf(propTypes.arrayOf(propTypes.string)),
  todaysDate: propTypes.object,
  user: propTypes.object,
}

export default compose(
  withRouter,
  withUser,
  withTodaysDate,
  withStatsAndStreaks,
)(ArchiveViewer)
