import React from 'react'
import moment from 'moment'
import { shallow } from 'enzyme'

import DateSelector from '../../shared/DateSelector/DateSelector'
import ModeToggle, {
  GRID_MODE,
} from '../../shared/ModeToggle/ModeToggle'
import TabContents from '../../../shared/hub/shared/TabContents/TabContents'
import TabGroup from '../../../shared/hub/shared/TabGroup/TabGroup'

import { ArchiveViewer, formatDate } from './ArchiveViewer'

jest.mock('foundation', () => ({}))

jest.mock('user', () => ({
  withUser: jest.fn((Component) => (props) => <Component />),
}))

describe('COMPONENTS: ARCHIVE VIEWER', () => {
  const defaultProps = {
    match: { params: { type: 'daily' } },
    todaysDate: moment('2021-04-01'),
    user: { userType: { regiId: 123, hasXwd: true } },
  }

  test('builds mode toggle', () => {
    const viewer = shallow(<ArchiveViewer {...defaultProps} />)
    expect(viewer.find(ModeToggle)).toHaveLength(1)
  })

  test('builds date selectors', () => {
    const viewer = shallow(<ArchiveViewer {...defaultProps} />)
    expect(viewer.find(DateSelector)).toHaveLength(1)
  })

  test('builds tabs', () => {
    const viewer = shallow(<ArchiveViewer {...defaultProps} />)
    const tabContents = viewer.find(TabContents)

    /** renders tabs */
    expect(viewer.find(TabGroup)).toHaveLength(1)
    expect(tabContents).toHaveLength(1)

    /** renders children */
    expect(tabContents.children().find(DateSelector)).toHaveLength(1)
    expect(tabContents.find('.archive_viewer-content')).toHaveLength(
      1,
    )
  })

  test('default mode', () => {
    const viewer = shallow(<ArchiveViewer {...defaultProps} />)

    // Setting state and local storage is async - wait for mode to set, then check it
    setTimeout(() => {
      expect(viewer.find(ModeToggle).props().mode).toEqual(GRID_MODE)
    }, 5000)
  })

  test('defaults to daily puzzles', () => {
    const viewer = shallow(
      <ArchiveViewer {...defaultProps} match={{ params: {} }} />,
    )
    expect(viewer.find(TabContents).props().title).toEqual('daily')
  })

  test('if the current tab has months, display the month', () => {
    const viewer = shallow(
      <ArchiveViewer
        {...defaultProps}
        match={{ params: { type: 'daily', month: '07' } }}
      />,
    )

    expect(viewer.find(DateSelector).props().month).toEqual(6)
  })

  test('if the current tab does not have months, skip it', () => {
    const viewer = shallow(
      <ArchiveViewer
        {...defaultProps}
        match={{ params: { type: 'bonus' } }}
      />,
    )
    expect(viewer.find(DateSelector).props().month).toEqual(undefined)
  })

  describe('UTILS: FORMATDATE', () => {
    const now = moment('2020-05')

    test('use the year from url params when provided', () => {
      const res = formatDate(now, { month: undefined, year: '2012' })
      expect(res.year).toEqual(2012)
    })

    test('use the month from url params when provided', () => {
      const res = formatDate(now, { month: '07', year: undefined })
      expect(res.month).toEqual(6)
    })

    test("if we don't have a year from the url, use current year", () => {
      const res = formatDate(now, {
        month: undefined,
        year: undefined,
      })
      expect(res.year).toEqual(2020)
    })

    test("if we don't have a month from the url, use current month", () => {
      const res = formatDate(now, {
        month: undefined,
        year: undefined,
      })
      expect(res.month).toEqual(4)
    })
  })
})
