import React from 'react'
import propTypes from 'prop-types'
import moment from 'moment'
import cn from 'classnames'

import { useUser } from 'user'

import { crosswordPuzzlePropTypes } from '../../../shared/sharedPropTypes'
import { getStreakLines } from '../../../shared/hub/utils/time'
import PuzzleCard from '../../../shared/hub/PuzzleCard/PuzzleCard'

export const formatPuzzleData = ({
  date,
  month,
  puzzles = [],
  streakDates,
  type,
  userType,
  year,
}) => {
  const puzzleCards = []
  const today = moment().format('MM-DD-YYYY')
  let brandNewCard

  // Create an easily searchable dictionary of existing puzzles
  // Skip this step for any unsubscribed/logged out users
  // (we use dummy puzzle data in those cases)
  const puzzleMap = userType.hasXwd
    ? puzzles.reduce((map, puzz) => {
        const puzzDate = moment(puzz.printDate)
        const puzzKey = puzzDate.date()

        if (puzzDate.format('MM-DD-YYYY') === today) {
          brandNewCard = puzzKey
        }

        return {
          ...map,
          ...{ [puzzKey]: puzz },
        }
      }, {})
    : {}

  // Iterate over each day in the month
  for (let day = 1; day <= date.daysInMonth(); day += 1) {
    // If a user is logged out/unsubscribed or if there simply is no puzzle, this will be false
    const puzzleExistsOnDay = !!puzzleMap[day]

    // Check for streak info
    const streaksObj =
      puzzleExistsOnDay && type === 'daily'
        ? getStreakLines(streakDates, puzzleMap[day].printDate)
        : {}

    /*
     * If user has crossword sub and there is a puzzle for this day, use puzzle data
     * Otherwise, display dummy puzzle data based on the date
     * If a puzzle does not exist for this day, then display fake card
     * If user does not have crossword sub, display a static puzzle card
     */
    puzzleCards.push(
      <div key={day} className="archive_calendar-item">
        <PuzzleCard
          isBrandNew={brandNewCard === day}
          isFake={!puzzleExistsOnDay}
          isStatic={!userType.hasXwd}
          puzzle={
            puzzleExistsOnDay
              ? puzzleMap[day]
              : {
                  publishType: type === 'mini' ? 'Mini' : 'Daily',
                  printDate: moment({
                    year,
                    month,
                    date: day,
                  }).format('YYYY-MM-DD'),
                }
          }
          type="Calendar"
          user={userType}
          hasStreakBefore={streaksObj.hasStreakBefore}
          hasStreakAfter={streaksObj.hasStreakAfter}
        />
      </div>,
    )
  }

  return puzzleCards
}

const CalendarView = ({
  puzzles,
  month,
  year,
  type,
  streakDates = [],
}) => {
  const { userType } = useUser()

  const date = moment({ year, month })
  const spacerWidth = `${date.day() * (100 / 7)}%`

  const puzzleCards = formatPuzzleData({
    date,
    month,
    puzzles,
    streakDates,
    type,
    userType,
    year,
  })

  return (
    <div>
      <div className="archive_calendar-header">
        <div className="archive_calendar-body">
          <div className="archive_calendar-item">S</div>
          <div className="archive_calendar-item">M</div>
          <div className="archive_calendar-item">T</div>
          <div className="archive_calendar-item">W</div>
          <div className="archive_calendar-item">T</div>
          <div className="archive_calendar-item">F</div>
          <div className="archive_calendar-item">S</div>
        </div>
      </div>
      <div
        className={cn(
          'archive_calendar-body',
          'archive_viewer-content',
          { isLoading: !puzzles },
        )}
      >
        <div
          className={cn(
            'archive_calendar-item',
            'archive_calendar-spacer',
          )}
          style={{ width: spacerWidth }}
        />
        {puzzleCards}
      </div>
    </div>
  )
}

CalendarView.propTypes = {
  puzzles: propTypes.arrayOf(crosswordPuzzlePropTypes),
  month: propTypes.number,
  year: propTypes.number,
  type: propTypes.string,
  streakDates: propTypes.arrayOf(propTypes.arrayOf(propTypes.string)),
}

export default CalendarView
