import React from 'react'
import { mount, shallow } from 'enzyme'
import moment from 'moment'
import { createStore } from 'redux'
import { Provider } from 'react-redux'

import PuzzleCard from '../../../shared/hub/PuzzleCard/PuzzleCard'

import CalendarView, { formatPuzzleData } from './CalendarView'

jest.mock('foundation', () => ({}))

jest.mock('user', () => ({
  useUser: jest
    .fn()
    .mockReturnValue({ userType: { regiId: 123, hasXwd: true } }),
}))

jest.mock('../../../shared/hub/utils/time', () => ({
  ...jest.requireActual('../../../shared/hub/utils/time'),
  getStreakLines: () => ({
    hasStreakBefore: '01-01-04',
    hasStreakAfter: '01-01-01',
  }),
}))

describe('ARCHIVE VIEWER / CALENDAR VIEW', () => {
  const now = moment()

  const defaultProps = {
    month: now.month(),
    puzzles: [],
    streakDates: [],
    type: '',
    year: now.year(),
  }

  test('renders a brand new card for today only', () => {
    const wrapper = shallow(
      <CalendarView
        {...defaultProps}
        puzzles={[
          { printDate: now.format('YYYY-MM-DD'), title: 'a' },
        ]}
      />,
    )

    let brandNewCards = 0

    wrapper.find(PuzzleCard).forEach((card) => {
      if (card.props().isBrandNew) {
        brandNewCards += 1
      }
    })

    expect(brandNewCards).toEqual(1)
  })

  test('renders the correct spacer and days for a given month', () => {
    const leapYearFebruary = moment({ year: 2011, month: 1 })
    const view = shallow(
      <CalendarView
        {...defaultProps}
        type="daily"
        month={leapYearFebruary.month()}
        year={leapYearFebruary.year()}
      />,
    )

    const spacer = view.find(
      '.archive_calendar-body .archive_calendar-spacer',
    )
    expect(parseInt(spacer.props().style.width, 10)).toBe(
      Math.floor((2 / 7) * 100),
    )
    expect(leapYearFebruary.format('dddd')).toBe('Tuesday')
  })

  describe('Util: formatPuzzleData', () => {
    const defaultpuzzleData = {
      date: moment().month('January'),
      month: 0,
      puzzles: [],
      streakDates: [],
      type: '',
      userType: { regiId: 123, hasXwd: true },
      year: 2021,
    }

    test('a puzzle card is returned for each day of the month', () => {
      const puzzleData = formatPuzzleData(defaultpuzzleData)
      expect(puzzleData).toHaveLength(31)
    })

    test('puzzle cards all have the expected shared properties', () => {
      const puzzleData = formatPuzzleData(defaultpuzzleData)

      const matchingProperties = puzzleData.every((card) => {
        const classname = card.props.className
        const PuzzleCardType = card.props.children.props.type

        return (
          PuzzleCardType === 'Calendar' &&
          classname === 'archive_calendar-item'
        )
      })

      expect(matchingProperties).toBeTruthy()
    })

    test('a fake puzzle card is generated for subscribed users when there is no puzzle available', () => {
      const puzzleData = formatPuzzleData(defaultpuzzleData)
      const fakePuzzles = puzzleData.every(
        (card) => card.props.children.props.isFake,
      )

      expect(fakePuzzles).toBeTruthy()
    })

    test('a static puzzle card is generated for unsubscribed users', () => {
      const puzzleData = formatPuzzleData({
        ...defaultpuzzleData,
        userType: { regiId: 123, hasXwd: false },
      })

      const staticPuzzles = puzzleData.every(
        (card) => card.props.children.props.isStatic,
      )

      expect(staticPuzzles).toBeTruthy()
    })

    test('streak info is set for existing daily puzzles', () => {
      const puzzleData = formatPuzzleData({
        ...defaultpuzzleData,
        puzzles: [
          { printDate: '2020-01-01' },
          { printDate: '2020-01-02' },
          { printDate: '2020-01-03' },
          { printDate: '2020-01-04' },
        ],
        type: 'daily',
      })

      const januaryThird = puzzleData.find((card) => card.key === '2')
      const PuzzleCardProps = januaryThird.props.children.props

      expect(PuzzleCardProps.hasStreakBefore).toEqual('01-01-04')
      expect(PuzzleCardProps.hasStreakAfter).toEqual('01-01-01')
    })
  })

  test('sends subscribed user to the correct puzzle', () => {
    const view = mount(
      <Provider store={createStore(() => {})}>
        <CalendarView
          puzzles={[
            {
              formatType: 'Normal',
              percentFilled: 0,
              printDate: '2021-06-18',
              publishType: 'Daily',
              puzzleId: 19306,
            },
          ]}
        />
      </Provider>,
    )

    // Find the puzzle card we've mocked
    // (each month renders a zero-indexed list of puzzles - one for each calendar day),
    // and use the tracking context and link to make sure the card is formatted correctly
    const card = view.find(PuzzleCard).at(17)
    const cardTracking = card.find('.js-hub-tracker').props()
    const cardLink = card.find('.puzzleAction').props().href

    expect(cardTracking['data-track-hub-context']).toEqual(19306)
    expect(cardLink).toEqual('/crosswords/game/daily/2021/06/18')
  })
})
