import React from 'react'
import { shallow } from 'enzyme'

import ListItem from './ListItem'
import ListView, { SORT_ASC, SORT_DESC } from './ListView'

jest.mock('foundation', () => ({}))

jest.mock('../../../shared/hub/PuzzleCard/PuzzleCard', () => () => (
  <div />
))

const HEADER_CONTAINER_CLASS = 'archive_list-view-header'
const DATE_SORT_TOGGLE_CLASS = 'archive_date-sort-toggle'

describe('ARCHIVE VIEWER / LIST VIEW', () => {
  const getPrintDates = (v) =>
    v.find(ListItem).map((card) => card.props().puzzle.printDate)

  let defaultPuzzles

  beforeEach(() => {
    defaultPuzzles = [
      {
        puzzleId: '19340',
        author: 'Napoleon Dynamite',
        formatType: 'Normal',
        printDate: '2021-06-24',
        publishType: 'Daily',
      },
      { printDate: '2017-06-01', author: 'Pedro' },
      { printDate: '2017-06-02', author: 'Deb' },
      { printDate: '2017-06-05', author: 'Uncle Rico' },
    ]
  })

  test('renders list cards', () => {
    const view = shallow(<ListView puzzles={defaultPuzzles} />)
    expect(view.find(ListItem).length).toBe(4)
  })

  describe('daily puzzles', () => {
    test('renders correct headers for daily puzzles', () => {
      const view = shallow(
        <ListView type="daily" puzzles={defaultPuzzles} />,
      )
      const header = view.find(`.${HEADER_CONTAINER_CLASS}`)
      const headerText = header.text().toUpperCase()
      expect(headerText).toContain('DAY OF WEEK')
      expect(headerText).toContain('CONSTRUCTOR')
    })

    // the tests below are written with the intention of
    // integration testing
    test('renders a link to a pdf of the puzzle', () => {
      // render the ListView
      const view = shallow(
        <ListView type="daily" puzzles={defaultPuzzles} />,
      )

      // the correct pdf link is rendered in the child component
      expect(
        view
          .find(ListItem)
          .first()
          .dive()
          .find('.archive-puzzle-pdf a')
          .prop('href'),
      ).toEqual('/svc/crosswords/v2/puzzle/print/Jun2421.pdf')
    })

    test('renders a link to a pdf with the solutions for the puzzle', () => {
      // render the ListView
      const view = shallow(
        <ListView type="daily" puzzles={defaultPuzzles} />,
      )

      // the correct solutions pdf link is rendered in the child component
      expect(
        view
          .find(ListItem)
          .first()
          .dive()
          .find('.archive-solutions-pdf a')
          .prop('href'),
      ).toEqual('/svc/crosswords/v2/puzzle/19340.ans.pdf')
    })
  })

  test('renders correct headers for mini puzzles', () => {
    const view = shallow(
      <ListView type="mini" puzzles={defaultPuzzles} />,
    )
    const header = view.find(`.${HEADER_CONTAINER_CLASS}`)
    const headerText = header.text().toUpperCase()
    expect(headerText).toContain('DAY OF WEEK')
    expect(headerText).toContain('CONSTRUCTOR')
  })

  test('renders correct headers for bonus puzzles', () => {
    const view = shallow(
      <ListView type="bonus" puzzles={defaultPuzzles} />,
    )
    const header = view.find(`.${HEADER_CONTAINER_CLASS}`)
    const headerText = header.text().toUpperCase()
    expect(headerText).toContain('TITLE')
    expect(headerText).toContain('DATE')
    expect(headerText).toContain('CONSTRUCTOR')
  })

  test('allows user to toggle date sort', () => {
    const sortedDates = [
      '2017-06-01',
      '2017-06-02',
      '2017-06-05',
      '2021-06-24',
    ]

    const setSort = jest.fn()
    const view = shallow(
      <ListView
        type="bonus"
        puzzles={defaultPuzzles}
        setSort={setSort}
        sort={SORT_ASC}
      />,
    )
    const toggle = view.find(`.${DATE_SORT_TOGGLE_CLASS}`)

    expect(getPrintDates(view)).toEqual(sortedDates)

    toggle.simulate('click')
    expect(setSort).toHaveBeenCalledWith(SORT_DESC)

    view.setProps({ sort: SORT_DESC })

    expect(getPrintDates(view)).toEqual(sortedDates.slice().reverse())
  })
})
