import React, { Component } from 'react'
import PT from 'prop-types'

import logger from '../../shared/utils/clientLogger'
import makeCancelable from '../../shared/utils/makeCancelable'

import fetchArchivePuzzles from './fetchArchivePuzzles'

let localCache = {}

/** mostly for testing */
export const clearCache = () => {
  localCache = {}
}

const getLocalKey = ({ type, year, month }) => {
  if (typeof month === 'undefined') return [type, year].join('-')

  return [type, year, month].join('-')
}

const withArchiveData = (ArchiveComponent) => {
  class ArchiveData extends Component {
    constructor(props) {
      super(props)

      const initialKey = getLocalKey(this.props)

      this.state = {
        key: initialKey,
        data: localCache[initialKey],
      }
    }

    componentWillUnmount = this.cleanup

    componentDidMount() {
      const { data, key } = this.state
      if (data) return
      this.fetchData(key)
    }

    // eslint-disable-next-line
    componentWillReceiveProps(nextProps) {
      const { key } = this.state
      const nextKey = getLocalKey(nextProps)
      if (nextKey !== key) {
        this.cleanup()
        this.setState({ key: nextKey })
      }
    }

    componentDidUpdate(prevProps, prevState) {
      const {
        user: { userType },
      } = this.props
      const { key } = this.state
      const userHasChanged =
        prevProps.user.userType.regiId !== userType.regiId

      if (!userHasChanged && key === prevState.key) return

      if (localCache[key]) {
        this.setData(localCache[key])
      } else {
        this.fetchData(key)
      }
    }

    setData(data) {
      this.setState({ data })
    }

    fetchData(key) {
      const {
        fetchArchivePuzzles,
        user: { userType },
      } = this.props
      const [type, year, month] = key.split('-')

      this.setData(undefined)

      /**
       * We make the promise cancelable
       * so that we can cancel it if the component unmounts
       * before the data cames back
       * see more here: https://facebook.github.io/react/blog/2015/12/16/ismounted-antipattern.html
       */
      this.cancelablePromise = makeCancelable(
        fetchArchivePuzzles(userType, type, year, month),
      )

      this.cancelablePromise.promise
        .then((data) => {
          localCache[key] = data
          this.setData(data)
          this.cancelablePromise = undefined
        })
        .catch((err) => {
          if (err instanceof Error) logger.error(err)
        })
    }

    cleanup() {
      if (this.cancelablePromise) this.cancelablePromise.cancel()
    }

    render() {
      const { data } = this.state

      return <ArchiveComponent puzzles={data} {...this.props} />
    }
  }

  ArchiveData.propTypes = {
    // TODO: figure out what happens when user does not exist
    // eslint-disable-next-line
    user: PT.shape({ userType: PT.shape({ regiId: PT.string }) }),
    fetchArchivePuzzles: PT.func,
  }

  ArchiveData.defaultProps = {
    fetchArchivePuzzles,
  }

  return ArchiveData
}

export default withArchiveData
