import React from 'react'
import { mount } from 'enzyme'

import logger from '../../shared/utils/clientLogger'
import makeCancelable from '../../shared/utils/makeCancelable'

import withArchiveData, { clearCache } from './withArchiveData'
import fetchArchivePuzzles from './fetchArchivePuzzles'

jest.mock('foundation', () => ({}))

jest.mock('user', () => ({
  withUser: jest.fn((Component) => (props) => <Component />),
}))

jest.mock('./fetchArchivePuzzles', () => jest.fn())
jest.mock('../../shared/utils/makeCancelable', () => jest.fn())
jest.mock('../../shared/utils/clientLogger', () => ({
  error: jest.fn(),
}))

describe('ARCHIVE VIEWER / DATA MGMT', () => {
  let ArchiveComponent
  let WrappedArchiveComponent
  let archive
  let defaultUser
  let defaultProps
  let getView
  let resolver
  let getResolvers
  let catcher
  let getCatchers
  let cancelled

  beforeEach(() => {
    jest.resetAllMocks()
    clearCache()

    cancelled = 0
    makeCancelable.mockImplementation((v) => ({
      promise: v,
      cancel: () =>
        v.catch(() => {
          cancelled += 1
          return { isCanceled: true }
        }),
    }))

    resolver = jest.fn()
    catcher = jest.fn()
    fetchArchivePuzzles.mockImplementation(() => {
      const promise = {}
      promise.catch = (cb) => {
        catcher(cb)
        cb()
      }

      promise.then = (cb) => {
        resolver(cb)
        return promise
      }

      return promise
    })
    getResolvers = () => resolver.mock.calls.map((args) => args[0])
    getCatchers = () => catcher.mock.calls.map((args) => args[0])

    defaultUser = { userType: { regiId: '123' } }

    defaultProps = {
      fetchArchivePuzzles,
      user: defaultUser,
      mode: 'grid',
      type: 'daily',
      year: 2000,
      month: 0,
    }

    ArchiveComponent = () => <div />
    WrappedArchiveComponent = withArchiveData(ArchiveComponent)
    archive = mount(<WrappedArchiveComponent {...defaultProps} />)
    getView = (mountedArchive) => {
      mountedArchive.update()
      return mountedArchive.find(ArchiveComponent).props()
    }
  })

  test('mount', () => {
    const puzzles = [1, 2, 3]

    /** passes props through to view */
    expect(getView(archive)).toEqual({
      ...defaultProps,
      puzzles: undefined,
    })

    /** starts fetching puzzles */
    expect(fetchArchivePuzzles).toHaveBeenCalledWith(
      defaultUser.userType,
      'daily',
      '2000',
      '0',
    )

    /** resolve the initial request */
    getResolvers()[0](puzzles)

    /** first mount resolves */
    expect(getView(archive)).toEqual({ ...defaultProps, puzzles })

    /** assert cache is used for same props */
    const archive2 = mount(
      <WrappedArchiveComponent {...defaultProps} />,
    )
    expect(getView(archive2)).toEqual({ ...defaultProps, puzzles })
    expect(fetchArchivePuzzles).toHaveBeenCalledTimes(1)
  })

  test('update', () => {
    const puzzles = [1, 2, 3]
    const nextPuzzles = [4, 5, 6]

    /** resolve the initial request */
    getResolvers()[0](puzzles)

    /** first mount resolves */
    expect(getView(archive)).toEqual({ ...defaultProps, puzzles })

    archive.setProps({
      mode: 'list',
      type: 'mini',
      year: 2015,
      month: undefined,
    })

    expect(getView(archive)).toEqual({
      mode: 'list',
      type: 'mini',
      year: 2015,
      month: undefined,
      puzzles: undefined,
      user: defaultUser,
      fetchArchivePuzzles,
    })

    /** resolve update */
    getResolvers()[1](nextPuzzles)

    /** component updates */
    expect(getView(archive)).toEqual({
      mode: 'list',
      type: 'mini',
      year: 2015,
      puzzles: nextPuzzles,
      user: defaultUser,
      fetchArchivePuzzles,
    })

    /** asset caching works */

    /** no change should not do anything */
    archive.setProps({
      mode: 'list',
      type: 'mini',
      year: 2015,
    })

    /** switch year to 2000 */
    archive.setProps({
      year: 2000,
    })

    /** resolve year 2000 */
    getResolvers()[2]([7, 8, 9])

    /** switch back to 2015 from cache */
    archive.setProps({
      year: 2015,
    })

    expect(fetchArchivePuzzles).toHaveBeenCalledTimes(3)
    expect(getView(archive)).toEqual({
      mode: 'list',
      type: 'mini',
      year: 2015,
      month: undefined,
      puzzles: nextPuzzles,
      user: defaultUser,
      fetchArchivePuzzles,
    })
  })

  test('cleans up pending call on unmount', () => {
    const attachedArchive = mount(
      <WrappedArchiveComponent {...defaultProps} />,
      {
        attachTo: document.body,
      },
    )

    /** unmount BEFORE api resolves */
    attachedArchive.detach()

    /** check that canceled was called */
    expect(cancelled).toBe(1)
  })

  test('cleans up pending calls on updates', () => {
    archive.setProps({ year: 2015 })
    archive.setProps({ month: 1 })

    /** check that canceled was called for both incomplete promises */
    expect(cancelled).toBe(2)
  })

  test('handles errors from api', () => {
    const err = new Error('bad call')
    getCatchers()[0](err)
    expect(logger.error).toHaveBeenCalledWith(err)
  })
})
