import React from 'react'
import { shallow } from 'enzyme'

import { DimensionsState } from '../../store/state/puzzle/puzzleTypes'

import Cell, { CellProps } from './Cell'

const defaults: CellProps = {
  ariaLabel: '',
  answer: '',
  guess: '',
  type: 'EMPTY',
  isSolved: false,
  skipFocus: false,
  isSelected: false,
  isInSelectedClue: false,
  isPenciled: false,
  isConfirmed: false,
  isChecked: false,
  isModified: false,
  isRevealed: false,
  isRelated: false,
  clickHandler: jest.fn(),
  openRebus: jest.fn(),
  index: 0,
  innards: [],
  background: {
    name: '',
    styles: [],
    children: [],
    attributes: {},
  },
  dimensions: { cellSize: 1, columnCount: 15 } as DimensionsState,
}

const renderCell = (props: Partial<CellProps>) => {
  return shallow(<Cell {...defaults} {...props} />)
}

describe('<Cell />', () => {
  afterEach(jest.clearAllMocks)

  describe('should put correct classnames on svg rect based on props', () => {
    const testCases = [
      {
        fillLogicClassName: 'xwd__cell--selected',
        props: { isSelected: true },
      },
      {
        fillLogicClassName: 'xwd__cell--highlighted',
        props: { isInSelectedClue: true },
      },
      {
        fillLogicClassName: 'xwd__cell--related',
        props: { isRelated: true },
      },
      {
        fillLogicClassName: 'xwd__cell--cell',
        props: {},
      },
      {
        fillLogicClassName: 'xwd__cell--block',
        props: { type: 'BLOCK' },
      },
      {
        fillLogicClassName: 'xwd__cell--shaded',
        props: { type: 'SHADED' },
      },
      {
        fillLogicClassName: 'xwd__cell--penciled',
        props: { isPenciled: true },
      },
      {
        fillLogicClassName: 'xwd__assistance--confirmed',
        props: { isConfirmed: true },
      },
    ]

    testCases.forEach(({ fillLogicClassName, props }) => {
      it(fillLogicClassName, () => {
        const wrapper = renderCell(props)
        const { className } = wrapper.find('rect').props()
        expect(className?.includes(fillLogicClassName)).toBeTruthy()
      })
    })
  })

  it('should include a revealed assist badge if revealed', () => {
    const wrapper = renderCell({ isRevealed: true })
    const useTag = wrapper.find('use')
    expect(useTag).toHaveLength(1)
    expect(useTag.props().xlinkHref).toEqual('#revealed')
  })

  it('should include a checked assist badge if checked', () => {
    const wrapper = renderCell({ isChecked: true })
    const useTag = wrapper.find('use')
    expect(useTag).toHaveLength(1)
    expect(useTag.props().xlinkHref).toEqual('#checked')
  })

  it('should include a modified assist badge if modified', () => {
    const wrapper = renderCell({ isChecked: true, isModified: true })
    const useTag = wrapper.find('use')
    expect(useTag).toHaveLength(1)
    expect(useTag.props().xlinkHref).toEqual('#modified')
  })

  it('should render innards if included', () => {
    const wrapper = renderCell({
      innards: [
        {
          name: 'circle',
          attributes: { id: 'hello' },
          children: [],
          styles: [],
        },
      ],
    })

    expect(wrapper.find('circle').props().id).toEqual('hello')
  })

  it('should handle clicks if isNavigable', () => {
    const wrapper = renderCell({})
    wrapper.find('g').first().simulate('click', {
      preventDefault: jest.fn(),
      button: 2,
      persist: jest.fn(),
    })
    expect(defaults.clickHandler).toHaveBeenCalledTimes(1)
    expect(defaults.openRebus).toHaveBeenCalledTimes(0)
  })

  it('should not handle clicks if not isNavigable', () => {
    const wrapper = renderCell({ type: 'BLOCK' })
    wrapper.find('g').first().simulate('click', {
      preventDefault: jest.fn(),
      button: 0,
      persist: jest.fn(),
    })
    expect(defaults.clickHandler).toHaveBeenCalledTimes(0)
    expect(defaults.openRebus).toHaveBeenCalledTimes(0)
  })
  it('should call openRebus if event button is type 1 (middle)', () => {
    const wrapper = renderCell({})
    wrapper.find('g').first().simulate('click', {
      preventDefault: jest.fn(),
      button: 1,
      persist: jest.fn(),
    })
    expect(defaults.openRebus).toHaveBeenCalled()
  })
})
