import React from 'react'
import classNames from 'classnames'

import { getCellDomId } from '../../utils/misc'
import {
  BoardState,
  DimensionsState,
} from '../../store/state/puzzle/puzzleTypes'

export type CellProps = {
  index: number
  innards: BoardState[]
  background: BoardState
  skipFocus: boolean
  isSolved: boolean
  dimensions: DimensionsState
  answer: string
  guess: string
  type: string
  isPenciled: boolean
  isConfirmed: boolean
  isChecked: boolean
  isModified: boolean
  isRevealed: boolean
  isSelected: boolean
  isInSelectedClue: boolean
  isRelated: boolean
  ariaLabel: string
  clickHandler: () => void
  openRebus: () => void
}

type HandleClickEvent = React.MouseEvent<SVGGElement, MouseEvent> &
  React.KeyboardEvent

class Cell extends React.PureComponent<CellProps> {
  ref: React.RefObject<SVGRectElement>

  value: string

  constructor(props: CellProps) {
    super(props)

    this.ref = React.createRef()
    this.value = ''
  }

  componentDidUpdate() {
    const { isSelected, skipFocus } = this.props

    if (isSelected && this.ref.current && !skipFocus) {
      const isNotFocused = document.activeElement !== this.ref.current
      if (isNotFocused) {
        this.ref.current.focus()
      }
    }
  }

  handleClick = (event: HandleClickEvent) => {
    const { type, clickHandler, openRebus } = this.props
    event?.persist()
    if (type === 'BLOCK' || type === 'EXTERNAL') return
    clickHandler()
    if (event?.button === 1) openRebus()
  }

  renderChild = (child: BoardState, idx: number) => {
    const { isSolved, answer, guess } = this.props
    const { attributes, content, name: Tag } = child

    let fontSize = attributes.fontSize as number

    const value = isSolved ? answer : guess
    const isGuessElement = attributes.textAnchor === 'middle'

    if (isGuessElement && value.length > 1) {
      fontSize /= value.length / 2
    }

    const adjustedAttrs = { ...attributes, fontSize }

    this.value = isGuessElement ? `${value}` : ''

    return (
      // @ts-ignore ts doesn't like using a string from the API as a JSX tagname
      <Tag key={idx} {...adjustedAttrs}>
        <text className="xwd__cell--hidden" aria-live="polite">
          {this.value}
        </text>
        {content || value}
      </Tag>
    )
  }

  renderAssistBadge = () => {
    const {
      isChecked,
      isRevealed,
      isModified,
      isConfirmed,
      dimensions,
      index,
    } = this.props
    const isAssisted = isChecked || isRevealed
    if (!isAssisted) {
      return null
    }

    let assistanceType

    if (isRevealed) {
      assistanceType = 'revealed'
    } else if (isModified || isConfirmed) {
      assistanceType = 'modified'
    } else {
      assistanceType = 'checked'
    }

    const { cellSize, columnCount } = dimensions
    const row = Math.floor(index / columnCount)
    const col = index % columnCount

    return (
      <use
        xlinkHref={`#${assistanceType}`}
        className={`xwd__assistance--${assistanceType}`}
        x={cellSize * col}
        y={cellSize * row}
      />
    )
  }

  render() {
    const {
      ariaLabel,
      isSelected,
      isInSelectedClue,
      isRelated,
      isPenciled,
      isConfirmed,
      background,
      innards,
      type,
      index,
    } = this.props

    const fillLogic = {
      'xwd__cell--selected': isSelected,
      'xwd__cell--highlighted': isInSelectedClue,
      'xwd__cell--related': isRelated,
      'xwd__cell--cell': type !== 'BLOCK',
      'xwd__cell--block': type === 'BLOCK',
      'xwd__cell--shaded': type === 'SHADED',
      'xwd__cell--penciled': isPenciled,
      'xwd__assistance--confirmed': isConfirmed,
    }

    return (
      <g
        className="xwd__cell"
        onClick={this.handleClick}
        onContextMenu={this.handleClick}
      >
        <rect
          role="cell"
          tabIndex={isSelected ? 0 : -1}
          aria-label={ariaLabel}
          ref={this.ref}
          id={getCellDomId(index)}
          className={classNames(fillLogic, 'xwd__cell--nested')}
          {...background.attributes}
        />
        {innards.map(this.renderChild)}
        {this.renderAssistBadge()}
      </g>
    )
  }
}

export default Cell
