import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import Cells from './Cells'

jest.mock('./Cell')

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

describe('Cells', () => {
  const dimensions = {
    cellSize: 10,
    columnCount: 8,
    rowCount: 8,
    aspectRatio: 1,
  }

  const cellBoard = [
    {
      index: 0,
      children: ['background', 'innard1', 'innard2'],
    },
  ]

  const makeStore = ({ clue, notes }) => ({
    getState: () => ({
      cells: [
        {
          answer: 'A',
          guess: 'B',
          type: 0,
          penciled: false,
          checked: false,
          confirmed: false,
          modified: false,
          revealed: false,
        },
      ],
      clues: [
        {
          label: 'a',
          direction: 'b',
          text: 'c',
          alternativeAriaLabelText: 'd',
        },
      ],
      puzzle: {
        data: {
          meta: { notes },
        },
      },
      modal: {},
      toolbar: {
        inRebusMode: false,
      },
      status: {
        isSolved: false,
      },
      selection: {
        clue,
        clueCells: [0],
        relatedCells: [],
        relatedClues: [],
      },
    }),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  })

  const getWrapper = ({ clue = 0, notes = null } = {}) =>
    mount(
      <Provider store={makeStore({ clue, notes })}>
        <Cells
          dimensions={dimensions}
          attributes={{}}
          cellBoard={cellBoard}
        />
      </Provider>,
    )

  it('should pass correct props to Cell - common props', () => {
    const wrapper = getWrapper()
    const props = wrapper.find('Cell').props()
    expect(props.index).toEqual(0)
    expect(props.dimensions).toEqual(dimensions)
    expect(props.background).toEqual('background')
    expect(props.innards).toEqual(['innard1', 'innard2'])
    expect(props.isSolved).toEqual(false)
    expect(props.skipFocus).toEqual(false)
    expect(props.answer).toEqual('A')
    expect(props.guess).toEqual('B')
    expect(props.type).toEqual('BLOCK')
    expect(props.isPenciled).toEqual(false)
    expect(props.isChecked).toEqual(false)
    expect(props.isConfirmed).toEqual(false)
    expect(props.isModified).toEqual(false)
    expect(props.isRevealed).toEqual(false)
  })

  it('should pass correct ariaLabel to Cell - no selected clue, no notes', () => {
    const wrapper = getWrapper({ clue: -1 })
    const props = wrapper.find('Cell').props()
    expect(props.ariaLabel).toEqual(
      'No Clue, Answer: 1 letter, Letter: 1',
    )
  })

  it('should pass correct ariaLabel to Cell - no selected clue, some notes', () => {
    const wrapper = getWrapper({
      clue: -1,
      notes: [{ text: 'NOTES' }],
    })
    const props = wrapper.find('Cell').props()
    expect(props.ariaLabel).toEqual(
      'No Clue, Notes: NOTES, Answer: 1 letter, Letter: 1',
    )
  })

  it('should pass correct ariaLabel to Cell - selected clue', () => {
    const wrapper = getWrapper({ clue: 0 })
    const props = wrapper.find('Cell').props()
    expect(props.ariaLabel).toEqual(
      'ab: d, Answer: 1 letters, Letter: 1',
    )
  })
})
