import React from 'react'
import { useDispatch, useSelector } from 'react-redux'

import { isMobile } from 'responsive'

import { getIsModalOpen } from '../../store/state/modal/modalSelectors'
import { selectCell } from '../../store/state/selection/selectionActions'
import {
  getSelectedCellIndex,
  getSelectedClue,
  getSelectedClueCells,
  getSelectedRelatedCells,
} from '../../store/state/selection/selectionSelectors'
import {
  getAllCells,
  getPuzzleMetadata,
} from '../../store/state/puzzle/puzzleSelectors'
import {
  BoardState,
  DimensionsState,
} from '../../store/state/puzzle/puzzleTypes'
import { getIsSolved } from '../../store/state/status/statusSelectors'
import { getInRebusMode } from '../../store/state/toolbar/toolbarSelectors'
import { toggleRebus } from '../../store/thunks/toolbarThunks'

import Cell from './Cell'

const getType = (code: number) => {
  switch (code) {
    case 1:
      return 'EMPTY'
    case 2:
      return 'CIRCLED'
    case 3:
      return 'SHADED'
    case 4:
      return 'EXTERNAL'
    default:
      return 'BLOCK'
  }
}

const dirs: Record<string, string> = {
  Across: 'A',
  Down: 'D',
  Heart: 'Heart',
  Diagonal: 'DG',
}

type CellsProps = {
  attributes: Record<string, string | number>
  cellBoard: BoardState[]
  dimensions: DimensionsState
}

export default function Cells({
  attributes,
  cellBoard,
  dimensions,
}: CellsProps) {
  const isModalOpen = useSelector(getIsModalOpen)
  const inRebusMode = useSelector(getInRebusMode)
  const isSolved = useSelector(getIsSolved)
  const cells = useSelector(getAllCells)
  const selectedClue = useSelector(getSelectedClue)
  const selectedCellIndex = useSelector(getSelectedCellIndex)
  const selectedClueCells = useSelector(getSelectedClueCells)
  const selectedRelatedCells = useSelector(getSelectedRelatedCells)
  const { notes } = useSelector(getPuzzleMetadata) || {}
  const dispatch = useDispatch()

  const isIOS = /iPad|iPhone/.test(navigator.userAgent)

  const getAriaLabel = React.useCallback(
    (index: number) => {
      if (!selectedClue || !Object.keys(selectedClue).length) {
        return notes && notes.length > 0
          ? `No Clue, Notes: ${notes[0].text}, Answer: 1 letter, Letter: 1`
          : 'No Clue, Answer: 1 letter, Letter: 1'
      }

      const { label, direction, text, alternativeAriaLabelText } =
        selectedClue

      if (label && direction && text) {
        const clueLength = selectedClueCells.length
        const clueIndexPos = selectedClueCells.indexOf(index)

        return (
          `${label}${
            dirs[direction] || direction
          }: ${alternativeAriaLabelText}, ` +
          `Answer: ${clueLength} letters, ` +
          `Letter: ${clueIndexPos + 1}`
        )
      }

      return ''
    },
    [selectedClue, selectedClueCells, notes],
  )

  return (
    <g {...attributes} role="table">
      {cellBoard.map(({ children, index }) => {
        if (typeof index !== 'number') return

        const [background, ...others] = children
        const cell = cells[index]

        const isSelected = index === selectedCellIndex

        const clickHandler = () =>
          dispatch(selectCell(index, isSelected))
        const openRebus = () => dispatch(toggleRebus())

        return (
          <Cell
            key={index}
            index={index}
            dimensions={dimensions}
            background={background}
            innards={others}
            isSolved={isSolved}
            skipFocus={isModalOpen || inRebusMode}
            answer={cell.answer || ''}
            guess={cell.guess || ''}
            type={getType(cell.type)}
            isPenciled={cell.penciled}
            isChecked={cell.checked}
            isConfirmed={cell.confirmed}
            isModified={cell.modified}
            isRevealed={cell.revealed}
            isSelected={isSelected}
            isInSelectedClue={selectedClueCells.includes(index)}
            isRelated={selectedRelatedCells.includes(index)}
            ariaLabel={getAriaLabel(index)}
            clickHandler={clickHandler}
            openRebus={openRebus}
          />
        )
      })}
    </g>
  )
}
