import React from 'react'

import { DimensionsState } from '../../store/state/puzzle/puzzleTypes'

type LineFunc = (index: number) => string

const drawLines = (
  dimensions: DimensionsState,
  externalCells: boolean[],
) => {
  const { rowCount, columnCount, cellSize } = dimensions
  const frameWidth = 3
  const isTopRow = (index: number) => index < columnCount
  const isBottomRow = (index: number) =>
    index >= columnCount * (rowCount - 1)
  const isLeftColumn = (index: number) => index % columnCount === 0
  const isRightColumn = (index: number) =>
    index % columnCount === columnCount - 1
  const getNeighborTop = (index: number) => index - columnCount
  const getNeighborBottom = (index: number) => index + columnCount
  const getNeighborLeft = (index: number) => index - 1
  const getNeighborRight = (index: number) => index + 1
  const getX = (index: number) =>
    (index % columnCount) * cellSize + frameWidth

  const getY = (index: number) =>
    Math.floor(index / columnCount) * cellSize + frameWidth

  const moveToCorner = (
    i: number,
    xOffset: number,
    yOffset: number,
  ) => `M${getX(i) + xOffset},${getY(i) + yOffset}`
  const drawHorizontal = () => `l${cellSize},0`
  const drawVertical = () => `l0,${cellSize}`
  const topLine: LineFunc = (index) =>
    moveToCorner(index, 0, 0) + drawHorizontal()

  const bottomLine: LineFunc = (index) =>
    moveToCorner(index, 0, cellSize) + drawHorizontal()

  const leftLine: LineFunc = (index) =>
    moveToCorner(index, 0, 0) + drawVertical()

  const rightLine: LineFunc = (index) =>
    moveToCorner(index, cellSize, 0) + drawVertical()

  return externalCells.reduce(
    (path, isExternal, index) => {
      if (isExternal) return path

      const drawLine = (side: LineFunc) => {
        path.lines += side(index)
      }

      const drawFrame = (side: LineFunc) => {
        path.frame += side(index)
      }

      const isFrame = (
        isEdge: (index: number) => boolean,
        getNeighbor: (index: number) => number,
      ) => isEdge(index) || externalCells[getNeighbor(index)]
      const topIsFrame = isFrame(isTopRow, getNeighborTop)
      const rightIsFrame = isFrame(isRightColumn, getNeighborRight)
      const bottomIsFrame = isFrame(isBottomRow, getNeighborBottom)
      const leftIsFrame = isFrame(isLeftColumn, getNeighborLeft)

      if (topIsFrame) {
        drawFrame(topLine)
      }

      if (leftIsFrame) {
        drawFrame(leftLine)
      }

      if (rightIsFrame) {
        drawFrame(rightLine)
      } else {
        drawLine(rightLine)
      }

      if (bottomIsFrame) {
        drawFrame(bottomLine)
      } else {
        drawLine(bottomLine)
      }

      return path
    },
    { frame: '', lines: '' },
  )
}

type LinesProps = {
  dimensions: DimensionsState
  grid: Record<string, string | number>
  group: Record<string, string | number>
  frame: Record<string, string | number>
  externalCells: boolean[]
}

export default function Lines({
  group,
  grid,
  frame,
  dimensions,
  externalCells,
}: LinesProps) {
  const hasExternalCells = externalCells.some(
    (isExternal) => isExternal,
  )

  if (hasExternalCells) {
    const path = drawLines(dimensions, externalCells)
    return (
      <g {...group}>
        <path {...grid} d={path.lines} />
        <path {...frame} d={path.frame} strokeLinecap="round" />
      </g>
    )
  }

  return (
    <g {...group}>
      <path {...grid} />
      <rect {...frame} fill="none" />
    </g>
  )
}
