import React, { Component, ReactNode } from 'react'
import Hammer from 'hammerjs'

const MIN_SCALE = 1
const MAX_SCALE = 2
const clamp = (n: number, min: number, max: number) =>
  Math.min(max, Math.max(min, n))

type ZoomableProps = {
  className?: string
  children: ReactNode
}

type ZoomableState = {
  transitionTime: number
  scale: number
  x: number
  y: number
  prevScale: number
  prevX: number
  prevY: number
  startDeltaX: number
  startDeltaY: number
}

export default class Zoomable extends Component<
  ZoomableProps,
  ZoomableState
> {
  wrapperRef: React.RefObject<HTMLDivElement>

  constructor(props: ZoomableProps) {
    super(props)
    this.state = {
      transitionTime: 0,
      scale: 1,
      x: 0,
      y: 0,
      prevScale: 1,
      prevX: 0,
      prevY: 0,
      startDeltaX: 0,
      startDeltaY: 0,
    }

    this.wrapperRef = React.createRef()
    this.handleStart = this.handleStart.bind(this)
    this.handleMove = this.handleMove.bind(this)
    this.handleEnd = this.handleEnd.bind(this)
  }

  componentDidMount() {
    if (!this.wrapperRef.current) return

    const hammer = new Hammer(
      this.wrapperRef.current,
      // @ts-ignore
      { preventDefault: true },
    )
    hammer.get('pinch').set({ enable: true })
    hammer.on('pinchstart panstart', this.handleStart)
    hammer.on('pinchmove panmove', this.handleMove)
    hammer.on('pinchend pinchcancel panend pancancel', this.handleEnd)
  }

  handleStart(event: { deltaX: number; deltaY: number }) {
    this.setState({
      transitionTime: 0,
      startDeltaX: event.deltaX,
      startDeltaY: event.deltaY,
    })
  }

  handleMove(event: {
    deltaX: number
    deltaY: number
    scale: number
  }) {
    this.setState((prevState) => {
      const { prevScale, prevX, prevY, startDeltaX, startDeltaY } =
        prevState
      const { scale, deltaX, deltaY } = event
      const newScale = prevScale * scale

      /* Only pan if somewhat zoomed */
      if (newScale === 1 && prevScale === 1) {
        return { ...prevState, scale: newScale }
      }

      return {
        ...prevState,
        scale: newScale,
        x: prevX + (deltaX - startDeltaX),
        y: prevY + (deltaY - startDeltaY),
      }
    })
  }

  handleEnd() {
    this.setState(({ x, y, scale }) => {
      const wrapper = this.wrapperRef.current
      if (!wrapper) {
        return null
      }

      const clampedScale = clamp(scale, MIN_SCALE, MAX_SCALE)
      const maxPosX = Math.ceil(
        (clampedScale - 1) * (wrapper.clientWidth / 2),
      )

      const maxPosY = Math.ceil(
        (clampedScale - 1) * (wrapper.clientHeight / 2),
      )
      const clampedX = clamp(x, -maxPosX, maxPosX)
      const clampedY = clamp(y, -maxPosY, maxPosY)

      return {
        transitionTime: 0.5,
        scale: clampedScale,
        x: clampedX,
        y: clampedY,
        prevScale: clampedScale,
        prevX: clampedX,
        prevY: clampedY,
      }
    })
  }

  createStyle() {
    const { scale, x, y, transitionTime } = this.state

    return {
      transition: `transform ${transitionTime}s`,
      transform: `translate(${x}px, ${y}px) scale(${scale})`,
    }
  }

  render() {
    const { children, className } = this.props

    return (
      <div
        className={className}
        ref={this.wrapperRef}
        style={this.createStyle()}
      >
        {children}
      </div>
    )
  }
}
