import React from 'react'
import { mount } from 'enzyme'
import { Provider } from 'react-redux'

import { ClueListState } from '../../../store/state/puzzle/puzzleTypes'
import { ClueState } from '../../../store/state/clues/cluesTypes'
import { SelectionState } from '../../../store/state/selection/selectionTypes'

import Clue from './Clue'

describe('ClueList', () => {
  const handleClueClick = jest.fn()
  const clueLists: ClueListState = {
    clues: [0, 1],
    cells: [0, 1],
    index: 0,
    prev: 1,
    next: 1,
    name: 'Across',
  }

  const clues: ClueState[] = [
    {
      list: 0,
      cells: [0, 1],
      direction: 'Across',
      label: '1',
      text: 'Dip for chips',
      index: 0,
      prev: 1,
      next: 1,
      unfilledCount: 0,
      isImageClue: false,
      alternativeAriaLabelText: '',
    },
    {
      list: 0,
      cells: [1, 2],
      direction: 'Across',
      label: '2',
      text: 'Popular plant',
      index: 1,
      prev: 2,
      next: 2,
      unfilledCount: 0,
      isImageClue: false,
      alternativeAriaLabelText: '',
    },
  ]

  const makeStore = (selection: SelectionState) => {
    return {
      getState: () => ({
        clueLists,
        clues,
        selection,
      }),
      dispatch: jest.fn(),
      subscribe: jest.fn(),
    }
  }

  const defaultSelection = {
    cell: 135,
    cellClues: [0, 1],
    clue: 65,
    clueCells: [0, 1],
    clueList: 1,
    relatedCells: [],
    relatedClues: [6],
  }

  const createWrapper = (selection: SelectionState) => {
    const store = makeStore(selection)
    return mount(
      // @ts-ignore
      <Provider store={store}>
        <Clue
          clickHandler={handleClueClick}
          index={0}
          label="Favorite millenial plant"
          text="foo"
          key={1}
          isModalOpen={false}
        />
      </Provider>,
    )
  }

  it('renders clue with text', () => {
    const wrapper = createWrapper(defaultSelection)
    expect(wrapper.find('span.xwd__clue--label').text()).toEqual(
      'Favorite millenial plant',
    )
  })

  it('renders clue with image tag', () => {
    const store = makeStore(defaultSelection)
    const wrapper = mount(
      // @ts-ignore
      <Provider store={store}>
        <Clue
          clickHandler={handleClueClick}
          index={0}
          label="Clue placeholder"
          text={'<img src="[IMAGE_CLUE_URL]" alt="Alt text here." />'}
          key={1}
          isModalOpen={false}
        />
      </Provider>,
    )

    expect(
      wrapper
        .html()
        .includes(
          '<img src="[IMAGE_CLUE_URL]" alt="Alt text here.">',
        ),
    ).toBe(true)
  })

  it('utilizes click handler when list element is clicked', () => {
    const wrapper = createWrapper(defaultSelection)
    wrapper.find('li.xwd__clue--li').simulate('click')
    expect(handleClueClick).toHaveBeenCalledTimes(1)
  })

  it('adds highlighted class when selection contains seleted cell and not in selected list', () => {
    const wrapper = createWrapper(defaultSelection)
    expect(wrapper.find('li.xwd__clue--highlighted')).toHaveLength(1)
  })

  it('does not add related class when relatedClues does not include clue index', () => {
    const wrapper = createWrapper(defaultSelection)
    expect(wrapper.find('li.xwd__clue--related')).toHaveLength(0)
  })

  it('adds the selected class when li element is selected', () => {
    const selected = {
      cell: 135,
      cellClues: [0, 1],
      clue: 65,
      clueCells: [0, 1],
      clueList: 0,
      relatedCells: [],
      relatedClues: [6],
    }
    const wrapper = createWrapper(selected)
    expect(wrapper.find('li.xwd__clue--selected')).toHaveLength(1)
  })

  it('adds the filled class when unfilled count is 0', () => {
    const wrapper = createWrapper(defaultSelection)
    expect(wrapper.find('li.xwd__clue--filled')).toHaveLength(1)
  })
})
