import React, { Component, createRef, RefObject } from 'react'
import cn from 'classnames'
import { connect } from 'react-redux'

import { CrosswordStoreState } from '../../../store/state/crosswordReducer'
import { getAllClues } from '../../../store/state/puzzle/puzzleSelectors'
import { getCurrentSelection } from '../../../store/state/selection/selectionSelectors'

type ClueStateProps = {
  hasNoUnfilledCells: boolean
  isHighlighted: boolean
  isRelated: boolean
  isSelected: boolean
  isImageClue: boolean
}

type ClueOwnProps = {
  clickHandler?: (clueIndex: number, isSelected: boolean) => void
  index: number | null
  label: string
  text: string
  isModalOpen: boolean
}

type ClueProps = ClueOwnProps & ClueStateProps

class Clue extends Component<ClueProps> {
  listRef: RefObject<HTMLLIElement>

  constructor(props: ClueProps) {
    super(props)
    this.listRef = createRef()
  }

  componentDidUpdate() {
    const { isHighlighted, isSelected, isModalOpen } = this.props
    const notSelectedOrHighlighted = !isHighlighted && !isSelected
    if (notSelectedOrHighlighted || isModalOpen) return
    const listItem = this.listRef.current
    const list = listItem?.parentElement
    if (listItem && list) {
      const liScrollTop = listItem.offsetTop
      const listTop = list.offsetTop
      list.scrollTo({
        top: liScrollTop - listTop,
        behavior: 'smooth',
      })
    }
  }

  handleClick = () => {
    const { clickHandler, index, isSelected } = this.props
    if (typeof index === 'number' && clickHandler) {
      clickHandler(index, isSelected)
    }
  }

  render() {
    const {
      hasNoUnfilledCells,
      isHighlighted,
      isRelated,
      isSelected,
      label,
      text = '',
      isImageClue,
    } = this.props

    const clueClasses = cn('xwd__clue--li', {
      'xwd__clue--filled': hasNoUnfilledCells,
      'xwd__clue--highlighted': isHighlighted,
      'xwd__clue--selected': isSelected,
      'xwd__clue--related': isRelated,
    })

    return (
      <li // eslint-disable-line jsx-a11y/no-noninteractive-element-interactions
        className={clueClasses}
        onClick={this.handleClick}
        ref={this.listRef}
        onKeyDown={(e) => {
          if (e.key === 'Enter') this.handleClick()
        }}
      >
        <span className="xwd__clue--label">{label}</span>
        <span
          className={cn('xwd__clue--text xwd__clue-format', {
            'xwd__clue--label--has-image': isImageClue,
          })}
          dangerouslySetInnerHTML={{ __html: text }}
        />
      </li>
    )
  }
}

const mapStateToProps = (
  state: CrosswordStoreState,
  ownProps: ClueOwnProps,
): ClueStateProps => {
  const selection = getCurrentSelection(state)
  const clues = getAllClues(state)
  const { index } = ownProps
  if (typeof index === 'number' && clues) {
    const clue = clues[index]
    const containsSelectedCell = selection.cellClues.includes(index)
    const inSelectedList = clue.list === selection.clueList
    const isSelected = containsSelectedCell && inSelectedList
    const isRelated = selection.relatedClues.includes(index)
    const hasNoUnfilledCells = clue.unfilledCount === 0
    const isHighlighted = containsSelectedCell && !isSelected
    return {
      hasNoUnfilledCells,
      isHighlighted,
      isSelected,
      isRelated,
      isImageClue: clue.isImageClue,
    }
  }

  return {
    hasNoUnfilledCells: false,
    isHighlighted: false,
    isSelected: false,
    isRelated: false,
    isImageClue: false,
  }
}

export default connect(mapStateToProps)(Clue)
