import React, { useCallback, useEffect, useRef } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import classNames from 'classnames'

import { isMobile } from 'responsive'
import { useUser } from 'user'
import { getGameData } from 'foundation'

import { getIsModalOpen } from '../../store/state/modal/modalSelectors'
import {
  getAllClueLists,
  getPageTitle,
  getPuzzleError,
  getPuzzleHasLoaded,
  getPuzzleId,
} from '../../store/state/puzzle/puzzleSelectors'
import { toggleHelpMenu } from '../../store/state/toolbar/toolbarActions'
import { puzzleEscaped } from '../../store/state/transient/transientActions'
import { recallPuzzleProgress } from '../../store/thunks/sharedThunks'
import keyDownHandler from '../../utils/keyDownHandler'
import keyPressHandler from '../../utils/keyPressHandler'
import { openModal } from '../../store/state/modal/modalActions'
import { CrosswordModal } from '../../store/state/modal/modalTypes'
import {
  recallUserSettings,
  recallPrintPrefs,
} from '../../store/thunks/userThunks'
import {
  getDoEscape,
  getIsReady,
} from '../../store/state/transient/transientSelectors'
import { getIsSolved } from '../../store/state/status/statusSelectors'
import {
  getMeterGranted,
  getMeterGrantReason,
} from '../../store/state/meter/meterSelectors'
import {
  getInRebusMode,
  getActiveMenu,
} from '../../store/state/toolbar/toolbarSelectors'
import Layout from '../Layout/Layout'
import useAbra from '../../../../shared/abra/useAbra'
import {
  callMeter,
  loadPuzzleWithMeter,
} from '../../store/thunks/meterThunks'

import CrosswordError from './CrosswordError'
import CrosswordLoading from './CrosswordLoading'

type FocusProps = {
  inRebusMode: boolean
  modalIsOpen: boolean
  isReady: boolean
  ref: HTMLElement | null
}

export const updateFocus = ({
  isReady,
  modalIsOpen,
  inRebusMode,
  ref,
}: FocusProps) => {
  const focusBody = isReady && !(modalIsOpen || inRebusMode)

  // If our ref exists, set focused classes
  if (ref) {
    ref.className = classNames('xwd__franklin', {
      xwd__focused: focusBody,
    })
  }

  // Skip focusing if we're in a modal or rebus
  if (focusBody) {
    const x = window.pageXOffset
    const y = window.pageYOffset

    ref && ref.focus()
    window.scrollTo(x, y)
  }
}

const CrosswordPage = () => {
  const mainRef = useRef<HTMLElement>(null)

  // STATE
  const clueLists = useSelector(getAllClueLists)
  const isReady = useSelector(getIsReady)
  const doEscape = useSelector(getDoEscape)
  const isSolved = useSelector(getIsSolved)
  const modalIsOpen = useSelector(getIsModalOpen)
  const inRebusMode = useSelector(getInRebusMode)
  const puzzleHasLoaded = useSelector(getPuzzleHasLoaded)
  const puzzleId = useSelector(getPuzzleId)
  const puzzleError = useSelector(getPuzzleError)
  const pageTitle = useSelector(getPageTitle)
  const activeMenu = useSelector(getActiveMenu)

  // DISPATCH
  const dispatch = useDispatch()

  const { userType, hasLoaded: hasUserInfoLoaded } = useUser()
  const { getVariant, hasAbraLoaded } = useAbra()

  // TODO: Test if memoizing dispatch-dependent callbacks is necessary
  const handleKeyDown = useCallback((event) => {
    const { action, payload } = keyDownHandler(event)

    if (!action) return

    dispatch(action(payload))

    event.preventDefault()
    event.stopPropagation()
  }, [])

  const handleKeyPress = useCallback((event) => {
    const { action, payload, shiftKey } = keyPressHandler(event)

    if (!action || !payload) return

    dispatch(action(payload, shiftKey))
  }, [])

  const handleClick = () => {
    if (activeMenu) dispatch(toggleHelpMenu(null))
  }

  useEffect(() => {
    if (hasUserInfoLoaded && puzzleHasLoaded) {
      /** get user specific state */
      dispatch(recallUserSettings(isMobile, userType))
      dispatch(recallPrintPrefs(userType))
      dispatch(recallPuzzleProgress(userType))
    }
  }, [hasUserInfoLoaded, puzzleHasLoaded, puzzleId])

  /* METER LOGIC */
  useEffect(() => {
    if (hasAbraLoaded) {
      // TODO: Abra cleanup
      const callMeterVariant = getVariant(
        'GAMES_xwdMeterIntegration_0927',
      )
      const hasCallMeterVariant = callMeterVariant === `1_callMeter`

      const gateMeterVariant = getVariant(
        'GAMES_xwdMeterEnforcement_0927',
      )
      const hasGateMeterVariant = gateMeterVariant === `1_gateMeter`

      // CHECK IF IOS
      const isIOSGamesApp = window.gamesAppPlatform === 'ios'
      // CHECK IF MINI
      // Cannot use getIsMini selector since selector returns false if Puzzle Data is not loaded
      const isMini = window.gameData?.stream === 'mini'

      // Do not use Meter for Mini or iOS XWD
      if (isIOSGamesApp || isMini) return

      if (hasGateMeterVariant) {
        const { filename, stream } = getGameData()
        dispatch(loadPuzzleWithMeter(filename, stream))
      } else if (hasCallMeterVariant) {
        dispatch(callMeter())
      }
    }
  }, [hasAbraLoaded])

  useEffect(() => {
    window.scrollTo(0, 0) // Always return to top of page on load
    return () => window.scrollTo(0, 0) // return to top of page on cleanup
  }, [])

  useEffect(() => {
    updateFocus({
      isReady,
      modalIsOpen,
      inRebusMode,
      ref: mainRef.current,
    })
  }, [isReady, isSolved, modalIsOpen, inRebusMode])

  useEffect(() => {
    if (!isReady && !modalIsOpen) {
      dispatch(openModal(CrosswordModal.Start))
    }
  }, [isReady, modalIsOpen, dispatch])

  useEffect(() => {
    if (puzzleHasLoaded) {
      document.title = pageTitle
    }
  }, [puzzleHasLoaded, pageTitle])

  if (!puzzleHasLoaded || !hasUserInfoLoaded) {
    return <CrosswordLoading />
  }

  if (puzzleError) {
    return <CrosswordError error={puzzleError} userType={userType} />
  }

  return (
    <main // eslint-disable-line
      tabIndex={0} // eslint-disable-line
      ref={mainRef}
      onKeyDown={(e) => {
        if (modalIsOpen) return
        handleKeyDown(e)
      }}
      onKeyPress={(e) => {
        if (modalIsOpen) return
        handleKeyPress(e)
      }}
      onClick={handleClick}
    >
      <Layout
        clueLists={clueLists}
        doEscape={doEscape}
        puzzleEscaped={() => dispatch(puzzleEscaped())}
        inRebusMode={inRebusMode}
        isMobile={isMobile}
      />
    </main>
  )
}

export default CrosswordPage
