import React from 'react'
import { mount } from 'enzyme'
import { Provider } from 'react-redux'

import { trackModuleInteraction } from '../../store/utils/tracker'

import EditorialCard, { EditorialCardProps } from './EditorialCard'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

jest.mock('../../../../shared/hub/store/selectors', () => ({
  getCurrentPuzzle: jest.fn((state) => ({ meta: { id: 0 } })),
}))

jest.mock('../../store/utils/tracker', () => ({
  trackModuleInteraction: jest.fn(),
}))

describe('EDITORIAL CONTENT: EDITORIAL CARDS', () => {
  const makeStore = () => ({
    getState: () => ({
      puzzle: {
        data: {
          meta: {
            id: 0,
          },
        },
      },
    }),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  })

  const getWrapper = (
    store = makeStore(),
    customProps: EditorialCardProps,
  ) => {
    return mount(
      // @ts-ignore
      <Provider store={store}>
        <EditorialCard {...customProps} />
      </Provider>,
    )
  }

  const requiredArticleProps = {
    headline: 'What The Heck Is That?: The River ALPH',
    url: 'https://www.nytimes.com',
  }

  it('should add proper class depending on card type', () => {
    const type = 'featuredCard' as const
    const fakeProps = { ...requiredArticleProps, type }

    const wrapper = getWrapper(makeStore(), fakeProps)
    expect(
      wrapper.find('.xwd__editorial-content--featuredCard').exists(),
    ).toEqual(true)
  })

  it('should render the component with all props', () => {
    const type = 'secondaryCard' as const
    const fakeProps = {
      ...requiredArticleProps,
      byline: 'DEB AMLEN',
      kicker: 'a',
      thumbURL: 'img.png',
      summary: 'lorem ipsum',
      type,
    }

    const wrapper = getWrapper(makeStore(), fakeProps)

    expect(
      wrapper.find('.xwd__editorial-content--secondaryCard').exists(),
    ).toEqual(true)
    expect(wrapper.find('img').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--kicker').exists(),
    ).toEqual(true)
    expect(wrapper.find('h2').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--summary').exists(),
    ).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--byline').exists(),
    ).toEqual(true)
  })

  it('should not render an image if thumbUrl is missing', () => {
    const type = 'secondaryCard' as const
    const fakeProps = {
      ...requiredArticleProps,
      byline: 'DEB AMLEN',
      kicker: 'a',
      summary: 'lorem ipsum',
      type,
    }

    const wrapper = getWrapper(makeStore(), fakeProps)

    expect(wrapper.find('img').exists()).toEqual(false)
    expect(
      wrapper.find('.xwd__editorial-content--kicker').exists(),
    ).toEqual(true)
    expect(wrapper.find('h2').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--summary').exists(),
    ).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--byline').exists(),
    ).toEqual(true)
  })

  it('should not render the kicker if it is missing', () => {
    const type = 'secondaryCard' as const
    const fakeProps = {
      ...requiredArticleProps,
      byline: 'DEB AMLEN',
      thumbURL: 'img.png',
      summary: 'lorem ipsum',
      type,
    }

    const wrapper = getWrapper(makeStore(), fakeProps)

    expect(wrapper.find('img').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--kicker').exists(),
    ).toEqual(false)
    expect(wrapper.find('h2').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--summary').exists(),
    ).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--byline').exists(),
    ).toEqual(true)
  })

  it('should not render the summary if it is missing', () => {
    const type = 'secondaryCard' as const
    const fakeProps = {
      ...requiredArticleProps,
      byline: 'DEB AMLEN',
      kicker: 'a',
      thumbURL: 'img.png',
      type,
    }

    const wrapper = getWrapper(makeStore(), fakeProps)

    expect(wrapper.find('img').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--kicker').exists(),
    ).toEqual(true)
    expect(wrapper.find('h2').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--summary').exists(),
    ).toEqual(false)
    expect(
      wrapper.find('.xwd__editorial-content--byline').exists(),
    ).toEqual(true)
  })

  it('should not render the byline if it is missing', () => {
    const type = 'secondaryCard' as const
    const fakeProps = {
      ...requiredArticleProps,
      kicker: 'a',
      thumbURL: 'img.png',
      summary: 'lorem ipsum',
      type,
    }

    const wrapper = getWrapper(makeStore(), fakeProps)

    expect(wrapper.find('img').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--kicker').exists(),
    ).toEqual(true)
    expect(wrapper.find('h2').exists()).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--summary').exists(),
    ).toEqual(true)
    expect(
      wrapper.find('.xwd__editorial-content--byline').exists(),
    ).toEqual(false)
  })

  it('should track items properly', () => {
    const type = 'secondaryCard' as const
    const fakeProps = {
      ...requiredArticleProps,
      kicker: 'a',
      thumbURL: 'img.png',
      summary: 'lorem ipsum',
      type,
    }

    const wrapper = getWrapper(makeStore(), fakeProps)

    wrapper
      .find('.xwd__editorial-content--secondaryCard')
      .simulate('click')

    // @ts-ignore
    expect(trackModuleInteraction.mock.calls.length).toEqual(1)
  })
})
