// @ts-nocheck
import React from 'react'
import { mount } from 'enzyme'
import { act } from 'react-dom/test-utils'
import { Provider } from 'react-redux'

import responsive from 'responsive'

import logger from '../../../shared/utils/clientLogger'
import getEditorialContent from '../../store/utils/editorialContentApi'
import { getIsMini } from '../../store/state/puzzle/puzzleSelectors'

import EditorialContent from './EditorialContent'
import SubGameplayGrid from './SubGameplayGrid'
import EditorialCard from './EditorialCard'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

jest.mock('../../store/state/puzzle/puzzleSelectors', () => ({
  ...jest.requireActual('../../store/state/puzzle/puzzleSelectors'),
  getIsMini: jest.fn(() => false),
}))

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('../../../shared/utils/clientLogger', () => ({
  warn: jest.fn(),
}))

jest.mock('../../store/utils/editorialContentApi', () => jest.fn())

describe('EDITORIAL CONTENT: SUBGAMEPLAY GRID', () => {
  const fakeArticles = {
    featuredArticle: {
      headline: 'a',
      byline: 'b',
      url: 'c',
      type: 'featuredCard',
    },
    secondaryArticles: [
      {
        headline: 'd',
        byline: 'e',
        url: 'f',
        type: 'secondaryCard',
      },
      {
        headline: 'g',
        byline: 'h',
        url: 'i',
        type: 'secondaryCard',
      },
    ],
  }

  const publicationDate = '2021-11-03'

  const makeStore = () => ({
    getState: () => ({
      puzzle: {
        data: {
          meta: {
            publicationDate,
            id: 0,
          },
        },
      },
      modal: {
        name: 'name',
      },
    }),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  })

  let div: HTMLDivElement
  let wrapper: ReactWrapper

  const getWrapper = (store = makeStore()) => {
    return mount(
      <Provider store={store}>
        <EditorialContent />
      </Provider>,
      {
        attachTo: div,
      },
    )
  }

  beforeEach(() => {
    div = document.createElement('div')
    div.setAttribute('id', 'portal-editorial-content')
    document.body.appendChild(div)

    responsive.isMobile = false
  })

  it('should render grid with articles from API', async () => {
    getEditorialContent.mockImplementation(() =>
      Promise.resolve(fakeArticles),
    )

    // eslint-disable-next-line require-await
    await act(async () => {
      wrapper = getWrapper(makeStore())
    })
    wrapper.update()

    expect(wrapper.find(EditorialCard).first().props()).toEqual(
      fakeArticles.featuredArticle,
    )
    expect(wrapper.find(EditorialCard).last().props()).toEqual(
      fakeArticles.secondaryArticles[
        fakeArticles.secondaryArticles.length - 1
      ],
    )
    expect(
      wrapper.find(SubGameplayGrid).props().children.flat().length,
    ).toEqual(fakeArticles.secondaryArticles.length + 1)
  })

  it('should not render SubGameplayGrid component if API fails', async () => {
    const mockError = new Error('api fail')
    getEditorialContent.mockImplementation(() =>
      Promise.reject(mockError),
    )

    // eslint-disable-next-line require-await
    await act(async () => {
      wrapper = getWrapper(makeStore())
    })
    wrapper.update()

    expect(wrapper.find(SubGameplayGrid).exists()).toBe(false)
    expect(logger.warn).toHaveBeenCalledWith(mockError)
  })

  it('should not render SubGameplayGrid component if is mobile', async () => {
    getEditorialContent.mockImplementation(() =>
      Promise.resolve(fakeArticles),
    )

    responsive.isMobile = true

    // eslint-disable-next-line require-await
    await act(async () => {
      wrapper = getWrapper(makeStore())
    })
    wrapper.update()

    expect(wrapper.find(SubGameplayGrid).exists()).toBe(false)
  })

  it('should not render SubGameplayGrid component if isMini', async () => {
    getEditorialContent.mockImplementation(() =>
      Promise.resolve(fakeArticles),
    )

    getIsMini.mockImplementation(() => true)

    // eslint-disable-next-line require-await
    await act(async () => {
      wrapper = getWrapper(makeStore())
    })
    wrapper.update()

    expect(wrapper.find(SubGameplayGrid).exists()).toBe(false)
  })

  it('should not render SubGameplayGrid component if ref does not exist', async () => {
    div.setAttribute('id', 'portal-editorial-content-doesnt-exist')

    getEditorialContent.mockImplementation(() =>
      Promise.resolve(fakeArticles),
    )

    // eslint-disable-next-line require-await
    await act(async () => {
      wrapper = getWrapper(makeStore())
    })
    wrapper.update()

    expect(wrapper.find(SubGameplayGrid).exists()).toBe(false)
  })
})
