import React from 'react'
import mitt from 'mitt'
import { mount } from 'enzyme'

import Juice from './index'

import { createMilestoneAnimation } from './createMilestoneAnimation'
import Milestone, { ANIMATION_TIME, CANVAS_SIZE } from './Milestone'

jest.mock('./createMilestoneAnimation', () => ({
  createMilestoneAnimation: jest.fn(),
}))

jest.mock('./findTriggerCell', () => ({
  findTriggerCell: () => ({ left: 20, top: 20 }),
}))

jest.useFakeTimers()

describe('JUICE: MILESTONE COMPONENT', () => {
  let animate
  let emitter
  let milestoneEventName

  Object.defineProperty(global, 'document.getElementById', {
    getBoundingClientRect: () => ({
      left: 20,
      top: 20,
    }),
    writable: true,
  })

  beforeEach(() => {
    milestoneEventName = 'milestone'
    emitter = mitt()
    animate = jest.fn()
    createMilestoneAnimation.mockReturnValue(animate)
  })

  it('animates on milestone events and cleans up on unmount', () => {
    const MockMilestone = () => (
      <Juice.Provider value={emitter}>
        <Milestone boardId="xwd-board" />
      </Juice.Provider>
    )

    const wrapper = mount(<MockMilestone />)
    const canvas = wrapper.find('canvas').instance()
    const fakeEvent = { id: 1, text: 'hello' }

    // sets up animation on mount
    expect(createMilestoneAnimation).toHaveBeenCalledWith(
      canvas,
      CANVAS_SIZE,
    )

    // does not animate immediately
    expect(animate).toHaveBeenCalledTimes(0)

    // animates on milestone events
    emitter.emit(milestoneEventName, fakeEvent)

    expect(animate).toHaveBeenCalledTimes(1)

    // does not animate while animation is in progress
    jest.runTimersToTime(ANIMATION_TIME - 1)
    emitter.emit(milestoneEventName, fakeEvent)

    expect(animate).toHaveBeenCalledTimes(1)

    // animates again if event is after animation is complete
    jest.runTimersToTime(ANIMATION_TIME + 1)
    emitter.emit(milestoneEventName, fakeEvent)

    expect(animate).toHaveBeenCalledTimes(2)

    // cleans up on unmount
    wrapper.unmount()
    jest.runTimersToTime(ANIMATION_TIME * 2 + 1)
    emitter.emit(milestoneEventName, fakeEvent)

    expect(animate).toHaveBeenCalledTimes(2)
  })

  it("sets position based on findTriggerCell's DOM calculations", () => {
    const MockMilestone = () => (
      <Juice.Provider value={emitter}>
        <Milestone boardId="xwd-board" />
      </Juice.Provider>
    )

    const wrapper = mount(<MockMilestone />)

    emitter.emit(milestoneEventName, {})
    wrapper.update()

    const { style } = wrapper.find('.xwd__milestone').props()

    expect(style.left).toEqual(20)
    expect(style.top).toEqual(20)
  })

  it('computes text from message', () => {
    const msg = { text: 'hello message' }

    const MockMilestone = () => (
      <Juice.Provider value={emitter}>
        <Milestone boardId="xwd-board" />
      </Juice.Provider>
    )

    const wrapper = mount(<MockMilestone />)

    emitter.emit(milestoneEventName, msg)
    wrapper.update()

    const messagePill = wrapper.find('.xwd__pill')

    expect(messagePill.text()).toBe(msg.text)
  })

  it('handles visibility', () => {
    const msg = { text: 'hello message' }

    const MilestoneMock = () => (
      <Juice.Provider value={emitter}>
        <Milestone boardId="xwd-board" />
      </Juice.Provider>
    )

    const wrapper = mount(<MilestoneMock />)

    expect(
      wrapper.find('.xwd__milestone').props().className,
    ).not.toContain('xwd__show')

    emitter.emit(milestoneEventName, msg)
    wrapper.update()

    expect(
      wrapper.find('.xwd__milestone').props().className,
    ).toContain('xwd__show')

    jest.runOnlyPendingTimers()
    wrapper.update()

    expect(
      wrapper.find('.xwd__milestone').props().className,
    ).not.toContain('xwd__show')
  })

  it('sets up canvas container for retina', () => {
    const MockComponent = () => (
      <Juice.Provider value={emitter}>
        <Milestone boardId="xwd-board" />
      </Juice.Provider>
    )

    const mounted = mount(<MockComponent />)

    const canvas = mounted.find('canvas')

    expect(canvas.prop('style')).toEqual({
      width: 70,
      height: 70,
    })
    expect(canvas.prop('width')).toEqual(140)
    expect(canvas.prop('height')).toEqual(140)
  })
})
