import React from 'react'
import cn from 'classnames'

import Juice, { JuiceEmitter } from './index'

import { findTriggerCell } from './findTriggerCell'
import { createMilestoneAnimation } from './createMilestoneAnimation'

export const CANVAS_SIZE = 70
export const ANIMATION_TIME = 1450 // NOTE: matches css animation

export type Message = {
  idx: number
  left?: number
  text: string
  top?: number
}

type JuicyMilestoneProps = {
  boardId: string
}

type MilestoneProps = JuicyMilestoneProps & {
  emitter: JuiceEmitter
}

type MilestoneState = {
  message: Message | null
}

const computeText = (message: Message) => message?.text

export class Milestone extends React.PureComponent<
  MilestoneProps,
  MilestoneState
> {
  animate: (() => void) | undefined

  canvas: React.RefObject<HTMLCanvasElement>

  timer: NodeJS.Timeout | null | undefined

  constructor(props: MilestoneProps) {
    super(props)
    this.state = { message: null }
    this.canvas = React.createRef()
    this.handleMessage = this.handleMessage.bind(this)
  }

  componentDidMount() {
    const { emitter } = this.props

    this.setupCanvas()

    emitter.on('milestone', this.handleMessage)
  }

  componentWillUnmount() {
    const { emitter } = this.props

    if (this.timer) clearTimeout(this.timer)

    emitter.off('milestone', this.handleMessage)
  }

  handleMessage(newMessage: Message) {
    const { message } = this.state

    if (message) return

    this.setState({ message: newMessage })

    if (this.animate) this.animate()

    this.timer = setTimeout(() => {
      this.timer = null
      this.setState({ message: null })
    }, ANIMATION_TIME + 20)
  }

  setupCanvas() {
    this.animate = createMilestoneAnimation(
      this.canvas.current,
      CANVAS_SIZE,
    )
  }

  render() {
    const { boardId } = this.props
    const { message } = this.state
    const position = message && findTriggerCell(message, boardId)

    return (
      <div
        className={cn('xwd__milestone', { xwd__show: !!message })}
        style={{ ...position }}
      >
        <span className="xwd__pill">
          {message && computeText && computeText(message)}
        </span>
        <canvas
          className="xwd__canvas"
          ref={this.canvas}
          style={{
            width: CANVAS_SIZE,
            height: CANVAS_SIZE,
          }}
          width={CANVAS_SIZE * 2}
          height={CANVAS_SIZE * 2}
        />
      </div>
    )
  }
}

const JuicyMilestone = (props: JuicyMilestoneProps) => (
  <Juice.Consumer>
    {(e) => <Milestone emitter={e} {...props} />}
  </Juice.Consumer>
)

export default JuicyMilestone
