import React from 'react'
import { useSelector, useDispatch } from 'react-redux'
import ReactSimpleKeyboard from 'react-simple-keyboard'

import {
  navigate,
  switchDirection,
} from '../../store/state/selection/selectionActions'
import {
  togglePencil,
  updateRebus,
} from '../../store/state/toolbar/toolbarActions'
import {
  getInPencilMode,
  getInRebusMode,
  getRebusValue,
} from '../../store/state/toolbar/toolbarSelectors'
import { getSelectedClue } from '../../store/state/selection/selectionSelectors'
import { backspace } from '../../store/thunks/cellThunks'
import { guess } from '../../store/thunks/statusThunks'
import {
  exitRebus,
  toggleRebus,
} from '../../store/thunks/toolbarThunks'

import {
  layout,
  CrosswordKeyboardKey,
  customButtons,
  iconButtons,
  tabletOnlyButtons,
} from './keys'

export default function Keyboard() {
  const [layoutName, setLayoutName] = React.useState('primary')
  const inRebusMode = useSelector(getInRebusMode)
  const inPencilMode = useSelector(getInPencilMode)
  const rebusValue = useSelector(getRebusValue)
  const selectedButtons = `${inRebusMode && 'rebus'} ${
    inPencilMode && 'pencil'
  }`
  const selectedClue = useSelector(getSelectedClue)
  const dispatch = useDispatch()

  const navigateAndExitRebus = (direction: string) => {
    if (inRebusMode) {
      dispatch(exitRebus(true))
    }
    dispatch(navigate(direction))
  }

  const onKeyPress = (key: CrosswordKeyboardKey) => {
    if (key === 'primary') {
      return setLayoutName('primary')
    }

    if (key === 'secondary') {
      return setLayoutName('secondary')
    }

    if (key === 'tertiary') {
      return setLayoutName('tertiary')
    }

    if (key === 'backspace') {
      if (inRebusMode && rebusValue) {
        return dispatch(updateRebus(rebusValue.slice(0, -1)))
      }

      return dispatch(backspace())
    }

    if (key === 'rebus') {
      return dispatch(toggleRebus())
    }

    if (key === 'pencil') {
      return dispatch(togglePencil())
    }

    if (key === 'prev') {
      return navigateAndExitRebus('PreviousClue')
    }

    if (key === 'next') {
      return navigateAndExitRebus('NextClue')
    }

    if (key === 'switch') {
      return dispatch(switchDirection())
    }

    if (key === 'right') {
      return navigateAndExitRebus('Right')
    }

    if (key === 'left') {
      return navigateAndExitRebus('Left')
    }

    if (inRebusMode) {
      dispatch(updateRebus(rebusValue + key))
    } else {
      dispatch(guess(key))
    }
  }

  const getImageUrl = (filename: string) => {
    // base path comes from server in getCrossword.js
    // is hardcoded locally and points to the CDN when deployed
    const base = window.gameData?.assetBasePath
    return `<img src="${base}/assets/${filename}.svg" />`
  }

  const display: Partial<Record<CrosswordKeyboardKey, string>> = {
    backspace: getImageUrl('delete-outline'),
    primary: 'ABC',
    secondary: '123',
    tertiary: '#+=',
    rebus: 'Rebus',
    pencil: inPencilMode
      ? getImageUrl('pencil-outline-inverse')
      : getImageUrl('pencil-outline'),
    prev: getImageUrl('arrow-left-open'),
    next: getImageUrl('arrow-right-open'),
    switch:
      selectedClue?.direction === 'Across'
        ? getImageUrl('switch-across')
        : getImageUrl('switch-down'),
  }

  return (
    <ReactSimpleKeyboard
      baseClass="xwd__keyboard"
      layout={layout}
      layoutName={layoutName}
      display={display}
      buttonTheme={[
        { class: 'icon', buttons: iconButtons.join(' ') },
        { class: 'custom', buttons: customButtons.join(' ') },
        { class: 'selected', buttons: selectedButtons },
        {
          class: 'tablet-only',
          buttons: tabletOnlyButtons.join(' '),
        },
      ]}
      preventDefault
      preventMouseDownDefault
      stopMouseDownPropagation
      onKeyPress={onKeyPress}
      disableButtonHold
      autoUseTouchEvents
    />
  )
}
