import React from 'react'
import * as redux from 'react-redux'
import { shallow } from 'enzyme'

import { ClueListsState } from '../../store/state/puzzle/puzzleTypes'
import LandscapeWarning from '../LandscapeWarning/LandscapeWarning'
import useScreenSizeWarning from '../../../shared/hooks/use-screen-size-warning'

import Layout from './Layout'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  track: jest.fn(),
}))

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('../../../shared/hooks/use-screen-size-warning', () => ({
  __esModule: true,
  ...jest.requireActual(
    '../../../shared/hooks/use-screen-size-warning',
  ),
  default: jest.fn(() => false),
}))

const useSelectorSpy = jest.spyOn(redux, 'useSelector')
useSelectorSpy.mockReturnValue({})

const useDispatchSpy = jest.spyOn(redux, 'useDispatch')
useDispatchSpy.mockReturnValue(jest.fn())

const clueLists: ClueListsState = [
  {
    clues: [1, 2, 3, 4],
    cells: [1, 3, 5],
    index: 1,
    prev: 0,
    next: 2,
    name: 'B',
  },
]

describe('LAYOUT', () => {
  it('should not render mobile controls div on desktop', () => {
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode={false}
        isMobile={false}
        puzzleEscaped={jest.fn()}
      />,
    )

    expect(
      wrapper.find('.xwd__layout_controls--mobile'),
    ).toHaveLength(0)
  })

  it('render the breaker span on mobile', () => {
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode={false}
        isMobile
        puzzleEscaped={jest.fn()}
      />,
    )

    expect(wrapper.find('.xwd__layout_breaker')).toHaveLength(1)
  })

  it('should render article wrapper for mobile board when mode isMobile', () => {
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode={false}
        isMobile
        puzzleEscaped={jest.fn()}
      />,
    )
    expect(wrapper.find('.xwd__layout_puzzle--mobile')).toHaveLength(
      1,
    )
    expect(wrapper.find('.xwd__layout_puzzle--desktop')).toHaveLength(
      0,
    )
    expect(
      wrapper.find('.xwd__layout_breaker--desktop'),
    ).toHaveLength(0)
  })

  it('should render the mobile controls div when mode is isMobile and rebusMode turned off', () => {
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode={false}
        isMobile
        puzzleEscaped={jest.fn()}
      />,
    )
    expect(
      wrapper.find('.xwd__layout_controls--mobile'),
    ).toHaveLength(1)
  })

  it('should not render the Puzzle Header on Mobile', () => {
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode={false}
        isMobile
        puzzleEscaped={jest.fn()}
      />,
    )
    expect(wrapper.find('PuzzleHeader').length).toBe(0)
  })

  it('should render the rebus field if flag is on', () => {
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode
        isMobile={false}
        puzzleEscaped={jest.fn()}
      />,
    )

    expect(wrapper.find('Toolbar').length).toBe(1)
    expect(wrapper.find('Board').length).toBe(1)
    expect(wrapper.find('Connect(RebusField)').length).toBe(1)
  })

  it('should render a landscape warning on mobile devices', () => {
    // @ts-ignore - property mockImplementation doesn't exist
    useScreenSizeWarning.mockImplementation(() => true)
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode={false}
        puzzleEscaped={jest.fn()}
        isMobile
      />,
    )

    expect(wrapper.find(LandscapeWarning)).toHaveLength(1)
  })

  it('does not add keyboard layout class if mobile view is in the newsreader app', () => {
    window.newsreaderAppPlatform = 'android'
    const wrapper = shallow(
      <Layout
        clueLists={clueLists}
        inRebusMode={false}
        isMobile
        puzzleEscaped={jest.fn()}
      />,
    )
    expect(wrapper.find('.xwd__layout_keyboard')).toHaveLength(0)
  })
})
