import React, { useEffect, useRef, useState } from 'react'
import { createPortal } from 'react-dom'
import classnames from 'classnames'
import { useSelector } from 'react-redux'

import useDeltaBanner from 'shared/marketing/useDeltaBanner'

import type { ModalMap } from '../../../shared/ModalSystem/modalTypes'
import { debounce } from '../../../helpers/gameUtils'
import { ClueListsState } from '../../store/state/puzzle/puzzleTypes'
import { CrosswordModal } from '../../store/state/modal/modalTypes'
import {
  getOpenModalName,
  getIsModalClosing,
} from '../../store/state/modal/modalSelectors'
import {
  closeModal,
  removeModal,
} from '../../store/state/modal/modalActions'
import CongratsModal from '../Modals/CongratsModal/CongratsModal'
import StartModal from '../Modals/StartModal/StartModal'
import PauseModal from '../Modals/PauseModal/PauseModal'
import InfoModal from '../Modals/InfoModal/InfoModal'
import RatsModal from '../Modals/RatsModal/RatsModal'
import SettingsModal from '../Modals/SettingsModal/SettingsModal'
import ConfirmationModal from '../Modals/ConfirmationModal/ConfirmationModal'
import PrintOptionsModal from '../Modals/PrintOptionsModal/PrintOptionsModal'
import ShareModal from '../Modals/ShareModal/ShareModal'
import Keyboard from '../Keyboard/Keyboard'
import Toolbar from '../Toolbar/Toolbar'
import Board from '../Board/Board'
import PuzzleHeader from '../PuzzleHeader/PuzzleHeader'
import EditorialContent from '../EditorialContent/EditorialContent'
import ClueBar from '../ClueBar/ClueBar'
import ClueListWrapper from '../ClueList/ClueListWrapper'
import RebusField from '../RebusField/RebusField'
import ModalManager from '../../../shared/ModalSystem/ModalManager'
import LandscapeWarning from '../LandscapeWarning/LandscapeWarning'
import TopSaleBanner from '../../../../server/views/components/Marketing/Sale/TopSaleBanner'
import useSaleBannerHybridSubscribe from '../../../shared/hooks/use-sale-banner-hybrid-subscribe'

type LayoutProps = {
  clueLists: ClueListsState | null
  inRebusMode: boolean
  doEscape?: boolean
  puzzleEscaped?: Function
  isMobile: boolean
}

const modalMap: ModalMap = {
  [CrosswordModal.Info]: InfoModal,
  [CrosswordModal.Settings]: SettingsModal,
  [CrosswordModal.Rats]: RatsModal,
  [CrosswordModal.Congrats]: CongratsModal,
  [CrosswordModal.Confirmation]: ConfirmationModal,
  [CrosswordModal.Print]: PrintOptionsModal,
  [CrosswordModal.Start]: StartModal,
  [CrosswordModal.Pause]: PauseModal,
  [CrosswordModal.Share]: ShareModal,
}

export default function Layout({
  clueLists,
  inRebusMode,
  doEscape,
  puzzleEscaped,
  isMobile,
}: LayoutProps) {
  const escapeRef = useRef<HTMLDivElement>(null)
  const mobileContainerRef = useRef<HTMLDivElement>(null)
  const isNewsReader = !!window.newsreaderAppPlatform
  const isAndroidNews = window.newsreaderAppPlatform === 'android'
  useSaleBannerHybridSubscribe()

  useEffect(() => {
    if (escapeRef.current && doEscape && !isMobile) {
      escapeRef.current.focus()
      puzzleEscaped && puzzleEscaped()
    }
  }, [doEscape])

  /**
   * Get the layout container's distance from the top of the viewport.
   * This will help us set the Rebus height on desktop.
   */
  const offsetRef = useRef<HTMLDivElement>(null)
  const [offSetTop, setOffSetTop] = useState(0)

  useEffect(() => {
    const offSetTop =
      offsetRef.current?.getBoundingClientRect().top || 0
    setOffSetTop(offSetTop)
  }, [inRebusMode])

  const { willShowDeltaBanner, portalEl } = useDeltaBanner()

  useEffect(() => {
    // set height to pixels to account for nav bar on phone devices
    if (isMobile && !isNewsReader) {
      const setVh = () => {
        const mobileDiv = mobileContainerRef.current
        const headerHeight =
          document.getElementsByClassName('pz-header')[0]
            .clientHeight || 0

        if (mobileDiv) {
          const vh = window.innerHeight - headerHeight // $nav-height-mobile
          mobileDiv.setAttribute('style', `height:${vh}px`)
        }
      }
      setVh()
      const debouncedVh = debounce(setVh, 50)
      window.addEventListener('resize', debouncedVh)
      return () => window.removeEventListener('resize', debouncedVh)
    }
  }, [isMobile, willShowDeltaBanner])

  return (
    <div className="xwd__layout_container" ref={offsetRef}>
      {willShowDeltaBanner &&
        portalEl &&
        createPortal(
          <TopSaleBanner url={document.location.href} isDelta />,
          portalEl,
        )}
      {isMobile && <LandscapeWarning inRebusMode={inRebusMode} />}
      <ModalManager
        getOpenModalName={getOpenModalName}
        getIsClosing={getIsModalClosing}
        modalMap={modalMap}
        onClose={closeModal}
        onRemove={removeModal}
        portalId="portal-game-modals"
      />
      {!isMobile && <PuzzleHeader />}
      <Toolbar />
      <div
        className={classnames({
          'xwd__layout_container--mobile': isMobile,
          'xwd__layout--rebusmode': inRebusMode,
          'xwd__layout_container--mobile-newsreader': isNewsReader,
          'xwd__news-android': isAndroidNews,
        })}
        ref={mobileContainerRef}
      >
        <article
          id="puzzle"
          aria-label="Main Puzzle Layout"
          className={classnames({
            'xwd__layout_puzzle--mobile': isMobile,
            'xwd__layout_puzzle--desktop': !isMobile,
          })}
        >
          <section
            className={classnames({
              xwd__layout_clueBarAndBoard: !isMobile,
              'xwd__mobile_layout--board_container': isMobile,
            })}
            aria-label="Game Board with Clue Bar"
          >
            {!isMobile && <ClueBar />}
            <Board isMobile={isMobile} />
            {isMobile && (
              <span key="breaker" className="xwd__layout_breaker" />
            )}
          </section>
          {clueLists && <ClueListWrapper lists={clueLists} />}
        </article>
        {inRebusMode && <RebusField offset={offSetTop} />}
        {isMobile && (
          <div className="xwd__layout_controls--mobile">
            <ClueBar />
            <Keyboard />
          </div>
        )}
      </div>
      <EditorialContent />
    </div>
  )
}
