import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import {
  CrosswordModal,
  CrosswordModalConfig,
} from '../../../store/state/modal/modalTypes'
import { CLEAR } from '../../../store/state/cells/cellsActions'
import { Scope } from '../../../store/state/cells/cellsTypes'
import { changeUserSetting } from '../../../store/state/user/userActions'

import ConfirmationModal from './ConfirmationModal'

jest.mock('../../../store/thunks/statusThunks', () => ({
  proceedWithAction: (actionType: string, scope: Scope) => ({
    actionType,
    scope,
  }),
}))

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

describe('<ConfirmationModal />', () => {
  const makeStore = (config: Partial<CrosswordModalConfig> = {}) => ({
    dispatch: jest.fn(),
    subscribe: jest.fn(),
    getState: () => ({
      modal: {
        name: CrosswordModal.Confirmation,
        config: {
          needToWarn: true,
          actionType: CLEAR,
          scope: Scope.Puzzle,
          ...config,
        },
      },
    }),
  })

  const close = jest.fn()
  const isClosing = false
  const onRemove = jest.fn()

  const getWrapper = (config: Partial<CrosswordModalConfig> = {}) => {
    const store = makeStore(config)
    const wrapper = mount(
      // @ts-ignore
      <Provider store={store}>
        <ConfirmationModal
          close={close}
          isClosing={isClosing}
          onRemove={onRemove}
        />
      </Provider>,
    )

    return { wrapper, store }
  }

  afterEach(jest.clearAllMocks)

  it('should render warning markup if need to warn', () => {
    const { wrapper } = getWrapper()
    expect(wrapper.find('.xwd__modal-confirm--warning')).toHaveLength(
      1,
    )
    expect(wrapper.find('input')).toHaveLength(1)
  })

  it('should not render warning markup if does not need to warn', () => {
    const { wrapper } = getWrapper({ needToWarn: false })
    expect(wrapper.find('.xwd__modal-confirm--warning')).toHaveLength(
      0,
    )
    expect(wrapper.find('input')).toHaveLength(0)
  })

  it('should pass true to close when cancelling', () => {
    const { wrapper, store } = getWrapper({ needToWarn: false })
    // @ts-ignore
    const { buttons } = wrapper.find('ModalWrapper').props()
    buttons[0].clickHandler()
    expect(close).toHaveBeenCalledTimes(1)
    expect(close).toHaveBeenCalledWith(true)
    expect(store.dispatch).not.toHaveBeenCalled()
  })

  it('should update setting on cancelling when warned', () => {
    const { wrapper, store } = getWrapper()
    // @ts-ignore
    const { buttons } = wrapper.find('ModalWrapper').props()
    buttons[0].clickHandler()
    expect(close).toHaveBeenCalledTimes(1)
    expect(close).toHaveBeenCalledWith(true)
    expect(store.dispatch).toHaveBeenCalledWith(
      changeUserSetting({ suppressDisqualificationWarnings: false }),
    )
  })

  it('should proceed with action and close when confirming', () => {
    const { wrapper, store } = getWrapper()
    // @ts-ignore
    const { buttons } = wrapper.find('ModalWrapper').props()
    buttons[1].clickHandler()
    expect(close).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      actionType: CLEAR,
      scope: Scope.Puzzle,
    })
  })
})
