import React from 'react'
import { useSelector, useDispatch } from 'react-redux'

import { ModalProps } from '../../../../shared/ModalSystem/modalTypes'
import {
  Title,
  Description,
} from '../../../../shared/moments/components/subcomponents'
import { getOpenModalConfig } from '../../../store/state/modal/modalSelectors'
import { changeUserSetting } from '../../../store/state/user/userActions'
import { proceedWithAction } from '../../../store/thunks/statusThunks'
import { CheckBox } from '../formfield/CheckBox'
import ModalWrapper from '../../../../shared/ModalSystem/ModalWrapper'

import getConfirmParams from './getConfirmParams'

export default function ConfirmationModal({
  close,
  isClosing,
  onRemove,
}: ModalProps) {
  const { actionType, scope, needToWarn } = useSelector(
    getOpenModalConfig,
  )

  const { title, text, buttonLabel } = getConfirmParams(
    actionType,
    needToWarn,
  )

  const dispatch = useDispatch()

  const updateDQSetting = (shouldSuppress: boolean) => {
    dispatch(
      changeUserSetting({
        suppressDisqualificationWarnings: shouldSuppress,
      }),
    )
  }

  const closeAndCancel = () => {
    close(true)
    if (needToWarn) {
      updateDQSetting(false)
    }
  }

  const closeAndConfirm = () => {
    // close must happen first, since otherwise it could close the
    // congrats/rats modals which might open in proceedWithAction
    close()
    dispatch(proceedWithAction(actionType, scope))
  }

  const buttons = [
    {
      label: 'Never mind',
      className: 'secondary',
      clickHandler: closeAndCancel,
    },
    {
      label: buttonLabel,
      clickHandler: closeAndConfirm,
    },
  ]

  const keyConfig = {
    Enter: closeAndConfirm,
    NumpadEnter: closeAndConfirm,
    Escape: closeAndCancel,
  }

  const warningMarkup = (
    <div className="xwd__modal-confirm--warning">
      <Title size="medium" text={title} />
      <Description text={text} variant="small" />
    </div>
  )

  const footer = needToWarn ? (
    <div className="xwd__modal-confirm--footer">
      <CheckBox
        name="suppressDisqualificationWarnings"
        label="Don’t show again"
        onChange={(e) => updateDQSetting(e.target.checked)}
      />
    </div>
  ) : null

  return (
    <ModalWrapper
      bodyClassName="xwd__confirmation-modal--wrapper"
      buttons={buttons}
      keyConfig={keyConfig}
      close={close}
      footer={footer}
      isClosing={isClosing}
      onRemove={onRemove}
    >
      {needToWarn ? (
        warningMarkup
      ) : (
        <Description text={text} variant="small" />
      )}
    </ModalWrapper>
  )
}
