import React from 'react'
import { mount, shallow } from 'enzyme'
import { Provider } from 'react-redux'

import ModalWrapper from '../../../../shared/ModalSystem/ModalWrapper'

import CongratsModal from './CongratsModal'
import DefaultCongrats from './DefaultCongrats'
import GuideCongrats from './GuideCongrats'
import GetAppLink from './GetAppLink'
import { secondsToString, formatSolveTime } from './helper'
import StreakMessage from './StreakMessage'

jest.mock('foundation', () => ({
  userType: {
    hasXwd: true,
  },
  abra: {
    getVariant: jest.fn(),
  },
}))

jest.mock('../../../store/state/user/userSelectors', () => ({
  getCurrentUserSettings: () => ({
    showTimer: true,
    soundOn: true,
  }),
}))

jest.mock('../../../store/state/status/statusSelectors', () => ({
  getGoldStarEligibility: () => true,
}))

jest.mock('../../../store/utils/tracker', () => ({
  trackModuleInteraction: jest.fn(),
  trackImpression: jest.fn(),
}))

jest.mock('responsive', () => ({
  isMobile: true,
}))

const mockDispatch = jest.fn()

const mockStore = (config) => {
  return {
    default: jest.fn(),
    subscribe: jest.fn(),
    dispatch: mockDispatch,
    getState: () => ({
      puzzle: {
        data: {
          meta: {
            publicationDate: '10',
            id: config.puzzleId || 0,
            publishStream: 'foo',
          },
        },
      },
      timer: {
        totalElapsedTime: 10,
        sessionStartTimestamp: 5,
      },
      stats: {
        streaks: {
          currentStreak: config?.currentStreak || 0,
          verticalStreaks: config?.verticalStreaks,
        },
      },
    }),
  }
}

describe('CONGRATS MODAL', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('renders congrats modal with two buttons', () => {
    const store = mockStore({})
    const view = mount(
      <Provider store={store}>
        <CongratsModal />
      </Provider>,
    )
    expect(view.find(CongratsModal).length).toBe(1)
    expect(view.find(ModalWrapper).length).toBe(1)
    expect(view.find(ModalWrapper).props().buttons.length).toBe(2)
  })

  it('renders the default congrats modal if puzzle is not a guide puzzle', () => {
    const puzzleId = 15023 // non guide puzzle id
    const store = mockStore({ puzzleId })
    const view = mount(
      <Provider store={store}>
        <CongratsModal />
      </Provider>,
    )
    expect(view.find(DefaultCongrats).length).toBe(1)
    expect(view.find(StreakMessage).length).toBe(0)
  })

  it('renders the streak message if a streak is present', () => {
    const puzzleId = 15023 // non guide puzzle id
    const store = mockStore({ puzzleId, currentStreak: 2 })
    const view = mount(
      <Provider store={store}>
        <CongratsModal />
      </Provider>,
    )
    expect(view.find(DefaultCongrats).length).toBe(1)
    expect(view.find(StreakMessage).length).toBe(1)
    expect(
      view.find('.xwd__congrats-modal--streakLength').text(),
    ).toBe('YOU HAVE A 3-DAY STREAK')
  })

  it('renders the vertical streak message if vertical streak exists', () => {
    const puzzleId = 15023 // non guide puzzle id
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="daily"
          close={jest.fn()}
          dayIdx={1} // monday
          isMini={false}
          streamLabel="daily"
          showTimer
          registeredNonSub={false}
          hasXwd={false}
          isGoldStarEligible
          streakLength={2}
          formattedSolveTime="45:03"
          verticalStreakLength={3}
        />
      </Provider>,
    )
    expect(
      view.find('.xwd__congrats-modal--streakLength').text(),
    ).toBe('4 Mondays in a row')
  })

  it('does not render promotional content for mobile subscribers', () => {
    const puzzleId = 15023 // non guide puzzle id
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={jest.fn()}
          dayIdx={1} // monday
          isMini
          streamLabel="mini"
          showTimer
          registeredNonSub={false}
          hasXwd
          isGoldStarEligible
          streakLength={2}
          formattedSolveTime="45:03"
          verticalStreakLength={3}
        />
      </Provider>,
    )

    const nextGameSuggestionDiv = view.find(
      '.xwd__nextGameSuggestion',
    )
    expect(nextGameSuggestionDiv.length).toBe(0)
  })

  it('renders the promotional content for Spelling Bee for users not logged in playing the mini', () => {
    const puzzleId = 15023 // non guide puzzle id
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={jest.fn()}
          isMini
          showTimer
          registeredNonSub={false}
          hasXwd={false}
          formattedSolveTime="45:03"
        />
      </Provider>,
    )
    expect(
      view.find('.xwd__nextGameSuggestion--gameTitle').text(),
    ).toBe('Spelling Bee')
  })

  it('renders the promotional content for Tiles for registered users', () => {
    const puzzleId = 15023 // non guide puzzle id
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={jest.fn()}
          isMini
          showTimer
          registeredNonSub
          hasXwd={false}
          formattedSolveTime="45:03"
        />
      </Provider>,
    )
    expect(
      view.find('.xwd__promotional-content--minimodalMessage').text(),
    ).toBe('Have you played our new matching game? It’s mesmerizing.')
    expect(view.find('.xwd__promotional-content--link').text()).toBe(
      'Try Tiles >>',
    )
  })

  it('does not render the get app link for newsreader users', () => {
    global.newsreaderAppPlatform = 'android'
    global.localStorage = {}
    const store = mockStore({})
    const view = mount(
      <Provider store={store}>
        <CongratsModal />
      </Provider>,
    )
    expect(view.find(GetAppLink).length).toBe(0)
  })

  it('does not render the get app link for desktop users', () => {
    const puzzleId = 15023
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="daily"
          close={jest.fn()}
          isMini={false}
          streamLabel="daily"
          showTimer
          registeredNonSub={false}
          hasXwd
          isMobile={false}
        />
      </Provider>,
    )
    expect(view.find(GetAppLink).length).toBe(0)
  })

  it('renders guide puzzle modal with button to return to guides', () => {
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <CongratsModal />
      </Provider>,
    )

    expect(view.find(CongratsModal).length).toBe(1)
    expect(view.find('.xwd__modal--button').text()).toBe(
      'Back to the guide',
    )
    expect(view.find('.xwd__modal--button').props().href).toBe(
      'https://www.nytimes.com/guides/crosswords/how-to-solve-a-crossword-puzzle#monday-level-easy',
    )
  })

  it('renders share button and close label on mini win', () => {
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={jest.fn()}
          isMini
          showTimer
          registeredNonSub={false}
          hasXwd={false}
          formattedSolveTime="45:03"
        />
      </Provider>,
    )
    expect(view.find('.secondary').text()).toBe('Share your score')
    expect(view.find('.secondary').text()).not.toBe(
      'Admire your puzzle',
    )

    expect(view.find('.xwd__modal--close').text()).toBe(
      'Back to puzzle',
    )
  })

  it('does not have close label for non-mini puzzles', () => {
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="daily"
          close={jest.fn()}
          isMini={false}
          streamLabel="daily"
          showTimer
          registeredNonSub={false}
          hasXwd
          isMobile={false}
        />
      </Provider>,
    )
    expect(view.find('.xwd__modal--close').text()).toBe('')
  })

  it('share button click should cause modal to close', () => {
    const mockClose = jest.fn()
    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={mockClose}
          isMini
          showTimer
          registeredNonSub={false}
          hasXwd={false}
          formattedSolveTime="45:03"
        />
      </Provider>,
    )

    view.find('.xwd__congrats-modal--shareButton').simulate('click')
    expect(mockClose.mock.calls.length).toEqual(1)
  })

  it('share button click should cause modal to close, call native share api', async () => {
    const mockClose = jest.fn()
    const mockCanShare = jest.fn(() => true)
    const mockShare = jest.fn(() => Promise.resolve())

    global.navigator.canShare = mockCanShare
    global.navigator.share = mockShare

    const oldUA = window.navigator.userAgent // to restore at end of test

    Object.defineProperty(window.navigator, 'userAgent', {
      value: '',
      writable: true,
    })
    navigator.userAgent = 'iPhone'

    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={mockClose}
          isMini
          showTimer
          registeredNonSub={false}
          hasXwd={false}
          formattedSolveTime="45:03"
        />
      </Provider>,
    )

    view.find('.xwd__congrats-modal--shareButton').simulate('click')

    expect(mockCanShare.mock.calls.length).toEqual(1)
    await expect(mockShare.mock.calls.length).toEqual(1)
    expect(mockClose.mock.calls.length).toEqual(1)

    navigator.userAgent = oldUA
  })

  it('share button click should cause modal to close, and NOT call native share api for safari mac', async () => {
    const mockClose = jest.fn()
    const mockCanShare = jest.fn(() => true)
    const mockShare = jest.fn(() => Promise.resolve())

    global.navigator.canShare = mockCanShare
    global.navigator.share = mockShare

    const oldUA = window.navigator.userAgent // to restore at end of test

    Object.defineProperty(window.navigator, 'userAgent', {
      value: '',
      writable: true,
    })
    navigator.userAgent = 'Safari Mac'

    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={mockClose}
          isMini
          showTimer
          registeredNonSub={false}
          hasXwd={false}
          formattedSolveTime="45:03"
        />
      </Provider>,
    )

    view.find('.xwd__congrats-modal--shareButton').simulate('click')

    expect(mockCanShare.mock.calls.length).toEqual(1)
    await expect(mockShare.mock.calls.length).toEqual(0)
    expect(mockClose.mock.calls.length).toEqual(1)

    navigator.userAgent = oldUA
  })

  it('share button click should cause modal to close, native share api not available ', async () => {
    const mockClose = jest.fn()
    const mockCanShare = jest.fn()
    const mockShare = jest.fn()

    global.navigator.canShare = mockCanShare
    global.navigator.share = mockShare

    const oldUA = window.navigator.userAgent // to restore at end of test

    Object.defineProperty(window.navigator, 'userAgent', {
      value: '',
      writable: true,
    })
    navigator.userAgent = 'Firefox'

    const store = mockStore({ puzzleId: 14177 })
    const view = mount(
      <Provider store={store}>
        <DefaultCongrats
          displayAppLink
          appCta="mini"
          close={mockClose}
          isMini
          showTimer
          registeredNonSub={false}
          hasXwd={false}
          formattedSolveTime="45:03"
        />
      </Provider>,
    )

    view.find('.xwd__congrats-modal--shareButton').simulate('click')

    expect(mockCanShare.mock.calls.length).toEqual(1)
    await expect(mockShare.mock.calls.length).toEqual(0)
    expect(mockClose.mock.calls.length).toEqual(1)

    navigator.userAgent = oldUA
  })

  it('builds correct string for time ending in 1 second', () => {
    expect(secondsToString(1)).toBe('1 second')
  })

  it('formats solve time for solves less than 1 minute', () => {
    expect(formatSolveTime(10)).toBe('10 seconds')
  })

  it('formats solve time for solves greater than 1 minute', () => {
    expect(formatSolveTime(90)).toBe('1:30')
  })
})
