import React from 'react'
import { connect } from 'react-redux'
import { compose } from 'redux'

import { isMobile } from 'responsive'
import { withUser } from 'user'
import { abra } from 'foundation'

import { ModalProps } from '../../../../shared/ModalSystem/modalTypes'
import {
  trackModuleInteraction,
  trackImpression,
} from '../../../store/utils/tracker'
import type { CrosswordStoreState } from '../../../store/state/crosswordReducer'
import { getCurrentUserSettings } from '../../../store/state/user/userSelectors'
import { getGoldStarEligibility } from '../../../store/state/status/statusSelectors'
import { getSolveTime } from '../../../store/state/timer/timerSelectors'
import { getPuzzleMetadata } from '../../../store/state/puzzle/puzzleSelectors'
import {
  getStreakLength,
  getStreaks,
} from '../../../store/state/stats/statsSelectors'
import { MetaState } from '../../../store/state/puzzle/puzzleTypes'
import { getPuzzleLabel } from '../../../utils/misc'
import { UserContextProps } from '../../../../../shared/user/UserContext'
import { navigateBack } from '../../../../helpers/navigateBack'

import { guidePuzzleIds, formatSolveTime } from './helper'
import DefaultCongrats from './DefaultCongrats'
import GuideCongrats from './GuideCongrats'

type CongratsModalState = {
  dayIdx: number | null
  appLinkClicked: boolean
}

type CongratsModalUserProps = {
  user: UserContextProps
}

type CongratsModalStateProps = {
  solveTime: number
  showTimer: boolean
  soundOn: boolean
  isGoldStarEligible: boolean
  meta: MetaState | null
  streakLength: number

  /* ignored to proceed with merge commit */
  /* eslint-disable-next-line react/require-default-props */
  streaks?: {
    verticalStreaks: Array<{ length: number; nextDate: string }>
  }
}

type CongratsModalProps = CongratsModalStateProps &
  ModalProps &
  CongratsModalUserProps
class CongratsModal extends React.Component<
  CongratsModalProps,
  CongratsModalState
> {
  constructor(props: CongratsModalProps) {
    super(props)
    this.state = {
      dayIdx: null,
      appLinkClicked: false,
    }
  }

  componentDidMount() {
    const { meta, solveTime, soundOn } = this.props
    const appLinkClicked =
      localStorage.getItem('app-link-clicked') === 'true'
    const dayIdx = new Date().getDay()
    const eventAction =
      meta?.publishStream === 'mini' ? 'mini-page' : 'game-page'
    const strutJingle = document.getElementById('strut')
    this.setState({ dayIdx, appLinkClicked })
    if (soundOn && strutJingle instanceof HTMLAudioElement) {
      strutJingle?.play()
    }

    trackModuleInteraction(
      'complete',
      meta?.id,
      eventAction,
      '100%-complete',
      `${solveTime}`,
    )

    if (meta?.publishStream === 'mini') {
      trackImpression(
        'congrats-modal',
        'mini-page',
        this.shouldDisplayAppLink() ? 'download-app' : undefined,
      )
    }
  }

  shouldDisplayAppLink() {
    const {
      user: {
        userType: { hasXwd },
      },
    } = this.props
    const { appLinkClicked } = this.state
    const isNewsreader = !!window.newsreaderAppPlatform

    const mobileSub = isMobile && hasXwd
    return mobileSub && !appLinkClicked && !isNewsreader
  }

  render() {
    const {
      close,
      solveTime,
      showTimer,
      isGoldStarEligible,
      user: { userType },
      meta,
      streakLength,
      streaks,
      isClosing,
      onRemove,
    } = this.props
    const { dayIdx } = this.state
    const variant = abra.getVariant('GAMES_BackToPlayTab_1222')

    if (!meta) {
      return null
    }

    const {
      publicationDate,
      publishStream,
      title: puzzleTitle,
      id: puzzleId,
    } = meta
    const { hasXwd, isLoggedIn } = userType
    const streamLabel = getPuzzleLabel(publicationDate, publishStream)
    const formattedSolveTime = formatSolveTime(solveTime)
    const isMini = publishStream === 'mini'
    const isGuidePuzzle = guidePuzzleIds.includes(puzzleId)
    const isNewsreader = !!window.newsreaderAppPlatform
    const registeredNonSub = isLoggedIn && !hasXwd
    const displayAppLink = this.shouldDisplayAppLink()

    const appCta =
      isMobile && !isNewsreader ? 'congrats-download-app' : ''

    const verticalStreakLength = () => {
      const verticalStreaks = streaks?.verticalStreaks || []
      if (verticalStreaks.length && dayIdx) {
        return verticalStreaks[dayIdx].length || 0
      }
    }

    const defaultCongratsModal = (
      <DefaultCongrats
        displayAppLink={displayAppLink}
        appCta={appCta}
        close={close}
        dayIdx={dayIdx}
        isMini={isMini}
        streamLabel={streamLabel}
        showTimer={showTimer}
        formattedSolveTime={formattedSolveTime}
        solveTime={solveTime}
        publicationDate={publicationDate}
        registeredNonSub={registeredNonSub}
        hasXwd={hasXwd}
        isMobile={isMobile}
        isNewsreader={isNewsreader}
        isGoldStarEligible={isGoldStarEligible}
        streakLength={streakLength}
        verticalStreakLength={verticalStreakLength()}
        navigateBack={() => navigateBack(variant)}
        isClosing={isClosing}
        onRemove={onRemove}
      />
    )

    const guideCongratsModal = (
      <GuideCongrats
        displayAppLink={displayAppLink}
        appCta={appCta}
        close={close}
        showTimer={showTimer}
        puzzleId={puzzleId}
        puzzleTitle={puzzleTitle || ''}
        formattedSolveTime={formattedSolveTime}
        isClosing={isClosing}
        onRemove={onRemove}
      />
    )

    return isGuidePuzzle ? guideCongratsModal : defaultCongratsModal
  }
}

const mapState = (
  state: CrosswordStoreState,
): CongratsModalStateProps => {
  const settings = getCurrentUserSettings(state)
  const meta = getPuzzleMetadata(state)
  const { showTimer, soundOn } = settings
  const isGoldStarEligible = getGoldStarEligibility(state)
  const solveTime = getSolveTime(state)
  const streakLength = getStreakLength(state)
  const streaks = getStreaks(state)

  return {
    solveTime,
    showTimer,
    soundOn,
    isGoldStarEligible,
    meta,
    streakLength,
    streaks,
  }
}

type C = (props: ModalProps) => JSX.Element
export default compose<C>(withUser, connect(mapState))(CongratsModal)
