import React, { useState, useEffect } from 'react'
import { useDispatch } from 'react-redux'

import ModalWrapper from '../../../../shared/ModalSystem/ModalWrapper'
import PuzzleIcon from '../../PuzzleIcon/PuzzleIcon'
import { Title } from '../../../../shared/moments/components/subcomponents'
import { CrosswordModal } from '../../../store/state/modal/modalTypes'
import { openModal } from '../../../store/state/modal/modalActions'
import { trackModuleInteraction } from '../../../store/utils/tracker'
import { getShareInfo } from '../ShareModal/ShareModal'
import type { ModalProps } from '../../../../shared/ModalSystem/modalTypes'

import PromotionalContent from './Promotion/PromotionalContent'
import StreakMessage from './StreakMessage'
import GetAppLink from './GetAppLink'

type DefaultCongratsProps = ModalProps & {
  displayAppLink: boolean
  appCta: string
  dayIdx: number | null
  isMini: boolean
  streamLabel: string
  showTimer: boolean
  registeredNonSub: boolean
  hasXwd: boolean
  isGoldStarEligible: boolean
  streakLength: number
  formattedSolveTime: string
  verticalStreakLength?: number
  isMobile: boolean
  isNewsreader: boolean
  navigateBack: () => void
  solveTime: number
  publicationDate: string
}

export default function DefaultCongrats({
  displayAppLink,
  appCta,
  close,
  dayIdx,
  isMini,
  streamLabel,
  showTimer,
  registeredNonSub,
  hasXwd,
  isGoldStarEligible,
  formattedSolveTime,
  streakLength,
  verticalStreakLength = 0,
  isMobile,
  isNewsreader = false,
  publicationDate,
  solveTime,
  navigateBack,
  isClosing,
  onRemove,
}: DefaultCongratsProps) {
  const showPromotionalContent = !hasXwd
  const timeString = (
    <span>
      in <span className="xwd__bold">{formattedSolveTime}</span>
    </span>
  )

  const [canNavigatorShare, setCanNavigatorShare] = useState(false)

  useEffect(() => {
    // this is just faux data to suss out if the method is supported
    const shareData = {
      title: 'NYTimes',
      text: 'Test share',
      url: 'https://www.nytimes.com',
    }

    if (navigator.canShare && navigator.canShare(shareData)) {
      setCanNavigatorShare(true)
    }
  }, [])

  const dispatch = useDispatch()

  const handleShareClick = () => {
    const isIOS = /iPad|iPhone/.test(navigator.userAgent)

    if (canNavigatorShare && isIOS) {
      const { shareUrl } = getShareInfo(publicationDate, solveTime)
      navigator
        .share({
          title: 'I solved the daily mini!',
          url: `${shareUrl}&smid=na-share`,
        })
        .then(() => {
          trackModuleInteraction(
            'share-tools',
            undefined,
            'native-share',
            'share',
          )
        })
        .catch((error) =>
          console.warn('Sharing Mini time failed', error),
        )

      close()
    } else {
      close()
      dispatch(openModal(CrosswordModal.Share))
    }
  }

  const actionButtons = [
    isMini
      ? {
          label: 'Share your score',
          clickHandler: handleShareClick,
          className: 'secondary xwd__congrats-modal--shareButton',
        }
      : {
          label: 'Admire your puzzle',
          className: 'secondary',
        },
    {
      label: 'View all games',
      clickHandler: navigateBack,
    },
  ]

  const streamSpecificMessaging =
    streamLabel && streamLabel.toLowerCase() === 'mini' ? (
      <div className="xwd__center xwd__congrats-modal--message">
        You solved{' '}
        <span className="xwd__bold">The {streamLabel}</span>
        <br /> {showTimer && timeString}.
      </div>
    ) : (
      <div className="xwd__center xwd__congrats-modal--message">
        You solved a <span className="xwd__bold">{streamLabel}</span>
        <br />
        Crossword {showTimer && timeString}.
      </div>
    )

  const defaultMessaging = (
    <div className="xwd__center">
      You solved the crossword
      <br />
      {showTimer && timeString}.
    </div>
  )

  const footer = (
    <>
      {displayAppLink && <GetAppLink appCta={appCta} />}
      {showPromotionalContent && (
        <PromotionalContent
          isMini={isMini}
          appCta={appCta}
          registeredNonSub={registeredNonSub}
          hasXwd={hasXwd}
        />
      )}
    </>
  )

  const showFooter = isMini || !hasXwd || (isMobile && !isNewsreader)
  const isIOSNewsreader = window.newsreaderAppPlatform === 'ios'

  return (
    <ModalWrapper
      bodyClassName="xwd__congrats-modal"
      close={close}
      buttons={isMobile ? actionButtons.reverse() : actionButtons}
      buttonsWrapperClassName="xwd__congrats-modal--buttons-wrapper"
      closeLabel={isMini ? 'Back to puzzle' : undefined}
      footer={showFooter && footer}
      isClosing={isClosing}
      onRemove={onRemove}
      containerClassName={
        isIOSNewsreader ? 'container-with-inset' : ''
      }
    >
      <div className="xwd__congrats-modal--content">
        <PuzzleIcon isMini={isMini} isGold={isGoldStarEligible} />
        <Title size="large" text="Congratulations!" />
        {streamLabel ? streamSpecificMessaging : defaultMessaging}
        {isGoldStarEligible &&
          (streakLength > 0 || verticalStreakLength > 0) && (
            <StreakMessage
              streakLength={streakLength}
              verticalStreakLength={verticalStreakLength}
              dayIdx={dayIdx || 0}
            />
          )}
      </div>
    </ModalWrapper>
  )
}
