import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import { MetaState } from '../../../store/state/puzzle/puzzleTypes'

import PrintOptionsModal from './PrintOptionsModal'

const defaultMeta: MetaState = {
  category: 0,
  constructors: ['Dan Harris'],
  copyright: '2021',
  editor: 'Will Shortz',
  id: 19658,
  lastUpdated: '2021-10-25 20:16:32 +0000 UTC',
  publicationDate: '2021-11-03',
  relatedContent: {
    text: 'Read and comment on the Wordplay blog',
    url: 'https://www.nytimes.com/2021/11/02/crosswords/daily-puzzle-2021-11-03.html',
  },
  goldStarCutoff: '2021-11-03T23:59:59-07:00',
  publishStream: 'daily',
  subcategory: 0,
}

const defaultPref = {
  layout: 'large_print',
  opacity: 100,
}

const dispatch = jest.fn()

const makeStore = (printOptions: any, meta = defaultMeta) => {
  return {
    getState: () => ({
      puzzle: {
        data: {
          meta,
        },
      },
      user: {
        printPrefs: {
          opacity: printOptions.opacity,
          layout: printOptions.layout,
        },
      },
      printOptions: {
        version: printOptions.version,
        showBlack: printOptions.showBlack,
        showSpoiler: printOptions.showSpoiler,
      },
    }),
    dispatch,
    subscribe: jest.fn(),
  }
}

const createWrapper = (printOptions: any, puzzle?: any) => {
  const store = makeStore(printOptions, puzzle)
  return mount(
    // @ts-ignore
    <Provider store={store}>
      <PrintOptionsModal
        close={jest.fn()}
        isClosing={false}
        onRemove={jest.fn()}
      />
    </Provider>,
  )
}

describe('Print Options Modal', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('renders the print modal', () => {
    const wrapper = createWrapper({
      version: 'puzzle',
      showBlack: false,
      showSpoiler: false,
      ...defaultPref,
    })
    expect(
      wrapper.find('div.xwd__print-modal--printModalContent'),
    ).toHaveLength(1)
  })
  it('dispatches change event to change layout setting', () => {
    const wrapper = createWrapper({
      version: 'puzzle',
      showBlack: false,
      showSpoiler: false,
      ...defaultPref,
    })

    const largePrint = wrapper
      .find('input#large_print')
      .simulate('change')

    const standardPrint = wrapper
      .find('input#standard')
      .simulate('change')

    const leftPrint = wrapper
      .find('input#southpaw')
      .simulate('change')

    expect(dispatch).toHaveBeenCalledTimes(3)
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      payload: { layout: 'large_print' },
      type: 'crossword/user/CHANGE_PRINT_LAYOUT',
    })
    expect(dispatch).toHaveBeenNthCalledWith(2, {
      payload: { layout: 'standard' },
      type: 'crossword/user/CHANGE_PRINT_LAYOUT',
    })
    expect(dispatch).toHaveBeenNthCalledWith(3, {
      payload: { layout: 'southpaw' },
      type: 'crossword/user/CHANGE_PRINT_LAYOUT',
    })
  })
  it('dispatches changes to print option settings', () => {
    const wrapper = createWrapper({
      version: 'puzzle',
      showBlack: false,
      showSpoiler: false,
      ...defaultPref,
    })

    const puzzleVersion = wrapper
      .find('input#puzzle')
      .simulate('change')

    const solutionVersion = wrapper
      .find('input#solution')
      .simulate('change')

    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: 'crossword/printOptions/CHANGE_PRINT_OPTION',
      payload: {
        version: 'puzzle',
      },
    })

    expect(dispatch).toHaveBeenNthCalledWith(2, {
      type: 'crossword/printOptions/CHANGE_PRINT_OPTION',
      payload: {
        version: 'solution',
      },
    })
  })

  it('should show option for news paper version if puzzle has both versions', () => {
    const wrapper = createWrapper({
      version: 'puzzle',
      showBlack: false,
      showSpoiler: false,
      ...defaultPref,
    })
    expect(wrapper.html()).toContain('Newspaper Version')
  })

  it('should not show news paper version option if the puzzle doesnt have a category set', () => {
    const puz = {
      constructors: ['Foo'],
      copyright: '2021',
      editor: 'Will Shortz',
      id: 19658,
      lastUpdated: '2021-10-25 20:16:32 +0000 UTC',
      publicationDate: '',
      relatedContent: {
        text: '',
        url: '',
      },
      goldStarCutoff: '',
      publishStream: 'daily',
      subcategory: 0,
      stream: '',
    }

    const wrapper = createWrapper(
      {
        version: 'puzzle',
        showBlack: false,
        showSpoiler: false,
        ...defaultPref,
      },
      puz,
    )
    expect(wrapper.find('input#newspaper')).toBeFalsy
  })
})
