import React from 'react'
import { useSelector, useDispatch } from 'react-redux'
import cn from 'classnames'

import ModalWrapper from '../../../../shared/ModalSystem/ModalWrapper'
import { ModalProps } from '../../../../shared/ModalSystem/modalTypes'
import {
  getPrintOptions,
  getPrintPath,
  getPuzzleDate,
  getPrintLayoutsUnavailable,
  getPrintOpacityUnavailable,
  getSolutionIsAvailable,
  getHasBothVersions,
} from '../../../store/state/printOptions/printOptionsSelectors'
import { RadioButton } from '../formfield/RadioButton'
import { CheckBox } from '../formfield/CheckBox'
import { PrintOptionsState } from '../../../store/state/printOptions/printOptionsTypes'
import { changePrintOption } from '../../../store/state/printOptions/printOptionActions'
import {
  changePrintLayout,
  changePrintOpacity,
} from '../../../store/state/user/userActions'
import { PrintLayoutOptions } from '../../../store/state/user/userTypes'
import { getPuzzleMetadata } from '../../../store/state/puzzle/puzzleSelectors'
import { getPrintPrefs } from '../../../store/state/user/userSelectors'
import { Title } from '../../../../shared/moments/components/subcomponents'

export default function PrintOptionsModal({
  close,
  isClosing,
  onRemove,
}: ModalProps) {
  const dispatch = useDispatch()

  const { version, showBlack, showSpoiler } =
    useSelector(getPrintOptions)
  const { opacity, layout } = useSelector(getPrintPrefs)
  const puzzle = useSelector(getPuzzleMetadata)
  const printPath = useSelector(getPrintPath)
  const puzzleDate = useSelector(getPuzzleDate)

  const printLayoutsUnavailable = useSelector(
    getPrintLayoutsUnavailable,
  )

  const printOpacityUnavailable = useSelector(
    getPrintOpacityUnavailable,
  )
  const hasBothVersions = useSelector(getHasBothVersions)
  const solutionIsAvailable = useSelector(getSolutionIsAvailable)
  const hasLayoutOptions = puzzle?.category === 0

  const handlePrintOptionChange = (
    option: Partial<PrintOptionsState>,
  ) => dispatch(changePrintOption(option))

  const sendChangeToStore = (
    event: React.ChangeEvent<HTMLInputElement>,
  ) => {
    const { target } = event
    const { checked, name, type, value } = target
    const isRadio = type === 'radio'
    const sendValue = isRadio ? value : checked
    handlePrintOptionChange({ [name]: sendValue })
  }

  const updateLayout = (layoutVal: PrintLayoutOptions) => {
    const newLayout = layoutVal || PrintLayoutOptions.standard
    dispatch(changePrintLayout(newLayout))
  }

  const updateOpacity = (checked: boolean) => {
    const newOpacity = checked ? 30 : 100
    dispatch(changePrintOpacity(newOpacity))
  }

  const clickHandler = () =>
    window.open(`https://www.nytimes.com${printPath}`)

  return (
    <ModalWrapper
      buttons={[{ label: 'Print', clickHandler }]}
      keyConfig={{ Enter: clickHandler, NumpadEnter: clickHandler }}
      ignoreBackgroundClick
      close={close}
      isClosing={isClosing}
      onRemove={onRemove}
    >
      <Title size="medium" text="Print" />
      <div className="xwd__print-modal--printModalContent">
        {puzzle?.formatType && (
          <div className="xwd__print-modal--puzzleDetails">
            <div className="xwd__print-modal--puzzleTitle">
              {puzzle.title || 'Daily Puzzle'}
            </div>
            <div className="xwd__print-modal--puzzleDate">
              {puzzleDate}
            </div>
          </div>
        )}
        <div className="xwd__print-modal--printOptions">
          <div className="xwd__print-modal--section">
            <div className="xwd__print-modal--left">
              <RadioButton
                type="radio"
                id="puzzle"
                name="version"
                value="puzzle"
                checked={version === 'puzzle'}
                onChange={sendChangeToStore}
                tabIndex={0}
                label="Puzzle"
              />
              {hasLayoutOptions && (
                <div
                  className={cn('xwd__print-modal--inset', {
                    'xwd__print-modal--disabled':
                      printLayoutsUnavailable,
                  })}
                >
                  <RadioButton
                    type="radio"
                    id="standard"
                    name="layout"
                    value="standard"
                    checked={layout === 'standard'}
                    onChange={() =>
                      updateLayout(PrintLayoutOptions.standard)
                    }
                    disabled={printLayoutsUnavailable}
                    label="Standard Layout"
                  />
                  <RadioButton
                    type="radio"
                    id="large_print"
                    name="layout"
                    value="large_print"
                    checked={layout === 'large_print'}
                    onChange={() =>
                      updateLayout(PrintLayoutOptions.large_print)
                    }
                    disabled={printLayoutsUnavailable}
                    label="Large Print"
                  />
                  <RadioButton
                    type="radio"
                    id="southpaw"
                    name="layout"
                    value="southpaw"
                    checked={layout === 'southpaw'}
                    onChange={() =>
                      updateLayout(PrintLayoutOptions.southpaw)
                    }
                    disabled={printLayoutsUnavailable}
                    label="Left-handed"
                  />
                </div>
              )}

              <RadioButton
                type="radio"
                id="solution"
                name="version"
                value="solution"
                checked={version === 'solution'}
                onChange={sendChangeToStore}
                disabled={!solutionIsAvailable}
                label="Solution"
                subText="(available Thursday 10 p.m. ET)"
                showSubText={!solutionIsAvailable}
                subTextClass="xwd__print-modal--subscript"
              />
            </div>

            <div
              className={cn('xwd__print-modal--cellDarkness', {
                disabled: printOpacityUnavailable,
              })}
            >
              <CheckBox
                onChange={(e) => updateOpacity(e.target.checked)}
                type="checkbox"
                id="opacity"
                name="opacity"
                checked={opacity < 100}
                disabled={printOpacityUnavailable}
                aria-disabled={printOpacityUnavailable}
                label="Ink Saver"
              />
              <div>
                <div className="xwd__print-modal--opacityIcon">
                  <div className="xwd__print-modal--opacityReference" />
                  <div
                    className="xwd__print-modal--userOpacity"
                    style={{ opacity: opacity / 100 }}
                  />
                </div>
              </div>
            </div>
          </div>
          {hasBothVersions && (
            <div className="xwd__print-modal--section">
              <RadioButton
                type="radio"
                id="newspaper"
                name="version"
                value="newspaper"
                checked={version === 'newspaper'}
                onChange={sendChangeToStore}
                label="Newspaper Version"
              />
            </div>
          )}
        </div>
      </div>
    </ModalWrapper>
  )
}
