import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import {
  CrosswordSettings,
  desktopSettings,
} from '../../../store/state/user/userTypes'
import { useShortzMode } from '../../providers/ShortzModeProvider'

import SettingsModal from './SettingsModal'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

jest.mock('../../providers/ShortzModeProvider', () => ({
  useShortzMode: jest.fn(),
}))

type ConfigSettings = {
  settings: CrosswordSettings
}

const close = jest.fn()
const dispatch = jest.fn()
const makeStore = (config: ConfigSettings) => {
  return {
    getState: () => ({
      user: {
        settings: config.settings,
      },
    }),
    dispatch,
    subscribe: jest.fn(),
  }
}

const createWrapper = (settings: ConfigSettings) => {
  const store = makeStore(settings)
  return mount(
    // @ts-ignore
    <Provider store={store}>
      <SettingsModal
        close={close}
        isClosing={false}
        onRemove={jest.fn()}
      />
    </Provider>,
  )
}

describe('Settings Modal', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })
  it('renders the settings modal', () => {
    const config = {
      settings: {
        skipFilled: true,
        backspaceAcrossWords: false,
        skipPenciled: false,
        autoAdvance: false,
        jumpBack: false,
        soundOn: false,
        showTimer: false,
        suppressDisqualificationWarnings: true,
        showMilestones: true,
        spaceTriggers: 'toggle' as const,
        onSwitch: 'stay',
        suppressAutocheckNotice: false,
        speedUpSyncInterval: false,
        forceGoldStarEligibility: false,
      },
    }

    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: false,
    })
    const wrapper = createWrapper(config)
    expect(wrapper.find('SettingsShortzMode')).toHaveLength(0)
    expect(wrapper.html()).toContain('Puzzle Settings')
    expect(wrapper.html()).toContain('Restore defaults')
    expect(wrapper.html()).toContain('Save and close')
  })

  it('renders the keyboard section on desktop', () => {
    const config = {
      settings: {
        skipFilled: true,
        backspaceAcrossWords: false,
        skipPenciled: false,
        autoAdvance: false,
        jumpBack: false,
        soundOn: false,
        showTimer: false,
        suppressDisqualificationWarnings: true,
        showMilestones: true,
        spaceTriggers: 'toggle' as const,
        onSwitch: 'stay',
        suppressAutocheckNotice: false,
        speedUpSyncInterval: false,
        forceGoldStarEligibility: false,
      },
    }

    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: false,
    })

    const wrapper = createWrapper(config)
    expect(wrapper.find('SettingsKeyboard')).toHaveLength(1)
  })

  it('disables restore defaults button when settings are at the default', () => {
    const config = {
      settings: desktopSettings,
    }

    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: false,
    })

    const wrapper = createWrapper(config)
    expect(wrapper.find('.secondary').props().disabled).toBe(true)
  })

  it('updates skipPenciled to false when skipFilled is false', () => {
    const config = {
      settings: {
        ...desktopSettings,
        skipPenciled: true,
        skipFilled: false,
      },
    }

    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: false,
    })

    createWrapper(config)

    expect(dispatch).toHaveBeenNthCalledWith(1, {
      payload: { skipPenciled: false },
      type: 'crossword/user/CHANGE_SETTING',
    })
  })

  it('dispatches when any form change occurs', () => {
    const config = {
      settings: desktopSettings,
    }

    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: false,
    })

    const wrapper = createWrapper(config)
    const form = wrapper.find('form')
    const settingsOptions = [
      'backspaceAcrossWords',
      'autoAdvance',
      'jumpBack',
      'soundOn',
      'showTimer',
      'suppressDisqualificationWarnings',
      'showMilestones',
      'spaceTriggers',
      'onSwitch',
      'suppressAutocheckNotice',
      'speedUpSyncInterval',
      'forceGoldStarEligibility',
    ]

    settingsOptions.forEach((el) => {
      form.simulate('change', {
        target: {
          checked: true,
          name: el,
        },
      })
    })
    expect(dispatch).toHaveBeenCalledTimes(settingsOptions.length)
  })

  it('displays shortz mode section for shortz mode users', () => {
    const config = {
      settings: desktopSettings,
    }

    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: true,
    })
    const wrapper = createWrapper(config)
    expect(wrapper.find('SettingsShortzMode')).toHaveLength(1)
  })
})
