import React from 'react'
import { useSelector, useDispatch } from 'react-redux'

import { isMobile } from 'responsive'

import { ModalProps } from '../../../../shared/ModalSystem/modalTypes'
import {
  changeUserSetting,
  resetToDefaults,
} from '../../../store/state/user/userActions'
import {
  mobileSettings,
  desktopSettings,
  CrosswordSettings,
} from '../../../store/state/user/userTypes'
import { getCurrentUserSettings } from '../../../store/state/user/userSelectors'
import ModalWrapper from '../../../../shared/ModalSystem/ModalWrapper'
import { keysMatch } from '../../../store/utils/misc'
import { Title } from '../../../../shared/moments/components/subcomponents'
import { useShortzMode } from '../../providers/ShortzModeProvider'

import SettingsKeyboard from './sections/SettingsKeyboard'
import SettingsPreword from './sections/SettingsPreword'
import SettingsIntraword from './sections/SettingsIntraword'
import SettingsPostword from './sections/SettingsPostword'
import SettingsOther from './sections/SettingsOther'
import SettingsShortzMode from './sections/SettingsShortzMode'
import ShortzToggle from './ShortzToggle'

export default function SettingsModal({
  close,
  isClosing,
  onRemove,
}: ModalProps) {
  const { isErsatzShortz, inShortzMode } = useShortzMode()
  const settings = useSelector(getCurrentUserSettings)
  const {
    skipFilled,
    backspaceAcrossWords,
    skipPenciled,
    autoAdvance,
    jumpBack,
    soundOn,
    showTimer,
    suppressDisqualificationWarnings,
    showMilestones,
    spaceTriggers,
    onSwitch,
    suppressAutocheckNotice,
    speedUpSyncInterval,
    forceGoldStarEligibility,
  } = settings
  const dispatch = useDispatch()

  const changeHandler = (
    name: string,
    selectionInput: string | boolean,
  ) => {
    dispatch(changeUserSetting({ [name]: selectionInput }))
  }

  const sendChangeToStore = (
    event: React.ChangeEvent<HTMLFormElement>,
  ) => {
    const { target } = event
    const { checked, name, type, value } = target
    const selectionInput = type === 'radio' ? value : checked
    changeHandler(name, selectionInput)
  }

  const allSettingsAtDefault = () => {
    const defaultSettings = isMobile
      ? mobileSettings
      : desktopSettings

    const settingNames = Object.keys(defaultSettings) as Array<
      keyof CrosswordSettings
    >

    return keysMatch(defaultSettings, settings, settingNames)
  }

  const buttons = [
    {
      label: 'Save and close',
      clickHandler: () => close(),
    },
    {
      label: 'Restore defaults',
      disabled: allSettingsAtDefault(),
      className: 'pz-moment__button secondary',
      clickHandler: () => dispatch(resetToDefaults(isMobile)),
    },
  ]

  return (
    <ModalWrapper
      bodyClassName="xwd__settings-modal--wrapper"
      close={close}
      keyConfig={{ Escape: () => close() }}
      buttons={buttons}
      buttonsWrapperClassName="xwd__settings-btns--wrapper"
      isClosing={isClosing}
      onRemove={onRemove}
    >
      <div className="xwd__settings-modal--title-container">
        <Title size="medium" text="Puzzle Settings" />
        {isErsatzShortz && <ShortzToggle />}
      </div>
      <form
        id="settings-panel"
        className="xwd__settings-modal--form"
        onChange={sendChangeToStore}
      >
        <div className="xwd__settings-modal--column">
          {!isMobile && (
            <SettingsKeyboard
              onSwitch={onSwitch}
              spaceTriggers={spaceTriggers}
            />
          )}
          <SettingsPreword
            backspaceAcrossWords={backspaceAcrossWords}
          />
          <SettingsIntraword
            subSettingsHandler={changeHandler}
            skipFilled={skipFilled}
            skipPenciled={skipPenciled}
          />
        </div>
        <div className="xwd__settings-modal--column">
          <SettingsPostword
            jumpBack={jumpBack}
            autoAdvance={autoAdvance}
          />
          <SettingsOther
            soundOn={soundOn}
            showTimer={showTimer}
            suppressDq={suppressDisqualificationWarnings}
            showMilestones={showMilestones}
          />
          {inShortzMode && (
            <SettingsShortzMode
              speedUpSyncInterval={speedUpSyncInterval || false}
              forceGoldStarEligibility={
                forceGoldStarEligibility || false
              }
              suppressAutocheckNotice={
                suppressAutocheckNotice || false
              }
            />
          )}
        </div>
      </form>
    </ModalWrapper>
  )
}
