import React, { useState, useRef, useEffect } from 'react'
import cn from 'classnames'
import { useSelector } from 'react-redux'
import md5 from 'blueimp-md5'

import { ModalProps } from '../../../../shared/ModalSystem/modalTypes'
import { getPublicationDate } from '../../../store/state/puzzle/puzzleSelectors'
import { getSolveTime } from '../../../store/state/timer/timerSelectors'
import { Title } from '../../../../shared/moments/components/subcomponents'
import { trackModuleInteraction } from '../../../store/utils/tracker'
import ModalWrapper from '../../../../shared/ModalSystem/ModalWrapper'

const SHARE_URL_BASE =
  'https://www.nytimes.com/badges/games/mini.html'
const IMAGE_URL_BASE = 'https://www.nytimes.com/badges/games/mini.png'

// Generates a URL suitable for sharing containing the tracking param (smid)
export const encodeUrlWithSMID = (url: string, smid: string) => {
  if (!url || !smid) {
    return url
  }
  const sep = url.indexOf('?') === -1 ? '?' : '&'
  return encodeURIComponent(`${url}${sep}smid=${smid}`)
}

export const getShareInfo = (
  publicationDate: string | undefined,
  solveTime: number,
) => {
  const baseParams = `d=${publicationDate}&t=${solveTime}`
  const hash = md5(baseParams)

  // https://www.nytimes.com/badges/games/mini.html?d=2022-04-28&c=nohash&t=50
  const shareUrl = `${SHARE_URL_BASE}?${baseParams}&c=${hash}`
  const shareImageSrc = `${IMAGE_URL_BASE}?${baseParams}&c=${hash}`

  return {
    shareUrl,
    shareImageSrc,
  }
}

const SMID = {
  twitter: 'tw-share',
  facebook: 'fb-share',
  wa: 'wa-share',
  reddit: 're-share',
  link: 'url-share',
  telegram: 'tel-share',
}

const fbUrlEncode = encodeURIComponent('https://www.facebook.com/')

const ShareModal = ({ close, isClosing, onRemove }: ModalProps) => {
  const [shouldShowCopyText, setShouldShowCopyText] = useState(false)
  const buttonRef = useRef<HTMLButtonElement>(null)
  const timeoutRef = React.useRef<null | NodeJS.Timeout>(null)

  const solveTime = useSelector(getSolveTime)
  const publicationDate = useSelector(getPublicationDate)

  const { shareUrl, shareImageSrc } = getShareInfo(
    publicationDate,
    solveTime,
  )

  const textEncoded = encodeURIComponent('I solved the daily mini!')

  // clear any remaining timeout on unmount
  useEffect(() => {
    return () => {
      clearTimeout(timeoutRef.current as NodeJS.Timeout)
    }
  }, [])

  const handleCopyClick = () => {
    if (buttonRef.current) {
      const copyLinkUrl = `${shareUrl}&smid=${SMID.link}`

      // set up the element
      const shareInput = document.createElement('input')
      shareInput.type = 'text'
      shareInput.value = copyLinkUrl

      // append and copy the link
      buttonRef.current.append(shareInput)
      shareInput.select()
      document.execCommand('copy')

      // remove the element
      shareInput.remove()
      setShouldShowCopyText(true)

      trackModuleInteraction(
        'share-tools',
        undefined,
        'share-url',
        'Permalink',
      )

      clearTimeout(timeoutRef.current as NodeJS.Timeout)

      timeoutRef.current = setTimeout(() => {
        setShouldShowCopyText(false)
      }, 5000)
    }
  }

  return (
    <ModalWrapper
      close={close}
      isClosing={isClosing}
      onRemove={onRemove}
      bodyClassName="xwd__share-modal"
    >
      <Title size="large" text="Share Your Victory" />
      <div className="xwd__share-modal_toutImageWrapper">
        <img
          src={shareImageSrc}
          alt="social media achievement"
          className="xwd__share-modal_toutImage"
        />
      </div>
      <ul className="xwd__share-modal_shareNetworks">
        <li className="xwd__share-modal_shareItem">
          <button
            ref={buttonRef}
            type="button"
            aria-label="Copy Link"
            id="copyUrlId"
            onClick={handleCopyClick}
            className={cn('xwd__share-modal_shareLinkButton', {
              'xwd__share-modal_copiedLink': shouldShowCopyText,
            })}
          >
            <i className={cn('xwd__share-modal_shareIcon', 'link')} />
            <div className="xwd__share-modal_shareLink">
              {shouldShowCopyText ? 'Copied' : 'Copy Link'}
            </div>
          </button>
        </li>

        <li className="xwd__share-modal_shareItem">
          <a
            href={`https://www.facebook.com/dialog/feed?app_id=9869919170&link=${encodeUrlWithSMID(
              shareUrl,
              SMID.facebook,
            )}&name=${textEncoded}&redirect_uri=${fbUrlEncode}`}
            target="_blank"
            aria-label="Share on Facebook"
            rel="noopener noreferrer"
            onClick={() => {
              trackModuleInteraction(
                'share-tools',
                undefined,
                'share-facebook',
                'Facebook',
              )
            }}
          >
            <i
              className={cn('xwd__share-modal_shareIcon', 'facebook')}
            />
            <div className="xwd__share-modal_shareLink">Facebook</div>
          </a>
        </li>

        <li className="xwd__share-modal_shareItem">
          <a
            href={`https://twitter.com/intent/tweet?url=${encodeUrlWithSMID(
              shareUrl,
              SMID.twitter,
            )}&text=${textEncoded}`}
            target="_blank"
            aria-label="Share on Twitter"
            rel="noopener noreferrer"
            onClick={() => {
              trackModuleInteraction(
                'share-tools',
                undefined,
                'share-twitter',
                'Twitter',
              )
            }}
          >
            <i
              className={cn('xwd__share-modal_shareIcon', 'twitter')}
            />
            <div className="xwd__share-modal_shareLink">Twitter</div>
          </a>
        </li>

        <li className="xwd__share-modal_shareItem">
          <a
            href={`https://t.me/share/url?url=${encodeUrlWithSMID(
              shareUrl,
              SMID.telegram,
            )}&text=${textEncoded} via @nytimes`}
            target="_blank"
            aria-label="Share on Telegram"
            rel="noopener noreferrer"
            onClick={() => {
              trackModuleInteraction(
                'share-tools',
                undefined,
                'share-telegram',
                'Telegram',
              )
            }}
          >
            <i
              className={cn('xwd__share-modal_shareIcon', 'telegram')}
            />
            <div className="xwd__share-modal_shareLink">Telegram</div>
          </a>
        </li>
        <li className="xwd__share-modal_shareItem">
          <a
            href={`https://api.whatsapp.com/send?text=${textEncoded}%20${encodeUrlWithSMID(
              shareUrl,
              SMID.wa,
            )}`}
            target="_blank"
            aria-label="Share on WhatsApp"
            rel="noopener noreferrer"
            onClick={() => {
              trackModuleInteraction(
                'share-tools',
                undefined,
                'share-whatsapp',
                'WhatsApp',
              )
            }}
          >
            <i
              className={cn('xwd__share-modal_shareIcon', 'whatsapp')}
            />
            <div className="xwd__share-modal_shareLink">Whatsapp</div>
          </a>
        </li>
      </ul>
    </ModalWrapper>
  )
}

export default ShareModal
