import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import {
  trackImpression,
  trackModuleInteraction,
} from '../../../store/utils/tracker'
import { getIsMini } from '../../../store/state/puzzle/puzzleSelectors'
import * as useAbra from '../../../../../shared/abra/useAbra'

import StartModal from './StartModal'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  getQueryParams: () => false,
}))

jest.mock('responsive', () => ({
  isMobile: true,
}))

jest.mock('../../PuzzleHeader/Notes', () => 'Notes')
jest.mock(
  '../../../../shared/ModalSystem/ModalWrapper',
  () => 'ModalWrapper',
)
jest.mock('../../../store/thunks/transientThunks', () => ({
  setReady: () => ({ type: 'READY' }),
}))
jest.mock('../../../store/utils/tracker', () => ({
  trackModuleInteraction: jest.fn(),
  trackImpression: jest.fn(),
}))

jest.mock('../../../store/state/puzzle/puzzleSelectors', () => ({
  ...jest.requireActual(
    '../../../store/state/puzzle/puzzleSelectors',
  ),
  getIsMini: jest.fn(),
}))

describe('<StartModal />', () => {
  const close = jest.fn()
  const onRemove = jest.fn()
  const dispatch = jest.fn()
  const testVariant = '1_icon'

  const getWrapper = () => {
    const store = {
      dispatch,
      subscribe: jest.fn(),
      getState: () => ({
        puzzle: {
          data: {
            meta: {
              title: 'The Crossword',
              publishStream: 'daily',
            },
          },
        },
      }),
    }

    return mount(
      // @ts-ignore
      <Provider store={store}>
        <StartModal
          close={close}
          isClosing={false}
          onRemove={onRemove}
        />
      </Provider>,
    )
  }

  afterEach(jest.clearAllMocks)

  it('should set ready and close', () => {
    const wrapper = getWrapper()
    // @ts-ignore
    const onClose = wrapper.find('ModalWrapper').props().close
    onClose()
    expect(dispatch).toHaveBeenCalledWith({ type: 'READY' })
    expect(close).toHaveBeenCalled()
  })

  it('should close on "Play without an account"', () => {
    const wrapper = getWrapper()
    const playWithoutAccountButton = wrapper
      .find('ModalWrapper')
      // @ts-ignore
      .props().footer
    playWithoutAccountButton.props.onClick()
    expect(dispatch).toHaveBeenCalledWith({ type: 'READY' })
    expect(close).toHaveBeenCalled()
  })

  it('should track and navigate on "Create a free account"', () => {
    const wrapper = getWrapper()
    window.navigationLinks = window.navigationLinks || {}
    window.navigationLinks.register = 'register'
    const createAccountButton = wrapper.find('ModalWrapper').props() // @ts-ignore
      .buttons[0]
    createAccountButton.clickHandler()
    expect(trackModuleInteraction).toHaveBeenCalled()
  })

  it('shows mini app banner on mobile web', () => {
    window.isHybridWebView = false // @ts-ignore
    getIsMini.mockReturnValue(true)

    jest.spyOn(useAbra, 'default').mockImplementation(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
      hasAbraLoaded: true,
      tests: {},
    }))

    const wrapper = getWrapper()
    const miniDownloadCTAText = wrapper.find({
      children: 'Play The Mini and more in the NYT Games app.',
    })

    const xwDownloadCTAText = wrapper.find({
      children: 'Play The Crossword and more in the NYT Games app.',
    })

    expect(miniDownloadCTAText).toHaveLength(1)
    expect(xwDownloadCTAText).toHaveLength(0)
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome',
      'mini-page',
      'download-app',
    )
  })

  it('shows crossword app banner on mobile web', () => {
    window.isHybridWebView = false // @ts-ignore
    getIsMini.mockReturnValue(false)

    jest.spyOn(useAbra, 'default').mockImplementation(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
      hasAbraLoaded: true,
      tests: {},
    }))

    const wrapper = getWrapper()
    const miniDownloadCTAText = wrapper.find({
      children: 'Play The Mini and more in the NYT Games app.',
    })

    const xwDownloadCTAText = wrapper.find({
      children: 'Play The Crossword and more in the NYT Games app.',
    })
    expect(miniDownloadCTAText).toHaveLength(0)
    expect(xwDownloadCTAText).toHaveLength(1)
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome',
      'game-page',
      'download-app',
    )
  })

  it('does not render crossword app banner in newsreader', () => {
    window.newsreaderAppPlatform = 'ios' // @ts-ignore
    getIsMini.mockReturnValue(false)

    jest.spyOn(useAbra, 'default').mockImplementation(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
      hasAbraLoaded: true,
      tests: {},
    }))

    const wrapper = getWrapper()
    const miniDownloadCTAText = wrapper.find({
      children: 'Play The Mini and more in the NYT Games app.',
    })

    const xwDownloadCTAText = wrapper.find({
      children: 'Play The Crossword and more in the NYT Games app.',
    })

    expect(miniDownloadCTAText).toHaveLength(0)
    expect(xwDownloadCTAText).toHaveLength(0)
  })

  it('does not render mini app banner in newsreader', () => {
    window.newsreaderAppPlatform = 'ios' // @ts-ignore
    getIsMini.mockReturnValue(true)

    jest.spyOn(useAbra, 'default').mockImplementation(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
      hasAbraLoaded: true,
      tests: {},
    }))

    const wrapper = getWrapper()

    const miniDownloadCTAText = wrapper.find({
      children: 'Play The Mini and more in the NYT Games app.',
    })

    const xwDownloadCTAText = wrapper.find({
      children: 'Play The Crossword and more in the NYT Games app.',
    })
    expect(miniDownloadCTAText).toHaveLength(0)
    expect(xwDownloadCTAText).toHaveLength(0)
  })
})
