import React, { useCallback } from 'react'
import cn from 'classnames'
import { useDispatch, useSelector } from 'react-redux'

import { useUser } from 'user'
import { getQueryParams } from 'foundation'

import { ModalProps } from '../../../../shared/ModalSystem/modalTypes'
import {
  getIsMini,
  getPuzzleMetadata,
  getPuzzleId,
} from '../../../store/state/puzzle/puzzleSelectors'
import { setReady } from '../../../store/thunks/transientThunks'
import ModalWrapper from '../../../../shared/ModalSystem/ModalWrapper'
import {
  Title,
  Description,
} from '../../../../shared/moments/components/subcomponents'
import { getTitleFormatted } from '../../PuzzleDetails/PuzzleDetails'
import DateAndBylineFooter from '../DateAndBylineFooter'
import {
  trackImpression,
  trackModuleInteraction,
} from '../../../store/utils/tracker'
import NonDismissalBanner from '../../../../shared/Banner/NonDismissalBanner'
import { shouldShowAppDownloadCTA } from '../../../../shared/Banner/bannerHelpers'
import useAbra from '../../../../../shared/abra/useAbra'
import {
  canCheckForAdTest,
  isInMiniAdVariant,
} from '../AdModal/validation'
import AdModal from '../AdModal'

export default function StartModal({
  close,
  isClosing,
  onRemove,
}: ModalProps) {
  const meta = useSelector(getPuzzleMetadata)
  const isMini = useSelector(getIsMini)
  const id = useSelector(getPuzzleId)
  const { userType } = useUser()
  const { getVariant, reportExposure, hasAbraLoaded } = useAbra()
  const [showAd, setShowAd] = React.useState(false)
  const [adViewComplete, setAdViewComplete] = React.useState(false)
  const dispatch = useDispatch()

  const setReadyAndClose = () => {
    dispatch(setReady(userType))
    close()
  }

  const triggerGamePlay = useCallback(() => {
    if (
      canCheckForAdTest({
        isMini,
        entitlement: userType.entitlement,
      }) &&
      hasAbraLoaded &&
      isInMiniAdVariant({ getVariant, reportExposure })
    ) {
      setShowAd(true)
    } else {
      setReadyAndClose()
    }
  }, [hasAbraLoaded])

  React.useEffect(() => {
    if (adViewComplete) {
      setReadyAndClose()
    }
  }, [adViewComplete])

  const isIOSNewsreader = window.newsreaderAppPlatform === 'ios'

  const miniDownloadCTAVariant = getVariant(
    'GAMES_minixwWelcomeAppDownloadCTA_0823',
  )

  const xwDownloadCTAVariant = getVariant(
    'GAMES_xwWelcomeAppDownloadCTA_0823',
  )

  const miniShowAppDownloadCTA = React.useMemo(
    () =>
      isMini &&
      shouldShowAppDownloadCTA(
        miniDownloadCTAVariant,
        '0_control',
        window,
      ),
    [miniDownloadCTAVariant, isMini],
  )

  const xwShowAppDownloadCTA = React.useMemo(
    () =>
      !isMini &&
      shouldShowAppDownloadCTA(
        xwDownloadCTAVariant,
        '0_control',
        window,
      ),
    [xwDownloadCTAVariant, isMini],
  )

  React.useEffect(() => {
    if (miniDownloadCTAVariant) {
      reportExposure('GAMES_minixwWelcomeAppDownloadCTA_0823')
    }

    if (xwDownloadCTAVariant) {
      reportExposure('GAMES_xwWelcomeAppDownloadCTA_0823')
    }
  }, [miniDownloadCTAVariant, xwDownloadCTAVariant])

  React.useEffect(() => {
    if (xwShowAppDownloadCTA) {
      trackImpression('welcome', 'game-page', 'download-app')
    }

    if (miniShowAppDownloadCTA) {
      trackImpression('welcome', 'mini-page', 'download-app')
    }
  }, [xwShowAppDownloadCTA, miniShowAppDownloadCTA])

  if (!meta) {
    return null
  }

  const { title, publishStream } = meta
  const fromNewsletter = !!getQueryParams('nl')
  const showAnonMessaging =
    !userType.isLoggedIn &&
    !window.newsreaderAppPlatform &&
    !fromNewsletter

  const buttons = showAnonMessaging
    ? [
        {
          label: 'Create a free account',
          clickHandler: () => {
            if (window.navigationLinks.register) {
              trackModuleInteraction('', '', 'mini-page', 'register')
              window.location.href = window.navigationLinks.register
            }
          },
        },
      ]
    : [{ label: 'Play' }]

  const footer = showAnonMessaging ? (
    <button
      type="button"
      onClick={triggerGamePlay}
      className="xwd__modal--subtle-button"
    >
      Play without an account
    </button>
  ) : (
    <DateAndBylineFooter meta={meta} />
  )

  return showAd ? (
    <AdModal
      bodyClassName={cn('xwd__start-modal', { mini: isMini })}
      close={() => {
        setAdViewComplete(true)
      }}
      containerClassName={cn('start-modal-container', {
        'ios-newsreader': isIOSNewsreader,
      })}
      isClosing={isClosing}
      onRemove={onRemove}
    />
  ) : (
    <ModalWrapper
      buttons={buttons}
      close={triggerGamePlay}
      hideCloseX
      ignoreBackgroundClick
      bodyClassName={cn('xwd__start-modal', { mini: isMini })}
      footer={footer}
      showHeader
      isClosing={isClosing}
      onRemove={onRemove}
      noAnimateOpen
      containerClassName={cn('start-modal-container', {
        'ios-newsreader': isIOSNewsreader,
      })}
    >
      <i className={cn('xwd__start-modal--icon', { mini: isMini })} />
      <Title
        size="large"
        text={getTitleFormatted(title, publishStream)}
      />
      {showAnonMessaging ? (
        <Description
          text="Save your progress across devices and compare times with friends."
          variant="small"
        />
      ) : (
        <Description text="Ready to start solving?" />
      )}
      {(miniShowAppDownloadCTA || xwShowAppDownloadCTA) && (
        <NonDismissalBanner
          title={`Play ${
            isMini ? 'The Mini' : 'The Crossword'
          } and more in the NYT Games app.`}
          icon="crossword-app"
          ctaLink="https://fpx3r.app.goo.gl/app-store"
          newTab
          position="fixed"
          onClickLink={() => {
            const page = isMini ? 'mini-page' : 'game-page'
            trackModuleInteraction(
              'welcome',
              id,
              page,
              'welcome-download-app',
              'https://fpx3r.app.goo.gl/app-store',
            )
          }}
          showMultipleIconsBanner
        />
      )}
    </ModalWrapper>
  )
}
