import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import { MetaState } from '../../store/state/puzzle/puzzleTypes'
import { useShortzMode } from '../providers/ShortzModeProvider'

import PuzzleDetails from './PuzzleDetails'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

jest.mock('../providers/ShortzModeProvider', () => ({
  useShortzMode: jest.fn(() => ({
    inShortzMode: false,
  })),
}))

const defaultProps = {
  constructors: ['Bugs Bunny', 'Daffy Duck'],
  editor: 'Elmer Fudd',
  id: 42,
  publicationDate: '2021-09-27',
}

const makeStore = (stream: string) => ({
  dispatch: jest.fn(),
  subscribe: jest.fn(),
  getState: () => ({
    puzzle: {
      data: {
        meta: {
          publishStream: stream,
        },
      },
    },
  }),
})

const getWrapper = (
  props: Partial<MetaState> = {},
  stream = 'daily',
) => {
  const store = makeStore(stream)
  return mount(
    // @ts-ignore
    <Provider store={store}>
      <PuzzleDetails {...defaultProps} {...props} />
    </Provider>,
  )
}

describe('<PuzzleDetails />', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })

  afterEach(() => {
    jest.clearAllMocks()
  })

  it('should render title and date once each on desktop', () => {
    const wrapper = getWrapper()
    const title = wrapper.find('.xwd__details--title')
    const date = wrapper.find('.xwd__details--date')
    expect(title).toHaveLength(1)
    expect(date).toHaveLength(1)
  })

  it('should render month and year on bonus', () => {
    const wrapper = getWrapper({}, 'bonus')
    expect(wrapper.find('.xwd__details--date').text()).toEqual(
      'September 2021',
    )
  })

  it('should render full date on desktop', () => {
    const wrapper = getWrapper()
    expect(wrapper.find('.xwd__details--date').text()).toEqual(
      'Monday, September 27, 2021',
    )
  })

  it('if the title is provided, then text should be the title', () => {
    const wrapper = getWrapper({ title: 'Grape' })
    expect(wrapper.find('.xwd__details--title').text()).toEqual(
      'Grape',
    )
  })

  it('if the title is provided and it has the word "and", then text should be the title with ampersands', () => {
    const wrapper = getWrapper({ title: 'Grapes and Melons' })
    expect(wrapper.find('.xwd__details--title').text()).toEqual(
      'Grapes & Melons',
    )
  })

  it('if title is not an easy mode, then constructor should be By and editor should be Edited by', () => {
    const wrapper = getWrapper({ title: 'Foo Bar' })
    const byline = wrapper.find('.xwd__details--byline')

    const constructor = byline.children().first()
    expect(constructor.text()).toEqual('By Bugs Bunny and Daffy Duck')

    const editor = byline.children().last()
    expect(editor.text()).toEqual('Edited by Elmer Fudd')
  })

  it('if the easy mode title is provided, then constructor should be Grid By and editor should be Clues by', () => {
    const wrapper = getWrapper({ title: 'Easy Mode No. 1' })
    const byline = wrapper.find('.xwd__details--byline')

    const constructor = byline.children().first()
    expect(constructor.text()).toEqual(
      'Grid by Bugs Bunny and Daffy Duck',
    )

    const editor = byline.children().last()
    expect(editor.text()).toEqual('Clues by Elmer Fudd')
  })

  it('if there is no title, then text should be "The $stream Crossword"', () => {
    const wrapper = getWrapper({}, 'melon')
    expect(wrapper.find('.xwd__details--title').text()).toEqual(
      'The Melon Crossword',
    )
  })

  it('if there is no title and stream is "daily", then text should be "The Crossword"', () => {
    const wrapper = getWrapper()
    expect(wrapper.find('.xwd__details--title').text()).toEqual(
      'The Crossword',
    )
  })

  it('should render shortz links when in shortz mode on desktop', () => {
    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: true,
    })

    const wrapper = getWrapper({}, '')

    expect(wrapper.find('a')).toHaveLength(2)
  })

  it('should not render shortz links when not in shortz mode', () => {
    // @ts-ignore
    useShortzMode.mockReturnValue({
      inShortzMode: false,
    })

    const wrapper = getWrapper()
    expect(wrapper.find('a')).toHaveLength(0)
  })
})
