import React from 'react'
import { useSelector } from 'react-redux'

import { formatDate } from '../../utils/time'
import {
  capitalize,
  constructorPrefix,
  editorPrefix,
  prettyJoin,
} from '../../utils/misc'
import { getPuzzlePublishStream } from '../../store/state/puzzle/puzzleSelectors'
import { useShortzMode } from '../providers/ShortzModeProvider'

const getNamedType = () => {
  const url = window.location?.pathname // TODO: does this work in newsreader?
  if (/tips-and-tricks/.test(url)) {
    return 'Tips & Tricks'
  }

  if (/special/.test(url)) {
    return 'Special Puzzle'
  }
}

export const getTitleFormatted = (title?: string, stream = '') => {
  // Specified Titles
  if (title) {
    return title.replace(/ and /gi, ' & ')
  }

  // Daily-Specific Titles
  if (stream === 'daily') {
    return 'The Crossword'
  }

  // All Mini etc. titles
  return `The ${capitalize(stream)} Crossword`
}

export const getDateContents = (
  publicationDate: string,
  stream = '',
) => {
  if (stream === 'bonus') {
    return formatDate(publicationDate, 'MMMM YYYY')
  }

  const dayOfWeek = formatDate(publicationDate, 'dddd')
  const monthDayYear = formatDate(publicationDate, 'MMMM D, YYYY')
  return `${dayOfWeek}, ${monthDayYear}`
}

type PuzzleDetailsProps = {
  constructors: string[]
  editor: string
  id: number
  publicationDate: string
  title?: string
}

export default function PuzzleDetails({
  constructors,
  editor = '',
  publicationDate,
  title,
  id,
}: PuzzleDetailsProps) {
  const { inShortzMode } = useShortzMode()
  const stream = useSelector(getPuzzlePublishStream)

  const titleFormatted = getTitleFormatted(title, stream)
  const byline = prettyJoin(constructors)
  const namedType = getNamedType()

  const shortzLinks = inShortzMode && (
    <>
      <a
        className="xwd__details--link"
        href={`https://admin.games.prd.nyt.net/console/xwd/edit/${id}`}
        target="_blank"
        rel="noopener noreferrer"
      >
        {`Edit #${id}`}
      </a>
      {` | `}
      <a
        className="xwd__details--link"
        href={`/svc/crosswords/v6/puzzle/${id}.json`}
        target="_blank"
        rel="noopener noreferrer"
      >
        View JSON
      </a>
    </>
  )

  return (
    <div className="xwd__details--container">
      {namedType ? (
        <div className="xwd__details--date">{namedType}</div>
      ) : (
        <>
          <div className="xwd__details--title">{titleFormatted}</div>
          <div className="xwd__details--date">
            {getDateContents(publicationDate, stream)}
          </div>
        </>
      )}
      {shortzLinks}
      <div className="xwd__details--byline">
        {byline && (
          <span>{`${constructorPrefix(title)} ${byline}`}</span>
        )}
        {editor && <span>{`${editorPrefix(title)} ${editor}`}</span>}
      </div>
    </div>
  )
}
