import React from 'react'
import { Provider } from 'react-redux'
import { mount, shallow } from 'enzyme'

import { SelectionState } from '../../store/state/selection/selectionTypes'
import { exitRebus } from '../../store/thunks/toolbarThunks'

import RebusField from './RebusField'
import { cells } from './rebusFieldTestData'

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

jest.mock('../../store/thunks/toolbarThunks', () => ({
  exitRebus: jest.fn(),
}))

type StorePayload = {
  selection: SelectionState
  isSolved: boolean
  inRebusMode: boolean
}

const defaultOffset = 150
const defaultStore = {
  selection: {
    cell: 1,
    cellClues: [1, 2],
    clue: 2,
    clueCells: [],
    clueList: 0,
    relatedCells: [],
    relatedClues: [],
  },
  isSolved: false,
  inRebusMode: true,
}

const makeStore = ({
  selection,
  isSolved,
  inRebusMode,
}: StorePayload) => {
  return {
    getState: () => ({
      assets: [],
      board: {},
      cells,
      clueLists: [],
      selection,
      transient: {
        isReady: false,
        doEscape: false,
      },
      meta: {},
      modal: {
        name: null,
      },
      status: {
        isSolved,
        autocheckEnabled: false,
      },
      toolbar: {
        activeMenu: '',
        inRebusMode,
        inPencilMode: false,
      },
      user: {
        settings: {},
      },
    }),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
    replaceReducer: jest.fn(),
  }
}

const createWrapper = (payload: StorePayload, offset: number) => {
  const store = makeStore(payload)
  return mount(
    // @ts-ignore
    <Provider store={store}>
      <RebusField offset={offset} />
    </Provider>,
  )
}

describe('Rebus Field on Desktop', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  it('renders rebus field', () => {
    const wrapper = createWrapper(defaultStore, defaultOffset)
    expect(wrapper.find('.xwd__rebus')).toHaveLength(1)
  })

  it('uses offset to figure out top position of rebus field', () => {
    const divOffset = 250
    const wrapper = createWrapper(defaultStore, divOffset)
    const rebusField = wrapper.find('.xwd__rebus--input').html()

    expect(rebusField).toContain(`top: -${divOffset}`)
  })

  it('does not render Rebus header text on desktop', () => {
    const wrapper = createWrapper(defaultStore, defaultOffset)
    expect(wrapper.contains('Rebus')).toBe(false)
  })

  it('keydown events dispatch and rebus is saved if key is Enter', () => {
    const wrapper = createWrapper(defaultStore, defaultOffset)
    const input = wrapper.find('#rebus-input')
    input.simulate('keyDown', { key: 'Enter' })

    expect(exitRebus).toHaveBeenCalledTimes(1)
    expect(exitRebus).toHaveBeenCalledWith(true)
  })

  it('keydown events dispatch and rebus is saved if key is NumpadEnter', () => {
    const wrapper = createWrapper(defaultStore, defaultOffset)
    const input = wrapper.find('#rebus-input')
    input.simulate('keyDown', { key: 'NumpadEnter' })

    expect(exitRebus).toHaveBeenCalledTimes(1)
    expect(exitRebus).toHaveBeenCalledWith(true)
  })

  it('keydown events dispatch but rebus is not saved if key is Escape', () => {
    const wrapper = createWrapper(defaultStore, defaultOffset)
    const input = wrapper.find('#rebus-input')
    input.simulate('keyDown', { key: 'Escape' })

    expect(exitRebus).toHaveBeenCalledTimes(1)
    expect(exitRebus).toHaveBeenCalledWith(false)
  })

  it('handleKeyDownInRebus doesnt dispatch if Tab is key type', () => {
    const wrapper = createWrapper(defaultStore, defaultOffset)
    const input = wrapper.find('#rebus-input')
    input.simulate('keyDown', { key: 'Tab' })

    expect(exitRebus).toHaveBeenCalledTimes(0)
  })
})
