import React, { createRef } from 'react'
import { connect } from 'react-redux'
import cn from 'classnames'

import { isMobile } from 'responsive'

import { getCellDomId } from '../../utils/misc'
import { CrosswordStoreState } from '../../store/state/crosswordReducer'
import { getSelectedCell } from '../../store/state/selection/selectionSelectors'
import { exitRebus } from '../../store/thunks/toolbarThunks'
import { CrosswordThunkDispatch } from '../../store/thunks/helpers'
import { updateRebus } from '../../store/state/toolbar/toolbarActions'

type RebusFieldStateProps = {
  index: number
  value?: string
  isSolved: boolean
}

type RebusFieldDispatchProps = {
  exit: (accept: boolean) => void
  onRebusChange: (value: string) => void
}

type RebusFieldOwnProps = {
  offset: number
}

type RebusFieldState = {
  width: number
  height: number
  left: number
  top: number
  visible: boolean
}

type RebusFieldProps = RebusFieldDispatchProps &
  RebusFieldOwnProps &
  RebusFieldStateProps

export const getActiveCellPosition = (
  index: number,
  rebusRef: React.RefObject<HTMLDivElement>,
) => {
  // Grab the position of the currently selected cell
  const cell = document
    .getElementById(getCellDomId(index))
    ?.getBoundingClientRect() || {
    top: 0,
    left: 0,
    height: 0,
    width: 0,
  }

  // check our current rebus input width
  const { width: inputWidth } =
    rebusRef.current?.getBoundingClientRect() || { width: 0 }

  // if the current rebus input is wider than the original cell, fit to the input
  cell.width = Math.max(inputWidth, cell.width)

  return cell
}

class RebusField extends React.Component<
  RebusFieldProps,
  RebusFieldState
> {
  inputRef: React.RefObject<HTMLInputElement>

  invisibleRef: React.RefObject<HTMLDivElement>

  constructor(props: RebusFieldProps) {
    super(props)
    this.inputRef = createRef()
    this.invisibleRef = createRef()
    this.state = {
      width: 0,
      height: 0,
      left: 0,
      top: 0,
      visible: false,
    }
  }

  componentDidMount() {
    const { index } = this.props
    const { current } = this.inputRef

    if (isMobile) this.setState({ visible: true })

    if (current) current.focus({ preventScroll: true })

    // After state and focus are updated, get the active cell position
    // This will help us position the rebus correctly over the selected cell
    const { height, left, top, width } = getActiveCellPosition(
      index,
      this.invisibleRef,
    )
    this.setState({ width, height, left, top })
  }

  componentDidUpdate(_: RebusFieldProps, prevState: RebusFieldState) {
    const { index } = this.props
    const { width } = getActiveCellPosition(index, this.invisibleRef)

    if (width && prevState.width !== width) {
      this.setState({ width })
    }
  }

  handleKeyDown = (event: React.KeyboardEvent) => {
    switch (event.key) {
      case 'Enter':
      case 'NumpadEnter':
        event.preventDefault()
        this.saveAndExit()
        break
      case 'Escape':
        this.saveAndExit(false)
        break
      case 'Tab':
        event.preventDefault()
        break
      default:
        break
    }
    event.stopPropagation()
  }

  checkForExitMobile = (event: React.KeyboardEvent) => {
    const { key } = event

    if (key === 'Tab') event.preventDefault()

    let delay = 0

    if (key && ['Escape', 'Enter', 'NumpadEnter'].includes(key)) {
      this.setState({ visible: false })
      event.preventDefault()
      delay = 300 // If we're exiting rebus mode, wait for the slide out
    }

    setTimeout(() => this.handleKeyDown(event), delay)
  }

  shiftField = (event: React.FormEvent<HTMLInputElement>) => {
    const target = event.target as HTMLInputElement
    const { index } = this.props

    if (index && target && !isMobile) {
      const { left, width } = getActiveCellPosition(
        index,
        this.invisibleRef,
      )
      let leftPosition = left + window.pageXOffset
      const rebusWiderThanCell = target.offsetWidth > width

      if (rebusWiderThanCell) {
        leftPosition += width / 2 - target.offsetWidth / 2
      }

      target.style.left = `${leftPosition}px`
    }

    event.stopPropagation()
  }

  updateRebus = (event: React.FormEvent<HTMLInputElement>) => {
    const target = event.target as HTMLInputElement
    const content = target.value || ''
    const { onRebusChange } = this.props

    onRebusChange(content)

    this.shiftField(event)
  }

  saveAndExit = (accept: boolean = true) => {
    const { exit } = this.props
    exit(accept)
  }

  render() {
    const { value = '', offset, isSolved } = this.props
    const { height, left, top, visible, width } = this.state

    /**
     * On desktop, we are lining up the rebus with the selected cell. The selected cell's
     * 'top' is the viewport, while the rebus's 'top' is the Layout container
     * Use the offset from layout to line up the two elements.
     *
     * On all devices, we need an updating 'width' property for an expanding input
     */
    const topPosition = top - offset
    const leftPosition = left + window.pageXOffset
    const style = isMobile
      ? { width }
      : {
          top: topPosition,
          left: leftPosition,
          height,
          width,
        }

    return (
      <div
        className={cn('xwd__rebus', {
          'xwd__rebus--mobile': isMobile,
          'xwd__rebus--visible': visible,
        })}
      >
        {isMobile && <div>Rebus</div>}
        <input
          id="rebus-input"
          name="rebus"
          inputMode={isMobile ? 'none' : 'text'} // ensures native keyboard doesnt pop up & allows paste
          disabled={isSolved}
          ref={this.inputRef}
          onChange={(e) => this.updateRebus(e)}
          onBlur={() => this.saveAndExit(true)}
          onKeyDown={(e) => {
            if (!isMobile) {
              this.handleKeyDown(e)
            } else {
              e.persist()
              this.checkForExitMobile(e)
              e.stopPropagation()
            }
          }}
          onKeyPress={(e) => e.stopPropagation()}
          className="xwd__rebus--input"
          style={style}
          value={value}
        />
        <div
          ref={this.invisibleRef}
          className="xwd__rebus--invisible"
          dangerouslySetInnerHTML={{ __html: value }}
        />
      </div>
    )
  }
}

const mapStateToProps = (
  state: CrosswordStoreState,
): RebusFieldStateProps => {
  const selectedCell = getSelectedCell(state)

  return {
    index: selectedCell ? selectedCell.index : -1,
    value: state.toolbar.rebusValue,
    isSolved: state.status.isSolved,
  }
}

const mapDispatchToProps = (
  dispatch: CrosswordThunkDispatch,
): RebusFieldDispatchProps => ({
  exit: (accept) => dispatch(exitRebus(accept)),
  onRebusChange: (value) => dispatch(updateRebus(value)),
})

export default connect(
  mapStateToProps,
  mapDispatchToProps,
)(RebusField)
