import React from 'react'
import { connect } from 'react-redux'
import { Dispatch } from 'redux'

import { isMobile } from 'responsive'

import { CrosswordStoreState } from '../../store/state/crosswordReducer'
import {
  pauseTimer,
  unpauseTimer,
} from '../../store/state/timer/timerActions'
import { getIsModalOpen } from '../../store/state/modal/modalSelectors'
import { getCurrentUserSettings } from '../../store/state/user/userSelectors'
import { formatTime, getElapsedTime } from '../../utils/time'
import Icon from '../../../../shared/components/Icon/Icon'

type TimerStateProps = {
  shouldBePaused: boolean
  totalElapsedTime: number
  sessionStartTimestamp?: number
  timerIsVisible: boolean
}

type TimerDispatchProps = {
  start: () => void
  stop: () => void
  pauseFailsafe: () => void
}

type TimerOwnProps = {
  clickHandler: () => void
}

type TimerProps = TimerStateProps & TimerDispatchProps & TimerOwnProps

export class TimerBase extends React.PureComponent<TimerProps> {
  interval?: number | void

  pauseTimeout?: number | void

  tick?: number

  componentDidMount = () => {
    const { pauseFailsafe } = this.props
    window.addEventListener('visibilitychange', () => {
      if (document.hidden || document.visibilityState === 'hidden') {
        this.handleBlur()
      } else {
        this.handleFocus()
      }
    })
    // pagehide supported on all browsers
    window.addEventListener('pagehide', () => {
      pauseFailsafe() // save progress on server
    })
    window.addEventListener('blur', this.handleBlur) // For when mobile devices are locked
    window.addEventListener('focus', this.handleFocus)
  }

  componentDidUpdate = () => {
    const { shouldBePaused } = this.props
    if (shouldBePaused === this.isPaused()) return

    if (shouldBePaused) {
      this.stopTicking()
    } else {
      this.startTicking()
    }
  }

  isPaused = () => !this.interval

  startTicking = () => {
    const { clickHandler, start } = this.props

    const updateRegularly = () => {
      // A tick being blocked for over 10 seconds probably means
      // the machine has been suspended (otherwise something is very wrong)
      const tookTooLong = this.tick && Date.now() - this.tick > 10000
      if (tookTooLong) {
        clickHandler()
      } else {
        this.forceUpdate()
      }
      this.tick = Date.now()
    }

    this.tick = Date.now()
    this.interval = window.setInterval(updateRegularly, 1000)
    start()
  }

  stopTicking = () => {
    const { stop } = this.props
    if (this.isPaused()) return // Game can only be so paused

    if (this.interval) {
      this.interval = window.clearInterval(this.interval)
    }
    stop()
  }

  handleFocus = () => {
    if (!this.pauseTimeout) return
    window.clearTimeout(this.pauseTimeout)
    delete this.pauseTimeout
  }

  handleBlur = () => {
    const { clickHandler, timerIsVisible } = this.props
    const willNeedToPause = timerIsVisible && !this.isPaused()

    if (!willNeedToPause) return

    if (isMobile && document.hidden) clickHandler()

    if (!this.pauseTimeout) {
      this.pauseTimeout = window.setTimeout(clickHandler, 30000)
    }
  }

  render() {
    const {
      clickHandler,
      totalElapsedTime,
      shouldBePaused,
      sessionStartTimestamp,
      timerIsVisible,
    } = this.props

    const formatted = formatTime(
      getElapsedTime(totalElapsedTime, sessionStartTimestamp),
    )

    const ariaLabel = `Timer ${
      shouldBePaused ? 'Play' : 'Pause'
    } Button`

    const timer = (
      <button
        type="button"
        onClick={() => !this.isPaused() && clickHandler()}
        aria-label={ariaLabel}
      >
        <div className="timer-count">{formatted}</div>
        {!this.isPaused() && <Icon icon="pause" />}
      </button>
    )

    return (
      <li className="xwd__tool--button xwd__timer--button">
        {timerIsVisible && timer}
      </li>
    )
  }
}

const mapState = (state: CrosswordStoreState): TimerStateProps => {
  const { transient, status, timer } = state
  const { isReady } = transient
  const { showTimer } = getCurrentUserSettings(state)
  const { isSolved } = status
  const { totalElapsedTime, sessionStartTimestamp } = timer
  const modalIsOpen = getIsModalOpen(state)
  const shouldBePaused = !isReady || modalIsOpen || isSolved

  return {
    shouldBePaused,
    totalElapsedTime,
    sessionStartTimestamp,
    timerIsVisible: showTimer,
  }
}

const mapDispatch = (dispatch: Dispatch): TimerDispatchProps => ({
  start: () => dispatch(unpauseTimer()),
  stop: () => dispatch(pauseTimer()),
  pauseFailsafe: () => dispatch(pauseTimer(true)),
})

export default connect(mapState, mapDispatch)(TimerBase)
