import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import { MetaState } from '../../store/state/puzzle/puzzleTypes'

import Toolbar from './Toolbar'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

jest.mock('responsive', () => ({
  isMobile: true,
}))

jest.mock(
  '../../../shared/ToolbarAdapter/ToolbarAdapter',
  () =>
    ({ children }: { children: React.ReactElement }) =>
      children,
)

type StorePayload = {
  inRebusMode?: boolean
  activeMenu?: string
  inPencilMode?: boolean
  autocheckEnabled?: boolean
  isSolved?: boolean
  notes?: Pick<MetaState, 'notes'>
}

const makeStore = ({
  inRebusMode,
  activeMenu,
  inPencilMode,
  isSolved,
  autocheckEnabled,
  notes,
}: StorePayload) => {
  return {
    getState: () => ({
      assets: [],
      board: {},
      clueLists: [],
      transient: {
        isReady: false,
        doEscape: false,
      },
      puzzle: {
        data: {
          meta: {
            notes,
          },
        },
      },
      modal: {
        name: null,
      },
      status: {
        isSolved,
        autocheckEnabled,
      },
      toolbar: {
        activeMenu,
        inRebusMode,
        inPencilMode,
      },
      timer: {},
      user: {
        settings: {},
      },
    }),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
    replaceReducer: jest.fn(),
  }
}

const createWrapper = (payload: StorePayload) => {
  const store = makeStore(payload)
  return mount(
    // @ts-ignore
    <Provider store={store}>
      <Toolbar />
    </Provider>,
  )
}

describe('Toolbar component', () => {
  it('should display the reset tool if puzzle is solved on mobile', () => {
    const state = {
      inRebusMode: true,
      activeMenu: '',
      inPencilMode: true,
      autocheckEnabled: false,
      isSolved: true,
    }
    const wrapper = createWrapper(state)
    expect(
      wrapper.find('.xwd__toolbar--mobileResetButton').length,
    ).toBe(1)
  })

  it('should display the info icon on mobile', () => {
    const state = {
      inRebusMode: true,
      activeMenu: '',
      inPencilMode: true,
      autocheckEnabled: false,
      isSolved: true,
    }
    const wrapper = createWrapper(state)
    expect(wrapper.find('.xwd__toolbar_icon--info').length).toBe(1)
  })

  it('should display the settings gear icon on mobile', () => {
    const state = {
      inRebusMode: true,
      activeMenu: '',
      inPencilMode: true,
      autocheckEnabled: false,
      isSolved: true,
    }
    const wrapper = createWrapper(state)
    expect(
      wrapper.find('.xwd__toolbar_icon--settings-gear').length,
    ).toBe(1)
  })

  it('should display cheat menu icon if puzzle isnt solved', () => {
    const state = {
      inRebusMode: true,
      activeMenu: '',
      inPencilMode: true,
      autocheckEnabled: false,
      isSolved: false,
    }
    const wrapper = createWrapper(state)
    expect(wrapper.find('.xwd__toolbar--condensedMenu').length).toBe(
      1,
    )
    expect(
      wrapper.find('.xwd__toolbar_icon--cheat-menu').length,
    ).toBe(1)
  })

  it('should display an active menu icon if auto check enabled or menu set is to mobile', () => {
    const state = {
      inRebusMode: true,
      activeMenu: 'MOBILE',
      inPencilMode: true,
      autocheckEnabled: true,
      isSolved: false,
    }
    const wrapper = createWrapper(state)
    expect(
      wrapper.find('.xwd__toolbar_icon--cheat-menu-active').length,
    ).toBe(1)
  })

  it('should not render the help menu after puzzle is solved', () => {
    const state = {
      inRebusMode: true,
      activeMenu: 'MOBILE',
      inPencilMode: true,
      autocheckEnabled: true,
      isSolved: true,
    }
    const wrapper = createWrapper(state)
    expect(wrapper.find('HelpMenu').length).toBe(0)
  })
})
