import React, { useState } from 'react'
import { useSelector, useDispatch } from 'react-redux'
import cn from 'classnames'

import { isMobile } from 'responsive'

import { Scope } from '../../store/state/cells/cellsTypes'
import { RESET_GAME } from '../../store/state/status/statusActions'
import {
  execute,
  ExecutedActionType,
} from '../../store/thunks/statusThunks'
import {
  togglePencil,
  toggleHelpMenu,
} from '../../store/state/toolbar/toolbarActions'
import { openModal } from '../../store/state/modal/modalActions'
import { CrosswordModal } from '../../store/state/modal/modalTypes'
import { toggleRebus } from '../../store/thunks/toolbarThunks'
import { getGameStatus } from '../../store/state/status/statusSelectors'
import { getToolbarState } from '../../store/state/toolbar/toolbarSelectors'
import { getIsReady } from '../../store/state/transient/transientSelectors'
import { getPuzzleMetadata } from '../../store/state/puzzle/puzzleSelectors'
import Timer from '../Timer/Timer'
import ToolbarAdapter from '../../../shared/ToolbarAdapter/ToolbarAdapter'

import Tool from './Tool/Tool'
import HelpMenu from './HelpMenu/HelpMenu'
import ToolMenu from './ToolMenu'
import {
  menuData,
  mobileMenuData,
  MenuAction,
} from './HelpMenu/helpMenuData'

export type ToolbarAction =
  | ExecutedActionType
  | MenuAction
  | (() => void)

export type ToggleType =
  | 'PENCIL'
  | 'REBUS'
  | 'MENU'
  | 'AUTOCHECK'
  | 'REVEAL'
  | 'CLEAR'

export default function Toolbar() {
  const { inRebusMode, activeMenu, inPencilMode } =
    useSelector(getToolbarState)
  const { isSolved, autocheckEnabled } = useSelector(getGameStatus)
  const { notes } = useSelector(getPuzzleMetadata) || {}
  const isReady = useSelector(getIsReady)
  const isNewsReader = !!window.newsreaderAppPlatform
  const shouldDisplayNavDrawer = isMobile && !isNewsReader
  const [infoIconClicked, setInfoIconClicked] = useState(false)
  const dispatch = useDispatch()

  const executeAction = (action: ToolbarAction, scope?: Scope) => {
    if (typeof action === 'function') dispatch(action())
    else dispatch(execute(action, scope))
  }

  const toggleAction = (type: ToggleType, menu?: string) => {
    if (type === 'PENCIL') {
      dispatch(togglePencil())
    } else if (type === 'REBUS') {
      dispatch(toggleRebus())
    } else if (menu) {
      dispatch(toggleHelpMenu(menu))
    }
  }

  const openModalAction = (name: CrosswordModal) => {
    dispatch(openModal(name))
  }

  const toolbar = (
    <div className={cn('xwd__toolbar--wrapper')}>
      <ul className="xwd__toolbar--tools">
        {!shouldDisplayNavDrawer && (
          <div>
            <Tool
              clickHandler={() =>
                openModalAction(CrosswordModal.Settings)
              }
              icon="xwd__toolbar_icon--settings-gear"
              ariaLabel="Puzzle Settings Menu"
            />
          </div>
        )}
        <Timer
          clickHandler={() => openModalAction(CrosswordModal.Pause)}
        />
        {!isMobile && (
          <Tool
            clickHandler={() => toggleAction('REBUS')}
            label="Rebus"
            active={inRebusMode}
          />
        )}
        {isSolved ? (
          <div
            className={cn(
              'xwd__toolbar--mobileResetButton',
              isMobile,
            )}
          >
            <Tool
              clickHandler={() => executeAction(RESET_GAME)}
              label={!isMobile ? 'Reset' : ''}
              icon={isMobile ? 'xwd__toolbar_icon--reset-puzzle' : ''}
            />
          </div>
        ) : (
          <ToolMenu
            autoCheckEnabled={autocheckEnabled}
            activeMenu={activeMenu}
            execute={executeAction}
            toggleMenuAction={toggleAction}
            menuData={menuData}
          />
        )}
        {isMobile && (
          <Tool
            clickHandler={() => {
              if (notes) setInfoIconClicked(true)
              openModalAction(CrosswordModal.Info)
            }}
            icon={cn('xwd__toolbar_icon--info', {
              'xwd__toolbar_icon--info-animation':
                notes && notes.length > 0 && !infoIconClicked,
            })}
          />
        )}
        {!isMobile && (
          <Tool
            clickHandler={() => toggleAction('PENCIL')}
            icon={`xwd__toolbar_icon--pencil${
              inPencilMode ? '-active' : ''
            }`}
            active={inPencilMode}
          />
        )}
        {!isSolved && (
          <div className="xwd__toolbar--condensedMenu">
            <Tool
              active={activeMenu === 'MOBILE'}
              icon={`xwd__toolbar_icon--cheat-menu${
                activeMenu === 'MOBILE' || autocheckEnabled
                  ? '-active'
                  : ''
              }`}
              clickHandler={() => toggleAction('MENU', 'MOBILE')}
              isAutocheckOn={autocheckEnabled}
            >
              <HelpMenu
                items={mobileMenuData}
                isAutocheckOn={autocheckEnabled}
                itemHandler={executeAction}
              />
            </Tool>
          </div>
        )}
        {shouldDisplayNavDrawer && (
          <div>
            <Tool
              clickHandler={() =>
                openModalAction(CrosswordModal.Settings)
              }
              icon="xwd__toolbar_icon--settings-gear"
              ariaLabel="Puzzle Settings Menu"
            />
          </div>
        )}
      </ul>
    </div>
  )

  return (
    <ToolbarAdapter isHidden={!isReady}>{toolbar}</ToolbarAdapter>
  )
}
