import React from 'react'
import { act, waitFor, render, screen } from '@testing-library/react'

import { useUser } from 'user'

import { getShortzToggleState } from '../../store/utils/forageUtils'

import {
  ShortzModeProvider,
  useShortzMode,
} from './ShortzModeProvider'

jest.mock('user', () => ({
  useUser: jest.fn(() => ({ userType: { inShortzMode: true } })),
}))

jest.mock('../../store/utils/forageUtils', () => ({
  getShortzToggleState: jest.fn(() => Promise.resolve(false)),
  storeShortzToggleState: jest.fn(),
}))

const Consumer = () => {
  const { isErsatzShortz, inShortzMode } = useShortzMode()
  return (
    <>
      <div data-testid="ersatz">{`${isErsatzShortz}`}</div>
      <div data-testid="mode">{`${inShortzMode}`}</div>
    </>
  )
}

describe('useShortzMode', () => {
  afterEach(jest.clearAllMocks)

  it('should use inShortzMode from useUser as isErsatzShortz', async () => {
    await act(async () => {
      render(
        <ShortzModeProvider>
          <Consumer />
        </ShortzModeProvider>,
      )

      const ersatzElem = screen.getByTestId('ersatz')
      const modeElem = screen.getByTestId('mode')

      await waitFor(() => {
        expect(ersatzElem.textContent).toEqual('true')
        expect(modeElem.textContent).toEqual('false')
      })
    })
  })

  it('should should get initial toggle state from local storage', async () => {
    // @ts-ignore
    getShortzToggleState.mockReturnValueOnce(Promise.resolve(true))
    await act(async () => {
      render(
        <ShortzModeProvider>
          <Consumer />
        </ShortzModeProvider>,
      )

      const ersatzElem = screen.getByTestId('ersatz')
      const modeElem = screen.getByTestId('mode')

      await waitFor(() => {
        expect(ersatzElem.textContent).toEqual('true')
        expect(modeElem.textContent).toEqual('true')
      })
    })
  })

  it('should not hit local storage for non shortz users', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({ userType: { inShortzMode: false } })
    await act(async () => {
      render(
        <ShortzModeProvider>
          <Consumer />
        </ShortzModeProvider>,
      )

      const ersatzElem = screen.getByTestId('ersatz')
      const modeElem = screen.getByTestId('mode')

      await waitFor(() => {
        expect(ersatzElem.textContent).toEqual('false')
        expect(modeElem.textContent).toEqual('false')
        expect(getShortzToggleState).not.toHaveBeenCalled()
      })
    })
  })
})
