import React, {
  useCallback,
  useState,
  useEffect,
  useMemo,
  createContext,
  useContext,
} from 'react'

import { useUser } from 'user'

import {
  getShortzToggleState,
  storeShortzToggleState,
} from '../../store/utils/forageUtils'

type ShortzModeContextType = {
  /** If the user's regi id is on the shortz mode list */
  isErsatzShortz: boolean
  /** If both isErsatzShortz and the toggle in the settings modal title is active */
  inShortzMode: boolean
  setShortzMode: (inShortzMode: boolean) => void
}

const ShortzModeContext = createContext<ShortzModeContextType>({
  isErsatzShortz: false,
  inShortzMode: false,
  setShortzMode: () => {},
})

export const ShortzModeProvider = ({
  children,
}: {
  children: React.ReactNode
}) => {
  const {
    userType: { inShortzMode: isErsatzShortz },
  } = useUser()
  const [isToggleActive, setToggleActive] = useState(false)

  // sync toggle state with local storage
  useEffect(() => {
    if (isErsatzShortz) {
      getShortzToggleState().then((isActive) => {
        setToggleActive(!!isActive)
      })
    }
  }, [isErsatzShortz])

  // set both hook state and local storage
  const setShortzMode = useCallback(
    (isActive: boolean) => {
      setToggleActive(isActive)
      storeShortzToggleState(isActive)
    },
    [setToggleActive],
  )

  const shortzModeContextValue = useMemo<ShortzModeContextType>(
    () => ({
      isErsatzShortz,
      inShortzMode: isErsatzShortz && isToggleActive,
      setShortzMode,
    }),
    [isErsatzShortz, isToggleActive, setShortzMode],
  )

  return (
    <ShortzModeContext.Provider value={shortzModeContextValue}>
      {children}
    </ShortzModeContext.Provider>
  )
}

export const useShortzMode = () => useContext(ShortzModeContext)
