import { APPLY_PROGRESS } from '../../state/progressActions'
import { PUZZLE_READY } from '../../state/transient/transientActions'
import { GUESS } from '../../state/cells/cellsActions'
import { isBlank, isNavigable } from '../../utils/cellUtils'
import {
  NAVIGATE,
  SELECT_CELL,
  SELECT_CLUE,
  SWITCH_DIRECTION,
} from '../../state/selection/selectionActions'
import {
  DEBUG_REVEAL,
  RESET_GAME,
} from '../../state/status/statusActions'
import { getNextOrFirstElement } from '../../utils/arrays'
import { CrosswordStoreState } from '../../state/crosswordReducer'
import { CrosswordAction } from '../../state/crosswordAction'
import { ClueListsState } from '../../state/puzzle/puzzleTypes'
import { SelectionState } from '../../state/selection/selectionTypes'
import { getCurrentUser } from '../../state/user/userSelectors'

import navigate from './navigator'

const VERTICAL_RE = /[Up|Down]$/

const cardinalDirections = ['Across', 'Down']

const isCardinalDirection = (
  clueListIdx: number,
  clueLists: ClueListsState,
) => cardinalDirections.includes(clueLists[clueListIdx]?.name)

const getAcrossIdx = (clueLists: ClueListsState) =>
  clueLists.findIndex((l) => l.name === 'Across') || 0

const getDownIdx = (clueLists: ClueListsState) =>
  clueLists.findIndex((l) => l.name === 'Down') || 0

const highlightRelatedClues = (
  { clues }: CrosswordStoreState,
  newSelection: SelectionState,
) => {
  const selectedClue = clues[newSelection.clue as number]
  let relatedCells: number[] = []
  let relatedClues: number[] = []
  if (selectedClue) {
    relatedClues = selectedClue.relatives || []
    relatedCells = relatedClues
      .map((clue) => clues[clue])
      .reduce<number[]>(
        (clueCells, clue) => clueCells.concat(clue.cells),
        [],
      )
  }

  return {
    ...newSelection,
    relatedCells,
    relatedClues,
  }
}

const getInitialSelection = ({
  cells,
  clues,
}: CrosswordStoreState): SelectionState => {
  // If the grid is completely filled, just select the first navigable cell
  const firstSelectedCell =
    cells.find(isBlank) || cells.find(isNavigable)

  const [firstSelectedClueIndex = null] =
    firstSelectedCell?.clues || []
  const firstSelectedClue = clues[firstSelectedClueIndex as number]

  return {
    cell: firstSelectedCell?.index ?? null,
    cellClues: firstSelectedCell?.clues || [],
    clue: firstSelectedClueIndex,
    clueCells: firstSelectedClue?.cells || [],
    clueList: firstSelectedClue?.list || 0, // Clues in list 0 have not yet been normalized in the clues reducer
    relatedCells: [],
    relatedClues: [],
  }
}

const getInitialSelectionIfNeeded = (state: CrosswordStoreState) => {
  const { selection } = state
  if (selection.cell == null) return getInitialSelection(state)

  return null
}

const getSelectionFromCell = (
  { cells, clues, selection }: CrosswordStoreState,
  newIndex: number,
): SelectionState => {
  const newCell = cells[newIndex]
  const { clueList } = selection
  const { clues: cellClues } = newCell
  const cellClueInCurrentList = cellClues.find(
    (index) => clues[index].list === clueList,
  )

  const newClue = cellClueInCurrentList
    ? clues[cellClueInCurrentList]
    : clues[cellClues[0]]

  return newClue
    ? {
        cell: newIndex,
        cellClues,
        clue: newClue.index,
        clueCells: newClue.cells,
        clueList: newClue.list ?? null,
        relatedCells: [],
        relatedClues: [],
      }
    : {
        cell: newIndex,
        cellClues: [],
        clue: null,
        clueCells: [],
        clueList,
        relatedCells: [],
        relatedClues: [],
      }
}

const getSelectionFromClue = (
  { cells, clues }: CrosswordStoreState,
  newIndex: number,
): SelectionState => {
  const newClue = clues[newIndex]
  const firstEmptyCell = newClue.cells
    .map((c) => cells[c])
    .find((c) => isBlank(c))

  const newCellIndex = firstEmptyCell
    ? firstEmptyCell.index
    : newClue.cells[0]

  return {
    cell: newCellIndex,
    cellClues: cells[newCellIndex].clues,
    clue: newClue.index,
    clueCells: newClue.cells,
    clueList: newClue.list ?? null,
    relatedCells: [],
    relatedClues: [],
  }
}

const switchDirection = (
  state: CrosswordStoreState,
  intermediateSelection: SelectionState | null = null,
): SelectionState => {
  const { clues } = state
  const selection = intermediateSelection || state.selection
  const { clue: selectedClue, cellClues } = selection
  if (cellClues.length < 2) return selection // No direction to switch to!

  const newClueIndex = getNextOrFirstElement(
    cellClues,
    selectedClue as number,
  )
  const newClue = clues[newClueIndex]
  return {
    ...selection,
    clueCells: newClue.cells,
    clue: newClue.index,
    clueList: newClue.list ?? null,
  }
}

const hasDirectionMismatch = (
  state: CrosswordStoreState,
  targetDirection: string,
) => {
  const {
    puzzle: { data },
    selection: { clueList },
  } = state

  if (!data) return false

  const selectedClueList = data.clueLists[clueList as number]
  const currentOrientation = selectedClueList.name
  const verticalArrowPressed = VERTICAL_RE.test(targetDirection)
  const desiredOrientation = verticalArrowPressed ? 'Down' : 'Across'
  return currentOrientation !== desiredOrientation
}

const determineAction = (
  state: CrosswordStoreState,
  targetDirection: string,
) => {
  const user = getCurrentUser(state)
  const {
    settings: { onSwitch },
  } = user

  const {
    puzzle: { data },
    selection: { cellClues, clueList },
  } = state

  if (!data) return null

  if (hasDirectionMismatch(state, targetDirection)) {
    const isClueless = cellClues.length === 0
    const shouldMove =
      onSwitch === 'move' ||
      isClueless ||
      !isCardinalDirection(clueList as number, data.clueLists)

    const intermediateSelection = shouldMove
      ? navigate(state, targetDirection.substring(5))
      : null

    return switchDirection(state, intermediateSelection)
  }

  return navigate(state, targetDirection.substring(5))
}

const shiftClueMovements: Record<string, string> = {
  ShiftArrowUp: 'PreviousClue',
  ShiftArrowDown: 'NextClue',
  ShiftArrowLeft: 'PreviousClue',
  ShiftArrowRight: 'NextClue',
}

const strafe = (
  state: CrosswordStoreState,
  targetDirection: string,
) => {
  const {
    puzzle: { data },
    cells,
    selection: { clueList },
  } = state
  const direction = targetDirection.replace('ShiftArrow', '')
  const isClueless = clueList == null

  if (!data) {
    return null
  }

  if (
    isClueless ||
    !isCardinalDirection(clueList as number, data.clueLists)
  ) {
    const preferredCluelist = VERTICAL_RE.test(direction)
      ? getDownIdx(data.clueLists)
      : getAcrossIdx(data.clueLists)

    return navigate(state, direction, preferredCluelist)
  }

  if (hasDirectionMismatch(state, targetDirection)) {
    // move to first navigable, but maintain cluelist, and then find first landable in that clue
    const intermediateSelection = navigate(state, direction)

    // if there is nowhere to strafe to, do nothing
    if (!intermediateSelection) return null

    // if every cell in this clue is filled, then just strafe
    const isNotEmpty = (c: number) => !isBlank(cells[c])
    if (intermediateSelection.clueCells.every(isNotEmpty)) {
      return intermediateSelection
    }

    // otherwise strafe and then find first landable in new clue
    return getSelectionFromClue(
      state,
      intermediateSelection.clue as number,
    )
  }

  // if directions match, act like next/previous clue
  return navigate(state, shiftClueMovements[targetDirection] || '')
}

const gamesMachine = (
  state: CrosswordStoreState,
  action: CrosswordAction,
) => {
  let newSelection: SelectionState | null = null
  switch (action.type) {
    case APPLY_PROGRESS:
      if ('selection' in action.payload) {
        newSelection = action.payload.selection || null
      }
      break
    case PUZZLE_READY:
      newSelection = getInitialSelectionIfNeeded(state)
      break
    case RESET_GAME:
      newSelection = getInitialSelection(state)
      break
    case DEBUG_REVEAL:
      newSelection = getSelectionFromCell(state, action.payload.index)
      break
    case SELECT_CELL:
      newSelection = getSelectionFromCell(state, action.payload.index)
      break
    case SELECT_CLUE:
      newSelection = getSelectionFromClue(state, action.payload.index)
      break
    case SWITCH_DIRECTION:
      newSelection = switchDirection(state)
      break
    case NAVIGATE: {
      const { direction } = action.payload
      if (direction?.startsWith('Arrow')) {
        newSelection = determineAction(state, direction)
      } else if (direction?.startsWith('Shift')) {
        newSelection = strafe(state, direction)
      } else {
        newSelection = navigate(state, direction)
      }
      break
    }
    case GUESS:
      newSelection = navigate(state, 'Advance')
      break
    default:
      return null
  }

  if (!newSelection) return null
  newSelection = highlightRelatedClues(state, newSelection)

  return newSelection
}

export default gamesMachine
