import { CellState } from '../../state/cells/cellsTypes'
import { ClueState } from '../../state/clues/cluesTypes'

/*
 * returns a clue given a cell, clues, and a preferred clue list
 * preferring a clue that matches, but 'rotating' clockwise if needed
 * where clockwise is the clue list with a higher index (or 0 if the list is the max)
 */
const findBestFit = (values: number[], target?: number | null) =>
  values.reduce((b, a) => {
    if (a === target) return a // first match

    if (b === target) return b // maintain match

    if (
      typeof target === 'number' &&
      ((a < target && b < target) || (a > target && b > target))
    ) {
      return Math.min(a, b) // same side
    }

    return Math.max(a, b) // opposite sides, prefer larger
  })

const getClueFromCell = (
  cell: CellState,
  clues: ClueState[],
  preferredCluelistIndex?: number | null,
): Pick<ClueState, 'index' | 'cells' | 'list'> => {
  /** handle clueless cells first */
  if (cell.clues.length === 0) {
    return { index: null, cells: [], list: preferredCluelistIndex }
  }

  /** no need for complexity if there is only one clue to pick */
  if (cell.clues.length === 1) return clues[cell.clues[0]]

  const cellClues = cell.clues
  const cellClueLists = cellClues.map((c) => clues[c].list as number)

  /*
   * Select the clue that belongs to the preferred clue list
   * otherwise, prefer the cluelist that is minimum higher index
   * otherwise get the minimum lower index
   */
  const closestCluelist = findBestFit(
    cellClueLists,
    preferredCluelistIndex,
  )

  // translate that best fit back to a clue in the cell's clue list
  const bestFit = cellClueLists.findIndex(
    (v) => v === closestCluelist,
  )

  return clues[cellClues[bestFit]]
}

export default getClueFromCell
