import { CellState } from '../../state/cells/cellsTypes'
import { CrosswordStoreState } from '../../state/crosswordReducer'
import { getSelectedCell } from '../../state/selection/selectionSelectors'
import { SelectionState } from '../../state/selection/selectionTypes'
import { getCurrentUserSettings } from '../../state/user/userSelectors'
import {
  getPreviousOrLastElement,
  getElementsAfter,
  getElementsBefore,
} from '../../utils/arrays'
import { setLandableConditions } from '../../utils/cellUtils'

const findLandableCell = (
  indexes: number[],
  cells: CellState[],
  isLandable: (cell: CellState) => boolean,
) => {
  const landableIndex = indexes.find((i) => isLandable(cells[i]))
  if (landableIndex !== undefined) {
    return cells[landableIndex]
  }
}

const moveToFirstCell = () => 0
const moveToLastCell = (_: number, list: number[]) => list.length - 1
const moveToCellBefore = (pos: number) => pos - 1
const moveToCellAfter = (pos: number) => pos + 1

const moveToNewPositionInClue = (
  cells: CellState[],
  selection: SelectionState,
  newPositionInClue: number,
) => {
  const newCellIndex = selection.clueCells[newPositionInClue]

  const newCell = cells[newCellIndex]
  return {
    ...selection,
    cell: newCellIndex,
    cellClues: newCell.clues,
  }
}

const moveWithinClue = (
  { cells, selection }: CrosswordStoreState,
  findPositionInClue: (
    cellIndex: number,
    clueCells: number[],
  ) => number,
) => {
  const { cell, clueCells } = selection
  if (!clueCells.length) return null // Nothing to do for clueless cells

  const newPositionInClue = findPositionInClue(
    clueCells.indexOf(cell as number),
    clueCells,
  )

  return moveToNewPositionInClue(cells, selection, newPositionInClue)
}

const moveToLandableCellViaClue = (
  state: CrosswordStoreState,
  reverse = false,
): SelectionState | null => {
  const settings = getCurrentUserSettings(state)
  const { cells, clues, selection } = state
  const isLandable = setLandableConditions(settings)
  const currentClue = clues[selection.clue as number]
  const neighbor = reverse ? 'prev' : 'next'

  let startClueIdx: number
  if (currentClue) {
    startClueIdx = currentClue[neighbor]
  } else {
    startClueIdx = reverse ? clues.length - 1 : 0 // For clueless cells
  }

  let firstLandableCell: CellState | undefined
  let newClue = clues[startClueIdx]
  do {
    firstLandableCell = findLandableCell(
      newClue.cells,
      cells,
      isLandable,
    )
    if (firstLandableCell) break
    newClue = clues[newClue[neighbor]]
  } while (newClue.index !== startClueIdx)

  const newCellIndex = firstLandableCell
    ? firstLandableCell.index
    : newClue.cells[0]
  const newCell = cells[newCellIndex]
  return {
    cell: newCellIndex,
    cellClues: newCell.clues,
    clue: newClue.index,
    clueCells: newClue.cells,
    clueList: newClue.list ?? null,
    relatedCells: [],
    relatedClues: [],
  }
}

const moveToNextLandableCell = (
  state: CrosswordStoreState,
): SelectionState | null => {
  const settings = getCurrentUserSettings(state)
  const { cells, selection } = state
  const { cell, clueCells } = selection
  const isLandable = setLandableConditions(settings)
  const start = getSelectedCell(state)
  const isOverwriting = start ? start.guess : false
  const cellsAfter = getElementsAfter(clueCells, cell as number)
  const isLastCellOfClue = cellsAfter.length === 0

  if (isOverwriting && !isLastCellOfClue) {
    return moveWithinClue(state, moveToCellAfter)
  }

  const cellsInClueToSearch = settings.jumpBack
    ? cellsAfter.concat(getElementsBefore(clueCells, cell as number))
    : cellsAfter

  const nextLandableInClue = findLandableCell(
    cellsInClueToSearch,
    cells,
    isLandable,
  )

  if (nextLandableInClue) {
    return moveToNewPositionInClue(
      cells,
      selection,
      clueCells.indexOf(nextLandableInClue.index),
    )
  }

  if (settings.autoAdvance) return moveToLandableCellViaClue(state)

  return null
}

const moveToPreviousLandableCell = (
  state: CrosswordStoreState,
): SelectionState | null => {
  const settings = getCurrentUserSettings(state)
  const { cells, clues, selection } = state
  const { cell, clue, clueCells } = selection
  const isFirstCellOfClue =
    cell !== null && clueCells.indexOf(cell) === 0

  if (!isFirstCellOfClue) {
    return moveWithinClue(state, moveToCellBefore)
  }

  if (!settings.backspaceAcrossWords) return null // Stop at word boundary if user prefers

  const previousClue = getPreviousOrLastElement(
    clues,
    clues[clue as number],
  )
  const cellsOfPreviousClue = previousClue.cells
  const lastCellOfPreviousClue =
    cells[cellsOfPreviousClue[cellsOfPreviousClue.length - 1]]

  return {
    ...selection,
    cell: lastCellOfPreviousClue.index,
    cellClues: lastCellOfPreviousClue.clues,
    clue: previousClue.index,
    clueCells: cellsOfPreviousClue,
    clueList: previousClue.list ?? null,
  }
}

const moveByCell = (
  state: CrosswordStoreState,
  direction?: string,
): SelectionState | null => {
  switch (direction) {
    case 'ClueStart':
      return moveWithinClue(state, moveToFirstCell)
    case 'ClueEnd':
      return moveWithinClue(state, moveToLastCell)
    case 'Advance':
      return moveToNextLandableCell(state)
    case 'Backspace':
      return moveToPreviousLandableCell(state)
    case 'NextClue':
      return moveToLandableCellViaClue(state)
    case 'PreviousClue':
      return moveToLandableCellViaClue(state, true)
    default:
      return null
  }
}

export default moveByCell
