import { createStore, applyMiddleware } from 'redux'

import { GUESS } from '../../../state/cells/cellsActions'
import { APPLY_PROGRESS } from '../../../state/progressActions'
import {
  NAVIGATE,
  SELECT_CELL,
  SELECT_CLUE,
  SWITCH_DIRECTION,
} from '../../../state/selection/selectionActions'
import { RESET_GAME } from '../../../state/status/statusActions'
import { PUZZLE_READY } from '../../../state/transient/transientActions'
import crossware from '..'
import navigator from '../navigator'

// test navigator and mover separately
jest.mock('../navigator')

let createEmptyGameState
let getSelection
let setupStore
let store

beforeEach(() => {
  navigator.mockReset()

  createEmptyGameState = () => ({
    cells: [
      { index: 0, type: 0 },
      { index: 1, type: 1, guess: 'F', clues: [0, 1] },
      { index: 2, type: 1, guess: undefined, clues: [2, 3] },
    ],
    clues: [
      { index: 0, list: 0, cells: [1] },
      { index: 1, list: 1, cells: [1] },
      { index: 2, list: 0, cells: [2] },
      { index: 3, list: 1, cells: [2] },
    ],
    puzzle: {
      data: {
        clueLists: [{ name: 'Across' }, { name: 'Down' }],
      },
    },
    selection: {
      cell: null,
      clueCells: [],
      clue: null,
      cellClues: [],
      clueList: null,
      relatedCells: [],
      relatedClues: [],
    },
  })

  setupStore = (initialState) => {
    const reducer = (state = initialState, action) => {
      if (action.selection) {
        return { ...state, selection: action.selection }
      }

      return state
    }

    return createStore(reducer, applyMiddleware(crossware))
  }

  getSelection = (s) => s.getState().selection

  store = setupStore(createEmptyGameState())
})

describe('crossware: on applying progress', () => {
  test('should apply current game selection', () => {
    store.dispatch({
      type: APPLY_PROGRESS,
      payload: {
        selection: {
          newSelection: true,
        },
      },
    })

    expect(getSelection(store)).toHaveProperty('newSelection', true)
  })
})

describe('crossware: on puzzle ready', () => {
  test('if initial cell is null', () => {
    const initialState = createEmptyGameState()
    initialState.selection.cell = null
    store = setupStore(initialState)
    store.dispatch({ type: PUZZLE_READY, payload: {} })

    expect(getSelection(store)).toMatchObject({
      cell: 2,
      cellClues: [2, 3],
      clue: 2,
      clueCells: [2],
    })
  })

  test('if initial cell is undefined', () => {
    const initialState = createEmptyGameState()
    initialState.selection.cell = undefined
    store = setupStore(initialState)
    store.dispatch({ type: PUZZLE_READY, payload: {} })

    expect(getSelection(store)).toMatchObject({
      cell: 2,
      cellClues: [2, 3],
      clue: 2,
      clueCells: [2],
    })
  })

  test('if initial cell is set', () => {
    const initialState = createEmptyGameState()
    initialState.selection.cell = 3
    store = setupStore(initialState)
    store.dispatch({ type: PUZZLE_READY, payload: {} })

    expect(getSelection(store)).toMatchObject({
      cell: 3,
    })
  })

  test('if initial cell is not set and but cells are all filled', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 0 },
      { index: 1, type: 1, guess: 'F', clues: [0, 1] },
      { index: 2, type: 1, guess: 'G', clues: [2, 3] },
    ]

    store = setupStore(initialState)
    store.dispatch({ type: PUZZLE_READY, payload: {} })

    expect(getSelection(store)).toMatchObject({
      cell: 1,
      cellClues: [0, 1],
      clue: 0,
      clueCells: [1],
    })
  })

  test('if initial cell is clueless', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, guess: undefined, clues: [] },
    ]

    store = setupStore(initialState)
    store.dispatch({ type: PUZZLE_READY, payload: {} })

    expect(getSelection(store)).toMatchObject({
      cell: 0,
      cellClues: [],
      clue: null,
      clueCells: [],
    })
  })
})

describe('crossware: on reset game', () => {
  test('reset selection even if it exists', () => {
    const initialState = createEmptyGameState()
    initialState.selection.cell = 3
    store = setupStore(initialState)
    store.dispatch({ type: PUZZLE_READY, payload: {} })

    // before reset
    expect(getSelection(store)).toMatchObject({ cell: 3 })

    store.dispatch({ type: RESET_GAME, payload: {} })

    // after reset
    expect(getSelection(store)).toMatchObject({ cell: 2 })
  })
})

describe('crossware: on select cell', () => {
  test('preserve cluelist (direction) when possible', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 1, type: 1, clues: [0, 1] },
      { index: 2, type: 1, clues: [2, 3] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 0, cells: [1] },
      { index: 3, list: 1, cells: [1] },
    ]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CELL, payload: { index: 0 } })

    expect(getSelection(store)).toMatchObject({
      cell: 0,
      cellClues: [0, 1],
      clue: 0,
      clueCells: [0],
      clueList: 0,
    })

    store.dispatch({ type: SELECT_CELL, payload: { index: 1 } })

    expect(getSelection(store)).toMatchObject({
      cell: 1,
      cellClues: [2, 3],
      clue: 2,
      clueCells: [1],
      clueList: 0,
    })
  })

  test('if cluelist must change, select first applicable cluelist', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 1, type: 1, clues: [0, 1] },
      { index: 2, type: 1, clues: [2, 3] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0] },
      { index: 1, list: 0, cells: [0] },
      { index: 2, list: 1, cells: [1] },
      { index: 3, list: 1, cells: [1] },
    ]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CELL, payload: { index: 0 } })

    expect(getSelection(store)).toMatchObject({
      cell: 0,
      cellClues: [0, 1],
      clue: 0,
      clueCells: [0],
      clueList: 0,
    })

    store.dispatch({ type: SELECT_CELL, payload: { index: 1 } })

    expect(getSelection(store)).toMatchObject({
      cell: 1,
      cellClues: [2, 3],
      clue: 2,
      clueCells: [1],
      clueList: 1,
    })
  })

  test('respect clueless cells by nullifying clue and maintaining last clue list', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 1, type: 1, clues: [0, 1] },
      { index: 2, type: 1, clues: [] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0] },
      { index: 1, list: 1, cells: [0] },
    ]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CELL, payload: { index: 0 } })

    expect(getSelection(store)).toMatchObject({
      cell: 0,
      cellClues: [0, 1],
      clue: 0,
      clueCells: [0],
      clueList: 0,
    })

    store.dispatch({ type: SELECT_CELL, payload: { index: 1 } })

    expect(getSelection(store)).toMatchObject({
      cell: 1,
      cellClues: [],
      clue: null,
      clueCells: [],
      clueList: 0,
    })
  })
})

describe('crossware: on select clue', () => {
  test('select the first empty cell in the clue, if there is one', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, guess: 'F', clues: [0] },
      { index: 1, type: 1, guess: undefined, clues: [0] },
      { index: 2, type: 1, guess: undefined, clues: [0] },
    ]

    initialState.clues = [{ index: 0, list: 2, cells: [0, 1, 2] }]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })

    expect(getSelection(store)).toMatchObject({
      cell: 1,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 1, 2],
      clueList: 2,
    })
  })

  test('otherwise, select the first one', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, guess: 'F', clues: [0] },
      { index: 1, type: 1, guess: 'G', clues: [0] },
      { index: 2, type: 1, guess: 'H', clues: [0] },
    ]

    initialState.clues = [{ index: 0, list: 2, cells: [0, 1, 2] }]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })

    expect(getSelection(store)).toMatchObject({
      cell: 0,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 1, 2],
      clueList: 2,
    })
  })
})

describe('crossware: on switch direction', () => {
  test('switches to next direction', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, guess: undefined, clues: [0, 1, 2] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 2, cells: [0] },
    ]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CELL, payload: { index: 0 } })
    expect(getSelection(store)).toHaveProperty('clue', 0)

    store.dispatch({ type: SWITCH_DIRECTION })
    expect(getSelection(store)).toHaveProperty('clue', 1)
  })

  test('returns to first direction after last', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, guess: undefined, clues: [0, 1, 2] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 2, cells: [0] },
    ]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CLUE, payload: { index: 2 } })
    expect(getSelection(store)).toHaveProperty('clue', 2)

    store.dispatch({ type: SWITCH_DIRECTION })
    expect(getSelection(store)).toHaveProperty('clue', 0)
  })

  test('does not switch if there is only one direction', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, guess: undefined, clues: [1] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 2, cells: [0] },
    ]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CELL, payload: { index: 0 } })
    expect(getSelection(store)).toHaveProperty('clue', 1)

    store.dispatch({ type: SWITCH_DIRECTION })
    expect(getSelection(store)).toHaveProperty('clue', 1)
  })
})

describe('crossware: on navigate', () => {
  beforeEach(() => {
    const initialState = createEmptyGameState()
    initialState.user = { settings: { onSwitch: 'move' } }

    initialState.cells = [
      { index: 0, type: 1, clues: [0, 1] },
      { index: 1, type: 1, clues: [2, 3] },
      { index: 2, type: 1, clues: [0] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 2] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 0, cells: [1] },
      { index: 3, list: 1, cells: [1] },
    ]

    store = setupStore(initialState)
  })

  test('navigate with arrows (UP/DOWN)', () => {
    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })
    const state1 = store.getState()
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowUp' },
    })
    const state2 = store.getState()
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowDown' },
    })
    const firstCallArgs = navigator.mock.calls[0]
    const secondCallArgs = navigator.mock.calls[1]
    expect(firstCallArgs[0]).toEqual(state1) // passes state
    expect(firstCallArgs[1]).toEqual('Up') // passes direction

    expect(secondCallArgs[0]).toEqual(state2) // passes state
    expect(secondCallArgs[1]).toEqual('Down') // passes direction
  })

  test('navigate with arrows (LEFT/RIGHT)', () => {
    store.dispatch({ type: SELECT_CLUE, payload: { index: 1 } })
    const state1 = store.getState()
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowLeft' },
    })
    const state2 = store.getState()
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowRight' },
    })

    const firstCallArgs = navigator.mock.calls[0]
    const secondCallArgs = navigator.mock.calls[1]

    expect(firstCallArgs[0]).toEqual(state1) // passes state
    expect(firstCallArgs[1]).toEqual('Left') // passes direction

    expect(secondCallArgs[0]).toEqual(state2) // passes state
    expect(secondCallArgs[1]).toEqual('Right') // passes direction
  })

  test('arrow moves away from a cell with no cardinal direction clues', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0, 1] },
      { index: 1, type: 1, clues: [2] }, // only has strange clue list
      { index: 2, type: 1, clues: [0] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 2] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 2, cells: [1] },
    ]

    initialState.puzzle.data.clueLists = [
      { name: 'across' },
      { name: 'down' },
      { name: 'strange' },
    ]

    // make this strict so we assure user wont get stuck
    initialState.user = { settings: { onSwitch: false } }

    initialState.selection = {
      cell: 1,
      cellClues: [2],
      clue: 2,
      clueCells: [1],
      clueList: 2,
    }

    navigator.mockReturnValueOnce({
      cell: 2,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 2],
      clueList: 0,
    })

    store = setupStore(initialState)
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowRight' },
    })
    const firstCallArgs = navigator.mock.calls[0]

    expect(firstCallArgs[0]).toMatchObject(initialState)
    expect(firstCallArgs[1]).toBe('Right')
    expect(getSelection(store)).toMatchObject({
      cell: 2,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 2],
      clueList: 0,
    })
  })

  test('arrow moves away from clueless cell', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0, 1] },
      { index: 1, type: 1, clues: [] }, // empty out clues on this cell
      { index: 2, type: 1, clues: [0] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 2] },
      { index: 1, list: 1, cells: [0] },
    ]

    initialState.clueLists = [{ name: 'across' }, { name: 'down' }]

    // make this strict so we assure user wont get stuck
    initialState.user = { settings: { onSwitch: false } }

    initialState.selection = {
      cell: 1,
      cellClues: [],
      clue: null,
      clueCells: [],
      clueList: 1,
    }

    navigator.mockReturnValueOnce({
      cell: 2,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 2],
      clueList: 0,
    })

    store = setupStore(initialState)
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowRight' },
    })
    const firstCallArgs = navigator.mock.calls[0]

    expect(firstCallArgs[0]).toMatchObject(initialState)
    expect(firstCallArgs[1]).toBe('Right')
    expect(getSelection(store)).toMatchObject({
      cell: 2,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 2],
      clueList: 0,
    })
  })

  test('arrow agrees with orientation', () => {
    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })

    expect(getSelection(store).clueList).toEqual(0)

    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowRight' },
    })

    const firstCallArgs = navigator.mock.calls[0]

    expect(getSelection(store).clueList).toEqual(0) // keeps the clueList
    expect(firstCallArgs[1]).toEqual('Right') // passes direction
  })

  test('arrow disagrees with orientation and wants to move on change', () => {
    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })

    expect(getSelection(store).clueList).toEqual(0)

    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowDown' },
    })

    const firstCallArgs = navigator.mock.calls[0]

    expect(getSelection(store).clueList).toEqual(1) // change the clueList
    expect(firstCallArgs[1]).toEqual('Down') // passes direction
  })

  test('user does not want to move on switch', () => {
    const initialState = createEmptyGameState()
    initialState.user = { settings: { onSwitch: false } }

    initialState.cells = [
      { index: 0, type: 1, clues: [0, 1] },
      { index: 1, type: 1, clues: [2, 3] },
      { index: 2, type: 1, clues: [0] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 2] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 0, cells: [1] },
      { index: 3, list: 1, cells: [1] },
    ]

    store = setupStore(initialState)
    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })
    expect(getSelection(store).clueList).toEqual(0)
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ArrowDown' },
    })
    expect(getSelection(store).clueList).toEqual(1) // change the clueList
    expect(navigator.mock.calls.length).toEqual(0) // dont navigate when direction changes
  })

  test('non-arrow navigation', () => {
    store.dispatch({ type: SELECT_CLUE, payload: { index: 1 } })
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'OtherNav' },
    })

    const firstCallArgs = navigator.mock.calls[0]

    expect(firstCallArgs[0]).toEqual(store.getState()) // passes state
    expect(firstCallArgs[1]).toEqual('OtherNav') // passes direction
  })

  test('strafe with clue orientation behaves like next/prev clue', () => {
    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowLeft' },
    })
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowRight' },
    })

    const firstCallArgs = navigator.mock.calls[0]
    const secondCallArgs = navigator.mock.calls[1]

    expect(firstCallArgs[0]).toEqual(store.getState()) // passes state
    expect(firstCallArgs[1]).toEqual('PreviousClue') // passes direction

    expect(secondCallArgs[0]).toEqual(store.getState()) // passes state
    expect(secondCallArgs[1]).toEqual('NextClue') // passes direction
  })

  test('strafe can handle if across and down are not in the same order', () => {
    const initialState = createEmptyGameState()

    // atypical order
    initialState.puzzle.data.clueLists = [
      { name: 'other' },
      { name: 'Across' },
      { name: 'Down' },
    ]

    initialState.clues[0].list = 2

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowUp' },
    })
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowDown' },
    })

    const firstCallArgs = navigator.mock.calls[0]
    const secondCallArgs = navigator.mock.calls[1]

    expect(firstCallArgs[0]).toEqual(store.getState()) // passes state
    expect(firstCallArgs[1]).toEqual('PreviousClue') // passes direction

    expect(secondCallArgs[0]).toEqual(store.getState()) // passes state
    expect(secondCallArgs[1]).toEqual('NextClue') // passes direction
  })

  test('strafe away from clueless cell switches clue list to movement orientation', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0, 1] },
      { index: 1, type: 1, clues: [] }, // empty out clues on this cell
      { index: 2, type: 1, clues: [0] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 2] },
      { index: 1, list: 1, cells: [0] },
    ]

    navigator.mockReturnValueOnce({ mocked: true })

    store = setupStore(initialState)
    store.dispatch({ type: SELECT_CELL, payload: { index: 1 } })
    const stateAfterSelect = store.getState()
    expect(getSelection(store).clueList).toEqual(null)
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowDown' },
    })
    const firstCallArgs = navigator.mock.calls[0]

    const DOWN_LIST = 1

    expect(getSelection(store)).toMatchObject({ mocked: true })
    expect(firstCallArgs[0]).toMatchObject(stateAfterSelect)
    expect(firstCallArgs[1]).toBe('Down')
    expect(firstCallArgs[2]).toBe(DOWN_LIST)
  })

  test('strafe while on extra clue list switches clue list to movement orientation', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0, 1] },
      { index: 1, type: 1, clues: [2] },
      { index: 2, type: 1, clues: [0] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 2] },
      { index: 1, list: 1, cells: [0] },
      { index: 2, list: 2, cells: [1] }, // some special clue list!
    ]

    initialState.puzzle.data.clueLists.push({ name: 'special' })

    navigator.mockReturnValueOnce({ mocked: true })

    store = setupStore(initialState)
    store.dispatch({ type: SELECT_CELL, payload: { index: 1 } })
    const stateAfterSelect = store.getState()
    expect(getSelection(store).clueList).toEqual(2)
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowRight' },
    })
    const firstCallArgs = navigator.mock.calls[0]

    const ACROSS_LIST = 0

    expect(getSelection(store)).toMatchObject({ mocked: true })
    expect(firstCallArgs[0]).toMatchObject(stateAfterSelect)
    expect(firstCallArgs[1]).toBe('Right')
    expect(firstCallArgs[2]).toBe(ACROSS_LIST)
  })

  test('strafe perpendicular to orientation, finds nearest landable in direction, maintains cluelist', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0], guess: 'A' }, // filled
      { index: 1, type: 1, clues: [0] }, // should go to
      { index: 2, type: 1, clues: [0] }, // lands on
      { index: 3, type: 1, clues: [1] }, // starts on
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 1, 2] },
      { index: 1, list: 0, cells: [0] },
    ]

    navigator.mockReturnValueOnce({
      cell: 2,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 1, 2],
      clueList: 0,
    })

    store = setupStore(initialState)
    store.dispatch({ type: SELECT_CELL, payload: { index: 3 } })
    expect(getSelection(store).clueList).toEqual(0)

    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowUp' },
    })

    expect(getSelection(store)).toMatchObject({
      cell: 1,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 1, 2],
      clueList: 0,
    })
  })

  test('strafe perpendicular to orientation, lands on full clue, keeps intermediate selection', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0], guess: 'A' }, // filled
      { index: 1, type: 1, clues: [0], guess: 'A' }, // filled
      { index: 2, type: 1, clues: [0], guess: 'A' }, // lands on and stays
      { index: 3, type: 1, clues: [1] }, // starts on
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 1, 2] },
      { index: 1, list: 0, cells: [0] },
    ]

    navigator.mockReturnValueOnce({
      cell: 2,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 1, 2],
      clueList: 0,
    })

    store = setupStore(initialState)
    store.dispatch({ type: SELECT_CELL, payload: { index: 3 } })
    expect(getSelection(store).clueList).toEqual(0)

    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowUp' },
    })

    expect(getSelection(store)).toMatchObject({
      cell: 2,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 1, 2],
      clueList: 0,
    })
  })

  test('strafe perpendicular to orientation, does nothing if nowhere to go', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0], guess: 'A' }, // filled
      { index: 1, type: 1, clues: [0] }, // should go to
      { index: 2, type: 1, clues: [0] }, // lands on
      { index: 3, type: 1, clues: [1] }, // starts on
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0, 1, 2] },
      { index: 1, list: 0, cells: [0] },
    ]

    store = setupStore(initialState)
    store.dispatch({ type: SELECT_CELL, payload: { index: 3 } })

    const initialSelection = getSelection(store)

    // intermediate selection doesn't find a cell
    navigator.mockReturnValueOnce(null)
    store.dispatch({
      type: NAVIGATE,
      payload: { direction: 'ShiftArrowDown' },
    })

    // keeps the initial selection
    expect(getSelection(store)).toBe(initialSelection)
  })
})

describe('crossware: on guess', () => {
  test('should use navigate to compute next selection on guess', () => {
    navigator.mockReturnValue({ navigated: true })
    store.dispatch({ type: GUESS })
    expect(getSelection(store)).toMatchObject({
      navigated: true,
    })
  })
})

describe('crossware: on other actions', () => {
  test('middleware ignores other actions', () => {
    const stateBefore = store.getState()
    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })
    const stateAfter = store.getState()

    store.dispatch({ type: 'UNHANDLED_ACTION' })
    const finalState = store.getState()

    expect(stateBefore).not.toBe(stateAfter)
    expect(stateAfter).toBe(finalState)
  })
})

describe('crossware: highlight related clues', () => {
  test('middleware highlights all related clues and cells', () => {
    const initialState = createEmptyGameState()

    initialState.cells = [
      { index: 0, type: 1, clues: [0] },
      { index: 1, type: 1, clues: [1] },
      { index: 2, type: 1, clues: [2] },
    ]

    initialState.clues = [
      { index: 0, list: 0, cells: [0], relatives: [1] },
      { index: 1, list: 1, cells: [1] },
      { index: 2, list: 1, cells: [2] },
    ]

    store = setupStore(initialState)

    store.dispatch({ type: SELECT_CLUE, payload: { index: 0 } })

    expect(getSelection(store)).toMatchObject({
      relatedCells: [1],
      relatedClues: [1],
    })
  })
})
