import mover from '../mover'

let cells
let clues
let selection
let settings
let makeGameState

describe('mover', () => {
  beforeEach(() => {
    settings = {}

    /*
      Test grid:
      |1|■|2|
      |3| | |
      | |■| |

      (■ = black cell
       # = clue number)
    */

    // prettier-ignore
    cells = [
      { index: 0, type: 1, clues: [ 0 ] }, { index: 1, type: 0              }, { index: 2, type: 1, clues: [ 1 ] },
      { index: 3, type: 1, clues: [0, 2] }, { index: 4, type: 1, clues: [ 2 ] }, { index: 5, type: 1, clues: [1, 2] },
      { index: 6, type: 1, clues: [ 0 ] }, { index: 7, type: 0              }, { index: 8, type: 1, clues: [ 1 ] },
    ]

    clues = [
      { index: 0, cells: [0, 3, 6], prev: 2, next: 1, list: 0 },
      { index: 1, cells: [2, 5, 8], prev: 0, next: 2, list: 0 },
      { index: 2, cells: [3, 4, 5], prev: 1, next: 0, list: 0 },
    ]

    selection = {
      cell: 0,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 3, 6],
      clueList: 0,
    }

    makeGameState = () => ({
      user: { settings },
      cells,
      clues,
      selection,
    })
  })

  describe('jumping to clue boundaries', () => {
    test('if there is no clue, dont jump anywhere', () => {
      selection = {
        cell: 4,
        cellClues: [],
        clue: null,
        clueCells: [],
        clueList: null,
      }

      const stateBefore = makeGameState()
      const selectionAfterStart = mover(stateBefore, 'ClueStart')
      const selectionAfterEnd = mover(stateBefore, 'ClueEnd')

      expect(selectionAfterStart).toBe(null)
      expect(selectionAfterEnd).toBe(null)
    })

    test('move to beginning of clue', () => {
      selection = {
        cell: 4,
        cellClues: [2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'ClueStart')

      expect(selectionAfter).toMatchObject({
        cell: 3,
        cellClues: [0, 2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      })
    })

    test('move to end of clue', () => {
      selection = {
        cell: 4,
        cellClues: [2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'ClueEnd')

      expect(selectionAfter).toMatchObject({
        cell: 5,
        cellClues: [1, 2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      })
    })
  })

  describe('advancing within clue', () => {
    test('when overwriting, move forward until clue boundary', () => {
      selection = {
        cell: 3,
        cellClues: [0, 2],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      cells[3].guess = 'A' // make guess so we will be overwriting
      cells[6].guess = 'B' // fill landing cell to prove it lands there anyway

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'Advance')

      expect(selectionAfter).toMatchObject({
        cell: 6,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      })
    })

    test('will jump back to landable cells if user wants', () => {
      selection = {
        cell: 3,
        cellClues: [0, 2],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      settings.jumpBack = true
      settings.skipFilled = true
      cells[6].guess = 'B' // fill landing cell so we will jump back to begining

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'Advance')

      expect(selectionAfter).toMatchObject({
        cell: 0,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      })
    })

    test('will stop at clue boundary if user doesn’t want to auto advance', () => {
      selection = {
        cell: 6,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'Advance')

      expect(selectionAfter).toBe(null)
    })

    test('will advance to next clue if user wants', () => {
      selection = {
        cell: 6,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      settings.autoAdvance = true

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'Advance')

      expect(selectionAfter).toMatchObject({
        cell: 2,
        cellClues: [1],
        clue: 1,
        clueCells: [2, 5, 8],
        clueList: 0,
      })
    })
  })

  describe('backspace', () => {
    test('move backwards if not first clue cell', () => {
      selection = {
        cell: 3,
        cellClues: [0, 2],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'Backspace')

      expect(selectionAfter).toMatchObject({
        cell: 0,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      })
    })

    test('do nothing at beginning of clue', () => {
      selection = {
        cell: 0,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'Backspace')

      expect(selectionAfter).toBe(null)
    })

    test('go back to previous clue if user wants', () => {
      selection = {
        cell: 3,
        cellClues: [0, 2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      }

      settings.backspaceAcrossWords = true

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'Backspace')

      expect(selectionAfter).toMatchObject({
        cell: 8,
        cellClues: [1],
        clue: 1,
        clueCells: [2, 5, 8],
        clueList: 0,
      })
    })
  })

  describe('jumping from clue to clue', () => {
    test('from clueless cell, nextClue jumps to first clue', () => {
      selection = {
        cell: 3,
        cellClues: [0, 2],
        clue: null,
        clueCells: [],
        clueList: null,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'NextClue')

      expect(selectionAfter).toMatchObject({
        cell: 0,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      })
    })

    test('from clueless cell, previousClue jumps to last clue', () => {
      selection = {
        cell: 3,
        cellClues: [0, 2],
        clue: null,
        clueCells: [],
        clueList: null,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'PreviousClue')

      expect(selectionAfter).toMatchObject({
        cell: 3,
        cellClues: [0, 2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      })
    })

    test('jump to next clue', () => {
      selection = {
        cell: 0,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'NextClue')

      expect(selectionAfter).toMatchObject({
        cell: 2,
        cellClues: [1],
        clue: 1,
        clueCells: [2, 5, 8],
        clueList: 0,
      })
    })

    test('jump to previous clue', () => {
      selection = {
        cell: 3,
        cellClues: [0, 2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      }

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'PreviousClue')

      expect(selectionAfter).toMatchObject({
        cell: 2,
        cellClues: [1],
        clue: 1,
        clueCells: [2, 5, 8],
        clueList: 0,
      })
    })

    test('skip clues with no landable cells', () => {
      selection = {
        cell: 0,
        cellClues: [0],
        clue: 0,
        clueCells: [0, 3, 6],
        clueList: 0,
      }

      cells[2].guess = 'A'
      cells[5].guess = 'A'
      cells[8].guess = 'A'

      settings.skipFilled = true

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'NextClue')

      expect(selectionAfter).toMatchObject({
        cell: 3,
        cellClues: [0, 2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      })
    })

    test('if all board cells are completely filled', () => {
      selection = {
        cell: 2,
        cellClues: [1],
        clue: 1,
        clueCells: [2, 5, 8],
        clueList: 0,
      }

      settings.skipFilled = true
      cells.forEach((c) => {
        c.guess = 'A'
      })

      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'NextClue')

      expect(selectionAfter).toMatchObject({
        cell: 3,
        cellClues: [0, 2],
        clue: 2,
        clueCells: [3, 4, 5],
        clueList: 0,
      })
    })
  })

  describe('other movements', () => {
    test('should pass state through', () => {
      const stateBefore = makeGameState()
      const selectionAfter = mover(stateBefore, 'direction')
      expect(selectionAfter).toBe(null)
    })
  })
})
