import navigator from '../navigator'
import mover from '../mover'

// test mover separately
jest.mock('../mover')

let makeGameState
let cells
let clues
let dimensions
let selection
let clueLists

beforeEach(() => {
  mover.mockReset()

  dimensions = {
    columnCount: 3,
    rowCount: 3,
  }

  /*
    Test grid:
    |1|■|2|
    |3| | |
    | |■| |

    (■ = black cell
     # = clue number)
  */

  // prettier-ignore
  cells = [
    { index: 0, type: 1, clues: [ 0 ] }, { index: 1, type: 0              }, { index: 2, type: 1, clues: [ 1 ] },
    { index: 3, type: 1, clues: [0, 2] }, { index: 4, type: 1, clues: [ 2 ] }, { index: 5, type: 1, clues: [1, 2] },
    { index: 6, type: 1, clues: [ 0 ] }, { index: 7, type: 0              }, { index: 8, type: 1, clues: [ 1 ] },
  ]

  clues = [
    { index: 0, list: 1, cells: [0, 3, 6] },
    { index: 1, list: 1, cells: [2, 5, 8] },
    { index: 2, list: 0, cells: [3, 4, 5] },
  ]

  clueLists = [{ name: 'Across' }, { name: 'Down' }]

  selection = {
    cell: 0,
    clueList: 1,
  }

  makeGameState = () => ({
    puzzle: {
      data: {
        clueLists,
        dimensions,
      },
    },
    cells,
    clues,
    selection,
  })
})

describe('navigator', () => {
  test('passes through non-directions to mover', () => {
    const state = {}
    const direction = 'non-direction'

    navigator(state, direction)
    const args = mover.mock.calls[0]

    expect(args[0]).toBe(state)
    expect(args[1]).toBe(direction)
  })

  test('simple Up', () => {
    selection = {
      cell: 3,
      clueList: 1,
    }

    const newSelection = navigator(makeGameState(), 'Up')

    expect(newSelection).toMatchObject({
      cell: 0,
      cellClues: [0],
      clue: 0,
      clueCells: [0, 3, 6],
      clueList: 1,
    })
  })

  test('simple Down', () => {
    selection = {
      cell: 0,
      clueList: 1,
    }

    const newSelection = navigator(makeGameState(), 'Down')

    expect(newSelection).toMatchObject({
      cell: 3,
      cellClues: [0, 2],
      clue: 0,
      clueCells: [0, 3, 6],
      clueList: 1,
    })
  })

  test('simple Right', () => {
    selection = {
      cell: 3,
      clueList: 0,
    }

    const newSelection = navigator(makeGameState(), 'Right')

    expect(newSelection).toMatchObject({
      cell: 4,
      cellClues: [2],
      clue: 2,
      clueCells: [3, 4, 5],
      clueList: 0,
    })
  })

  test('simple Left', () => {
    selection = {
      cell: 4,
      clueList: 0,
    }

    const newSelection = navigator(makeGameState(), 'Left')

    expect(newSelection).toMatchObject({
      cell: 3,
      cellClues: [0, 2],
      clue: 2,
      clueCells: [3, 4, 5],
      clueList: 0,
    })
  })

  test('skip non-navigable', () => {
    selection = {
      cell: 0,
      clueList: 1,
    }

    const newSelection = navigator(makeGameState(), 'Right')

    expect(newSelection).toMatchObject({
      cell: 2,
      cellClues: [1],
      clue: 1,
      clueCells: [2, 5, 8],
      clueList: 1,
    })
  })

  test('return null if off the board', () => {
    selection = {
      cell: 0,
      clueList: 0,
    }

    const newSelection = navigator(makeGameState(), 'Up')

    expect(newSelection).toBe(null)
  })

  test('handles clueless cell', () => {
    cells[4].clues = [] // empty the clues for landing cell

    selection = {
      cell: 3,
      clueList: 2,
    }

    const newSelection = navigator(makeGameState(), 'Right')

    expect(newSelection).toMatchObject({
      cell: 4,
      cellClues: [],
      clue: null,
      clueCells: [],
    })
  })

  test('throws if grid dimensions are missing', () => {
    dimensions = {
      columnCount: undefined,
      rowCount: 3,
    }

    expect(() => {
      navigator(makeGameState(), 'Down')
    }).toThrow(/gridwidth must be a positive number/)
  })

  test('allows a preferred cluelist', () => {
    selection = {
      cell: 0,
      clueList: 1,
    }

    const preferredCluelist = 0
    const newSelection = navigator(
      makeGameState(),
      'Down',
      preferredCluelist,
    )

    expect(newSelection).toMatchObject({
      cell: 3,
      cellClues: [0, 2],
      clue: 2,
      clueCells: [3, 4, 5],
      clueList: 0,
    })
  })
})
