import { emitter } from '../../components/Juice'
import { CLEAR, GUESS, REVEAL } from '../state/cells/cellsActions'
import { PUZZLE_READY } from '../state/transient/transientActions'
import {
  DEBUG_REVEAL,
  RESET_GAME,
} from '../state/status/statusActions'
import { CrosswordStoreState } from '../state/crosswordReducer'
import { getStatsAndStreaks } from '../thunks/statsThunks'
import {
  getSelectedCell,
  getSelectedClue,
} from '../state/selection/selectionSelectors'
import { getAllClues } from '../state/puzzle/puzzleSelectors'
import {
  getGameStatus,
  getGoldStarEligibility,
  getPercentFilled,
} from '../state/status/statusSelectors'
import { CellState } from '../state/cells/cellsTypes'
import { getCurrentUserSettings } from '../state/user/userSelectors'

import { CrosswordMiddleware } from './type'

const FILL_PERCENTAGE_ALTERING_ACTIONS = [
  CLEAR,
  REVEAL,
  GUESS,
  DEBUG_REVEAL,
  RESET_GAME,
]

type Milestone = {
  reached: boolean
  seen: boolean
  text: string
  threshold: number
}

const threeQtrMilestone: Milestone = {
  reached: false,
  seen: false,
  text: '3/4 Done!',
  threshold: 0.75,
}

const halfwayMilestone: Milestone = {
  reached: false,
  seen: false,
  text: 'Halfway!',
  threshold: 0.5,
}

const sendBurst = (milestone: Milestone, idx: number) => {
  const { text } = milestone
  emitter.emit('milestone', { text, idx })

  milestone.seen = true
}

const updateMilestoneEligibility = (
  percentBefore: number,
  percentAfter: number,
) => {
  ;[threeQtrMilestone, halfwayMilestone].forEach((milestone) => {
    const { threshold } = milestone
    const hasReachedMilestone =
      percentBefore <= threshold && threshold < percentAfter

    const fellBelowMilestone =
      percentAfter <= threshold && threshold < percentBefore

    if (hasReachedMilestone || fellBelowMilestone) {
      Object.assign(milestone, {
        reached: hasReachedMilestone,
        seen: false,
      })
    }
  })
}

const actionDidFillClue = (
  stateBeforeAction: CrosswordStoreState,
  stateAfterAction: CrosswordStoreState,
) => {
  const clueBefore = getSelectedClue(stateBeforeAction)
  const clueAfter =
    clueBefore &&
    getAllClues(stateAfterAction)[clueBefore.index as number]

  return clueBefore?.unfilledCount && !clueAfter?.unfilledCount
}

const actionDidSolvePuzzle = (
  stateAfterAction: CrosswordStoreState,
) => {
  const statusAfter = getGameStatus(stateAfterAction)
  return statusAfter.incorrectCells <= 0
}

const isReadyToBurst = (
  stateBefore: CrosswordStoreState,
  stateAfter: CrosswordStoreState,
) => {
  const clueNowFilled = actionDidFillClue(stateBefore, stateAfter)
  const puzzleNowSolved = actionDidSolvePuzzle(stateAfter)
  return clueNowFilled && !puzzleNowSolved
}

const makeBurstSender = (stateBeforeAction: CrosswordStoreState) => {
  const { index } = getSelectedCell(stateBeforeAction) as CellState
  return (milestone: Milestone) => {
    const shouldSendBurst = milestone.reached && !milestone.seen
    if (shouldSendBurst) sendBurst(milestone, index)

    return shouldSendBurst
  }
}

const fillMonitorMiddleware: CrosswordMiddleware =
  (store) => (next) => (action) => {
    const stateBeforeAction = store.getState()
    const result = next(action)
    const stateAfterAction = store.getState()

    /**
     * In order to correctly show the streak messages in the congrats modal
     * the stats and streaks are fetched at the beginning of the game
     *
     * the stats are only fetched if the solver is gold star eligible at the beginning
     * to reduce calls to the API
     *
     * to determine the gold star eligibility the puzzles progress must be loaded
     * so 'PUZZLE_READY' is used as the trigger for fetching the stats
     */
    if (
      action.type === PUZZLE_READY &&
      getGoldStarEligibility(stateAfterAction)
    ) {
      const userType = action.payload.user
      getStatsAndStreaks(userType)(
        store.dispatch,
        store.getState,
        null,
      )
    }

    if (!getCurrentUserSettings(stateAfterAction).showMilestones) {
      return result
    }

    if (!FILL_PERCENTAGE_ALTERING_ACTIONS.includes(action.type)) {
      return result
    }

    const percentBefore = getPercentFilled(stateBeforeAction)
    const percentAfter = getPercentFilled(stateAfterAction)
    if (percentBefore === percentAfter) return result

    updateMilestoneEligibility(percentBefore, percentAfter)

    if (isReadyToBurst(stateBeforeAction, stateAfterAction)) {
      const sendBurstIfEligible = makeBurstSender(stateBeforeAction)

      if (!sendBurstIfEligible(threeQtrMilestone)) {
        sendBurstIfEligible(halfwayMilestone)
      }
    }

    return result
  }

export default fillMonitorMiddleware
