import { UserType } from 'foundation'

import {
  CHECK,
  CLEAR,
  GUESS,
  REVEAL,
} from '../state/cells/cellsActions'
import {
  NAVIGATE,
  SELECT_CELL,
  SELECT_CLUE,
  SWITCH_DIRECTION,
} from '../state/selection/selectionActions'
import {
  DISABLE_AUTOCHECK,
  ENABLE_AUTOCHECK,
  MARK_SOLVED,
  RESET_GAME,
} from '../state/status/statusActions'
import {
  COMMIT_PROGRESS,
  PAUSE_TIMER,
  RESUME_TIMER,
  forceCommit,
} from '../state/timer/timerActions'
import {
  PUZZLE_READY,
  PUZZLE_SYNCED,
  setSynced,
} from '../state/transient/transientActions'
import {
  storeGame,
  getOrGenerateDeviceID,
} from '../utils/forageUtils'
import { pickShallow } from '../utils/misc'
import { syncGameState } from '../utils/syncGameState'
import logger from '../../../shared/utils/clientLogger'
import { getCurrentUserSettings } from '../state/user/userSelectors'
import { getPuzzleId } from '../state/puzzle/puzzleSelectors'
import { getTimerState } from '../state/timer/timerSelectors'
import { syncableKeys } from '../state/progressActions'

import { CrosswordMiddleware } from './type'

export const potentialSyncActions = [
  PUZZLE_SYNCED,
  MARK_SOLVED,
  RESUME_TIMER,
  PAUSE_TIMER,
  NAVIGATE,
  SELECT_CELL,
  SELECT_CLUE,
  SWITCH_DIRECTION,
  GUESS,
  REVEAL,
  CLEAR,
  CHECK,
  COMMIT_PROGRESS,
  RESET_GAME,
  ENABLE_AUTOCHECK,
  DISABLE_AUTOCHECK,
]

const dirtyActions = [GUESS, REVEAL, CLEAR, CHECK, RESET_GAME]

const hardSyncActions = [RESET_GAME, COMMIT_PROGRESS]

const SYNC_INTERVAL = 1000 * 60 * 5 // 5 minutes
const DEBUG_INTERVAL = 1000 * 5 // 5 seconds

const persistGameMiddleware: CrosswordMiddleware = (store) => {
  let boardHasDirt = false
  let syncInterval: number
  let userType: UserType

  const checkForDirt = () => {
    if (boardHasDirt) {
      boardHasDirt = false
      store.dispatch(forceCommit())
    }
  }

  const dirtPatrol = (isDebugging = false) => {
    clearInterval(syncInterval)
    syncInterval = window.setInterval(
      checkForDirt,
      isDebugging ? DEBUG_INTERVAL : SYNC_INTERVAL,
    )
  }

  if (typeof window === 'object') dirtPatrol()

  return (next) => (action) => {
    const { type } = action

    if (type === PUZZLE_READY) {
      userType = action.payload.user
      return next(action)
    }

    const stateBefore = store.getState()
    const timerBefore = getTimerState(stateBefore)
    const settingsBefore = getCurrentUserSettings(stateBefore)

    /** execute the action */
    const result = next(action)

    if (!userType) return result

    const stateAfter = store.getState()
    const timerAfter = getTimerState(stateAfter)
    const settingsAfter = getCurrentUserSettings(stateAfter)
    const puzzleID = getPuzzleId(stateAfter)

    const isLogging = userType.inShortzMode

    if (!puzzleID) {
      return
    }

    if (
      settingsBefore.speedUpSyncInterval !==
      settingsAfter.speedUpSyncInterval
    ) {
      dirtPatrol(settingsAfter.speedUpSyncInterval) // Will only fire client-side
    }

    if (potentialSyncActions.includes(type)) {
      /** first, we persist the local game */
      const localProgress = pickShallow(stateAfter, syncableKeys)
      storeGame(userType.regiId, puzzleID, localProgress)
      if (isLogging) {
        logger.log('info: PERSISTED LOCAL GAME due to', action.type)
      }

      /** then we decide if we need to persist to the server */
      const isHardSync = hardSyncActions.includes(type)
      const isLeaving = timerAfter.isLeavingGame
      const wasPaused = !timerBefore.sessionStartTimestamp
      const isPaused = !timerAfter.sessionStartTimestamp
      const wasSolved = stateBefore.status.isSolved
      const { isSolved } = stateAfter.status
      const justGotPaused = !wasPaused && isPaused
      const justGotSolved = !wasSolved && isSolved
      const needToSync =
        isHardSync || justGotPaused || (justGotSolved && isPaused)

      /** something has made the content state dirty */
      if (dirtyActions.includes(type)) boardHasDirt = true

      /** finally, we send the local progress to the server */
      if (needToSync && userType.isLoggedIn && userType.regiId) {
        if (isLogging) logger.time('info: PERSISTED REMOTE GAME')

        getOrGenerateDeviceID()
          .then((deviceID) => {
            const ids = {
              deviceID,
              puzzleID,
              userID: userType.regiId,
            }
            syncGameState(
              ids,
              userType,
              localProgress,
              !!isLeaving,
            ).then((syncedState) => {
              store.dispatch(setSynced(syncedState?.lastCommitID))
              if (isLogging) {
                logger.timeEnd('info: PERSISTED REMOTE GAME')
              }
            })
          })
          .catch((e) => {
            console.error('Failed to generate device ID:', e)
          })
      }
    }

    return result
  }
}

export default persistGameMiddleware
