import analyticsTracking from '../analyticsTracking'
import { trackModuleInteraction } from '../../utils/tracker'
import { CHECK, CLEAR, REVEAL } from '../../state/cells/cellsActions'
import { CLOSE_MODAL } from '../../state/modal/modalActions'
import {
  PAUSE_TIMER,
  RESUME_TIMER,
} from '../../state/timer/timerActions'
import { RESET_GAME } from '../../state/status/statusActions'
import { TOGGLE_PENCIL } from '../../state/toolbar/toolbarActions'
import { getIsMini } from '../../state/puzzle/puzzleSelectors'

jest.mock('../../state/puzzle/puzzleSelectors', () => ({
  ...jest.requireActual('../../state/puzzle/puzzleSelectors'),
  getIsMini: jest.fn(),
}))

jest.mock('../../utils/tracker', () => ({
  trackModuleInteraction: jest.fn(),
}))

describe('middleware: analyticsTracking', () => {
  let next
  let getState
  let store
  let middleware

  beforeEach(() => {
    jest.resetAllMocks()
    next = jest.fn()
    getState = jest.fn()
    store = { getState }
    middleware = analyticsTracking(store)(next)
    getIsMini.mockReturnValue(false)
    getState.mockReturnValue({
      puzzle: {
        data: {
          meta: { id: 123 },
        },
      },
      cells: [
        { answer: 'A' },
        { answer: 'B' },
        { answer: 'C' },
        { answer: 'D' },
        { answer: 'A' },
        { answer: 'B' },
        { answer: 'C' },
        { answer: 'D' },
      ],
    })
  })

  test('calls next with the provided action', () => {
    const action = { type: 'AN_ACTION' }
    middleware(action)
    expect(next).toHaveBeenCalledWith(action)
  })

  test('calls impression events for revealing on the daily puzzle and mini puzzle', () => {
    middleware({
      type: REVEAL,
      payload: { scope: 'UNCHECKED_SCOPE' },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'reveal-incomplete',
    )

    middleware({ type: REVEAL, payload: { scope: 'LETTER_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'reveal-square',
    )

    middleware({
      type: REVEAL,
      payload: { scope: 'WORD_SCOPE', cellsInScope: [1, 4, 7] },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'reveal-word',
      'BAD',
    )

    middleware({ type: REVEAL, payload: { scope: 'PUZZLE_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'reveal-puzzle',
    )

    getIsMini.mockReturnValue(true)

    middleware({ type: REVEAL, payload: { scope: 'LETTER_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'mini-page',
      'reveal-square',
    )
  })

  test('calls impression events for checking on the daily and mini puzzle', () => {
    middleware({
      type: CHECK,
      payload: { scope: 'UNCHECKED_SCOPE' },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'check-incomplete',
    )

    middleware({ type: CHECK, payload: { scope: 'LETTER_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'check-square',
    )

    middleware({ type: CHECK, payload: { scope: 'WORD_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'check-word',
      undefined,
    )

    middleware({ type: CHECK, payload: { scope: 'PUZZLE_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'check-puzzle',
    )

    getIsMini.mockReturnValue(true)

    middleware({ type: CHECK, payload: { scope: 'WORD_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'mini-page',
      'check-word',
      undefined,
    )
  })

  test('calls impression events for clearing the daily and mini puzzle', () => {
    middleware({
      type: CLEAR,
      payload: { scope: 'UNCHECKED_SCOPE' },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'clear-incomplete',
    )

    middleware({ type: CLEAR, payload: { scope: 'LETTER_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'clear-square',
    )

    middleware({ type: CLEAR, payload: { scope: 'WORD_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'clear-word',
      undefined,
    )

    middleware({ type: CLEAR, payload: { scope: 'PUZZLE_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'clear-puzzle',
    )

    getIsMini.mockReturnValue(true)

    middleware({ type: CLEAR, payload: { scope: 'PUZZLE_SCOPE' } })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'mini-page',
      'clear-puzzle',
    )
  })

  test('calls impression event for toggling pencil on and off', () => {
    getState.mockReturnValue({
      puzzle: {
        data: {
          meta: { id: 123 },
        },
      },
      toolbar: { inPencilMode: true },
    })

    middleware({ type: TOGGLE_PENCIL })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'pen',
    )

    getState.mockReturnValue({
      puzzle: {
        data: {
          meta: { id: 123 },
        },
      },
      toolbar: { inPencilMode: false },
    })

    middleware({ type: TOGGLE_PENCIL })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'pencil',
    )
  })

  test('calls clear-cancel impression event when closing the clear modal on the daily', () => {
    getState.mockReturnValue({
      puzzle: {
        data: {
          meta: { id: 123 },
        },
      },
      modal: { config: { actionType: CLEAR } },
    })

    middleware({
      type: CLOSE_MODAL,
      payload: { isCancelled: true },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'clear-cancel',
    )
  })

  test('calls clear-cancel impression event when closing the clear modal on the mini puzzle', () => {
    getIsMini.mockReturnValue(true)
    getState.mockReturnValue({
      puzzle: {
        data: {
          meta: { id: 123 },
        },
      },
      modal: { config: { actionType: RESET_GAME } },
    })

    middleware({
      type: CLOSE_MODAL,
      payload: { isCancelled: true },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'mini-page',
      'clear-cancel',
    )
  })

  test('does not call clear-cancel impression event when isCancelled action payload is false', () => {
    getState.mockReturnValue({
      puzzle: {
        data: {
          meta: { id: 123 },
        },
      },
      modal: { config: { modalType: CLEAR } },
    })

    middleware({
      type: CLOSE_MODAL,
      payload: { isCancelled: false },
    })
    expect(trackModuleInteraction).not.toHaveBeenCalled()
  })

  test('calls clear-confirm impression event when resetting the puzzle', () => {
    middleware({ type: RESET_GAME })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'clear-confirm',
    )
  })

  test('calls stop-timer impression event when the timer stops', () => {
    middleware({ type: PAUSE_TIMER })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'stop-timer',
    )
  })

  test('calls start-timer impression event when the timer starts', () => {
    middleware({ type: RESUME_TIMER })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      123,
      'game-page',
      'start-timer',
    )
  })
})
