import fillMonitor from '../fillMonitor'
import Juice from '../../../components/Juice'
import { GUESS, REVEAL } from '../../state/cells/cellsActions'
import { PUZZLE_READY } from '../../state/transient/transientActions'
import {
  getGameStatus,
  getGoldStarEligibility,
  getPercentFilled,
} from '../../state/status/statusSelectors'
import { getCurrentUserSettings } from '../../state/user/userSelectors'
import {
  getSelectedCell,
  getSelectedClue,
} from '../../state/selection/selectionSelectors'
import { getAllClues } from '../../state/puzzle/puzzleSelectors'

jest.mock('../../state/status/statusSelectors', () => ({
  getGameStatus: jest.fn(),
  getPercentFilled: jest.fn(),
  getGoldStarEligibility: jest.fn(),
}))

jest.mock('../../state/selection/selectionSelectors', () => ({
  getSelectedCell: jest.fn(),
  getSelectedClue: jest.fn(),
}))

jest.mock('../../state/user/userSelectors', () => ({
  getCurrentUserSettings: jest.fn(),
}))

jest.mock('../../state/puzzle/puzzleSelectors', () => ({
  getAllClues: jest.fn(),
}))

jest.mock('../../thunks/statsThunks', () => ({
  getStatsAndStreaks: () => (dispatch, getState) => {
    dispatch('dispatch from getStatsAndStreaks')
    getState('getState from getStatsAndStreaks')
  },
}))

jest.mock('../../../components/Juice', () => ({
  emitter: {
    emit: jest.fn(),
  },
}))

jest.mock('foundation', () => ({
  env: { name: 'test' },
}))

describe('middleware: fillMonitor', () => {
  let getState
  let next
  let middleware
  let store
  let dispatch

  beforeEach(() => {
    jest.resetAllMocks()
    next = jest.fn()
    dispatch = jest.fn()
    getState = jest.fn()
    store = { getState, dispatch }

    getPercentFilled.mockImplementation(
      (state) => state.percentFilled,
    )
    getCurrentUserSettings.mockReturnValue({ showMilestones: true })
  })

  const setupState = ({
    percentBefore,
    percentAfter,
    clueIndex = 0,
    cellIndex = 0,
    unfilledBefore = 1,
    unfilledAfter = 0,
    incorrectCells = 0,
    goldStarEligibility = false,
  }) => {
    getSelectedCell.mockReturnValue({ index: cellIndex })
    getSelectedClue.mockReturnValue({
      unfilledCount: unfilledBefore,
      index: clueIndex,
    })
    getAllClues.mockReturnValue([{ unfilledCount: unfilledAfter }])
    getGameStatus.mockReturnValue({ incorrectCells })
    getGoldStarEligibility.mockReturnValue(goldStarEligibility)

    getState.mockReturnValue({ percentFilled: percentBefore })

    next.mockImplementation((action) => {
      getState.mockReturnValue({ percentFilled: percentAfter })
      return action.payload
    })

    middleware = fillMonitor(store)(next)
  }

  test('ignore other actions', () => {
    middleware = fillMonitor({ getState })(next)

    const action = { type: 'OTHER_ACTION' }
    middleware(action)
    expect(next).toHaveBeenCalledWith(action)
    expect(getPercentFilled).not.toHaveBeenCalled()
  })

  test('fetches stats in background if game is still gold star eligible', () => {
    setupState({ goldStarEligibility: true })
    middleware({
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true } },
    })
    expect(dispatch).toHaveBeenCalledWith(
      'dispatch from getStatsAndStreaks',
    )
    expect(getState).toHaveBeenCalledWith(
      'getState from getStatsAndStreaks',
    )
  })

  test('does not fetch stats in background if game is no longer gold star eligible', () => {
    setupState({ goldStarEligibility: false })
    middleware({ type: PUZZLE_READY })
    expect(dispatch).not.toHaveBeenCalledWith(
      'dispatch from getStatsAndStreaks',
    )
    expect(getState).not.toHaveBeenCalledWith(
      'getState from getStatsAndStreaks',
    )
  })

  test('halfway milestone does not fire because percent filled has not passed threshold', () => {
    setupState({ percentBefore: 0.48, percentAfter: 0.49 })

    const action = { type: GUESS }
    middleware(action)
    expect(Juice.emitter.emit).not.toHaveBeenCalled()
  })

  test('halfway milestone does not fire because guess is not last letter of clue', () => {
    setupState({
      percentBefore: 0.49,
      percentAfter: 0.5,
      unfilledBefore: 2,
      unfilledAfter: 1,
    })

    const action = { type: GUESS }
    middleware(action)
    expect(Juice.emitter.emit).not.toHaveBeenCalled()
  })

  test('halfway milestone does not fire because percent filled has not passed threshold', () => {
    setupState({ percentBefore: 0.48, percentAfter: 0.49 })

    const action = { type: GUESS }
    middleware(action)
    expect(Juice.emitter.emit).not.toHaveBeenCalled()
  })

  test('milestones do not fire because setting is turned off', () => {
    setupState({ percentBefore: 0.74, percentAfter: 0.76 })
    getCurrentUserSettings.mockReturnValue({ showMilestones: false })
    middleware({ type: GUESS })
    expect(Juice.emitter.emit).not.toHaveBeenCalled()
  })

  test('3/4 milestone fires', () => {
    setupState({
      percentBefore: 0.74,
      percentAfter: 0.76,
      incorrectCells: 1,
    })

    const action = { type: GUESS }
    middleware(action)
    expect(Juice.emitter.emit).toHaveBeenCalledWith('milestone', {
      text: '3/4 Done!',
      idx: 0,
    })
  })

  test('3/4 milestone fires when both thresholds were passed at once', () => {
    setupState({
      percentBefore: 0.25,
      percentAfter: 0.76,
      incorrectCells: 1,
    })

    const action = { type: GUESS }
    middleware(action)
    expect(Juice.emitter.emit).toHaveBeenCalledWith('milestone', {
      text: '3/4 Done!',
      idx: 0,
    })
    expect(Juice.emitter.emit).toHaveBeenCalledTimes(1)
  })

  test('milestone does not fire on reveal puzzle', () => {
    setupState({ percentBefore: 0, percentAfter: 1 })

    const action = {
      type: REVEAL,
      payload: { scope: 'PUZZLE_SCOPE' },
    }
    middleware(action)
    expect(Juice.emitter.emit).not.toHaveBeenCalled()
  })
})
