import persistGame, { potentialSyncActions } from '../persistGame'
import {
  getOrGenerateDeviceID,
  storeGame,
} from '../../utils/forageUtils'
import { syncGameState } from '../../utils/syncGameState'
import {
  PUZZLE_READY,
  setSynced,
} from '../../state/transient/transientActions'
import { getCurrentUserSettings } from '../../state/user/userSelectors'
import { getTimerState } from '../../state/timer/timerSelectors'
import { getPuzzleId } from '../../state/puzzle/puzzleSelectors'

jest.mock('redux') // eliminates a warning in the test
jest.mock('../../../../shared/utils/clientLogger')

jest.mock('../../utils/forageUtils', () => ({
  storeGame: jest.fn(),
  getOrGenerateDeviceID: jest.fn(),
}))

jest.mock('../../utils/syncGameState', () => ({
  syncGameState: jest.fn(),
}))

jest.mock('../../state/user/userSelectors', () => ({
  getCurrentUserSettings: jest.fn(),
}))

jest.mock('../../state/timer/timerSelectors', () => ({
  getTimerState: jest.fn(),
}))

jest.mock('../../state/puzzle/puzzleSelectors', () => ({
  getPuzzleId: jest.fn(),
}))

jest.mock('../../state/transient/transientActions', () => ({
  PUZZLE_SYNCED: 'PUZZLE_SYNCED',
  setSynced: jest.fn(),
}))

describe('middleware: persist game progress', () => {
  let user
  let syncedState
  let timerBefore
  let timerAfter
  let puzzleBefore
  let puzzleAfter
  let getState
  let dispatch
  let next
  let middleware
  let userID
  let deviceID
  let puzzleID
  let lastCommitID

  beforeEach(() => {
    jest.resetAllMocks()
    next = jest.fn()
    getState = jest.fn()
    dispatch = jest.fn()
    user = { isLoggedIn: true, regiId: '123456789' }
    deviceID = 120
    puzzleID = 42
    lastCommitID = 400
    timerBefore = { sessionStartTimestamp: 100 }
    timerAfter = { sessionStartTimestamp: 100 }
    puzzleBefore = {
      cells: [1, 2, 3],
      status: { isSolved: false },
      timer: timerBefore,
      selection: { foo: true },
    }
    puzzleAfter = {
      cells: [1, 2, 3],
      status: { isSolved: false },
      timer: timerAfter,
      selection: { foo: true },
    }
    syncedState = { lastCommitID }

    /** simplified selectors */
    getCurrentUserSettings.mockImplementation((state) => state.user)
    getTimerState.mockImplementation((state) => state.timer)
    getPuzzleId.mockImplementation(() => puzzleID)

    /** make promises sync */
    getOrGenerateDeviceID.mockImplementation(() => ({
      then: (fn) => {
        fn(deviceID)
        return {
          catch: (err) => {
            console.log('a test error', err)
          },
        }
      },
    }))
    syncGameState.mockImplementation(() => ({
      then: (fn) => {
        fn(syncedState)
      },
    }))

    /** make action creator pass through */
    setSynced.mockImplementation((v) => ({ payload: v }))

    /** before next is fired return before state */
    getState.mockReturnValue({
      user,
      timer: timerBefore,
      ...puzzleBefore,
    })

    next.mockImplementation((action) => {
      /** after next fires return after state */
      if (
        action.type === 'PUZZLE_READY' ||
        action.type === undefined
      ) {
        getState.mockReturnValue({
          user,
          timer: timerBefore,
          ...puzzleBefore,
        })
      } else {
        getState.mockReturnValue({
          user,
          timer: timerAfter,
          ...puzzleAfter,
        })
      }

      return action.payload
    })

    middleware = persistGame({ getState, dispatch })(next)
  })

  test('all action types are defined', () => {
    potentialSyncActions.forEach((action) => {
      expect(typeof action).toBe('string')
    })
  })

  test('middleware should respond to set synced (ensures local is up to date)', () => {
    expect(potentialSyncActions.includes('PUZZLE_SYNCED')).toBe(true)
  })

  test('ignore other actions', () => {
    const preliminary_action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true } },
    }
    middleware(preliminary_action)

    const action = { type: 'OTHER_ACTION' }
    middleware(action)
    expect(next).toHaveBeenCalledWith(action)
    expect(storeGame).not.toHaveBeenCalled()
  })

  test('store local progress', () => {
    const preliminary_action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true } },
    }
    middleware(preliminary_action)

    const action = { type: potentialSyncActions[0] }
    middleware(action)
    expect(next).toHaveBeenCalledWith(action)

    expect(storeGame).toHaveBeenCalledWith(userID, puzzleID, {
      cells: [1, 2, 3],
      status: { isSolved: false },
      timer: { sessionStartTimestamp: 100 },
      selection: { foo: true },
    })
  })

  test('store remote game', () => {
    const preliminary_action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true, regiId: user.regiId } },
    }
    middleware(preliminary_action)

    const action = { type: potentialSyncActions[0] }
    const isLeavingGame = true
    const expectedIDs = { deviceID, puzzleID, userID: user.regiId }
    const expectedLocalProgress = {
      cells: [1, 2, 3],
      status: { isSolved: false },
      timer: {
        sessionStartTimestamp: undefined,
        isLeavingGame: true,
      },
      selection: { foo: true },
    }

    /** simulate a server-worthy change */
    timerAfter.sessionStartTimestamp = undefined // paused before
    timerAfter.isLeavingGame = isLeavingGame

    /** trigger the update */
    middleware(action)

    /** make sure everything got the correct arguments */
    expect(syncGameState).toHaveBeenCalledWith(
      expectedIDs,
      user,
      expectedLocalProgress,
      isLeavingGame,
    )
    expect(dispatch).toHaveBeenCalledWith({ payload: lastCommitID })
  })

  test('do not store remote if user is logged out', () => {
    const preliminary_action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: false } },
    }
    middleware(preliminary_action)

    /** user is logged out */
    user.isLoggedIn = false
    /** simulate a server-worthy change */
    timerAfter.sessionStartTimestamp = undefined
    middleware({ type: potentialSyncActions[0] })
    expect(syncGameState).not.toHaveBeenCalled()
  })

  test('do not store remote if user unpauses', () => {
    const action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true } },
    }
    middleware(action)

    /** simulate paused before */
    timerBefore.sessionStartTimestamp = undefined
    middleware({ type: potentialSyncActions[0] })
    expect(syncGameState).not.toHaveBeenCalled()
  })

  test('store remote if user pauses', () => {
    const action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true, regiId: user.regiId } },
    }
    middleware(action)

    /** simulate paused after */
    timerAfter.sessionStartTimestamp = undefined
    middleware({ type: potentialSyncActions[0] })
    expect(syncGameState).toHaveBeenCalled()
  })

  test('store remote if user just solved and game is paused', () => {
    const action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true, regiId: user.regiId } },
    }
    middleware(action)

    /** simulate paused after and solved */
    timerAfter.sessionStartTimestamp = undefined
    puzzleAfter.status.isSolved = true
    middleware({ type: potentialSyncActions[0] })
    expect(syncGameState).toHaveBeenCalled()
  })

  test('do not store remote if user just solved and game is not paused', () => {
    const action = {
      type: PUZZLE_READY,
      payload: { user: { isLoggedIn: true } },
    }
    middleware(action)

    /** simulate paused after and solved */
    puzzleAfter.status.isSolved = true
    middleware({ type: potentialSyncActions[0] })
    expect(syncGameState).not.toHaveBeenCalled()
  })
})
