import { CrosswordAction } from '../crosswordAction'
import { DEBUG_REVEAL, RESET_GAME } from '../status/statusActions'

import { CHECK, CLEAR, GUESS, REVEAL } from './cellsActions'
import cellsReducer from './cellsReducer'
import { CellState } from './cellsTypes'

const CELL_INDEX = 0
const ORIGINAL_TIME = 10000
const ACTION_TIME = 20000
const INCORRECT_GUESS = 'A'
const CORRECT_GUESS = 'B'
const EMPTY_GUESS = ''

const emptyCell: CellState = {
  type: 1,
  clues: [],
  answer: CORRECT_GUESS,
  label: '1',
  index: CELL_INDEX,
  guess: EMPTY_GUESS,
  penciled: false,
  checked: false,
  confirmed: false,
  revealed: false,
  modified: false,
  timestamp: ORIGINAL_TIME,
}

const incorrectCell = {
  ...emptyCell,
  guess: INCORRECT_GUESS,
}

const correctCell = {
  ...emptyCell,
  guess: CORRECT_GUESS,
}

const confirmedCell = {
  ...correctCell,
  confirmed: true,
}

const checkedCell = {
  ...incorrectCell,
  confirmed: false,
  checked: true,
  modified: false,
}

const actionPayload = {
  affectedCells: [CELL_INDEX], // Forces action to target this cell
  cellsInScope: [],
  blankDeltas: [],
  incorrectDeltas: [],
  now: ACTION_TIME, // Actions with effects will update the cell timestamp
}

const clearAction: CrosswordAction = {
  type: CLEAR,
  payload: actionPayload,
}

const revealAction: CrosswordAction = {
  payload: actionPayload,
  type: REVEAL,
}

const checkAction: CrosswordAction = {
  payload: actionPayload,
  type: CHECK,
}

const debugAction: CrosswordAction = {
  payload: {
    ...actionPayload,
    index: 1,
  },
  type: DEBUG_REVEAL,
}

const resetAction: CrosswordAction = {
  payload: actionPayload,
  type: RESET_GAME,
}

const makeGuessAction = (payload = {}): CrosswordAction => ({
  type: GUESS,
  payload: {
    inPencilMode: false,
    index: CELL_INDEX,
    value: INCORRECT_GUESS,
    now: ACTION_TIME,
    blankDelta: 0,
    incorrectDelta: 0,
    autocheckEnabled: false,
    fromRebus: false,
    ...payload,
  },
})
const incorrectGuessAction = makeGuessAction()
const correctGuessAction = makeGuessAction({ value: CORRECT_GUESS })
const rebusGuessAction = makeGuessAction({ fromRebus: true })
const penciledGuessAction = makeGuessAction({ inPencilMode: true })
const autocheckedWrongGuessAction = makeGuessAction({
  autocheckEnabled: true,
})

const autoconfirmedGuessAction = makeGuessAction({
  autocheckEnabled: true,
  value: CORRECT_GUESS,
})

describe('cells reducer', () => {
  it('clear should empty filled cell', () => {
    const cleared = cellsReducer([incorrectCell], clearAction)
    expect(cleared[0].guess).toEqual(EMPTY_GUESS)
    expect(cleared[0].timestamp).toEqual(ACTION_TIME)
  })
  it('clear should not affect confirmed or blank cell', () => {
    expect(
      cellsReducer([emptyCell], clearAction)[0].timestamp,
    ).toEqual(ORIGINAL_TIME)
    expect(
      cellsReducer([confirmedCell], clearAction)[0].timestamp,
    ).toEqual(ORIGINAL_TIME)
  })
  it('reset should override cell’s confirmed state', () => {
    const reset = cellsReducer([confirmedCell], resetAction)[0]
    expect(reset.guess).toEqual(EMPTY_GUESS)
    expect(reset.confirmed).toEqual(false)
    expect(reset.penciled).toEqual(false)
    expect(reset.timestamp).toEqual(ACTION_TIME)
  })

  it('reveal should confirm correct cell but not mark revealed', () => {
    const revealed = cellsReducer([correctCell], revealAction)[0]
    expect(revealed.guess).toEqual(CORRECT_GUESS)
    expect(revealed.confirmed).toEqual(true)
    expect(revealed.revealed).toEqual(false)
  })
  it('reveal should confirm an empty cell and mark it revealed', () => {
    const revealed = cellsReducer([emptyCell], revealAction)[0]
    expect(revealed.guess).toEqual(CORRECT_GUESS)
    expect(revealed.confirmed).toEqual(true)
    expect(revealed.revealed).toEqual(true)
  })
  it('debug should apply correct guess and remove checked flag', () => {
    const debugged = cellsReducer([checkedCell], debugAction)[0]
    expect(debugged.guess).toEqual(CORRECT_GUESS)
    expect(debugged.checked).toEqual(false)
    expect(debugged.timestamp).toEqual(ACTION_TIME)
  })

  it('checking while incorrect should mark incorrect and unmodified', () => {
    const checked = cellsReducer([incorrectCell], checkAction)[0]
    expect(checked.checked).toEqual(true)
    expect(checked.modified).toEqual(false)
    expect(checked.guess).toEqual(INCORRECT_GUESS)
  })

  it('check should not affect confirmed or blank cell', () => {
    expect(
      cellsReducer([emptyCell], checkAction)[0].timestamp,
    ).toEqual(ORIGINAL_TIME)

    expect(
      cellsReducer([confirmedCell], checkAction)[0].timestamp,
    ).toEqual(ORIGINAL_TIME)
  })

  it('check should confirm correct cell but not mark checked', () => {
    const checked = cellsReducer([correctCell], checkAction)[0]
    expect(checked.guess).toEqual(CORRECT_GUESS)
    expect(checked.confirmed).toEqual(true)
    expect(checked.checked).toEqual(false)
  })

  it('guessing should update an unconfirmed cell', () => {
    const cell = cellsReducer([checkedCell], incorrectGuessAction)[0]
    expect(cell.guess).toEqual(INCORRECT_GUESS)
    expect(cell.checked).toEqual(true)
    expect(cell.modified).toEqual(true)
    expect(cell.confirmed).toEqual(false)
    expect(cell.revealed).toEqual(false)
  })

  it('guessing should not update a confirmed cell...', () => {
    const cell = cellsReducer(
      [confirmedCell],
      incorrectGuessAction,
    )[0]
    expect(cell).toEqual(confirmedCell)
  })
  it('...unless using rebus mode', () => {
    const cell = cellsReducer([confirmedCell], rebusGuessAction)[0]
    expect(cell.guess).toEqual(INCORRECT_GUESS)
    expect(cell.confirmed).toEqual(false)
  })

  it('guessing the same guess should not update a cell...', () => {
    const cell = cellsReducer(
      [incorrectCell],
      incorrectGuessAction,
    )[0]
    expect(cell).toEqual(incorrectCell)
  })
  it('...unless the pencil state is changing', () => {
    const cell = cellsReducer([incorrectCell], penciledGuessAction)[0]
    expect(cell.penciled).toEqual(true)
    expect(
      cellsReducer([cell], incorrectGuessAction)[0].penciled,
    ).toEqual(false)
  })

  it('manual and autocheck should behave the same with an incorrect guess', () => {
    const autochecked = cellsReducer(
      [emptyCell],
      autocheckedWrongGuessAction,
    )
    const guessed = cellsReducer([emptyCell], incorrectGuessAction)
    const manuallyChecked = cellsReducer(guessed, checkAction)
    expect(autochecked).toEqual(manuallyChecked)
  })

  it('manual and autocheck should behave the same with a correct guess', () => {
    const autochecked = cellsReducer(
      [emptyCell],
      autoconfirmedGuessAction,
    )
    const guessed = cellsReducer([emptyCell], correctGuessAction)
    const manuallyChecked = cellsReducer(guessed, checkAction)
    expect(autochecked).toEqual(manuallyChecked)
  })

  it('all valid rebus cell answers should be marked correct', () => {
    const rebusCell = {
      ...emptyCell,
      answer: 'CHICKEN/ROAD',
      moreAnswers: {
        valid: ['CHICKEN', 'ROAD', 'CR', 'C', 'R', 'CHICKENROAD'],
      },
    }
    rebusCell.moreAnswers.valid
      .concat(rebusCell.answer)
      .forEach((guess) => {
        const guessed = cellsReducer(
          [rebusCell],
          makeGuessAction({ value: guess }),
        )
        expect(
          cellsReducer(guessed, checkAction)[0].confirmed,
        ).toEqual(true)
        expect(cellsReducer(guessed, revealAction)[0].guess).toEqual(
          rebusCell.answer,
        )
      })
  })

  it('black and external cells should never change', () => {
    const blackCell = { ...incorrectCell, type: 0 }
    const externalCell = { ...incorrectCell, type: 4 }
    expect(cellsReducer([blackCell], makeGuessAction())).toEqual([
      blackCell,
    ])
    expect(cellsReducer([blackCell], checkAction)).toEqual([
      blackCell,
    ])
    expect(cellsReducer([blackCell], revealAction)).toEqual([
      blackCell,
    ])
    expect(cellsReducer([externalCell], makeGuessAction())).toEqual([
      externalCell,
    ])
    expect(cellsReducer([externalCell], checkAction)).toEqual([
      externalCell,
    ])
    expect(cellsReducer([externalCell], revealAction)).toEqual([
      externalCell,
    ])
  })
})
