import { Reducer } from 'redux'

import type { CrosswordAction } from '../crosswordAction'
import { APPLY_PROGRESS } from '../progressActions'
import {
  DEBUG_REVEAL,
  ENABLE_AUTOCHECK,
  RESET_GAME,
} from '../status/statusActions'
import { PUZZLE_LOADED } from '../puzzle/puzzleActions'
import {
  hasModifiableContent,
  isCorrect,
  isLocked,
  isNavigable,
  isUnlockedAndNavigable,
  willBeAffectedByChecking,
} from '../../utils/cellUtils'
import { keysMatch } from '../../utils/misc'

import { CellState } from './cellsTypes'
import { CHECK, CLEAR, GUESS, REVEAL } from './cellsActions'

const revealCell = (cell: CellState): CellState => {
  if (!isUnlockedAndNavigable(cell)) return cell
  const correct = isCorrect(cell)
  return {
    ...cell,
    confirmed: true,
    checked: cell.checked && correct,
    penciled: false,
    revealed: !correct,
    guess: cell.answer,
  }
}

const debugCell = (cell: CellState) => ({
  ...cell,
  guess: cell.answer,
  checked: false,
})

const checkCell = (cell: CellState) => {
  if (!willBeAffectedByChecking(cell)) return cell
  const correct = isCorrect(cell)
  return {
    ...cell,
    confirmed: correct,
    checked: !correct || cell.checked,
    penciled: !correct && cell.penciled,
    modified: false,
  }
}

const updateGuess = (
  cell: CellState,
  guess: string,
  payload?: {
    inPencilMode: boolean
    fromRebus: boolean
    autocheckEnabled: boolean
  },
) => {
  const {
    inPencilMode: penciled = false,
    fromRebus: overrideLock = false,
    autocheckEnabled = false,
  } = payload || {}

  if (isLocked(cell) && !overrideLock) return cell

  const guessed: CellState = {
    ...cell,
    guess,
    penciled,
    modified: cell.checked,
    confirmed: false,
  }

  return autocheckEnabled ? checkCell(guessed) : guessed
}

// Clear only cell contents, and only of unlocked cells
const eraseCell = (cell: CellState) => {
  if (!hasModifiableContent(cell)) return cell

  return updateGuess(cell, '')
}

const resetCell = (cell: CellState) => {
  return {
    ...cell,
    penciled: false,
    revealed: false,
    checked: false,
    confirmed: false,
    modified: false,
    guess: '',
  }
}

export const monitoredProps: Array<keyof CellState> = [
  'guess',
  'checked',
  'confirmed',
  'modified',
  'revealed',
  'penciled',
]

// Applies the given transform to each navigable affectedCell
// If any key in monitoredProps is affected by the transformation,
// the cell's timestamp is updated
const cellsReducerHelper = (
  cells: CellState[],
  transform: (c: CellState) => CellState,
  affectedCells: Array<number>,
  now: number,
): CellState[] => {
  return cells.map((cell) => {
    if (!isNavigable(cell) || !affectedCells.includes(cell.index)) {
      return cell
    }

    const next = transform(cell)
    return keysMatch(cell, next, monitoredProps)
      ? next
      : { ...next, timestamp: now }
  })
}

// Cell state is represented as an array of objects, each containing
// state for a specific cell. With 2 exceptions,
// all actions are handled by calling out to the helper function
// defined above with the appropriate transform and list of affected cells
const cellsReducer: Reducer<CellState[], CrosswordAction> = (
  state = [],
  action,
) => {
  switch (action.type) {
    case PUZZLE_LOADED: {
      return action.payload.cells
    }

    case APPLY_PROGRESS: {
      return state.map((cell) => {
        if (!isNavigable(cell)) {
          return cell
        }

        return { ...cell, ...action.payload.cells[cell.index] }
      })
    }

    case RESET_GAME: {
      return cellsReducerHelper(
        state,
        resetCell,
        action.payload.affectedCells,
        action.payload.now,
      )
    }

    case DEBUG_REVEAL: {
      return cellsReducerHelper(
        state,
        (cell) =>
          action.payload.index <= cell.index
            ? resetCell(cell)
            : debugCell(cell),
        action.payload.affectedCells,
        action.payload.now,
      )
    }

    case GUESS: {
      const affectedCells =
        typeof action.payload.index === 'number'
          ? [action.payload.index]
          : []

      return cellsReducerHelper(
        state,
        (cell) =>
          updateGuess(cell, action.payload.value, action.payload),
        affectedCells,
        action.payload.now,
      )
    }

    case CLEAR: {
      return cellsReducerHelper(
        state,
        eraseCell,
        action.payload.affectedCells,
        action.payload.now,
      )
    }

    case REVEAL: {
      return cellsReducerHelper(
        state,
        revealCell,
        action.payload.affectedCells,
        action.payload.now,
      )
    }

    case CHECK:
    case ENABLE_AUTOCHECK: {
      return cellsReducerHelper(
        state,
        checkCell,
        action.payload.affectedCells,
        action.payload.now,
      )
    }

    default:
      return state
  }
}

export default cellsReducer
