import { createSelector } from 'reselect'
import moment from 'moment'
import 'moment-timezone'

import { getMomentNow } from '../../../../../shared/hub/utils/time'
import { getPuzzleMetadata } from '../puzzle/puzzleSelectors'
import { getPrintPrefs } from '../user/userSelectors'
import type { CrosswordStoreState } from '../crosswordReducer'

/**
 * Selectors for print options
 */

export const getPrintOptions = (state: CrosswordStoreState) =>
  state.printOptions

export const getIsGeneratable = createSelector(
  getPuzzleMetadata,
  (meta) => meta && !meta.category,
)

export const getPuzzleVersion = createSelector(
  getPrintOptions,
  (printOptions) => printOptions.version,
)

const getPdfVersion = (state: CrosswordStoreState) =>
  state.printOptions.version

export const getSolutionIsAvailable = createSelector(
  getPuzzleMetadata,
  (meta) => {
    if (!meta) {
      return false
    }
    const { category, publicationDate } = meta
    const thursdayAt10PM = moment(publicationDate)
      .tz('America/New_York')
      .add({ days: 4, hours: 22 })
      .format()

    const isSolutionAvailableNow = getMomentNow()
      .tz('America/New_York')
      .isSameOrAfter(thursdayAt10PM)

    // For PDF-only (category "other") puzzles, solution PDFs are
    // only available at 10pm the Thursday after publish date
    if (category === 8) return isSolutionAvailableNow

    return true
  },
)

export const getHasBothVersions = createSelector(
  getPuzzleMetadata,
  (meta) => {
    if (!meta) {
      return false
    }
    const { category, publicationDate } = meta

    if (category !== 0) return false // Only crosswords have both versions

    const stream = meta.publishStream

    let firstDateOfPdf

    if (stream === 'daily') {
      firstDateOfPdf = '2011-04-01'
    } else {
      return false
    }

    return moment(publicationDate).isSameOrAfter(firstDateOfPdf)
  },
)

export const getPrintLayoutsUnavailable = createSelector(
  getIsGeneratable,
  getPdfVersion,
  (generatable, version) => !generatable || version === 'newspaper',
)

export const getPrintOpacityUnavailable = createSelector(
  getIsGeneratable,
  getPdfVersion,
  (generatable, version) => !generatable || version === 'newspaper',
)

const getGeneratedPdfUrl = createSelector(
  getPuzzleMetadata,
  getPdfVersion,
  (meta, version) => {
    if (!meta) return ''
    const base = '/svc/crosswords/v2/puzzle/'
    const extension = version === 'solution' ? '.ans.pdf' : '.pdf'
    return `${base + meta.id + extension}`
  },
)

export const getPuzzleDate = createSelector(
  getPuzzleMetadata,
  (meta) => {
    if (!meta) return ''
    const fullMonth = moment(meta.publicationDate).format('MMMM')
    const year = moment(meta.publicationDate).format('YYYY')
    const dayOfMonth = moment(meta.publicationDate).format('D')

    return `${fullMonth} ${dayOfMonth}, ${year}`
  },
)

const getNewspaperPdfUrl = createSelector(
  getPuzzleMetadata,
  getPdfVersion,
  (puzzle, version) => {
    if (!puzzle) return ''
    const base = '/svc/crosswords/v2/puzzle/print/'
    const stream = puzzle.publishStream

    const dateString = moment(puzzle.publicationDate).format(
      'MMMDDYY',
    )
    const extension = version === 'solution' ? '.ans.pdf' : '.pdf'

    switch (stream) {
      case 'daily':
        break
      default:
        return ''
    }

    return `${base + dateString + extension}`
  },
)

const getQueryStrings = createSelector(
  getPrintPrefs,
  getPrintOpacityUnavailable,
  (printPrefs, inkSaverDisabled) => {
    const { opacity, layout } = printPrefs
    const params: Record<string, any> = {}

    if (!inkSaverDisabled && opacity < 100) {
      params.block_opacity = opacity
    }

    if (layout === 'southpaw') {
      params.southpaw = true
    } else if (layout === 'large_print') {
      params.large_print = true
    }

    const constructQuery = (parameters: Record<string, any>) =>
      Object.entries(parameters)
        .map((keyValue) => keyValue.join('='))
        .join('&')

    const populatedParams = Object.keys(params).length > 0
    return populatedParams ? `?${constructQuery(params)}` : ''
  },
)

export const getPrintPath = createSelector(
  getPrintOptions,
  getIsGeneratable,
  getGeneratedPdfUrl,
  getNewspaperPdfUrl,
  getQueryStrings,
  (
    options,
    generatable,
    generatedPdfUrl,
    newspaperPdfUrl,
    queryStrings,
  ) => {
    const pdfUrl = generatable ? generatedPdfUrl : newspaperPdfUrl
    switch (options.version) {
      case 'puzzle':
      case 'solution':
        return `${pdfUrl}${queryStrings}`
      case 'newspaper':
        return `${newspaperPdfUrl}${queryStrings}`
      default:
        return `${generatedPdfUrl}${queryStrings}`
    }
  },
)
