import { createSelector } from 'reselect'

import { capitalize } from '../../../utils/misc'
import { formatDate } from '../../../utils/time'
import type { CrosswordStoreState } from '../crosswordReducer'

export const getAllCells = (state: CrosswordStoreState) => state.cells

export const getExternalCells = createSelector(getAllCells, (cells) =>
  cells.map((c) => c.type === 4),
)

export const getAllClues = (state: CrosswordStoreState) => state.clues

const getPuzzleData = (state: CrosswordStoreState) =>
  state.puzzle.data

export const getPuzzleHasLoaded = (state: CrosswordStoreState) =>
  state.puzzle.hasLoaded

export const getPuzzleError = (state: CrosswordStoreState) =>
  state.puzzle.error

export const getPuzzleMetadata = createSelector(
  getPuzzleData,
  (data) => data?.meta || null,
)

export const getPuzzleBoard = createSelector(
  getPuzzleData,
  (data) => data?.board || null,
)

export const getAllClueLists = createSelector(
  getPuzzleData,
  (data) => data?.clueLists || [],
)

export const getOverlays = createSelector(
  getPuzzleData,
  (data) => data?.overlays || null,
)

export const getPuzzleDimensions = createSelector(
  getPuzzleData,
  (data) => data?.dimensions || null,
)

export const getPuzzleId = createSelector(
  getPuzzleMetadata,
  (meta) => meta?.id,
)

const getPuzzleMetadataTitle = createSelector(
  getPuzzleMetadata,
  (meta) => meta?.title,
)

export const getPuzzlePublishStream = createSelector(
  getPuzzleMetadata,
  (meta) => meta?.publishStream || '',
)

export const getIsMini = createSelector(
  getPuzzlePublishStream,
  (stream) => stream === 'mini',
)

export const getPublicationDate = createSelector(
  getPuzzleMetadata,
  (meta) => meta?.publicationDate,
)

export const getPageTitle = createSelector(
  [
    getPuzzlePublishStream,
    getPublicationDate,
    getPuzzleMetadataTitle,
    getPuzzleError,
  ],
  (publishStream, publicationDate, title, error) => {
    if (error) {
      if (error.status === 403) {
        return 'Subscription Required'
      }

      if (error.status === 404) {
        return 'Page Not Found'
      }

      return 'Oops! Something went wrong.'
    }

    const isSpecial = !publishStream

    let dateFormat = 'MMMM D, YYYY'

    if (publishStream === 'bonus') {
      dateFormat = 'MMMM YYYY'
    } else if (publishStream === 'daily') {
      dateFormat = 'dddd, MMMM D, YYYY'
    }

    const puzzleType =
      publishStream === 'mini'
        ? `Daily ${capitalize(publishStream)}`
        : capitalize(publishStream || 'special')

    const shouldIncludePuzzleTitle =
      isSpecial || publishStream === 'bonus'

    let base = `${puzzleType} Crossword Puzzle`

    if (!isSpecial && publicationDate) {
      base = `${formatDate(publicationDate, dateFormat)} ${base}`
    }

    return shouldIncludePuzzleTitle ? `${base} - ${title}` : base
  },
)
