import { Reducer, combineReducers } from 'redux'

import type { CrosswordAction } from '../crosswordAction'
import { CHECK, CLEAR, GUESS, REVEAL } from '../cells/cellsActions'
import {
  APPLY_PROGRESS,
  RECALCULATE_COUNTS,
} from '../progressActions'
import {
  PUZZLE_READY,
  PUZZLE_SYNCED,
} from '../transient/transientActions'
import { isBlank, isIncorrect } from '../../utils/cellUtils'

import {
  DEBUG_REVEAL,
  DISABLE_AUTOCHECK,
  ENABLE_AUTOCHECK,
  MARK_FILLED,
  MARK_SOLVED,
  RESET_GAME,
  SET_INITIAL_CELL_STATUS,
  TRACK_PROGRESS,
  UNMARK_FILLED,
} from './statusActions'
import { calculateProgressPercentages } from './statusSelectors'
import { FirstTimestamps } from './statusTypes'

export const PROGRESS_STEP_INTERVAL = 0.2

const setFirst = (
  firsts: FirstTimestamps,
  key: keyof FirstTimestamps,
  now: number,
): FirstTimestamps => {
  if (firsts[key]) return firsts

  return {
    ...firsts,
    [key]: now,
  }
}

const firsts: Reducer<FirstTimestamps, CrosswordAction> = (
  state = {},
  action,
) => {
  switch (action.type) {
    case APPLY_PROGRESS:
      return { ...state, ...action.payload.status.firsts }
    case PUZZLE_READY:
      return setFirst(state, 'opened', action.payload.now)
    case MARK_SOLVED:
      return setFirst(state, 'solved', action.payload.now)
    case REVEAL:
      return setFirst(state, 'revealed', action.payload.now)
    case CHECK:
      return setFirst(state, 'checked', action.payload.now)
    case ENABLE_AUTOCHECK:
      return setFirst(state, 'checked', action.payload.now)
    case CLEAR:
      return setFirst(state, 'cleared', action.payload.now)
    case RESET_GAME:
      return setFirst(state, 'timerReset', action.payload.now)
    default:
      return state
  }
}

const blankCells: Reducer<number, CrosswordAction> = (
  state = 0,
  action,
) => {
  switch (action.type) {
    case GUESS: {
      return state + (action.payload.blankDelta || 0)
    }

    case SET_INITIAL_CELL_STATUS:
    case RECALCULATE_COUNTS: {
      return action.payload.cells.filter(isBlank).length
    }

    case DEBUG_REVEAL:
    case REVEAL:
    case CLEAR: {
      const { blankDeltas } = action.payload
      return (
        state + blankDeltas.reduce((total, delta) => total + delta, 0)
      )
    }

    case RESET_GAME: {
      return action.payload.affectedCells.length
    }

    default:
      return state
  }
}

const incorrectCells: Reducer<number, CrosswordAction> = (
  state = 0,
  action,
) => {
  switch (action.type) {
    case GUESS: {
      return state + (action.payload.incorrectDelta || 0)
    }

    case SET_INITIAL_CELL_STATUS:
    case RECALCULATE_COUNTS: {
      return action.payload.cells.filter(isIncorrect).length
    }

    case DEBUG_REVEAL:
    case REVEAL:
    case CLEAR: {
      const { incorrectDeltas } = action.payload
      return (
        state +
        incorrectDeltas.reduce((total, delta) => total + delta, 0)
      )
    }

    case RESET_GAME: {
      return action.payload.affectedCells.length
    }

    default:
      return state
  }
}

const isSolved: Reducer<boolean, CrosswordAction> = (
  state = false,
  action,
) => {
  switch (action.type) {
    case RESET_GAME:
      return false
    case MARK_SOLVED:
      return true
    case APPLY_PROGRESS:
      return action.payload.status.isSolved || false
    default:
      return state
  }
}

const isFilled: Reducer<boolean, CrosswordAction> = (
  state = false,
  action,
) => {
  switch (action.type) {
    case MARK_FILLED:
      return true
    case UNMARK_FILLED:
      return false
    default:
      return state
  }
}

const autocheckEnabled: Reducer<boolean, CrosswordAction> = (
  state = false,
  action,
) => {
  switch (action.type) {
    // Default to false for games started prior to implementation of the feature
    case APPLY_PROGRESS:
      return action.payload.status.autocheckEnabled || false
    case ENABLE_AUTOCHECK:
      return true
    case DISABLE_AUTOCHECK:
      return false
    default:
      return state
  }
}

const lastCommitID: Reducer<string | null, CrosswordAction> = (
  state = null,
  action,
) => {
  switch (action.type) {
    case PUZZLE_SYNCED: {
      return action.payload.commitID || state
    }

    default:
      return state
  }
}

const currentProgress: Reducer<number, CrosswordAction> = (
  state = 0,
  action,
) => {
  switch (action.type) {
    case TRACK_PROGRESS: {
      return action.payload.currentProgress
    }

    case APPLY_PROGRESS: {
      return Math.floor(
        calculateProgressPercentages(action.payload) /
          PROGRESS_STEP_INTERVAL,
      )
    }

    case RESET_GAME: {
      return 0
    }

    default:
      return state
  }
}

export default combineReducers({
  firsts,
  isSolved,
  isFilled,
  autocheckEnabled,
  blankCells,
  incorrectCells,
  lastCommitID,
  currentProgress,
})
