import moment from 'moment'
import { createSelector } from 'reselect'

import { isNavigable } from '../../utils/cellUtils'
import type { CrosswordStoreState } from '../crosswordReducer'
import { ApplyProgressPayload } from '../progressActions'
import {
  getAllCells,
  getOverlays,
  getPuzzleMetadata,
} from '../puzzle/puzzleSelectors'
import { getCurrentUserSettings } from '../user/userSelectors'

export const getGameStatus = (state: CrosswordStoreState) =>
  state.status

export const getCurrentProgress = createSelector(
  getGameStatus,
  (status) => status.currentProgress,
)

export const getIsSolved = createSelector(
  getGameStatus,
  (status) => status.isSolved,
)

export const getAutocheckEnabled = createSelector(
  getGameStatus,
  (status) => status.autocheckEnabled,
)

export const calculateProgressPercentages = ({
  cells,
  status,
}:
  | Pick<CrosswordStoreState, 'cells' | 'status'>
  | ApplyProgressPayload) => {
  const navigableCells = cells.filter((c) => isNavigable(c)).length
  const blankCells = status?.blankCells || 0
  const filledCellCount = navigableCells - blankCells
  return filledCellCount / navigableCells
}

export const getPercentFilled = createSelector(
  [getAllCells, getGameStatus],
  (cells, status) => calculateProgressPercentages({ cells, status }),
)

export const getGoldStarEligibility = createSelector(
  [getGameStatus, getPuzzleMetadata, getCurrentUserSettings],
  ({ firsts }, meta, settings) => {
    if (!meta) return false

    switch (true) {
      case meta.publishStream !== 'daily':
        return false
      case settings.forceGoldStarEligibility:
        return true
      case !!firsts.revealed:
        return false
      case !!firsts.timerReset:
        return false
      case !!firsts.checked:
        return false
      // note the anti pattern of having a selector be time dependent
      case moment().isAfter(meta.goldStarCutoff):
        return false
      default:
        return true
    }
  },
)

export const getCurrentOverlay = createSelector(
  [getOverlays, getIsSolved],
  (overlays, isSolved) => {
    if (!overlays) return ''
    const { beforeStart, afterSolve } = overlays
    return isSolved ? afterSolve || beforeStart : beforeStart
  },
)
