import { MARK_SOLVED, RESET_GAME } from '../status/statusActions'
import { PUZZLE_SYNCED } from '../transient/transientActions'
import { MultiCellActionPayload } from '../cells/cellsTypes'

import timerReducer from './timerReducer'
import {
  RESUME_TIMER,
  PAUSE_TIMER,
  COMMIT_PROGRESS,
} from './timerActions'

const cellPayload: MultiCellActionPayload = {
  cellsInScope: [],
  affectedCells: [],
  blankDeltas: [],
  incorrectDeltas: [],
  now: 1631735116,
}

const basicTimerPayload = {
  now: 1631735117,
  commitID: '12345',
  isLeavingGame: false,
}

const timerState = {
  sessionStartTimestamp: 1631733110,
  totalElapsedTime: 1600,
  sessionElapsedTime: 5000,
  resetSinceLastCommit: true,
}

describe('Timer reducer', () => {
  it('puzzle sync sets reset flag to false', () => {
    const puzzleSynced = timerReducer(undefined, {
      type: PUZZLE_SYNCED,
      payload: { commitID: '123345' },
    })
    expect(puzzleSynced.resetSinceLastCommit).toBe(false)
  })

  it('resuming timer sets the session timestamp to current timestamp', () => {
    const resumeTimer = timerReducer(undefined, {
      type: RESUME_TIMER,
      payload: basicTimerPayload,
    })
    expect(resumeTimer.sessionStartTimestamp).toBe(1631735117)
  })

  it('pausing the timer removes the session start timestamp', () => {
    const pauseTimer = timerReducer(undefined, {
      type: PAUSE_TIMER,
      payload: basicTimerPayload,
    })
    expect(pauseTimer.sessionStartTimestamp).toBeUndefined
  })

  it('pausing the timer updates total elapsed time', () => {
    const pauseTimer = timerReducer(timerState, {
      type: PAUSE_TIMER,
      payload: basicTimerPayload,
    })
    const UPDATED_ELAPSED_TIME = 3607
    expect(pauseTimer.totalElapsedTime).toEqual(UPDATED_ELAPSED_TIME)
  })

  it('commiting progress updates session start timestamp to be current timestamp', () => {
    const commitProgress = timerReducer(undefined, {
      type: COMMIT_PROGRESS,
      payload: basicTimerPayload,
    })
    expect(commitProgress.sessionStartTimestamp).toBe(
      basicTimerPayload.now,
    )
  })

  it('commiting progress updates session start timestamp to be current timestamp and updates totalElapsed time', () => {
    const commitProgress = timerReducer(timerState, {
      type: COMMIT_PROGRESS,
      payload: basicTimerPayload,
    })
    const UPDATED_ELAPSED_TIME = 3607
    expect(commitProgress.totalElapsedTime).toEqual(
      UPDATED_ELAPSED_TIME,
    )
    expect(commitProgress.sessionStartTimestamp).toEqual(
      basicTimerPayload.now,
    )
  })

  it('resetting game resets timestamps and updates session timestamp to current timestamp', () => {
    const resetGame = timerReducer(undefined, {
      type: RESET_GAME,
      payload: cellPayload,
    })
    expect(resetGame.sessionStartTimestamp).toBe(cellPayload.now)
    expect(resetGame.totalElapsedTime).toBe(0)
    expect(resetGame.sessionElapsedTime).toBe(0)
    expect(resetGame.resetSinceLastCommit).toBe(true)
  })

  it('resetting game resets timestamps and updates session timestamp to current timestamp', () => {
    const resetGame = timerReducer(timerState, {
      type: RESET_GAME,
      payload: cellPayload,
    })
    expect(resetGame.sessionStartTimestamp).toBe(cellPayload.now)
    expect(resetGame.totalElapsedTime).toBe(0)
    expect(resetGame.sessionElapsedTime).toBe(0)
    expect(resetGame.resetSinceLastCommit).toBe(true)
  })

  it('marking solved updates session elapsed time to 0', () => {
    const markSolved = timerReducer(undefined, {
      type: MARK_SOLVED,
      payload: basicTimerPayload,
    })
    expect(markSolved.sessionElapsedTime).toBe(0)
  })

  it('marking solved updates session elapsed time to 0', () => {
    const markSolved = timerReducer(timerState, {
      type: MARK_SOLVED,
      payload: basicTimerPayload,
    })
    expect(markSolved.sessionElapsedTime).toBe(0)
  })
})
