import { Reducer } from 'redux'

import type { CrosswordAction } from '../crosswordAction'
import { APPLY_PROGRESS } from '../progressActions'
import { MARK_SOLVED, RESET_GAME } from '../status/statusActions'
import { PUZZLE_SYNCED } from '../transient/transientActions'

import {
  COMMIT_PROGRESS,
  PAUSE_TIMER,
  RESUME_TIMER,
} from './timerActions'
import { TimerState } from './timerTypes'

const initialState: TimerState = {
  sessionStartTimestamp: undefined,
  totalElapsedTime: 0,
  sessionElapsedTime: 0,
  resetSinceLastCommit: false,
}

const reducer: Reducer<TimerState, CrosswordAction> = (
  state = initialState,
  action,
) => {
  switch (action.type) {
    case APPLY_PROGRESS: {
      return {
        ...state,
        ...action.payload.timer,
      }
    }

    case PUZZLE_SYNCED: {
      return {
        ...state,
        resetSinceLastCommit: false,
      }
    }

    case RESUME_TIMER: {
      return {
        ...state,
        sessionStartTimestamp: action.payload.now,
        sessionElapsedTime: 0,
      }
    }

    case PAUSE_TIMER: {
      const { now, isLeavingGame } = action.payload
      const { sessionStartTimestamp, totalElapsedTime } = state

      // Failsafe in case timer is already paused
      const sessionElapsedTime = sessionStartTimestamp
        ? now - sessionStartTimestamp
        : 0

      return {
        ...state,
        sessionElapsedTime,
        sessionStartTimestamp: undefined,
        totalElapsedTime: totalElapsedTime + sessionElapsedTime,
        isLeavingGame,
      }
    }

    case COMMIT_PROGRESS: {
      const { sessionStartTimestamp, totalElapsedTime } = state
      const { now } = action.payload

      // Failsafe in case timer is already paused
      const sessionElapsedTime = sessionStartTimestamp
        ? now - sessionStartTimestamp
        : 0

      return {
        ...state,
        sessionElapsedTime,
        sessionStartTimestamp: now,
        totalElapsedTime: totalElapsedTime + sessionElapsedTime,
        isLeavingGame: false,
      }
    }

    case RESET_GAME: {
      return {
        totalElapsedTime: 0,
        sessionElapsedTime: 0,
        sessionStartTimestamp: action.payload.now,
        resetSinceLastCommit: true,
      }
    }

    case MARK_SOLVED: {
      // If the user is solving the puzzle via an action that requires confirmation
      // (e.g. Reveal Puzzle), the game will have already been paused by the opening of
      // the confirm modal, and upon confirming, the game will not have time to resume
      // the timer, where `sessionElapsedTime` would normally get reset, since the congrats
      // modal will immediately open after the confirmation one closes.
      return {
        ...state,
        sessionElapsedTime: 0,
      }
    }

    default:
      return state
  }
}

export default reducer
