import { AnyAction } from 'redux'
import { ThunkAction, ThunkDispatch } from 'redux-thunk'

import {
  acceptsGuess,
  hasModifiableContent,
  isBlank,
  isIncorrect,
  isLocked,
  isNavigable,
  isUnlockedAndNavigable,
  willBeAffectedByChecking,
} from '../utils/cellUtils'
import { CHECK, CLEAR, REVEAL } from '../state/cells/cellsActions'
import {
  DEBUG_REVEAL,
  ENABLE_AUTOCHECK,
  RESET_GAME,
} from '../state/status/statusActions'
import { CrosswordStoreState } from '../state/crosswordReducer'
import { CellState, Scope } from '../state/cells/cellsTypes'
import {
  LocalProgress,
  RemoteProgress,
} from '../state/progressActions'

export type CrosswordThunk<ReturnType = void> = ThunkAction<
  ReturnType,
  CrosswordStoreState,
  unknown,
  AnyAction
>

export type CrosswordThunkDispatch = ThunkDispatch<
  CrosswordStoreState,
  void,
  AnyAction
>

export const isDisqualifyingAction = (action: string) =>
  [CHECK, REVEAL, RESET_GAME, ENABLE_AUTOCHECK].includes(action)

export const isLocalLatest = (
  local: LocalProgress | null,
  remote: RemoteProgress | null,
) =>
  !remote || // no remote game means local is fresher
  !remote.lastCommitID || // you have local progress but never synced successfully
  local?.status?.lastCommitID === remote.lastCommitID

export const getPercentileByScope = (scope: Scope | null) => {
  switch (scope) {
    case Scope.Fifty:
      return 0.5
    case Scope.SeventyFive:
      return 0.75
    default:
      return 1
  }
}

export const getCellAtThresholdForScope = (
  cells: CellState[],
  scope: Scope | null,
) => {
  const percentile = getPercentileByScope(scope)
  const fillableCells = cells.filter((c) => isNavigable(c))
  const highestCellUnderPercentage = Math.ceil(
    fillableCells.length * percentile - 1,
  )

  return fillableCells[highestCellUnderPercentage].index
}

export const findCellsInScope = (
  state: CrosswordStoreState,
  scope: Scope | null,
) => {
  const { cells, selection } = state
  return (
    cells &&
    cells.filter((_, index) => {
      switch (scope) {
        case Scope.Letter:
          return selection && selection.cell === index
        case Scope.Word:
          return selection && selection.clueCells.includes(index)
        case Scope.Puzzle:
          return true
        default:
          return true // Default scope is all cells
      }
    })
  )
}

const findSelectedClueCellsWithoutFilledOpposingClues = ({
  cells,
  clues,
  selection,
}: CrosswordStoreState) => {
  const cellsOfSelectedClue = selection
    ? selection.clueCells.map((i) => cells && cells[i])
    : []

  return cellsOfSelectedClue.filter((c) => {
    const opposingClues = c?.clues.filter(
      (clueIndex) => clueIndex !== selection?.clue,
    )

    return opposingClues?.some(
      (clueIndex) => clues && clues[clueIndex].unfilledCount > 0,
    )
  })
}

export const determineCellsInScope = (
  state: CrosswordStoreState,
  scope: Scope | null,
) => {
  const scopeFilter =
    scope === Scope.Unchecked
      ? findSelectedClueCellsWithoutFilledOpposingClues
      : findCellsInScope

  return scopeFilter(state, scope)
}

export const getFilterByAction = (action: string) => {
  switch (action) {
    case CHECK:
    case ENABLE_AUTOCHECK:
      return willBeAffectedByChecking
    case REVEAL:
      return isUnlockedAndNavigable
    case DEBUG_REVEAL:
    case RESET_GAME:
      return isNavigable
    case CLEAR:
    default:
      return hasModifiableContent
  }
}

export const determineAffectedCells = (
  state: CrosswordStoreState,
  action: string,
  scope: Scope | null,
) => {
  const cellsInScope = determineCellsInScope(state, scope)
  const actionFilter = getFilterByAction(action)
  return cellsInScope?.filter(actionFilter)
}

export const getDelta = (
  cell: CellState,
  willBeAfter: boolean,
  wasBefore: boolean,
  isDebug: boolean,
) => {
  if (isLocked(cell) && !isDebug) return 0

  if (willBeAfter !== wasBefore) return willBeAfter ? 1 : -1

  return 0
}

export const getBlankDelta = (
  cell: CellState,
  newGuess: string,
  isDebug?: boolean,
) => getDelta(cell, !newGuess, isBlank(cell), !!isDebug)

export const getIncorrectDelta = (
  cell: CellState,
  newGuess: string,
  isDebug?: boolean,
) =>
  getDelta(
    cell,
    !acceptsGuess(cell, newGuess),
    isIncorrect(cell),
    !!isDebug,
  )

export const convertRebusToGuess = (string = '') =>
  string.toUpperCase().trim()
