import { captureSentryError, xhr } from 'foundation'

import { getMeterHasLoaded } from '../state/meter/meterSelectors'
import { retrieveMeterValue } from '../../../../shared/meter'
import { MeterProperties } from '../../../../shared/meter/MeterTypes'
import { meterLoaded, meterError } from '../state/meter/meterActions'
import { RawPuzzleData } from '../utils/adaptCrossword/types'
import { edgeHost } from '../utils/remoteUtils'
import adaptCrossword from '../utils/adaptCrossword'
import { setInitialCellStatus } from '../state/status/statusActions'
import {
  puzzleLoaded,
  puzzleError,
} from '../state/puzzle/puzzleActions'
import {
  getPuzzleHasLoaded,
  getPuzzleError,
} from '../state/puzzle/puzzleSelectors'

import { CrosswordThunk } from './helpers'

const sendMeterErrorToSentry = (error: Error, status: number) => {
  captureSentryError(error, {
    api: 'meter',
    category: 'meter-get',
  })
}

export const loadPuzzleWithMeter =
  (filename: string, stream: string): CrosswordThunk =>
  (dispatch, getState) => {
    const hasLoaded = getPuzzleHasLoaded(getState())
    const hasError = getPuzzleError(getState())

    // Do not reload with Meter if initial call is successful
    // TODO (XWD-21575): Remove first call after Meter integration is rolled out
    if (hasLoaded && !hasError) {
      return
    }

    const sendPuzzleErrorToSentry = (
      error: Error,
      status: number,
    ) => {
      // don't capture expected error codes (like if someone hits the page logged out)
      if (![401, 403, 404].includes(status) && status < 500) {
        captureSentryError(error, {
          api: 'games-puzzles',
          category: 'load-puzzle',
        })
      }
    }

    const puzzleApiGet: Promise<RawPuzzleData> =
      xhr.get<RawPuzzleData>(
        `${edgeHost}/v6/puzzle/${filename}.json`,
        {
          withStatus: true,
          headers: {
            'X-Games-Auth-Bypass': 'true',
          },
        },
      )

    const meterGet: Promise<MeterProperties> = retrieveMeterValue({})

    Promise.all([puzzleApiGet, meterGet])
      .then(([puzzleData, meterData]) => {
        if (
          (meterData.granted && meterData.grantReason === 'IP') ||
          !!puzzleData.freePuzzle
        ) {
          if (puzzleData.status === 200) {
            const { cells, clues, ...data } = adaptCrossword(
              puzzleData,
              stream,
            )
            dispatch(puzzleLoaded(cells, clues, data))
            dispatch(setInitialCellStatus(cells))
          } else {
            // if puzz call succeeds with anything other than 200, send an error to sentry
            const puzzErrorMessage = `puzzle call failed with status ${puzzleData.status}`

            sendPuzzleErrorToSentry(
              new Error(puzzErrorMessage),
              puzzleData.status,
            )
            dispatch(puzzleError(puzzleData.status))
          }
        }
      })
      .catch((e) => {
        // if puzz call fails or adapt crossword code throws, send an error to sentry
        sendPuzzleErrorToSentry(
          new Error('Puzzle API Fetch with bypass failed'),
          e.status,
        )
        console.error(e)
        dispatch(puzzleError(e.status))
      })
  }

export const callMeter =
  (): CrosswordThunk => (dispatch, getState) => {
    const hasLoaded = getMeterHasLoaded(getState())
    if (hasLoaded) {
      return
    }

    const meterGet: Promise<MeterProperties> = retrieveMeterValue({})

    meterGet
      .then((res: MeterProperties) => {
        if (!res.isOffline) {
          dispatch(meterLoaded(res))
        }
      })
      .catch((err) => {
        // if Meter call fails, send an error to sentry
        sendMeterErrorToSentry(
          new Error('Meter call failed on XWD URL'),
          err.status,
        )
        console.error(err)
        dispatch(meterError(err.status))
      })
  }
