import { captureSentryError, xhr } from 'foundation'

import { edgeHost } from '../utils/remoteUtils'
import adaptCrossword from '../utils/adaptCrossword'
import { setInitialCellStatus } from '../state/status/statusActions'
import {
  puzzleLoaded,
  puzzleError,
} from '../state/puzzle/puzzleActions'
import { RawPuzzleData } from '../utils/adaptCrossword/types'
import { getPuzzleHasLoaded } from '../state/puzzle/puzzleSelectors'

import { CrosswordThunk } from './helpers'

export const loadPuzzle =
  (filename: string, stream: string): CrosswordThunk =>
  (dispatch, getState) => {
    const hasLoaded = getPuzzleHasLoaded(getState())
    if (hasLoaded) {
      return
    }

    const sendErrorToSentry = (error: Error, status: number) => {
      // don't capture expected error codes (like if someone hits the page logged out)
      if (![401, 403, 404].includes(status) && status < 500) {
        captureSentryError(error, {
          api: 'games-puzzles',
          category: 'load-puzzle',
        })
      }
    }

    xhr
      .get<RawPuzzleData>(`${edgeHost}/v6/puzzle/${filename}.json`, {
        withStatus: true,
      })
      .then((res) => {
        if (res.status === 200) {
          const { cells, clues, ...data } = adaptCrossword(
            res,
            stream,
          )
          dispatch(puzzleLoaded(cells, clues, data))
          dispatch(setInitialCellStatus(cells))
        } else {
          // if puzz call succeeds with anything other than 200, send an error to sentry
          const puzzErrorMessage = `puzzle call failed with status ${res.status}`

          sendErrorToSentry(new Error(puzzErrorMessage), res.status)
          dispatch(puzzleError(res.status))
        }
      })
      .catch((err) => {
        // if puzz call fails or adapt crossword code throws, send an error to sentry
        sendErrorToSentry(
          new Error('Could not load XWD puzzle'),
          err.status,
        )
        console.error(err)
        dispatch(puzzleError(err.status))
      })
  }
