import moment from 'moment'

import {
  CHECK,
  CLEAR,
  GUESS,
  REVEAL,
} from '../state/cells/cellsActions'
import { PROGRESS_STEP_INTERVAL } from '../state/status/statusReducer'
import { getElapsedTime } from '../../utils/time'
import {
  DEBUG_REVEAL,
  DISABLE_AUTOCHECK,
  ENABLE_AUTOCHECK,
  MARK_FILLED,
  MARK_SOLVED,
  RESET_GAME,
  TRACK_PROGRESS,
  UNMARK_FILLED,
} from '../state/status/statusActions'
import { trackModuleInteraction } from '../utils/tracker'
import {
  CellState,
  MultiCellActionPayload,
  Scope,
} from '../state/cells/cellsTypes'
import {
  getIsMini,
  getPuzzleId,
} from '../state/puzzle/puzzleSelectors'
import {
  getAutocheckEnabled,
  getCurrentProgress,
  getGameStatus,
  getGoldStarEligibility,
  getPercentFilled,
} from '../state/status/statusSelectors'
import { getSelectedCell } from '../state/selection/selectionSelectors'
import { getCurrentUserSettings } from '../state/user/userSelectors'
import { openModal } from '../state/modal/modalActions'
import { CrosswordModal } from '../state/modal/modalTypes'
import {
  getSessionStartTimestamp,
  getTotalElapsedTime,
} from '../state/timer/timerSelectors'

import {
  CrosswordThunk,
  determineAffectedCells,
  determineCellsInScope,
  getBlankDelta,
  getCellAtThresholdForScope,
  getIncorrectDelta,
  isDisqualifyingAction,
} from './helpers'
import { exitRebus } from './toolbarThunks'

const PROGRESS_STEPS = [20, 40, 60, 80]

const getNewGuessByAction = (
  action: string,
  thresholdIndex: number,
) => {
  switch (action) {
    case DEBUG_REVEAL:
      return (cell: CellState) =>
        cell.index >= thresholdIndex ? '' : cell.answer
    case REVEAL:
      return (cell: CellState) => cell.answer
    default:
      return () => ''
  }
}

export const checkPuzzleProgress =
  (): CrosswordThunk => (dispatch, getState) => {
    const state = getState()
    const puzzleId = getPuzzleId(state)
    const currentProgress = getCurrentProgress(state)
    const totalElapsedTime = getTotalElapsedTime(state)
    const sessionStartTimestamp = getSessionStartTimestamp(state)

    if (!puzzleId) return

    const isMini = getIsMini(state)
    const eventAction = isMini ? 'mini-page' : 'game-page'
    const percentFilled = getPercentFilled(state)
    const newProgress = Math.floor(
      percentFilled / PROGRESS_STEP_INTERVAL,
    )

    const solveTime = getElapsedTime(
      totalElapsedTime,
      sessionStartTimestamp,
    )

    if (newProgress > currentProgress) {
      for (
        let i = currentProgress;
        i < newProgress && i < PROGRESS_STEPS.length;
        i += 1
      ) {
        trackModuleInteraction(
          'complete',
          puzzleId,
          eventAction,
          `${PROGRESS_STEPS[i]}%-complete`,
          `${solveTime}`,
        )
      }
      dispatch({
        type: TRACK_PROGRESS,
        payload: { currentProgress: newProgress },
      })
    }
  }

export const checkSolvedState =
  (): CrosswordThunk => (dispatch, getState) => {
    const { status } = getState()
    const {
      incorrectCells: incorrectCount,
      blankCells: blankCount,
      isFilled: wasFilled,
    } = status

    let action: string = ''
    const hasNoBlankCells = blankCount === 0
    const isFilledCorrectly = incorrectCount === 0
    const isFilledIncorrectly = hasNoBlankCells && !isFilledCorrectly
    const isNotFilled = !hasNoBlankCells

    if (isFilledCorrectly) {
      action = MARK_SOLVED
      dispatch(openModal(CrosswordModal.Congrats))
    } else if (isFilledIncorrectly && !wasFilled) {
      action = MARK_FILLED
      dispatch(openModal(CrosswordModal.Rats))
    } else if (isNotFilled && wasFilled) action = UNMARK_FILLED

    dispatch(checkPuzzleProgress())
    if (action) {
      dispatch({
        type: action,
        payload: { now: moment().unix() },
      })
    }
  }

export const guess =
  (value: string, fromRebus?: boolean): CrosswordThunk =>
  (dispatch, getState) => {
    const state = getState()
    const { isSolved, autocheckEnabled } = getGameStatus(state)

    if (isSolved) return

    const selectedCell = getSelectedCell(state)
    const { inPencilMode } = state.toolbar

    dispatch({
      type: GUESS,
      payload: {
        blankDelta:
          selectedCell && getBlankDelta(selectedCell, value),
        incorrectDelta:
          selectedCell && getIncorrectDelta(selectedCell, value),
        index: selectedCell && selectedCell.index,
        inPencilMode,
        autocheckEnabled,
        value,
        fromRebus: !!fromRebus,
        now: moment().unix(),
      },
    })

    dispatch(checkSolvedState())
  }

export type ExecutedActionType =
  | typeof RESET_GAME
  | typeof CLEAR
  | typeof REVEAL
  | typeof DEBUG_REVEAL
  | typeof CHECK
  | typeof ENABLE_AUTOCHECK
  | typeof GUESS

export const proceedWithAction =
  (
    actionType: ExecutedActionType,
    scope: Scope | null,
  ): CrosswordThunk =>
  (dispatch, getState) => {
    const puzzle = getState()
    const cellsInScope = determineCellsInScope(puzzle, scope)
    const affectedCells = determineAffectedCells(
      puzzle,
      actionType,
      scope,
    )

    const newGuess = getNewGuessByAction(
      actionType,
      getCellAtThresholdForScope(puzzle.cells, scope),
    )
    const isDebug = actionType === DEBUG_REVEAL
    const payload: MultiCellActionPayload = {
      scope: scope ?? undefined,
      cellsInScope: cellsInScope.map((c) => c.index),
      affectedCells: affectedCells.map((c) => c.index),
      blankDeltas: affectedCells.map((c) =>
        getBlankDelta(c, newGuess(c), isDebug),
      ),
      incorrectDeltas: affectedCells.map((c) =>
        getIncorrectDelta(c, newGuess(c), isDebug),
      ),
      now: moment().unix(),
    }

    if (isDebug) {
      Object.assign(payload, {
        index: getCellAtThresholdForScope(puzzle.cells, scope),
      })
    }

    dispatch({
      type: actionType,
      payload,
    })

    dispatch(checkSolvedState())
  }

export const execute =
  (
    actionType: ExecutedActionType,
    scope: Scope | null = null,
  ): CrosswordThunk =>
  (dispatch, getState) => {
    const state = getState()

    const isGoldStarEligible = getGoldStarEligibility(state)
    const { suppressDisqualificationWarnings } =
      getCurrentUserSettings(state)
    const { isSolved } = getGameStatus(state)

    // Only valid action once solved is to reset the game, which should always proceed as requested
    if (isSolved) {
      if (actionType === RESET_GAME) {
        dispatch(exitRebus(false))
        dispatch(proceedWithAction(actionType, scope))
      }

      return
    }

    const needToConfirm =
      scope === Scope.Puzzle && actionType !== CHECK

    const needToWarn =
      isDisqualifyingAction(actionType) &&
      isGoldStarEligible &&
      !suppressDisqualificationWarnings

    const callback = () => proceedWithAction(actionType, scope)
    // If there is no need for a warning or confirmation then just proceed as requested
    if (!needToWarn && !needToConfirm) return dispatch(callback())

    return dispatch(
      openModal(CrosswordModal.Confirmation, {
        needToWarn,
        actionType,
        scope,
      }),
    )
  }

export const toggleAutocheck =
  (): CrosswordThunk => (dispatch, getState) => {
    const state = getState()
    const puzzleId = getPuzzleId(state)
    const isMini = getIsMini(state)
    const eventAction = isMini ? 'mini-page' : 'game-page'
    const autocheckEnabled = getAutocheckEnabled(state)

    if (autocheckEnabled) {
      dispatch({ type: DISABLE_AUTOCHECK })
      trackModuleInteraction(
        'gameplay',
        puzzleId,
        eventAction,
        'auto-check',
        'turn-off',
      )
    } else {
      dispatch(execute(ENABLE_AUTOCHECK))
      trackModuleInteraction(
        'gameplay',
        puzzleId,
        eventAction,
        'auto-check',
        'turn-on',
      )
    }
  }
