import {
  isDisqualifyingAction,
  isLocalLatest,
  getPercentileByScope,
  getCellAtThresholdForScope,
  convertRebusToGuess,
  findCellsInScope,
  getDelta,
} from '../helpers'
import {
  TRACK_PROGRESS,
  DISABLE_AUTOCHECK,
  MARK_FILLED,
  UNMARK_FILLED,
} from '../../state/status/statusActions'
import { Scope } from '../../state/cells/cellsTypes'

const cellState = [
  {
    type: 1,
    clues: [1, 3, 4, 5],
    answer: 'B',
    label: 'foo',
    index: 2,
    confirmed: true,
    checked: false,
    penciled: false,
    revealed: false,
    guess: '',
    timestamp: 1632165687297,
    modified: false,
  },
  {
    type: 1,
    clues: [6, 7],
    answer: 'C',
    label: 'bar',
    index: 3,
    confirmed: false,
    checked: false,
    penciled: false,
    revealed: false,
    guess: '',
    timestamp: 1632165687298,
    modified: false,
  },
]

const emptySelection = {
  cell: 0,
  clueCells: [],
  clue: null,
  cellClues: [],
  clueList: null,
  relatedCells: [],
  relatedClues: [],
}

const filledSelection = {
  cell: 1,
  clueCells: [1, 3],
  clue: null,
  cellClues: [],
  clueList: null,
  relatedCells: [],
  relatedClues: [],
}

describe('helper functions', () => {
  it('returns false if action is qualifying', () => {
    expect(isDisqualifyingAction(TRACK_PROGRESS)).toBe(false)
    expect(isDisqualifyingAction(DISABLE_AUTOCHECK)).toBe(false)
    expect(isDisqualifyingAction(MARK_FILLED)).toBe(false)
    expect(isDisqualifyingAction(UNMARK_FILLED)).toBe(false)
  })

  it('returns true if remote commit id equals local status commit id', () => {
    const localArg = {
      status: {
        lastCommitID: '12345',
      },
    }

    const remote = {
      lastCommitID: '12345',
    }
    // @ts-ignore
    expect(isLocalLatest(localArg, remote)).toBe(true)
  })

  it('returns true if no remote game present', () => {
    const localArg = {
      status: {
        lastCommitID: '12345',
      },
    }
    // @ts-ignore
    expect(isLocalLatest(localArg, null)).toBe(true)
  })

  it('returns true if remote game does not have a last commit id', () => {
    const localArg = {
      status: {
        lastCommitID: '12345',
      },
    }
    // @ts-ignore
    expect(isLocalLatest(localArg, { lastCommitID: '' })).toBe(true)
  })

  it('returns the appropriate integer based on the scope string', () => {
    expect(getPercentileByScope(Scope.Fifty)).toEqual(0.5)
    expect(getPercentileByScope(Scope.SeventyFive)).toEqual(0.75)
  })

  it('defaults to one if no scope is provided', () => {
    expect(getPercentileByScope(null)).toEqual(1)
  })

  it('returns the index of the highest cell at the threshold scope', () => {
    const scopeSeventyFive = getCellAtThresholdForScope(
      cellState,
      Scope.SeventyFive,
    )

    const scopeFifty = getCellAtThresholdForScope(
      cellState,
      Scope.Fifty,
    )
    expect(scopeSeventyFive).toEqual(3)
    expect(scopeFifty).toEqual(2)
  })

  it('to find cells in scope, based on cells and selection', () => {
    const crosswordState = {
      cells: cellState,
      selection: emptySelection,
    }

    const cellScopeResult = findCellsInScope(
      // @ts-ignore
      crosswordState,
      Scope.Letter,
    )
    expect(cellScopeResult).toHaveLength(1)
    expect(cellScopeResult[0].index).toEqual(2)
    expect(cellScopeResult[0].label).toEqual('foo')
  })

  it('does not find any cells if selection clues dont include cell index', () => {
    const crosswordState = {
      cells: cellState,
      selection: emptySelection,
    }

    const cellScopeResult = findCellsInScope(
      // @ts-ignore
      crosswordState,
      Scope.Word,
    )
    expect(cellScopeResult).toHaveLength(0)
  })

  it('does find cells if selection clues includes cell index', () => {
    const crosswordState = {
      cells: cellState,
      selection: filledSelection,
    }

    const cellScopeResult = findCellsInScope(
      // @ts-ignore
      crosswordState,
      Scope.Word,
    )
    expect(cellScopeResult).toHaveLength(1)
  })

  it('formats and converts rebus strings to guess strings', () => {
    expect(convertRebusToGuess(' aB ')).toEqual('AB')
    expect(convertRebusToGuess(' AA  ')).toEqual('AA')
    expect(convertRebusToGuess(' ')).toEqual('')
  })

  it('will return 1 if cell is different and willBeAfter is true', () => {
    const res = getDelta(cellState[0], true, false, true)
    expect(res).toEqual(1)
  })

  it('will return 0 if cell is locked and debug mode is off', () => {
    const res = getDelta(cellState[0], false, false, false)
    expect(res).toEqual(0)
  })

  it('will return -1 if cell is different and willBeAfter is false', () => {
    const res = getDelta(cellState[0], false, true, true)
    expect(res).toEqual(-1)
  })
})
