import {
  guess,
  toggleAutocheck,
  proceedWithAction,
  execute,
  checkPuzzleProgress,
} from '../statusThunks'
import {
  TRACK_PROGRESS,
  MARK_FILLED,
} from '../../state/status/statusActions'
import { Scope } from '../../state/cells/cellsTypes'
import { openModal } from '../../state/modal/modalActions'
import { CrosswordModal } from '../../state/modal/modalTypes'
import { CLEAR } from '../../state/cells/cellsActions'

import { data } from './data'

jest.mock('../../utils/tracker', () => ({
  tracker: jest.fn(),
  trackModuleInteraction: jest.fn(),
}))

const dispatch = jest.fn()

describe('status thunk', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  it('checks puzzle progress if track progress action is dispatched if new progress is more than current game progress', () => {
    const expectedActions = {
      payload: { currentProgress: 3 },
      type: 'crossword/status/TRACK_PROGRESS',
    }

    const getState = () => {
      return {
        status: { currentProgress: 1, blankCells: 7 },
        cells: data.cells,
        timer: data.timer,
        puzzle: data.puzzle,
      }
    }
    // @ts-ignore // suppress extra argument warning
    checkPuzzleProgress()(dispatch, getState)
    expect(dispatch).toHaveBeenCalledWith(expectedActions)
  })

  it('does not dispatch track progress action if new progress is less than current progress', () => {
    const getState = () => {
      return {
        status: { currentProgress: 4, blankCells: 7 },
        cells: data.cells,
        timer: data.timer,
        puzzle: data.puzzle,
      }
    }

    // @ts-ignore // suppress extra argument warning
    checkPuzzleProgress()(dispatch, getState)
    expect(dispatch).toHaveBeenCalledTimes(0)
  })

  it('dispatches guess action with correct payload and checked solved state', () => {
    const getState = () => {
      return {
        status: { currentProgress: 4, blankCells: 7 },
        cells: data.cells,
        timer: data.timer,
        puzzle: data.puzzle,
        isSolved: false,
        autocheckEnabled: false,
        toolbar: { inPencilMode: false },
        selection: data.selection,
      }
    }

    // @ts-ignore // suppress extra argument warning
    guess('B')(dispatch, getState)
    expect(dispatch).toHaveBeenCalledTimes(2)
  })

  it('dispatches once when autocheck is already enabled', () => {
    const getState = () => {
      return {
        status: {
          currentProgress: 4,
          blankCells: 7,
          autocheckEnabled: true,
        },
        cells: data.cells,
        timer: data.timer,
        puzzle: data.puzzle,
      }
    }

    // @ts-ignore // suppress extra argument warning
    toggleAutocheck()(dispatch, getState)
    expect(dispatch).toHaveBeenCalledTimes(1)
  })

  it('proceedWithAction dispatches twice', () => {
    const getState = () => {
      return {
        status: {
          currentProgress: 4,
          blankCells: 7,
          autocheckEnabled: true,
        },
        cells: data.cells,
        timer: data.timer,
        puzzle: data.puzzle,
      }
    }

    // @ts-ignore // suppress extra argument warning
    proceedWithAction(TRACK_PROGRESS, Scope.Fifty)(dispatch, getState)
    expect(dispatch).toHaveBeenCalledTimes(2)
    expect(dispatch).toEqual(expect.any(Function))
  })

  it('executes dispatch if user does not need to be warned or perform a confirmation', () => {
    const getState = () => {
      return {
        status: {
          currentProgress: 4,
          blankCells: 7,
          autocheckEnabled: true,
        },
        cells: data.cells,
        timer: data.timer,
        puzzle: data.puzzle,
        user: {
          settings: {
            forceGoldStarEligibility: true,
            suppressDisqualificationWarnings: true,
          },
        },
      }
    }

    // @ts-ignore // suppress extra argument warning
    execute(MARK_FILLED, Scope.Word)(dispatch, getState)
    expect(dispatch).toHaveBeenCalledTimes(1)
    expect(dispatch).toEqual(expect.any(Function))
  })

  it('execute opens confirmation modal if user needs to be warned or confirm modal', () => {
    const getState = () => {
      return {
        status: {
          currentProgress: 4,
          blankCells: 7,
          autocheckEnabled: true,
        },
        cells: data.cells,
        timer: data.timer,
        puzzle: data.puzzle,
        user: {
          settings: {
            forceGoldStarEligibility: false,
            suppressDisqualificationWarnings: false,
          },
        },
      }
    }
    // @ts-ignore // suppress extra argument warning
    execute(CLEAR, Scope.Puzzle)(dispatch, getState)
    expect(dispatch).toHaveBeenCalledTimes(1)
    expect(dispatch).toHaveBeenCalledWith(
      openModal(CrosswordModal.Confirmation, {
        needToWarn: false,
        actionType: CLEAR,
        scope: Scope.Puzzle,
      }),
    )
  })
})
