import adaptPuzzle, {
  getAriaLabel,
  performImgLogic,
} from './adaptPuzzle'

describe('normalize puzzle data', () => {
  it('should spread unknown puzzle attributes', () => {
    const puzzle = adaptPuzzle({
      cells: [],
      clues: [],
      clueLists: [],
      a: true,
      b: false,
    })
    expect(puzzle).toMatchObject({ a: true, b: false })
  })

  it('should link clues in a loop and add indexes', () => {
    const puzzle = adaptPuzzle({
      cells: [],
      clues: [
        { a: 1, text: [{ plain: 'plain txt' }] },
        { a: 2, text: [{ formatted: 'formatted txt' }] },
        { a: 3, text: [{ formatted: 'formatted txt' }] },
      ],
      clueLists: [],
    })

    expect(puzzle.clues).toMatchObject([
      { a: 1, index: 0, next: 1, prev: 2 },
      { a: 2, index: 1, next: 2, prev: 0 },
      { a: 3, index: 2, next: 0, prev: 1 },
    ])
  })

  it('should link clueLists in a loop and add indexes', () => {
    const puzzle = adaptPuzzle({
      cells: [{}, {}, {}],
      clues: [
        { text: [{ formatted: 'txt' }], cells: [1] },
        { text: [{ formatted: 'txt' }], cells: [0, 2] },
      ],
      clueLists: [
        { a: 1, clues: [0] },
        { a: 2, clues: [1] },
      ],
    })

    expect(puzzle.clueLists).toMatchObject([
      { a: 1, index: 0, next: 1, prev: 1, cells: [1], clues: [0] },
      { a: 2, index: 1, next: 0, prev: 0, cells: [0, 2], clues: [1] },
    ])
  })

  it('should add index to cell data', () => {
    const puzzle = adaptPuzzle({
      cells: [{ a: 1 }, { a: 2 }, { a: 3 }],
      clues: [],
      clueLists: [],
    })

    expect(puzzle.cells).toMatchObject([
      { index: 0, a: 1 },
      { index: 1, a: 2 },
      { index: 2, a: 3 },
    ])
  })

  it('should normalize cells', () => {
    const puzzle = adaptPuzzle({
      cells: [{}, {}, {}],
      clues: [],
      clueLists: [],
    })

    expect(puzzle.cells).toMatchObject([
      { index: 0, clues: [], type: 0 },
      { index: 1, clues: [], type: 0 },
      { index: 2, clues: [], type: 0 },
    ])
  })

  it('should normalize clues', () => {
    const puzzle = adaptPuzzle({
      cells: [],
      clues: [
        { text: [{ plain: 'plain txt' }], cells: [0, 1, 2] },
        {
          text: [{ formatted: 'formatted txt', plain: 'plain txt' }],
        },
        {
          text: [
            { formatted: "prefix<img src='[IMAGE_CLUE_URL]' />" },
          ],
          list: 2,
        },
      ],
      clueLists: [],
    })

    expect(puzzle.clues).toMatchObject([
      {
        index: 0,
        text: 'plain txt',
        list: 0,
        unfilledCount: 3,
        cells: [0, 1, 2],
        isImageClue: false,
      },
      {
        index: 1,
        text: 'formatted txt',
        list: 0,
        unfilledCount: Number.POSITIVE_INFINITY,
        isImageClue: false,
      },
      {
        index: 2,
        text: "<img src='[IMAGE_CLUE_URL]' />",
        list: 2,
        unfilledCount: Number.POSITIVE_INFINITY,
        isImageClue: true,
      },
    ])
  })
})

describe('performImgLogic()', () => {
  const baseClue = {
    formatted: 'foobar<img src="misc.svg" />',
    plain: 'hi hello',
  }

  describe('when passed image', () => {
    it('returns true and removes prefix on image text when present and image tag does not close (still valid)', () => {
      const clue = {
        ...baseClue,
        formatted: 'foobar<img src="misc.svg" >',
      }
      expect(performImgLogic(clue)).toEqual(true)
      expect(clue).toEqual({
        ...clue,
        formatted: '<img src="misc.svg" >',
      })
    })

    it('returns true and removes prefix on image text when present and image tag is self closing', () => {
      const clue = {
        ...baseClue,
        formatted: 'foobar<img src="misc.svg" />',
      }
      expect(performImgLogic(clue)).toEqual(true)
      expect(clue).toEqual({
        ...clue,
        formatted: '<img src="misc.svg" />',
      })
    })

    it('returns true and removes prefix on image text when present and image tag has matching close tag', () => {
      const clue = {
        ...baseClue,
        formatted: 'foobar<img src="misc.svg" ></img>',
      }
      expect(performImgLogic(clue)).toEqual(true)
      expect(clue).toEqual({
        ...clue,
        formatted: '<img src="misc.svg" ></img>',
      })
    })

    it('returns true and keeps image as is when prefix text not present', () => {
      const clue = {
        ...baseClue,
        formatted: '<img src="misc.svg" />',
      }
      expect(performImgLogic(clue)).toEqual(true)
      expect(clue).toEqual(clue)
    })
  })

  describe('when not passed image', () => {
    it('returns false and keeps image as is when prefix text not present', () => {
      const clue = {
        ...baseClue,
        formatted: 'hello hello there not a real img hello',
      }
      expect(performImgLogic(clue)).toEqual(false)
      expect(clue).toEqual(clue)
    })
  })
})

describe('getAriaLabel()', () => {
  describe('when passed clue with formatted text and a specified aria label in the plain text', () => {
    it('with whitespace, returns text after the aria label', () => {
      const clue = {
        formatted: 'foobar',
        plain: '[aria-label] hi hello',
      }
      expect(getAriaLabel(clue)).toEqual('hi hello')
    })

    it('with no whitespace, returns text after the aria label', () => {
      const clue = {
        formatted: 'foobar',
        plain: '[aria-label]hi hello',
      }
      expect(getAriaLabel(clue)).toEqual('hi hello')
    })
  })

  describe('when not passed formatted text, returns plain text', () => {
    it('returns unaltered plain text', () => {
      const clue = {
        plain: '[aria-label] hi hello',
      }
      expect(getAriaLabel(clue)).toEqual('[aria-label] hi hello')
    })
  })

  describe('when passed formatted text and no aria label specified, returns formatted text', () => {
    it('returns unaltered plain text', () => {
      const clue = {
        plain: 'foobar',
        formatted: 'lorem ipsum',
      }
      expect(getAriaLabel(clue)).toEqual('lorem ipsum')
    })
  })
})
