import { RawCell, RawClue, RawClueList, RawPuzzleBody } from './types'

const IMAGE_PREFIX_REGEX = new RegExp(/^.+?(?=<img)/)
const IMAGE_REGEX = new RegExp(/<img/)
const ARIA_LABEL_REGEX = new RegExp(/^\[aria-label\]/)

const addIndex = <T extends object>(item: T, index: number) => ({
  ...item,
  index,
})

const addPrevAndNext = <T extends object>(
  element: T,
  index: number,
  collection: T[],
) => ({
  ...element,
  prev: index === 0 ? collection.length - 1 : index - 1,
  next: index === collection.length - 1 ? 0 : index + 1,
})

const normalizeCell = (c: RawCell) => ({
  type: 0,
  clues: [],
  confirmed: false,
  checked: false,
  penciled: false,
  revealed: false,
  modified: false,
  guess: '',
  ...c,
})

export const getAriaLabel = (text: {
  formatted?: string
  plain: string
}): string => {
  if (!text.formatted) return text.plain

  if (ARIA_LABEL_REGEX.test(text.plain)) {
    text.plain = text.plain.replace(ARIA_LABEL_REGEX, '').trimStart()
    return text.plain
  }

  return text.formatted
}

export const performImgLogic = (text: {
  formatted?: string
  plain: string
}): boolean => {
  if (text.formatted && IMAGE_REGEX.test(text.formatted)) {
    text.formatted = text.formatted.replace(IMAGE_PREFIX_REGEX, '')
    return true
  }

  return false
}

const normalizeClue = (clue: RawClue) => {
  const {
    text: [text],
  } = clue

  const isImageClue = performImgLogic(text)
  const alternativeAriaLabelText = getAriaLabel(text)

  return {
    list: 0, // add default; protobuf omits 0s
    cells: [],
    ...clue,
    text: text.formatted || text.plain,
    unfilledCount: clue.cells ? clue.cells.length : Infinity,
    isImageClue,
    alternativeAriaLabelText,
  }
}

const normalizeClueList = (list: RawClueList, clues: RawClue[]) => {
  const listClues = list.clues.map((c) => clues[c])
  return {
    ...list,
    cells: listClues.reduce<number[]>(
      (listCells, listClue) => listCells.concat(listClue.cells || []),
      [],
    ),
  }
}

const adaptPuzzle = (puzzle: Omit<RawPuzzleBody, 'SVG'>) => {
  const normalCells = puzzle.cells.map(normalizeCell).map(addIndex)

  const normalClues = puzzle.clues
    .map(normalizeClue)
    .map(addPrevAndNext)
    .map(addIndex)

  const normalClueLists = puzzle.clueLists
    .map((list) => normalizeClueList(list, puzzle.clues))
    .map(addPrevAndNext)
    .map(addIndex)

  return {
    ...puzzle,
    clues: normalClues,
    cells: normalCells,
    clueLists: normalClueLists,
  }
}

export default adaptPuzzle
