import type { CellState } from '../state/cells/cellsTypes'

const isFilled = (cell: CellState) => !!cell.guess

export const isNavigable = (cell: CellState) =>
  ![0, 4].includes(cell.type)

export const isBlank = (cell: CellState) =>
  isNavigable(cell) && !isFilled(cell)

export const isLocked = (cell: CellState | null) =>
  cell && cell.confirmed

const isUnlocked = (cell: CellState) => !isLocked(cell)

export const acceptsGuess = (cell: CellState, guess: string) => {
  const { answer, moreAnswers } = cell
  // If there's only one right answer, it must match the guess (or both answer and guess must be falsy)
  if (!moreAnswers) return answer ? answer === guess : !guess
  const { valid = [] } = moreAnswers
  return valid.concat(answer).includes(guess)
}

export const isCorrect = (cell: CellState) =>
  acceptsGuess(cell, cell.guess || '')

export const isIncorrect = (cell: CellState) =>
  isNavigable(cell) && !isCorrect(cell)

export const isUnlockedAndNavigable = (cell: CellState) =>
  isUnlocked(cell) && isNavigable(cell)

const isUncheckedOrModified = (cell: CellState) =>
  !cell.checked || cell.modified

export const hasModifiableContent = (cell: CellState) =>
  isFilled(cell) && isUnlocked(cell)

export const willBeAffectedByChecking = (cell: CellState) =>
  hasModifiableContent(cell) && isUncheckedOrModified(cell)

const isCheckedIncorrectAndUnmodified = (cell: CellState) =>
  cell.checked && !cell.modified && isUnlocked(cell)

export const setLandableConditions =
  (settings: { skipFilled: boolean; skipPenciled: boolean }) =>
  (cell: CellState) => {
    if (isCheckedIncorrectAndUnmodified(cell)) return true

    if (settings.skipFilled) {
      if (isBlank(cell)) return true

      if (!settings.skipPenciled) return cell.penciled

      return false
    }

    return true
  }
