import lf from 'localforage'

import { UserType } from 'foundation'

import { LocalProgress } from '../state/progressActions'
import { UserState } from '../state/user/userTypes'

import { newID } from './misc'
import type { Commit } from './syncGameState'

/**
 * iOS 14.5 and up have issues with indexedDB eventually crashing on open after navigating away from the page.
 * All known workarounds as of 09/2021 do not appear to be working. We will force local storage for these devices to resolve.
 */
if (/iPad|iPhone|Safari/.test(navigator.userAgent)) {
  lf.config({
    driver: lf.LOCALSTORAGE,
  })
}

const ANONYMOUS_ID = 'anon'

const gameKey = (userID: string | null, puzzleID: number) =>
  `${userID || ANONYMOUS_ID}@${puzzleID}`

export const getLastPlayedTime = (key: string) =>
  lf.getItem<number | null>(`${key}#lastPlayed`)

export const deleteStoredPuzzleData = (key: string) => {
  lf.removeItem(`${key}#lastPlayed`)
  lf.removeItem(key)
}

/**
 * Called by storeGame. Sets a timestamp on a property gameKey#lastPlayed
 */
export const addRecentPuzzle = (
  key: string,
  timestamp: number = new Date().getTime(),
) => lf.setItem(`${key}#lastPlayed`, timestamp)

/**
 * Check the timestamp on the #lastPlayed stoage entry and delete if it's
 * older than a week. Delete if there's no timestamp, too.
 */
export const checkIfStoredPuzzleIsStale = (key: string) => {
  const lastWeek = new Date().getTime() - 604800000

  getLastPlayedTime(key).then((timestamp: number | null) => {
    if (timestamp && timestamp <= lastWeek) {
      deleteStoredPuzzleData(key)
    } else if (timestamp === null) {
      deleteStoredPuzzleData(key)
    }
  })
}

/**
 * Scans the browser storage used by localforage for crossword puzzle data
 * If the locally stored data is that is older than one week it will be deleted
 */
export const manageXWDBrowserStorage = () => {
  /**
   * IndexedDB entries have keys userid@puzzleid
   * LocalStorage entries have keys localforage/userid@puzzleid
   */
  const keyRegex = /(localforage\/)?([0-9]|anon)+@[0-9]+$/
  lf.keys().then((keys) => {
    keys.forEach((key) => {
      if (key.match(keyRegex)) {
        checkIfStoredPuzzleIsStale(key)
      }
    })
  })
}

export const fetchLocalProgress = (
  userInfo: UserType,
  puzzleID: number,
) => {
  return lf.getItem<LocalProgress>(gameKey(userInfo.regiId, puzzleID))
}

const commitsKey = (userID: string | null) =>
  `${userID}#pendingCommits`

export const fetchPendingCommits = (userID: string | null) =>
  lf.getItem(commitsKey(userID)).then((c) => c || undefined)

export const clearPendingCommits = (userID: string | null) => {
  lf.removeItem(commitsKey(userID)).catch((error: any) => {
    console.error('error clearing pending commits', error)
  })
}

const updatePendingCommits = (
  userID: string | null,
  commitID: string,
  newCommit?: Commit,
) => {
  fetchPendingCommits(userID).then((commits: any = {}) => {
    if (newCommit) commits[commitID] = newCommit
    else delete commits[commitID]

    if (Object.keys(commits).length) {
      lf.setItem(commitsKey(userID), commits).catch((error: any) => {
        console.error('error setting commit key', error)
      })
    } else {
      clearPendingCommits(userID) // Delete object once final commit is synced
    }
  })
}

export const addPendingCommit = (
  userID: string | null,
  commitID: string,
  newCommit: Commit,
) => updatePendingCommits(userID, commitID, newCommit)

export const removePendingCommit = (
  userID: string,
  commitID: string,
) => updatePendingCommits(userID, commitID)

export const storeGame = (
  userID: string | null,
  puzzleID: number,
  localGame: LocalProgress,
) => {
  const key = gameKey(userID, puzzleID)
  lf.setItem<LocalProgress>(key, localGame).catch((error: any) => {
    console.error('error storing game', error)
  })
  addRecentPuzzle(key)
}

let deviceID: string
export const getOrGenerateDeviceID = (): Promise<string> => {
  if (deviceID) return Promise.resolve(deviceID)

  return lf.getItem<string>('deviceID').then((storedID) => {
    deviceID = storedID || `${newID()}-web`
    if (!storedID) {
      lf.setItem('deviceID', deviceID).catch((error: any) => {
        console.error('error setting device id', error)
      })
    }

    return deviceID
  })
}

export const storeUserPrefs = (
  user: UserState,
  userType: UserType,
) => {
  const { printPrefs, settings } = user
  lf.setItem(`${userType.regiId || ANONYMOUS_ID}#preferences`, {
    printPrefs,
    settings,
  }).catch((error: any) => {
    console.error('error storing user prefs', error)
  })
}

export const getLocalUserPreferences = (userId = ANONYMOUS_ID) =>
  lf.getItem<UserState>(`${userId}#preferences`)

export const storeShortzToggleState = (isActive: boolean) =>
  lf.setItem('shortz-toggle', isActive).catch((error: any) => {
    console.error('error setting shortz toggle state', error)
  })

export const getShortzToggleState = () =>
  lf.getItem<boolean>('shortz-toggle')
