import { env, UserType, xhr } from 'foundation'

import logger from '../../../shared/utils/clientLogger'
import { RemoteProgress } from '../state/progressActions'

import { camelizeKeysDeep } from './misc'

export const edgeHost = `${env.edge}/svc/crosswords`

type Config = {
  method: string
  url: string
  data?: Record<any, any>
  isSync?: boolean
}

// On the server, we can set the cookie, but on the client, we use a custom header
export const apiCallWithUser = <R = any>(
  config: Config,
  user: UserType,
) => {
  const { method, url, data = null, isSync } = config
  // log all calls if you are on the team
  if (user.inShortzMode) {
    logger.info(`info: API CALL ${JSON.stringify(config, null, 2)}`)
  }

  return xhr.request<R>(method, url, { data, isSync })
}

export const getV6Game = (user: UserType, puzzleID: number) => {
  const path = `${edgeHost}/v6/game/${puzzleID}`
  const url = `${path}.json`
  const config = { method: 'GET', url }
  return apiCallWithUser(config, user)
}

export const putV6Game = (
  user: UserType,
  data: any,
  isSync = false,
) => {
  const path = `${edgeHost}/v6/game`
  const url = `${path}.json`
  const config = { method: 'PUT', url, data, isSync }
  return apiCallWithUser(config, user).catch((error) => {
    logger.error('Failed to fetch puzzle progress: ', error)
  })
}

export const fetchRemoteProgress = (
  user: UserType,
  puzzleID: number,
  parser: (raw: any) => Promise<RemoteProgress>,
): Promise<RemoteProgress | null> => {
  const remotePromise = user.isLoggedIn
    ? getV6Game(user, puzzleID).then((progress) => parser(progress))
    : Promise.resolve(null)

  return remotePromise as Promise<RemoteProgress | null>
}

const fetchStatsAndStreaks = (user: UserType) => {
  const url = `${edgeHost}/v3/${user.regiId}/stats-and-streaks.json?date_start=2014-01-01&start_on_monday=true`
  const config = { method: 'GET', url }

  return apiCallWithUser(config, user)
    .then((response) =>
      camelizeKeysDeep({
        dailyStats: response.results.stats,
        streaks: response.results.streaks,
      }),
    )
    .catch((error) => {
      logger.error('failed to fetch stats and streaks', error)
    })
}

const fetchMiniStats = (user: UserType) => {
  const url = `${edgeHost}/v3/${user.regiId}/mini-stats.json?date_start=2014-01-01&start_on_monday=true`
  const config = { method: 'GET', url }

  return apiCallWithUser(config, user)
    .then((response) =>
      camelizeKeysDeep({
        miniStats: response.results,
      }),
    )
    .catch((error) => {
      logger.error('failed to fetch mini stats', error)
    })
}

export const fetchAllStats = (user: UserType) =>
  Promise.all([
    fetchMiniStats(user),
    fetchStatsAndStreaks(user),
  ]).then((results) =>
    results.reduce((final, v) => Object.assign(final, v), {}),
  )
