import moment from 'moment'

import { UserType } from '../../../../shared/foundation'
import { monitoredProps } from '../state/cells/cellsReducer'
import type { CellState } from '../state/cells/cellsTypes'
import type { CrosswordStoreState } from '../state/crosswordReducer'
import { RemoteProgress } from '../state/progressActions'
import type { FirstTimestamps } from '../state/status/statusTypes'

import { isNavigable } from './cellUtils'
import { addPendingCommit, removePendingCommit } from './forageUtils'
import { newID, pickShallow } from './misc'
import { putV6Game } from './remoteUtils'

type RawTimestamps = {
  opened?: number
  checked?: number
  revealed?: number
  cleared?: number
  timerReset?: number
  solved?: number
}

type Calcs = {
  eligible: boolean
  secondsSpentSolving: number
  solved: boolean
}

type RawData = {
  autocheckEnabled: boolean
  board: any
  calcs: Calcs
  firsts: RawTimestamps
  lastCommitID: string
  minGuessTime: number
  userID: string
}

interface RawCell extends CellState {
  blank: boolean
}

const syncableProps: Array<keyof CellState> = [
  ...monitoredProps,
  'timestamp',
]

const serializeCells = (cells: CellState[], minTS: number) =>
  cells.map((cell) => {
    if (!isNavigable(cell)) return { blank: true }

    return {
      ...pickShallow(cell, syncableProps),
      timestamp: cell.timestamp ? cell.timestamp - minTS : undefined,
    }
  })

export type Commit = {
  deviceID: string
  puzzleID: number
  userID: number | null // API expecting a number for userID
  commitID: string
  timestamp: number
  minGuessTime: number
  timerDiff: number
  reset: boolean
  solved: boolean
  board: { cells: ReturnType<typeof serializeCells> }
  firsts: FirstTimestamps
  autocheckEnabled: boolean
}

const buildCommit = (
  ids: { deviceID: string; puzzleID: number; userID: string | null },
  {
    cells,
    status,
    timer,
  }: Pick<CrosswordStoreState, 'cells' | 'status' | 'timer'>,
) => {
  const { firsts, isSolved, autocheckEnabled } = status
  const { userID, deviceID, puzzleID } = ids
  const numberUserID = userID ? parseInt(userID) : null
  const newIds = {
    deviceID,
    puzzleID,
    userID: numberUserID,
  }
  const { sessionElapsedTime, resetSinceLastCommit } = timer
  const minGuessTime =
    cells.map((c) => c.timestamp || Infinity).sort()[0] - 1 || 0
  const serializedCells = serializeCells(cells, minGuessTime)
  const commitID = newID()
  const newCommit: Commit = {
    ...newIds, // device/puzzle/user
    commitID,
    timestamp: moment().unix(),
    minGuessTime,
    timerDiff: sessionElapsedTime,
    reset: resetSinceLastCommit,
    solved: isSolved,
    board: { cells: serializedCells },
    firsts,
    autocheckEnabled,
  }

  addPendingCommit(ids.userID, commitID, newCommit)
  return newCommit
}

const deserializeCells = (
  cells: RawCell[],
  baseTS: number,
): CellState[] => {
  const defaultCell = {
    guess: '',
    label: '',
    penciled: false,
    checked: false,
    confirmed: false,
    revealed: false,
    modified: false,
  }

  return cells.map((cell) => {
    if (cell.blank) {
      return {
        ...defaultCell,
        type: 0,
        clues: [],
        index: cell.index,
        answer: cell.answer,
      }
    }

    if (cell.timestamp) {
      cell.timestamp += baseTS
    }

    return {
      ...defaultCell,
      ...cell,
    }
  })
}

export const parseApiProgress = (
  raw: RawData,
): Promise<RemoteProgress> => {
  if (!raw || !raw.calcs) return Promise.reject()

  const {
    board,
    minGuessTime,
    calcs,
    firsts,
    lastCommitID,
    userID,
    autocheckEnabled = false,
  } = raw

  const {
    secondsSpentSolving = 0,
    solved = false,
    eligible = false,
  } = calcs

  removePendingCommit(userID, lastCommitID)

  return Promise.resolve({
    lastCommitID,
    cells: board ? deserializeCells(board.cells, minGuessTime) : [],
    status: {
      firsts,
      isSolved: solved,
      isGoldStarEligible: eligible,
      autocheckEnabled,
    },
    timer: {
      totalElapsedTime: secondsSpentSolving,
    },
  })
}

export const syncGameState = (
  ids: { deviceID: string; puzzleID: number; userID: string | null },
  user: UserType,
  localState: Pick<CrosswordStoreState, 'cells' | 'status' | 'timer'>,
  isLeavingGame: boolean = false,
) => {
  const data = {
    now: moment().unix(),
    commits: [buildCommit(ids, localState)],
  }
  const isSyncXhr = isLeavingGame
  return putV6Game(user, data, isSyncXhr)
    .then(parseApiProgress)
    .catch(() => {
      console.error('Failed to sync game state')
    })
}
