import { getSubscribeLink, handleSubscribe } from '../subscribeLinks'

describe('HELPERS: SUBSCRIBE LINKS', () => {
  describe('getSubscribeLink()', () => {
    const mockWindow = {
      navigationLinks: {
        subscribe: {
          default: 'DEFAULT',
          vertexTodaysPuzzle:
            'https://www.nytimes.com/subscription/games?campaignId=9KK7K',
          tilesWelcomeMobile:
            'https://www.nytimes.com/subscription/games?campaignId=7Y97X',
        },
      },
    }

    it('should return a default URL if provided nothing', () => {
      expect(getSubscribeLink()).toBe(
        'https://www.nytimes.com/subscription/games',
      )
    })

    it('should return a default URL if provided a bogus window', () => {
      const brokenWindow = {}
      expect(
        getSubscribeLink('vertexTodaysPuzzle', brokenWindow),
      ).toBe('https://www.nytimes.com/subscription/games')
    })

    it('should return a default URL if provided a bogus campaign', () => {
      expect(getSubscribeLink('goober', mockWindow)).toBe('DEFAULT')
    })

    it('should return the correct URL if provided a valid campaign', () => {
      expect(getSubscribeLink('vertexTodaysPuzzle', mockWindow)).toBe(
        'https://www.nytimes.com/subscription/games?campaignId=9KK7K',
      )
    })

    it('should return another correct URL if provided a valid campaign', () => {
      expect(getSubscribeLink('tilesWelcomeMobile', mockWindow)).toBe(
        'https://www.nytimes.com/subscription/games?campaignId=7Y97X',
      )
    })

    it('should return the default link off the window object with no campaign', () => {
      expect(getSubscribeLink(undefined, mockWindow)).toBe('DEFAULT')
    })
  })

  describe('handleSubscribe', () => {
    const { location } = window

    it('should call gamesAuthenticateUser when in hybrid webview and native bridge exists', () => {
      window.isHybridWebView = true
      const data = {
        success: true,
        values: {
          gamesAuthenticateUser: {},
        },
      }

      const gamesAuthenticateUser = jest
        .fn()
        .mockImplementation(() => Promise.resolve(data))
      window.NativeBridge = { gamesAuthenticateUser }
      const tag = 'tag'
      handleSubscribe(tag)
      expect(gamesAuthenticateUser).toHaveBeenCalled()
    })

    it('should add an anchor element to the document if deep link', () => {
      window.isHybridWebView = false
      window.navigationLinks = {
        subscribe: { tag: 'nytimes://hello' },
      }
      const tag = 'tag'
      const clickMock = jest.fn()
      const createElementMock = jest
        .fn()
        .mockImplementation((href) => ({
          href,
          click: clickMock,
        }))
      document.createElement = createElementMock
      handleSubscribe(tag)
      expect(createElementMock).toHaveBeenCalled()
      expect(clickMock).toHaveBeenCalled()
    })

    it('should change the window location if not hybrid or deep link', () => {
      delete window.location

      const assignMock = jest.fn()
      window.location = {
        assign: assignMock,
      }

      const url = 'http://hello'
      window.isHybridWebView = false
      window.navigationLinks = {
        subscribe: { tag: url },
      }
      const tag = 'tag'
      handleSubscribe(tag)
      expect(assignMock).toHaveBeenCalledWith(url)

      window.location = location
    })
  })
})
