import { track } from 'foundation'

export type TrackingEvent =
  | 'moduleInteraction'
  | 'impression'
  | 'expose'

const INTERACTION_CATEGORIES = [
  'modal',
  'gameplay',
  'settings',
  'general',
] as const

type InteractionCategory = typeof INTERACTION_CATEGORIES[number]

const IMPRESSION_CATEGORIES = [
  'tech',
  'gameplay',
  'settings',
] as const

type ImpressionCategory = typeof IMPRESSION_CATEGORIES[number]

type TrackingFunction = (
  event: TrackingEvent,
  eventCategory: string,
  elementNameOrModuleRegion: string,
  elementLabelOrModuleLabel?: string | object,
  eventLabel?: string | null,
  useBeacon?: boolean,
) => void

type ImpressionTrackingFunction = (
  moduleRegion: string,
  moduleLabel: string | object,
  eventLabel: string | null,
) => void

type InteractionTrackingFunction = (
  elementName: string,
  elementLabel: string | object,
  eventLabel: string | null,
) => void

type CustomTrackingObject = {
  module: {
    name?: string
    element: {
      name: string
      label: string
    }
    region?: string
    label?: string
  }
}

type CustomTrackingFunction = (
  arg0: CustomTrackingObject,
  useBeacon?: boolean,
) => void

type StandardTracker = {
  interaction: {
    [key in InteractionCategory]: InteractionTrackingFunction
  }
  impression: {
    [key in ImpressionCategory]: ImpressionTrackingFunction
  }
}

type CustomTracker = {
  interaction: {
    custom: CustomTrackingFunction
  }
  impression: {
    custom: CustomTrackingFunction
  }
}

type GameTracker = StandardTracker & CustomTracker

const createGameTracker = (
  gameAbbreviation: string,
  gameId: string | number,
) => {
  const context = `${gameAbbreviation}-${gameId}`

  const trackAction: TrackingFunction = (
    event,
    eventCategory,
    elementNameOrModuleRegion,
    elementLabelOrModuleLabel,
    eventLabel,
    useBeacon,
  ) => {
    track(
      event,
      event === 'moduleInteraction'
        ? {
            // moduleInteraction schema
            module: {
              name: eventCategory,
              context,
              label: eventLabel,
              element: {
                name: elementNameOrModuleRegion || '',
                label:
                  typeof elementLabelOrModuleLabel === 'string'
                    ? elementLabelOrModuleLabel
                    : JSON.stringify(elementLabelOrModuleLabel),
              },
            },
            eventData: {
              pageType: 'game',
              type: useBeacon ? 'ob_click' : 'click',
              trigger: 'module',
              // important to leave an empty string for `status`
              // for the event to pass correctly to google analytics
              // consult with someone from tracking team before removing
              // and confirm that the events are still available in big query
              status: '',
            },
          }
        : {
            // impression schema
            module: {
              name: eventCategory,
              context,
              region: elementNameOrModuleRegion || '',
              label:
                typeof elementLabelOrModuleLabel === 'string'
                  ? elementLabelOrModuleLabel
                  : JSON.stringify(elementLabelOrModuleLabel),
            },
          },
    )
  }

  const api: GameTracker = {
    interaction: {
      /**
       * Fires a moduleInteraction event with a customized argument structure
       *
       * @param {Object} module
       * @param {String} module.name
       * @param {Object} module.element
       * @param {String} module.element.name
       * @param {String} module.element.label
       * @param {boolean} useBeacon If true, fires the event immediately
       */
      custom: (
        {
          module: {
            name: eventCategory = 'gameplay',
            element: { name: elementName, label: elementLabel = '' },
          },
        }: CustomTrackingObject,
        useBeacon: boolean = false,
      ) => {
        trackAction(
          'moduleInteraction',
          eventCategory,
          elementName,
          elementLabel,
          '',
          useBeacon,
        )
      },
      // Stub for TypeScript
      modal: () => {},
      gameplay: () => {},
      settings: () => {},
      general: () => {},
    },
    impression: {
      /**
       * Fires an impression event with a customized argument structure
       *
       * @param {Object} module
       * @param {String} module.name
       * @param {String} module.region
       * @param {String} module.label
       */
      custom: ({
        module: {
          name: eventCategory = 'gameplay',
          region: moduleRegion = '',
          label: moduleLabel = '',
        },
      }: CustomTrackingObject) => {
        trackAction(
          'impression',
          eventCategory,
          moduleRegion,
          moduleLabel,
        )
      },
      // Stub for TypeScript
      tech: () => {},
      gameplay: () => {},
      settings: () => {},
    },
  }

  // Replace stubs with trackers
  INTERACTION_CATEGORIES.forEach((c) => {
    api.interaction[c] = (
      elementName: string,
      elementLabel: string | object,
      eventLabel: string | null = null,
    ) =>
      trackAction(
        'moduleInteraction',
        c,
        elementName,
        elementLabel,
        eventLabel,
      )
  })

  IMPRESSION_CATEGORIES.forEach((c) => {
    api.impression[c] = (
      moduleRegion: string,
      moduleLabel: string | object,
    ) => trackAction('impression', c, moduleRegion, moduleLabel)
  })

  return api
}

export default createGameTracker
