/* eslint-disable no-console */

export type SubModalContent = {
  name: string
  Content: () => React.ReactElement
}

export type ModalOptions = {
  name: string
  className?: string
  closeByBgClick?: boolean
  closeByEx?: boolean
  Content?: () => React.ReactElement
  subModalContent?: SubModalContent
  create?: (view: ModalView) => void
}

export type ModalConfirmOptions =
  | {
      message?: string
      cancelText?: string
      confirmText?: string
    }
  | string

export type ModalSystem = {
  add: (modalConfig: ModalOptions) => void
  get: (name: string) => any
  getAll: () => Record<string, any>
  open: (name: string, trigger?: string) => void
  close: () => void
  getContent: () => HTMLElement
  setScrimColor: (color: string) => void
  confirm?: (
    options: ModalConfirmOptions,
    cbConfirm: () => void,
    cbCancel: () => void,
  ) => void
}

export type ModalView = {
  root: HTMLElement
  subRoot?: HTMLElement
  name: string
  meta: { Content?: () => React.ReactElement }
  viewTime: number
  viewCount: number
  openTrigger?: string
  closeTrigger?: string
}

type ModalConfig = Pick<
  ModalOptions,
  'name' | 'className' | 'closeByBgClick' | 'closeByEx'
> &
  Pick<
    ModalView,
    'root' | 'subRoot' | 'meta' | 'viewTime' | 'viewCount'
  > & {
    id: string
    onOpen: (view: ModalView, prevView?: ModalView) => void
    onClose: (view: ModalView, prevView?: ModalView) => void
    create: (view: ModalView, publicApi: ModalSystem) => void
    wrap: HTMLElement
    ex?: HTMLElement
    subModalContent?: SubModalContent
  }

type ModalSystemOptions = {
  root: HTMLElement
  modals: Array<ModalOptions>
  onOpen: (view: ModalView, prevView?: ModalView) => void
  onClose: (view: ModalView, prevView?: ModalView) => void
}

const createModalSystem = (
  systemOptions: Partial<ModalSystemOptions>,
) => {
  /** ********** */
  /* CONSTANTS */
  /** ********** */
  const MODAL_SYSTEM_PREFIX = 'pzm'
  const MODAL_SYSTEM_CLASS = `${MODAL_SYSTEM_PREFIX}-system`
  const MODAL_CONTENT_CLASS = `${MODAL_SYSTEM_PREFIX}-modals-wrapper`
  const MODAL_SUB_CLASS = `${MODAL_SYSTEM_PREFIX}-submodal`
  const MODAL_WRAPPER_CLASS = `${MODAL_SYSTEM_PREFIX}-modal`
  const MODAL_CLASS = `${MODAL_WRAPPER_CLASS}-content`
  const MODAL_EX = `${MODAL_WRAPPER_CLASS}-ex`
  const ACTIVE_CLASS = `${MODAL_SYSTEM_PREFIX}-active`

  /** ******** */
  /* HELPERS */
  /** ******** */
  const noop = () => {}

  const addClass = (elm: HTMLElement, className: string) => {
    elm.classList.add(className)
    return elm
  }

  const removeClass = (elm: HTMLElement, className: string) => {
    elm.classList.remove(className)
    return elm
  }

  /** ********* */
  /* DEFAULTS */
  /** ********* */

  const systemOpts: ModalSystemOptions = {
    root: document.body,
    onOpen: noop,
    onClose: noop,
    modals: [],
    ...systemOptions,
  }

  /** ******** */
  /* PRIVATE */
  /** ******** */

  const modals: Record<string, ModalConfig> = {}
  const systemWrapper = addClass(systemOpts.root, MODAL_SYSTEM_CLASS)
  const subContentWrapper = addClass(
    document.createElement('div'),
    MODAL_SUB_CLASS,
  )

  const systemContent = addClass(
    document.createElement('div'),
    MODAL_CONTENT_CLASS,
  )

  systemContent.setAttribute('tabindex', '0')

  let publicApi: ModalSystem
  let currentModal: ModalConfig | undefined
  let openTrigger: string | undefined
  let closeTrigger: string | undefined
  let openTime: number

  const closeOnBgClick = (e: MouseEvent) => {
    if (e.target !== systemContent) return
    publicApi.close()
  }

  const closeOnEscape = (e: KeyboardEvent) => {
    if (e.key === 'Escape') {
      e.preventDefault()
      publicApi.close()
    }
  }

  const getModal = (name: string) => {
    if (!modals[name]) {
      throw new Error(
        `unknown modal ${name}\n\nchoose one of\n\t${Object.keys(
          modals,
        ).join(`\n\t`)}\n`,
      )
    }

    return modals[name]
  }

  const isOpen = (name: string) => {
    return name === (currentModal && currentModal.name)
  }

  const getPublicView = ({
    root,
    subRoot,
    name,
    meta,
    viewTime,
    viewCount,
  }: ModalConfig): ModalView => ({
    root,
    subRoot,
    name,
    meta,
    viewTime,
    viewCount,
    openTrigger,
    closeTrigger,
  })

  const runLifeCycle = (
    event: 'onOpen' | 'onClose',
    modal: ModalConfig,
    prevModal?: ModalConfig,
  ) => {
    const view = getPublicView(modal)
    const prevView = prevModal && getPublicView(prevModal)

    if (systemOpts[event]) {
      systemOpts[event](view, prevView)
    }

    if (modal[event]) {
      modal[event](view, prevView)
    }
  }

  const setupModal = (config: ModalOptions) => {
    if (!config.name || modals[config.name]) {
      throw new Error(`modals need a unique name`)
    }

    const c: ModalConfig = {
      meta: {},
      create: noop,
      id: `${MODAL_SYSTEM_PREFIX}-${config.name}`,
      className: '',
      onOpen: noop,
      onClose: noop,
      closeByEx: true,
      closeByBgClick: true,
      viewTime: 0,
      viewCount: 0,
      subModalContent: undefined,
      ...config,

      // begin creating the ui
      root: addClass(document.createElement('div'), MODAL_CLASS),
      wrap: addClass(
        document.createElement('div'),
        MODAL_WRAPPER_CLASS,
      ),
    }

    /** finish creating the ui */
    c.wrap.setAttribute('id', c.id)

    if (c.closeByEx) {
      c.ex = addClass(document.createElement('div'), MODAL_EX)
      c.ex.innerHTML = '×'
      c.ex.setAttribute('role', 'button')
      c.ex.setAttribute('tabindex', '0')
      c.ex.addEventListener('click', publicApi.close)
      c.ex.addEventListener('keydown', (evt) => {
        if (evt.key === 'Enter' || evt.key === 'NumpadEnter') {
          evt.preventDefault()
          evt.stopPropagation()
          publicApi.close()
        }
      })
      c.wrap.appendChild(c.ex)
    }

    if (c.subModalContent?.name) {
      c.subRoot = addClass(
        document.createElement('div'),
        `${MODAL_SYSTEM_PREFIX}-submodal-content-${config.name}`,
      )

      subContentWrapper.appendChild(c.subRoot)
      addClass(
        c.wrap,
        `${MODAL_SYSTEM_PREFIX}-with-${c.subModalContent.name}`,
      )
    }

    c.wrap.appendChild(c.root)

    if (c.className) {
      addClass(c.wrap, c.className)
    }

    /** mount the ui */
    systemContent.appendChild(c.wrap)

    /** run user creator */
    Object.assign(c, c.create(getPublicView(c), publicApi))

    modals[c.name] = c
  }

  const open = (name: string, trigger?: string) => {
    /** don't do anything if the modal has not changed */
    if (isOpen(name)) return

    /** get the modal object */
    const modal = getModal(name)

    /** keep track of analytics information */
    openTrigger = trigger
    modal.viewCount += 1
    openTime = Date.now()

    /** update the ui */
    addClass(modal.wrap, ACTIVE_CLASS)

    if (modal.subRoot) addClass(modal.subRoot, ACTIVE_CLASS)
    systemContent.focus()

    if (modal.closeByBgClick) {
      systemContent.addEventListener('click', closeOnBgClick)
    }

    if (modal.closeByBgClick || modal.closeByEx) {
      document.body.addEventListener('keydown', closeOnEscape)
    }

    /** run lifecycle hooks */
    runLifeCycle('onOpen', modal, currentModal)

    /** update the state */
    currentModal = modal
  }

  const close = (trigger?: string) => {
    const t = typeof trigger === 'string' ? trigger : undefined
    /** prevent dom event from being treated as trigger name */

    /** don't do anything if there is nothing open */
    if (!currentModal) return

    /** keep track of analytics information */
    closeTrigger = t
    currentModal.viewTime += Date.now() - openTime

    /** update the ui */
    removeClass(currentModal.wrap, ACTIVE_CLASS)

    if (currentModal.subRoot) {
      removeClass(currentModal.subRoot, ACTIVE_CLASS)
    }

    if (currentModal.closeByBgClick) {
      systemContent.removeEventListener('click', closeOnBgClick)
    }

    if (currentModal.closeByBgClick || currentModal.closeByEx) {
      document.body.removeEventListener('keydown', closeOnEscape)
    }

    /** update the state */
    const prevModal = currentModal
    currentModal = undefined

    runLifeCycle('onClose', prevModal)
  }

  /** ******* */
  /* PUBLIC */
  /** ******* */
  publicApi = {
    add(modalConfig) {
      setupModal(modalConfig)
    },

    get(name) {
      return getPublicView(getModal(name))
    },

    getAll() {
      return Object.keys(modals).map((name) => this.get(name))
    },

    open(name, trigger) {
      if (currentModal) {
        close()
        open(name, trigger)
      } else {
        addClass(systemWrapper, ACTIVE_CLASS)
        open(name, trigger)
      }
    },

    close() {
      removeClass(systemWrapper, ACTIVE_CLASS)
      close()
    },

    getContent() {
      return systemContent
    },

    setScrimColor(color) {
      systemContent.style.background = color
    },
  }

  /** *********** */
  /* INITIALIZE */
  /** *********** */
  systemWrapper.appendChild(systemContent)
  systemOpts.modals.forEach(setupModal)
  systemContent.appendChild(subContentWrapper)

  return publicApi
}

export default createModalSystem
