/** Do two lists contain exactly the same entries? */
export const areSetEqual = (
  list1: Array<any> = [],
  list2: Array<any> = [],
) => [...list1].sort().join() === [...list2].sort().join()

/**
 * Fisher-Yates shuffle taken from https://bost.ocks.org/mike/shuffle/
 * NOTE:
 * A derangement is a permutation in which none of the objects appear
 * in their "natural" (i.e., ordered) place
 * (we use it to make sure nothing ends up in the same spot after shuffling)
 */
export const shuffle = (
  arr: Array<any>,
  assertDerangement: boolean,
): Array<any> => {
  const copy = [...arr]

  let m = copy.length

  // While there remain elements to shuffle…
  while (m) {
    // Pick a remaining element…
    m -= 1

    const i = Math.floor(Math.random() * m)

    // And swap it with the current element.
    const t = copy[m]

    copy[m] = copy[i]
    copy[i] = t

    if (
      assertDerangement &&
      (arr[m] === copy[m] || arr[i] === copy[i])
    ) {
      return shuffle(arr, true)
    }
  }

  return copy
}

export const wait = (ms: number) =>
  new Promise((resolve) => {
    setTimeout(resolve, ms)
  })

export const throttle = (func: () => void, limit: number) => {
  let inThrottle: boolean
  return function fn(this: any, ...args: []) {
    const context = this
    if (!inThrottle) {
      func.apply(context, args)
      inThrottle = true
      setTimeout(() => {
        inThrottle = false
      }, limit)
    }
  }
}

export const debounce = (func: () => void, delay: number) => {
  let inDebounce: ReturnType<typeof setTimeout>
  return function fn(this: any, ...args: []) {
    const context = this
    clearTimeout(inDebounce)
    inDebounce = setTimeout(() => func.apply(context, args), delay)
  }
}

export const deepCloneSimple = (
  obj: Record<string | number, any>,
) => {
  if (!obj) return obj

  const newObj: Record<string | number, any> = Array.isArray(obj)
    ? []
    : {}

  let v: any

  const keys = Object.keys(obj) as Array<keyof typeof obj>
  keys.forEach((k) => {
    v = obj[k]
    newObj[k] = typeof v === 'object' ? deepCloneSimple(v) : v
  })

  return newObj
}
