import { UserType } from 'foundation'

import {
  isDeepLink,
  openDeepLink,
  handleUserInformationUpdate,
} from './hybridLinkHelpers'

const SUBSCRIBE_URL = 'https://www.nytimes.com/subscription/games'

/**
 * Returns an absolute URL to a subscription landing page.
 *
 * @param campaign Context where the subscribe link is being rendered
 * @param w Browser window object
 * @returns An absolute URL to a subscription landing page w/ campaign code query param.
 */
export const getSubscribeLink = (campaign?: string, w = window) => {
  if (!w || !w.navigationLinks || !w.navigationLinks.subscribe) {
    return SUBSCRIBE_URL
  }

  const subscribeUrls = w.navigationLinks.subscribe

  if (!campaign || !subscribeUrls[campaign]) {
    return subscribeUrls.default || SUBSCRIBE_URL
  }

  return subscribeUrls[campaign]
}

/**
 * Use the helper methods defined above to generate the correct subscribe link
 * for the campaign and immediate open it, whether it's a deep link or not.
 *
 * @param tag The campaign to use for the subscribe URL
 */
export const handleSubscribe = (
  tag: string,
  setUserType?: (userData: Partial<UserType>) => void,
) => {
  const link = getSubscribeLink(tag)

  if (window.isHybridWebView && window.NativeBridge) {
    window.NativeBridge.gamesAuthenticateUser('subscribe')
      .then((data: any) => {
        if (!data.success) throw new Error(data.error)

        // TODO: remove dispatch in client side entitlements milestone 3
        window.dispatchEvent(
          new CustomEvent('gamesUserCredentialChanged', {
            detail: data,
          }),
        )

        if (setUserType) {
          handleUserInformationUpdate(
            data.values.gamesAuthenticateUser,
            setUserType,
          )
        }
      })
      .catch((e: Error) => {
        console.error('Failed to authenticate user', e)
      })
  } else if (isDeepLink(link)) {
    openDeepLink(link)
  } else {
    window.location.assign(link)
  }

  return null
}
