import React, { useState, useEffect, useContext } from 'react'
import { createPortal } from 'react-dom'
import { useSelector, useDispatch } from 'react-redux'
import cn from 'classnames'

import useDeltaBanner from 'shared/marketing/useDeltaBanner'
import { setSentryData } from 'foundation' /* eslint-disable-line import/no-unresolved */
import { shouldFailOpen } from 'shared/meter'
import { useUser } from 'user'
import {
  MeterProperties,
  grantReasons,
} from 'shared/meter/MeterTypes'

import { handleUserInformationUpdate } from '../../helpers/hybridLinkHelpers'
import ModalManager from '../../shared/ModalSystem/ModalManager'
import useGameDataHybridEvent from '../../shared/hooks/use-game-data-hybrid-event'
import useAndroidBack from '../../shared/hooks/use-android-back'
import { EVENT_ON_NAVIGATE_BACK } from '../../../shared/foundation/js/hybrid/commands'
import TopSaleBanner from '../../../server/views/components/Marketing/Sale/TopSaleBanner'
import useSaleBannerHybridSubscribe from '../../shared/hooks/use-sale-banner-hybrid-subscribe'

import { didCompleteGame, saveAsCompleted } from './handleCompletion'
import Square from './Square'
import Toolbar from './Toolbar'
import WordList from './WordList'
import TextField from './TextField'
import track from './letterBoxedTracker'
import Message from './Message'
import { Moments, momentSystem } from './moments'
import Congrats from './modals/Congrats'
import HowToPlay from './modals/HowToPlay'
import YesterdaysSolution from './modals/YesterdaysSolution'
import {
  getOpenModalName,
  getIsClosing,
  getIsModalOpen,
} from './store/puzzleContainer/selectors'
import { unlockAndResetGame } from './store/puzzleContainer/actions'
import { getConfirmReset } from './store/settings'
import ConfirmReset from './modals/ConfirmReset'
import { disableActionButtons } from './util'
import { MeterContext } from './meter/meterProvider'

type Letter = {
  isActiveLetter: boolean
  isInActiveWord: boolean
  isUsed: boolean
}

type LayoutProps = {
  puzzle: {
    animating: boolean
    printDate: string
    activeWord: string
    activeLetter: string
    message: {
      isError: boolean
    }
    submittedWords: string[]
    letters: Letter[]
    par: number
    isCompleted: boolean
    preventMoves: boolean
    sides: string[][]
    yesterdaysSides: string[]
    yesterdaysSolution: string[]
    id: number
  }
  actions: {
    unlockPuzzle: () => void
    lockPuzzleForNonSubs: () => void
    handleTextChange: () => void
    addLetter: () => void
    removeLetter: () => void
    submitWord: () => void
    removeModal: () => {
      type: string
    }
    closeModal: () => {
      type: string
      payload: {
        isCancelled: boolean
      }
    }
    openModal: (arg0: string) => {
      type: string
      payload: {
        isCancelled: boolean
      }
    }
  }
}

const Layout = (props: LayoutProps) => {
  const dispatch = useDispatch()
  const confirmReset = useSelector(getConfirmReset)
  const { puzzle, actions } = props
  const { animating, printDate, isCompleted, id } = puzzle

  const { hasLoaded, updateUserType, userType } = useUser()
  const showErrorMessage = puzzle.message.isError

  useSaleBannerHybridSubscribe()

  const meterContext = useContext(MeterContext)

  const [meterResponse, setMeterResponse] = useState({
    granted: false,
    grantReason: grantReasons.METER_LIMIT,
    isOffline: true,
    loggedIn: false,
  })

  const getGranted = (meterContext: MeterProperties) => {
    const isOnline = !meterContext.isOffline
    // if the meter is online, use the meter response
    if (isOnline) {
      return meterContext.granted
    }

    // if the meter is offline, verify if the last response should fail open
    if (shouldFailOpen(meterContext)) {
      return meterContext.granted
    }

    // when all else fails, fall back on lb paywall logic
    return false
  }

  useEffect(() => {
    momentSystem.load('loading')
  }, [])

  useEffect(() => {
    // If there is an issue with the day's puzzle, it may default to show this puzzle, where the answers are "BREAKING" and "GLITCH"
    if (
      JSON.stringify(puzzle.sides) ===
      JSON.stringify([
        ['L', 'R', 'C'],
        ['E', 'G', 'K'],
        ['H', 'I', 'B'],
        ['N', 'T', 'A'],
      ])
    ) {
      setSentryData('incorrect puzzle is being shown', {
        values: JSON.stringify(puzzle.sides),
      })
    }
  }, [puzzle])

  useEffect(() => {
    if (meterContext) {
      const granted = getGranted(meterContext)
      const meter = {
        granted,
        grantReason: meterContext.grantReason,
        isOffline: meterContext.isOffline,
        loggedIn: meterContext.loggedIn,
      }
      setMeterResponse(meter)
    }
  }, [meterContext])

  const handleCredentialChanged = (event: CustomEvent) => {
    const userData = event?.detail?.values.gamesAuthenticateUser
    handleUserInformationUpdate(userData, updateUserType)
  }

  useEffect(() => {
    if (window.isHybridWebView && window.NativeBridge) {
      window.addEventListener(
        'gamesUserCredentialChanged',
        handleCredentialChanged as EventListener,
      )

      return () => {
        window.removeEventListener(
          'gamesUserCredentialChanged',
          handleCredentialChanged as EventListener,
        )
      }
    }
  }, [])

  useEffect(() => {
    if (!hasLoaded) return

    if (isCompleted) {
      track.interaction.gameplay(
        'solve-game',
        puzzle.submittedWords.length.toString(),
        null,
      )
      if (userType.hasXwd || meterResponse?.granted) {
        actions.openModal('congrats')
      } else {
        actions.lockPuzzleForNonSubs()
        saveAsCompleted(printDate)
        actions.openModal('congrats')
      }
    }
  }, [hasLoaded, userType, isCompleted])

  const didComplete = didCompleteGame(puzzle.printDate)
  const currentGame = {
    name: 'letter-boxed',
    ...puzzle,
  }

  const isModalOpen = useSelector(getIsModalOpen)

  useAndroidBack(
    (event) => {
      type Response = {
        gamesOnNavigateBack?: boolean
      }
      const response: Response = {}

      if (isModalOpen) {
        actions.closeModal()
        response[EVENT_ON_NAVIGATE_BACK] = true
      } else {
        // go back to app feed
        response[EVENT_ON_NAVIGATE_BACK] = false
      }
      event.respondWith(response)
    },
    [isModalOpen],
  )

  useGameDataHybridEvent({
    analyticsPuzzleID: id.toString(),
    puzzleID: id,
    game: 'letterBoxed',
  })

  const disableButtons = disableActionButtons(
    puzzle,
    userType.hasXwd || meterResponse.granted,
  )

  const { willShowDeltaBanner, portalEl } = useDeltaBanner()

  return (
    <div className="lb-content-box">
      <Moments
        contentProps={{
          puzzle,
          isCompleted: didComplete,
          hasLoaded,
          meterResponse,
        }}
      />
      {willShowDeltaBanner &&
        portalEl &&
        createPortal(
          <TopSaleBanner url={document.location.href} isDelta />,
          portalEl,
        )}
      <Toolbar
        actions={actions}
        animating={animating}
        printDate={printDate}
        meterResponse={meterResponse}
      />
      <ModalManager
        modalMap={{
          yesterday: YesterdaysSolution,
          congrats: Congrats,
          howToPlay: HowToPlay,
          reset: ConfirmReset,
        }}
        portalId="portal-modal-system"
        getOpenModalName={getOpenModalName}
        getIsClosing={getIsClosing}
        onClose={actions.closeModal}
        onRemove={actions.removeModal}
        actions={actions}
        data={currentGame}
        meterResponse={meterResponse}
      />
      <div className="lb-game-container">
        <div className="lb-word-container">
          <TextField
            value={puzzle.activeWord}
            action={actions.handleTextChange}
            submittedWords={puzzle.submittedWords}
            par={puzzle.par}
          >
            {showErrorMessage && <Message message={puzzle.message} />}
          </TextField>
          <div className="lb-list-container">
            {puzzle.submittedWords.length > 0 && (
              <WordList {...props} />
            )}
          </div>
          <div
            className={cn('lb-par', {
              hidden: !puzzle.submittedWords.length,
            })}
          >
            Try to solve in {puzzle.par}{' '}
            {puzzle.par > 1 ? 'words' : 'word'}
          </div>
        </div>
        <div className="lb-square-container">
          {!showErrorMessage && <Message message={puzzle.message} />}
          <Square {...props} />
          <div className="lb-button-container">
            <button
              type="button"
              className="lb-button"
              data-testid="restart"
              disabled={disableButtons}
              onClick={() => {
                if (!confirmReset) {
                  track.interaction.gameplay(
                    'reset-game',
                    `${currentGame.par || 0}`,
                    null,
                  )
                  dispatch(unlockAndResetGame())
                } else {
                  actions.openModal('reset')
                }
              }}
            >
              Restart
            </button>
            <button
              type="button"
              className="lb-button"
              data-testid="delete"
              disabled={disableButtons || puzzle.animating}
              onClick={actions.removeLetter}
            >
              Delete
            </button>
            <button
              type="button"
              className="lb-button"
              data-testid="enter"
              disabled={disableButtons || puzzle.animating}
              onClick={actions.submitWord}
            >
              Enter
            </button>
          </div>
        </div>
      </div>
    </div>
  )
}

export default Layout
