import React from 'react'
import pt from 'prop-types'
import FontFaceObserver from 'fontfaceobserver'

import {
  coordTypes,
  lineStates,
  colors,
  fontTimeout,
  fontFailHandler,
} from './constants'
import {
  calculateCoordinates,
  getSquareInfo,
  getWordListLines,
  getLineDashConfig,
} from './canvasUtils'

class PreviewSquare extends React.PureComponent {
  constructor(props) {
    super(props)

    const { config } = this.props

    this.canvas = React.createRef()

    this.letters = [].concat(
      ...props.sides.map((side) =>
        side.split('').map((letter) => ({ letter })),
      ),
    )

    this.squareInfo = getSquareInfo(
      config.canvasSize,
      config.squarePercentage,
    )
  }

  componentDidMount() {
    const fontFaceObserver = new FontFaceObserver('nyt-franklin')
    fontFaceObserver
      .load(null, fontTimeout)
      .then(() => this.drawPuzzle())
      .catch(fontFailHandler)
  }

  componentDidUpdate() {
    this.clearPuzzle()
    this.drawPuzzle()
  }

  getCtx = () => {
    return this.canvas.current.getContext('2d')
  }

  setScale = (drawFunc) => {
    // Every time we draw something, we need to reset the scale so it does not compound
    const ctx = this.getCtx()
    ctx.save()
    ctx.scale(window.devicePixelRatio, window.devicePixelRatio)

    drawFunc()

    ctx.restore()
  }

  drawPuzzle = () => {
    this.calculateCoordinates()
    this.setScale(() => {
      const { config } = this.props

      this.drawSquare()
      this.drawLines()

      if (config.outlineColor) {
        this.drawSquareOutline()
      }

      if (config.pointStyle) {
        this.drawPoints()
      }

      if (config.letterStyle) {
        this.drawLetters()
      }
    })
  }

  clearPuzzle = () => {
    const ctx = this.getCtx()
    ctx.clearRect(
      0,
      0,
      this.canvas.current.width,
      this.canvas.current.height,
    )
  }

  calculateCoordinates = () => {
    const { config } = this.props
    const letterCoords = calculateCoordinates(
      coordTypes.letter,
      this.squareInfo,
      config,
    )

    const pointCoords = calculateCoordinates(
      coordTypes.point,
      this.squareInfo,
      config,
    )

    this.updateLetterState(coordTypes.letter, letterCoords)
    this.updateLetterState(coordTypes.point, pointCoords)
  }

  drawSquare = () => {
    const { config } = this.props
    const { startingX, startingY, squareSize } = this.squareInfo

    const ctx = this.getCtx()
    ctx.fillStyle = config.squareColor
    ctx.fillRect(startingX, startingY, squareSize, squareSize)
  }

  updateLetterState = (type, coords) => {
    const updatedLetters = this.letters.map((letterObj, i) => ({
      ...letterObj,
      [`${type}Coords`]: coords[i],
    }))

    this.letters = updatedLetters
  }

  drawLines = () => {
    const { solution } = this.props
    const wordListCoords = getWordListLines(solution, this.letters)

    if (wordListCoords) {
      wordListCoords.map((line) =>
        this.drawLine(line, lineStates.isSubmitted),
      )
    }
  }

  drawLine = ({ from, to }) => {
    const { config } = this.props

    const { canvasSize, lineWidth } = config
    const { color, isDashed, lineCap } = config.lineStyle

    const lineThickness = lineWidth - 1
    const lineDash = isDashed ? getLineDashConfig(canvasSize) : []

    const ctx = this.getCtx()

    ctx.save()
    ctx.strokeStyle = color
    ctx.fillStyle = color
    ctx.lineWidth = lineThickness
    ctx.lineCap = lineCap
    ctx.setLineDash(lineDash)
    ctx.beginPath()
    ctx.moveTo(from.x, from.y)
    ctx.lineTo(to.x, to.y)
    ctx.stroke()
    ctx.restore()
  }

  drawSquareOutline = () => {
    const { config } = this.props

    const { startingX, startingY, squareSize } = this.squareInfo

    const ctx = this.getCtx()

    ctx.strokeStyle = config.outlineColor
    ctx.lineWidth = config.lineWidth - 1
    ctx.strokeRect(startingX, startingY, squareSize, squareSize)
  }

  drawPoint = (coord) => {
    const { config } = this.props

    const { dotRadius, lineWidth } = config
    const style = config.pointStyle

    const ctx = this.getCtx()

    ctx.lineWidth = lineWidth - 1
    ctx.strokeStyle = style.strokeStyle
    ctx.fillStyle = style.fillStyle
    ctx.beginPath()
    ctx.arc(coord.x, coord.y, dotRadius, 0, 2 * Math.PI, false)
    ctx.fill()
    ctx.stroke()
  }

  drawPoints = () => {
    this.letters.forEach(({ pointCoords }) =>
      this.drawPoint(pointCoords),
    )
  }

  drawLetter = (letter, coord) => {
    const { config } = this.props
    const { fontSize, letterStyle } = config

    const ctx = this.getCtx()

    ctx.textAlign = 'center'
    ctx.font = `${letterStyle.fontWeight} ${fontSize}px nyt-franklin`
    ctx.strokeStyle = letterStyle.strokeStyle
    ctx.fillStyle = letterStyle.fillStyle
    ctx.fillText(letter, coord.x, coord.y)
  }

  drawLetters = () => {
    this.letters.forEach(({ letter, letterCoords }) =>
      this.drawLetter(letter, letterCoords),
    )
  }

  render() {
    const { config } = this.props
    const { canvasSize } = config
    return (
      <canvas
        ref={this.canvas}
        width={canvasSize * window.devicePixelRatio}
        height={canvasSize * window.devicePixelRatio}
        style={{
          width: canvasSize,
          height: canvasSize,
        }}
      />
    )
  }
}

PreviewSquare.defaultProps = {
  config: pt.shape({
    squareColor: colors.white,
    outlineColor: null,
    pointStyle: null,
    letterStyle: null,
    lineStyle: {
      isDashed: false,
      color: colors.pink,
      lineCap: 'round',
    },
    canvasSize: 200,
    squarePercentage: 0.9,
    dotRadius: 2,
    fontSize: 12,
    lineWidth: 5,
  }),
}

PreviewSquare.propTypes = {
  solution: pt.arrayOf(pt.string).isRequired,
  sides: pt.arrayOf(pt.string).isRequired,
  config: pt.shape({
    squareColor: pt.string,
    outlineColor: pt.string,
    pointStyle: pt.shape({
      strokeStyle: pt.string,
      fillStyle: pt.string,
    }),
    letterStyle: pt.shape({
      strokeStyle: pt.string,
      fillStyle: pt.string,
      fontWeight: pt.number,
    }),
    lineStyle: pt.shape({
      isDashed: pt.bool,
      color: pt.string,
      lineCap: pt.string,
    }),
    canvasSize: pt.number,
    squarePercentage: pt.number,
    dotRadius: pt.number,
    fontSize: pt.number,
    lineWidth: pt.number,
  }),
}

export default PreviewSquare
