import React from 'react'
import pt from 'prop-types'
import TWEEN from '@tweenjs/tween.js'
import FontFaceObserver from 'fontfaceobserver'

import { wait } from '../../helpers/gameUtils'

import { getResponsiveConfig } from './sharedUtils'
import {
  areLettersOnSameSide,
  getSuccessMessage,
} from './containers/PuzzleContainer/utils'
import {
  colors,
  coordTypes,
  canvasConfig,
  lineStates,
  letterStateConfigs,
  squishTime,
  fontTimeout,
  fontFailHandler,
} from './constants'
import {
  isInSquare,
  getLineColor,
  getWordLines,
  getSquareInfo,
  getPointStyle,
  getLetterStyle,
  getLetterWeights,
  getWordListLines,
  changeLetterState,
  getLineDashConfig,
  getActiveWordLines,
  calculateCoordinates,
  isUserClickingLetter,
} from './canvasUtils'

class Square extends React.Component {
  constructor(props) {
    super(props)

    this.tweens = []
    this.startingPos = null
    this.canvas = React.createRef()
    this.letters = props.puzzle.letters

    this.state = {
      ...getResponsiveConfig(
        window.innerWidth,
        window.innerHeight,
        canvasConfig,
      ),
      currentLine: null,
    }

    window.addEventListener('resize', () => {
      if (document.body.clientWidth > window.innerWidth) {
        return // For pinch-to-zoom issue
      }
      const { actions } = this.props

      const canvasConfigNext = getResponsiveConfig(
        window.innerWidth,
        window.innerHeight,
        canvasConfig,
      )

      const canvasConfigPrev = this.state

      if (
        canvasConfigNext.canvasSize !== canvasConfigPrev.canvasSize
      ) {
        // If the resize triggers a redrawing of the canvas
        // End the animation before triggering componentDidUpdate with a state change
        // Othwerwise the animation will restart
        actions.endAnimation()
        this.setState(canvasConfigNext)
      }
    })
  }

  componentDidMount() {
    this.drawPuzzle()
    this.addEventListeners()

    window.requestAnimationFrame(this.animate)
  }

  shouldComponentUpdate() {
    const { puzzle } = this.props
    return !puzzle.animating
  }

  componentDidUpdate(prevProps) {
    const { puzzle } = this.props

    if (puzzle.letters !== prevProps.puzzle.letters) {
      this.letters = puzzle.letters
      this.startingPos = null
    }

    this.clearPuzzle()
    this.drawPuzzle()

    if (puzzle.animating && !prevProps.puzzle.animating) {
      this.squish()
      wait(squishTime * 2 + 30).then(this.animateLines)
    }
  }

  getCtx = () => {
    return this.canvas.current.getContext('2d')
  }

  getAdjustedMousePosition = (e) => {
    const rect = this.canvas.current.getBoundingClientRect()
    const userPos = e.clientX ? e : e.changedTouches[0]

    return {
      x: userPos.clientX - rect.left,
      y: userPos.clientY - rect.top,
    }
  }

  setScale = (drawFunc) => {
    // Every time we draw something, we need to reset the scale so it does not compound
    const ctx = this.getCtx()

    ctx.save()
    ctx.scale(window.devicePixelRatio, window.devicePixelRatio)
    drawFunc()
    ctx.restore()
  }

  animate = (time) => {
    window.requestAnimationFrame(this.animate)
    TWEEN.update(time)
  }

  animateLines = () => {
    const { puzzle } = this.props
    const activeWordCoords = getWordLines(
      puzzle.activeWord,
      this.letters,
    )
    activeWordCoords.forEach((coords, i) =>
      this.animateLineDraw(coords, i === activeWordCoords.length - 1),
    )

    // Chaining the tweens together to fire one after another
    for (let i = 1; i < this.tweens.length; i += 1) {
      this.tweens[i - 1].chain(this.tweens[i])
    }

    this.tweens[0].start()
  }

  animateLineDraw = (line, isLast) => {
    const { from, to } = line

    const startObj = this.letters.find(
      ({ pointCoords }) =>
        pointCoords.x === from.x && pointCoords.y === from.y,
    )

    const endObj = this.letters.find(
      ({ pointCoords }) =>
        pointCoords.x === to.x && pointCoords.y === to.y,
    )

    const tween = new TWEEN.Tween({
      ...startObj.pointCoords,
    })
      .to({ ...endObj.pointCoords }, 350)
      .onUpdate((increment) => {
        this.drawOnAnimationUpdate(line, startObj, endObj, increment)
      })
      .interpolation(TWEEN.Interpolation.Linear)
      .easing(TWEEN.Easing.Exponential.InOut)
      .onComplete(() =>
        this.drawOnAnimationComplete(startObj, endObj, isLast),
      )

    this.tweens.push(tween)
  }

  /* 
    DERIVED VALUES
    These values change based on the canvas size
  */
  calculateCoordinates = (rel = 1) => {
    const { canvasSize } = this.state
    const letterCoords = calculateCoordinates(
      coordTypes.letter,
      getSquareInfo(canvasSize),
      this.state,
    )

    const pointCoords = calculateCoordinates(
      coordTypes.point,
      getSquareInfo(canvasSize),
      this.state,
      rel,
    )

    this.updateLetterState(coordTypes.letter, letterCoords)
    this.updateLetterState(coordTypes.point, pointCoords)
  }

  /* 
    DRAWING HELPERS
    These functions are for interacting with canvas
  */

  drawSquare = (rel = 1) => {
    const { canvasSize } = this.state
    const { startingX, startingY, squareSize } =
      getSquareInfo(canvasSize)
    const pxModifier = (squareSize - squareSize * rel) / 2

    const ctx = this.getCtx()
    ctx.fillStyle = colors.white
    ctx.fillRect(
      startingX + pxModifier,
      startingY + pxModifier,
      squareSize * rel,
      squareSize * rel,
    )
  }

  drawSquareOutline = (rel = 1) => {
    const { canvasSize, lineWidth } = this.state
    const { startingX, startingY, squareSize } =
      getSquareInfo(canvasSize)

    const pxModifier = (squareSize - squareSize * rel) / 2

    const ctx = this.getCtx()

    ctx.strokeStyle = colors.black
    ctx.lineWidth = lineWidth - 1
    ctx.strokeRect(
      startingX + pxModifier,
      startingY + pxModifier,
      squareSize * rel,
      squareSize * rel,
    )
  }

  drawLetter = (letterObj, coord) => {
    const { fontSize } = this.state
    const style = getLetterStyle(letterObj)
    const ctx = this.getCtx()

    ctx.textAlign = 'center'
    ctx.font = `${style.fontWeight} ${fontSize}px nyt-franklin`
    ctx.strokeStyle = style.strokeStyle
    ctx.fillStyle = style.fillStyle
    ctx.fillText(letterObj.letter, coord.x, coord.y)
  }

  drawLetters = () => {
    this.letters.forEach(({ letterCoords, ...letterObj }) =>
      this.drawLetter(letterObj, letterCoords),
    )
  }

  drawPoint = (letterObj, coord) => {
    const { dotRadius, lineWidth } = this.state
    const style = getPointStyle(letterObj)
    const ctx = this.getCtx()

    ctx.lineWidth = lineWidth - 1
    ctx.strokeStyle = style.strokeStyle
    ctx.fillStyle = style.fillStyle
    ctx.beginPath()
    ctx.arc(coord.x, coord.y, dotRadius, 0, 2 * Math.PI, false)
    ctx.fill()
    ctx.stroke()
  }

  drawPoints = () => {
    this.letters.forEach(({ pointCoords, ...letterObj }) =>
      this.drawPoint(letterObj, pointCoords),
    )
  }

  drawLine = (
    { from, to, repeated = 0, repeatCount = 0 },
    lineState = lineStates.isActive,
  ) => {
    const { canvasSize, lineWidth } = this.state
    const { isActive, isAnimating } = lineStates
    const lineColor = getLineColor(lineState, repeated)

    const lineDash =
      lineState === isActive ? getLineDashConfig(canvasSize) : []

    const lineThickness =
      lineState === isAnimating
        ? lineWidth + repeatCount
        : lineWidth + repeated

    const ctx = this.getCtx()

    ctx.save()
    ctx.strokeStyle = lineColor
    ctx.fillStyle = lineColor
    ctx.lineWidth = lineThickness
    ctx.setLineDash(lineDash)
    ctx.beginPath()
    ctx.moveTo(from.x, from.y)
    ctx.lineTo(to.x, to.y)
    ctx.globalCompositeOperation = 'source-atop'
    ctx.stroke()
    ctx.restore()
  }

  drawLines = () => {
    const { puzzle } = this.props
    const { currentLine } = this.state
    const wordListCoords = getWordListLines(
      puzzle.submittedWords,
      this.letters,
    )

    const activeWordCoords = getActiveWordLines(
      puzzle.activeWord,
      this.letters,
    )

    if (wordListCoords) {
      wordListCoords.map((line) =>
        this.drawLine(line, lineStates.isSubmitted),
      )
    }

    if (activeWordCoords) {
      activeWordCoords.map((line) => this.drawLine(line))
    }

    if (currentLine) {
      this.drawLine(currentLine)
    }
  }

  drawPuzzle = () => {
    const fontFaceObservers = getLetterWeights().map((weight) => {
      return new FontFaceObserver('nyt-franklin', { weight }).load(
        null,
        fontTimeout,
      )
    })

    Promise.all(fontFaceObservers)
      .then(() => {
        // Ready to use the font in a canvas context
        this.calculateCoordinates()
        this.setScale(() => {
          this.drawSquare()
          this.drawLines()
          this.drawSquareOutline()
          this.drawPoints()
          this.drawLetters()
        })
      })
      .catch(fontFailHandler)
  }

  clearPuzzle = () => {
    const ctx = this.getCtx()

    ctx.clearRect(
      0,
      0,
      this.canvas.current.width,
      this.canvas.current.height,
    )
  }

  clearSquare = () => {
    const { canvasSize, dotRadius, lineWidth } = this.state
    const { startingX, startingY, squareSize } =
      getSquareInfo(canvasSize)
    const mod = dotRadius + lineWidth

    this.getCtx().clearRect(
      startingX - mod,
      startingY - mod,
      squareSize + mod * 2,
      squareSize + mod * 2,
    )
  }

  updateLetterState = (type, coords) => {
    const updatedLetters = this.letters.map((letterObj, i) => ({
      ...letterObj,
      [`${type}Coords`]: coords[i],
    }))

    this.letters = updatedLetters
  }

  /*
    ANIMATION FUNCTIONS
  */
  squish = () => {
    const { puzzle, actions } = this.props
    const draw = ({ scale }) => {
      const ctx = this.getCtx()
      ctx.save()

      this.calculateCoordinates(scale)

      ctx.scale(window.devicePixelRatio, window.devicePixelRatio)

      this.clearSquare()

      this.drawSquare(scale)
      this.drawLines(scale)
      this.drawSquareOutline(scale)
      this.drawPoints()

      ctx.restore()
    }

    const rel = { scale: 1 }
    const tweenA = new TWEEN.Tween(rel)
      .to({ scale: 0.85 }, squishTime)
      .onUpdate(draw)

    const tweenB = new TWEEN.Tween(rel)
      .to({ scale: 1 }, squishTime)
      .onUpdate(draw)

    tweenA.chain(tweenB).start()

    wait(squishTime).then(() => {
      actions.showMessage(getSuccessMessage(puzzle), false)
    })
    wait(800).then(actions.clearErrorMessage)
  }

  /* 
    INTERACTION HANDLERS
    These methods are concerned with the logic involved in down, move, and up events but
    are not concerned with support
  */
  interactionDown = (e) => {
    // If we detect more than one touch point, i.e. the user is pinching
    // to zoom in/out on a mobile device, skip the rest so the browser
    // can perform its default zoom gestures
    if (e.touches && e.touches.length > 1) return

    // otherwise prevent the brwoser's default behavior so we can
    // do custom interaction
    e.preventDefault()

    const { puzzle, actions } = this.props

    if (puzzle.animating) return

    const mousePos = this.getAdjustedMousePosition(e)
    const letterObj = isUserClickingLetter(mousePos, this.letters)
    const { activeLetter } = puzzle

    // Only draw a line if the user is touching a letter AND EITHER
    // 1. There is no activeLetter OR
    // 2. The user is starting with the activeLetter
    const isStartingWithActiveLetter =
      letterObj &&
      (activeLetter === '' || letterObj.letter === activeLetter)

    if (!isStartingWithActiveLetter) return

    // If there is no activeLetter and user is drawing a line from a letter
    // make that the new activeLetter
    if (activeLetter === '') {
      actions.addLetter(letterObj.letter)
    }

    this.startingPos = letterObj.pointCoords
  }

  interactionMove = (e) => {
    // If we detect more than one touch point, i.e. the user is pinching
    // to zoom in/out on a mobile device, skip the rest so the browser
    // can perform its default zoom gestures
    if (e.touches && e.touches.length > 1) return

    // otherwise prevent the brwoser's default behavior so we can
    // do custom interaction
    e.preventDefault()

    const { puzzle, actions } = this.props
    const { canvasSize } = this.state

    if (puzzle.animating) return

    const mousePos = this.getAdjustedMousePosition(e)
    const letterObj = isUserClickingLetter(mousePos, this.letters)
    const { activeLetter } = puzzle

    if (
      !this.startingPos ||
      !isInSquare(mousePos, getSquareInfo(canvasSize))
    ) {
      return
    }

    this.setState({
      currentLine: { from: this.startingPos, to: mousePos },
    })

    // Check to see if current letter is not the same as activeLetter
    // that way we don't call addLetter a bunch of times
    // and validate that next letter is not on same side as previous letter
    const nextLetterIsValid =
      letterObj &&
      letterObj.letter !== activeLetter &&
      !areLettersOnSameSide(
        this.letters,
        letterObj.letter,
        activeLetter,
      )

    if (!nextLetterIsValid) return

    actions.addLetter(letterObj.letter)
    this.startingPos = letterObj.pointCoords
  }

  interactionUp = (e) => {
    // If we detect more than one touch point, i.e. the user is pinching
    // to zoom in/out on a mobile device, skip the rest so the browser
    // can perform its default zoom gestures
    if (e.touches && e.touches.length > 1) return

    // otherwise prevent the brwoser's default behavior so we can
    // do custom interaction
    e.preventDefault()

    const { puzzle, actions } = this.props

    if (puzzle.animating) return

    const mousePos = this.getAdjustedMousePosition(e)
    const letterObj = isUserClickingLetter(mousePos, this.letters)
    const { activeLetter } = puzzle

    this.setState({ currentLine: null })

    if (!letterObj || letterObj.letter === activeLetter) {
      return
    }

    actions.addLetter(letterObj.letter)
    this.startingPos = null
  }

  /* 
   EVENT LISTENERS
   Supporting mouse and touch interactions
 */
  addEventListeners = () => {
    let canDrag = false

    this.canvas.current.addEventListener('mousedown', (e) => {
      e.preventDefault()
      canDrag = true
      this.interactionDown(e)
    })

    this.canvas.current.addEventListener('mousemove', (e) => {
      e.preventDefault()
      if (!canDrag) return
      this.interactionMove(e)
    })

    this.canvas.current.addEventListener('mouseup', (e) => {
      e.preventDefault()
      canDrag = false
      this.interactionUp(e)
    })

    this.canvas.current.addEventListener('touchstart', (e) => {
      this.interactionDown(e)
    })

    this.canvas.current.addEventListener('touchmove', (e) => {
      this.interactionMove(e)
    })

    this.canvas.current.addEventListener('touchend', (e) => {
      this.interactionUp(e)
    })
  }

  drawOnAnimationUpdate = (
    originalLine,
    startObj,
    endObj,
    increment,
  ) => {
    const { puzzle } = this.props
    const activeLetterObj = this.letters.find(
      ({ letter }) => puzzle.activeLetter === letter,
    )

    // Draw line from starting point to increment
    const lineObj = {
      from: startObj.pointCoords,
      to: increment,
      repeated: originalLine.repeated,
      repeatCount: originalLine.repeatCount,
    }

    this.setScale(() => {
      this.drawLine(lineObj, lineStates.isAnimating)

      // Active letter should not have 'inactive' state during animation
      this.drawPoint(
        changeLetterState(
          activeLetterObj,
          letterStateConfigs.inactive,
        ),
        activeLetterObj.pointCoords,
      )

      // Starting point should be active during animation
      this.drawPoint(
        changeLetterState(startObj, letterStateConfigs.active),
        startObj.pointCoords,
      )

      // So that line does not appear to be drawn on top of end point
      this.drawPoint(
        changeLetterState(endObj, letterStateConfigs.inactive),
        endObj.pointCoords,
      )
    })
  }

  drawOnAnimationComplete = (startObj, endObj, isLast) => {
    const { actions } = this.props
    // When line has completed drawing, change the starting point to the animating state
    // and the ending point to the active state
    this.setScale(() => {
      this.drawPoint(
        changeLetterState(startObj, letterStateConfigs.animating),
        startObj.pointCoords,
      )
      this.drawPoint(
        changeLetterState(endObj, letterStateConfigs.active),
        endObj.pointCoords,
      )
    })

    // Once the final tween has completed, add word to word list and reset tweens array
    if (isLast) {
      actions.addWord()

      this.tweens = []
    }
  }

  render() {
    const { canvasSize } = this.state

    return (
      <canvas
        ref={this.canvas}
        width={canvasSize * window.devicePixelRatio}
        height={canvasSize * window.devicePixelRatio}
        style={{
          width: canvasSize,
          height: canvasSize,
        }}
      />
    )
  }
}

Square.propTypes = {
  puzzle: pt.shape({
    animating: pt.bool.isRequired,
    activeWord: pt.string.isRequired,
    activeLetter: pt.string.isRequired,
    submittedWords: pt.arrayOf(pt.string).isRequired,
    letters: pt.arrayOf(
      pt.shape({
        isActiveLetter: pt.bool.isRequired,
        isInActiveWord: pt.bool.isRequired,
        isUsed: pt.bool.isRequired,
      }).isRequired,
    ),
  }).isRequired,
  actions: pt.shape({
    addLetter: pt.func.isRequired,
    addWord: pt.func.isRequired,
    clearErrorMessage: pt.func.isRequired,
    endAnimation: pt.func.isRequired,
    removeLetter: pt.func.isRequired,
    showMessage: pt.func.isRequired,
  }).isRequired,
}

export default Square
