import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import Square from '../Square'
import Toolbar from '../Toolbar'
import Layout from '../Layout'
import useAndroidBack from '../../../shared/hooks/use-android-back'
import { unlockAndResetGame } from '../store/puzzleContainer/actions'

jest.mock('foundation', () => ({
  getGameData: () => ({}),
  env: {
    name: 'dev',
  },
  track: jest.fn(),
  appendSupportedParams: jest.fn(),
}))

jest.mock(
  '../Square',
  () =>
    function Square() {
      return <div />
    },
)
jest.mock(
  '../Toolbar',
  () =>
    function Toolbar() {
      return <div />
    },
)

jest.mock(
  '../WordList',
  () =>
    function WordList() {
      return <div />
    },
)

jest.mock(
  '../TextField',
  () =>
    function TextField() {
      return <div />
    },
)

jest.mock(
  '../Message',
  () =>
    function Message() {
      return <div />
    },
)

jest.mock('../moments/index', () => ({
  Moments: jest.fn().mockReturnValue(<div />),
  momentSystem: {
    load: jest.fn(),
  },
}))

jest.mock('../../../shared/hooks/use-android-back', () => jest.fn())

jest.mock('../store/puzzleContainer/actions', () => ({
  unlockAndResetGame: jest.fn(),
}))

describe('LETTER BOXED: LAYOUT', () => {
  const puzzle = {
    animating: false,
    printDate: '2023-04-25',
    activeWord: 'ERA',
    activeLetter: 'A',
    message: { isError: false },
    submittedWords: [],
    letters: [],
    par: 6,
    isCompleted: false,
    preventMoves: false,
    sides: [],
    yesterdaysSides: [],
    yesterdaysSolution: [],
    id: 1595,
  }

  const actions = {
    unlockPuzzle: jest.fn(),
    lockPuzzleForNonSubs: jest.fn(),
    handleTextChange: jest.fn(),
    addLetter: jest.fn(),
    removeLetter: jest.fn(),
    submitWord: jest.fn(),
    removeModal: jest.fn(),
    closeModal: jest.fn(),
    openModal: jest.fn(),
  }

  const user = {
    userType: {
      hasXwd: false,
    },
    hasLoaded: true,
  }

  const renderLayout = (store, puzzle) =>
    mount(
      <Provider store={store}>
        <Layout
          actions={actions}
          puzzle={{ ...puzzle }}
          user={user}
        />
      </Provider>,
    )

  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('should render the toolbar, square and buttons', () => {
    expect.assertions(6)
    const store = {
      getState: () => ({
        puzzleContainer: { modalSystem: {} },
        settings: { confirmReset: true },
      }),
      subscribe: jest.fn(),
      dispatch: jest.fn(),
    }
    const layout = renderLayout(store, puzzle)
    expect(layout.find(Toolbar).exists()).toEqual(true)
    expect(layout.find(Square).exists()).toEqual(true)

    const restartButton = layout.find('button[data-testid="restart"]')

    const deleteButton = layout.find('button[data-testid="delete"]')

    const submitButton = layout.find('button[data-testid="enter"]')

    expect(restartButton.text()).toEqual('Restart')
    restartButton.simulate('click')
    expect(actions.openModal).toHaveBeenCalledWith('reset')

    deleteButton.simulate('click')
    expect(actions.removeLetter).toHaveBeenCalled()

    submitButton.simulate('click')
    expect(actions.submitWord).toHaveBeenCalled()
  })

  it('has buttons disabled if no active word', () => {
    expect.assertions(5)

    const store = {
      getState: () => ({
        puzzleContainer: { modalSystem: {} },
        settings: { confirmReset: true },
      }),
      subscribe: jest.fn(),
      dispatch: jest.fn(),
    }
    const layout = renderLayout(store, { ...puzzle, activeWord: '' })
    expect(layout.find(Toolbar).exists()).toEqual(true)
    expect(layout.find(Square).exists()).toEqual(true)

    const restartButton = layout.find('button[data-testid="restart"]')

    const deleteButton = layout.find('button[data-testid="delete"]')

    const submitButton = layout.find('button[data-testid="enter"]')

    expect(restartButton.props().disabled).toBe(true)
    expect(deleteButton.props().disabled).toBe(true)
    expect(submitButton.props().disabled).toBe(true)
  })

  it('uses the android back hook', () => {
    renderLayout(
      {
        getState: () => ({
          puzzleContainer: { modalSystem: {} },
          settings: { confirmReset: true },
        }),
        subscribe: jest.fn(),
        dispatch: jest.fn(),
      },
      puzzle,
    )
    expect(useAndroidBack).toHaveBeenCalled()
  })

  it('opens the reset modal by default when a user clicks the reset button', () => {
    const wrapper = renderLayout(
      {
        getState: () => ({
          puzzleContainer: { modalSystem: {} },
          settings: { confirmReset: true },
        }),
        subscribe: jest.fn(),
        dispatch: jest.fn(),
      },
      puzzle,
    )
    wrapper.find('button[data-testid="restart"]').simulate('click')

    expect(actions.openModal).toHaveBeenCalledWith('reset')
    expect(unlockAndResetGame).not.toHaveBeenCalled()
  })

  it('resets the game when a user has opted out of reset confirmation and clicks the reset button', () => {
    const wrapper = renderLayout(
      {
        getState: () => ({
          puzzleContainer: { modalSystem: {} },
          settings: { confirmReset: false },
        }),
        subscribe: jest.fn(),
        dispatch: jest.fn(),
      },
      puzzle,
    )
    wrapper.find('button[data-testid="restart"]').simulate('click')

    expect(actions.openModal).not.toHaveBeenCalled()
    expect(unlockAndResetGame).toHaveBeenCalledTimes(1)
  })
})
