import { colors } from '../constants'
import {
  hasSamePath,
  getPointStyle,
  getLetterStyle,
  getLetterWeights,
  checkForRepetition,
  getRepetitionCount,
  getWordCoordinates,
  areCoordinatesEqual,
  getWordListCoordinates,
  getActiveWordCoordinates,
} from '../canvasUtils'

const someLetters = [
  {
    letter: 'U',
    pointCoords: { x: 0, y: 0 },
    isActiveLetter: true,
    isInActiveWord: true,
    isUsed: false,
    side: 'NORTH',
  },
  {
    letter: 'N',
    pointCoords: { x: 1, y: 1 },
    isActiveLetter: false,
    isInActiveWord: false,
    isUsed: false,
    side: 'NORTH',
  },
  {
    letter: 'S',
    pointCoords: { x: 2, y: 2 },
    isActiveLetter: false,
    isInActiveWord: false,
    isUsed: true,
    side: 'NORTH',
  },
  {
    letter: 'H',
    pointCoords: { x: 3, y: 3 },
    isActiveLetter: false,
    isInActiveWord: true,
    isUsed: false,
    side: 'EAST',
  },
]

jest.mock('responsive', () => ({
  isMobile: false,
}))

describe('LETTER BOXED: CANVAS UTILS', () => {
  it('should get the coordinates of the active word', () => {
    expect(getWordCoordinates('NUNS', someLetters)).toEqual([
      { from: { x: 1, y: 1 }, to: { x: 0, y: 0 } },
      { from: { x: 0, y: 0 }, to: { x: 1, y: 1 } },
      { from: { x: 1, y: 1 }, to: { x: 2, y: 2 } },
    ])
  })

  it('should get the coordinates of the word list', () => {
    expect(
      getWordListCoordinates(['SUN', 'NUN'], someLetters),
    ).toEqual([
      { from: { x: 2, y: 2 }, to: { x: 0, y: 0 } },
      { from: { x: 0, y: 0 }, to: { x: 1, y: 1 } },
      { from: { x: 1, y: 1 }, to: { x: 0, y: 0 } },
      { from: { x: 0, y: 0 }, to: { x: 1, y: 1 } },
    ])
  })

  it('should find repetitions and return new line with one set of coords for each repeated path', () => {
    const wordLineList = getActiveWordCoordinates('NUNS', someLetters)
    expect(wordLineList).toEqual([
      { from: { x: 1, y: 1 }, to: { x: 0, y: 0 } },
      { from: { x: 1, y: 1 }, to: { x: 0, y: 0 } },
      { from: { x: 1, y: 1 }, to: { x: 2, y: 2 } },
    ])
  })

  it('should determine if two sets of coordinates are the same', () => {
    expect(
      areCoordinatesEqual({ x: 0, y: 0 }, { x: 0, y: 0 }),
    ).toEqual(true)
  })

  it('should determine if two lines have the same path', () => {
    const pointA = { x: 0, y: 0 }
    const pointB = { x: 1, y: 1 }
    const lineA = { from: pointA, to: pointB }
    const lineB = { from: pointB, to: pointA }
    expect(hasSamePath(lineA, lineB)).toEqual(true)
  })

  it('should determine how many times a line has been repeated', () => {
    const wordLineList = getWordCoordinates('NUNS', someLetters)

    const wordListlineList = getWordListCoordinates(
      ['SUN', 'NUN'],
      someLetters,
    )

    const line = {
      from: { x: 1, y: 1 },
      to: { x: 0, y: 0 },
    } // N to U
    expect(getRepetitionCount(line, wordLineList)).toEqual(2)
    expect(getRepetitionCount(line, wordListlineList)).toEqual(3)
  })

  it('should return new line list with repetition values', () => {
    const wordLineList = getWordCoordinates('NUNS', someLetters)

    const wordListlineList = getWordListCoordinates(
      ['SUN', 'NUN'],
      someLetters,
    )
    expect(checkForRepetition(wordLineList)).toEqual([
      {
        from: { x: 1, y: 1 },
        to: { x: 0, y: 0 },
        repeated: 0,
        repeatCount: 2,
      },
      {
        from: { x: 0, y: 0 },
        to: { x: 1, y: 1 },
        repeated: 1,
        repeatCount: 2,
      },
      {
        from: { x: 1, y: 1 },
        to: { x: 2, y: 2 },
        repeated: 0,
        repeatCount: 1,
      },
    ])
    expect(checkForRepetition(wordListlineList)).toEqual([
      {
        from: { x: 2, y: 2 },
        to: { x: 0, y: 0 },
        repeated: 0,
        repeatCount: 1,
      },
      {
        from: { x: 0, y: 0 },
        to: { x: 1, y: 1 },
        repeated: 0,
        repeatCount: 3,
      },
      {
        from: { x: 1, y: 1 },
        to: { x: 0, y: 0 },
        repeated: 1,
        repeatCount: 3,
      },
      {
        from: { x: 0, y: 0 },
        to: { x: 1, y: 1 },
        repeated: 2,
        repeatCount: 3,
      },
    ])
  })

  it('should get the correct style for an active letter', () => {
    const result = {
      strokeStyle: colors.black,
      fillStyle: colors.black,
      fontWeight: 600,
    }
    expect(getLetterStyle(someLetters[0])).toEqual(result)
  })

  it('should get the correct style for a non-active letter', () => {
    const result = {
      strokeStyle: colors.white,
      fillStyle: colors.white,
      fontWeight: 600,
    }
    expect(getLetterStyle(someLetters[1])).toEqual(result)
  })

  it('should get the correct style for a used letter', () => {
    const result = {
      strokeStyle: colors.black,
      fillStyle: colors.black,
      fontWeight: 500,
    }
    expect(getLetterStyle(someLetters[2])).toEqual(result)
  })

  it('should get the correct style for a letter in an active word', () => {
    const result = {
      strokeStyle: colors.black,
      fillStyle: colors.black,
      fontWeight: 600,
    }
    expect(getLetterStyle(someLetters[3])).toEqual(result)
  })

  it('should get the correct style for a point with an active letter', () => {
    const result = {
      strokeStyle: colors.pink,
      fillStyle: colors.black,
    }
    expect(getPointStyle(someLetters[0])).toEqual(result)
  })

  it('should get the correct style for a point with an inactive letter', () => {
    const result = {
      strokeStyle: colors.black,
      fillStyle: colors.white,
    }
    expect(getPointStyle(someLetters[1])).toEqual(result)
  })

  it('should get the correct style for a point with a used letter', () => {
    const result = {
      strokeStyle: colors.black,
      fillStyle: colors.pink,
    }
    expect(getPointStyle(someLetters[2])).toEqual(result)
  })

  it('should get the correct style for a point with a letter in the active word', () => {
    const result = {
      strokeStyle: colors.pink,
      fillStyle: colors.white,
    }
    expect(getPointStyle(someLetters[3])).toEqual(result)
  })

  it('should get a list of all font weights used in letter styles', () => {
    const fontWeights = getLetterWeights()

    expect(fontWeights).toHaveLength(2)
    expect(fontWeights).toContain(600)
  })
})
