import { didCompleteGame, saveAsCompleted } from '../handleCompletion'

class LocalStorageMock {
  constructor() {
    this.store = {}
  }

  getItem(key) {
    return this.store[key] || null
  }

  setItem(key, value) {
    this.store[key] = value.toString()
  }

  removeItem(key) {
    delete this.store[key]
  }

  clear() {
    this.store = {}
  }
}
const localStorageMock = new LocalStorageMock()
const original = window.localStorage

describe('LETTER BOXED: HANDLE COMPLETION  - CLIENT SIDE', () => {
  const expectedKey = 'letterboxed-today'
  const printDate = '2021-05-11'
  let removeSpy
  let errorSpy

  beforeEach(() => {
    localStorageMock.clear()
    Object.defineProperty(window, 'localStorage', {
      value: localStorageMock,
    })
    removeSpy = jest.spyOn(localStorageMock, 'removeItem')
    errorSpy = jest.spyOn(global.console, 'error')
    errorSpy.mockImplementation(jest.fn())
  })

  afterEach(() => {
    jest.resetAllMocks()
    Object.defineProperty(window, 'localStorage', {
      value: original,
    })
  })

  describe('didCompleteGame()', () => {
    it('when id matches saved, returns true and does not clear storage', () => {
      saveAsCompleted(printDate)
      expect(didCompleteGame(printDate)).toEqual(true)
      expect(removeSpy).not.toHaveBeenCalled()
    })

    it('when id does not match saved, returns false and clears storage', () => {
      saveAsCompleted('2021-05-11')
      expect(didCompleteGame('2021-05-12')).toEqual(false)
      expect(removeSpy).toHaveBeenCalledWith(expectedKey)
    })

    it('with no saved value, returns false and does not clear storage', () => {
      expect(didCompleteGame(printDate)).toEqual(false)
      expect(removeSpy).not.toHaveBeenCalled()
    })

    it('when value cannot be parsed, returns false and clears storage', () => {
      const unparseable = 'notParseable'
      localStorageMock.setItem(expectedKey, unparseable)

      expect(didCompleteGame(printDate)).toEqual(false)

      expect(removeSpy).toHaveBeenCalledWith(expectedKey)
      expect(errorSpy).toHaveBeenCalledWith(
        `could not parse local progress: ${unparseable}`,
      )
    })
  })

  describe('saveAsCompleted()', () => {
    it('when setItem errors, logs error', () => {
      const setItemSpy = jest.spyOn(localStorageMock, 'setItem')
      const error = new Error('ahhh!')
      setItemSpy.mockImplementation(() => {
        throw error
      })

      saveAsCompleted(printDate)
      expect(errorSpy).toHaveBeenCalledWith(
        'local storage failure:',
        error,
      )
    })
  })
})
