import {
  colors,
  coordTypes,
  lineStates,
  squareSize,
  letterStateConfigs,
  letterTouchAreaConfig,
} from './constants'

/* 
  COORDINATE CALCULATION & LINE HELPERS
*/
export const calculateCoordinates = (
  type = coordTypes.letter,
  squareInfo,
  canvasElemsConfig,
  rel = 1,
) => {
  const { startingX, startingY, squareSize, padding } = squareInfo

  const { fontSize, dotRadius } = canvasElemsConfig

  let x0
  let x1
  let x2 // Shared coordinates between NORTH and SOUTH

  let y0
  let y1
  let y2 // Shared coordinates between WEST & EAST

  let northY
  let southY
  let westX
  let eastX

  const pxModifier = (squareSize - squareSize * rel) / 2

  if (type === coordTypes.point) {
    x0 = startingX + padding + pxModifier
    x1 = startingX + squareSize / 2
    x2 = startingX + squareSize - padding - pxModifier

    y0 = startingY + padding + pxModifier
    y1 = startingY + squareSize / 2
    y2 = startingY + squareSize - padding - pxModifier

    northY = startingY + pxModifier
    southY = startingY + squareSize - pxModifier

    westX = startingX + pxModifier
    eastX = startingX + squareSize - pxModifier
  } else if (type === coordTypes.letter) {
    x0 = startingX + padding
    x1 = x0 + squareSize / 2 - padding
    x2 = startingX + squareSize - padding

    y0 = startingY + padding + fontSize / 3
    y1 = startingY + squareSize / 2 + fontSize / 3
    y2 = startingY + squareSize - padding + fontSize / 3

    northY = startingY - dotRadius - padding / 4
    southY =
      startingY + squareSize + fontSize + dotRadius + padding / 12

    westX = startingX - fontSize / 2 - padding / 2.5
    eastX = startingX + squareSize + fontSize / 2 + padding / 2.5
  }

  const northCoords = [
    { x: x0, y: northY },
    { x: x1, y: northY },
    { x: x2, y: northY },
  ]

  const southCoords = [
    { x: x0, y: southY },
    { x: x1, y: southY },
    { x: x2, y: southY },
  ]

  const westCoords = [
    { x: westX, y: y0 },
    { x: westX, y: y1 },
    { x: westX, y: y2 },
  ]

  const eastCoords = [
    { x: eastX, y: y0 },
    { x: eastX, y: y1 },
    { x: eastX, y: y2 },
  ]

  return [
    ...northCoords,
    ...eastCoords,
    ...southCoords,
    ...westCoords,
  ]
}

export const getWordCoordinates = (word, letters) => {
  const wordLetters = word.split('')
  const wordCoords = []

  if (wordLetters.length < 2) return null

  for (let a = 1; a < wordLetters.length; a += 1) {
    const prevLetter = wordLetters[a - 1]
    const currLetter = wordLetters[a]

    const from = letters.find(
      ({ letter }) => letter === prevLetter,
    ).pointCoords

    const to = letters.find(
      ({ letter }) => letter === currLetter,
    ).pointCoords

    wordCoords.push({ from, to })
  }

  return wordCoords
}

export const areCoordinatesEqual = (coords1, coords2) =>
  coords1.x === coords2.x && coords1.y === coords2.y

export const hasSamePath = (coords1, coords2) =>
  (areCoordinatesEqual(coords1.from, coords2.from) ||
    areCoordinatesEqual(coords1.from, coords2.to)) &&
  (areCoordinatesEqual(coords1.to, coords2.from) ||
    areCoordinatesEqual(coords1.to, coords2.to))

export const getActiveWordCoordinates = (activeWord, letters) => {
  const activeWordLetters = activeWord.split('')
  const activeWordCoords = []

  if (activeWordLetters.length < 2) return null

  for (let a = 1; a < activeWordLetters.length; a += 1) {
    const prevLetter = activeWordLetters[a - 1]
    const currLetter = activeWordLetters[a]

    const newCoord = {
      from: letters.find(({ letter }) => letter === prevLetter)
        .pointCoords,
      to: letters.find(({ letter }) => letter === currLetter)
        .pointCoords,
    }

    // Takes into account if current path has been used before
    // i.e. if a user has drawn from A -> B and then draws from B -> A
    // the path has been used, and instead of storing the coords from B -> A
    // we will store them the same way we stored the first line (A -> B)
    // This is specifically for issue with dashed line not quite lining up when repeated
    activeWordCoords.forEach((coord) => {
      if (hasSamePath(coord, newCoord)) {
        newCoord.from = coord.from
        newCoord.to = coord.to
      }
    })

    activeWordCoords.push(newCoord)
  }

  return activeWordCoords
}

export const getWordListCoordinates = (list, letters) =>
  [...list].reduce(
    (acc, word) => [
      ...acc,
      ...getWordCoordinates(word, letters, list),
    ],
    [],
  )

export const getRepetitionCount = (currLine, linesDrawn) => {
  let count = 0
  linesDrawn.forEach((line) => {
    if (hasSamePath(line, currLine)) count += 1
  })
  return count
}

export const checkForRepetition = (list) =>
  list &&
  list.map((item, i) => {
    item.repeated = getRepetitionCount(item, list.slice(0, i)) // repeated x times so far
    item.repeatCount = getRepetitionCount(item, list) // repeated x times in total
    return item
  })

export const getActiveWordLines = (word, letters) =>
  checkForRepetition(getActiveWordCoordinates(word, letters))

export const getWordLines = (word, letters) =>
  checkForRepetition(getWordCoordinates(word, letters))

export const getWordListLines = (list, letters) =>
  checkForRepetition(getWordListCoordinates(list, letters))

/* 
  USER POSITION HELPERS
*/
const isUserTouchingLetter = (coords, mousePos, leeway) =>
  Math.abs(coords.x - mousePos.x) < leeway.x &&
  coords.y - mousePos.y < leeway.y &&
  coords.y - mousePos.y > 0

const isUserTouchingPoint = (coords, mousePos, leeway) =>
  Math.abs(coords.x - mousePos.x) < leeway &&
  Math.abs(coords.y - mousePos.y) < leeway

export const isUserClickingLetter = (mousePos, letters) => {
  const {
    letterWidth,
    letterHeight,
    letterPadding,
    pointSize,
    pointPadding,
  } = letterTouchAreaConfig

  const letterLeeway = {
    x: letterWidth / 2 + letterPadding,
    y: letterHeight + letterPadding,
  }
  const pointLeeway = pointSize / 2 + pointPadding

  for (let i = 0; i < letters.length; i += 1) {
    const { letterCoords, pointCoords } = letters[i]
    if (
      isUserTouchingLetter(letterCoords, mousePos, letterLeeway) ||
      isUserTouchingPoint(pointCoords, mousePos, pointLeeway)
    ) {
      return letters[i]
    }
  }

  return null
}

export const isInSquare = (
  mousePos,
  { startingX, startingY, squareSize },
) =>
  mousePos.x >= startingX &&
  mousePos.x <= startingX + squareSize &&
  mousePos.y >= startingY &&
  mousePos.y <= startingY + squareSize

/* 
  POINT, LETTER & LINE STYLES
*/
export const getPointStyle = ({
  isActiveLetter,
  isInActiveWord,
  isUsed,
  isAnimating,
}) => {
  let output = {
    strokeStyle: colors.black,
    fillStyle: colors.white,
  }

  if (isActiveLetter && isInActiveWord) {
    output = {
      strokeStyle: colors.pink,
      fillStyle: colors.black,
    }
  } else if (isInActiveWord) {
    output = {
      strokeStyle: colors.pink,
      fillStyle: colors.white,
    }
  } else if (isUsed) {
    output = {
      strokeStyle: colors.black,
      fillStyle: colors.pink,
    }
  } else if (isAnimating) {
    output = {
      strokeStyle: colors.white,
      fillStyle: colors.pink,
    }
  }

  return output
}

// Single source of truth for LB letter styling/fonts
export const letterStyles = {
  default: {
    strokeStyle: colors.white,
    fillStyle: colors.white,
    fontWeight: 600,
  },
  activeLetter: {
    strokeStyle: colors.black,
    fillStyle: colors.black,
    fontWeight: 600,
  },
  activeWord: {
    strokeStyle: colors.black,
    fillStyle: colors.black,
    fontWeight: 600,
  },
  used: {
    strokeStyle: colors.black,
    fillStyle: colors.black,
    fontWeight: 500,
  },
}

export const getLetterStyle = ({
  isActiveLetter,
  isInActiveWord,
  isUsed,
}) => {
  let output = letterStyles.default

  if (isActiveLetter && isInActiveWord) {
    output = letterStyles.activeLetter
  } else if (isInActiveWord) {
    output = letterStyles.activeWord
  } else if (isUsed) {
    output = letterStyles.used
  }

  return output
}

// Cycle through all letter styles for LB, extracting font weights
export const getLetterWeights = () =>
  Object.values(letterStyles).reduce((acc, curr) => {
    if (!acc.find((el) => el === curr.fontWeight)) {
      acc.push(curr.fontWeight)
    }

    return acc
  }, [])

/*
  COLOR CALCULATION
  Using -Version 2 Hex- from this StackOverflow post:
  https://stackoverflow.com/questions/5560248/programmatically-lighten-or-darken-a-hex-color-or-rgb-and-blend-colors
*/
const shadeColor = (color, percent) => {
  const f = parseInt(color.slice(1), 16)
  const t = percent < 0 ? 0 : 255
  const p = percent < 0 ? percent * -1 : percent
  const R = f >> 16
  const G = (f >> 8) & 0x00ff
  const B = f & 0x0000ff
  return `#${(
    0x1000000 +
    (Math.round((t - R) * p) + R) * 0x10000 +
    (Math.round((t - G) * p) + G) * 0x100 +
    (Math.round((t - B) * p) + B)
  )
    .toString(16)
    .slice(1)}`
}

export const getLineColor = (lineState, repeated) => {
  const { isAnimating, isSubmitted } = lineStates
  let output = colors.pink

  if (lineState === isAnimating && repeated > 0) {
    output = shadeColor(colors.pink, -(repeated / 15))
  } else if (lineState === isSubmitted) {
    output = colors.palePink
  }

  return output
}

export const changeLetterState = (letterObj, letterState) => {
  let output = letterObj

  if (letterState === letterStateConfigs.active) {
    output = {
      ...letterObj,
      isActiveLetter: true,
      isInActiveWord: true,
    }
  } else if (letterState === letterStateConfigs.inactive) {
    output = {
      ...letterObj,
      isActiveLetter: false,
      isInActiveWord: true,
    }
  } else if (letterState === letterStateConfigs.animating) {
    output = {
      ...letterObj,
      isActiveLetter: false,
      isInActiveWord: false,
      isUsed: false,
      isAnimating: true,
    }
  }

  return output
}

/* 
  CANVAS SIZE SQUARE CALCULATION
*/
export const getSquareInfo = (
  canvasSize,
  squarePercentage = squareSize,
) => ({
  startingX: canvasSize * ((1 - squarePercentage) / 2),
  startingY: canvasSize * ((1 - squarePercentage) / 2),
  squareSize: canvasSize * squarePercentage,
  padding: canvasSize / 10,
})

export const getLineDashConfig = (canvasSize) => [
  canvasSize / 30,
  canvasSize / 38,
]
