import React, { useEffect } from 'react'

import { UserType } from 'foundation'
import { useUser } from 'user'
import useCTA from 'shared/marketing/useCTA'

import ModalCongratsShared from '../../../shared/ModalSystem/ModalCongrats'
import { navigateBack } from '../../../helpers/navigateBack'
import type {
  ButtonProps,
  ModalProps,
} from '../../../shared/ModalSystem/modalTypes'
import { getCTA } from '../../../shared/ModalSystem/helpers'
import handleHybridLoginClick from '../../../helpers/hybridLoginLinkClick'
import { handleSubscribe } from '../../../helpers/subscribeLinks'
import { trackImpression } from '../../../helpers/tracking'
import track from '../letterBoxedTracker'
import { trackLBInteraction } from '../analytics'

type Props = {
  data?: {
    submittedWords: string[]
    sides: string[]
    par: number
  }
  actions?: {
    unlockAndResetGame: () => void
  }
}

const size = 'medium'

const login = (
  updateUserType: (val: Partial<UserType>) => void,
  { showAnonText }: { showAnonText: boolean },
): ButtonProps => ({
  size,
  onClick: (e) => {
    trackLBInteraction('CONGRATS_LOGIN')
    handleHybridLoginClick(e, updateUserType)
  },
  text: 'Log In',
  largeText: true,
  key: 'login-btn-congrats',
  filled: false,
  descriptionText:
    (showAnonText && [
      {
        content: 'Log In to Play:',
        emphasized: true,
      },
    ]) ||
    undefined,
})

const subscribe = (
  updateUserType: (val: Partial<UserType>) => void,
  { showAnonText }: { showAnonText: boolean },
  buttonText: string,
): ButtonProps => {
  const descriptionText = [
    {
      content: 'No plays left today.',
      emphasized: true,
    },
  ]

  if (showAnonText) {
    descriptionText.push({
      content: 'Log In or Subscribe to Play:',
      emphasized: true,
    })
  }

  return {
    descriptionText,
    size,
    onClick: () => {
      trackLBInteraction('CONGRATS_SUBSCRIBE')
      handleSubscribe('letterboxedCutoffSettings', updateUserType)
    },
    text: buttonText,
    largeText: true,
    filled: false,
    key: 'sub-btn-congrats',
  }
}

const playAgainBtn = (
  onClick: (e: React.MouseEvent<Element, MouseEvent>) => void,
): ButtonProps => {
  return {
    onClick,
    size,
    text: 'Play again',
    filled: true,
    largeText: true,
    key: 'play-again-btn-congrats',
    descriptionText: undefined,
  }
}

const viewAllGamesBtn = (navigateBack: () => void): ButtonProps => {
  return {
    size,
    onClick: () => {
      navigateBack()
    },
    text: 'View all games',
    filled: false,
    largeText: true,
    key: 'view-all-games-btn-congrats',
  }
}

const DEFAULT_DATA = {
  submittedWords: [],
  sides: [],
  par: 0,
}

const DEFAULT_ACTIONS = {
  unlockAndResetGame: () => {},
}

const Congrats = ({
  close,
  isClosing,
  onRemove,
  data = DEFAULT_DATA,
  actions = DEFAULT_ACTIONS,
  meterResponse,
}: CongratsProps) => {
  const { submittedWords, par, sides } = data
  const { unlockAndResetGame } = actions
  const { granted } = meterResponse

  const { userType, updateUserType } = useUser()
  const { getButtonText } = useCTA()
  const buttonText = getButtonText()

  useEffect(() => {
    if (!userType.hasXwd) {
      trackImpression('hardpaywall', 'letter-boxed-no-plays-congrats')
    }

    trackImpression('congrats-modal', 'letter-boxed')
    trackImpression('next-game', 'letter-boxed', 'spelling-bee')
  }, [userType])

  const playAgainOnClick = () => {
    track.interaction.gameplay('play-again', par.toString(), null)
    unlockAndResetGame()
  }

  const getButtons = () => {
    let buttons: ButtonProps[]
    if (userType.hasXwd || granted) {
      buttons = [playAgainBtn(playAgainOnClick)]
    } else if (userType.isLoggedIn) {
      buttons = [
        subscribe(
          updateUserType,
          { showAnonText: false },
          buttonText,
        ),
      ]
    } else {
      buttons = [
        subscribe(updateUserType, { showAnonText: true }, buttonText),
        login(updateUserType, { showAnonText: false }),
      ]
    }

    if (window.isHybridWebView && window.NativeBridge) {
      buttons.push(viewAllGamesBtn(navigateBack))
    }

    return buttons
  }

  let btnProps = getButtons()
  useEffect(() => {
    btnProps = getButtons()
  }, [userType])

  const subheading =
    submittedWords.length <= par ? (
      <>
        You solved within{' '}
        <span className="btn__bold">
          {par} {par > 1 ? `words` : `word`}
        </span>
        .
      </>
    ) : (
      <>
        You solved in{' '}
        <span className="btn__bold">
          {submittedWords.length}{' '}
          {submittedWords.length > 1 ? `words` : `word`}
        </span>
        .
        {` 
        For an extra challenge, try to solve in `}
        {par} {par > 1 ? `words` : `word`} or less.
      </>
    )

  return (
    <ModalCongratsShared
      close={close}
      onRemove={onRemove}
      isClosing={isClosing}
      showStar={false}
      heading={
        submittedWords.length <= par
          ? 'Congratulations!'
          : 'Super solving!'
      }
      subheading={subheading}
      buttons={btnProps}
      cta={{
        game: getCTA('spelling-bee'),
      }}
      currentGame={{
        name: 'letter-boxed',
        sides,
        submittedWords,
      }}
    />
  )
}

export type CongratsProps = Props & ModalProps

export default Congrats
