import React from 'react'
import { mount } from 'enzyme'

import { useUser } from 'user'
import { grantReasons } from 'shared/meter/MeterTypes'

import { trackImpression } from '../../../../helpers/tracking'
import Congrats from '../Congrats'

const injectedState = {
  submittedWords: [],
  par: 1,
  sides: [],
}

const componentObject = {
  title: {
    dataTestid: 'header',
    text: 'Congratulations!',
    text2: 'Super solving!',
  },
  description: {
    dataTestid: 'subheading',
    text: 'You solved within 1 word.',
    text2:
      'You solved in 2 words. For an extra challenge, try to solve in 1 word or less.',
    text3: 'You solved within 2 words.',
    text4:
      'You solved in 3 words. For an extra challenge, try to solve in 2 words or less.',
  },
}

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('../../letterBoxedTracker.js', () => ({
  interaction: {
    gameplay: jest.fn(),
    general: jest.fn(),
  },
}))

jest.mock('../../../../helpers/tracking', () => ({
  trackImpression: jest.fn(),
}))

jest.mock('user', () => ({
  useUser: jest.fn(() => ({
    userType: { hasXwd: true },
    hasLoaded: true,
  })),
}))

type MockedUserHookReturn = {
  userType: {
    isLoggedIn?: boolean
    hasXwd?: boolean
  }
  hasLoaded: boolean
}

const setUser = (user: MockedUserHookReturn) => {
  // @ts-ignore mock with partial user return type
  useUser.mockReturnValue(user)
}

const mockTrackImpression = trackImpression as jest.MockedFunction<
  typeof trackImpression
>

describe(`LETTER BOXED: CONGRATS MODAL`, () => {
  beforeEach(() => {
    jest.resetAllMocks()
    setUser({
      userType: { hasXwd: true },
      hasLoaded: true,
    })
  })

  it('render a modal with respective H3 text', () => {
    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )
    expect(
      wrapper
        .find(`h1[data-testid="${componentObject.title.dataTestid}"]`)
        .text(),
    ).toBe(componentObject.title.text)
    const testData = {
      ...injectedState,
      submittedWords: Array.of('a', 'b', 'v', 'c', 'c', 'c'),
    }

    const wrapper2 = mount(
      <Congrats
        data={testData}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )
    expect(
      wrapper2
        .find(`h1[data-testid="${componentObject.title.dataTestid}"]`)
        .text(),
    ).toBe(componentObject.title.text2)
  })

  it('should render a modal with respective congrats text', () => {
    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )

    expect(
      wrapper
        .find(
          `p[data-testid="${componentObject.description.dataTestid}"]`,
        )
        .text()
        .replace(/\s\s+/g, ' '),
    ).toBe(componentObject.description.text)
    const wrapper2 = mount(
      <Congrats
        data={{
          ...injectedState,
          submittedWords: Array.of('a', 'b'),
        }}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )
    expect(
      wrapper2
        .find(
          `p[data-testid="${componentObject.description.dataTestid}"]`,
        )
        .text()
        .replace(/\s\s+/g, ' '),
    ).toBe(componentObject.description.text2)

    const wrapper3 = mount(
      <Congrats
        data={{
          ...injectedState,
          submittedWords: Array.of('a', 'b'),
          par: 2,
        }}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )
    expect(
      wrapper3
        .find(
          `p[data-testid="${componentObject.description.dataTestid}"]`,
        )
        .text()
        .replace(/\s\s+/g, ' '),
    ).toBe(componentObject.description.text3)

    const wrapper4 = mount(
      <Congrats
        data={{
          ...injectedState,
          submittedWords: Array.of('a', 'b', 'c'),
          par: 2,
        }}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )

    expect(
      wrapper4
        .find(
          `p[data-testid="${componentObject.description.dataTestid}"]`,
        )
        .text()
        .replace(/\s\s+/g, ' '),
    ).toBe(componentObject.description.text4)
  })

  it('should render a modal with a square canvas', () => {
    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )

    const actual = wrapper.find('canvas')
    const expected = 1
    expect(actual).toHaveLength(expected)
  })

  it('should render "Play again button" for subscribers', () => {
    setUser({
      userType: { hasXwd: true, isLoggedIn: true },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )

    expect(
      wrapper
        .find(`button[data-testid="btn-play-again"]`)
        .text()
        .replace(/\s\s+/g, ' '),
    ).toBe('Play again')
  })

  it('should render "Play again button" for anons on delta', () => {
    setUser({
      userType: { hasXwd: false, isLoggedIn: false },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{
          granted: true,
          grantReason: grantReasons.IP,
          loggedIn: false,
          isOffline: false,
        }}
      />,
    )

    expect(
      wrapper
        .find(`button[data-testid="btn-play-again"]`)
        .text()
        .replace(/\s\s+/g, ' '),
    ).toBe('Play again')
  })

  it('should render "Play again button" for regis on delta', () => {
    setUser({
      userType: { hasXwd: false, isLoggedIn: true },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{
          granted: true,
          grantReason: grantReasons.IP,
          loggedIn: true,
          isOffline: false,
        }}
      />,
    )

    expect(
      wrapper
        .find(`button[data-testid="btn-play-again"]`)
        .text()
        .replace(/\s\s+/g, ' '),
    ).toBe('Play again')
  })

  it('should NOT render "Play again button" for regular anons', () => {
    setUser({
      userType: { hasXwd: false, isLoggedIn: false },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{
          granted: false,
          grantReason: grantReasons.METER_LIMIT,
          loggedIn: false,
          isOffline: false,
        }}
      />,
    )

    expect(
      wrapper.find(`button[data-testid="btn-play-again"]`),
    ).toHaveLength(0)
  })

  it('should NOT render "Play again button" for regular regis', () => {
    setUser({
      userType: { hasXwd: false, isLoggedIn: true },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{
          granted: false,
          grantReason: grantReasons.METER_LIMIT,
          loggedIn: true,
          isOffline: false,
        }}
      />,
    )

    expect(
      wrapper.find(`button[data-testid="btn-play-again"]`),
    ).toHaveLength(0)
  })

  it('should fire a hardpaywall impression event for anons', () => {
    setUser({
      userType: { hasXwd: false, isLoggedIn: false },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )
    expect(mockTrackImpression).toHaveBeenCalledWith(
      'hardpaywall',
      'letter-boxed-no-plays-congrats',
    )
  })

  it('should fire a hardpaywall impression event for regis', () => {
    setUser({
      userType: { hasXwd: false, isLoggedIn: true },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )
    expect(mockTrackImpression).toHaveBeenCalledWith(
      'hardpaywall',
      'letter-boxed-no-plays-congrats',
    )
  })

  it('should NOT fire a hardpaywall impression event for subs', () => {
    setUser({
      userType: { hasXwd: true, isLoggedIn: true },
      hasLoaded: true,
    })

    const wrapper = mount(
      <Congrats
        data={injectedState}
        actions={{ unlockAndResetGame: () => {} }}
        close={() => {}}
        isClosing={false}
        onRemove={() => {}}
        meterResponse={{}}
      />,
    )
    expect(mockTrackImpression).not.toHaveBeenCalledWith(
      'hardpaywall',
      'letter-boxed-no-plays-congrats',
    )
  })
})
