import React from 'react'

import { UserType, mobileNavTools } from 'foundation'
import { MeterProperties } from 'shared/meter/MeterTypes'

import { momentSystem } from './index'

import track from '../letterBoxedTracker'
import { handleSubscribe } from '../../../helpers/subscribeLinks'
import handleHybridLoginClick from '../../../helpers/hybridLoginLinkClick'
import { trackLBInteraction } from '../analytics'

import { PuzzleProps } from './MomentWelcomeManager'

type MouseEventClick = (
  event:
    | MouseEvent
    | React.MouseEvent<HTMLButtonElement, MouseEvent>
    | React.MouseEvent<HTMLAnchorElement, MouseEvent>,
) => void

type Button = {
  text: string
  action: MouseEventClick | (() => void)
  icon?: string
  color?: string
}

type ButtonGroups = {
  buttons: Button[]
  header?: string | JSX.Element
  desktopButtonOrder?: string
}[]

const trackInteractionSubscribe = () => {
  const useBeacon = true
  track.interaction.custom(
    {
      module: {
        element: {
          name: 'letter-boxed',
          label: 'subscribe',
        },
      },
    },
    useBeacon,
  )
}

export const createButtons = (
  meterResponse: MeterProperties,
  puzzle: PuzzleProps,
  isCompleted: Boolean,
  hasXwd: Boolean,
  isLoggedIn: Boolean,
  updateUserType: (val: Partial<UserType>) => void,
  getButtonText: { (): string; (): any },
): ButtonGroups => {
  const subscribeMessage = 'Subscribe for Unlimited Play:'
  const { granted, grantReason } = meterResponse

  const playButton = {
    text: 'Play',
    action: () => {
      mobileNavTools.activate()
      momentSystem.transition('welcome', 'game')
      track.interaction.gameplay(
        'letter-boxed',
        'start-game',
        `${puzzle.par}`,
      )
    },
  }

  const subscribeButton = {
    text: getButtonText(),
    color: 'secondary',
    action: () => {
      handleSubscribe('letterBoxedCard', updateUserType)
      trackInteractionSubscribe()
    },
  }

  const loginButton = {
    text: 'Log In',
    color: 'secondary',
    action: (
      event:
        | MouseEvent
        | React.MouseEvent<HTMLButtonElement, MouseEvent>
        | React.MouseEvent<HTMLAnchorElement, MouseEvent>
        | React.MouseEvent<Element, MouseEvent>,
    ) => {
      handleHybridLoginClick(event, updateUserType)
      trackLBInteraction('WELCOME')
    },
  }

  const subButtons: Button[] = [playButton]

  // Anon on Delta
  if (!isLoggedIn && grantReason === 'IP') {
    subButtons.push(loginButton, subscribeButton)
  }

  // Regi on Delta
  if (!hasXwd && isLoggedIn && grantReason === 'IP') {
    subButtons.push(subscribeButton)
  }

  // Subscriber - doesn't matter if on Delta
  if (hasXwd || granted) {
    return [
      {
        buttons: subButtons,
        desktopButtonOrder: 'vertical',
      },
    ]
  }

  const nonSubButtons: Button[] = [subscribeButton]

  if (!isLoggedIn) {
    nonSubButtons.splice(0, 0, loginButton)
  }

  if (isLoggedIn && !isCompleted) {
    return [
      {
        header: '1 play left today',
        buttons: [playButton],
        desktopButtonOrder: 'vertical',
      },
      {
        header: subscribeMessage,
        buttons: nonSubButtons,
        desktopButtonOrder: 'vertical',
      },
    ]
  }

  if (isLoggedIn && isCompleted) {
    return [
      {
        header: (
          <>
            <span>0 plays left today.</span>
            <br />
            <span id="letter-boxed-subscribe-message">
              {subscribeMessage}
            </span>
          </>
        ),
        buttons: nonSubButtons,
        desktopButtonOrder: 'vertical',
      },
    ]
  }

  if (!isLoggedIn && !isCompleted) {
    return [
      {
        header: '1 play left today',
        buttons: [playButton],
        desktopButtonOrder: 'vertical',
      },
      {
        header: subscribeMessage,
        buttons: nonSubButtons,
        desktopButtonOrder: 'vertical',
      },
    ]
  }

  // not logged in, completed
  return [
    {
      header: (
        <>
          <span>0 plays left today.</span>
          <br />
          <span id="letter-boxed-subscribe-message">
            {subscribeMessage}
          </span>
        </>
      ),
      buttons: nonSubButtons,
      desktopButtonOrder: 'vertical',
    },
  ]
}

export default createButtons
