import React from 'react'
import { mount } from 'enzyme'
import { fireEvent, render, screen } from '@testing-library/react'

import { useUser } from 'user'
import { trackImpression } from 'foundation'
import { grantReasons } from 'shared/meter/MeterTypes'

import MomentWelcomeManager from '../MomentWelcomeManager'
import * as useAbra from '../../../../../shared/abra/useAbra'
import track from '../../letterBoxedTracker'

jest.mock('foundation', () => ({
  env: { account: 'https://myaccount.nytimes.com' },
  getGameData: () => {
    return { id: 0 }
  },
  trackImpression: jest.fn(),
  mobileNavTools: {
    activate: jest.fn(),
    deactivate: jest.fn(),
  },
}))

jest.mock('responsive', () => ({
  isMobile: true,
}))

jest.mock('../../moments', () => ({
  momentSystem: {
    transition: jest.fn(),
  },
}))

jest.mock('user', () => ({
  useUser: jest.fn(() => ({
    userType: { hasXwd: false, isLoggedIn: false },
    hasLoaded: true,
  })),
}))

jest.mock('../../letterBoxedTracker', () => ({
  interaction: {
    gameplay: jest.fn(),
    custom: jest.fn(),
  },
  impression: {
    custom: jest.fn(),
  },
  track: jest.fn(),
}))

describe('Welcome Moment Manager', () => {
  const { location } = window
  const testVariant = '1_icon'
  const props = {
    puzzle: {
      printDate: '2022-02-03',
      editor: 'Sam Ezersky',
      par: 6,
    },
    isCompleted: false,
    meterResponse: {
      granted: false,
      grantReason: grantReasons.METER_LIMIT,
      loggedIn: false,
      isOffline: false,
    },
  }

  afterEach(() => {
    window.location = location
  })

  beforeEach(() => {
    // @ts-ignore
    delete window.location
    window.location = {
      ...location,
      reload: jest.fn(),
      assign: jest.fn(),
    }

    jest.spyOn(window.location, 'reload').mockImplementation(() => {})
    jest.spyOn(window.location, 'assign').mockImplementation(() => {})

    jest.clearAllMocks()
  })

  it('Welcome moment: Analytics Tracking', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false },
      hasLoaded: true,
    })

    const wrapper = mount(<MomentWelcomeManager {...props} />)
    expect.assertions(5)

    expect(trackImpression).toHaveBeenCalledWith(
      'welcome-modal',
      'letter-boxed',
      ['log-in', 'subscribe'],
      '1',
    )

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    expect(buttons.length).toEqual(3)
    buttons.at(0).simulate('click')
    expect(track.interaction.gameplay).toHaveBeenCalledWith(
      'letter-boxed',
      'start-game',
      '6',
    )
    buttons.at(1).simulate('click')
    expect(track.interaction.custom).toHaveBeenCalledWith(
      {
        module: {
          element: {
            label: 'log-in',
            name: 'letter-boxed',
          },
          name: 'welcome',
        },
      },
      true,
    )
    buttons.at(2).simulate('click')
    expect(track.interaction.custom).toHaveBeenCalledWith(
      {
        module: {
          element: {
            label: 'subscribe',
            name: 'letter-boxed',
          },
        },
      },
      true,
    )
  })

  it('Welcome moment: Anon 1 play left', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false, isLoggedIn: false },
      hasLoaded: true,
    })

    const anon1Props = {
      puzzle: {
        printDate: '2022-02-03',
        editor: 'Sam Ezersky',
        par: 6,
      },
      isCompleted: false,
      meterResponse: {
        granted: false,
        grantReason: grantReasons.METER_LIMIT,
        loggedIn: false,
        isOffline: false,
      },
    }
    const wrapper = mount(<MomentWelcomeManager {...anon1Props} />)

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    const plays = wrapper.find('.header')

    expect(buttons.length).toEqual(3)
    expect(buttons.at(0).text()).toEqual('Play')
    expect(buttons.at(1).text()).toEqual('Log In')
    expect(buttons.at(2).text()).toEqual('Subscribe')

    expect(plays.at(0).text()).toEqual('1 play left today')
  })

  it('Welcome moment: Anon 0 plays left', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false, isLoggedIn: false },
      hasLoaded: true,
    })

    const anon0Props = {
      puzzle: {
        printDate: '2022-02-03',
        editor: 'Sam Ezersky',
        par: 6,
      },
      isCompleted: true,
      meterResponse: {
        granted: false,
        grantReason: grantReasons.METER_LIMIT,
        loggedIn: false,
        isOffline: false,
      },
    }

    const wrapper = mount(<MomentWelcomeManager {...anon0Props} />)

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    const plays = wrapper.find('.header')

    expect(buttons.length).toEqual(2)
    expect(buttons.at(0).text()).toEqual('Log In')
    expect(buttons.at(1).text()).toEqual('Subscribe')

    expect(plays.at(0).text()).toEqual(
      '0 plays left today.Subscribe for Unlimited Play:',
    )
  })

  it('Welcome moment: Anon Infinite plays left', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false, isLoggedIn: false },
      hasLoaded: true,
    })

    const anonInfProps = {
      puzzle: {
        printDate: '2022-02-03',
        editor: 'Sam Ezersky',
        par: 6,
      },
      isCompleted: true,
      meterResponse: {
        granted: true,
        grantReason: grantReasons.IP,
        loggedIn: false,
        isOffline: false,
      },
    }

    const wrapper = mount(<MomentWelcomeManager {...anonInfProps} />)

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    const plays = wrapper.find('.header')

    expect(buttons.length).toEqual(3)
    expect(buttons.at(0).text()).toEqual('Play')
    expect(buttons.at(1).text()).toEqual('Log In')
    expect(buttons.at(2).text()).toEqual('Subscribe')
  })

  it('Welcome moment: Regi 1 play left', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false, isLoggedIn: true },
      hasLoaded: true,
    })

    const regi1Props = {
      puzzle: {
        printDate: '2022-02-03',
        editor: 'Sam Ezersky',
        par: 6,
      },
      isCompleted: false,
      meterResponse: {
        granted: false,
        grantReason: grantReasons.METER_LIMIT,
        loggedIn: true,
        isOffline: false,
      },
    }
    const wrapper = mount(<MomentWelcomeManager {...regi1Props} />)

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    const plays = wrapper.find('.header')

    expect(buttons.length).toEqual(2)
    expect(buttons.at(0).text()).toEqual('Play')
    expect(buttons.at(1).text()).toEqual('Subscribe')

    expect(plays.at(0).text()).toEqual('1 play left today')
  })

  it('Welcome moment: Regi 0 plays left', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false, isLoggedIn: true },
      hasLoaded: true,
    })

    const regi0Props = {
      puzzle: {
        printDate: '2022-02-03',
        editor: 'Sam Ezersky',
        par: 6,
      },
      isCompleted: true,
      meterResponse: {
        granted: false,
        grantReason: grantReasons.METER_LIMIT,
        loggedIn: true,
        isOffline: false,
      },
    }

    const wrapper = mount(<MomentWelcomeManager {...regi0Props} />)

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    const plays = wrapper.find('.header')

    expect(buttons.length).toEqual(1)
    expect(buttons.at(0).text()).toEqual('Subscribe')

    expect(plays.at(0).text()).toEqual(
      '0 plays left today.Subscribe for Unlimited Play:',
    )
  })

  it('Welcome moment: Regi Infinite plays left', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false, isLoggedIn: true },
      hasLoaded: true,
    })

    const regiInfProps = {
      puzzle: {
        printDate: '2022-02-03',
        editor: 'Sam Ezersky',
        par: 6,
      },
      isCompleted: true,
      meterResponse: {
        granted: true,
        grantReason: grantReasons.IP,
        loggedIn: true,
        isOffline: false,
      },
    }

    const wrapper = mount(<MomentWelcomeManager {...regiInfProps} />)

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    const plays = wrapper.find('.header')

    expect(buttons.length).toEqual(2)
    expect(buttons.at(0).text()).toEqual('Play')
    expect(buttons.at(1).text()).toEqual('Subscribe')
  })

  it('Welcome moment: subscribed', () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: true, isLoggedIn: true },
      hasLoaded: true,
    })

    const subProps = {
      puzzle: {
        printDate: '2022-02-03',
        editor: 'Sam Ezersky',
        par: 6,
      },
      isCompleted: true,
      meterResponse: {
        granted: true,
        grantReason: grantReasons.SUBSCRIBER,
        loggedIn: true,
        isOffline: false,
      },
    }

    const wrapper = mount(<MomentWelcomeManager {...subProps} />)

    const buttons = wrapper
      .find('.pz-moment__button-wrapper')
      .children()

    expect(buttons.length).toEqual(1)
    expect(buttons.at(0).text()).toEqual('Play')
  })

  it('shows app banner on mobile web', () => {
    window.isHybridWebView = false

    // @ts-ignore
    jest.spyOn(useAbra, 'default').mockImplementationOnce(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
      hasAbraLoaded: true,
      tests: {},
    }))

    const wrapper = mount(<MomentWelcomeManager {...props} />)

    const downloadCTAText = wrapper.find({
      children: 'Play Letter Boxed and more in the NYT Games app.',
    })

    expect(downloadCTAText).toHaveLength(1)
  })

  it('does not render app banner in newsreader', () => {
    window.newsreaderAppPlatform = 'ios'

    // @ts-ignore
    jest.spyOn(useAbra, 'default').mockImplementationOnce(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
      hasAbraLoaded: true,
      tests: {},
    }))

    const wrapper = mount(<MomentWelcomeManager {...props} />)

    const downloadCTAText = wrapper.find({
      children: 'Play Letter Boxed and more in the NYT Games app.',
    })

    expect(downloadCTAText).toHaveLength(0)
  })

  it('does not render app banner in hybrid', () => {
    window.isHybridWebView = true

    // @ts-ignore
    jest.spyOn(useAbra, 'default').mockImplementationOnce(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
      hasAbraLoaded: true,
      tests: {},
    }))

    const wrapper = mount(<MomentWelcomeManager {...props} />)

    const downloadCTAText = wrapper.find({
      children: 'Play Letter Boxed and more in the NYT Games app.',
    })

    expect(downloadCTAText).toHaveLength(0)
  })
})
