import { createStore, applyMiddleware } from 'redux'
import thunk from 'redux-thunk'

import { puzzleContainerReducer, initialState } from '../reducer'
import * as actions from '../actions'

const prepareStore = (state) => {
  const store = createStore(
    puzzleContainerReducer,
    state,
    applyMiddleware(thunk),
  )

  return { store, dispatch: store.dispatch }
}

describe('LETTER BOXED: Puzzle Container Reducer', () => {
  const reducerState = {
    ...initialState,
    sides: ['ADR', 'MEO', 'BXU', 'ITS'],
  }

  let store
  let dispatch

  beforeEach(() => {
    ;({ store, dispatch } = prepareStore(reducerState))
  })

  it('should return the initial state', () => {
    expect(puzzleContainerReducer(undefined, {})).toEqual(
      initialState,
    )
  })

  it('should return a store state equal to initial state', () => {
    expect(store.getState()).toEqual(initialState)
  })

  it('unlocks game on unlock and reset game', () => {
    dispatch(actions.addLetter('T'))
    dispatch(actions.addLetter('O'))
    dispatch(actions.addLetter('T'))
    dispatch(actions.addLetter('E'))

    expect(store.getState()).toEqual({
      ...initialState,
      activeLetter: 'E',
      activeWord: 'TOTE',
    })

    dispatch(actions.openModal('foobar'))

    expect(store.getState()).toEqual({
      ...initialState,
      activeLetter: 'E',
      activeWord: 'TOTE',
      modalSystem: { ...initialState.modalSystem, modal: 'foobar' },
      isLocked: true,
    })

    dispatch(actions.unlockAndResetGame())

    expect(store.getState()).toEqual({
      ...initialState,
      isLocked: false,
    })
  })

  it('locks game on opening modal', () => {
    dispatch(actions.openModal('foobar'))

    expect(store.getState()).toEqual({
      ...initialState,
      modalSystem: { ...initialState.modalSystem, modal: 'foobar' },
      isLocked: true,
    })
  })

  it('unlocks game on closing modal', () => {
    dispatch(actions.openModal('foobar'))
    expect(store.getState()).toEqual({
      ...initialState,
      modalSystem: { ...initialState.modalSystem, modal: 'foobar' },
      isLocked: true,
    })

    dispatch(actions.closeModal('foobar'))

    expect(store.getState()).toEqual({
      ...initialState,
      modalSystem: {
        ...initialState.modalSystem,
        isClosing: true,
        modal: undefined,
      },
      isLocked: false,
    })
  })

  it('should return a store with a success message state', () => {
    dispatch(actions.showSuccessMessage('test'))

    expect(store.getState()).toEqual({
      ...initialState,
      message: { isError: false, value: 'test' },
      previousSuccess: true,
    })
  })
})
