import { createStore, applyMiddleware } from 'redux'
import thunk from 'redux-thunk'

import { puzzleContainerReducer, initialState } from '../reducer'
import * as actions from '../actions'

const reducerState = {
  ...initialState,
  sides: ['ADR', 'MEO', 'BXU', 'ITS'],
}

const prepareStore = (state) => {
  const store = createStore(
    puzzleContainerReducer,
    state,
    applyMiddleware(thunk),
  )

  return { store, dispatch: store.dispatch }
}

describe('Puzzle Container Reducer Thunks', () => {
  let store
  let dispatch
  let getPuzzleState
  beforeEach(() => {
    store = createStore(
      puzzleContainerReducer,
      reducerState,
      applyMiddleware(thunk),
    )

    dispatch = store.dispatch

    getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })
  })

  it('should return a store state updated with an new active word and letter', () => {
    const letter = 'A'
    actions.addLetter(letter)(dispatch, store.getState)

    expect(store.getState()).toEqual({
      ...initialState,
      activeLetter: letter,
      activeWord: letter,
    })
  })

  it('should return a store state updated with a single letter due to the second failing validation', () => {
    const letter = 'B'

    actions.addLetter(letter)(dispatch, store.getState)
    actions.addLetter(letter)(dispatch, store.getState)

    expect(store.getState()).toEqual({
      ...initialState,
      activeLetter: letter,
      activeWord: letter,
    })
  })

  it('should return a store state updated with two letter active word', () => {
    const letter = 'A'
    const letter2 = 'B'

    actions.addLetter(letter)(dispatch, store.getState)
    actions.addLetter(letter2)(dispatch, store.getState)

    expect(store.getState()).toEqual({
      ...initialState,
      activeLetter: letter2,
      activeWord: letter + letter2,
    })
  })

  it('should return a store state updated with a single active word and letter', () => {
    const letter = 'A'
    const letter2 = 'B'

    actions.addLetter(letter)(dispatch, store.getState)
    actions.addLetter(letter2)(dispatch, store.getState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.removeLetter()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...initialState,
      activeLetter: letter,
      activeWord: letter,
    })
  })

  it('should return a letter not found action', () => {
    const letter = 'Z'

    actions.addLetter(letter)(dispatch, store.getState)

    expect(
      actions.addLetter(letter)(dispatch, store.getState),
    ).toEqual({
      payload: letter,
      type: 'LetterBoxed/LETTER_NOT_FOUND',
    })
  })

  it('should return an updated state with an new active letter and word through the handleTextChange fx', () => {
    const key = 'A'
    const command = { key, preventDefault: () => {} }

    actions.handleTextChange(command)(dispatch, getPuzzleState)

    expect(store.getState().activeLetter).toEqual(key)
    expect(store.getState().activeWord).toEqual(key)
  })

  it('should return a store state should continue to have an empty active word/letter', () => {
    actions.removeLetter()(dispatch, getPuzzleState)
    expect(store.getState()).toEqual({ ...initialState })
  })

  it('should remove a letter from active letter + word', () => {
    const removeWordState = {
      ...reducerState,
      activeWord: 'RED',
      activeLetter: 'D',
    }

    const { store, dispatch } = prepareStore(removeWordState)

    actions.removeLetter()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...removeWordState,
      activeWord: 'RE',
      activeLetter: 'E',
    })
  })

  it('should remove the last letter from active letter + word', () => {
    const removeWordState = {
      ...reducerState,
      activeWord: 'D',
      activeLetter: 'D',
    }

    const { store, dispatch } = prepareStore(removeWordState)

    actions.removeLetter()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...removeWordState,
      activeWord: '',
      activeLetter: '',
    })
  })

  it('should remove the last letter from active letter + word and move to the next submitted word', () => {
    const removeWordState = {
      ...reducerState,
      activeWord: 'M',
      activeLetter: 'M',
      submittedWords: ['EAT', 'TRIM'],
    }

    const { store, dispatch } = prepareStore(removeWordState)

    actions.removeLetter()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...removeWordState,
      activeWord: 'TRIM',
      activeLetter: 'M',
      submittedWords: ['EAT'],
    })
  })

  it('should reset the game', () => {
    const resetGameState = {
      ...reducerState,
      activeWord: 'SOUR',
      activeLetter: 'S',
      submittedWords: ['MIXED', 'DIBS'],
    }

    const { store, dispatch } = prepareStore(resetGameState)
    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.unlockAndResetGame()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual(initialState)
  })

  it('should unlock and reset the game', () => {
    const modifiedGameState = {
      ...reducerState,
      activeWord: 'SOUR',
      activeLetter: 'S',
      submittedWords: ['MIXED', 'DIBS'],
      isLocked: true,
    }

    const { store, dispatch } = prepareStore(modifiedGameState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.unlockAndResetGame()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual(initialState)
  })

  it('should add word to word list', () => {
    const addWordState = {
      ...reducerState,
      activeWord: 'RED',
      activeLetter: 'D',
      submittedWords: [],
    }
    const { store, dispatch } = prepareStore(addWordState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.addWord()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...addWordState,
      activeWord: 'D',
      activeLetter: 'D',
      submittedWords: ['RED'],
    })
  })

  it('should not add word to word list and throw a too short message', () => {
    const addWordState = {
      ...reducerState,
      activeWord: 'RE',
      activeLetter: 'E',
    }
    const { store, dispatch } = prepareStore(addWordState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.submitWord()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...addWordState,
      message: { isError: true, value: 'Too short' },
    })
  })

  it('should not add word to word list and throw a not a word message', () => {
    const addWordState = {
      ...reducerState,
      activeWord: 'REAXDFAS',
      activeLetter: 'S',
    }
    const { store, dispatch } = prepareStore(addWordState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.submitWord()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...addWordState,
      message: { isError: true, value: 'Not a word' },
    })
  })

  it('should not add word to word list and throw a too short message, then add a word', () => {
    const shortWordState = {
      ...reducerState,
      activeWord: 'RE',
      activeLetter: 'E',
      dictionary: ['RED'],
    }

    const { store, dispatch } = prepareStore(shortWordState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.submitWord()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...shortWordState,
      message: { isError: true, value: 'Too short' },
    })

    actions.addLetter('D')(dispatch, store.getState)
    actions.submitWord()(dispatch, getPuzzleState)
    actions.addWord()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...shortWordState,
      activeWord: 'D',
      activeLetter: 'D',
      submittedWords: ['RED'],
    })
  })

  it('should submit a word', () => {
    const addWordState = {
      ...reducerState,
      activeWord: 'ARE',
      activeLetter: 'E',
      dictionary: ['ARE'],
    }
    const { store, dispatch } = prepareStore(addWordState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })
    actions.submitWord()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...addWordState,
      activeWord: 'ARE',
      activeLetter: 'E',
      animating: true,
    })
  })

  it('should flag the game as complete when all twelve letters are used', () => {
    const modifiedGameState = {
      ...reducerState,
      activeWord: 'AMBIDEXTROUS',
      activeLetter: 'S',
      submittedWords: [],
    }

    const { store, dispatch } = prepareStore(modifiedGameState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.addWord()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual({
      ...reducerState,
      activeWord: 'S',
      activeLetter: 'S',
      submittedWords: ['AMBIDEXTROUS'],
      isCompleted: true,
    })
  })

  it('should reset the game after complete', () => {
    const modifiedGameState = {
      ...reducerState,
      activeWord: 'AMBIDEXTROUS',
      activeLetter: 'S',
      submittedWords: [],
    }

    const { store, dispatch } = prepareStore(modifiedGameState)

    const getPuzzleState = () => ({
      puzzleContainer: store.getState(),
    })

    actions.addWord()(dispatch, getPuzzleState)
    expect(store.getState()).toEqual({
      ...reducerState,
      activeWord: 'S',
      activeLetter: 'S',
      submittedWords: ['AMBIDEXTROUS'],
      isCompleted: true,
    })

    actions.unlockAndResetGame()(dispatch, getPuzzleState)

    expect(store.getState()).toEqual(reducerState)
  })
})
