import { wait } from 'helpers/gameUtils'

import * as constants from '../../containers/PuzzleContainer/constants'
import {
  validateLetter,
  validateWord,
  isPuzzleLocked,
  isPuzzleComplete,
  validateWordLength,
} from '../../containers/PuzzleContainer/utils'

import { selectPuzzleData } from './selectors'

/*
 * letterNotFound
 */
export function letterNotFound(payload) {
  return {
    type: constants.LETTER_NOT_FOUND,
    payload,
  }
}

/*
 * addLetter
 */
export function addLetter(payload) {
  return (dispatch, getState) => {
    const state = getState()
    if (isPuzzleLocked(state)) return

    if (validateLetter(state, payload)) {
      return dispatch({
        type: constants.ADD_LETTER,
        payload,
      })
    }

    return dispatch(letterNotFound(payload))
  }
}

/*
 * lockPuzzleForNonSubs
 */
export const lockPuzzleForNonSubs = () => (dispatch) => {
  dispatch({
    type: constants.PREVENT_MOVES,
  })
}

/*
 * removeLetter
 */
export const removeLetter = () => (dispatch, getState) => {
  const state = getState()
  const puzzleLocked = isPuzzleLocked(state)

  if (
    !puzzleLocked &&
    state.puzzleContainer.activeWord.length === 1 &&
    state.puzzleContainer.submittedWords.length === 0
  ) {
    dispatch({
      type: constants.UNLOCK_AND_RESET_GAME,
    })
    return
  }

  if (!puzzleLocked) {
    dispatch({
      type: constants.REMOVE_LETTER,
    })
  }
}

/*
 * clearErrorMessage
 */
export const clearErrorMessage = () => ({
  type: constants.CLEAR_MESSAGE,
})

/*
 * showErrorMessage
 */
export const showErrorMessage = (payload) => ({
  type: constants.SHOW_ERROR_MESSAGE,
  payload,
})

/*
 * showSuccessMessage
 */
export const showSuccessMessage = (payload) => ({
  type: constants.SHOW_SUCCESS_MESSAGE,
  payload,
})

/*
 * showMessage
 */
export const showMessage =
  (msg, isError = false) =>
  (dispatch, getState) => {
    if (isError) {
      dispatch(showErrorMessage(msg))
    } else {
      dispatch(showSuccessMessage(msg))
    }

    const duration = isError ? 1200 : 800 // These durations correspond to the animation durations in animations.scss
    wait(duration).then(() => {
      const { value, isError } = getState().puzzleContainer.message

      if (value && isError) {
        dispatch(clearErrorMessage())
      }
    })
  }

/*
 * submitWord
 */
export const submitWord = () => (dispatch, getState) => {
  const puzzleState = selectPuzzleData(getState())

  if (!isPuzzleLocked(puzzleState)) {
    if (!validateWordLength(puzzleState.activeWord)) {
      dispatch(showMessage('Too short', true))
    } else if (!validateWord(puzzleState)) {
      dispatch(showMessage('Not a word', true))
    } else {
      if (puzzleState.message.value) {
        dispatch(clearErrorMessage())
      }

      dispatch({ type: constants.ANIMATE_START })
    }
  }
}

/*
 * handleTextChange
 */
export const handleTextChange = (e) => (dispatch, getState) => {
  if (isPuzzleLocked(getState())) return

  const letter = e.key.toUpperCase()

  if (letter === constants.BACKSPACE) {
    e.preventDefault()
    dispatch(removeLetter())
  } else if (letter === constants.SUBMIT) {
    e.preventDefault()
    dispatch(submitWord())
  } else {
    dispatch(addLetter(letter))
  }
}

/*
 * completeGame
 */
export const completeGame = () => ({
  type: constants.COMPLETE_GAME,
})

/*
 * endAnimation
 */
export const endAnimation = () => ({
  type: constants.ANIMATE_END,
})

/*
 * addWord
 */
export const addWord = () => (dispatch, getState) => {
  dispatch(endAnimation())
  dispatch({ type: constants.ADD_WORD })

  const puzzleState = selectPuzzleData(getState())

  if (isPuzzleComplete(puzzleState)) {
    dispatch(completeGame())
  }
}

/*
 * unlockAndResetGame
 */
export const unlockAndResetGame = () => (dispatch) => {
  dispatch({
    type: constants.UNLOCK_AND_RESET_GAME,
  })
}

/*
 * openModal
 */
export const openModal = (modal) => ({
  type: constants.OPEN_MODAL,
  modal,
})

/*
 * closeModal
 */
export const closeModal = () => ({ type: constants.CLOSE_MODAL })

/*
 * removeModal
 */
export const removeModal = () => ({ type: constants.REMOVE_MODAL })
