import React from 'react'
import { fireEvent, render, screen } from '@testing-library/react'
import MockDate from 'mockdate'
import { addHours } from 'date-fns'

import Banner from './Banner'

describe('<Banner />', () => {
  beforeAll(() => jest.useFakeTimers())
  afterAll(() => jest.useRealTimers())

  afterEach(() => {
    window.localStorage.clear()
    MockDate.reset()
  })

  const title = 'Test Banner Title'
  const assertBannerExists = () => {
    expect(screen.queryByText(title)).toBeTruthy()
  }

  const assertBannerNotExists = () => {
    expect(screen.queryByText(title)).toBeFalsy()
  }

  const dismissBanner = () => {
    const closeButton = screen.getByText('Close')
    fireEvent.click(closeButton)
    jest.advanceTimersByTime(401)
  }

  const renderBanner = (lsKey?: string, onDismiss?: () => void) => {
    return render(
      <Banner
        title={title}
        icon="icon"
        message="testing this shared banner"
        localStorageKey={lsKey}
        onDismiss={onDismiss}
      />,
    )
  }

  it('should show up by default', () => {
    renderBanner()
    assertBannerExists()
  })

  it('should disappear when dismissed', () => {
    renderBanner()
    dismissBanner()
    assertBannerNotExists()
  })

  it('should reappear if remounted, when no local storage is used', () => {
    renderBanner()
    dismissBanner()
    assertBannerNotExists()

    renderBanner()
    assertBannerExists()
  })

  it('should stay hidden if remounted, when local storage is used', () => {
    renderBanner('banner-key')
    dismissBanner()
    assertBannerNotExists()

    renderBanner('banner-key')
    assertBannerNotExists()
  })

  it('should reappear if remounted, when a diff local storage key is used', () => {
    renderBanner('banner-key')
    dismissBanner()
    assertBannerNotExists()

    renderBanner('banner-key-2')
    assertBannerExists()
  })

  it('should reappear after expiration time', () => {
    MockDate.set(1677512788787) // Feb 27 2023 9:46 AM

    const onDismiss = () => {
      const inThreeHours = addHours(new Date(), 3) // Feb 27 2023 12:46 PM
      window.localStorage.setItem(
        'banner-key-3',
        JSON.stringify(inThreeHours),
      )
    }

    renderBanner('banner-key-3', onDismiss)
    dismissBanner()

    MockDate.set(1677526388787) // Feb 27 2023 1:33 PM
    renderBanner('banner-key-3')
    assertBannerExists()
  })

  it('should not reappear before expiration time', () => {
    MockDate.set(1677512788787) //  Feb 27 2023 9:46 AM

    const onDismiss = () => {
      const inThreeHours = addHours(new Date(), 3) // Feb 27 2023 12:46 PM
      window.localStorage.setItem(
        'banner-key-3',
        JSON.stringify(inThreeHours),
      )
    }

    renderBanner('banner-key-3', onDismiss)
    dismissBanner()

    MockDate.set(1677522788787) // Feb 27 2023 12:33 PM
    renderBanner('banner-key-3')
    assertBannerNotExists()
  })
})
