import React, { useEffect, useState } from 'react'
import { CSSTransition } from 'react-transition-group'
import cn from 'classnames'
import { isAfter } from 'date-fns'

import { getDateOverride } from '../../../server/views/components/Marketing/Sale/functions'
import Icon from '../../../shared/components/Icon/Icon'

import styles from './Banner.module.scss'

const saveBannerDismissed = (localStorageKey: string) => {
  try {
    // If a local storage key exists, save the dismissed
    // state to local storage
    if (localStorageKey) {
      window.localStorage.setItem(localStorageKey, 'true')
    }
  } catch (e) {
    console.warn(
      'Failed to save banner visibility status. Error: ',
      e,
    )
    return null
  }
}

const getBannerDismissed = (localStorageKey: string) => {
  if (!localStorageKey) {
    return false
  }

  try {
    // Get from local storage whether banner was dismissed
    const ls = window.localStorage.getItem(localStorageKey)
    const isDismissed = ls && JSON.parse(ls)

    // If dismissed, return true
    return isDismissed
  } catch (e) {
    console.warn(
      'Failed to read banner visibility status. Error: ',
      e,
    )
    return null
  }
}

/**
 * Banner component displays a banner at the bottom
 * of a page with an icon, title, and an "x" to dismiss the banner.
 *
 * When dismissed, the component will write to local storage
 * and will remember that choice.
 *
 * The component supports two different layouts one with
 * and without an optional message.
 */

type BannerProps = {
  title: string | null
  message?: string | null
  icon: string | null
  localStorageKey?: string | null
  ctaLink?: string | null
  newTab?: boolean
  onClickLink?: () => void
  onDismiss?: () => void
  onVisible?: () => void
}

const Banner = ({
  title,
  icon,
  message,
  localStorageKey,
  ctaLink,
  newTab,
  onClickLink,
  onDismiss,
  onVisible,
}: BannerProps) => {
  // State
  const [showMessage, setShowMessage] = useState(false)

  useEffect(() => {
    // Find out whether user has already dismissed
    // the banner by checking local storage
    const isDismissed = getBannerDismissed(localStorageKey ?? '')

    // Check type of local storage key, if not boolean, evaluate expiration timestamp
    if (typeof isDismissed === 'boolean') {
      if (!isDismissed) setShowMessage(true)
    } else {
      const dateOverride = getDateOverride(window.location.href)
      const now =
        (dateOverride && new Date(dateOverride)) || new Date()

      if (isAfter(now, new Date(isDismissed))) setShowMessage(true)
    }
  }, [])

  useEffect(() => {
    if (showMessage && onVisible) {
      onVisible()
    }
  }, [showMessage, onVisible])

  const banner = (
    <div className={styles.banner} aria-live="polite">
      <div className={styles.bannerClose}>
        <button
          className={styles.iconButton}
          type="button"
          onClick={() => {
            setShowMessage(false)
            saveBannerDismissed(localStorageKey ?? '')
            if (onDismiss) {
              onDismiss()
            }
          }}
        >
          <span className="visually-hidden">Close</span>
          <Icon icon="close" />
        </button>
      </div>
      <div
        className={cn(styles.bannerInformation, {
          [styles.bannerMessageIcon]: message,
        })}
        aria-label="Information"
      >
        <Icon icon={icon ?? ''} />
        <div
          className={cn(styles.bannerBody, {
            [styles.clickArea]: !message,
          })}
        >
          <h3 className={message ? styles.bannerMessageTitle : ''}>
            {ctaLink ? (
              <a
                href={ctaLink ?? ''}
                target={newTab ? '_blank' : '_self'}
                rel="noopener noreferrer"
                onClick={() => {
                  if (onClickLink) {
                    onClickLink()
                  }
                }}
              >
                {title}
              </a>
            ) : (
              title
            )}
          </h3>
          {message && <p>{message}</p>}
        </div>
      </div>
    </div>
  )

  return (
    <CSSTransition
      in={showMessage}
      unmountOnExit
      timeout={400}
      classNames={{
        exit: styles.exit,
        exitActive: styles.exitActive,
      }}
    >
      {banner}
    </CSSTransition>
  )
}

export default Banner
