import React from 'react'
import pt from 'prop-types'

import { track } from 'foundation'

import useAbra from '../../../shared/abra/useAbra'
import { trackImpression } from '../../helpers/tracking'
import { selectIsPastPuzzle } from '../../spelling-bee/js/store/game/game.selectors'
import { useSbSelector } from '../../spelling-bee/js/store/game/hooks'

import ConversionBanner from './ConversionBanner'

const ConversionBannerWrapper = ({
  gameType,
  subAbraTest,
  regiAbraTest,
  userType,
  isLocked,
}) => {
  const isSubscriber = userType.hasXwd
  const isNewsreaderApp = !!window.newsreaderAppPlatform
  const isGamesApp = !!window.gamesAppPlatform
  const abra = useAbra()
  const subVariant = abra.getVariant(subAbraTest)
  const regiVariant = abra.getVariant(regiAbraTest)
  const isSubGiftAllocated = isSubscriber && subVariant === '1_gift'
  const isRegiGiftAllocated =
    !isSubscriber && regiVariant === '1_gift'
  const isGiftAllocated = isSubGiftAllocated || isRegiGiftAllocated
  const isPastPuzzle = useSbSelector(selectIsPastPuzzle)

  const bannerUrl = `https://www.nytimes.com/subscription/games/${
    isGiftAllocated ? 'gift' : ''
  }?campaignId=${gameType === 'spelling-bee' ? '8URFH' : '89YHK'}`

  const handleImpression = () => {
    trackImpression(
      'dock', // module.name
      null, // module.region
      isSubscriber ? subAbraTest : regiAbraTest, // module.label
      isGiftAllocated ? 'gift' : 'subscribe', // module.context
    )
  }

  const setFreqCapCookie = () => {
    /* Set frequency cap cookie with expiration date one month from current date */
    const expiryDate = new Date()
    expiryDate.setMonth(expiryDate.getMonth() + 1)
    document.cookie = `standcon-conversion-banner=true; expires=${expiryDate.toGMTString()};`
  }

  const isFreqCapped = () => {
    const value = `; ${document.cookie}`
    const parts = value.split('; standcon-conversion-banner=')
    return parts.length === 2
  }

  const handleExpose = () => {
    isSubscriber
      ? abra.reportExposure(subAbraTest)
      : abra.reportExposure(regiAbraTest)
  }

  const fireInteractionEvent = (type) => {
    track('moduleInteraction', {
      module: {
        name: 'dock',
        label: isSubscriber ? subAbraTest : regiAbraTest,
        context: isGiftAllocated ? 'gift' : 'subscribe',
        element: {
          name: type === 'close' ? 'close-dock' : 'goto-subscribe',
          label: type === 'close' ? 'X' : '>',
          url: type === 'close' ? null : bannerUrl,
        },
      },
    })
  }

  const [closeBanner, setCloseBanner] = React.useState(false)

  const handleClose = (event) => {
    event.preventDefault()
    event.stopPropagation()
    setFreqCapCookie()

    setCloseBanner(true)

    fireInteractionEvent('close')
  }

  let banner = null

  // Show gift banner if allocated to '1_gift' and have not hit sub freq cap limit
  const isGiftBanner =
    (isSubGiftAllocated && !isFreqCapped()) || isRegiGiftAllocated

  if (isGiftBanner) {
    banner = (
      <ConversionBanner
        isGiftBanner
        gameType={gameType}
        fireInteractionEvent={fireInteractionEvent}
        handleClose={handleClose}
        isSubscriber={isSubscriber}
        bannerUrl={bannerUrl}
      />
    )
  }

  // Show subscribe banner if not a sub and allocated to '2_sub'
  const isSubscribeBanner = !isSubscriber && regiVariant === '2_sub'
  if (isSubscribeBanner) {
    banner = (
      <ConversionBanner
        fireInteractionEvent={fireInteractionEvent}
        handleClose={handleClose}
        gameType={gameType}
        bannerUrl={bannerUrl}
      />
    )
  }

  React.useEffect(() => {
    if (banner && !isLocked) {
      handleImpression()
    }

    if (!isLocked && (subVariant || regiVariant)) {
      handleExpose()
    }
  }, [isLocked])

  return (
    (!closeBanner &&
      !isNewsreaderApp &&
      !isGamesApp &&
      !isPastPuzzle &&
      banner) ||
    null
  )
}

ConversionBannerWrapper.propTypes = {
  gameType: pt.string.isRequired,
  subAbraTest: pt.string.isRequired,
  regiAbraTest: pt.string.isRequired,
  userType: pt.object.isRequired,
  isLocked: pt.bool.isRequired,
}

export default ConversionBannerWrapper
