import React from 'react'
import PT from 'prop-types'
import moment from 'moment'
import { withRouter } from 'react-router-dom'

import {
  noop,
  getUpdate,
  getRange,
  isDisabledMonth,
  updateUrl,
} from './utils'

export const DateSelector = ({
  earliestDate,
  history,
  match: { params },
  month,
  todaysDate,
  year,
}) => {
  const useMonths = typeof month !== 'undefined'
  const thisYear = todaysDate.year()
  const thisMonth = todaysDate.month()
  const earliestYear = earliestDate.year()
  const months = moment.months()
  const years = getRange(earliestYear, thisYear)

  const date = useMonths
    ? moment(`${month + 1}-${year}`, 'M-YYYY')
    : moment(`${year}`, 'YYYY')
  const nextDate = moment(date).add(1, useMonths ? 'months' : 'years')
  const prevDate = moment(date).subtract(
    1,
    useMonths ? 'months' : 'years',
  )
  const hasNext = todaysDate.isSameOrAfter(nextDate)
  const hasPrev = earliestDate.isSameOrBefore(prevDate)
  const next = hasNext
    ? () =>
        updateUrl(history, params, ...getUpdate(useMonths, nextDate))
    : noop

  const prev = hasPrev
    ? () =>
        updateUrl(history, params, ...getUpdate(useMonths, prevDate))
    : noop

  const today = hasNext
    ? () =>
        updateUrl(
          history,
          params,
          thisYear,
          useMonths ? thisMonth : undefined,
        )
    : noop

  const changeYear = (newYear) => {
    if (useMonths && newYear === earliestYear) {
      updateUrl(
        history,
        params,
        newYear,
        Math.max(month, earliestDate.month()),
      )
    } else if (useMonths && newYear === thisYear) {
      updateUrl(history, params, newYear, Math.min(month, thisMonth))
    } else if (useMonths) {
      updateUrl(history, params, newYear, month)
    } else {
      updateUrl(history, params, newYear)
    }
  }

  return (
    <div className="archive_date-selector-container">
      <button
        label="prev"
        className="archive_prev"
        onClick={prev}
        disabled={!hasPrev}
        type="button"
      />
      <div className="archive_dropdowns">
        {useMonths && (
          <select
            className="archive_dropdown"
            label="month"
            value={month}
            onChange={(event) =>
              updateUrl(
                history,
                params,
                year,
                parseInt(event.target.value, 10),
              )
            }
          >
            {months.map((m, monthIdx) => {
              const disabled = isDisabledMonth({
                earliestDate,
                monthIdx,
                todaysDate,
                year,
              })

              return (
                <option value={monthIdx} key={m} disabled={disabled}>
                  {m}
                </option>
              )
            })}
          </select>
        )}
        <select
          className="archive_dropdown"
          label="year"
          value={year}
          onChange={(event) =>
            changeYear(parseInt(event.target.value, 10))
          }
        >
          {years.map((y) => (
            <option value={y} key={y}>
              {y}
            </option>
          ))}
        </select>
      </div>
      <div className="archive_buttons-right">
        <button
          className="archive_today"
          onClick={today}
          disabled={!hasNext}
          type="button"
        >
          Today
        </button>
        <button
          label="next"
          className="archive_next"
          onClick={next}
          disabled={!hasNext}
          type="button"
        />
      </div>
    </div>
  )
}

DateSelector.propTypes = {
  earliestDate: PT.instanceOf(moment),
  history: PT.object,
  match: PT.shape({ params: PT.object }),
  month: PT.number,
  todaysDate: PT.instanceOf(moment),
  year: PT.number,
}

export default withRouter(DateSelector)
