import React from 'react'
import moment from 'moment'
import { shallow } from 'enzyme'

import { DateSelector } from './DateSelector'
import { updateUrl } from './utils'

jest.mock('./utils', () => ({
  ...jest.requireActual('./utils'),
  updateUrl: jest.fn(),
}))

describe('ARCHIVE VIEWER / DATE SELECTOR', () => {
  let fakeToday
  let defaultProps

  beforeEach(() => {
    jest.resetAllMocks()

    fakeToday = moment({ year: 2017, month: 9 })

    defaultProps = {
      earliestDate: moment('2000', 'YYYY'),
      history: { push: () => {} },
      match: { params: { type: 'daily' } },
      month: 4,
      todaysDate: fakeToday,
      year: 2015,
    }
  })

  test('renders years back to earliest date', () => {
    const selector = shallow(<DateSelector {...defaultProps} />)

    const oldest = selector.find({ value: 2000 })
    const tooOld = selector.find({ value: 1998 })

    expect(oldest).toHaveLength(1)
    expect(tooOld).toHaveLength(0)
  })

  test('only renders select for month if necessary', () => {
    const monthSelector = shallow(<DateSelector {...defaultProps} />)
    const yearSelector = shallow(
      <DateSelector {...defaultProps} month={undefined} />,
    )

    expect(monthSelector.find({ label: 'month' })).toHaveLength(1)
    expect(yearSelector.find({ label: 'month' })).toHaveLength(0)
  })

  test('renders 12 month options', () => {
    const selector = shallow(<DateSelector {...defaultProps} />)
    const monthOptions = selector
      .find({ label: 'month' })
      .find('option')

    expect(monthOptions).toHaveLength(12)
  })

  test('updating to earliest possible month if selected month is before earliest month', () => {
    /* EXAMPLE: April 2017 is the current selected date.
    User clicks the dropdown to go to 2000, which only
    has puzzles starting in June. The month should auto-update
    to June. */
    const dateSelector = shallow(
      <DateSelector
        {...defaultProps}
        earliestDate={moment('6-2000', 'MM-YYYY')}
      />,
    )

    const {
      history,
      match: { params },
    } = defaultProps

    const yearSelect = dateSelector.find('select').last()
    yearSelect.simulate('change', { target: { value: 2000 } })

    expect(updateUrl).toHaveBeenCalledWith(history, params, 2000, 5)
  })

  test('updating to current month if selected month is in the future', () => {
    /* EXAMPLE: April 2017 is the current selected date.
    User clicks the dropdown to go to 2021, which only
    has puzzles through February (because in our fake world,
    it's February, 2021). The month should auto-update to February. */
    const fakeToday = moment({ year: 2021, month: 1, date: 6 })

    const dateSelector = shallow(
      <DateSelector {...defaultProps} todaysDate={fakeToday} />,
    )

    const {
      history,
      match: { params },
    } = defaultProps
    const yearSelect = dateSelector.find('select').last()
    yearSelect.simulate('change', { target: { value: 2021 } })

    expect(updateUrl).toHaveBeenCalledWith(history, params, 2021, 1)
  })

  describe('next and previous buttons', () => {
    const fakeToday = moment({ year: 2021, month: 6, date: 1 })

    test('next is active if there are more years available', () => {
      const dateSelector = shallow(
        <DateSelector
          {...defaultProps}
          month={null}
          todaysDate={fakeToday}
        />,
      )

      expect(
        dateSelector.find({ label: 'next' }).props().disabled,
      ).toBe(false)
    })

    test('next is disabled if there are no more years available', () => {
      const dateSelector = shallow(
        <DateSelector
          {...defaultProps}
          year={2021}
          month={undefined}
          todaysDate={fakeToday}
        />,
      )

      expect(
        dateSelector.find({ label: 'next' }).props().disabled,
      ).toBe(true)
    })

    test('next is active if there are months available', () => {
      const dateSelector = shallow(
        <DateSelector {...defaultProps} todaysDate={fakeToday} />,
      )

      expect(
        dateSelector.find({ label: 'next' }).props().disabled,
      ).toBe(false)
    })

    test('previous is active if there are years available', () => {
      const dateSelector = shallow(
        <DateSelector
          {...defaultProps}
          month={undefined}
          earliestDate={moment('6-2000', 'MM-YYYY')}
        />,
      )

      expect(
        dateSelector.find({ label: 'prev' }).props().disabled,
      ).toBe(false)
    })

    test('previous is disabled if there are no more years available', () => {
      const dateSelector = shallow(
        <DateSelector
          {...defaultProps}
          month={undefined}
          year={2000}
          earliestDate={moment('6-2000', 'MM-YYYY')}
        />,
      )

      expect(
        dateSelector.find({ label: 'prev' }).props().disabled,
      ).toBe(true)
    })

    test('previous is active if there are months available', () => {
      const dateSelector = shallow(
        <DateSelector
          {...defaultProps}
          earliestDate={moment('6-2000', 'MM-YYYY')}
        />,
      )

      expect(
        dateSelector.find({ label: 'prev' }).props().disabled,
      ).toBe(false)
    })
  })
})
