import moment from 'moment'

import {
  getUpdate,
  getRange,
  isDisabledMonth,
  updateUrl,
} from './utils'

describe('UTILS: DateSelector', () => {
  describe('updateUrl', () => {
    let dummyHistory = []
    const history = { push: (val) => dummyHistory.push(val) }
    const params = { type: 'daily' }

    beforeEach(() => {
      dummyHistory = []
    })

    it('updates a year without a month', () => {
      updateUrl(history, params, 2005)
      expect(dummyHistory[0]).toEqual(
        '/crosswords/archive/daily/2005',
      )
    })

    it('updates year with month', () => {
      updateUrl(history, params, 2005, 4)
      expect(dummyHistory[0]).toEqual(
        '/crosswords/archive/daily/2005/05',
      )
    })

    it('updates with different params', () => {
      updateUrl(history, { type: 'mini' }, 2005, 4)
      expect(dummyHistory[0]).toEqual(
        '/crosswords/archive/mini/2005/05',
      )
    })

    it('defaults to daily if no params are provided', () => {
      updateUrl(history, null, 2015, 8)
      expect(dummyHistory[0]).toEqual(
        '/crosswords/archive/daily/2015/09',
      )
    })
  })

  describe('getRange', () => {
    it('returns the expected range in reverse', () => {
      const res = getRange(4, 10)
      expect(res).toEqual([10, 9, 8, 7, 6, 5, 4])
    })
  })

  describe('getUpdate', () => {
    const year = moment('2012')
    const month = moment('2012-05')

    it('gives a year back if useMonths is false', () => {
      const res = getUpdate(false, year)
      expect(res).toEqual([2012])
    })

    it('gives a year and month back if useMonths is true', () => {
      const res = getUpdate(true, month)
      expect(res).toEqual([2012, 4])
    })
  })

  describe('isDisabledMonth', () => {
    const fakeToday = moment({ year: 2017, month: 9 })

    test('returns true for future months', () => {
      const disabled = isDisabledMonth({
        earliestDate: moment('2000-07-01'),
        monthIdx: 11,
        todaysDate: fakeToday,
        year: 2017,
      })

      expect(disabled).toBe(true)
    })

    test('returns true for months older than the oldest month', () => {
      const disabled = isDisabledMonth({
        earliestDate: moment('2000-07-01'),
        monthIdx: 4,
        todaysDate: fakeToday,
        year: 2000,
      })

      expect(disabled).toBe(true)
    })

    test('returns false for valid months', () => {
      const disabled = isDisabledMonth({
        earliestDate: moment('2000-07-01'),
        monthIdx: 4,
        todaysDate: fakeToday,
        year: 2011,
      })

      expect(disabled).toBe(false)
    })
  })
})
