import React, {
  useState,
  useRef,
  ReactNode,
  ReactElement,
} from 'react'
import cn from 'classnames'

import { isMobile } from 'responsive'
import { trackModuleInteraction } from 'foundation'

import useClickOutside from '../hooks/use-click-outside'

type DropDownProps = {
  buttonLabel: string | ReactElement
  children: ReactNode | ReactNode[]
  removeArrow?: boolean
  customLabelClass?: string
  ariaLabel?: string
  trackData?: {
    context: number | string
    elementName: string
    elementLabel: string
    moduleLabel?: string
  }
}

/**
 * Reusable DropDown component.
 * Dropdown comes with an up/down arrow, a dropdown container
 * that displays a list of clickable items, an optional
 * icon that can be displayed next to a dropdown item, and
 * optional tracking for "drop down" click events.
 *
 * @param {string | ReactElement} buttonLabel  The text label or icon for the dropdown button
 * @param {array} children      An array of items to go in the dropdown. Can be of type
 *                              'button' or 'navigation' (a link).
 * @param {boolean} removeArrow removes the arrow for dropdown, but defaults to display it
 * @param {string} customLabelClass to add additional classes to the button label
 * @param {string} ariaLabel encouraged for use with icon buttons
 * @param {object} trackData Optional parameter for tracking drop-down clicks (NOT minimize clicks)
 * @returns {ReactElement}
 *
 */
const DropDown = ({
  buttonLabel,
  removeArrow = false,
  customLabelClass = '',
  trackData,
  ariaLabel,
  children,
}: DropDownProps) => {
  const [showMenu, setShowMenu] = useState(false)
  const helpMenu = useRef(null)
  const dropdownRef = useRef<HTMLDivElement>(null)

  useClickOutside(dropdownRef, () => {
    if (isMobile) {
      const button = document.getElementsByClassName(
        'pz-dropdown__button',
      )[0]
      // @ts-ignore
      button.style.backgroundColor = 'white'
    }
    setShowMenu(false)
  })

  const menuToggle = () => {
    if (isMobile) {
      const button = document.getElementsByClassName(
        'pz-dropdown__button',
      )[0]

      if (showMenu) {
        // @ts-ignore
        button.style.backgroundColor = 'white'
      } else {
        // @ts-ignore
        button.style.backgroundColor = '#ebebeb'
      }
    }
    setShowMenu(!showMenu)
  }

  const handleKeyboardMenu = (e: React.KeyboardEvent) => {
    if (e.key === 'Enter') {
      menuToggle()
    }
  }

  const arrowClasses = cn('pz-dropdown__arrow', {
    reverse: showMenu,
  })

  const trackDropDown = () => {
    if (trackData) {
      const { elementName, moduleLabel, elementLabel, context } =
        trackData
      trackModuleInteraction(
        'click',
        context,
        elementName,
        elementLabel,
        moduleLabel,
      )
    }
  }

  return (
    <div className="pz-dropdown" ref={dropdownRef}>
      <button
        type="button"
        onClick={() => {
          // Only track when expanding the menu, not when collapsing
          if (!showMenu) {
            trackDropDown()
          }
          menuToggle()
        }}
        onKeyDown={handleKeyboardMenu}
        className="pz-dropdown__button"
        aria-label={ariaLabel}
      >
        <span className={cn('pz-dropdown__label', customLabelClass)}>
          {buttonLabel}
        </span>
        {!removeArrow && <span className={arrowClasses} />}
      </button>
      {showMenu && (
        <nav ref={helpMenu} className="pz-dropdown__list">
          <ul>
            {/**
             * The menuToggle prop is passed to the child so
             * that the child can call it (to close the
             * menu) if desired.
             */}
            {React.Children.map(children, (child: ReactNode) => {
              if (child) {
                return React.cloneElement(child as ReactElement, {
                  menuToggle,
                })
              }
            })}
          </ul>
        </nav>
      )}
    </div>
  )
}

export default DropDown
