import React, { FunctionComponent } from 'react'
import cn from 'classnames'

import { trackClick } from 'foundation'

import { congratsSquareConfig } from '../../letter-boxed/js/previewSquareConfig'
import PreviewSquare from '../../letter-boxed/js/PreviewSquare'

import ModalWrapper from './ModalWrapper'
import { ButtonProps, ModalProps } from './modalTypes'

type LetterBoxedOptions = {
  submittedWords: string[]
  sides: string[]
}

type CurrentGameOptions =
  | {
      name: 'tiles'
    }
  | {
      name: 'sudoku'
    }
  | ({
      name: 'letter-boxed'
    } & LetterBoxedOptions)

type Props = {
  heading: string
  subheading: JSX.Element
  buttons: ButtonProps[]
  showStar: boolean
  currentGame: CurrentGameOptions
  cta: {
    game: {
      href: string
      name: string
      displayName: string
      subtitle: JSX.Element
      iconClass: string
    }
  }
  hideClose?: boolean
}

type ModalCongratsWrapperProps = Props & ModalProps

export const ModalCongrats: FunctionComponent<
  ModalCongratsWrapperProps
> = ({
  isClosing,
  onRemove,
  showStar,
  heading,
  subheading,
  buttons,
  cta,
  currentGame,
  close,
  hideClose = false,
}: ModalCongratsWrapperProps) => {
  const isLetterBoxed = currentGame.name === 'letter-boxed'
  let submittedWords: string[] = []
  let sides: string[] = []
  if (isLetterBoxed) {
    const letterBoxedOptions = currentGame as LetterBoxedOptions
    submittedWords = letterBoxedOptions.submittedWords
    sides = letterBoxedOptions.sides
  }

  return (
    <ModalWrapper
      bodyClassName="modal-congrats-body"
      close={close}
      isClosing={isClosing}
      onRemove={onRemove}
      hideCloseX={hideClose}
      ignoreBackgroundClick={hideClose}
    >
      <div className="xwd__modal--content">
        {showStar && <i className="su-modal__star" />}
        {isLetterBoxed && (
          <div className="top-image">
            <PreviewSquare
              solution={submittedWords}
              sides={sides}
              config={congratsSquareConfig}
            />
          </div>
        )}
        <h1 className="modal-system-header" data-testid="header">
          {heading}
        </h1>
        <p
          className={`modal-system-subheader${
            isLetterBoxed ? ' restrict-width' : ''
          }`}
          data-testid="subheading"
        >
          {subheading}
        </p>
        <div
          className="modal-congrats__actions"
          data-testid="actions"
        >
          {buttons.map((btn, i) => (
            <div key={btn.key} className="modal-congrats__action">
              {btn.descriptionText && (
                <div
                  className="btn__message"
                  data-testid={`action-message-${i + 1}`}
                >
                  {btn.descriptionText.map(
                    ({ content, emphasized }) => (
                      <p key={content}>
                        {emphasized ? (
                          <span className="btn__emphasized">
                            {content}
                          </span>
                        ) : (
                          { content }
                        )}
                      </p>
                    ),
                  )}
                </div>
              )}
              <button
                className={`pz-modal__button ${
                  btn.filled ? 'dark' : 'white'
                } ${btn.largeText ? 'lrg_text' : ''} ${btn.size}`}
                type="button"
                onClick={btn.onClick}
                data-testid={`btn-${btn.text
                  .replace(/\s+/g, '-')
                  .toLowerCase()}`}
              >
                {btn.text}
              </button>
            </div>
          ))}
          <a
            href={cta.game.href}
            onClick={() =>
              trackClick({
                name: currentGame.name,
                label: 'next-game',
                context: cta.game.name,
              })
            }
            className={cn(
              'modal-congrats__sb',
              'sb',
              'modal-card',
              `${currentGame.name}-modal-card`,
            )}
          >
            <div className="sb-content__link">
              <div
                className={`modal-card__icon ${cta.game.iconClass}`}
              />
              <div className="modal-card__info">
                <div className="modal-card__title">
                  {cta.game.displayName}
                </div>
                <div className="modal-card__text">
                  {cta.game.subtitle}
                </div>
              </div>
              <div className="modal-card__arrow" />
            </div>
          </a>
        </div>
      </div>
    </ModalWrapper>
  )
}
export default ModalCongrats
