import React from 'react'
import cn from 'classnames'

import Icon from '../../../shared/components/Icon/Icon'
import ModalButtons, {
  ModalButtonConfig,
} from '../../crossword/components/Modals/ModalButtons'

/*
  This Modal Wrapper component has a few concerns:
  --
  1) This component receives a close function (a way to close the modal),
  which serves as the default click behavior for buttons not explicitly
  passed a clickHandler.

  2) A modal by default is closed when a user clicks on the background
  (anything that isn't the modal itself). This component gives you the
  option to turn this default behavior off, by setting the
  ignoreBackgroundClick prop to true.

  3) This component freezes the scroll position on the page so that when
  the modal is opened and closed the page does not appear to have moved
  beneath it.

  4) This component also deals with the positioning of the modal.

  5) Making sure that the modal body is in focus

  6) Handling key down events: Pressing Enter or Escape
  should close the modal by default but these can be overridden

  7) Rendering any children

  8) Creating and rendering actual ModalButtons from the
  buttons array

  9) Deciding whether to display the close X or not
*/

/* eslint-disable react/require-default-props */
type ModalWrapperProps = {
  close: () => void
  children: React.ReactNode
  bodyClassName?: string
  buttons?: ModalButtonConfig[]
  buttonsWrapperClassName?: string
  handleKeyDown?: (key: string) => void
  hideCloseX?: boolean
  ignoreBackgroundClick?: boolean
  keyConfig?: Record<
    string,
    (e: React.KeyboardEvent, focusRef: HTMLDivElement | null) => void
  >
  footer?: React.ReactNode
  showHeader?: boolean
  closeLabel?: string | undefined
  isClosing: boolean
  onRemove: () => void
  noAnimateOpen?: boolean
  containerClassName?: string
}

class ModalWrapper extends React.Component<ModalWrapperProps> {
  focusRef: React.RefObject<HTMLDivElement>

  frozenFocus: Element | null

  constructor(props: ModalWrapperProps) {
    super(props)
    this.focusRef = React.createRef()
    this.frozenFocus = document.activeElement
  }

  // We should always stop close events from bubbling up;
  // this prevents unexpected behavior in parent components
  closeWithoutPropagation = (
    event: React.KeyboardEvent | React.MouseEvent,
  ) => {
    const { close } = this.props
    event.preventDefault()
    event.stopPropagation()
    close()
  }

  onKeyDown = (event: React.KeyboardEvent) => {
    const { handleKeyDown, keyConfig } = this.props
    const { key, ctrlKey, metaKey } = event

    if (handleKeyDown) {
      handleKeyDown(key)
    }

    if (ctrlKey || metaKey) return // Allow key combos (refresh etc.) to proceed

    const keyAction = keyConfig?.[key]

    if (keyAction) {
      return keyAction(event, this.focusRef.current)
    }

    if (['Enter', 'NumpadEnter', 'Escape'].includes(key)) {
      return this.closeWithoutPropagation(event)
    }
  }

  componentDidMount = () => {
    if (this.focusRef.current) {
      this.focusRef.current.focus({ preventScroll: true })
    }
  }

  componentWillUnmount = () => {
    if (this.frozenFocus instanceof HTMLElement) {
      this.frozenFocus.focus({ preventScroll: true })
    }
  }

  handleBackgroundClick = (event: React.MouseEvent) => {
    const { close, ignoreBackgroundClick } = this.props

    if (ignoreBackgroundClick) return

    if (event.target === event.currentTarget) close()
  }

  handleAnimationEnd = (event: React.AnimationEvent) => {
    const { onRemove } = this.props

    if (event.animationName === 'SlideOut') {
      onRemove()
    }
  }

  trapFocus = (event: React.KeyboardEvent<HTMLElement> | any) => {
    /**
     * If a user tries to navigate away from the modal while it's open, via click or tab,
     * send them back to the top of the modal instead. Make an exception for links,
     * allowing users to click on footer and editorial content while the modal is open.
     */
    if (
      event.relatedTarget &&
      !event.currentTarget?.contains(event.relatedTarget) &&
      event.relatedTarget.nodeName !== 'A'
    ) {
      this.focusRef.current?.focus()
    }
  }

  render() {
    const {
      buttons,
      children,
      hideCloseX,
      bodyClassName,
      footer,
      buttonsWrapperClassName,
      showHeader,
      closeLabel,
      isClosing,
      noAnimateOpen,
      containerClassName,
    } = this.props

    const closeX = (
      <div
        role="button"
        aria-label="close"
        className="xwd__modal--close"
        onClick={this.closeWithoutPropagation}
        tabIndex={0}
        onKeyDown={this.onKeyDown}
      >
        {closeLabel}
        <Icon icon="close" />
      </div>
    )

    return (
      <div
        className={cn('modal-system-container', containerClassName)}
      >
        <div className="xwd__modal--wrapper">
          <div
            id="modalWrapper-overlay"
            role="none"
            onClick={(e) => this.handleBackgroundClick(e)}
            className={cn('xwd__modal--overlay', { showHeader })}
          />
          <div
            role="textbox"
            className={cn('xwd__modal--body', bodyClassName, {
              'animate-opening': !noAnimateOpen,
              closing: isClosing,
            })}
            onAnimationEnd={this.handleAnimationEnd}
            onKeyDown={this.onKeyDown}
            onBlur={this.trapFocus}
            ref={this.focusRef}
            tabIndex={0}
          >
            {!hideCloseX && closeX}
            <article className="xwd__modal--content">
              {children}
              {buttons && (
                <ModalButtons
                  buttons={buttons}
                  close={this.closeWithoutPropagation}
                  wrapperClassName={buttonsWrapperClassName}
                />
              )}
              {footer}
            </article>
          </div>
        </div>
      </div>
    )
  }
}

export default ModalWrapper
