import React from 'react'
import cn from 'classnames'

import styles from './SkipNav.module.scss'

/**
 * Skip navigation link is intended to be used as a mechanism to allow users
 * to quickly skip header and navigation to jump the main content of the page.
 * The skip link will be read by screenreaders but is visually hidden by default.
 * When users navigate with keyboard it will become visible and then user can skip
 * to main content by pressing enter.
 *
 * This component should be used as close to the <body> tag as possible and needs the main content
 * area to have a matching id which the link will jump to when selected
 *
 * This is a known pattern for a11y: https://webaim.org/techniques/skipnav/
 */
const handleSkipNavEnter = (event: React.SyntheticEvent) => {
  // prevent enter from dispatching further events up the DOM
  event.stopPropagation()
}

const SkipNav = ({
  associatedSectionId,
  linkText = 'Skip to content',
}: {
  associatedSectionId: string
  linkText?: string
}) => {
  return (
    <a
      onKeyDown={handleSkipNavEnter}
      className={cn(styles.skipNav)}
      href={`#${associatedSectionId}`}
    >
      {linkText}
    </a>
  )
}

export default SkipNav
