import React, { useState } from 'react'
import cn from 'classnames'

export const useToggle = (
  defaultToggle: boolean,
): [boolean, () => void] => {
  const [isToggled, setisToggled] = useState(defaultToggle)

  const handleToggle = () => setisToggled(!isToggled)
  return [isToggled, handleToggle]
}

interface ToggleProps {
  labelA: string
  labelB: string
  isToggled?: boolean
  handleToggle: () => void
}

export const Toggle = ({
  labelA = 'One',
  labelB = 'Two',
  isToggled = false, // Option A is selected by default
  handleToggle,
}: ToggleProps) => (
  <button type="button" role="tablist" className="pz-toggle">
    <div
      role="tab"
      tabIndex={0}
      aria-controls="tabpanel-a"
      aria-selected={!isToggled ? 'true' : 'false'}
      onClick={() => (!isToggled ? null : handleToggle())}
      onKeyDown={() => (!isToggled ? null : handleToggle())}
      className={cn('pz-toggle__option', {
        selected: !isToggled,
      })}
    >
      {labelA}
    </div>
    <div
      role="tab"
      tabIndex={0}
      aria-controls="tabpanel-b"
      aria-selected={isToggled ? 'true' : 'false'}
      onClick={() => (isToggled ? null : handleToggle())}
      onKeyDown={() => (isToggled ? null : handleToggle())}
      className={cn('pz-toggle__option', {
        selected: isToggled,
      })}
    >
      {labelB}
    </div>
  </button>
)

const withToggle =
  <P extends object>(
    Component: React.ComponentType<P>,
    defaultToggle: boolean = false,
  ) =>
  (props: { labelA: string; labelB: string }) => {
    const [isToggled, handleToggle] = useToggle(defaultToggle)
    const { labelA, labelB } = props

    return (
      <>
        <Toggle
          labelA={labelA}
          labelB={labelB}
          isToggled={isToggled}
          handleToggle={handleToggle}
        />
        <Component isToggled={isToggled} {...(props as P)} />
      </>
    )
  }

export default withToggle
