import React from 'react'
import { mount, ReactWrapper } from 'enzyme'

import {
  globalNavId,
  mobileToolbarId,
  showToolbarClass,
} from 'foundation'
import * as responsive from 'responsive'

import ToolbarAdapter from './ToolbarAdapter'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  globalNavId: 'GLOBAL_NAV_ID',
  mobileToolbarId: 'MOBILE_TOOLBAR_ID',
  showToolbarClass: 'SHOW_TOOLBAR_CLASS',
}))

const mockIsMobile = (isMobile: boolean) => {
  Object.defineProperty(responsive, 'isMobile', { value: isMobile })
}

const getWrapper = (isHidden = false) => {
  return mount(
    <ToolbarAdapter isHidden={isHidden}>
      <div id="toolbar" />
    </ToolbarAdapter>,
  )
}

describe('<MobileToolbarWrapper />', () => {
  let wrapper: ReactWrapper

  beforeAll(() => {
    const globalNav = document.createElement('div')
    globalNav.setAttribute('id', globalNavId)
    const mobileToolbar = document.createElement('div')
    mobileToolbar.setAttribute('id', mobileToolbarId)
    const body = document.querySelector('body')
    body?.appendChild(globalNav)
    body?.appendChild(mobileToolbar)
  })

  afterEach(() => wrapper.unmount())

  it('should have no effect on desktop', () => {
    mockIsMobile(false)
    wrapper = getWrapper()
    expect(wrapper.find('#toolbar')).toHaveLength(1)
    const parent = document.querySelector(`#${mobileToolbarId}`)
    expect(parent?.hasChildNodes()).toBeFalsy()
  })

  it('should send children through a portal on mobile', () => {
    mockIsMobile(true)
    wrapper = getWrapper()
    expect(wrapper.find('#toolbar')).toHaveLength(1)
    const parent = document.querySelector(`#${mobileToolbarId}`)
    expect(parent?.hasChildNodes()).toBeTruthy()
  })

  it('should apply class name to global nav when not hidden', () => {
    mockIsMobile(true)
    wrapper = getWrapper()
    const globalNav = document.querySelector(`#${globalNavId}`)
    expect(globalNav?.classList).toContain(showToolbarClass)
  })

  it('should remove class name from global nav when shown', () => {
    mockIsMobile(true)
    wrapper = getWrapper(true)
    const globalNav = document.querySelector(`#${globalNavId}`)
    expect(globalNav?.classList).not.toContain(showToolbarClass)
  })
})
