import React, { useState } from 'react'
import { render, screen } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import '@testing-library/jest-dom'

import useKeyboardTrap from './use-keyboard-trap'

const ComponentWithKeyboardTrap = () => {
  const wrappingDivRef = React.useRef()

  useKeyboardTrap(true, wrappingDivRef)

  return (
    <>
      <div ref={wrappingDivRef} data-testid="wrapping-div">
        <h2>A second level header</h2>
        <a href="#fake-link">I am the first link</a>
        <button type="button">Just a button hanging out</button>
        <button type="submit">Submit button</button>
      </div>
    </>
  )
}

const ComponentWithKeyboardTrapAndDependencies = ({
  // eslint-disable-next-line react/prop-types
  shouldReveal = true,
}) => {
  const wrappingDivRef = React.useRef()

  const [revealAdditionalLink, setRevealAdditionalLink] =
    useState(false)

  const [anotherStateObject, updateStateObject] = useState({
    fine: 'ok',
  })

  useKeyboardTrap(true, wrappingDivRef, [
    revealAdditionalLink,
    anotherStateObject,
  ])

  return (
    <>
      <div ref={wrappingDivRef} data-testid="wrapping-div">
        <h2>A second level header</h2>
        <a href="#fake-link">I am the first link</a>
        <button
          type="button"
          onClick={() => {
            setRevealAdditionalLink(true)
          }}
        >
          Just a button hanging out
        </button>
        <button type="submit">Submit button</button>
        {shouldReveal && revealAdditionalLink && (
          <>
            <p>
              Random paragraph which should not affect any tab order
            </p>
            <a href="#new">Link revealed on state change</a>
          </>
        )}
      </div>
    </>
  )
}

const initialDOMElements = () => {
  const firstAnchor = screen.queryByText('I am the first link')
  const firstButton = screen.queryByText('Just a button hanging out')
  const secondButton = screen.queryByText('Submit button')

  return { firstAnchor, firstButton, secondButton }
}

describe('HOOK: use-keyboard-trap', () => {
  it('should focus first focusable element when tab used', async () => {
    render(<ComponentWithKeyboardTrap />)

    const wrappingDiv = screen.getByTestId('wrapping-div')
    expect(wrappingDiv).toBeTruthy()

    expect(document.body).toHaveFocus()

    const firstAnchor = screen.queryByText('I am the first link')

    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()
  })

  it('should focus elements in order and loop to top as expected', async () => {
    render(<ComponentWithKeyboardTrap />)

    const { firstAnchor, firstButton, secondButton } =
      initialDOMElements()

    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()

    await userEvent.tab()
    expect(firstButton).toHaveFocus()

    await userEvent.tab()
    expect(secondButton).toHaveFocus()

    // tabbing at the end loops back to the first focusable element
    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()
  })

  it('should focus elements in order with shift tab and loop back to bottom', async () => {
    render(<ComponentWithKeyboardTrap />)

    const { firstAnchor, firstButton, secondButton } =
      initialDOMElements()

    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()

    await userEvent.tab()
    expect(firstButton).toHaveFocus()

    // use shift-tab to go back one elemement
    // note: shift option with `true` moves tab backwards
    await userEvent.tab({ shift: true })
    expect(firstAnchor).toHaveFocus()

    // tabbing backwards again loops to first element
    await userEvent.tab({ shift: true })
    expect(secondButton).toHaveFocus()
  })

  it('should pick up any new dependencies and add to tab order if focusable', async () => {
    render(<ComponentWithKeyboardTrapAndDependencies />)

    const { firstAnchor, firstButton, secondButton } =
      initialDOMElements()

    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()

    await userEvent.tab()
    expect(firstButton).toHaveFocus()

    await firstButton.click()
    const revealedLink = await screen.queryByText(
      'Link revealed on state change',
    )
    expect(revealedLink).toBeInTheDocument()

    await userEvent.tab()
    expect(secondButton).toHaveFocus()

    // ensure element just revealed now is part of tab order
    await userEvent.tab()
    expect(revealedLink).toHaveFocus()

    // tabbing after revealed element brings back to first element
    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()
  })

  it('should not resync DOM elements if dependencies have not changed', async () => {
    render(
      <ComponentWithKeyboardTrapAndDependencies
        shouldReveal={false}
      />,
    )

    const { firstAnchor, firstButton, secondButton } =
      initialDOMElements()

    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()

    await userEvent.tab()
    expect(firstButton).toHaveFocus()

    await firstButton.click()
    const revealedLink = await screen.queryByText(
      'Link revealed on state change',
    )
    // ensure reveal element does not display or become part of focus order
    expect(revealedLink).not.toBeInTheDocument()

    await userEvent.tab()
    expect(secondButton).toHaveFocus()

    // tabbing after revealed element brings back to first element
    await userEvent.tab()
    expect(firstAnchor).toHaveFocus()
  })
})
