import React, { useEffect } from 'react'

/**
 * A hook to use for a11y keyboard navigation to trap focus in a particular React element when the keyboard is used -
 * this hook will check all focusable elements within and if user has tabbed or shift-tabbed to first or last element
 * traps the focus in the area the user is focused on
 *
 * Expects an boolean to determine if trapping should be enabled, the element to to watch focus on,
 * and any additional dependencies that should be watched and have the focus detection rerun as a result of changes,
 * for example: if more elements get added to the DOM or are finished loading
 *
 * Note: This hook does not handle initial programatic focus of the first element, see `use-toggle-el-focus.ts` if you need
 * to manually focus the first element of a given area (such as in the case of a modal) and return to the originating el if needed
 */
const useKeyboardTrap = (
  shouldElementBeTrapped: boolean,
  elementRef: React.RefObject<HTMLElement>,
  additionalDeps?: React.DependencyList,
) => {
  useEffect(() => {
    const wrappingElement = elementRef.current

    if (shouldElementBeTrapped && wrappingElement) {
      // html elements that by their nature are considered "focusable", list adapted from
      // https://piccalil.li/quick-tip/load-all-focusable-elements-with-javascript/
      const focusableElements: NodeListOf<HTMLInputElement> =
        wrappingElement.querySelectorAll(
          'button:not([disabled]), [href], input:not([disabled]), select:not([disabled]), textarea:not([disabled]), [tabindex]:not([tabindex="-1"]):not([disabled]), details:not([disabled]), summary:not(:disabled)',
        )

      if (!focusableElements) {
        return
      }

      const firstElement = focusableElements[0]
      const lastElement =
        focusableElements[focusableElements.length - 1]

      const handleTabKeyPress = (event: KeyboardEvent) => {
        if (event.key === 'Tab') {
          if (
            event.shiftKey &&
            document.activeElement === firstElement
          ) {
            event.preventDefault()
            lastElement.focus()
          } else if (
            !event.shiftKey &&
            document.activeElement === lastElement
          ) {
            event.preventDefault()
            firstElement.focus()
          }
        }
      }

      wrappingElement.addEventListener('keydown', handleTabKeyPress)

      return () => {
        wrappingElement.removeEventListener(
          'keydown',
          handleTabKeyPress,
        )
      }
    }
  }, [shouldElementBeTrapped, elementRef, additionalDeps])
}

export default useKeyboardTrap
