/* eslint-disable require-await, react/prop-types  */

import React from 'react'
import { act } from 'react-dom/test-utils'
import { mount } from 'enzyme'
import lf from 'localforage'

import useLocalStore from './use-local-store'

const COUNTER_KEY = 'COUNTER_KEY'

const CounterComponent = ({ initialValue }) => {
  const [value, setValue] = useLocalStore(COUNTER_KEY, initialValue)

  return (
    <div>
      <p>{String(value)}</p>
      <button onClick={() => setValue(value + 1)}>Increment</button>
    </div>
  )
}

describe('useLocalStore hook', () => {
  let div

  beforeEach(() => {
    div = document.createElement('div')
    // prevents warnings about mounting component to the document element
    document.body.appendChild(div)
  })

  it('uses null value while waiting to fetch from local storage', async () => {
    await act(async () => {
      const counter = mount(<CounterComponent initialValue={0} />, {
        attachTo: div,
      })
      // using expect() within the act call allows us to test the first render
      expect(counter.find('p').text()).toEqual('null')
    })
  })

  it('resolves to initial value when data from local storage is null', async () => {
    let counter

    await act(async () => {
      counter = mount(<CounterComponent initialValue={0} />, {
        attachTo: div,
      })
    })

    expect(counter.find('p').text()).toEqual('0')
  })

  it('resolves to value from local storage when it is not null', async () => {
    let counter

    await lf.setItem(COUNTER_KEY, 1)
    await act(async () => {
      counter = mount(<CounterComponent initialValue={0} />, {
        attachTo: div,
      })
    })

    expect(counter.find('p').text()).toEqual('1')
  })

  it('updates the value in state and in local storage', async () => {
    let counter

    await lf.setItem(COUNTER_KEY, 2)
    await act(async () => {
      counter = mount(<CounterComponent initialValue={0} />, {
        attachTo: div,
      })
    })

    counter.find('button').simulate('click')

    const counterInState = counter.find('p').text()
    const counterInStorage = await lf.getItem(COUNTER_KEY)

    expect(counterInState).toEqual('3')
    expect(counterInStorage).toEqual(3)
  })
})
