import React, { useState } from 'react'
import { render, screen, waitFor } from '@testing-library/react'

import useToggleElFocus from './use-toggle-el-focus'

// eslint-disable-next-line react/prop-types
const ComponentWithFocusableHook = ({ shouldFocus = true }) => {
  const refAnchor = React.useRef()

  useToggleElFocus(shouldFocus, refAnchor)

  return (
    <>
      <div>
        <a href="#fake-link" ref={refAnchor} data-testid="anchorEl">
          Link to focus
        </a>
      </div>
    </>
  )
}

// eslint-disable-next-line react/prop-types
const ComponentToLaunchFocusableLink = () => {
  // note this component might represent launching a modal for example
  // from another el or state that would auto-focusing that new component
  const [isLinkShown, setLinkShown] = useState(false)

  return (
    <>
      <a
        href="#lauch"
        data-testid="toggleHookComponent"
        onClick={(event) => {
          setLinkShown(!isLinkShown)
        }}
      >
        Trigger to show focusable link
      </a>
      <a href="#extra" data-testid="unconcernedLink">
        Random link to ensure that focus works outside of any
        component interaction
      </a>
      <ComponentWithFocusableHook shouldFocus={isLinkShown} />
    </>
  )
}

describe('HOOK: use-toggle-el-focus', () => {
  it('should focus el in component if hook used', () => {
    render(<ComponentWithFocusableHook shouldFocus />)

    const anchor = screen.getByTestId('anchorEl')
    expect(anchor).toBeTruthy()

    expect(document.activeElement).toBe(anchor)
  })

  it('should keep focus outside of component if hook focus is false', () => {
    render(<ComponentWithFocusableHook shouldFocus={false} />)

    const anchor = screen.getByTestId('anchorEl')
    expect(anchor).toBeTruthy()

    // ensure body retains focus if effect doesn't toggle focus
    expect(document.activeElement).toBe(document.body)
  })

  it('should return focus outside of component if hook sets focus and then removes component from tree', async () => {
    render(<ComponentToLaunchFocusableLink />)

    // by default document body is still active until user interacts
    expect(document.activeElement).toBe(document.body)

    // focus a random link in HOC component
    const extraLink = screen.getByTestId('unconcernedLink')
    extraLink.focus()

    // click link which should toggle component with hook to open
    const parentEl = screen.getByTestId('toggleHookComponent')
    parentEl.click()

    await waitFor(() => {
      // confirm anchor is now focused
      const anchor = screen.getByTestId('anchorEl')
      expect(document.activeElement).toBe(anchor)
    })

    // take action that will close focused component
    parentEl.click()
    await waitFor(() => {
      // ensure that fousable link first focused before component open is now refocused
      expect(document.activeElement).toEqual(extraLink)
    })
  })
})
