import React, { useEffect } from 'react'

/**
 * A hook to use for a11y keyboard navigation that triggers JS focus of an element and then returns focus
 * to previous active element when original focus is cleared
 *
 * This hook takes a boolean and a React ref HTMLElement to focus and will focus provided element,
 * keeping track of element that was focused before the intended focus of the hook occurs
 * (via native `activeElement` in the `document`).
 * When boolean toggles focus off, the focus is returned to the originating `activeElement`
 */
export default (
  shouldElementBeFocused: boolean,
  refElementToFocus: React.RefObject<HTMLElement>,
) => {
  const originatingActiveElement = React.useRef<HTMLElement | null>(
    null,
  )

  useEffect(() => {
    if (shouldElementBeFocused && refElementToFocus?.current) {
      // track DOM element active before new element should be focused
      originatingActiveElement.current =
        document.activeElement as HTMLElement

      // focus provided element
      refElementToFocus.current.focus()
    }

    // when element should no longer be focused, return focus to prior active element
    if (!shouldElementBeFocused) {
      originatingActiveElement.current?.focus()
    }
  }, [shouldElementBeFocused])

  // note: hooks by their nature do not have to return anything:
  // https://reactjs.org/docs/hooks-custom.html#extracting-a-custom-hook
}
