# Moment System Docs

## Intro

### This moment system is comprised of code in

1. ./index.js which exports a function called `createMomentSystem`
1. ./momentHelpers.js which holds some animation functions and exports other helpers that are mainly used by `createMomentSystem`
1. ./components which has some often used components for creating moments
1. src/shared/foundation-game/moments.scss which holds all of our universal moment styles

## Setup

1. Create a moments folder your game's root folder
1. In this folder, add React components that hold the content of your moments. An example is included below.

```javascript
import React from 'react'

const MomentWelcome = () => (
  <>
    <h3 className="pzm-moment__title">Placeholder Moment</h3>
  </>
)

export default MomentWelcome
```

3. Import all misc. moments into the game's App.js file and add them to a moments configuration object

```javascript
import MomentWelcome from './moments/MomentWelcome'
import MomentCongrats from './moments/MomentCongrats'

const moments = [
  {
    name: 'welcome',
    Content: MomentWelcome,
  },
  {
    name: 'congrats',
    Content: MomentCongrats,
  },
]
```

4. Create a `loadConfig` object which takes a bunch of special loader specific props like `transitionTo` or `barColor`.

```javascript
const loadConfig = {
  icon: 'spelling-bee',
  title: 'Spelling Bee',
  description: 'Use letters in the hive',
  bgColor: '$spelling-bee-gold',
  barBgColor: 'rgba(255, 255, 255, 0.6)',
  barColor: 'black',
  transitionTo: 'welcome',
}
```

4. Create the `momentSystem` using the `createMomentSystem` function included in the index.js file in this folder. `createMomentSystem` takes the array of moments above and the loadConfig and returns both the `Moments` component as well as a `momentSystem` object.

Here is an example of an App.js file that uses moments:

```javascript
import React from 'react'
import createMomentSystem from '../shared/moments'
import MomentContent from './moments'

const moments = [
  {
    name: 'welcome',
    Content: MomentContent.Welcome,
  },
  {
    name: 'congrats',
    Content: MomentContent.Congrats,
  },
]

const loadConfig = {
  icon: 'spelling-bee',
  title: 'Spelling Bee',
  description:
    'How many words can you make with&nbsp;7&nbsp;letters?',
  bgColor: '$spelling-bee-gold',
  barBgColor: 'rgba(255, 255, 255, 0.6)',
  barColor: 'black',
  transitionTo: 'welcome',
}

const { Moments, momentSystem } = createMomentSystem(
  moments,
  loadConfig,
)

const App = () => {
  React.useEffect(() => {
    momentSystem.load('loading')
  }, []) // To show loader on mount

  return (
    <>
      <Moments />
      <Game momentSystem={momentSystem} />
    </>
  )
}
```

## Using Moments

The `momentSystem` object has a helpful `transition` method which allows you to transition between moments you registered in your array and your game and also has a `sequencingClass` that you can add to your moments as a simple way to add animation. Here is an example of how a moment might utilize these:

```javascript
const Welcome = ({ momentSystem }) => (
  <div
    className={cn('pz-moment__welcome', momentSystem.sequencingClass)}
  >
    <h1>Welcome Moment</h1>
    <button
      type="button"
      onClick={() => {
        momentSystem
          .transition('welcome', 'game')
          .then(() => console.log('hey!'))
      }}
    >
      Play
    </button>
  </div>
)
```

Note: The `transition` function returns a promise so that if you need to do something special on completion, you can!

In addition to being able to transition between registered moments, 'game' is also registered automatically by targeting the `.pz-game-screen` div (which can be found in `GameContainer.js`)

Here is an example of how a game component might use the moment system:

```javascript
const Game = ({ momentSystem }) => (
  <div className="test-content">
    <h1>Game</h1>
    <button
      type="button"
      onClick={() => {
        momentSystem.transition('game', 'congrats')
      }}
    >
      Show Congrats
    </button>
  </div>
)
```

We also have some helpful premade components in the `/components` folder, like `Welcome` or `WelcomeBack`. Here is an example of how you might create a Welcome Moment using one of these:

```javascript
import Welcome from './moments/components/Welcome'

const WelcomeMoment = ({ momentSystem }) => {
  const welcomeProps = {
    icon: 'spelling-bee',
    title: 'Spelling Bee',
    description: {
      text: 'Create words using the letters in the hive',
    },
    buttons: [
      {
        text: 'Play',
        action: () => momentSystem.transition('welcome', 'game'),
      },
    ],
    date: 'March 1st, 2021',
    editor: 'Sam Ezersky',
    bgColor: '$spelling-bee-gold',
  }

  return <Welcome game={welcomeProps} />
}
```

Note: The only background colors that can be passed in are ones that we have created variables for in both `src/shared/scss-helpers/colors.scss` and `src/games/shared/colors/colors.js`
