import React from 'react'
import { mount, shallow } from 'enzyme'

import { trackModuleInteraction } from 'foundation'

import IconComponent from '../../../../../shared/components/Icon/Icon'
import {
  HybridBackButton,
  CloseButton,
  CloseButtonWithText,
  Icon,
  Title,
  Description,
  BodyText,
  Button,
  ButtonGroup,
  Date,
  Editor,
  Wrapper,
  Meta,
} from '../subcomponents'

jest.mock('foundation', () => ({
  trackModuleInteraction: jest.fn(),
}))

describe('MOMENTS: COMPONENTS', () => {
  describe('HYBRID BACK BUTTON', () => {
    it('should render & be clickable if native bridge is initialized', () => {
      const mockBackToHub = jest.fn()

      window.isHybridWebView = 'games'
      window.NativeBridge = {
        gamesBackToHub: mockBackToHub,
      }

      const button = mount(<HybridBackButton />)
      expect(button.text()).toBe('Back')

      button.simulate('click')
      expect(mockBackToHub).toHaveBeenCalled()
    })

    it('should not fire tracking event if no trackData is present', () => {
      const button = mount(<HybridBackButton />)
      button.simulate('click')
      expect(trackModuleInteraction).toBeCalledTimes(0)
    })

    it('should fire tracking event if trackData is present', () => {
      const trackData = {
        context: 'abc',
        elementName: 'def',
        moduleLabel: 'ghi',
      }

      const button = mount(<HybridBackButton trackData={trackData} />)
      button.simulate('click')
      expect(trackModuleInteraction).toBeCalled()
    })

    it('should not render if native bridge is not initialized', () => {
      delete window.isHybridWebView
      delete window.NativeBridge

      const button = shallow(<HybridBackButton />)
      expect(button.html()).toBeFalsy()
    })
  })

  describe('CLOSE BUTTON', () => {
    it('should display an X button', () => {
      const button = mount(<CloseButton />)
      expect(button.text()).toBe('×')
    })

    it('should call provided function when clicked', () => {
      const mockAction = jest.fn()
      const button = mount(<CloseButton action={mockAction} />)
      button.simulate('click')
      expect(mockAction).toHaveBeenCalled()
    })
  })

  describe('CLOSE BUTTON WITH TEXT', () => {
    it('should display an close button with text', () => {
      const button = mount(
        <CloseButtonWithText closeText="Stop nowwww" />,
      )
      expect(button.text()).toBe('Stop nowwww')
    })

    it('should contain icon component', () => {
      const button = mount(<CloseButtonWithText closeText="x" />)
      expect(button.find(IconComponent)).toHaveLength(1)
    })

    it('should call provided function when clicked', () => {
      const mockAction = jest.fn()
      const button = mount(
        <CloseButtonWithText closeText="x" action={mockAction} />,
      )
      button.simulate('click')
      expect(mockAction).toHaveBeenCalled()
    })
  })

  describe('ICON', () => {
    it('should display correct icon when passed', () => {
      const icon = 'spelling-bee'
      const wrapper = shallow(<Icon icon={icon} size="small" />)

      expect(wrapper.hasClass('spelling-bee')).toEqual(true)
    })

    it('should show correct icon size when passed', () => {
      const icon = 'spelling-bee'
      const wrapper = shallow(<Icon icon={icon} size="small" />)

      expect(wrapper.hasClass('small')).toBe(true)
    })
  })

  describe('TITLE', () => {
    it('should display correct title when passed', () => {
      const name = 'Spelling Bee'
      const wrapper = mount(<Title text={name} />)

      expect(wrapper.find('.pz-moment__title')).toHaveLength(1)
      expect(wrapper.find('.pz-moment__title').text()).toEqual(
        'Spelling Bee',
      )
    })

    it('should show correct title size when passed', () => {
      const name = 'Spelling Bee'
      const wrapper = shallow(<Title text={name} size="small" />)

      expect(wrapper.hasClass('small')).toEqual(true)
    })
  })

  describe('DESCRIPTION', () => {
    it('should display correct description when passed', () => {
      const description =
        'Connect the dots and <br /> reveal the hidden picture.'
      const wrapper = mount(<Description text={description} />)

      expect(wrapper.find('.pz-moment__description')).toHaveLength(1)
      expect(
        wrapper.find('.pz-moment__description').html(),
      ).toContain(
        'Connect the dots and <br> reveal the hidden picture.',
      )
    })

    it('should apply correct class name when passed', () => {
      const description =
        'Connect the dots and <br /> reveal the hidden picture.'

      const wrapper = mount(
        <Description text={description} variant="small" />,
      )

      expect(wrapper.find('.pz-moment__description')).toHaveLength(1)
      expect(
        wrapper.find('.pz-moment__description').hasClass('small'),
      ).toBe(true)
    })
  })

  describe('BODY TEXT', () => {
    it('should display correct text when passed', () => {
      const text = 'Some text here'
      const wrapper = mount(<BodyText>{text}</BodyText>)

      expect(wrapper.find('.pz-moment__body')).toHaveLength(1)
      expect(wrapper.find('.pz-moment__body').html()).toContain(
        'Some text here',
      )
    })
  })

  describe('BUTTON', () => {
    it('should display correct text in Button when passed', () => {
      const buttonText = "Today's Puzzle"
      const wrapper = mount(<Button text={buttonText} />)

      expect(wrapper.find('.pz-moment__button')).toHaveLength(1)
      expect(wrapper.find('.pz-moment__button').text()).toBe(
        "Today's Puzzle",
      )
    })

    it('should fire action when Button is clicked', () => {
      const action = jest.fn()
      const wrapper = mount(<Button action={action} />)

      wrapper.simulate('click')

      expect(action).toHaveBeenCalled()
    })

    it('should apply correct class names when passed', () => {
      const wrapper = mount(
        <Button color="secondary" variant="wide" />,
      )

      expect(wrapper.find('.pz-moment__button')).toHaveLength(1)
      expect(
        wrapper.find('.pz-moment__button').hasClass('secondary'),
      ).toBe(true)
      expect(
        wrapper.find('.pz-moment__button').hasClass('wide'),
      ).toBe(true)
    })
  })

  describe('DATE', () => {
    it('should display correct date when passed', () => {
      const date = 'Thursday, February 25'
      const wrapper = mount(<Date date={date} />)

      expect(wrapper.find('.pz-moment__info-date')).toHaveLength(1)
      expect(wrapper.find('.pz-moment__info-date').text()).toEqual(
        'Thursday, February 25',
      )
    })
  })

  describe('EDITOR', () => {
    it('should display correct editor when passed', () => {
      const editor = 'sample editor'
      const wrapper = mount(<Editor editor={editor} />)

      expect(wrapper.find('.pz-moment__info-editor')).toHaveLength(1)
      expect(wrapper.find('.pz-moment__info-editor').text()).toEqual(
        'Edited by sample editor',
      )
    })
  })

  describe('META', () => {
    it('should display meta info when passed', () => {
      const text = 'Easy, Medium, Hard'
      const wrapper = mount(<Meta text={text} />)

      expect(wrapper.find('.pz-moment__meta')).toHaveLength(1)
      expect(wrapper.find('.pz-moment__meta').text()).toEqual(
        'Easy, Medium, Hard',
      )
    })
  })

  describe('WRAPPER', () => {
    it('should use bgColor', () => {
      const bgColor = '$daily-crossword-blue' // #6493e6
      const wrapper = shallow(<Wrapper bgColor={bgColor} />)

      expect(wrapper.prop('style')).toHaveProperty(
        'backgroundColor',
        '#6493e6',
      )
    })

    it('should render children', () => {
      const FakeContent = () => (
        <div className="fake-content">hey</div>
      )

      const wrapper = mount(
        <Wrapper>
          <FakeContent />
        </Wrapper>,
      )
      const content = wrapper.find('.fake-content')
      expect(content).toHaveLength(1)
    })
  })

  describe('BUTTON GROUP', () => {
    const createWrapper = (customProps) => {
      const props = {
        buttons: [
          { text: 'Button one', action: () => {} },
          { text: 'Button two', action: () => {} },
        ],
        ...customProps,
      }

      return shallow(<ButtonGroup {...props} />)
    }

    it('should render a header if one is provided', () => {
      const wrapper = createWrapper({
        header: 'Log In or Subscribe to Play',
      })
      expect(wrapper.find('.header')).toHaveLength(1)
    })

    it('should not render a Meta component without a header', () => {
      const wrapper = createWrapper()
      expect(wrapper.find(Meta)).toHaveLength(0)
    })

    it('should render the correct number of buttons', () => {
      const wrapper = createWrapper()
      expect(wrapper.find(Button)).toHaveLength(2)
    })
  })
})
