import React from 'react'
import cn from 'classnames'
import propTypes from 'prop-types'

import { trackModuleInteraction } from 'foundation'

import Pill from '../../Pill'
import IconComponent from '../../../../shared/components/Icon/Icon'
import HybridBackButtonBase from '../../../../shared/components/HybridBackButton'
import colors from '../../colors/colors'

export const HybridBackButton = ({ trackData }) => {
  // only render in hybrid webviews with a native bridge initialized
  if (!window.isHybridWebView || !window.NativeBridge) {
    return null
  }

  // hide the button on the ios newsreader app, since it has its own navigation
  if (window.newsreaderAppPlatform === 'ios') {
    return null
  }

  return (
    <HybridBackButtonBase
      onClick={() => {
        if (trackData) {
          const { elementName, context, moduleLabel } = trackData
          const elementLabel = 'back'

          trackModuleInteraction(
            'click',
            context,
            elementName,
            elementLabel,
            moduleLabel,
          )
        }

        window.NativeBridge.gamesBackToHub()
      }}
    />
  )
}

HybridBackButton.propTypes = {
  trackData: propTypes.shape({
    context: propTypes.string,
    elementName: propTypes.string.isRequired,
    moduleLabel: propTypes.string.isRequired,
  }),
}

export const CloseButton = ({ action }) => (
  <button
    type="button"
    className="pz-moment__close"
    aria-label="Close"
    onClick={action}
  >
    ×
  </button>
)

CloseButton.propTypes = {
  action: propTypes.func,
}

export const CloseButtonWithText = ({ closeText, action }) => (
  <button
    type="button"
    className="pz-moment__close_text"
    onClick={action}
  >
    <span className="inner-text">{closeText}</span>
    <IconComponent icon="close" />
  </button>
)

CloseButtonWithText.propTypes = {
  closeText: propTypes.string.isRequired,
  action: propTypes.func,
}

export const Icon = ({ icon, size = 'large' }) => (
  <div className={cn('pz-moment__icon', size, icon)} />
)

// Icons can be found in public/assets/expansion-games/
Icon.propTypes = {
  icon: propTypes.oneOf([
    'spelling-bee',
    'spelling-bee-loved',
    'spelling-bee-smarty',
    'spelling-bee-queen',
    'spelling-bee-birthday',
    'spelling-bee-spooky',
    'spelling-bee-valentines',
    'spelling-bee-ny-2024',
    'spelling-bee-ny-2025',
    'spelling-bee-ny-2026',
    'spelling-bee-ny-2027',
    'spelling-bee-ny-2028',
    'spelling-bee-ny-2029',
    'spelling-bee-ny-2030',
    'old-spelling-bee-gold-star',
    'spelling-bee-gold-star',
    'vertex',
    'letter-boxed',
    'sudoku',
    'tiles',
    'connections',
  ]),
  size: propTypes.oneOf(['small', 'medium', 'large']),
}

export const Title = ({ text, size = 'large', font = 'karnak' }) => (
  <h1 className={cn('pz-moment__title', size, font)}>{text}</h1>
)

Title.propTypes = {
  text: propTypes.string,
  size: propTypes.oneOf([
    'extra-small',
    'small',
    'medium',
    'med-large',
    'large',
  ]),
  font: propTypes.oneOf(['karnak', 'franklin']),
}

export const Description = ({
  text,
  variant = 'default',
  font = 'karnak',
}) => (
  <h1
    className={cn('pz-moment__description', variant, font)}
    dangerouslySetInnerHTML={{ __html: text }}
  />
)

Description.propTypes = {
  text: propTypes.string,
  variant: propTypes.oneOf(['default', 'small', 'extra-small']),
  font: propTypes.oneOf(['karnak', 'franklin']),
}

export const Meta = ({ text }) => (
  <span className="pz-moment__meta">{text}</span>
)

Meta.propTypes = {
  text: propTypes.oneOfType([propTypes.string, propTypes.node]),
}

export const BodyText = ({ children }) => (
  <div className="pz-moment__body">{children}</div>
)

BodyText.propTypes = {
  children: propTypes.node,
}

export const InfoText = ({ children }) => (
  <span className="pz-moment__info-text ">{children}</span>
)

InfoText.propTypes = {
  children: propTypes.node,
}

export const Button = ({
  text,
  action,
  icon = 'none',
  color = 'primary',
  variant = 'default',
  isEmphasizedNew = false,
}) => (
  <button
    type="button"
    className={cn('pz-moment__button', color, variant)}
    onClick={action}
  >
    {isEmphasizedNew && (
      <Pill text="new" position="top-right-outside" />
    )}
    {icon === 'checked' && (
      <div className="pz-moment__button--checked" />
    )}
    {icon === 'locked' && (
      <div className="pz-moment__button--padlock" />
    )}
    {text}
  </button>
)

Button.propTypes = {
  color: propTypes.oneOf(['primary', 'secondary', 'transparent']),
  variant: propTypes.oneOf([
    'default',
    'wide',
    'disabled',
    'extra-wide',
    'extra-extra-wide',
    'link',
  ]),
  icon: propTypes.oneOf(['none', 'locked', 'checked']),
  isEmphasizedNew: propTypes.bool,
  action: propTypes.func,
  text: propTypes.string,
}

export const ButtonGroup = ({
  buttons,
  header,
  order = 'default',
}) => (
  <div className="pz-moment__button-group">
    {header && (
      <div id="buttongroup-header" className="header">
        {header}
      </div>
    )}
    <div
      aria-labelledby="buttongroup-header"
      className={cn('pz-moment__button-wrapper', { [order]: order })}
    >
      {(buttons || []).map((props, i) => {
        return <Button {...props} key={i} />
      })}
    </div>
  </div>
)

ButtonGroup.propTypes = {
  buttons: propTypes.arrayOf(
    propTypes.shape({
      text: propTypes.string,
      action: propTypes.func,
    }),
  ),
  header: propTypes.oneOfType([propTypes.node, propTypes.string]),
  order: propTypes.oneOf(['default', 'vertical']),
}

export const Date = ({ date }) => (
  <span
    data-testid="pz-moment__date"
    className="pz-moment__info-date"
  >
    {date}
  </span>
)

Date.propTypes = {
  date: propTypes.string,
}

export const Editor = ({ editor }) =>
  editor ? (
    <span
      data-testid="pz-moment__editor"
      className="pz-moment__info-editor"
    >{`Edited by ${editor}`}</span>
  ) : null

Editor.propTypes = {
  editor: propTypes.string,
}

export const Wrapper = ({ bgColor = '$gray3', children }) => (
  <div
    className="pz-moment"
    style={{ backgroundColor: colors[bgColor] }}
  >
    {children}
  </div>
)

// The only colors accepted are those that we have stored in variables
// See src/shared/scss-helpers/colors.scss and/or
// src/games/shared/colors/colors.js
Wrapper.propTypes = {
  bgColor: propTypes.oneOf(Object.keys(colors)),
  children: propTypes.oneOfType([propTypes.element, propTypes.func]),
}

export const Promo = ({ promo }) =>
  promo === 'oxford' && (
    <p>
      {/* eslint-disable jsx-a11y/anchor-has-content */}
      <a
        href="https://www.oxforddictionaries.com"
        target="_blank"
        rel="noopener noreferrer"
        className="pz-moment__promo"
      />
    </p>
  )

Promo.propTypes = {
  promo: propTypes.string,
}
