import React, { Component } from 'react'
import pt from 'prop-types'
import ReactDOM from 'react-dom'
import cn from 'classnames'

import {
  transition,
  setUpMoments,
  putOnStage,
  reanimateMoment,
  CLASS_ANIMATION,
} from './momentHelpers'

const createMomentSystem = (moments, loadConfig) => {
  /* root can be found in GameContainer.js */
  const momentRoot = document.getElementById('portal-game-moments')

  /* the names of all registered and predefined moments */
  const momentNames = [...moments.map((m) => m.name), 'game']

  /* helper to find moment divs on the page */
  const getMoment = (name) =>
    document.getElementById(`js-hook-pz-moment__${name}`)

  /* expected functions for object that will be exported from moment system */
  let momentSystem = {
    load: (modal) => {},
    getCurrentMoment: (moment) => null,
    transition: async (from, to) => {},
  }
  class Moments extends Component {
    constructor(props) {
      super(props)

      this.state = {
        currentMoment: '',
      }

      momentSystem.load = this.load.bind(this)
      momentSystem.getCurrentMoment = this.getCurrentMoment.bind(this)
      momentSystem.transition = this.momentTransition.bind(this)
      momentSystem.reanimateMoment = reanimateMoment.bind(this)
    }

    componentDidMount() {
      /* so that the game can be transitioned to or from */
      setUpMoments()
    }

    componentDidUpdate(_, prevState) {
      const { currentMoment } = this.state
      const { onEnter } = this.props

      if (prevState.currentMoment !== currentMoment) {
        onEnter(currentMoment)
      }
    }

    getCurrentMoment() {
      const { currentMoment } = this.state
      return currentMoment
    }

    /*
      moment names are registered in moment system
      and used for transitioning between them and the game
    */
    momentTransition(from, to) {
      if (!momentNames.includes(from) || !momentNames.includes(to)) {
        console.error(`${from} or ${to} is not a registered moment`)
        return
      }

      const leaving = getMoment(from)
      const entering = getMoment(to)
      const toGame = to === 'game'

      /* transition() returns a promise */
      return transition(
        leaving,
        entering,
        () => this.setState({ currentMoment: to }),
        toGame,
      )
    }

    /* directly load the given moment, without any transitions/delay */
    load(to) {
      if (!momentNames.includes(to)) {
        console.error(`${to} is not a registered moment`)
        return
      }

      const entering = getMoment(to)

      putOnStage(entering)
      this.setState({ currentMoment: to })
    }

    render() {
      const { currentMoment } = this.state
      const { contentProps } = this.props

      return ReactDOM.createPortal(
        <>
          {moments.map(({ name, Content }) => (
            <section
              className={cn(`pz-moment__frame pz-moment__${name}`, {
                'playtab-fade-in': name === 'loading',
              })}
              key={name}
              id={`js-hook-pz-moment__${name}`}
            >
              <Content
                momentSystem={momentSystem}
                isActive={currentMoment === name}
                {...contentProps}
              />
            </section>
          ))}
        </>,
        momentRoot,
      )
    }
  }

  Moments.propTypes = {
    onEnter: pt.func,
    /**
     * Provide any props to all of the subsequently rendered Content components in your system.
     */
    contentProps: pt.object,
  }

  Moments.defaultProps = {
    onEnter: () => {},
    contentProps: {},
  }

  momentSystem = {
    sequencingClass: CLASS_ANIMATION,
    load: Moments.load,
    getCurrentMoment: Moments.getCurrentMoment,
    transition: Moments.transition,
  }

  return {
    Moments,
    momentSystem,
  }
}

export default createMomentSystem
