import { wait } from '../../helpers/gameUtils'

const ATTR_TOP = 'top'
const ATTR_POSITION = 'position'
const ATTR_OVERFLOW = 'overflow'
const ATTR_VISIBILITY = 'visibility'
const ATTR_MIN_HEIGHT = 'minHeight'
const ATTR_BG_COLOR = 'backgroundColor'

const VALUE_HIDDEN = 'hidden'
const VALUE_STATIC = 'static'
const VALUE_VISIBLE = 'visible'
const VALUE_RELATIVE = 'relative'
const VALUE_ABSOLUTE = 'absolute'

const CLASS_ON_STAGE = 'on-stage'
const CLASS_FLY_IN = 'fly-in'
const CLASS_FLY_OUT = 'fly-out'
const CLASS_SLIDE_UP = 'slide-up'
const CLASS_MODAL_CARD = 'modal-card'
export const CLASS_ANIMATION = 'sequence-animation'

const TIME_GROW = 400

/*
  INTERNAL HELPER FUNCTIONS
*/
const TWEEN_CUBIC_IN_OUT = (k) => {
  let tween

  if (k * 2 < 1) {
    tween = k * 2
    return 0.5 * tween ** 3
  }

  tween = k - 2
  return 0.5 * tween ** 3 + 2
}

const now = () => Date.now()

const style = (elm, attr, value) => {
  elm.style[attr] = value
}

const addClass = (elm, className) => {
  elm.classList.add(className)
}

const removeClass = (elm, className) => {
  if (!elm.length) {
    elm.classList.remove(className)
  } else {
    elm.forEach((node) => {
      node.classList.remove(className)
    })
  }
}

const animateProp = (
  elm,
  prop,
  from,
  to,
  duration,
  tween = TWEEN_CUBIC_IN_OUT,
  unit = 'px',
) => {
  return new Promise((resolve) => {
    const delta = to - from
    const start = now()
    const animate = () => {
      let k = (now() - start) / duration

      k = k >= 1 ? 1 : k

      elm.style[prop] = Math.min(from + tween(k) * delta + unit, to)

      if (k === 1) {
        resolve()
      } else {
        window.requestAnimationFrame(animate)
      }
    }

    animate()
  })
}

/*
  EXPORTED HELPER FUNCTIONS
*/

// Adds special 'frame' class to container that holds the game.
// This allows us to transition between moments and the game.
export const setUpMoments = () => {
  const game = document.getElementById('js-hook-pz-moment__game')
  if (game) {
    addClass(game, 'pz-moment__frame')
  }
}

// Performs sequencing animation on a list of elements.
// I.E. when elements slide up into screen one after the other.
export const sequence = (nodes, duration) => {
  let delay = 0
  nodes.forEach((node) => {
    if (!node) return
    delay += duration
    style(node, 'transitionDelay', `${delay}ms`)
    addClass(node, CLASS_SLIDE_UP)
  })

  return delay
}

export const putOnStage = (entering) => {
  addClass(entering, CLASS_ON_STAGE)
}

// For re-rendering the welcome moment after a user's status changes
// we'll slide in the content replacements
export const reanimateMoment = (moment) => {
  wait(TIME_GROW).then(() => {
    const entering = document.getElementById(
      `js-hook-pz-moment__${moment}`,
    )

    const nodesToSequence = entering.querySelector(
      `.${CLASS_ANIMATION}`,
    )

    if (nodesToSequence) {
      sequence(Array.from(nodesToSequence.children), 200)
    }
  })
}

export const playtabTransition = (
  leaving,
  entering,
  onTransition,
  toGame,
) => {
  const sequencedNodes = document.querySelectorAll(
    `.${CLASS_SLIDE_UP}`,
  )
  const modalCard = document.querySelector(`.${CLASS_MODAL_CARD}`)
  const body = document.querySelector('body')

  if (onTransition) {
    onTransition()
  }

  addClass(entering, CLASS_ON_STAGE)
  addClass(entering, 'playtab-fade-in')
  removeClass(leaving, 'playtab-fade-in')

  if (toGame) {
    addClass(body, 'game-moment')
  } else {
    removeClass(body, 'game-moment')
  }

  wait(600).then(() => {
    if (sequencedNodes.length) {
      removeClass(sequencedNodes, CLASS_SLIDE_UP)
    }

    const nodesToSequence = entering.querySelector(
      `.${CLASS_ANIMATION}`,
    )

    if (nodesToSequence) {
      wait(
        sequence(
          Array.from(nodesToSequence.children).concat(modalCard),
          25,
        ),
      ).then(() => {
        removeClass(leaving, CLASS_ON_STAGE)
        return Promise.resolve()
      })
    } else {
      removeClass(leaving, CLASS_ON_STAGE)
    }
  })

  return Promise.resolve()
}

// Takes a 'leaving' frame and 'entering' frame.
// Responsible for transitioning between the two.
export const transition = (
  leaving,
  entering,
  onTransition,
  toGame,
) => {
  if (window.isPlayTab || window.newsreaderAppPlatform === 'ios') {
    return playtabTransition(leaving, entering, onTransition, toGame)
  }

  const container = document.getElementById('js-hook-game-wrapper')
  const fromHeight = leaving.offsetHeight
  const modalCard = document.querySelector(`.${CLASS_MODAL_CARD}`)
  const sequencedNodes = document.querySelectorAll(
    `.${CLASS_SLIDE_UP}`,
  )

  // Set the container color to the background color of
  // the last screen for smoother transitions.
  // If there's no background color defined,
  // set it to the default background color on web (white),
  // as a transparent background would reveal the webview's background color
  style(
    container,
    ATTR_BG_COLOR,
    leaving.style.backgroundColor || 'white',
  )
  style(container, ATTR_OVERFLOW, VALUE_HIDDEN)
  style(container, ATTR_MIN_HEIGHT, `${fromHeight}px`)

  style(entering, ATTR_VISIBILITY, VALUE_HIDDEN)
  style(entering, ATTR_POSITION, VALUE_ABSOLUTE)
  style(entering, ATTR_TOP, 0)
  addClass(entering, CLASS_ON_STAGE)

  style(leaving, ATTR_POSITION, VALUE_ABSOLUTE)
  style(leaving, ATTR_TOP, 0)
  addClass(leaving, CLASS_FLY_OUT)

  if (onTransition) {
    onTransition()
  }

  const toHeight = entering.offsetHeight

  // Set the background color for the body if in hybrid (to style iOS safe insets)
  if (window.isHybridWebView) {
    style(
      document.body,
      ATTR_BG_COLOR,
      leaving.style.backgroundColor || 'white',
    )
  }

  wait(TIME_GROW).then(() => {
    if (sequencedNodes.length) {
      removeClass(sequencedNodes, CLASS_SLIDE_UP)
    }

    style(entering, ATTR_VISIBILITY, VALUE_VISIBLE)
    addClass(entering, CLASS_FLY_IN)

    const nodesToSequence = entering.querySelector(
      `.${CLASS_ANIMATION}`,
    )

    if (nodesToSequence) {
      sequence(
        Array.from(nodesToSequence.children).concat(modalCard),
        25,
      )
    }

    // set background to orange in sudoku
    if (window.isHybridWebView) {
      if (
        entering.id === 'js-hook-pz-moment__welcome' &&
        leaving.id === 'js-hook-pz-moment__game'
      ) {
        style(document.body, ATTR_BG_COLOR, '#fb9b00')
      }
    }

    animateProp(
      container,
      ATTR_MIN_HEIGHT,
      fromHeight,
      toHeight,
      TIME_GROW,
    ).then(() => {
      removeClass(leaving, CLASS_ON_STAGE)
      removeClass(leaving, CLASS_FLY_OUT)
      removeClass(entering, CLASS_FLY_IN)

      style(container, ATTR_OVERFLOW, VALUE_VISIBLE)
      style(container, ATTR_MIN_HEIGHT, 0)
      style(leaving, ATTR_POSITION, VALUE_STATIC)
      style(entering, ATTR_POSITION, VALUE_RELATIVE)
    })
  })

  // Returns promise after animation is complete
  return new Promise((resolve) => setTimeout(resolve, TIME_GROW))
}
