import React from 'react'
import { render, screen } from '@testing-library/react'

import { CardProgressMap } from '../../spelling-bee/js/store/api/api.selectors'

import PastPuzzlesSections from './PastPuzzlesSections'
import { mockGameData } from './mockGameData'

const progressMap: CardProgressMap = {
  '1236': { rank: '', isRevealed: false },
  '12236': { rank: 'Amazing', isRevealed: false },
  '12637': { rank: '', isRevealed: false },
  '13520': { rank: '', isRevealed: false },
  '19644': { rank: 'Great', isRevealed: false },
  '21677': { rank: '', isRevealed: false },
  '21769': { rank: '', isRevealed: false },
  '23921': { rank: '', isRevealed: false },
  '27517': { rank: '', isRevealed: false },
}

describe('PastPuzzlesSections', () => {
  beforeEach(() => {
    window.gamesAppPlatform = ''
    window.newsreaderAppPlatform = ''
    window.isHybridWebView = false
  })

  it('renders oops if no puzzles', () => {
    render(
      <PastPuzzlesSections
        progressMap={{}}
        pastPuzzles={null}
        isProgressLoading={false}
        hasXwd
        hasUserLoaded
      />,
    )
    const copy = screen.getByText(
      'Oops, we’re off our game. Spelling Bee past puzzles couldn’t be loaded. Please reload the page or try again later.',
    )

    expect(copy).toBeTruthy()
  })

  it('renders default header when subscribed', () => {
    render(
      <PastPuzzlesSections
        progressMap={{}}
        pastPuzzles={mockGameData.pastPuzzles}
        isProgressLoading={false}
        hasXwd
        hasUserLoaded
      />,
    )
    const copy = screen.getByText(
      'Catch up on puzzles from previous days.',
    )

    expect(copy).toBeTruthy()
  })

  it('renders subscribe link when *not* subscribed', () => {
    render(
      <PastPuzzlesSections
        progressMap={{}}
        pastPuzzles={mockGameData.pastPuzzles}
        isProgressLoading={false}
        hasXwd={false}
        hasUserLoaded
      />,
    )
    const link = screen.getByText('Subscribe')

    expect(link).toBeTruthy()
  })

  it('renders puzzle cards for last week', () => {
    render(
      <PastPuzzlesSections
        progressMap={{}}
        pastPuzzles={mockGameData.pastPuzzles}
        isProgressLoading={false}
        hasXwd
        hasUserLoaded
      />,
    )
    const header = screen.getByText('Last Week')
    const lastWeekrange = screen.getByText('June 11 - June 5')

    expect(header).toBeTruthy()
    expect(lastWeekrange).toBeTruthy()
  })

  it('renders puzzle cards for this week', () => {
    render(
      <PastPuzzlesSections
        progressMap={{}}
        pastPuzzles={mockGameData.pastPuzzles}
        isProgressLoading={false}
        hasXwd
        hasUserLoaded
      />,
    )
    const header = screen.getByText('This Week')
    const thisWeekRange = screen.getByText('June 18 - June 12')

    expect(header).toBeTruthy()
    expect(thisWeekRange).toBeTruthy()
  })

  it('renders correct link for a past puzzle', () => {
    render(
      <PastPuzzlesSections
        progressMap={{}}
        pastPuzzles={mockGameData.pastPuzzles}
        isProgressLoading={false}
        hasXwd
        hasUserLoaded
      />,
    )

    const pastPuzzleLink = screen.getByRole('link', {
      name: 'past puzzle from 2023-06-05 Monday',
    })
    expect(pastPuzzleLink).toHaveAttribute(
      'href',
      '/puzzles/spelling-bee/2023-06-05',
    )
  })

  it('does not render link for first day of week', () => {
    render(
      <PastPuzzlesSections
        progressMap={{}}
        pastPuzzles={mockGameData.pastPuzzles}
        isProgressLoading={false}
        hasXwd
        hasUserLoaded
      />,
    )

    // based on mock data today is 6-13 so that item should not exist
    // since we don't show "today" in past puzzle
    const pastPuzzleLink = screen.queryByRole('link', {
      name: 'past puzzle from 2023-06-13 Monday',
    })
    expect(pastPuzzleLink).not.toBeInTheDocument()
  })

  it('displays user rank for stats', () => {
    render(
      <PastPuzzlesSections
        progressMap={progressMap}
        pastPuzzles={mockGameData.pastPuzzles}
        isProgressLoading={false}
        hasXwd
        hasUserLoaded
      />,
    )

    // even though not real rank verify that rank in fixture data is one displayed
    const rank = screen.getByText('Great')

    expect(rank).toBeTruthy()
  })
})
