import React from 'react'

import { trackClick } from 'foundation'

import type {
  SpellingBeeGameData,
  SpellingBeePuzzle,
} from '../../spelling-bee/js/store/game/game.state'
import { getSlashFormattedPrintDate } from '../../shared/utils/dateUtils'
import { handleSubscribe } from '../../helpers/subscribeLinks'
import useCTA from '../../../shared/marketing/useCTA'
import type { CardProgressMap } from '../../spelling-bee/js/store/api/api.selectors'

import {
  getWeekRangesToDisplay,
  getReversedWeekRanges,
} from './utils/helpers'
import styles from './PastPuzzlesSections.module.scss'
import PastPuzzleCard from './components/PastPuzzleCard'

const getPuzzleCards = (
  puzzles: SpellingBeePuzzle[],
  progressMap: CardProgressMap,
  hasXwd: boolean,
  hasUserLoaded: boolean,
  isProgressLoading: boolean,
  isInModalBody?: boolean,
) => {
  return puzzles.map((puzzle) => {
    const puzzleIdString = puzzle.id.toString()
    // if puzzle is matched to stats, it means user has a rank for
    // that day so make sure to pass along rank to PastPuzzleCard component
    const progress = progressMap[puzzleIdString]

    return (
      <PastPuzzleCard
        rank={progress ? progress.rank : ''}
        isRevealed={progress ? progress.isRevealed : false}
        hasXwd={hasXwd}
        hasUserLoaded={hasUserLoaded}
        printDate={puzzle.printDate}
        displayWeekday={puzzle.displayWeekday}
        key={puzzle.id}
        puzzleId={puzzleIdString}
        isProgressLoading={isProgressLoading}
        isInModalBody={isInModalBody}
      />
    )
  })
}

const getSectionRow = (
  sectionHeader: string,
  weekRangeText: string,
  puzzles: SpellingBeePuzzle[],
  progressMap: CardProgressMap,
  hasXwd: boolean,
  hasUserLoaded: boolean,
  isProgressLoading: boolean,
  isInModalBody: boolean,
) => {
  return (
    <div className={styles.weekSectionContainer}>
      <div className={styles.weekHeaderSection}>
        <div className={styles.weekHeader}>
          <h3>{sectionHeader}</h3>
        </div>
        <div className={styles.weekRange}>{weekRangeText}</div>
      </div>
      <div className={styles.weekSection}>
        {getPuzzleCards(
          puzzles,
          progressMap,
          hasXwd,
          hasUserLoaded,
          isProgressLoading,
          isInModalBody,
        )}
      </div>
    </div>
  )
}

const PastPuzzlesSections = ({
  progressMap,
  pastPuzzles,
  isProgressLoading,
  hasXwd,
  hasUserLoaded,
  isInModalBody = false,
}: {
  progressMap: CardProgressMap
  pastPuzzles: SpellingBeeGameData['pastPuzzles']
  isProgressLoading: boolean
  hasXwd: boolean
  hasUserLoaded: boolean
  isInModalBody?: boolean
}) => {
  const { isGamesSaleActive, getUrgencyMessage } = useCTA()

  if (!pastPuzzles) {
    return (
      <div className={styles.pastPuzzlesContent}>
        <p>
          Oops, we’re off our game. Spelling Bee past puzzles couldn’t
          be loaded. Please reload the page or try again later.
        </p>
      </div>
    )
  }

  const { lastWeek, thisWeek } = pastPuzzles
  const { printDate } = pastPuzzles.today

  // get full range for this week and last week, exclusive of actual puzzles,
  // to use as display text along this week and last week headers
  const { thisWeekDateRange, lastWeekDateRange } =
    getWeekRangesToDisplay(getSlashFormattedPrintDate(printDate))

  // get puzzle data for this week and last week,
  // making sure to remove "today" from this week for display purposes
  const { thisWeekWithoutTodayReversed, lastWeekReversed } =
    getReversedWeekRanges(thisWeek, lastWeek)

  return (
    <>
      <div className={styles.pastPuzzlesHeaderContainer}>
        <div className={styles.pastPuzzlesHeader}>
          <h2>Past Puzzles</h2>
        </div>
        {hasXwd ? (
          <p>Catch up on puzzles from previous days.</p>
        ) : (
          <div>
            <p>
              <button
                type="button"
                className={styles.subscribeLink}
                onClick={() => {
                  trackClick({
                    context: null,
                    element: {
                      name: 'spelling-bee-hub',
                      label: 'subscribe',
                    },
                  })
                  handleSubscribe('spellingBeeHub')
                }}
              >
                Subscribe
              </button>{' '}
              <span className="visually-hidden"> to games</span>
              to play past puzzles.{' '}
              {/** TODO: remove after game sale */}
              {isGamesSaleActive &&
                `Enjoy 50% off: ${getUrgencyMessage()}.`}
            </p>
          </div>
        )}
      </div>
      <div className={styles.pastPuzzlesContent}>
        {/* Hide this week section if there are no past puzzlesfrom this week yet  */}
        {thisWeekWithoutTodayReversed &&
          thisWeekWithoutTodayReversed.length > 0 && (
            <>
              {getSectionRow(
                'This Week',
                thisWeekDateRange,
                thisWeekWithoutTodayReversed,
                progressMap,
                hasXwd,
                hasUserLoaded,
                isProgressLoading,
                isInModalBody,
              )}
            </>
          )}
        {getSectionRow(
          'Last Week',
          lastWeekDateRange,
          lastWeekReversed,
          progressMap,
          hasXwd,
          hasUserLoaded,
          isProgressLoading,
          isInModalBody,
        )}
      </div>
    </>
  )
}

export default PastPuzzlesSections
