import React from 'react'
import {
  render,
  screen,
  createEvent,
  fireEvent,
} from '@testing-library/react'

import useFeatureFlag from '../../../../shared/abra/useFeatureFlag'

import PastPuzzleCard from './PastPuzzleCard'
import styles from './PastPuzzleCard.module.scss'

const mockResponse = jest.fn()
Object.defineProperty(window, 'location', {
  value: {
    hash: {
      endsWith: mockResponse,
      includes: mockResponse,
    },
    assign: mockResponse,
  },
  writable: true,
})

jest.mock('../../../../shared/abra/useFeatureFlag', () => jest.fn())

describe('PastPuzzleCard', () => {
  const { location } = window
  let url = ''
  const getHrefSpy = jest.fn(() => url)
  const setHrefSpy = jest.fn((href) => {
    url = href
  })

  beforeAll(() => {
    // @ts-ignore
    delete window.location
    // @ts-ignore
    window.location = {}

    Object.defineProperty(window.location, 'href', {
      get: getHrefSpy,
      set: setHrefSpy,
    })
  })

  afterAll(() => {
    window.location = location
  })

  beforeEach(() => {
    window.gamesAppPlatform = ''
    window.newsreaderAppPlatform = ''
    window.isHybridWebView = false
  })

  it('renders a link to a past puzzle', () => {
    render(
      <PastPuzzleCard
        rank="Beginner"
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )
    const link = screen.getByRole('link')

    expect(link).toHaveAttribute(
      'href',
      '/puzzles/spelling-bee/2023-03-27',
    )
  })

  it('reroutes user to proper link in hybrid webviews', () => {
    window.newsreaderAppPlatform = 'ios'
    window.isHybridWebView = true

    render(
      <PastPuzzleCard
        rank="Beginner"
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
        isInModalBody
      />,
    )
    const link = screen.getByRole('link')
    expect(link).toHaveAttribute(
      'href',
      '/puzzles/spelling-bee/2023-03-27',
    )
    fireEvent.click(link)
    expect(window.location.href).toBe(
      '/puzzles/spelling-bee/2023-03-27?hybrid=news',
    )
  })
  it('redirects if in modal boday', () => {
    window.newsreaderAppPlatform = 'ios'
    window.isHybridWebView = true

    render(
      <PastPuzzleCard
        rank="Beginner"
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
        isInModalBody
      />,
    )
    const link = screen.getByRole('link')

    // make sure event is prevented by default when in modal boday
    const event = createEvent.click(link)
    fireEvent(link, event)
    expect(event.defaultPrevented).toBeTruthy()
  })

  it('still renders puzzle link if unsubscribed', () => {
    render(
      <PastPuzzleCard
        rank="Beginner"
        hasXwd={false}
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )
    const link = screen.getByRole('link')

    expect(link).toHaveAttribute(
      'href',
      '/puzzles/spelling-bee/2023-03-27',
    )
  })

  it('renders a foldover if played, if in the feature flag is enabled', () => {
    // @ts-ignore
    useFeatureFlag.mockImplementation(() => true)

    const { container } = render(
      <PastPuzzleCard
        rank="Beginner"
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )

    const foldOver = container.querySelector(
      `.${styles.playedFoldoverTop}`,
    )
    expect(foldOver).toBeInTheDocument()
  })

  it('renders beeatrice icon if unplayed ', () => {
    const { container } = render(
      <PastPuzzleCard
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )

    const icon = container.querySelector(
      `.${styles.beeatriceUnplayed}`,
    )
    expect(icon).toBeInTheDocument()
  })

  it('renders correct progress icon ', () => {
    const { container } = render(
      <PastPuzzleCard
        rank="Good Start"
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )

    const icon = container.querySelector(
      `.${styles.beeatriceUnplayed}`,
    )
    expect(icon).toBeInTheDocument()
  })

  it('renders correct progress icon when user has revealed', () => {
    const { container } = render(
      <PastPuzzleCard
        rank="Good Start"
        isRevealed
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )

    const icon = container.querySelector(`.${styles.goldStar}`)
    expect(icon).toBeInTheDocument()
  })

  it('renders correct genius hat icon ', () => {
    const { container } = render(
      <PastPuzzleCard
        rank="Genius"
        isRevealed
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )

    const icon = container.querySelector(`.${styles.geniusHat}`)
    expect(icon).toBeInTheDocument()
  })

  it('renders correct queen bee icon ', () => {
    const { container } = render(
      <PastPuzzleCard
        rank="Queen Bee"
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )

    const icon = container.querySelector(`.${styles.queenBeeHat}`)
    expect(icon).toBeInTheDocument()
  })

  it('renders a rank if played', () => {
    render(
      <PastPuzzleCard
        rank="Great"
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )
    const rank = screen.queryByText('Great')

    expect(rank).toBeInTheDocument()
  })

  it('still renders a rank, and lock if non-sub and has played', () => {
    const { container } = render(
      <PastPuzzleCard
        rank="Great"
        hasXwd={false}
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )
    const rank = screen.queryByText('Great')
    const lockIcon = container.querySelector(`.${styles.lockIcon}`)

    expect(lockIcon).toBeTruthy()
    expect(rank).toBeInTheDocument()
  })

  it('does not render lock if user status has not loaded', () => {
    const { container } = render(
      <PastPuzzleCard
        hasXwd={false}
        hasUserLoaded={false}
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )
    const lockIcon = container.querySelector(`.${styles.lockIcon}`)
    expect(lockIcon).toBeFalsy()
  })

  it('renders lock if anon', () => {
    const { container } = render(
      <PastPuzzleCard
        hasXwd={false}
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )
    const lockIcon = container.querySelector(`.${styles.lockIcon}`)
    expect(lockIcon).toBeTruthy()
  })

  it('renders loading icon and text when progress is still loading', () => {
    const { container } = render(
      <PastPuzzleCard
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading
      />,
    )

    const loadingIcon = container.querySelector(
      `.${styles.loadingIcon}`,
    )

    const loadingText = container.querySelector(
      `.${styles.loadingText}`,
    )

    expect(loadingIcon).toBeTruthy()
    expect(loadingText).toBeTruthy()
  })

  it('renders revealed message and no foldover when puzzle has been revealed', () => {
    const { container } = render(
      <PastPuzzleCard
        rank="Solid"
        isRevealed
        hasXwd
        hasUserLoaded
        printDate="2023-03-27"
        displayWeekday="Monday, March 27"
        puzzleId="cats"
        isProgressLoading={false}
      />,
    )

    const revealed = screen.queryByText('Revealed: Solid')
    const foldOver = container.querySelector(
      `.${styles.playedFoldoverTop}`,
    )

    expect(revealed).toBeInTheDocument()
    expect(foldOver).not.toBeInTheDocument()
  })
})
