import React from 'react'

import { DateString } from '../../../shared/utils/dateUtils'
import { trackInteraction } from '../../../../shared/foundation'
import createHybridNewsLink from '../../../shared/utils/createHybridNewsLink'
import useFeatureFlag from '../../../../shared/abra/useFeatureFlag'
import { useReportExposure } from '../../../../shared/abra/FeatureFlagProvider'
import type { SpellingBeeRankWithDefault } from '../../../spelling-bee/js/store/utils'

import styles from './PastPuzzleCard.module.scss'

export type PastPuzzlesCardProps = {
  rank?: SpellingBeeRankWithDefault
  isRevealed?: boolean
  hasXwd: boolean
  hasUserLoaded: boolean
  printDate: DateString
  displayWeekday: string
  puzzleId: string
  isProgressLoading: boolean
  isInModalBody?: boolean
}

const PastPuzzleCard = ({
  rank = '',
  isRevealed = false,
  hasXwd,
  hasUserLoaded,
  printDate,
  displayWeekday,
  puzzleId,
  isProgressLoading,
  isInModalBody = false,
}: PastPuzzlesCardProps) => {
  const showOldIcons = useFeatureFlag('sbHubIconsOld')
  // TODO: foldover is going away with the new icons, so remove
  // this comment and logic when experiment is done
  // if your rank is either an empty string because you have
  // not yet started the game, or your rank is Queen Bee, or
  // because you have revealed the answers to the puzzle
  // we do not display a foldover on the card

  const showFoldOver =
    showOldIcons &&
    !(rank === '' || rank === 'Queen Bee' || isRevealed)

  let oldProgressIcon
  let progressIcon

  switch (rank) {
    case 'Beginner':
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankBeginner
      break
    case 'Good Start':
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankGoodStart
      break
    case 'Moving Up':
      // Design said Good Start and Moving Up icons will overlap because
      // we don't have enough beehive status icons
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankGoodStart
      break
    case 'Good':
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankGood
      break
    case 'Solid':
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankSolid
      break
    case 'Nice':
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankNice
      break
    case 'Great':
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankGreat
      break
    case 'Amazing':
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.rankAmazing
      break
    case 'Genius':
      oldProgressIcon = styles.geniusHat
      progressIcon = styles.beeatriceGenius
      break
    case 'Queen Bee':
      oldProgressIcon = styles.queenBeeHat
      progressIcon = styles.beeatriceQueenBee
      break
    default:
      oldProgressIcon = styles.beeatriceUnplayed
      progressIcon = styles.beeatriceUnplayed
  }

  if (rank !== 'Genius' && rank !== 'Queen Bee' && isRevealed) {
    oldProgressIcon = styles.goldStar
    progressIcon = styles.amazingRevealed
  }

  useReportExposure('sbHubIconsOld')

  return (
    <>
      <a
        href={`/puzzles/spelling-bee/${printDate}`}
        onClick={(event) => {
          trackInteraction({
            element: {
              name: 'spelling-bee-hub',
              label: 'spelling-bee',
            },
            context: `SB-${puzzleId}`,
            name: {
              p: printDate,
              t: 'archive',
            },
          })
          // for past puzzle cards rendered in modal, when clicked force a
          // location change, otherwise direct link to past puzzle opens natively in "sheet"
          if (
            isInModalBody &&
            (!!window.gamesAppPlatform ||
              !!window.newsreaderAppPlatform)
          ) {
            event.preventDefault()
            window.location.href = createHybridNewsLink(
              `/puzzles/spelling-bee/${printDate}`,
            )
          }
        }}
      >
        <span className="visually-hidden">
          past puzzle from {printDate}
        </span>
        <div className={styles.puzzleCardWrapper}>
          {hasUserLoaded && !hasXwd && (
            <>
              <span className="visually-hidden">
                {' '}
                subscribe to play
              </span>
              <i className={styles.lockIcon} />
            </>
          )}
          {showFoldOver && (
            <>
              <div
                className={`${styles.playedFoldoverTop} ${
                  isInModalBody
                    ? styles.foldoverInModal
                    : styles.foldoverDefault
                }`}
                role="presentation"
              />
              <div
                className={styles.playedFoldoverBottom}
                role="presentation"
              />
            </>
          )}
          <div className={styles.puzzleCardContent}>
            {isProgressLoading ? (
              <>
                <div className={styles.loadingIcon} />
                <div className={styles.loadingText} />
              </>
            ) : (
              <>
                <i
                  className={
                    showOldIcons ? oldProgressIcon : progressIcon
                  }
                  role="presentation"
                />
                <div className={styles.progressText}>
                  {rank && (
                    <div className={styles.rankType}>
                      {isRevealed && 'Revealed: '}
                      {rank}
                    </div>
                  )}
                  <div className={styles.dayOfTheWeekType}>
                    {displayWeekday}
                  </div>
                </div>
              </>
            )}
          </div>
        </div>
      </a>
    </>
  )
}

export default PastPuzzleCard
