import React, { useState, useEffect, useCallback } from 'react'
import cn from 'classnames'
import { useDispatch } from 'react-redux'

import { wait } from '../../helpers/gameUtils'

import HiveCell from './HiveCell'
import HiveActions from './HiveActions'
import { useSbSelector } from './store/game/hooks'
import { shuffleLetters } from './store/game/game.actions'
import {
  deleteLetter,
  inputLetter,
  submitWord,
} from './store/api/thunks'

type HiveProps = {
  boundingParentRef: React.RefObject<HTMLDivElement>
}

const isLetter = (key: string) => {
  const re = new RegExp('^([a-z]{1}$)')
  return re.test(key)
}

const shuffleTransitionTime = 300 // This should match $shuffleTransitionTime in hive.scss

function Hive(props: HiveProps) {
  const isRevealed = useSbSelector((state) => state.game.isRevealed)

  const { boundingParentRef = { current: null } } = props

  const [activeKey, setActiveKey] = useState<string | null>(null)
  const [shuffleStage, setShuffleStage] = useState<
    'beforeShuffle' | 'shuffling' | 'afterShuffle' | null
  >(null)

  const centerLetter = useSbSelector(
    (state) => state.game.centerLetter,
  )

  const outerLetters = useSbSelector(
    (state) => state.game.outerLetters,
  )

  const currentMoment = useSbSelector(
    (state) => state.game.currentMoment,
  )
  const dispatch = useDispatch()

  const handleKeyup = () => {
    setActiveKey(null)
  }

  const handleShuffle = useCallback(() => {
    if (shuffleStage) return

    setShuffleStage('beforeShuffle') // Kick off first stage of shuffle animation and lock shuffle button
    wait(shuffleTransitionTime) // Wait for 'beforeShuffle' animation to finish
      .then(() => {
        setShuffleStage('shuffling') // Hide letters
        dispatch(shuffleLetters()) // Actually shuffle letters while they are hidden
      })
      .then(() => wait(10)) // Wait again so that letters don't appear before shuffling is done
      .then(() => setShuffleStage('afterShuffle')) // Now trigger 'afterShuffle' animation
      .then(() => wait(shuffleTransitionTime)) // Wait for animation to finish
      .then(() => setShuffleStage(null)) // Unlock shuffle button    },
  }, [shuffleStage])

  const handleKeydown = (e: KeyboardEvent) => {
    let key = null

    if (e.metaKey) return

    if (e.key === 'Backspace' || e.key === 'Delete') {
      key = 'delete'

      e.preventDefault()
      dispatch(deleteLetter())
    } else if (e.key === 'Enter') {
      key = 'submit'

      e.preventDefault()
      dispatch(submitWord())
    } else if (e.key === ' ') {
      key = 'shuffle'

      e.preventDefault()
      handleShuffle()
    } else {
      const key = e.key.toLowerCase()
      if (isLetter(key)) {
        dispatch(inputLetter(key))
      }
    }

    setActiveKey(key)
  }

  useEffect(() => {
    window.addEventListener('keydown', handleKeydown, false)
    window.addEventListener('keyup', handleKeyup, false)

    return () => {
      window.removeEventListener('keydown', handleKeydown)
      window.removeEventListener('keyup', handleKeyup)
    }
  }, [shuffleStage])

  const cells = [
    <HiveCell
      key={centerLetter}
      activeKey={activeKey}
      letter={centerLetter}
      onDown={() => dispatch(inputLetter(centerLetter))}
      onUp={() => handleKeyup()}
      type="center"
    />,
    ...[
      outerLetters.map((letter) => (
        <HiveCell
          key={letter}
          letter={letter}
          activeKey={activeKey}
          onDown={() => dispatch(inputLetter(letter))}
          onUp={() => handleKeyup()}
          type="outer"
        />
      )),
    ],
  ]

  return (
    <>
      <div className="sb-hive" data-testid="hive">
        <div
          className={cn('hive', {
            'fade-out': shuffleStage === 'beforeShuffle',
            shuffling: shuffleStage === 'shuffling',
            'fade-in': shuffleStage === 'afterShuffle',
            revealed: isRevealed,
          })}
        >
          {cells}
        </div>
      </div>
      <HiveActions
        activeKey={activeKey}
        handleShuffle={() => handleShuffle()}
        submitWord={() => dispatch(submitWord())}
        deleteLetter={() => dispatch(deleteLetter())}
        isVisible={currentMoment === 'game'}
        boundingParentRef={boundingParentRef}
        isRevealed={isRevealed}
      />
    </>
  )
}

export default Hive
