import React from 'react'
import cn from 'classnames'

import Button from '../../shared/Button'
import { debounce } from '../../helpers/gameUtils'
import { usePastPuzzles } from '../../../shared/abra/usePastPuzzles'

import { toggleModal } from './modals/index'
import TouchButton from './TouchButton'
import { useTracking } from './store/game/hooks'

type HiveActionsProps = {
  activeKey: string | null
  handleShuffle: () => void
  submitWord: () => void
  deleteLetter: () => void
  isVisible: boolean
  boundingParentRef: React.RefObject<HTMLElement>
  isRevealed: boolean
}

const getStyleNumberValue = (
  element: HTMLElement,
  property: string,
) =>
  window.getComputedStyle
    ? parseInt(
        window.getComputedStyle(element).getPropertyValue(property),
      ) || 0
    : 0

const getElementWidthWithMargin = (element: HTMLElement | null) => {
  if (!element) {
    return 0
  }

  const width = element.clientWidth
  const marginLeft = getStyleNumberValue(element, 'marginLeft')
  const marginRight = getStyleNumberValue(element, 'marginRight')
  return width + marginLeft + marginRight
}

const HiveActions = ({
  activeKey,
  deleteLetter,
  handleShuffle,
  submitWord,
  isVisible,
  boundingParentRef,
  isRevealed,
}: HiveActionsProps) => {
  const isPastPuzzlesActive = usePastPuzzles()
  const deleteRef = React.useRef<HTMLElement | null>(null)
  const shuffleRef = React.useRef<HTMLElement | null>(null)
  const submitRef = React.useRef<HTMLElement | null>(null)
  const [isVertical, setVertical] = React.useState(false)
  const trackSB = useTracking()

  const setVerticalIfTooWide = () => {
    if (!isVisible) {
      return
    }

    const totalHorizontalWidth: number = [
      deleteRef.current,
      shuffleRef.current,
      submitRef.current,
    ].reduce(
      (acc, el: HTMLElement | null) =>
        acc + getElementWidthWithMargin(el),
      0,
    )

    // this should only happen if font-size has been increased by the player using a11y settings
    const isTooWide = boundingParentRef?.current
      ? boundingParentRef.current.clientWidth < totalHorizontalWidth
      : false

    setVertical(isTooWide)
  }

  React.useEffect(() => {
    setVerticalIfTooWide()
    const debounced = debounce(setVerticalIfTooWide, 50)
    window.addEventListener('resize', debounced)
    return () => window.removeEventListener('resize', debounced)
  }, [isVisible])

  const GameButtons = (
    <div className={cn('hive-actions', { vertical: isVertical })}>
      <TouchButton
        className={cn('hive-action', 'hive-action__submit', {
          'action-active': activeKey === 'submit',
        })}
        onDown={() => submitWord()}
        outerRef={submitRef}
      >
        Enter
      </TouchButton>
      <TouchButton
        allowBurst
        className={cn('hive-action', 'hive-action__delete', {
          'action-active': activeKey === 'delete',
        })}
        onDown={() => deleteLetter()}
        outerRef={deleteRef}
      >
        Delete
      </TouchButton>
      <TouchButton
        className={cn('hive-action', 'hive-action__shuffle', {
          'action-active': activeKey === 'shuffle',
        })}
        onDown={() => handleShuffle()}
        outerRef={shuffleRef}
      />
    </div>
  )

  const PlayPastPuzzles = (
    <div className="sb-hive-play-past-puzzles">
      <Button
        text="Play past puzzles"
        type="secondary"
        onClick={() => {
          if (
            !!window.gamesAppPlatform ||
            !!window.newsreaderAppPlatform
          ) {
            // within hybrid webview launch modal with hub cards
            // rather than redirect user
            toggleModal('pastPuzzles')
          } else {
            trackSB('spelling-bee-hub', 'revealed-puzzle')
            window.location.href = '/puzzles/spelling-bee/hub'
          }
        }}
      />
    </div>
  )

  return isRevealed && isPastPuzzlesActive
    ? PlayPastPuzzles
    : GameButtons
}

export default HiveActions
