import React, {
  useRef,
  useState,
  useLayoutEffect,
  useEffect,
} from 'react'
import cn from 'classnames'

import { isMobile } from 'responsive'

import { useSbSelector } from './store/game/hooks'
import { selectIsRevealed } from './store/game/game.selectors'

const HiveInput = () => {
  const message = useSbSelector((state) => state.game.message)
  const input = useSbSelector((state) => state.game.input)
  const isRevealed = useSbSelector(selectIsRevealed)
  const centerLetter = useSbSelector(
    (state) => state.game.centerLetter,
  )

  const outerLetters = useSbSelector(
    (state) => state.game.outerLetters || [],
  )
  const validLetters = [centerLetter, ...outerLetters]

  const containerRef = useRef<HTMLDivElement>(null)
  const hiddenInputRef = useRef<HTMLSpanElement>(null)

  const [scale, setScale] = useState(1)
  const [showPlaceholder, setShowPlaceholder] = useState(true)

  useLayoutEffect(() => {
    if (!containerRef.current || !hiddenInputRef.current) {
      return
    }

    const { width: containerWidth } =
      containerRef.current.getBoundingClientRect()

    const { width: fullSizeTextWidth } =
      hiddenInputRef.current.getBoundingClientRect()

    // if the full-sized text exceeds the width of this component, scale the text down to fit
    if (fullSizeTextWidth > containerWidth) {
      setScale(containerWidth / fullSizeTextWidth)
    } else {
      setScale(1)
    }
  }, [input])

  useEffect(() => {
    // placeholder is removed once user types a value
    if (input.length > 0 || isRevealed) setShowPlaceholder(false)
  }, [input, isRevealed])

  // renders either our regular visible input,
  // or a hidden input used to measure and potentially scale down the font-size
  const renderContent = (
    size: number,
    ref?: React.RefObject<HTMLSpanElement>,
  ) => {
    return (
      <span
        className={cn('sb-hive-input-content', {
          'has-error': message && message.isError,
          'non-empty': input.length > 0,
          'is-accepting': message && !message.isError,
        })}
        style={{ fontSize: `${size}em` }}
        ref={ref}
      >
        {/* display a placeholder for desktop to let user know 
        they can use their keyboard */}
        {showPlaceholder && !isMobile && (
          <span className="sb-hive-input-placeholder">
            Type or click
          </span>
        )}
        {input.map((l, i) => (
          <span
            key={l + i}
            className={cn({
              'sb-input-bright': centerLetter === l,
              'sb-input-invalid': !validLetters.includes(l),
            })}
          >
            {l}
          </span>
        ))}
      </span>
    )
  }

  return (
    <div className="sb-hive-input" ref={containerRef}>
      {renderContent(scale)}
      <div className="sb-hive-hidden-input">
        {renderContent(1, hiddenInputRef)}
      </div>
    </div>
  )
}

export default HiveInput
