import React from 'react'

import { useUser } from 'user'
import { isMobile } from 'responsive'

import DropDown from '../../shared/DropDown/DropDown'
import DropDownItem from '../../shared/DropDown/DropDownItem'
import ToolbarIcon from '../../shared/DropDown/ToolbarIcon'
import { usePersistHide } from '../../shared/hooks/use-persist-hide'
import { usePastPuzzles } from '../../../shared/abra/usePastPuzzles'
import ToolbarAdapter from '../../shared/ToolbarAdapter/ToolbarAdapter'
import createHybridNewsLink from '../../shared/utils/createHybridNewsLink'
import useFeatureFlag from '../../../shared/abra/useFeatureFlag'

import { useSbSelector, useTracking } from './store/game/hooks'
import { ModalButton, toggleModal } from './modals/index'
import { createForumLink } from './createForumLink'
import { ToolbarTooltip } from './ToolbarTooltip'
import { selectIsPastPuzzle } from './store/game/game.selectors'

const Toolbar = () => {
  const track = useTracking()
  const { userType } = useUser()
  const printDate = useSbSelector((state) => state.game.printDate)
  const yesterday = useSbSelector((state) => state.game.yesterday)
  const isPastPuzzle = useSbSelector(selectIsPastPuzzle)
  const showNewYAFlow = userType.hasXwd
  const momentIsOpen = useSbSelector(
    (state) => state.game.currentMoment !== 'game',
  )

  const enableLifetimeStats = useFeatureFlag('sbLifetimeStats')

  const hideLifetimeStatsTooltip = usePersistHide(
    'sb-lifetime-stats-tooltip',
    2,
  )

  const tooltipButtonRef = React.useRef<HTMLAnchorElement | null>(
    null,
  )

  const showPastPuzzles = usePastPuzzles()
  const isPlayingInApp =
    !!window.gamesAppPlatform || !!window.newsreaderAppPlatform

  const yesterdaysPuzzle = yesterday
    ? createHybridNewsLink(
        `/puzzles/spelling-bee/${yesterday.printDate}`,
      )
    : ''

  const yesterdaysAnswersModalButton = yesterday && (
    <ModalButton
      className="pz-toolbar-button pz-toolbar-button__yesterday"
      modal="yesterday"
      triggerName="toolbar"
    >
      {isMobile ? 'Yesterday' : 'Yesterday’s Answers'}
    </ModalButton>
  )

  const newYesterdaysAnswersModalButton = yesterday && (
    <ModalButton
      className="pz-toolbar-button pz-toolbar-button__yesterday"
      modal="yesterdayNew"
      triggerName="toolbar"
    >
      {isMobile ? 'Yesterday' : 'Yesterday’s Answers'}
    </ModalButton>
  )

  const yesterdaysPuzzleLink = yesterday && (
    // eslint-disable-next-line jsx-a11y/anchor-is-valid,jsx-a11y/click-events-have-key-events,jsx-a11y/no-static-element-interactions
    <a
      className="pz-toolbar-button pz-toolbar-button__yesterday"
      onClick={() => {
        track('yesterday', 'toolbar')
        window.location.href = yesterdaysPuzzle
      }}
    >
      {isMobile ? 'Yesterday' : 'Play Yesterday’s Puzzle'}
      <ToolbarIcon icon="external" />
    </a>
  )

  const answersModalButton = (
    <ModalButton
      className="pz-toolbar-button pz-toolbar-button__answers"
      modal="answers"
      triggerName="toolbar"
    >
      Answers
    </ModalButton>
  )

  // Logic for yesterdays answers vs yesterdays puzzle vs answers
  const getAnswersOrYesterdayCTA = () => {
    if (!showPastPuzzles) {
      return yesterdaysAnswersModalButton
    }

    if (isPastPuzzle) {
      return answersModalButton
    }

    if (showNewYAFlow) {
      return newYesterdaysAnswersModalButton
    }

    return yesterdaysPuzzleLink
  }

  return (
    <ToolbarAdapter isHidden={momentIsOpen}>
      <div className="pz-toolbar-left" />
      <div className="pz-toolbar-right">
        <ModalButton
          className="pz-toolbar-button pz-toolbar-button__stats"
          modal="stats"
          triggerName="toolbar"
          buttonRef={tooltipButtonRef}
          onClick={() => {
            if (enableLifetimeStats) {
              hideLifetimeStatsTooltip.hide()
            }
          }}
        >
          Stats
        </ModalButton>
        {getAnswersOrYesterdayCTA()}
        {!isMobile && (
          <a
            className="pz-toolbar-button pz-toolbar-button__hints"
            href={createForumLink(printDate)}
            target={window.isHybridWebView ? '_self' : '_blank'}
            rel="noreferrer"
            onClick={() => track('todays-hints', 'toolbar')}
          >
            Hints
            <ToolbarIcon icon="external" />
          </a>
        )}
        <DropDown buttonLabel="More">
          <DropDownItem
            type="button"
            value="How to Play"
            buttonAction={() => toggleModal('help', 'toolbar')}
          />
          <DropDownItem
            type="button"
            value="Rankings"
            buttonAction={() => toggleModal('ranks', 'toolbar')}
          />
          {isMobile && (
            <DropDownItem
              type="navigation"
              value="Hints"
              url={createForumLink(printDate)}
              buttonAction={() => track('todays-hints', 'toolbar')}
              icon="external"
            />
          )}
          <DropDownItem
            type="navigation"
            value="Community"
            url={`${createForumLink(printDate)}#commentsContainer`}
            buttonAction={() => track('community', 'toolbar')}
            icon="external"
          />
          {showPastPuzzles && (
            <DropDownItem
              type={isPlayingInApp ? 'button' : 'navigation'}
              value="Past Puzzles"
              url={
                isPlayingInApp
                  ? undefined
                  : '/puzzles/spelling-bee/hub'
              }
              buttonAction={() => {
                track('spelling-bee-hub', 'toolbar')
                if (isPlayingInApp) {
                  toggleModal('pastPuzzles')
                }
              }}
              icon="external"
            />
          )}
        </DropDown>
      </div>
      {enableLifetimeStats && (
        <ToolbarTooltip
          isHidden={hideLifetimeStatsTooltip.isHidden}
          persistHide={hideLifetimeStatsTooltip.hide}
          targetRef={tooltipButtonRef}
        />
      )}
    </ToolbarAdapter>
  )
}

export default Toolbar
