import React from 'react'
import cn from 'classnames'
import { CSSTransition } from 'react-transition-group'
import { createPortal } from 'react-dom'
import {
  useFloating,
  arrow,
  FloatingArrow,
  autoUpdate,
} from '@floating-ui/react'

import { trackImpression } from 'foundation'

import useTimeout from '../../shared/hooks/use-timeout'
import Pill from '../../shared/Pill'
import useWindowSize from '../../shared/hooks/use-window-size'

import { useSbSelector, useTracking } from './store/game/hooks'
import styles from './ToolbarTooltip.module.scss'

type Props = {
  targetRef: { current: HTMLElement | null }
  persistHide: () => void
  isHidden: boolean
}

/**
 * NOTE: this tooltip component is not intended for reuse
 */
export const ToolbarTooltip = ({
  targetRef,
  persistHide,
  isHidden,
}: Props) => {
  const size = useWindowSize()
  const overridePositioning = size.winWidth <= 440
  const tooltipRoot = document.getElementById('portal-sb-tooltip')
  const [isGameVisible, setGameVisible] = React.useState(false)
  const currentMoment = useSbSelector(
    (state) => state.game.currentMoment,
  )

  const isModalOpen = useSbSelector((state) => state.game.isModalOpen)
  const puzzleId = useSbSelector((state) => state.game.id)

  const trackSB = useTracking()

  const tooltipIsHidden =
    isHidden ||
    !targetRef.current ||
    !isGameVisible ||
    currentMoment !== 'game' ||
    isModalOpen

  // fade in tooltip only after moment transition has concluded
  useTimeout(
    () => setGameVisible(true),
    currentMoment === 'game' ? 1200 : null,
  )

  React.useEffect(() => {
    if (!tooltipIsHidden) {
      trackImpression(
        'tool-tip',
        'spelling-bee',
        'lifetime-stats', // TODO: update once we have a spec
        `SB-${puzzleId}`,
      )
    }
  }, [tooltipIsHidden])

  const close = () => {
    trackSB('tool-tip', 'close')
    persistHide()
  }

  // escape key closes tooltip
  React.useEffect(() => {
    if (tooltipIsHidden) {
      return
    }

    const listener = (e: KeyboardEvent) => {
      if (e.key === 'Escape') {
        close()
      }
    }

    window.addEventListener('keydown', listener)
    return () => window.removeEventListener('keydown', listener)
  }, [tooltipIsHidden])

  const arrowRef = React.useRef<SVGSVGElement | null>(null)
  // this library handles positioning for us
  const floating = useFloating({
    middleware: [arrow({ element: arrowRef })],
    whileElementsMounted: window.ResizeObserver
      ? autoUpdate
      : undefined,
  })

  // tell the library about the button from the toolbar that we are targetting
  React.useEffect(() => {
    if (targetRef.current) {
      floating.refs.setReference(targetRef.current)
    }
  })

  React.useEffect(() => {
    floating.update()
  }, [tooltipIsHidden])

  const [arrowX, setArrowX] = React.useState(0)
  const arrowWidth = 20

  // position the arrow manually (without the floating library) if we are overriding positioning on small devices
  React.useEffect(() => {
    if (!overridePositioning || tooltipIsHidden) {
      return
    }

    const setArrowPosition = () => {
      // rectangle around button that tooltip is pointing to
      const targetRect = targetRef.current?.getBoundingClientRect()

      // rectangle around tooltip box
      const tooltipRect =
        floating.refs.floating.current?.getBoundingClientRect()

      if (targetRect && tooltipRect) {
        const x =
          targetRect.left + // left edge of target
          targetRect.width / 2 - // center of target
          arrowWidth / 2 - // adjust for half of the arrow width
          tooltipRect.x // adjust for positioning of tooltip box
        setArrowX(x)
      }
    }

    setArrowPosition()
    window.addEventListener('resize', setArrowPosition)
    return () =>
      window.removeEventListener('resize', setArrowPosition)
  }, [tooltipIsHidden])

  if (!tooltipRoot) {
    return null
  }

  const title = 'More Stats'

  const subtitle =
    'See your total puzzles, words, pangrams, and more.'

  const tooltip = (
    <CSSTransition
      in={!tooltipIsHidden}
      unmountOnExit
      timeout={200}
      classNames={{
        exit: styles.exit,
        exitActive: styles.exitActive,
        enter: styles.enter,
        enterActive: styles.enterActive,
      }}
    >
      <div
        ref={floating.refs.setFloating}
        className={styles.tooltip}
        style={{
          position: floating.strategy,
          top: overridePositioning ? 0 : Math.round(floating.y ?? 0),
          left: overridePositioning ? 'auto' : floating.x ?? 0,
          right: overridePositioning ? 20 : undefined,
        }}
      >
        <FloatingArrow
          ref={arrowRef}
          context={floating.context}
          width={arrowWidth}
          height={10}
          fill="#2a2a2a"
          staticOffset={arrowX}
        />
        <div
          role="button"
          className={styles.close}
          onClick={() => close()}
        >
          ×
        </div>
        <div>
          <Pill text="New" className={styles.pill} />
          <div className={cn(styles.text, styles.bold)}>{title}</div>
          <div className={styles.text}>{subtitle}</div>
        </div>
      </div>
    </CSSTransition>
  )

  return createPortal(tooltip, tooltipRoot)
}
