import React from 'react'
import { connect } from 'react-redux'
import cn from 'classnames'

import { isMobile } from 'responsive'

import { throttle, debounce } from '../../helpers/gameUtils'
import { handleNavigation } from '../../../shared/components/Pagination/handleScroll'

import Anagram from './Anagram'
import { useSbSelector } from './store/game/hooks'
import { selectTodayPangramSet } from './store/game/game.selectors'
import type { SpellingBeeState } from './store'

type WordlistBaseProps = {
  words: string[]
}

export const WordlistBase = ({ words }: WordlistBaseProps) => {
  const containerRef = React.useRef<HTMLDivElement | null>(null)
  const lastItemRef = React.useRef<HTMLLIElement | null>(null)
  const [numScrollAnchors, setNumScrollAnchors] = React.useState(1)
  const [activePage, setActivePage] = React.useState(0)
  const pangramSet = useSbSelector(selectTodayPangramSet)

  const showKebobs = numScrollAnchors > 1
  const showKebobArrows = showKebobs && !isMobile

  const handleScroll = React.useCallback(
    throttle(() => {
      if (!containerRef.current) {
        return
      }

      const newActivePage = Math.round(
        containerRef.current.scrollLeft /
          containerRef.current.offsetWidth,
      )
      setActivePage(newActivePage)
    }, 100),
    [],
  )

  const calculateNumScrollAnchors = React.useCallback(() => {
    if (!containerRef.current) {
      return
    }

    // when no words are present, there is no lastItemRef
    if (!lastItemRef.current) {
      setNumScrollAnchors(1)
      return
    }

    const scrollAnchorWidth = containerRef.current.offsetWidth
    if (!scrollAnchorWidth) {
      return // avoid a divide by 0
    }

    const containerScrollLeft = containerRef.current.scrollLeft

    const { left: containerLeft } =
      containerRef.current.getBoundingClientRect()

    const { right: lastItemRight } =
      lastItemRef.current.getBoundingClientRect()

    const listWidth =
      lastItemRight + containerScrollLeft - containerLeft

    const newNumScrollAnchors = Math.ceil(
      listWidth / scrollAnchorWidth,
    )

    if (newNumScrollAnchors > 0) {
      setNumScrollAnchors(newNumScrollAnchors)
    }
  }, [setNumScrollAnchors])

  // recalculate on each render, since this needs to be triggered on the moment transition
  React.useEffect(calculateNumScrollAnchors)

  // recalculate when window is resized
  React.useEffect(() => {
    const debouncedCalc = debounce(calculateNumScrollAnchors, 100)
    window.addEventListener('resize', debouncedCalc)
    return () => window.removeEventListener('resize', debouncedCalc)
  }, [])

  return (
    <>
      <div
        className="sb-wordlist-pag"
        ref={containerRef}
        onScroll={handleScroll}
      >
        {[...Array(numScrollAnchors)].map((_, i) => (
          <div
            key={i.toString()}
            className="sb-wordlist-scroll-anchor"
            style={{ left: `${100 * i}%` }}
          />
        ))}
        <ul
          className={cn('sb-wordlist-items-pag', {
            single: words.length === 1,
          })}
        >
          {words.map((w, i) => (
            <li
              key={w}
              ref={i === words.length - 1 ? lastItemRef : undefined}
            >
              <Anagram value={w} isPangram={pangramSet.has(w)} />
            </li>
          ))}
        </ul>
      </div>
      <div className="sb-kebob">
        {showKebobArrows && (
          <button
            type="button"
            onClick={() => handleNavigation(containerRef, 'prev')}
            className={cn('sb-bob-arrow', 'sb-bob-prev', {
              active: activePage > 0,
            })}
          >
            <span className="sb-bob-text">Prev</span>
          </button>
        )}
        {showKebobs &&
          [...Array(numScrollAnchors)].map((_, index) => (
            <div
              className={cn('sb-bob', {
                active: index === activePage,
              })}
              key={index.toString()}
            />
          ))}
        {showKebobArrows && (
          <button
            type="button"
            onClick={() => handleNavigation(containerRef, 'next')}
            className={cn('sb-bob-arrow', 'sb-bob-next', {
              active: activePage < numScrollAnchors - 1,
            })}
          >
            <span className="sb-bob-text">Next</span>
          </button>
        )}
      </div>
    </>
  )
}

const mapStateToProps = (state: SpellingBeeState) => ({
  words: state.game.foundWords
    .slice()
    .sort((a, b) => (a > b ? 1 : -1)),
})

export default connect(mapStateToProps)(WordlistBase)
