import React from 'react'
import { Provider } from 'react-redux'
import {
  act,
  render,
  fireEvent,
  waitFor,
} from '@testing-library/react'

import Hive from '../Hive'
import { shuffleLetters } from '../store/game/game.actions'
import {
  submitWord,
  deleteLetter,
  inputLetter,
} from '../store/api/thunks'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  getGameData: () => {
    return { id: 0, today: { id: 0 } }
  },
}))

jest.mock('../store/api/thunks', () => ({
  submitWord: jest.fn(),
  deleteLetter: jest.fn(),
  inputLetter: jest.fn(),
}))

jest.mock('../store/game/game.actions', () => ({
  shuffleLetters: jest.fn(),
}))

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('foundation', () => ({
  trackClick: jest.fn(),
  userType: {
    hasXwd: true,
  },
  getGameData: () => ({
    today: { id: 1 },
  }),
}))

const centerLetter = 'C'
const outerLetters = ['T', 'I', 'P', 'A', 'Y', 'V']
const dispatch = jest.fn()
const subscribe = jest.fn()

function makeStore(centerLetter, outerLetters, isRevealed = false) {
  return {
    getState: () => ({
      game: {
        centerLetter,
        outerLetters,
        isRevealed,
      },
    }),
    subscribe,
    dispatch,
  }
}

const renderHive = (centerLetter, outerLetters, isRevealed) => {
  const store = makeStore(centerLetter, outerLetters, isRevealed)
  return render(
    <Provider store={store}>
      <Hive />
    </Provider>,
  )
}

describe('SPELLING BEE: HIVE', () => {
  it('should render a hive shape', () => {
    const hive = renderHive(centerLetter, outerLetters, false)
    expect(hive.queryByTestId('hive')).toBeTruthy()
    expect(hive.queryAllByTestId('hive-cell')).toHaveLength(7)
  })

  it('should show center letter in center hive', () => {
    const hive = renderHive(centerLetter, outerLetters, false)
    const hiveCells = hive.queryAllByTestId('hive-cell')

    expect(hiveCells[0].textContent).toBe('C')
  })

  it('should show outer letters in outer hives', () => {
    const hive = renderHive(centerLetter, outerLetters, false)
    const hiveCells = hive.queryAllByTestId('hive-cell')

    expect(hiveCells[1].textContent).toBe('T')
    expect(hiveCells[2].textContent).toBe('I')
    expect(hiveCells[3].textContent).toBe('P')
    expect(hiveCells[4].textContent).toBe('A')
    expect(hiveCells[5].textContent).toBe('Y')
    expect(hiveCells[6].textContent).toBe('V')
  })

  it('should have correct actions', () => {
    const hive = renderHive(centerLetter, outerLetters, false)
    const touchButtons = hive.queryAllByTestId('touch-button')

    expect(touchButtons).toHaveLength(3)

    expect(touchButtons[0].textContent).toContain('Enter')
    expect(touchButtons[1].textContent).toContain('Delete')
    expect(touchButtons[2].textContent).toContain('')
  })

  it('should show "Play past puzzles" button when revealed', () => {
    const hive = renderHive(centerLetter, outerLetters, true)
    const touchButtons = hive.queryAllByTestId('touch-button')
    const buttons = hive.queryAllByRole('button')

    expect(touchButtons).toHaveLength(0)
    expect(buttons).toHaveLength(1)

    expect(buttons[0].textContent).toContain('Play past puzzles')
  })
})

describe('SPELLING BEE: KEY PRESSDOWNS', () => {
  afterEach(() => {
    jest.resetAllMocks()
  })

  it('should input letter on letter input', () => {
    renderHive(centerLetter, outerLetters, false)
    fireEvent.keyDown(document, { key: 'A', code: 'KeyA' })

    expect(inputLetter).toHaveBeenCalled()
    expect(inputLetter.mock.calls).toEqual([['a']])
  })

  it('should submit word on enter', () => {
    renderHive(centerLetter, outerLetters, false)
    fireEvent.keyDown(document, { key: 'Enter', code: 'Enter' })

    expect(submitWord).toHaveBeenCalled()
  })

  it('should delete letter on delete', () => {
    renderHive(centerLetter, outerLetters, false)
    fireEvent.keyDown(document, { key: 'Delete', code: 'Delete' })

    expect(deleteLetter).toHaveBeenCalled()
  })

  it('should delete letter on backspace', () => {
    renderHive(centerLetter, outerLetters, false)
    fireEvent.keyDown(document, {
      key: 'Backspace',
      code: 'Backspace',
    })

    expect(deleteLetter).toHaveBeenCalled()
  })

  it('should shuffle on space', async () => {
    await act(async () => {
      const hive = renderHive(centerLetter, outerLetters, false)
      const hiveDiv = hive.container.firstElementChild

      setTimeout(() => {
        fireEvent.keyDown(hiveDiv, { key: ' ', code: ' ' })
      }, 300)

      await waitFor(() => {
        expect(shuffleLetters).toHaveBeenCalled()
      })
    })
  })

  it('should do nothing on number input', () => {
    renderHive(centerLetter, outerLetters, false)
    fireEvent.keyDown(document, { key: '1', code: 'Key1' })

    expect(inputLetter).not.toHaveBeenCalled()
  })

  it('should do nothing on F-key input', () => {
    renderHive(centerLetter, outerLetters, false)
    fireEvent.keyDown(document, { key: 'F1', code: 'F1' })

    expect(inputLetter).not.toHaveBeenCalled()
  })

  it('should do nothing on non F-key non-number non-letter input', () => {
    renderHive(centerLetter, outerLetters, false)
    fireEvent.keyDown(document, { key: 'Escape', code: 'Escape' })

    expect(inputLetter).not.toHaveBeenCalled()
  })
})
