import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import HiveInput from '../HiveInput'

const dispatch = jest.fn()
const subscribe = jest.fn()

jest.mock('responsive', () => ({
  isMobile: false,
}))

function makeStore(centerLetter, outerLetters, input, message) {
  return {
    getState: () => ({
      game: {
        centerLetter,
        outerLetters,
        input,
        message,
      },
    }),
    subscribe,
    dispatch,
  }
}

describe('HIVEINPUT: CORRECT TAGS', () => {
  const getWrapper = (store) =>
    mount(
      <Provider store={store}>
        <HiveInput />
      </Provider>,
    )

  it('should exist', () => {
    const store = makeStore(
      'C',
      ['T', 'I', 'P', 'A', 'Y', 'V'],
      ['C', 'I', 'T', 'Y'],
      {
        isError: true,
      },
    )
    const wrapper = getWrapper(store)

    expect(wrapper.find('.sb-hive-input')).toHaveLength(1)
    expect(wrapper.find('.sb-hive-input-content')).toHaveLength(2)
  })

  it('should have special tag for center letter only', () => {
    const store = makeStore(
      'C',
      ['T', 'I', 'P', 'A', 'Y', 'V'],
      ['C', 'I', 'T', 'Y'],
      {
        isError: true,
      },
    )
    const wrapper = getWrapper(store)

    expect(wrapper.find('.sb-input-bright')).toHaveLength(2)
    const letter = wrapper.find('.sb-input-bright').first()
    expect(letter.text()).toEqual('C')
  })

  it('should have special tag for invalid letters', () => {
    const store = makeStore(
      'C',
      ['T', 'I', 'P', 'A', 'Y', 'V'],
      ['Z', 'I', 'T', 'Y'],
      {
        isError: true,
      },
    )
    const wrapper = getWrapper(store)

    expect(wrapper.find('.sb-input-invalid')).toHaveLength(2)
    const letter = wrapper.find('.sb-input-invalid').first()
    expect(letter.text()).toEqual('Z')
  })

  it('should have special tag for explicit error message', () => {
    const store = makeStore(
      'C',
      ['T', 'I', 'P', 'A', 'Y', 'V'],
      ['Z', 'I', 'T', 'Y'],
      {
        isError: true,
      },
    )
    const wrapper = getWrapper(store)

    expect(wrapper.find('.has-error')).toHaveLength(2)
  })

  it('should have special tag for explicit non-error message', () => {
    const store = makeStore(
      'C',
      ['T', 'I', 'P', 'A', 'Y', 'V'],
      ['Z', 'I', 'T', 'Y'],
      {
        isError: false,
      },
    )
    const wrapper = getWrapper(store)

    expect(wrapper.find('.is-accepting')).toHaveLength(2)
  })

  describe('HIVEINPUT: PLACEHOLDER', () => {
    it('should render a placeholder', () => {
      // initially there is no input (an empty array)
      const store = makeStore(
        'C',
        ['T', 'I', 'P', 'A', 'Y', 'V'],
        [],
        {
          isError: false,
        },
      )

      // render the content
      const wrapper = mount(
        <Provider store={store}>
          <HiveInput />
        </Provider>,
      )

      // placeholder is rendered (twice because of the hidden input)
      expect(wrapper.find('.sb-hive-input-placeholder')).toHaveLength(
        2,
      )
    })

    it('should remove the placeholder on input', () => {
      // there is an input
      const store = makeStore(
        'C',
        ['T', 'I', 'P', 'A', 'Y', 'V'],
        ['P'],
        {
          isError: false,
        },
      )

      // render the content
      const wrapper = mount(
        <Provider store={store}>
          <HiveInput />
        </Provider>,
      )

      // placeholder is *not* rendered
      expect(wrapper.find('.sb-hive-input-placeholder')).toHaveLength(
        0,
      )
    })
  })
})
