import { mount } from 'enzyme'
import React from 'react'
import { act } from 'react-dom/test-utils'
import { Provider } from 'react-redux'

import './domSetup'
import { setUpStore } from '../store'
import { insertFoundWords } from '../store/game/game.actions'
import Layout from '../Layout'

import { mockGameData } from './mockGameData'

jest.mock('../../../helpers/FeedbackLink', () =>
  jest.fn().mockReturnValue(<span>Email us!</span>),
)

jest.mock('../../../shared/moments/components/LoadingBar', () =>
  jest.fn().mockReturnValue(<div>Loading</div>),
)

jest.mock('../Hive', () => jest.fn().mockReturnValue(<div>Hive</div>))

jest.mock('../Toolbar', () =>
  jest.fn().mockReturnValue(<div>Toolbar</div>),
)

jest.mock('../../../helpers/gameUtils', () => ({
  ...jest.requireActual('../../../helpers/gameUtils'),
  shuffle: jest.fn(),
  wait: jest.fn().mockImplementation(() => Promise.resolve()),
}))

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('foundation', () => ({
  trackInteraction: jest.fn(),
  mobileNavTools: {
    create: jest.fn(),
    activate: jest.fn(),
  },
  abra: {
    get: jest.fn(),
    reportExposure: jest.fn(),
  },
  appendSupportedParams: jest.fn(),
  getGameData: () => require('./mockGameData').mockGameData,
  env: {
    account: 'https://myaccount.nytimes.com',
  },
  track: jest.fn(),
  trackPageUpdate: jest.fn(),
  getFeedbackLink: jest.fn().mockResolvedValue('href'),
  SEND_EMAIL_TYPE: { SUGGESTION: 'mock-suggestion' },
}))

jest.mock('user', () => ({
  useUser: () => ({
    userType: {
      hasXwd: true,
    },
    hasLoaded: true,
  }),
}))

jest.mock('../hooks/useProgress', () => ({
  useProgress: jest.fn(),
}))

function mountLayout(store) {
  return mount(
    <Provider store={store}>
      <Layout />
    </Provider>,
  )
}

describe('LAYOUT: MOMENTS & MODALS', () => {
  let wrapper

  beforeEach(() => {
    jest.useFakeTimers()
    wrapper = mountLayout(setUpStore())
  })

  it('loads the loading moment on initialization', () => {
    const loadingMoment = wrapper.find('.pz-moment__loading')
    expect(loadingMoment).toBeTruthy()
  })

  it('loads the welcome moment on initialization', () => {
    const startMoment = wrapper.find('.pz-moment').first()
    expect(startMoment.find('.pz-moment__title').text()).toEqual(
      'Spelling Bee',
    )
    expect(startMoment.find('.pz-moment__button').text()).toEqual(
      'Play',
    )
  })

  it('loads the rankings modal on clicking the progress bar', () => {
    // unlock the gameboard by closing the welcome moment
    jest.runAllTimers()
    wrapper
      .find('.pz-moment')
      .first()
      .find('.pz-moment__button')
      .simulate('click')
    expect(wrapper.find('.sb-game-locked')).toHaveLength(0)

    wrapper.find('.sb-progress').simulate('click')
    expect(wrapper.find('.sb-game-locked')).toHaveLength(1)
    jest.runAllTimers()

    const openModalHtml = wrapper.find('.sb-modal-wrapper').html()
    expect(openModalHtml).toContain('Rankings')

    // confirm the modal can be closed
    document.querySelector('.sb-modal-close').click()
    jest.runAllTimers()

    const closedModalHtml = wrapper.find('.sb-modal-wrapper').html()
    expect(closedModalHtml).not.toContain('Rankings')
  })
})

describe('LAYOUT: RANKS', () => {
  let store
  let wrapper

  const expectations = [
    ['Beginner', 0],
    ['Good', 2],
    ['Solid', 4],
    ['Nice', 10],
    ['Great', 20],
    ['Amazing', 25],
    ['Genius', 35],
  ]

  beforeEach(() => {
    store = setUpStore()
    wrapper = mountLayout(store)
  })

  const { answers } = mockGameData.today

  test.each(expectations)(
    `with ${answers.length} possible right answers, displays "%s" with %i found answer(s)`,
    (rank, count) => {
      store.dispatch(insertFoundWords(answers.slice(0, count)))
      expect(wrapper.find('.sb-progress-rank').text()).toEqual(rank)
    },
  )
})

describe('LAYOUT: OFFLINE TICKER', () => {
  let onlineGetter
  let store
  const { addEventListener } = window
  const eventMap = {}
  const tickerVisibleSelector = '.pz-offline-ticker.is-offline'

  beforeAll(() => {
    onlineGetter = jest.spyOn(window.navigator, 'onLine', 'get')
    store = setUpStore()
    window.addEventListener = jest
      .fn()
      .mockImplementation((event, cb) => {
        eventMap[event] = cb
      })
  })

  afterAll(() => {
    jest.clearAllMocks()
    window.addEventListener = addEventListener
  })

  it('displays offline ticker on hybrid while offline', () => {
    window.isHybridWebView = true
    onlineGetter.mockReturnValue(false)
    const wrapper = mountLayout(store)
    expect(wrapper.exists(tickerVisibleSelector)).toBeTruthy()
  })

  it('does not display offline ticker if not on hybrid', () => {
    window.isHybridWebView = false
    onlineGetter.mockReturnValue(false)
    const wrapper = mountLayout(store)
    expect(wrapper.exists(tickerVisibleSelector)).toBeFalsy()
  })

  it('does not display offline ticker if online', () => {
    window.isHybridWebView = true
    onlineGetter.mockReturnValue(true)
    const wrapper = mountLayout(store)
    expect(wrapper.exists(tickerVisibleSelector)).toBeFalsy()
  })

  it('displays offline ticker if connection is lost, and hides it once connection comes back', () => {
    window.isHybridWebView = true
    onlineGetter.mockReturnValue(true)
    const wrapper = mountLayout(store)
    expect(wrapper.exists(tickerVisibleSelector)).toBeFalsy()

    act(() => {
      eventMap.offline()
    })

    setTimeout(() => {
      expect(wrapper.exists(tickerVisibleSelector)).toBeTruthy()
    }, 0)

    act(() => {
      eventMap.online()
    })

    setTimeout(() => {
      expect(wrapper.exists(tickerVisibleSelector)).toBeFalsy()
    }, 0)
  })
})
