import React from 'react'
import { mount } from 'enzyme'

import { Progress } from '../Progress'

const mockToggleModal = jest.fn()

jest.mock('../modals/createModalSystem', () => {
  const createModalSystem = jest.requireActual(
    '../modals/createModalSystem',
  ).default

  return function createMockModalSystem(
    modals,
    ModalWrap,
    modalRoot,
  ) {
    const mockSystem = createModalSystem(modals, ModalWrap, modalRoot)
    mockSystem.toggleModal = function toggleModal(...args) {
      return mockToggleModal(...args)
    }

    return mockSystem
  }
})

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('foundation', () => ({
  trackClick: jest.fn(),
  userType: {
    hasXwd: true,
  },
  getGameData: () => ({
    today: { id: 1 },
  }),
  env: { account: 'https://myaccount.nytimes.com' },
}))

function makeProgressProps(score, rank, rankIdx) {
  return {
    score,
    rank,
    rankIdx,
    ranksUpToGenius: [
      { title: 'Beginner', minScore: 0 },
      { title: 'Good Start', minScore: 2 },
      { title: 'Moving Up', minScore: 5 },
      { title: 'Good', minScore: 8 },
      { title: 'Solid', minScore: 15 },
      { title: 'Nice', minScore: 25 },
      { title: 'Great', minScore: 40 },
      { title: 'Amazing', minScore: 50 },
      { title: 'Genius', minScore: 69 },
    ],
  }
}

describe('PROGRESS: WITH RANKINGS MODAL', () => {
  it('calls triggerModal with "ranks" and "in-game" arguments on click', () => {
    const progressProps = makeProgressProps(0, 'Beginner', 0)
    const wrapper = mount(<Progress {...progressProps} />)
    expect(mockToggleModal).not.toHaveBeenCalled()
    wrapper.find('.sb-progress').simulate('click')
    expect(mockToggleModal).toHaveBeenCalledWith('ranks', 'in-game')
  })
})

describe('PROGRESS: SHOWS PROPER PROGRESS', () => {
  it('should be correct for Rank 1', () => {
    const progressProps = makeProgressProps(0, 'Beginner', 0)

    const wrapper = mount(<Progress {...progressProps} />)

    expect(wrapper.find('.sb-progress-rank')).toHaveLength(1)
    expect(wrapper.find('.sb-progress-rank').text()).toEqual(
      'Beginner',
    )
    expect(wrapper.find('.sb-progress-value').text()).toEqual('0')
    expect(wrapper.find('.completed')).toHaveLength(0)
  })

  it('should be correct for Rank 4', () => {
    const progressProps = makeProgressProps(20, 'Solid', 4)
    const wrapper = mount(<Progress {...progressProps} />)

    expect(wrapper.find('.sb-progress-rank')).toHaveLength(1)
    expect(wrapper.find('.sb-progress-rank').text()).toEqual('Solid')
    expect(wrapper.find('.sb-progress-value').text()).toEqual('20')
    expect(wrapper.find('.completed')).toHaveLength(4)
  })

  it('should be correct for Rank 8', () => {
    const progressProps = makeProgressProps(70, 'Genius', 8)
    const wrapper = mount(<Progress {...progressProps} />)

    expect(wrapper.find('.sb-progress-rank')).toHaveLength(1)
    expect(wrapper.find('.sb-progress-rank').text()).toEqual('Genius')
    expect(wrapper.find('.sb-progress-value').text()).toEqual('70')
    expect(wrapper.find('.completed')).toHaveLength(8)
  })

  it('should have the correct number of progress dots for Genius', () => {
    const progressProps = makeProgressProps(70, 'Genius', 8)
    const wrapper = mount(<Progress {...progressProps} />)

    expect(wrapper.find('.sb-progress-dot')).toHaveLength(9)
  })

  it('Queen Bee should have the same number of progress dots as Genius', () => {
    const progressProps = makeProgressProps(100, 'Queen Bee', 9)
    const wrapper = mount(<Progress {...progressProps} />)

    expect(wrapper.find('.sb-progress-dot')).toHaveLength(9)
  })
})
