import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import Toolbar from '../Toolbar'
import { SB_FORUM, createForumLink } from '../createForumLink'

Object.defineProperty(window, 'location', {
  value: {
    href: '',
  },
  writable: true,
})

const mockToggleModal = jest.fn()

jest.mock('../modals/createModalSystem', () => {
  const createModalSystem = jest.requireActual(
    '../modals/createModalSystem',
  ).default

  return function createMockModalSystem(
    modals,
    ModalWrap,
    modalRoot,
  ) {
    const mockSystem = createModalSystem(modals, ModalWrap, modalRoot)
    mockSystem.toggleModal = function toggleModal(...args) {
      return mockToggleModal(...args)
    }

    return mockSystem
  }
})

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('foundation', () => ({
  getGameData: () => ({
    today: { id: 1 },
  }),
  env: {
    name: 'dev',
  },
  trackInteraction: jest.fn(),
}))

jest.mock('react-dom', () => ({
  createPortal: (jsx) => jsx,
}))

const mockStore = (state) => {
  return {
    default: jest.fn(),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
    getState: () => state,
  }
}

describe('TOOLBAR: SHOWS PROPER STYLING FOR DESKTOP AND MOBILE', () => {
  const puzzleData = {
    game: {
      isPastPuzzle: true,
      printDate: '2021-09-24',
      yesterday: {
        printDate: '2021-09-23',
      },
    },
  }

  const createComponent = (puzzleData) => {
    const store = mockStore(puzzleData)

    const wrapper = mount(
      <Provider store={store}>
        <Toolbar />
      </Provider>,
    )

    return wrapper
  }

  beforeEach(() => {
    jest.resetAllMocks()
  })

  it('should include Stats, Answers, Today’s Hints and More buttons for Desktop', () => {
    const wrapper = createComponent(puzzleData)
    const statsButton = wrapper.find('ModalButton').at(0)
    const yesterdayAnswersButton = wrapper.find('ModalButton').at(1)

    expect(statsButton).toHaveLength(1)
    expect(statsButton.prop('modal')).toEqual('stats')

    expect(yesterdayAnswersButton).toHaveLength(1)
    expect(yesterdayAnswersButton.prop('modal')).toEqual('answers')
    expect(yesterdayAnswersButton.childAt(0).text()).toEqual(
      'Answers',
    )

    const todaysHintsButton = wrapper.find(
      '.pz-toolbar-button__hints',
    )
    expect(todaysHintsButton).toHaveLength(1)
    expect(todaysHintsButton.childAt(0).text()).toEqual('Hints')

    const MoreButton = wrapper.find('DropDown')
    expect(MoreButton).toHaveLength(1)
    expect(MoreButton.prop('buttonLabel')).toEqual('More')
  })

  it('should not show yesterday button if yesterday data isnt available', () => {
    const wrapper = createComponent({
      game: { printDate: '2021-09-24' },
    })

    const modalButtons = wrapper.find('ModalButton')
    expect(modalButtons).toHaveLength(1)
    expect(modalButtons.at(0).prop('modal')).toEqual('stats')
  })

  it('should show correct link', () => {
    createComponent(puzzleData)
    const validDateLink = createForumLink('2021-09-30')
    const invalidDateLink = createForumLink('date')
    expect.assertions(2)
    expect(validDateLink.includes('2021/09/30')).toBe(true)
    expect(invalidDateLink).toBe(SB_FORUM)
  })

  it('should call toggle modal when clicked', () => {
    const wrapper = createComponent(puzzleData)
    const statsButton = wrapper.find('ModalButton').at(0)
    expect(statsButton.prop('modal')).toEqual('stats')

    statsButton.simulate('click')
    expect(mockToggleModal).toHaveBeenCalled()
    expect(mockToggleModal).toHaveBeenCalledWith('stats', 'toolbar')
  })

  it('should open answers modal when answers button clicked', () => {
    const wrapper = createComponent(puzzleData)
    const answersButton = wrapper.find('ModalButton').at(1)
    expect(answersButton.prop('modal')).toEqual('answers')

    answersButton.simulate('click')
    expect(mockToggleModal).toHaveBeenCalled()
    expect(mockToggleModal).toHaveBeenCalledWith('answers', 'toolbar')
  })

  it('if feature flag is on and on a past puzzle, show Answers in toolbar', () => {
    const wrapper = createComponent(puzzleData)
    const answersButton = wrapper.find('ModalButton').at(1)

    expect(answersButton).toHaveLength(1)
    expect(answersButton.prop('modal')).toEqual('answers')
    expect(answersButton.childAt(0).text()).toEqual('Answers')
  })

  it('if feature flag is on and not on a past puzzle, show Yesterday’s Puzzle in toolbar', () => {
    const todaysPuzzle = { ...puzzleData }
    todaysPuzzle.game.isPastPuzzle = false
    const wrapper = createComponent(todaysPuzzle)
    const yesterdaysPuzzleButton = wrapper.find(
      '.pz-toolbar-button__yesterday',
    )

    expect(yesterdaysPuzzleButton).toHaveLength(1)
    expect(yesterdaysPuzzleButton.childAt(0).text()).toEqual(
      'Play Yesterday’s Puzzle',
    )
    yesterdaysPuzzleButton.prop('onClick')()
    expect(window.location.href).toEqual(
      '/puzzles/spelling-bee/2021-09-23',
    )
  })

  it('if in the newsreader and in a hybrid web view force hybrid on yesterdays puzzle link', () => {
    window.newsreaderAppPlatform = 'ios'
    window.isHybridWebView = true

    const todaysPuzzle = { ...puzzleData }
    todaysPuzzle.game.isPastPuzzle = false
    const wrapper = createComponent(todaysPuzzle)
    const yesterdaysPuzzleButton = wrapper.find(
      '.pz-toolbar-button__yesterday',
    )

    expect(yesterdaysPuzzleButton).toHaveLength(1)
    yesterdaysPuzzleButton.prop('onClick')()
    expect(window.location.href).toEqual(
      '/puzzles/spelling-bee/2021-09-23?hybrid=news',
    )
  })

  it('should toggle past puzzle modal in apps', () => {
    window.newsreaderAppPlatform = 'ios'
    window.isHybridWebView = true

    const wrapper = createComponent(puzzleData)
    wrapper.find('.pz-dropdown__button').at(0).simulate('click')

    const pastPuzzlesCTA = wrapper.find('DropDownItem').at(3)
    expect(pastPuzzlesCTA.prop('value')).toEqual('Past Puzzles')
    expect(pastPuzzlesCTA.prop('type')).toEqual('button')
    pastPuzzlesCTA.prop('buttonAction')()
    expect(mockToggleModal).toHaveBeenCalledWith(
      'pastPuzzles',
      undefined,
    )
  })
})
