import moment from 'moment'
import MockDate from 'mockdate'
import { AnyAction, Store } from 'redux'
import { ThunkDispatch } from 'redux-thunk'

import { getGameData, UserType, trackInteraction } from 'foundation'

import { devToolsTest } from '../../../helpers/__tests__/boilerplate'
import { setUpStore, SpellingBeeState } from '../store'
import {
  toggleWordlist,
  shuffleLetters,
  updateUser,
  setYesterdaysProgress,
  updateModalRedux,
  revealAnswers,
} from '../store/game/game.actions'
import {
  inputLetter,
  deleteLetter,
  submitWord,
  SUCCESS_MSG_DURATION,
  ERROR_MSG_DURATION,
  MAX_INPUT_LENGTH,
} from '../store/api/thunks'
import {
  selectRanks,
  selectMaxScore,
  selectCurrentScore,
  selectCurrentRankIdx,
  selectCurrentRank,
  selectHasWon,
  selectHasCompleted,
  selectYesterdayPangramSet,
  selectYesterdayFoundWordSet,
  selectHasReachedCutoff,
} from '../store/game/game.selectors'
import { momentSystem } from '../moments/index'

jest.mock('../../js/moments', () => ({
  momentSystem: {
    getCurrentMoment: jest.fn(),
    transition: jest.fn(),
    reanimateMoment: jest.fn(),
  },
}))

jest.mock('../../js/modals/index', () => ({
  toggleModal: jest.fn(),
}))

jest.mock('../../js/gating', () => ({
  CUTOFF_RANK_IDX: 4,
}))

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  trackInteraction: jest.fn(),
  abra: {
    get: jest.fn(),
    reportExposure: jest.fn(),
  },
  getGameData: () => {
    const centerLetter = 'n'
    const outerLetters = 'cehitk'
    const pangramAnswer = 'kitchen'
    const goodAnswer = 'cent'
    const niceAnswer = 'chink'
    const awesomeAnswer = 'nineteenth'

    const p1 = {
      id: 123,
      centerLetter,
      outerLetters: outerLetters.split(''),
      validLetters: ['n', ...outerLetters.split('')],
      pangrams: [pangramAnswer],
      answers: [pangramAnswer, goodAnswer, niceAnswer, awesomeAnswer],
      foundWords: [awesomeAnswer],
      printDate: '2023-04-30',
    }

    return { yesterday: p1, today: p1, isPastPuzzle: true }
  },
}))

jest.mock('user', () => ({
  useUser: () => ({
    userType: {
      hasXwd: true,
      inShortzMode: false,
    },
    hasLoaded: true,
  }),
}))

const {
  answers,
  outerLetters: ol,
  centerLetter,
  pangrams,
} = getGameData().today
const outerLetters = ol.join('')

const [firstAnswer, secondAnswer, thirdAnswer, awesomeAnswer] =
  answers

const pangramAnswer = firstAnswer
const goodAnswer = secondAnswer
const niceAnswer = thirdAnswer

const failures = [
  ['Too\u00a0short', outerLetters.slice(0, 2)],
  ['Bad\u00a0letters', '0123456789'],
  ['Missing\u00a0center\u00a0letter', outerLetters.slice(0, 5)],
  [
    'Not\u00a0in\u00a0word\u00a0list',
    outerLetters.slice(0, 4) + centerLetter,
  ],
  ['Already\u00a0found', firstAnswer],
]

const successes: Array<[string, string, number, boolean]> = [
  ['Good!', goodAnswer, 1, false],
  ['Nice!', niceAnswer, niceAnswer.length, false],
  ['Awesome!', awesomeAnswer, awesomeAnswer.length, false],
  ['Pangram!', pangramAnswer, pangramAnswer.length + 7, true],
]

describe('SPELLING BEE: STORE', () => {
  let store: Store<SpellingBeeState>
  let dispatch: ThunkDispatch<SpellingBeeState, null, AnyAction>
  let isDev: boolean

  beforeEach(() => {
    // @ts-ignore TODO: get rid of whole __DEV__ concept
    isDev = global.__DEV__
    // @ts-ignore
    global.__DEV__ = false

    jest.resetAllMocks()
    jest.useFakeTimers()
    store = setUpStore()
    dispatch = store.dispatch
    dispatch(updateUser({ hasXwd: true } as UserType))
  })

  afterEach(() => {
    // @ts-ignore
    global.__DEV__ = isDev
    MockDate.reset()
  })

  devToolsTest(() => setUpStore())

  it('should initialize state correctly', () => {
    const state = store.getState()

    const totalScore =
      pangramAnswer.length +
      7 +
      goodAnswer.length / goodAnswer.length +
      niceAnswer.length +
      awesomeAnswer.length

    expect(selectMaxScore(state)).toBe(totalScore)
    expect(selectCurrentScore(state)).toBe(0)
    expect(selectCurrentRankIdx(state)).toBe(0)
    expect(selectCurrentRank(state)).toBe('Beginner')
    expect(selectHasWon(state)).toBe(false)
    expect(selectHasCompleted(state)).toBe(false)
    expect(selectYesterdayFoundWordSet(state).size).toBe(0)
    expect(selectYesterdayPangramSet(state)).toContain(pangrams[0])
    expect(selectYesterdayPangramSet(state).size).toBe(
      pangrams.length,
    )
    expect(selectHasReachedCutoff(state)).toBe(false)
    expect(selectRanks(state)).toEqual([
      { title: 'Beginner', minScore: 0 },
      { title: 'Good Start', minScore: 1 },
      { title: 'Moving Up', minScore: 2 },
      { title: 'Good', minScore: 2 },
      { title: 'Solid', minScore: 5 },
      { title: 'Nice', minScore: 8 },
      { title: 'Great', minScore: 12 },
      { title: 'Amazing', minScore: 15 },
      { title: 'Genius', minScore: 21 },
      { title: 'Queen Bee', minScore: 30 },
    ])

    expect(state.game.input).toEqual([])
    expect(state.game.foundWords).toEqual([])
    expect(state.game.message).toEqual(null)

    // because we shuffle up front
    expect([...state.game.outerLetters].sort()).toEqual(
      outerLetters.split('').sort(),
    )

    expect(state.game.yesterday?.foundWords).toEqual(null)
  })

  it('should be able to shuffle', () => {
    // do this a lot to ensure the next is another derrangement
    for (let i = 0; i < 1000; i += 1) {
      const before = store.getState()
      store.dispatch(shuffleLetters())
      const after = store.getState()
      expect(before).not.toEqual(after)
    }
  })

  it('should shuffle on set up', () => {
    const state = store.getState()
    expect(state.game.outerLetters).not.toEqual(
      outerLetters.split(''),
    )
  })

  it('should allow users to cheat in dev', () => {
    // @ts-ignore
    global.__DEV__ = true
    const s = setUpStore()
    const d: ThunkDispatch<SpellingBeeState, null, AnyAction> =
      s.dispatch
    d(updateUser({ hasXwd: true, inShortzMode: true } as UserType))

    // cheat

    d(inputLetter(centerLetter))
    d(submitWord())
    expect(s.getState().game.foundWords).toEqual([firstAnswer])

    // reset

    d(submitWord(outerLetters))

    expect(s.getState().game.foundWords).toEqual([])
  })

  it('should allow users to cheat with window method', () => {
    // @ts-ignore
    global.__DEV__ = true
    const s = setUpStore()
    s.dispatch(updateUser({ hasXwd: true } as UserType))

    window.cheat?.()

    expect(s.getState().game.foundWords).toEqual([firstAnswer])

    window.reset?.()

    expect(s.getState().game.foundWords).toEqual([])
  })

  it('should allow users to cheat if they are inShortzMode', () => {
    const s = setUpStore()
    const d: ThunkDispatch<SpellingBeeState, null, AnyAction> =
      s.dispatch
    d(updateUser({ hasXwd: true, inShortzMode: true } as UserType))

    // cheat

    d(inputLetter(centerLetter))
    d(inputLetter(centerLetter))
    d(submitWord())

    expect(s.getState().game.foundWords).toEqual([
      firstAnswer,
      secondAnswer,
    ])

    // reset

    d(submitWord(outerLetters))

    expect(s.getState().game.foundWords).toEqual([])
  })

  it('should not let other people cheat', () => {
    const s = setUpStore()
    const d: ThunkDispatch<SpellingBeeState, null, AnyAction> =
      s.dispatch
    d(updateUser({ hasXwd: true, inShortzMode: false } as UserType))

    // enter a word

    d(submitWord(thirdAnswer))

    // try to cheat

    d(inputLetter(centerLetter))
    d(inputLetter(centerLetter))
    d(submitWord())

    expect(s.getState().game.foundWords).toEqual([thirdAnswer])
  })

  successes.forEach(([successMessage, word, points, isPangram]) => {
    it(`should accept word [${word}]`, (done) => {
      word.split('').forEach((l) => {
        dispatch(inputLetter(l))
      })

      dispatch(submitWord())

      expect(store.getState().game.foundWords).toEqual([word])
      expect(store.getState().game.message).toEqual({
        value: successMessage,
        points,
        isPangram,
      })

      jest.advanceTimersByTime(SUCCESS_MSG_DURATION)

      process.nextTick(() => {
        expect(store.getState().game.message).toEqual(null)
        done()
      })
    })
  })

  failures.forEach(([errorMessage, word]) => {
    it(`should reject input [${errorMessage}]`, (done) => {
      dispatch(submitWord(firstAnswer))
      dispatch(submitWord(word))

      expect(store.getState().game.foundWords).toEqual([firstAnswer])
      expect(store.getState().game.message).toEqual({
        value: errorMessage,
        isError: true,
      })

      jest.advanceTimersByTime(ERROR_MSG_DURATION)

      process.nextTick(() => {
        expect(store.getState().game.message).toEqual(null)
        done()
      })
    })
  })

  it('should freeze input and warn after MAX_INPUT_LENGTH + 1 characters', (done) => {
    for (let i = 0; i < 24; i += 1) {
      dispatch(inputLetter(centerLetter))
    }

    expect(store.getState().game.input).toHaveLength(
      MAX_INPUT_LENGTH + 1,
    )
    expect(store.getState().game.message).toEqual({
      value: 'Too long',
      isError: true,
    })

    jest.advanceTimersByTime(ERROR_MSG_DURATION)
    process.nextTick(() => {
      expect(store.getState().game.input).toEqual([])
      done()
    })
  })

  it('should handle deleting letters', () => {
    dispatch(inputLetter('a'))
    dispatch(inputLetter('b'))

    expect(store.getState().game.input).toEqual(['a', 'b'])

    dispatch(deleteLetter())

    expect(store.getState().game.input).toEqual(['a'])

    dispatch(deleteLetter())

    expect(store.getState().game.input).toEqual([])

    dispatch(deleteLetter())

    expect(store.getState().game.input).toEqual([])
  })

  it('should not input letter if revealed', () => {
    dispatch(revealAnswers())
    dispatch(inputLetter('a'))
    expect(store.getState().game.input).toEqual([])
  })

  it('submitting an empty word should not show a message', () => {
    dispatch(submitWord())
    expect(store.getState().game.message).toEqual(null)
  })

  it('should only accept letters when unlocked', () => {
    dispatch(inputLetter('a'))
    expect(store.getState().game.input).toEqual(['a'])

    dispatch(updateModalRedux(true))
    dispatch(inputLetter('b'))
    expect(store.getState().game.input).toEqual(['a'])

    dispatch(updateModalRedux(false))
    dispatch(inputLetter('c'))
    expect(store.getState().game.input).toEqual(['a', 'c'])
  })

  it('should only delete letters when unlocked', () => {
    dispatch(inputLetter('a'))
    expect(store.getState().game.input).toEqual(['a'])
    dispatch(deleteLetter())
    expect(store.getState().game.input).toEqual([])

    dispatch(inputLetter('b'))
    dispatch(updateModalRedux(true))
    dispatch(deleteLetter())
    expect(store.getState().game.input).toEqual(['b'])

    dispatch(updateModalRedux(false))
    dispatch(deleteLetter())
    expect(store.getState().game.input).toEqual([])
  })

  it('should only submit words when unlocked', () => {
    dispatch(inputLetter('c'))
    dispatch(inputLetter('e'))
    dispatch(inputLetter('n'))
    dispatch(inputLetter('t'))
    dispatch(updateModalRedux(true))
    dispatch(submitWord())
    expect(store.getState().game.message).toEqual(null)
    expect(store.getState().game.input).toEqual(['c', 'e', 'n', 't'])
  })

  function submitTooShortWord() {
    dispatch(inputLetter('a'))
    dispatch(inputLetter('b'))
    dispatch(submitWord())
  }

  it('after invalid submission, input should not clear immediately', () => {
    submitTooShortWord()
    expect(store.getState().game.input).toEqual(['a', 'b'])
  })

  it('when user short-circuits error message with input, message & input should be cleared, and only once', (done) => {
    submitTooShortWord()
    dispatch(inputLetter('a'))
    expect(store.getState().game.input).toEqual(['a'])
    jest.advanceTimersByTime(ERROR_MSG_DURATION)
    process.nextTick(() => {
      expect(store.getState().game.input).toEqual(['a'])
      done()
    })
  })

  it('when user short-circuits error message with delete key, message & input should be cleared, and only once', (done) => {
    submitTooShortWord()
    dispatch(deleteLetter())
    expect(store.getState().game.input).toEqual([])
    expect(store.getState().game.message).toEqual(null)
    dispatch(inputLetter('a'))
    jest.advanceTimersByTime(ERROR_MSG_DURATION)
    process.nextTick(() => {
      expect(store.getState().game.input).toEqual(['a'])
      done()
    })
  })

  it('success messages should *not* be short-circuited', () => {
    dispatch(inputLetter('c'))
    dispatch(inputLetter('e'))
    dispatch(inputLetter('n'))
    dispatch(inputLetter('t'))
    dispatch(submitWord())
    dispatch(inputLetter('c'))
    expect(store.getState().game.message).not.toEqual(null)
  })

  it('on valid submission, input should clear immediately and not again after message clears', (done) => {
    dispatch(inputLetter('c'))
    dispatch(inputLetter('e'))
    dispatch(inputLetter('n'))
    dispatch(inputLetter('t'))
    expect(store.getState().game.input).toEqual(['c', 'e', 'n', 't'])
    dispatch(submitWord())
    expect(store.getState().game.input).toEqual([])
    dispatch(inputLetter('a'))
    jest.advanceTimersByTime(SUCCESS_MSG_DURATION)
    process.nextTick(() => {
      expect(store.getState().game.input).toEqual(['a'])
      done()
    })
  })

  it('handles toggling of mobile found word list', () => {
    expect(store.getState().game.isExpandedWordlist).toBe(false)

    store.dispatch(toggleWordlist())
    expect(store.getState().game.isExpandedWordlist).toBe(true)

    store.dispatch(toggleWordlist())
    expect(store.getState().game.isExpandedWordlist).toBe(false)
  })

  it('cuts the a non-sub off once they pass the gating index', () => {
    dispatch(
      updateUser({
        hasXwd: false,
      } as UserType),
    )

    expect(selectHasReachedCutoff(store.getState())).toBe(false)

    dispatch(submitWord(niceAnswer))

    expect(selectHasReachedCutoff(store.getState())).toBe(true)
  })

  it('allows a user to log in after cutoff', () => {
    // @ts-ignore
    momentSystem.getCurrentMoment.mockImplementationOnce(
      () => 'cutoff',
    )

    dispatch(
      updateUser({
        hasXwd: false,
      } as UserType),
    )

    dispatch(
      updateUser({
        hasXwd: true,
      } as UserType),
    )

    expect(momentSystem.transition).toHaveBeenCalledWith(
      'cutoff',
      'game',
    )
  })

  it('allows a user to log in at welcome moment', () => {
    // @ts-ignore
    momentSystem.getCurrentMoment.mockImplementationOnce(
      () => 'welcome',
    )

    dispatch(
      updateUser({
        hasXwd: false,
      } as UserType),
    )

    dispatch(
      updateUser({
        hasXwd: true,
      } as UserType),
    )
    // @ts-ignore
    expect(momentSystem.reanimateMoment).toHaveBeenCalled()
  })

  it('allows a subscriber to play entire game', () => {
    dispatch(
      updateUser({
        hasXwd: true,
      } as UserType),
    )

    expect(selectHasReachedCutoff(store.getState())).toBe(false)

    answers.forEach((word: string) => {
      dispatch(submitWord(word))
    })

    expect(selectHasReachedCutoff(store.getState())).toBe(false)
  })

  it('opens congrats moment a bit after user has reached congrats', () => {
    dispatch(
      updateUser({
        hasXwd: true,
      } as UserType),
    )

    answers.forEach((word: string) => {
      dispatch(submitWord(word))
    })

    expect(momentSystem.transition).not.toHaveBeenCalled()

    jest.advanceTimersByTime(500)

    expect(momentSystem.transition).toHaveBeenCalledWith(
      'game',
      'congrats',
    )
  })

  it.only('hybrid web view: opens congrats moment a bit after user has reached cutoff', () => {
    window.isHybridWebView = true

    dispatch(
      updateUser({
        hasXwd: false,
      } as UserType),
    )

    dispatch(submitWord(niceAnswer))

    expect(momentSystem.transition).not.toHaveBeenCalled()

    jest.advanceTimersByTime(500)

    expect(momentSystem.transition).toHaveBeenCalledWith(
      'game',
      'cutoff',
    )
  })

  it.only('newsreader app: opens congrats moment a bit after user has reached cutoff', () => {
    window.newsreaderAppPlatform = 'ios'

    dispatch(
      updateUser({
        hasXwd: false,
      } as UserType),
    )

    dispatch(submitWord(niceAnswer))

    expect(momentSystem.transition).not.toHaveBeenCalled()

    jest.advanceTimersByTime(500)

    expect(momentSystem.transition).toHaveBeenCalledWith(
      'game',
      'cutoff',
    )
  })

  it.only('web view: opens congrats moment a bit after user has reached cutoff', () => {
    window.isHybridWebView = false
    window.newsreaderAppPlatform = ''

    dispatch(
      updateUser({
        hasXwd: false,
      } as UserType),
    )

    dispatch(submitWord(niceAnswer))

    expect(momentSystem.transition).not.toHaveBeenCalled()

    jest.advanceTimersByTime(500)

    expect(momentSystem.transition).not.toHaveBeenCalled()
  })

  it('handles edge case where yesterdays progress is set with `undefined`', () => {
    // @ts-ignore
    dispatch(setYesterdaysProgress())
    expect(store.getState().game.yesterday?.foundWords).toEqual([])
    expect(store.getState().game.yesterday?.isRevealed).toEqual(false)
  })

  it('fires a module interaction when a user advances', () => {
    // trackInteraction.mockReset()
    dispatch(submitWord(goodAnswer))
    // @ts-ignore
    expect(trackInteraction.mock.calls).toEqual([
      [
        {
          element: {
            name: 'spelling-bee',
            label: 'level-up',
          },
          context: 'SB-123',
          name: {
            p: '2023-04-30',
            t: 'archive',
          },
          label: '1-good-start',
        },
      ],
    ])

    answers.forEach((word: string) => {
      if (word !== goodAnswer) {
        dispatch(submitWord(word))
      }
    })

    const createTestEvent = (label: string) => ({
      element: {
        name: 'spelling-bee',
        label: 'level-up',
      },
      context: 'SB-123',
      name: {
        p: '2023-04-30',
        t: 'archive',
      },
      label,
    })

    const events = [
      [createTestEvent('1-good-start')],
      [createTestEvent('2-moving-up')],
      [createTestEvent('3-good')],
      [createTestEvent('4-solid')],
      [createTestEvent('5-nice')],
      [createTestEvent('6-great')],
      [createTestEvent('7-amazing')],
      [createTestEvent('8-genius')],
      [createTestEvent('9-queen-bee')],
    ]
    events.forEach((e) => {
      // @ts-ignore
      expect(trackInteraction.mock.calls).toContainEqual(e)
    })
  })

  it('fires a module interaction when a finds a pangram', () => {
    dispatch(submitWord(pangramAnswer))

    // @ts-ignore
    expect(trackInteraction.mock.calls).toContainEqual([
      {
        element: {
          name: 'spelling-bee',
          label: 'pangram',
        },
        context: 'SB-123',
        name: {
          p: '2023-04-30',
          t: 'archive',
        },
        label: undefined,
      },
    ])
  })

  it('fires an analytics event to track time in mobile word list', () => {
    MockDate.set(
      moment('2018-05-24 00:00:01', 'YYYY-MM-DD HH:mm:ss').toDate(),
    )

    store.dispatch(toggleWordlist())

    MockDate.set(
      moment('2018-05-24 00:00:10', 'YYYY-MM-DD HH:mm:ss').toDate(),
    )

    store.dispatch(toggleWordlist())
    expect(trackInteraction).toHaveBeenCalledWith({
      element: {
        name: 'spelling-bee',
        label: 'word-list',
      },
      context: 'SB-123',
      name: {
        p: '2023-04-30',
        t: 'archive',
      },
      label: 9000,
    })
  })
})
