import { render, waitFor } from '@testing-library/react'
import React from 'react'
import { Provider } from 'react-redux'

import { trackPageUpdate } from 'foundation'

import {
  EVENT_ON_NAVIGATE_BACK,
  GamesNativeBridge,
} from '../../../../shared/foundation/js/hybrid/commands'
import { setUpStore } from '../store'
import UserContext from '../../../../shared/user/UserContext'
import { defaultUserType } from '../../../../shared/user/userHelpers'
import { momentSystem } from '../moments'
import { isModalOpen, toggleModal } from '../modals'
import { TOGGLE_WORDLIST } from '../store/game/game.actions'

import { useSBHybridEffects } from './useSBHybridEffects'

jest.mock('foundation', () => ({
  trackPageUpdate: jest.fn(),
  trackInteraction: jest.fn(),
  env: {},
  getGameData: () =>
    require('../__tests__/mockGameData').mockGameData,
}))

jest.mock('../moments', () => ({
  momentSystem: {
    load: jest.fn(),
    getCurrentMoment: jest.fn(() => ''),
    transition: jest.fn(),
  },
}))

jest.mock('../modals', () => ({
  isModalOpen: jest.fn(() => false),
  toggleModal: jest.fn(),
}))

describe('useSBHybridEffects', () => {
  let element: Element

  const mockHybrid = (
    isReturningFromBackground = false,
    success = true,
  ) => {
    element = document.createElement('div')

    window.isHybridWebView = true
    const mockBridge: Partial<GamesNativeBridge> = {
      gamesInitializeState: jest.fn(() =>
        Promise.resolve({
          success,
          error: '',
          values: {
            gamesInitializeState: {
              isReturningFromBackground,
            },
          },
        }),
      ),
      addEventListener: element.addEventListener.bind(element),
      removeEventListener: element.removeEventListener.bind(element),
    }
    window.NativeBridge = mockBridge as GamesNativeBridge

    return mockBridge
  }

  const Component = () => {
    useSBHybridEffects()
    return null
  }

  const updateUserType = jest.fn()

  const MockUserProvider = ({
    children,
  }: {
    children: React.ReactNode
  }) => {
    return (
      <UserContext.Provider
        value={{
          updateUserType,
          userType: defaultUserType,
          hasLoaded: true,
          shortzHasLoaded: true,
        }}
      >
        {children}
      </UserContext.Provider>
    )
  }

  const renderHook = (expandWordlist = false) => {
    const store = setUpStore()
    if (expandWordlist) {
      store.dispatch({ type: TOGGLE_WORDLIST })
    }

    const component = render(
      <Provider store={store}>
        <MockUserProvider>
          <Component />
        </MockUserProvider>
      </Provider>,
    )

    return { store, component }
  }

  afterEach(() => {
    window.isHybridWebView = undefined
    window.NativeBridge = undefined
    window.newsreaderAppPlatform = ''
    jest.resetAllMocks()
  })

  it('should track page update on ios newsreader', () => {
    mockHybrid()
    window.newsreaderAppPlatform = 'ios'
    renderHook()
    expect(trackPageUpdate).toHaveBeenCalled()
  })

  it('should not track page update on other surfaces', () => {
    mockHybrid()
    renderHook()
    expect(trackPageUpdate).not.toHaveBeenCalled()
  })

  it('should load loading moment when not in hybrid', () => {
    renderHook()
    expect(momentSystem.load).toHaveBeenCalledTimes(1)
    expect(momentSystem.load).toHaveBeenCalledWith('loading')
  })

  it('should load game moment when returning from background', async () => {
    mockHybrid(true)
    renderHook()
    await waitFor(() => expect(momentSystem.load).toHaveBeenCalled())
    expect(momentSystem.load).toHaveBeenCalledWith('game')
  })

  it('should load loading moment when bridge errors', async () => {
    mockHybrid(false, false)
    renderHook()
    await waitFor(() => expect(momentSystem.load).toHaveBeenCalled())
    expect(momentSystem.load).toHaveBeenCalledWith('loading')
  })

  it('should load loading moment when not returning from background', async () => {
    mockHybrid()
    renderHook()
    await waitFor(() => expect(momentSystem.load).toHaveBeenCalled())
    expect(momentSystem.load).toHaveBeenCalledWith('loading')
  })

  const dispatchUserEvent = () => {
    const e = new Event('gamesUserCredentialChanged')
    // @ts-ignore
    e.detail = {
      values: {
        gamesAuthenticateUser: {
          isUserLoggedIn: true,
          isSubscribed: true,
          regiID: '12345',
        },
      },
    }
    window.dispatchEvent(e)
  }

  it('should update user type per Native Bridge event', () => {
    mockHybrid()
    renderHook()
    dispatchUserEvent()
    expect(updateUserType).toHaveBeenCalled()
    expect(updateUserType.mock.calls[0][0].regiId).toEqual('12345')
  })

  it('should not update user type if component has unmounted', () => {
    mockHybrid()
    const { component } = renderHook()
    component.unmount()
    dispatchUserEvent()
    expect(updateUserType).not.toHaveBeenCalled()
  })

  const respondWith = jest.fn()
  const dispatchNavigateBackEvent = () => {
    const e = new Event(EVENT_ON_NAVIGATE_BACK)
    // @ts-ignore
    e.respondWith = respondWith
    element.dispatchEvent(e)
  }

  it('should default to native natigation on android back button', () => {
    mockHybrid()
    renderHook()
    dispatchNavigateBackEvent()

    expect(respondWith).toHaveBeenCalledWith({
      [EVENT_ON_NAVIGATE_BACK]: false,
    })
  })

  it('should close modal on android back button', () => {
    // @ts-ignore
    isModalOpen.mockReturnValueOnce(true)
    mockHybrid()
    renderHook()
    dispatchNavigateBackEvent()

    expect(toggleModal).toHaveBeenCalled()
    expect(respondWith).toHaveBeenCalledWith({
      [EVENT_ON_NAVIGATE_BACK]: true,
    })
  })

  it('should close congrats moment on android back button', () => {
    // @ts-ignore
    momentSystem.getCurrentMoment.mockReturnValueOnce('congrats')
    mockHybrid()
    renderHook()
    dispatchNavigateBackEvent()

    expect(momentSystem.transition).toHaveBeenCalled()
    expect(respondWith).toHaveBeenCalledWith({
      [EVENT_ON_NAVIGATE_BACK]: true,
    })
  })

  it('should collapse wordlist on android back button', () => {
    mockHybrid()
    const { store } = renderHook(true)
    dispatchNavigateBackEvent()

    expect(store.getState().game.isExpandedWordlist).toEqual(false)
    expect(respondWith).toHaveBeenCalledWith({
      [EVENT_ON_NAVIGATE_BACK]: true,
    })
  })
})
