import React from 'react'
import { useDispatch } from 'react-redux'

import { trackPageUpdate } from 'foundation'

import { momentSystem } from '../moments'
import { handleUserInformationUpdate } from '../../../helpers/hybridLinkHelpers'
import { useUser } from '../../../../shared/user'
import {
  BridgeListener,
  EVENT_ON_NAVIGATE_BACK,
} from '../../../../shared/foundation/js/hybrid/commands'
import { isModalOpen, toggleModal } from '../modals'
import { useSbSelector } from '../store/game/hooks'
import { toggleWordlist } from '../store/game/game.actions'
import useGameDataHybridEvent from '../../../shared/hooks/use-game-data-hybrid-event'
import { sbTrackingContext } from '../spellingBeeTracker'
import { selectWordlistExpanded } from '../store/game/game.selectors'

const useHybridEffect = (
  effect: React.EffectCallback,
  deps?: React.DependencyList,
) => {
  React.useEffect(() => {
    if (window.isHybridWebView && window.NativeBridge) {
      return effect()
    }
  }, deps)
}

const useNonHybridEffect = (
  effect: React.EffectCallback,
  deps?: React.DependencyList,
) => {
  React.useEffect(() => {
    if (!window.isHybridWebView || !window.NativeBridge) {
      return effect()
    }
  }, deps)
}

export const useSBHybridEffects = () => {
  const { updateUserType } = useUser()
  const isWordlistExpanded = useSbSelector(selectWordlistExpanded)
  const puzzleID = useSbSelector((state) => state.game.id)
  const dispatch = useDispatch()

  useHybridEffect(() => {
    if (window.newsreaderAppPlatform === 'ios') {
      trackPageUpdate()
    }
  }, [])

  // bypass the welcome moment if we're returning after having been backgrounded
  useHybridEffect(() => {
    window.NativeBridge?.gamesInitializeState()
      .then((data) => {
        if (!data.success) throw new Error(data.error)

        const {
          gamesInitializeState: { isReturningFromBackground },
        } = data.values

        if (isReturningFromBackground) {
          momentSystem.load('game')
        } else {
          momentSystem.load('loading')
        }
      })
      .catch((e) => {
        console.error('Failed to get initial state', e)
        momentSystem.load('loading')
      })
  }, [])

  // if we're not in hybrid, show the loading moment
  useNonHybridEffect(() => {
    momentSystem.load('loading')
  }, [])

  // keep UserProvider in sync with native app
  // NOTE: does this belong in UserProvider, instead?
  useHybridEffect(() => {
    const handleCredentialChanged = (event: CustomEvent) => {
      const userData = event?.detail?.values.gamesAuthenticateUser
      handleUserInformationUpdate(userData, updateUserType)
    }

    window.addEventListener(
      'gamesUserCredentialChanged',
      handleCredentialChanged as EventListener,
    )

    return () => {
      window.removeEventListener(
        'gamesUserCredentialChanged',
        handleCredentialChanged as EventListener,
      )
    }
  }, [])

  // handle Android physical back button press
  // sometimes we want to prevent native navigation and have the webview change state, instead
  useHybridEffect(() => {
    const navigateBack: BridgeListener = (event) => {
      const respondToNative = (preventNativeNavigation: boolean) =>
        event.respondWith({
          [EVENT_ON_NAVIGATE_BACK]: preventNativeNavigation,
        })

      if (isModalOpen()) {
        toggleModal()
        respondToNative(true)
      } else if (momentSystem.getCurrentMoment() === 'congrats') {
        momentSystem.transition('congrats', 'game')
        respondToNative(true)
      } else if (isWordlistExpanded) {
        dispatch(toggleWordlist())
        respondToNative(true)
      } else {
        respondToNative(false)
      }
    }

    window.NativeBridge?.addEventListener(
      EVENT_ON_NAVIGATE_BACK,
      navigateBack,
    )

    return () => {
      window.NativeBridge?.removeEventListener(
        EVENT_ON_NAVIGATE_BACK,
        navigateBack,
      )
    }
  }, [isWordlistExpanded])

  // communicate analytics info back to the app
  useGameDataHybridEvent({
    analyticsPuzzleID: sbTrackingContext,
    puzzleID,
    game: 'spellingBee',
  })
}
