import React from 'react'

import { trackInteraction } from 'foundation'
import { isMobile } from 'responsive'

import {
  selectCurrentScore,
  selectCurrentRank,
  selectCurrentRankTrackingLabel,
} from '../store/game/game.selectors'
import { useSbSelector } from '../store/game/hooks'

import { ModalHeader, ModalBody } from './ModalComponents'
import {
  CopyLink,
  ShareButton,
  ShareFacebook,
  ShareTwitter,
  ShareWhatsApp,
  DownloadImage,
} from './ShareComponents'
import { getShareInfo } from './shareUtils'

const ModalShare = () => {
  // Puzzle Data
  const date = useSbSelector((s) => s.game.printDate)
  const letters = useSbSelector((s) => s.game.validLetters.join(''))
  const points = useSbSelector(selectCurrentScore)
  const rank = useSbSelector(selectCurrentRank)
  const rankLabel = useSbSelector(selectCurrentRankTrackingLabel)
  const id = useSbSelector((state) => state.game.id)
  const isPastPuzzle = useSbSelector(
    (state) => state.game.isPastPuzzle,
  )

  // combine rank and date separated by `-` for file name
  const downloadImageFileName = 'spelling-bee-'
    .concat(rank, '-', date)
    .replace(' ', '-')
    .toLowerCase()

  // Share Info
  const textEncoded = `I got ${rank}!`
  const { shareUrl, shareImageSrc } = getShareInfo(
    date,
    letters,
    points,
    rank,
  )

  const shareData = {
    title: 'NYTimes',
    text: 'Test share',
    url: 'https://www.nytimes.com',
  }

  const canNavigatorShare =
    !!navigator.canShare && navigator.canShare(shareData)

  const trackShareButtonClick = (label: string) => {
    trackInteraction({
      label,
      element: { name: 'spelling-bee', label: 'share-option' },
      context: `SB-${id}`,
      region: rankLabel,
      name: {
        p: date,
        t: isPastPuzzle ? 'archive' : 'today',
      },
    })
  }
  const showNativeShare = isMobile && canNavigatorShare
  const showDownload = window.gamesAppPlatform !== 'android'

  return (
    <>
      <ModalHeader>
        <h3 className="sb-modal-title">Share Your Achievement</h3>
      </ModalHeader>
      <ModalBody>
        <div className="sb-share__tout-image-wrapper">
          <img
            src={shareImageSrc}
            alt="NYT spelling bee social media achievement"
            className="sb-share__tout-image"
          />
        </div>
        <ul className="sb-share__share-networks">
          {showNativeShare ? (
            <>
              <li className="sb-share__share-item">
                <ShareButton
                  shareUrl={shareUrl}
                  textEncoded={textEncoded}
                  trackShareClick={() => {
                    trackShareButtonClick('share')
                  }}
                />
              </li>
              {showDownload && (
                <li className="sb-share__share-item">
                  <DownloadImage
                    shareImageSrc={shareImageSrc}
                    downloadFileName={downloadImageFileName}
                    trackShareClick={() => {
                      trackShareButtonClick('download-image')
                    }}
                  />
                </li>
              )}
            </>
          ) : (
            <>
              {showDownload && (
                <li className="sb-share__share-item">
                  <DownloadImage
                    shareImageSrc={shareImageSrc}
                    downloadFileName={downloadImageFileName}
                    trackShareClick={() => {
                      trackShareButtonClick('download-image')
                    }}
                  />
                </li>
              )}
              <li className="sb-share__share-item">
                <CopyLink
                  shareUrl={shareUrl}
                  trackShareClick={() => {
                    trackShareButtonClick('copy-link')
                  }}
                />
              </li>
              <li className="sb-share__share-item">
                <ShareFacebook
                  shareUrl={shareUrl}
                  textEncoded={textEncoded}
                  trackShareClick={() => {
                    trackShareButtonClick('facebook')
                  }}
                />
              </li>
              <li className="sb-share__share-item">
                <ShareTwitter
                  shareUrl={shareUrl}
                  textEncoded={textEncoded}
                  trackShareClick={() => {
                    trackShareButtonClick('twitter')
                  }}
                />
              </li>
              <li className="sb-share__share-item">
                <ShareWhatsApp
                  shareUrl={shareUrl}
                  textEncoded={textEncoded}
                  trackShareClick={() => {
                    trackShareButtonClick('whats-app')
                  }}
                />
              </li>
            </>
          )}
        </ul>
      </ModalBody>
    </>
  )
}

export default ModalShare
