import React, { useEffect, useState, useRef } from 'react'
import cn from 'classnames'

import { encodeUrlWithSMID, SMID } from './shareUtils'

const fbUrlEncode = encodeURIComponent('https://www.facebook.com/')

interface ShareProps {
  shareUrl: string
  textEncoded: string
  trackShareClick?: () => void
}

const ShareLink = ({
  shareUrl,
  appName,
  trackingCallback,
}: {
  shareUrl: ShareProps['shareUrl']
  appName: string
  trackingCallback?: () => void
}) => {
  return (
    <a
      href={shareUrl}
      target="_blank"
      aria-label={`Share on ${appName}`}
      rel="noopener noreferrer"
      onClick={() => {
        trackingCallback?.()
      }}
    >
      <i
        className={cn('sb-share__share-icon', appName.toLowerCase())}
      />
      <div className="sb-share__share-link">{appName}</div>
    </a>
  )
}

export const ShareFacebook = ({
  shareUrl,
  textEncoded,
  trackShareClick,
}: ShareProps) => {
  const shareFacebookUrl = `https://www.facebook.com/dialog/feed?app_id=9869919170&link=${encodeUrlWithSMID(
    shareUrl,
    SMID.facebook,
  )}&name=${textEncoded}&redirect_uri=${fbUrlEncode}`

  return (
    <ShareLink
      shareUrl={shareFacebookUrl}
      appName="Facebook"
      trackingCallback={() => {
        trackShareClick?.()
      }}
    />
  )
}

export const ShareTwitter = ({
  shareUrl,
  textEncoded,
  trackShareClick,
}: ShareProps) => {
  const shareTwitterUrl = `https://twitter.com/intent/tweet?url=${encodeUrlWithSMID(
    shareUrl,
    SMID.twitter,
  )}&text=${textEncoded}`

  return (
    <ShareLink
      shareUrl={shareTwitterUrl}
      appName="Twitter"
      trackingCallback={() => {
        trackShareClick?.()
      }}
    />
  )
}

export const ShareWhatsApp = ({
  shareUrl,
  textEncoded,
  trackShareClick,
}: ShareProps) => {
  const shareWhatsAppUrl = `https://api.whatsapp.com/send?text=${textEncoded}%20${encodeUrlWithSMID(
    shareUrl,
    SMID.whatsapp,
  )}`

  return (
    <ShareLink
      shareUrl={shareWhatsAppUrl}
      appName="WhatsApp"
      trackingCallback={() => {
        trackShareClick?.()
      }}
    />
  )
}

export const CopyLink = ({
  shareUrl,
  trackShareClick,
}: {
  shareUrl: string
  trackShareClick: () => void
}) => {
  const [shouldShowCopyText, setShouldShowCopyText] = useState(false)
  const buttonRef = useRef<HTMLButtonElement>(null)
  const timeoutRef = useRef<null | number>(null)

  const handleCopyClick = () => {
    if (buttonRef.current) {
      const copyLinkUrl = `${shareUrl}&smid=${SMID.link}`

      // set up the element
      const shareInput = document.createElement('input')
      shareInput.type = 'text'
      shareInput.value = copyLinkUrl

      // append and copy the link
      buttonRef.current.append(shareInput)
      shareInput.select()
      document.execCommand('copy')

      // remove the element
      shareInput.remove()
      setShouldShowCopyText(true)

      // Tracking
      trackShareClick?.()

      clearTimeout(timeoutRef.current as number)

      timeoutRef.current = window.setTimeout(() => {
        setShouldShowCopyText(false)
      }, 5000)
    }
  }

  useEffect(() => {
    return () => {
      window.clearTimeout(timeoutRef.current as number)
    }
  }, [])

  return (
    <button
      ref={buttonRef}
      type="button"
      aria-label="Copy Link"
      id="copyUrlId"
      onClick={handleCopyClick}
      className={cn('sb-share__share-link-button', {
        'sb-share__copied-link': shouldShowCopyText,
      })}
    >
      <i className={cn('sb-share__share-icon', 'copy-link')} />
      <div className="sb-share__share-link">
        {shouldShowCopyText ? 'Copied' : 'Copy link'}
      </div>
    </button>
  )
}

export const DownloadImage = ({
  shareImageSrc,
  downloadFileName,
  trackShareClick,
}: {
  shareImageSrc: string
  downloadFileName: string
  trackShareClick: () => void
}) => (
  <a
    href={shareImageSrc}
    download={downloadFileName}
    target="_blank"
    aria-label="Download image"
    rel="noopener noreferrer"
    onClick={() => {
      trackShareClick?.()
    }}
  >
    <i className={cn('sb-share__share-icon', 'download')} />
    Download image
  </a>
)

export const ShareButton = ({
  shareUrl,
  textEncoded,
  trackShareClick,
}: ShareProps) => {
  const handleShareClick = () => {
    navigator
      .share({
        title: textEncoded,
        url: `${shareUrl}&smid=na-share`,
      })
      .then(() => {
        trackShareClick?.()
      })
      .catch((error) =>
        console.error('Sharing Spelling Bee rank failed', error),
      )
  }

  return (
    <button
      type="button"
      onClick={handleShareClick}
      className="sb-share__share-link-button"
    >
      <i className={cn('sb-share__share-icon', 'share')} />
      Share
    </button>
  )
}
