import React from 'react'
import { Provider } from 'react-redux'
import { render, screen, fireEvent } from '@testing-library/react'

import { setUpStore } from '../../store'
import ModalAnswers from '../ModalAnswers'
import { mockGameData } from '../../__tests__/mockGameData'
import { ACCEPT_WORD } from '../../store/game/game.actions'

const toggleModal = jest.fn()

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  getGameData: () => {
    return require('../../__tests__/mockGameData').mockGameData
  },
}))

jest.mock('../../moments', () => ({
  momentSystem: {
    transition: jest.fn(),
  },
}))

const renderModal = () => {
  const store = setUpStore()

  let answerIdx = 0

  const enterWord = () => {
    const { answers } = mockGameData.today
    const word = answers[answerIdx]
    store.dispatch({
      type: ACCEPT_WORD,
      payload: word,
    })
    answerIdx += 1
    return word
  }

  const answersModal = render(
    <Provider store={store}>
      <ModalAnswers toggleModal={toggleModal} />
    </Provider>,
  )

  return { answersModal, store, enterWord }
}

test('should render title', () => {
  renderModal()
  expect(screen.getByRole('heading').textContent).toBe('Answers')
})

test('should render confirmation message when puzzle has not been revealed', () => {
  renderModal()

  const buttons = screen.getAllByRole('button')

  expect(buttons[0].textContent).toBe('Reveal')
  expect(buttons[1].textContent).toBe('Keep Playing')
})

test('should close modal after pressing Keep Playing', () => {
  renderModal()

  const keepPlaying = screen.getAllByRole('button')[1]

  fireEvent.click(keepPlaying)
  expect(toggleModal).toHaveBeenCalled()
})

test('should show answers after pressing Reveal', () => {
  renderModal()

  const reveal = screen.getAllByRole('button')[0]
  fireEvent.click(reveal)

  const answers = screen.queryAllByTestId('answers-word')
  expect(answers.length > 0).toBe(true)
})

test('should show checkmark only next to found words', () => {
  const { enterWord, store } = renderModal()

  const word1 = enterWord()
  const word2 = enterWord()
  const word3 = store.getState().game.answers[2]

  const reveal = screen.getAllByRole('button')[0]
  fireEvent.click(reveal)

  const answers = screen.queryAllByTestId('answers-word')
  const firstWord = answers[0]
  const secondWord = answers[1]
  const thirdWord = answers[2]

  expect(firstWord.textContent).toBe(word1)
  expect(firstWord.innerHTML.includes('checked')).toBe(true)

  expect(secondWord.textContent).toBe(word2)
  expect(secondWord.innerHTML.includes('checked')).toBe(true)

  expect(thirdWord.textContent).toBe(word3)
  expect(thirdWord.innerHTML.includes('checked')).toBe(false)
})
