import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'
import { act, waitFor } from '@testing-library/react'

import { trackInteraction } from 'foundation'

import { ModalHeader, ModalBody } from '../ModalComponents'
import ModalHelp from '../ModalHelp'

jest.mock('foundation', () => ({
  getFeedbackLink: () => Promise.resolve('feedback'),
  SEND_EMAIL_TYPE: { SUGGESTION: 'mock-suggestion' },
  trackInteraction: jest.fn(),
}))

jest.mock('../../../../../shared/abra/useAbra', () => () => ({
  getVariant: () => '1_TipsAndTricks',
}))

jest.mock('user', () => ({
  useUser: () => ({ userType: {} }),
}))

describe('SPELLING BEE: MODAL HELP', () => {
  let connectModal
  const store = {
    getState: () => ({
      game: {
        id: 1,
        printDate: '2023-04-28',
        isPastPuzzle: false,
      },
    }),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  }

  beforeAll(() => {
    connectModal = () =>
      mount(
        <Provider store={store}>
          <ModalHelp />
        </Provider>,
      )
  })

  it('should render without crashing', async () => {
    await act(async () => {
      const component = connectModal()
      await waitFor(() => {
        expect(component.length).toBeTruthy()
      })
    })
  })

  it('should render rules list', async () => {
    await act(async () => {
      const component = connectModal().find(ModalBody)

      const list = component.find('.sb-modal-list').first()
      await waitFor(() => {
        expect(list.childAt(0).text()).toEqual(
          'Words must contain at least 4 letters.',
        )
        expect(list.children()).toHaveLength(5)
      })
    })
  })

  it('should render scoring details', async () => {
    await act(async () => {
      const component = connectModal().find(ModalBody)

      const heading = component.find('.sb-modal-heading')
      await waitFor(() => {
        expect(heading.text()).toEqual(
          'Score points to increase your rating.',
        )
      })
      const list = component.find('.sb-modal-list').last()
      await waitFor(() => {
        expect(list.childAt(0).text()).toEqual(
          '4-letter words are worth 1 point each.',
        )
        expect(list.children()).toHaveLength(3)
      })
    })
  })

  it('should render link to tips and tricks', async () => {
    await act(async () => {
      const component = connectModal().find(ModalBody)

      const message = component.find('.sb-modal-message').first()
      await waitFor(() => {
        expect(message.text()).toContain('Need more solving tips?')
      })
    })
  })

  it('should render release line', async () => {
    await act(async () => {
      const component = connectModal().find(ModalBody)

      const message = component.find('.sb-modal-message').at(1)
      await waitFor(() => {
        expect(message.text()).toEqual(
          'New puzzles are released daily at 3 a.m. ET.',
        )
      })
    })
  })

  it('should display correct feedback links', async () => {
    await act(async () => {
      const component = connectModal().find(ModalBody)

      const buzzwordsLink = component.find('FeedbackLink').at(0)
      await waitFor(() => {
        expect(buzzwordsLink.prop('href')).toEqual(
          'mailto:buzzwords@nytimes.com',
        )
        expect(buzzwordsLink.render().text()).toContain(
          'buzzwords@nytimes.com',
        )
        expect(buzzwordsLink.prop('subject')).toEqual(
          'Spelling Bee Word Suggestion',
        )

        const feedbackLink = component.find('FeedbackLink').at(1)
        expect(feedbackLink.render().text()).toEqual(
          'nytgames@nytimes.com',
        )
        expect(feedbackLink.prop('subject')).toEqual(
          'Spelling Bee Feedback',
        )
      })
    })
  })

  it('should render correct title and message', async () => {
    await act(async () => {
      const component = connectModal().find(ModalHeader)

      const title = component.find('.sb-modal-title')
      await waitFor(() => {
        expect(title.text()).toEqual('How to Play Spelling Bee')
        const message = component.find('.sb-modal-subtitle')
        expect(message.text()).toEqual(
          'Create words using letters from the hive.',
        )
      })
    })
  })

  describe('LINK TO FORUM', () => {
    let forumButton

    beforeEach(() => {
      jest.resetAllMocks()
    })

    afterEach(() => {})

    it('should render link to SB forum', async () => {
      await act(async () => {
        const component = connectModal().find(ModalBody)
        const message = component.find('.sb-modal-message').first()
        forumButton = message.find('a')
        await waitFor(() => {
          expect(forumButton.text()).toBe('This article can help.')
        })
      })
    })

    it('should fire tracking when clicked', async () => {
      await act(async () => {
        const component = connectModal().find(ModalBody)
        const message = component.find('.sb-modal-message').first()
        forumButton = message.find('a')
        window.isHybridWebView = false
        window.open = jest.fn(() => ({ focus: jest.fn() }))
        forumButton.simulate('click')
        await waitFor(() => {
          expect(trackInteraction).toHaveBeenCalled()
        })
      })
    })
  })
})
