import React from 'react'
import { Provider } from 'react-redux'
import { render, screen, fireEvent } from '@testing-library/react'

import { setUpStore } from '../../store'
import ModalYesterdayNew from '../ModalYesterdayNew'
import {
  SET_YESTERDAYS_PROGRESS,
  REVEAL_YESTERDAY,
} from '../../store/game/game.actions'

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  getGameData: () => {
    return require('../../__tests__/mockGameData').mockGameData
  },
}))

jest.mock('../../moments', () => ({
  momentSystem: {
    transition: jest.fn(),
  },
}))

const renderModal = () => {
  const store = setUpStore()

  const revealYesterday = () => {
    store.dispatch({
      type: REVEAL_YESTERDAY,
      payload: true,
    })
  }

  const setYesterdaysProgress = (words) => {
    store.dispatch({
      type: SET_YESTERDAYS_PROGRESS,
      payload: {
        foundWords: words,
        isRevealed: false,
      },
    })
  }

  const yesterdayModal = render(
    <Provider store={store}>
      <ModalYesterdayNew />
    </Provider>,
  )

  return {
    yesterdayModal,
    store,
    revealYesterday,
    setYesterdaysProgress,
  }
}

describe('ModalYesterdayNew', () => {
  const { location } = window
  let url = ''
  const getHrefSpy = jest.fn(() => url)
  const setHrefSpy = jest.fn((href) => {
    url = href
  })

  beforeAll(() => {
    // @ts-ignore
    delete window.location
    // @ts-ignore
    window.location = {}

    Object.defineProperty(window.location, 'href', {
      get: getHrefSpy,
      set: setHrefSpy,
    })
  })

  afterAll(() => {
    window.location = location
  })

  test('should render title', () => {
    renderModal()
    expect(screen.getByRole('heading').textContent).toBe(
      'Yesterday’s Answers',
    )
  })

  test('should render confirmation message when puzzle has not been revealed', () => {
    renderModal()

    const buttons = screen.getAllByRole('button')

    expect(buttons[0].textContent).toBe('Reveal')
    expect(buttons[1].textContent).toBe('Play instead')
  })

  test('should navigate to yesterdays puzzle after pressing Play instead', () => {
    renderModal()

    const playYesterday = screen.getAllByRole('button')[1]

    fireEvent.click(playYesterday)
    expect(window.location.href).toBe(
      '/puzzles/spelling-bee/2023-07-22',
    )
  })

  test('should show answers after pressing Reveal', () => {
    renderModal()

    const reveal = screen.getAllByRole('button')[0]
    fireEvent.click(reveal)

    const answers = screen.queryAllByTestId('yesterdays-answer-word')
    expect(answers.length > 0).toBe(true)
  })

  test('should show checkmark only next to found words', () => {
    const { store, setYesterdaysProgress } = renderModal()
    const firstAnswer = store.getState().game.yesterday.answers[0]
    setYesterdaysProgress([firstAnswer])

    const reveal = screen.getAllByRole('button')[0]
    fireEvent.click(reveal)

    const answers = screen.queryAllByTestId('yesterdays-answer-word')
    const firstWord = answers[0]
    const secondWord = answers[1]
    const thirdWord = answers[2]

    expect(firstWord.innerHTML.includes('checked')).toBe(true)

    expect(secondWord.innerHTML.includes('checked')).toBe(false)

    expect(thirdWord.innerHTML.includes('checked')).toBe(false)
  })
})
