import React, { Component } from 'react'
import ReactDOM from 'react-dom'
import cn from 'classnames'

const TIMEOUT_DURATION = 150

const noop = () => {}

export type SpellingBeeModalName =
  | 'help'
  | 'ranks'
  | 'yesterday'
  | 'yesterdayNew'
  | 'stats'
  | 'share'
  | 'answers'
  | 'pastPuzzles'

type ModalProps = {
  initialModal: SpellingBeeModalName | null
  onClose: (modal?: SpellingBeeModalName, trigger?: string) => void
  onOpen: (modal?: SpellingBeeModalName, trigger?: string) => void
}

type ModalState = {
  currentModal: SpellingBeeModalName | null
  isOpen: boolean
  isClosing: boolean
  hasAnimatedIn: boolean
}

type ModalButtonProps = {
  modal: SpellingBeeModalName
  className?: string
  triggerName: string
  children: React.ReactNode
  buttonRef?: React.RefObject<HTMLElement>
  onClick?: () => void
}

type ModalSystem = {
  toggleModal: (
    inModal?: SpellingBeeModalName,
    trigger?: string,
  ) => void
  getCurrentModal: () => SpellingBeeModalName | null
  Modals?: React.ComponentClass<ModalProps, ModalState>
  ModalButton: (props: ModalButtonProps) => React.ReactElement
}

export type ModalContentProps = {
  toggleModal: () => void
  modalType: SpellingBeeModalName
}

export type ModalWrapperProps = {
  children: React.ReactNode
  toggleModal: () => void
  modalType: SpellingBeeModalName
}

const createModalSystem = (
  modals: {
    [K in SpellingBeeModalName]: (
      props: ModalContentProps,
    ) => React.ReactElement | null
  },
  ModalWrap: (props: ModalWrapperProps) => React.ReactElement,
  modalRoot: HTMLElement | null,
) => {
  const ModalButton = ({
    modal,
    children,
    className,
    triggerName,
    buttonRef,
    onClick,
  }: ModalButtonProps) => (
    <span
      role="presentation"
      className={className}
      onClick={() => {
        if (onClick) {
          onClick()
        }
        system.toggleModal(modal, triggerName) // eslint-disable-line
      }}
      ref={buttonRef}
    >
      {children}
    </span>
  )

  const system: ModalSystem = {
    toggleModal: noop,
    getCurrentModal: () => null,
    ModalButton,
  }

  class Modals extends Component<ModalProps, ModalState> {
    constructor(props: ModalProps) {
      super(props)
      this.state = {
        currentModal: props.initialModal || null,
        isOpen: !!props.initialModal,
        isClosing: false,
        hasAnimatedIn: false,
      }

      system.toggleModal = this.toggleModal.bind(this)
      system.getCurrentModal = this.getCurrentModal.bind(this)
      this.enqueueStates = this.enqueueStates.bind(this)
      this.handleScrimClick = this.handleScrimClick.bind(this)
    }

    handleScrimClick() {
      const { hasAnimatedIn } = this.state

      // we allow a user to click on the scrim to dismiss the modal
      // but only after it has fully animated in, to avoid a page flash
      // if a user double clicks or double taps a modal trigger
      if (hasAnimatedIn) {
        system.toggleModal()
      }
    }

    getCurrentModal() {
      return this.state.currentModal || null // eslint-disable-line
    }

    enqueueStates(
      states: Partial<ModalState>[],
      delay = TIMEOUT_DURATION,
      index = 0,
    ) {
      if (!states[index]) {
        return
      }

      setTimeout(
        () => {
          this.setState({ ...(states[index] as ModalState) }, () =>
            this.enqueueStates(states, delay, index + 1),
          )
        },
        index ? delay : 0,
      )
    }

    toggleModal(inModal?: SpellingBeeModalName, trigger?: string) {
      const { isOpen, currentModal } = this.state
      const { onOpen, onClose } = this.props

      // allow toggleModal to be passed directly to onClick()
      // where first arg would be event object
      const nextModal = typeof inModal === 'string' && inModal

      if (nextModal && nextModal !== currentModal) {
        if (onOpen) {
          onOpen(nextModal, trigger)
        }

        if (isOpen) {
          this.setState({ currentModal: nextModal })
        } else {
          this.enqueueStates([
            { hasAnimatedIn: true },
            { isOpen: true, currentModal: nextModal },
          ])
        }
      } else {
        if (onClose && currentModal) {
          onClose(currentModal, trigger)
        }

        this.enqueueStates([
          { isClosing: true, hasAnimatedIn: false },
          {
            isClosing: false,
            isOpen: false,
            currentModal: null,
          },
        ])
      }
    }

    render() {
      const { isOpen, isClosing, currentModal } = this.state
      const ModalContent =
        (currentModal && modals[currentModal]) || null

      if (!modalRoot) return null

      return ReactDOM.createPortal(
        <div
          className={cn('sb-modal-system', {
            'sb-modal-open': isOpen,
            'sb-modal-closing': isClosing,
          })}
        >
          <div
            role="presentation"
            className="sb-modal-scrim"
            onClick={this.handleScrimClick}
          >
            <div className="sb-modal-wrapper">
              {currentModal && ModalContent && (
                <ModalWrap
                  key={currentModal}
                  toggleModal={system.toggleModal}
                  modalType={currentModal}
                >
                  <ModalContent
                    toggleModal={system.toggleModal}
                    modalType={currentModal}
                  />
                </ModalWrap>
              )}
            </div>
          </div>
        </div>,
        modalRoot,
      )
    }
  }

  system.Modals = Modals
  system.ModalButton = ModalButton

  return system
}

export default createModalSystem
