import React from 'react'
import cn from 'classnames'
import { useDispatch } from 'react-redux'

import { useSbSelector, useTracking } from '../store/game/hooks'
import { updateModalRedux } from '../store/game/game.actions'
import { trackModals } from '../spellingBeeTracker'
import { selectCurrentRankTrackingLabel } from '../store/game/game.selectors'

import createModalSystem, {
  ModalContentProps,
  ModalWrapperProps,
  SpellingBeeModalName,
} from './createModalSystem'
import ModalHelp from './ModalHelp'
import ModalRanks from './ranks/ModalRanks'
import ModalYesterday from './ModalYesterday'
import ModalYesterdayNew from './ModalYesterdayNew'
import ModalStats from './stats/ModalStats'
import ModalShare from './ModalShare'
import ModalAnswers from './ModalAnswers'
import ModalPastPuzzles from './ModalPastPuzzles'

const ModalTypes: {
  [K in SpellingBeeModalName]: ({
    toggleModal,
    modalType,
  }: ModalContentProps) => JSX.Element | null
} = {
  help: ModalHelp,
  ranks: ModalRanks,
  yesterday: ModalYesterday,
  yesterdayNew: ModalYesterdayNew,
  stats: ModalStats,
  share: ModalShare,
  answers: ModalAnswers,
  pastPuzzles: ModalPastPuzzles,
}

const ModalWrapper = ({
  children,
  toggleModal,
  modalType,
}: ModalWrapperProps) => {
  const closeX = (
    <div className="sb-modal-top">
      <div
        role="button"
        className="sb-modal-close"
        onClick={() => toggleModal()}
      >
        ×
      </div>
    </div>
  )

  const [hasOverflow, setOverflow] = React.useState(false)
  const contentRef = React.useRef<HTMLDivElement>(null)

  const detectOverflow = () => {
    if (contentRef.current) {
      setOverflow(
        contentRef.current.scrollTop +
          contentRef.current.offsetHeight <
          contentRef.current.scrollHeight,
      )
    }
  }

  React.useEffect(() => {
    detectOverflow()

    contentRef.current?.addEventListener('scroll', detectOverflow)
    return () => {
      contentRef.current?.removeEventListener(
        'scroll',
        detectOverflow,
      )
    }
  }, [])

  return (
    <div
      role="button"
      className={cn('sb-modal-frame', modalType)}
      onClick={(e) => e.stopPropagation()}
    >
      {closeX}
      <div
        ref={contentRef}
        className={cn('sb-modal-content', {
          'has-overflow': hasOverflow,
        })}
      >
        {children}
      </div>
    </div>
  )
}

const system = createModalSystem(
  ModalTypes,
  ModalWrapper,
  document.getElementById('portal-sb-modals'),
)

export const toggleModal = (
  m?: SpellingBeeModalName,
  origin?: string,
) => system.toggleModal(m, origin)

export const isModalOpen = () => {
  const { getCurrentModal } = system
  return !!getCurrentModal()
}

export const { ModalButton } = system

export const SBModals = () => {
  const trackSB = useTracking()
  const dispatch = useDispatch()
  const rankLabel = useSbSelector(selectCurrentRankTrackingLabel)

  const handleModalClose = (
    currentModal: SpellingBeeModalName,
    trigger?: string,
  ) => {
    if (currentModal === 'share') {
      trackSB('close-congrats-share', rankLabel)
    } else if (
      currentModal === 'answers' ||
      currentModal === 'yesterdayNew'
    ) {
      trackSB('close-answers', trigger || 'x')
    }
  }

  if (!system.Modals) {
    return null
  }

  return (
    <system.Modals
      onOpen={(modal?: SpellingBeeModalName, trigger?: string) => {
        dispatch(updateModalRedux(true))
        if (modal) trackModals(modal, trigger)
      }}
      onClose={(
        currentModal?: SpellingBeeModalName,
        trigger?: string,
      ) => {
        dispatch(updateModalRedux(false))
        if (currentModal) {
          handleModalClose(currentModal, trigger)
        }
      }}
      initialModal={null}
    />
  )
}
