import React from 'react'
import { mount } from 'enzyme'

import DailyRecapBar from './DailyRecapBar'

jest.mock('foundation', () => ({
  userType: { hasXwd: false, regId: 2222 },
}))

jest.mock('../../../../shared/utils/createHybridNewsLink', () =>
  jest.fn().mockImplementation((url) => url),
)

jest.mock('../../store/game/hooks', () => ({
  useSbSelector: jest.fn().mockReturnValue('2022-04-27'),
}))

describe('SPELLING BEE: DAILY RECAP BAR', () => {
  const { location } = window
  let url = ''
  const getHrefSpy = jest.fn(() => url)
  const setHrefSpy = jest.fn((href) => {
    url = href
  })

  beforeAll(() => {
    // @ts-ignore
    delete window.location
    // @ts-ignore
    window.location = {}

    Object.defineProperty(window.location, 'href', {
      get: getHrefSpy,
      set: setHrefSpy,
    })
  })

  afterAll(() => {
    window.location = location
  })

  const today = '2022-04-27'
  const tomorrow = '2022-04-28'
  const yesterday = '2022-04-26'
  const grayText = '.sb-stats-bar__gray-text'

  const props = {
    rank: '',
    wordsFound: 0,
    totalWords: 50,
  }

  it('renders rank recap', () => {
    const wrapper = mount(
      <DailyRecapBar publishedDate={today} {...props} />,
    )

    expect(wrapper.find('.sb-stats-bar').length).toBe(1)
  })

  it('renders letter to represent day of week', () => {
    const wrapper = mount(
      <DailyRecapBar
        {...props}
        isCurrentPuzzle
        publishedDate={today}
      />,
    )

    expect(wrapper.find('.sb-stats-bar__weekday').text()).toBe('W')
  })

  it('renders release date if game date is in the future', () => {
    const wrapper = mount(
      <DailyRecapBar publishedDate={tomorrow} {...props} />,
    )

    expect(wrapper.find(grayText).text()).toBe(
      'Releases on 4/28 at 3 a.m. E.T.',
    )
  })

  it('renders unplayed text if day was unplayed', () => {
    const wrapper = mount(
      <DailyRecapBar publishedDate={yesterday} {...props} />,
    )

    const text = wrapper.find('.sb-stats-bar__text').text()
    expect(text).toContain('Not Started')
  })

  it('renders no progress if date is today and no words were found', () => {
    const wrapper = mount(
      <DailyRecapBar
        {...props}
        isCurrentPuzzle
        publishedDate={today}
      />,
    )

    expect(wrapper.text()).toContain('Play today’s Spelling Bee')
  })

  it('renders a rank label', () => {
    const wrapper = mount(
      <DailyRecapBar
        publishedDate={today}
        rank="Moving Up"
        wordsFound={3}
        totalWords={50}
      />,
    )

    expect(wrapper.find('.sb-stats-bar-rank__text').text()).toBe(
      'Moving Up',
    )
  })

  it('renders genius icon if rank is Genius', () => {
    const wrapper = mount(
      <DailyRecapBar
        publishedDate={today}
        rank="Genius"
        wordsFound={45}
        totalWords={50}
      />,
    )

    expect(wrapper.find('.genius').length).toBe(1)
  })

  it('renders queen bee icon if rank is Genius', () => {
    const wrapper = mount(
      <DailyRecapBar
        publishedDate={today}
        rank="Queen Bee"
        wordsFound={50}
        totalWords={50}
      />,
    )

    expect(wrapper.find('.queen-bee').length).toBe(1)
  })

  it('renders word count for past puzzles', () => {
    const wrapper = mount(
      <DailyRecapBar
        publishedDate={yesterday}
        rank="Moving Up"
        wordsFound={3}
        totalWords={50}
      />,
    )

    expect(
      wrapper.find('.sb-stats-bar-rank__word-count').text(),
    ).toBe('3 of 50')
  })

  it('does not render word count for today`s puzzle', () => {
    const wrapper = mount(
      <DailyRecapBar
        publishedDate={today}
        rank="Moving Up"
        wordsFound={3}
        totalWords={50}
      />,
    )

    expect(
      wrapper.find('.sb-stats-bar-rank__word-count').length,
    ).toBe(0)
  })

  it('renders a link for a past puzzle and calls tracking function when clicked', () => {
    const track = jest.fn()
    const wrapper = mount(
      <DailyRecapBar
        publishedDate={yesterday}
        trackClick={track}
        {...props}
      />,
    )
    const bar = wrapper.find('button')
    bar.simulate('click')

    expect(bar.length).toBe(1)
    expect(window.location.href).toBe(
      `/puzzles/spelling-bee/${yesterday}`,
    )
    expect(track).toHaveBeenCalled()
  })

  it('renders a button that closes the modal for the current puzzle you are on', () => {
    const toggleModal = jest.fn()
    const wrapper = mount(
      <DailyRecapBar
        publishedDate={today}
        isCurrentPuzzle
        toggleModal={toggleModal}
        {...props}
      />,
    )
    const bar = wrapper.find('button')
    bar.simulate('click')

    expect(bar.length).toBe(1)
    expect(toggleModal).toHaveBeenCalled()
  })

  it('does not render a link for future puzzles', () => {
    const wrapper = mount(
      <DailyRecapBar publishedDate={tomorrow} {...props} />,
    )

    expect(wrapper.find('button').length).toBe(0)
    expect(wrapper.find('a').length).toBe(0)
  })

  it('does not render a link for a fake bar', () => {
    const wrapper = mount(
      <DailyRecapBar
        {...props}
        disableLink
        publishedDate={yesterday}
      />,
    )

    expect(wrapper.find('button').length).toBe(0)
  })

  it('renders indicator for current puzzle', () => {
    const wrapper = mount(
      <DailyRecapBar
        {...props}
        isCurrentPuzzle
        publishedDate={today}
      />,
    )

    expect(wrapper.find('.sb-stats-bar__current').length).toBe(1)
  })
})
