import React, { useEffect, useRef } from 'react'
import cn from 'classnames'

import {
  getUTCDateString,
  getUTCShortPuzzleDateString,
} from '../../../../shared/utils/dateUtils'
import createHybridNewsLink from '../../../../shared/utils/createHybridNewsLink'
import { useSbSelector } from '../../store/game/hooks'
import { RANKS, getRankIdxFromRank } from '../../store/utils'

const DailyRecapBar = ({
  publishedDate,
  rank = 'Beginner',
  wordsFound = 0,
  totalWords,
  trackClick,
  isCurrentPuzzle = false,
  toggleModal,
  disableLink,
}: {
  publishedDate: string
  rank: string
  wordsFound: number
  totalWords: number
  trackClick?: () => void
  isCurrentPuzzle?: boolean
  toggleModal?: () => void
  disableLink?: boolean
}) => {
  const puzzleDateString: string = getUTCDateString(publishedDate)
  const weekdayInitials: string = puzzleDateString.charAt(0)
  const shortPuzzleDateString: string = getUTCShortPuzzleDateString(
    new Date(publishedDate),
  )

  const todaysPuzzleDate = useSbSelector((state) => {
    // get today's print date for stats - if pastPuzzlesData not available
    // we default back to what we have in store for today's data
    return (
      state.game.pastPuzzlesData?.today.printDate ||
      state.game.printDate
    )
  })

  const isToday = todaysPuzzleDate === publishedDate

  // date is in future if current day is less than puzzle date
  const isDateInFuture =
    new Date(todaysPuzzleDate) < new Date(puzzleDateString)

  const showUnplayed = !isToday && !isDateInFuture && !wordsFound
  const showNoProgress: boolean = isToday && !wordsFound
  const showRank: boolean =
    Boolean(totalWords) && !(isToday && !wordsFound)

  const showLink = !isDateInFuture && !isCurrentPuzzle && !disableLink

  const showArrow = !isDateInFuture && !disableLink

  const progressBar = useRef<HTMLDivElement>(null)
  let barColor: string = 'pale-yellow'
  let progress: string = '0%'
  let icon: string | undefined
  let rankIndex: number = 0
  let rankPercentage: number = 0

  if (rank) {
    rankIndex = getRankIdxFromRank(rank)
  }

  if (rankIndex < RANKS.length) {
    // eslint-disable-next-line prefer-destructuring
    rankPercentage = RANKS[rankIndex][1]
  }

  /* progress calculation below: because Genius is 70% of total points
  but the bar goes to 100% of the width, other ranks' percentages 
  are multiplied by (100 / 70) to keep the increments between ranks
  visually consistent. */
  progress = `${Math.round(rankPercentage * (100 / 70)).toString()}%`

  if (rank === 'Genius' || rank === 'Queen Bee') {
    barColor = 'bright-yellow'
    progress = '100%'
  }

  if (rank === 'Genius') {
    icon = 'genius'
  } else if (rank === 'Queen Bee') {
    icon = 'queen-bee'
  }

  const Rank = () => (
    <>
      <div className="sb-stats-bar-rank">
        <div className="sb-stats-bar-rank__text">{rank}</div>
        {icon && (
          <div className={cn('sb-stats-bar-rank__icon', icon)} />
        )}
      </div>
      {!isToday && (
        <div className="sb-stats-bar-rank__word-count">{`${wordsFound} of ${totalWords}`}</div>
      )}
    </>
  )

  const Unplayed = () => (
    <>
      Not Started
      {showLink && <div className="sb-stats-bar__play">Play</div>}
    </>
  )

  const NoProgressToday = () => (
    <>
      Play today’s Spelling Bee
      <div className="sb-stats-bar__play">Play</div>
    </>
  )

  const ReleaseDate = () => (
    <div className="sb-stats-bar__gray-text">{`Releases on ${shortPuzzleDateString} at 3 a.m. E.T.`}</div>
  )

  const BarWrapper = ({
    children,
  }: {
    children: React.ReactNode
  }) => {
    const baseLink = '/puzzles/spelling-bee'
    const puzzleLink = isToday
      ? baseLink
      : `${baseLink}/${publishedDate}`

    if (showLink) {
      return (
        <button
          type="button"
          className="sb-stats-bar"
          onClick={() => {
            if (trackClick) trackClick()
            window.location.href = createHybridNewsLink(puzzleLink)
          }}
        >
          {children}
        </button>
      )
    }

    if (isCurrentPuzzle) {
      return (
        <button
          type="button"
          className="sb-stats-bar"
          onClick={toggleModal}
        >
          {children}
          <div className="sb-stats-bar__current" />
        </button>
      )
    }

    return <div className="sb-stats-bar">{children}</div>
  }

  useEffect(() => {
    if (progressBar.current) {
      progressBar.current.style.width = progress
    }
  })

  return (
    <BarWrapper>
      <div className="sb-stats-bar__weekday">{weekdayInitials}</div>
      <div className="sb-stats-bar__inner">
        <div className="sb-stats-bar__text">
          {showUnplayed && <Unplayed />}
          {!showUnplayed && (
            <>
              {showRank && <Rank />}
              {showNoProgress && <NoProgressToday />}
              {isDateInFuture && <ReleaseDate />}
            </>
          )}
          {showArrow && <div className="sb-stats-bar__arrow" />}
        </div>
        <div
          ref={progressBar}
          className={cn(
            'sb-stats-percentage-bar',
            showRank ? barColor : '',
          )}
        />
      </div>
    </BarWrapper>
  )
}

export default DailyRecapBar
