import React from 'react'
import { Provider } from 'react-redux'
import { Player } from '@nyt/games-schema'
import { render } from '@testing-library/react'

import {
  MOOGLE_POST_ERROR,
  MOOGLE_POST_SUCCESS,
} from 'shared/moogle/moogleActions'

import { setUpStore } from '../../store'
import { UPDATE_USER } from '../../store/game/game.actions'
import useWindowSize from '../../../../shared/hooks/use-window-size'

import * as fakeStats from './player'
import { LifetimeStats } from './LifetimeStats'

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  getGameData: () => {
    return require('../../__tests__/mockGameData').mockGameData
  },
}))

jest.mock('../../../js/moments', () => ({
  momentSystem: {
    getCurrentMoment: jest.fn(),
  },
}))

jest.mock('../../../../shared/hooks/use-window-size', () => ({
  __esModule: true,
  default: jest.fn(),
}))

const getStatValue = (el: HTMLElement) =>
  el.querySelector('.sb-lifetime-stat__value')?.textContent

const getStatLabel = (el: HTMLElement) =>
  el.querySelector('.sb-lifetime-stat__label')?.textContent

describe('SPELLING BEE: LIFETIME STATS', () => {
  beforeEach(() => {
    jest.resetAllMocks()

    // @ts-ignore
    useWindowSize.mockReturnValue({
      winWidth: 600,
    })
  })

  const renderLifetimeStats = () => {
    const store = setUpStore()
    const puzzleId = store.getState().game.id.toString()

    const loadPlayerDoc = (player: Player) => {
      store.dispatch({
        type: MOOGLE_POST_SUCCESS,
        payload: { puzzleId, player },
      })
    }

    const forcePostError = () => {
      store.dispatch({
        type: MOOGLE_POST_ERROR,
        payload: { puzzleId, message: 'forced error' },
      })
    }

    const updateUser = (hasXwd: boolean) => {
      store.dispatch({
        type: UPDATE_USER,
        payload: {
          isLoggedIn: hasXwd,
          hasXwd,
          hasDigi: hasXwd,
          hasHd: hasXwd,
          inShortzMode: hasXwd,
          entitlement: hasXwd ? 'sub' : 'anon',
          regiId: '1',
        },
      })
    }

    const lifetimeStats = render(
      <Provider store={store}>
        <LifetimeStats />
      </Provider>,
    )

    return {
      lifetimeStats,
      loadPlayerDoc,
      updateUser,
      forcePostError,
    }
  }

  it('renders puzzles, words and pangrams', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.beginner)
    const puzzles = lifetimeStats.getByTestId('stat-puzzles')
    const words = lifetimeStats.getByTestId('stat-words')
    const pangrams = lifetimeStats.getByTestId('stat-pangrams')

    expect(getStatLabel(puzzles)).toBe('Puzzles')
    expect(getStatValue(puzzles)).toBe('4')

    expect(getStatLabel(words)).toBe('Words')
    expect(getStatValue(words)).toBe('4')

    expect(getStatLabel(pangrams)).toBe('Pangrams')
    expect(getStatValue(pangrams)).toBe('0')
  })

  it('renders the longest word with the center letter highlighted', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.beginner)
    const longestWord = lifetimeStats.getByTestId('stat-longest-word')
    const centerLetter = longestWord.querySelector('.center-letter')

    expect(getStatLabel(longestWord)).toBe('Longest Word')
    expect(getStatValue(longestWord)).toBe('snow')
    expect(centerLetter?.textContent).toBe('s')
  })

  it('does not show longest word if there is none', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.newPlayer)
    const puzzles = lifetimeStats.getByTestId('stat-puzzles')
    const words = lifetimeStats.getByTestId('stat-words')
    const pangrams = lifetimeStats.getByTestId('stat-pangrams')
    const longestWord = lifetimeStats.queryByTestId(
      'stat-longest-word',
    )

    expect(puzzles).toBeInTheDocument()
    expect(words).toBeInTheDocument()
    expect(pangrams).toBeInTheDocument()
    expect(longestWord).not.toBeInTheDocument()
  })

  it('renders larger numbers with commas', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.maxedOut)
    const puzzles = lifetimeStats.getByTestId('stat-puzzles')
    const words = lifetimeStats.getByTestId('stat-words')
    const pangrams = lifetimeStats.getByTestId('stat-pangrams')

    expect(getStatLabel(puzzles)).toBe('Puzzles')
    expect(getStatValue(puzzles)).toBe('999,999')

    expect(getStatLabel(words)).toBe('Words')
    expect(getStatValue(words)).toBe('999,999')

    expect(getStatLabel(pangrams)).toBe('Pangrams')
    expect(getStatValue(pangrams)).toBe('999,999')
  })

  it('shows the highest rank when geniuses are zero', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.amazing)
    const highestRank = lifetimeStats.getByTestId('stat-highest-rank')
    const geniuses = lifetimeStats.getByTestId('stat-geniuses')

    expect(getStatLabel(highestRank)).toBe('Highest Rank')
    expect(getStatValue(highestRank)).toBe('Amazing')

    expect(getStatLabel(geniuses)).toBe('Geniuses')
    expect(getStatValue(geniuses)).toBe('0')
  })

  it('does not show the highest rank when geniuses are greater than zero', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.genius)
    const highestRank = lifetimeStats.queryByTestId(
      'stat-highest-rank',
    )
    const geniuses = lifetimeStats.getByTestId('stat-geniuses')

    expect(highestRank).not.toBeInTheDocument()
    expect(getStatLabel(geniuses)).toBe('Geniuses')
    expect(getStatValue(geniuses)).toBe('20')
  })

  it('does not show queen bees if they are zero', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.genius)
    const geniuses = lifetimeStats.getByTestId('stat-geniuses')
    const queenBees = lifetimeStats.queryByTestId('stat-queen-bees')

    expect(getStatLabel(geniuses)).toBe('Geniuses')
    expect(getStatValue(geniuses)).toBe('20')
    expect(queenBees).not.toBeInTheDocument()
  })

  it('shows queen bees if they are greater than zero', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.queenBee)
    const geniuses = lifetimeStats.getByTestId('stat-geniuses')
    const queenBees = lifetimeStats.getByTestId('stat-queen-bees')

    expect(getStatLabel(geniuses)).toBe('Geniuses')
    expect(getStatValue(geniuses)).toBe('22')

    expect(getStatLabel(queenBees)).toBe('Queen Bees')
    expect(getStatValue(queenBees)).toBe('2')
  })

  it('shows geniuses and queen bees with icons', () => {
    const { lifetimeStats, loadPlayerDoc, updateUser } =
      renderLifetimeStats()
    loadPlayerDoc(fakeStats.queenBee)
    updateUser(true)

    const geniuses = lifetimeStats.getByTestId('stat-geniuses')
    const geniusIcon = geniuses.querySelector(
      '.sb-lifetime-stats__icon',
    )

    const queenBees = lifetimeStats.getByTestId('stat-queen-bees')
    const queenBeeIcon = queenBees.querySelector(
      '.sb-lifetime-stats__icon',
    )

    expect(geniusIcon).toHaveClass('genius')
    expect(queenBeeIcon).toHaveClass('queen-bee')
  })

  it('shows geniuses with a lock if user is not a sub and number of geniuses is zero', () => {
    const { lifetimeStats, loadPlayerDoc, updateUser } =
      renderLifetimeStats()
    loadPlayerDoc(fakeStats.beginner)
    updateUser(false)

    const geniuses = lifetimeStats.getByTestId('stat-geniuses')
    expect(getStatLabel(geniuses)).toBe('Geniuses')
    expect(getStatValue(geniuses)).toBe('0')
    expect(geniuses).toHaveClass('locked')
  })

  it('shows geniuses with a lock if user is not a sub and geniuses are greater than zero', () => {
    const { lifetimeStats, loadPlayerDoc, updateUser } =
      renderLifetimeStats()
    loadPlayerDoc(fakeStats.genius)
    updateUser(false)

    const geniuses = lifetimeStats.getByTestId('stat-geniuses')
    expect(getStatLabel(geniuses)).toBe('Geniuses')
    expect(getStatValue(geniuses)).toBe('20')
    expect(geniuses).toHaveClass('locked')
  })

  it('shows queen bees with a lock if user is not a sub and queen bees are greater than zero', () => {
    const { lifetimeStats, loadPlayerDoc, updateUser } =
      renderLifetimeStats()
    loadPlayerDoc(fakeStats.queenBee)
    updateUser(false)

    const geniuses = lifetimeStats.getByTestId('stat-queen-bees')
    expect(getStatLabel(geniuses)).toBe('Queen Bees')
    expect(getStatValue(geniuses)).toBe('2')
    expect(geniuses).toHaveClass('locked')
  })

  it('labels should be singular when value is one', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.ones)
    const puzzles = lifetimeStats.getByTestId('stat-puzzles')
    const words = lifetimeStats.getByTestId('stat-words')
    const pangrams = lifetimeStats.getByTestId('stat-pangrams')
    const geniuses = lifetimeStats.getByTestId('stat-geniuses')
    const queenBees = lifetimeStats.getByTestId('stat-queen-bees')

    expect(getStatLabel(puzzles)).toBe('Puzzle')
    expect(getStatValue(puzzles)).toBe('1')

    expect(getStatLabel(words)).toBe('Word')
    expect(getStatValue(words)).toBe('1')

    expect(getStatLabel(pangrams)).toBe('Pangram')
    expect(getStatValue(pangrams)).toBe('1')

    expect(getStatLabel(geniuses)).toBe('Genius')
    expect(getStatValue(geniuses)).toBe('1')

    expect(getStatLabel(queenBees)).toBe('Queen Bee')
    expect(getStatValue(queenBees)).toBe('1')
  })

  it('should have three rows and no divider if the width of the screen is small', () => {
    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.queenBee)

    const rows = lifetimeStats.queryAllByTestId(
      'sb-lifetime-stats__row',
    )

    const divider = lifetimeStats.queryByTestId(
      'sb-lifetime-stats__divider',
    )

    expect(rows.length).toBe(3)
    expect(divider).not.toBeInTheDocument()
  })

  it('should have two rows and a divider if the width of the screen is large', () => {
    // @ts-ignore
    useWindowSize.mockReturnValue({
      winWidth: 1000,
    })

    const { lifetimeStats, loadPlayerDoc } = renderLifetimeStats()
    loadPlayerDoc(fakeStats.queenBee)

    const rows = lifetimeStats.queryAllByTestId(
      'sb-lifetime-stats__row',
    )

    const divider = lifetimeStats.queryByTestId(
      'sb-lifetime-stats__divider',
    )

    expect(rows.length).toBe(2)
    expect(divider).toBeInTheDocument()
  })

  it('should show error message when there is a post error in addition to stats', () => {
    const { lifetimeStats, loadPlayerDoc, forcePostError } =
      renderLifetimeStats()
    loadPlayerDoc(fakeStats.amazing)
    const puzzles = lifetimeStats.getByTestId('stat-puzzles')
    const words = lifetimeStats.getByTestId('stat-words')
    const pangrams = lifetimeStats.getByTestId('stat-pangrams')
    const getErrorMsg = () =>
      lifetimeStats.queryByTestId('sb-lifetime-stats__post-error')

    expect(getStatLabel(puzzles)).toBe('Puzzles')
    expect(getStatValue(puzzles)).toBe('50')

    expect(getStatLabel(words)).toBe('Words')
    expect(getStatValue(words)).toBe('1,000')

    expect(getStatLabel(pangrams)).toBe('Pangrams')
    expect(getStatValue(pangrams)).toBe('40')

    expect(getErrorMsg()).not.toBeInTheDocument()

    forcePostError()

    expect(getStatLabel(puzzles)).toBe('Puzzles')
    expect(getStatValue(puzzles)).toBe('50')

    expect(getStatLabel(words)).toBe('Words')
    expect(getStatValue(words)).toBe('1,000')

    expect(getStatLabel(pangrams)).toBe('Pangrams')
    expect(getStatValue(pangrams)).toBe('40')

    expect(getErrorMsg()).toBeInTheDocument()
  })
})
