import React from 'react'
import cn from 'classnames'

import { useSbSelector } from '../../store/game/hooks'
import {
  selectStats,
  selectPostError,
} from '../../store/api/api.selectors'
import useWindowSize from '../../../../shared/hooks/use-window-size'

import * as fakeStats from './player'
import withControls from './withControls'

const StatLabelIcon = ({
  icon,
}: {
  icon: 'genius' | 'queen-bee'
}) => <div className={cn('sb-lifetime-stats__icon', icon)} />

export const Stat = ({
  id,
  label,
  children,
  icon,
  isLocked,
}: {
  id: string
  label: string
  children: React.ReactNode
  icon?: 'genius' | 'queen-bee'
  isLocked?: boolean
}) => (
  <div
    data-testid={`stat-${id}`}
    id={`sb-lifetime-stat__${id}`}
    className={cn('sb-lifetime-stat', { locked: isLocked })}
  >
    <div className="sb-lifetime-stat__value">{children}</div>
    <div className="sb-lifetime-stat__label">
      {icon && <StatLabelIcon icon={icon} />}
      {label}
    </div>
  </div>
)

const LongestWord = ({
  word,
  centerLetter,
}: {
  word: string
  centerLetter: string
}) => (
  <div className="sb-lifetime-stat__longest-word">
    {word.split('').map((l: string, i: number) =>
      l === centerLetter ? (
        <span key={i} className="center-letter">
          {l}
        </span>
      ) : (
        l
      ),
    )}
  </div>
)

export const Row = ({ children }: { children: React.ReactNode }) => (
  <div
    className="sb-lifetime-stats__row"
    data-testid="sb-lifetime-stats__row"
  >
    {children}
  </div>
)

const formatNumber = (num: number) =>
  Number(num).toLocaleString('en-US')

const getLabel = (labelSingle: string, value: number) => {
  if (value === 1) return labelSingle

  if (labelSingle === 'Genius') return 'Geniuses'

  return `${labelSingle}s`
}

export const LifetimeStats = ({
  useFakeUser,
  hasFakeXwd,
}: {
  useFakeUser?: boolean
  hasFakeXwd?: boolean
}) => {
  const windowSize = useWindowSize()
  const stats = useSbSelector(selectStats)
  const hasXwd = useSbSelector((state) => state.game.userType.hasXwd)
  const isSubscribed = useFakeUser ? hasFakeXwd : hasXwd
  const showPostErrorMsg = useSbSelector(selectPostError)

  if (!stats) return null

  const {
    puzzlesStarted,
    totalWords,
    totalPangrams,
    highestRank,
    geniuses,
    queenBees,
    longestWord,
  } = stats

  const showHighestRank = geniuses === 0
  const isDesktop = windowSize.winWidth >= 992

  return (
    <>
      {showPostErrorMsg && (
        <p
          className="sb-lifetime-stats__post-error"
          data-testid="sb-lifetime-stats__post-error"
        >
          We couldn’t sync your stats. Try again in a few.
        </p>
      )}
      <div className="sb-lifetime-stats">
        <Row>
          <Stat
            id="puzzles"
            label={getLabel('Puzzle', puzzlesStarted)}
          >
            {formatNumber(puzzlesStarted)}
          </Stat>
          <Stat id="words" label={getLabel('Word', totalWords)}>
            {formatNumber(totalWords)}
          </Stat>
          <Stat
            id="pangrams"
            label={getLabel('Pangram', totalPangrams)}
          >
            {formatNumber(totalPangrams)}
          </Stat>
        </Row>
        <Row>
          <div className="sb-lifetime-stats__group">
            {showHighestRank && (
              <Stat id="highest-rank" label="Highest Rank">
                {highestRank}
              </Stat>
            )}
            <Stat
              id="geniuses"
              label={getLabel('Genius', geniuses)}
              icon="genius"
              isLocked={!isSubscribed}
            >
              {formatNumber(geniuses)}
            </Stat>
            {!!queenBees && (
              <Stat
                id="queen-bees"
                label={getLabel('Queen Bee', queenBees)}
                icon="queen-bee"
                isLocked={!isSubscribed}
              >
                {formatNumber(queenBees)}
              </Stat>
            )}
          </div>
          {isDesktop && !!longestWord ? (
            <>
              <div
                className="sb-lifetime-stats__divider"
                data-testid="sb-lifetime-stats__divider"
              />
              <Stat id="longest-word" label="Longest Word">
                <LongestWord
                  word={longestWord.word}
                  centerLetter={longestWord.centerLetter}
                />
              </Stat>
            </>
          ) : null}
        </Row>
        {!isDesktop && !!longestWord ? (
          <Row>
            <Stat id="longest-word" label="Longest Word">
              <LongestWord
                word={longestWord.word}
                centerLetter={longestWord.centerLetter}
              />
            </Stat>
          </Row>
        ) : null}
      </div>
    </>
  )
}

export default withControls(LifetimeStats, fakeStats)
