import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import { getGameData } from 'foundation'

import { setUpStore } from '../../store'
import {
  UPDATE_MODAL,
  UPDATE_USER,
} from '../../store/game/game.actions'
import { mockLatestsState } from '../../__tests__/mockLatestsState'
import { mockGameData } from '../../__tests__/mockGameData'
import {
  MULTI_STATES_ERROR,
  MULTI_STATES_SUCCESS,
} from '../../../../../shared/moogle/moogleActions'

import ModalStats from './ModalStats'

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  userType: { hasXwd: false, regId: 2222 },
  trackInteraction: jest.fn(),
  track: jest.fn(),
  getGameData: jest.fn(() => {
    return require('../../__tests__/mockGameData').mockGameData
  }),
  env: { account: 'https://myaccount.nytimes.com' },
}))

jest.mock('../../../../../shared/abra/useFeatureFlag', () =>
  jest.fn(() => true),
)

describe('SPELLING BEE: MODAL STATS', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })

  const toggleModal = () => {}

  const connectModal = ({
    isLoggedIn = true,
    isMultiStateError = false,
    isModalOpen = true,
  } = {}) => {
    const store = setUpStore()
    store.dispatch({ type: UPDATE_USER, payload: { isLoggedIn } })
    if (isModalOpen) {
      store.dispatch({
        type: UPDATE_MODAL,
        payload: { isOpen: true },
      })
    }

    if (isMultiStateError) {
      store.dispatch({
        type: MULTI_STATES_ERROR,
        payload: { message: '' },
      })
    } else {
      store.dispatch({
        type: MULTI_STATES_SUCCESS,
        payload: { data: mockLatestsState },
      })
    }

    return mount(
      <Provider store={store}>
        <ModalStats toggleModal={toggleModal} />
      </Provider>,
    )
  }

  it('should render without crashing', () => {
    const component = connectModal()
    expect(component.length).toBeTruthy()
  })

  it('should render a modal header ', () => {
    const component = connectModal()
    const modalHeader = component.find('.sb-modal-header')

    expect(modalHeader.childAt(0).text()).toBe('Statistics')
  })

  it('should render a modal body', () => {
    const component = connectModal()
    const lastThisWeekItem = component
      .find('DailyRecapBar')
      .find('.sb-stats-bar')
      .last()
    expect(lastThisWeekItem.text()).toContain(
      'Play today’s Spelling Bee',
    )
  })

  it('should render a RankList component', () => {
    const component = connectModal()

    const rankList = component.find('RankList')

    expect(rankList.length).toBe(1)
  })

  it('should show regiwall if user is anon', () => {
    const component = connectModal({ isLoggedIn: false })
    const ranksHeader = component.find('.sb-modal-letters--condensed')
    const regiwall = component.find('.sb-regiwall-message')

    expect(ranksHeader.length).toBe(0)
    expect(regiwall.length).toBe(1)
  })

  it('should show message on API error from /latests', () => {
    const component = connectModal({ isMultiStateError: true })
    const message = component.find('.sb-modal-body').find('p').first()

    expect(message.text()).toContain('Oops')
  })

  it('should show error message if past puzzles data is null', () => {
    // @ts-ignore
    getGameData.mockReturnValueOnce({
      ...mockGameData,
      pastPuzzles: null,
    })
    const component = connectModal()
    const message = component.find('.sb-modal-body').find('p').first()

    expect(message.text()).toContain('Oops')
  })

  it('should not render a RankList component if transitioning back to game', () => {
    const component = connectModal({ isModalOpen: false })
    const rankList = component.find('RankList')

    expect(rankList.length).toBe(0)
  })

  it('should show dismissable message if user is not anon and there is no error message', () => {
    const component = connectModal()
    const dismissableMessage = component.find('DismissableMessage')

    expect(dismissableMessage.length).toBe(1)
  })

  it('should not show dismissable message if user is anon', () => {
    const component = connectModal({ isLoggedIn: false })
    const dismissableMessage = component.find('DismissableMessage')

    expect(dismissableMessage.length).toBe(0)
  })

  it('should not show dismissable message there is an error message', () => {
    const component = connectModal({ isMultiStateError: true })
    const dismissableMessage = component.find('DismissableMessage')

    expect(dismissableMessage.length).toBe(0)
  })
})
