import React from 'react'
import cn from 'classnames'

import useFeatureFlag from 'shared/abra/useFeatureFlag'

import {
  selectIsLocked,
  selectIsCurrentPuzzleFromLastWeek,
} from '../../store/game/game.selectors'
import { useSbSelector } from '../../store/game/hooks'
import withToggle from '../../../../shared/Toggle/Toggle'
import { ModalHeader, ModalBody } from '../ModalComponents'
import DismissableMessage from '../../../../shared/DismissableMessage'

import RankList from './RankList'
import LifetimeStats from './LifetimeStats'
import StatsRegiWall from './StatsRegiWall'

const ModalStats = ({
  toggleModal,
}: {
  toggleModal: (modalName?: string, trigger?: string) => void
}) => {
  /* Lifetime Stats Feature Flag */
  const showLifetimeStats = useFeatureFlag('sbLifetimeStats')

  /* Error Messaging State */
  const multiStateError = useSbSelector(
    (state) => state.moogle.getRequest.error,
  )

  const pastPuzzlesData = useSbSelector(
    (state) => state.game.pastPuzzlesData,
  )

  const isAnon = useSbSelector(
    (state) => !state.game.userType.isLoggedIn,
  )

  const shouldShowErrorMessage =
    !isAnon && (!pastPuzzlesData || multiStateError)

  /* Dismissable Message State */
  const showMessage =
    !isAnon && !shouldShowErrorMessage && showLifetimeStats

  /* Returning to Game State */
  const isLocked = useSbSelector(selectIsLocked)
  const isReturningToGame = !isLocked

  /* Toggle State */
  const isCurrentPuzzleFromLastWeek = useSbSelector(
    selectIsCurrentPuzzleFromLastWeek,
  )

  /* Helper Components */
  const ListWithToggle = withToggle(
    RankList,
    isCurrentPuzzleFromLastWeek,
  )

  const StatsContent = () => {
    if (isAnon) {
      return <StatsRegiWall />
    }

    return (
      <>
        {showLifetimeStats && <LifetimeStats />}
        <h4 className="sb-modal-letters--condensed">RANKS</h4>
        <div>
          <ListWithToggle
            labelA="This Week"
            labelB="Last Week"
            {...{ toggleModal }}
          />
        </div>
      </>
    )
  }

  const ErrorMessage = () => {
    return (
      <div className="sb-modal-message-container">
        <div className="sb-error__icon" />
        <p>
          Oops, something went wrong.
          <br />
          Please try again later.
        </p>
      </div>
    )
  }

  // if user is returning to game don't render modal stats,
  // prevents double render of stats
  if (isReturningToGame) return null

  return (
    <>
      <ModalHeader>
        {/* hide header if StatsRegiWall is showing and not in lifetime stats test */}
        <h3
          className={cn('sb-modal-title', {
            'visually-hidden': !showLifetimeStats && isAnon,
          })}
        >
          Statistics
        </h3>
      </ModalHeader>
      {showMessage && (
        <div className="sb-dismissable-message">
          <DismissableMessage
            title="New stats."
            text="See your puzzle count, pangrams, Geniuses, and more. Your stats are a reflection of August 2023 onward."
            localStorageKey="sb-ls-stats-msg"
          />
        </div>
      )}
      <ModalBody>
        {shouldShowErrorMessage ? <ErrorMessage /> : <StatsContent />}
      </ModalBody>
    </>
  )
}

export default ModalStats
