import React from 'react'
import { useDispatch } from 'react-redux'
import { Player } from '@nyt/games-schema'

import {
  MULTI_STATES_ERROR,
  MOOGLE_POST_ERROR,
  MOOGLE_POST_SUCCESS,
} from 'shared/moogle/moogleActions'

import { useSbSelector } from '../../store/game/hooks'

/*
  withControls is a temporary component meant for testing
  out different combinations of stats and user entitlements
  until we have data from the API to test. It also displays
  errors and the JSON response from the API. Can only be seen
  if you are in ShortzMode and if you append 'useFakeStats=true'
  to the url.
*/
const withControls = <P extends object>(
  Component: React.ComponentType<P>,
  fakeStats: { [key: string]: Player },
) => {
  const WithControls = (props: P) => {
    const dispatch = useDispatch()
    const puzzleId = useSbSelector((state) => state.game.id)
    const playerDocument = useSbSelector(
      (state) => state.moogle.player,
    )

    // Access
    const inShortzMode = useSbSelector(
      (state) => state.game.userType.inShortzMode,
    )

    const useFakeStats = window.location.href.includes(
      'useFakeStats=true',
    )

    const showControls = useFakeStats && inShortzMode

    // Default selection for dropdowns
    const [userOptionsState, setUserOptionsState] =
      React.useState('default')

    const [statsOptionsState, setStatsOptionsState] =
      React.useState('default')

    // Fake entitlement status
    const [useFakeUser, setUseFakeUser] = React.useState(false)
    const [hasFakeXwd, setHasFakeXwd] = React.useState(false)

    // Fake player stats helper
    const loadPlayer = (player: Player) => ({
      type: MOOGLE_POST_SUCCESS,
      payload: { puzzleId, player },
    })

    // Show API Response
    const resHidden = 'hidden'
    const [apiRes, setApiRes] = React.useState(resHidden)
    const toggleAPIResponse = () => {
      if (apiRes === resHidden) {
        setApiRes(JSON.stringify(playerDocument, null, 2))
      } else {
        setApiRes(resHidden)
      }
    }

    // Copy Button
    const [copyState, setCopyState] = React.useState(false)
    const copyJSON = () => {
      if (apiRes === resHidden) return
      navigator.clipboard.writeText(apiRes)
      setCopyState(true)
      setTimeout(() => setCopyState(false), 1000)
    }

    const Controls = () => (
      <>
        <button type="button" onClick={toggleAPIResponse}>{`${
          apiRes === resHidden ? 'Show' : 'Hide'
        } Response from API`}</button>
        {apiRes !== resHidden ? (
          <>
            <button type="button" onClick={copyJSON}>
              {copyState ? 'Copied!' : 'Copy'}
            </button>
            <pre id="api-json">{apiRes}</pre>
          </>
        ) : null}
        <div className="sb-lifetime-stats__controls">
          <select
            name="Fake User Options"
            value={userOptionsState}
            onChange={(event) => {
              const { value } = event.target
              setUserOptionsState(value)

              if (value === 'default') {
                setUseFakeUser(false)
              } else if (value === 'sub') {
                setUseFakeUser(true)
                setHasFakeXwd(true)
              } else if (value === 'nonsub') {
                setUseFakeUser(true)
                setHasFakeXwd(false)
              }
            }}
          >
            <option value="default">Default entitlements</option>
            <option value="sub">Subscriber</option>
            <option value="nonsub">Non-subscriber</option>
          </select>
          <select
            name="Fake Stats Options"
            value={statsOptionsState}
            onChange={(event) => {
              const { value } = event.target
              setStatsOptionsState(value)

              if (value === 'error') {
                dispatch({
                  type: MOOGLE_POST_ERROR,
                  payload: { puzzleId, message: 'forced error' },
                })
              } else if (value === 'get-error') {
                dispatch({
                  type: MULTI_STATES_ERROR,
                  payload: { message: 'forced error' },
                })
              } else {
                dispatch(loadPlayer(fakeStats[value]))
              }
            }}
          >
            <option value="default">Select fake stats</option>
            <option value="beginner">Beginner</option>
            <option value="amazing">Amazing</option>
            <option value="genius">Genius</option>
            <option value="queenBee">Queen Bee</option>
            <option value="newPlayer">New Player</option>
            <option value="maxedOut">Maxed Out</option>
            <option value="ones">Ones</option>
            <option value="emptyStats">Empty Stats</option>
            <option value="emptySBStats">Empty SB Stats</option>
            <option value="get-error">Get error</option>
            <option value="error">
              Post error - select state first
            </option>
          </select>
        </div>
      </>
    )

    return (
      <>
        {showControls && <Controls />}
        <Component
          {...props}
          useFakeUser={useFakeUser}
          hasFakeXwd={hasFakeXwd}
        />
      </>
    )
  }

  return WithControls
}

export default withControls
