import React, { useEffect, useState } from 'react'

import { trackImpression, trackInteraction } from 'foundation'

import Revealed, {
  RevealedProps,
} from '../../../shared/moments/components/Revealed'
import { useSbSelector } from '../store/game/hooks'
import {
  selectRankLabel,
  selectWordCount,
  selectIsModalOpen,
  selectCurrentRank,
  selectCurrentScore,
} from '../store/game/game.selectors'
import useFeatureFlag from '../../../../shared/abra/useFeatureFlag'

export type MomentRevealedProps = {
  toggleModal: (m?: string, origin?: string) => {}
  momentSystem: {
    transition: (from: string, to: string) => Promise<void>
  }
}

const MomentRevealed = ({ momentSystem }: MomentRevealedProps) => {
  const rank = useSbSelector(selectCurrentRank)
  const score = useSbSelector(selectCurrentScore)
  const wordCount = useSbSelector(selectWordCount)
  const rankLabel = useSbSelector(selectRankLabel)
  const isModalOpen = useSbSelector(selectIsModalOpen)
  const { id, printDate, isPastPuzzle, currentMoment } =
    useSbSelector((state) => state.game)

  const transitionToGame = () => {
    momentSystem.transition('revealed', 'game')
  }

  const [hasShownReveal, setHasShownReveal] = useState(false)
  const showOldIcons = useFeatureFlag('sbHubIconsOld')

  useEffect(() => {
    // Because the revealed moment gets rendered below the
    // Answers modal, we want want to wait for the modal
    // to be closed and fire off a trackImpression.
    // But we only want to do this once, right after
    // the user has seen the reveal moment.
    if (
      currentMoment === 'revealed' &&
      !isModalOpen &&
      !hasShownReveal
    ) {
      trackImpression(
        'congrats-revealed',
        'spelling-bee',
        rankLabel,
        `SB-${id}`,
      )
      setHasShownReveal(true)
    }
  }, [currentMoment, isModalOpen, hasShownReveal])

  const words = wordCount === 1 ? 'word' : 'words'
  const points = score === 1 ? 'point' : 'points'
  const description =
    wordCount === 0
      ? ''
      : `You found ${wordCount} ${words} with a<br />total of ${score} ${points}.`

  const admirePuzzleButton = {
    text: 'Admire this puzzle',
    action: () => {
      trackInteraction({
        element: {
          name: 'spelling-bee',
          label: 'close-reveal',
        },
        context: `SB-${id}`,
        name: {
          p: printDate,
          t: isPastPuzzle ? 'archive' : 'today',
        },
        region: 'revealed-congrats',
      })
      transitionToGame()
    },
    color: 'secondary',
    variant: 'extra-wide',
  }

  const revealedProps: RevealedProps = {
    title: rank,
    bgColor: '$white',
    close: {
      action: () => {
        transitionToGame()
      },
    },
    description: {
      text: description,
      variant: 'small',
    },
    icon: showOldIcons
      ? 'old-spelling-bee-gold-star'
      : 'spelling-bee-gold-star',
    buttons: [admirePuzzleButton],
  }

  return <Revealed {...revealedProps} />
}

export default MomentRevealed
