import React from 'react'

import { trackImpression, trackInteraction } from 'foundation'
import { useUser } from 'user'
import { isMobile } from 'responsive'
import useCTA from 'shared/marketing/useCTA'
import useAbra from 'shared/abra/useAbra'

import NonDismissalBanner from '../../../shared/Banner/NonDismissalBanner'
import { willUseMeterToGate } from '../../../../shared/meter'
import { handleSubscribe } from '../../../helpers/subscribeLinks'
import { useSbSelector } from '../store/game/hooks'
import {
  selectHasReachedCutoff,
  selectHasWon,
  selectHasCompleted,
  selectIsRevealed,
} from '../store/game/game.selectors'
import { selectHasReachedCutoff as selectHasReachedMeterCutoff } from '../store/meter/meter.selector'

import { MomentWelcomeProps } from './constants'
import MomentWelcome from './MomentWelcome'
import MomentWelcomePast from './MomentWelcomePast'
import MomentWelcomeBack from './MomentWelcomeBack'
import MomentWelcomeRevealed from './MomentWelcomeRevealed'
import MomentWelcomeBackPast from './MomentWelcomeBackPast'
import MomentWelcomeBackCutoff from './MomentWelcomeBackCutoff'

export const MomentWelcomeManager = ({
  isActive,
  toggleModal,
  momentSystem,
}: {
  isActive: boolean
  toggleModal: (m?: string, origin?: string) => {}
  momentSystem: {
    transition: (from: string, to: string) => Promise<void>
    sequencingClass: string
  }
}) => {
  const { isGamesSaleActive, getButtonText } = useCTA()

  // User
  const { hasLoaded, userType, updateUserType } = useUser()

  // Abra
  const [hasEarlyPaywall, setHasEarlyPaywall] = React.useState(false)
  const { getVariant, hasAbraLoaded } = useAbra()
  React.useEffect(() => {
    if (hasAbraLoaded) {
      // TODO: Abra cleanup
      const earlyPaywallVariant = getVariant(
        'GAMES_appPaywallGood_1214',
      )

      const hasEarlyPaywallVariant =
        earlyPaywallVariant === `1_paywallGood`
      setHasEarlyPaywall(hasEarlyPaywallVariant)
    }
  }, [hasAbraLoaded])

  // State
  const {
    editor,
    printDate,
    foundWords,
    displayDate,
    id: puzzleId,
    isPastPuzzle,
  } = useSbSelector((state) => state.game)
  const wordCount = foundWords.length
  const cutoff = useSbSelector(selectHasReachedCutoff)
  const earlyCutoff = useSbSelector(selectHasReachedCutoff)
  const meterCutOff = useSbSelector(selectHasReachedMeterCutoff)
  const gotQueenBee = useSbSelector(selectHasCompleted)
  const isRevealed = useSbSelector(selectIsRevealed)
  const gotGenius = useSbSelector(selectHasWon)

  // Tracking
  const trackSB = (
    elementLabel: string,
    label?: string,
    elementName?: string,
  ) => {
    trackInteraction({
      element: {
        name: elementName || 'spelling-bee',
        label: elementLabel,
      },
      context: `SB-${puzzleId}`,
      name: {
        p: printDate,
        t: isPastPuzzle ? 'archive' : 'today',
      },
      label,
    })
  }

  const transitionToGame = () => {
    let transitionTo = 'game'

    // if a player reached Queen Bee or Genius, display
    // the 'congrats' moment immediately after 'welcome'
    if (gotGenius || gotQueenBee) transitionTo = 'congrats'

    // if viewing the 'congrats' modal, deactivate mobile nav
    // tools so players can't do things in the background
    momentSystem.transition('welcome', transitionTo)
  }

  const openPastPuzzleHub = () => {
    // In the apps launch modal with hub
    // cards rather than redirect user
    if (!!window.gamesAppPlatform || !!window.newsreaderAppPlatform) {
      toggleModal('pastPuzzles')
    } else {
      window.location.href = '/puzzles/spelling-bee/hub'
    }
  }

  // Banner
  const onBannerVisible = React.useCallback(() => {
    const name = wordCount > 0 ? 'welcome-back' : 'welcome'
    trackImpression(
      name,
      'spelling-bee',
      'download-app',
      `SB-${puzzleId}`,
    )
  }, [wordCount])

  const isNewsreader = !!window.newsreaderAppPlatform
  // we want to show the app download CTA only when
  // a user is on desktop or webviews within the Games app
  // and allocated to variant, but also not in Past Puzzles
  const showAppDownloadCta =
    !isNewsreader &&
    !window.isHybridWebView &&
    isMobile &&
    !isPastPuzzle

  let shouldCutOff: boolean
  const usingMeter = willUseMeterToGate()

  if (usingMeter) {
    shouldCutOff = meterCutOff
  } else if (hasEarlyPaywall) {
    shouldCutOff = earlyCutoff
  } else {
    shouldCutOff = cutoff
  }

  // Moment specific props - any other info needed
  // can be retrieved in individual moment
  const welcomeProps: MomentWelcomeProps = {
    icon: 'spelling-bee',
    date: displayDate,
    title: 'Spelling Bee',
    editor,
    bgColor: '$spelling-bee-gold',
    wordCount,
    isActive,
    usingMeter,
    shouldCutOff,
    isSubscribed: hasLoaded && userType.hasXwd,
    transitionToGame,
    openPastPuzzleHub,
    isInGamesApp: !!window.gamesAppPlatform,
    handleSubscribe: (tag: string) =>
      handleSubscribe(tag, updateUserType),
    trackInteraction: trackSB,
    isGamesSale: isGamesSaleActive,
    gamesSaleText: getButtonText(),
  }

  const getWelcomeContent = () => {
    if (isPastPuzzle) {
      if (isRevealed) {
        return <MomentWelcomeRevealed {...welcomeProps} />
      }

      if (wordCount > 0) {
        return <MomentWelcomeBackPast {...welcomeProps} />
      }

      return <MomentWelcomePast {...welcomeProps} />
    }

    if (wordCount > 0) {
      return shouldCutOff ? (
        <MomentWelcomeBackCutoff {...welcomeProps} />
      ) : (
        <MomentWelcomeBack {...welcomeProps} />
      )
    }

    return <MomentWelcome {...welcomeProps} />
  }

  return (
    <>
      {getWelcomeContent()}

      {showAppDownloadCta && (
        <NonDismissalBanner
          title="Play Spelling Bee and more in the NYT Games app."
          icon="crossword-app"
          // Configuration: https://console.firebase.google.com/project/nyt-games-prd/durablelinks/links/https:~2F~2Ffpx3r.app.goo.gl
          ctaLink="https://fpx3r.app.goo.gl/app-sb"
          newTab
          onClickLink={() => {
            trackInteraction({
              type: 'load',
              element: {
                name: 'spelling-bee',
                label: 'welcome-download-app',
              },
              context: `SB-${puzzleId}`,
              name: {
                p: printDate,
                t: isPastPuzzle ? 'archive' : 'today',
              },
              label: 'https://fpx3r.app.goo.gl/app-sb',
            })
          }}
          onVisible={
            isActive // only call impression tracking once moment is visible
              ? onBannerVisible
              : undefined
          }
          showMultipleIconsBanner
        />
      )}
    </>
  )
}

export default MomentWelcomeManager
