import { fireEvent } from '@testing-library/react'
import { Middleware } from 'redux'

import { trackImpression } from 'foundation'

import { SpellingBeeState } from '../../store'
import MomentRevealed, {
  MomentRevealedProps,
} from '../MomentRevealed'
import {
  ACCEPT_WORD,
  updateModalRedux,
  updateCurrentMoment,
} from '../../store/game/game.actions'
import { mockGameData } from '../../__tests__/mockGameData'
import renderWithRankUp from '../../__tests__/renderWithRankUp'

const props = {
  toggleModal: jest.fn(),
  momentSystem: {
    transition: jest.fn().mockReturnValue(Promise.resolve()),
  },
}

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  getGameData: () => {
    return require('../../__tests__/mockGameData').mockGameData
  },
  trackImpression: jest.fn(),
}))

jest.mock('../../store/middleware/analytics', () => {
  const middlewareNoop: Middleware<{}, SpellingBeeState> =
    () => (next) => (action) =>
      next(action)

  return {
    analyticsMiddleware: middlewareNoop,
    trackGameStateMiddleware: middlewareNoop,
    trackUserStateMiddleware: middlewareNoop,
  }
})

jest.mock('../../../../../shared/meter', () => ({
  generateMeterUrl: jest.fn(),
  retrieveMeterValue: () => Promise.resolve(),
  willUseMeterToGate: () => false,
}))

const renderMoment = (props: MomentRevealedProps) =>
  renderWithRankUp(MomentRevealed, props)

describe('SPELLING BEE: MOMENTS - REVEALED', () => {
  const { location } = window
  let url = ''
  const getHrefSpy = jest.fn(() => url)
  const setHrefSpy = jest.fn((href) => {
    url = href
  })

  beforeAll(() => {
    // @ts-ignore
    delete window.location
    // @ts-ignore
    window.location = {}

    Object.defineProperty(window.location, 'href', {
      get: getHrefSpy,
      set: setHrefSpy,
    })

    beforeEach(() => {
      window.gamesAppPlatform = ''
      window.newsreaderAppPlatform = ''
      window.isHybridWebView = false
    })
  })

  test('User reveals with no progress', () => {
    const { component: moment } = renderMoment(props)

    const headings = moment.queryAllByRole('heading')
    const rank = headings[0]
    const summary = headings[1]

    expect(rank.textContent).toBe('Beginner')
    expect(summary.textContent).toBe('')
  })

  test('User reveals with one word entered', () => {
    const { component: moment, store } = renderMoment(props)
    const fourLetterWord =
      mockGameData.today.answers.find((w) => w.length === 4) || 'alit'

    store.dispatch({
      type: ACCEPT_WORD,
      payload: fourLetterWord,
    })

    const headings = moment.queryAllByRole('heading')
    const rank = headings[0]
    const summary = headings[1]

    expect(rank.textContent).toBe('Beginner')
    expect(summary.textContent).toContain('1 word')
    expect(summary.textContent).toContain('1 point')
  })

  test('User reveals after reaching Amazing', () => {
    const { component: moment, rankUp } = renderMoment(props)
    rankUp(7)

    const headings = moment.queryAllByRole('heading')
    const rank = headings[0]
    const summary = headings[1]

    expect(rank.textContent).toBe('Amazing')
    expect(summary.textContent).toContain('words')
    expect(summary.textContent).toContain('points')
  })

  test('User reveals after reaching Genius', () => {
    const { component: moment, rankUp } = renderMoment(props)
    rankUp(8)

    const headings = moment.queryAllByRole('heading')
    const rank = headings[0]

    expect(rank.textContent).toBe('Genius')
  })

  test('User reveals after reaching Queen Bee', () => {
    const { component: moment, rankUp } = renderMoment(props)
    rankUp(9)

    const headings = moment.queryAllByRole('heading')
    const rank = headings[0]

    expect(rank.textContent).toBe('Queen Bee')
  })

  test('Clicking x button closes moment', () => {
    const { component: moment } = renderMoment(props)

    const buttons = moment.queryAllByRole('button')
    const close = buttons[0]

    expect(close.textContent).toBe('×')
    fireEvent.click(close)
    expect(props.momentSystem.transition).toHaveBeenCalledWith(
      'revealed',
      'game',
    )
  })

  test('Clicking close button closes moment', () => {
    const { component: moment } = renderMoment(props)

    const buttons = moment.queryAllByRole('button')
    const close = buttons[1]

    expect(close.textContent).toBe('Admire this puzzle')
    fireEvent.click(close)
    expect(props.momentSystem.transition).toHaveBeenCalledWith(
      'revealed',
      'game',
    )
  })

  test('trackImpression gets called after answers modal is closed', () => {
    jest.resetAllMocks()
    const { store } = renderMoment(props)
    store.dispatch(updateCurrentMoment('revealed'))
    store.dispatch(updateModalRedux(false))

    expect(trackImpression).toHaveBeenCalled()
  })

  test('trackImpression gets called first time modal is closed and not when modal is opened and closed again', () => {
    jest.resetAllMocks()
    const { store } = renderMoment(props)
    store.dispatch(updateCurrentMoment('revealed'))
    store.dispatch(updateModalRedux(false))
    store.dispatch(updateModalRedux(true))
    store.dispatch(updateModalRedux(false))

    expect(trackImpression).toHaveBeenCalledTimes(1)
  })

  afterAll(() => {
    window.location = location
  })
})
