import { act } from 'react-dom/test-utils'
import { fireEvent } from '@testing-library/react'

import { trackImpression } from 'foundation'

import { MomentWelcomeProps } from '../constants'
import MomentWelcomeBackPast from '../MomentWelcomeBackPast'
import renderWithRankUp from '../../__tests__/renderWithRankUp'
import { updateCurrentMoment } from '../../store/game/game.actions'
import { mockGameData } from '../../__tests__/mockGameData'

const props = {
  icon: 'spelling-bee',
  date: mockGameData.today.displayDate,
  title: 'Spelling Bee',
  editor: 'Sam Ezersky',
  bgColor: '$spelling-bee-gold',
  wordCount: 0,
  isActive: false,
  usingMeter: true,
  isSubscribed: false,
  shouldCutOff: false,
  isInGamesApp: false,
  transitionToGame: jest.fn(),
  handleSubscribe: jest.fn(),
  trackInteraction: jest.fn(),
  openPastPuzzleHub: jest.fn(),
  isGamesSale: false,
  gamesSaleText: 'Subscribe for 50% off',
}

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  trackImpression: jest.fn(),
  getGameData: () => {
    return require('../../__tests__/mockGameData').mockGameData
  },
}))

const renderMoment = (props: MomentWelcomeProps) =>
  renderWithRankUp(MomentWelcomeBackPast, props)

describe('SPELLING BEE: MOMENTS - WELCOME BACK PAST', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  test('should see Past Puzzles heading and puzzle info', () => {
    const { component: moment } = renderMoment(props)
    const headings = moment.getAllByRole('heading')
    const date = moment.getByTestId('pz-moment__date')
    const editor = moment.getByTestId('pz-moment__editor')
    const { displayWeekday, displayDate } = mockGameData.today
    expect(headings[0].textContent).toBe('PAST PUZZLES')
    expect(headings[1].textContent).toBe(displayWeekday)
    expect(date.firstChild?.textContent).toBe(displayDate)
    expect(editor.firstChild?.textContent).toBe(
      'Edited by Sam Ezersky',
    )
  })

  test('if user has access, they should be able to play the puzzle', () => {
    const { component: moment } = renderMoment(props)
    const button = moment.queryAllByRole('button')[0]
    expect(button.textContent).toBe('Continue')
    fireEvent.click(button)
    expect(props.transitionToGame).toHaveBeenCalled()
  })

  test('if user does not have access, they should be taken to subscribe page', () => {
    const propsNoAccess = { ...props, shouldCutOff: true }
    const { component: moment } = renderMoment(propsNoAccess)
    const button = moment.getByRole('button')
    expect(button.textContent).toBe('Subscribe')
    fireEvent.click(button)
    expect(props.handleSubscribe).toHaveBeenCalled()
  })

  test('if user has access and they have reached genius or higher, they should see their score', () => {
    const { component: moment, rankUp } = renderMoment(props)
    rankUp(8)
    const description = moment.getAllByRole('heading')[2].textContent
    const button = moment.queryAllByRole('button')[0]
    const recapMsg = ['You have', 'points.']
    const geniusMsg = ['You’re', 'points away from Genius!']
    expect(description?.includes(recapMsg[0])).toBe(true)
    expect(description?.includes(recapMsg[1])).toBe(true)
    expect(description?.includes(geniusMsg[0])).toBe(false)
    expect(description?.includes(geniusMsg[1])).toBe(false)
    expect(button.textContent).toBe('Play')
  })

  test('if user has access and they have not reached genius, they should see their score and number of points to Genius', () => {
    const { component: moment } = renderMoment(props)
    const description = moment.getAllByRole('heading')[2].textContent
    const button = moment.queryAllByRole('button')[0]
    const recapMsg = ['You have', 'points.']
    const geniusMsg = ['You’re', 'points away from Genius!']
    expect(description?.includes(recapMsg[0])).toBe(true)
    expect(description?.includes(recapMsg[1])).toBe(true)
    expect(description?.includes(geniusMsg[0])).toBe(true)
    expect(description?.includes(geniusMsg[1])).toBe(true)
    expect(button.textContent).toBe('Continue')
  })

  test('if user does not have access, they should see subscribe message', () => {
    const propsNoAccess = { ...props, shouldCutOff: true }
    const { component: moment } = renderMoment(propsNoAccess)
    const button = moment.getByRole('button')
    const description = moment.getAllByRole('heading')[2].textContent
    expect(button.textContent).toBe('Subscribe')
    expect(description?.includes('Want to continue')).toBe(true)
  })

  test('if not using meter and user has a subscription, they should be able to play the puzzle', () => {
    const propsSub = {
      ...props,
      usingMeter: false,
      isSubscribed: true,
    }
    const { component: moment } = renderMoment(propsSub)
    const button = moment.queryAllByRole('button')[0]
    expect(button.textContent).toBe('Continue')
    fireEvent.click(button)
    expect(props.transitionToGame).toHaveBeenCalled()
  })

  test('if not using meter and user does not have a subscription, they should see subscribe message', () => {
    const propsNonSub = {
      ...props,
      usingMeter: false,
      isSubscribed: false,
    }
    const { component: moment } = renderMoment(propsNonSub)
    const button = moment.getByRole('button')
    const description = moment.getAllByRole('heading')[2].textContent
    expect(button.textContent).toBe('Subscribe')
    expect(description?.includes('Want to continue')).toBe(true)
  })

  test('if the user is a sub, in the games app, they should be able go to past puzzle hub', () => {
    const propsSubGamesApp = {
      ...props,
      isSubscribed: true,
      isInGamesApp: true,
    }
    const { component: moment } = renderMoment(propsSubGamesApp)
    const firstButton = moment.queryAllByRole('button')[0]
    const secondButton = moment.queryAllByRole('button')[1]
    expect(firstButton.textContent).toBe('Continue')
    expect(secondButton.textContent).toContain('Past Puzzles')
    fireEvent.click(secondButton)
    expect(props.openPastPuzzleHub).toHaveBeenCalled()
  })

  test('if user does not have access, a hardpaywall impression event should fire', () => {
    const propsNoAccess = { ...props, shouldCutOff: true }
    const { store } = renderMoment(propsNoAccess)
    act(() => {
      store.dispatch(updateCurrentMoment('welcome'))
    })
    expect(trackImpression).toHaveBeenCalled()
  })

  test('if user has access, a hardpaywall impression event should not fire', () => {
    const { store } = renderMoment(props)
    act(() => {
      store.dispatch(updateCurrentMoment('welcome'))
    })

    expect(trackImpression).not.toHaveBeenCalled()
  })
})
