import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import * as responsive from 'responsive'

import { MomentWelcomeManager } from '../MomentWelcomeManager'
import { MomentWelcome } from '../MomentWelcome'
import { MomentWelcomeBack } from '../MomentWelcomeBack'
import { MomentWelcomeBackCutoff } from '../MomentWelcomeBackCutoff'
import { MomentWelcomePast } from '../MomentWelcomePast'
import { MomentWelcomeBackPast } from '../MomentWelcomeBackPast'
import { MomentWelcomeRevealed } from '../MomentWelcomeRevealed'

jest.mock('foundation', () => ({
  trackClick: jest.fn(),
  trackImpression: jest.fn(),
  trackModuleInteraction: jest.fn(),
  track: jest.fn(),
  getGameData: () => ({
    today: { id: 1 },
  }),
  mobileNavTools: {
    activate: jest.fn(),
    deactivate: jest.fn(),
  },
}))

jest.mock('user', () => ({
  useUser: () => ({
    userType: {
      hasXwd: false,
    },
  }),
}))

jest.mock('../../../../../shared/abra/useAbra', () => () => ({
  getVariant: () => '1_Download_App_Banner',
  reportExposure: () => null,
}))

const props = {
  momentSystem: {
    transition: jest.fn().mockReturnValue(Promise.resolve()),
  },
  toggleModal: jest.fn(),
  isActive: true,
}

const mockStore = (state) => {
  return {
    default: jest.fn(),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
    getState: () => state,
  }
}

const puzzleData = {
  displayDate: 'March 5, 2021',
  editor: 'Sam Ezersky',
  foundWords: [],
  outerLetters: ['b', 'c', 'i', 'e', 'l', 'a'],
  centerLetter: 'k',
  pangrams: ['clickable'],
  answers: [
    'clickable',
    'aback',
    'babka',
    'bake',
    'balk',
    'cackle',
    'cake',
    'kebab',
    'kickball',
    'likeable',
  ],
  isRevealed: false,
  isPastPuzzle: false,
  userType: {
    hasXwd: true,
  },
}

const createComponent = (puzzleData) => {
  const store = mockStore({
    game: puzzleData,
    meter: {
      granted: true,
    },
  })

  const wrapper = mount(
    <Provider store={store}>
      <MomentWelcomeManager {...props} />
    </Provider>,
  )

  return wrapper
}

describe('MOMENT WELCOME MANAGER: MANAGEMENT', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  it('should show MomentWelcome if user has no progress', () => {
    const wrapper = createComponent(puzzleData)

    expect(wrapper.find(MomentWelcome)).toHaveLength(1)
  })

  it('should show WelcomeBack if user has not reached cutoff rank', () => {
    const puzzleDataWithProgress = {
      ...puzzleData,
      foundWords: ['clickable'],
    }
    const wrapper = createComponent(puzzleDataWithProgress)

    expect(wrapper.find(MomentWelcomeBack)).toHaveLength(1)
  })

  it('should show WelcomeBackCutoff if user reached cutoff rank', () => {
    const puzzleDataWithProgress = {
      ...puzzleData,
      foundWords: ['aback', 'babka'],
      userType: {
        hasXwd: false,
      },
    }
    const wrapper = createComponent(puzzleDataWithProgress)

    expect(wrapper.find(MomentWelcomeBackCutoff)).toHaveLength(1)
  })

  it('should show WelcomePast if is past puzzle and user has no progress', () => {
    const pastPuzzleData = { ...puzzleData, isPastPuzzle: true }
    const wrapper = createComponent(pastPuzzleData)

    expect(wrapper.find(MomentWelcomePast)).toHaveLength(1)
  })

  it('should show WelcomeBackPast if is past puzzle and user has progress', () => {
    const pastPuzzleDataWithProgress = {
      ...puzzleData,
      foundWords: ['clickable'],
      isPastPuzzle: true,
    }
    const wrapper = createComponent(pastPuzzleDataWithProgress)

    expect(wrapper.find(MomentWelcomeBackPast)).toHaveLength(1)
  })

  it('should show welcome revealed & prepare to show game after welcome if user has revealed', () => {
    const pastPuzzleDataWithProgress = {
      ...puzzleData,
      foundWords: ['clickable'],
      isRevealed: true,
      isPastPuzzle: true,
    }
    const wrapper = createComponent(pastPuzzleDataWithProgress)
    wrapper.find(MomentWelcomeRevealed).props().transitionToGame()
    expect(props.momentSystem.transition).toHaveBeenCalledWith(
      'welcome',
      'game',
    )
  })

  it('should show app download cta in mobile web', () => {
    responsive.isMobile = true
    const wrapper = createComponent(puzzleData)
    expect(wrapper.text()).toContain('Play Spelling Bee and more')
  })

  it('should not show app download cta in desktop', () => {
    responsive.isMobile = false

    const wrapper = createComponent(puzzleData)
    expect(wrapper.text()).not.toContain('Play Spelling Bee and more')
  })

  it('should not show app download cta in the games app', () => {
    responsive.isMobile = true
    window.gamesAppPlatform = 'ios'
    window.isHybridWebView = true

    const wrapper = createComponent(puzzleData)
    expect(wrapper.text()).not.toContain('Play Spelling Bee and more')
  })

  it('should not show app download cta in newsreader', () => {
    responsive.isMobile = true
    window.newsreaderAppPlatform = 'ios'

    const wrapper = createComponent(puzzleData)
    expect(wrapper.text()).not.toContain('Play Spelling Bee and more')
  })
})
