import React from 'react'
import { Provider } from 'react-redux'
import { act } from 'react-dom/test-utils'
import { render, screen, fireEvent } from '@testing-library/react'

import { trackImpression } from 'foundation'

import { setUpStore } from '../../store'
import { MomentWelcomeProps } from '../constants'
import MomentWelcomePast from '../MomentWelcomePast'
import { updateCurrentMoment } from '../../store/game/game.actions'
import { mockGameData } from '../../__tests__/mockGameData'

const props = {
  icon: 'spelling-bee',
  date: mockGameData.today.displayDate,
  title: 'Spelling Bee',
  editor: 'Sam Ezersky',
  bgColor: '$spelling-bee-gold',
  wordCount: 0,
  isActive: false,
  usingMeter: true,
  isSubscribed: false,
  shouldCutOff: false,
  isInGamesApp: false,
  transitionToGame: jest.fn(),
  handleSubscribe: jest.fn(),
  trackInteraction: jest.fn(),
  openPastPuzzleHub: jest.fn(),
  isGamesSale: false,
  gamesSaleText: 'Subscribe for 50% off',
}

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  trackImpression: jest.fn(),
  getGameData: () => {
    return require('../../__tests__/mockGameData').mockGameData
  },
}))

const renderMoment = (props: MomentWelcomeProps) => {
  const store = setUpStore()

  const welcomePast = render(
    <Provider store={store}>
      <MomentWelcomePast {...props} />
    </Provider>,
  )

  return { welcomePast, store }
}

describe('SPELLING BEE: MOMENTS - WELCOME PAST', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  test('should see Past Puzzles heading and puzzle info', () => {
    renderMoment(props)
    const headings = screen.getAllByRole('heading')
    const date = screen.getByTestId('pz-moment__date')
    const editor = screen.getByTestId('pz-moment__editor')
    expect(headings[0].textContent).toBe('PAST PUZZLES')
    const { displayDate, displayWeekday } = mockGameData.today
    expect(headings[1].textContent).toBe(displayWeekday)
    expect(date.firstChild?.textContent).toBe(displayDate)
    expect(editor.firstChild?.textContent).toBe(
      'Edited by Sam Ezersky',
    )
  })

  test('if user has access, they should be able to play the puzzle', () => {
    renderMoment(props)
    const button = screen.queryAllByRole('button')[0]
    expect(button.textContent).toBe('Play')
    fireEvent.click(button)
    expect(props.transitionToGame).toHaveBeenCalled()
  })

  test('if user does not have access, they should be taken to subscribe page', () => {
    const propsNoAccess = { ...props, shouldCutOff: true }
    renderMoment(propsNoAccess)
    const button = screen.getByRole('button')
    expect(button.textContent).toBe('Subscribe')
    fireEvent.click(button)
    expect(props.handleSubscribe).toHaveBeenCalled()
  })

  test('if user has access, they should see welcome message', () => {
    renderMoment(props)
    const description = screen.getAllByRole('heading')[2].textContent
    const playMsg = 'Can you get to Genius?'
    expect(description?.includes(playMsg)).toBe(true)
  })

  test('if user does not have access, they should see subscribe message', () => {
    const propsNoAccess = { ...props, shouldCutOff: true }
    renderMoment(propsNoAccess)
    const button = screen.getByRole('button')
    const description = screen.getAllByRole('heading')[2].textContent
    expect(button.textContent).toBe('Subscribe')
    expect(description?.includes('Want to play past puzzles?')).toBe(
      true,
    )
  })

  test('if not using meter and user has a subscription, they should see welcome message', () => {
    const propsSub = {
      ...props,
      usingMeter: false,
      isSubscribed: true,
    }
    renderMoment(propsSub)
    const description = screen.getAllByRole('heading')[2].textContent
    const playMsg = 'Can you get to Genius?'
    expect(description?.includes(playMsg)).toBe(true)
  })

  test('if not using meter and user does not have a subscription, they should see a subscribe message', () => {
    const propsNonSub = {
      ...props,
      usingMeter: false,
      isSubscribed: false,
    }
    renderMoment(propsNonSub)
    const button = screen.getByRole('button')
    const description = screen.getAllByRole('heading')[2].textContent
    expect(button.textContent).toBe('Subscribe')
    expect(description?.includes('Want to play past puzzles?')).toBe(
      true,
    )
  })

  test('if the user is a sub, in the games app, they should be able go to past puzzle hub', () => {
    const propsSubGamesApp = {
      ...props,
      isSubscribed: true,
      isInGamesApp: true,
    }
    renderMoment(propsSubGamesApp)
    const firstButton = screen.queryAllByRole('button')[0]
    const secondButton = screen.queryAllByRole('button')[1]
    expect(firstButton.textContent).toBe('Play')
    expect(secondButton.textContent).toContain('Past Puzzles')
    fireEvent.click(secondButton)
    expect(props.openPastPuzzleHub).toHaveBeenCalled()
  })

  test('if user does not have access, a hardpaywall impression event should fire', () => {
    const propsNoAccess = { ...props, shouldCutOff: true }
    const { store } = renderMoment(propsNoAccess)
    act(() => {
      store.dispatch(updateCurrentMoment('welcome'))
    })
    expect(trackImpression).toHaveBeenCalled()
  })

  test('if user has access, a hardpaywall impression event should not fire', () => {
    const { store } = renderMoment(props)
    act(() => {
      store.dispatch(updateCurrentMoment('welcome'))
    })

    expect(trackImpression).not.toHaveBeenCalled()
  })
})
