import React from 'react'
import { Provider } from 'react-redux'
import { render, screen, fireEvent } from '@testing-library/react'

import { trackInteraction } from 'foundation'

import { setUpStore } from '../../store'
import { MomentWelcomeProps } from '../constants'
import MomentWelcomeRevealed from '../MomentWelcomeRevealed'
import { mockGameData } from '../../__tests__/mockGameData'

const props = {
  icon: 'spelling-bee',
  date: mockGameData.today.displayDate,
  title: 'Spelling Bee',
  editor: 'Sam Ezersky',
  bgColor: '$spelling-bee-gold',
  wordCount: 0,
  isActive: false,
  usingMeter: true,
  isSubscribed: false,
  shouldCutOff: false,
  isInGamesApp: false,
  transitionToGame: jest.fn(),
  handleSubscribe: jest.fn(),
  trackInteraction: jest.fn(),
  openPastPuzzleHub: jest.fn(),
  isGamesSale: false,
  gamesSaleText: 'Subscribe for 50% off',
}

jest.mock('foundation', () => ({
  ...jest.requireActual('foundation'),
  trackInteraction: jest.fn(),
  getGameData: () => {
    return require('../../__tests__/mockGameData').mockGameData
  },
}))

const renderMoment = (props: MomentWelcomeProps) => {
  const store = setUpStore()

  const welcomeRevealed = render(
    <Provider store={store}>
      <MomentWelcomeRevealed {...props} />
    </Provider>,
  )

  return { welcomeRevealed, store }
}

describe('SPELLING BEE: MOMENTS - WELCOME REVEALED', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  test('should see Past Puzzles heading and puzzle info', () => {
    renderMoment(props)
    const headings = screen.getAllByRole('heading')
    const date = screen.getByTestId('pz-moment__date')
    expect(headings[0].textContent).toBe('PAST PUZZLES')
    const { displayDate, displayWeekday } = mockGameData.today
    expect(headings[1].textContent).toBe(displayWeekday)
    expect(date.firstChild?.textContent).toBe(displayDate)
  })

  test('should display description and buttons', () => {
    const { welcomeRevealed } = renderMoment(props)
    const description =
      welcomeRevealed.getAllByRole('heading')[2].textContent
    const buttons = welcomeRevealed.queryAllByRole('button')
    const recapMsg = ['You got', 'this puzzle!']
    expect(description?.includes(recapMsg[0])).toBe(true)
    expect(description?.includes(recapMsg[1])).toBe(true)
    expect(buttons[0].textContent).toBe('All Past Puzzles')
    expect(buttons[1].textContent).toBe('Admire this puzzle >')
  })

  test('should fire tracking event and open puzzle hub when "All Past Puzzles" button is clicked', () => {
    const { welcomeRevealed } = renderMoment(props)
    const pastPuzzlesButton =
      welcomeRevealed.queryAllByRole('button')[0]
    fireEvent.click(pastPuzzlesButton)
    expect(props.openPastPuzzleHub).toHaveBeenCalled()
    expect(trackInteraction).toHaveBeenCalled()
  })

  test('should fire tracking event and close moment when "Admire this puzzle" button is clicked', () => {
    const { welcomeRevealed } = renderMoment(props)
    const admirePuzzleButton =
      welcomeRevealed.queryAllByRole('button')[1]
    fireEvent.click(admirePuzzleButton)
    expect(props.transitionToGame).toHaveBeenCalled()
    expect(trackInteraction).toHaveBeenCalled()
  })
})
