import { SpellingBeeState } from '..'
import {
  INSERT_FOUND_WORDS,
  REVEAL,
  SET_HAS_LOADED,
} from '../game/game.actions'
import {
  cleanSpellingBeeLocalStorage,
  options,
  yesterdayOptions,
} from '../spellingBeeMoogleConfig'

type DeepPartial<T> = T extends object
  ? {
      [P in keyof T]?: DeepPartial<T[P]>
    }
  : T

describe('spellingBeeMoogleConfig', () => {
  afterEach(() => {
    window.localStorage.clear()
  })

  it('reconciles states if both are null', () => {
    expect(options.reconcileRemoteAndLocalState(null, null)).toEqual(
      null,
    )
  })

  it('reconciles state if one is null', () => {
    const state = {
      answers: ['hey'],
      isRevealed: false,
      timestamp: 42,
      schemaVersion: '0.0.0',
    }
    expect(options.reconcileRemoteAndLocalState(state, null)).toEqual(
      state,
    )
    expect(options.reconcileRemoteAndLocalState(null, state)).toEqual(
      state,
    )
  })

  it('reconciles states with overlapping words', () => {
    const state1 = {
      answers: ['hey'],
      isRevealed: false,
      timestamp: 42,
      schemaVersion: '0.0.0',
    }

    const state2 = {
      answers: ['hey', 'hello'],
      isRevealed: false,
      timestamp: 42,
      schemaVersion: '0.0.0',
    }

    expect(
      options.reconcileRemoteAndLocalState(state1, state2),
    ).toEqual({
      answers: ['hey', 'hello'],
      isRevealed: false,
    })
  })

  it('inserts progress, not revealed', () => {
    const gameData = { answers: ['hey'], isRevealed: false }
    const dispatch = jest.fn()
    // @ts-ignore
    options.insertProgress(gameData)(dispatch)

    expect(dispatch).toHaveBeenCalledTimes(3)
    expect(dispatch).toHaveBeenCalledWith({
      type: INSERT_FOUND_WORDS,
      payload: ['hey'],
    })
    expect(dispatch).toHaveBeenCalledWith({ type: SET_HAS_LOADED })
  })

  it('inserts progress, revealed', () => {
    const gameData = { answers: ['hey'], isRevealed: true }
    const dispatch = jest.fn()
    // @ts-ignore
    options.insertProgress(gameData)(dispatch)

    expect(dispatch).toHaveBeenCalledTimes(4)
    expect(dispatch).toHaveBeenCalledWith({
      type: INSERT_FOUND_WORDS,
      payload: ['hey'],
    })
    expect(dispatch).toHaveBeenCalledWith({ type: SET_HAS_LOADED })
    expect(dispatch).toHaveBeenCalledWith({ type: REVEAL })
  })

  it('clears away old local storage', () => {
    window.localStorage.setItem('nytsb/ALONGLONGTIMEAGO/ANON', 'test')
    window.localStorage.setItem('nytsb/05-11-2023/ANON', 'test')

    const state: DeepPartial<SpellingBeeState> = {
      game: {
        printDate: '05-11-2023',
        pastPuzzlesData: {
          thisWeek: [{ printDate: '05-11-2023' }],
          lastWeek: [{ printDate: '05-05-2023' }],
        },
      },
    }

    const dispatch = jest.fn()
    const getState = () => state

    // @ts-ignore
    cleanSpellingBeeLocalStorage()(dispatch, getState)
    expect(
      window.localStorage.getItem('nytsb/ALONGLONGTIMEAGO/ANON'),
    ).toEqual(null)
    expect(
      window.localStorage.getItem('nytsb/05-11-2023/ANON'),
    ).toEqual('test')
  })

  it('should persist rank on todays puzzle', () => {
    const state: DeepPartial<SpellingBeeState> = {
      game: {
        foundWords: ['hello'],
        isRevealed: false,
        answers: ['hello'],
        pangrams: [],
      },
    }

    const persistedState = options.selectPersistedState(
      state as SpellingBeeState,
    )
    expect(persistedState).toEqual({
      answers: ['hello'],
      isRevealed: false,
      rank: 'Queen Bee',
    })
  })

  it('should persist rank on yesterdays puzzle', () => {
    const state: DeepPartial<SpellingBeeState> = {
      game: {
        yesterday: {
          foundWords: ['hello'],
          isRevealed: false,
          answers: ['hello'],
          pangrams: [],
        },
      },
    }

    const persistedState = yesterdayOptions.selectPersistedState(
      state as SpellingBeeState,
    )
    expect(persistedState).toEqual({
      answers: ['hello'],
      isRevealed: false,
      rank: 'Queen Bee',
    })
  })
})
