import { mockGameData } from '../../__tests__/mockGameData'
import { mockLatestsState } from '../../__tests__/mockLatestsState'

import {
  getRankStats,
  getProgressMap,
  formatStats,
  getPostError,
} from './api.selectors'
import { SpellingBeeMoogleData } from './api.state'

describe('Spelling Bee store: API selectors', () => {
  it('should fill in remaining days of this week, when not included in activePuzzles', () => {
    const activePuzzles = {
      ...mockGameData.pastPuzzles,
      thisWeek: mockGameData.pastPuzzles.thisWeek.slice(0, 3),
    }

    const rankStats = getRankStats(
      mockLatestsState as SpellingBeeMoogleData[],
      activePuzzles,
    )

    expect(rankStats?.thisWeek.length).toEqual(7)
    expect(rankStats?.thisWeek[6].printDate).toEqual('2023-07-23')
  })

  it('should give an empty progressMap when no rankStats exist', () => {
    const emptyRankStats = getRankStats(
      null,
      mockGameData.pastPuzzles,
    )
    const progressMap = getProgressMap(emptyRankStats)
    expect(progressMap).toEqual({})
  })

  it('should flatten rankStats into progressMap', () => {
    const rankStats = getRankStats(
      mockLatestsState as SpellingBeeMoogleData[],
      mockGameData.pastPuzzles,
    )
    const progressMap = getProgressMap(rankStats)

    expect(progressMap[15707].rank).toEqual('Genius')
    expect(progressMap[19100].rank).toEqual('Amazing')
    expect(progressMap[27090].rank).toEqual('Nice')
  })

  it('should pass in value for puzzles, words, pangrams', () => {
    const formattedStats = formatStats({
      puzzles_started: 5,
      total_words: 10,
      total_pangrams: 2,
    })

    expect(formattedStats).toMatchObject({
      puzzlesStarted: 5,
      totalWords: 10,
      totalPangrams: 2,
    })
  })

  it('should provide default values if object is empty', () => {
    const formattedStats = formatStats({})
    expect(formattedStats).toMatchObject({
      puzzlesStarted: 0,
      totalWords: 0,
      totalPangrams: 0,
      longestWord: null,
      geniuses: 0,
      queenBees: 0,
    })
  })

  it('should create longest word object', () => {
    const formattedStats = formatStats({
      longest_word: {
        word: 'milennium',
        center_letter: 'm',
        print_date: '2022-10-21',
      },
    })

    expect(formattedStats?.longestWord).toEqual({
      word: 'milennium',
      centerLetter: 'm',
    })
  })

  it('should return highest rank value', () => {
    const formattedStats = formatStats({
      ranks: {
        'Good Start': 2,
        'Moving Up': 4,
        Good: 5,
      },
    })

    expect(formattedStats?.highestRank).toBe('Good')
  })

  it('should return number of Geniuses inclusive of number of Queen Bees', () => {
    const formattedStats = formatStats({
      ranks: {
        'Good Start': 2,
        'Moving Up': 4,
        Good: 5,
        Genius: 10,
        'Queen Bee': 2,
      },
    })

    expect(formattedStats?.geniuses).toBe(12)
    expect(formattedStats?.queenBees).toBe(2)
  })

  it('should return false when there is no post request data for a given puzzle id or at all', () => {
    const isErrNoPostReqs = getPostError(1, {})
    const isErrNoPuzzleId = getPostError(1, {
      '2': {
        isLoading: false,
        error: true,
        errorMessage: 'some error',
        isErrorRetry: false,
      },
    })

    expect(isErrNoPostReqs).toBe(false)
    expect(isErrNoPuzzleId).toBe(false)
  })

  it('should return post error for given puzzle id', () => {
    const isErr = getPostError(1, {
      '1': {
        isLoading: false,
        error: true,
        errorMessage: 'some error',
        isErrorRetry: false,
      },
    })
    expect(isErr).toBe(true)
  })
})
