import { createSelector } from 'reselect'
import { parse, add, format } from 'date-fns'
import { SpellingBeeStats } from '@nyt/games-schema'

import { MoogleState } from 'shared/moogle/moogleState'

import type { SpellingBeeState } from '..'
import {
  selectActivePuzzles,
  selectCurrentRank,
  calculateRank,
  selectWordCount,
  selectPuzzleId,
  selectIsRevealed,
} from '../game/game.selectors'
import type {
  SpellingBeeGameData,
  SpellingBeePuzzle,
} from '../game/game.state'
import type { DateString } from '../../../../shared/utils/dateUtils'
import { RANKS } from '../utils'
import type {
  SpellingBeeRank,
  SpellingBeeRankWithDefault,
} from '../utils'

import { SpellingBeeMoogleData } from './api.state'

type RankRow = {
  puzzleId: string
  printDate: string
  rank: SpellingBeeRankWithDefault
  wordsFound: number
  totalWords: number
  isRevealed: boolean
}

// the data structure that serves the RankList in the Stats modal
export type RankStats = {
  thisWeek: RankRow[]
  lastWeek: RankRow[]
}

// the data structure that serves the cards in PastPuzzlesSection
export type CardProgressMap = {
  [puzzleId: string]: {
    rank: SpellingBeeRankWithDefault
    isRevealed: boolean
  }
}

/**
 * Combines the output of two APIs (active puzzles and latest states)
 * into the data structure needed for the RankList in the Stats modal.
 *
 * The three last arguments are optional, and used to source the data for
 * the puzzle that is currently being played. If a user has made progress
 * on this puzzle since loading the page, the response from /latests would not suffice.
 *
 * @param multiStateData A list of the user's states within the active puzzle range, from games-state /latests
 * @param activePuzzles The "pastPuzzles" part of the response from games-puzzles active.json
 * @param currentPuzzleId
 * @param currentRank
 * @param currentWordCount
 * @returns {RankStats | null} null if one or both API responses aren't present
 */
export const getRankStats = (
  multiStateData: SpellingBeeMoogleData[] | null,
  activePuzzles: SpellingBeeGameData['pastPuzzles'] | null,
  currentPuzzleId: number | null = null,
  currentRank: SpellingBeeRankWithDefault = '',
  currentWordCount = 0,
  currentRevealed = false,
): RankStats | null => {
  if (!activePuzzles || !multiStateData) {
    return null
  }

  const createRankRow = (puzzle: SpellingBeePuzzle) => {
    const gameData = multiStateData.find(
      (d) => d.puzzle_id === `${puzzle?.id}`,
    )?.game_data

    let rank: SpellingBeeRankWithDefault = ''
    let wordsFound = 0
    let isRevealed = gameData?.isRevealed || false

    if (puzzle.id === currentPuzzleId) {
      rank = currentRank
      wordsFound = currentWordCount
      isRevealed = currentRevealed
    } else if (gameData && (gameData.answers.length || isRevealed)) {
      rank = calculateRank(puzzle, gameData.answers)
      wordsFound = gameData.answers.length
    }

    return {
      puzzleId: puzzle.id.toString(),
      printDate: puzzle.printDate,
      rank,
      wordsFound,
      isRevealed,
      totalWords: puzzle.answers.length,
    }
  }

  const thisWeek = activePuzzles.thisWeek.map(createRankRow)

  // activePuzzles does not return future days,
  // but RankList requires thisWeek to have the full 7 items for the week
  while (thisWeek.length < 7) {
    const date = parse(
      thisWeek.slice(-1)[0].printDate,
      'yyyy-MM-dd',
      new Date(),
    )
    const nextDay = add(date, { days: 1 })
    thisWeek.push({
      puzzleId: '',
      printDate: format(nextDay, 'yyyy-MM-dd') as DateString,
      rank: '',
      wordsFound: 0,
      totalWords: 0,
      isRevealed: false,
    })
  }

  return {
    thisWeek,
    lastWeek: activePuzzles.lastWeek.map(createRankRow),
  }
}

/**
 * Flattens the RankStats structure into an object for easy lookup of rank by puzzleId
 * @param rankStats
 * @returns {CardProgressMap}
 */
export const getProgressMap = (
  rankStats: RankStats | null,
): CardProgressMap => {
  if (!rankStats) {
    return {}
  }

  const flattened = [...rankStats.thisWeek, ...rankStats.lastWeek]
  return flattened.reduce<CardProgressMap>((acc, puzzleState) => {
    if (puzzleState.puzzleId) {
      acc[puzzleState.puzzleId] = {
        rank: puzzleState.rank,
        isRevealed: puzzleState.isRevealed,
      }
    }

    return acc
  }, {})
}

const selectMultiStateData = (state: SpellingBeeState) => {
  const { isLoading, error, statesByPuzzleId } =
    state.moogle.getRequest

  if (isLoading || error) {
    return null
  }

  return Object.values(statesByPuzzleId) as SpellingBeeMoogleData[]
}

export const selectRankStats = createSelector(
  [
    selectMultiStateData,
    selectActivePuzzles,
    selectPuzzleId,
    selectCurrentRank,
    selectWordCount,
    selectIsRevealed,
  ],
  getRankStats,
)

export const selectProgressMap = createSelector(
  selectRankStats,
  getProgressMap,
)

const formatPlayerRanks = (
  ranks: [SpellingBeeRank, number][],
  playerRanks: SpellingBeeStats['ranks'],
) => {
  const newRanksObject = ranks.reduce<{
    [key in SpellingBeeRank]: number
  }>(
    (acc, val) => {
      const rank = val[0]
      const rankValue = playerRanks?.[rank]
      acc[rank] = typeof rankValue === 'number' ? rankValue : 0
      return acc
    },
    {} as {
      [key in SpellingBeeRank]: number
    },
  )

  return newRanksObject
}

export const selectPlayerDocument = (state: SpellingBeeState) => {
  if (!state.moogle.player?.stats?.spelling_bee) return null

  return state.moogle.player.stats.spelling_bee
}

export const formatStats = (sbStats: SpellingBeeStats | null) => {
  if (!sbStats) return null

  const {
    puzzles_started,
    total_words,
    total_pangrams,
    ranks = {},
    longest_word,
  } = sbStats

  const rankCounts = formatPlayerRanks(RANKS, ranks)

  return {
    puzzlesStarted: puzzles_started || 0,
    totalWords: total_words || 0,
    totalPangrams: total_pangrams || 0,
    longestWord:
      longest_word?.word && longest_word?.center_letter
        ? {
            word: longest_word.word,
            centerLetter: longest_word.center_letter,
          }
        : null,
    geniuses: rankCounts.Genius + rankCounts['Queen Bee'],
    queenBees: rankCounts['Queen Bee'],
    highestRank:
      RANKS.map((arr) => arr[0])
        .reverse()
        .find((r) => rankCounts[r] > 0) || 'Beginner',
  }
}

export const selectStats = createSelector(
  selectPlayerDocument,
  formatStats,
)

export const getPostError = (
  puzzleId: number,
  postRequests: MoogleState['postRequest'],
) => postRequests[puzzleId.toString()]?.error || false

const selectPostRequest = (state: SpellingBeeState) =>
  state.moogle?.postRequest

export const selectPostError = createSelector(
  [selectPuzzleId, selectPostRequest],
  getPostError,
)
