import { ThunkAction } from 'redux-thunk'

import { trackInteraction } from 'foundation'

import { areSetEqual, wait } from '../../../../helpers/gameUtils'
import {
  ACCEPT_WORD,
  CLEAR_INPUT,
  CLEAR_MESSAGE,
  CLEAR_MESSAGE_AND_INPUT,
  DELETE_LETTER,
  INPUT_LETTER,
  SHOW_MESSAGE,
} from '../game/game.actions'
import { SpellingBeeMessage } from '../game/game.state'
import {
  checkInput,
  computeWordValue,
  valueToMessage,
  TOO_LONG,
} from '../utils'
import { selectIsLocked } from '../game/game.selectors'
import type { SpellingBeeState, SpellingBeeAction } from '..'

export const MAX_INPUT_LENGTH = 18
export const SUCCESS_MSG_DURATION = 900
export const ERROR_MSG_DURATION = 1100

export type SpellingBeeThunk<ReturnType = void> = ThunkAction<
  ReturnType,
  SpellingBeeState,
  unknown,
  SpellingBeeAction
>

const sendMessage = (msg: SpellingBeeMessage): SpellingBeeThunk => {
  return (dispatch, getState) => {
    dispatch({ type: SHOW_MESSAGE, payload: msg })
    const duration = msg.isError
      ? ERROR_MSG_DURATION
      : SUCCESS_MSG_DURATION

    const clearAction = msg.isError
      ? CLEAR_MESSAGE_AND_INPUT
      : CLEAR_MESSAGE

    return wait(duration).then(() => {
      if (getState().game.message) {
        dispatch({ type: clearAction })
      }
    })
  }
}

export const inputLetter =
  (letter: string): SpellingBeeThunk =>
  (dispatch, getState) => {
    const state = getState()
    const isLocked = selectIsLocked(state)
    if (isLocked) return

    if (state.game.input.length > MAX_INPUT_LENGTH) {
      dispatch(sendMessage({ value: TOO_LONG, isError: true }))
    } else {
      if (state.game.message && state.game.message.isError) {
        dispatch({ type: CLEAR_MESSAGE_AND_INPUT })
      }
      dispatch({ type: INPUT_LETTER, payload: letter })
    }
  }

export const deleteLetter =
  (): SpellingBeeThunk => (dispatch, getState) => {
    const state = getState()

    const isLocked = selectIsLocked(state)
    if (isLocked) return

    if (state.game.message && state.game.message.isError) {
      dispatch({ type: CLEAR_MESSAGE_AND_INPUT })
    } else {
      dispatch({ type: DELETE_LETTER })
    }
  }

export const submitWord = (
  providedWord?: string,
): SpellingBeeThunk => {
  return (dispatch, getState) => {
    const state = getState()
    const { inShortzMode } = state.game.userType

    const isLocked = selectIsLocked(state)
    if (isLocked) return

    const word = providedWord || state.game.input.join('')

    if (!word.length) return

    // Allow dev and our team to cheat
    if (
      inShortzMode &&
      window.cheat &&
      word.split('').every((l) => l === state.game.centerLetter)
    ) {
      return window.cheat(word.length)
    }

    // allow team to reset
    if (
      inShortzMode &&
      window.reset &&
      areSetEqual(word.split(''), state.game.outerLetters)
    ) {
      window.reset()
    }

    const errorType = checkInput(word, state)

    if (errorType) {
      dispatch(sendMessage({ value: errorType, isError: true }))
    } else {
      dispatch({ type: ACCEPT_WORD, payload: word })

      const isPangram = state.game.pangrams.includes(word)
      const points = computeWordValue(word, isPangram)
      const msg = valueToMessage(points, isPangram)

      dispatch({ type: CLEAR_INPUT })
      dispatch(sendMessage({ value: msg, points, isPangram }))
    }
  }
}

export const trackSBEvent =
  (elementLabel: string, label?: string | number): SpellingBeeThunk =>
  (dispatch, getState) => {
    const state = getState()
    const { id, printDate, isPastPuzzle } = state.game

    trackInteraction({
      element: {
        name: 'spelling-bee',
        label: elementLabel,
      },
      context: `SB-${id}`,
      name: {
        p: printDate,
        t: isPastPuzzle ? 'archive' : 'today',
      },
      label,
    })
  }
