import { createSelector } from 'reselect'

import type { SpellingBeeState } from '..'
import { CUTOFF_RANK_IDX } from '../../gating'
import {
  RANKS,
  getRank,
  getRankIdx,
  computeScore,
  computeRanks,
  getActivePuzzleIds,
} from '../utils'

import type { SpellingBeePuzzle } from './game.state'

export const calculateRank = (
  puzzle: SpellingBeePuzzle,
  foundWords: string[],
) => {
  const { answers, pangrams } = puzzle
  const score = computeScore(foundWords, pangrams)
  const maxScore = computeScore(answers, pangrams)
  const ranks = computeRanks(maxScore)
  const rankIdx = getRankIdx(score, ranks)
  return getRank(rankIdx)
}

export const selectMaxScore = (state: SpellingBeeState) =>
  computeScore(state.game.answers, state.game.pangrams)
export const selectCurrentScore = (state: SpellingBeeState) =>
  computeScore(state.game.foundWords, state.game.pangrams)

export const selectRanks = createSelector(
  selectMaxScore,
  computeRanks,
)

export const selectCurrentRankIdx = createSelector(
  [selectCurrentScore, selectRanks],
  getRankIdx,
)

export const selectCurrentRank = createSelector(
  selectCurrentRankIdx,
  getRank,
)

// Determines whether the user reached Genius rank
export const selectHasWon = createSelector(
  [selectCurrentRankIdx],
  (rankIdx) => rankIdx === RANKS.length - 2,
)

// Determines whether the user reached Queen Bee rank
export const selectHasCompleted = createSelector(
  [selectCurrentScore, selectMaxScore],
  (currentScore, maxScore) => currentScore === maxScore,
)

export const selectWordCount = (state: SpellingBeeState) =>
  state.game.foundWords.length

export const selectCurrentRankTrackingLabel = createSelector(
  [selectHasWon, selectHasCompleted],
  (geniusRank, queenBeeRank) => {
    let rankLabel = null

    if (geniusRank) {
      rankLabel = '8-genius'
    } else if (queenBeeRank) {
      rankLabel = '9-completed'
    }

    return rankLabel
  },
)

export const selectRankLabel = createSelector(
  [selectCurrentRankIdx, selectCurrentRank],
  (rankIdx, rank) => {
    let rankLabel: string = rank
    if (rank === 'Queen Bee') {
      rankLabel = 'completed'
    }

    return `${rankIdx}-${rankLabel.replace(' ', '-').toLowerCase()}`
  },
)

export const selectUserType = (state: SpellingBeeState) =>
  state.game.userType

export const selectHasReachedCutoff = createSelector(
  [selectCurrentRankIdx, selectUserType],
  (currentRankIdx, userType) => {
    return !userType.hasXwd
      ? currentRankIdx >= CUTOFF_RANK_IDX
      : false
  },
)

const selectTodayPangrams = (state: SpellingBeeState) =>
  state.game.pangrams

export const selectTodayPangramSet = createSelector(
  [selectTodayPangrams],
  (pangrams) => new Set(pangrams),
)

// TODO: all the state and selectors for yesterday found words
export const selectYesterdayFoundWords = (state: SpellingBeeState) =>
  state.game.yesterday?.foundWords

export const selectYesterdayFoundWordSet = createSelector(
  [selectYesterdayFoundWords],
  (foundWords) => new Set(foundWords),
)

const selectYesterdayPangrams = (state: SpellingBeeState) =>
  state.game.yesterday?.pangrams

export const selectYesterdayPangramSet = createSelector(
  [selectYesterdayPangrams],
  (pangrams) => new Set(pangrams),
)

export const selectYesterdaysMaxScore = (state: SpellingBeeState) => {
  const { yesterday } = state.game

  if (!yesterday) {
    return 0
  }

  return computeScore(yesterday.answers, yesterday.pangrams)
}

export const selectYesterdaysScore = (state: SpellingBeeState) => {
  const { yesterday } = state.game
  if (!yesterday || yesterday.foundWords === null) return 0

  return computeScore(yesterday.foundWords, yesterday.pangrams)
}

export const selectYesterdaysRanks = createSelector(
  selectYesterdaysMaxScore,
  computeRanks,
)

export const selectYesterdaysRankIdx = createSelector(
  [selectYesterdaysScore, selectYesterdaysRanks],
  getRankIdx,
)

export const selectYesterdaysRank = createSelector(
  selectYesterdaysRankIdx,
  getRank,
)

export const selectCompletedYesterday = createSelector(
  [selectYesterdaysScore, selectYesterdaysMaxScore],
  (currentScore, maxScore) => currentScore === maxScore,
)

export const selectWordlistExpanded = (state: SpellingBeeState) =>
  state.game.isExpandedWordlist

export const selectMoment = (state: SpellingBeeState) =>
  state.game.currentMoment

export const selectIsModalOpen = (state: SpellingBeeState) =>
  state.game.isModalOpen

const selectIsMomentOpen = createSelector(
  selectMoment,
  (moment) => moment && moment !== 'game',
)

export const selectIsRevealed = (state: SpellingBeeState) =>
  state.game.isRevealed

export const selectIsLocked = createSelector(
  [selectIsMomentOpen, selectIsModalOpen, selectIsRevealed],
  (isMomentOpen, isModalOpen, isRevealed) => {
    return isMomentOpen || isModalOpen || isRevealed
  },
)

export const selectPuzzleId = (state: SpellingBeeState) =>
  state.game.id

const selectPrintDate = (state: SpellingBeeState) =>
  state.game.printDate

export const selectIsPastPuzzle = (state: SpellingBeeState) =>
  state.game.isPastPuzzle

export const selectBeatriceHat = createSelector(
  [selectPrintDate],
  (printDate) => {
    const todaysDate = printDate

    // switch statement + regex explained here:
    // https://stackoverflow.com/questions/2896626/switch-statement-for-string-matching-in-javascript
    switch (true) {
      // Halloween, recurring every year
      case /10-31/.test(todaysDate):
        return 'spelling-bee-spooky'

      // Valentines, recurring every year
      case /02-14/.test(todaysDate):
        return 'spelling-bee-valentines'

      // Beatrice's birthday, recurring every year
      case /05-09/.test(todaysDate):
        return 'spelling-bee-birthday'

      // New Years 2024
      case /2023-12-31/.test(todaysDate) ||
        /2024-01-01/.test(todaysDate):
        return 'spelling-bee-ny-2024'

      // New Years 2025
      case /2024-12-31/.test(todaysDate) ||
        /2025-01-01/.test(todaysDate):
        return 'spelling-bee-ny-2025'

      // New Years 2026
      case /2025-12-31/.test(todaysDate) ||
        /2026-01-01/.test(todaysDate):
        return 'spelling-bee-ny-2026'

      // New Years 2027
      case /2026-12-31/.test(todaysDate) ||
        /2027-01-01/.test(todaysDate):
        return 'spelling-bee-ny-2027'

      // New Years 2028
      case /2027-12-31/.test(todaysDate) ||
        /2028-01-01/.test(todaysDate):
        return 'spelling-bee-ny-2028'

      // New Years 2029
      case /2028-12-31/.test(todaysDate) ||
        /2029-01-01/.test(todaysDate):
        return 'spelling-bee-ny-2029'

      // New Years 2030
      case /2029-12-31/.test(todaysDate) ||
        /2030-01-01/.test(todaysDate):
        return 'spelling-bee-ny-2030'

      default:
        return 'spelling-bee-smarty'
    }
  },
)

export const selectActivePuzzles = (state: SpellingBeeState) =>
  state.game.pastPuzzlesData

export const selectActivePuzzleIds = createSelector(
  selectActivePuzzles,
  getActivePuzzleIds,
)

export const selectIsCurrentPuzzleFromLastWeek = createSelector(
  [selectActivePuzzles, selectPuzzleId],
  (activePuzzles, currentPuzzId) =>
    !!activePuzzles?.lastWeek.find(
      (puzz) => puzz.id === currentPuzzId,
    ),
)
