import { Middleware } from 'redux'

import hybridDebugger from '../../../../../shared/foundation/js/hybrid/hybridDebugger'
import { willUseMeterToGate } from '../../../../../shared/meter'
import { momentSystem as momentSystemJS } from '../../moments/index'
import {
  ACCEPT_WORD,
  REVEAL,
  REVEAL_YESTERDAY,
  TOGGLE_WORDLIST,
  UPDATE_USER,
} from '../game/game.actions'
import type { SpellingBeeState } from '..'
import {
  selectCurrentRankIdx,
  selectHasCompleted,
  selectHasWon,
  selectIsLocked,
  selectUserType,
  selectHasReachedCutoff,
} from '../game/game.selectors'
import { selectHasReachedCutoff as selectHasReachedMeterCutoff } from '../meter/meter.selector'
import { RANKS } from '../utils'
import { SET_METER } from '../meter/meter.actions'
import { trackSBEvent } from '../api/thunks'

// TEMP until those files are converted
const momentSystem = momentSystemJS as any

const WIN_DWELL_TIME = 450

type SbMiddleware = Middleware<{}, SpellingBeeState>

export const analyticsMiddleware: SbMiddleware = (store) => {
  let wordlistTimeStamp: number

  return (next) => (action) => {
    const prevState = store.getState()

    const result = next(action)
    const { dispatch } = store

    // track closing of word list for analytics
    if (action.type === TOGGLE_WORDLIST) {
      if (prevState.game.isExpandedWordlist) {
        dispatch(
          trackSBEvent(
            'word-list',
            new Date().getTime() - wordlistTimeStamp,
          ),
        )
      } else {
        wordlistTimeStamp = new Date().getTime()
      }
    }

    // only do work if a word was just accepted
    if (action.type === ACCEPT_WORD) {
      const state = store.getState()

      // track level changes for analytics
      const prevRankIdx = selectCurrentRankIdx(prevState)
      const rankIdx = selectCurrentRankIdx(state)

      if (rankIdx > prevRankIdx) {
        // you can level up by more than one rank with a single word entry
        for (let r = prevRankIdx + 1; r <= rankIdx; r += 1) {
          const rName = RANKS[r][0]
          dispatch(
            trackSBEvent(
              'level-up',
              `${r}-${rName.replace(' ', '-').toLowerCase()}`,
            ),
          )
        }
      }

      // track pangram solves
      const isPangram = state.game.pangrams.includes(action.payload)
      if (isPangram) {
        dispatch(trackSBEvent('pangram'))
      }

      const justCompleted =
        !selectHasCompleted(prevState) && selectHasCompleted(state)

      if (justCompleted) {
        dispatch(
          trackSBEvent(
            'level-up',
            `${selectCurrentRankIdx(state) + 1}-completed`,
          ),
        )
      }
    }

    return result
  }
}

export const trackGameStateMiddleware: SbMiddleware =
  (store) => (next) => (action) => {
    const prevState = store.getState()

    const result = next(action)
    const { dispatch } = store

    if (action.type === SET_METER) {
      const state = store.getState()

      if (
        willUseMeterToGate() &&
        selectHasReachedMeterCutoff(state)
      ) {
        hybridDebugger('Using Meter service to cut off gameplay')
        switch (momentSystem.getCurrentMoment()) {
          case 'game':
            momentSystem.transition('game', 'cutoff')
            break
          case 'welcome':
            momentSystem.reanimateMoment('welcome')
            break
          default:
        }
      }
    }

    // only do work if a word was just accepted
    if (action.type === ACCEPT_WORD) {
      const state = store.getState()

      const isLocked = selectIsLocked(state)
      if (isLocked) return

      // determine if user reached Genius rank
      const justWon = !selectHasWon(prevState) && selectHasWon(state)

      // determine if the user reached Queen Bee rank
      const justCompleted =
        !selectHasCompleted(prevState) && selectHasCompleted(state)

      if (!willUseMeterToGate() && selectHasReachedCutoff(state)) {
        setTimeout(() => {
          momentSystem.transition('game', 'cutoff')
        }, WIN_DWELL_TIME)
      }

      if (justWon || justCompleted) {
        setTimeout(() => {
          momentSystem.transition('game', 'congrats')
        }, WIN_DWELL_TIME)
      }
    }

    if (action.type === REVEAL && store.getState().game.hasLoaded) {
      dispatch(trackSBEvent('reveal', 'gameplay'))
      momentSystem.transition('game', 'revealed')
    }

    if (
      action.type === REVEAL_YESTERDAY &&
      store.getState().game.hasLoaded
    ) {
      dispatch(trackSBEvent('reveal', 'yesterday'))
    }

    return result
  }

export const trackUserStateMiddleware: SbMiddleware =
  (store) => (next) => (action) => {
    const prevState = store.getState()
    const hadXwd = selectUserType(prevState).hasXwd
    const hadShortz = selectUserType(prevState).inShortzMode

    const result = next(action)

    // only do work if the user's status changed
    if (action.type === UPDATE_USER) {
      const state = store.getState()
      const { hasXwd, inShortzMode } = selectUserType(state)

      // check if user previously did not have Xwd but now does
      if (hasXwd && !hadXwd) {
        switch (momentSystem.getCurrentMoment()) {
          case 'cutoff':
            momentSystem.transition('cutoff', 'game')
            break
          case 'welcome':
            momentSystem.reanimateMoment('welcome')
            break
          default:
        }
      }

      // @ts-ignore
      if (__DEV__ || (!hadShortz && inShortzMode)) {
        window.cheat = (limit = 1) => {
          const { answers, foundWords } = store.getState().game
          const remainingWords = answers.filter(
            (a) => !foundWords.includes(a),
          )

          const submit = (word: string) =>
            store.dispatch({ type: ACCEPT_WORD, payload: word })
          remainingWords.slice(0, limit).forEach(submit)
        }
      }
    }

    return result
  }
