import { SpellingBeeData } from '@nyt/games-schema'

import { createMoogleMiddleware } from 'shared/moogle/moogleMiddleware'
import { MoogleOptions } from 'shared/moogle/types'
import { createMoogleLoader } from 'shared/moogle/moogleActions'

import type { SpellingBeeState, SpellingBeeAction } from '.'

import {
  ACCEPT_WORD,
  insertFoundWords,
  RESET_GAME,
  REVEAL,
  REVEAL_YESTERDAY,
  setHasLoaded,
  setYesterdaysProgress,
} from './game/game.actions'
import {
  selectActivePuzzleIds,
  selectCurrentRank,
  selectYesterdaysRank,
} from './game/game.selectors'
import type { DateString } from './game/game.state'
import type { SpellingBeeThunk } from './api/thunks'

const getSpellingBeeLocalStorageKey = (
  printDate: DateString,
  regiId?: string | number | null,
) => `nytsb/${printDate}/${regiId || 'ANON'}`

const cleanLocalStorageExcept = (dates: string[]) => {
  Object.keys(window.localStorage)
    .filter((key) => key.startsWith('nytsb'))
    .forEach((key) => {
      const storedDate = key.split('/')[1]
      if (storedDate && !dates.includes(storedDate)) {
        window.localStorage.removeItem(key)
      }
    })
}

/**
 * In order to prevent a memory leak in local storage, this will wipe
 * out any local storage for puzzles outside the active range.
 */
export const cleanSpellingBeeLocalStorage =
  (): SpellingBeeThunk => (_, getState) => {
    const state = getState()
    const { pastPuzzlesData, printDate } = state.game

    if (!pastPuzzlesData) {
      return
    }

    const activeDates = [
      printDate,
      ...pastPuzzlesData.thisWeek.map((p) => `${p.printDate}`),
      ...pastPuzzlesData.lastWeek.map((p) => `${p.printDate}`),
    ]

    cleanLocalStorageExcept(activeDates)
  }

const dedupe = (words: string[]) => Array.from(new Set(words))

const reconcileStates = (
  state1: SpellingBeeData | null,
  state2: SpellingBeeData | null,
): SpellingBeeData | null => {
  if (!state1 && !state2) {
    return null
  }

  if (!state2) {
    return state1
  }

  if (!state1) {
    return state2
  }

  return {
    answers: dedupe([
      ...(state1.answers || []),
      ...(state2.answers || []),
    ]),
    isRevealed: state1.isRevealed || state2.isRevealed,
  }
}

export const options: MoogleOptions<
  SpellingBeeState,
  SpellingBeeAction,
  SpellingBeeData
> = {
  game: 'spelling_bee',
  shouldSave: (_, action) => {
    return (
      action.type === ACCEPT_WORD ||
      action.type === REVEAL ||
      action.type === RESET_GAME
    )
  },
  selectPersistedState: (state) => ({
    answers: state.game.foundWords,
    isRevealed: state.game.isRevealed,
    rank: selectCurrentRank(state),
  }),
  getLocalStorageKey: (state, regiId) =>
    getSpellingBeeLocalStorageKey(state.game.printDate, regiId),
  selectMoogleState: (state) => state.moogle,
  selectPuzzleId: (state) => `${state.game.id}`,
  selectAdditionalPuzzleIdsToFetch: selectActivePuzzleIds,
  reconcileRemoteAndLocalState: reconcileStates,
  handleMidGameLogin: reconcileStates,
  insertProgress: (gameData) => (dispatch) => {
    const foundWords = gameData?.answers || []
    dispatch(insertFoundWords(foundWords))

    if (gameData?.isRevealed) {
      dispatch({ type: REVEAL })
    }

    dispatch(setHasLoaded())
    dispatch(cleanSpellingBeeLocalStorage())
  },
}

export const yesterdayOptions: MoogleOptions<
  SpellingBeeState,
  SpellingBeeAction,
  SpellingBeeData
> = {
  game: 'spelling_bee',
  reconcileRemoteAndLocalState: reconcileStates,
  selectMoogleState: (state) => state.moogle,
  shouldSave: (_, action) => {
    return action.type === REVEAL_YESTERDAY
  },
  selectPersistedState: (state) => ({
    answers: state.game.yesterday?.foundWords || [],
    isRevealed: state.game.yesterday?.isRevealed || false,
    rank: selectYesterdaysRank(state),
  }),
  selectPuzzleId: (state) => {
    const { yesterday } = state.game
    return yesterday ? `${yesterday.id}` : ''
  },
  getLocalStorageKey: (state, regiId) => {
    const { yesterday } = state.game
    if (!yesterday) {
      return ''
    }

    return getSpellingBeeLocalStorageKey(yesterday.printDate, regiId)
  },
  insertProgress: (yesterdayData) => (dispatch) => {
    if (yesterdayData) {
      const { answers, isRevealed } = yesterdayData
      dispatch(setYesterdaysProgress(answers, isRevealed))
    }
  },
}

export const saveSpellingBee = createMoogleMiddleware(options)
export const saveYesterdaysSpellingBee =
  createMoogleMiddleware(yesterdayOptions)

export const loadSpellingBeeProgress = createMoogleLoader(
  options,
  yesterdayOptions,
)
