import type { SpellingBeeState } from './index'

// ----------------
// INPUT VALIDATION
// ----------------
const ERR_TOO_SHORT = 'Too\u00a0short'
const ERR_BAD_INPUT = 'Bad\u00a0letters'
const ERR_MISSING_KEY_LETTER = 'Missing\u00a0center\u00a0letter'
const ERR_NON_WORD = 'Not\u00a0in\u00a0word\u00a0list'
const ERR_ALREADY_FOUND = 'Already\u00a0found'
export const TOO_LONG = 'Too\u00a0long'

export const checkInput = (word: string, state: SpellingBeeState) => {
  const letters = word.split('')

  if (word.length < 4) return ERR_TOO_SHORT

  if (letters.some((c) => !state.game.validLetters.includes(c))) {
    return ERR_BAD_INPUT
  }

  if (!word.includes(state.game.centerLetter)) {
    return ERR_MISSING_KEY_LETTER
  }

  if (!state.game.answers.includes(word)) return ERR_NON_WORD

  if (state.game.foundWords.includes(word)) {
    return ERR_ALREADY_FOUND
  }
}

export type SpellingBeeRank =
  | 'Beginner'
  | 'Good Start'
  | 'Moving Up'
  | 'Good'
  | 'Solid'
  | 'Nice'
  | 'Great'
  | 'Amazing'
  | 'Genius'
  | 'Queen Bee'

export type SpellingBeeRankWithDefault = SpellingBeeRank | ''

// ----------------
// USEFUL SPELLING BEE COMPUTATION
// ----------------
export const RANKS: Array<[SpellingBeeRank, number]> = [
  ['Beginner', 0],
  ['Good Start', 2],
  ['Moving Up', 5],
  ['Good', 8],
  ['Solid', 15],
  ['Nice', 25],
  ['Great', 40],
  ['Amazing', 50],
  ['Genius', 70],
  ['Queen Bee', 100],
]

export const computeWordValue = (
  guess: string,
  isPangram: boolean,
) => {
  if (guess.length < 5) return 1
  const value = guess.length
  const bonus = isPangram ? 7 : 0
  return value + bonus
}

export const computeScore = (words: string[], pangrams: string[]) => {
  return words.reduce(
    (total, guess) =>
      total + computeWordValue(guess, pangrams.includes(guess)),
    0,
  )
}

export const computeRanks = (maxScore: number) =>
  RANKS.map(([title, percent]) => ({
    title,
    minScore: Math.round((percent / 100) * maxScore),
  }))

export const getRankIdx = (
  currentScore: number,
  ranks: { title: string; minScore: number }[],
) => {
  let l
  for (l = 0; l < ranks.length; l += 1) {
    if (currentScore < ranks[l].minScore) break
  }

  return l - 1
}

export const getRank = (rankIdx: number) => RANKS[rankIdx][0]

export const getRankIdxFromRank = (rank: string) => {
  return RANKS.findIndex((arr) => arr[0] === rank)
}

export const valueToMessage = (value: number, isPangram: boolean) => {
  if (isPangram) return 'Pangram!'

  if (value >= 7) return 'Awesome!'

  if (value > 1) return 'Nice!'

  return 'Good!'
}

export const getActivePuzzleIds = (
  activePuzzles: SpellingBeeState['game']['pastPuzzlesData'],
) => {
  if (!activePuzzles) {
    return []
  }

  const { thisWeek, lastWeek } = activePuzzles
  return [...lastWeek, ...thisWeek].map((puzz) => `${puzz.id}`)
}
