import React from 'react'
import { shallow, mount } from 'enzyme'

import { trackEvent } from '../../helpers/sudokuTracker'

import Keyboard from './Keyboard'

jest.mock('../../helpers/sudokuTracker', () => ({
  trackEvent: jest.fn(),
}))

describe('SUDOKU: KEYBOARD', () => {
  let act
  let view

  beforeEach(() => {
    act = jest.fn()
    view = {
      auto: false,
      candidateMode: true,
      completed: { 1: 9 },
      difficulty: 'hard',
    }
  })

  test('renders all buttons', () => {
    // candidate buttons
    let keyboard = shallow(<Keyboard act={act} view={view} />)

    const candidateButtons = keyboard.find('.su-keyboard__svg')

    expect(candidateButtons).toHaveLength(9)
    expect(candidateButtons.at(1).props()).toEqual({
      className: 'su-keyboard__svg candidate row-0 col-1',
      number: 2,
      weight: 'bold',
    })

    // normal buttons
    view.candidateMode = false
    keyboard = shallow(<Keyboard act={act} view={view} />)
    const normalButtons = keyboard.find('.su-keyboard__svg')

    expect(normalButtons).toHaveLength(9)
    expect(normalButtons.at(3).props()).toEqual({
      className: 'su-keyboard__svg row-1 col-0',
      number: 4,
      weight: 'bold',
    })
  })

  test('handles completed numbers', () => {
    const keyboard = shallow(<Keyboard act={act} view={view} />)
    const buttons = keyboard.find('.su-keyboard__svg')

    expect(buttons.at(0).props()).toEqual({
      className: 'su-keyboard__svg candidate completed row-0 col-0',
      number: 1,
      weight: 'light',
    })
  })

  test('handles value changes', () => {
    const keyboard = mount(<Keyboard act={act} view={view} />)
    const buttons = keyboard.find('.su-keyboard__number')
    buttons.at(0).simulate('click')
    expect(act).toHaveBeenCalledWith('guess', 1)

    buttons.at(3).simulate('click')
    expect(act).toHaveBeenCalledWith('guess', 4)
  })

  test('handles deletions', () => {
    const keyboard = mount(<Keyboard act={act} view={view} />)

    const deleteButton = keyboard.find('.su-keyboard__delete')
    deleteButton.simulate('click')
    expect(act).toHaveBeenCalledWith('guess', undefined)
    expect(trackEvent).toHaveBeenCalledWith('hard', 'clear-square')
  })

  test('toggles between normal and candidate modes', () => {
    const keyboard = mount(<Keyboard act={act} view={view} />)

    const normalMode = keyboard.find('.su-keyboard__mode.normal')
    normalMode.simulate('click')
    expect(act).toHaveBeenCalledWith('disable', 'candidateMode')
    expect(trackEvent).toHaveBeenCalledWith('hard', 'normal-mode')

    const candidateMode = keyboard.find(
      '.su-keyboard__mode.candidate',
    )
    candidateMode.simulate('click')
    expect(act).toHaveBeenCalledWith('enable', 'candidateMode')
    expect(trackEvent).toHaveBeenCalledWith('hard', 'candidate-mode')
  })

  test('toggles auto on', () => {
    const keyboard = mount(<Keyboard act={act} view={view} />)
    const checkbox = keyboard.find('.su-keyboard__checkbox')
    expect(checkbox.props().checked).toBe(false)

    checkbox.simulate('change')

    expect(trackEvent).toHaveBeenCalledWith(
      'hard',
      'auto-candidate-mode',
      'turn-on',
    )
  })

  test('toggles auto off', () => {
    const viewProps = { ...view, ...{ auto: true } }
    const keyboard = mount(<Keyboard act={act} view={viewProps} />)
    const checkbox = keyboard.find('.su-keyboard__checkbox')
    expect(checkbox.props().checked).toBe(true)

    checkbox.simulate('change')

    expect(trackEvent).toHaveBeenCalledWith(
      'hard',
      'auto-candidate-mode',
      'turn-off',
    )
  })
})
