import React from 'react'
import pt from 'prop-types'
import cn from 'classnames'

import { isMobile } from 'responsive'
import { mobileNavTools } from 'foundation'

import { useAppSelector, useAppDispatch, store } from '../../store'
import Toolbar from '../Toolbar/Toolbar'
import Board from '../Board/Board'
import Timer from '../Timer/Timer'
import Keyboard from '../Keyboard/Keyboard'
import { Moments, momentSystem } from '../Moments'
import useSave from '../../hooks/useSave/useSave'
import useGame from '../../hooks/useGame/useGame'
import useLevel from '../../hooks/useLevel/useLevel'
import useSizer from '../../hooks/useSizer/useSizer'
import useSettings from '../../hooks/useSettings/useSettings'
import useKeyboard from '../../hooks/useKeyboard/useKeyboard'
import {
  load,
  puzzleAnalyticsId,
  puzzleInfo,
} from '../../helpers/persist'
import { trackEvent } from '../../helpers/sudokuTracker'
import { trackPuzzleCompletionProgress } from '../../helpers/completionHelper'
import ModalManager from '../../../shared/ModalSystem/ModalManager'
import {
  open,
  close,
  remove,
  selectOpenModal,
  selectIsClosing,
} from '../../store/slices/modalSystem'
import { SudokuModal } from '../Modals/modalTypes'
import ModalRules from '../Modals/ModalRules'
import ModalPause from '../Modals/ModalPause'
import ModalCongrats from '../Modals/ModalCongrats'
import ModalSettingsWrapper from '../Modals/ModalSettings'
import { LEVELS, Welcome } from '../../constants/constants'
import { EVENT_ON_NAVIGATE_BACK } from '../../../../shared/foundation/js/hybrid/commands'
import useAndroidBack from '../../../shared/hooks/use-android-back'
import useGameDataHybridEvent from '../../../shared/hooks/use-game-data-hybrid-event'
import useSaleBannerHybridSubscribe from '../../../shared/hooks/use-sale-banner-hybrid-subscribe'

const modalMap = {
  [SudokuModal.Pause]: ModalPause,
  [SudokuModal.Rules]: ModalRules,
  [SudokuModal.Congrats]: ModalCongrats,
  [SudokuModal.Settings]: ModalSettingsWrapper,
}

const Layout = ({ data, showWelcomeMoment }) => {
  const dispatch = useAppDispatch()

  const isModalOpen = useAppSelector(selectOpenModal)

  // Keep track of if we can see the game board. The board is not yet visible
  // if we are rendering welcome moments. But it is visible if we did not
  // initialized the moment system.

  const [isGameBoardVisible, setIsGameBoardVisible] = React.useState(
    !showWelcomeMoment,
  )

  // unfortunate anti-pattern used to avoid hybrid transition issues
  const [getShowWelcomeMoment, setGetShowWelcomeMoment] =
    React.useState(showWelcomeMoment)

  // Bool used to keep track of when user info, progress, etc has been fetched
  // Currently, this is immediate, but won't always be
  const [hasLoaded, setHasLoaded] = React.useState(true)

  const sizer = useSizer()
  const settings = useSettings()

  useSaleBannerHybridSubscribe()

  const level = useLevel()

  const games = {
    easy: useGame(data.easy, load('easy'), settings),
    medium: useGame(data.medium, load('medium'), settings),
    hard: useGame(data.hard, load('hard'), settings),
  }

  games.welcome = useGame(
    {
      difficulty: '',
      puzzle_data: {
        puzzle: [],
        solution: [],
      },
    },
    {
      id: '0',
    },
    settings,
  )

  const game = games[level.current] || games.welcome

  useSave(game)
  useKeyboard(game.act, game.state)

  function startGame() {
    setIsGameBoardVisible(true)
    if (isMobile) {
      const element = document.querySelector(
        '.pz-nav__hamburger-helper',
      )

      if (element) {
        element.style.display = 'none'
      }
    }

    if (getShowWelcomeMoment) {
      momentSystem.transition('welcome', 'game')
    } else {
      setGetShowWelcomeMoment(true)
      momentSystem.load('game')
    }
    sizer.resize()
    if (isMobile) {
      mobileNavTools.activate()
    }
  }

  React.useEffect(() => {
    if (
      momentSystem.getCurrentMoment() === 'welcome' &&
      level.current !== 'welcome'
    ) {
      startGame()
    }

    if (
      momentSystem.getCurrentMoment() === 'game' &&
      level.current === 'welcome'
    ) {
      const element = document.querySelector(
        '.pz-nav__hamburger-helper',
      )
      element.style.display = 'flex'
      momentSystem.transition('game', 'welcome')
      if (isMobile) {
        mobileNavTools.deactivate()
      }
    }
  }, [level])

  React.useEffect(() => {
    // Only start the timer when the game board is visible, aka we've
    // gotten through the welcome moments
    if (!game.view.solved && isGameBoardVisible) {
      game.timer.resume()
    } else if (!isGameBoardVisible) {
      game.timer.pause()
    }
  }, [isGameBoardVisible, game.timer])

  React.useEffect(() => {
    setHasLoaded(true)
    if (showWelcomeMoment) {
      momentSystem.load('loading')
    } else {
      if (
        level.current !== Welcome &&
        (load(level.current).difficulty === null ||
          load(level.current).initialTime === 0 ||
          load(level.current).id !== data.easy.print_date)
      ) {
        trackEvent(level.current, 'start-game')
      }
      startGame()
    }
  }, [showWelcomeMoment])

  React.useEffect(() => {
    // every time there is a cell change track user progress in puzzle
    if (game.view.justSolved) {
      trackPuzzleCompletionProgress(
        game.state.difficulty,
        game.state.cells,
        game.state.selected,
      )
    }
  }, [game.state.cells])

  React.useEffect(() => {
    if (!game.view.solved && isGameBoardVisible && !isModalOpen) {
      game.timer.resume()
    }
  })

  const setLevel = level.set

  const puzzleDate = data.easy.print_date

  React.useEffect(() => {
    const backButton = document.getElementById('js-hybrid-back')

    const goBack = () => {
      if (momentSystem.getCurrentMoment() === 'game') {
        game.timer.pause()
        trackEvent(level.current, 'back', 'in-game')
        momentSystem.transition('game', 'welcome')
        setLevel('welcome', { method: 'pushState' })
        mobileNavTools.deactivate()
      }
    }

    if (
      window.isHybridWebView &&
      window.isHybridWebView !== 'news' &&
      backButton
    ) {
      // in navigation.ts, we prevent the default hybrid back button behavior
      backButton.addEventListener('click', goBack)
    }

    return () => {
      backButton.removeEventListener('click', goBack)
    }
  }, [level])

  React.useEffect(() => {
    // remove any open modal when not playing the game
    const isNotPlaying = !LEVELS.map((l) => l.name).includes(
      level.current,
    )

    if (isNotPlaying && isModalOpen) {
      store.dispatch(close())
    }
  }, [level])

  React.useEffect(() => {
    if (game.view.justSolved) {
      setTimeout(() => {
        dispatch(open('Congrats'))
      }, 400)
    }
  }, [game.view.justSolved])

  useAndroidBack(
    (event) => {
      const response = {}

      if (isModalOpen) {
        dispatch(close())
        response[EVENT_ON_NAVIGATE_BACK] = true
      } else if (momentSystem.getCurrentMoment() === 'welcome') {
        // go back to app feed
        response[EVENT_ON_NAVIGATE_BACK] = false
      } else {
        // game is being played. go to welcome moment
        // update level and deactivate toolbar
        momentSystem.transition('game', 'welcome')
        setLevel('welcome', { method: 'pushState' })
        if (isMobile) {
          mobileNavTools.deactivate()
        }
        response[EVENT_ON_NAVIGATE_BACK] = true
      }

      event.respondWith(response)
    },
    [isModalOpen],
  )

  useGameDataHybridEvent({
    analyticsPuzzleID: puzzleAnalyticsId(data, level),
    puzzleID: puzzleInfo(data, level).puzzle_id,
    game: 'sudoku',
  })

  return (
    <>
      {/* Toolbar */}
      <Toolbar game={game} level={level} settings={settings} />
      <Moments
        contentProps={{
          displayDate: data.displayDate,
          startGame,
          level,
          hasLoaded,
          setLevel,
          puzzleDate,
        }}
      />
      <ModalManager
        getOpenModalName={selectOpenModal}
        getIsClosing={selectIsClosing}
        modalMap={modalMap}
        onClose={close}
        portalId="portal-modal-system"
        game={game}
        level={level}
        settings={settings}
        timer={game.timer}
        onRemove={remove}
      />

      {/* Mobile Level + Timer */}
      {isMobile && (
        <div
          className={cn('su-level-timer-mobile', {
            mweb: isMobile,
          })}
        >
          {level.current !== Welcome && (
            <div className="su-level">{level.current}</div>
          )}
          {settings.get('clock') && (
            <Timer game={game} level={level.current} />
          )}
        </div>
      )}

      {/* Game Board */}
      <div className="su-app">
        <div className="su-app__main">
          <div className="su-app__play">
            <div className="su-app__board-square">
              <div
                className="su-app__board su-stretch"
                ref={sizer.ref}
              >
                {!!sizer.shapes.cell && (
                  <Board
                    {...game}
                    shapes={sizer.shapes}
                    level={level.current}
                  />
                )}
              </div>
            </div>
          </div>

          <div className="su-app__divider" />

          {/* Keyboard */}
          <div className="su-app__control">
            <Keyboard {...game} />
          </div>
        </div>
      </div>
    </>
  )
}

Layout.propTypes = {
  data: pt.shape({
    easy: pt.shape({
      print_date: pt.string,
    }),
    medium: pt.shape({}),
    hard: pt.shape({}),
    displayDate: pt.string,
  }),
  showWelcomeMoment: pt.bool,
}

export default Layout
