import React from 'react'
import { shallow } from 'enzyme'

import ModalSettings from '../ModalSettings'

jest.mock('../../../constants/constants', () => ({
  LEVELS: [
    { displayName: 'Easy', name: 'easy' },
    { displayName: 'Medium', name: 'medium' },
    { displayName: 'Hard', name: 'hard' },
  ],
}))

const findInputByLabel = (wrapper, txt) =>
  wrapper
    .find('label')
    .findWhere((n) => n.text() === txt)
    .at(0)
    .find('input')

describe('SUDOKU: MODAL SETTINGS', () => {
  let settings
  let props

  beforeEach(() => {
    settings = {
      autoCheck: false,
    }

    props = {
      settings: {
        get: (k) => settings[k],
        set: (k, v) => {
          settings[k] = v
        },
        toggle: (k) => {
          settings[k] = !settings[k]
        },
      },
      game: {
        act: jest.fn(),
        view: {},
      },
    }
  })

  test('renders modal', () => {
    const wrapper = shallow(<ModalSettings {...props} />)
    expect(wrapper.exists()).toBe(true)
  })

  test('Autocheck - enable', () => {
    settings.autoCheck = false

    const wrapper = shallow(<ModalSettings {...props} />)
    const input = findInputByLabel(
      wrapper,
      'Check guesses when entered',
    )

    expect(input.prop('checked')).toBe(false)

    input.simulate('change')

    expect(settings.autoCheck).toBe(true)
    expect(props.game.act).toHaveBeenCalledWith('check', 'puzzle')
  })

  test('Autocheck - disable', () => {
    settings.autoCheck = true

    const wrapper = shallow(<ModalSettings {...props} />)
    const input = findInputByLabel(
      wrapper,
      'Check guesses when entered',
    )

    expect(input.prop('checked')).toBe(true)

    input.simulate('change')

    expect(settings.autoCheck).toBe(false)
    expect(props.game.act).not.toHaveBeenCalled()
  })

  test('Auto Candidate Mode - enable', () => {
    settings.autoCandidateMode = false

    const wrapper = shallow(<ModalSettings {...props} />)
    const input = findInputByLabel(wrapper, 'Start in automatic mode')

    expect(input.prop('checked')).toBe(false)

    input.simulate('change')

    expect(settings.autoCandidateMode).toBe(true)
    expect(props.game.act).toHaveBeenCalledWith('enable', 'auto')
  })

  test('Auto Candidate Mode - disable', () => {
    settings.autoCandidateMode = true

    const wrapper = shallow(<ModalSettings {...props} />)
    const input = findInputByLabel(wrapper, 'Start in automatic mode')

    expect(input.prop('checked')).toBe(true)

    input.simulate('change')

    expect(settings.autoCandidateMode).toBe(false)
    expect(props.game.act).not.toHaveBeenCalled()
  })

  const simpleSettings = {
    highlightConflicts: 'Highlight conflicts',
    sound: 'Play sound on solve',
    clock: 'Show clock',
  }

  Object.entries(simpleSettings).forEach(([setting, label]) => {
    test(`${label} - enable`, () => {
      settings[setting] = false

      const wrapper = shallow(<ModalSettings {...props} />)
      const input = findInputByLabel(wrapper, label)

      expect(input.prop('checked')).toBe(false)
      input.simulate('change')
      expect(settings[setting]).toBe(true)
    })

    test(`${label} - disable`, () => {
      settings[setting] = true

      const wrapper = shallow(<ModalSettings {...props} />)
      const input = findInputByLabel(wrapper, label)

      expect(input.prop('checked')).toBe(true)
      input.simulate('change')
      expect(settings[setting]).toBe(false)
    })
  })
})
