import React from 'react'
import { useSelector } from 'react-redux'

import { useUser } from 'user'
import {
  trackClick,
  trackImpression,
  trackModuleInteraction,
} from 'foundation'

import { selectCompletedDifficulties } from '../../store/slices/gameStatus'
import handleHybridLoginClick from '../../../helpers/hybridLoginLinkClick'
import Welcome from '../../../shared/moments/components/Welcome'
import NonDismissalBanner from '../../../shared/Banner/NonDismissalBanner'
import { shouldShowAppDownloadCTA } from '../../../shared/Banner/bannerHelpers'
import { LEVELS } from '../../constants/constants'
import useAbra from '../../../../shared/abra/useAbra'

type MomentWelcomeProps = {
  displayDate: string
  startGame: () => void
  level: {
    current: 'easy' | 'medium' | 'hard' | 'welcome'
  }
  setLevel: (
    level: string,
    options?: { puzzleDate?: string; origin?: string },
  ) => void
  puzzleDate: string
}

const MomentWelcome = ({
  displayDate,
  startGame,
  level,
  setLevel,
  puzzleDate,
}: MomentWelcomeProps) => {
  type ButtonsType = {
    text?: string
    icon?: string
    color?: string
    action?: (
      event:
        | MouseEvent
        | React.MouseEvent<HTMLButtonElement, MouseEvent>
        | React.MouseEvent<HTMLAnchorElement, MouseEvent>,
    ) => void
  }[]

  type MomentWelcomePropsType = {
    game?: string
    icon?: string
    title?: string
    date?: string
    infoText?: string
    bgColor?: string
    meta?: string
    buttonGroups?: [
      {
        header?: string
        buttons?: ButtonsType
        desktopButtonOrder?: 'vertical' | null
      },
    ]
    description?: {
      text: string
    }
  }
  let momentWelcomeProps: MomentWelcomePropsType = {
    buttonGroups: [{ buttons: [] }],
  }

  const { getVariant, reportExposure } = useAbra()

  const downloadCTAVariant = getVariant(
    'GAMES_sudokuWelcomeAppDownloadCTA_0823',
  )

  const showAppDownloadCTA = React.useMemo(
    () =>
      shouldShowAppDownloadCTA(
        downloadCTAVariant,
        '0_control',
        window,
      ),
    [downloadCTAVariant],
  )

  const sharedWelcomeProps = {
    game: 'sudoku',
    icon: 'sudoku',
    title: 'Sudoku',
    date: displayDate,
    bgColor: '$sudoku-orange',
  }

  const stateCompletedDifficulties = useSelector(
    selectCompletedDifficulties,
  )

  momentWelcomeProps = {
    ...sharedWelcomeProps,
    meta: 'Choose Your Puzzle:',
    buttonGroups: [
      {
        buttons: LEVELS.map(({ name, displayName }) => ({
          text: displayName,
          icon: stateCompletedDifficulties.includes(name)
            ? 'checked'
            : 'none',
          color: stateCompletedDifficulties.includes(name)
            ? 'transparent'
            : undefined,
          action: () => {
            setLevel(name, { puzzleDate })
            const hamburger = document.getElementById('js-nav-burger')

            const hamburgerMenuIsOpen =
              !!hamburger?.classList.contains('active')

            if (hamburgerMenuIsOpen) {
              hamburger?.click()
            }

            startGame()
          },
        })),
        desktopButtonOrder: 'vertical',
      },
    ],
    description: {
      text: 'Try this numbers game, minus the math.',
    },
  }

  const { hasLoaded, userType, updateUserType } = useUser()

  if (hasLoaded) {
    const { isLoggedIn } = userType
    const isNewsreader = !!window.newsreaderAppPlatform

    if (
      !isLoggedIn &&
      !isNewsreader &&
      momentWelcomeProps.buttonGroups
    ) {
      momentWelcomeProps.buttonGroups[0].buttons?.push({
        text: 'Log in',
        color: 'secondary',
        action: (
          event:
            | MouseEvent
            | React.MouseEvent<HTMLButtonElement, MouseEvent>
            | React.MouseEvent<HTMLAnchorElement, MouseEvent>,
        ) => {
          handleHybridLoginClick(event, updateUserType, '')

          trackClick({
            name: 'sudoku',
            context: '',
            label: 'log-in-welcome',
          })
        },
      })
    }
  }

  React.useEffect(() => {
    if (downloadCTAVariant) {
      reportExposure('GAMES_sudokuWelcomeAppDownloadCTA_0823')
    }

    if (showAppDownloadCTA) {
      trackImpression('welcome-modal', 'sudoku', 'download-app', '')
    }
  }, [showAppDownloadCTA, downloadCTAVariant])

  React.useEffect(() => {
    if (hasLoaded) {
      const { isLoggedIn } = userType
      const isNewsreader = !!window.newsreaderAppPlatform

      if (level.current === 'welcome') {
        if (!isLoggedIn && !isNewsreader) {
          trackImpression(
            'welcome-modal',
            'sudoku',
            'log-in',
            JSON.stringify(stateCompletedDifficulties),
          )
        } else {
          trackImpression(
            'welcome-modal',
            'sudoku',
            null,
            JSON.stringify(stateCompletedDifficulties),
          )
        }
      }
    }
  }, [hasLoaded, level.current])

  return (
    <>
      <Welcome game={momentWelcomeProps} />
      {showAppDownloadCTA && (
        <NonDismissalBanner
          title="Play Sudoku and more in the NYT Games app."
          icon="crossword-app"
          ctaLink="https://fpx3r.app.goo.gl/app-store"
          onClickLink={() => {
            trackModuleInteraction(
              '',
              '',
              'sudoku',
              'welcome-download-app',
              'https://fpx3r.app.goo.gl/app-store',
            )
          }}
          showMultipleIconsBanner
        />
      )}
    </>
  )
}

export default MomentWelcome
