import React from 'react'
import pt from 'prop-types'

import { isMobile } from 'responsive'

import { open, selectOpenModal } from '../../store/slices/modalSystem'
import { useAppSelector, useAppDispatch } from '../../store'
import { Welcome } from '../../constants/constants'
import { trackEvent } from '../../helpers/sudokuTracker'

const Timer = ({ game, level }) => {
  const dispatch = useAppDispatch()
  const openModal = useAppSelector(selectOpenModal)
  const { timeFormatted } = game.timer.useTick()
  const pauseTimeout = React.useRef()

  // the functionality in this effect is largely ripped from crossword/Timer.tsx
  React.useEffect(() => {
    const pause = () => {
      if (game.timer.isPaused) return

      if (openModal) return

      game.timer.pause()
      if (!game.view.solved) {
        dispatch(open('Pause'))
      }
    }

    const handleBlur = () => {
      // pause immediately on mobile
      if (isMobile && document.hidden) {
        trackEvent(level, 'pause')
        return pause()
      }

      // wait 30 seconds before pausing on desktop
      if (!pauseTimeout.current) {
        pauseTimeout.current = window.setTimeout(pause, 30000)
      }
    }

    const handleFocus = () => {
      if (pauseTimeout.current) {
        window.clearTimeout(pauseTimeout.current)
        pauseTimeout.current = null
      }
    }

    const handleVisibilityChange = () => {
      if (document.hidden || document.visibilityState === 'hidden') {
        handleBlur()
      } else {
        handleFocus()
      }
    }

    if (!isMobile) {
      window.addEventListener(
        'visibilitychange',
        handleVisibilityChange,
      )
    }

    if (isMobile) {
      window.addEventListener('blur', handleBlur) // For when mobile devices are locked
      window.addEventListener('focus', handleFocus)
    }

    return () => {
      if (!isMobile) {
        window.removeEventListener(
          'visibilitychange',
          handleVisibilityChange,
        )
      }

      if (isMobile) {
        window.removeEventListener('blur', handleBlur)
        window.removeEventListener('focus', handleFocus)
      }
    }
  }, [level])

  return (
    <div
      role="button"
      className="su-timer"
      onClick={() => {
        if (!game.view.solved) {
          game.timer.toggle()
          trackEvent(level, 'pause')
        }
      }}
    >
      {level !== Welcome && (
        <span className="su-timer__value">
          {game.view.solved
            ? game.timer.timeFormatted
            : timeFormatted}
        </span>
      )}
      {game.view.solved || (
        <div className="su-timer__pause">
          <i className="su-timer__pause-icon" />
        </div>
      )}
    </div>
  )
}

Timer.propTypes = {
  game: pt.shape({
    view: pt.shape({
      solved: pt.bool,
    }),
    timer: pt.shape({
      toggle: pt.func,
      useTick: pt.func,
      pause: pt.func,
      timeFormatted: pt.string,
      isPaused: pt.bool,
    }),
  }),
  level: pt.string,
}

export default Timer
