import React from 'react'
import { mount } from 'enzyme'
import { Provider } from 'react-redux'

import DropDown from '../../../../shared/DropDown/DropDown'
import DropDownItem from '../../../../shared/DropDown/DropDownItem'
import * as toolbarHelpers from '../../../helpers/toolbar'
import { trackEvent } from '../../../helpers/sudokuTracker'

import HelpSelectMenu from './HelpSelectMenu'

jest.mock(
  '../../../../shared/DropDown/DropDownCloseHook',
  () => () => <></>,
)

jest.mock('../../../helpers/toolbar', () => ({
  onGameReset: jest.fn(),
  onShowHint: jest.fn(),
  onCheckCell: jest.fn(),
  onCheckBoard: jest.fn(),
  onRevealCell: jest.fn(),
  onRevealBoard: jest.fn(),
}))

jest.mock('../../../helpers/sudokuTracker', () => ({
  trackEvent: jest.fn(),
  gameIdMap: {
    easy: 'e-12345',
    medium: 'm-12345',
    hard: 'h-12345',
  },
}))

jest.mock('../../../store/slices/gameStatus', () => ({
  addToCompletedDifficulties: jest.fn(),
  removeFromCompletedDifficulties: jest.fn(),
  selectCompletedDifficulties: jest.fn(),
}))

jest.mock('../../../store/index', () => ({
  useAppDispatch: jest.fn(),
  useAppSelector: jest.fn(),
}))

jest.mock('../../../store/slices/modalSystem', () => ({
  open: jest.fn(),
  selectOpenModal: jest.fn(),
}))

jest.mock('../../../store/index', () => ({
  useAppDispatch: jest.fn(),
}))

jest.mock('../../../store/slices/modalSystem', () => ({
  open: jest.fn(),
}))

describe('SUDOKU: TOOLBAR: HELP SELECT MENU BEFORE SOLVED', () => {
  let wrapper
  let defaultProps
  const makeStore = () => ({
    getState: () => ({}),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  })

  const store = makeStore()

  beforeEach(async () => {
    defaultProps = {
      game: {
        act: jest.fn(),
        view: {},
        state: { difficulty: 'easy' },
        timer: {},
      },
      level: {
        current: 'welcome',
      },
    }

    wrapper = await mount(
      <Provider store={store}>
        <HelpSelectMenu {...defaultProps} />
      </Provider>,
    )
  })

  test('renders help menu', () => {
    expect(wrapper.exists()).toBe(true)
  })

  test('help button exists', () => {
    const dropdown = wrapper.find(DropDown)

    expect(dropdown.props().ariaLabel).toBe('help')
  })

  test('dropdown item does not exist by default', () => {
    const dropdownitem = wrapper.find(DropDownItem)

    expect(dropdownitem.exists()).toBe(false)
  })

  test('dropdown item exists after clicking help button', () => {
    const dropdown = wrapper.find(DropDown)
    dropdown.find('.pz-dropdown__button').simulate('click')
    const dropdownitem = wrapper.find(DropDownItem)

    expect(dropdownitem.exists()).toBe(true)
  })

  test('fires callbacks for hints on click', () => {
    const dropdown = wrapper.find(DropDown)
    dropdown.find('.pz-dropdown__button').simulate('click')

    const dropdownitem = wrapper.find(DropDownItem)
    dropdownitem.first().find('button').simulate('click')

    expect(toolbarHelpers.onShowHint).toBeCalled()
    expect(trackEvent).toHaveBeenCalledWith('easy', 'hint')
  })

  test('fires callbacks for check cell on click', () => {
    const dropdown = wrapper.find(DropDown)
    dropdown.find('.pz-dropdown__button').simulate('click')
    const dropdownitem = wrapper.find(DropDownItem)
    dropdownitem.at(1).find('button').simulate('click')
    expect(dropdownitem.at(1).text()).toEqual('Check Cell')
    expect(toolbarHelpers.onCheckCell).toBeCalled()
    expect(trackEvent).toHaveBeenCalledWith('easy', 'check-cell')
  })
})

describe('SUDOKU: TOOLBAR: HELP SELECT MENU AFTER SOLVED', () => {
  let wrapper
  let defaultProps
  const makeStore = () => ({
    getState: () => ({}),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  })

  const store = makeStore()

  beforeEach(async () => {
    defaultProps = {
      game: {
        act: jest.fn(),
        view: { solved: true },
        state: { difficulty: 'easy' },
        timer: {},
      },
      level: {
        current: 'welcome',
      },
    }

    wrapper = await mount(
      <Provider store={store}>
        <HelpSelectMenu {...defaultProps} />
      </Provider>,
    )
  })

  test('renders help menu', () => {
    expect(wrapper.exists()).toBe(true)
  })

  test('help button exists', () => {
    const dropdown = wrapper.find(DropDown)

    expect(dropdown.props().ariaLabel).toBe('help')
  })

  test('dropdown item does not exist by default', () => {
    const dropdownitem = wrapper.find(DropDownItem)

    expect(dropdownitem.exists()).toBe(false)
  })

  test('does not show check cell if solved, reset puzzle is first item', () => {
    const dropdown = wrapper.find(DropDown)
    dropdown.find('.pz-dropdown__button').simulate('click')
    const dropdownitem = wrapper.find(DropDownItem)
    expect(dropdownitem.at(0).text()).not.toEqual('Check Cell')
    expect(dropdownitem.at(0).text()).toEqual('Reset Puzzle')
  })
})
