import React from 'react'
import ReactDOM from 'react-dom'
import cn from 'classnames'
import pt from 'prop-types'

import { isMobile as mobile } from 'responsive'
import { mobileNavTools } from 'foundation'

import { momentSystem } from '../Moments'
import Timer from '../Timer/Timer'
import { trackEvent } from '../../helpers/sudokuTracker'
import { onPrint } from '../../helpers/toolbar'
import { useAppDispatch } from '../../store'
import { open } from '../../store/slices/modalSystem'
import { Welcome } from '../../constants/constants'

import HelpSelectMenu from './HelpSelectMenu/HelpSelectMenu'

const toolbarEl = (mobile) =>
  document.getElementById(
    `${mobile ? 'js-mobile' : 'portal-game'}-toolbar`,
  )

const Toolbar = ({
  game,
  level,
  settings,
  children,
  forceMobile,
}) => {
  const isMobile = mobile || forceMobile === true
  const isNewsReader = !!window.newsreaderAppPlatform
  const isMobileWeb =
    isMobile && !window.isHybridWebView && !isNewsReader
  const isDesktopWeb = !isMobile && !window.isHybridWebView
  const dispatch = useAppDispatch()

  const toolbarClasses = cn({
    'pz-module': true,
    'pz-flex-row': true,
    'pz-game-toolbar-content': true,
    'su-toolbar': true,
    'su-toolbar__mobile': isMobile,
  })

  return ReactDOM.createPortal(
    <>
      <div className={toolbarClasses}>
        {/* Left */}
        <div className="su-toolbar__left">
          {isMobileWeb && (
            <button
              type="button"
              className="su-back__text-container"
              onClick={() => {
                mobileNavTools.deactivate()
                trackEvent(level.current, 'back', 'in-game')
                level.set('welcome')
                momentSystem.transition('game', 'welcome')
                // to make hamburger menu appear again
                const element = document.querySelector(
                  '.pz-nav__hamburger-helper',
                )
                element.style.display = 'flex'
                game.timer.pause()
              }}
            >
              <i className="su-back__arrow" />
              <span className="su-back__text">Back</span>
            </button>
          )}
          {isDesktopWeb && (
            <button
              type="button"
              className="su-back__text-container"
              onClick={() => {
                trackEvent(level.current, 'back', 'in-game')
                game.timer.pause()
                level.set('welcome')
                momentSystem.transition('game', 'welcome')
              }}
            >
              <i className="su-back__arrow" />
              <span className="su-back__text">Back</span>
            </button>
          )}
        </div>

        {/* Center */}
        {/* Only show level + timer in toolbar for desktop web */}
        {!isMobile && (
          <div className="su-toolbar__center">
            {level.current !== Welcome && (
              <div className="su-level su-level-desktop">
                {level.current}
              </div>
            )}
            {settings.get('clock') && (
              <div className="su-timer su-timer-desktop">
                <Timer game={game} level={level.current} />
              </div>
            )}
          </div>
        )}

        {/* Right */}
        {/* to-do refac this to use nested su-toolbar__mobile */}
        <div className="su-toolbar__right">
          {/* Icons+ */}
          {children}
          {/* Buttons */}
          <button
            type="button"
            className="pz-toolbar-button su-app__print"
            onClick={() => {
              onPrint(game)
              trackEvent(level.current, 'print')
            }}
          >
            Print
          </button>
          <button
            type="button"
            aria-label="how to play"
            data-testid="how-to-play"
            className="pz-toolbar-button su-app__howto"
            onClick={() => {
              dispatch(open('Rules'))
              game.timer.pause()
              trackEvent(level.current, 'how-to-play')
            }}
          >
            <i className="su-app__help-icon" />
          </button>
          <HelpSelectMenu game={game} level={level} />
          <button
            type="button"
            aria-label="settings"
            data-testid="settings"
            className="pz-toolbar-button su-app__settings"
            onClick={() => {
              game.timer.pause()
              dispatch(open('Settings'))
              trackEvent(level.current, 'settings')
            }}
          >
            <i className="su-app__settings-icon" />
          </button>
        </div>
      </div>
    </>,
    toolbarEl(mobile),
  )
}

Toolbar.propTypes = {
  game: pt.shape({
    timer: pt.shape({
      pause: pt.func,
    }),
  }),
  level: pt.shape({
    current: pt.string,
  }),
  settings: pt.shape({
    set: pt.func,
    get: pt.func,
    toggle: pt.func,
  }),
  forceMobile: pt.bool,
}

export default Toolbar
