import * as React from 'react'
import { Provider } from 'react-redux'
import { mount, shallow } from 'enzyme'

import Timer from '../Timer/Timer'
import { trackEvent } from '../../helpers/sudokuTracker'
import { onPrint } from '../../helpers/toolbar'
import { open, close } from '../../store/slices/modalSystem'

import Toolbar from './Toolbar'

jest.mock('responsive', () => ({
  isMobile: false,
}))

jest.mock('react-dom', () => ({
  createPortal: (jsx) => jsx,
}))

jest.mock(
  '../../../../../node_modules/react-redux/lib/utils/batch.js',
  () => ({
    setBatch: jest.fn(),
    getBatch: () => (fn) => fn(),
  }),
)

jest.mock('../../helpers/sudokuTracker', () => ({
  trackEvent: jest.fn(),
  gameIdMap: {
    easy: 'e-12345',
    medium: 'm-12345',
    hard: 'h-12345',
  },
}))

jest.mock('../../helpers/toolbar', () => ({
  onPrint: jest.fn(),
}))

jest.mock('../../store/slices/gameStatus', () => ({
  addToCompletedDifficulties: jest.fn(),
  removeFromCompletedDifficulties: jest.fn(),
  selectCompletedDifficulties: jest.fn(),
}))

jest.mock('../../store/index', () => ({
  useAppDispatch: jest.fn(() => jest.fn()),
  useAppSelector: jest.fn(),
}))

jest.mock('../../store/slices/modalSystem', () => ({
  open: jest.fn(),
  close: jest.fn(),
}))

describe('SUDOKU: TOOLBAR', () => {
  let wrapper

  let left
  let right

  let timer

  let printBtn
  let settingsBtn
  let howToPlayBtn

  let defaultProps
  let storedSettings
  const makeStore = () => ({
    getState: () => ({}),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  })

  const store = makeStore()

  beforeEach(() => {
    storedSettings = {
      clock: true,
    }
    const tick = {
      timeFormatted: 'timeBefore',
      isPaused: true,
    }

    defaultProps = {
      game: {
        act: jest.fn(),
        view: {},
        state: { difficulty: 'medium' },
        timer: {
          toggle: jest.fn(),
          useTick: () => tick,
          pause: jest.fn(),
        },
      },
      level: {
        current: 'medium',
        set: jest.fn(),
      },
      settings: {
        get: (k) => storedSettings[k],
      },
    }

    wrapper = mount(
      <Provider store={store}>
        <Toolbar {...defaultProps} />
      </Provider>,
    )

    left = () => wrapper.find('.su-toolbar__left')
    right = () => wrapper.find('.su-toolbar__right')

    timer = () => wrapper.find(Timer)

    printBtn = () => right().find('.su-app__print')
    settingsBtn = () => right().find('.su-app__settings')
    howToPlayBtn = () => right().find('.su-app__howto')
  })

  describe('Left', () => {
    test('renders', () => {
      expect(left().exists()).toBe(true)
    })
  })

  describe('Center', () => {
    test('renders the timer', () => {
      expect(timer()).toHaveLength(1)
    })

    test('does not render the timer if hidden', () => {
      storedSettings.clock = false
      wrapper = shallow(<Toolbar {...defaultProps} />)
      expect(timer()).toHaveLength(0)
    })
  })

  describe('Right', () => {
    test('renders', () => {
      expect(right().exists()).toBe(true)
    })

    describe('print button', () => {
      test('renders', () => {
        expect(printBtn().exists()).toBe(true)
      })

      test('fires onPrint event onClick', () => {
        printBtn().simulate('click')
        expect(onPrint).toBeCalled()
      })

      test('fires tracking event onClick', () => {
        printBtn().simulate('click')
        expect(trackEvent).toHaveBeenCalledWith('medium', 'print')
      })
    })

    describe('settings button', () => {
      test('renders', () => {
        expect(settingsBtn().exists()).toBe(true)
      })

      test('calls open action on click with Settings as argument', () => {
        settingsBtn().simulate('click')
        expect(open).toHaveBeenCalledWith('Settings')
      })

      test('fires tracking event onClick', () => {
        printBtn().simulate('click')
        expect(trackEvent).toHaveBeenCalledWith('medium', 'settings')
      })
    })

    describe('how to play button', () => {
      test('renders', () => {
        expect(howToPlayBtn().exists()).toBe(true)
      })

      test('calls open action on click with Rules as argument', () => {
        howToPlayBtn().simulate('click')
        expect(open).toHaveBeenCalledWith('Rules')
      })

      test('fires tracking event onClick', () => {
        howToPlayBtn().simulate('click')
        expect(trackEvent).toHaveBeenCalledWith(
          'medium',
          'how-to-play',
        )
      })
    })
  })
})
