import whenIdle from '../../helpers/whenIdle'

import { load, save, saveDifficulty } from './persist'

jest.mock('../../helpers/whenIdle', () => jest.fn())

describe('SUDOKU PERSIST DATA', () => {
  let getItem
  let setItem
  let logError
  let stored

  beforeEach(() => {
    stored = {}

    getItem = jest.spyOn(window.Storage.prototype, 'getItem')
    setItem = jest.spyOn(window.Storage.prototype, 'setItem')

    logError = jest.spyOn(console, 'error')

    getItem.mockImplementation((k) => stored[k])
    setItem.mockImplementation((k, v) => {
      stored[k] = v
    })
    logError.mockImplementation(() => {})
  })

  afterEach(() => {
    getItem.mockClear()
    setItem.mockClear()
    logError.mockClear()
    whenIdle.mockReset()
  })

  test('Load reads once from local storage and then from memory', () => {
    const foo = { foo: true }

    setItem('xwd-sudoku-1', JSON.stringify(foo))

    const a = load('1')
    const b = load('1')

    expect(a).toEqual(foo)
    expect(b).toEqual(foo)
    expect(getItem).toHaveBeenCalledTimes(1)
  })

  test('Load catches error', () => {
    const err = new Error('')
    getItem.mockImplementation(() => {
      throw err
    })
    load('2')
    expect(logError).toHaveBeenCalledWith(err)
  })

  test('Save urgently', () => {
    save('foo', { foo: true }, true)
    expect(getItem('xwd-sudoku-foo')).toEqual(
      JSON.stringify({ foo: true }),
    )
  })

  test('Save deferred', () => {
    const expectedkey = 'xwd-sudoku-bar'
    const bar = { bar: true }

    save('bar', bar)

    expect(getItem(expectedkey)).toBe(undefined)

    const [key, callback] = whenIdle.mock.calls[0]

    callback()

    expect(key).toEqual(expectedkey)
    expect(getItem(expectedkey)).toBe(JSON.stringify(bar))
  })

  test('Save catches error', () => {
    const err = new Error('')

    setItem.mockImplementation(() => {
      throw err
    })

    save('2', {}, true)
    expect(logError).toHaveBeenCalledWith(err)
  })

  test('Load and save share memory', () => {
    save('foo', { v: 1 }, true)
    const v1 = load('foo')

    expect(v1).toEqual({ v: 1 })

    save('foo', { v: 2 }, true)
    const v2 = load('foo')

    expect(v2).toEqual({ v: 2 })
  })

  test('Load and save share memory', () => {
    saveDifficulty('completedDifficulties', 'easy', true, true)
    const v1 = load('completedDifficulties')

    expect(v1).toEqual(['easy'])

    saveDifficulty('completedDifficulties', 'medium', true, true)
    const v2 = load('completedDifficulties')

    expect(v2).toEqual(['easy', 'medium'])
  })
})
