export const STATE_GUESSED = 'guessed'
export const STATE_REJECTED = 'rejected'
export const STATE_CONFIRMED = 'confirmed'

export const select = (cell) => ({ selected: cell })
export const toggle = (key, state) => ({
  [key]: !state[key],
})
export const enable = (key) => ({ [key]: true })
export const disable = (key) => ({ [key]: false })

export const move = (direction, state) => {
  let row = (state.selected / 9) >> 0
  let col = state.selected % 9

  if (direction === 'down' && row < 8) row += 1

  if (direction === 'up' && row > 0) row -= 1

  if (direction === 'left' && col > 0) col -= 1

  if (direction === 'right' && col < 8) col += 1

  return {
    selected: row * 9 + col,
  }
}

export const clearCell = (_, state, view) => {
  const current = state.cells[state.selected]

  if (current.value) {
    return {
      cells: {
        [state.selected]: {
          value: undefined,
          state: STATE_GUESSED,
        },
      },
    }
  }

  if (!state.auto) {
    const rendered = view.candidates[state.selected]
    const changed = {}

    Object.keys(rendered).forEach((c) => {
      if (rendered[c]) {
        changed[c] = undefined
      }
    })

    return {
      manualCandidates: {
        [state.selected]: changed,
      },
    }
  }
}

export const toggleCandidate = (candidate, state, view) => {
  const rendered = view.candidates[state.selected]

  const change = {
    [state.auto ? 'autoCandidates' : 'manualCandidates']: {
      [state.selected]: {
        [candidate]: !rendered[candidate],
      },
    },
  }

  if (state.cells[state.selected].value) {
    change.cells = {
      [state.selected]: { value: undefined },
    }
  }

  return change
}

export const enterValue = (value, state, view, settings) => {
  const current = state.cells[state.selected]

  let autoCheck

  if (current.value === value) return

  if (settings.get('autoCheck')) {
    if (value === current.answer) {
      autoCheck = STATE_CONFIRMED
    } else {
      autoCheck = STATE_REJECTED
    }
  } else {
    autoCheck = STATE_GUESSED
  }

  const change = {
    cells: {
      [state.selected]: {
        value,

        state: autoCheck,
      },
    },
  }

  if (state.auto) {
    change.autoCandidates = view.pendingAutoChanges
  }

  return change
}

export const guess = (value, state, view, settings) => {
  if (
    state.cells[state.selected].prefilled ||
    state.cells[state.selected].state === STATE_CONFIRMED
  ) {
    return
  }

  if (!value) {
    return clearCell(value, state, view)
  }

  if (state.candidateMode) {
    return toggleCandidate(value, state, view)
  }

  return enterValue(value, state, view, settings)
}

export const reveal = (scope, state) => {
  const start = scope === 'cell' ? state.selected : 0
  const end = scope === 'cell' ? state.selected + 1 : state.size
  const newCells = {}

  for (let i = start; i < end; i += 1) {
    if (!state.cells[i].prefilled) {
      newCells[i] = {
        value: state.cells[i].answer,
        state: STATE_CONFIRMED,
      }
    }
  }

  return {
    cells: newCells,
  }
}

export const check = (scope, state) => {
  const start = scope === 'cell' ? state.selected : 0
  const end = scope === 'cell' ? state.selected + 1 : state.size
  const newCells = {}

  for (let i = start; i < end; i += 1) {
    if (!state.cells[i].prefilled && state.cells[i].value) {
      newCells[i] = {
        state:
          state.cells[i].value === state.cells[i].answer
            ? STATE_CONFIRMED
            : STATE_REJECTED,
      }
    }
  }

  return {
    cells: newCells,
  }
}

export const debugFill = (_, state) => {
  const newCells = {}

  let skipped
  let selected

  for (let i = 0; i < state.size; i += 1) {
    if (!state.cells[i].prefilled) {
      newCells[i] = {
        value: state.cells[i].answer,
        state: STATE_GUESSED,
      }

      if (!skipped) {
        skipped = true
        selected = i
        newCells[i].value = undefined
      }
    }
  }

  return {
    selected,
    cells: newCells,
  }
}

export const showHint = (_, state) => {
  const { hints } = state

  for (let i = 0; i < hints.length; i += 1) {
    if (!state.cells[hints[i]].value) {
      return { selected: hints[i] }
    }

    if (
      state.cells[hints[i]].value !== state.cells[hints[i]].answer
    ) {
      return {
        selected: hints[i],
        cells: { [hints[i]]: { state: STATE_REJECTED } },
      }
    }
  }
}
