import {
  select,
  toggle,
  enable,
  disable,
  clearCell,
  toggleCandidate,
  move,
  guess,
  reveal,
  check,
  showHint,
  debugFill,
} from './actions'

describe('SUDOKU: SELECT CELL', () => {
  test('Selected is set to the index of the input cell', () => {
    const change = select(0)
    expect(change).toEqual({ selected: 0 })
  })
})

describe('SUDOKU: TOGGLE', () => {
  test('enable', () => {
    const state = toggle('foo', { foo: false })
    expect(state).toEqual({ foo: true })
  })

  test('disable', () => {
    const state = toggle('bar', { bar: true })
    expect(state).toEqual({ bar: false })
  })
})

describe('SUDOKU: ENABLE / DISABLE', () => {
  test('enable', () => {
    const state = enable('foo', { foo: false })
    expect(state).toEqual({ foo: true })
  })

  test('disable', () => {
    const state = disable('bar', { bar: true })
    expect(state).toEqual({ bar: false })
  })
})

describe('SUDOKU: MOVE CURSOR', () => {
  test('Up moves up one cell', () => {
    const state = { selected: 9 }
    const change = move('up', state)
    expect(change).toEqual({ selected: 0 })
  })

  test('Up stops at the top edge', () => {
    const state = { selected: 0 }
    const change = move('up', state)
    expect(change).toEqual({ selected: 0 })
  })

  test('Down moves down one cell', () => {
    const state = { selected: 0 }
    const change = move('down', state)
    expect(change).toEqual({ selected: 9 })
  })

  test('Down stops at the bottom edge', () => {
    const state = { selected: 72 }
    const change = move('down', state)
    expect(change).toEqual({ selected: 72 })
  })

  test('Right moves right one cell', () => {
    const state = { selected: 0 }
    const change = move('right', state)
    expect(change).toEqual({ selected: 1 })
  })

  test('Right stops at the right edge', () => {
    const state = { selected: 8 }
    const change = move('right', state)
    expect(change).toEqual({ selected: 8 })
  })

  test('Left moves left one cell', () => {
    const state = { selected: 1 }
    const change = move('left', state)
    expect(change).toEqual({ selected: 0 })
  })

  test('Left stops at the left edge', () => {
    const state = { selected: 0 }
    const change = move('left', state)
    expect(change).toEqual({ selected: 0 })
  })
})

describe('SUDOKU: TOGGLE CANDIDATE', () => {
  test('Manual candidates can be toggled', () => {
    const state = {
      selected: 0,
      cells: {
        0: { value: 0 },
      },
    }

    const view = {
      candidates: {
        0: { 1: true },
      },
    }
    let change = toggleCandidate(1, state, view)
    expect(change.manualCandidates[0][1]).toBe(false)
    change = toggleCandidate(4, state, view)
    expect(change.manualCandidates[0][4]).toBe(true)
  })

  test('Auto candidates can be toggled', () => {
    const state = {
      auto: true,
      selected: 0,
      cells: {
        0: { value: 0 },
      },
    }

    const view = {
      candidates: {
        0: { 1: true },
      },
    }
    let change = toggleCandidate(1, state, view)
    expect(change.autoCandidates[0][1]).toBe(false)
    change = toggleCandidate(4, state, view)
    expect(change.autoCandidates[0][4]).toBe(true)
  })
})

describe('CLEAR CELL', () => {
  let state
  let view

  beforeEach(() => {
    state = {
      auto: false,
      selected: 1,
      cells: {
        1: { value: 1 },
      },
    }
    view = {
      candidates: {
        1: { 1: true, 2: true, 3: false },
      },
    }
  })

  test('clears value first (manual)', () => {
    expect(clearCell(undefined, state, view)).toEqual({
      cells: { 1: { value: undefined, state: 'guessed' } },
    })

    state.auto = true

    expect(guess(undefined, state, view)).toEqual({
      cells: { 1: { value: undefined, state: 'guessed' } },
    })

    state.candidateMode = true

    expect(guess(undefined, state, view)).toEqual({
      cells: { 1: { value: undefined, state: 'guessed' } },
    })
  })

  test('clears manual candidates if no value', () => {
    state.cells[1].value = undefined
    expect(clearCell(undefined, state, view)).toEqual({
      manualCandidates: {
        1: { 1: undefined, 2: undefined },
      },
    })
  })

  test('auto candidates are not deleted in auto mode', () => {
    state.auto = true
    state.cells[1].value = undefined
    expect(clearCell(undefined, state, view)).toEqual(undefined)
  })
})

describe('SUDOKU: GUESS', () => {
  let settings
  let state
  let view

  beforeEach(() => {
    state = {
      selected: 1,
      cells: {
        1: {},
      },
    }
    view = {}
    settings = { get: jest.fn() }
    settings.get.mockReturnValue(false)
  })

  test('Prefilled cell cannot be edited', () => {
    state.cells[state.selected].prefilled = true
    const change = guess(2, state, view, settings)
    expect(change).toBeUndefined() // just returns when prefilled
  })

  test('Confirmed cell cannot be edited', () => {
    state.cells[state.selected].state = 'confirmed'
    const change = guess(2, state, view, settings)
    expect(change).toBeUndefined() // just returns when confirmed
  })

  test('Guessing the same number does not change the cell', () => {
    state.cells[state.selected].value = 1
    const change = guess(1, state, view, settings)
    expect(change).toBeUndefined() // just returns when same number is guessed
  })

  test('Blank cell value can be edited', () => {
    const change = guess(2, state, view, settings)
    expect(change).toEqual({
      cells: { 1: { value: 2, state: 'guessed' } },
    })
  })

  test('Existing value of non-prefilled, unconfirmed cell can be edited', () => {
    state.cells[state.selected] = {
      prefilled: false,
      state: 'guessed',
      value: 2,
    }
    const change = guess(1, state, view, settings)
    expect(change).toEqual({
      cells: { 1: { state: 'guessed', value: 1 } },
    })
  })

  test('Conflicting candidates are not rendered in auto mode', () => {
    state = {
      auto: true,
      selected: 0,
      cells: {
        0: { prefilled: false, state: 'guessed', value: 2 },
        1: { prefilled: false },
      },
      autoCandidates: {
        1: { 1: true, 4: true },
      },
    }

    view = {
      pendingAutoChanges: {
        1: { 1: undefined },
      },
    }

    const change = guess(1, state, view, settings)
    expect(change.autoCandidates[1][1]).toBe(undefined)
  })

  test('Candidates overwrite existing cell value when in candidate mode', () => {
    state = {
      candidateMode: true,
      selected: 0,
      cells: {
        0: { prefilled: false, state: 'guessed', value: 2 },
      },
    }

    view = {
      candidates: { 0: {} },
    }

    const change = guess(3, state, view, settings)
    expect(change.cells[0].value).toBe(undefined)
  })

  test('Confirms correct guess while autochecking', () => {
    settings.get.mockImplementation((k) => k === 'autoCheck')
    state.cells[state.selected] = { answer: 1 }
    const change = guess(1, state, view, settings)
    expect(change).toEqual({
      cells: { 1: { state: 'confirmed', value: 1 } },
    })
  })

  test('Rejects incorrect guess while autochecking', () => {
    settings.get.mockImplementation((k) => k === 'autoCheck')
    state.cells[state.selected] = { answer: 1 }
    const change = guess(2, state, view, settings)
    expect(change).toEqual({
      cells: { 1: { state: 'rejected', value: 2 } },
    })
  })
})

describe('SUDOKU: REVEAL CELLS', () => {
  test('Prefilled cells are not affected by reveal', () => {
    const initial = {
      size: 2,
      cells: {
        0: { prefilled: true, value: 2, answer: 1 },
        1: { prefilled: true, value: 2, answer: 2 },
      },
    }
    const expected = { cells: {} }
    const change = reveal(undefined, initial)
    expect(change).toEqual(expected)
  })

  test('Reveal cell only reveals the selected cell', () => {
    const initial = {
      selected: 0,
      size: 2,
      cells: {
        0: { value: 0, answer: 1 },
        1: { value: 0, answer: 2 },
      },
    }
    const change = reveal('cell', initial)
    expect(change.cells[0].value).toEqual(1)
    expect(change.cells[0].state).toBe('confirmed')
    expect(change.cells[1]).toBeUndefined()
  })

  test('Reveal board reveals all non-prefilled cells', () => {
    const initial = {
      size: 3,
      cells: {
        0: { prefilled: true, value: 1, answer: 1 },
        1: { prefilled: false, value: 3, answer: 2 },
        2: { prefilled: false, value: 3, answer: 4 },
      },
    }
    const change = reveal('board', initial)
    expect(change.cells[0]).toBeUndefined()
    expect(change.cells[1].state).toBe('confirmed')
    expect(change.cells[1].value).toBe(2)
    expect(change.cells[2].state).toBe('confirmed')
    expect(change.cells[2].value).toBe(4)
  })

  test('Rejected state switches to confirmed after reveal', () => {
    const initial = {
      size: 2,
      cells: {
        0: {
          prefilled: false,
          value: 2,
          answer: 1,
          state: 'rejected',
        },
        1: {
          prefilled: false,
          value: 3,
          answer: 2,
          state: 'rejected',
        },
      },
    }

    const expected = {
      cells: {
        0: { value: 1, state: 'confirmed' },
        1: { value: 2, state: 'confirmed' },
      },
    }
    const change = reveal('board', initial)
    expect(change).toEqual(expected)
  })
})

describe('SUDOKU: CHECK CELLS', () => {
  test('Check cell only checks the selected cell', () => {
    const initial = {
      selected: 1,
      size: 2,
      cells: {
        0: { prefilled: false, value: 2, answer: 1 },
        1: { prefilled: false, value: 3, answer: 2 },
      },
    }

    const expected = {
      cells: {
        1: { state: 'rejected' },
      },
    }
    const change = check('cell', initial)
    expect(change).toEqual(expected)
  })

  test('Check board checks the entire board', () => {
    const initial = {
      selected: 1,
      size: 3,
      cells: {
        0: { prefilled: false, value: 2, answer: 1 },
        1: { prefilled: false, value: 2, answer: 2 },
        2: { prefilled: false, value: 3, answer: 4 },
      },
    }

    const expected = {
      cells: {
        0: { state: 'rejected' },
        1: { state: 'confirmed' },
        2: { state: 'rejected' },
      },
    }
    const change = check('board', initial)
    expect(change).toEqual(expected)
  })

  test('Check does not affect prefilled cells', () => {
    const initial = {
      selected: 1,
      size: 2,
      cells: {
        0: { prefilled: true, value: 2, answer: 1 },
        1: { prefilled: false, value: 2, answer: 2 },
      },
    }

    const expected = {
      cells: {
        1: { state: 'confirmed' },
      },
    }
    const change = check('board', initial)
    expect(change).toEqual(expected)
  })
})

describe('SUDOKU: HINT', () => {
  test('refocuses to next cell in hint list', () => {
    const initial = {
      selected: 1,
      size: 2,
      hints: [0, 1],
      cells: {
        0: { value: 0, answer: 1 },
        1: { value: 0, answer: 2 },
      },
    }
    const expected = { selected: 0 }
    const change = showHint('', initial)
    expect(change).toEqual(expected)
  })

  test('checks next cell in hint list if value is incorrect', () => {
    const initial = {
      selected: 1,
      size: 2,
      hints: [0, 1],
      cells: {
        0: { value: 3, answer: 1 },
        1: { value: 0, answer: 2 },
      },
    }
    const expected = { 0: { state: 'rejected' } }
    const change = showHint('', initial)
    expect(change.cells).toEqual(expected)
    expect(change.selected).toBe(0)
  })

  test('skips over next cell in hint list if value is correct', () => {
    const initial = {
      selected: 1,
      size: 2,
      hints: [0, 1],
      cells: {
        0: { value: 1, answer: 1 },
        1: { value: 0, answer: 2 },
      },
    }
    const expected = { selected: 1 }
    const change = showHint('', initial)
    expect(change).toEqual(expected)
  })
})

describe('SUDOKU: DEBUG FILL', () => {
  test('reveals all but first cell and selects it', () => {
    const initial = {
      size: 4,
      cells: {
        0: { prefilled: true, value: 1 },
        1: { prefilled: false, value: 9, answer: 1 },
        2: { prefilled: false, answer: 2 },
        3: { prefilled: false, answer: 3 },
      },
    }

    const expected = {
      selected: 1,
      cells: {
        1: { value: undefined, state: 'guessed' },
        2: { value: 2, state: 'guessed' },
        3: { value: 3, state: 'guessed' },
      },
    }
    const change = debugFill(undefined, initial)
    expect(change).toEqual(expected)
  })
})
