import React from 'react'
import { useDispatch } from 'react-redux'

import { addToCompletedDifficulties } from '../../store/slices/gameStatus'
import { saveDifficulty } from '../../helpers/persist'
import useTimer from '../useTimer/useTimer'

import view from './view'
import reducer from './reducer'
import createInitialState from './createInitialState'
import * as actions from './actions'

const useGame = (puzzle, inState, settings) => {
  /**
   * Do not use yesterdays state
   */
  let initialState = inState
  if (initialState && initialState.id !== puzzle.print_date) {
    initialState = undefined
  }

  const stateRef = React.useRef()
  const viewRef = React.useRef()
  const uniqueDispatch = useDispatch()

  let init
  if (!stateRef.current) {
    init = initialState || createInitialState(puzzle)
    init.auto = settings.get('autoCandidateMode')
  }

  const [state, dispatch] = React.useReducer(reducer, init)
  const timer = useTimer(state.initialTime, state.difficulty)

  const wasSolved = viewRef.current ? viewRef.current.solved : true // when game is just created act as if solved so extra sound and modal is not triggered

  stateRef.current = state
  viewRef.current = view(state, settings, wasSolved)

  // NOTE:
  // In order to
  // 1. play the sound only when you first solve
  // 2. ensure the browser links the sound to the user interaction
  // This seems like the easiest place to trigger the sound (future thought on this would be good)
  if (viewRef.current.justSolved) {
    uniqueDispatch(
      addToCompletedDifficulties(viewRef.current.difficulty),
    )
    saveDifficulty(
      'completedDifficulties',
      viewRef.current.difficulty,
      false,
      true,
    )
    timer.pause()

    if (settings.get('sound')) {
      document.getElementById('strut').play()
    }
  }

  const act = React.useRef()

  if (!act.current) {
    act.current = (action, payload) => {
      if (action === 'reset') {
        timer.reset()

        const reset = createInitialState(puzzle)
        reset.auto = settings.get('autoCandidateMode')

        return dispatch({ reset })
      }

      if (viewRef.current.solved || timer.isPaused) return

      dispatch(
        actions[action](
          payload,
          stateRef.current,
          viewRef.current,
          settings,
        ),
      )
    }
  }

  /**
   * NOTE:
   * `act` and `view` are similar to `dispatch` and `state` in redux.
   * `act` is a function that takes a payload and the current state and returns a `change`
   * `view` is a function that translates the sate into a view that is optimized for rendering UI.
   *
   * It is crucial that the act function...
   *   A) is only created once
   *   B) always has access to the current state
   *
   * this is why useRef() is used for creating `act` and accessing state
   */
  return {
    act: act.current,
    view: viewRef.current,
    state: stateRef.current,
    timer,
  }
}

export default useGame
