import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import useTimer from '../useTimer/useTimer'

import useGame from './useGame'

jest.mock('./actions', () => ({
  change: (delta, state) => {
    return {
      value: state.value + delta,
    }
  },
}))

jest.mock('./view', () => (state) => {
  return {
    viewValue: state.value,
  }
})

jest.mock('./reducer', () => (state, change) => {
  if (change.reset) return change.reset

  return {
    ...state,
    ...change,
  }
})

jest.mock('./createInitialState', () => (puzzle) => {
  return {
    id: puzzle.id,
    value: puzzle.value * 2,
  }
})

jest.mock('../useTimer/useTimer', () => jest.fn())

jest.mock('../../store/slices/gameStatus', () => ({
  addToCompletedDifficulties: jest.fn(),
  removeFromCompletedDifficulties: jest.fn(),
  selectCompletedDifficulties: jest.fn(),
}))

describe('SUDOKU: USE GAME', () => {
  let timer
  let settings
  const makeStore = () => ({
    getState: () => ({}),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  })

  const store = makeStore()

  beforeEach(() => {
    timer = {
      time: 0,
      reset: jest.fn(),
    }
    useTimer.mockReset()
    useTimer.mockReturnValue(timer)

    settings = {
      get: jest.fn(),
    }
  })

  test('combines state mgmt correctly', () => {
    const count = jest.fn()

    const MyComponent = () => {
      const { act, view } = useGame({ value: 2 }, undefined, settings)
      const refAct = React.useRef(act)
      // use effect hook to verify if act is only created once
      // using a ref to act ensurse act has access to fresh state
      React.useEffect(() => {
        count()
      }, [act])
      return (
        <div>
          <span id="current">{view.viewValue}</span>
          <button
            id="plus-2"
            onClick={() => refAct.current('change', 2)}
          />
          <button
            id="minus-3"
            onClick={() => refAct.current('change', -3)}
          />
        </div>
      )
    }

    const c = mount(
      <Provider store={store}>
        <MyComponent />
      </Provider>,
    )

    expect(c.text()).toBe('4')
    c.find('#plus-2').simulate('click')
    expect(c.text()).toBe('6')
    c.find('#minus-3').simulate('click')
    expect(c.text()).toBe('3')

    expect(count).toHaveBeenCalledTimes(1) // ensure act function is identical
  })

  test('supports initial state', () => {
    let result
    const MyComponent = () => {
      result = useGame(
        { value: 1, print_date: 2 },
        { value: 5, id: 2 },
        settings,
      )
      return null
    }
    mount(
      <Provider store={store}>
        <MyComponent />
      </Provider>,
    )

    expect(result.state.value).toEqual(5)
  })

  test('ensures initial state is current puzzle', () => {
    let result
    const MyComponent = () => {
      result = useGame(
        { value: 1, print_date: 1 },
        { value: 3, id: 2 },
        settings,
      )
      return null
    }

    mount(
      <Provider store={store}>
        <MyComponent />
      </Provider>,
    )
    expect(result.state.value).toEqual(2)
  })

  test('exposes timer and state', () => {
    let result
    const MyComponent = () => {
      result = useGame({ value: 2 }, undefined, settings)
      return null
    }

    mount(
      <Provider store={store}>
        <MyComponent />
      </Provider>,
    )

    expect(result.state).toEqual({ value: 4 })
    expect(result.timer.time).toBe(0)
  })

  test('resets timer and returns initial state on reset', () => {
    let timer
    const MyComponent = () => {
      const game = useGame({ value: 2 }, undefined, settings)
      timer = game.timer
      return (
        <div>
          <span id="current">{game.view.viewValue}</span>
          <button id="plus-1" onClick={() => game.act('change', 1)} />
          <button id="reset" onClick={() => game.act('reset')} />
        </div>
      )
    }

    const c = mount(
      <Provider store={store}>
        <MyComponent />
      </Provider>,
    )

    expect(c.text()).toBe('4')
    c.find('#plus-1').simulate('click')
    expect(c.text()).toBe('5')

    expect(timer.reset).not.toHaveBeenCalled()

    c.find('#reset').simulate('click')
    expect(c.text()).toBe('4')

    expect(timer.reset).toHaveBeenCalledTimes(1)
  })
})
