import createView from './view'

describe('SUDOKU: USE GAME / VIEW', () => {
  let state
  let settings
  let storedSettings

  beforeEach(() => {
    storedSettings = { highlightConflicts: true }
    settings = { get: (k) => storedSettings[k] }
  })

  test('Filled set to true when board is filled', () => {
    state = {
      size: 3,
      cells: {
        0: { value: 1 },
        1: { value: 1, prefilled: true },
        2: { value: 1 },
      },
    }

    const final = createView(state, settings)

    expect(final.filled).toEqual(true)
  })

  test('Filled set to false when cells have missing values', () => {
    state = {
      size: 3,
      cells: {
        0: { value: 1 },
        1: { value: 1, prefilled: true },
        2: { value: undefined },
      },
    }

    const final = createView(state, settings)

    expect(final.filled).toEqual(false)
  })

  test('Solved set to true when cell values match answers', () => {
    state = {
      size: 3,
      cells: {
        0: { value: 1, answer: 1 },
        1: { value: 1, answer: 1 },
        2: { value: 2, answer: 2 },
      },
    }

    const final = createView(state, settings)

    expect(final.solved).toEqual(true)
  })

  test('Solved set to false when cell values do not match answers', () => {
    state = {
      size: 3,
      cells: {
        0: { value: 1, answer: 1 },
        1: { value: 5, answer: 2 },
        2: { value: 3, answer: 3 },
      },
    }

    const final = createView(state, settings)

    expect(final.solved).toEqual(false)
  })

  test('Just solved set to true if not previously solved', () => {
    state = {
      size: 3,
      cells: {
        0: { value: 1, answer: 1 },
        1: { value: 5, answer: 5 },
        2: { value: 3, answer: 3 },
      },
    }

    const final = createView(state, settings, false)

    expect(final.justSolved).toEqual(true)
  })

  test('Just solved set to false if previously solved', () => {
    state = {
      size: 3,
      cells: {
        0: { value: 1, answer: 1 },
        1: { value: 5, answer: 5 },
        2: { value: 3, answer: 3 },
      },
    }

    const final = createView(state, settings, true)

    expect(final.justSolved).toEqual(false)
  })

  test('Extra properties of cells are preserved in the view object', () => {
    state = {
      size: 1,
      cells: {
        0: { foo: 0, value: 1, answer: 0 },
      },
    }

    const final = createView(state, settings)

    expect(final.cells[0].foo).toEqual(0)
  })

  test('Conflicting cells in row are detected', () => {
    state = { size: 81, cells: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.cells[0].value = 1

    const noConflict = createView(state, settings)

    expect(noConflict.cells[0].conflicted).toBeUndefined()

    state.cells[8].value = 1

    const conflict = createView(state, settings)

    expect(conflict.cells[0].conflicted).toBe(true)
    expect(conflict.cells[8].conflicted).toBe(true)

    // disable highlight conflicts
    storedSettings.highlightConflicts = false

    const noHighlight = createView(state, settings)

    expect(noHighlight.cells[0].conflicted).toBeUndefined()
    expect(noHighlight.cells[0].conflicted).toBeUndefined()
  })

  test('Conflicting cells in column are detected', () => {
    state = { size: 81, cells: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.cells[0].value = 1

    const noConflict = createView(state, settings)

    expect(noConflict.cells[0].conflicted).toBeUndefined()

    state.cells[72].value = 1

    const conflict = createView(state, settings)

    expect(conflict.cells[0].conflicted).toBe(true)
    expect(conflict.cells[72].conflicted).toBe(true)

    // disable highlight conflicts
    storedSettings.highlightConflicts = false

    const noHighlight = createView(state, settings)

    expect(noHighlight.cells[0].conflicted).toBeUndefined()
    expect(noHighlight.cells[72].conflicted).toBeUndefined()
  })

  test('Conflicting cells in region are detected', () => {
    state = { size: 81, cells: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.cells[0].value = 1

    const noConflict = createView(state, settings)

    expect(noConflict.cells[0].conflicted).toBeUndefined()

    state.cells[20].value = 1

    const conflict = createView(state, settings)

    expect(conflict.cells[0].conflicted).toBe(true)
    expect(conflict.cells[20].conflicted).toBe(true)

    // disable highlight conflicts
    storedSettings.highlightConflicts = false

    const noHighlight = createView(state, settings)

    expect(noHighlight.cells[0].conflicted).toBeUndefined()
    expect(noHighlight.cells[20].conflicted).toBeUndefined()
  })

  test('Possible candidates have correct values', () => {
    state = { size: 81, cells: {}, autoCandidates: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.auto = true

    const view = createView(state, settings)
    const expected = {
      1: true,
      2: true,
      3: true,
      4: true,
      5: true,
      6: true,
      7: true,
      8: true,
      9: true,
    }

    expect(view.candidates[0]).toEqual(expected)
  })

  test('Possible candidates adapt to values in row', () => {
    state = { size: 81, cells: {}, autoCandidates: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.auto = true
    state.cells[8].value = 1

    const view = createView(state, settings)

    expect(view.candidates[0][1]).toBe(false)
  })

  test('Possible candidates adapt to values in column', () => {
    state = { size: 81, cells: {}, autoCandidates: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.auto = true
    state.cells[72].value = 1

    const view = createView(state, settings)

    expect(view.candidates[0][1]).toBe(false)
  })

  test('Possible candidates adapt to values in region', () => {
    state = { size: 81, cells: {}, autoCandidates: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.auto = true
    state.cells[20].value = 1

    const view = createView(state, settings)

    expect(view.candidates[0][1]).toBe(false)
  })

  test('Disabling conflict highlighting does not break auto', () => {
    state = { size: 81, cells: {}, autoCandidates: {} }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.auto = true
    state.cells[20].value = 1

    storedSettings.highlightConflicts = false

    expect(() => {
      const view = createView(state, settings)

      expect(view.candidates[0][1]).toBe(false)
    }).not.toThrow()
  })

  test('Auto mode is set appropriately in the view object', () => {
    state = { auto: true }

    let view = createView(state, settings)

    expect(view.auto).toBe(true)

    state.auto = false
    view = createView(state, settings)

    expect(view.auto).toBe(false)
  })

  test('Only manual candidates are rendered and modified when auto mode is disabled', () => {
    state = { size: 81, cells: {}, auto: false }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.manualCandidates = {}
    state.autoCandidates = { 0: { 2: true } }

    let view = createView(state, settings)

    expect(view.candidates[0]).toEqual({})

    state.manualCandidates = { 0: { 1: true } }
    view = createView(state, settings)

    expect(view.candidates[0][1]).toBe(true)
    expect(view.candidates[0][2]).toBeUndefined()
  })

  test('Only auto candidates are rendered and modified when auto mode is enabled', () => {
    state = { size: 81, cells: {}, auto: true }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.autoCandidates = {
      1: {
        1: true,
        2: false,
        3: false,
        4: false,
        5: false,
        6: false,
        7: false,
        8: false,
        9: false,
      },
    }

    state.manualCandidates = { 1: { 2: true } }

    let view = createView(state, settings)

    expect(view.candidates[1][1]).toBe(true)
    expect(view.candidates[1][2]).toBe(false)

    state.autoCandidates[1][2] = true
    state.manualCandidates[1][2] = false

    view = createView(state, settings)

    expect(view.candidates[1][2]).toBe(true)
  })

  test('Pending auto changes (conflicting candidates) are updated', () => {
    state = { size: 81, cells: {}, auto: true }

    for (let i = 0; i < state.size; i += 1) {
      state.cells[i] = { id: i, value: 0 }
    }

    state.autoCandidates = {
      1: { 1: true, 2: true },
      2: { 1: false, 2: true },
    }

    state.cells[0].value = 1

    const view = createView(state, settings)

    expect(view.candidates[1]).toBeDefined() // trigger getter
    expect(view.candidates[2]).toBeDefined() // trigger getter
    expect(view.candidates[2][1]).toBe(false)
    expect(view.pendingAutoChanges).toEqual({
      1: { 1: undefined, 2: undefined },
      2: { 2: undefined },
    })
  })
})
