import React from 'react'
import { mount } from 'enzyme'

import { trackEvent } from '../../helpers/sudokuTracker'

import useKeyboard from './useKeyboard'

jest.mock('../../helpers/sudokuTracker', () => ({
  trackEvent: jest.fn(),
}))

describe('SUDOKU: USE KEYBOARD', () => {
  let MyComponent
  let component
  let act
  let state

  beforeEach(() => {
    act = jest.fn()
    state = {
      difficulty: 'medium',
      candidateMode: false,
    }

    MyComponent = function MyComponent() {
      useKeyboard(act, state)
      return <div />
    }

    component = mount(React.createElement(MyComponent))
  })

  afterEach(() => {
    component.unmount()
  })

  for (let i = 0; i < 10; i += 1) {
    test(`guesses: ${i}`, () => {
      const evt = new window.KeyboardEvent('keydown', {
        keyCode: i + 48,
      })
      jest.spyOn(evt, 'preventDefault')
      document.dispatchEvent(evt)

      if (i > 0 && i < 10) {
        expect(act).toHaveBeenCalledWith('guess', i)
      } else {
        expect(act).not.toHaveBeenCalled()
      }

      expect(evt.preventDefault).not.toHaveBeenCalled()
    })

    test(`guesses numpad: ${i}`, () => {
      const evt = new window.KeyboardEvent('keydown', {
        keyCode: i + 96,
      })
      jest.spyOn(evt, 'preventDefault')
      document.dispatchEvent(evt)

      if (i > 0 && i < 10) {
        expect(act).toHaveBeenCalledWith('guess', i)
      } else {
        expect(act).not.toHaveBeenCalled()
      }

      expect(evt.preventDefault).not.toHaveBeenCalled()
    })
  }

  Object.entries({
    up: 38,
    down: 40,
    left: 37,
    right: 39,
  }).forEach(([direction, keyCode]) => {
    test(`moves ${direction}`, () => {
      const evt = new window.KeyboardEvent('keydown', {
        keyCode,
      })
      jest.spyOn(evt, 'preventDefault')
      document.dispatchEvent(evt)
      expect(act).toHaveBeenCalledWith('move', direction)
      expect(evt.preventDefault).toHaveBeenCalled()
    })
  })

  Object.entries({
    delete: 46,
    backspace: 8,
  }).forEach(([key, keyCode]) => {
    test(`clears on ${key}`, () => {
      const evt = new window.KeyboardEvent('keydown', {
        keyCode,
      })
      jest.spyOn(evt, 'preventDefault')
      document.dispatchEvent(evt)
      expect(act).toHaveBeenCalledWith('guess', undefined)
      expect(evt.preventDefault).toHaveBeenCalled()
    })
  })

  test('switches mode on space bar', () => {
    const down = new window.KeyboardEvent('keydown', {
      keyCode: 32,
    })

    const up = new window.KeyboardEvent('keyup', {
      keyCode: 32,
    })
    jest.spyOn(down, 'preventDefault')
    document.dispatchEvent(down)
    document.dispatchEvent(up)
    expect(act).toHaveBeenCalledWith('toggle', 'candidateMode')
    expect(act).toHaveBeenCalledTimes(1)
    expect(down.preventDefault).toHaveBeenCalled()
    expect(trackEvent).toHaveBeenCalledWith(
      'medium',
      'candidate-mode',
    )
  })

  Object.entries({
    alt: 18,
    shift: 16,
  }).forEach(([key, keyCode]) => {
    test(`temporary toggle on ${key}`, () => {
      const down = new window.KeyboardEvent('keydown', {
        keyCode,
      })

      const downRepeat = new window.KeyboardEvent('keydown', {
        keyCode,
        repeat: true,
      })

      const up = new window.KeyboardEvent('keyup', {
        keyCode,
      })
      jest.spyOn(down, 'preventDefault')
      document.dispatchEvent(down)
      document.dispatchEvent(downRepeat)

      expect(act).toHaveBeenCalledWith('toggle', 'candidateMode')
      expect(act).toHaveBeenCalledTimes(1)
      expect(down.preventDefault).toHaveBeenCalled()
      expect(trackEvent).toHaveBeenCalledWith(
        'medium',
        'candidate-mode',
      )

      act.mockReset()

      document.dispatchEvent(up)
      expect(act).toHaveBeenCalledWith('toggle', 'candidateMode')
    })
  })

  test('cleans up', () => {
    const act = jest.fn()
    const state = {
      difficulty: 'medium',
      candidateMode: false,
    }

    const MyComponent = () => {
      useKeyboard(act, state)
      return <div />
    }

    const c = mount(<MyComponent />)

    const down1 = new window.KeyboardEvent('keydown', {
      keyCode: 16,
    })

    const up1 = new window.KeyboardEvent('keyup', {
      keyCode: 16,
    })
    document.dispatchEvent(down1)
    document.dispatchEvent(up1)

    c.unmount()
    const down2 = new window.KeyboardEvent('keydown', {
      keyCode: 16,
    })

    const up2 = new window.KeyboardEvent('keyup', {
      keyCode: 16,
    })
    document.dispatchEvent(down2)
    document.dispatchEvent(up2)

    expect(act).toHaveBeenCalledTimes(2)
  })

  test('cleans up when act changes', () => {
    const act = jest.fn()
    const act2 = jest.fn()
    const state = {
      difficulty: 'medium',
      candidateMode: false,
    }

    /* eslint-disable */
    const MyComponent = ({ act }) => {
      useKeyboard(act, state)
      return <div />
    }
    /* eslint-enable */

    const c = mount(<MyComponent act={act} />)
    c.setProps({ act: act2 })
    const evt = new window.KeyboardEvent('keydown', {
      keyCode: 32,
    })
    document.dispatchEvent(evt)
    expect(act).not.toHaveBeenCalled()
    expect(act2).toHaveBeenCalled()
  })
})
