import React from 'react'
import ReactDOM from 'react-dom'
import { act } from 'react-dom/test-utils'

import { save } from '../../helpers/persist'
import onExit from '../../../helpers/onExit'

import useSave from './useSave'

jest.mock('../../helpers/persist', () => ({
  save: jest.fn(),
}))
jest.mock('../../../helpers/onExit', () => jest.fn())

describe('SUDOKU: USE SAVE', () => {
  let MyComponent
  let container
  let update
  let timer
  let unSubscribeFromExit
  let unSubscribeFromTimer

  beforeEach(() => {
    timer = { subscribe: jest.fn(), time: 100 }

    unSubscribeFromExit = jest.fn()
    unSubscribeFromTimer = jest.fn()

    save.mockReset()
    onExit.mockReset()

    timer.subscribe.mockReturnValue(unSubscribeFromTimer)
    onExit.mockReturnValue(unSubscribeFromExit)

    container = document.createElement('div')
    document.body.appendChild(container)

    MyComponent = function MyComponent() {
      const [game, setGame] = React.useState({
        timer,
        state: { difficulty: 'easy', v: 1 },
      })
      update = setGame
      useSave(game)
      return null
    }
  })

  afterEach(() => {
    ReactDOM.unmountComponentAtNode(container)
    document.body.removeChild(container)
    container = null
  })

  test('saves on state changes', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    expect(save).toHaveBeenCalledWith('easy', {
      v: 1,
      difficulty: 'easy',
      initialTime: 100,
    })

    timer.time = 200
    act(() => {
      update({ timer, state: { difficulty: 'easy', v: 2 } })
    })

    expect(save).toHaveBeenCalledWith('easy', {
      v: 2,
      difficulty: 'easy',
      initialTime: 200,
    })
  })

  test('saves on pause', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    const [listenter] = timer.subscribe.mock.calls[0]

    expect(save).toHaveBeenCalledTimes(1)

    timer.time = 200
    listenter('tick')

    expect(save).toHaveBeenCalledTimes(1)

    timer.time = 300
    listenter('pause')

    expect(save).toHaveBeenCalledTimes(2)
    expect(save).toHaveBeenCalledWith('easy', {
      v: 1,
      difficulty: 'easy',
      initialTime: 300,
    })
  })

  test('saves on exit', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    const [listenter] = onExit.mock.calls[0]

    timer.time = 400

    listenter()

    const isUrgent = true

    expect(save).toHaveBeenCalledWith(
      'easy',
      {
        v: 1,
        difficulty: 'easy',
        initialTime: 400,
      },
      isUrgent,
    )
  })

  test('cleans up when difficulty changes', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    expect(unSubscribeFromExit).not.toHaveBeenCalled()
    expect(unSubscribeFromTimer).not.toHaveBeenCalled()

    act(() => {
      update({
        timer,
        state: { difficulty: 'medium', v: 1 },
      })
    })

    expect(unSubscribeFromExit).toHaveBeenCalledTimes(1)
    expect(unSubscribeFromTimer).toHaveBeenCalledTimes(1)
  })
})
