import React from 'react'
import ReactDOM from 'react-dom'
import { act } from 'react-dom/test-utils'

import { load, save } from '../../helpers/persist'

import useSettings from './useSettings'

jest.mock('../../helpers/persist', () => ({
  save: jest.fn(),
  load: jest.fn(),
}))

describe('SUDOKU: USE SETTINGS', () => {
  let MyComponent
  let container
  let settings
  let storage

  beforeEach(() => {
    storage = {}
    load.mockImplementation((k) => storage[k])
    save.mockImplementation((k, v) => {
      storage[k] = v
    })

    container = document.createElement('div')
    document.body.appendChild(container)

    MyComponent = function MyComponent() {
      settings = useSettings()
      return null
    }
  })

  afterEach(() => {
    ReactDOM.unmountComponentAtNode(container)
    document.body.removeChild(container)
    container = null

    load.mockReset()
    save.mockReset()
  })

  test('loads default settings', () => {
    ReactDOM.render(<MyComponent />, container)

    expect(settings.get('autoCheck')).toBe(false)
    expect(settings.get('autoCandidateMode')).toBe(false)
    expect(settings.get('highlightConflicts')).toBe(true)
    expect(settings.get('sound')).toBe(true)
    expect(settings.get('clock')).toBe(true)
  })

  test('loads initial settings from local storage', () => {
    storage.settings = { toggle: true }

    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    expect(settings.get('toggle')).toBe(true)
  })

  test('toggles settings', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    act(() => {
      settings.toggle('clock', 'medium')
    })

    expect(settings.get('clock')).toBe(false)

    act(() => {
      settings.toggle('clock', 'medium')
    })

    expect(settings.get('clock')).toBe(true)
  })

  test('saves settings initially', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    expect(save).toHaveBeenCalledWith('settings', {
      autoCheck: false,
      autoCandidateMode: false,
      highlightConflicts: true,
      sound: true,
      clock: true,
    })
  })

  test('saves settings on changes', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    save.mockReset()

    act(() => {
      settings.set('clock', false)
    })

    expect(save).toHaveBeenCalledWith('settings', {
      autoCheck: false,
      autoCandidateMode: false,
      highlightConflicts: true,
      sound: true,
      clock: false, // note that clock changes
    })
  })

  test('settings is identical across renders (for memoizing)', () => {
    act(() => {
      ReactDOM.render(<MyComponent />, container)
    })

    const before = settings

    act(() => {
      settings.set('clock', false)
    })

    const after = settings

    expect(before).toBe(after)
  })
})
