import mockDate from 'mockdate'

import { trackEvent } from '../../helpers/sudokuTracker'
import { close, open } from '../../store/slices/modalSystem'

import useTick from './useTick'
import createTimer from './createTimer'

jest.mock('./formatTime', () => (t) => `formatTime(${t})`)
jest.mock('./useTick', () => jest.fn())
jest.mock('../../helpers/sudokuTracker', () => ({
  trackEvent: jest.fn(),
}))
jest.mock('../../store/slices/modalSystem', () => ({
  close: jest.fn(),
  open: jest.fn(),
}))

describe('SUDOKU - CREATE TIMER', () => {
  let timer
  let currentTime
  let addOneSecond

  const mockDispatch = jest.fn()

  beforeEach(() => {
    useTick.mockImplementation((timer) => ({ timer }))

    currentTime = 0
    jest.useFakeTimers()
    timer = createTimer(0, 'easy', mockDispatch)
    mockDate.set(currentTime)

    addOneSecond = () => {
      currentTime += 1000
      mockDate.set(currentTime)
      jest.advanceTimersByTime(1000)
    }
  })

  afterEach(() => {
    mockDate.reset()
    jest.resetAllMocks()
  })

  test('getTime', () => {
    addOneSecond()

    expect(timer.time).toBe(0)
    expect(timer.timeFormatted).toBe(`formatTime(0)`)

    timer.resume()

    addOneSecond()

    expect(timer.time).toBe(1000)
    expect(timer.timeFormatted).toBe(`formatTime(1)`)
  })

  test('control', () => {
    expect(timer.isPaused).toBe(true)

    timer.resume()
    addOneSecond()
    expect(timer.isPaused).toBe(false)
    expect(timer.time).toBe(1000)

    timer.pause()
    addOneSecond()
    expect(timer.isPaused).toBe(true)
    expect(timer.time).toBe(1000)

    timer.resume()
    addOneSecond()
    expect(timer.isPaused).toBe(false)
    expect(timer.time).toBe(2000)

    timer.toggle()
    addOneSecond()
    expect(timer.isPaused).toBe(true)
    expect(timer.time).toBe(2000)

    timer.toggle()
    addOneSecond()
    expect(timer.isPaused).toBe(false)
    expect(timer.time).toBe(3000)

    timer.reset()
    expect(timer.isPaused).toBe(false)
    expect(timer.time).toBe(0)
  })

  test('subscribe', () => {
    const foo = jest.fn()
    const bar = jest.fn()
    const cleanupFoo = timer.subscribe(foo)
    const cleanupBar = timer.subscribe(bar)

    timer.resume()
    timer.resume() // test double call
    addOneSecond()

    timer.pause()
    addOneSecond()

    addOneSecond()

    timer.resume()
    addOneSecond()

    cleanupFoo()

    addOneSecond()

    addOneSecond()

    cleanupBar()
    cleanupBar() // shouldnt throw

    addOneSecond()

    expect(foo.mock.calls).toEqual([
      ['resume', 0],
      ['tick', 1000],
      ['pause', 1000],
      ['resume', 1000],
      ['tick', 2000],
    ])

    expect(bar.mock.calls).toEqual([
      ['resume', 0],
      ['tick', 1000],
      ['pause', 1000],
      ['resume', 1000],
      ['tick', 2000],
      ['tick', 3000],
      ['tick', 4000],
    ])
  })

  test('paused is always accurate', () => {
    expect.assertions(2)

    timer.subscribe((evt) => {
      if (evt === 'pause') expect(timer.isPaused).toBe(true)

      if (evt === 'resume') {
        expect(timer.isPaused).toBe(false)
      }
    })

    timer.resume()
    timer.pause()
  })

  test('exposes useTick', () => {
    const tick = timer.useTick()

    expect(useTick).toHaveBeenCalledWith(timer)
    expect(tick.timer).toBe(timer)
  })

  test('warn on possible leak', () => {
    const spy = jest.spyOn(console, 'warn')
    const foo = jest.fn()

    spy.mockImplementation(() => {})

    for (let i = 0; i < 11; i += 1) {
      timer.subscribe(foo)
    }

    expect(spy).toHaveBeenCalledWith(
      'POSSIBLE GAME TIMER SUBSCRIPTION LEAK',
    )
    spy.mockRestore()
  })

  test('tracking', () => {
    timer.resume()
    expect(trackEvent).not.toHaveBeenCalled()

    timer.toggle()
    expect(trackEvent).toHaveBeenCalledTimes(1)
    expect(trackEvent).toHaveBeenCalledWith('easy', 'stop-timer')

    timer.toggle()
    expect(trackEvent).toHaveBeenCalledTimes(2)
    expect(trackEvent).toHaveBeenCalledWith('easy', 'start-timer')
  })

  test('calls modal system', () => {
    expect(mockDispatch).not.toHaveBeenCalled()

    timer.resume()
    expect(mockDispatch).toHaveBeenCalledTimes(1)
    expect(mockDispatch).toHaveBeenLastCalledWith(close())

    timer.toggle()
    expect(mockDispatch).toHaveBeenCalledTimes(2)
    expect(mockDispatch).toHaveBeenLastCalledWith(open())

    timer.resume()
    expect(mockDispatch).toHaveBeenCalledTimes(3)
    expect(mockDispatch).toHaveBeenLastCalledWith(close())
  })
})
