import { deepCloneSimple } from '../../helpers/gameUtils'

export const createBoard = (
  { zLayer, layers, infiniteMode = false, boardSize = 30 },
  mockBoard = undefined,
) => {
  if (mockBoard) {
    return JSON.parse(mockBoard)
  }

  const zLayerIds = [...zLayer]
  const layersIds = [...layers]

  const board = []
  const layersPool = [...Array(layersIds.length)].map(() => [])

  const randomIndex = (arr) => Math.floor(Math.random() * arr.length)

  const removeRandom = (arr) => arr.splice(randomIndex(arr), 1)[0]

  const getRandomLayers = (layersPool) =>
    layersPool.map((layer) => ({
      id: removeRandom(layer),
      hidden: true,
    }))

  // If not in infinite mode, we have to make sure that there are at least two
  // of each randomly selected layer so that clearing the board is possible
  const tileCount = infiniteMode ? boardSize : boardSize / 2

  for (let i = 0; i < tileCount; i += 1) {
    layersIds.forEach((layerIdsArr, ind) => {
      layersPool[ind].push(layerIdsArr[randomIndex(layerIdsArr)])
    })
  }

  if (!infiniteMode) {
    layersPool.forEach((layerPool, ind) => {
      layersPool[ind] = layerPool.concat(layerPool)
    })
  }

  // The zLayer only needs two randomly selected svgs for the
  // checkerboard effect
  const zPool =
    zLayerIds.length <= 2
      ? zLayerIds
      : [zLayerIds.shift(), removeRandom(zLayerIds)]

  for (let i = 0; i < boardSize; i += 1) {
    board.push({
      id: i,
      zLayer: { id: zPool[i % 2], hidden: false },
      layers: getRandomLayers(layersPool),
    })
  }

  return board
}

export const getMatchingLayers = (tileOne, tileTwo) => {
  const matchingLayers = []

  tileOne.layers.forEach((layer, idx) => {
    const tileTwoLayer = tileTwo.layers[idx]

    if (
      layer.id === tileTwoLayer.id &&
      !layer.hidden &&
      !tileTwoLayer.hidden
    ) {
      matchingLayers.push(idx)
    }
  })

  return matchingLayers.length === 0 ? null : matchingLayers
}

export const revealAllLayers = (board) => {
  const newBoard = deepCloneSimple(board)

  newBoard.forEach((tile) => {
    tile.layers.forEach((layer) => {
      layer.hidden = false
    })
  })

  return newBoard
}

export const hideLayers = (
  board,
  { tileOneId, tileTwoId, matchingLayers },
) => {
  const newBoard = deepCloneSimple(board)

  matchingLayers.forEach((layer) => {
    newBoard[tileOneId].layers[layer].hidden = true
    newBoard[tileTwoId].layers[layer].hidden = true
  })

  return newBoard
}

export const isAboutToBeClear = (tile, matchingLayers) =>
  tile.layers.every(
    (layer, idx) => layer.hidden || matchingLayers.includes(idx),
  )

export const createNewTile = (layersIds) =>
  layersIds.map((layerArr) => ({
    id: layerArr[Math.floor(Math.random() * layerArr.length)],
    hidden: false,
  }))

export const generateNewLayers = (layerIds, board) => {
  const newLayerIds = [...Array(layerIds.length)].map(() => [])

  // go through palette layers
  layerIds.forEach((layersArr, index) =>
    layersArr.forEach((layer) => {
      let count = 0

      // and find out how many times each layer appears on board
      board.forEach((tile) => {
        tile.layers.forEach((l) => {
          if (l.id === layer) count += 1
        })
      })

      // if count is odd put in new layers arr
      if (count % 2 !== 0) {
        newLayerIds[index].push(layer)
      }
    }),
  )

  // if type of layer (e.g. aLayer) has no layer IDs with an odd number of repetitions
  // then just use whole array of layer IDs from palette
  newLayerIds.forEach((newLayers, ind) => {
    if (newLayers.length === 0) {
      newLayerIds[ind] = layerIds[ind]
    }
  })

  // then randomly use one from each layers arr to construct new tile
  return createNewTile(newLayerIds)
}

export const replaceTileLayers = (board, tileId, newTile) => {
  const newBoard = deepCloneSimple(board)

  newBoard[tileId].layers = newTile

  return newBoard
}

export const isThereAMatch = (tileOne, tileTwo) => {
  for (let i = 0; i < tileOne.layers.length; i += 1) {
    const layerOne = tileOne.layers[i]
    const layerTwo = tileTwo.layers[i]

    if (
      layerOne.id === layerTwo.id &&
      !layerOne.hidden &&
      !layerTwo.hidden
    ) {
      return true
    }
  }

  return false
}

export const isMoveAvailable = (board, tile) => {
  for (let i = tile.id + 1; i < board.length; i += 1) {
    const currTile = board[i]

    if (isThereAMatch(tile, currTile)) return true
  }

  return false
}

export const areThereAnyValidMovesOnBoard = (board) =>
  board.some((el) => isMoveAvailable(board, el))

export const animationDelay = (id) => `${id * 0.05 + 1}s`

export const transitionDelay = (id) =>
  id === 0 ? '0.05s' : `${id * 0.05 + 0.05}s`

export const checkWinState = (action) => {
  const { infiniteMode, board, matchedLayers } = action.state
  const { matchingLayers } = action
  const tileLayers = board[0].layers.length

  return (
    !infiniteMode &&
    (board.length / 2) * tileLayers ===
      matchedLayers + matchingLayers.length
  )
}
