import {
  createBoard,
  hideLayers,
  createNewTile,
  isThereAMatch,
  isMoveAvailable,
  isAboutToBeClear,
  replaceTileLayers,
  getMatchingLayers,
  areThereAnyValidMovesOnBoard,
  checkWinState,
  revealAllLayers,
} from './helpers'

describe('TILES: HELPER FUNCTIONS', () => {
  it('should return a static board if local storage object is set', () => {
    expect(createBoard({}, '[1, 2, 3]')).toEqual([1, 2, 3])
  })

  it('should get the matching layers excluding the zLayer', () => {
    const tileOne = {
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A1', hidden: false },
        { id: 'B2', hidden: false },
        { id: 'C3', hidden: false },
      ],
    }

    const tileTwo = {
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A1', hidden: true },
        { id: 'B2', hidden: false },
        { id: 'C13', hidden: false },
      ],
    }
    expect(getMatchingLayers(tileOne, tileTwo)).toEqual([1])
  })

  it('should return null if there are no matching layers', () => {
    const tileOne = {
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A1', hidden: false },
        { id: 'B2', hidden: false },
        { id: 'C3', hidden: false },
      ],
    }

    const tileTwo = {
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A11', hidden: false },
        { id: 'B12', hidden: false },
        { id: 'C13', hidden: false },
      ],
    }
    expect(getMatchingLayers(tileOne, tileTwo)).toEqual(null)
  })

  it('should hide tile layers on the board', () => {
    const board = [
      {
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: false },
          { id: 'C3', hidden: false },
        ],
      },
      {
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: false },
          { id: 'C13', hidden: false },
        ],
      },
    ]

    const newBoard = [
      {
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: true },
          { id: 'C3', hidden: false },
        ],
      },
      {
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C13', hidden: false },
        ],
      },
    ]

    const action = {
      tileOneId: 0,
      tileTwoId: 1,
      matchingLayers: [1],
    }
    expect(hideLayers(board, action)).toEqual(newBoard)
    expect(board[0].layers[1].hidden).toBe(false)
  })

  it('should indicate if all tile layers will be hidden', () => {
    expect.assertions(2)

    const tile = {
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A1', hidden: true },
        { id: 'B2', hidden: true },
        { id: 'C3', hidden: false },
      ],
    }

    let matchingLayers = [2]
    expect(isAboutToBeClear(tile, matchingLayers)).toBe(true)

    matchingLayers = []
    expect(isAboutToBeClear(tile, matchingLayers)).toBe(false)
  })

  it('should create a new tile', () => {
    const layers = [
      ['A1', 'A2'],
      ['B1', 'B2'],
      ['C1', 'C2'],
    ]
    const flatTiles = ['A1', 'A2', 'B1', 'B2', 'C1', 'C2']
    const newTileLayers = createNewTile(layers)

    newTileLayers.forEach((layer) => {
      expect(flatTiles.includes(layer.id)).toBe(true)
      expect(layer.hidden).toBe(false)
    })
  })

  it('should replace a tile on the board', () => {
    const board = [
      {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: false },
          { id: 'C3', hidden: false },
        ],
      },
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: false },
          { id: 'C13', hidden: false },
        ],
      },
    ]

    const newBoard = [
      {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: false },
          { id: 'C3', hidden: false },
        ],
      },
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A10', hidden: false },
          { id: 'B12', hidden: false },
          { id: 'C23', hidden: false },
        ],
      },
    ]

    const newTile = [
      { id: 'A10', hidden: false },
      { id: 'B12', hidden: false },
      { id: 'C23', hidden: false },
    ]
    expect(replaceTileLayers(board, 1, newTile)).toEqual(newBoard)
  })

  it('should check for a match', () => {
    expect.assertions(2)
    const tileOne = {
      id: 0,
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A1', hidden: false },
        { id: 'B2', hidden: false },
        { id: 'C3', hidden: false },
      ],
    }

    const tileTwo = {
      id: 0,
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A1', hidden: false },
        { id: 'B2', hidden: true },
        { id: 'C13', hidden: false },
      ],
    }

    const tileThree = {
      id: 0,
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A10', hidden: false },
        { id: 'B21', hidden: false },
        { id: 'C31', hidden: false },
      ],
    }
    expect(isThereAMatch(tileOne, tileTwo)).toBe(true)
    expect(isThereAMatch(tileOne, tileThree)).toBe(false)
  })

  it('should check if a move is available for given tile', () => {
    expect.assertions(2)
    const tile = {
      id: 0,
      zLayer: { id: 'Z1', hidden: false },
      layers: [
        { id: 'A1', hidden: false },
        { id: 'B2', hidden: false },
        { id: 'C3', hidden: false },
      ],
    }

    const boardWithMatch = [
      tile,
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B12', hidden: false },
          { id: 'C3', hidden: true },
        ],
      },
      {
        id: 2,
        zLayer: { id: 'Z2', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C13', hidden: false },
        ],
      },
    ]

    const boardWithoutMatch = [
      tile,
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A11', hidden: false },
          { id: 'B21', hidden: false },
          { id: 'C31', hidden: false },
        ],
      },
      {
        id: 2,
        zLayer: { id: 'Z2', hidden: false },
        layers: [
          { id: 'A11', hidden: true },
          { id: 'B12', hidden: false },
          { id: 'C13', hidden: false },
        ],
      },
    ]
    expect(isMoveAvailable(boardWithMatch, tile)).toBe(true)
    expect(isMoveAvailable(boardWithoutMatch, tile)).toBe(false)
  })
  it('should check if there are any valid moves on the board', () => {
    expect.assertions(2)
    const boardWithMoves = [
      {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: false },
          { id: 'C3', hidden: false },
        ],
      },
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B12', hidden: false },
          { id: 'C3', hidden: true },
        ],
      },
      {
        id: 2,
        zLayer: { id: 'Z2', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C13', hidden: false },
        ],
      },
    ]

    const boardWithoutMoves = [
      {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: false },
          { id: 'C3', hidden: false },
        ],
      },
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A11', hidden: false },
          { id: 'B21', hidden: false },
          { id: 'C31', hidden: false },
        ],
      },
      {
        id: 2,
        zLayer: { id: 'Z2', hidden: false },
        layers: [
          { id: 'A11', hidden: true },
          { id: 'B12', hidden: false },
          { id: 'C13', hidden: false },
        ],
      },
    ]
    expect(areThereAnyValidMovesOnBoard(boardWithMoves)).toBe(true)
    expect(areThereAnyValidMovesOnBoard(boardWithoutMoves)).toBe(
      false,
    )
  })
  it('should check win state', () => {
    const action = {
      type: 'CLEAR_MATCH',
      matchingLayers: [0, 1, 2],
      state: {
        board: [
          {
            id: 0,
            zLayer: { id: 'Z1', hidden: false },
            layers: [
              { id: 'A11', hidden: true },
              { id: 'B2', hidden: true },
              { id: 'C3', hidden: true },
            ],
          },
          {
            id: 1,
            zLayer: { id: 'Z1', hidden: false },
            layers: [
              { id: 'A11', hidden: true },
              { id: 'B2', hidden: true },
              { id: 'C3', hidden: true },
            ],
          },
          {
            id: 2,
            zLayer: { id: 'Z1', hidden: false },
            layers: [
              { id: 'A5', hidden: false },
              { id: 'B3', hidden: false },
              { id: 'C4', hidden: false },
            ],
          },
          {
            id: 3,
            zLayer: { id: 'Z1', hidden: false },
            layers: [
              { id: 'A5', hidden: false },
              { id: 'B3', hidden: false },
              { id: 'C4', hidden: false },
            ],
          },
        ],
        infiniteMode: false,
        matchedLayers: 3,
      },
    }

    expect(checkWinState(action)).toBe(true)
  })
  it('should reveal the board', () => {
    const board = [
      {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C3', hidden: true },
        ],
      },
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C13', hidden: true },
        ],
      },
    ]

    expect.assertions(2)

    revealAllLayers(board).forEach((tile) => {
      const allRevealed = tile.layers.every(
        (layer) => layer.hidden === false,
      )
      expect(allRevealed).toBe(true)
    })
  })
})
