import React, { useState, useEffect, useContext } from 'react'
import { createPortal } from 'react-dom'
import pt from 'prop-types'
import { connect, useSelector } from 'react-redux'
import { compose } from 'redux'

import useDeltaBanner from 'shared/marketing/useDeltaBanner'
import { useUser, withUser } from 'user'
import { handleUserInformationUpdate } from 'helpers/hybridLinkHelpers'
import { shouldFailOpen } from 'shared/meter'

import useGameDataHybridEvent from '../../../shared/hooks/use-game-data-hybrid-event'
import useAndroidBack from '../../../shared/hooks/use-android-back'
import useSaleBannerHybridSubscribe from '../../../shared/hooks/use-sale-banner-hybrid-subscribe'
import { EVENT_ON_NAVIGATE_BACK } from '../../../../shared/foundation/js/hybrid/commands'
import Toolbar from '../Toolbar/Toolbar'
import Tileset from '../Tileset/Tileset'
import { wait } from '../../../helpers/gameUtils'
import ModalManager from '../../../shared/ModalSystem/ModalManager'
import {
  closeModal,
  getIsModalOpen,
  openModal,
  removeModal,
} from '../store/store'
import Congrats from '../modals/Congrats'
import HowToPlay from '../modals/HowToPlay'
import Settings from '../modals/Settings'
import ComboCounter from '../ComboCounter/ComboCounter'
import {
  setTilesCredits,
  getCurrentCredits,
} from '../tilesCredits/tilesCredits'
import { Moments, momentSystem } from '../moments/index'
import { trackEvent, getPuzzleID } from '../tilesTracker/tilesTracker'
import { MeterContext } from '../meter/MeterProvider'
import TopSaleBanner from '../../../../server/views/components/Marketing/Sale/TopSaleBanner'

const Layout = ({
  links,
  bgColor,
  fontColor,
  svg,
  displayName,
  filename,
  winState,
  boardSize,
  user,
  openModal,
  createdBy,
}) => {
  const { userType, updateUserType } = useUser()
  const meterContext = useContext(MeterContext)

  const [loaded, setLoaded] = useState(false)
  const [creditsLeft, setCreditsLeft] = useState(null)
  const [hasPlayed, setHasPlayed] = useState(false)
  const [editor, setEditor] = useState('Robert Vinluan')
  const [meterResponse, setMeterResponse] = useState({
    granted: false,
    grantReason: 'METER_LIMIT',
    isOffline: true,
  })

  const getGranted = (meterContext) => {
    const isOnline = !meterContext.isOffline
    // if the meter is online, use the meter response
    if (isOnline) {
      return meterContext.granted
    }

    // if the meter is offline, verify if the last response should fail open
    if (shouldFailOpen(meterContext)) {
      return meterContext.granted
    }

    // when all else fails, fall back on tiles paywall logic
    return false
  }

  useEffect(() => {
    if (meterContext) {
      const granted = getGranted(meterContext)
      const meter = {
        granted,
        grantReason: meterContext.grantReason,
      }
      setMeterResponse(meter)
    }
  }, [meterContext])

  const handleCredentialChanged = (event) => {
    const userData = event?.detail?.values.gamesAuthenticateUser
    handleUserInformationUpdate(userData, updateUserType)
  }

  useEffect(() => {
    if (window.isHybridWebView && window.NativeBridge) {
      window.addEventListener(
        'gamesUserCredentialChanged',
        handleCredentialChanged,
      )

      return () => {
        window.removeEventListener(
          'gamesUserCredentialChanged',
          handleCredentialChanged,
        )
      }
    }
  }, [])

  useEffect(() => {
    if (user.hasLoaded) {
      setLoaded(true)
    }
  }, [user])

  useEffect(() => {
    if (loaded) {
      setCreditsLeft(getCurrentCredits(userType, meterResponse))
    }
  }, [loaded])

  useEffect(() => {
    if (createdBy && loaded) {
      setEditor(createdBy)
    }
  }, [createdBy, loaded])

  useEffect(() => {
    momentSystem.load('loading')
  }, [])

  useEffect(() => {
    if (!winState) return
    const delayMS = (boardSize * 0.05 + 2) * 1000

    wait(delayMS).then(() => {
      openModal('congrats')
    })
  }, [winState])

  useGameDataHybridEvent({
    analyticsPuzzleID: getPuzzleID(),
    game: 'tiles',
  })

  const isModalOpen = useSelector(getIsModalOpen)

  useAndroidBack(
    (event) => {
      const response = {}

      if (isModalOpen) {
        closeModal()
        response[EVENT_ON_NAVIGATE_BACK] = true
      } else {
        // go back to app feed
        response[EVENT_ON_NAVIGATE_BACK] = false
      }
      event.respondWith(response)
    },
    [isModalOpen],
  )

  useSaleBannerHybridSubscribe()

  useEffect(() => {
    const backButton =
      document.getElementsByClassName('pz-hybrid-back')[0]

    if (window.isHybridWebView && window.NativeBridge) {
      if (backButton) {
        backButton.addEventListener('click', () => {
          if (momentSystem.getCurrentMoment() === 'game') {
            trackEvent('BACK', 'in-game')
          } else {
            trackEvent('BACK', 'welcome')
          }
        })
      }
    }

    return () => {
      backButton.removeEventListener('click', null)
    }
  }, [])

  const updateCredits = () => {
    setCreditsLeft(setTilesCredits(userType, meterResponse))
  }

  const { willShowDeltaBanner, portalEl } = useDeltaBanner()

  return (
    <>
      <div id="tls-svg" dangerouslySetInnerHTML={{ __html: svg }} />
      <div
        id="tls-content-box"
        className={hasPlayed ? 'conversion-tls-content-box' : ''}
        style={{ backgroundColor: bgColor, fontColor }}
      >
        {willShowDeltaBanner &&
          portalEl &&
          createPortal(
            <TopSaleBanner url={document.location.href} isDelta />,
            portalEl,
          )}
        <Toolbar />
        <Moments
          contentProps={{
            creditsLeft,
            updateCredits,
            links,
            hasLoaded: loaded,
            paletteName: displayName,
            filename,
            close: () => {
              momentSystem.transition('welcome', 'game')
              setHasPlayed(true)
            },
            editor,
            meterResponse,
          }}
        />
        <ModalManager
          modalMap={{
            congrats: Congrats,
            howToPlay: HowToPlay,
            settings: Settings,
          }}
          portalId="portal-modal-system"
          getOpenModalName={(state) => state.modalSystem.modal}
          getIsClosing={(state) => state.modalSystem.isClosing}
          onClose={closeModal}
          onRemove={removeModal}
          creditsLeft={creditsLeft}
          updateCredits={updateCredits}
          meterResponse={meterResponse}
        />
        <div id="tls-game-container">
          <Tileset />
          <ComboCounter fontColor={fontColor} />
        </div>
      </div>
    </>
  )
}

Layout.propTypes = {
  openModal: pt.func.isRequired,
  links: pt.shape({}).isRequired,
  bgColor: pt.string.isRequired,
  fontColor: pt.string,
  svg: pt.string.isRequired,
  displayName: pt.string.isRequired,
  filename: pt.string.isRequired,
  createdBy: pt.string.isRequired,
  winState: pt.bool.isRequired,
  boardSize: pt.number.isRequired,
  user: pt.shape({
    hasLoaded: pt.bool.isRequired,
    userType: pt.shape({
      isLoggedIn: pt.bool.isRequired,
      hasXwd: pt.bool.isRequired,
    }).isRequired,
  }).isRequired,
}

const mapStateToProps = ({ palette, boardSize, winState }) => ({
  bgColor: palette.bgColor,
  fontColor: palette.fontColor,
  svg: palette.svg,
  displayName: palette.displayName,
  filename: palette.filename,
  createdBy: palette.createdBy,
  boardSize,
  winState,
})

const mapDispatchToProps = { openModal }

export default compose(
  withUser,
  connect(mapStateToProps, mapDispatchToProps),
)(Layout)
