import React from 'react'
import { waitFor, render } from '@testing-library/react'

import { useUser } from 'user'
import { retrieveMeterValue, willUseMeterToGate } from 'shared/meter'

import { MeterProvider } from './MeterProvider'

jest.mock('user', () => ({
  useUser: jest.fn(() => ({
    userType: { isLoggedIn: true, hasXwd: true },
    hasLoaded: true,
  })),
}))

jest.mock('../../../../shared/meter', () => ({
  retrieveMeterValue: jest.fn(),
  willUseMeterToGate: jest.fn(),
}))

describe('MeterProvider', () => {
  beforeEach(jest.clearAllMocks)
  afterEach(jest.clearAllMocks)

  it('should not call Meter if user is subscribed on web', async () => {
    // @ts-ignore
    willUseMeterToGate.mockReturnValueOnce(true)

    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).not.toHaveBeenCalled()
    })
  })

  it('should not call Meter if user is subscribed on games app', async () => {
    window.isHybridWebView = true
    window.newsreaderAppPlatform = ''
    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).not.toHaveBeenCalled()
    })
  })

  it('should not call Meter if user is subscribed on newsreader', async () => {
    window.isHybridWebView = false
    window.newsreaderAppPlatform = 'ios'
    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )
    await waitFor(() => {
      expect(retrieveMeterValue).not.toHaveBeenCalled()
    })
  })

  it('should call Meter if user is anon and on web', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: false, hasXwd: false },
      hasLoaded: true,
    })

    window.isHybridWebView = false
    window.newsreaderAppPlatform = ''

    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).toHaveBeenCalled()
    })
  })

  it('should not call Meter if user is anon and on games app', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: false, hasXwd: false },
      hasLoaded: true,
    })

    window.isHybridWebView = true
    window.newsreaderAppPlatform = ''

    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).not.toHaveBeenCalled()
    })
  })

  it('should not call Meter if user is anon and on newsreader app', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: false, hasXwd: false },
      hasLoaded: true,
    })

    window.isHybridWebView = false
    window.newsreaderAppPlatform = 'ios'

    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).not.toHaveBeenCalled()
    })
  })

  it('should call Meter if user is regi and on web', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: true, hasXwd: false },
      hasLoaded: true,
    })

    // @ts-ignore
    willUseMeterToGate.mockReturnValueOnce(true)

    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).toHaveBeenCalled()
    })
  })

  it('should not call Meter if user is regi and on games app', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: true, hasXwd: false },
      hasLoaded: true,
    })

    window.isHybridWebView = true
    window.newsreaderAppPlatform = ''

    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).not.toHaveBeenCalled()
    })
  })

  it('should not call Meter if user is regi newsreader app', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: true, hasXwd: false },
      hasLoaded: true,
    })

    window.isHybridWebView = false
    window.newsreaderAppPlatform = 'ios'

    render(
      <MeterProvider>
        <></>
      </MeterProvider>,
    )

    await waitFor(() => {
      expect(retrieveMeterValue).not.toHaveBeenCalled()
    })
  })
})
