import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'
import { configureStore } from '@reduxjs/toolkit'

import { useUser } from 'user'

import { trackEvent } from '../tilesTracker/tilesTracker'
import { trackImpression } from '../../../helpers/tracking'
import { initialState } from '../store/store'

import ModalCongrats from './Congrats'

const mockedUseUser = useUser as jest.MockedFunction<typeof useUser>

jest.mock('foundation', () => ({
  mobileNavTools: {
    activate: jest.fn(),
  },
  getGameData: () => ({}),
  env: {
    account: 'mockedAccount',
  },
}))

jest.mock('user', () => ({
  useUser: jest.fn(),
}))

jest.mock('../tilesTracker/tilesTracker', () => ({
  trackEvent: jest.fn(),
}))

jest.mock('../../../helpers/tracking', () => ({
  trackImpression: jest.fn(),
}))

jest.mock('../../../helpers/navigateBack', () => ({
  navigateBack: jest.fn(),
}))

const userProps = {
  hasDigi: false,
  hasHd: false,
  inShortzMode: false,
  entitlement: 'anon' as 'anon',
  regiId: 'mockedRegi',
}

const LONGEST_COMBO = 5

const store = configureStore({
  reducer: () => ({
    ...initialState,
    winState: true,
    longestCombo: LONGEST_COMBO,
    perfectSolve: false,
  }),
})

const props = {
  creditsLeft: 0,
  updateCredits: jest.fn(),
  links: {
    login: '/login',
    register: '/auth/register',
  },
  deviceType: {
    isMobile: true,
  },
  close: jest.fn(),
  isClosing: false,
  onRemove: jest.fn(),
  perfectSolve: true,
  meterResponse: {
    granted: false,
    grantReason: 'METER_LIMIT',
  },
}

describe('TILES: CONGRATS MODAL', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  const mountCongrats = () =>
    mount(
      <Provider store={store}>
        <ModalCongrats {...props} />
      </Provider>,
    )

  it('title is "Perfect" on perfect solve', () => {
    mockedUseUser.mockReturnValue({
      updateUserType: () => {},
      userType: {
        ...userProps,
        hasXwd: true,
        isLoggedIn: false,
      },
      hasLoaded: true,
    })

    const perfStore = configureStore({
      reducer: () => ({
        ...initialState,
        winState: true,
        longestCombo: 5,
        perfectSolve: true,
      }),
    })

    const congrats = mount(
      <Provider store={perfStore}>
        <ModalCongrats {...props} />
      </Provider>,
    )
    expect(congrats.find('h1[data-testid="header"]').text()).toEqual(
      'Perfect!',
    )
  })

  it('title is "Congratulations" on imperfect solve', () => {
    mockedUseUser.mockReturnValue({
      updateUserType: () => {},
      userType: {
        ...userProps,
        hasXwd: true,
        isLoggedIn: false,
      },
      hasLoaded: true,
    })

    const congrats = mountCongrats()
    expect(congrats.find('h1[data-testid="header"]').text()).toEqual(
      'Congratulations!',
    )
  })

  it('fires impression analytics', () => {
    mockedUseUser.mockReturnValue({
      userType: {
        ...userProps,
        hasXwd: true,
        isLoggedIn: false,
      },
      hasLoaded: true,
      updateUserType: () => {},
    })
    mountCongrats()

    expect(trackImpression).toHaveBeenCalledWith(
      'congrats-modal',
      'tiles',
    )
    expect(trackImpression).toHaveBeenCalledWith(
      'next-game',
      'tiles',
      'spelling-bee',
    )
  })

  it('description displays longest combo', () => {
    mockedUseUser.mockReturnValue({
      userType: {
        ...userProps,
        hasXwd: true,
        isLoggedIn: false,
      },
      hasLoaded: true,
      updateUserType: () => {},
    })

    const congrats = mountCongrats()
    expect(
      congrats.find('p[data-testid="subheading"]').text(),
    ).toEqual(`Your highest combo was ${LONGEST_COMBO} moves.`)
  })

  it('clicking "Play Again" closes modal, adjusts credits, and fires analytics event', () => {
    mockedUseUser.mockReturnValue({
      userType: {
        ...userProps,
        hasXwd: true,
        isLoggedIn: true,
      },
      hasLoaded: true,
      updateUserType: () => {},
    })
    props.creditsLeft = Infinity

    const congrats = mountCongrats()
    const actionDiv = congrats.find('div[data-testid="actions"]')
    const playAgain = actionDiv.find('button').first()

    expect(playAgain.text()).toEqual('Play again')

    playAgain.simulate('click')

    expect(trackEvent).toHaveBeenCalledWith('PLAY_AGAIN')
    expect(props.close).toHaveBeenCalledWith(false)
    expect(props.updateCredits).toHaveBeenCalled()
  })

  it('for subs, does not render credits messaging', () => {
    mockedUseUser.mockReturnValue({
      userType: {
        ...userProps,
        hasXwd: true,
        isLoggedIn: true,
      },
      hasLoaded: true,
      updateUserType: () => {},
    })

    const congrats = mountCongrats()
    const actionDiv = congrats.find('div[data-testid="actions"]')
    const playAgain = actionDiv.find('button').first()
    expect(playAgain.text()).toEqual('Play again')

    expect(
      actionDiv.find('div[data-testid="action-message-1"]').length,
    ).toEqual(0)
  })

  it('for regis and anons with credits, renders button and credits messaging', () => {
    mockedUseUser.mockReturnValue({
      userType: {
        ...userProps,
        hasXwd: false,
        isLoggedIn: true,
      },
      hasLoaded: true,
      updateUserType: () => {},
    })
    const CREDITS_LEFT = 2
    props.creditsLeft = CREDITS_LEFT

    const congrats = mountCongrats()
    const actionDiv = congrats.find('div[data-testid="actions"]')
    const playAgain = actionDiv.find('button').first()
    expect(playAgain.text()).toEqual('Play again')

    const actionMessage = actionDiv.find(
      'div[data-testid="action-message-1"]',
    )
    expect(actionMessage.text()).toEqual(
      `${CREDITS_LEFT} plays left today.`,
    )
  })

  describe('for anons without credits', () => {
    it('renders a button and registers an upsell', () => {
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: false,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })

      props.creditsLeft = 0

      const congrats = mountCongrats()
      const actionDiv = congrats.find('div[data-testid="actions"]')
      const actionMessage = actionDiv.find(
        'div[data-testid="action-message-1"]',
      )
      expect(actionMessage.text()).toContain('No plays left today.')
      expect(actionMessage.text()).toContain(
        'Log In or Subscribe to Play',
      )

      const subBtn = actionDiv.find(
        'button[data-testid="btn-subscribe"]',
      )
      expect(subBtn.text()).toContain('Subscribe')
      const loginBtn = actionDiv.find(
        'button[data-testid="btn-log-in"]',
      )
      expect(loginBtn.text()).toEqual('Log In')
    })

    it('does not trigger hard paywall impression analytics, that is regi only', () => {
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: false,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })

      props.creditsLeft = 0

      mountCongrats()

      expect(trackImpression).not.toHaveBeenCalledWith(
        'hardregiwall',
        'tiles-no-plays-congrats',
      )
    })
  })

  describe('for regis without credits', () => {
    it('renders a subscribe button and registers an upsell', () => {
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: true,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })
      props.creditsLeft = 0

      const congrats = mountCongrats()
      const actionDiv = congrats.find('div[data-testid="actions"]')
      const actionMessage = actionDiv.find(
        'div[data-testid="action-message-1"]',
      )
      expect(actionMessage.text()).toContain('No plays left today.')
      expect(actionMessage.text()).not.toContain(
        'Log In or Subscribe to Play',
      )

      const subBtn = actionDiv.find(
        'button[data-testid="btn-subscribe"]',
      )
      expect(subBtn.text()).toContain('Subscribe')

      expect(
        actionDiv.find('button[data-testid="btn-log-in"]').length,
      ).toEqual(0)
    })

    it('fires the proper impression analytics', () => {
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: true,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })
      props.creditsLeft = 0

      mountCongrats()

      expect(trackImpression).toHaveBeenCalledWith(
        'hardpaywall',
        'tiles-no-plays-congrats',
      )
    })

    // Using Meter Response
    it('for anons on Delta - renders play again button and does not render credits messaging', () => {
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: false,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })
      props.meterResponse.granted = true

      const congrats = mountCongrats()
      const actionDiv = congrats.find('div[data-testid="actions"]')
      const playAgain = actionDiv.find('button').first()
      expect(playAgain.text()).toEqual('Play again')

      expect(
        actionDiv.find('div[data-testid="action-message-1"]').length,
      ).toEqual(0)
    })

    it('for regis on Delta - renders play again button and does not render credits messaging', () => {
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: true,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })
      props.meterResponse.granted = true

      const congrats = mountCongrats()
      const actionDiv = congrats.find('div[data-testid="actions"]')
      const playAgain = actionDiv.find('button').first()
      expect(playAgain.text()).toEqual('Play again')

      expect(
        actionDiv.find('div[data-testid="action-message-1"]').length,
      ).toEqual(0)
    })
  })

  describe('for non-hybrid users', () => {
    it('does not render a view all games button', () => {
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: true,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })
      props.creditsLeft = 0

      const congrats = mountCongrats()
      const actionDiv = congrats.find('div[data-testid="actions"]')

      const viewAllGamesBtnLength = actionDiv.find(
        'button[data-testid="btn-view-all-games"]',
      ).length

      expect(viewAllGamesBtnLength).toEqual(0)
    })
  })

  describe('for hybrid users', () => {
    it('renders a view all games button that takes hybrid users back to the hub', () => {
      window.isHybridWebView = true
      // @ts-ignore
      window.NativeBridge = true
      mockedUseUser.mockReturnValue({
        userType: {
          ...userProps,
          hasXwd: false,
          isLoggedIn: true,
        },
        hasLoaded: true,
        updateUserType: () => {},
      })
      props.creditsLeft = 0

      const congrats = mountCongrats()
      const actionDiv = congrats.find('div[data-testid="actions"]')

      const viewAllGamesBtn = actionDiv.find(
        'button[data-testid="btn-view-all-games"]',
      )
      expect(viewAllGamesBtn.text()).toEqual('View all games')

      viewAllGamesBtn.simulate('click')

      expect(trackEvent).toHaveBeenCalledWith('BACK_TO_HUB')
    })
  })
})
