import React, { useEffect } from 'react'
import { useDispatch, useSelector } from 'react-redux'

import { UserType } from 'foundation'
import { useUser } from 'user'
import useCTA from 'shared/marketing/useCTA'

import ModalCongratsShared from '../../../shared/ModalSystem/ModalCongrats'
import { navigateBack } from '../../../helpers/navigateBack'
import type {
  ButtonProps,
  ModalProps,
} from '../../../shared/ModalSystem/modalTypes'
import { getCTA } from '../../../shared/ModalSystem/helpers'
import handleHybridLoginClick from '../../../helpers/hybridLoginLinkClick'
import { handleSubscribe } from '../../../helpers/subscribeLinks'
import { trackEvent } from '../tilesTracker/tilesTracker'
import { trackImpression } from '../../../helpers/tracking'
import {
  generateNewBoard,
  getLongestCombo,
  getPerfectSolve,
} from '../store/store'

import { playCopyHelper } from './helpers'

type Props = {
  creditsLeft: number
  updateCredits: () => void
}

const size = 'medium'

const playCopy = (playCount: number) => {
  const { numberText, playText } = playCopyHelper(playCount, true)
  return `${numberText} ${playText} left today.`
}

const login = (
  updateUserType: (val: Partial<UserType>) => void,
  { showAnonText }: { showAnonText: boolean },
): ButtonProps => ({
  size,
  onClick: (e) => {
    trackEvent('LOGIN', 'congrats-log-in')
    handleHybridLoginClick(e, updateUserType)
  },
  text: 'Log In',
  largeText: true,
  key: 'login-btn-congrats',
  filled: false,
  descriptionText:
    (showAnonText && [
      {
        content: 'Log In to Play:',
        emphasized: true,
      },
    ]) ||
    undefined,
})

const subscribe = (
  updateUserType: (val: Partial<UserType>) => void,
  { showAnonText }: { showAnonText: boolean },
  subscribeText: string,
): ButtonProps => {
  const descriptionText = [
    {
      content: playCopy(0),
      emphasized: true,
    },
  ]

  if (showAnonText) {
    descriptionText.push({
      content: 'Log In or Subscribe to Play:',
      emphasized: true,
    })
  }

  return {
    descriptionText,
    size,
    onClick: () => {
      trackEvent('SUBSCRIBE_CONGRATS')
      handleSubscribe('tilesCutoffSettings', updateUserType)
    },
    text: subscribeText,
    largeText: true,
    filled: false,
    key: 'sub-btn-congrats',
  }
}

const playAgainBtn = (
  onClick: (e: React.MouseEvent<Element, MouseEvent>) => void,
  creditsLeft: false | number,
): ButtonProps => {
  return {
    onClick,
    size,
    text: 'Play again',
    filled: true,
    largeText: true,
    key: 'play-again-btn-congrats',
    descriptionText:
      (creditsLeft && [
        {
          content: playCopy(creditsLeft),
          emphasized: true,
        },
      ]) ||
      undefined,
  }
}

const viewAllGamesBtn = (navigateBack: () => void): ButtonProps => {
  return {
    size,
    onClick: () => {
      navigateBack()
      trackEvent('BACK_TO_HUB')
    },
    text: 'View all games',
    filled: false,
    largeText: true,
    key: 'view-all-games-btn-congrats',
  }
}

const Congrats = ({
  close,
  isClosing,
  onRemove,
  creditsLeft,
  updateCredits,
  meterResponse,
}: ModalCongratsProps) => {
  const dispatch = useDispatch()
  const longestCombo = useSelector(getLongestCombo)
  const perfectSolve = useSelector(getPerfectSolve)
  const { userType, updateUserType } = useUser()
  const { getButtonText } = useCTA()
  const { granted } = meterResponse

  const subscribeText = getButtonText()
  const noCreditsLeft = creditsLeft === 0
  const subAccess = granted || userType.hasXwd

  useEffect(() => {
    if (noCreditsLeft && userType.isLoggedIn) {
      trackImpression('hardpaywall', 'tiles-no-plays-congrats')
    }

    trackImpression('congrats-modal', 'tiles')
    trackImpression('next-game', 'tiles', 'spelling-bee')
  }, [noCreditsLeft, userType])

  const playAgainOnClick = () => {
    if (noCreditsLeft) return
    updateCredits()
    dispatch(generateNewBoard(true))
    close(false)
    trackEvent('PLAY_AGAIN')
  }

  const getButtons = () => {
    let buttons = []
    if (userType.isLoggedIn || subAccess) {
      buttons =
        subAccess || creditsLeft > 0
          ? [
              playAgainBtn(
                playAgainOnClick,
                !subAccess && creditsLeft,
              ),
            ]
          : [
              subscribe(
                updateUserType,
                { showAnonText: false },
                subscribeText,
              ),
            ]
    } else {
      buttons =
        creditsLeft > 0
          ? [
              playAgainBtn(playAgainOnClick, creditsLeft),
              login(updateUserType, { showAnonText: true }),
            ]
          : [
              subscribe(
                updateUserType,
                { showAnonText: true },
                subscribeText,
              ),
              login(updateUserType, { showAnonText: false }),
            ]
    }

    if (window.isHybridWebView && window.NativeBridge) {
      buttons.push(viewAllGamesBtn(navigateBack))
    }

    return buttons
  }

  let btnProps = getButtons()
  useEffect(() => {
    btnProps = getButtons()
  }, [userType])

  const subheading: JSX.Element = (
    <>
      Your highest combo was{' '}
      <span className="btn__emphasized">
        {longestCombo} {longestCombo === 1 ? 'move' : 'moves'}
      </span>
      .
    </>
  )

  return (
    <ModalCongratsShared
      hideClose
      close={close}
      onRemove={onRemove}
      isClosing={isClosing}
      showStar={false}
      heading={perfectSolve ? 'Perfect!' : 'Congratulations!'}
      subheading={subheading}
      buttons={btnProps}
      cta={{
        game: getCTA('spelling-bee'),
      }}
      currentGame={{ name: 'tiles' }}
    />
  )
}

type ModalCongratsProps = Props & ModalProps

export default Congrats
