import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'
import { configureStore } from '@reduxjs/toolkit'

import { useUser } from 'user'

import { initialState } from '../store/store'

import ModalSettings from './Settings'

jest.mock('foundation', () => ({
  getGameData: () => ({}),
  env: {
    account: 'mockedAccount',
  },
  appendSupportedParams: () => {},
}))

const mockChangePalette = jest.fn()

jest.mock('../store/store', () => ({
  ...jest.requireActual('../store/store'),
  changePalette: () =>
    jest.fn().mockImplementation(() => {
      mockChangePalette()
    }),
}))

jest.mock('user', () => ({
  useUser: jest.fn(),
}))

jest.mock('../tilesTracker/tilesTracker', () => ({
  setPaletteName: jest.fn(),
}))

const mockedUseUser = useUser as jest.MockedFunction<typeof useUser>

const allPalettes = [
  { displayName: 'Test1', filename: 'test1' },
  { displayName: 'Test2', filename: 'test2' },
  { displayName: 'Test3', filename: 'test3' },
]

const initState = {
  ...initialState,
  winState: true,
  longestCombo: 3,
  perfectSolve: false,
  palette: {
    displayName: 'Test1',
    filename: 'test1',
  },
  allPalettes,
  dispatch: jest.fn(),
  subscribe: jest.fn(),
}

const store = configureStore({
  reducer: () => initState,
})

const userTypeProps = {
  hasDigi: false,
  hasHd: false,
  inShortzMode: false,
  entitlement: 'anon' as 'anon',
  regiId: 'mockedRegi',
  hasXwd: true,
  isLoggedIn: false,
}

const props = {
  creditsLeft: 0,
  updateCredits: jest.fn(),
  links: {
    register: '',
  },
  deviceType: {
    isMobile: true,
  },
  store,
  close: jest.fn(),
  isClosing: false,
  onRemove: jest.fn(),
  meterResponse: {
    granted: false,
    grantReason: 'METER_LIMIT',
  },
}

const mountModal = () =>
  mount(
    <Provider store={store}>
      <div className="pzm-modals-wrapper">
        <div className="pzm-modal-ex" />
        <ModalSettings {...props} />
      </div>
    </Provider>,
  )

const mockAnon = () => {
  mockedUseUser.mockReturnValue({
    updateUserType: () => {},
    userType: {
      ...userTypeProps,
      isLoggedIn: false,
      hasXwd: false,
    },
    hasLoaded: true,
  })
}

const mockRegi = () => {
  mockedUseUser.mockReturnValue({
    updateUserType: () => {},
    userType: {
      ...userTypeProps,
      isLoggedIn: true,
      hasXwd: false,
    },
    hasLoaded: true,
  })
}

const mockSub = () => {
  mockedUseUser.mockReturnValue({
    updateUserType: () => {},
    userType: {
      ...userTypeProps,
      isLoggedIn: true,
      hasXwd: true,
    },
    hasLoaded: true,
  })
}

describe('TILES: CONGRATS MODAL', () => {
  beforeEach(() => {
    jest.resetAllMocks()
  })

  it('prompt anons to register or subscribe', () => {
    mockAnon()
    props.creditsLeft = 3
    const modalSettings = mountModal()
    const description = modalSettings.find('p[data-testid="message"]')

    expect(description.text()).toBe(
      'Register or subscribe to unlock more features.',
    )
  })

  it('should not display upsell for subs', () => {
    mockSub()
    const modalSettings = mountModal()

    expect(
      modalSettings.find('p[data-testid="message"]'),
    ).toHaveLength(0)
  })

  it('should prompt anons to register', () => {
    mockAnon()
    const modalSettings = mountModal()
    const credits = modalSettings.find(
      'h3[data-testid="credits-left"]',
    )

    expect(credits.text()).toBe('You have 3 plays left.')
  })

  it('should prompt regis to subscribe with a link', () => {
    mockRegi()
    const modalSettings = mountModal()
    const credits = modalSettings.find(
      'h3[data-testid="credits-left"]',
    )

    const description = modalSettings.find('p[data-testid="message"]')

    expect(credits.text()).toBe('You have 3 plays left.')
    expect(description.text()).toBe(
      'Subscribe to unlock more features.',
    )
  })

  // Using Meter Response
  it('should not display upsell for Anons on Delta', () => {
    mockAnon()
    props.meterResponse.granted = true
    const modalSettings = mountModal()

    expect(
      modalSettings.find('p[data-testid="message"]'),
    ).toHaveLength(0)
  })

  it('should not display upsell for Regis on Delta', () => {
    mockRegi()
    props.meterResponse.granted = true
    const modalSettings = mountModal()

    expect(
      modalSettings.find('p[data-testid="message"]'),
    ).toHaveLength(0)
  })
})
