import React, { useEffect, useState } from 'react'
import { connect } from 'react-redux'

import {
  mobileNavTools,
  track,
  trackClick,
  trackModuleInteraction,
  abra,
} from 'foundation'
import { useUser } from 'user'
import useCTA from 'shared/marketing/useCTA'

import { trackEvent, setPlayMode } from '../tilesTracker/tilesTracker'
import Welcome from '../../../shared/moments/components/Welcome'
import NonDismissalBanner from '../../../shared/Banner/NonDismissalBanner'
import { shouldShowAppDownloadCTA } from '../../../shared/Banner/bannerHelpers'
import { handleSubscribe } from '../../../helpers/subscribeLinks'
import handleHybridLoginClick from '../../../helpers/hybridLoginLinkClick'
import { trackImpression } from '../../../helpers/tracking'
import { startGame, infiniteModeToggle } from '../store/store'
import { paletteDisplayText } from '../MenuOptions/shared'
import useWindowSize from '../../../shared/hooks/use-window-size'
import { MeterProperties } from '../../../../shared/meter/MeterTypes'

export type WelcomeMomentProps = {
  close: Function
  creditsLeft: number
  paletteName: string
  filename: string
  editor: string
  infiniteModeToggle: Function
  startGame: Function
  updateCredits: Function
  meterResponse: MeterProperties
}

const WelcomeMoment = ({
  close,
  creditsLeft,
  paletteName,
  filename,
  editor,
  infiniteModeToggle,
  startGame,
  updateCredits,
  meterResponse,
}: WelcomeMomentProps) => {
  const { userType, updateUserType, hasLoaded } = useUser()
  const { hasXwd, isLoggedIn, entitlement } = userType
  const { winWidth } = useWindowSize()
  const { getVariant, reportExposure } = abra

  // to prevent duplicates after clicking play
  const [sendTrackEvent, setSendTrackEvent] = useState(true)

  const navigationLink = `tilesWelcome${
    winWidth < 500 ? 'Mobile' : 'Desktop'
  }`

  const tilesDownloadCtaVariant = getVariant(
    'GAMES_tilesWelcomeAppDownloadCTA_0823',
  )

  const showDownloadCTA = React.useMemo(
    () =>
      shouldShowAppDownloadCTA(
        tilesDownloadCtaVariant,
        '0_control',
        window,
      ),
    [tilesDownloadCtaVariant],
  )

  // Tracking
  useEffect(() => {
    const creditsLoaded = creditsLeft !== null
    const sendAnon =
      entitlement === 'anon' && creditsLoaded && sendTrackEvent

    const sendRegi =
      isLoggedIn && !hasXwd && creditsLoaded && sendTrackEvent

    if (!hasLoaded) return

    if (sendAnon) {
      track('impression', {
        module: {
          name: 'welcome-modal',
          region: 'tiles',
          label: ['log-in', 'subscribe'],
          context: creditsLeft,
        },
      })
    }

    if (sendRegi) {
      trackImpression(
        'welcome-modal',
        'tiles',
        'log-in',
        `${creditsLeft}`,
      )
    }
  }, [hasLoaded, hasXwd, isLoggedIn, entitlement, creditsLeft])

  React.useEffect(() => {
    if (tilesDownloadCtaVariant) {
      reportExposure('GAMES_tilesWelcomeAppDownloadCTA_0823')
    }

    if (showDownloadCTA) {
      trackImpression('welcome-modal', 'tiles', 'download-app', '')
    }
  }, [showDownloadCTA, tilesDownloadCtaVariant])

  useEffect(() => {
    if (hasXwd) {
      trackImpression('welcome-modal', 'tiles')
    }
  }, [hasXwd])

  useEffect(() => {
    if (!hasLoaded) return

    // Non sub users without credits
    if (creditsLeft === 0 && sendTrackEvent && !hasXwd) {
      trackImpression('hardpaywall', 'tiles-no-plays-welcome')
    }
  }, [creditsLeft, hasLoaded, hasXwd])

  type MouseEventClick = (
    event:
      | MouseEvent
      | React.MouseEvent<HTMLButtonElement, MouseEvent>
      | React.MouseEvent<HTMLAnchorElement, MouseEvent>,
  ) => void

  type Button = {
    text: string
    action: MouseEventClick | (() => void)
    icon?: string
    color?: string
  }

  type ButtonGroups = {
    buttons: Button[]
    header?: string | JSX.Element
    desktopButtonOrder?: string
  }[]

  const { getButtonText } = useCTA()

  const subscribeText = getButtonText()

  const createButtons: () => ButtonGroups = () => {
    const { granted, grantReason } = meterResponse
    const subscribeMessage = isLoggedIn
      ? 'Subscribe to Play:'
      : 'Log In or Subscribe to Play:'

    const playButton = {
      text: 'Play',
      action: () => {
        setSendTrackEvent(false)
        mobileNavTools.activate()
        updateCredits()
        startGame()
        close()
        trackEvent('START_GAME')
      },
    }

    const subscribeButton = {
      text: subscribeText,
      color: 'secondary',
      action: () => {
        handleSubscribe(navigationLink, updateUserType)
        trackClick({
          context: null,
          element: { name: 'tiles', label: 'subscribe' },
          label: 'welcome',
        })
      },
    }

    const loginButton = {
      text: 'Log In',
      color: 'secondary',
      action: (
        event:
          | MouseEvent
          | React.MouseEvent<HTMLButtonElement, MouseEvent>
          | React.MouseEvent<HTMLAnchorElement, MouseEvent>,
      ) => {
        handleHybridLoginClick(event, updateUserType)
        trackClick({
          context: null,
          element: { name: 'tiles', label: 'log-in' },
          label: 'welcome',
        })
      },
    }

    const zenButtonSub = {
      text: 'Zen Mode',
      action: () => {
        mobileNavTools.activate()
        close()
        infiniteModeToggle()
        setPlayMode('z')
        trackEvent('START_GAME')
      },
    }

    const zenButtonNonSub = {
      text: 'Zen Mode',
      icon: 'locked',
      action: () => {
        handleSubscribe(navigationLink, updateUserType)
        trackEvent('ZEN_MODE')
      },
    }

    const subButtons: Button[] = [playButton, zenButtonSub]
    // Anon on Delta
    if (!isLoggedIn && grantReason === 'IP') {
      subButtons.push(loginButton, subscribeButton)
    }

    // Regi on Delta
    if (!hasXwd && isLoggedIn && grantReason === 'IP') {
      subButtons.push(subscribeButton)
    }

    if (hasXwd || granted) {
      return [
        {
          buttons: subButtons,
          desktopButtonOrder: 'vertical',
        },
      ]
    }

    const nonSubButtons: Button[] = [zenButtonNonSub, subscribeButton]

    if (!isLoggedIn) {
      nonSubButtons.splice(1, 0, loginButton)
    }

    if (creditsLeft === 0) {
      return [
        {
          header: (
            <>
              <span>0 plays left today.</span>
              <br />
              <span id="tiles-subscribe-message">
                {subscribeMessage}
              </span>
            </>
          ),
          buttons: nonSubButtons,
          desktopButtonOrder: 'vertical',
        },
      ]
    }

    return [
      {
        header: `${creditsLeft} play${
          creditsLeft > 1 ? 's' : ''
        } left today`,
        buttons: [playButton],
        desktopButtonOrder: 'vertical',
      },
      {
        header: subscribeMessage,
        buttons: nonSubButtons,
        desktopButtonOrder: 'vertical',
      },
    ]
  }

  const capitalize = (str: string) =>
    str && str.length > 1
      ? str
          .split('-')
          .map((s) => s[0].toUpperCase() + s.slice(1))
          .join(' ')
      : ''

  const momentProps = {
    bgColor: '$tiles-green',
    game: 'tiles',
    icon: 'tiles',
    title: 'Tiles',
    description: {
      text: 'Match elements and keep your chain going.',
    },
    buttonGroups: createButtons(),
    infoText: (
      <span style={{ fontWeight: 600 }}>
        <span>{`${paletteDisplayText(
          paletteName !== 'Default'
            ? paletteName
            : capitalize(filename),
          {
            addTilesetSuffix: false,
          },
        )} Tileset`}</span>
        <br />
        <span>{`Tileset by ${editor}`}</span>
      </span>
    ),
  }

  return (
    <>
      <Welcome game={momentProps} />
      {showDownloadCTA && (
        <NonDismissalBanner
          title="Play Tiles and more in the NYT Games app."
          icon="crossword-app"
          ctaLink="https://fpx3r.app.goo.gl/app-store"
          onClickLink={() => {
            trackModuleInteraction(
              '',
              '',
              'tiles',
              'welcome-download-app',
              'https://fpx3r.app.goo.gl/app-store',
            )
          }}
          showMultipleIconsBanner
        />
      )}
    </>
  )
}

const mapStateToProps = () => ({})

const mapDispatchToProps = {
  infiniteModeToggle,
  startGame,
}

export default connect(
  mapStateToProps,
  mapDispatchToProps,
)(WelcomeMoment)
