import React from 'react'
import { Provider } from 'react-redux'
import { act } from 'react-dom/test-utils'
import { fireEvent, render, screen } from '@testing-library/react'

import { abra, mobileNavTools, trackClick } from 'foundation'
import { useUser } from 'user'
import { isMobile } from 'responsive'

import { trackEvent } from '../../tilesTracker/tilesTracker'
import { trackImpression } from '../../../../helpers/tracking'
import WelcomeMoment from '../WelcomeMoment'

jest.mock('foundation', () => ({
  mobileNavTools: {
    activate: jest.fn(),
  },
  getGameData: () => {},
  env: { account: 'https://myaccount.nytimes.com' },
  trackClick: () => {},
  abra: {
    getVariant: jest.fn(),
    reportExposure: jest.fn(),
  },
}))

jest.mock('responsive', () => ({
  isMobile: jest.fn(),
}))

jest.mock('user', () => ({
  useUser: jest.fn(),
}))

jest.mock('../../tilesTracker/tilesTracker', () => ({
  trackEvent: jest.fn(),
  setPlayMode: jest.fn(),
}))

jest.mock('../../../../helpers/tracking', () => ({
  trackImpression: jest.fn(),
}))

describe('TILES: WELCOME MOMENT', () => {
  const store = {
    getState: () => ({}),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  }

  const props = {
    updateCredits: jest.fn(),
    deviceType: {
      isMobile: true,
    },
    open: jest.fn(),
    close: jest.fn(),
    startGame: jest.fn(),
    infiniteModeToggle: jest.fn(),
    links: {},
    meterResponse: {
      granted: false,
    },
  }

  const assignMock = jest.fn()

  delete window.location
  window.location = { assign: assignMock }

  afterEach(() => {
    assignMock.mockClear()
  })

  beforeEach(() => {
    jest.resetAllMocks()
  })

  const renderMoment = (creditsLeft) =>
    render(
      <Provider store={store}>
        <WelcomeMoment creditsLeft={creditsLeft} {...props} />
      </Provider>,
    )

  it('should render SVG, title and description', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
        isLoggedIn: false,
      },
    })
    renderMoment(Infinity)
    expect(
      document.querySelector('.pz-moment__icon.tiles'),
    ).toBeTruthy()
    expect(
      document.querySelector('.pz-moment__title').textContent,
    ).toEqual('Tiles')
    expect(
      document.querySelector('.pz-moment__description'),
    ).toBeTruthy()
  })

  it('for regis with credits, should show buttons and credit count', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: true,
      },
    })
    // @ts-ignore
    abra.getVariant.mockReturnValue('0_control')
    renderMoment(2)
    const buttons = screen.getAllByRole('button')

    expect(buttons).toHaveLength(3)
    expect(screen.queryByText('2 plays left today')).toBeTruthy()
  })

  it('for regis without credits, should show buttons and upsell', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: true,
      },
    })
    // @ts-ignore
    abra.getVariant.mockReturnValue('0_control')
    renderMoment(0)
    const buttons = screen.getAllByRole('button')
    const creditsMessaging = document.getElementById(
      'tiles-subscribe-message',
    ).textContent

    expect(buttons).toHaveLength(2)
    expect(creditsMessaging).toEqual('Subscribe to Play:')
  })

  it('for anons with credits, should show buttons and credit count', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: false,
      },
    })
    // @ts-ignore
    abra.getVariant.mockReturnValue('0_control')
    renderMoment(1)
    const buttons = screen.getAllByRole('button')

    expect(buttons).toHaveLength(4)
    expect(screen.queryByText('1 play left today')).toBeTruthy()
  })

  it('for regis with no credits, show zen and subscribe button', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: true,
      },
    })

    renderMoment(0)
    const zenButton = screen.getAllByRole('button')[0]
    const subscribeButton = screen.getAllByRole('button')[1]

    expect(zenButton.textContent).toEqual('Zen Mode')
    expect(subscribeButton.textContent).toContain('Subscribe')
    expect(screen.queryByText('0 plays left today.')).toBeTruthy()
    fireEvent.click(subscribeButton)
    expect(store.subscribe).toHaveBeenCalled()
  })

  it('for anons with no credits, show zen, login and subscribe button', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: false,
      },
    })

    const connectedMoment = renderMoment(0)
    const zenButton = screen.getAllByRole('button')[0]
    const loginButton = screen.getAllByRole('button')[1]
    const subscribeButton = screen.getAllByRole('button')[2]

    expect(zenButton.textContent).toEqual('Zen Mode')
    expect(loginButton.textContent).toEqual('Log In')
    expect(subscribeButton.textContent).toContain('Subscribe')
    expect(screen.queryByText('0 plays left today.')).toBeTruthy()
    expect(
      screen.queryByText('Log In or Subscribe to Play:'),
    ).toBeTruthy()
    fireEvent.click(subscribeButton)
    expect(store.subscribe).toHaveBeenCalled()
  })

  it('for regis or anons with credits and subs, play button should work', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: false,
      },
    })

    renderMoment(2)
    const playButton = screen.getAllByRole('button')[0]

    expect(playButton.textContent).toEqual('Play')
    fireEvent.click(playButton)

    expect(mobileNavTools.activate).toHaveBeenCalled()
    expect(trackEvent).toHaveBeenCalledWith('START_GAME')
    expect(props.close).toHaveBeenCalled()
  })

  it('for anons and regis with Infinity credits, credits should not be visible', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
        isLoggedIn: false,
      },
    })

    renderMoment(Infinity)
    expect(
      document.querySelector('.pz-moment__meta').textContent,
    ).toEqual('')
  })

  it('for subs, zen mode button should work', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
        isLoggedIn: false,
      },
    })

    renderMoment(Infinity)
    const zenModeButton = screen.getAllByRole('button')[1]

    expect(zenModeButton.textContent).toEqual('Zen Mode')
    fireEvent.click(zenModeButton)

    expect(mobileNavTools.activate).toHaveBeenCalled()
    expect(trackEvent).toHaveBeenCalledWith('START_GAME')
    expect(props.close).toHaveBeenCalled()
  })

  it('for non subs, zen mode button should trigger a zen mode event', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: false,
      },
    })

    renderMoment(Infinity)

    const zenModeButton = screen.getAllByRole('button')[1]
    expect(zenModeButton.textContent).toEqual('Zen Mode')
    fireEvent.click(zenModeButton)

    expect(trackEvent).toHaveBeenCalledWith('ZEN_MODE')
  })

  it('for regis without credits, should fire proper analytics function', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: true,
      },
      hasLoaded: true,
    })

    act(() => {
      renderMoment(0)
    })

    expect(trackImpression).toHaveBeenCalledWith(
      'hardpaywall',
      'tiles-no-plays-welcome',
    )
  })

  it('for anons without credits, should fire proper analytics function', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })

    // @ts-ignore
    abra.getVariant.mockReturnValue('0_control')

    act(() => {
      renderMoment(0)
    })

    expect(trackImpression).toHaveBeenCalledTimes(1)
    expect(trackImpression).toHaveBeenCalledWith(
      'hardpaywall',
      'tiles-no-plays-welcome',
    )
  })

  it('should display banner on Mobile Web', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    window.isHybridWebView = false
    isMobile.mockReturnValue(true)
    // @ts-ignore
    abra.getVariant.mockReturnValue('1_icon')

    renderMoment(1)

    const bannerText = screen.queryByText(
      'Play Tiles and more in the NYT Games app.',
    )

    expect(bannerText).toBeVisible()
  })

  it('should not display banner on newsreader', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    window.newsreaderAppPlatform = 'ios'
    isMobile.mockReturnValue(true)
    // @ts-ignore
    abra.getVariant.mockReturnValue('1_icon')

    renderMoment(1)

    const bannerText = screen.queryByText(
      'Play Tiles and more in the NYT Games app.',
    )

    expect(bannerText).toBeNull()
  })

  it('should not display banner on hybrid webviews', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    window.isHybridWebView = true
    isMobile.mockReturnValue(true)
    // @ts-ignore
    abra.getVariant.mockReturnValue('1_icon')

    renderMoment(1)

    const bannerText = screen.queryByText(
      'Play Tiles and more in the NYT Games app.',
    )

    expect(bannerText).toBeNull()
  })
})

// Using Meter Response
describe('TILES: WELCOME MOMENT - DELTA USERS', () => {
  const store = {
    getState: () => ({}),
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  }

  const props = {
    updateCredits: jest.fn(),
    deviceType: {
      isMobile: true,
    },
    open: jest.fn(),
    close: jest.fn(),
    startGame: jest.fn(),
    infiniteModeToggle: jest.fn(),
    links: {},
    meterResponse: {
      granted: true,
      grantReason: 'IP',
    },
  }

  const assignMock = jest.fn()

  delete window.location
  window.location = { assign: assignMock }

  beforeEach(() => {
    jest.resetAllMocks()
  })

  const renderMoment = (creditsLeft) =>
    render(
      <Provider store={store}>
        <WelcomeMoment creditsLeft={creditsLeft} {...props} />
      </Provider>,
    )

  it('for anons on Delta - should show buttons and no credit count', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: false,
      },
    })
    renderMoment(Infinity)
    const buttons = screen.getAllByRole('button')
    const loginButton = screen.getAllByRole('button')[2]
    const subscribeButton = screen.getAllByRole('button')[3]

    expect(buttons).toHaveLength(4)
    expect(loginButton.textContent).toEqual('Log In')
    expect(subscribeButton.textContent).toContain('Subscribe')
    expect(
      document.querySelector('.pz-moment__meta').textContent,
    ).toEqual('')
  })

  it('for anons on Delta - play button should show and work', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: false,
      },
    })
    renderMoment(Infinity)
    const playButton = screen.getAllByRole('button')[0]

    expect(playButton.textContent).toEqual('Play')
    fireEvent.click(playButton)
    expect(mobileNavTools.activate).toHaveBeenCalled()
    expect(trackEvent).toHaveBeenCalledWith('START_GAME')
    expect(props.close).toHaveBeenCalled()
  })

  it('for anons on Delta - zen mode button should show and work', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: false,
      },
    })
    renderMoment(Infinity)
    const zenButton = screen.getAllByRole('button')[1]

    expect(zenButton.textContent).toEqual('Zen Mode')
    fireEvent.click(zenButton)
    expect(mobileNavTools.activate).toHaveBeenCalled()
    expect(trackEvent).toHaveBeenCalledWith('START_GAME')
    expect(props.close).toHaveBeenCalled()
  })

  it('for regis on Delta - should show buttons and no credit count', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: true,
      },
    })
    renderMoment(Infinity)
    const buttons = screen.getAllByRole('button')
    const subscribeButton = screen.getAllByRole('button')[2]

    expect(buttons).toHaveLength(3)
    expect(subscribeButton.textContent).toContain('Subscribe')
    expect(
      document.querySelector('.pz-moment__meta').textContent,
    ).toEqual('')
  })

  it('for regis on Delta - play button should show and work', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: true,
      },
    })
    renderMoment(Infinity)
    const playButton = screen.getAllByRole('button')[0]

    expect(playButton.textContent).toEqual('Play')
    fireEvent.click(playButton)
    expect(mobileNavTools.activate).toHaveBeenCalled()
    expect(trackEvent).toHaveBeenCalledWith('START_GAME')
    expect(props.close).toHaveBeenCalled()
  })

  it('for regis on Delta - zen mode button should show and work', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
        isLoggedIn: true,
      },
    })
    renderMoment(Infinity)
    const zenButton = screen.getAllByRole('button')[1]

    expect(zenButton.textContent).toEqual('Zen Mode')
    fireEvent.click(zenButton)
    expect(mobileNavTools.activate).toHaveBeenCalled()
    expect(trackEvent).toHaveBeenCalledWith('START_GAME')
    expect(props.close).toHaveBeenCalled()
  })
})
