import thunk from 'redux-thunk'
import {
  createStore,
  compose,
  applyMiddleware,
  combineReducers,
} from 'redux'

import { getGameData, xhr } from 'foundation'

import { wait } from '../../../helpers/gameUtils'
import { trackImpression } from '../../../helpers/tracking'
import {
  setPlayMode,
  setPaletteName,
  trackEvent,
} from '../tilesTracker/tilesTracker'
import {
  hideLayers,
  createBoard,
  revealAllLayers,
  isAboutToBeClear,
  replaceTileLayers,
  getMatchingLayers,
  areThereAnyValidMovesOnBoard,
  checkWinState,
  generateNewLayers,
} from '../../helpers/helpers'

const gameData = getGameData()

export const initialState = {
  board: [],
  boardSize: 0,
  selected: null,
  free: null,
  currentCombo: 0,
  longestCombo: 0,
  matchedLayers: 0,
  winState: false,
  perfectSolve: true,
  message: { id: null, text: '', animate: false },
  infiniteMode: false,
  palette: {
    filename: '',
    displayName: '',
    createdBy: '',
    // as entitlements haven't loaded, start with a white background
    // so that when the actual background comes in, the change isn't so jarring
    // as going from today's background to the stored background
    bgColor: 'white',
    fontColor: '',
    selectionColor: '',
    zLayer: [],
    layers: [],
    svg: '',
  },
  allPalettes: gameData?.allPalettes || [],
  modalSystem: {
    isClosing: false,
    modal: null,
  },
}

// CONSTANTS
const BUILD_MATCH = 'BUILD_MATCH'
const MAKE_MATCH = 'MAKE_MATCH'
const CLEAR_MATCH = 'CLEAR_MATCH'
const BREAK_MATCH = 'BREAK_MATCH'
const SHOW_MESSAGE = 'SHOW_MESSAGE'
const HIDE_MESSAGE = 'HIDE_MESSAGE'
const TOGGLE_INFINITE_MODE = 'TOGGLE_INFINITE_MODE'
const REGENERATE_TILE = 'REGENERATE_TILE'
const REGENERATE_BOARD = 'REGENERATE_BOARD'
const RESET_SELECTION = 'RESET_SELECTION'
const START_GAME = 'START_GAME'
const SET_INITIAL_PALETTE = 'SET_INITIAL_PALETTE'
const UPDATE_PALETTE = 'UPDATE_PALETTE'
const OPEN_MODAL = 'OPEN_MODAL'
const CLOSE_MODAL = 'CLOSE_MODAL'
const REMOVE_MODAL = 'REMOVE_MODAL'

// ACTIONS
export const buildMatch = (id) => ({
  type: BUILD_MATCH,
  id,
})
export const makeMatch = (
  tileOneId,
  tileTwoId,
  matchingLayers,
  combo,
  state,
) => ({
  type: MAKE_MATCH,
  tileOneId,
  tileTwoId,
  matchingLayers,
  combo,
  state,
})
export const clearMatch = (
  tileOneId,
  tileTwoId,
  matchingLayers,
  combo,
  state,
) => ({
  type: CLEAR_MATCH,
  tileOneId,
  tileTwoId,
  matchingLayers,
  combo,
  state,
})
export const breakMatch = () => ({ type: BREAK_MATCH })
export const showMessage = (id, text) => ({
  type: SHOW_MESSAGE,
  id,
  text,
})
export const hideMessage = () => ({ type: HIDE_MESSAGE })
export const toggleInfiniteMode = () => ({
  type: TOGGLE_INFINITE_MODE,
})
export const regenerateTile = (id, newTile) => ({
  type: REGENERATE_TILE,
  id,
  newTile,
})
export const regenerateBoard = (board, infiniteMode) => ({
  type: REGENERATE_BOARD,
  board,
  infiniteMode,
})
export const resetSelection = () => ({
  type: RESET_SELECTION,
})
export const startGame = () => ({ type: START_GAME })
export const updatePalette = (palette) => ({
  type: UPDATE_PALETTE,
  palette,
})
export const openModal = (modal) => ({ type: OPEN_MODAL, modal })
export const closeModal = () => ({ type: CLOSE_MODAL })
export const removeModal = () => ({ type: REMOVE_MODAL })

const setInitialPalette = (palette, board) => ({
  type: SET_INITIAL_PALETTE,
  palette,
  board,
})

// HELPERS
const animateMessage =
  ({ delay, id, text }) =>
  (dispatch) => {
    wait(delay).then(() => {
      dispatch(showMessage(id, text))
      wait(700).then(() => {
        dispatch(hideMessage())
      })
    })
  }

export const generateNewBoard =
  (fullReset = false) =>
  (dispatch, getState) => {
    if (fullReset) dispatch(resetSelection())
    const { palette, infiniteMode, board } = getState()
    const boardConfig = {
      zLayer: palette.zLayer,
      layers: palette.layers,
      infiniteMode,
      boardSize: board.length,
    }

    dispatch(regenerateBoard(createBoard(boardConfig), infiniteMode))
    wait(700).then(() => dispatch(startGame()))
  }

const generateNewTile = (id) => (dispatch, getState) => {
  const { palette, board } = getState()
  dispatch(
    regenerateTile(id, generateNewLayers(palette.layers, board)),
  )
}

// INFINITE MODE
export const infiniteModeToggle = () => (dispatch, getState) => {
  const { infiniteMode, palette, board } = getState()
  const boardConfig = {
    zLayer: palette.zLayer,
    layers: palette.layers,
    infiniteMode: !infiniteMode,
    boardSize: board.length,
  }
  setPlayMode(boardConfig.infiniteMode ? 'z' : 'r') // for analytics
  dispatch(regenerateBoard(createBoard(boardConfig)))
  dispatch(toggleInfiniteMode())
  wait(700).then(() => dispatch(startGame()))
}

// BOARD SIZE
export const boardSizeToggle = () => (dispatch, getState) => {
  const { infiniteMode, palette, board } = getState()
  const boardConfig = {
    zLayer: palette.zLayer,
    layers: palette.layers,
    infiniteMode,
    boardSize: board.length === 30 ? 4 : 30,
  }

  dispatch(regenerateBoard(createBoard(boardConfig)))
  wait(700).then(() => dispatch(startGame()))
}

// PALETTE SELECTION
export const changePalette = (paletteObj) => (dispatch, getState) => {
  const { board, infiniteMode, allPalettes } = getState()
  if (!allPalettes.find((p) => p.filename === paletteObj.filename)) {
    return
  }

  xhr
    .get(`/puzzles/tiles/palette/${paletteObj.filename}`)
    .then((data) => {
      const { links, allPalettes, ...paletteData } = data

      paletteData.displayName = paletteObj.displayName
      paletteData.createdBy = paletteObj.createdBy

      const boardConfig = {
        zLayer: paletteData.zLayer,
        layers: paletteData.layers,
        infiniteMode,
        boardSize: board.length,
      }

      const searchParams = new URLSearchParams(window.location.search)

      // eslint-disable-next-line no-restricted-globals
      if (history.pushState && searchParams.get('palette')) {
        searchParams.set('palette', paletteObj.filename)
        const newurl = `${window.location.protocol}//${
          window.location.host
        }${window.location.pathname}?${searchParams.toString()}`
        window.history.pushState({ path: newurl }, '', newurl)
      }

      setPaletteName(paletteData.filename) // for analytics

      dispatch(updatePalette(paletteData))
      dispatch(regenerateBoard(createBoard(boardConfig)))
      dispatch(startGame())
      trackEvent('START_GAME', 'palette-changed')

      try {
        window.localStorage.setItem(
          'tilesPalette',
          JSON.stringify(paletteData),
        )
      } catch (e) {
        console.error('Tiles: Failed to store palette preference.', e)
      }
    })
    .catch((e) =>
      console.error('Tiles: Failed to change palette.', e),
    )
}

// GAME LOGIC
export const initiateLoop = (clicked) => (dispatch, getState) => {
  const { board, selected, currentCombo, infiniteMode, free } =
    getState()

  const nextCombo = currentCombo + 1

  if (infiniteMode && free !== null) {
    dispatch(generateNewTile(free))
  }

  if (infiniteMode && !areThereAnyValidMovesOnBoard(board)) {
    trackImpression('tiles', 'no-moves-left')

    dispatch(
      animateMessage({
        delay: 200,
        id: clicked,
        text: 'no moves available',
      }),
    )

    wait(1000).then(() => dispatch(generateNewBoard()))

    return
  }

  if (selected === null) {
    return dispatch(buildMatch(clicked))
  }

  const selectedTile = board[selected]
  const clickedTile = board[clicked]
  const matchingLayers = getMatchingLayers(selectedTile, clickedTile)

  if (
    matchingLayers &&
    isAboutToBeClear(clickedTile, matchingLayers)
  ) {
    dispatch(
      clearMatch(
        selected,
        clicked,
        matchingLayers,
        nextCombo,
        getState(),
      ),
    )
    if (!getState().winState) {
      dispatch(
        animateMessage({
          delay: 700,
          id: clicked,
          text: nextCombo < 16 ? 'go anywhere' : nextCombo.toString(),
        }),
      )
    }
  } else if (
    matchingLayers &&
    !isAboutToBeClear(clickedTile, matchingLayers)
  ) {
    dispatch(
      makeMatch(
        selected,
        clicked,
        matchingLayers,
        nextCombo,
        getState(),
      ),
    )
    dispatch(
      animateMessage({
        delay: 700,
        id: clicked,
        text: nextCombo.toString(),
      }),
    )
  } else {
    dispatch(breakMatch())
    dispatch(
      animateMessage({
        delay: 200,
        id: clicked,
        text: 'no match',
      }),
    )
  }

  if (
    infiniteMode &&
    matchingLayers &&
    isAboutToBeClear(selectedTile, matchingLayers)
  ) {
    wait(700).then(() => dispatch(generateNewTile(selected)))
  }
}

// REDUCERS
const boardReducer = (state = [], action) => {
  switch (action.type) {
    case MAKE_MATCH:
      return hideLayers(state, action)
    case CLEAR_MATCH:
      return hideLayers(state, action)
    case REGENERATE_TILE:
      return replaceTileLayers(state, action.id, action.newTile)
    case SET_INITIAL_PALETTE:
    case REGENERATE_BOARD:
      return action.board
    case START_GAME:
      return revealAllLayers(state)
    default:
      return state
  }
}

const boardSizeReducer = (state = 0, action) => {
  switch (action.type) {
    case SET_INITIAL_PALETTE:
    case REGENERATE_BOARD:
      return action.board.length
    default:
      return state
  }
}

const selectedReducer = (state = null, action) => {
  switch (action.type) {
    case BUILD_MATCH:
      return action.id
    case MAKE_MATCH:
      return action.tileTwoId
    case CLEAR_MATCH:
      return null
    case BREAK_MATCH:
      return null
    case TOGGLE_INFINITE_MODE:
      return null
    case RESET_SELECTION:
      return null
    case UPDATE_PALETTE:
      return null
    default:
      return state
  }
}

const freeReducer = (state = null, action) => {
  switch (action.type) {
    case BUILD_MATCH:
      return null
    case MAKE_MATCH:
      return null
    case CLEAR_MATCH:
      return checkWinState(action) ? null : action.tileTwoId
    case BREAK_MATCH:
      return null
    case TOGGLE_INFINITE_MODE:
      return null
    case REGENERATE_BOARD:
      return null
    case RESET_SELECTION:
      return null
    default:
      return state
  }
}

const currentComboReducer = (state = 0, action) => {
  switch (action.type) {
    case MAKE_MATCH:
      return state + 1
    case CLEAR_MATCH:
      return state + 1
    case BREAK_MATCH:
      return 0
    case REGENERATE_BOARD:
      return 0
    case TOGGLE_INFINITE_MODE:
      return 0
    default:
      return state
  }
}

const longestComboReducer = (state = 0, action) => {
  switch (action.type) {
    case MAKE_MATCH:
      return action.combo > state ? action.combo : state
    case CLEAR_MATCH:
      return action.combo > state ? action.combo : state
    case TOGGLE_INFINITE_MODE:
      return 0
    case REGENERATE_BOARD:
      return action.infiniteMode ? state : 0
    default:
      return state
  }
}

const matchedLayersReducer = (state = 0, action) => {
  switch (action.type) {
    case CLEAR_MATCH:
      return state + action.matchingLayers.length
    case MAKE_MATCH:
      return state + action.matchingLayers.length
    case REGENERATE_BOARD:
      return 0
    default:
      return state
  }
}

const winStateReducer = (state = false, action) => {
  switch (action.type) {
    case MAKE_MATCH:
      return checkWinState(action)
    case CLEAR_MATCH:
      return checkWinState(action)
    case REGENERATE_BOARD:
      return false
    default:
      return state
  }
}

const perfectSolveReducer = (state = true, action) => {
  switch (action.type) {
    case BREAK_MATCH:
      return false
    case REGENERATE_BOARD:
      return true
    default:
      return state
  }
}

const messageReducer = (state = {}, action) => {
  switch (action.type) {
    case SHOW_MESSAGE:
      return {
        id: action.id,
        text: action.text,
        animate: true,
      }
    case HIDE_MESSAGE:
      return {
        ...state,
        animate: false,
      }
    default:
      return state
  }
}

const infiniteModeReducer = (state = false, action) => {
  switch (action.type) {
    case TOGGLE_INFINITE_MODE:
      return !state
    default:
      return state
  }
}

const paletteReducer = (state = {}, action) => {
  switch (action.type) {
    case SET_INITIAL_PALETTE:
    case UPDATE_PALETTE:
      return action.palette
    default:
      return state
  }
}

const allPalettesReducer = (state = [], action) => {
  switch (action.type) {
    default:
      return state
  }
}

const modalReducer = (state = {}, action) => {
  switch (action.type) {
    case OPEN_MODAL:
      return {
        modal: action.modal,
        isClosing: false,
      }
    case CLOSE_MODAL:
      return {
        isClosing: true,
      }
    case REMOVE_MODAL:
      return {
        isClosing: false,
        modal: null,
      }
    default:
      return state
  }
}

export const rootReducer = combineReducers({
  board: boardReducer,
  boardSize: boardSizeReducer,
  selected: selectedReducer,
  free: freeReducer,
  currentCombo: currentComboReducer,
  longestCombo: longestComboReducer,
  matchedLayers: matchedLayersReducer,
  winState: winStateReducer,
  perfectSolve: perfectSolveReducer,
  message: messageReducer,
  infiniteMode: infiniteModeReducer,
  palette: paletteReducer,
  allPalettes: allPalettesReducer,
  modalSystem: modalReducer,
})

// Analytics
const percentageCompleteTracker = (store) => (next) => (action) => {
  const prevState = store.getState()
  const result = next(action)
  const nextState = store.getState()

  if (action.type === MAKE_MATCH || action.type === CLEAR_MATCH) {
    const { boardSize, board, perfectSolve, longestCombo } = nextState

    const matchesToWin = (boardSize * board[0].layers.length) / 2

    const prevPercentageComplete =
      prevState.matchedLayers / matchesToWin

    const nextPercentageComplete =
      nextState.matchedLayers / matchesToWin

    const solveTypeAndCombo = `${
      perfectSolve ? 'p' : 'i'
    }-${longestCombo}`

    if (
      prevPercentageComplete < 0.5 &&
      nextPercentageComplete >= 0.5
    ) {
      trackEvent('COMPLETE_50')
    } else if (
      prevPercentageComplete < 1 &&
      nextPercentageComplete === 1
    ) {
      trackEvent('COMPLETE_100', solveTypeAndCombo)
    }
  }

  return result
}

export const loadInitialPalette = (hasXwd) => (dispatch) => {
  const { mockBoard } = window.localStorage
  const { links, iconSVG, allPalettes, ...todaysPalette } = gameData
  let storedPalette = null

  if (hasXwd) {
    try {
      const stored = JSON.parse(
        window.localStorage.getItem('tilesPalette'),
      )

      const urlParams = new URLSearchParams(window.location.search)
      const queryParamPalette = urlParams.get('palette')
      const validPalette =
        queryParamPalette &&
        !!allPalettes.find((p) => p.filename === queryParamPalette)

      if (validPalette) {
        storedPalette = {
          ...todaysPalette,
          ...validPalette,
        }
      } else if (stored) {
        if (
          !allPalettes.find((p) => p.filename === stored.filename)
        ) {
          window.localStorage.removeItem('tilesPalette')
        } else if (stored.displayName === 'Default') {
          storedPalette = {
            ...todaysPalette,
            displayName: 'Default',
          }
        } else storedPalette = stored
      }
    } catch (e) {
      console.error('Tiles: Failed to get stored palette', e)
    }
  }

  const palette = storedPalette || todaysPalette

  const board = createBoard(palette, mockBoard)
  dispatch(setInitialPalette(palette, board))

  // Analytics
  setPlayMode('r')
  setPaletteName(palette.filename)
}

export const setUpStore = () => {
  const middleware = [
    applyMiddleware(thunk, percentageCompleteTracker),
  ]

  if (window.__REDUX_DEVTOOLS_EXTENSION__) {
    middleware.push(window.__REDUX_DEVTOOLS_EXTENSION__())
  }

  return createStore(
    rootReducer,
    initialState,
    compose(...middleware),
  )
}

// SELECTORS
export const getLongestCombo = (state) => state.longestCombo
export const getPerfectSolve = (state) => state.perfectSolve
export const getIsModalOpen = (state) =>
  state.modalSystem.modal !== null
