import { DEFAULT_PALETTE } from '../MenuOptions/shared'

import {
  initialState,
  rootReducer,
  setUpStore,
  buildMatch,
  makeMatch,
  clearMatch,
  breakMatch,
  showMessage,
  hideMessage,
  toggleInfiniteMode,
  startGame,
  loadInitialPalette,
} from './store'

jest.mock('foundation', () => ({
  xhr: {
    get: jest.fn(),
  },
  getGameData: () => ({
    zLayer: ['Z1', 'Z1'],
    layers: [['A1'], ['B1'], ['C1']],
  }),
}))

jest.mock('../tilesTracker/tilesTracker', () => ({
  trackEvent: jest.fn(),
  setPlayMode: jest.fn(),
  setPaletteName: jest.fn(),
}))

describe('TILES: STORE', () => {
  it('should set up the store', () => {
    const data = {
      zLayer: ['Z1', 'Z1'],
      layers: [['A1'], ['B1'], ['C1']],
      allPalettes: [DEFAULT_PALETTE.toLowerCase()],
    }

    const user = {
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    }

    const store = setUpStore(data, user)
    store.dispatch(loadInitialPalette(false))

    const state = store.getState()

    expect(state.board).toHaveLength(30)
    expect(state.selected).toBe(null)
    expect(state.currentCombo).toBe(0)
    expect(state.longestCombo).toBe(0)
  })

  // it('should render initial state', () => {
  //   expect(rootReducer(initialState, { type: null })).toBe(
  //     initialState,
  //   )
  // })

  it('should build a match by selecting a tile', () => {
    const firstSelectState = rootReducer(initialState, buildMatch(4))
    expect(firstSelectState.selected).toBe(4)
    const secondSelectState = rootReducer(
      firstSelectState,
      buildMatch(13),
    )
    expect(secondSelectState.selected).toBe(13)
  })

  it('should show and hide message', () => {
    const showMessageState = rootReducer(
      initialState,
      showMessage(4, 'matched'),
    )
    expect(showMessageState.message).toEqual({
      id: 4,
      text: 'matched',
      animate: true,
    })
    const hideMessageState = rootReducer(
      showMessageState,
      hideMessage(),
    )
    expect(hideMessageState.message).toEqual({
      id: 4,
      text: 'matched',
      animate: false,
    })
  })

  it('should toggle infinite mode', () => {
    const infiniteModeState = rootReducer(
      initialState,
      toggleInfiniteMode(),
    )
    expect(infiniteModeState.infiniteMode).toBe(true)
    const classicModeState = rootReducer(
      infiniteModeState,
      toggleInfiniteMode(),
    )
    expect(classicModeState.infiniteMode).toBe(false)
  })

  describe('MAKE MATCH', () => {
    let newTile0
    let newTile1
    let makeMatchState
    let makeWinState

    beforeEach(() => {
      const board = [
        {
          id: 0,
          zLayer: { id: 'Z1', hidden: false },
          layers: [
            { id: 'A1', hidden: false },
            { id: 'B2', hidden: false },
            { id: 'C3', hidden: false },
          ],
        },
        {
          id: 1,
          zLayer: { id: 'Z1', hidden: false },
          layers: [
            { id: 'A1', hidden: true },
            { id: 'B2', hidden: false },
            { id: 'C13', hidden: false },
          ],
        },
      ]

      newTile0 = {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: true },
          { id: 'C3', hidden: false },
        ],
      }

      newTile1 = {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C13', hidden: false },
        ],
      }

      const stateWithMatchingTileLayers = {
        ...initialState,
        board,
      }

      const action = makeMatch(
        0,
        1,
        [1],
        1,
        stateWithMatchingTileLayers,
      )

      const winAction = makeMatch(
        0,
        1,
        [0, 1, 2],
        1,
        stateWithMatchingTileLayers,
      )
      makeMatchState = rootReducer(
        stateWithMatchingTileLayers,
        action,
      )
      makeWinState = rootReducer(
        stateWithMatchingTileLayers,
        winAction,
      )
    })

    it('should update the board with hidden layers', () => {
      expect(makeMatchState.board[0]).toEqual(newTile0)
      expect(makeMatchState.board[1]).toEqual(newTile1)
    })

    it('should update the selected tile', () => {
      expect(makeMatchState.selected).toEqual(newTile1.id)
    })

    it('should not update free', () => {
      expect(makeMatchState.free).toEqual(null)
    })

    it('should increase matched layers', () => {
      expect(makeMatchState.matchedLayers).toEqual(1)
    })

    it('should increase the current combo', () => {
      expect(makeMatchState.currentCombo).toEqual(
        initialState.currentCombo + 1,
      )
    })

    it('should increase the longest combo', () => {
      expect(makeMatchState.longestCombo).toEqual(
        initialState.longestCombo + 1,
      )
    })

    it('should determine win state', () => {
      expect.assertions(2)

      expect(makeMatchState.winState).toEqual(false)
      expect(makeWinState.winState).toEqual(true)
    })
  })

  describe('COMPLETE MATCH', () => {
    let newTile0
    let newTile1
    let makeMatchState
    let makeWinState

    beforeEach(() => {
      const board = [
        {
          id: 0,
          zLayer: { id: 'Z1', hidden: false },
          layers: [
            { id: 'A1', hidden: false },
            { id: 'B2', hidden: false },
            { id: 'C3', hidden: false },
          ],
        },
        {
          id: 1,
          zLayer: { id: 'Z1', hidden: false },
          layers: [
            { id: 'A1', hidden: false },
            { id: 'B2', hidden: false },
            { id: 'C13', hidden: true },
          ],
        },
      ]

      newTile0 = {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C3', hidden: false },
        ],
      }

      newTile1 = {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C13', hidden: true },
        ],
      }

      const stateWithMatchingTileLayers = {
        ...initialState,
        board,
      }

      const action = clearMatch(
        0,
        1,
        [0, 1],
        1,
        stateWithMatchingTileLayers,
      )

      const winAction = clearMatch(
        0,
        1,
        [0, 1, 2],
        1,
        stateWithMatchingTileLayers,
      )
      makeMatchState = rootReducer(
        stateWithMatchingTileLayers,
        action,
      )
      makeWinState = rootReducer(
        stateWithMatchingTileLayers,
        winAction,
      )
    })

    it('should update the board with hidden layers', () => {
      expect(makeMatchState.board[0]).toEqual(newTile0)
      expect(makeMatchState.board[1]).toEqual(newTile1)
    })

    it('should make no tile selected', () => {
      expect(makeMatchState.selected).toEqual(null)
    })

    it('should make cleared tile free', () => {
      expect(makeMatchState.free).toEqual(newTile1.id)
    })

    it('should not make cleared tile free if game won', () => {
      expect(makeWinState.free).toEqual(null)
    })

    it('should increase matched layers', () => {
      expect(makeMatchState.matchedLayers).toEqual(2)
    })

    it('should increase the current combo', () => {
      expect(makeMatchState.currentCombo).toEqual(
        initialState.currentCombo + 1,
      )
    })

    it('should increase the longest combo', () => {
      expect(makeMatchState.longestCombo).toEqual(
        initialState.longestCombo + 1,
      )
    })

    it('should not alter perfect solve', () => {
      expect(makeMatchState.perfectSolve).toEqual(true)
    })

    it('should determine win state', () => {
      expect.assertions(2)

      expect(makeMatchState.winState).toEqual(false)
      expect(makeWinState.winState).toEqual(true)
    })
  })

  describe('BREAK MATCH', () => {
    let newTile0
    let newTile1
    let breakMatchState

    beforeEach(() => {
      const board = [
        {
          id: 0,
          zLayer: { id: 'Z1', hidden: false },
          layers: [
            { id: 'A11', hidden: false },
            { id: 'B12', hidden: false },
            { id: 'C13', hidden: true },
          ],
        },
        {
          id: 1,
          zLayer: { id: 'Z1', hidden: false },
          layers: [
            { id: 'A1', hidden: false },
            { id: 'B2', hidden: false },
            { id: 'C13', hidden: true },
          ],
        },
      ]

      newTile0 = {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A11', hidden: false },
          { id: 'B12', hidden: false },
          { id: 'C13', hidden: true },
        ],
      }

      newTile1 = {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: false },
          { id: 'B2', hidden: false },
          { id: 'C13', hidden: true },
        ],
      }

      const stateWithNoMatchingLayers = {
        ...initialState,
        board,
      }
      const action = breakMatch(0, 1, [])
      breakMatchState = rootReducer(stateWithNoMatchingLayers, action)
    })

    it('should not update the board with hidden layers', () => {
      expect(breakMatchState.board[0]).toEqual(newTile0)
      expect(breakMatchState.board[1]).toEqual(newTile1)
    })

    it('should make no tile selected', () => {
      expect(breakMatchState.selected).toEqual(null)
    })

    it('should have no free tile', () => {
      expect(breakMatchState.free).toEqual(null)
    })

    it('should reset the current combo', () => {
      expect(breakMatchState.currentCombo).toEqual(0)
    })

    it('should break perfect solve', () => {
      expect(breakMatchState.perfectSolve).toEqual(false)
    })
  })

  describe('START GAME', () => {
    const board = [
      {
        id: 0,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A11', hidden: true },
          { id: 'B12', hidden: true },
          { id: 'C13', hidden: true },
        ],
      },
      {
        id: 1,
        zLayer: { id: 'Z1', hidden: false },
        layers: [
          { id: 'A1', hidden: true },
          { id: 'B2', hidden: true },
          { id: 'C13', hidden: true },
        ],
      },
    ]

    it('should reveal all board layers after game start', () => {
      expect.assertions(2)

      const stateWithHiddenLayers = Object.assign(initialState, {
        board,
      })
      const action = startGame()
      const startGameState = rootReducer(
        stateWithHiddenLayers,
        action,
      )
      startGameState.board.forEach((tile) => {
        const allRevealed = tile.layers.every(
          (layer) => layer.hidden === false,
        )
        expect(allRevealed).toBe(true)
      })
    })
  })
})
