import MockDate from 'mockdate'

import { abra } from 'foundation'

import {
  setTilesCredits,
  getCurrentCredits,
  exposeTilesCreditsTest,
} from './tilesCredits'

const moment = require('moment')

jest.mock('foundation', () => ({
  abra: {
    get: jest.fn(),
    reportExposure: jest.fn(),
  },
}))

describe('TILES: CHECKING FOR CREDITS', () => {
  const getCredits = () =>
    parseInt(window.localStorage.getItem('tilesCredits'))
  const userType = {}
  const meterResponse = {
    granted: false,
    grantReason: 'METER_LIMIT',
  }

  afterEach(() => {
    window.localStorage.clear()
    MockDate.reset()
  })

  it('should return infinite credits when user is a subscriber', () => {
    abra.get.mockReturnValue('2_games')
    userType.isLoggedIn = true
    userType.hasXwd = true

    expect(setTilesCredits(userType, meterResponse)).toEqual(Infinity)
  })

  it('should decrement credits for each same-day visit', () => {
    MockDate.set(moment('2021-01-01'))

    userType.isLoggedIn = true
    userType.hasXwd = false

    const visits = [
      setTilesCredits(userType, meterResponse),
      setTilesCredits(userType, meterResponse),
      setTilesCredits(userType, meterResponse),
    ]

    const expectedCredits = [2, 1, 0]

    expect(visits).toEqual(expectedCredits)
  })

  it('should reset credits each day', () => {
    userType.isLoggedIn = true
    userType.hasXwd = false
    window.localStorage.setItem('tilesDate', 'Tue Feb 05 2019')
    window.localStorage.setItem('tilesCredits', '0')

    MockDate.set(moment('2019-02-06'))
    setTilesCredits(userType, meterResponse)
    expect(getCredits()).toEqual(2)
  })

  it('should fail open if local storage read breaks', () => {
    /* eslint-disable */
    const getItemSpy = jest.spyOn(Storage.prototype, 'getItem')
    /* eslint-enable */

    getItemSpy.mockImplementation(() => {
      throw new Error('overflow')
    })

    expect(() => {
      expect(setTilesCredits(userType, meterResponse)).toEqual(
        Infinity,
      )
    }).not.toThrow()

    getItemSpy.mockRestore()
  })

  it('should fail open if local storage write breaks', () => {
    /* eslint-disable */
    const setItemSpy = jest.spyOn(Storage.prototype, 'setItem')
    /* eslint-enable */

    setItemSpy.mockImplementation(() => {
      throw new Error('overflow')
    })

    expect(() => {
      expect(setTilesCredits(userType, meterResponse)).toEqual(
        Infinity,
      )
    }).not.toThrow()

    setItemSpy.mockRestore()
  })

  it('should expose test if user is anon', () => {
    userType.hasXwd = false
    userType.isLoggedIn = false
    exposeTilesCreditsTest(userType)

    expect(abra.reportExposure).toHaveBeenCalledWith(
      'GAMES_TP3_TilesRegiWall2',
    )
  })

  it('should get correct number of credits for registered users', () => {
    MockDate.set(moment('2020-11-20'))

    userType.isLoggedIn = true
    userType.hasXwd = false

    const visits = [
      setTilesCredits(userType, meterResponse),
      getCurrentCredits(userType, meterResponse),
      setTilesCredits(userType, meterResponse),
      getCurrentCredits(userType, meterResponse),
    ]

    const expectedCredits = [2, 2, 1, 1]

    expect(visits).toEqual(expectedCredits)
  })

  it('should get correct number of credits for anonymous users', () => {
    MockDate.set(moment('2020-11-20'))

    userType.isLoggedIn = false
    userType.hasXwd = false

    const visits = [
      setTilesCredits(userType, meterResponse),
      getCurrentCredits(userType, meterResponse),
      setTilesCredits(userType, meterResponse),
      getCurrentCredits(userType, meterResponse),
    ]

    const expectedCredits = [1, 1, 0, 0]

    expect(visits).toEqual(expectedCredits)
  })

  it('should reset credits when user logs in or registers', () => {
    MockDate.set(moment('2021-01-01'))

    userType.isLoggedIn = false
    userType.hasXwd = false

    const loggedOutCredits = [
      setTilesCredits(userType, meterResponse),
      setTilesCredits(userType, meterResponse),
    ]

    userType.isLoggedIn = true

    const loggedInCredits = [
      setTilesCredits(userType, meterResponse),
      setTilesCredits(userType, meterResponse),
    ]

    expect(loggedOutCredits).toEqual([1, 0])
    expect(loggedInCredits).toEqual([2, 1])
  })

  it('getCurrentCredits should reset credits if number of credits cannot be retrieved', () => {
    MockDate.set(moment('2021-01-01'))
    userType.isLoggedIn = true
    userType.hasXwd = false

    getCurrentCredits(userType, meterResponse)
    expect(getCredits()).toEqual(2)
  })

  it('getCurrentCredits should reset credits if the credits stored are not from today', () => {
    userType.isLoggedIn = true
    userType.hasXwd = false
    window.localStorage.setItem('tilesDate', 'Tue Feb 05 2021')
    window.localStorage.setItem('tilesCredits', '0')
    window.localStorage.setItem('tilesUserStatus', 'regi')

    MockDate.set(moment('2021-02-06'))
    getCurrentCredits(userType, meterResponse)
    expect(getCredits()).toEqual(2)
  })

  it('getCurrentCredits should reset credits if the user was anonymous and has logged in or registered', () => {
    MockDate.set(moment('2021-01-01'))

    userType.isLoggedIn = false
    userType.hasXwd = false

    const loggedOutCredits = getCurrentCredits(
      userType,
      meterResponse,
    )

    userType.isLoggedIn = true

    const loggedInCredits = getCurrentCredits(userType, meterResponse)

    expect(loggedOutCredits).toEqual(1)
    expect(loggedInCredits).toEqual(2)
  })

  it('getCurrentCredits should reset credits if the user was logged in as regi and has logged out', () => {
    MockDate.set(moment('2021-01-01'))

    userType.isLoggedIn = false
    userType.hasXwd = false

    const anonVisit = getCurrentCredits(userType, meterResponse)
    userType.isLoggedIn = true // log in
    const loggedInVisit = getCurrentCredits(userType, meterResponse)
    userType.isLoggedIn = false // log out
    const loggedOutVisit = getCurrentCredits(userType, meterResponse)

    expect(anonVisit).toEqual(1)
    expect(loggedInVisit).toEqual(2)
    expect(loggedOutVisit).toEqual(1)
  })

  // Using Meter Response
  it('should return infinite credits for anons on Delta', () => {
    userType.isLoggedIn = false
    userType.hasXwd = false
    meterResponse.granted = true

    expect(setTilesCredits(userType, meterResponse)).toEqual(Infinity)
  })

  it('should return infinite credits for regis on Delta', () => {
    userType.isLoggedIn = true
    userType.hasXwd = false
    meterResponse.granted = true

    expect(setTilesCredits(userType, meterResponse)).toEqual(Infinity)
  })
})
