import { track } from 'foundation'

import { DEFAULT_PALETTE } from '../MenuOptions/shared'

export const EVENT_TYPE = 'type'
export const WHERE_IT_HAPPENED = 'where'
export const WHAT_IT_DOES = 'what'
export const GAME_ID = 'id'
export const ADDITIONAL_INFO = 'extra'

let playMode = 'r'
let paletteName = DEFAULT_PALETTE.toLowerCase()

/**
 * For event specs
 * @see https://docs.google.com/spreadsheets/d/1ia9nBlnUrVi_jkE-CYsCoKElSweGi8MWlcRJWSNFG0E/edit#gid=420489817
 */
const events = {
  START_GAME: {
    // clicks "Play" or "Zen Mode" button on welcome modal
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'start-game',
  },
  RULES: {
    // clicks "Rules" button on welcome modal
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'how-to-play',
  },
  HELP: {
    // clicks "Help" button in toolbar
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'help',
  },
  COMPLETE_50: {
    // clears 50% of board in regular mode
    [EVENT_TYPE]: 'complete',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: '50%-complete',
  },
  COMPLETE_100: {
    // clears 100% of board in regular mode
    [EVENT_TYPE]: 'complete',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: '100%-complete',
  },
  PLAY_AGAIN: {
    // clicks "Play again" button on congrats modal
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'play-again',
  },
  SETTINGS: {
    // clicks "settings" during Tiles gameplay
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'settings',
  },
  ZEN_MODE: {
    // clicks "Zen Mode" on Welcome modal as non sub
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'zen-mode-subscribe',
  },
  SUBSCRIBE: {
    // clicks "Subscribe"
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'subscribe',
  },
  SUBSCRIBE_CONGRATS: {
    // clicks "Subscribe" from Congrats modal
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'subscribe-congrats',
  },
  LOGIN: {
    // clicks "Login"
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'log-in',
  },
  BACK: {
    // clicks back button
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'back',
  },
  BACK_TO_HUB: {
    // clicks "view all games"
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'tiles',
    [WHAT_IT_DOES]: 'back-to-hub',
  },
}

export const setPlayMode = (mode) => {
  playMode = mode
}

export const setPaletteName = (name) => {
  paletteName = name
}

export const getPuzzleID = () => `${playMode}-${paletteName}`

export const trackModuleInteraction = (evt) => {
  const moduleObj = {
    module: {
      name: evt[EVENT_TYPE] || 'click',
      context: evt[GAME_ID],
      label: evt[ADDITIONAL_INFO],
      element: {
        name: evt[WHERE_IT_HAPPENED],
        label: evt[WHAT_IT_DOES],
      },
    },
    eventData: {
      pageType: 'game',
      type: evt[EVENT_TYPE] || 'click',
      trigger: 'module',
      // important to leave an empty string for `status`
      // for the event to pass correctly to google analytics
      // consult with someone from tracking team before removing
      // and confirm that the events are still available in big query
      status: '',
    },
  }

  track('moduleInteraction', moduleObj)
}

export const trackEvent = (name, extra = '') => {
  if (!events[name]) {
    // we dont throw here so that we dont crash other front-end code for tracking fail
    console.error(
      new Error(
        `unrecognized tracking event '${name}' in:\n${Object.keys(
          events,
        ).join(', ')}`,
      ),
    )
    return
  }

  const evt = {
    [GAME_ID]: getPuzzleID(),
    [ADDITIONAL_INFO]: extra,
    ...events[name],
  }

  trackModuleInteraction(evt)
}
