import React, { useContext } from 'react'
import { createPortal } from 'react-dom'
import pt from 'prop-types'
import { connect } from 'react-redux'

import useDeltaBanner from 'shared/marketing/useDeltaBanner'
import { useUser } from 'user'
import { trackImpression } from 'helpers/tracking'

import TopSaleBanner from '../../../server/views/components/Marketing/Sale/TopSaleBanner'
import useSaleBannerHybridSubscribe from '../../shared/hooks/use-sale-banner-hybrid-subscribe'
import { wait } from '../../helpers/gameUtils'
import { isLastPuzzleInPack } from '../helpers/helpers'
import { savePackCompletion } from '../helpers/progress'
import { trackEvent } from '../vertexTracker/vertexTracker'
import { showCongrats } from '../Concierge/conciergeHelpers'
import { Modals, modalSystem } from '../modals/Modals'
import Game from '../Game/Game'
import { Toolbar } from '../Toolbar/Toolbar'
import Concierge from '../Concierge/Concierge'
import {
  resetGame,
  setCurrentPuzzle,
  completeTutorial,
  debugCompleteGame,
  nextTutorialPuzzle,
  completeStarterPack,
  nextStarterPackPuzzle,
} from '../store/store'
import { MeterExperimentsContext } from '../meter/MeterExperimentsProvider'

const CONGRATS_WAIT = 3000 // 500ms zoom out and center animation then 2500ms wait

const App = ({
  id,
  isMobile,
  resetGame,
  isComplete,
  dailyPuzzle,
  tutorialPuzzles,
  setCurrentPuzzle,
  completeTutorial,
  debugCompleteGame,
  starterPackPuzzles,
  nextTutorialPuzzle,
  completeStarterPack,
  hasCompletedTutorial,
  nextStarterPackPuzzle,
  hasCompletedStarterPack,
}) => {
  const [congrats, setCongrats] = React.useState('daily') // because of transition weirdness, congrats can't be tied to state
  const [hasStartedGame, startGame] = React.useState(false) // so that congrats doesn't appear on first rerender

  const { userType } = useUser()

  const { updateMeterLevel, meterGranted } = useContext(
    MeterExperimentsContext,
  )

  const saveAndCompleteTutorial = () => {
    completeTutorial(true)
    setCurrentPuzzle(starterPackPuzzles[0])
    savePackCompletion('tutorial', true)
  }

  const saveAndCompleteStarterPack = () => {
    completeStarterPack(true)
    updateMeterLevel(hasCompletedStarterPack)

    if (userType.hasXwd || meterGranted) setCurrentPuzzle(dailyPuzzle)

    savePackCompletion('starter-pack', true)
  }

  /**
   * METER LOGIC
   */
  React.useEffect(() => {
    // On load, update Meter provider with most recent level info
    updateMeterLevel(hasCompletedStarterPack)
  }, [hasCompletedStarterPack])

  /*
    TUTORIAL & STARTER PACK FLOW
  */

  React.useEffect(() => {
    if (!isComplete) return

    if (!hasCompletedTutorial) {
      if (isLastPuzzleInPack(id, tutorialPuzzles)) {
        setCongrats('tutorial')
        trackImpression('congrats-modal', 'vertex')

        wait(CONGRATS_WAIT)
          .then(showCongrats)
          .then(saveAndCompleteTutorial)
      } else {
        wait(600).then(nextTutorialPuzzle)
      }
    } else if (!hasCompletedStarterPack) {
      if (isLastPuzzleInPack(id, starterPackPuzzles)) {
        setCongrats('starter pack')
        trackImpression('congrats-modal', 'vertex')
        trackImpression('next-game', 'vertex', 'spelling-bee')

        wait(CONGRATS_WAIT)
          .then(showCongrats)
          .then(saveAndCompleteStarterPack)
      } else {
        setCongrats('midway starter pack')
        trackImpression('congrats-modal', 'vertex')
        trackImpression('next-game', 'vertex', 'spelling-bee')

        wait(CONGRATS_WAIT)
          .then(showCongrats)
          .then(nextStarterPackPuzzle)
      }
    } else if (hasStartedGame) {
      setCongrats('daily')
      trackImpression('congrats-modal', 'vertex')
      trackImpression('next-game', 'vertex', 'spelling-bee')

      wait(CONGRATS_WAIT).then(showCongrats)
    }
  }, [isComplete])

  /*
    ANALYTICS
  */
  const getGameType = () => {
    if (!hasCompletedTutorial && !hasCompletedStarterPack) {
      return 'tutorial'
    }

    if (hasCompletedTutorial && !hasCompletedStarterPack) {
      return 'starter-pack'
    }

    return 'daily'
  }

  const track = (event) => trackEvent(event, id, getGameType())

  /*
    TOOLBAR PROPS
  */
  const resetGameAndTrack = () => {
    resetGame()
    track('RESET')
  }

  const skipTutorial = () => {
    setCongrats('tutorial')
    showCongrats().then(saveAndCompleteTutorial)
    track('SKIP_TUTORIAL')
  }

  const openRulesModal = () => {
    modalSystem.open('help')
    track('RULES')
  }

  /*
    CONCIERGE PROPS
  */
  const conciergeProps = {
    track,
    congrats,
    startGame,
    openRulesModal,
    hasStartedGame,
    saveAndCompleteTutorial,
    tutorialPuzzles,
    hasCompletedDailyPuzzle: isComplete,
  }

  const { willShowDeltaBanner, portalEl } = useDeltaBanner()

  useSaleBannerHybridSubscribe()

  return (
    <div className="geo-main-container">
      {willShowDeltaBanner &&
        portalEl &&
        createPortal(
          <TopSaleBanner url={document.location.href} isDelta />,
          portalEl,
        )}
      <Toolbar
        resetGame={resetGameAndTrack}
        debugCompleteGame={debugCompleteGame}
        skipTutorial={
          hasStartedGame && !hasCompletedTutorial
            ? skipTutorial
            : null
        }
        openRulesModal={openRulesModal}
      />
      <Modals />
      <Concierge {...conciergeProps}>
        <Game isMobile={isMobile} track={track} />
      </Concierge>
    </div>
  )
}

App.propTypes = {
  id: pt.string.isRequired,
  isMobile: pt.bool.isRequired,
  resetGame: pt.func.isRequired,
  isComplete: pt.bool.isRequired,
  completeTutorial: pt.func.isRequired,
  dailyPuzzle: pt.shape({}).isRequired,
  setCurrentPuzzle: pt.func.isRequired,
  debugCompleteGame: pt.func.isRequired,
  nextTutorialPuzzle: pt.func.isRequired,
  completeStarterPack: pt.func.isRequired,
  hasCompletedTutorial: pt.bool.isRequired,
  nextStarterPackPuzzle: pt.func.isRequired,
  hasCompletedStarterPack: pt.bool.isRequired,
  tutorialPuzzles: pt.arrayOf(pt.shape({})).isRequired,
  starterPackPuzzles: pt.arrayOf(pt.shape({})).isRequired,
}

const mapStateToProps = ({
  id,
  isComplete,
  dailyPuzzle,
  tutorialPuzzles,
  starterPackPuzzles,
  hasCompletedTutorial,
  hasCompletedStarterPack,
}) => ({
  id,
  isComplete,
  dailyPuzzle,
  tutorialPuzzles,
  starterPackPuzzles,
  hasCompletedTutorial,
  hasCompletedStarterPack,
})

const mapDispatchToProps = {
  resetGame,
  setCurrentPuzzle,
  completeTutorial,
  debugCompleteGame,
  nextTutorialPuzzle,
  completeStarterPack,
  nextStarterPackPuzzle,
}

export default connect(mapStateToProps, mapDispatchToProps)(App)
