import React from 'react'
import pt from 'prop-types'
import cn from 'classnames'

import { colors, zoomRange } from '../constants'
import { wait } from '../../helpers/gameUtils'
import usePrevious from '../usePrevious/usePrevious'
import { getVertexSpecs, getArtEdges } from '../helpers/canvasHelpers'
import {
  isLineIncluded,
  isLastPuzzleInPack,
} from '../helpers/helpers'

import createCanvases from './createCanvases'

const CanvasManager = ({ puzzleState, isMobile, actions, track }) => {
  const {
    id,
    shapes,
    vertices,
    isComplete,
    puzzleType,
    drawnLines,
    drawnShapes,
    preDrawnLines,
    preDrawnShapes,
    tutorialPuzzles,
    pendingConnections,
  } = puzzleState

  const height = '100vh'
  const points = Object.values(vertices)

  const canvasContainerRef = React.useRef()
  const apiRef = React.useRef()

  const prevDrawnLines = usePrevious(drawnLines) || []
  const prevDrawnShapes = usePrevious(drawnShapes) || []

  const [zoom, setZoom] = React.useState(zoomRange.min)

  /**
   * HELPER FUNCTIONS
   */

  const trackCompletion = (
    prevDrawnShapes,
    currentDrawnShapes,
    shapes,
  ) => {
    const prevPercentage =
      (prevDrawnShapes.length / Object.keys(shapes).length) * 100

    const currentPercentage =
      (currentDrawnShapes.length / Object.keys(shapes).length) * 100

    if (prevPercentage < 100 && currentPercentage === 100) {
      track('COMPLETE_100')
    } else if (prevPercentage < 80 && currentPercentage >= 80) {
      track('COMPLETE_80')
    } else if (prevPercentage < 60 && currentPercentage >= 60) {
      track('COMPLETE_60')
    } else if (prevPercentage < 40 && currentPercentage >= 40) {
      track('COMPLETE_40')
    } else if (prevPercentage < 20 && currentPercentage >= 20) {
      track('COMPLETE_20')
    }
  }

  const drawPuzzle = () => {
    points.forEach((v) => {
      apiRef.current.addPoint(v)
    })

    preDrawnLines.forEach((line) => {
      apiRef.current.addLine(
        vertices[line[0]].coords,
        vertices[line[1]].coords,
      )
    })

    preDrawnShapes.forEach((shape) => {
      apiRef.current.addTriangle(
        vertices[shape.vertices[0]].coords,
        vertices[shape.vertices[1]].coords,
        vertices[shape.vertices[2]].coords,
        shape.color,
        shape.vertices,
        shape.id,
        true,
      )
    })

    apiRef.current.drawLines()
  }

  const resetLines = () => {
    apiRef.current.resetLines()
    preDrawnLines.forEach((line) => {
      apiRef.current.addLine(
        vertices[line[0]].coords,
        vertices[line[1]].coords,
      )
    })
    apiRef.current.drawLines()
  }

  const redrawLines = () => {
    apiRef.current.resetLines()

    preDrawnLines.forEach((line) => {
      apiRef.current.addLine(
        vertices[line[0]].coords,
        vertices[line[1]].coords,
      )
    })

    drawnLines.forEach((line) =>
      apiRef.current.addLine(
        apiRef.current.points[line[0]].coords,
        apiRef.current.points[line[1]].coords,
      ),
    )

    apiRef.current.drawLines()
  }

  const resetShapes = () => {
    apiRef.current.points.forEach((point) => {
      point.completedShapes = []
      const { radius, tapArea } = getVertexSpecs(
        apiRef.current.zoom,
        point,
      )
      point.radius = radius
      point.tapArea = tapArea
    })

    apiRef.current.resetTriangles()

    preDrawnShapes.forEach((shape) => {
      apiRef.current.addTriangle(
        vertices[shape.vertices[0]].coords,
        vertices[shape.vertices[1]].coords,
        vertices[shape.vertices[2]].coords,
        shape.color,
        shape.vertices,
        shape.id,
        true,
      )
    })

    apiRef.current.drawTriangles()
  }

  const addNewLines = (newlyDrawnLines) => {
    newlyDrawnLines.forEach((line) => {
      const newLine = apiRef.current.addLine(
        vertices[line[0]].coords,
        vertices[line[1]].coords,
      )

      newLine.draw()
    })
  }

  const addNewShapes = (newShapes) => {
    newShapes.forEach((shapeId) => {
      const newTriangle = apiRef.current.addTriangle(
        apiRef.current.points[shapes[shapeId].vertices[0]].coords,
        apiRef.current.points[shapes[shapeId].vertices[1]].coords,
        apiRef.current.points[shapes[shapeId].vertices[2]].coords,
        shapes[shapeId].color,
        shapes[shapeId].vertices,
        shapes[shapeId].id,
      )
      newTriangle.draw()
    })
  }

  /**
   * HOOKS
   */
  React.useEffect(() => {
    const W = 800
    const H = 1400 // 1200px for puzzle + 100px for ShapeCounter height
    const canvasesConfig = {
      width: W,
      height: H,
      isMobile,
      artEdges: getArtEdges(points),
    }

    const canvases = createCanvases(
      canvasContainerRef.current,
      canvasesConfig,
      actions,
    )
    apiRef.current = canvases
    drawPuzzle()

    return canvases.destroy
  }, [vertices]) // If the points of the puzzle change, then it must be a new puzzle

  React.useEffect(() => {
    // If current is greater than previous, we should add new lines to canvas
    if (drawnLines.length > prevDrawnLines.length) {
      const newlyDrawnLines = drawnLines.slice(prevDrawnLines.length)
      addNewLines(newlyDrawnLines)
      return
    }

    // We want to make sure here that the user is trying to reset the puzzle
    // And not trying to clear their first line
    const isResettingPuzzle =
      prevDrawnLines.length > 2 &&
      drawnLines.length === 0 &&
      !apiRef.current.cleared

    if (isResettingPuzzle) {
      resetLines()
      apiRef.current.selected = null
      apiRef.current.clearCurrentLineCanvas()

      return
    }

    // If current is less than previous, the user is deleting lines and
    // we should remove them from the canvas
    if (drawnLines.length < prevDrawnLines.length) {
      const clearedLines = prevDrawnLines.filter(
        (prevLine) =>
          !isLineIncluded(prevLine, drawnLines.concat(preDrawnLines)),
      )

      // If there is a point set as "selected",
      // that means a user double tapped a point
      // which means we need to reel back in all the lines to that point
      if (apiRef.current.selected !== null) {
        const clearedPoint =
          apiRef.current.points[apiRef.current.selected]
        const clearedStart = clearedPoint.coords[0]
        const clearedEnd = clearedPoint.coords[1]

        clearedLines.forEach((clearedLine) => {
          const lineObj = apiRef.current.addLine(
            apiRef.current.points[clearedLine[0]].coords,
            apiRef.current.points[clearedLine[1]].coords,
            false,
            true,
          )

          const reverse =
            clearedStart === lineObj.b[0] &&
            clearedEnd === lineObj.b[1]

          apiRef.current.removeLine(clearedLine)
          lineObj
            .reelIn(reverse, apiRef.current.drawLines, clearedPoint)
            .start()
        })
      }

      // If there is no point set as "selected"
      // that means the user pressed the 'Undo' button
      // so we clear the line that was drawn last
      if (
        apiRef.current.selected === null &&
        clearedLines.length > 0
      ) {
        const clearedLine = clearedLines[0]
        const lineObj = apiRef.current.addLine(
          apiRef.current.points[clearedLine[0]].coords,
          apiRef.current.points[clearedLine[1]].coords,
          false,
          true,
        )
        apiRef.current.removeLine(clearedLines[0])
        lineObj.reelIn(false, apiRef.current.drawLines).start()
      }

      // TODO: Rethink this - just a temporary fix for weird issue when
      // line stays on canvas after clearing it
      wait(200).then(() => {
        redrawLines()
        apiRef.current.cleared = null
        apiRef.current.drawPoints() // to make sure cleared point doesn't stay red
      })
    }
  }, [drawnLines])

  React.useEffect(() => {
    // If current is less than previous, we must be trying to reset
    if (drawnShapes.length < prevDrawnShapes.length) {
      resetShapes()
    }

    // Otherwise, only new shapes should be added to canvas
    const newShapes = drawnShapes.slice(prevDrawnShapes.length)
    addNewShapes(newShapes)
    trackCompletion(prevDrawnShapes, drawnShapes, shapes)
  }, [drawnShapes])

  React.useEffect(() => {
    // If pending connections change
    // change point size to correspond to new number
    // and redraw points
    apiRef.current.points.forEach((p) => {
      if (pendingConnections[p.id] === p.pendingConnections) {
        return
      }

      p.pendingConnections = pendingConnections[p.id]

      const {
        fontSize,
        radius,
        innerStroke,
        outerStroke,
        lineDashSegments,
        tapArea,
      } = getVertexSpecs(apiRef.current.zoom, p)

      p.fontSize = fontSize
      p.radius = p.overrides.radius || radius
      p.innerStroke = innerStroke
      p.outerStroke = outerStroke
      p.lineDashSegments = lineDashSegments
      p.tapArea = p.overrides.tapArea || tapArea
    })

    apiRef.current.drawPoints()
  }, [pendingConnections])

  React.useEffect(() => {
    if (!isComplete) return

    apiRef.current.zoomOutAnimation().start()
    setZoom(apiRef.current.zoom)

    // NOTE: The tutorial puzzle is a special case here.
    // Each "puzzle" though separate in the code,
    // is conceptually made up of different stages
    // of the same puzzle. Therefore we do not want to
    // clear the lines unless it is the last one.
    if (
      puzzleType !== 'tutorial' ||
      isLastPuzzleInPack(id, tutorialPuzzles)
    ) {
      apiRef.current.resetLines()
    }
  }, [isComplete])

  const UndoButton = () => (
    <button
      type="button"
      className="geo-undo"
      onClick={actions.undoLine}
    >
      <p>Undo</p>
    </button>
  )

  const ZoomControls = () => (
    <div className="geo-zoom-panel">
      <button
        type="button"
        className={cn({ disabled: zoom === zoomRange.max })}
        onClick={() => {
          setZoom(apiRef.current.setZoom(1.25))
        }}
      >
        <p>+</p>
      </button>
      <button
        type="button"
        className={cn({ disabled: zoom === zoomRange.min })}
        onClick={() => {
          setZoom(apiRef.current.setZoom(1.25, 'out'))
        }}
      >
        <p>–</p>
      </button>
    </div>
  )

  return (
    <div>
      <div
        ref={canvasContainerRef}
        className="geo-canvas"
        id="js-hook-geo-canvas"
        style={{
          width: '100vw',
          height,
          background: `${colors.tan}`,
          position: 'relative',
        }}
      />
      <UndoButton />
      {!isMobile && <ZoomControls />}
    </div>
  )
}

CanvasManager.propTypes = {
  actions: pt.shape({
    drawLine: pt.func,
    undoLines: pt.func,
    clearConnections: pt.func,
    undoLine: pt.func,
  }),
  puzzleState: pt.shape({
    vertices: pt.shape({
      id: pt.number,
      coords: pt.arrayOf(pt.number),
      shapes: pt.arrayOf(pt.number),
    }),
    shapes: pt.shape({
      id: pt.number,
      color: pt.string,
      lines: pt.arrayOf(pt.arrayOf(pt.number)),
    }),
    pendingConnections: pt.shape({}),
    drawnShapes: pt.arrayOf(pt.number),
    drawnLines: pt.arrayOf(pt.arrayOf(pt.number)),
    preDrawnLines: pt.shape({
      forEach: pt.func,
    }),
    preDrawnShapes: pt.shape({
      forEach: pt.func,
    }),
    tutorialPuzzles: pt.arrayOf(
      pt.shape({
        id: pt.string,
      }),
    ),
    id: pt.string,
    isComplete: pt.bool,
    puzzleType: pt.string,
  }).isRequired,
  track: pt.func.isRequired,
  isMobile: pt.bool.isRequired,
}

export default CanvasManager
