import Hammer from 'hammerjs'
import TWEEN from '@tweenjs/tween.js'

import { colors, zoomRange } from '../constants'
import {
  prop,
  getCenter,
  isUsingMultiTouch,
  doesLineIntersect,
} from '../helpers/helpers'
import {
  getVertexSpecs,
  getVertexStyles,
  getLineSpecs,
} from '../helpers/canvasHelpers'

const ARC_FULL = 2 * Math.PI
const PIXEL_RATIO = window.devicePixelRatio
const TOOLBAR_HEIGHT = 45
const THEME_BAR_HEIGHT = 100

const createCanvases = (elm, options, actions) => {
  const pointsCanvasElm = document.createElement('canvas')
  const pointsCtx = pointsCanvasElm.getContext('2d')

  const trianglesCanvasElm = document.createElement('canvas')
  const trianglesCtx = trianglesCanvasElm.getContext('2d')

  const linesCanvasElm = document.createElement('canvas')
  const linesCtx = linesCanvasElm.getContext('2d')

  const currentLineCanvasElm = document.createElement('canvas')
  const currentLineCtx = currentLineCanvasElm.getContext('2d')

  const focusX = options.width / 2
  const focusY =
    options.height / 2 - (THEME_BAR_HEIGHT + TOOLBAR_HEIGHT)

  const { width, height, isMobile, artEdges } = options

  const c = Object.create(
    {},
    {
      zoom: prop('_zoom', 1),
      focusX: prop('_focus_x', focusX),
      focusY: prop('_focus_y', focusY),
      minFocusX: prop('_min_focus_x', artEdges.minX),
      maxFocusX: prop('_max_focus_x', artEdges.maxX),
      minFocusY: prop('_min_focus_y', artEdges.minY),
      maxFocusY: prop('_max_focus_y', artEdges.maxY),
      artWidth: prop('_art_w', width),
      artHeight: prop('_art_h', height),
      isMobile: prop('_isMobile', isMobile),
      canvasWidth: prop('_canvas_w', width),
      canvasHeight: prop('_canvas_h', height),
      hover: {
        writable: true,
        configurable: true,
        value: null,
      },
      selected: {
        writable: true,
        configurable: true,
        value: null,
      },
      cleared: {
        writable: true,
        configurable: true,
        value: null,
      },
      isAnimating: {
        writable: true,
        configurable: true,
        value: null,
      },

      handleResize: {
        value() {
          const rect = pointsCanvasElm.getBoundingClientRect()
          const width = rect.width * PIXEL_RATIO
          const height = rect.height * PIXEL_RATIO

          pointsCanvasElm.setAttribute('width', width)
          pointsCanvasElm.setAttribute('height', height)

          trianglesCanvasElm.setAttribute('width', width)
          trianglesCanvasElm.setAttribute('height', height)

          linesCanvasElm.setAttribute('width', width)
          linesCanvasElm.setAttribute('height', height)

          currentLineCanvasElm.setAttribute('width', width)
          currentLineCanvasElm.setAttribute('height', height)

          this.top = rect.top
          this.left = rect.left
          this.canvasWidth = width
          this.canvasHeight = height
        },
      },

      handlePinchStart: {
        value(e) {
          e.preventDefault()

          // set center of pinch
          let center
          if (this.isMobile) {
            if (e.touches) {
              center = getCenter(e.touches)
            }
          } else {
            center = { x: e.clientX, y: e.clientY }
          }

          this._initialFocus = {
            x: this.focusX,
            y: this.focusY,
          }

          this._initialZoom = this.zoom

          if (
            center &&
            typeof center.x !== 'undefined' &&
            typeof center.y !== 'undefined'
          ) {
            this.clientX = center.x
            this.clientY = center.y
          }
        },
      },

      handlePinch: {
        value(e) {
          e.preventDefault()

          const scaleAdjustment = this._scaleAdjustment

          if (
            typeof this._initialFocus === 'undefined' ||
            this._initialFocus === null
          ) {
            // make sure to set initial focus and zoom if
            // if web touch has not been detected yet
            this._initialFocus = {
              x: this.focusX,
              y: this.focusY,
            }

            this._initialZoom = this.zoom
          }

          const nextFocusX =
            this._initialFocus.x -
            (e.deltaX / scaleAdjustment) * PIXEL_RATIO

          const nextFocusY =
            this._initialFocus.y -
            (e.deltaY / scaleAdjustment) * PIXEL_RATIO

          // TODO: this causes 3 updates to helper function
          this.setZoom(e.scale)
          if (nextFocusX >= this.maxFocusX) {
            this.focusX = this.maxFocusX
          } else if (nextFocusX <= this.minFocusX) {
            this.focusX = this.minFocusX
          } else {
            this.focusX = nextFocusX
          }

          if (nextFocusY >= this.maxFocusY) {
            this.focusY = this.maxFocusY
          } else if (nextFocusY <= this.minFocusY) {
            this.focusY = this.minFocusY
          } else {
            this.focusY = nextFocusY
          }
        },
      },

      handleTouchStart: {
        value(e) {
          e.preventDefault()

          const selected = this.isUserTouchingPoint(e)

          if (selected) {
            if (isUsingMultiTouch(e)) return

            this.selected = selected.id
            const selectedPoint = this.points[selected.id]
            selectedPoint
              .animate(selectedPoint.tapArea * 2, true)
              .start()
          } else {
            this.handlePinchStart(e)
          }
        },
      },

      handleTouchMove: {
        value(e) {
          e.preventDefault()
          this.isTapping = false

          if (isUsingMultiTouch(e)) return

          const selected = this.points[this.selected]

          // User is panning
          if (
            !selected &&
            typeof this._initialFocus !== 'undefined' &&
            this._initialFocus !== null
          ) {
            this.setFocus(e)
          }

          // User is drawing
          if (selected && selected.pendingConnections !== 0) {
            const userPos = this.getAdjustedMousePosition(e)
            this.handleHover(e)
            this.drawCurrentLine(
              selected.coords,
              this.toArtPoint(userPos),
            )
          }
        },
      },

      handleTouchEnd: {
        value(e) {
          e.preventDefault()

          // Don't fire touchend when user is double tapping
          if (this.isTapping) {
            this.isTapping = false
            return
          }

          this.cleared = null

          // If no point was ever selected originally, don't do anything
          if (this.selected === null) return

          // Determine whether line can be drawn
          const startPoint = this.points[this.selected]
          const endPoint = this.isUserTouchingPoint(e, true)
          const canDrawLine =
            endPoint &&
            startPoint.id !== endPoint.id &&
            startPoint.pendingConnections > 0 &&
            endPoint.pendingConnections > 0 &&
            !doesLineIntersect(
              startPoint.coords,
              endPoint.coords,
              this.lines,
            )

          if (canDrawLine) {
            // If the line is valid, draw it
            this.clearCurrentLineCanvas()
            actions.drawLine([startPoint.id, endPoint.id])

            // Shrink starting point animation
            const startPointTapArea = getVertexSpecs(
              c.zoom,
              startPoint,
            ).tapArea

            const shrinkStartPoint =
              startPoint.animate(startPointTapArea)
            shrinkStartPoint.start()
          } else {
            // If not, then reel the line back into its origin point
            const shrinkPoint = startPoint.animate(
              startPoint.tapArea / 2,
            )
            const userPos = this.getAdjustedMousePosition(e)
            const reelInLine = this.reelInCurrentLine(
              c.toArtPoint(userPos),
            )

            if (startPoint.pendingConnections !== 0) {
              shrinkPoint.chain(reelInLine)
            }

            shrinkPoint.start()
          }

          if (this.isMobile) this.hover = null
          this.selected = null
          c.drawPoints()
        },
      },

      handleDoubleTap: {
        value(e) {
          if (this.isAnimating) return

          // Get selected point
          let selected
          if (this.selected !== null) {
            // For some strangeness with iOS12 and other iOS versions
            selected = this.points[this.selected]
          } else {
            selected = this.isUserTouchingPoint(e)
            if (selected === null) return
            this.selected = selected.id
          }

          // Clear lines connected to selected point
          actions.clearConnections(this.selected)

          // Shrink selected point back to normal
          const selectedPointTapArea = getVertexSpecs(
            c.zoom,
            selected,
          ).tapArea

          const clearSelected = () => {
            c.selected = null
          }

          const shrinkSelectedPoint = this.points[
            this.selected
          ].animate(selectedPointTapArea, false, clearSelected)
          shrinkSelectedPoint.start()

          // Sometimes lines are accidentally drawn
          // when double tapping, so clear them
          currentLineCtx.clearRect(
            0,
            0,
            c.canvasWidth,
            c.canvasHeight,
          )
        },
      },

      handleHover: {
        value(e) {
          e.preventDefault()
          const hover = this.isUserTouchingPoint(e, true)

          const isHoverNull = hover === null && this.hover === null
          const isSameAsPrevHover = hover && hover.id === this.hover

          if (isHoverNull || isSameAsPrevHover) return

          if (hover) this.hover = hover.id
          else this.hover = null

          c.drawPoints()
        },
      },

      getAdjustedMousePosition: {
        value(e) {
          e.preventDefault()

          // If the event is coming from Hammer, we want the srcEvent object
          const sourceEvent = e.srcEvent ? e.srcEvent : e

          // If the event has changedTouches, it means the user is on a mobile device
          const userPos = sourceEvent.changedTouches
            ? sourceEvent.changedTouches[0]
            : sourceEvent

          const adjustedX =
            (userPos.clientX - this.left) * PIXEL_RATIO
          const adjustedY = (userPos.clientY - this.top) * PIXEL_RATIO

          return [adjustedX, adjustedY]
        },
      },

      isUserTouchingPoint: {
        value(e, isConnecting = false) {
          const [userPosX, userPosY] =
            this.getAdjustedMousePosition(e)

          let minDistance = Infinity
          let closestTouched = null
          const leeway = isConnecting ? 3 : 2 // larger tap area for when drawing a line to a point as opposed to selecting a vertex

          this.points.forEach((obj) => {
            const [canvasPointX, canvasPointY] = this.toCanvasPoint(
              obj.coords,
            )

            const distFromPoint = Math.sqrt(
              (userPosX - canvasPointX) ** 2 +
                (userPosY - canvasPointY) ** 2,
            )

            const touchArea =
              (obj.radius + obj.tapArea) * PIXEL_RATIO * leeway
            const isTouching = distFromPoint <= touchArea

            if (isTouching && distFromPoint < minDistance) {
              minDistance = distFromPoint
              closestTouched = obj
            }
          })

          return closestTouched
        },
      },

      setup: {
        value() {
          elm.style.position = 'relative'

          pointsCanvasElm.style.position = 'absolute'
          pointsCanvasElm.style.top = 0
          pointsCanvasElm.style.left = 0
          pointsCanvasElm.style.width = '100%'
          pointsCanvasElm.style.height = '100%'

          trianglesCanvasElm.style.position = 'absolute'
          trianglesCanvasElm.style.top = 0
          trianglesCanvasElm.style.left = 0
          trianglesCanvasElm.style.width = '100%'
          trianglesCanvasElm.style.height = '100%'

          linesCanvasElm.style.position = 'absolute'
          linesCanvasElm.style.top = 0
          linesCanvasElm.style.left = 0
          linesCanvasElm.style.width = '100%'
          linesCanvasElm.style.height = '100%'

          currentLineCanvasElm.style.position = 'absolute'
          currentLineCanvasElm.style.top = 0
          currentLineCanvasElm.style.left = 0
          currentLineCanvasElm.style.width = '100%'
          currentLineCanvasElm.style.height = '100%'

          const handleResize = (e) => this.handleResize(e)
          const handlePinchStart = (e) => this.handlePinchStart(e)
          const handlePinch = (e) => this.handlePinch(e)
          const handleTouchStart = (e) => this.handleTouchStart(e)
          const handleTouchMove = (e) => this.handleTouchMove(e)
          const handleTouchEnd = (e) => this.handleTouchEnd(e)
          const handleHover = (e) => this.handleHover(e)
          const handleDoubleTap = (e) => this.handleDoubleTap(e)

          const handleMouseDown = (e) => {
            c.isDrawing = true
            handleTouchStart(e)
          }

          const handleMouseMove = (e) => {
            if (c.isDrawing) handleTouchMove(e)
            else handleHover(e)
          }

          const handleMouseUp = (e) => {
            c.isDrawing = false
            handleTouchEnd(e)
          }

          // Disabling the recognizers that are enabled by default
          const mc = new Hammer.Manager(elm, {
            recognizers: [
              [Hammer.Press, { enable: false }],
              [Hammer.Pinch, { enable: true, pointers: 2 }],
              [Hammer.Tap, { enable: true, pointers: 1, taps: 2 }],
              [Hammer.Pan, { enable: false }],
              [Hammer.Swipe, { enable: false }],
            ],
          })

          // For weirdness with transition from welcome moment
          window.addEventListener('playGame', handleResize)

          // For desktop + orientation change on mobile
          window.addEventListener('resize', handleResize)

          mc.on('tap', (e) => {
            this.isTapping = true
            handleDoubleTap(e)
          })

          if (this.isMobile) {
            pointsCanvasElm.addEventListener(
              'touchstart',
              handleTouchStart,
            )
            pointsCanvasElm.addEventListener(
              'touchmove',
              handleTouchMove,
            )
            pointsCanvasElm.addEventListener(
              'touchend',
              handleTouchEnd,
            )

            mc.on('pinchstart', handlePinchStart)
            mc.on('pinchmove', handlePinch)
          } else {
            window.addEventListener('scroll', handleResize)

            pointsCanvasElm.addEventListener(
              'mousedown',
              handleMouseDown,
            )
            pointsCanvasElm.addEventListener(
              'mousemove',
              handleMouseMove,
            )
            pointsCanvasElm.addEventListener('mouseup', handleMouseUp)
          }

          elm.appendChild(trianglesCanvasElm)
          elm.appendChild(linesCanvasElm)
          elm.appendChild(currentLineCanvasElm)
          elm.appendChild(pointsCanvasElm)

          handleResize()

          this.destroy = () => {
            if (this.requestedFrame) {
              window.cancelAnimationFrame(this._requestedFrame)
            }

            window.removeEventListener('resize', handleResize)
            window.removeEventListener('scroll', handleResize)
            pointsCanvasElm.removeEventListener(
              'touchstart',
              handleTouchStart,
            )
            pointsCanvasElm.removeEventListener(
              'touchmove',
              handleTouchMove,
            )
            pointsCanvasElm.removeEventListener(
              'touchend',
              handleTouchEnd,
            )
            pointsCanvasElm.removeEventListener(
              'mousedown',
              handleMouseDown,
            )
            pointsCanvasElm.removeEventListener(
              'mousemove',
              handleMouseMove,
            )
            pointsCanvasElm.removeEventListener(
              'mouseup',
              handleMouseUp,
            )

            mc.destroy()

            pointsCanvasElm.remove()
            linesCanvasElm.remove()
            trianglesCanvasElm.remove()
            currentLineCanvasElm.remove()
          }
        },
      },

      update: {
        value() {
          const {
            zoom,
            focusX,
            focusY,
            artWidth,
            artHeight,
            canvasWidth,
            canvasHeight,
          } = this
          const artRatio = artWidth / artHeight
          const canvasRatio = canvasWidth / canvasHeight
          const isCanvasTaller = canvasRatio < artRatio
          const fitZoom = isCanvasTaller
            ? canvasWidth / artWidth
            : canvasHeight / artHeight
          const scaleAdjustment = fitZoom * zoom
          const renderedArtWidth = artWidth * scaleAdjustment

          const renderedArtHeight = artHeight * scaleAdjustment
          const surplusWidth = (canvasWidth - renderedArtWidth) / 2
          const surplusHeight = (canvasHeight - renderedArtHeight) / 2
          const xAdjustment =
            (artWidth / 2 - focusX) * scaleAdjustment + surplusWidth

          const yAdjustment =
            (artHeight / 2 - focusY) * scaleAdjustment + surplusHeight

          this.toArtPoint = ([x, y]) => {
            return [
              Math.round((x - xAdjustment) / scaleAdjustment),
              Math.round((y - yAdjustment) / scaleAdjustment),
            ]
          }

          this.toCanvasPoint = ([x, y]) => {
            return [
              Math.round(x * scaleAdjustment + xAdjustment),
              Math.round(y * scaleAdjustment + yAdjustment),
            ]
          }

          this._scaleAdjustment = scaleAdjustment

          this.points.forEach((point) => {
            const {
              fontSize,
              radius,
              innerStroke,
              outerStroke,
              lineDashSegments,
              tapArea,
            } = getVertexSpecs(c.zoom, point)

            point.fontSize = fontSize
            point.radius = point.overrides.radius || radius
            point.innerStroke = innerStroke
            point.outerStroke = outerStroke
            point.lineDashSegments = lineDashSegments
            point.tapArea = point.overrides.tapArea || tapArea
          })

          c.redrawAll()
        },
      },

      setFocus: {
        value(e) {
          // return if client coordinates haven't been set yet
          if (
            typeof this.clientX === 'undefined' ||
            Object.is(this.clientX, NaN) ||
            typeof this.clientY === 'undefined' ||
            Object.is(this.clientY, NaN)
          ) {
            return
          }

          const scaleAdjustment = this._scaleAdjustment
          const deltaX = e.changedTouches
            ? e.changedTouches[0].clientX - this.clientX
            : e.clientX - this.clientX

          const deltaY = e.changedTouches
            ? e.changedTouches[0].clientY - this.clientY
            : e.clientY - this.clientY

          const nextFocusX =
            this._initialFocus.x -
            (deltaX / scaleAdjustment) * PIXEL_RATIO

          const nextFocusY =
            this._initialFocus.y -
            (deltaY / scaleAdjustment) * PIXEL_RATIO

          // TODO: this causes 3 updates to helper function
          if (nextFocusX >= this.maxFocusX) {
            this.focusX = this.maxFocusX
          } else if (nextFocusX <= this.minFocusX) {
            this.focusX = this.minFocusX
          } else {
            this.focusX = nextFocusX
          }

          if (nextFocusY >= this.maxFocusY) {
            this.focusY = this.maxFocusY
          } else if (nextFocusY <= this.minFocusY) {
            this.focusY = this.minFocusY
          } else {
            this.focusY = nextFocusY
          }
        },
      },

      setZoom: {
        value(scale, direction = 'in') {
          let nextZoom

          if (this.isMobile) {
            nextZoom = this._initialZoom * scale
          } else {
            nextZoom =
              direction === 'in'
                ? this.zoom * scale
                : this.zoom / scale
          }

          if (nextZoom <= zoomRange.min) {
            this.zoom = zoomRange.min
          } else if (nextZoom >= zoomRange.max) {
            this.zoom = zoomRange.max
          } else {
            this.zoom = nextZoom
          }

          return this.zoom
        },
      },

      zoomOutAnimation: {
        value() {
          const centerFocusX = this.artWidth / 2
          const centerFocusY =
            this.artHeight / 2 - (THEME_BAR_HEIGHT + TOOLBAR_HEIGHT)

          const currentZoomAndFocus = {
            z: this.zoom,
            x: this.focusX,
            y: this.focusY,
          }

          const tween = new TWEEN.Tween(currentZoomAndFocus)
            .to(
              {
                z: zoomRange.min,
                x: centerFocusX,
                y: centerFocusY,
              },
              500,
            )
            .onUpdate((increment) => {
              this.zoom = increment.z
              this.focusX = increment.x
              this.focusY = increment.y
              c.redrawAll()
            })

          return tween
        },
      },

      points: {
        value: [],
      },

      lines: {
        value: [],
      },

      triangles: {
        value: [],
      },

      drawCurrentLine: {
        value(a, b, isClearing = false) {
          c.clearCurrentLineCanvas()

          const vertexNearby =
            c.hover !== null && c.hover !== c.selected
          const [ax, ay] = c.toCanvasPoint(a)
          const [bx, by] = c.toCanvasPoint(b)
          const lineSpecs = getLineSpecs(c.zoom, c.isMobile)

          // Line
          currentLineCtx.beginPath()

          if (isClearing) {
            currentLineCtx.strokeStyle = colors.red
          } else if (vertexNearby) {
            currentLineCtx.strokeStyle = colors.gold
          } else {
            currentLineCtx.strokeStyle = colors.black
          }

          currentLineCtx.lineWidth = lineSpecs * PIXEL_RATIO

          currentLineCtx.moveTo(ax, ay)
          currentLineCtx.lineTo(bx, by)
          currentLineCtx.setLineDash([
            lineSpecs * PIXEL_RATIO,
            lineSpecs * PIXEL_RATIO,
          ])
          currentLineCtx.stroke()

          // End Point
          if (c.selected === null) return

          const selected = c.points[c.selected]

          // Inner circle
          currentLineCtx.beginPath()
          currentLineCtx.arc(
            bx,
            by,
            4 * PIXEL_RATIO,
            0,
            ARC_FULL,
            false,
          )

          if (isClearing) {
            currentLineCtx.fillStyle = colors.red
          } else if (vertexNearby) {
            currentLineCtx.fillStyle = colors.gold
          } else {
            currentLineCtx.fillStyle = colors.black
          }

          currentLineCtx.fill()

          // Outer circle
          currentLineCtx.beginPath()
          currentLineCtx.lineWidth =
            selected.outerStroke * PIXEL_RATIO
          currentLineCtx.strokeStyle = colors.black
          currentLineCtx.arc(
            bx,
            by,
            (selected.radius + selected.tapArea) * PIXEL_RATIO,
            0,
            ARC_FULL,
            false,
          )
          currentLineCtx.setLineDash([
            selected.lineDashSegments[0] * PIXEL_RATIO,
            selected.lineDashSegments[1] * PIXEL_RATIO,
          ])

          currentLineCtx.stroke()
        },
      },

      reelInCurrentLine: {
        value(endCoordinates) {
          const selected = c.points[c.selected]
          const from = [...endCoordinates]
          const to = [...selected.coords]
          const tween = new TWEEN.Tween(from)
            .to(to, 200)
            .onUpdate((increment) => {
              c.drawCurrentLine(to, increment, true)
            })
            .easing(TWEEN.Easing.Exponential.InOut)
            .onComplete(() => {
              c.clearCurrentLineCanvas()
              c.selected = null
            })

          return tween
        },
      },

      addPoint: {
        value(point) {
          const {
            coords,
            id,
            pendingConnections,
            shapes,
            completedShapes,
            preDrawnShapes,
          } = point

          const {
            radius,
            fontSize,
            innerStroke,
            outerStroke,
            lineDashSegments,
            tapArea,
          } = getVertexSpecs(c.zoom, point)

          c.points.push({
            id,
            coords,
            pendingConnections,
            radius,
            fontSize,
            innerStroke,
            outerStroke,
            lineDashSegments,
            tapArea,
            shapes,
            completedShapes,
            preDrawnShapes,
            overrides: {
              radius: null,
              fontSize: null,
              innerStroke: null,
              outerStroke: null,
              lineDashSegments: null,
              tapArea: null,
            },

            draw() {
              if (this.radius === 0) return

              const [x, y] = c.toCanvasPoint(this.coords)
              const fontStyle = `600 ${
                this.fontSize * PIXEL_RATIO
              }px nyt-franklin`

              const {
                innerFillStyle,
                outerFillStyle,
                innerStrokeStyle,
                outerStrokeStyle,
                fontColor,
              } = getVertexStyles(id, c.selected, c.hover, c.cleared)

              // Inner circle
              pointsCtx.beginPath()
              pointsCtx.lineWidth = innerStroke * PIXEL_RATIO
              pointsCtx.textAlign = 'center'
              pointsCtx.textBaseline = 'middle'
              pointsCtx.font = fontStyle
              pointsCtx.arc(
                x,
                y,
                this.radius * PIXEL_RATIO,
                0,
                ARC_FULL,
                false,
              )
              pointsCtx.fillStyle = innerFillStyle
              pointsCtx.strokeStyle = innerStrokeStyle
              pointsCtx.fill()
              pointsCtx.setLineDash([])
              pointsCtx.stroke()

              // Outer circle
              pointsCtx.beginPath()
              pointsCtx.arc(
                x,
                y,
                (this.radius + this.tapArea) * PIXEL_RATIO,
                0,
                ARC_FULL,
                false,
              )
              pointsCtx.lineWidth = this.outerStroke * PIXEL_RATIO
              pointsCtx.strokeStyle = outerStrokeStyle
              pointsCtx.fillStyle = outerFillStyle
              pointsCtx.setLineDash([
                this.lineDashSegments[0] * PIXEL_RATIO,
                this.lineDashSegments[1] * PIXEL_RATIO,
              ])
              pointsCtx.fill()
              pointsCtx.stroke()

              pointsCtx.setLineDash([])
              pointsCtx.fillStyle = fontColor
              pointsCtx.fillText(
                this.pendingConnections.toString(),
                x,
                y,
              )
            },

            animate(
              newTapArea,
              fillForward = false,
              postAnimation = () => {},
            ) {
              const oldTapArea = { t: this.tapArea }
              const tween = new TWEEN.Tween(oldTapArea)
                .to({ t: newTapArea }, 100)
                .onUpdate((increment) => {
                  this.tapArea = increment.t
                  this.overrides.tapArea = increment.t
                  c.drawPoints()
                })
                .onComplete(() => {
                  if (fillForward) {
                    this.overrides.tapArea = newTapArea
                  } else {
                    this.tapArea = getVertexSpecs(
                      c.zoom,
                      this,
                    ).tapArea
                    this.overrides.tapArea = null
                  }
                  postAnimation()
                  c.drawPoints()
                })

              return tween
            },

            animateShrink(
              newRadius,
              newTapArea,
              fillForward = false,
            ) {
              const oldSpecs = {
                r: this.radius,
                t: this.tapArea,
              }

              const tween = new TWEEN.Tween(oldSpecs)
                .to({ r: newRadius, t: newTapArea }, 100)
                .onUpdate((increment) => {
                  this.radius = increment.r
                  this.tapArea = increment.t
                  this.overrides.radius = increment.r
                  this.overrides.tapArea = increment.t
                  c.drawPoints()
                })
                .onComplete(() => {
                  if (fillForward) {
                    this.overrides.radius = newRadius
                  } else {
                    this.radius = getVertexSpecs(c.zoom, this).tapArea
                    this.overrides.radius = null
                  }

                  c.drawPoints()
                })

              return tween
            },
          })
        },
      },

      addLine: {
        value(a, b, inProgress = false, isClearing = false) {
          if (this.lines.includes((l) => l.a === a && l.b === b)) {
            return
          }

          this.lines.push({
            a,
            b,
            draw() {
              const vertexNearby =
                c.hover !== null &&
                inProgress &&
                c.hover !== c.selected
              const [ax, ay] = c.toCanvasPoint(this.a)
              const [bx, by] = c.toCanvasPoint(this.b)
              const lineSpecs = getLineSpecs(c.zoom, c.isMobile)

              const drawingCtx = inProgress
                ? currentLineCtx
                : linesCtx

              drawingCtx.beginPath()

              if (isClearing) {
                drawingCtx.strokeStyle = colors.red
              } else if (vertexNearby) {
                drawingCtx.strokeStyle = colors.gold
              } else {
                drawingCtx.strokeStyle = colors.black
              }

              drawingCtx.lineWidth =
                inProgress || isClearing
                  ? lineSpecs * PIXEL_RATIO
                  : PIXEL_RATIO

              drawingCtx.moveTo(ax, ay)
              drawingCtx.lineTo(bx, by)

              if (inProgress || isClearing) {
                drawingCtx.setLineDash([
                  lineSpecs * PIXEL_RATIO,
                  lineSpecs * PIXEL_RATIO,
                ])
              } else {
                drawingCtx.setLineDash([])
              }

              drawingCtx.stroke()

              if (inProgress) {
                this.drawEndPoint(bx, by)
              }
            },

            drawEndPoint(x, y) {
              if (c.selected === null) return

              const selected = c.points[c.selected]
              const vertexNearby =
                selected !== null &&
                c.hover !== null &&
                selected.id !== c.hover

              // Inner circle
              currentLineCtx.beginPath()
              currentLineCtx.arc(
                x,
                y,
                4 * PIXEL_RATIO,
                0,
                ARC_FULL,
                false,
              )
              currentLineCtx.fillStyle = vertexNearby
                ? colors.gold
                : colors.black
              currentLineCtx.fill()

              // Outer circle
              currentLineCtx.beginPath()
              currentLineCtx.lineWidth =
                selected.outerStroke * PIXEL_RATIO
              currentLineCtx.strokeStyle = colors.black
              currentLineCtx.arc(
                x,
                y,
                (selected.radius + selected.tapArea) * PIXEL_RATIO,
                0,
                ARC_FULL,
                false,
              )
              currentLineCtx.setLineDash([
                selected.lineDashSegments[0] * PIXEL_RATIO,
                selected.lineDashSegments[1] * PIXEL_RATIO,
              ])
              currentLineCtx.stroke()
            },

            reelIn(reverse = false, drawFunc = () => {}, cleared) {
              c.cleared = cleared
              this.isAnimating = true

              const from = reverse ? [...this.a] : [...this.b]
              const to = reverse ? [...this.b] : [...this.a]
              const tween = new TWEEN.Tween(from)
                .to(to, 200)
                .onUpdate((increment) => {
                  if (reverse) this.a = increment
                  else this.b = increment
                  drawFunc()
                })
                .easing(TWEEN.Easing.Exponential.InOut)
                .onComplete(() => {
                  c.cleared = null
                  drawFunc()
                  this.isAnimating = false
                })

              return tween
            },
          })

          return this.lines[this.lines.length - 1]
        },
      },

      animateLine: {
        value(a, b) {
          this.addLine(a.coords, a.coords)

          const newLineInd = this.lines.length - 1
          const tween = new TWEEN.Tween([...a.coords])
            .to([...b.coords], 1000)
            .onUpdate((increment) => {
              this.lines[newLineInd].b = increment
            })

          tween.start()
        },
      },

      addTriangle: {
        value(
          a,
          b,
          c,
          color = '#00F',
          vertices,
          id,
          preDrawn = false,
        ) {
          vertices.forEach((v) => {
            const point = this.points[v]

            if (!preDrawn) point.completedShapes.push(id)

            if (
              point.completedShapes.length +
                point.preDrawnShapes.length ===
              this.points[v].shapes.length
            ) {
              this.points[v].animateShrink(0, 0).start()
            }
          })

          this.triangles.push({
            draw: () => {
              const [ax, ay] = this.toCanvasPoint(a)
              const [bx, by] = this.toCanvasPoint(b)
              const [cx, cy] = this.toCanvasPoint(c)

              trianglesCtx.fillStyle = color
              trianglesCtx.strokeStyle = color
              trianglesCtx.beginPath()
              trianglesCtx.moveTo(ax, ay)
              trianglesCtx.lineTo(bx, by)
              trianglesCtx.lineTo(cx, cy)
              trianglesCtx.lineTo(ax, ay)
              trianglesCtx.closePath()
              trianglesCtx.fill()
              trianglesCtx.stroke()
            },
          })
          return this.triangles[this.triangles.length - 1]
        },
      },

      removeLine: {
        value(lineArr) {
          const ind = this.lines.findIndex((lineObj) => {
            const a = this.points[lineArr[0]].coords
            const b = this.points[lineArr[1]].coords

            return (
              lineObj.a.toString() === a.toString() &&
              lineObj.b.toString() === b.toString()
            )
          })

          this.lines.splice(ind, 1)
        },
      },

      resetLines: {
        value() {
          this.lines.length = 0
        },
      },

      resetTriangles: {
        value() {
          this.triangles.length = 0
        },
      },

      clearCurrentLineCanvas: {
        value() {
          currentLineCtx.clearRect(
            0,
            0,
            c.canvasWidth,
            c.canvasHeight,
          )
        },
      },

      redrawAll: {
        value() {
          currentLineCtx.clearRect(
            0,
            0,
            c.canvasWidth,
            c.canvasHeight,
          )
          trianglesCtx.clearRect(0, 0, c.canvasWidth, c.canvasHeight)
          linesCtx.clearRect(0, 0, c.canvasWidth, c.canvasHeight)

          pointsCtx.clearRect(0, 0, c.canvasWidth, c.canvasHeight)
          c.triangles.forEach((o) => o.draw())
          c.lines.forEach((o) => o.draw())
          c.points.forEach((o) => o.draw())
        },
      },

      drawLines: {
        value() {
          linesCtx.clearRect(0, 0, c.canvasWidth, c.canvasHeight)
          c.lines.forEach((o) => o.draw())
        },
      },

      drawPoints: {
        value() {
          pointsCtx.clearRect(0, 0, c.canvasWidth, c.canvasHeight)
          c.points.forEach((o) => o.draw())
        },
      },

      drawTriangles: {
        value() {
          trianglesCtx.clearRect(0, 0, c.canvasWidth, c.canvasHeight)
          c.triangles.forEach((o) => o.draw())
        },
      },
    },
  )

  c.setup()

  const animationLoop = (time) => {
    TWEEN.update(time)

    c.requestedFrame = window.requestAnimationFrame(animationLoop)
  }

  animationLoop()

  return c
}

export default createCanvases
