import React, { useContext } from 'react'
import pt from 'prop-types'
import cn from 'classnames'

import { useUser } from 'user'
import { trackImpression } from 'helpers/tracking'
import useCTA from 'shared/marketing/useCTA'

import {
  Icon,
  Title,
  Description,
  Button,
  BodyText,
} from '../../shared/moments/components/subcomponents'

const PrimaryButton = ({
  userType,
  hasCompletedDailyPuzzle,
  hasProgress,
  toSubLandingPage,
  playGame,
  track,
  meterGranted,
  buttonText,
}) => {
  const startGame = () => {
    playGame()
    track('START_GAME')
  }

  const continueGame = () => {
    playGame()
    track('CONTINUE')
  }

  const viewSolved = () => {
    playGame()
    track('VIEW_SOLVED')
  }

  if (userType.hasXwd || meterGranted) {
    const buttons = []

    if (meterGranted && !userType.hasXwd) {
      buttons.push(
        <Button
          text={buttonText}
          action={() => {
            toSubLandingPage('vertexWelcomeMomentDailyPuzzle')
            track('SUBSCRIBE')
          }}
          key="subscribe"
        />,
      )
    }

    if (hasCompletedDailyPuzzle) {
      buttons.push(
        <Button text="Solved" action={viewSolved} key="viewSolved" />,
      )
    }

    if (!hasCompletedDailyPuzzle) {
      buttons.push(
        <Button
          action={hasProgress ? continueGame : startGame}
          text={hasProgress ? 'Continue' : "Today's Puzzle"}
          key="dailyPuzzle"
        />,
      )
    }

    return buttons
  }

  return (
    <>
      <Button
        text={buttonText}
        action={() => {
          toSubLandingPage('vertexWelcomeMomentDailyPuzzle')
          track('SUBSCRIBE')
        }}
      />
      <Button
        action={() => {
          toSubLandingPage('vertexTodaysPuzzle')
          track('LOCKED_PLAY')
        }}
        text="Today’s Puzzle"
        icon="locked"
        color="secondary"
      />
    </>
  )
}

PrimaryButton.propTypes = {
  track: pt.func.isRequired,
  playGame: pt.func.isRequired,
  hasProgress: pt.bool.isRequired,
  toSubLandingPage: pt.func.isRequired,
  hasCompletedDailyPuzzle: pt.bool.isRequired,
  userType: pt.shape({ hasXwd: pt.bool.isRequired }).isRequired,
  meterGranted: pt.bool.isRequired,
  buttonText: pt.string.isRequired,
}

const CallToAction = ({
  hasProgress,
  hasCompletedDailyPuzzle,
  hasLoaded,
  userType,
  meterGranted,
}) => {
  if (userType.hasXwd || meterGranted) {
    if (!hasProgress) {
      return (
        <Description text="Connect the dots and <br/> reveal the hidden picture." />
      )
    }

    if (!hasCompletedDailyPuzzle) {
      return <Description text="Reveal the full picture." />
    }

    return (
      <Description text="Come back tomorrow to play a new puzzle." />
    )
  }

  if (hasLoaded && !userType.hasXwd) {
    return (
      <Description text="Subscribe to New York Times Games to<br/> play Vertex every day." />
    )
  }

  return null
}

CallToAction.propTypes = {
  hasLoaded: pt.bool.isRequired,
  hasProgress: pt.bool.isRequired,
  userType: pt.shape({ hasXwd: pt.bool.isRequired }).isRequired,
  hasCompletedDailyPuzzle: pt.bool.isRequired,
  meterGranted: pt.bool.isRequired,
}

const WelcomeDaily = ({
  track,
  playGame,
  displayDate,
  hasProgress,
  toSubLandingPage,
  puzzleConstructor,
  hasCompletedDailyPuzzle,
  meterGranted,
}) => {
  const { userType, hasLoaded } = useUser()
  const { getButtonText } = useCTA()
  const buttonText = getButtonText()

  const welcomeBack = (userType.hasXwd || meterGranted) && hasProgress

  React.useEffect(() => {
    if (
      hasLoaded &&
      !userType.hasXwd &&
      !meterGranted &&
      !hasCompletedDailyPuzzle
    ) {
      // !hasCompletedDailyPuzzle prevents this event to fire at the end of the starter pack congrats
      trackImpression(
        'hardpaywall',
        'vertex-end-of-starter-pack-welcome',
      )
    }
  }, [hasLoaded, userType, hasCompletedDailyPuzzle])

  return (
    <>
      <div
        className={cn(
          'geo-moment__container',
          'geo-moment__welcome',
          {
            back: welcomeBack,
          },
        )}
        id="geo-welcome-daily"
      >
        <div className="geo-moment__content">
          {welcomeBack ? (
            <>
              <div className="geo-moment__icon" />
              <h1 className="geo-moment__title">Vertex</h1>
            </>
          ) : (
            <>
              <Icon icon="vertex" />
              <Title text="Vertex" />
            </>
          )}
          {welcomeBack && (
            <h2 className="geo-moment__subtitle">Welcome back.</h2>
          )}
          <CallToAction
            hasProgress={hasProgress}
            hasCompletedDailyPuzzle={hasCompletedDailyPuzzle}
            hasLoaded={hasLoaded}
            userType={userType}
            meterGranted={meterGranted}
          />
          <div className="geo-moment__actions">
            <PrimaryButton
              userType={userType}
              hasCompletedDailyPuzzle={hasCompletedDailyPuzzle}
              hasProgress={hasProgress}
              toSubLandingPage={toSubLandingPage}
              playGame={playGame}
              track={track}
              meterGranted={meterGranted}
              buttonText={buttonText}
            />
          </div>
          <BodyText>
            <span className="pz-moment__info-date">
              {displayDate}
            </span>
          </BodyText>
          <BodyText>{`By ${puzzleConstructor}`}</BodyText>
        </div>
      </div>
    </>
  )
}

WelcomeDaily.propTypes = {
  track: pt.func.isRequired,
  playGame: pt.func.isRequired,
  hasProgress: pt.bool.isRequired,
  puzzleConstructor: pt.string,
  displayDate: pt.string.isRequired,
  toSubLandingPage: pt.func.isRequired,
  hasCompletedDailyPuzzle: pt.bool.isRequired,
  meterGranted: pt.bool.isRequired,
}

export default WelcomeDaily
