import React from 'react'
import { mount } from 'enzyme'

import { useUser } from 'user'

import CongratsStarterPack from '../CongratsStarterPack'
import { trackImpression } from '../../../helpers/tracking'

const props = {
  track: jest.fn(),
  backToGame: jest.fn(),
  toSubLandingPage: jest.fn(),
  meterGranted: false,
}

jest.mock('foundation', () => ({}))

jest.mock('user', () => ({
  useUser: jest.fn(),
}))

jest.mock('../../../helpers/tracking', () => ({
  trackImpression: jest.fn(),
}))

jest.mock('shared/marketing/useCTA', () => ({
  __esModule: true,
  default: () => ({
    isGamesSaleActive: false,
    getButtonText: () => 'Subscribe',
  }),
}))

describe('VERTEX: MOMENT - CONGRATS STARTER PACK', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('should render text', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: false,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const title = congratsStarterPack.find('.pz-moment__title')

    expect(title.text()).toEqual('Congrats on your masterpiece!')
  })

  it('should render info for subs', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const info = congratsStarterPack.find('.pz-moment__description')

    expect(info.text()).toEqual(
      'As a Games subscriber, you can play Vertex every day.',
    )
  })

  it('should render info for non-subs', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const info = congratsStarterPack.find('.pz-moment__description')

    expect(info.text()).toEqual(
      'Subscribe to New York Times Games to play Vertex every day.',
    )
  })

  it('should render working play button for subs', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const primaryButton = congratsStarterPack
      .find('.pz-moment__button')
      .at(0)

    expect(primaryButton.text()).toEqual(`Today’s Puzzle`)

    primaryButton.simulate('click')

    expect(props.backToGame).toHaveBeenCalled()
    expect(props.track).toHaveBeenCalledWith('START_GAME')
  })

  it('should not render subcribe button if user has a sub', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const secondaryButton = congratsStarterPack
      .find('.pz-moment__button')
      .at(1)

    expect(secondaryButton).toHaveLength(0)
  })

  it('does not render the Subscribe CTA user is subscribed and does not have Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const buttons = congratsStarterPack.find('.pz-moment__button')
    expect(buttons.length).toEqual(1)

    const primaryButton = buttons.at(0)
    expect(primaryButton.text()).toEqual(`Today’s Puzzle`)
  })

  it('does not render the Subscribe CTA if user is subscribed and has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const meterGrantedProps = {
      ...props,
      meterGranted: true,
    }

    const congratsStarterPack = mount(
      <CongratsStarterPack {...meterGrantedProps} />,
    )

    const buttons = congratsStarterPack.find('.pz-moment__button')
    expect(buttons.length).toEqual(1)

    const primaryButton = buttons.at(0)
    expect(primaryButton.text()).toEqual(`Today’s Puzzle`)
  })

  it('renders the daily puzzle CTA user is subscribed and does not have Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const primaryButton = congratsStarterPack
      .find('.pz-moment__button')
      .at(0)

    primaryButton.simulate('click')

    expect(props.backToGame).toHaveBeenCalled()
    expect(props.track).toHaveBeenCalledWith('START_GAME')
  })

  it('renders the daily puzzle CTA user is subscribed and has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const meterGrantedProps = {
      ...props,
      meterGranted: true,
    }

    const congratsStarterPack = mount(
      <CongratsStarterPack {...meterGrantedProps} />,
    )

    const primaryButton = congratsStarterPack
      .find('.pz-moment__button')
      .at(0)

    primaryButton.simulate('click')

    expect(props.backToGame).toHaveBeenCalled()
    expect(props.track).toHaveBeenCalledWith('START_GAME')
  })

  it('renders the Subscribe CTA if non-sub user does not have Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const buttons = congratsStarterPack.find('.pz-moment__button')
    expect(buttons.length).toEqual(2)

    const subButton = buttons.at(0)
    expect(subButton.text()).toEqual(`Subscribe`)
  })

  it('renders the Subscribe CTA if non-sub user has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const meterGrantedProps = {
      ...props,
      meterGranted: true,
    }

    const congratsStarterPack = mount(
      <CongratsStarterPack {...meterGrantedProps} />,
    )

    const buttons = congratsStarterPack.find('.pz-moment__button')
    expect(buttons.length).toEqual(2)

    const subButton = buttons.at(0)
    expect(subButton.text()).toEqual(`Subscribe`)
  })

  it('renders locked daily puzzle CTA if non-sub user does not have Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })

    const congratsStarterPack = mount(
      <CongratsStarterPack {...props} />,
    )

    const buttons = congratsStarterPack.find('.pz-moment__button')
    expect(buttons.length).toEqual(2)

    const lockIcon = congratsStarterPack.find(
      '.pz-moment__button--padlock',
    )
    expect(lockIcon.length).toEqual(1)

    const subButton = buttons.at(0)
    expect(subButton.text()).toEqual(`Subscribe`)

    const dailyPlayButton = buttons.at(1)
    expect(dailyPlayButton.text()).toEqual(`Today’s Puzzle`)

    dailyPlayButton.simulate('click')

    expect(props.backToGame).not.toHaveBeenCalled()
    expect(props.toSubLandingPage).toHaveBeenCalledWith(
      'vertexTodaysPuzzle',
    )
  })

  it('renders unlocked daily puzzle CTA if non-sub user has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const meterGrantedProps = {
      ...props,
      meterGranted: true,
    }

    const congratsStarterPack = mount(
      <CongratsStarterPack {...meterGrantedProps} />,
    )

    const buttons = congratsStarterPack.find('.pz-moment__button')
    expect(buttons.length).toEqual(2)

    const subButton = buttons.at(0)
    expect(subButton.text()).toEqual(`Subscribe`)

    const dailyPlayButton = buttons.at(1)
    expect(dailyPlayButton.text()).toEqual(`Today’s Puzzle`)

    const lockIcon = congratsStarterPack.find(
      '.pz-moment__button--padlock',
    )
    expect(lockIcon.length).toEqual(0)

    dailyPlayButton.simulate('click')

    expect(props.backToGame).toHaveBeenCalled()
    expect(props.track).toHaveBeenCalledWith('START_GAME')
    expect(props.toSubLandingPage).not.toHaveBeenCalled()
  })

  it('should fire trackImpression for paywall once on initial render if no Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })

    mount(<CongratsStarterPack {...props} />)

    expect(trackImpression).toHaveBeenCalledTimes(1)
  })

  it('should not trackImpression for paywall once on initial render if Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const meterProps = {
      ...props,
      meterGranted: true,
    }

    mount(<CongratsStarterPack {...meterProps} />)

    expect(trackImpression).not.toHaveBeenCalled()
  })
})
