import React from 'react'
import { mount, debug } from 'enzyme'

import { useUser } from 'user'

import WelcomeDaily from '../WelcomeDaily'
import { trackImpression } from '../../../helpers/tracking'

const defaultProps = {
  toHub: jest.fn(),
  track: jest.fn(),
  playGame: jest.fn(),
  hasProgress: false,
  openRulesModal: jest.fn(),
  toSubLandingPage: jest.fn(),
  hasCompletedDailyPuzzle: false,
  puzzleConstructor: 'Albus Dumbledore',
  displayDate: 'Sunday, November 10',
  meterGranted: false,
}

jest.mock('foundation', () => ({}))

jest.mock('user', () => ({
  useUser: jest.fn(),
}))

jest.mock('../../../helpers/tracking', () => ({
  trackImpression: jest.fn(),
}))

jest.mock('shared/marketing/useCTA', () => ({
  __esModule: true,
  default: () => ({
    isGamesSaleActive: false,
    getButtonText: () => 'Subscribe',
  }),
}))

describe('VERTEX: MOMENT - WELCOME DAILY', () => {
  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('should render text', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const welcomeDaily = mount(<WelcomeDaily {...defaultProps} />)
    const title = welcomeDaily.find('.pz-moment__title')
    const info = welcomeDaily.find('.pz-moment__body')

    expect(title.text()).toEqual('Vertex')
    expect(info.at(0).text()).toEqual('Sunday, November 10')

    expect(info.at(1).text()).toEqual('By Albus Dumbledore')
  })

  it('should render proper info if user is sub and has completed puzzle', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      hasCompletedDailyPuzzle: true,
      hasProgress: true,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)
    const subtitle = welcomeDaily.find('.geo-moment__subtitle')
    const callToAction = welcomeDaily.find('.pz-moment__description')

    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    expect(subtitle.text()).toEqual('Welcome back.')
    expect(primaryButton.text()).toEqual('Solved')
    expect(callToAction.text()).toEqual(
      'Come back tomorrow to play a new puzzle.',
    )

    primaryButton.simulate('click')

    expect(props.playGame).toHaveBeenCalledTimes(1)
    expect(props.track).toHaveBeenCalledWith('VIEW_SOLVED')
  })

  it('should render proper info if user is sub and has not completed puzzle', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      hasCompletedDailyPuzzle: false,
      hasProgress: false,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)
    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    expect(primaryButton.text()).toEqual("Today's Puzzle")

    primaryButton.simulate('click')

    expect(props.playGame).toHaveBeenCalledTimes(1)
    expect(props.track).toHaveBeenCalledWith('START_GAME')
  })

  it('should render proper info if user is sub and has not completed puzzle but has progress', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      hasProgress: true,
      hasCompletedDailyPuzzle: false,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)
    const callToAction = welcomeDaily.find('.pz-moment__description')

    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    expect(primaryButton.text()).toEqual('Continue')
    expect(callToAction.text()).toEqual('Reveal the full picture.')

    primaryButton.simulate('click')

    expect(props.playGame).toHaveBeenCalledTimes(1)
    expect(props.track).toHaveBeenCalledWith('CONTINUE')
  })

  it('should render proper info if user is not a sub', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      hasProgress: false,
      hasCompletedDailyPuzzle: false,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)
    const callToAction = welcomeDaily.find('.pz-moment__description')

    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    const secondaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(1)

    expect(primaryButton.text()).toEqual('Subscribe')
    expect(secondaryButton.text()).toEqual('Today’s Puzzle')
    expect(callToAction.text()).toEqual(
      'Subscribe to New York Times Games to play Vertex every day.',
    )

    primaryButton.simulate('click')

    expect(props.playGame).not.toHaveBeenCalled()

    secondaryButton.simulate('click')

    expect(props.toSubLandingPage).toHaveBeenCalledWith(
      'vertexWelcomeMomentDailyPuzzle',
    )
  })

  it('should fire trackImpression once on initial render', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })

    mount(<WelcomeDaily {...defaultProps} />)

    expect(trackImpression).toHaveBeenCalledTimes(1)
  })

  it('should not render the Subscribe CTA if user is subscribed and has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      meterGranted: true,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)

    const buttons = welcomeDaily.find('.pz-moment__button')
    expect(buttons.length).toEqual(1)

    const primaryButton = buttons.at(0)
    expect(primaryButton.text()).toEqual(`Today's Puzzle`)
  })

  it('should render the daily puzzle CTA if user is subscribed and has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      meterGranted: true,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)

    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    expect(primaryButton.text()).toEqual(`Today's Puzzle`)

    primaryButton.simulate('click')

    expect(props.playGame).toHaveBeenCalledTimes(1)
    expect(props.track).toHaveBeenCalledWith('START_GAME')
  })

  it('should render the view puzzle CTA if user is subscribed and has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: true,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      hasCompletedDailyPuzzle: true,
      hasProgress: true,
      meterGranted: true,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)

    const buttons = welcomeDaily.find('.pz-moment__button')
    expect(buttons.length).toEqual(1)

    const primaryButton = buttons.at(0)

    expect(primaryButton.text()).toEqual('Solved')

    primaryButton.simulate('click')

    expect(props.playGame).toHaveBeenCalledTimes(1)
    expect(props.track).toHaveBeenCalledWith('VIEW_SOLVED')
  })

  it('should render the Subscribe CTA if non-sub user has Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      meterGranted: true,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)

    const buttons = welcomeDaily.find('.pz-moment__button')
    expect(buttons.length).toEqual(2)

    const subButton = buttons.at(0)
    expect(subButton.text()).toEqual(`Subscribe`)
  })

  it('should render locked daily puzzle CTA if non-sub user does not have Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })

    const welcomeDaily = mount(<WelcomeDaily {...defaultProps} />)
    const callToAction = welcomeDaily.find('.pz-moment__description')

    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    const secondaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(1)

    expect(primaryButton.text()).toEqual('Subscribe')
    expect(secondaryButton.text()).toEqual('Today’s Puzzle')
    expect(callToAction.text()).toEqual(
      'Subscribe to New York Times Games to play Vertex every day.',
    )

    primaryButton.simulate('click')

    expect(defaultProps.playGame).not.toHaveBeenCalled()

    secondaryButton.simulate('click')

    expect(defaultProps.toSubLandingPage).toHaveBeenCalledWith(
      'vertexWelcomeMomentDailyPuzzle',
    )
  })

  it('should render unlocked daily puzzle CTA if non-sub user does have Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      meterGranted: true,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)

    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    const secondaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(1)

    expect(primaryButton.text()).toEqual('Subscribe')
    expect(secondaryButton.text()).toEqual(`Today's Puzzle`)

    secondaryButton.simulate('click')

    expect(props.playGame).toHaveBeenCalledTimes(1)
    expect(props.track).toHaveBeenCalledWith('START_GAME')
  })

  it('should render unlocked solved puzzle CTA if non-sub user does have Meter access', () => {
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      hasCompletedDailyPuzzle: true,
      hasProgress: true,
      meterGranted: true,
    }

    const welcomeDaily = mount(<WelcomeDaily {...props} />)

    const primaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(0)

    const secondaryButton = welcomeDaily
      .find('.pz-moment__button')
      .at(1)

    expect(primaryButton.text()).toEqual('Subscribe')
    expect(secondaryButton.text()).toEqual('Solved')

    secondaryButton.simulate('click')

    expect(props.playGame).toHaveBeenCalledTimes(1)
    expect(props.track).toHaveBeenCalledWith('VIEW_SOLVED')
  })

  it('should not fire trackImpression at the end of congrats of starter pack', () => {
    // sets setting as a non subscriber finishing vertex starter pack
    useUser.mockReturnValue({
      userType: {
        hasXwd: false,
      },
      hasLoaded: true,
    })
    const props = {
      ...defaultProps,
      hasCompletedDailyPuzzle: true,
    }

    mount(<WelcomeDaily {...props} />)

    expect(trackImpression).not.toHaveBeenCalled()
  })
})
