import {
  verticesSpecs,
  getVertexSpecs,
  getVertexStyles,
  getLineSpecs,
} from '../canvasHelpers'
import { colors } from '../../constants'

const minZoom = 1
const maxZoom = 3
const vertex = {
  pendingConnections: 3,
  shapes: [1, 2],
  completedShapes: [],
  preDrawnShapes: [],
}

describe('VERTEX: CANVAS HELPERS', () => {
  it('should return zoomedOut specs at minimum zoom', () => {
    window.innerWidth = 444
    expect(getVertexSpecs(minZoom, vertex)).toEqual(
      verticesSpecs.zoomedOut.smallScreen.smallVertex,
    )
  })

  it('should return zoomedIn specs at maximum zoom', () => {
    window.innerWidth = 444
    expect(getVertexSpecs(maxZoom, vertex)).toEqual(
      verticesSpecs.zoomedIn.smallScreen.smallVertex,
    )
  })

  it('should return small screen specs when the screen width is smaller than 444px', () => {
    window.innerWidth = 400
    expect(getVertexSpecs(minZoom, vertex)).toEqual(
      verticesSpecs.zoomedOut.smallScreen.smallVertex,
    )
  })

  it('should return larger screen specs when the screen width is greater than 444px', () => {
    window.innerWidth = 600
    expect(getVertexSpecs(minZoom, vertex)).toEqual(
      verticesSpecs.zoomedOut.largerScreen.smallVertex,
    )
  })

  it('should return small vertices when connections are 3 or less', () => {
    window.innerWidth = 444
    expect(getVertexSpecs(minZoom, vertex)).toEqual(
      verticesSpecs.zoomedOut.smallScreen.smallVertex,
    )
  })

  it('should return medium vertices when vertex connections are between 3 and 6', () => {
    window.innerWidth = 444
    vertex.pendingConnections = 5
    expect(getVertexSpecs(minZoom, vertex)).toEqual(
      verticesSpecs.zoomedOut.smallScreen.mediumVertex,
    )
  })

  it('should return large vertices when vertex connections are 7 or more', () => {
    window.innerWidth = 444
    vertex.pendingConnections = 8
    expect(getVertexSpecs(minZoom, vertex)).toEqual(
      verticesSpecs.zoomedOut.smallScreen.largeVertex,
    )
  })

  it('should return 0 size radius and tapArea when there are no pending connections or shapes', () => {
    window.innerWidth = 444
    vertex.pendingConnections = 0
    vertex.completedShapes = [1, 2]
    vertex.preDrawnShapes = [3]
    vertex.shapes = [1, 2, 3]
    expect(getVertexSpecs(minZoom, vertex).radius).toEqual(0)
    expect(getVertexSpecs(minZoom, vertex).tapArea).toEqual(0)
  })

  it('should return hover styles', () => {
    const hoverStyles = getVertexStyles(1, null, 1)
    expect(hoverStyles.innerFillStyle).toEqual(colors.black)
    expect(hoverStyles.outerFillStyle).toEqual(
      colors.transparentBlack,
    )
    expect(hoverStyles.innerStrokeStyle).toEqual(colors.black)
    expect(hoverStyles.outerStrokeStyle).toEqual(colors.black)
    expect(hoverStyles.fontColor).toEqual(colors.tan)
  })

  it('should return selected styles', () => {
    const selectedStyles = getVertexStyles(1, 1, 2)
    expect(selectedStyles.innerFillStyle).toEqual(colors.gold)
    expect(selectedStyles.outerFillStyle).toEqual(
      colors.transparentGold,
    )
    expect(selectedStyles.innerStrokeStyle).toEqual(colors.gold)
    expect(selectedStyles.outerStrokeStyle).toEqual(colors.gold)
    expect(selectedStyles.fontColor).toEqual(colors.black)
  })

  it('should return nearby vertex styles', () => {
    const selectedStyles = getVertexStyles(0, 2, 0)
    expect(selectedStyles.innerFillStyle).toEqual(colors.gold)
    expect(selectedStyles.outerFillStyle).toEqual(
      colors.transparentGold,
    )
    expect(selectedStyles.innerStrokeStyle).toEqual(colors.gold)
    expect(selectedStyles.outerStrokeStyle).toEqual(colors.gold)
    expect(selectedStyles.fontColor).toEqual(colors.black)
  })

  it('should return minimum mobile line thickness at minimum zoom', () => {
    const minWidth = 2
    const lineStyles = getLineSpecs(minZoom, true)
    expect(lineStyles).toEqual(minWidth)
  })

  it('should return maximum mobile line thickness at maximum zoom', () => {
    const maxWidth = 4
    const lineStyles = getLineSpecs(maxZoom, true)
    expect(lineStyles).toEqual(maxWidth)
  })

  it('should return minimum desktop line thickness at minimum zoom', () => {
    const minWidth = 3
    const lineStyles = getLineSpecs(minZoom, false)
    expect(lineStyles).toEqual(minWidth)
  })

  it('should return maximum desktop line thickness at maximum zoom', () => {
    const maxWidth = 5
    const lineStyles = getLineSpecs(maxZoom, false)
    expect(lineStyles).toEqual(maxWidth)
  })
})
