import {
  shouldLineUndo,
  getConnections,
  isLineIncluded,
  isShapeComplete,
  clearConnections,
  doesLineIntersect,
  isLastPuzzleInPack,
  hasLineCompletedShape,
  getNewlyCompletedShapes,
  calculatePendingConnections,
} from '../helpers'

const connectedLines = [
  { a: [2, 1], b: [6, 3] },
  { a: [0, 3], b: [2, 8] },
  { a: [5, 9], b: [8, 7] },
]

describe('VERTEX: HELPERS', () => {
  it('should return false if line is not included in array', () => {
    const line = [0, 1]
    const lineArr = [
      [0, 2],
      [2, 0],
    ]
    expect(isLineIncluded(line, lineArr)).toBe(false)
  })

  it('should tell if lines drawn completed given shape', () => {
    const shapeLines = [
      [0, 1],
      [0, 2],
      [1, 2],
    ]

    const drawnLines = [
      [4, 5],
      [1, 2],
      [2, 0],
      [0, 1],
    ]

    expect(isShapeComplete(shapeLines, drawnLines)).toBe(true)
  })

  it('should tell if lines drawn did not complete given shape', () => {
    const shapeLines = [
      [0, 1],
      [0, 2],
      [1, 2],
    ]

    const drawnLines = [
      [4, 5],
      [1, 2],
      [2, 0],
    ]
    expect(isShapeComplete(shapeLines, drawnLines)).toBe(false)
  })

  it('should return true if new line has completed given shape', () => {
    const shapeLines = [
      [0, 1],
      [0, 2],
      [1, 2],
    ]

    const drawnLines = [
      [4, 5],
      [1, 2],
      [0, 1],
    ]
    const newLine = [2, 0]
    expect(
      hasLineCompletedShape(shapeLines, drawnLines, newLine),
    ).toBe(true)
  })

  it('should return false if new line has not completed given shape', () => {
    const shapeLines = [
      [0, 1],
      [0, 2],
      [1, 2],
    ]

    const drawnLines = [
      [4, 5],
      [1, 2],
      [0, 1],
    ]

    const newLine = [3, 0]

    expect(
      hasLineCompletedShape(shapeLines, drawnLines, newLine),
    ).toBe(false)
  })

  it('should ignore previously completed shapes', () => {
    const shapeLines = [
      [0, 1],
      [0, 2],
      [1, 2],
    ]

    const drawnLines = [
      [0, 1],
      [0, 2],
      [1, 2],
    ]

    const newLine = [3, 0]

    expect(
      hasLineCompletedShape(shapeLines, drawnLines, newLine),
    ).toBe(false)
  })

  it('should get all newly completed shapes given previously drawn lines and new line', () => {
    const shapes = {
      0: {
        lines: [
          [0, 1],
          [0, 2],
          [1, 2],
        ],
        vertices: [0, 1, 2],
      },
    }

    const drawnLines = [
      [4, 5],
      [1, 2],
      [2, 0],
    ]

    const newLine = [0, 1]

    expect(
      getNewlyCompletedShapes(shapes, drawnLines, newLine),
    ).toEqual([shapes[0]])
  })

  it('should return true if a given line has one intersection with a connected line', () => {
    expect(doesLineIntersect([4, 1], [1, 4], connectedLines)).toBe(
      true,
    )
  })

  it('should return false if a given line is parallel to a connected line', () => {
    expect(doesLineIntersect([3, 4], [5, 5], connectedLines)).toBe(
      false,
    )
  })

  it('should return true if a given line overlaps a connected line', () => {
    expect(doesLineIntersect([2, 8], [0, 3], connectedLines)).toBe(
      true,
    )
  })

  it('should return false if a given line does not intersect with a connected line', () => {
    expect(doesLineIntersect([8, 4], [8, 5], connectedLines)).toBe(
      false,
    )
  })

  it('should return new set of drawnLines and new pending connections when a vertex is cleared', () => {
    const state = {
      vertices: {
        0: { id: 0, pendingConnections: 1 },
        1: { id: 1, pendingConnections: 0 },
        2: { id: 2, pendingConnections: 1 },
      },
      drawnLines: [
        [0, 1],
        [1, 2],
        [3, 4],
      ],
      drawnShapes: [],
    }

    const result = {
      drawnLines: [[3, 4]],
      otherVertices: [0, 2],
    }

    expect(clearConnections(1, state)).toEqual(result)
  })

  it('should return proper number of pending connections for a vertex', () => {
    const shapes = {
      0: {
        lines: [
          [0, 1],
          [0, 2],
          [1, 2],
        ],
      },
      1: {
        lines: [
          [1, 5],
          [1, 6],
          [5, 6],
        ],
      },
      2: {
        lines: [
          [1, 5],
          [1, 8],
          [5, 8],
        ],
      },
    }

    expect(getConnections(shapes, 1)).toBe(5)
  })

  it('should undo connections if line is not in a completed shape', () => {
    const state = {
      shapes: {
        7: {
          id: 7,
          vertices: [0, 2, 7],
          lines: [
            [0, 2],
            [0, 7],
            [2, 7],
          ],
        },
        8: {
          id: 8,
          vertices: [0, 7, 9],
          lines: [
            [0, 7],
            [0, 9],
            [7, 9],
          ],
        },
      },
      drawnLines: [[7, 9]],
      drawnShapes: [],
    }

    expect(shouldLineUndo(state.drawnLines[0], state)).toBe(true)
  })

  it('should not undo connections if line is in a completed shape', () => {
    const state = {
      shapes: {
        7: {
          id: 7,
          vertices: [0, 2, 7],
          lines: [
            [0, 2],
            [0, 7],
            [2, 7],
          ],
        },
        8: {
          id: 8,
          vertices: [0, 7, 9],
          lines: [
            [0, 7],
            [0, 9],
            [7, 9],
          ],
        },
      },
      drawnLines: [[7, 0]],
      drawnShapes: [7],
    }

    expect(shouldLineUndo(state.drawnLines[0], state)).toBe(false)
  })

  it('should calculcate proper pending connection number when loading progress', () => {
    const puzzle = {
      pendingConnections: {
        0: 5,
        1: 5,
        2: 5,
      },
    }

    const newPendingConnections = {
      0: 3,
      1: 3,
      2: 3,
    }

    const progress = {
      drawnLines: [
        [0, 1],
        [0, 2],
        [1, 2],
      ],
    }

    expect(calculatePendingConnections(puzzle, progress)).toEqual(
      newPendingConnections,
    )
  })

  it('should return same pending connections if there is no progress', () => {
    const puzzle = {
      pendingConnections: {
        0: 5,
        1: 5,
        2: 5,
      },
    }

    const newPendingConnections = {
      0: 5,
      1: 5,
      2: 5,
    }

    const progress = {}

    expect(calculatePendingConnections(puzzle, progress)).toEqual(
      newPendingConnections,
    )
  })

  it('should determine if puzzle is last in pack', () => {
    expect.assertions(2)
    const pack = [{ id: 0 }, { id: 1 }]
    expect(isLastPuzzleInPack(0, pack)).toBe(false)
    expect(isLastPuzzleInPack(1, pack)).toBe(true)
  })
})
