export const isLineIncluded = (line, linesArr) =>
  linesArr.some(
    (lineEl) => lineEl.includes(line[0]) && lineEl.includes(line[1]),
  )

export const isShapeComplete = (shapeLines, drawnLines) =>
  shapeLines.every((shapeLine) =>
    isLineIncluded(shapeLine, drawnLines),
  )

export const hasLineCompletedShape = (
  shapeLines,
  drawnLines,
  newLine,
) =>
  isLineIncluded(newLine, shapeLines) &&
  isShapeComplete(shapeLines, [...drawnLines, newLine])

export const getNewlyCompletedShapes = (
  shapes,
  drawnLines,
  newLine,
) =>
  Object.values(shapes).filter((shape) =>
    hasLineCompletedShape(shape.lines, drawnLines, newLine),
  )

export const getConnections = (
  shapes,
  vertex,
  preDrawnLines = [],
) => {
  const preDrawn = preDrawnLines.map((line) => line.toString())
  const total = []

  Object.keys(shapes).forEach((shape) => {
    const { lines } = shapes[shape]

    lines.forEach((line) => {
      if (line.includes(parseInt(vertex))) {
        const lineStr = line.toString()

        if (!total.includes(lineStr) && !preDrawn.includes(lineStr)) {
          total.push(lineStr)
        }
      }
    })
  })

  return total.length
}

export const clearConnections = (vertexId, state) => {
  const { drawnLines, drawnShapes, shapes } = state

  const newLines = []
  const otherVertices = []

  const drawnShapesLines = drawnShapes
    .map((shapeId) => shapes[shapeId].lines.map((el) => el))
    .flat()

  drawnLines.forEach((line) => {
    if (
      line.includes(vertexId) &&
      !isLineIncluded(line, drawnShapesLines)
    ) {
      const otherVertex = vertexId === line[0] ? line[1] : line[0]

      otherVertices.push(otherVertex)
    } else {
      newLines.push(line)
    }
  })

  return { drawnLines: newLines, otherVertices }
}

// Restore pending connections after deletion of a line
export const restorePendingConnections = (
  state,
  affectedVertices,
  vertexId,
) => {
  const result = {}

  affectedVertices.forEach((vId) => {
    result[vId] = state.pendingConnections[vId] + 1
  })

  result[vertexId] =
    state.pendingConnections[vertexId] + affectedVertices.length

  return result
}

// Calculate new pending connections when loading a puzzle with progress
export const calculatePendingConnections = (puzzle, progress) => {
  if (Object.keys(progress).length === 0) {
    return puzzle.pendingConnections
  }

  const pendingConnections = {
    ...puzzle.pendingConnections,
  }

  const flatDrawnLines = progress.drawnLines.flat()

  Object.keys(pendingConnections).forEach((k) => {
    const numberOfConnections = flatDrawnLines.reduce(
      (totalConnections, vertexId) => {
        return vertexId === parseInt(k)
          ? totalConnections + 1
          : totalConnections
      },
      0,
    )

    pendingConnections[k] -= numberOfConnections
  })

  return pendingConnections
}

export const shouldLineUndo = (line, state) => {
  const { drawnShapes, shapes } = state

  const drawnShapesLines = drawnShapes
    .map((shapeId) => shapes[shapeId].lines.map((el) => el))
    .flat()

  return !isLineIncluded(line, drawnShapesLines)
}

/*
  DETERMINING COLLISIONS
  returns true if the line from (a,b)->(c,d) collides with (p,q)->(r,s)
  AND the point of intersection is not a shared vertex
  modified from https://stackoverflow.com/questions/9043805/test-if-two-lines-intersect-javascript-function
*/
const checkCollision = ([a, b], [c, d], [p, q], [r, s]) => {
  const slopeDiff = (c - a) * (s - q) - (r - p) * (d - b)

  if (slopeDiff === 0) {
    // lines are parallel
    if (
      (a === p && b === q && c === r && d === s) ||
      (a === r && b === s && c === p && d === q)
    ) {
      return true
    } // lines are the same

    return false
  }

  const lambda = ((s - q) * (r - a) + (p - r) * (s - b)) / slopeDiff

  const gamma = ((b - d) * (r - a) + (c - a) * (s - b)) / slopeDiff

  const doesCollide =
    lambda > 0 && lambda < 1 && gamma > 0 && gamma < 1

  if (!doesCollide) return false

  const getSlope = (pointA, pointB) =>
    (pointB[1] - pointA[1]) / (pointB[0] - pointA[0])

  // if lines collide then check to see if point of collision is a vertex
  const m1 = getSlope([a, b], [c, d]) // slope of first line
  const b1 = b - a * m1 // y-intercept of first line

  const m2 = getSlope([p, q], [r, s]) // slope of second line
  const b2 = q - p * m2 // y-intercept of second line

  const x = (b2 - b1) / (m1 - m2) // solve for x-coordinate of intersection
  const y = m1 * x + b1

  const intersectionPoint = [Math.round(x), Math.round(y)].toString()

  const lineOneStartPoint = [Math.round(a), Math.round(b)].toString()

  const lineOneEndPoint = [Math.round(c), Math.round(d)].toString()

  const lineTwoStartPoint = [Math.round(p), Math.round(q)].toString()

  const lineTwoEndPoint = [Math.round(r), Math.round(s)].toString()

  const isVertexPointOfCollision =
    lineOneStartPoint === intersectionPoint ||
    lineOneEndPoint === intersectionPoint ||
    lineTwoStartPoint === intersectionPoint ||
    lineTwoEndPoint === intersectionPoint

  // if a vertex is the only point of collision, the lines should be able to connect
  return !isVertexPointOfCollision
}

export const doesLineIntersect = (start, end, lines) =>
  lines.some((line) => checkCollision(line.a, line.b, start, end))

/*
  GET CENTER
  Used for determining center of all pointers
  Taken from: https://github.com/hammerjs/hammer.js/blob/master/src/inputjs/get-center.js
*/
export const getCenter = (pointers) => {
  const pointersLength = pointers.length

  // no need to loop when only one touch
  if (pointersLength === 1) {
    return {
      x: Math.round(pointers[0].clientX),
      y: Math.round(pointers[0].clientY),
    }
  }

  let x = 0
  let y = 0
  let i = 0

  while (i < pointersLength) {
    x += pointers[i].clientX
    y += pointers[i].clientY
    i += 1
  }

  return {
    x: Math.round(x / pointersLength),
    y: Math.round(y / pointersLength),
  }
}

export const prop = (name, initial) => ({
  get() {
    if (!this.hasOwnProperty(name)) {
      this[name] = initial
    }

    return this[name]
  },
  set(v) {
    this[name] = v

    this.update()
  },
})

export const isUsingMultiTouch = (e) =>
  e.touches && e.touches.length > 1

export const isLastPuzzleInPack = (puzzleId, puzzlePack) =>
  puzzleId === puzzlePack[puzzlePack.length - 1].id
