import { xhr, env } from 'foundation' // eslint-disable-line

const getCurrentTimestamp = () => Math.floor(Date.now() / 1000)
const isEmpty = (obj) => Object.keys(obj).length === 0

/*
  PACK COMPLETION
*/
export const saveLocalPackCompletion = (packId, isComplete) => {
  try {
    window.localStorage.setItem(
      'vertexPackCompletion',
      JSON.stringify({
        pack_id: packId,
        is_complete: isComplete,
        timestamp: getCurrentTimestamp(),
      }),
    )
  } catch (e) {
    console.error('Vertex: Failed to save local pack completion:', e)
  }
}

export const saveRemotePackCompletion = (packId, isComplete) => {
  return xhr
    .post(`${env.api}/svc/vertex/v1/game/store-completion`, {
      pack_id: packId,
      is_complete: isComplete,
      timestamp: getCurrentTimestamp(),
    })
    .catch((e) => {
      console.error(
        'Vertex: Failed to save remote pack completion:',
        e,
      )
    })
}

export const savePackCompletion = (packId, isComplete) => {
  saveLocalPackCompletion(packId, isComplete)
  saveRemotePackCompletion(packId, isComplete)
}

export const getLocalPackCompletion = () => {
  try {
    const status = JSON.parse(
      window.localStorage.getItem('vertexPackCompletion'),
    )

    if (status) {
      return {
        packId: status.pack_id,
        isComplete: status.is_complete,
        timestamp: status.timestamp,
      }
    }

    return {}
  } catch (e) {
    console.error('Vertex: Failed to get local pack completion:', e)
    return null
  }
}

export const getRemotePackCompletion = () => {
  return xhr
    .get(`${env.api}/svc/vertex/v1/game/recent-pack-completion.json`)
    .then((res) => ({
      packId: res.pack_id,
      isComplete: res.is_complete,
      timestamp: res.timestamp,
    }))
    .catch((e) => {
      console.error('Vertex: Failed to get remote pack completion', e)
      return {}
    })
}

export const getPackCompletion = () => {
  const localPackCompletion = getLocalPackCompletion()

  return getRemotePackCompletion().then((remotePackCompletion) => {
    if (isEmpty(remotePackCompletion)) {
      return localPackCompletion
    }

    const localTimestamp = isEmpty(localPackCompletion)
      ? 0
      : localPackCompletion.timestamp

    const remoteTimestamp = isEmpty(remotePackCompletion)
      ? 0
      : remotePackCompletion.timestamp

    if (localTimestamp === 0 && remoteTimestamp === 0) {
      return {}
    }

    if (remoteTimestamp > localTimestamp) {
      const { packId, isComplete } = remotePackCompletion
      saveLocalPackCompletion(packId, isComplete)
      return remotePackCompletion
    }

    const { packId, isComplete } = localPackCompletion
    saveRemotePackCompletion(packId, isComplete)
    return localPackCompletion
  })
}

/*
  PUZZLE PROGRESS
*/
export const saveLocalProgress = ({
  id,
  packId,
  puzzleType,
  isComplete,
  drawnLines,
  drawnShapes,
}) => {
  try {
    const progress = {
      puzzle_id: id,
      pack_id: packId,
      puzzle_type: puzzleType,
      is_complete: isComplete,
      drawn_lines: drawnLines,
      drawn_shapes: drawnShapes,
      timestamp: getCurrentTimestamp(),
    }

    if (puzzleType === 'daily') {
      window.localStorage.setItem(
        'vertexProgress',
        JSON.stringify(progress),
      )
    } else {
      window.localStorage.setItem(
        'vertexPackProgress',
        JSON.stringify([progress]),
      )
    }
  } catch (e) {
    console.error('Vertex: Failed to save local progress:', e)
  }
}

export const saveRemoteProgress = ({
  id,
  packId,
  puzzleType,
  isComplete,
  drawnLines,
  drawnShapes,
}) => {
  return xhr
    .post(`${env.api}/svc/vertex/v1/game/store-progress`, {
      puzzle_id: id,
      pack_id: packId,
      puzzle_type: puzzleType,
      is_complete: isComplete,
      drawn_lines: drawnLines,
      drawn_shapes: drawnShapes,
      timestamp: getCurrentTimestamp(),
    })
    .catch((e) => {
      console.error('Vertex: Failed to save remote progress:', e)
    })
}

export const saveProgress = (progress) => {
  saveLocalProgress(progress)
  saveRemoteProgress(progress)
}

export const getLocalPuzzleProgress = (puzzleId) => {
  try {
    const progress = JSON.parse(
      window.localStorage.getItem('vertexProgress'),
    )

    if (progress && progress.puzzle_id === puzzleId) {
      return {
        id: progress.puzzle_id,
        packId: progress.pack_id,
        puzzleType: progress.puzzle_type,
        timestamp: progress.timestamp,
        isComplete: progress.is_complete,
        drawnLines: Array.isArray(progress.drawn_lines)
          ? progress.drawn_lines
          : [],
        drawnShapes: Array.isArray(progress.drawn_shapes)
          ? progress.drawn_shapes
          : [],
      }
    }

    return {}
  } catch (e) {
    console.error('Vertex: Failed to get local progress:', e)
    return {}
  }
}

export const getRemotePuzzleProgress = (puzzleId) => {
  return xhr
    .get(
      `${env.api}/svc/vertex/v1/game/puzzle-progress/${puzzleId}.json`,
    )
    .then((res) => ({
      id: res.puzzle_id,
      packId: res.pack_id,
      puzzleType: res.puzzle_type,
      timestamp: res.timestamp,
      isComplete: res.is_complete,
      drawnLines: Array.isArray(res.drawn_lines)
        ? res.drawn_lines
        : [],
      drawnShapes: Array.isArray(res.drawn_shapes)
        ? res.drawn_shapes
        : [],
    }))
    .catch((e) => {
      console.error('Vertex: Failed to get remote progress', e)
      return {}
    })
}

export const getPuzzleProgress = (puzzleId) => {
  const localProgress = getLocalPuzzleProgress(puzzleId)

  return getRemotePuzzleProgress(puzzleId).then((remoteProgress) => {
    if (isEmpty(remoteProgress)) return localProgress

    const localTimestamp = isEmpty(localProgress)
      ? 0
      : localProgress.timestamp

    const remoteTimestamp = isEmpty(remoteProgress)
      ? 0
      : remoteProgress.timestamp

    if (localTimestamp === 0 && remoteTimestamp === 0) {
      return {}
    }

    if (remoteTimestamp > localTimestamp) {
      saveLocalProgress(remoteProgress)
      return remoteProgress
    }

    saveRemoteProgress(localProgress)
    return localProgress
  })
}

/*
  PACK PROGRESS
*/
export const getLocalPackProgress = (packId) => {
  try {
    const packProgress = JSON.parse(
      window.localStorage.getItem('vertexPackProgress'),
    )

    if (packProgress && packProgress.length !== 0) {
      const filtered = packProgress.filter(
        (progress) => progress.pack_id === packId,
      )

      const pp = filtered.map((progress) => ({
        id: progress.puzzle_id,
        packId: progress.pack_id,
        puzzleType: progress.puzzle_type,
        timestamp: progress.timestamp,
        isComplete: progress.is_complete,
        drawnLines: Array.isArray(progress.drawn_lines)
          ? progress.drawn_lines
          : [],
        drawnShapes: Array.isArray(progress.drawn_shapes)
          ? progress.drawn_shapes
          : [],
      }))

      return pp
    }

    return []
  } catch (e) {
    console.error('Vertex: Failed to get local pack progress:', e)
    return []
  }
}

export const getRemotePackProgress = (packId) => {
  return xhr
    .get(`${env.api}/svc/vertex/v1/game/pack-progress/${packId}.json`)
    .then((res) =>
      res.map((puzzProgress) => ({
        id: puzzProgress.puzzle_id,
        timestamp: puzzProgress.timestamp,
        isComplete: puzzProgress.is_complete,
        drawnLines: Array.isArray(puzzProgress.drawn_lines)
          ? puzzProgress.drawn_lines
          : [],
        drawnShapes: Array.isArray(puzzProgress.drawn_shapes)
          ? puzzProgress.drawn_shapes
          : [],
      })),
    )
    .catch((e) => {
      console.error('Vertex: Failed to get remote pack progress', e)
      return []
    })
}

export const getPackProgress = (packId) => {
  const localProgress = getLocalPackProgress(packId)

  return getRemotePackProgress(packId).then((remoteProgress) => {
    if (remoteProgress.length === 0) return localProgress

    return remoteProgress
  })
}

/*
  HANDLE PROGRESS
*/
export const handleProgress = (store, setCurrentPuzzle) => {
  getPackCompletion().then(({ packId = '', isComplete = false }) => {
    const {
      tutorialPuzzles,
      starterPackPuzzles,
      dailyPuzzle,
    } = store.getState()

    const hasCompletedTutorial = packId === 'tutorial' && isComplete

    const hasCompletedStarterPack =
      packId === 'starter-pack' && isComplete

    if (hasCompletedTutorial) {
      store.dispatch({ type: 'COMPLETE_TUTORIAL' })
    } else if (hasCompletedStarterPack) {
      store.dispatch({ type: 'COMPLETE_STARTER_PACK' })
    }

    if (hasCompletedStarterPack) {
      getPuzzleProgress(dailyPuzzle.id).then((storedProgress) =>
        setCurrentPuzzle(dailyPuzzle, storedProgress),
      )
    } else if (hasCompletedTutorial) {
      getPackProgress('starter-pack').then((storedProgress) => {
        let progress = {}
        const reversedStarterPuzzles = [
          ...starterPackPuzzles,
        ].reverse()

        const starterPuzzle = reversedStarterPuzzles.find(
          (starterPuzz) => {
            progress =
              storedProgress &&
              storedProgress.find(
                (progress) => progress.id === starterPuzz.id,
              )
            if (progress) return starterPuzz

            return false
          },
        )

        if (starterPuzzle) {
          setCurrentPuzzle(starterPuzzle, progress)
        } else setCurrentPuzzle(starterPackPuzzles[0])
      })
    } else {
      setCurrentPuzzle(tutorialPuzzles[0])
    }
  })

  setInterval(() => {
    const currState = store.getState()
    if (currState.puzzleType !== 'tutorial') {
      saveRemoteProgress(currState)
    }
  }, 10000)

  /**
   * When the user goes offline
   * their remote saves will begin to fail,
   * so when they come back online,
   * save wherever they are
   */
  window.addEventListener('online', () => {
    const currState = store.getState()
    if (currState.puzzleType !== 'tutorial') {
      saveRemoteProgress(currState)
    }
  })
}
