import React, { useContext } from 'react'
import { act, waitFor, render, screen } from '@testing-library/react'

import { useUser } from 'user'
import { retrieveMeterValue, willUseMeterToGate } from 'shared/meter'
import useAbra from 'shared/abra/useAbra'

import {
  MeterExperimentsContext,
  MeterExperimentProvider,
} from './MeterExperimentsProvider'

jest.mock('user', () => ({
  useUser: jest.fn(() => ({
    userType: { hasXwd: true },
    hasLoaded: true,
  })),
}))

jest.mock('shared/abra/useAbra', () =>
  jest.fn(() => ({
    reportExposure: () => {},
  })),
)

jest.mock('../../../shared/meter', () => ({
  retrieveMeterValue: jest.fn(),
  willUseMeterToGate: jest.fn(),
}))

const Consumer = () => {
  const { meterGranted } = useContext(MeterExperimentsContext)

  return (
    <>
      <div data-testid="meterGranted">{`${meterGranted}`}</div>
    </>
  )
}

describe('MeterExperimentProvider', () => {
  beforeEach(jest.clearAllMocks)
  afterEach(jest.clearAllMocks)

  it('should not call Meter if user is subscribed', () => {
    render(<MeterExperimentProvider />)
    expect(retrieveMeterValue).not.toHaveBeenCalled()
  })

  it('should call Meter if user is not subscribed', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false },
      hasLoaded: true,
    })

    await act(async () => {
      render(<MeterExperimentProvider />)

      await waitFor(() => {
        expect(retrieveMeterValue).toHaveBeenCalled()
      })
    })
  })



  it('should grant access if Meter grants access', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false },
      hasLoaded: true,
    })

    // @ts-ignore
    useAbra.mockReturnValueOnce({
      reportExposure: () => {},
    })

    // @ts-ignore
    willUseMeterToGate.mockReturnValueOnce(true)

    // @ts-ignore
    retrieveMeterValue.mockReturnValueOnce({
      granted: true,
      grantReason: 'IP',
    })

    await act(async () => {
      render(
        <MeterExperimentProvider>
          <Consumer />
        </MeterExperimentProvider>,
      )

      const meterGrantedElem = screen.getByTestId('meterGranted')

      await waitFor(() => {
        expect(retrieveMeterValue).toHaveBeenCalled()
        expect(meterGrantedElem.textContent).toEqual('true')
      })
    })
  })

  it('should not grant access if Meter does not grant access', async () => {
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { hasXwd: false },
      hasLoaded: true,
    })

    // @ts-ignore
    useAbra.mockReturnValueOnce({
      reportExposure: () => {},
    })

    // @ts-ignore
    willUseMeterToGate.mockReturnValueOnce(true)
    
    // @ts-ignore
    retrieveMeterValue.mockReturnValueOnce({
      granted: false,
      grantReason: 'METER_LIMIT',
    })

    await act(async () => {
      render(
        <MeterExperimentProvider>
          <Consumer />
        </MeterExperimentProvider>,
      )

      const meterGrantedElem = screen.getByTestId('meterGranted')

      await waitFor(() => {
        expect(retrieveMeterValue).toHaveBeenCalled()
        expect(meterGrantedElem.textContent).toEqual('false')
      })
    })
  })
})
