import React, {
  createContext,
  useEffect,
  useState,
  FC,
} from 'react'

import { useUser } from 'user'
import { willUseMeterToGate, retrieveMeterValue } from 'shared/meter/index'
import type { MeterProperties } from 'shared/meter/MeterTypes'

interface MeterContextType {
  meterValue: MeterProperties | null
  updateMeterLevel: (hasCompletedStarterPack: boolean) => void
  meterGranted: boolean
}

const defaultValue: MeterContextType = {
  meterValue: null,
  updateMeterLevel: () => {},
  meterGranted: false,
}

export const MeterExperimentsContext =
  createContext<MeterContextType>(defaultValue)

export const MeterExperimentProvider: FC = ({ children }) => {
  const [meterValue, setMeterValue] =
    useState<MeterProperties | null>(null)

  const [meterLevel, setMeterLevel] = useState<number>(1)

  const [allowMeterGrant, setAllowMeterGrant] =
    useState<boolean>(false)
  const [meterGranted, setMeterGranted] = useState<boolean>(false)

  const { hasLoaded: hasUserTypeLoaded, userType } = useUser()

  /**
   * Updates the Meter level according to current user's level
   * If they have not completed the starter pack, they will be at Meter level 1
   * If they have, they will be at Meter level 2
   * @param hasCompletedStarterPack whether or not the user has completed the starter pack
   */
  const updateMeterLevel = (hasCompletedStarterPack: boolean) => {
    if (hasCompletedStarterPack) setMeterLevel(2)
    else setMeterLevel(1)
  }

  useEffect(() => {
    const shouldFetchMeter =
      hasUserTypeLoaded && !userType.hasXwd

    const retrieveMeterValueAsync = async () => {
      try {
        const meterProperties: MeterProperties =
          await retrieveMeterValue({
            level: meterLevel,
          })
        setMeterValue(meterProperties)
      } catch (error) {
        console.error('Failed to retrieve meter value:', error)
      }
    }

    shouldFetchMeter && retrieveMeterValueAsync()
  }, [
    hasUserTypeLoaded,
    userType.hasXwd,
    meterLevel,
  ])

  // Allow Meter Gate logic
  useEffect(() => {
    if (willUseMeterToGate()) {
      setAllowMeterGrant(true)
    }
  }, [])
  
  useEffect(() => {
    if (
      meterValue?.granted &&
      meterValue?.grantReason === 'IP' &&
      allowMeterGrant
    ) {
      setMeterGranted(true)
    }
  }, [meterValue, allowMeterGrant])

  const contextValue: MeterContextType = {
    meterValue,
    updateMeterLevel,
    meterGranted,
  }

  return (
    <MeterExperimentsContext.Provider value={contextValue}>
      {children}
    </MeterExperimentsContext.Provider>
  )
}
