import reducer from '../reducer'

const v0 = {
  id: 0,
  coords: [0, 0],
  shapes: [0],
}

const v1 = {
  id: 1,
  coords: [1, 1],
  shapes: [0],
}

const v2 = {
  id: 2,
  coords: [2, 2],
  shapes: [0],
}

const shape0 = {
  id: 0,
  color: '#000',
  lines: [
    [0, 1],
    [0, 2],
    [1, 2],
  ],
  vertices: [0, 1, 2],
}

const state = {
  isComplete: false,
  drawnLines: [],
  drawnShapes: [],
  pendingConnections: {
    0: 2,
    1: 2,
    2: 2,
  },
  vertices: {
    0: { ...v0 },
    1: { ...v1 },
    2: { ...v2 },
  },
  shapes: {
    0: { ...shape0 },
  },
}

describe('VERTEX - REDUCER', () => {
  it('should return default state', () => {
    expect(reducer(state, { type: 'UNKNOWN_ACTION' })).toEqual(state)
  })

  it('should make correct updates when drawing a line', () => {
    const action = { type: 'DRAW_LINE', line: [0, 1] }
    const newState = {
      ...state,
      pendingConnections: {
        ...state.pendingConnections,
        0: 1,
        1: 1,
      },
      drawnLines: [[0, 1]],
    }

    expect(reducer(state, action)).toEqual(newState)
  })

  it('should make correct updates when drawing a shape', () => {
    const action = {
      type: 'DRAW_SHAPE',
      newShapeIds: [0],
      line: [0, 1],
    }

    const oldState = {
      ...state,
      pendingConnections: { 0: 1, 1: 1, 2: 0 },
      drawnLines: [
        [0, 2],
        [1, 2],
      ],
    }

    const newState = {
      ...state,
      pendingConnections: { 0: 0, 1: 0, 2: 0 },
      drawnLines: [
        [0, 2],
        [1, 2],
        [0, 1],
      ],
      drawnShapes: [0],
    }

    expect(reducer(oldState, action)).toEqual(newState)
  })

  it('should make correct updates when completing game', () => {
    const action = {
      type: 'COMPLETE_GAME',
      newShapeIds: [0],
      line: [0, 1],
    }

    const oldState = {
      ...state,
      pendingConnections: { 0: 1, 1: 1, 2: 0 },
      drawnLines: [
        [0, 2],
        [1, 2],
      ],
    }

    const newState = {
      ...state,
      isComplete: true,
      pendingConnections: { 0: 0, 1: 0, 2: 0 },
      drawnLines: [
        [0, 2],
        [1, 2],
        [0, 1],
      ],
      drawnShapes: [0],
    }

    expect(reducer(oldState, action)).toEqual(newState)
  })

  it('should only clear lines related to given vertex', () => {
    const action = {
      type: 'CLEAR_CONNECTIONS',
      vertexId: 1,
    }

    const oldState = {
      ...state,
      pendingConnections: { 0: 0, 1: 1, 2: 1 },
      drawnLines: [
        [0, 1],
        [0, 2],
        [4, 5],
      ],
    }

    const newState = {
      ...state,
      pendingConnections: { 0: 1, 1: 2, 2: 1 },
      drawnLines: [
        [0, 2],
        [4, 5],
      ],
    }
    expect(reducer(oldState, action)).toEqual(newState)
  })

  it('should not clear lines that are already part of a completed shape', () => {
    const action = {
      type: 'CLEAR_CONNECTIONS',
      vertexId: 2,
    }

    const oldState = {
      ...state,
      pendingConnections: { 0: 0, 1: 0, 2: 0 },
      drawnLines: [
        [0, 1],
        [0, 2],
        [1, 2],
      ],
      drawnShapes: [0],
    }

    expect(reducer(oldState, action)).toEqual(oldState)
  })

  it('should reset game', () => {
    const action = { type: 'RESET_GAME' }

    const oldState = {
      ...state,
      pendingConnections: { 0: 0, 1: 0, 2: 0 },
      drawnLines: [
        [0, 1],
        [0, 2],
        [1, 2],
      ],
      drawnShapes: [0],
    }

    expect(reducer(oldState, action)).toEqual(state)
  })

  it('should complete the game', () => {
    const action = { type: 'DEBUG_COMPLETE_GAME' }

    const newState = {
      ...state,
      isComplete: true,
      drawnShapes: [0],
    }

    expect(reducer(state, action)).toEqual(newState)
  })

  it('should complete the tutorial', () => {
    const action = { type: 'COMPLETE_TUTORIAL' }

    const newState = {
      ...state,
      hasCompletedTutorial: true,
    }

    expect(reducer(state, action)).toEqual(newState)
  })

  it('should complete the starter pack', () => {
    const action = { type: 'COMPLETE_STARTER_PACK' }

    const newState = {
      ...state,
      hasCompletedTutorial: true,
      hasCompletedStarterPack: true,
    }

    expect(reducer(state, action)).toEqual(newState)
  })
})
