import {
  drawLine,
  undoLine,
  resetGame,
  setUpStore,
  clearConnections,
  setCurrentPuzzle,
  completeTutorial,
  debugCompleteGame,
  nextTutorialPuzzle,
  completeStarterPack,
  nextStarterPackPuzzle,
} from '../store'
import { saveLocalProgress } from '../../helpers/progress'

import mockGameData from './mockGameData'

jest.mock('../../helpers/progress', () => ({
  saveLocalProgress: jest.fn(),
  handleProgress: jest.fn(),
}))

describe('VERTEX: STORE', () => {
  let store

  beforeEach(() => {
    store = setUpStore(mockGameData)
    jest.clearAllMocks()
  })

  it('should render initital state', () => {
    const state = store.getState()

    expect(state.drawnLines).toHaveLength(0)
    expect(state.drawnShapes).toHaveLength(0)
    expect(state.hasCompletedTutorial).toBe(false)
    expect(state.hasCompletedStarterPack).toBe(false)
    expect(state.id).toBe(mockGameData.api.tutorial[0].id)
    expect(state.dailyPuzzle.id).toBe(mockGameData.api.id)
    expect(state.tutorialPuzzles).toHaveLength(
      mockGameData.api.tutorial.length,
    )
    expect(state.starterPackPuzzles).toHaveLength(
      mockGameData.api.starterPuzzles.length,
    )
  })

  it('should complete the tutorial', () => {
    store.dispatch(completeTutorial())
    const state = store.getState()
    expect(state.hasCompletedTutorial).toBe(true)
  })

  it('should complete the starter pack', () => {
    store.dispatch(completeStarterPack())
    const state = store.getState()
    expect(state.hasCompletedStarterPack).toBe(true)
  })

  it('should set current puzzle', () => {
    const { dailyPuzzle } = store.getState()
    store.dispatch(setCurrentPuzzle(dailyPuzzle))
    expect(store.getState().id).toBe(dailyPuzzle.id)
  })

  it('should draw a line', () => {
    const { dailyPuzzle } = store.getState()
    store.dispatch(setCurrentPuzzle(dailyPuzzle))
    store.dispatch(drawLine([0, 1]))
    const state = store.getState()
    expect(state.drawnLines).toEqual([[0, 1]])
  })

  it('should complete a shape', () => {
    const { dailyPuzzle } = store.getState()
    store.dispatch(setCurrentPuzzle(dailyPuzzle))
    store.dispatch(drawLine([0, 1]))
    store.dispatch(drawLine([1, 2]))
    store.dispatch(drawLine([0, 2]))
    const state = store.getState()
    expect(state.drawnLines).toEqual([
      [0, 1],
      [1, 2],
      [0, 2],
    ])
    expect(state.drawnShapes).toEqual([0])
  })

  it('should complete game', () => {
    store.dispatch(drawLine([0, 1]))
    store.dispatch(drawLine([1, 2]))
    store.dispatch(drawLine([0, 2]))
    const state = store.getState()
    expect(state.isComplete).toBe(true)
  })

  it('should debug complete game', () => {
    store.dispatch(debugCompleteGame())
    const state = store.getState()
    expect(state.drawnShapes).toEqual([0])
    expect(state.isComplete).toBe(true)
  })

  it('should undo line', () => {
    const { dailyPuzzle } = store.getState()
    store.dispatch(setCurrentPuzzle(dailyPuzzle))
    store.dispatch(drawLine([0, 1]))
    store.dispatch(drawLine([4, 5]))
    store.dispatch(undoLine())
    const state = store.getState()
    expect(state.drawnLines).toEqual([[0, 1]])
  })

  it('should reset game', () => {
    const { dailyPuzzle } = store.getState()
    store.dispatch(setCurrentPuzzle(dailyPuzzle))
    store.dispatch(drawLine([0, 1]))
    store.dispatch(drawLine([4, 5]))
    store.dispatch(resetGame())
    const state = store.getState()
    expect(state.drawnLines).toEqual([])
  })

  it('should clear connections', () => {
    const { dailyPuzzle } = store.getState()
    store.dispatch(setCurrentPuzzle(dailyPuzzle))
    store.dispatch(drawLine([0, 1]))
    store.dispatch(drawLine([4, 5]))
    store.dispatch(drawLine([0, 3]))
    store.dispatch(clearConnections(0))
    const state = store.getState()
    expect(state.drawnLines).toEqual([[4, 5]])
  })

  it('should display next tutorial puzzle', () => {
    store.dispatch(nextTutorialPuzzle())
    const state = store.getState()
    expect(state.id).toBe(mockGameData.api.tutorial[1].id)
  })

  it('should display next starter pack puzzle', () => {
    const firstStarterPuzzle = store.getState().starterPackPuzzles[0]
    store.dispatch(setCurrentPuzzle(firstStarterPuzzle))
    store.dispatch(nextStarterPackPuzzle())
    const state = store.getState()
    expect(state.id).toBe(mockGameData.api.starterPuzzles[1].id)
  })

  it('should save progress locally', () => {
    const firstStarterPuzzle = store.getState().starterPackPuzzles[0]
    store.dispatch(setCurrentPuzzle(firstStarterPuzzle))
    store.dispatch(drawLine([0, 1]))
    expect(saveLocalProgress).toHaveBeenCalled()
  })

  it('should not save progress locally', () => {
    store.dispatch(drawLine([0, 1]))
    expect(saveLocalProgress).not.toHaveBeenCalled()
  })
})
