import { createStore, compose, applyMiddleware } from 'redux'
import thunk from 'redux-thunk'

import {
  saveLocalProgress,
  handleProgress,
} from '../helpers/progress'
import {
  getNewlyCompletedShapes,
  shouldLineUndo,
} from '../helpers/helpers'

import reducer from './reducer'
import formatApiData from './formatApiData'

export const initialState = {
  // Current Puzzle
  id: '1',
  theme: '',
  packId: '',
  shapes: {},
  palette: [],
  vertices: {},
  puzzleType: '',
  displayDate: '',
  drawnLines: [],
  drawnShapes: [],
  isComplete: false,
  preDrawnLines: [],
  preDrawnShapes: [],
  puzzleConstructor: '',
  pendingConnections: [],

  // Game
  dailyPuzzle: {},
  tutorialPuzzles: [],
  starterPackPuzzles: [],
  hasCompletedTutorial: false,
  hasCompletedStarterPack: false,
}

/*
  ACTIONS
*/
export const resetGame = () => ({ type: 'RESET_GAME' })
export const completeTutorial = () => ({
  type: 'COMPLETE_TUTORIAL',
})
export const debugCompleteGame = () => ({
  type: 'DEBUG_COMPLETE_GAME',
})
export const completeStarterPack = () => ({
  type: 'COMPLETE_STARTER_PACK',
})

export const clearConnections = (vertex) => ({
  type: 'CLEAR_CONNECTIONS',
  vertexId: vertex,
})
export const setCurrentPuzzle = (puzzle, progress = {}) => ({
  type: 'SET_CURRENT_PUZZLE',
  puzzle,
  progress,
})

export const drawLine = (line) => (dispatch, getState) => {
  const {
    shapes,
    preDrawnLines = [],
    drawnLines,
    drawnShapes,
  } = getState()

  const totalShapeCount = Object.keys(shapes).length

  const newShapes = getNewlyCompletedShapes(
    shapes,
    [...preDrawnLines, ...drawnLines],
    line,
  )

  const newShapeIds = newShapes.map((s) => s.id)

  if (drawnLines.includes((l) => line.toString === l.toString)) {
    return
  }

  if (
    newShapes.length &&
    newShapes.length + drawnShapes.length === totalShapeCount
  ) {
    dispatch({ type: 'COMPLETE_GAME', newShapeIds, line })
  } else if (newShapes.length) {
    dispatch({ type: 'DRAW_SHAPE', newShapeIds, line })
  } else {
    dispatch({ type: 'DRAW_LINE', line })
  }
}

export const undoLine = () => (dispatch, getState) => {
  const { drawnLines } = getState()
  let drawnLinesLength = drawnLines.length - 1

  while (drawnLinesLength > -1) {
    if (shouldLineUndo(drawnLines[drawnLinesLength], getState())) {
      dispatch({
        type: 'UNDO_CONNECTION',
        idx: drawnLinesLength,
      })
      break
    } else {
      drawnLinesLength -= 1
    }
  }
}

export const nextTutorialPuzzle = () => (dispatch, getState) => {
  const { id, tutorialPuzzles } = getState()
  const index = tutorialPuzzles.findIndex((puzz) => puzz.id === id)
  const nextPuzzle = tutorialPuzzles[index + 1]
  dispatch(setCurrentPuzzle(nextPuzzle))
}

export const nextStarterPackPuzzle = () => (dispatch, getState) => {
  const { id, starterPackPuzzles } = getState()
  const index = starterPackPuzzles.findIndex((puzz) => puzz.id === id)
  const nextPuzzle = starterPackPuzzles[index + 1]
  dispatch(setCurrentPuzzle(nextPuzzle))
}

/*
  MIDDLEWARE
*/
const saveProgressLocally = (store) => (next) => (action) => {
  const result = next(action)
  const nextState = store.getState()

  if (
    nextState.puzzleType !== 'tutorial' &&
    action !== 'COMPLETE_TUTORIAL' &&
    action !== 'COMPLETE_STARTER_PACK'
  ) {
    saveLocalProgress(nextState)
  }

  return result
}

/*
  STORE
*/
export const setUpStore = (gameData) => {
  // Middleware
  const middleware = [applyMiddleware(thunk, saveProgressLocally)]

  if (window.__REDUX_DEVTOOLS_EXTENSION__) {
    middleware.push(window.__REDUX_DEVTOOLS_EXTENSION__())
  }

  // Format API Data
  const { tutorial, starterPuzzles, ...daily } = gameData.api
  const dailyPuzzle = formatApiData(daily, 'daily')
  const tutorialPuzzles = tutorial.map((puzz) =>
    formatApiData(puzz, 'tutorial'),
  )

  const starterPackPuzzles = starterPuzzles.map((puzz) =>
    formatApiData(puzz, 'starter-pack'),
  )

  const state = {
    ...initialState,
    ...tutorialPuzzles[0],
    tutorialPuzzles,
    starterPackPuzzles,
    dailyPuzzle,
    hasCompletedTutorial: false,
    hasCompletedStarterPack: false,
  }

  // Initialize Store
  const store = createStore(reducer, state, compose(...middleware))

  // Deal with loading and saving progress
  handleProgress(store, (puzzle, progress) =>
    store.dispatch(setCurrentPuzzle(puzzle, progress)),
  )
  return store
}
