import { track } from 'foundation'

export const EVENT_TYPE = 'type'
export const WHERE_IT_HAPPENED = 'where'
export const WHAT_IT_DOES = 'what'
export const GAME_ID = 'id'
export const ADDITIONAL_INFO = 'extra'

/**
 * For event specs
 * @see https://docs.google.com/spreadsheets/d/1ia9nBlnUrVi_jkE-CYsCoKElSweGi8MWlcRJWSNFG0E/edit#gid=420489817
 */
const events = {
  START_GAME: {
    // clicks "Play" welcome moment
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'start-game',
  },
  RULES: {
    // clicks "Rules" button in toolbar or in welcome moment
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'help',
  },
  COMPLETE_20: {
    // completes 20% of puzzle
    [EVENT_TYPE]: 'complete',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: '20%-complete',
  },
  COMPLETE_40: {
    // completes 40% of puzzle
    [EVENT_TYPE]: 'complete',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: '40%-complete',
  },
  COMPLETE_60: {
    // completes 60% of puzzle
    [EVENT_TYPE]: 'complete',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: '60%-complete',
  },
  COMPLETE_80: {
    // completes 80% of puzzle
    [EVENT_TYPE]: 'complete',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: '80%-complete',
  },
  COMPLETE_100: {
    // completes 100% of puzzle
    [EVENT_TYPE]: 'complete',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: '100%-complete',
  },
  CONTINUE: {
    // clicks "Continue" button on welcome moment after having previously started a puzzle
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'continue',
  },
  NEXT_PUZZLE: {
    // clicks "Next puzzle" button on welcome moment - specifically within starter pack flow
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'next-puzzle',
  },
  VIEW_SOLVED: {
    // clicks "Solved" on welcome moment after having completed puzzle
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'view-solved',
  },
  ADMIRE_MY_PUZZLE: {
    // clicks "Admire my puzzle" on congrats moment after having completed puzzle
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'view-my-puzzle',
  },
  BACK_TO_HUB: {
    // clicks "Back to hub" on congrats moment after having completed puzzle
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'back-to-hub',
  },
  RESET: {
    // clicks "Reste" button in Toolbar during gameplay
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'reset',
  },
  SKIP_TUTORIAL: {
    // clicks "Skip Tutorial" button in Toolbar
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'skip-tutorial',
  },
  SUBSCRIBE: {
    // clicks "Subscribe" button in Welcome Moment
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'subscribe',
  },
  LOCKED_PLAY: {
    // clicks "Today’s Puzzle" button in Welcome Moment
    [EVENT_TYPE]: 'click',
    [WHERE_IT_HAPPENED]: 'vertex',
    [WHAT_IT_DOES]: 'play-subscribe',
  },
}

export const trackModuleInteraction = (evt) => {
  const moduleObj = {
    module: {
      name: evt[EVENT_TYPE] || 'click',
      context: evt[GAME_ID],
      label: evt[ADDITIONAL_INFO],
      element: {
        name: evt[WHERE_IT_HAPPENED],
        label: evt[WHAT_IT_DOES],
      },
    },
    eventData: {
      pageType: 'game',
      type: evt[EVENT_TYPE] || 'click',
      trigger: 'module',
      // important to leave an empty string for `status`
      // for the event to pass correctly to google analytics
      // consult with someone from tracking team before removing
      // and confirm that the events are still available in big query
      status: '',
    },
  }

  track('moduleInteraction', moduleObj)
}

export const trackEvent = (name, gameId, gameType) => {
  if (!events[name]) {
    // we dont throw here so that we dont crash other front-end code for tracking fail
    console.error(
      new Error(
        `unrecognized tracking event '${name}' in:\n${Object.keys(
          events,
        ).join(', ')}`,
      ),
    )
    return
  }

  const evt = {
    [GAME_ID]: gameId,
    [ADDITIONAL_INFO]: gameType,
    ...events[name],
  }

  trackModuleInteraction(evt)
}
