import React from 'react'
import { Provider } from 'react-redux'
import { render, screen, waitFor } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import { format } from 'date-fns'

import createStore from '../../store'
import { selectModal } from '../../store/overlays/overlays.selectors'
import { changeSetting } from '../../store/settings/settings.actions'
import { selectHardMode } from '../../store/settings/settings.selectors'
import {
  selectBoardState,
  selectGameStatus,
} from '../../store/game/game.selectors'
import * as apiActions from '../../store/api/api.actions'
import { getDayOffsetForDate } from '../../utils/gamelogic'
import { bridgeCommands } from '../../../../../shared/foundation/js/hybrid/commands'
import { applyContract } from '../../../../../shared/foundation/js/hybrid/initMock'
import { selectCurrentStreak } from '../../store/stats/stats.selectors'

import App from './App'

const today = new Date()
const mockTodaysFormattedDate = format(today, 'yyyy-MM-dd')
const mockTodaysDayOffset = getDayOffsetForDate(today)

const mockHybridDebugger = jest.fn()
jest.mock(
  '../../../../../shared/foundation/js/hybrid/hybridDebugger',
  () => jest.fn().mockImplementation(() => mockHybridDebugger()),
)

jest.mock('user', () => ({
  useUser: () => ({
    hasLoaded: true,
    userType: require('../../../../../shared/user/userHelpers')
      .defaultUserType,
  }),
}))

jest.mock('foundation', () => ({
  env: { version: 'test', name: 'foobar', edge: 'domain.com' },
  trackModuleInteraction: jest.fn(),
  trackInteraction: jest.fn(),
  trackImpression: jest.fn(),
  trackClick: jest.fn(),
  abra: {
    reportExposure: jest.fn(),
    getVariant: jest.fn(),
  },
  xhr: {
    get: (url: string) => {
      let dayOffset
      let solution

      switch (url) {
        case 'domain.com/svc/wordle/v2/2022-04-20.json':
          dayOffset = 305
          solution = 'mimic'
          break
        case 'domain.com/svc/wordle/v2/2022-04-21.json':
          dayOffset = 306
          solution = 'linen'
          break
        case `domain.com/svc/wordle/v2/${mockTodaysFormattedDate}.json`:
          dayOffset = mockTodaysDayOffset
          solution = 'testy'
          break
        case `domain.com/svc/wordle/v2/2022-04-22.json`:
          dayOffset = null
          break
        case `domain.com/svc/wordle/v2/2022-04-23.json`:
          dayOffset = 307
          solution = 'guess'
          break
        default:
          return Promise.reject(
            new Error(
              `url "${url} not recognized as API route option in mock`,
            ),
          )
      }

      return Promise.resolve({
        days_since_launch: dayOffset,
        id: dayOffset,
        solution,
      })
    },
  },
}))
jest.mock('../Ad/useAd', () => ({
  __esModule: true,
  default: jest.fn(() => ({})),
  reportExposure: jest.fn(),
}))
jest.mock('../../../../../shared/abra/useAbra', () => () => ({
  hasAbraLoaded: jest.fn(() => true),
  getVariant: jest.fn(() => 'foo'),
  reportExposure: jest.fn(),
}))

jest.mock('../../../../../shared/abra/useAbra', () => ({
  __esModule: true,
  default: () => ({
    hasAbraLoaded: true,
    getVariant: () => '',
    reportExposure: jest.fn(),
  }),
}))

describe('<App />', () => {
  let loadSolutionSpy: jasmine.Spy
  window.navigationLinks = { login: '', subscribe: { default: '' } }

  const renderApp = async () => {
    const store = createStore()

    const app = render(
      <Provider store={store}>
        <App />
      </Provider>,
    )

    await loadSolutionSpy
    await waitFor(
      () => store.getState().persist.game.dayOffset !== null,
    )
    await (await app.findByText(/Play|Continue/)).click()

    return { store, app }
  }

  beforeEach(() => {
    jest.setSystemTime(jest.getRealSystemTime())
    window.localStorage.clear()
    loadSolutionSpy = spyOn(
      apiActions,
      'loadSolution',
    ).and.callThrough()
  })

  beforeAll(() => jest.useFakeTimers('modern'))

  afterAll(() => jest.useRealTimers())

  it('should render with empty state if local storage is empty', async () => {
    await renderApp()
    expect(screen.getAllByTestId('tile')).toHaveLength(30)
  })

  it('should play the happy path', async () => {
    const user = userEvent.setup({ delay: null })
    const { store } = await renderApp()

    await user.keyboard('t')
    await user.keyboard('e')
    await user.keyboard('s')
    await user.keyboard('t')
    await user.keyboard('y')
    await user.keyboard('{Enter}')

    expect(selectGameStatus(store.getState())).toEqual('WIN')
  })

  it('should save/load guessed words', async () => {
    const user = userEvent.setup({ delay: null })
    const { app } = await renderApp()

    await user.keyboard('a')
    await user.keyboard('a')
    await user.keyboard('r')
    await user.keyboard('t')
    await user.keyboard('i')
    await user.keyboard('{Enter}')
    app.unmount()

    // render app again, should load from localStorage
    const { store: newStore } = await renderApp()
    expect(selectBoardState(newStore.getState())).toEqual([
      'aarti',
      '',
      '',
      '',
      '',
      '',
    ])
  })

  it('should start new game if refreshed next day', async () => {
    jest.setSystemTime(new Date('April 20, 2022 23:59:00'))
    const user = userEvent.setup({ delay: null })
    const { app } = await renderApp()

    await user.keyboard('a')
    await user.keyboard('a')
    await user.keyboard('r')
    await user.keyboard('t')
    await user.keyboard('i')
    await user.keyboard('{Enter}')
    app.unmount()
    jest.advanceTimersByTime(1000 * 60 * 2) // two minutes, should be next day

    const { store } = await renderApp()

    await waitFor(
      () => store.getState().persist.game.dayOffset !== null,
    )

    expect(selectBoardState(store.getState())).toEqual([
      '',
      '',
      '',
      '',
      '',
      '',
    ])
  })

  it('should not break streak if puzzle call fails', async () => {
    jest.setSystemTime(new Date('April 21, 2022 23:59:00'))
    const user = userEvent.setup({ delay: null })
    const { app, store } = await renderApp()
    await user.keyboard('l')
    await user.keyboard('i')
    await user.keyboard('n')
    await user.keyboard('e')
    await user.keyboard('n')
    await user.keyboard('{Enter}')
    expect(selectGameStatus(store.getState())).toEqual('WIN')
    expect(selectCurrentStreak(store.getState())).toEqual(1)
    app.unmount()

    // puzzle call should fail
    jest.advanceTimersByTime(1000 * 60 * 2) // two minutes, should be next day
    const { app: app2 } = await renderApp()
    app2.unmount()

    // puzzle call should succeed
    jest.advanceTimersByTime(1000 * 60 * 60 * 24) // advance by a day
    const { store: store3 } = await renderApp()
    await user.keyboard('g')
    await user.keyboard('u')
    await user.keyboard('e')
    await user.keyboard('s')
    await user.keyboard('s')
    await user.keyboard('{Enter}')

    expect(selectCurrentStreak(store3.getState())).toEqual(2)
  })

  it('should save/load settings', async () => {
    const { app, store: firstStore } = await renderApp()

    firstStore.dispatch(changeSetting('hardMode', true))
    app.unmount()

    // render app again, should load from localStorage
    const { store } = await renderApp()
    expect(selectHardMode(store.getState())).toEqual(true)
  })

  // start/continue game

  it('should open help modal if never played', async () => {
    const { store } = await renderApp()
    jest.advanceTimersByTime(101)
    expect(selectModal(store.getState())).toEqual('help')
  })

  it('should set class on body for dark mode', async () => {
    const { store } = await renderApp()
    store.dispatch(changeSetting('darkMode', true))
    jest.advanceTimersByTime(0)
    expect(document.body.classList).toContain('dark')
    expect(document.body.classList).not.toContain('colorblind')
  })

  it('should set class on body for colorblind mode', async () => {
    const { store } = await renderApp()
    store.dispatch(changeSetting('colorblindMode', true))
    jest.advanceTimersByTime(0)
    expect(document.body.classList).toContain('colorblind')
  })

  describe('gamesSetNativeColorTheme() native bridge function', () => {
    function applyNativeBridge(
      mockGamesSetNativeColorTheme: jest.Mock,
    ) {
      window.NativeBridge = {
        ...applyContract(bridgeCommands),
        gamesSetNativeColorTheme: mockGamesSetNativeColorTheme,
        addEventListener: jest.fn(),
        removeEventListener: jest.fn(),
      }
    }

    let mockGamesSetNativeColorTheme: jest.Mock
    let mockError: jest.SpyInstance
    beforeEach(() => {
      mockError = jest.spyOn(console, 'error').mockImplementation()
    })

    afterEach(() => {
      mockError.mockRestore()
    })

    describe('on resolve', () => {
      beforeEach(() => {
        mockGamesSetNativeColorTheme = jest
          .fn()
          .mockResolvedValue(true)
        applyNativeBridge(mockGamesSetNativeColorTheme)
      })

      it('in hybrid views, calls gamesSetNativeColorTheme with "light" on load, then "dark" on dark selection', async () => {
        window.isHybridWebView = true
        const { store } = await renderApp()

        expect(mockGamesSetNativeColorTheme).toHaveBeenCalledTimes(1)
        expect(mockGamesSetNativeColorTheme).toHaveBeenNthCalledWith(
          1,
          'light',
        )

        store.dispatch(changeSetting('darkMode', true))
        jest.advanceTimersByTime(0)
        expect(mockGamesSetNativeColorTheme).toHaveBeenCalledTimes(2)
        expect(mockGamesSetNativeColorTheme).toHaveBeenNthCalledWith(
          2,
          'dark',
        )

        expect(mockHybridDebugger).not.toHaveBeenCalled()
        expect(mockError).not.toHaveBeenCalled()
      })

      it('in non-hybrid views, does not call gamesSetNativeColorTheme', async () => {
        window.isHybridWebView = false
        const { store } = await renderApp()
        store.dispatch(changeSetting('darkMode', true))
        jest.advanceTimersByTime(0)

        expect(mockGamesSetNativeColorTheme).not.toHaveBeenCalled()
      })
    })

    describe('on reject', () => {
      beforeEach(() => {
        mockGamesSetNativeColorTheme = jest
          .fn()
          .mockRejectedValue(new Error('eeeek'))
        applyNativeBridge(mockGamesSetNativeColorTheme)
      })

      it('calls gamesSetNativeColorTheme with "light", logs error to console and hybrid debugger', async () => {
        window.isHybridWebView = true
        await renderApp()
        expect(mockGamesSetNativeColorTheme).toHaveBeenCalledTimes(1)

        expect(mockHybridDebugger).toHaveBeenCalledTimes(1)
        expect(mockError).toHaveBeenCalledWith(
          'Failed to set native color theme: light',
          new Error('eeeek'),
        )
      })
    })
  })
})
