import React from 'react'

import { isMobileBrowser } from 'shared/responsive'
import { useUser } from 'user'

import { startRefreshTimer } from '../../game-refresher'
import MomentSystem from '../Moment/MomentSystem'
import { useThemeManager } from '../Settings/useThemeManager'
import {
  addToast,
  willClosePage,
  trackAuthImpression,
} from '../../store/overlays/overlays.actions'
import {
  finishLoading,
  loadApp,
  loadPreview,
} from '../../store/api/api.actions'
import LandscapeWarning from '../LandscapeWarning/LandscapeWarning'
import {
  selectIsFirstMoogleSave,
  selectAppLoaded,
  selectDataFetched,
} from '../../store/api/api.selectors'
import {
  selectGameDayOffset,
  selectGameId,
} from '../../store/game/game.selectors'
import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import {
  wordleMomentConfig,
  getInitialMoment,
} from '../Moment/wordleMomentConfig'
import useGameDataHybridEvent from '../../../../shared/hooks/use-game-data-hybrid-event'
import useSaleBannerHybridSubscribe from '../../../../shared/hooks/use-sale-banner-hybrid-subscribe'

import { useAndroidBack } from './useAndroidBack'

export default function App() {
  const isFirstSave = useWordleSelector(selectIsFirstMoogleSave)
  const hasAppLoaded = useWordleSelector(selectAppLoaded)
  const hasDataFetched = useWordleSelector(selectDataFetched)
  const gameDayOffset = useWordleSelector(selectGameDayOffset)
  const gameId = useWordleSelector(selectGameId)
  const { hasLoaded: hasUserLoaded } = useUser()

  const dispatch = useWordleDispatch()

  React.useEffect(() => {
    const timer = startRefreshTimer()
    return () => clearInterval(timer)
  }, [])

  useAndroidBack()

  useSaleBannerHybridSubscribe()

  // triggers API calls to puzzles and state
  React.useEffect(() => {
    if (!hasAppLoaded) {
      // wordle previews use "puzzles" routes and should contain print dates
      const isPreview =
        window.location.pathname.includes('/puzzles/shortz')

      const printDate =
        (isPreview && window.gameData?.wordleShortzDate) || null

      if (printDate) {
        dispatch(loadPreview(printDate))
        return
      }

      dispatch(loadApp())
    }
  }, [hasAppLoaded])

  // once the API calls are complete (including any triggered by UserProvider),
  // finish the loading flow by setting the initial state in redux
  React.useEffect(() => {
    if (hasUserLoaded && hasDataFetched && !hasAppLoaded) {
      dispatch(finishLoading())
    }
  }, [hasDataFetched, hasAppLoaded, hasUserLoaded])

  // show this toast to users when they are first opted in
  React.useEffect(() => {
    const displayToast = hasAppLoaded && isFirstSave
    const impressionRegion = 'wordle-stats-link'
    const impressionName = 'stats-successful'
    const impressionLabel = 'automatic-opt-in'

    if (displayToast) {
      dispatch(
        addToast({
          text: 'Your stats have been linked to your account.',
          duration: 2000,
          isSystem: true,
        }),
      )
      dispatch(willClosePage())
      dispatch(
        trackAuthImpression(
          impressionName,
          impressionRegion,
          impressionLabel,
        ),
      )
    }
  }, [isFirstSave, hasAppLoaded])

  useGameDataHybridEvent({
    analyticsPuzzleID: `${gameDayOffset}`,
    puzzleID: gameId,
    game: 'wordle',
  })

  useThemeManager()

  return (
    <>
      {isMobileBrowser() && <LandscapeWarning />}
      <MomentSystem
        hasLoaded={hasAppLoaded}
        config={wordleMomentConfig}
        initialMoment={getInitialMoment()}
      />
    </>
  )
}
