import React from 'react'
import { Provider } from 'react-redux'
import userEvent from '@testing-library/user-event'
import { render, screen, fireEvent } from '@testing-library/react'

import { useUser } from 'user'
import { abra } from 'foundation'

import createStore from '../../store'
import { openModal } from '../../store/overlays/overlays.actions'

import AppHeader from './AppHeader'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  trackClick: jest.fn(),
  trackModuleInteraction: jest.fn(),
  abra: {
    getVariant: jest.fn(),
  },
}))

jest.mock('user', () => ({
  useUser: jest.fn(() => ({
    userType: { isLoggedIn: true, hasXwd: false },
  })),
}))

describe('<AppHeader />', () => {
  const renderHeader = (isLoading: boolean = false) => {
    const store = createStore()

    const header = render(
      <Provider store={store}>
        <AppHeader isLoading={isLoading} />
      </Provider>,
    )

    return { store, header }
  }

  it('should display top level header', () => {
    renderHeader()

    expect(
      screen.getByRole('heading', {
        name: 'Wordle',
        level: 1,
      }),
    )
  })

  it('should toggle nav button to expanded when clicked', () => {
    renderHeader()
    const navButton = screen.getByLabelText('Menu')
    expect(
      screen.getByRole('button', {
        name: 'Menu',
        expanded: false,
      }),
    )

    fireEvent.click(navButton)

    expect(navButton.getAttribute('aria-haspopup')).toEqual('menu')

    // note some aria attributes that are expected to have state changes can be
    // queried by role and the attribute, below is an example
    expect(
      screen.getByRole('button', {
        name: 'Menu',
        expanded: true,
      }),
    )
  })

  it('should open help modal', () => {
    renderHeader()
    const helpButton = screen.getByLabelText('Help')

    expect(helpButton.getAttribute('aria-haspopup')).toEqual('dialog')
  })

  // GAMES_wordleWebStatsRegiWall_1023 - 0_control
  it('0_control - should open the stats modal when logged in', () => {
    // @ts-ignore
    abra.getVariant.mockReturnValue('0_control')

    renderHeader(true)
    expect(screen.getByLabelText('Statistics')).toBeTruthy()
  })

  // GAMES_wordleWebStatsRegiWall_1023 - 0_control
  it('0_control - should open the stats modal when NOT logged in', () => {
    // @ts-ignore
    abra.getVariant.mockReturnValue('0_control')
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: false, hasXwd: false },
      hasLoaded: true,
    })

    renderHeader(true)
    expect(screen.getByLabelText('Statistics')).toBeTruthy()
  })

  // GAMES_wordleWebStatsRegiWall_1023 - 1_modal
  it('1_modal - should open the regiwall modal when NOT logged in and stats icon is clicked in app header', () => {
    // @ts-ignore
    abra.getVariant.mockReturnValue('1_modal')
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: false, hasXwd: false },
      hasLoaded: true,
    })

    const { store, header } = renderHeader(true)
    store.dispatch(openModal('regiwallModal', true))
    expect(
      screen.findByText('Here to see your stats and streaks?'),
    ).toBeTruthy()
  })

  // GAMES_wordleWebStatsRegiWall_1023 - 2_congrats
  it('2_congrats - should open the regiwall modal when NOT logged in and stats icon is clicked in app header', () => {
    // @ts-ignore
    abra.getVariant.mockReturnValue('2_congrats')
    // @ts-ignore
    useUser.mockReturnValueOnce({
      userType: { isLoggedIn: false, hasXwd: false },
      hasLoaded: true,
    })

    const { store, header } = renderHeader(true)
    store.dispatch(openModal('regiwallModal', true))
    expect(
      screen.findByText('Here to see your stats and streaks?'),
    ).toBeTruthy()
  })

  it('should disable stats modal when loading', () => {
    renderHeader(true)
    const statsButton = screen.getByLabelText('Statistics')

    expect(statsButton).toBeDisabled()
  })

  it('should not release focus if user hits enter while settings button is focused', async () => {
    const user = userEvent.setup({ delay: null })
    renderHeader()
    const statsButton = screen.getByLabelText('Statistics')
    statsButton.focus()
    await user.keyboard('{Enter}')

    expect(statsButton).toHaveFocus()
  })

  it('should release focus if user is focused on button and then types letters', async () => {
    const user = userEvent.setup({ delay: null })
    renderHeader()
    const statsButton = screen.getByLabelText('Statistics')
    statsButton.focus()
    await user.keyboard('t')

    expect(statsButton).not.toHaveFocus()

    // if user then hits enter again focus is still not on stats button
    await user.keyboard('{Enter}')
    expect(statsButton).not.toHaveFocus()
  })

  it('should release focus if user is focused on button and then deletes a letter using backspace', async () => {
    const user = userEvent.setup({ delay: null })
    renderHeader()
    const statsButton = screen.getByLabelText('Statistics')
    statsButton.focus()
    await user.keyboard('t')
    await user.keyboard('Backspace')

    expect(statsButton).not.toHaveFocus()

    // if user then hits enter again focus is still not on stats button
    await user.keyboard('{Enter}')
    expect(statsButton).not.toHaveFocus()
  })

  it('should open settings modal', () => {
    renderHeader()
    const settingsButton = screen.getByLabelText('Settings')
    expect(settingsButton.getAttribute('aria-haspopup')).toEqual(
      'dialog',
    )
  })

  it('should disable settings modal when loading', () => {
    renderHeader(true)
    const settingsButton = screen.getByLabelText('Settings')

    expect(settingsButton).toBeDisabled()
  })

  it('should render Subscribe button on desktop web', () => {
    window.isHybridWebView = false
    window.newsreaderAppPlatform = ''
    window.isMobileDevice = false

    renderHeader()
    const subButton = screen.getByText('Subscribe to Games')
    expect(subButton).toBeTruthy()
  })

  it('should render Subscribe button on mobile web', () => {
    window.isHybridWebView = false
    window.newsreaderAppPlatform = ''
    window.isMobileDevice = true

    renderHeader()
    const subButton = screen.getByText('Subscribe to Games')
    expect(subButton).toBeTruthy()
  })

  it('should not render Subscribe button on games app', () => {
    window.isHybridWebView = true
    window.gamesAppPlatform = 'ios'

    renderHeader()
    const subButton = screen.queryByText('Subscribe to Games')
    expect(subButton).toBeNull()
  })
})
