import React from 'react'
import cn from 'classnames'

import { abra, trackClick } from 'foundation'
import { useUser } from 'user'

import { ALPHABET } from '../../utils/gamelogic'
import {
  openModal,
  openNavModal,
} from '../../store/overlays/overlays.actions'
import {
  selectError,
  selectNavModalOpen,
} from '../../store/overlays/overlays.selectors'
import Icon from '../Icon/Icon'
import NavIcon from '../Icon/NavIcon'
import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import {
  BackButton,
  WebOnly,
  HybridOnly,
} from '../../../../shared/Hybrid'
import { selectGameDayOffset } from '../../store/game/game.selectors'
import { handleSubscribe } from '../../../../helpers/subscribeLinks'
import { selectMoogleUser } from '../../store/api/api.selectors'

import styles from './AppHeader.module.scss'

type AppHeaderProps = {
  isLoading?: boolean
}

export const navMenuAriaControlsId = 'nav-modal'

export default function AppHeader({ isLoading }: AppHeaderProps) {
  const gameDayOffset = `${useWordleSelector(selectGameDayOffset)}`
  const isNavOpen = useWordleSelector(selectNavModalOpen)
  const isLoggedIn = !!useWordleSelector(selectMoogleUser)
  const dispatch = useWordleDispatch()
  const { userType } = useUser()
  const hasError = useWordleSelector(selectError)
  const trackData = {
    context: gameDayOffset || undefined,
    elementName: 'wordle',
    moduleLabel: 'in-game',
  }

  const disableStatsAndSettings = !!hasError || isLoading

  const blurOnLetterInput = (
    target: HTMLButtonElement,
    key: string,
  ) => {
    // when letters are typed on keyboard, release focus on button so gameplay keys work,
    // ie: enter and backspace
    if (ALPHABET.includes(key.toLowerCase()) || key === 'Backspace') {
      target.blur()
    }
  }

  // GAMES_wordleWebStatsRegiWall_1023 experiment
  const regiwallPromptVariant = abra.getVariant(
    'GAMES_wordleWebStatsRegiWall_1023',
  )

  const showRegiwallModal =
    !isLoggedIn &&
    (regiwallPromptVariant === '1_modal' ||
      regiwallPromptVariant === '2_congrats')

  return (
    // eslint-disable-next-line jsx-a11y/no-static-element-interactions
    <header
      className={cn(styles.appHeader, 'wordle-app-header')}
      onKeyDown={(event) => {
        // if user hits enter on any navigation item stop keyboard from submitting word via preventing propagation
        if (event.key === 'Enter') {
          event.stopPropagation()
        }
      }}
    >
      <div className={styles.menuLeft}>
        <WebOnly>
          <button
            type="button"
            id={styles.navButton}
            className={styles.icon}
            aria-label="Menu"
            aria-haspopup="menu"
            aria-controls={navMenuAriaControlsId}
            aria-expanded={isNavOpen}
            onClick={() => dispatch(openNavModal())}
            onKeyDown={(event) => {
              const { key, currentTarget } = event
              blurOnLetterInput(currentTarget, key)
            }}
          >
            <NavIcon />
          </button>
        </WebOnly>
        <HybridOnly>
          <BackButton trackData={trackData} />
        </HybridOnly>
      </div>
      <h1 className={styles.title}>Wordle</h1>

      {/* eslint-disable-next-line jsx-a11y/no-noninteractive-element-interactions */}
      <menu
        className={cn(styles.menuRight, {
          [styles.longTextVariant]: !userType.hasXwd,
        })}
        onKeyDown={(event) => {
          const { key, target } = event
          blurOnLetterInput(target as HTMLButtonElement, key)
        }}
      >
        <button
          type="button"
          id="help-button"
          className={styles.icon}
          aria-label="Help"
          aria-haspopup="dialog"
          aria-controls="help-dialog"
          onClick={() => {
            trackClick({
              name: 'wordle',
              label: 'help',
              context: gameDayOffset,
              useBeacon: true,
            })
            dispatch(openModal('help', true))
          }}
        >
          <Icon icon="help" />
        </button>
        <button
          type="button"
          id="statistics-button"
          className={styles.icon}
          aria-label="Statistics"
          aria-haspopup="dialog"
          aria-controls="stats-dialog"
          onClick={() =>
            dispatch(
              openModal(
                showRegiwallModal ? 'regiwallModal' : 'stats',
                true,
              ),
            )
          }
          disabled={disableStatsAndSettings}
        >
          <Icon
            icon="statistics"
            disabled={disableStatsAndSettings}
          />
        </button>
        <button
          type="button"
          id="settings-button"
          className={styles.icon}
          aria-label="Settings"
          aria-haspopup="dialog"
          aria-controls="settings-dialog"
          onClick={() => {
            trackClick({
              element: { name: 'wordle', label: 'settings' },
              region: 'toolbar',
              context: gameDayOffset,
              useBeacon: true,
            })
            dispatch(openModal('settings'))
          }}
          disabled={disableStatsAndSettings}
        >
          <Icon icon="settings" disabled={disableStatsAndSettings} />
        </button>
        {!userType.hasXwd && (
          <WebOnly>
            <button
              type="button"
              className={styles.subscribeLink}
              aria-label="Subscribe"
              onClick={() => {
                handleSubscribe('wordle')
                trackClick({
                  name: 'wordle',
                  element: {
                    name: 'wordle',
                    label: 'subscribe',
                  },
                  context: gameDayOffset,
                  useBeacon: true,
                })
              }}
            >
              Subscribe to Games
            </button>
          </WebOnly>
        )}
      </menu>
    </header>
  )
}
