import React from 'react'
import { Provider } from 'react-redux'
import { render, screen, fireEvent } from '@testing-library/react'

import createStore from '../../store'
import { selectModal } from '../../store/overlays/overlays.selectors'
import { selectHasPlayed } from '../../store/stats/stats.selectors'
import useFeatureFlag from '../../../../../shared/abra/useFeatureFlag'
import { changeSetting } from '../../store/settings/settings.actions'
import { closeModal } from '../../store/overlays/overlays.actions'

import Game from './Game'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  abra: {
    reportExposure: jest.fn(),
    getVariant: jest.fn(),
  },
  trackModuleInteraction: jest.fn(),
  trackInteraction: jest.fn(),
  trackImpression: jest.fn(),
}))
jest.mock('../Ad/useAd', () => ({
  __esModule: true,
  default: jest.fn(() => ({})),
}))
jest.mock('../../../../../shared/abra/useAbra', () => () => ({
  getVariant: jest.fn(() => 'foo'),
  reportExposure: jest.fn(),
}))

jest.mock('../../store/stats/stats.selectors', () => ({
  ...jest.requireActual('../../store/stats/stats.selectors'),
  selectHasPlayed: jest.fn(() => false),
}))

jest.mock('../../../../../shared/abra/useFeatureFlag', () =>
  jest.fn(),
)

describe('<Game />', () => {
  const renderGame = () => {
    const store = createStore()
    window.navigationLinks = { login: '', subscribe: { default: '' } }

    const game = render(
      <Provider store={store}>
        <Game hasLoaded transition={() => {}} />
      </Provider>,
    )

    return { store, game }
  }

  it('should show board items', () => {
    renderGame()
    const tiles = screen.getAllByTestId('tile')
    expect(tiles.length).toEqual(30)
  })

  it('should not show loading screen', () => {
    renderGame()
    const loading = screen.queryByTestId('loading')
    expect(loading).toBeNull()
  })

  describe('<Modal> and inert attribute', () => {
    it('game content should not be inert by default', () => {
      renderGame()

      // when modal is open game is set to `inert`
      expect(
        screen.queryByTestId('game-wrapper'),
      ).not.toHaveAttribute('inert')
    })

    it('should open nav modal and set inert to outside content', async () => {
      renderGame()
      const navButton = screen.getByLabelText('Statistics')
      await fireEvent.click(navButton)
      expect(screen.queryByTestId('modal-overlay')).toBeTruthy()
      expect(screen.queryByText('Statistics')).toBeTruthy()

      // when modal is open game is set to `inert`
      expect(screen.queryByTestId('game-wrapper')).toHaveAttribute(
        'inert',
      )
    })
  })

  describe('useAutoModal', () => {
    beforeAll(() => jest.useFakeTimers())
    afterAll(() => jest.useRealTimers())
    beforeEach(() => {
      window.isHybridWebView = true
    })
    afterEach(() => {
      window.localStorage.clear()
      jest.clearAllMocks()
    })

    it('should open help modal if never played', () => {
      const { store } = renderGame()
      jest.advanceTimersByTime(101)
      expect(selectModal(store.getState())).toEqual('help')
    })

    it('should open hard mode awareness if hard mode is not enabled', () => {
      // @ts-ignore
      selectHasPlayed.mockImplementation(() => true)
      // @ts-ignore
      useFeatureFlag.mockImplementation(() => true)
      const { store } = renderGame()

      jest.advanceTimersByTime(101)
      expect(selectModal(store.getState())).toEqual(
        'hardModeAwareness',
      )
    })

    it('should not open hard mode awareness if hard mode is enabled', () => {
      // @ts-ignore
      selectHasPlayed.mockImplementation(() => true)
      // @ts-ignore
      useFeatureFlag.mockImplementation(() => true)
      const { store } = renderGame()
      store.dispatch(changeSetting('hardMode', true))

      jest.advanceTimersByTime(101)
      expect(selectModal(store.getState())).toEqual(null)
    })

    it('should not open hard mode awareness if already dismissed', () => {
      // @ts-ignore
      selectHasPlayed.mockImplementation(() => true)
      // @ts-ignore
      useFeatureFlag.mockImplementation(() => true)
      const { store, game } = renderGame()

      jest.advanceTimersByTime(101)
      expect(selectModal(store.getState())).toEqual(
        'hardModeAwareness',
      )
      store.dispatch(closeModal())
      game.unmount()

      const { store: store2 } = renderGame()
      jest.advanceTimersByTime(101)
      expect(selectModal(store2.getState())).toEqual(null)
    })
  })
})

describe('<Game hasLoaded={false} />', () => {
  const renderGame = () => {
    const store = createStore()

    const game = render(
      <Provider store={store}>
        <Game hasLoaded={false} transition={() => {}} />
      </Provider>,
    )

    return { store, game }
  }

  it('should show loading screen', () => {
    renderGame()
    const loading = screen.queryByTestId('loading')
    expect(loading).toBeTruthy
  })

  it('should not show board items', () => {
    renderGame()
    const tiles = screen.queryAllByTestId('tile')
    expect(tiles).toEqual([])
  })
})
