import React, { useEffect } from 'react'

import SkipNav from '../../../../shared/SkipNav'
import Board from '../Board/Board'
import Icon from '../Icon/Icon'
import Loading from '../Loading/Loading'
import Modal from '../Modal/Modal'
import NavModal from '../NavModal/NavModal'
import Page from '../Page/Page'
import ToastContainer from '../Toast/ToastContainer'
import Error from '../Error/Error'
import ErrorBanner from '../Error/ErrorBanner'
import Keyboard from '../Keyboard/Keyboard'
import OfflineTicker from '../../../../shared/OfflineTicker/OfflineTicker'
import { useWordleSelector } from '../../store/types'
import {
  selectIsSavingState,
  selectMoogleError,
} from '../../store/api/api.selectors'
import {
  selectModal,
  selectPage,
} from '../../store/overlays/overlays.selectors'
import Ad from '../Ad'
import useAd from '../Ad/useAd'
import useAbra from '../../../../../shared/abra/useAbra'
import { LINKBACK_TEST } from '../../abra-constants'
import { useAutoModal } from '../Modal/useAutoModal'
import type {
  MomentComponentProps,
  WordleMoments,
} from '../Moment/wordleMomentConfig'

import styles from './App.module.scss'
import AppHeader from './AppHeader'

interface GameProps extends MomentComponentProps<WordleMoments> {}

export default function Game({ hasLoaded, transition }: GameProps) {
  const adProps = useAd()
  const abra = useAbra()
  const gameHTMLId = 'wordle-app-game'
  const { couldHaveAd, hasAd, hasAdError } = adProps
  const isSaving = useWordleSelector(selectIsSavingState)
  const moogleGetError = useWordleSelector(selectMoogleError)
  const linkbackVariant = abra.getVariant(LINKBACK_TEST)

  const modalType = useWordleSelector(selectModal)
  const pageType = useWordleSelector(selectPage)

  // game should be inert aka non-functional when modal or page are overlaid on game
  const isInert = modalType || pageType

  useAutoModal(hasLoaded)

  // Report exposure for Wordle Linkback Test
  // https://jira.nyt.net/browse/SEO-2571
  useEffect(() => {
    if (linkbackVariant) {
      abra.reportExposure(LINKBACK_TEST)
    }
  }, [linkbackVariant])

  const containerStyle =
    couldHaveAd && hasAd
      ? {
          // Ideally the 210px would be Board.height - Keyboard.height
          height: `calc(100% - 210px)`,
        }
      : {
          height: '100%',
        }

  const gameContents = (
    <main className={styles.game} id={gameHTMLId}>
      {isSaving && (
        <OfflineTicker dismissIcon={<Icon icon="close" />} />
      )}
      {moogleGetError && <ErrorBanner />}
      <Board adProps={adProps} />
      <Keyboard />
      <Error />
    </main>
  )

  return (
    <>
      <SkipNav associatedSectionId={gameHTMLId} />
      {hasLoaded && couldHaveAd && !hasAdError && (
        <Ad position="top" id="ad-top" />
      )}
      <Modal />
      <Page adProps={adProps} />
      <ToastContainer />
      <div
        className={styles.gameContainer}
        style={containerStyle}
        // set content outside of modal as `inert` when modal is open -  need to ignore TS and eslint
        // rules as a workaround with issue for inert not being flagged yet for React as a valid HTML attribute
        // see: https://github.com/facebook/react/issues/17157
        // @ts-ignore
        // eslint-disable-next-line react/no-unknown-property
        inert={isInert ? '' : undefined}
        data-testid="game-wrapper"
      >
        <NavModal />
        <AppHeader isLoading={!hasLoaded} />
        {hasLoaded ? gameContents : <Loading />}
      </div>
    </>
  )
}
