import React from 'react'

import { NUM_ROWS, NUM_TILES } from '../../store/game/game.constants'
import {
  lastTileRevealed,
  removeInvalid,
} from '../../store/transient/transient.actions'
import {
  selectLastRowInvalid,
  selectLastRowWin,
} from '../../store/transient/transient.selectors'
import {
  selectBoardState,
  selectEvaluations,
  selectCurrentRowIndex,
} from '../../store/game/game.selectors'
import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import Row from '../Row/Row'
import useWindowSize from '../../../../shared/hooks/use-window-size'

import styles from './Board.module.scss'

const MAX_BOARD_WIDTH_PX = 350

const clamp = (num: number, min: number, max: number): number =>
  Math.min(Math.max(num, min), max)

export default function Board({
  adProps = {},
}: {
  adProps?: {
    couldHaveAd?: boolean
    hasAd?: boolean
    pageScrollable?: boolean
  }
}) {
  const { couldHaveAd, hasAd, pageScrollable } = adProps
  const windowSize = useWindowSize()
  const boardState = useWordleSelector(selectBoardState)
  const evaluations = useWordleSelector(selectEvaluations)
  const currentRowIndex = useWordleSelector(selectCurrentRowIndex)
  const lastRowInvalid = useWordleSelector(selectLastRowInvalid)
  const lastRowWin = useWordleSelector(selectLastRowWin)
  const dispatch = useWordleDispatch()

  const boardContainerRef = React.useRef<HTMLDivElement>(null)
  const boardRef = React.useRef<HTMLDivElement>(null)

  const sizeBoard = () => {
    const boardContainer = boardContainerRef.current
    const board = boardRef.current
    if (!boardContainer || !board) return
    /*
      if we might have an ad we'll prevent the board from shrinking to an unusable height
      since the ad will take up a minimum height of 300px so it's more likely to be unusable
     */
    const MIN_BOARD_WIDTH_PX = couldHaveAd && hasAd ? 300 : 0

    const flooredWidth = Math.floor(
      boardContainer.clientHeight * (NUM_TILES / NUM_ROWS),
    )

    const boardWidth = clamp(
      flooredWidth,
      MIN_BOARD_WIDTH_PX,
      MAX_BOARD_WIDTH_PX,
    )

    const tileWidth = Math.floor(boardWidth / NUM_TILES)
    const boardHeight = tileWidth * NUM_ROWS
    board.style.width = `${boardWidth}px`
    board.style.height = `${boardHeight}px`
    /*
     unset overflow on board container to prevent siblings from flowing on top of element
     */
    boardContainer.style.overflow =
      couldHaveAd && hasAd && boardWidth === MIN_BOARD_WIDTH_PX
        ? 'unset'
        : ''
  }
  React.useEffect(() => {
    sizeBoard()
  }, [hasAd, windowSize, pageScrollable])

  return (
    <div className={styles.boardContainer} ref={boardContainerRef}>
      <div className={styles.board} ref={boardRef}>
        {Array(NUM_ROWS)
          .fill(true)
          .map((_, i) => (
            <Row
              key={i}
              letters={boardState[i] || ''}
              length={NUM_TILES}
              evaluation={evaluations[i]}
              rowIndex={i}
              invalid={i === currentRowIndex && lastRowInvalid}
              win={i === currentRowIndex - 1 && lastRowWin}
              removeInvalid={() => dispatch(removeInvalid())}
              onFlipEnd={() => dispatch(lastTileRevealed(i))}
            />
          ))}
      </div>
    </div>
  )
}
