import React from 'react'
import { Provider } from 'react-redux'
import { render, screen, fireEvent } from '@testing-library/react'

import createStore from '../../store'

import Explainer from './Explainer'

const onClose = jest.fn()
const trackClick = jest.fn()

const closeButtonEl = () => {
  return (
    <button type="button" data-testid="close" onClick={onClose}>
      Close mock
    </button>
  )
}
describe('<Explainer />', () => {
  const renderExplainer = (
    type: 'linkInfo' | 'statsProblem',
    loggedIn?: boolean,
  ) => {
    return render(
      <Provider store={createStore()}>
        <Explainer
          closeButton={() => {
            return closeButtonEl()
          }}
          trackExplainerClick={trackClick()}
          type={type}
          loggedIn={loggedIn}
        />
      </Provider>,
    )
  }
  describe('explainer - linkInfo', () => {
    it('linking stats to account', () => {
      renderExplainer('linkInfo')
      const title = screen.getByText(
        'You can now link your Wordle stats to a free New York Times account.',
      )
      expect(title).toBeTruthy()
    })

    it('calls the onClose function when close button is clicked', () => {
      renderExplainer('linkInfo')
      expect(onClose).not.toHaveBeenCalled()
      const closeButton = screen.getByTestId('close')
      fireEvent.click(closeButton)
      expect(onClose).toHaveBeenCalled()
    })
  })

  describe('explainer - statsProblem', () => {
    it('something is wrong with stats', () => {
      renderExplainer('statsProblem')
      const title = screen.getByText("Something doesn't look right.")
      expect(title).toBeTruthy()
    })

    it('renders logged out message', () => {
      renderExplainer('statsProblem', false)
      const loggedOutMessage = screen.getByText(
        /you may need to log in to your New York Times account/,
      )
      expect(loggedOutMessage).toBeTruthy()
    })

    it('renders opted in message', () => {
      renderExplainer('statsProblem', true)
      const loggedOutMessage = screen.getByText(
        /you may be seeing game data from another user of your account/,
      )
      expect(loggedOutMessage).toBeTruthy()
    })

    it('should display log in button', () => {
      // TODO: update to only show if isLoggedIn is false
      renderExplainer('statsProblem')
      const buttonText = screen.getByText(
        'Log in or create a free account',
      )
      expect(buttonText).toBeTruthy()
    })
  })
})
