import React from 'react'

import { trackClick } from 'foundation'

import { useWordleSelector } from '../../store/types'
import Tile from '../Tile/Tile'
import { Evaluation } from '../../utils/gamelogic'
import LoginCTA from '../LoginLink/LoginCTA'
import { selectGameDayOffset } from '../../store/game/game.selectors'
import { selectWasModalOpenedByClick } from '../../store/overlays/overlays.selectors'
import { selectMoogleUser } from '../../store/api/api.selectors'

import styles from './Help.module.scss'

type HelpWord = {
  helpWord: string
  letters: {
    letter: string
    evaluation?: Evaluation
  }[]
}

type HelpWords = HelpWord[]

export const helpWords: HelpWords = [
  {
    helpWord: 'weary',
    letters: [
      { letter: 'w', evaluation: 'correct' },
      { letter: 'e' },
      { letter: 'a' },
      { letter: 'r' },
      { letter: 'y' },
    ],
  },
  {
    helpWord: 'pills',
    letters: [
      { letter: 'p' },
      { letter: 'i', evaluation: 'present' },
      { letter: 'l' },
      { letter: 'l' },
      { letter: 's' },
    ],
  },
  {
    helpWord: 'vague',
    letters: [
      { letter: 'v' },
      { letter: 'a' },
      { letter: 'g' },
      { letter: 'u', evaluation: 'absent' },
      { letter: 'e' },
    ],
  },
]

const helpTileSection = ({ helpWord, letters }: HelpWord) => (
  <>
    {letters.map(({ letter, evaluation }, index) => (
      <div
        data-testid={`${helpWord}-letter`}
        className={styles.tileContainer}
        // eslint-disable-next-line react/no-array-index-key
        key={`${helpWord}-${letter}-${index}`}
      >
        <Tile
          letterIndex={index}
          letter={letter}
          evaluation={evaluation}
          small
        />
      </div>
    ))}
  </>
)

export default function Help() {
  const id = `${useWordleSelector(selectGameDayOffset)}`
  const isLoggedIn = !!useWordleSelector(selectMoogleUser)
  const isWelcome = !useWordleSelector(selectWasModalOpenedByClick)

  return (
    <>
      <h3 className={styles.subheading}>
        Guess the Wordle in 6 tries.
      </h3>
      <section className={styles.help}>
        <ul className={styles.instructions}>
          <li>Each guess must be a valid 5-letter word.</li>
          <li>
            The color of the tiles will change to show how close your
            guess was to the word.
          </li>
        </ul>
        <div className={styles.examples}>
          <p>
            <strong>Examples</strong>
          </p>
          <div
            aria-label={helpWords[0].helpWord}
            className={styles.example}
          >
            {helpTileSection(helpWords[0])}
            <p>
              <strong>W</strong> is in the word and in the correct
              spot.
            </p>
          </div>
          <div
            aria-label={helpWords[1].helpWord}
            className={styles.example}
          >
            {helpTileSection(helpWords[1])}
            <p>
              <strong>I</strong> is in the word but in the wrong spot.
            </p>
          </div>
          <div
            aria-label={helpWords[2].helpWord}
            className={styles.example}
          >
            {helpTileSection(helpWords[2])}
            <p>
              <strong>U</strong> is not in the word in any spot.
            </p>
          </div>
        </div>
        {!isLoggedIn && (
          <div className={styles.statsLogin}>
            <div className={styles.statsIcon}>
              <img
                alt="Green Bar Graph"
                style={{ content: 'var(--wordle-stats-mini-check)' }}
              />
            </div>
            <div className={styles.loginText}>
              <LoginCTA
                trackEvent={{
                  eventName: isWelcome ? 'log-in-welcome' : 'help',
                  sendContext: true,
                  element: {
                    name: 'wordle',
                    label: 'log-in-welcome',
                  },
                }}
                lireOfferAsset="wordle-how-to-play"
              >
                Log in or create a free NYT account
              </LoginCTA>{' '}
              to link your stats.
            </div>
          </div>
        )}
        <div className={styles.reminderSignUp}>
          <p>
            A new puzzle is released daily at midnight. If you haven’t
            already, you can{' '}
            <a
              href="https://www.nytimes.com/newsletters/signup/NTWO"
              target="_blank"
              rel="noreferrer"
              onClick={() => {
                trackClick({
                  name: 'wordle',
                  label: null,
                  element: {
                    name: 'wordle',
                    label: 'newsletter-sign-up',
                  },
                  context: id,
                })
              }}
            >
              sign up
            </a>{' '}
            for our daily reminder email.
          </p>
        </div>
      </section>
    </>
  )
}
