import React from 'react'
import cn from 'classnames'

import { Evaluation } from '../../utils/gamelogic'
import Icon from '../Icon/Icon'

import styles from './Key.module.scss'

const keyMap = {
  backspace: {
    character: '←',
    text: 'backspace',
    jsEventKeyName: ' ',
  },
  enter: {
    character: '↵',
    text: 'enter',
    jsEventKeyName: 'Enter',
  },
}

const getKeyContent = (value: string) => {
  if (value === keyMap.backspace.character) {
    return <Icon icon="backspace" />
  }

  if (value === keyMap.enter.character) {
    return keyMap.enter.text
  }

  if (value >= 'a' && value <= 'z') {
    return value
  }
}

type KeyProps = {
  value: string
  evaluation?: Evaluation
  isKeyDisabled?: boolean
}

export default function Key({
  value,
  evaluation,
  isKeyDisabled,
}: KeyProps) {
  const [fade, setFade] = React.useState(false)
  const content = getKeyContent(value)
  React.useEffect(() => {
    if (evaluation) {
      setFade(true)
    }
  }, [evaluation])

  if (!content) {
    // spacer
    return (
      <div
        data-testid="spacer"
        className={value.length === 1 ? styles.half : styles.one}
      />
    )
  }

  let ariaLabel = `add ${value}`
  if (value === keyMap.backspace.character) {
    ariaLabel = keyMap.backspace.text
  } else if (value === keyMap.enter.character) {
    ariaLabel = keyMap.enter.text
  } else if (evaluation) {
    ariaLabel = `${value} ${evaluation}`
  }

  return (
    <button
      type="button"
      data-key={value}
      data-state={evaluation}
      aria-label={ariaLabel}
      onKeyDown={(event) => {
        if (
          event.key === keyMap.enter.jsEventKeyName ||
          event.key === keyMap.backspace.jsEventKeyName
        ) {
          // if enter or backspace is pressed on a key via keydown, stop propagation
          // so that global `keydown` handler is not reached and default button behavior is not overriden
          // note that React's `click` handler will still fire so so any `enter` detection for `enter`
          // button will work
          event.stopPropagation()
        }
      }}
      aria-disabled={isKeyDisabled}
      className={cn(styles.key, {
        [styles.oneAndAHalf]:
          value === keyMap.backspace.character ||
          value === keyMap.enter.character,
        [styles.fade]: fade,
      })}
      onTransitionEnd={() => setFade(false)}
    >
      {content}
    </button>
  )
}
