import React from 'react'
import { Provider } from 'react-redux'
import { render, screen, fireEvent } from '@testing-library/react'
import userEvent from '@testing-library/user-event'

import createStore from '../../store'
import { ALPHABET } from '../../utils/gamelogic'
import { selectVirtualKeyboardOnlyMode } from '../../store/settings/settings.selectors'

import Keyboard from './Keyboard'
import keyStyles from './Key.module.scss'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

jest.mock('../../store/game/game.actions', () => ({
  addLetter: jest.fn((letter: string) => ({
    type: 'ADD_LETTER',
    letter,
  })),
  backspace: jest.fn(() => ({ type: 'BACKSPACE' })),
  submitGuess: jest.fn(() => ({ type: 'SUBMIT_GUESS' })),
}))

jest.mock('../../store/settings/settings.selectors', () => ({
  ...jest.requireActual('../../store/settings/settings.selectors'),
  selectVirtualKeyboardOnlyMode: jest.fn(() => false),
}))

jest.mock('../../store/transient/transient.selectors', () => ({
  selectKeyboardLetterEvaluations: jest.fn(() => ({
    b: 'correct',
  })),
}))

describe('<Keyboard />', () => {
  const renderKeyboard = () => {
    const store = createStore()
    store.dispatch = jest.fn()

    render(
      <Provider store={store}>
        <Keyboard />
      </Provider>,
    )

    return { store }
  }

  it('should render two spacers, backspace, enter, and 26 letters', () => {
    renderKeyboard()

    expect(screen.getAllByTestId('spacer')).toHaveLength(2)
    expect(screen.getAllByText('enter')).toHaveLength(1)

    expect(screen.getAllByTestId('icon-backspace')).toHaveLength(1)
    expect(
      screen
        .getAllByTestId('icon-backspace')[0]
        .parentElement?.getAttribute('aria-label'),
    ).toEqual('backspace')

    ALPHABET.toLowerCase()
      .split('')
      .forEach((letter) => {
        expect(screen.getAllByText(letter)).toHaveLength(1)
      })
  })

  it('should add a letter when that letter is clicked', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()

    const bKey = screen.getByText('b')
    await user.click(bKey)

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'ADD_LETTER',
      letter: 'b',
    })
  })

  it('should remove a letter when backspace is clicked', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()

    const backspace = screen.getByTestId('icon-backspace')
    await user.click(backspace)

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'BACKSPACE',
    })
  })

  it('should submit a guess when enter is clicked', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()

    const enter = screen.getByText('enter')
    await user.click(enter)

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'SUBMIT_GUESS',
    })
  })

  it('should add letter when letter is typed on keyboard', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()

    await user.keyboard('b')

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'ADD_LETTER',
      letter: 'b',
    })
  })

  it('should not add letter when letter is typed on keyboard and disabled virtual keyboard input only setting is on', async () => {
    // @ts-ignore
    selectVirtualKeyboardOnlyMode.mockImplementationOnce(() => true)

    const user = userEvent.setup()
    const { store } = renderKeyboard()

    await user.keyboard('b')

    expect(store.dispatch).toHaveBeenCalledTimes(0)
    expect(store.dispatch).not.toHaveBeenCalledWith({
      type: 'ADD_LETTER',
      letter: 'b',
    })
  })

  it('should add letter when letter is typed on keyboard and disabled virtual keyboard input only setting is off', async () => {
    // @ts-ignore
    selectVirtualKeyboardOnlyMode.mockImplementationOnce(() => false)

    const user = userEvent.setup()
    const { store } = renderKeyboard()

    await user.keyboard('b')

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'ADD_LETTER',
      letter: 'b',
    })
  })

  it('should not add number', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()

    await user.keyboard('2')

    expect(store.dispatch).toHaveBeenCalledTimes(0)
  })

  it('should remove letter when backspace is typed on keyboard', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()

    await user.keyboard('{Backspace}')

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'BACKSPACE',
    })
  })

  it('should submit just the letter on key focus and then enter', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()
    const bKey = screen.getByText('b')

    // focus `b` key before hitting enter
    bKey.focus()
    await user.keyboard('{Enter}')

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'ADD_LETTER',
      letter: 'b',
    })
  })

  it('should submit a guess when enter is typed', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()

    await user.keyboard('{Enter}')

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'SUBMIT_GUESS',
    })
  })

  it('should still submit word when enter focused and submitted', async () => {
    const user = userEvent.setup()
    const { store } = renderKeyboard()
    const enterKey = screen.getByText('enter')

    enterKey.focus()

    await user.keyboard('{Enter}')

    expect(store.dispatch).toHaveBeenCalledTimes(1)
    expect(store.dispatch).toHaveBeenCalledWith({
      type: 'SUBMIT_GUESS',
    })
  })

  it('should apply color and aria-labels to evaluated letter', async () => {
    renderKeyboard()
    const aKey = screen.getByText('a')
    const bKey = screen.getByText('b')

    expect(aKey.getAttribute('data-state')).toBeFalsy()
    expect(aKey.getAttribute('aria-label')).toBe('add a')
    expect(bKey.getAttribute('data-state')).toEqual('correct')
    expect(bKey.getAttribute('aria-label')).toEqual('b correct')

    expect(aKey.classList).not.toContain(keyStyles.fade)
    expect(bKey.classList).toContain(keyStyles.fade)

    await fireEvent.transitionEnd(bKey)
    expect(bKey.classList).not.toContain(keyStyles.fade)
  })
})
