import React, { useRef, useEffect, useState, MouseEvent } from 'react'

import { ALPHABET } from '../../utils/gamelogic'
import {
  addLetter,
  backspace,
  submitGuess,
} from '../../store/game/game.actions'
import { selectVirtualKeyboardOnlyMode } from '../../store/settings/settings.selectors'
import { selectDisabledKeys } from '../../store/game/game.selectors'
import { selectKeyboardLetterEvaluations } from '../../store/transient/transient.selectors'
import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'

import Key from './Key'
import styles from './Keyboard.module.scss'

const keyboardRows = [
  ['q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p'],
  ['-', 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', '-'],
  ['↵', 'z', 'x', 'c', 'v', 'b', 'n', 'm', '←'],
]

export default function Keyboard() {
  const keyboardRef = useRef<HTMLDivElement>(null)
  const dispatch = useWordleDispatch()
  const letterEvaluations = useWordleSelector(
    selectKeyboardLetterEvaluations,
  )
  const disabledKeys = useWordleSelector(selectDisabledKeys)

  const virtualKeyboardOnlyMode = useWordleSelector(
    selectVirtualKeyboardOnlyMode,
  )

  const dispatchKeyPressEvent = (key: string) => {
    if (key === '←' || key === 'Backspace') {
      dispatch(backspace())
    } else if (key === '↵' || key === 'Enter') {
      dispatch(submitGuess())
    } else if (ALPHABET.includes(key.toLowerCase())) {
      dispatch(addLetter(key.toLowerCase()))
    }
  }

  const handleClick = (e: MouseEvent<HTMLDivElement>) => {
    const button = (e.target as HTMLElement).closest('button')
    if (!button || !keyboardRef.current) return

    if (!keyboardRef.current.contains(button)) return

    if (button.dataset.key === undefined) return
    dispatchKeyPressEvent(button.dataset.key)
  }

  const handleKeyDown = (event: KeyboardEvent) => {
    if (event.repeat === true) return
    const { key, metaKey, ctrlKey } = event
    // Crude method of ignoring things like cmd + shift + r
    if (metaKey || ctrlKey) return

    if (
      ALPHABET.includes(key.toLowerCase()) ||
      key === 'Backspace' ||
      key === 'Enter'
    ) {
      dispatchKeyPressEvent(key)
    }
  }

  useEffect(() => {
    if (virtualKeyboardOnlyMode) {
      // exit function if virtual keyboard only setting is on,
      // we do not want to add event listener to global keydown
      return
    }

    window.addEventListener('keydown', handleKeyDown)

    // always remove event handler as part of cleanup return function -
    // will be run again when there are changed dependencies as well as on unmount
    return () => window.removeEventListener('keydown', handleKeyDown)
  }, [virtualKeyboardOnlyMode])

  return (
    // eslint-disable-next-line jsx-a11y/no-static-element-interactions, jsx-a11y/click-events-have-key-events, jsx-a11y/no-noninteractive-element-interactions
    <div
      className={styles.keyboard}
      role="group"
      aria-label="Keyboard"
      ref={keyboardRef}
      onClick={handleClick}
      onMouseDown={(event) => {
        // as part of initial `mousedown`s click we need to prevent implicit focus on button
        // this will allow event further up the tree to be handled correctly -
        // but is needed as part of Safari browser bug with click/focus
        // see: https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button#clicking_and_focus
        event.preventDefault()
      }}
    >
      {keyboardRows.map((row, r) => (
        <div className={styles.row} key={r}>
          {row.map((key, k) => (
            <Key
              key={`${key}-${r}-${k}`}
              value={key}
              isKeyDisabled={disabledKeys?.includes(key)}
              evaluation={letterEvaluations[key]}
            />
          ))}
        </div>
      ))}
    </div>
  )
}
