import React, {
  useState,
  AnimationEvent,
  MouseEvent,
  useRef,
} from 'react'
import cn from 'classnames'

import { trackClick, trackInteraction } from 'foundation'

import useToggleElFocus from '../../../../shared/hooks/use-toggle-el-focus'
import useKeyboardTrap from '../../../../shared/hooks/use-keyboard-trap'
import Icon from '../Icon/Icon'
import Help from '../Help/Help'
import Stats from '../Stats/Stats'
import RegiwallPrompt from '../Stats/RegiwallPrompt'
import HardModeAwareness from '../HardModeAwareness/HardModeAwareness'
import ActivationRegiModal from '../ActivationRegiModal/ActivationRegiModal'
import { Settings } from '../Settings/Settings'
import {
  closeModal,
  openModal,
} from '../../store/overlays/overlays.actions'
import {
  selectModal,
  selectWasModalOpenedByClick,
} from '../../store/overlays/overlays.selectors'
import { selectIsSavingState } from '../../store/api/api.selectors'
import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import useFeatureFlag from '../../../../../shared/abra/useFeatureFlag'
import {
  selectGameDayOffset,
  selectGameStatus,
} from '../../store/game/game.selectors'
import { selectHasPlayed } from '../../store/stats/stats.selectors'
import useAbra from '../../../../../shared/abra/useAbra'
import {
  ACTIVATION_REGI_MODAL,
  WORDLE_REGIWALL_TEST,
} from '../../abra-constants'
import type { ModalName } from '../../store/overlays/overlays.state'
import LoginPrompt from '../Login/LoginPrompt'

import styles from './Modal.module.scss'

const modalConfig: Record<
  NonNullable<ModalName>,
  {
    content: JSX.Element
    heading?: string
    onClose?: (context: string) => void
  }
> = {
  help: {
    content: <Help />,
    heading: 'How To Play',
  },
  stats: {
    content: <Stats />,
  },
  regiwallModal: {
    content: <RegiwallPrompt />,
  },
  loginPrompt: {
    content: <LoginPrompt />,
  },
  settings: {
    content: <Settings />,
    heading: 'settings',
  },
  // NSO-2385
  activationRegiModal: {
    content: <ActivationRegiModal />,
    onClose: (context) => {
      trackClick({
        name: 'wordle',
        label: 'close-regi-modal',
        context,
      })
    },
  },
  hardModeAwareness: {
    content: <HardModeAwareness />,
    onClose: (context) => {
      trackClick({
        element: { name: 'wordle', label: 'close' },
        context,
        region: 'tool-tip',
      })
    },
  },
}

export default function Modal() {
  const dispatch = useWordleDispatch()
  const modalType = useWordleSelector(selectModal)
  const [closing, setClosing] = useState(false)
  const isSavingState = useWordleSelector(selectIsSavingState)
  const shortStatsModal = modalType === 'stats' && isSavingState
  const hasPlayed = useWordleSelector(selectHasPlayed)
  const gameStatus = useWordleSelector(selectGameStatus)
  // NSO-2385 - remove min height on mobile to remove extra space on bottom of modal
  const removeMinHeight = modalType === 'activationRegiModal'
  const removePadding = modalType === 'stats'
  const gameId = `${useWordleSelector(selectGameDayOffset)}`
  const isEndScreen = !useWordleSelector(selectWasModalOpenedByClick)
  const isFullScreenStats =
    modalType === 'stats' || modalType === 'regiwallModal'

  // A/B Test
  const abra = useAbra()

  // focus close button when Modal is launched, when modal closes
  // release focus via hook
  const closeRef = useRef<HTMLButtonElement>(null)
  useToggleElFocus(Boolean(modalType), closeRef)

  // when modal is open enable keyboard trapping within element
  const modalRef = useRef<HTMLDialogElement>(null)
  useKeyboardTrap(Boolean(modalType), modalRef)

  /**
   * NSO-2385 - ab expose for control of `ON_wordle_regi_modal`
   * If allocated to control, Activation team's regi modal will not display
   * but an ab expose must still fire for control.
   * To be removed once test concludes.
   */
  const isRegiModalAbraControl = useFeatureFlag(
    'activationRegiModalControl',
  )

  if (
    !modalType &&
    !isSavingState &&
    hasPlayed &&
    isRegiModalAbraControl
  ) {
    abra.reportExposure(ACTIVATION_REGI_MODAL)
  }

  if (!modalType) return null

  const { content, heading, onClose } = modalConfig[modalType]

  const handleClose = () => {
    // GAMES_wordleWebStatsRegiWall_1023 experiment
    const regiwallPromptVariant = abra.getVariant(
      WORDLE_REGIWALL_TEST,
    )

    const showRegiwallStats =
      regiwallPromptVariant === '1_modal' ||
      regiwallPromptVariant === '2_congrats'

    if (showRegiwallStats) {
      let trackRegion = ''
      // lire form should send 'stats-regiwall'
      if (modalType === 'loginPrompt') {
        trackRegion = 'stats-regiwall'
        dispatch(openModal('regiwallModal'))
      } else if (
        modalType === 'regiwallModal' ||
        modalType === 'stats'
      ) {
        // end screen regiwall or end screen stats should send 'congrats'
        if (isEndScreen) trackRegion = 'congrats'
        // regular regiwall or regular stats should send 'stats'
        else trackRegion = 'stats'
      }

      // w-55 interaction: close stats regiwall popup / congrats modal
      trackInteraction({
        element: { name: 'wordle', label: 'close' },
        context: `${gameId}`,
        region: trackRegion,
      })
    }

    if (modalType !== 'loginPrompt') {
      if (onClose) {
        onClose(gameId)
      }
      setClosing(true)
    }
  }

  const closeOnOutsideClick = (e: MouseEvent) => {
    // close modal if user clicks on the modal overlay
    if (e.currentTarget === e.target) {
      handleClose()
    }
  }

  const handleAnimationEnd = (e: AnimationEvent) => {
    if (e.animationName === styles.SlideOut) {
      setClosing(false)
      dispatch(closeModal())
    }
  }

  return (
    // eslint-disable-next-line jsx-a11y/click-events-have-key-events, jsx-a11y/no-noninteractive-element-interactions
    <dialog
      ref={modalRef}
      open
      className={cn(styles.modalOverlay, {
        [styles.paddingTop]: modalType === 'help',
      })}
      data-testid="modal-overlay"
      aria-label={`${modalType} Modal`}
      aria-modal="true"
      id={`${modalType}-dialog`}
      onClick={closeOnOutsideClick}
      onAnimationEnd={handleAnimationEnd}
      onKeyDown={(event) => {
        if (event.key === 'Escape') {
          handleClose()
        }
      }}
    >
      <div
        className={cn(styles.content, styles.testExtraWidth, {
          [styles.closing]: closing,
          [styles.shortStatsModal]: shortStatsModal,
          [styles.noPadding]: removePadding,
          [styles.testNoMinHeight]: removeMinHeight, // NSO-2385
          [styles.flexContainer]: modalType === 'settings',
          [styles.extraPadding]: modalType === 'help',
          [styles.fullscreenStats]:
            modalType === 'stats' ||
            modalType === 'regiwallModal' ||
            modalType === 'loginPrompt',
        })}
      >
        {heading && (
          <h2
            className={cn(styles.heading, {
              [styles.newHeading]: modalType === 'help',
            })}
          >
            {heading}
          </h2>
        )}
        {modalType === 'stats' ||
        modalType === 'regiwallModal' ||
        modalType === 'loginPrompt' ? (
          <div className={cn(styles.fullscreenStatsExit)}>
            <div className={cn(styles.buttonContainer)}>
              <button
                ref={closeRef}
                className={cn(styles.closeIconButton)}
                type="button"
                aria-label="Close"
                onClick={handleClose}
              >
                <Icon icon="close" />
              </button>
            </div>
          </div>
        ) : (
          <button
            ref={closeRef}
            className={cn(styles.closeIcon)}
            type="button"
            aria-label="Close"
            onClick={handleClose}
          >
            <Icon icon="close" />
          </button>
        )}
        {content}
      </div>
    </dialog>
  )
}
