import { useEffect, useRef, useState } from 'react'

import useFeatureFlag from 'shared/abra/useFeatureFlag'

import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import {
  selectError,
  selectModal,
} from '../../store/overlays/overlays.selectors'
import { selectHasPlayed } from '../../store/stats/stats.selectors'
import { selectIsSavingState } from '../../store/api/api.selectors'
import { selectCurrentRowIndex } from '../../store/game/game.selectors'
import { selectHardMode } from '../../store/settings/settings.selectors'
import {
  getRegiModalViewCount,
  FREQ_CAP,
} from '../ActivationRegiModal/ActivationRegiModal'
import {
  isHybridWebView,
  isInNewsreader,
} from '../../../../../shared/utils/deviceIdentifiers'
import useTimeout from '../../../../shared/hooks/use-timeout'
import { openModal } from '../../store/overlays/overlays.actions'
import type { ModalName } from '../../store/overlays/overlays.state'
import { usePersistHide } from '../../../../shared/hooks/use-persist-hide'

// writes to local storage when modal is dismissed, so that it won't come back
const useHardModeAwarenessDismissal = () => {
  const currentModal = useWordleSelector(selectModal)
  const prevModal = useRef<ModalName>(null)
  const { isHidden, hide } = usePersistHide('wordle-hma-dismissed')

  useEffect(() => {
    if (
      prevModal.current === 'hardModeAwareness' &&
      currentModal !== 'hardModeAwareness'
    ) {
      hide()
    }
    prevModal.current = currentModal
  }, [currentModal])

  return isHidden
}

export const useAutoModal = (hasLoaded: boolean) => {
  const dispatch = useWordleDispatch()
  const [hasAutoOpened, setAutoOpened] = useState(false)
  const hasError = useWordleSelector(selectError)
  const hasPlayed = useWordleSelector(selectHasPlayed)
  const isSavingState = useWordleSelector(selectIsSavingState)
  const currentRowIndex = useWordleSelector(selectCurrentRowIndex)
  const hasStartedGame = currentRowIndex > 0
  const isActivationModalAbra = useFeatureFlag('activationRegiModal')
  const isActivationFreqCapped = () => {
    return getRegiModalViewCount() >= FREQ_CAP
  }

  const hardModeEnabled = useWordleSelector(selectHardMode)
  const hardModeDismissed = useHardModeAwarenessDismissal()

  const modal = (() => {
    // never auto-open a modal if the user is mid-puzzle
    if (hasAutoOpened || !hasLoaded || hasError || hasStartedGame) {
      return null
    }

    // opens the Help modal if a user has never played
    if (!hasPlayed) {
      return 'help'
    }

    /**
     * NSO-2385 - Activation Regi Modal test
     * Serves a modal that contains only a login/register button if the following conditions are met:
     * - user is in web browser only
     * - user has not started the game (has not entered a guess yet)
     * - frequency cap not met
     * - user is not logged in
     * - user is allocated to abra variant `1_regi_modal` of `ON_wordle_regi_modal`
     * - does not interfere with Help modal
     */
    if (
      !isInNewsreader() &&
      !isHybridWebView() &&
      !isActivationFreqCapped() &&
      !isSavingState &&
      isActivationModalAbra
    ) {
      return 'activationRegiModal'
    }

    // Open a modal that directs the player to settings if:
    // - hard mode is off
    // - the user has never dismissed this modal
    if (!hardModeEnabled && !hardModeDismissed) {
      return 'hardModeAwareness'
    }

    return null
  })()

  useTimeout(
    () => {
      dispatch(openModal(modal))
      setAutoOpened(true)
    },
    modal ? 100 : null,
  )
}
