import React, {
  useState,
  AnimationEvent,
  useRef,
  useEffect,
} from 'react'
import cn from 'classnames'

import { trackClick } from 'foundation'

import useToggleElFocus from '../../../../shared/hooks/use-toggle-el-focus'
import useKeyboardTrap from '../../../../shared/hooks/use-keyboard-trap'
import Explainer from '../Explainer/Explainer'
import {
  closePage,
  trackAuthClick,
} from '../../store/overlays/overlays.actions'
import {
  selectPage,
  selectWillClosePage,
} from '../../store/overlays/overlays.selectors'
import {
  selectMoogleError,
  selectMoogleUser,
} from '../../store/api/api.selectors'
import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import { PageName } from '../../store/overlays/overlays.state'
import Icon from '../Icon/Icon'
import { selectGameDayOffset } from '../../store/game/game.selectors'

import styles from './Page.module.scss'

export default function Page({
  adProps = {},
}: {
  adProps?: { couldHaveAd?: boolean; hasAd?: boolean }
}) {
  const { couldHaveAd, hasAd } = adProps
  const dispatch = useWordleDispatch()
  const pageType = useWordleSelector(selectPage)
  const [closing, setClosing] = useState(false)
  const loggedIn = !!useWordleSelector(selectMoogleUser)
  const willClosePage = useWordleSelector(selectWillClosePage)
  const mooglePostError = useWordleSelector(selectMoogleError)
  const gameId = `${useWordleSelector(selectGameDayOffset)}`

  useEffect(() => {
    if (willClosePage) {
      setClosing(true)
    }
  }, [willClosePage])

  // focus close button when Modal is launched, when modal closes
  // release focus via hook
  const pageCloseButton = useRef<HTMLButtonElement>(null)
  useToggleElFocus(Boolean(pageType), pageCloseButton)

  // when modal is open enable keyboard trapping within element
  const modalRef = useRef<HTMLDialogElement>(null)
  useKeyboardTrap(Boolean(pageType), modalRef)

  if (!pageType) return null

  const closeButtonEl = (
    pageTypeClosed?: PageName | undefined,
  ): JSX.Element => {
    return (
      <button
        ref={pageCloseButton}
        type="button"
        aria-label="Close"
        className={styles.close}
        data-testid="close"
        onClick={() => {
          setClosing(true)

          if (pageTypeClosed === 'statsProblem' && mooglePostError) {
            dispatch(
              trackAuthClick('', false, {
                label: 'stats-link-error',
                name: 'wordle',
              }),
            )
          }
        }}
      >
        <Icon icon="close" />
      </button>
    )
  }

  const pageOptions = {
    linkInfo: {
      content: (
        <Explainer
          closeButton={() => {
            return closeButtonEl('linkInfo')
          }}
          type="linkInfo"
          loggedIn={loggedIn}
          trackExplainerClick={() =>
            dispatch(trackAuthClick('more-info', false))
          }
        />
      ),
    },
    statsProblem: {
      content: (
        <Explainer
          closeButton={() => {
            return closeButtonEl('statsProblem')
          }}
          loggedIn={loggedIn}
          type="statsProblem"
          trackExplainerClick={(label: string) =>
            trackClick({
              name: '',
              label: null,
              element: {
                name: 'wordle',
                label,
              },
              context: gameId,
            })
          }
        />
      ),
    },
  }

  const handleAnimationEnd = (e: AnimationEvent) => {
    if (e.animationName !== styles.SlideOut) return
    setClosing(false)
    dispatch(closePage())
  }

  const { content } = pageOptions[pageType]

  const noDarkModePages = ['statsLink']
  const noDarkMode = noDarkModePages.includes(pageType)

  return (
    // eslint-disable-next-line jsx-a11y/click-events-have-key-events, jsx-a11y/no-noninteractive-element-interactions
    <dialog
      ref={modalRef}
      open
      data-testid="pageNew"
      className={cn(styles.pageNew, {
        [styles.pageFixed]: couldHaveAd && hasAd,
        [styles.closing]: closing,
        [styles.noDarkMode]: noDarkMode,
      })}
      onAnimationEnd={handleAnimationEnd}
      onKeyDown={(event) => {
        if (event.key === 'Escape') {
          setClosing(true)
        }
      }}
    >
      <div className={styles.content}>
        <div className={styles.contentContainer}>{content}</div>
      </div>
    </dialog>
  )
}
