import React from 'react'
import { Provider } from 'react-redux'
import {
  act,
  fireEvent,
  render,
  screen,
  waitFor,
} from '@testing-library/react'

import { Evaluation } from '../../utils/gamelogic'
import createStore from '../../store'
import { CONTINUE_GAME } from '../../store/game/game.actions'

import Row, { RowProps } from './Row'
import rowStyles from './Row.module.scss'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

describe('<Row />', () => {
  const removeInvalid = jest.fn()
  const defaultProps: RowProps = {
    letters: '',
    evaluation: null,
    rowIndex: 1,
    length: 5,
    invalid: false,
    win: false,
    removeInvalid,
    onFlipEnd: () => {},
  }

  const renderRow = (
    props: Partial<RowProps> = {},
    isRestoringSession = false,
  ) => {
    const store = createStore()
    if (isRestoringSession) {
      store.dispatch({
        type: CONTINUE_GAME,
        payload: { currentRowIndex: 1 },
      })
    }

    return render(
      <Provider store={store}>
        <Row {...defaultProps} {...props} />
      </Provider>,
    )
  }

  beforeAll(() => jest.useFakeTimers())
  afterAll(() => jest.useRealTimers())
  afterEach(jest.clearAllMocks)

  it('should render unrevealed tiles by default', () => {
    act(() => {
      renderRow()
      const tiles = screen.getAllByTestId('tile')
      tiles.forEach((tile) => {
        expect(tile.getAttribute('data-animation')).toEqual('idle')
      })
    })
  })

  it('should reveal tiles quickly when restoring session', async () => {
    await act(async () => {
      renderRow(
        {
          evaluation: [
            'correct',
            'absent',
            'present',
            'correct',
            'correct',
          ],
        },
        true,
      )
      const tiles = screen.getAllByTestId('tile')
      await waitFor(() => {
        tiles.forEach((tile) => {
          expect(tile.getAttribute('data-animation')).toEqual('idle')
        })
      })

      jest.advanceTimersByTime(501)

      await waitFor(() => {
        tiles.forEach((tile) => {
          expect(tile.getAttribute('data-animation')).toEqual(
            'flip-in',
          )
        })
      })
    })
  })

  it('should reveal tiles slowly on update', async () => {
    await act(async () => {
      const evaluation: Evaluation[] = [
        'correct',
        'absent',
        'present',
        'correct',
        'correct',
      ]
      renderRow({ evaluation })
      const tiles = screen.getAllByTestId('tile')
      await waitFor(() => {
        tiles.forEach((tile) => {
          expect(tile.getAttribute('data-animation')).toEqual('idle')
        })
      })
      jest.advanceTimersByTime(501)
      await waitFor(() => {
        tiles.forEach((tile, i) => {
          const expectedAnim = i > 1 ? 'idle' : 'flip-in'
          expect(tile.getAttribute('data-animation')).toEqual(
            expectedAnim,
          )
        })
      })
      jest.advanceTimersByTime(1500)
      await waitFor(() => {
        tiles.forEach((tile) => {
          expect(tile.getAttribute('data-animation')).toEqual(
            'flip-in',
          )
        })
      })
    })
  })

  it('should remove invalid status when animation ends', () => {
    const { container } = renderRow()
    if (!container.firstElementChild) throw Error('empty container')
    fireEvent.animationEnd(container.firstElementChild, {
      animationName: rowStyles.Shake,
    })
    expect(removeInvalid).toHaveBeenCalledTimes(1)
  })

  it('should include the win animation if just won', () => {
    const { container } = renderRow({ win: true })
    const winTiles = container.firstElementChild?.querySelectorAll(
      `.${rowStyles.win}`,
    )
    expect(winTiles).toHaveLength(defaultProps.length)
  })
})
