import React from 'react'
import { fireEvent, render, screen } from '@testing-library/react'
import { Provider } from 'react-redux'

import * as overlayActions from '../../store/overlays/overlays.actions'
import createStore from '../../store'

import { Settings } from './Settings'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  SEND_EMAIL_TYPE: {
    CUSTOM: 'custom',
  },
}))

jest.mock('../../store/settings/settings.selectors', () => ({
  ...jest.requireActual('../../store/settings/settings.selectors'),
  selectHardMode: () => false,
  selectVirtualKeyboardOnlyMode: () => false,
}))

jest.mock('../../store/game/game.selectors', () => ({
  ...jest.requireActual('../../store/game/game.selectors'),
  selectGameDayOffset: () => 50,
  selectGameStatus: () => 'IN_PROGRESS',
  selectCurrentRowIndex: jest.fn(() => 2),
}))

jest.mock('../../store/overlays/overlays.actions', () => ({
  addToast: jest.fn(() => ({
    type: 'wordle/overlays/ADD_TOAST',
    payload: {},
  })),
}))

describe('Settings', () => {
  const renderSettings = () => {
    const store = createStore()

    render(
      <Provider store={store}>
        <Settings />
      </Provider>,
    )

    return store
  }

  it('renders four switches - one for each mode', () => {
    renderSettings()

    expect(screen.getByText('Hard Mode')).toBeTruthy()
    expect(screen.getByText('Dark Theme')).toBeTruthy()
    expect(screen.getByText('High Contrast Mode')).toBeTruthy()
    expect(
      screen.getByText('Onscreen Keyboard Input Only'),
    ).toBeTruthy()
  })

  it('renders the expected feedback link', () => {
    renderSettings()
    const feedbackHref = screen
      .getByTitle('Send feedback to nytgames@nytimes.com')
      .getAttribute('href')

    expect(
      feedbackHref?.includes('subject=Wordle%20Feedback'),
    ).toBeTruthy()
    expect(
      feedbackHref?.includes('mailto:nytgames@nytimes.com'),
    ).toBeTruthy()
  })

  it('renders the expected bug report link for web users', () => {
    renderSettings()
    const bugReportHref = screen
      .getByTitle('Report a bug to nytgames@nytimes.com')
      .getAttribute('href')

    expect(
      bugReportHref?.includes('subject=Wordle%20Bug%20Report'),
    ).toBeTruthy()
    expect(
      bugReportHref?.includes('mailto:nytgames@nytimes.com'),
    ).toBeTruthy()
  })

  it('renders the expected bug report link for newsreader users', () => {
    window.newsreaderAppPlatform = 'ios'

    renderSettings()
    const bugReportHref = screen
      .getByTitle('Report a bug to nytgames@nytimes.com')
      .getAttribute('href')

    expect(
      bugReportHref?.includes(
        'subject=News%20App%3A%20Wordle%20Bug%20Report',
      ),
    ).toBeTruthy()
    expect(
      bugReportHref?.includes('mailto:nytgames@nytimes.com'),
    ).toBeTruthy()

    window.newsreaderAppPlatform = ''
  })

  it('renders the expected bug report link for hybrid users on xwd 4.53 and above', () => {
    // @ts-ignore
    window.NativeBridge = {}
    // @ts-ignore
    window.isHybridWebView = true
    // @ts-ignore
    window.gamesAppPlatform = 'ios'
    // @ts-ignore
    window.gamesAppVersion = '4.53'

    renderSettings()

    const bugReport = screen.queryByTitle(
      'Report a bug to nytgames@nytimes.com',
    )

    expect(bugReport).toBeTruthy()
  })

  it('does not render the bug report link for hybrid users below xwd 4.53', () => {
    // @ts-ignore
    window.NativeBridge = {}
    // @ts-ignore
    window.isHybridWebView = true
    // @ts-ignore
    window.gamesAppPlatform = 'ios'
    // @ts-ignore
    window.gamesAppVersion = '4.49'

    renderSettings()

    const bugReport = screen.queryByTitle(
      'Report a bug to nytgames@nytimes.com',
    )

    expect(bugReport).toBe(null)
  })

  it('renders the expected wordle review comumunity link', () => {
    renderSettings()
    const feedbackHref = screen
      .getByText('Wordle Review')
      .getAttribute('href')

    expect(feedbackHref).toBeTruthy()
    expect(feedbackHref).toEqual(
      'https://www.nytimes.com/spotlight/wordle-review',
    )
  })

  it('renders the expected puzzle number', () => {
    renderSettings()

    // See dayOffset selector mock
    expect(screen.getByText('#50')).toBeTruthy()
  })

  it('dispatches a toast if hardMode is clicked while disabled', () => {
    renderSettings()
    const toastSpy = jest.spyOn(overlayActions, 'addToast')
    const hardModeToggle = screen.getByRole('switch', {
      name: 'Hard Mode',
    })

    fireEvent.click(hardModeToggle)

    expect(toastSpy).toHaveBeenCalledTimes(1)
  })
})
