/* eslint-disable jsx-a11y/no-static-element-interactions,  */
/* eslint-disable jsx-a11y/click-events-have-key-events */
/* eslint-disable jsx-a11y/no-noninteractive-element-interactions */
import React from 'react'

import { SEND_EMAIL_TYPE, trackClick, env } from 'foundation'

import { meetsMinimumVersion } from '../../../../../shared/foundation/js/hybrid/detection'
import { changeSetting } from '../../store/settings/settings.actions'
import {
  selectCurrentRowIndex,
  selectGameDayOffset,
  selectGameStatus,
} from '../../store/game/game.selectors'
import {
  selectColorblindMode,
  selectDarkMode,
  selectVirtualKeyboardOnlyMode,
  selectHardMode,
} from '../../store/settings/settings.selectors'
import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import FeedbackLink from '../FeedbackLink/FeedbackLink'

import { Switch } from './Switch'
import styles from './Settings.module.scss'

export function getBugReportSubject() {
  if (window.newsreaderAppPlatform) {
    return 'News App: Wordle Bug Report'
  }

  if (window.gamesAppPlatform === 'ios') {
    return 'iOS Wordle Bug Report'
  }

  if (window.gamesAppPlatform === 'android') {
    return 'Android Wordle Bug Report'
  }

  return 'Wordle Bug Report'
}

/**
 * TODO: This check can be deleted once we reach full adoption of xwd 4.53
 */
export function shouldShowReportBugLink() {
  // Show for web and non hybrid webviews
  if (!window.isHybridWebView) return true

  // Show only if the user is on a hybrid webview above 4.53 of xwd app
  return meetsMinimumVersion(window.gamesAppVersion, '4.53')
}

export const Settings = () => {
  const dispatch = useWordleDispatch()
  const gameDayOffset = useWordleSelector(selectGameDayOffset)
  const gameStatus = useWordleSelector(selectGameStatus)
  const rowIndex = useWordleSelector(selectCurrentRowIndex)
  const colorblindMode = useWordleSelector(selectColorblindMode)
  const darkMode = useWordleSelector(selectDarkMode)
  const hardMode = useWordleSelector(selectHardMode)
  const hardModeDisabled =
    !hardMode && gameStatus === 'IN_PROGRESS' && rowIndex !== 0

  const virtualKeyboardOnlyMode = useWordleSelector(
    selectVirtualKeyboardOnlyMode,
  )

  return (
    <>
      <div className={styles.container}>
        <section>
          <div className={styles.setting}>
            <div className={styles.text}>
              <h3 className={styles.title}>Hard Mode</h3>
              <p className={styles.description}>
                Any revealed hints must be used in subsequent guesses
              </p>
            </div>
            <Switch
              disabled={hardModeDisabled}
              name="Hard Mode"
              handleClick={() => {
                if (!hardModeDisabled) {
                  dispatch(changeSetting('hardMode', !hardMode))
                }
              }}
              selected={hardMode}
              showAlert={hardModeDisabled}
            />
          </div>
          <div className={styles.setting}>
            <div className={styles.text}>
              <h3 className={styles.title}>Dark Theme</h3>
            </div>
            <Switch
              name="Dark Mode"
              handleClick={() =>
                dispatch(changeSetting('darkMode', !darkMode))
              }
              selected={darkMode}
            />
          </div>
          <div className={styles.setting}>
            <div className={styles.text}>
              <h3 className={styles.title}>High Contrast Mode</h3>
              <p className={styles.description}>
                Contrast and colorblindness improvements
              </p>
            </div>
            <Switch
              name="High Contrast Mode"
              handleClick={() =>
                dispatch(
                  changeSetting('colorblindMode', !colorblindMode),
                )
              }
              selected={colorblindMode}
            />
          </div>
          <div className={styles.setting}>
            <div className={styles.text}>
              <h3 className={styles.title}>
                Onscreen Keyboard Input Only
              </h3>
              <p className={styles.description}>
                Ignore key input except from the onscreen keyboard.
                Most helpful for users using speech recognition or
                other assistive devices.
              </p>
            </div>
            <Switch
              name="Virtual Keyboard Input Only"
              handleClick={() =>
                dispatch(
                  changeSetting(
                    'virtualKeyboardOnlyMode',
                    !virtualKeyboardOnlyMode,
                  ),
                )
              }
              selected={Boolean(virtualKeyboardOnlyMode)}
            />
          </div>
        </section>
        <section>
          <div className={styles.setting}>
            <div className={styles.text}>
              <h3 className={styles.title}>Feedback</h3>
            </div>
            <FeedbackLink styles={styles.feedbackLink}>
              Email
            </FeedbackLink>
          </div>
          {shouldShowReportBugLink() && (
            <div className={styles.setting}>
              <div className={styles.text}>
                <h3 className={styles.title}>Report a Bug</h3>
              </div>
              <FeedbackLink
                styles={styles.feedbackLink}
                description="Report a bug to nytgames@nytimes.com"
                subject={getBugReportSubject()}
                type={SEND_EMAIL_TYPE.CUSTOM}
                tracking={() =>
                  trackClick({
                    name: 'wordle',
                    label: 'report-bug',
                    context: `${gameDayOffset}`,
                  })
                }
              >
                Email
              </FeedbackLink>
            </div>
          )}
          <div className={styles.setting}>
            <div className={styles.text}>
              <h3 className={styles.title}>Community</h3>
            </div>
            {/* eslint-disable react/jsx-no-target-blank */}
            <a
              href="https://www.nytimes.com/spotlight/wordle-review"
              target="_blank"
            >
              Wordle Review
            </a>
          </div>
          <div className={styles.setting}>
            <div className={styles.text}>
              <h3 className={styles.title}>Questions?</h3>
            </div>
            {/* eslint-disable react/jsx-no-target-blank */}
            <a
              href="https://help.nytimes.com/hc/en-us/articles/360029050872-Word-Games-and-Logic-Puzzles#h_01FVGCB2Z00ZQMDMCYWBPWJNXB"
              target="_blank"
            >
              FAQ
            </a>
          </div>
        </section>
      </div>
      <div className={styles.footnote}>
        <p id="copyright">
          &#169;&nbsp;{new Date().getFullYear()}&nbsp;The New York
          Times Company
        </p>
        <p aria-label="game number">#{gameDayOffset}</p>
      </div>
    </>
  )
}
