import React from 'react'
import { fireEvent, render, screen } from '@testing-library/react'
import { Provider } from 'react-redux'

import createStore from '../../store'

import styles from './Switch.module.scss'
import { Switch, SwitchProps } from './Switch'

describe('Switch', () => {
  const renderSwitch = (customProps?: Partial<SwitchProps>) => {
    const defaultProps = {
      name: 'test',
      handleClick: jest.fn(),
      selected: false,
      ...customProps,
    }

    const store = createStore()

    render(
      <Provider store={store}>
        <Switch {...defaultProps} />
      </Provider>,
    )
  }

  it('should display one switch element', () => {
    renderSwitch()

    expect(screen.getByRole('switch')).toBeTruthy()
  })

  it('should style the switch properly', () => {
    renderSwitch()

    expect(screen.getByRole('switch').classList).toContain(
      styles.switch,
    )
  })

  it('should apply the correct styling and aria labels for a checked button', () => {
    renderSwitch({ selected: true })
    const wrapper = document.querySelector(`.${styles.checked}`)
    const btn = screen.getByRole('switch')

    expect(wrapper).toBeTruthy()
    expect(btn.getAttribute('aria-checked')).toBe('true')
  })

  it('should apply disabled styling for a disabled button', () => {
    renderSwitch({ disabled: true })
    const btn = screen.getByRole('switch')

    expect(btn).toHaveClass('disabled')
  })

  it('should apply disabled aria label for a disabled button', () => {
    renderSwitch({ disabled: true })
    const btn = screen.getByRole('switch')

    expect(btn.getAttribute('aria-disabled')).toBe('true')
  })

  it('onClick should should not execute for a disabled button', () => {
    const handleClick = jest.fn()
    renderSwitch({ disabled: true, handleClick })
    const btn = screen.getByRole('switch')
    fireEvent.click(btn)

    expect(handleClick.mock.calls).toHaveLength(0)
  })
})
