import React from 'react'
import cn from 'classnames'

import { addToast } from '../../store/overlays/overlays.actions'
import { useWordleDispatch } from '../../store/types'

import styles from './Switch.module.scss'

export type SwitchProps = {
  disabled?: boolean
  name: string
  handleClick: () => void
  selected: boolean
  showAlert?: boolean
}

export const Switch = ({
  disabled,
  name,
  handleClick,
  selected,
  showAlert,
}: SwitchProps) => {
  const dispatch = useWordleDispatch()

  const handleDisabledEvent = (event: React.MouseEvent) => {
    event?.preventDefault()
    if (showAlert) {
      dispatch(
        addToast({
          text: 'Hard mode can only be enabled at the start of a round',
          duration: 1500,
          isSystem: true,
        }),
      )
    }
  }

  // Instead of using the disabled property on the button,
  // we are using CSS to achieve a disabled style and conditionally
  // handle handleClick events to display a toast when disabled. In some instances
  // like this one, a "disabled" property on a button means tab navigation will
  // skip it all together, whereas aria-disabled=true will allow focus and interaction
  // to trigger the toast. Not every button needs this implementation, but
  // when a disabled button still needs to be perceived by assistive technology
  // users, this implementation is helpful
  // See https://medium.com/@zakwarsamee/make-disabled-buttons-more-accessible-on-react-c0d8330b232
  return (
    <div
      id={name}
      className={cn(styles.container, {
        [styles.checked]: selected,
      })}
    >
      <button
        aria-disabled={disabled}
        aria-checked={selected}
        aria-label={name}
        role="switch"
        type="button"
        className={cn(styles.switch, { [styles.disabled]: disabled })}
        onClick={disabled ? handleDisabledEvent : handleClick}
      >
        <span className={styles.knob} />
      </button>
    </div>
  )
}
