import React from 'react'
import cn from 'classnames'

import type { WordleStatus } from '../../store/game/game.state'
import type { WordleStatsState } from '../../store/stats/stats.state'
import { useWordleDispatch, useWordleSelector } from '../../store/types'
import { openPage } from '../../store/overlays/overlays.actions'
import { selectWasModalOpenedByClick } from '../../store/overlays/overlays.selectors'
import { ordinal } from '../../utils/gamelogic'

import styles from './Stats.module.scss'

const MIN_BAR_PERC = 7

type GuessRowProps = {
  rowGuess: number
  percentGuesses: number
  numGuesses: number
  addHighlight: boolean
  index: number
}

const guessRow = ({
  rowGuess,
  percentGuesses,
  numGuesses,
  addHighlight,
  index,
}: GuessRowProps) => {
  const ordinalRowGuess = ordinal(rowGuess)
  return (
    <div
      className={styles.graphContainer}
      key={index}
      role="img" // for screenreaders, because this graph is a visual representation
      aria-label={`${numGuesses} solved in ${ordinalRowGuess} attempt`}
    >
      <div className={styles.guess} aria-hidden="true">
        {rowGuess}
      </div>
      <div className={styles.graph} aria-hidden="true">
        <div
          style={{ width: `${percentGuesses}%` }}
          className={cn(styles.graphBar, {
            [styles.alignRight]: numGuesses > 0,
            [styles.highlight]: addHighlight,
          })}
        >
          <div className={styles.numGuesses}>{numGuesses}</div>
        </div>
      </div>
    </div>
  )
}

type GuessDistributionProps = {
  guesses: WordleStatsState['guesses']
  rowIndex: number
  gameStatus: WordleStatus
  loggedIn: boolean
  showRegiwall?: boolean // GAMES_wordleWebStatsRegiWall_1023 experiment
}

export default function GuessDistribution({
  guesses,
  rowIndex,
  gameStatus,
  loggedIn,
  showRegiwall,
}: GuessDistributionProps) {
  const guessRows = []
  const maxGuesses = Math.max(...Object.values(guesses))
  const noData = Object.values(guesses).every((guess) => guess === 0)
  const dispatch = useWordleDispatch()
  const isEndScreen = !useWordleSelector(selectWasModalOpenedByClick)

  if (noData) {
    return (
      <>
        <div className={styles.statsBtnLeft}>
          <h3 className={styles.guessDistributionCopy}>
            Guess Distribution
          </h3>
          <div className={styles.noData}>No Data</div>
        </div>
      </>
    )
  }

  for (let i = 1; i < Object.keys(guesses).length; i += 1) {
    const guess = i
    const numGuesses = guesses[i]
    const percentGuesses = Math.max(
      MIN_BAR_PERC,
      Math.round((numGuesses / maxGuesses) * 100),
    )
    const addHighlight = gameStatus === 'WIN' && i === rowIndex
    const guessData = {
      rowGuess: guess,
      percentGuesses,
      numGuesses,
      addHighlight,
      index: i,
    }
    const row = guessRow(guessData)
    guessRows.push(row)
  }

  return (
    <>
      <div className={styles.statsBtnLeft}>
        <h2 className={styles.guessDistributionCopy}>
          Guess Distribution
        </h2>
      </div>
      <div className={styles.guessDistribution}>{guessRows}</div>
      {loggedIn && !(showRegiwall && isEndScreen) && (
        <button
          type="button"
          className={styles.statsInfoBtn}
          onClick={() => {
            dispatch(openPage('statsProblem'))
          }}
        >
          My stats don&apos;t look right ›
        </button>
      )}
    </>
  )
}
