import React from 'react'
import { Provider } from 'react-redux'
import { render, screen } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import { WordleGameData } from '@nyt/games-schema'

import createStore from '../../store'
import { MOOGLE_GET_SUCCESS } from '../../store/api/api.actions'
import {
  getMoogleLocalStorageKey,
  writeToLocalStorage,
} from '../../store/moogle/moogle.helpers'

import { StatsSelectionTool } from './StatsSelectionTool'
import { mockGamesStateResponse, mockAnonLocalStorage } from './mocks'
import { useStatsSelectionTool } from './useStatsSelectionTool'

jest.mock('../../../../../shared/foundation', () => ({
  env: {},
  xhr: { get: jest.fn() },
  trackImpression: jest.fn(),
  trackInteraction: jest.fn(),
}))

describe('Stats Selection Tool', () => {
  const transitionMock = jest.fn()

  const renderSSSST = ({ noRemoteState = false } = {}) => {
    const store = createStore()

    if (!noRemoteState) {
      store.dispatch({
        type: MOOGLE_GET_SUCCESS,
        payload: {
          data: mockGamesStateResponse,
        },
      })
    }

    const component = render(
      <Provider store={store}>
        <StatsSelectionTool hasLoaded transition={transitionMock} />
      </Provider>,
    )

    return { component, store }
  }

  beforeEach(() => {
    writeToLocalStorage(
      getMoogleLocalStorageKey(),
      mockAnonLocalStorage,
    )

    jest.clearAllMocks()
  })

  it('should render null if no API state', () => {
    const { component } = renderSSSST({ noRemoteState: true })
    expect(component.container).toBeEmptyDOMElement()
    expect(transitionMock).toHaveBeenCalledWith('game')
  })

  it('should render null if no anon local storage', () => {
    window.localStorage.clear()
    const { component } = renderSSSST()
    expect(component.container).toBeEmptyDOMElement()
    expect(transitionMock).toHaveBeenCalledWith('game')
  })

  it('should render tool ', () => {
    const { component } = renderSSSST()
    expect(component.container).not.toBeEmptyDOMElement()
    expect(transitionMock).not.toHaveBeenCalled()
  })

  it('should have disabled button at first', () => {
    renderSSSST()
    const button = screen.getByText('Save Stats')
    expect(button).toBeDisabled()
  })

  it('should submit with account state after clicking first option', () => {
    const { store } = renderSSSST()
    const firstOption = screen.getByText('Saved to your account')
    firstOption.click()
    const button = screen.getByText(
      'Save Stats & Play Todayʼs Puzzle',
    )
    button.click()
    expect(transitionMock).toHaveBeenCalledWith('game')
    expect(store.getState().persist.stats.gamesPlayed).toEqual(317)
    expect(store.getState().persist.stats.hasMadeStatsChoice).toEqual(
      true,
    )
  })

  it('should submit with device state after clicking second option', () => {
    const { store } = renderSSSST()
    const secondOption = screen.getByText('Found on this device')
    secondOption.click()
    const button = screen.getByText(
      'Save Stats & Play Todayʼs Puzzle',
    )
    button.click()
    expect(transitionMock).toHaveBeenCalledWith('game')
    expect(store.getState().persist.stats.gamesPlayed).toEqual(1)
    expect(store.getState().persist.stats.hasMadeStatsChoice).toEqual(
      true,
    )
  })

  it('should not be submittable upon only tabbing to the form', async () => {
    const user = userEvent.setup({ delay: null })
    await renderSSSST()

    await user.tab() // focus form
    await user.keyboard('{Enter}') // submit

    expect(transitionMock).not.toHaveBeenCalled()
  })

  it('should submit first option with keyboard', async () => {
    const user = userEvent.setup({ delay: null })
    const { store } = await renderSSSST()

    await userEvent.tab() // focus form
    await user.keyboard(' ') // space will choose the first option
    await user.keyboard('{Enter}') // submit

    expect(transitionMock).toHaveBeenCalled()
    expect(store.getState().persist.stats.gamesPlayed).toEqual(317)
  })

  it('should submit second option with keyboard', async () => {
    const user = userEvent.setup({ delay: null })
    const { store } = await renderSSSST()

    await userEvent.tab() // focus form
    await user.keyboard('{ArrowDown}') // down arrow will choose the second option
    await user.keyboard('{Enter}') // submit

    expect(transitionMock).toHaveBeenCalled()
    expect(store.getState().persist.stats.gamesPlayed).toEqual(1)
  })

  it('should submit second option with keyboard, tabbing to submit button', async () => {
    const user = userEvent.setup({ delay: null })
    const { store } = await renderSSSST()

    await userEvent.tab() // focus form
    await user.keyboard('{ArrowDown}') // down arrow will choose the second option
    await userEvent.tab() // focus button
    await user.keyboard('{Enter}') // submit

    expect(transitionMock).toHaveBeenCalled()
    expect(store.getState().persist.stats.gamesPlayed).toEqual(1)
  })
})

describe('useStatsSelectionTool', () => {
  beforeEach(() => {
    writeToLocalStorage(
      getMoogleLocalStorageKey(),
      mockAnonLocalStorage,
    )

    jest.clearAllMocks()
  })

  const handleShow = jest.fn()

  const MockComponent = ({ hasLoaded }: { hasLoaded: boolean }) => {
    const show = useStatsSelectionTool(hasLoaded)
    handleShow(show)
    return null
  }

  const renderWithHook = ({ hasMadeStatsChoice = false } = {}) => {
    const store = createStore()

    store.dispatch({
      type: MOOGLE_GET_SUCCESS,
      payload: {
        data: {
          ...mockGamesStateResponse,
          game_data: {
            ...mockGamesStateResponse.game_data,
            stats: {
              ...(mockGamesStateResponse.game_data as WordleGameData)
                .stats,
              hasMadeStatsChoice,
            },
          },
        },
      },
    })

    const component = render(
      <Provider store={store}>
        <MockComponent hasLoaded />
      </Provider>,
    )

    return { component, store }
  }

  it('should show if auto opted in on another device', () => {
    renderWithHook()
    expect(handleShow).toHaveBeenCalledWith(true)
  })

  it('should not show if already made choice', () => {
    renderWithHook({ hasMadeStatsChoice: true })
    expect(handleShow).not.toHaveBeenCalledWith(true)
  })

  it('should not show if local stats have 0 games played', () => {
    const localState = {
      ...mockAnonLocalStorage,
      stats: {
        ...mockAnonLocalStorage.stats,
        gamesPlayed: 0,
      },
    }

    writeToLocalStorage(getMoogleLocalStorageKey(), localState)

    renderWithHook()
    expect(handleShow).not.toHaveBeenCalledWith(true)
  })
})
