import React from 'react'
import { WordleGameData } from '@nyt/games-schema'

import { trackImpression, trackInteraction } from 'foundation'

import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import { getLocalInitialState } from '../../store/moogle/moogle.helpers'
import {
  selectApiDayOffset,
  selectMoogleGameData,
  selectMoogleTimestamp,
} from '../../store/api/api.selectors'
import type {
  MomentComponentProps,
  WordleMoments,
} from '../Moment/wordleMomentConfig'
import { BackButton, HybridOnly } from '../../../../shared/Hybrid'
import { makeStatsSelection } from '../../store/api/api.actions'

import styles from './StatsSelectionTool.module.scss'
import { StatsSelectionOption } from './StatsSelectionOption'

interface StatsSelectionToolProps
  extends MomentComponentProps<WordleMoments> {}

export const StatsSelectionTool = ({
  transition,
}: StatsSelectionToolProps) => {
  const dispatch = useWordleDispatch()
  const todaysDayOffset = useWordleSelector(selectApiDayOffset)
  const accountState = useWordleSelector(selectMoogleGameData)
  const accountTimestamp = useWordleSelector(selectMoogleTimestamp)

  const anonState = getLocalInitialState()
  const anonTimestamp = anonState?.timestamp || 0

  const [selectedOption, setSelectedOption] = React.useState<
    'device' | 'account' | null
  >(null)

  const getTodayStatus = (
    state: WordleGameData,
  ): 'unplayed' | 'in progress' | 'completed' => {
    if (
      state.game.dayOffset !== todaysDayOffset ||
      state.game.currentRowIndex === 0
    ) {
      return 'unplayed'
    }

    if (state.game.status === 'IN_PROGRESS') {
      return 'in progress'
    }

    return 'completed'
  }

  const getSelectedState = (): WordleGameData | null => {
    if (selectedOption === 'account') {
      return accountState as WordleGameData
    }

    if (selectedOption === 'device') {
      return anonState
    }

    return null
  }

  const getButtonText = () => {
    const selectedState = getSelectedState()

    if (
      selectedState &&
      getTodayStatus(selectedState) !== 'completed'
    ) {
      return 'Save Stats & Play Todayʼs Puzzle'
    }

    return 'Save Stats'
  }

  const handleSubmit = (e: React.FormEvent) => {
    e.preventDefault()
    const selectedState = getSelectedState()
    if (selectedState) {
      trackInteraction({
        context: `${todaysDayOffset}`,
        element: { name: 'wordle', label: 'save-stats' },
        region: 'stats-self-select',
        label: selectedOption || '',
      })
      dispatch(makeStatsSelection(selectedState))
      transition('game')
    }
  }

  React.useEffect(() => {
    trackImpression(
      'stats-self-select',
      'wordle',
      null,
      `${todaysDayOffset}`,
    )
  }, [])

  React.useEffect(() => {
    // fail-safe just in case we're rendering null. this really should never happen
    if (!accountState || !anonState) {
      transition('game')
    }
  }, [accountState, anonState])

  if (!accountState || !anonState) {
    return null
  }

  const hybridBackTrackData = {
    context: `${todaysDayOffset}`,
    elementName: 'wordle',
    moduleLabel: 'stats-selection', // TODO: do we want an event for this?
  }

  return (
    <div className={styles.container}>
      <HybridOnly>
        <div className={styles.back}>
          <BackButton trackData={hybridBackTrackData} />
        </div>
      </HybridOnly>
      <div className={styles.header}>
        <h2 className="visually-hidden">Games</h2>
        <img
          className={styles.gamesIcon}
          style={{ content: `${'var(--gameslogo)'}` }}
          alt="New York Times Games Logo"
        />
      </div>
      <form className={styles.form} onSubmit={handleSubmit} id="">
        <fieldset>
          <legend className={styles.description}>
            Your stats on this device donʼt match the stats saved to
            your account.{' '}
            <span className={styles.bold}>Select one</span> to save
            permanently.
          </legend>
          <StatsSelectionOption
            option="account"
            state={accountState}
            label="Saved to your account"
            timestamp={accountTimestamp || 0}
            todayStatus={getTodayStatus(accountState)}
            selectedOption={selectedOption}
            setOption={setSelectedOption}
          />
          <StatsSelectionOption
            option="device"
            state={anonState}
            label="Found on this device"
            timestamp={anonTimestamp}
            todayStatus={getTodayStatus(anonState)}
            selectedOption={selectedOption}
            setOption={setSelectedOption}
          />
        </fieldset>
        <div className={styles.buttonContainer}>
          <button
            type="submit"
            className={styles.submitButton}
            disabled={!selectedOption}
          >
            {getButtonText()}
          </button>
        </div>
      </form>
    </div>
  )
}
