import React from 'react'
import { Provider } from 'react-redux'
import {
  act,
  fireEvent,
  render,
  waitFor,
} from '@testing-library/react'

import createStore from '../../store'

import Tile, { TileProps } from './Tile'
import tileStyles from './Tile.module.scss'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

describe('<Tile />', () => {
  beforeAll(() => jest.useFakeTimers())
  afterAll(() => jest.useRealTimers())

  const renderTile = (props: Partial<TileProps> = {}) => {
    const store = createStore()
    store.dispatch = jest.fn()

    const tile = render(
      <Provider store={store}>
        <Tile letterIndex={0} letter="" {...props} />
      </Provider>,
    )

    return { store, tile }
  }

  it('should render empty tile by default', () => {
    act(() => {
      const { tile } = renderTile()
      const tileDiv = tile.container.firstElementChild
      expect(tileDiv?.textContent).toEqual('')
      expect(tileDiv?.getAttribute('data-state')).toEqual('empty')
      expect(tileDiv?.getAttribute('data-animation')).toEqual('idle')
      expect(tileDiv?.getAttribute('aria-label')).toEqual(
        '1st letter, empty',
      )
    })
  })

  it('should pop tile when letter is typed', async () => {
    await act(async () => {
      const { tile, store } = renderTile()
      tile.rerender(
        <Provider store={store}>
          <Tile letterIndex={0} letter="a" />
        </Provider>,
      )
      const tileDiv = tile.container.firstElementChild
      if (!tileDiv) throw Error('tile div not found')
      await waitFor(() => {
        expect(tileDiv.getAttribute('data-animation')).toEqual('pop')
      })

      setTimeout(() => {}, 1)
      fireEvent.animationEnd(tileDiv, {
        animationName: tileStyles.PopIn,
      })

      await waitFor(() => {
        expect(tileDiv.getAttribute('data-animation')).toEqual('idle')
        expect(tileDiv?.getAttribute('aria-label')).toEqual(
          '1st letter, A',
        )
      })
    })
  })

  it('should animate flip on reveal', async () => {
    await act(async () => {
      const { tile, store } = renderTile({
        letter: 'a',
        evaluation: 'correct',
        flipDelay: 100,
      })
      jest.advanceTimersByTime(101)
      const tileDiv = tile.container.firstElementChild
      if (!tileDiv) throw Error('tile div not found')

      await waitFor(() => {
        expect(tileDiv.getAttribute('data-state')).toEqual('tbd')
        expect(tileDiv?.getAttribute('aria-label')).toEqual(
          '1st letter, A',
        )
        expect(tileDiv.getAttribute('data-animation')).toEqual(
          'flip-in',
        )
      })

      fireEvent.animationEnd(tileDiv, {
        animationName: tileStyles.FlipIn,
      })

      await waitFor(() => {
        expect(tileDiv.getAttribute('data-state')).toEqual('correct')
        expect(tileDiv?.getAttribute('aria-label')).toEqual(
          '1st letter, A, correct',
        )
        expect(tileDiv.getAttribute('data-animation')).toEqual(
          'flip-out',
        )
      })

      fireEvent.animationEnd(tileDiv, {
        animationName: tileStyles.FlipOut,
      })

      await waitFor(() => {
        expect(tileDiv.getAttribute('data-state')).toEqual('correct')
        expect(tileDiv?.getAttribute('aria-label')).toEqual(
          '1st letter, A, correct',
        )
        expect(tileDiv.getAttribute('data-animation')).toEqual('idle')
        expect(store.dispatch).not.toHaveBeenCalled()
      })
    })
  })

  it('should animate flip on reveal with correct aria description', async () => {
    await act(async () => {
      const { tile, store } = renderTile({
        letter: 'c',
        evaluation: 'present',
        flipDelay: 100,
      })
      jest.advanceTimersByTime(101)
      const tileDiv = tile.container.firstElementChild
      if (!tileDiv) throw Error('tile div not found')

      await waitFor(() => {
        expect(tileDiv.getAttribute('data-state')).toEqual('tbd')
        expect(tileDiv?.getAttribute('aria-label')).toEqual(
          '1st letter, C',
        )
        expect(tileDiv.getAttribute('data-animation')).toEqual(
          'flip-in',
        )
      })

      fireEvent.animationEnd(tileDiv, {
        animationName: tileStyles.FlipIn,
      })

      await waitFor(() => {
        expect(tileDiv.getAttribute('data-state')).toEqual('present')
        // present state has custom description
        expect(tileDiv?.getAttribute('aria-label')).toEqual(
          '1st letter, C, present in another position',
        )
        expect(tileDiv.getAttribute('data-animation')).toEqual(
          'flip-out',
        )
      })

      fireEvent.animationEnd(tileDiv, {
        animationName: tileStyles.FlipOut,
      })

      await waitFor(() => {
        expect(tileDiv.getAttribute('data-state')).toEqual('present')
        expect(tileDiv?.getAttribute('aria-label')).toEqual(
          '1st letter, C, present in another position',
        )
        expect(tileDiv.getAttribute('data-animation')).toEqual('idle')
        expect(store.dispatch).not.toHaveBeenCalled()
      })
    })
  })

  it('should call onFlipEnd if provided', async () => {
    await act(async () => {
      const onFlipEnd = jest.fn()
      const { tile } = renderTile({
        letter: 'a',
        onFlipEnd,
      })
      const tileDiv = tile.container.firstElementChild
      if (!tileDiv) throw Error('tile div not found')
      fireEvent.animationEnd(tileDiv, {
        animationName: tileStyles.FlipOut,
      })
      await waitFor(() => {
        expect(onFlipEnd).toHaveBeenCalledTimes(1)
      })
    })
  })
})
