import React from 'react'
import cn from 'classnames'

import { Evaluation, ordinal } from '../../utils/gamelogic'
import useTimeout from '../../../../shared/hooks/use-timeout'

import styles from './Tile.module.scss'

export type TileProps = {
  letter: string
  evaluation?: Evaluation | null
  flipDelay?: number
  letterIndex: number
  onFlipEnd?: () => void
  small?: boolean
  isAriaLiveActive?: boolean
}

type TileStateDescription = {
  [K in Evaluation | 'empty' | 'tbd']: string
}

const tilePlacementDescriptions: TileStateDescription = {
  unknown: 'unknown',
  absent: 'absent',
  present: 'present in another position',
  correct: 'correct',
  empty: 'empty',
  tbd: 'tbd',
} as const

const getTilePlacementDescription = (
  tilePlacement: TileStateDescription[keyof typeof tilePlacementDescriptions],
  letter: string,
  letterIndex: number,
  hasBeenEvaluated: boolean,
) => {
  const letterPlacementInRow = ordinal(letterIndex + 1)
  const letterUpper = letter?.toUpperCase()

  if (hasBeenEvaluated) {
    // when tiles have been revealed and their state evalulated, include in description
    return `${letterPlacementInRow} letter, ${letterUpper}, ${tilePlacement}`
  }

  if (letterUpper) {
    return `${letterPlacementInRow} letter, ${letterUpper}`
  }

  return `${letterPlacementInRow} letter, empty`
}

export default function Tile({
  letter,
  letterIndex,
  evaluation,
  flipDelay = 0,
  onFlipEnd,
  small = false,
  isAriaLiveActive = false,
}: TileProps) {
  const [animation, setAnimation] = React.useState('idle')
  const [revealed, setRevealed] = React.useState(false)
  const mounted = React.useRef(false)

  const handleAnimationEnd = (e: React.AnimationEvent) => {
    if (e.animationName === styles.PopIn && animation === 'pop') {
      setAnimation('idle')
    }

    if (e.animationName === styles.FlipIn) {
      setRevealed(true)
      setAnimation('flip-out')
    }

    if (e.animationName === styles.FlipOut) {
      setAnimation('idle')
      if (onFlipEnd) {
        onFlipEnd()
      }
    }
  }

  React.useEffect(() => {
    // don't pop on initial render
    if (mounted.current && letter) {
      // quick scaling animation when a letter is entered
      setAnimation('pop')
    }
    mounted.current = true
  }, [letter])

  // flip the tile if it has a yet-to-be-revealed evaluation
  const shouldFlip =
    !!evaluation && !revealed && animation !== 'flip-in'
  useTimeout(
    () => setAnimation('flip-in'),
    shouldFlip ? flipDelay : null,
  )

  // various states tiles can be during gameplay
  let dataState: keyof TileStateDescription = 'empty'
  if (revealed && evaluation) {
    dataState = evaluation
  } else if (letter) {
    dataState = 'tbd'
  }

  // get tile aria labels to display as user plays game
  const ariaLabel = getTilePlacementDescription(
    tilePlacementDescriptions[dataState],
    letter,
    letterIndex,
    revealed && Boolean(evaluation),
  )

  return (
    <div
      className={cn(styles.tile, { [styles.small]: small })}
      onAnimationEnd={handleAnimationEnd}
      role="img"
      aria-roledescription="tile"
      aria-label={ariaLabel}
      data-state={dataState}
      data-animation={animation || 'idle'}
      data-testid="tile"
      aria-live={isAriaLiveActive ? 'polite' : 'off'}
    >
      {letter}
    </div>
  )
}
