import React from 'react'
import { Provider } from 'react-redux'
import {
  fireEvent,
  render,
  screen,
  act,
} from '@testing-library/react'

import createStore from '../../store'
import { addToast } from '../../store/overlays/overlays.actions'

import toastStyles from './Toast.module.scss'
import ToastContainer from './ToastContainer'

jest.mock('foundation', () => ({
  env: { version: 'test' },
}))

describe('<ToastContainer />', () => {
  const renderToastContainer = () => {
    const store = createStore()
    render(
      <Provider store={store}>
        <ToastContainer />
      </Provider>,
    )
    return store
  }

  beforeAll(() => {
    jest.useFakeTimers()
  })

  afterAll(() => {
    jest.useRealTimers()
  })

  it('should display a toast', () => {
    const store = renderToastContainer()
    store.dispatch(addToast({ text: 'hello world' }))
    expect(screen.getByText('hello world')).toBeTruthy()
  })

  it('should fade a toast after specified time', () => {
    const store = renderToastContainer()
    store.dispatch(addToast({ text: 'hello world', duration: 200 }))
    const toast = screen.getByText('hello world')
    expect(toast.classList).not.toContain(toastStyles.fade)
    act(() => {
      jest.advanceTimersByTime(201)
    })
    expect(toast.classList).toContain(toastStyles.fade)
  })

  it('should remove a toast after fading', async () => {
    const store = renderToastContainer()
    store.dispatch(addToast({ text: 'hello world' }))
    const toast = screen.getByText('hello world')
    await fireEvent.transitionEnd(toast)
    expect(screen.queryByText('hello world')).toBeFalsy()
  })

  it('should never fade an Infinite toast', () => {
    const store = renderToastContainer()
    store.dispatch(
      addToast({ text: 'hello world', duration: Infinity }),
    )
    act(() => {
      jest.advanceTimersByTime(100000000)
    })
    const toast = screen.getByText('hello world')
    expect(toast.classList).not.toContain(toastStyles.fade)
  })
})
