import React from 'react'
import { Provider } from 'react-redux'
import { fireEvent, render, screen } from '@testing-library/react'

import { trackImpression, trackClick } from 'foundation'

import createStore from '../../store'
import {
  selectCurrentRowIndex,
  selectGameStatus,
} from '../../store/game/game.selectors'
import {
  selectMoogleTimestamp,
  selectMoogleUser,
} from '../../store/api/api.selectors'
import * as useAbra from '../../../../../shared/abra/useAbra'

import { Welcome, largeCTAWindowHeight } from './Welcome'

jest.mock('foundation', () => ({
  env: { version: 'test' },
  trackImpression: jest.fn(),
  trackClick: jest.fn(),
  abra: {
    getVariant: jest.fn(),
  },
}))

jest.mock('../../store/game/game.selectors', () => ({
  ...jest.requireActual('../../store/game/game.selectors'),
  selectCurrentRowIndex: jest.fn(() => 0),
  selectGameStatus: jest.fn(() => 'IN_PROGRESS'),
}))

jest.mock('../../store/api/api.selectors', () => ({
  ...jest.requireActual('../../store/api/api.selectors'),
  selectMoogleTimestamp: jest.fn(() => false),
  selectMoogleUser: jest.fn(() => false),
}))

jest.mock('../../../../../shared/responsive/hybrid', () => ({
  setHybridWindowProperties: () => {},
  setHybridBodyClasses: () => {},
}))

jest.mock('responsive', () => ({
  isMobile: true,
}))

const transition = jest.fn()

describe('Welcome', () => {
  const renderWelcome = () => {
    const store = createStore()

    render(
      <Provider store={store}>
        <Welcome transition={transition} hasLoaded />
      </Provider>,
    )
    return { store }
  }

  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('renders Welcome text and actions', () => {
    renderWelcome()
    const play = screen.getByText('Play', { selector: 'button' })
    const howToPlay = screen.queryByText('How to play', {
      selector: 'button',
    })

    const login = screen.queryByText('Log in', {
      selector: 'a',
    })

    const description = screen.getByText('5-letter')
    expect(play).toBeTruthy()
    expect(howToPlay).toBeTruthy()
    expect(login).toBeTruthy()
    expect(description).toBeTruthy()
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome-modal',
      'wordle-welcome',
      'log-in',
      'null',
    )
  })

  it('hides How To Play button if the user has moogle stats', () => {
    // @ts-ignore
    selectMoogleTimestamp.mockImplementation(() => true)
    renderWelcome()
    const play = screen.getByText('Play', { selector: 'button' })
    const howToPlay = screen.queryByText('How to play', {
      selector: 'button',
    })
    const description = screen.getByText('5-letter')
    expect(play).toBeTruthy()
    expect(howToPlay).toBeFalsy()
    expect(description).toBeTruthy()
  })

  it('hides login button if the user is logged in', () => {
    // @ts-ignore
    selectMoogleUser.mockImplementation(() => true)
    renderWelcome()
    const play = screen.getByText('Play', { selector: 'button' })
    const login = screen.queryByText('Log in', {
      selector: 'button',
    })
    const description = screen.getByText('5-letter')
    expect(play).toBeTruthy()
    expect(login).toBeFalsy()
    expect(description).toBeTruthy()
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome-modal',
      'wordle-welcome',
      '0',
      'null',
    )
  })

  it('fires actions when how to play is clicked', () => {
    // @ts-ignore
    selectMoogleTimestamp.mockImplementation(() => false)
    renderWelcome()
    const howToPlay = screen.getByText('How to play', {
      selector: 'button',
    })
    const description = screen.getByText('5-letter')
    expect(howToPlay).toBeTruthy()
    expect(description).toBeTruthy()

    fireEvent.click(howToPlay)
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome-modal',
      'wordle-welcome',
      '0',
      'null',
    )
    expect(transition).toHaveBeenCalled()
    expect(trackClick).toHaveBeenCalledWith({
      context: 'null',
      label: 'help-welcome',
      name: 'wordle',
    })
  })

  it('fires actions when how to play is clicked for moogle opted in user', () => {
    // @ts-ignore
    selectMoogleTimestamp.mockImplementation(() => false)
    renderWelcome()
    const howToPlay = screen.getByText('How to play', {
      selector: 'button',
    })
    const description = screen.getByText('5-letter')
    expect(howToPlay).toBeTruthy()
    expect(description).toBeTruthy()

    fireEvent.click(howToPlay)
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome-modal',
      'wordle-welcome',
      '0',
      'null',
    )
  })

  it('fires actions when play is clicked', () => {
    renderWelcome()
    const play = screen.getByText('Play', {
      selector: 'button',
    })
    const description = screen.getByText('5-letter')
    expect(play).toBeTruthy()
    expect(description).toBeTruthy()

    fireEvent.click(play)
    expect(trackClick).toHaveBeenCalledWith({
      context: 'null',
      label: 'play',
      name: 'wordle',
    })
  })

  it('renders Welcome Back text and actions', () => {
    // @ts-ignore
    selectCurrentRowIndex.mockImplementation(() => 1)
    renderWelcome()
    expect(screen.getByText('Welcome Back')).toBeTruthy()
    const description = screen.getByText(
      'Youʼre on attempt 2 out of 6. Keep it up!',
    )

    const continueButton = screen.getByText('Continue', {
      selector: 'button',
    })
    expect(continueButton).toBeTruthy()
    expect(description).toBeTruthy()

    fireEvent.click(continueButton)
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome-back-modal',
      'wordle-welcome',
      '0',
      'null',
    )
    expect(trackClick).toHaveBeenCalledWith({
      context: 'null',
      label: 'continue',
      name: 'wordle',
    })
  })

  it('renders Completed Game text and actions', () => {
    // @ts-ignore
    selectGameStatus.mockImplementation(() => 'WIN')

    renderWelcome()
    const seeStatsButton = screen.getByText('See Stats', {
      selector: 'button',
    })
    expect(seeStatsButton).toBeTruthy()

    fireEvent.click(seeStatsButton)
    expect(trackImpression).toHaveBeenCalledWith(
      'welcome-see-stats-modal',
      'wordle-welcome',
      '0',
      'null',
    )
    expect(trackClick).toHaveBeenCalledWith({
      context: 'null',
      label: 'see-stats',
      name: 'wordle',
    })
  })

  it('hides note when appropriate', () => {
    renderWelcome()
    const play = screen.queryByText(
      'See your stats wherever you play.',
    )
    expect(play).toBeNull()
  })

  it('hides back button when not hybrid', () => {
    renderWelcome()

    const backButton = screen.queryByTestId('back-button')
    expect(backButton).toBeNull()
  })

  it('shows back button when in hybrid & native bridge exists', () => {
    window.isHybridWebView = true
    // @ts-ignore
    window.NativeBridge = true
    renderWelcome()

    const backButton = screen.queryByTestId('back-button')
    expect(backButton).toBeTruthy()
  })

  it('shows note when appropriate', () => {
    window.isHybridWebView = true
    // @ts-ignore
    selectCurrentRowIndex.mockImplementation(() => 0)
    // @ts-ignore
    selectMoogleTimestamp.mockImplementation(() => false)

    renderWelcome()
    const play = screen.getByText('See your stats wherever you play.')
  })

  it('shows app banner on mobile web', () => {
    window.isHybridWebView = false
    renderWelcome()
    expect(
      screen.queryByText('Play Wordle', { exact: false }),
    ).toBeTruthy()
  })

  it('hides app banner on hybrid', () => {
    window.isHybridWebView = true
    renderWelcome()
    expect(
      screen.queryByText('Play Wordle', { exact: false }),
    ).toBeFalsy()
  })

  it('hides app banner on newsreader: andriod', () => {
    window.newsreaderAppPlatform = 'android'
    renderWelcome()
    expect(
      screen.queryByText('Play Wordle', { exact: false }),
    ).toBeFalsy()
  })

  it('hides app banner on newsreader: ios', () => {
    window.newsreaderAppPlatform = 'ios'
    renderWelcome()
    expect(
      screen.queryByText('Play Wordle', { exact: false }),
    ).toBeFalsy()
  })

  it('renders large app download banner variant on tall phones', () => {
    const testVariant = '1_iconButton'
    // @ts-ignore
    window.innerHeight = largeCTAWindowHeight + 1
    window.newsreaderAppPlatform = ''
    window.isHybridWebView = false

    // @ts-ignore
    jest.spyOn(useAbra, 'default').mockImplementationOnce(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
    }))
    renderWelcome()
    expect(screen.getByTestId(testVariant)).toBeTruthy()
  })

  it('renders default download banner on short phones', () => {
    const testVariant = '1_iconButton'
    // @ts-ignore
    window.innerHeight = largeCTAWindowHeight - 1
    window.newsreaderAppPlatform = ''
    window.isHybridWebView = false

    // @ts-ignore
    jest.spyOn(useAbra, 'default').mockImplementationOnce(() => ({
      getVariant: () => testVariant,
      reportExposure: jest.fn(),
    }))
    renderWelcome()
    expect(screen.queryByTestId(testVariant)).toBeNull()
  })
})
