import React from 'react'
import cn from 'classnames'

import {
  trackClick,
  trackImpression,
  trackModuleInteraction,
} from 'foundation'
import { isMobile } from 'responsive'

import {
  useWordleDispatch,
  useWordleSelector,
} from '../../store/types'
import { openModal } from '../../store/overlays/overlays.actions'
import {
  selectGameDayOffset,
  selectCurrentRowIndex,
} from '../../store/game/game.selectors'
import {
  selectMoogleUser,
  selectMoogleTimestamp,
  selectEditor,
} from '../../store/api/api.selectors'
import { BackButton, HybridOnly } from '../../../../shared/Hybrid'
import NonDismissalBanner from '../../../../shared/Banner/NonDismissalBanner'
import LargeCTABanner from '../../../../shared/Banner/LargeCTABanner'
import { getAppLink } from '../../../../shared/Banner/getAppLink'
import useAbra from '../../../../../shared/abra/useAbra'
import useWindowSize from '../../../../shared/hooks/use-window-size'
import type {
  MomentComponentProps,
  WordleMoments,
} from '../Moment/wordleMomentConfig'
import { useShowAdInterstitial } from '../AdInterstitial/adMoment'
import { useStatsSelectionTool } from '../StatsSelectionTool/useStatsSelectionTool'

import {
  IconAndTitle,
  Subtitle,
  MetadataDisplay,
  ButtonContainer,
  useWelcomeSubtitle,
  useWelcomeTitle,
  useWelcomePrimaryCTA,
  useWelcomeImpressionData,
  useWelcomeInteractionLabel,
} from './WelcomeHelpers'
import styles from './Welcome.module.scss'
import Note from './Note'

interface WelcomeProps extends MomentComponentProps<WordleMoments> {}

export const largeCTAWindowHeight = 667

export const Welcome = ({ transition, hasLoaded }: WelcomeProps) => {
  const dispatch = useWordleDispatch()
  const currentRowIndex = useWordleSelector(selectCurrentRowIndex)
  const loggedIn = !!useWordleSelector(selectMoogleUser)
  const dayOffset = useWordleSelector(selectGameDayOffset)
  const editor = useWordleSelector(selectEditor)
  const previousMoogleStats = !!useWordleSelector(
    selectMoogleTimestamp,
  )
  const { winHeight } = useWindowSize()
  const { getVariant, reportExposure } = useAbra()

  const hasStartedGame = currentRowIndex > 0
  const subtitle = useWelcomeSubtitle(hasStartedGame, loggedIn)
  const title = useWelcomeTitle(hasStartedGame)
  const primaryCTA = useWelcomePrimaryCTA(hasStartedGame)
  const primaryCTALabel = useWelcomeInteractionLabel(hasStartedGame)
  const isMobileWeb =
    !window.isHybridWebView &&
    !window.newsreaderAppPlatform &&
    isMobile

  const largeCTAVariant = getVariant(
    'GAMES_wordleAppDownloadCTASize_0623',
  )

  const inLargeCTAVariant =
    largeCTAVariant && isMobileWeb && winHeight > largeCTAWindowHeight

  const showLargeCTAVariant =
    inLargeCTAVariant && largeCTAVariant !== '0_control'

  const { name, region, label, context } = useWelcomeImpressionData(
    hasStartedGame,
    loggedIn,
  )

  const trackData = {
    context: `${dayOffset}` || undefined,
    elementName: 'wordle',
    moduleLabel: 'welcome',
  }

  const trackMomentClick = (label: string) => {
    trackClick({
      name: 'wordle',
      context: `${dayOffset}`,
      label,
    })
  }

  React.useEffect(() => {
    trackImpression(name, region, label, context)
    // Report GAMES_wordleAppDownloadCTASize_0623 exposure for non-holdout users with eligible devices
    if (inLargeCTAVariant) {
      reportExposure('GAMES_wordleAppDownloadCTASize_0623')
    }
  }, [])

  const showAdInterstitial = useShowAdInterstitial()
  const showStatsSelection = useStatsSelectionTool(hasLoaded)

  const nextMoment = (() => {
    if (showAdInterstitial) {
      return 'ad'
    }

    if (showStatsSelection) {
      return 'statsSelection'
    }

    return 'game'
  })()

  const buttons = [
    {
      hide: hasStartedGame || previousMoogleStats, // TODO: should we use lastPlayed instead?
      text: 'How to play',
      secondary: true,
      onClick: () => {
        transition(nextMoment)
        dispatch(openModal('help'))
        trackMomentClick('help-welcome')
      },
    },
    {
      hide:
        hasStartedGame || loggedIn || !!window.newsreaderAppPlatform,
      text: 'Log in',
      secondary: true,
      isLogin: true,
      onClick: () => {
        trackMomentClick('log-in-welcome')
      },
    },
    {
      text: primaryCTA,
      onClick: () => {
        transition(nextMoment)
        trackMomentClick(primaryCTALabel)
      },
    },
  ]

  const showNote =
    window.isHybridWebView && !previousMoogleStats && !hasStartedGame

  const content = (
    <div className={styles.contentWelcome}>
      <HybridOnly>
        <div className={styles.back}>
          <BackButton trackData={trackData} />
        </div>
      </HybridOnly>
      <div
        className={cn(styles.contentWelcomeContainer, {
          [styles.largeCTAVariant]: showLargeCTAVariant,
        })}
      >
        {hasStartedGame && <IconAndTitle small />}
        <div className={styles.contentWelcomeMain}>
          {!hasStartedGame && <IconAndTitle />}
          {title && <div className={styles.title}>{title}</div>}
          <Subtitle>{subtitle}</Subtitle>
          <ButtonContainer buttons={buttons} />
          <MetadataDisplay dayOffset={dayOffset} editor={editor} />
        </div>
        {showNote && (
          <Note
            closeMoment={() => transition(nextMoment)}
            trackLink={() => trackMomentClick('welcome-more-info')}
          />
        )}
      </div>
      {showLargeCTAVariant ? (
        <LargeCTABanner
          variant={largeCTAVariant}
          ctaLink="https://fpx3r.app.goo.gl/app-wordle"
          newTab
          onClickLink={() => {
            trackModuleInteraction(
              '',
              `${dayOffset}`,
              'wordle',
              'welcome-download-app',
              `${getAppLink()}`,
            )
          }}
          onVisible={() => {
            trackImpression('welcome', 'wordle', 'download-app')
          }}
          positionRelative
        />
      ) : (
        isMobileWeb && (
          <NonDismissalBanner
            title="Play Wordle and more in the NYT Games app."
            icon="crossword-app"
            ctaLink="https://fpx3r.app.goo.gl/app-wordle"
            newTab
            onVisible={() => {
              trackImpression('welcome', 'wordle', 'download-app')
            }}
            position="relative"
            onClickLink={() => {
              trackModuleInteraction(
                '',
                `${dayOffset}`,
                'wordle',
                'welcome-download-app',
                `${getAppLink()}`,
              )
            }}
            showMultipleIconsBanner
          />
        )
      )}
    </div>
  )

  return <>{content}</>
}
