import React from 'react'
import cn from 'classnames'

import LoginCTA from '../LoginLink/LoginCTA'
import { getDateForDayOffset } from '../../utils/gamelogic'
import { useWordleSelector } from '../../store/types'
import {
  selectCurrentRowIndex,
  selectGameDayOffset,
  selectGameStatus,
} from '../../store/game/game.selectors'
import {
  selectCurrentStreak,
  selectGetWonYesterday,
} from '../../store/stats/stats.selectors'

import styles from './Welcome.module.scss'

export const IconAndTitle = ({ small }: { small?: boolean }) => (
  <>
    <div
      role="img"
      aria-label="wordle grid icon"
      className={cn(styles.icon, { [styles.small]: small })}
    />
    <h1 className={cn(styles.title, { [styles.small]: small })}>
      Wordle
    </h1>
  </>
)

export const Subtitle = ({
  children,
}: {
  children: React.ReactNode
}) => <h2 className={styles.subtitle}>{children}</h2>

export const MetadataDisplay = ({
  dayOffset,
  editor,
}: {
  dayOffset: number | null
  editor?: string
}) => {
  if (!dayOffset) {
    return null
  }

  const dateString = getDateForDayOffset(
    dayOffset,
  ).toLocaleDateString('en-us', {
    year: 'numeric',
    month: 'long',
    day: 'numeric',
  })

  // if editor is present and it is on or after Nov. 7th (offset 506)
  // will remove this condition once we've launched, for now it serves
  // as an easy "feature flag"
  const showEditor = editor && dayOffset != null && dayOffset >= 506

  return (
    <div className={styles.dateContainer}>
      <h3 className={styles.date}>{dateString}</h3>
      <p className={styles.wordleMeta}>No. {dayOffset}</p>
      {showEditor && (
        <p className={styles.wordleMeta}>Edited by {editor}</p>
      )}
    </div>
  )
}

type ButtonContainerProps = {
  buttons: Array<{
    text: string
    onClick?: () => void
    secondary?: boolean
    hide?: boolean
    isLogin?: boolean
  }>
}

export const ButtonContainer = ({
  buttons,
}: ButtonContainerProps) => (
  <div className={styles.buttonContainer}>
    {buttons
      .filter((button) => !button.hide)
      .map(({ onClick, text, secondary, isLogin }) => {
        const buttonStyles = cn(styles.button, {
          [styles.secondary]: secondary,
        })

        const button = (
          <button
            key={text}
            data-testid={text}
            type="button"
            onClick={onClick}
            className={buttonStyles}
          >
            {text}
          </button>
        )

        if (isLogin) {
          return (
            <LoginCTA
              key={text}
              styles={buttonStyles}
              handleClick={onClick}
            >
              {text}
            </LoginCTA>
          )
        }

        return button
      })}
  </div>
)

export const useWelcomeTitle = (hasStarted: boolean) => {
  const gameStatus = useWordleSelector(selectGameStatus)

  if (!hasStarted) {
    // in this case, the title is rendered w/ the big icon in IconAndTitle
    return ''
  }

  if (gameStatus !== 'IN_PROGRESS') {
    // the user has already won or lost today
    return 'Hi Wordler'
  }

  return 'Welcome Back'
}

export const useWelcomeSubtitle = (
  hasStarted: boolean,
  loggedIn: boolean,
) => {
  const gameStatus = useWordleSelector(selectGameStatus)
  const streak = useWordleSelector(selectCurrentStreak)
  const currentRowIndex = useWordleSelector(selectCurrentRowIndex)
  const getWonYesterday = useWordleSelector(selectGetWonYesterday)
  const wonYesterday = getWonYesterday()

  if (hasStarted && gameStatus === 'IN_PROGRESS') {
    const attempts = currentRowIndex + 1
    return `Youʼre on attempt ${attempts} out of 6. Keep it up!`
  }

  if (gameStatus === 'WIN') {
    return 'Great job on todayʼs puzzle! Check out your progress.'
  }

  if (gameStatus === 'FAIL') {
    return 'Tomorrow’s a new day, with a new puzzle. See you then.'
  }

  if (loggedIn && !hasStarted && streak > 0 && wonYesterday) {
    return (
      <>
        <span className="visually-hidden">
          Go ahead, add another day to your {streak} day streak.
        </span>
        <span aria-hidden="true">
          Go ahead, add another day to your{' '}
          <span className={styles.bold}>{streak} day</span> streak.
        </span>
      </>
    )
  }

  return (
    <>
      <span className="visually-hidden">
        Get 6 chances to guess a 5-letter word.
      </span>
      <span aria-hidden="true">
        Get 6 chances to guess a{' '}
        <span className={styles.noWrap}>5-letter</span> word.
      </span>
    </>
  )
}

export const useWelcomePrimaryCTA = (hasStarted: boolean) => {
  const gameStatus = useWordleSelector(selectGameStatus)

  if (!hasStarted) {
    return 'Play'
  }

  if (gameStatus === 'IN_PROGRESS') {
    return 'Continue'
  }

  // the user has won or lost already today
  return 'See Stats'
}

export const useWelcomeImpressionData = (
  hasStarted: boolean,
  loggedIn: boolean,
) => {
  const currentStreak = useWordleSelector(selectCurrentStreak)
  const dayOffset = `${useWordleSelector(selectGameDayOffset)}`
  const gameStatus = useWordleSelector(selectGameStatus)

  let label = ''
  let name = 'welcome-modal'
  const region = 'wordle-welcome'
  const context = dayOffset

  if (loggedIn) {
    // DIG only wants to send the streak if they opted in to Moogle
    label = `${currentStreak}`
  } else {
    label = 'log-in'
  }

  if (hasStarted) {
    name = 'welcome-back-modal'

    if (gameStatus !== 'IN_PROGRESS') {
      name = 'welcome-see-stats-modal'
    }
  }

  return { label, name, region, context }
}

export const useWelcomeInteractionLabel = (hasStarted: boolean) => {
  const gameStatus = useWordleSelector(selectGameStatus)

  if (!hasStarted) {
    return 'play'
  }

  if (gameStatus === 'IN_PROGRESS') {
    return 'continue'
  }

  return 'see-stats'
}
