import { DeepPartial } from 'redux'

import { WordleStoreState } from '../types'

import { loadSolution } from './api.actions'

const SOLUTION_REJECTS_DATE = new Date('2022-06-25')
const SUCCESS_DATE = new Date('2022-01-02')
const SOLUTION_400s_DATE = new Date('2019-12-31')

jest.mock('foundation', () => {
  return {
    xhr: {
      get: (url: string) => {
        if (
          url ===
          `domain.com/svc/wordle/v2/${new Date(
            '2022-01-02',
          ).toLocaleDateString('en-CA')}.json`
        ) {
          return Promise.resolve({
            days_since_launch: 4,
            id: 5,
            solution: 'BACKEND SOLUTION',
          })
        }

        if (
          url ===
          `domain.com/svc/wordle/v2/${new Date(
            '2019-12-31',
          ).toLocaleDateString('en-CA')}.json`
        ) {
          return Promise.resolve({
            status: 'ERROR',
            errors: ['Not Found'],
            results: [],
          })
        }

        return Promise.reject(new Error('any error'))
      },
    },
    env: {
      name: 'foobar',
      edge: 'domain.com',
    },
  }
})

jest.mock('../../utils/gamelogic', () => {
  return {
    ...jest.requireActual('../../utils/gamelogic'),
    getDayOffsetForDate: (date: Date) => {
      if (
        date.toLocaleDateString('en-CA') ===
        new Date('2022-06-25').toLocaleDateString('en-CA')
      ) {
        return 2
      }

      return 0
    },
  }
})

describe('loadSolution', () => {
  const buildGetState = () => () => {
    const state: DeepPartial<WordleStoreState> = {
      persist: {
        game: {},
      },
      api: {
        solution: undefined,
      },
    }

    return state as WordleStoreState
  }

  it('if backend solution fetching succeeds, dispatches solution success action', async () => {
    const getState = buildGetState()
    const dispatch = jest.fn()
    await loadSolution({ today: SUCCESS_DATE })(
      dispatch,
      getState,
      null,
    )
    expect(dispatch).toHaveBeenCalledTimes(2)
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: 'wordle/api/LOAD_SOLUTION',
    })
    expect(dispatch).toHaveBeenNthCalledWith(2, {
      payload: {
        data: {
          dayOffset: 4,
          id: 5,
          solution: 'BACKEND SOLUTION',
        },
      },
      type: 'wordle/api/SOLUTION_SUCCESS',
    })
  })

  it('if backend solution endpoint 400s, renders and opens error', async () => {
    const getState = buildGetState()
    const dispatch = jest.fn()
    await loadSolution({
      today: SOLUTION_400s_DATE,
    })(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(3)
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: 'wordle/api/LOAD_SOLUTION',
    })
    expect(dispatch).toHaveBeenNthCalledWith(2, {
      type: 'wordle/api/SOLUTION_ERROR',
    })
    expect(dispatch).toHaveBeenNthCalledWith(3, {
      payload: {
        error: 'default',
      },
      type: 'wordle/overlays/OPEN_ERROR',
    })
  })

  it('if backend solution endpoint is down, renders and opens error', async () => {
    const getState = buildGetState()
    const dispatch = jest.fn()
    await loadSolution({
      today: SOLUTION_REJECTS_DATE,
    })(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(3)
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: 'wordle/api/LOAD_SOLUTION',
    })
    expect(dispatch).toHaveBeenNthCalledWith(2, {
      type: 'wordle/api/SOLUTION_ERROR',
    })
    expect(dispatch).toHaveBeenNthCalledWith(3, {
      payload: {
        error: 'default',
      },
      type: 'wordle/overlays/OPEN_ERROR',
    })
  })
})
