import type { CoreState, WordleGameData } from '@nyt/games-schema'
import { format } from 'date-fns'

import { captureSentryError, env, xhr } from 'foundation'

import {
  openError,
  closeModal,
  closeNavModal,
  closePage,
  addToast,
} from '../overlays/overlays.actions'
import {
  setRegiLocalState,
  loadInitialState,
  setInitialState,
} from '../moogle/moogle.actions'
import { startOrContinueGame } from '../game/game.actions'
import { selectGameId } from '../game/game.selectors'
import type { WordleThunk } from '../types'
import { WordleMoogleState } from '../moogle/moogle.helpers'

import { selectMoogleUser } from './api.selectors'

const MOOGLE_API =
  env.name === 'prod'
    ? 'https://www.nytimes.com'
    : 'https://www.stg.nytimes.com'

export const LOAD_MOOGLE_GET = 'wordle/api/LOAD_MOOGLE_GET'
export const MOOGLE_GET_SUCCESS = 'wordle/api/MOOGLE_GET_SUCCESS'
export const MOOGLE_GET_ERROR = 'wordle/api/MOOGLE_GET_ERROR'

export const LOAD_MOOGLE_POST = 'wordle/api/LOAD_MOOGLE_POST'
export const MOOGLE_POST_SUCCESS = 'wordle/api/MOOGLE_POST_SUCCESS'
export const MOOGLE_POST_ERROR = 'wordle/api/MOOGLE_POST_ERROR'
export const MOOGLE_POST_RESET = 'wordle/api/MOOGLE_POST_RESET'

export const LOAD_SOLUTION = 'wordle/api/LOAD_SOLUTION'
export const SOLUTION_SUCCESS = 'wordle/api/SOLUTION_SUCCESS'
export const SOLUTION_ERROR = 'wordle/api/SOLUTION_ERROR'

export const APP_LOADED = 'wordle/api/APP_LOADED'
export const DATA_FETCHED = 'wordle/api/DATA_FETCHED'
export const RESTART_LOAD = 'wordle/api/RESTART_LOAD'
export const MAKE_STATS_SELECTION = 'wordle/api/MAKE_STATS_SELECTION'

const SOLUTION_ENDPOINT = (d: Date) =>
  `${env.edge}/svc/wordle/v2/${format(d, 'yyyy-MM-dd')}.json`

type LoadMoogleGetAction = {
  type: typeof LOAD_MOOGLE_GET
}

type MoogleGetSuccessAction = {
  type: typeof MOOGLE_GET_SUCCESS
  payload: {
    data: CoreState
  }
}

type MoogleGetErrorAction = {
  type: typeof MOOGLE_GET_ERROR
  payload: {
    message: string
  }
}

type LoadMooglePostAction = {
  type: typeof LOAD_MOOGLE_POST
}

type MooglePostSuccessAction = {
  type: typeof MOOGLE_POST_SUCCESS
  payload: {
    data: CoreState
  }
}

type MooglePostErrorAction = {
  type: typeof MOOGLE_POST_ERROR
  payload: {
    saveData: WordleMoogleState
  }
}

type MooglePostResetAction = {
  type: typeof MOOGLE_POST_RESET
}

type LoadSolutionAction = {
  type: typeof LOAD_SOLUTION
}

type SolutionSuccessAction = {
  type: typeof SOLUTION_SUCCESS
  payload: {
    data: {
      solution: string
      id: number
      dayOffset: number
      editor?: string
    }
  }
}

type SolutionErrorAction = {
  type: typeof SOLUTION_ERROR
}

export type SolutionResponseError = {
  status: string
  errors: string[]
  results?: {}[]
}

export type SolutionResponseSuccess = {
  solution: string
  days_since_launch: number
  id: number
  editor?: string
}

type AppLoadedAction = {
  type: typeof APP_LOADED
}

type DataFetchedAction = {
  type: typeof DATA_FETCHED
}

type RestartLoadAction = {
  type: typeof RESTART_LOAD
}

type MakeStatsSelectionAction = {
  type: typeof MAKE_STATS_SELECTION
}

export type WordleApiAction =
  | LoadMoogleGetAction
  | MoogleGetSuccessAction
  | MoogleGetErrorAction
  | LoadMooglePostAction
  | MooglePostSuccessAction
  | MooglePostErrorAction
  | LoadSolutionAction
  | SolutionSuccessAction
  | SolutionErrorAction
  | MooglePostResetAction
  | AppLoadedAction
  | DataFetchedAction
  | RestartLoadAction
  | MakeStatsSelectionAction

const setAppLoaded = (): AppLoadedAction => ({
  type: APP_LOADED,
})

const setDataFetched = (): DataFetchedAction => ({
  type: DATA_FETCHED,
})

export const restartLoad = (): RestartLoadAction => ({
  type: RESTART_LOAD,
})

const handleError =
  (err: Error): WordleThunk =>
  (dispatch) => {
    // clear any overlays
    dispatch(closeModal())
    dispatch(closePage())
    dispatch(closeNavModal())

    if (err.message === 'malformed Moogle response data') {
      captureSentryError(new Error(err.message), {
        api: 'moogle',
        category: 'game_state',
      })
    }

    if (err.message && err.message === 'No internet') {
      dispatch(openError('offline'))
    } else if (err.message && err.message === 'profile') {
      dispatch(openError('profile'))
    } else {
      dispatch(openError('default'))
    }
  }

export const loadSolution =
  ({ today }: { today: Date }): WordleThunk<Promise<void>> =>
  (dispatch): Promise<void> => {
    dispatch({ type: LOAD_SOLUTION })

    const getSolution = () =>
      xhr
        .get<SolutionResponseSuccess | SolutionResponseError>(
          SOLUTION_ENDPOINT(today),
        )
        .catch((e) => {
          if (format(today, 'yyyy-MM-dd') === '1970-01-01') {
            const additionalInfo = [
              `message: ${e.message}`,
              `today: ${today}`,
              `new Date(): ${new Date()}`,
              `new Date().getTimezoneOffset(): ${new Date().getTimezoneOffset()}`,
              `Intl.DateTimeFormat().resolvedOptions().timeZone: ${
                Intl.DateTimeFormat().resolvedOptions().timeZone
              }`,
            ]

            e.message = additionalInfo.join(', ')
            captureSentryError(new Error(e.message), {
              api: 'games-puzzles',
              category: 'wordle-solution-date-invalid',
            })
          } else {
            captureSentryError(new Error(e.message), {
              api: 'games-puzzles',
              category: 'wordle-solution',
            })
          }

          throw e
        })

    function dispatchSuccess(data: SolutionResponseSuccess) {
      dispatch({
        type: SOLUTION_SUCCESS,
        payload: {
          data: {
            solution: data.solution,
            dayOffset: data.days_since_launch,
            id: data.id,
            editor: data.editor,
          },
        },
      })
    }

    return getSolution()
      .then(
        (data: SolutionResponseSuccess | SolutionResponseError) => {
          if (data.hasOwnProperty('solution')) {
            dispatchSuccess(data as SolutionResponseSuccess)
            return
          }
          throw new Error()
        },
      )
      .catch((err) => {
        dispatch({ type: SOLUTION_ERROR })

        if (err.message && err.message === 'No internet') {
          dispatch(openError('offline'))
        } else {
          dispatch(openError('default'))
        }
      })
  }

const loggedOutErrorMessage = 'user not authenticated for GET /state'

export const loadMoogle =
  (): WordleThunk<Promise<void>> => (dispatch) => {
    dispatch({ type: LOAD_MOOGLE_GET })

    return xhr
      .get<CoreState>(`${MOOGLE_API}/svc/games/state/wordle/latest`)
      .then((data) => {
        if (data.error === 'forbidden') {
          throw new Error(loggedOutErrorMessage)
        }

        if (!data.timestamp && !data.user_id) {
          throw new Error('malformed Moogle response data')
        } else {
          dispatch({
            type: MOOGLE_GET_SUCCESS,
            payload: { data },
          })
        }
      })
      .catch((err) => {
        if (err.message !== loggedOutErrorMessage) {
          dispatch({
            type: MOOGLE_GET_ERROR,
            payload: { message: err.message },
          })
        }
      })
  }

export const updateMoogle =
  (moogleState: WordleMoogleState): WordleThunk<Promise<void>> =>
  (dispatch, getState) => {
    const regiId = selectMoogleUser(getState())
    const puzzle_id = selectGameId(getState())
    const { timestamp, schemaVersion, ...persistedState } =
      moogleState

    // If the puzzle data didn't successfully load, don't update with bad data.
    if (!puzzle_id) {
      return Promise.resolve()
    }

    if (regiId) {
      dispatch({ type: LOAD_MOOGLE_POST })

      return xhr
        .post<CoreState>(`${MOOGLE_API}/svc/games/state`, {
          game: 'wordle',
          game_data: persistedState,
          puzzle_id: `${puzzle_id}`,
          schema_version: schemaVersion || '0.0.0',
          timestamp,
          user_id: regiId,
        })
        .then((data) => {
          if (!data.version) {
            throw new Error('malformed Moogle response data')
          } else {
            dispatch({
              type: MOOGLE_POST_SUCCESS,
              payload: {
                data,
              },
            })
          }
        })
        .catch((err) => {
          dispatch({
            type: MOOGLE_POST_ERROR,
            payload: { saveData: moogleState },
          })
          dispatch(
            addToast({
              text: 'There was an error while saving. Please refresh the page and try again',
              duration: 3000,
              isSystem: true,
            }),
          )
        })
    }

    return Promise.resolve()
  }

export const loadApp = (): WordleThunk => async (dispatch) => {
  const today = new Date()

  try {
    await Promise.all([
      dispatch(loadMoogle()),
      dispatch(loadSolution({ today })),
    ])

    dispatch(setDataFetched())
  } catch (e) {
    console.error('loadApp promise rejected', e)
  }
}

export const loadPreview =
  (printDate: string): WordleThunk =>
  async (dispatch) => {
    // set "today" to noon local time for the given date
    const today = new Date(`${printDate}T12:00:00`)

    try {
      await dispatch(loadSolution({ today }))
      dispatch(setDataFetched())
    } catch (e) {
      console.error('loadApp promise rejected', e)
    }
  }

export const finishLoading = (): WordleThunk => (dispatch) => {
  const initialStateForFirstSave = dispatch(setRegiLocalState())
  dispatch(loadInitialState(initialStateForFirstSave))
  dispatch(startOrContinueGame())
  dispatch(setAppLoaded())
}

export const makeStatsSelection =
  (chosenState: WordleGameData): WordleThunk =>
  (dispatch) => {
    dispatch(setInitialState(chosenState))
    dispatch(startOrContinueGame())
    dispatch({ type: MAKE_STATS_SELECTION })
    dispatch(
      addToast({
        text: 'Your saved stats have been updated',
        duration: 6000,
        isSystem: true,
      }),
    )
  }
