import { DeepPartial } from 'redux'

import { ModalName, PageName } from '../overlays/overlays.state'
import { WordleStoreState } from '../types'

import {
  EVALUATE_ROW,
  startOrContinueGame,
  submitGuess,
} from './game.actions'

let mockSelectGameDayOffset = 99
jest.mock('./game.selectors', () => ({
  ...jest.requireActual('./game.selectors'),
  selectGameDayOffset: () => mockSelectGameDayOffset,
}))

let mockSelectApiDayOffset = 5
jest.mock('../api/api.selectors', () => ({
  ...jest.requireActual('../api/api.selectors'),
  selectApiDayOffset: () => mockSelectApiDayOffset,
}))

jest.mock('foundation', () => {
  return {
    env: {
      name: 'notprod',
      edge: 'domain.com',
    },
  }
})

jest.mock('../overlays/overlays.actions', () => ({
  ...jest.requireActual('../overlays/overlays.actions'),
  addToast: ({ text = '', invalidate = false }) => ({
    type: 'TOAST',
    text,
    invalidate,
  }),
}))

const now = Date.now()

const buildGetState =
  ({
    boardState = ['testy'],
    status = 'IN_PROGRESS',
    hardMode = false,
    modalName = null,
    pageName = null,
  }: {
    boardState?: string[]
    status?: string
    hardMode?: boolean
    modalName?: ModalName | null
    pageName?: PageName | null
  } = {}) =>
  () => {
    const state: DeepPartial<WordleStoreState> = {
      persist: {
        game: {
          boardState,
          currentRowIndex: boardState.length - 1,
          // @ts-ignore
          status,
          timestamps: {
            lastPlayed: 42,
            lastCompleted: 42,
          },
        },
        settings: {
          hardMode,
        },
        stats: {},
      },
      transient: {
        isAnimatingRow: false,
      },
      overlays: {
        modal: modalName,
        page: pageName,
      },
      api: {
        solution: {
          data: {
            solution: 'testy',
            id: 1,
            dayOffset: 1,
          },
        },
      },
    }

    return state as WordleStoreState
  }

describe('startOrContinueGame', () => {
  describe('when reset is not true', () => {
    describe('and game dayOffset matches api dayOffset', () => {
      let ogMockSelectGameDayOffset: number
      let ogmockSelectApiDayOffset: number
      beforeEach(() => {
        ogMockSelectGameDayOffset = mockSelectGameDayOffset
        ogmockSelectApiDayOffset = mockSelectApiDayOffset
        mockSelectGameDayOffset = 2
        mockSelectApiDayOffset = 2
      })

      afterEach(() => {
        mockSelectGameDayOffset = ogMockSelectGameDayOffset
        mockSelectApiDayOffset = ogmockSelectApiDayOffset
      })

      it('continues game with current solution', () => {
        const getState = buildGetState({ status: 'WIN' })
        const dispatch = jest.fn()
        startOrContinueGame()(dispatch, getState, null)
        expect(dispatch).toHaveBeenCalledWith({
          payload: {
            currentRowIndex: 0,
          },
          type: 'wordle/game/CONTINUE_GAME',
        })
      })
    })

    describe('and game day offset matches api day offset', () => {
      let ogMockSelectGameDayOffset: number
      let ogmockSelectApiDayOffset: number
      beforeEach(() => {
        ogMockSelectGameDayOffset = mockSelectGameDayOffset
        ogmockSelectApiDayOffset = mockSelectApiDayOffset
        mockSelectGameDayOffset = 2
        mockSelectApiDayOffset = 2
      })

      afterEach(() => {
        mockSelectGameDayOffset = ogMockSelectGameDayOffset
        mockSelectApiDayOffset = ogmockSelectApiDayOffset
      })

      it('continues game', () => {
        const getState = buildGetState({ status: 'WIN' })
        const dispatch = jest.fn()
        startOrContinueGame()(dispatch, getState, null)
        expect(dispatch).toHaveBeenCalledWith({
          payload: {
            currentRowIndex: 0,
          },
          type: 'wordle/game/CONTINUE_GAME',
        })
      })
    })

    describe('and game day offset does not match api day offset', () => {
      let ogMockSelectGameDayOffset: number
      let ogmockSelectApiDayOffset: number
      beforeEach(() => {
        ogMockSelectGameDayOffset = mockSelectGameDayOffset
        ogmockSelectApiDayOffset = mockSelectApiDayOffset
        mockSelectGameDayOffset = 1
        mockSelectApiDayOffset = 333
      })

      afterEach(() => {
        mockSelectGameDayOffset = ogMockSelectGameDayOffset
        mockSelectApiDayOffset = ogmockSelectApiDayOffset
      })

      it('starts new game', () => {
        const getState = buildGetState({ status: 'WIN' })
        const dispatch = jest.fn()
        startOrContinueGame()(dispatch, getState, null)
        expect(dispatch).toHaveBeenCalledWith({
          payload: {
            id: 1,
            numRows: 6,
            dayOffset: mockSelectApiDayOffset,
            isOnStreak: false,
          },
          type: 'wordle/game/START_GAME',
        })
      })
    })
  })

  describe('when reset is true', () => {
    const savedLocation = window.location

    beforeEach(() => {
      // @ts-ignore
      delete window.location
      // @ts-ignore
      window.location = Object.assign(new URL('https://foobar.com'), {
        search: 'something-reset-something',
      })
    })

    afterEach(() => {
      window.location = savedLocation
    })

    it('starts a new game', () => {
      const getState = buildGetState({ status: 'WIN' })
      const dispatch = jest.fn()
      startOrContinueGame()(dispatch, getState, null)
      expect(dispatch).toHaveBeenCalledWith({
        payload: {
          id: 1,
          numRows: 6,
          dayOffset: mockSelectApiDayOffset,
          isOnStreak: true,
        },
        type: 'wordle/game/START_GAME',
      })
    })
  })
})

describe('submitGuess', () => {
  it('should do nothing if game not in progress', () => {
    const getState = buildGetState({ status: 'WIN' })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).not.toHaveBeenCalled()
  })

  it('should do nothing if modal is open while game is in progress', () => {
    const getState = buildGetState({
      status: 'IN_PROGRESS',
      modalName: 'help',
    })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).not.toHaveBeenCalled()
  })

  it('should warn if guess is too short', () => {
    const getState = buildGetState({ boardState: ['test'] })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(1)
    expect(dispatch).toHaveBeenCalledWith({
      type: 'TOAST',
      text: 'Not enough letters',
      invalidate: true,
    })
  })

  it('should warn if guess is not in valid words', () => {
    const getState = buildGetState({ boardState: ['testq'] })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(1)
    expect(dispatch).toHaveBeenCalledWith({
      type: 'TOAST',
      text: 'Not in word list',
      invalidate: true,
    })
  })

  it('should warn if guess does not include present letter in hard mode', () => {
    const getState = buildGetState({
      hardMode: true,
      boardState: ['guess', 'again'],
    })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(1)
    expect(dispatch).toHaveBeenCalledWith({
      type: 'TOAST',
      text: 'Guess must contain E',
      invalidate: true,
    })
  })

  it('should warn if guess does not include correct letter in hard mode', () => {
    const getState = buildGetState({
      hardMode: true,
      boardState: ['toast', 'again'],
    })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(1)
    expect(dispatch).toHaveBeenCalledWith({
      type: 'TOAST',
      text: '1st letter must be T',
      invalidate: true,
    })
  })

  it('should submit winning guess', () => {
    const getState = buildGetState({
      hardMode: true,
      boardState: ['toast', 'testy'],
    })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(1)
    const action = dispatch.mock.calls[0][0]
    expect(action.type).toEqual(EVALUATE_ROW)
    expect(action.payload.status).toEqual('WIN')
    expect(action.payload.numGuesses).toEqual(2)
  })

  it('should submit losing guess', () => {
    const getState = buildGetState({
      boardState: [
        'guess',
        'guess',
        'guess',
        'guess',
        'guess',
        'guess',
      ],
    })
    const dispatch = jest.fn()
    submitGuess()(dispatch, getState, null)
    expect(dispatch).toHaveBeenCalledTimes(1)
    const action = dispatch.mock.calls[0][0]
    expect(action.type).toEqual(EVALUATE_ROW)
    expect(action.payload.status).toEqual('FAIL')
    expect(action.payload.numGuesses).toEqual(6)
  })
})
