import { setRefreshFlag } from '../../game-refresher'
import {
  checkGuessHardMode,
  getDifferenceInDays,
  validateWord,
} from '../../utils/gamelogic'
import { selectHardMode } from '../settings/settings.selectors'
import { selectInputDisabled } from '../transient/transient.selectors'
import type { WordleThunk } from '../types'
import { addToast } from '../overlays/overlays.actions'
import {
  selectApiDayOffset,
  selectId,
  selectSolution,
} from '../api/api.selectors'
import { selectGetWonYesterday } from '../stats/stats.selectors'

import { NUM_ROWS, NUM_TILES } from './game.constants'
import {
  selectBoardState,
  selectCurrentRowIndex,
  selectEvaluations,
  selectGuess,
  selectGameDayOffset,
  selectGameStatus,
} from './game.selectors'
import { WordleStatus } from './game.state'

export const START_GAME = 'wordle/game/START_GAME'
export const CONTINUE_GAME = 'wordle/game/CONTINUE_GAME'
export const ADD_LETTER = 'wordle/game/ADD_LETTER'
export const BACKSPACE = 'wordle/game/BACKSPACE'
export const EVALUATE_ROW = 'wordle/game/EVALUATE_ROW'

type StartGameAction = {
  type: typeof START_GAME
  payload: {
    numRows: number
    id: number
    dayOffset: number
    isOnStreak: boolean
  }
}

type ContinueGameAction = {
  type: typeof CONTINUE_GAME
  payload: {
    currentRowIndex: number
  }
}

type AddLetterAction = {
  type: typeof ADD_LETTER
  payload: {
    letter: string
  }
}

type BackspaceAction = {
  type: typeof BACKSPACE
}

type EvaluateRowAction = {
  type: typeof EVALUATE_ROW
  payload: {
    now: number
    status: WordleStatus
    isStreak: boolean
    numGuesses: number
    guess: string
  }
}

export type WordleGameAction =
  | StartGameAction
  | ContinueGameAction
  | AddLetterAction
  | BackspaceAction
  | EvaluateRowAction

export const startOrContinueGame =
  (): WordleThunk => (dispatch, getState) => {
    const currentRowIndex = selectCurrentRowIndex(getState())
    const gameDayOffset = selectGameDayOffset(getState())
    const apiDayOffset = selectApiDayOffset(getState())
    const gameStatus = selectGameStatus(getState())

    // overrides the dayOffset logic and gives the tester a clean board
    const shouldReset =
      window.env?.name !== 'prod' &&
      document.location.search.includes('reset')

    if (
      !shouldReset &&
      gameDayOffset &&
      gameDayOffset === apiDayOffset
    ) {
      // already played today, continue game
      return dispatch({
        type: CONTINUE_GAME,
        payload: {
          currentRowIndex,
        },
      })
    }

    // start new game
    dispatch({
      type: START_GAME,
      payload: {
        id: selectId(getState()),
        numRows: NUM_ROWS,
        dayOffset: apiDayOffset,
        isOnStreak:
          gameStatus === 'WIN' &&
          !!gameDayOffset &&
          apiDayOffset - gameDayOffset <= 1, // gameDayOffset is either yesterday or today
      },
    })
  }

export const addLetter =
  (letter: string): WordleThunk =>
  (dispatch, getState) => {
    const inputDisabled = selectInputDisabled(getState())
    const guess = selectGuess(getState())
    if (inputDisabled || guess.length >= NUM_TILES) {
      return
    }

    dispatch({
      type: ADD_LETTER,
      payload: { letter },
    })
  }

export const backspace = (): WordleThunk => (dispatch, getState) => {
  const inputDisabled = selectInputDisabled(getState())
  if (inputDisabled) {
    return
  }

  dispatch({ type: BACKSPACE })
}

export const submitGuess =
  (): WordleThunk => (dispatch, getState) => {
    const state = getState()
    const inputDisabled = selectInputDisabled(state)
    const guess = selectGuess(state)
    const solution = selectSolution(state)
    const boardState = selectBoardState(state)
    const evaluations = selectEvaluations(state)
    const rowIndex = selectCurrentRowIndex(state)
    const hardMode = selectHardMode(state)
    const getWonYesterday = selectGetWonYesterday(state)

    if (inputDisabled) {
      return
    }

    if (guess.length !== NUM_TILES) {
      return dispatch(
        addToast({ text: 'Not enough letters', invalidate: true }),
      )
    }

    if (!validateWord(guess, solution)) {
      return dispatch(
        addToast({ text: 'Not in word list', invalidate: true }),
      )
    }

    if (hardMode) {
      const { validGuess, errorMessage } = checkGuessHardMode(
        guess,
        boardState[rowIndex - 1],
        evaluations[rowIndex - 1],
      )

      if (!validGuess) {
        return dispatch(
          addToast({
            text: errorMessage || 'Not valid in hard mode',
            invalidate: true,
          }),
        )
      }
    }

    const numGuesses = rowIndex + 1
    const isWin = guess === solution
    const isFail = !isWin && numGuesses >= NUM_ROWS
    const wonYesterday = getWonYesterday()
    const isStreak = isWin && wonYesterday

    let status: WordleStatus = 'IN_PROGRESS'
    if (isWin) {
      status = 'WIN'
    } else if (isFail) {
      status = 'FAIL'
    }

    if (status !== 'IN_PROGRESS') {
      setRefreshFlag()
    }

    dispatch({
      type: EVALUATE_ROW,
      payload: {
        now: Date.now(),
        status,
        isStreak,
        numGuesses,
        guess,
      },
    })
  }
