import { createSelector } from 'reselect'

import { evaluateGuess, ALPHABET } from '../../utils/gamelogic'
import type { WordleStoreState } from '../types'
import { selectSolution } from '../api/api.selectors'

import { NUM_TILES } from './game.constants'

const selectGame = (state: WordleStoreState) => state.persist.game

export const selectBoardState = createSelector(
  selectGame,
  (state) => state.boardState,
)

export const selectGameStatus = createSelector(
  selectGame,
  (game) => game.status,
)

export const selectCurrentRowIndex = createSelector(
  selectGame,
  (game) => game.currentRowIndex,
)

export const selectGuess = createSelector(
  [selectBoardState, selectCurrentRowIndex],
  (board, rowIndex) => board[rowIndex],
)

export const selectGameDayOffset = createSelector(
  selectGame,
  (game) => game.dayOffset || null,
)

export const selectGameId = createSelector(
  selectGame,
  (game) => game.id,
)

// DEPRECATED: clean up in XWD-17383
export const selectLastPlayed = createSelector(
  selectGame,
  (game) => game.timestamps.lastPlayed,
)

// DEPRECATED: clean up in XWD-17383
export const selectLastCompleted = createSelector(
  selectGame,
  (game) => game.timestamps.lastCompleted,
)

export const selectEvaluations = createSelector(
  [selectBoardState, selectCurrentRowIndex, selectSolution],
  (boardState, currentRowIndex, solution) => {
    return boardState.map((guess, i) => {
      if (i >= currentRowIndex) {
        return null
      }

      return evaluateGuess(guess, solution)
    })
  },
)

export const selectDisabledKeys = createSelector(
  [selectGuess, selectGameStatus],
  (currentGuess, gameStatus) => {
    // set specific keys as disabled as user progresses through playing game
    const enterCharacter = '↵'
    const backspaceCharacter = '←'
    const allValidLetters = ALPHABET
    const disabledKeys = []

    if (gameStatus === 'IN_PROGRESS') {
      if (!currentGuess) {
        // no letters have been guessed, enter and backspace are disabled
        disabledKeys.push(enterCharacter, backspaceCharacter)
      } else if (currentGuess.length === NUM_TILES) {
        // row is full so alphabet letters are disabled as you cannot add any more
        disabledKeys.push(...allValidLetters)
      } else if (currentGuess.length < NUM_TILES) {
        // enter is disabled when there are less then 5 letters
        disabledKeys.push(enterCharacter)
      }
    } else {
      // if game isn't in progress disable all keys
      disabledKeys.push(
        ...allValidLetters,
        enterCharacter,
        backspaceCharacter,
      )
    }

    return disabledKeys
  },
)
