import { AnyAction, Dispatch } from 'redux'

import {
  trackModuleInteraction,
  trackInteraction,
  trackImpression,
  trackClick,
} from 'foundation'

import {
  OPEN_MODAL,
  OPEN_PAGE,
  OPEN_NAV_MODAL,
  TRACK_NAV_MODAL_ITEM,
} from '../overlays/overlays.actions'
import { CHANGE_SETTING } from '../settings/settings.actions'
import { EVALUATE_ROW } from '../game/game.actions'
import { TRACK_STATS_IMPRESSION } from '../stats/stats.actions'

import analyticsTracking from './analyticsTracking'

jest.mock('foundation', () => ({
  trackModuleInteraction: jest.fn(),
  trackInteraction: jest.fn(),
  trackImpression: jest.fn(),
  trackClick: jest.fn(),
  abra: {
    reportExposure: jest.fn(),
    getVariant: jest.fn(),
  },
}))

describe('middleware: analyticsTracking', () => {
  let next: Dispatch<AnyAction>
  let getState
  let store
  let middleware: (arg1: AnyAction) => void

  beforeEach(() => {
    jest.resetAllMocks()
    next = jest.fn()
    getState = jest.fn()
    store = { getState, dispatch: jest.fn() }
    middleware = analyticsTracking(store)(next)
    getState.mockReturnValue({
      persist: {
        game: {
          boardState: [],
          dayOffset: 12345,
          currentRowIndex: 0,
          timestamps: {
            lastPlayed: null,
            lastCompleted: null,
          },
          status: 'IN_PROGRESS',
        },
        settings: {
          hardMode: false,
        },
        stats: {
          gamesPlayed: 100,
          gamesWon: 70,
          currentStreak: 20,
          maxStreak: 50,
          guesses: {
            1: 1,
            2: 2,
            3: 3,
            4: 4,
            5: 5,
            6: 6,
          },
        },
        modal: null,
        page: null,
      },
    })
  })

  test('calls next with the provided action', () => {
    const action = { type: 'AN_ACTION' }
    middleware(action)
    expect(next).toHaveBeenCalledWith(action)
  })

  test('calls open-modal interaction event when open modal is triggered', () => {
    middleware({
      type: OPEN_MODAL,
      payload: { modal: 'stats', isClicked: true },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      '12345',
      'wordle',
      'stats',
    )
  })

  test('calls open-page interaction event when open page is triggered', () => {
    middleware({
      type: OPEN_PAGE,
      payload: { page: 'help' },
    })
    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      '12345',
      'wordle',
      'help',
    )
  })

  test('calls open nav modal track event when nav is opened', () => {
    middleware({
      type: OPEN_NAV_MODAL,
    })
    expect(trackClick).toHaveBeenCalledWith({
      context: null,
      label: 'click-nav',
      name: 'wordle',
      useBeacon: true,
    })
  })

  test('calls track nav modal item event for nav items', () => {
    middleware({
      type: TRACK_NAV_MODAL_ITEM,
      payload: {
        name: 'wordle',
        label: 'vertex-nav',
        context: null,
        useBeacon: true,
      },
    })
    expect(trackClick).toHaveBeenCalledWith({
      context: null,
      label: 'vertex-nav',
      name: 'wordle',
      useBeacon: true,
    })
  })

  test('calls change setting interaction event when change_setting is triggered to off', () => {
    middleware({
      type: CHANGE_SETTING,
      payload: { name: 'darkMode', value: 0 },
    })

    expect(trackInteraction).toHaveBeenCalledWith({
      context: '12345',
      element: { label: 'dark-mode', name: 'wordle' },
      label: 'turn-off',
      region: null,
    })
  })

  test('calls change setting interaction event for virual keyboard with accessibility region', () => {
    middleware({
      type: CHANGE_SETTING,
      payload: { name: 'virtualKeyboardOnlyMode', value: 0 },
    })

    expect(trackInteraction).toHaveBeenCalledWith({
      context: '12345',
      element: { label: 'virtual-keyboard', name: 'wordle' },
      label: 'turn-off',
      region: 'settings-accessibility',
    })
  })

  test('calls change setting interaction event when change_setting is triggered to on', () => {
    middleware({
      type: CHANGE_SETTING,
      payload: { name: 'darkMode', value: 1 },
    })

    expect(trackInteraction).toHaveBeenCalledWith({
      context: '12345',
      element: { label: 'dark-mode', name: 'wordle' },
      label: 'turn-on',
      region: null,
    })
  })

  test('calls game start track interaction on first valid guess', () => {
    middleware({
      type: EVALUATE_ROW,
      payload: {
        numGuesses: 1,
        status: 'IN_PROGRESS',
        guess: 'hello',
      },
    })

    expect(trackInteraction).toHaveBeenCalledWith({
      context: '12345',
      element: { label: 'start-game', name: 'wordle' },
      label: 'CB=0 DM=0 VK=0',
    })
  })

  test('calls track interaction on every valid guess', () => {
    middleware({
      type: EVALUATE_ROW,
      payload: {
        numGuesses: 0,
        status: 'IN_PROGRESS',
        guess: 'hello',
      },
    })

    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      '12345',
      'wordle',
      'solve-attempt',
      '0-hello',
    )
  })

  test('calls track interaction on win', () => {
    middleware({
      type: EVALUATE_ROW,
      payload: {
        numGuesses: 6,
        status: 'WIN',
        guess: 'hello',
      },
    })

    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      '12345',
      'wordle',
      '100%-complete',
      'e-6',
    )
  })

  test('calls track interaction on fail', () => {
    middleware({
      type: EVALUATE_ROW,
      payload: {
        numGuesses: 6,
        status: 'FAIL',
        guess: 'hello',
      },
    })

    expect(trackModuleInteraction).toHaveBeenCalledWith(
      'click',
      '12345',
      'wordle',
      '100%-complete',
      'e-0',
    )
  })

  test('calls stats impression event when stats page is shown and game is complete', () => {
    middleware({
      type: TRACK_STATS_IMPRESSION,
      payload: { isGameComplete: true },
    })
    expect(trackImpression).toHaveBeenCalledWith(
      'wordle',
      'congrats-modal',
      [
        'p: 100',
        'w: 70',
        'cs: 20',
        'ms: 50',
        '1: 1',
        '2: 2',
        '3: 3',
        '4: 4',
        '5: 5',
        '6: 6',
      ],
      '12345',
    )
  })

  test('calls stats impression event when stats page is shown and game is not complete', () => {
    middleware({
      type: TRACK_STATS_IMPRESSION,
      payload: { isGameComplete: false },
    })
    expect(trackImpression).toHaveBeenCalledWith(
      'wordle',
      'stats',
      [
        'p: 100',
        'w: 70',
        'cs: 20',
        'ms: 50',
        '1: 1',
        '2: 2',
        '3: 3',
        '4: 4',
        '5: 5',
        '6: 6',
      ],
      '12345',
    )
  })
})
