import {
  abra,
  trackModuleInteraction,
  trackInteraction,
  trackImpression,
  trackClick,
} from 'foundation'

import {
  selectColorblindMode,
  selectDarkMode,
  selectHardMode,
  selectVirtualKeyboardOnlyMode,
} from '../settings/settings.selectors'
import {
  OPEN_MODAL,
  OPEN_NAV_MODAL,
  OPEN_PAGE,
  TRACK_NAV_MODAL_ITEM,
  TRACK_AUTH_CLICK,
  TRACK_AUTH_IMPRESSION,
} from '../overlays/overlays.actions'
import { CHANGE_SETTING } from '../settings/settings.actions'
import { selectGameDayOffset } from '../game/game.selectors'
import { EVALUATE_ROW } from '../game/game.actions'
import { TRACK_STATS_IMPRESSION } from '../stats/stats.actions'
import {
  selectGamesPlayed,
  selectCurrentStreak,
  selectMaxStreak,
  selectGuesses,
  selectGamesWon,
} from '../stats/stats.selectors'
import { WordleMiddleware, WordleAction } from '../types'

const analyticsTracking: WordleMiddleware =
  (store) => (next) => (action: WordleAction) => {
    const returnValue = next(action)
    const state = store.getState()
    const colorBlindMode = selectColorblindMode(state)
    const darkMode = selectDarkMode(state)
    const virtualKeyboard = selectVirtualKeyboardOnlyMode(state)

    const id = `${selectGameDayOffset(state)}`
    const hardMode = selectHardMode(state)

    switch (action.type) {
      case OPEN_MODAL: {
        const { modal, isClicked } = action.payload
        if (modal === 'stats' && isClicked) {
          // Don't track if help opens via modal - aka, they've never played before
          // Don't track if stats is opened automatically
          trackModuleInteraction('click', id, 'wordle', modal)
        }
        break
      }

      case OPEN_PAGE: {
        const { page } = action.payload

        // DIG reports these should not be sent
        if (page === 'linkInfo') {
          break
        }

        let elementLabel = page as string
        if (page === 'statsProblem') {
          // DIG wanted this to be hyphenated
          elementLabel = 'stats-problem'
        }

        if (page) {
          trackModuleInteraction('click', id, 'wordle', elementLabel)
        }
        break
      }

      case OPEN_NAV_MODAL: {
        trackClick({
          name: 'wordle',
          label: 'click-nav',
          useBeacon: true,
          context: null,
        })
        break
      }

      case TRACK_NAV_MODAL_ITEM: {
        const { name, label, useBeacon, context } = action.payload

        trackClick({
          name,
          label,
          useBeacon,
          context,
        })
        break
      }

      case CHANGE_SETTING: {
        const { name, value } = action.payload
        const settingsNameForTracking = {
          darkMode: 'dark-mode',
          colorblindMode: 'high-contrast',
          hardMode: 'hard-mode',
          virtualKeyboardOnlyMode: 'virtual-keyboard',
        }
        const isOn = value ? 'turn-on' : 'turn-off'

        trackInteraction({
          element: {
            name: 'wordle',
            label: settingsNameForTracking[name],
          },
          context: id,
          region:
            settingsNameForTracking[name] === 'virtual-keyboard'
              ? 'settings-accessibility'
              : null,
          label: isOn,
        })

        break
      }

      case EVALUATE_ROW: {
        const CB = colorBlindMode ? '1' : '0'
        const DM = darkMode ? '1' : '0'
        const VK = virtualKeyboard ? '1' : '0'
        const hardModeOrEasyModeId = hardMode ? 'h' : 'e'

        const { numGuesses, status, guess } = action.payload

        if (numGuesses === 1) {
          trackInteraction({
            element: {
              name: 'wordle',
              label: 'start-game',
            },
            context: id,
            label: `CB=${CB} DM=${DM} VK=${VK}`,
          })
        }

        trackModuleInteraction(
          'click',
          id,
          'wordle',
          'solve-attempt',
          `${numGuesses.toString()}-${guess}`,
        )

        if (status !== 'IN_PROGRESS') {
          trackModuleInteraction(
            'click',
            id,
            'wordle',
            '100%-complete',
            `${hardModeOrEasyModeId}-${
              status === 'WIN' ? numGuesses : 0
            }`,
          )
        }

        break
      }

      case TRACK_STATS_IMPRESSION: {
        const { isGameComplete } = action.payload
        const whereItHappened = isGameComplete
          ? 'congrats-modal'
          : 'stats'

        const gamesPlayed = selectGamesPlayed(state)
        const maxStreak = selectMaxStreak(state)
        const currentStreak = selectCurrentStreak(state)
        const guesses = selectGuesses(state)
        const gamesWon = selectGamesWon(state)

        trackImpression(
          'wordle',
          whereItHappened,
          [
            `p: ${gamesPlayed}`,
            `w: ${gamesWon}`,
            `cs: ${currentStreak}`,
            `ms: ${maxStreak}`,
            `1: ${guesses[1]}`,
            `2: ${guesses[2]}`,
            `3: ${guesses[3]}`,
            `4: ${guesses[4]}`,
            `5: ${guesses[5]}`,
            `6: ${guesses[6]}`,
          ],
          id,
        )

        if (isGameComplete) {
          // GAMES_wordlebotSBCTA_0822 test analytics
          if (abra.getVariant('GAMES_wordlebotSBCTA_0822')) {
            abra.reportExposure('GAMES_wordlebotSBCTA_0822')
          }
        }

        break
      }

      case TRACK_AUTH_CLICK: {
        const { label, sendContext, element } = action.payload
        const context = sendContext ? id : null
        const data = element
          ? {
              name: '',
              label,
              element,
              useBeacon: true,
              context,
            }
          : {
              name: 'wordle',
              label,
              useBeacon: true,
              context,
            }
        trackClick(data)
        break
      }

      case TRACK_AUTH_IMPRESSION: {
        const { name, region, label } = action.payload
        trackImpression(name, region, label)
        break
      }

      default:
        break
    }

    return returnValue
  }

export default analyticsTracking
