import type { WordleGameData } from '@nyt/games-schema'

import type { WordleStoreState, WordleThunk } from '../types'
import {
  selectIsFirstMoogleSave,
  selectMoogleGameData,
  selectMoogleTimestamp,
  selectMoogleUser,
} from '../api/api.selectors'

import {
  WordleMoogleState,
  getDefaultInitialState,
  getLocalInitialState,
  getMoogleLocalStorageKey,
  writeToLocalStorage,
} from './moogle.helpers'

export const SET_INITIAL_STATE = 'wordle/moogle/SET_INITIAL_STATE'

type SetInitialStateAction = {
  type: typeof SET_INITIAL_STATE
  payload: WordleStoreState['persist']
}

export type WordleMoogleAction = SetInitialStateAction

export const setInitialState = (
  initialState: WordleStoreState['persist'],
): SetInitialStateAction => ({
  type: SET_INITIAL_STATE,
  payload: initialState,
})

/**
 * If a user has opted into Moogle but hasn't synced before
 * we should copy ANON localstorage data to their regi ID localstorage,
 * making sure to convert from original localstorage state if necessary.
 *
 * We should only do this once, on a user's first sync to Moogle, to avoid
 * accidentally overwriting a user's progress with ANON localstorage data
 * on later revisits.
 */
export const setRegiLocalState =
  (): WordleThunk<WordleMoogleState | null> =>
  (dispatch, getState) => {
    const state = getState()
    const isFirstSave = selectIsFirstMoogleSave(state)
    const moogleUser = selectMoogleUser(state)

    if (isFirstSave) {
      const moogleKey = getMoogleLocalStorageKey(moogleUser)
      const anonKey = getMoogleLocalStorageKey()
      const anonLocalState =
        getLocalInitialState() || getDefaultInitialState()

      // we'll store a timestamp on both anon and regi state,
      // used by the Stats Selection Tool trigger logic
      anonLocalState.stats.autoOptInTimestamp = Date.now()

      writeToLocalStorage(moogleKey, anonLocalState)
      writeToLocalStorage(anonKey, anonLocalState)

      // also return the anon state, to account for potential edge case with full local storage
      return anonLocalState
    }

    return null
  }

/**
 * Loads the initial state from either Moogle or localStorage into redux.
 * Responsible for all within the top level "persist" key on redux state.
 *
 * @param {initialStateForFirstSave} - to handle an edge case with full local storage, fall back to the value returned by setRegiLocalState
 */
export const loadInitialState =
  (initialStateForFirstSave: WordleMoogleState | null): WordleThunk =>
  (dispatch, getState) => {
    const state = getState()
    const moogleGameData = selectMoogleGameData(state)
    const moogleUser = selectMoogleUser(state)
    const moogleTimestamp = selectMoogleTimestamp(state) || 0

    // get local initial state for a regi user, if there is one and they are opted in
    const newLocalState =
      getLocalInitialState(moogleUser) || initialStateForFirstSave
    const localStorageTimestamp = newLocalState?.timestamp || 0

    /**
     * If a moogle timestamp exists, compare it to the localstorage timestamp.
     * If it is more recent, or a localstorage timestamp doesn't exist,
     * prefer Moogle data
     */
    if (
      !!moogleTimestamp &&
      moogleTimestamp >= localStorageTimestamp
    ) {
      const wordleGameData = moogleGameData as WordleGameData
      return dispatch(setInitialState(wordleGameData))
    }

    /**
     * If moogle sync is not enabled, if a moogle timestamp doesn't exist,
     * or if the localStorage timestamp is newer, prefer localStorage data
     */
    if (newLocalState) return dispatch(setInitialState(newLocalState))

    /**
     * If the new format of initial state in localStorage doesn't exist,
     * we need to construct it from a set of defaults. This should
     * happen on a browser until a save action is taken, because then state
     * is saved in both formats
     */
    return dispatch(setInitialState(getDefaultInitialState()))
  }
