import type { WordleGameData } from '@nyt/games-schema'

import { captureSentryError } from 'foundation'

import { initialStatsState } from '../stats/stats.state'
import { NUM_ROWS } from '../game/game.constants'

const MOOGLE_KEY = 'nyt-wordle-moogle'
export const DARK_MODE_KEY = 'nyt-wordle-darkmode' // keep in sync with server-side wordleInitialDarkMode.js
export const COLORBLIND_MODE_KEY = 'nyt-wordle-cbmode'

export const getMoogleLocalStorageKey = (regiId?: number) => {
  return `${MOOGLE_KEY}/${regiId || 'ANON'}`
}

/**
 * WordleGameData types are generated from our games-schema repo, based on Moogle data structure.
 *
 * IMPORTANT: the @nyt/games-schema package version reflects which schema version we're importing.
 * If these types fall out of sync, be sure to re-install the games-schema package using the most recent version.
 */
export type WordleMoogleState = WordleGameData & {
  timestamp: number
  // if schema version is missing from local storage, its because we weren't writing schemaVersion there when it was 0.0.0
  schemaVersion?: string
}

export const getFromLocalStorage = <T>(
  key: string,
  defaultValue: T | null = null,
): T | null => {
  try {
    const storedString = window.localStorage.getItem(key)
    if (!storedString) return defaultValue

    // TODO: safety around this parse? making an assumption here
    return JSON.parse(storedString) as T
  } catch (e) {
    console.error(e)
    captureSentryError(e as Error)
    return defaultValue
  }
}

// what if this fails because storage is full?
export const writeToLocalStorage = <T>(key: string, value: T) => {
  try {
    window.localStorage.setItem(key, JSON.stringify(value))
    return true
  } catch (e) {
    console.error(e)
    captureSentryError(e as Error)
    return false
  }
}

export const getLocalInitialState = (
  regiId?: number,
): WordleMoogleState | null => {
  try {
    const moogleKey = getMoogleLocalStorageKey(regiId)

    const localMoogleState =
      getFromLocalStorage<WordleMoogleState>(moogleKey)

    // note: this might become nontrivial
    return localMoogleState
  } catch (e) {
    return null
  }
}

export const getDefaultInitialState = (): WordleMoogleState => {
  // until preference is set, use browser setting for dark mode
  const systemDarkMode =
    !!window.matchMedia &&
    window.matchMedia('(prefers-color-scheme: dark)').matches

  return {
    game: {
      id: null,
      dayOffset: null,
      boardState: Array(NUM_ROWS).fill(''),
      currentRowIndex: 0,
      status: 'IN_PROGRESS',
      timestamps: {
        lastPlayed: null,
        lastCompleted: null,
      },
    },
    settings: {
      hardMode: false,
      darkMode: systemDarkMode,
      colorblindMode: false,
    },
    stats: initialStatsState,
    timestamp: 0, // should get overwritten pretty much immediately with a real timestamp, by moogle.middleware
  }
}

const getItemOrDefault = (key: string, defaultValue: string) => {
  try {
    return localStorage.getItem(key) || defaultValue
  } catch (e) {
    return defaultValue
  }
}

type MoogleFeedbackData = {
  anon: string
  regi: string
}

export const getMoogleDataForFeedback = (
  regiId: number | undefined,
): MoogleFeedbackData => {
  const anonKey = 'nyt-wordle-moogle/ANON'
  const regiKey = `nyt-wordle-moogle/${regiId}`

  const anon = getItemOrDefault(anonKey, '[NONE]')
  const regi = getItemOrDefault(regiKey, '[NONE]')

  return {
    anon,
    regi,
  }
}
