import debounce from 'lodash-es/debounce'

import { WordleAction, WordleMiddleware } from '../types'
import { CHANGE_SETTING } from '../settings/settings.actions'
import {
  CONTINUE_GAME,
  EVALUATE_ROW,
  START_GAME,
} from '../game/game.actions'
import {
  MAKE_STATS_SELECTION,
  updateMoogle,
} from '../api/api.actions'
import {
  selectIsFirstMoogleSave,
  selectMoogleUser,
  selectMooglePostUnsavedData,
} from '../api/api.selectors'
import { getDayOffsetForDate } from '../../utils/gamelogic'

import {
  getMoogleLocalStorageKey,
  writeToLocalStorage,
  WordleMoogleState,
} from './moogle.helpers'

/**
 * If a user is on a stale tab or is playing past midnight, we don't
 * save to either local or remote storage. This should fix the issue where
 * the user would be locked out of playing the next day's puzzle.
 */
const isStaleData = (moogleState: WordleMoogleState) => {
  const {
    game: { dayOffset, id },
    timestamp,
  } = moogleState

  // never save if the puzzle hasn't loaded successfully
  if (!id) {
    return true
  }

  const savingDayOffset = getDayOffsetForDate(
    new Date(timestamp * 1000),
  )

  return dayOffset && dayOffset < savingDayOffset
}

/**
 * Certain actions save the "persist" key of the redux store
 * to localStorage and to Moogle, if a regi user has enabled that behavior.
 *
 * For now, we must support both formats of localStorage (see moogle.actions),
 * meaning each save is saving to both places.
 *
 * Once one save happens, as long as we don't roll back this code, the
 * new format is what's loaded. However we can't predict whether or not
 * a roll back will happen, thus we must keep the old format intact for now.
 */
export const save: WordleMiddleware = (store) => {
  const debouncedSaveToBackend = debounce(
    (state: WordleMoogleState) => store.dispatch(updateMoogle(state)),
    1200, // must wait at least 1 second, since we cannot send 2 states w/ the same timestamp
  )

  window.addEventListener('online', () => {
    const unsavedData = selectMooglePostUnsavedData(store.getState())
    if (unsavedData) {
      debouncedSaveToBackend(unsavedData)
    }
  })

  return (next) => (action: WordleAction) => {
    const returnValue = next(action)
    const state = store.getState()
    const moogleUser = selectMoogleUser(state)
    const isFirstSave = selectIsFirstMoogleSave(state)
    const timestamp = Math.floor(Date.now() / 1000) // Moogle expects seconds, not milliseconds
    const moogleState: WordleMoogleState = {
      ...state.persist,
      timestamp,
      schemaVersion: '0.13.1', // TODO: @nyt/games-schema should export this
    }

    const moogleKey = getMoogleLocalStorageKey(moogleUser)

    const moogleSyncActions = [
      CHANGE_SETTING,
      EVALUATE_ROW,
      START_GAME,
      MAKE_STATS_SELECTION,
    ]

    if (isStaleData(moogleState)) {
      return returnValue
    }

    // save to both local storage and backend
    if (moogleSyncActions.includes(action.type)) {
      writeToLocalStorage(moogleKey, moogleState)
      debouncedSaveToBackend(moogleState)
    }

    // make sure a save happens on page load for newly opted in users
    if (isFirstSave && action.type === CONTINUE_GAME) {
      debouncedSaveToBackend(moogleState)
    }

    return returnValue
  }
}
