import { Reducer } from 'redux'

import type { WordleAction } from '../types'
import { SET_INITIAL_STATE } from '../moogle/moogle.actions'
import { EVALUATE_ROW, START_GAME } from '../game/game.actions'
import { MAKE_STATS_SELECTION } from '../api/api.actions'

import { WordleStatsState, initialStatsState } from './stats.state'

const updateStats = (
  oldStats: WordleStatsState,
  {
    isWin,
    isStreak,
    numGuesses,
  }: {
    isWin: boolean
    isStreak: boolean
    numGuesses: number
  },
): WordleStatsState => {
  // clone stats object so can be mutated
  const stats = {
    ...oldStats,
    hasPlayed: true,
    guesses: { ...oldStats.guesses },
  }

  if (isWin) {
    stats.guesses[numGuesses] += 1

    if (isStreak) {
      stats.currentStreak += 1
    } else {
      stats.currentStreak = 1
    }
  } else {
    stats.currentStreak = 0
    stats.guesses.fail += 1
  }
  stats.maxStreak = Math.max(stats.currentStreak, stats.maxStreak)

  stats.gamesPlayed += 1
  stats.gamesWon += isWin ? 1 : 0
  stats.winPercentage = Math.round(
    (stats.gamesWon / stats.gamesPlayed) * 100,
  )
  if (stats.gamesWon === 0) {
    stats.averageGuesses = 0
  } else {
    stats.averageGuesses = Math.round(
      Object.entries(stats.guesses).reduce<number>(
        (acc, [key, val]) => {
          const new_acc = acc + parseInt(key, 10) * val
          return key !== 'fail' ? new_acc : acc
        },
        0,
      ) / stats.gamesWon,
    )
  }

  return stats
}

const reducer: Reducer<WordleStatsState, WordleAction> = (
  state = initialStatsState,
  action,
) => {
  switch (action.type) {
    case SET_INITIAL_STATE: {
      if (action.payload?.stats) {
        const { stats, game } = action.payload
        return {
          ...stats,
          isOnStreak: stats.isOnStreak ?? false,
          hasPlayed: stats.hasPlayed ?? !!game.timestamps.lastPlayed,
        }
      }

      return state
    }

    case START_GAME: {
      return {
        ...state,
        isOnStreak: action.payload.isOnStreak,
      }
    }

    case EVALUATE_ROW: {
      const { status, isStreak, numGuesses } = action.payload
      if (status === 'IN_PROGRESS') {
        return {
          ...state,
          hasPlayed: true,
        }
      }

      const isWin = status === 'WIN'
      return updateStats(state, { isWin, isStreak, numGuesses })
    }

    case MAKE_STATS_SELECTION: {
      return {
        ...state,
        hasMadeStatsChoice: true,
      }
    }

    default:
      return state
  }
}

export default reducer
