import { createSelector } from 'reselect'

import { getDifferenceInDays } from '../../utils/gamelogic'
import { NUM_ROWS } from '../game/game.constants'
import {
  selectCurrentRowIndex,
  selectGameDayOffset,
  selectEvaluations,
  selectGameStatus,
  selectLastPlayed,
  selectLastCompleted,
} from '../game/game.selectors'
import {
  selectColorblindMode,
  selectDarkMode,
  selectHardMode,
} from '../settings/settings.selectors'
import type { WordleStoreState } from '../types'

export const selectWordleStats = (state: WordleStoreState) =>
  state.persist.stats

export const selectHasPlayed = createSelector(
  [selectWordleStats, selectLastPlayed],
  (stats, lastPlayed) => stats.hasPlayed || !!lastPlayed,
)

const selectIsOnStreak = createSelector(
  selectWordleStats,
  (stats) => !!stats.isOnStreak,
)

export const selectGamesPlayed = createSelector(
  selectWordleStats,
  (stats) => stats.gamesPlayed,
)

export const selectGamesWon = createSelector(
  selectWordleStats,
  (stats) => stats.gamesWon,
)

export const selectMaxStreak = createSelector(
  selectWordleStats,
  (stats) => stats.maxStreak,
)

export const selectCurrentStreak = createSelector(
  selectWordleStats,
  (stats) => stats.currentStreak,
)

export const selectGuesses = createSelector(
  selectWordleStats,
  (stats) => stats.guesses,
)

// returns a function that takes a date and determines if user is on a streak
// this will be removed with XWD-17383, and replaced with selectIsOnStreak
export const selectGetWonYesterday = createSelector(
  [selectIsOnStreak, selectLastCompleted],
  (isOnStreak, lastCompleted) => {
    return (now = new Date()) => {
      if (isOnStreak) {
        return true
      }

      return (
        !!lastCompleted &&
        getDifferenceInDays(new Date(lastCompleted), now) === 1
      )
    }
  },
)

const getAbsentTile = (darkMode: boolean) => (darkMode ? '⬛' : '⬜')

const getCorrectTile = (colorblindMode: boolean) =>
  colorblindMode ? '🟧' : '🟩'

const getPresentTile = (colorblindMode: boolean) =>
  colorblindMode ? '🟦' : '🟨'

export const selectShareMessage = createSelector(
  [
    selectEvaluations,
    selectGameDayOffset,
    selectCurrentRowIndex,
    selectHardMode,
    selectGameStatus,
    selectDarkMode,
    selectColorblindMode,
  ],
  (
    evaluations,
    dayOffset,
    rowIndex,
    isHardMode,
    status,
    darkMode,
    colorblindMode,
  ) => {
    let title = `Wordle ${dayOffset}`

    title += ` ${status === 'WIN' ? rowIndex : 'X'}/${NUM_ROWS}`

    if (isHardMode) {
      title += '*'
    }

    let grid = ''

    evaluations.forEach((row) => {
      if (!row) return
      row.forEach((evaluation) => {
        if (!evaluation) return
        let evaluationString = ''
        switch (evaluation) {
          case 'correct': {
            evaluationString = getCorrectTile(colorblindMode)
            break
          }

          case 'present': {
            evaluationString = getPresentTile(colorblindMode)
            break
          }

          case 'absent': {
            evaluationString = getAbsentTile(darkMode)
            break
          }
          default:
            break
        }
        grid += evaluationString
      })
      grid += '\n'
    })

    // trim the grid to remove the last whitespace that was added
    const message = `${title}\n\n${grid.trim()}`

    return {
      text: message,
    }
  },
)
