import { validWords } from '../words/validWords'
import { deprecatedSolutionWords } from '../words/deprecated/deprecatedSolutionWords'

export type Evaluation = 'unknown' | 'absent' | 'present' | 'correct'

export function evaluateGuess(
  guess: string,
  solution: string,
): Evaluation[] {
  // Start by assuming all tiles are absent;
  const evaluation: Evaluation[] = Array(solution.length).fill(
    'absent',
  )
  const availableGuessLetters = Array(solution.length).fill(true)
  const availableSolutionLetters = Array(solution.length).fill(true)

  // CORRECT takes precedent over PRESENT, so we need to iterate all the way through and check for them first
  for (let i = 0; i < guess.length; i += 1) {
    const guessLetter = guess[i]
    const solutionLetter = solution[i]
    if (
      guessLetter === solutionLetter &&
      availableSolutionLetters[i]
    ) {
      evaluation[i] = 'correct'
      availableGuessLetters[i] = false
      availableSolutionLetters[i] = false
    }
  }

  for (let i = 0; i < guess.length; i += 1) {
    const guessLetter = guess[i]
    if (availableGuessLetters[i]) {
      for (let j = 0; j < solution.length; j += 1) {
        const solutionLetter = solution[j]
        if (availableSolutionLetters[j]) {
          if (guessLetter === solutionLetter) {
            evaluation[i] = 'present'
            availableSolutionLetters[j] = false
            break
          }
        }
      }
    }
  }

  return evaluation
}

// We use this to compare
const evaluationPrecedence: Record<Evaluation, number> = {
  unknown: 0,
  absent: 1,
  present: 2,
  correct: 3,
}

// Used for displaying keyboard letter states
// Take an set of evaluation and convert them to a map of letters -> evaluations
// Because letters can occur multiple times in one evaluation, we need to use precendence to
// ensure the high
export function getLetterEvaluations(
  boardState: string[],
  evaluations: Array<Array<Evaluation> | null>,
) {
  const letterEvaluations: Record<string, Evaluation> = {}

  boardState.forEach((row, rowIndex) => {
    const rowEvaluation = evaluations[rowIndex]
    if (!rowEvaluation) return

    for (let i = 0; i < row.length; i += 1) {
      const letter = row[i]
      const evaluation = rowEvaluation[i]

      const existingEvaluation =
        letterEvaluations[letter] || 'unknown'

      if (
        evaluationPrecedence[evaluation] >
        evaluationPrecedence[existingEvaluation]
      ) {
        letterEvaluations[letter] = evaluation
      }
    }
  })

  return letterEvaluations
}

export function ordinal(n: number) {
  const s = ['th', 'st', 'nd', 'rd']
  const v = n % 100
  return n + (s[(v - 20) % 10] || s[v] || s[0])
}

export function checkGuessHardMode(
  guess: string,
  previousRowGuess: string,
  previousRowEvaluation: Evaluation[] | null,
): { validGuess: boolean; errorMessage?: string } {
  // If something is missing, assume we're good.
  if (!guess || !previousRowGuess || !previousRowEvaluation) {
    return { validGuess: true }
  }

  // Check corrects
  for (let i = 0; i < previousRowEvaluation.length; i += 1) {
    if (previousRowEvaluation[i] === 'correct') {
      if (guess[i] !== previousRowGuess[i]) {
        // Didn't use a correct letter
        return {
          validGuess: false,
          errorMessage: `${ordinal(
            i + 1,
          )} letter must be ${previousRowGuess[i].toUpperCase()}`,
        }
      }
    }
  }
  // Check if all presents and corrects appear in guess
  const previousOccuranceCount: { [name: string]: number } = {}
  for (let i = 0; i < previousRowEvaluation.length; i += 1) {
    if (['correct', 'present'].includes(previousRowEvaluation[i])) {
      previousOccuranceCount[previousRowGuess[i]]
        ? (previousOccuranceCount[previousRowGuess[i]] += 1)
        : (previousOccuranceCount[previousRowGuess[i]] = 1)
    }
  }

  const guessOccuranceCount = guess
    .split('')
    .reduce<Record<string, number>>((acc, item) => {
      acc[item] ? (acc[item] += 1) : (acc[item] = 1)
      return acc
    }, {})

  const previousOccurances = Object.keys(previousOccuranceCount)

  for (let i = 0; i < previousOccurances.length; i += 1) {
    const letter = previousOccurances[i]

    if (
      (guessOccuranceCount[letter] || 0) <
      previousOccuranceCount[letter]
    ) {
      return {
        validGuess: false,
        errorMessage: `Guess must contain ${letter.toUpperCase()}`,
      }
    }
  }

  return { validGuess: true }
}

export function validateWord(word: string, solution: string) {
  return word === solution || validWords.includes(word)
}

// UTC 11:59:59pm vs 12:00:01am = 1 day difference
// the difference is dependent on the timezone the question is being asked from
// in UTC, these occured on different dates
// in PST these occured on the same day
// the time in millis between the values has not changed

// Store lastplayed as a ts
// when game loads, derive local date from Date.now()
// derive local date from lastPlayed
// if the two are different, you can play
// if the difference in days is 1, it counts towards streak

const ONE_DAY_EXACTLY_MS = 1000 * 60 * 60 * 24
// const LAUNCH_DATE = new Date('2021-06-19T00:00:00');

// Initializing with arguments to try and minimize date string
// parsing weirdness across browsers. Note month is 0 indexed.
// https://blog.lftechnology.com/date-ing-javascript-6203650b752c
const LAUNCH_DATE = new Date(2021, 5, 19, 0, 0, 0, 0)

export function getDifferenceInDays(startDate: Date, endDate: Date) {
  // TODO: better names for this?
  // We're gonna modify the date object so create new ones just in case
  const finalStartDate = new Date(startDate)
  const finalEndDate = new Date(endDate)
  // setHours returns MS since epoch
  const msDiff =
    finalEndDate.setHours(0, 0, 0, 0) -
    finalStartDate.setHours(0, 0, 0, 0)
  // Rounding is required here due to issues where the LAUNCH_DATE and endDate
  // are in different timezones. E.g. BST and GMT. The difference in MS no
  // longer cleanly divides. TBH, I barely understand this.
  const dayDiff = Math.round(msDiff / ONE_DAY_EXACTLY_MS)

  // console.log({ startDate: _startDate, endDate: _endDate, msDiff, dayDiff });
  return dayDiff
}

// deprecated, only used in moogle, please use selectApiDayOffset
export function getDayOffsetForDate(date: Date) {
  return getDifferenceInDays(LAUNCH_DATE, date)
}

export function getDateForDayOffset(dayOffset: number) {
  const date = new Date(LAUNCH_DATE.valueOf())
  date.setDate(date.getDate() + dayOffset)
  return date
}

// deprecated, please use selectSolution
export function getSolutionByOffset(offset: number | null) {
  if (offset === null) return ''
  const wrappedIndex = offset % deprecatedSolutionWords.length
  const solution = deprecatedSolutionWords[wrappedIndex]
  return solution
}

export const ALPHABET = 'abcdefghijklmnopqrstuvwxyz'
