const express = require('express')

const { MAGNOLIA_DAM_ASSET_PATH } = require('../constants')

const svc = require('./services')
const mw = require('./middleware')
const createPageProps = require('./util/createPageProps')
const createPageErrorProps = require('./util/createPageErrorProps')
const is404 = require('./util/archiveRouteGuard')

const router = express.Router()

// ------------
// CROSSWORD
// ------------

const crosswordRoutes = [
  {
    route: '/game/:stream(mini)/tips-and-tricks/:title',
    gameData: svc.getCrossword.byTutorial,
  },
  {
    route: '/game/:stream(daily|mini)',
    gameData: svc.getCrossword.byStream,
  },
  {
    route: '/game/:stream(daily|mini|bonus)/:yyyy/:m/:d?',
    gameData: svc.getCrossword.byDate,
  },
  {
    route: '/game/by-id/:id',
    gameData: svc.getCrossword.byId,
  },
  {
    route: '/game/:type(paid|special)/:slug',
    gameData: svc.getCrossword.bySlug,
  },
]

crosswordRoutes.forEach(({ route, gameData, metaUrl }) => {
  router.get(
    route,
    mw.injectMetadata(({ _parsedUrl, params }) => {
      const ogTitle =
        params.stream === 'mini'
          ? 'Play The Mini Crossword'
          : 'Play The Crossword'

      const ogDescription =
        params.stream === 'mini'
          ? 'All the fun of the larger New York Times Crossword, but you can solve it in seconds.'
          : 'Keep your mind sharp by making The Crossword part of your daily routine.'

      const ogImage =
        params.stream === 'mini' ? 'mini.png' : 'crossword.png'
      const ogImageUrl = `${MAGNOLIA_DAM_ASSET_PATH}${ogImage}`
      const { pathname } = _parsedUrl

      return {
        title: 'The New York Times Crossword',
        trackingPageType: 'puzzle | crossword',
        metaTags: [
          {
            property: 'og:title',
            content: ogTitle,
          },
          {
            property: 'og:url',
            content:
              metaUrl ||
              `https://www.nytimes.com/crosswords${pathname}`,
          },
          {
            property: 'og:description',
            content: ogDescription,
          },
          {
            property: 'og:image',
            content: ogImageUrl,
          },
          {
            property: 'og:type',
            content: 'website',
          },
          {
            name: 'CG',
            content: 'Crosswords/Games',
          },
          {
            name: 'PT',
            content: `puzzle | ${params.stream}`,
          },
        ],
      }
    }),
    mw.renderView(
      'Crossword',
      createPageProps({
        pageName: 'game-page',
        adUnitPath: 'crossword',
        entry: 'crossword',
        gameData,
      }),
    ),
    mw.renderError(
      'ErrorPage',
      createPageErrorProps({
        liraAsset: 'crossword',
      }),
    ),
  )
})

// -----------------------
// CROSSWORD ARCHIVE PAGES
// -----------------------

router.get(
  '/archive/:type?/:year?/:month?',
  mw.restrictAccess((req) => {
    if (is404(req.params)) {
      return 404
    }
  }),
  mw.injectMetadata(() => ({
    title: 'Puzzle Archive - The New York Times',
    trackingPageType: 'archive | daily',
    metaTags: [
      { name: 'robots', content: 'noindex' },
      { name: 'app', content: 'games-phoenix' },
    ],
  })),
  mw.renderView(
    'CrosswordArchive',
    createPageProps({
      pageName: 'archive',
      adUnitPath: 'archive',
      entry: 'crossword-archive',
    }),
  ),
  // TODO: mw.renderError
)

// --------
// HUB PAGE
// --------
router.get(
  '/',
  mw.renderView(
    'Hub',
    createPageProps({
      pageName: 'hub',
      metadata: {
        title: 'The Crossword',
        contentGroup: 'Crosswords/Games',
        pageType: 'puzzle hub',
        metaTags: [
          {
            property: 'og:title',
            content: 'Subscribe to New York Times Games',
          },
          {
            property: 'og:url',
            content: 'https://www.nytimes.com/crosswords',
          },
          {
            property: 'og:description',
            content:
              'Upgrade your downtime with full access to The Crossword, The Mini, Spelling Bee and more.',
          },
          {
            property: 'og:image',
            content: `${MAGNOLIA_DAM_ASSET_PATH}hub.png`,
          },
          {
            name: 'description',
            itemProp: 'description',
            content: `
        Play the Daily New York Times Crossword puzzle edited by Will Shortz online. Try
        free NYT games like the Mini Crossword, Ken Ken, Sudoku & SET plus our new
        subscriber-only puzzle Spelling Bee.
        `,
          },
        ],
        linkTags: [
          {
            rel: 'canonical',
            href: 'https://www.nytimes.com/crosswords',
          },
        ],
      },
      adUnitPath: 'crosswords',
      entry: 'hub',
      hubPuzzles: svc.getGamesHubPuzzles,
    }),
  ),
)

// --------
// TIPS & TRICKS
// --------

router.get(
  '/tips-and-tricks',
  mw.injectMetadata({
    title: 'Tips & Tricks',
    contentGroup: 'Crosswords/Games',
    pageType: 'puzzle hub',
    metaTags: [
      {
        property: 'og:CG',
        content: 'Crosswords/Games',
      },
      {
        property: 'og:PT',
        content: 'tips and tricks',
      },
    ],
  }),
  mw.renderView(
    'TipsAndTricks',
    createPageProps({
      pageName: 'tipsandtricks',
      adUnitPath: 'tipsandtricks',
      entry: 'tipsandtricks',
      enableClientSideEntitlements: true,
    }),
  ),

  // --------
  // Download
  // --------

  router.get(
    '/apps',
    mw.injectMetadata({
      title: 'Download The NY Times Crossword Apps',
      contentGroup: 'Crosswords/Games',
      pageType: 'puzzle hub',
      metaTags: [
        {
          property: 'og:CG',
          content: 'Crosswords/Games',
        },
        {
          property: 'og:PT',
          content: 'apps',
        },
        {
          property: 'og:description',
          content:
            "Play Wordle, Spelling Bee, Sudoku, The Crossword and more on iOS and Android. You'll find new puzzles daily.",
        },
        {
          property: 'og:title',
          content: 'Download the New York Times Games App',
        },
        {
          property: 'og:url',
          content: 'https://www.nytimes.com/crosswords/apps',
        },
        {
          property: 'og:image',
          content:
            'https://www.nytimes.com/games-assets/v2/assets/apps/Games_Social_Card.png',
        },
        {
          name: 'twitter:card',
          content: 'summary_large_image',
        },
        {
          name: 'twitter:site',
          content: '@NYTGames',
        },
        {
          name: 'twitter:title',
          content: 'Download the New York Times Games App',
        },
        {
          name: 'twitter:description',
          content:
            "Play Wordle, Spelling Bee, Sudoku, The Crossword and more on iOS and Android. You'll find new puzzles daily.",
        },
        {
          name: 'twitter:image',
          content:
            'https://www.nytimes.com/games-assets/v2/assets/apps/Games_Social_Card.png',
        },
      ],
    }),
    mw.renderView(
      'Apps',
      createPageProps({
        pageName: 'apps',
        entry: 'apps',
      }),
    ),
  ),
)

module.exports = router
