/**
 * NOTE: in order to use the same build config for
 * server and client code, we need to include the babel helpers on the server
 */
require('../../vendor/babelHelpers')
require('moment-timezone')

const express = require('express')
const compression = require('compression')
const cookieParser = require('cookie-parser')
const Sentry = require('@sentry/node')

const logger = require('../serverLogger')
const constants = require('../constants')

const puzzlesRouter = require('./puzzlesRouter')
const crosswordsRouter = require('./crosswordsRouter')
const gamesRouter = require('./gamesRouter')
const createPageErrorProps = require('./util/createPageErrorProps')
const { initSentry, isSentryEnabled } = require('./sentry')
const mw = require('./middleware')

const {
  USE_DIST,
  DIST_ROOT,
  PUBLIC_ROOT,
  ROUTE_DIST,
  ROUTE_ASSETS,
  PUZZLES_ROUTE_BASE,
  CROSSWORDS_ROUTE_BASE,
  GAMES_ROUTE_BASE,
  STORYBOOK_ASSETS,
  ROUTE_STORYBOOK,
  IS_PROD,
} = constants

const app = express()

// Enabling Sentry in production and staging
if (isSentryEnabled) {
  initSentry()
  // If enabled, Sentry requestHandler must be the first middleware in the chain
  app.use(Sentry.Handlers.requestHandler())
  app.use(mw.sentryMetaData())
}

app.use(cookieParser())

app.use(compression())
app.use(mw.handleRedirects)

// serve built assets from the dist root and not /games-assets
if (USE_DIST) {
  logger.info('serving static assets')
  app.use(ROUTE_DIST, mw.serveStaticAssets(DIST_ROOT))
  app.use(ROUTE_ASSETS, mw.serveStaticAssets(PUBLIC_ROOT))
}

if (!IS_PROD) {
  logger.info('serving storybook')
  app.use(ROUTE_STORYBOOK, mw.serveStaticAssets(STORYBOOK_ASSETS))
}

// shared middleware
app.use(mw.standardizeAuthCookie)
app.use(mw.verifyAnonSubs)
app.use(mw.injectUserType)
app.use(mw.injectDeviceType)
app.use(mw.injectFeatureFlags)
app.use(mw.injectCompliance)
app.use(mw.getSubInfo)
app.use(mw.injectApi)
app.use(mw.disableKeepAlive)
app.use(mw.httpLogger)

// use router
app.use(PUZZLES_ROUTE_BASE, puzzlesRouter)
app.use(CROSSWORDS_ROUTE_BASE, crosswordsRouter)
app.use(GAMES_ROUTE_BASE, gamesRouter)

// health check for deployment
app.get('/healthz', (req, res) => res.status(200).send('ok'))

// error handling
if (isSentryEnabled) {
  app.use(Sentry.Handlers.errorHandler())
}
app.use(mw.renderError('ErrorPage', createPageErrorProps()))
app.use(mw.staticErrorHandler)

module.exports = app
