const axios = require('axios')
const request = require('supertest')
const express = require('express')
const cookieParser = require('cookie-parser')

const { getSubInfo } = require('..')

jest.mock('axios')
jest.mock('../../services/getShortz', () => true)

describe('APP/MIDDLEWARE: GET SUB INFO', () => {
  let app

  beforeEach(() => {
    // Stub a fake app w/ our middleware and return
    // the locals object in the response body so we
    // can write assertions against them
    app = express()
    app.use(cookieParser())
    app.use(getSubInfo)
    app.get('*', (req, res) => {
      res.send(res.locals)
    })
  })

  it('should set userType.hasHd to false in response locals if we do not know anything', () => {
    return request(app)
      .get('/some-page')
      .then((response) => {
        expect(response.body).toEqual({
          userType: { hasHd: false },
        })
      })
  })

  it('should not call subInfo if we do not know anything', () => {
    axios.mockResolvedValue({})

    return request(app)
      .get('/some-page')
      .then(() => {
        expect(axios).toHaveBeenCalledTimes(0)
      })
  })

  it('should not call subInfo if the nyts-regiid is empty', () => {
    return request(app)
      .get('/some-page')
      .set('nyts-regiid', '')
      .then(() => {
        expect(axios).toHaveBeenCalledTimes(0)
      })
  })

  it('should not call subInfo if the request is to Wordle', () => {
    axios.mockResolvedValue({})

    return request(app)
      .get('/games/wordle/index.html')
      .then(() => {
        expect(axios).toHaveBeenCalledTimes(0)
      })
  })

  it('should set userType.hasHd to true in response locals if subInfo fetch returns a valid HD bundle', () => {
    axios.mockResolvedValue({
      data: [{ status: 'ACTIVE', bundleType: 'E' }],
    })

    return request(app)
      .get('/some-page')
      .set('Cookie', ['NYT-S=123.abc.123'])
      .set('nyts-regiid', 123456)
      .then((response) => {
        expect(response.body).toEqual({
          userType: { hasHd: true },
        })
      })
  })

  it('should set userType.hasHd to false in response locals if subInfo fetch returns an invalid HD bundle', () => {
    axios.mockResolvedValue({
      data: [{ status: 'ACTIVE', bundleType: 'NOT_AN_HD_BUNDLE' }],
    })

    return request(app)
      .get('/some-page')
      .set('Cookie', ['NYT-S=123.abc.123'])
      .set('nyts-regiid', 123456)
      .then((response) => {
        expect(response.body).toEqual({
          userType: { hasHd: false },
        })
      })
  })
})
