const request = require('supertest')
const express = require('express')
const MobileDetect = require('mobile-detect')
const platform = require('platform')
const deIndent = require('de-indent')

const { injectDeviceType } = require('..')

jest.mock('../renderView', () => ({}))
jest.mock('mobile-detect', () => jest.fn())
jest.mock('platform', () => ({ parse: jest.fn() }))
jest.mock('../../services/getShortz', () => true)

describe('APP/MIDDLEWARE: INJECT DEVICE TYPE', () => {
  let isMobile
  let isPhone
  let isTablet
  let app

  beforeEach(() => {
    isMobile = jest.fn()
    isPhone = jest.fn()
    isTablet = jest.fn()

    MobileDetect.mockImplementation(function MD() {
      this.mobile = () => isMobile()
      this.phone = () => isPhone()
      this.tablet = () => isTablet()
    })

    platform.parse.mockReturnValue({
      name: 'device-name',
      version: 'device-version',
      os: {
        family: 'os-family',
        version: 'os-version',
      },
    })

    app = express()
    app.use(injectDeviceType)
    app.get('*', (req, res) => {
      res.send(res.locals)
    })
  })

  it('adds device type to `res.locals`', () => {
    expect.assertions(1)

    return request(app)
      .get('/page')
      .then((r) => {
        expect(r.body.deviceType).toEqual({
          isMobile: false,
          isPhone: false,
          isTablet: false,
          newsreaderAppPlatform: '',
          newsreaderAppVersion: '',
          gamesAppPlatform: '',
          gamesAppVersion: '',
        })
      })
  })

  it('detects mobile', () => {
    expect.assertions(1)
    isMobile.mockReturnValue(true)

    return request(app)
      .get('/page')
      .then((r) => {
        expect(r.body.deviceType).toMatchObject({
          isMobile: true,
        })
      })
  })

  it('detects mobile from iOS Newsreader UA', () => {
    return request(app)
      .get('/page')
      .set(
        'User-agent',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.33.0',
      )
      .then((r) => {
        expect(r.body.deviceType).toMatchObject({
          isMobile: true,
        })
      })
  })

  it('does not call MobileDetect isMobile when we have an iOS Newsreader UA', () => {
    return request(app)
      .get('/page')
      .set(
        'User-agent',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.33.0',
      )
      .then(() => {
        expect(isMobile).toHaveBeenCalledTimes(0)
      })
  })

  it('detects mobile from Android Newsreader UA', () => {
    return request(app)
      .get('/page')
      .set('User-agent', '...nyt_android/9.13...')
      .then((r) => {
        expect(r.body.deviceType).toMatchObject({
          isMobile: true,
        })
      })
  })

  it('detects mobile from iOS Crossword UA', () => {
    return request(app)
      .get('/page')
      .set('User-agent', '...nyt_xwords_ios/4.10')
      .then((r) => {
        expect(r.body.deviceType).toMatchObject({
          isMobile: true,
        })
      })
  })

  it('detects mobile from Android Crossword UA', () => {
    return request(app)
      .get('/page')
      .set('User-agent', '...Crosswords/4.10')
      .then((r) => {
        expect(r.body.deviceType).toMatchObject({
          isMobile: true,
        })
      })
  })

  it('detects phones', () => {
    expect.assertions(1)
    isPhone.mockReturnValue(true)

    return request(app)
      .get('/page')
      .then((r) => {
        expect(r.body.deviceType).toMatchObject({
          isPhone: true,
        })
      })
  })

  it('detects tablets', () => {
    expect.assertions(1)
    isTablet.mockReturnValue(true)

    return request(app)
      .get('/page')
      .then((r) => {
        expect(r.body.deviceType).toMatchObject({
          isTablet: true,
        })
      })
  })

  it('injects the unique device information', () => {
    expect.assertions(1)

    return request(app)
      .get('/page')
      .then((r) => {
        expect(r.body.device).toMatchObject({
          name: 'device-name',
          version: 'device-version',
          os: {
            family: 'os-family',
            version: 'os-version',
          },
          feedback: deIndent(`\r\n\r\n
            ----
            Device summary:
            Page: %s
            Platform: Web (Desktop)
            Browser: device-name device-version
            OS: os-family os-version
            Screen Resolution: %s x %s
            Viewport Size: %s x %s
            Timezone: %s
            Regi: %s
            Digi: %s
            Xwd: %s
            Hd: %s
            Page View: %s
          `),
        })
      })
  })

  it('identifies mobile devices in the feedback template', () => {
    expect.assertions(1)
    isMobile.mockReturnValue(true)

    return request(app)
      .get('/page')
      .then((r) => {
        expect(r.body.device.feedback).toContain(
          'Platform: Web (Mobile)',
        )
      })
  })

  it('identifies the newsreader app in the feedback template', () => {
    expect.assertions(1)

    return request(app)
      .get('/page')
      .set('User-agent', '...nyt_android/9.13...')
      .then((r) => {
        expect(r.body.device.feedback).toContain(
          'Platform: Web (News App)',
        )
      })
  })
})
