/**
 * This is necessary to get the correct Puzzle People URL
 * @jest-environment node
 */

const request = require('supertest')
const express = require('express')

const { injectUserType } = require('..')

jest.mock('../renderView', () => ({}))
jest.mock(
  '../../services/getShortz',
  () => (regi) => regi === 2054201,
)

describe('APP/MIDDLEWARE: INJECT USER TYPE', () => {
  let app

  beforeEach(() => {
    app = express()
    app.use(injectUserType)
    app.get('*', (req, res) => {
      res.send(res.locals)
    })
  })

  it('supports anonymous users', () => {
    expect.assertions(1)

    return request(app)
      .get('/page')
      .then((r) => {
        expect(r.body).toEqual({
          user: {},
          userType: {
            entitlement: 'anon',
            hasDigi: false,
            hasHd: false,
            hasXwd: false,
            inShortzMode: false,
            isErsatzShortz: false,
            isLoggedIn: false,
          },
        })
      })
  })

  it('supports logged in users (without crosswords subscription)', () => {
    expect.assertions(1)

    return request(app)
      .get('/page')
      .set('nyts-regiid', '123')
      .set('nyts-username', 'user-name')
      .then((r) => {
        expect(r.body).toEqual({
          user: {
            displayName: 'user-name',
            id: 123,
          },
          userType: {
            entitlement: 'reg',
            hasDigi: false,
            hasHd: false,
            hasXwd: false,
            inShortzMode: false,
            isErsatzShortz: false,
            isLoggedIn: true,
            regiId: 123,
          },
        })
      })
  })

  it('detects users with crosswords subscription', () => {
    expect.assertions(1)

    return request(app)
      .get('/page')
      .set('nyts-regiid', '123')
      .set('nyts-xwd', 'true')
      .then((r) => {
        expect(r.body).toMatchObject({
          userType: {
            hasXwd: true,
            entitlement: 'cr',
          },
        })
      })
  })

  it('detects users with digital subscription', () => {
    const subscriptionHeaders = ['nyts-mm', 'nyts-msd', 'nyts-mtd']

    expect.assertions(subscriptionHeaders.length)

    return Promise.all(
      subscriptionHeaders.map((subHeader) =>
        request(app)
          .get('/page')
          .set('nyts-regiid', '123')
          .set(subHeader, 'true')
          .then((r) => {
            expect(r.body).toMatchObject({
              userType: {
                hasDigi: true,
                entitlement: 'sub',
              },
            })
          }),
      ),
    )
  })

  it('detects users with digital and crossword subscription', () => {
    const subscriptionHeaders = ['nyts-mm', 'nyts-msd', 'nyts-mtd']

    expect.assertions(subscriptionHeaders.length)

    return Promise.all(
      subscriptionHeaders.map((subHeader) =>
        request(app)
          .get('/page')
          .set('nyts-xwd', 'true')
          .set('nyts-regiid', '123')
          .set(subHeader, 'true')
          .then((r) => {
            expect(r.body).toMatchObject({
              userType: {
                hasDigi: true,
                entitlement: 'sub,cr',
              },
            })
          }),
      ),
    )
  })

  it('detects if user is on team', () => {
    expect.assertions(1)

    return request(app)
      .get('/page')
      .set('nyts-regiid', '2054201') // Will Shortz
      .then((r) => {
        expect(r.body).toMatchObject({
          userType: {
            isErsatzShortz: true,
            inShortzMode: true,
          },
        })
      })
  })
})
