const request = require('supertest')
const express = require('express')

const { restrictAccess } = require('..')

jest.mock('../renderView', () => ({}))
jest.mock('../../services/getShortz', () => true)

jest.mock('../../util/StatusError', () => {
  return class StatusError extends Error {
    constructor(message, status) {
      super(`${message} (status: ${status})`)
    }
  }
})

describe('APP/MIDDLEWARE: RESTRICT ACCESS', () => {
  it('doesnt restrict if there is no status returned', () => {
    const restriction = restrictAccess(() => {})

    const app = express()
    app.get('*', restriction, (req, res) => res.send('hello'))
    app.use((error, req, res) => {
      res.send(error.message)
    })

    return request(app).get('/page').expect('hello')
  })

  it('doesnt restrict if the status returned is 200', () => {
    const restriction = restrictAccess(() => 200)

    const app = express()
    app.get('*', restriction, (req, res) => res.send('hello'))
    app.use((error, req, res) => {
      res.send(error.message)
    })

    return request(app).get('/page').expect('hello')
  })

  it('supports async blocking', () => {
    const restriction = restrictAccess(() => Promise.resolve(404))

    const app = express()
    app.get('/page/:id', restriction, (req, res) => res.send('hello'))

    /* eslint-disable no-unused-vars */
    app.use((error, req, res, next) => {
      res.send(error.message)
    })

    return request(app)
      .get('/page/a')
      .expect('restricted /page/a (status: 404)')
  })

  it('handles async blocking', () => {
    const restriction = restrictAccess(() =>
      Promise.reject(new Error('uh oh!')),
    )

    const app = express()
    app.get('/page/:id', restriction, (req, res) => res.send('hello'))
    /* eslint-disable no-unused-vars */
    app.use((error, req, res, next) => {
      res.send(error.message)
    })

    return request(app).get('/page/a').expect('uh oh!')
  })

  it('supports blocking based on req data', () => {
    const restriction = restrictAccess((req) => {
      if (req.params.id === 'a') return 404
    })

    const app = express()
    app.get('/page/:id', restriction, (req, res) => res.send('hello'))
    app.use((error, req, res, next) => {
      res.send(error.message)
    })

    return request(app)
      .get('/page/a')
      .expect('restricted /page/a (status: 404)')
  })

  it('supports blocking based on response data', () => {
    const restriction = restrictAccess((req, res) => {
      if (res.locals.isLoggedOut) return 401
    })

    const app = express()
    app.use((req, res, next) => {
      res.locals.isLoggedOut = true
      next()
    })
    app.get('/page/:id', restriction, (req, res) => res.send('hello'))
    app.use((error, req, res, next) => {
      res.send(error.message)
    })

    return request(app)
      .get('/page/a')
      .expect('restricted /page/a (status: 401)')
  })
})
