/**
 * Enables caching via Fastly
 *
 * @param {Object} config
 * @param {number} config.httpCacheTTL
 * @param {number} config.httpCacheStaleIfErrorTTL
 * @param {number} config.staleWhileRevalidateTTL
 * @param {string} config.surrogateKey
 */
module.exports = function cacheInFastly(config = {}) {
  const {
    httpCacheTTL = 300,
    httpCacheStaleIfErrorTTL = 86400,
    staleWhileRevalidateTTL = 60,
    surrogateKey,
  } = config

  return (req, res, next) => {
    // no-cache | don't reuse resource in private caches (e.g., browser) without revalidating with server
    // max-age=0 | workaround for lack of no-cache support in HTTP/1.0
    // s-maxage=300 | cache in shared caches (e.g., fastly) for 5m
    // stale-if-error=86400 | serve stale responses for up to 1d if error
    // stale-while-revalidate=60 | serve stale for up to 60s while fetching updated content from the origin
    if (!res.headersSent) {
      res.set({
        'cache-control': `no-cache,max-age=0,s-maxage=${httpCacheTTL},stale-if-error=${httpCacheStaleIfErrorTTL},stale-while-revalidate=${staleWhileRevalidateTTL}`,
        'surrogate-control': `max-age=${httpCacheTTL},stale-if-error=${httpCacheStaleIfErrorTTL},stale-while-revalidate=${staleWhileRevalidateTTL}`,
        'surrogate-key': `games ${surrogateKey}`,
      })
    }

    next()
  }
}
