const path = require('path')

const deepmerge = require('deepmerge')
const axios = require('axios')
const { GoogleAuth } = require('google-auth-library')

const StatusError = require('../util/StatusError')
const logger = require('../../serverLogger')
const {
  API_HOST_SERVER,
  STATIC_API_ROOT,
  USE_STATIC_API,
  USE_OAUTH,
} = require('../../constants')

/**
 * Our kube.yml configures istio to drop all outbound requests that exceed 9 seconds.
 * Axios by default has no timeout, and we have noticed server-side requests wait indefinitely
 * and block sending an HTTP response to Fastly. This sets a timeout on all server-side GETs
 * so that Phoenix can fail earlier, with a proper status code, and allow Fastly to serve
 * stale content until it recovers.
 *
 * See: https://jira.nyt.net/browse/XWD-22652
 */
const timeout = 9000

/**
 * Once we authorize a google client
 * we shouldn't need to do it again
 */
let isClientAuthorized

if (USE_OAUTH) {
  logger.info(`Using oAuth for API calls`)
}

module.exports = async function injectApi(req, res, next) {
  const userCookie = req.cookies['NYT-S']
  const apiUrl = API_HOST_SERVER

  /**
   * create a getter for the api
   * but decide how it works based on environment
   */
  let get

  if (USE_STATIC_API) {
    /**
     * If desired, a fake api can be used
     * to load data from disk
     * This is useful for local development, but very brittle
     */
    get = (url) => {
      const domainUrl = url.replace('https://www.nytimes.com/', '')

      logger.info(`using static api ${domainUrl}`)

      return Promise.resolve(
        require(path.resolve(STATIC_API_ROOT, domainUrl)),
      )
    }
  } else if (USE_OAUTH) {
    /**
     * In staging, we need to authenticate
     * calls to the api using oauth since we have no fixed ip range
     */

    /**
     * First, we need to establish the oauth client
     * and on subsequent requests we can reuse it
     */
    const auth = new GoogleAuth({
      scopes: ['https://www.googleapis.com/auth/userinfo.email'],
    })

    if (!isClientAuthorized) {
      try {
        const client = await auth.getClient()
        logger.info(`established oAuthClient for ${client.email}`)
        isClientAuthorized = true // we only need to to this the first time
      } catch (e) {
        logger.error(e)
        return next(e)
      }
    }

    /**
     * Then, for each request we need an auth options
     */
    const opts = await auth.authorizeRequest()

    get = (url, config = {}) => {
      const fullURL = url.startsWith('http')
        ? url
        : `${apiUrl}/${url}`

      const mergedConfig = deepmerge(config, {
        headers: {
          Cookie: `NYT-S=${userCookie}`,
          ...opts.headers,
        },
        timeout,
      })

      return axios.get(fullURL, mergedConfig).catch((err) => {
        throw new StatusError(err)
      })
    }

    // ensure we have a client (only the first call)
    // create a token for all requests
  } else {
    // call api and insert cookie
    get = (url, config = {}) => {
      const fullURL = url.startsWith('http')
        ? url
        : `${apiUrl}/${url}`

      const mergedConfig = deepmerge(config, {
        headers: {
          Cookie: `NYT-S=${userCookie}`,
        },
        timeout,
      })

      return axios.get(fullURL, mergedConfig).catch((err) => {
        throw new StatusError(err)
      })
    }
  }

  res.locals.api = { get }

  next()
}
