const MobileDetect = require('mobile-detect')
const platform = require('platform')
const deIndent = require('de-indent')

const {
  getNewsreaderWebView,
  getNewsreaderVersion,
} = require('../util/newsreader')
const getCrosswordWebView = require('../util/getCrosswordWebView')
const getCrosswordVersion = require('../util/getCrosswordVersion')

const formatPlatform = (deviceType) => {
  if (deviceType.newsreaderAppPlatform) return 'Web (News App)'

  if (deviceType.isMobile) return 'Web (Mobile)'

  return 'Web (Desktop)'
}

/**
 * This is our server-side implementation of device detection.
 * For client-side device detection (which should be used on all cached routes)
 * see shared/responsive/index.ts and shared/responsive/hybrid.ts
 */
module.exports = function injectDeviceType(req, res, next) {
  const { headers, cookies = {}, query = {} } = req
  const ua = headers['user-agent']
  const md = new MobileDetect(ua)

  const newsreaderWebView = getNewsreaderWebView(query, ua)
  const crosswordWebView = getCrosswordWebView(ua, query)

  /*
   * This work was initially added for checking if the 50% off
   * for the Games sale is supported by specific iOS & Android
   * Crossword App & Newsreader versions, can be used in future to compare
   * versions using semver: https://github.com/nytimes/games-phoenix/pull/2718/files#diff-aad635e8bbba32522e64ab330192d2f9a1e3916e0829c60ba27dfad012f02a23
   */
  const newsreaderAppVersion = getNewsreaderVersion(ua)
  const gamesAppVersion = getCrosswordVersion(ua)

  // cachable
  res.locals.deviceType = {
    // If a UA is from the newsreader, we'll force (assume) that isMobile is true.
    // This will prevent some logic in device.js where we reload the page if an
    // iPad's UA didn't correctly determine we've got an iPad.
    isMobile:
      !!newsreaderWebView ||
      !!crosswordWebView ||
      !!cookies.inferredIpad ||
      !!md.mobile(),
    isPhone: !!md.phone(),
    isTablet: !!cookies.inferredIpad || !!md.tablet(),
    newsreaderAppPlatform: newsreaderWebView,
    newsreaderAppVersion,
    gamesAppPlatform: crosswordWebView,
    gamesAppVersion,
  }

  // not cachable
  const deviceType = platform.parse(req.headers['user-agent'])
  res.locals.device = {
    // NOTE: the order of the substitutions is very fragile
    // verify carefully if changing
    feedback: deIndent(`\r\n\r\n
      ----
      Device summary:
      Page: %s
      Platform: ${formatPlatform(res.locals.deviceType)}
      Browser: ${deviceType.name} ${deviceType.version}
      OS: ${deviceType.os.family} ${deviceType.os.version}
      Screen Resolution: %s x %s
      Viewport Size: %s x %s
      Timezone: %s
      Regi: %s
      Digi: %s
      Xwd: %s
      Hd: %s
      Page View: %s
    `),
    ...deviceType,
  }

  next()
}
