const url = require('url')

const express = require('express')
const cors = require('cors')

const { SUBSCRIBE_URL } = require('../constants')

const createPageProps = require('./util/createPageProps')
const createPageErrorProps = require('./util/createPageErrorProps')
const StatusError = require('./util/StatusError')
const svc = require('./services')
const mw = require('./middleware')
const { puzzleRouterConfig } = require('./puzzlesRouterConfig')

const router = express.Router()

const {
  letterBoxed,
  spellingBeeHub,
  spellingBeeDate,
  tiles,
  vertexDate,
  vertexShortz,
  sudoku,
  maintenance,
  connections,
  testing,
  tilesShortz,
  wordleShortz,
  letterBoxedShortz,
  spellingBeeShortz,
} = puzzleRouterConfig

// ----------------------------
// PUBLIC USER/DEVICE ENDPOINTS
// ----------------------------

const corsOptions = {
  origin: 'nytresource://nyt-local-asset-host',
  optionsSuccessStatus: 200,
}

router.get('/user', (req, res) => res.send(res.locals.user))
router.get('/device', (req, res) => res.send(res.locals.device))
router.get('/shortz/:regiId', cors(corsOptions), (req, res) => {
  const shortz = svc.getShortz(parseInt(req.params.regiId))
  res.send(shortz)
})

/**
 * Upon the deprecation of our internal subscribe URLs, we'll redirect
 * any traffic to an external URL.
 */
router.get(
  '/subscribe/:game(tiles|spelling-bee|vertex)',
  (req, res) => {
    res.redirect(
      301,
      url.format({
        pathname: SUBSCRIBE_URL,
      }),
    )
  },
)

// ------------
// Letter Boxed
// ------------

router.get(
  '/letter-boxed',
  mw.injectMetadata(letterBoxed.metaData),
  mw.cacheInFastly({
    surrogateKey: 'letterboxed',
  }),
  mw.renderView(
    'LetterBoxed',
    createPageProps(letterBoxed.pageProps),
  ),
)

router.get(
  '/shortz/letter-boxed/:date?',
  mw.injectMetadata(letterBoxedShortz.metaData),
  mw.renderView(
    'LetterBoxed',
    createPageProps(letterBoxedShortz.pageProps),
  ),
)

// ------------
// SPELLING BEE
// ------------

router.get(
  '/spelling-bee/hub',
  mw.injectMetadata(spellingBeeHub.metaData),
  mw.renderView(
    'SpellingBeeHub',
    createPageProps(spellingBeeHub.pageProps),
  ),
  mw.renderError(
    'ErrorPage',
    createPageErrorProps({
      liraAsset: 'spelling-bee',
      subscriptionCampaign: 'spellingBeePaywall',
    }),
  ),
)

router.get(
  '/spelling-bee/:date?',
  mw.injectMetadata(spellingBeeDate.metaData),
  mw.renderView(
    'SpellingBee',
    createPageProps(spellingBeeDate.pageProps),
  ),
  mw.renderError(
    'ErrorPage',
    createPageErrorProps({
      liraAsset: 'spelling-bee',
      subscriptionCampaign: 'spellingBeePaywall',
    }),
  ),
)

router.get(
  '/shortz/spelling-bee/:date?',
  mw.injectMetadata(spellingBeeShortz.metaData),
  mw.renderView(
    'SpellingBeeShortz',
    createPageProps(spellingBeeShortz.pageProps),
  ),
)

// ------
// TILES
// ------

router.get(
  '/tiles',
  mw.injectMetadata(tiles.metaData),
  mw.cacheInFastly({
    surrogateKey: 'tiles',
  }),
  mw.renderView('Tiles', createPageProps(tiles.pageProps)),
)

router.get(
  '/tiles/palette/:name',
  mw.cacheInFastly({
    surrogateKey: 'tiles-palette',
  }),
  async (req, res) => {
    try {
      const data = await svc.getTiles(req.params.name)
      res.status(200).send(data)
    } catch (e) {
      res.status(500).send({
        message: `Tiles: Could not fetch palette. ${e}`,
      })
    }
  },
)

router.get(
  '/shortz/tiles/:palette?',
  mw.injectMetadata(tilesShortz.metaData),
  async (req, res, next) => {
    const data = await svc.getTiles(req.params.palette)
    if (!data.success) {
      return res.status(400).send({ error: data.error })
    }
    res.locals.tilesData = data
    next()
  },
  mw.renderView(
    'TilesShortz',
    createPageProps({
      pageName: 'tiles',
      adUnitPath: 'tiles',
      entry: 'tilesshortz',
      gameData: (_, res) => res.locals.tilesData,
      includeScreenIntercept: true,
    }),
  ),
  mw.renderView(
    'TilesShortz',
    createPageProps(tilesShortz.pageProps),
  ),
)

// --------
// VERTEX
// --------

router.get(
  '/vertex',
  mw.injectMetadata(vertexDate.metaData),
  mw.renderView('Vertex', createPageProps(vertexDate.pageProps)),
)

router.get(
  '/shortz/vertex/:date',
  mw.injectMetadata(vertexShortz.metaData),
  mw.renderView(
    'VertexShortz',
    createPageProps(vertexShortz.pageProps),
  ),
)

// ------
// WORDLE PREVIEWS
//
// Wordle dated preview links are hosted at /puzzles/shortz/wordle instead
// of /games, where the public-facing game sits. This is to
// acommodate Fastly logic that redirects all routes that
// start with /games/wordle to /games/wordle/index.html.
// Wordle previews will exist at /puzzles until we move away
// from the /index.html pattern to skip the redirect and
// Fastly caching.
// ------

router.get(
  '/shortz/wordle/:date?',
  mw.injectMetadata(wordleShortz.metaData),
  mw.restrictAccess((req) => {
    const { date } = req.params
    const validDate = /\d{4}-\d{1,2}-\d{1,2}/.test(`${date}`)
    if (!validDate) {
      return 404
    }
  }),
  mw.renderView(
    'WordleShortz',
    createPageProps({
      pageName: 'wordle',
      adUnitPath: 'wordle',
      entry: 'wordleshortz',
      includeScreenIntercept: true,
      data: (req) => {
        const { date } = req.params
        return date
      },
    }),
  ),
)

// ------
// SUDOKU
// ------

router.get(
  '/sudoku/:level(easy|medium|hard)?',
  mw.injectMetadata(sudoku.metaData),
  mw.cacheInFastly({
    surrogateKey: 'sudoku',
  }),
  mw.renderView('Sudoku', createPageProps(sudoku.pageProps)),
)

// -----------------
// LEADERBOARD PAGES
// -----------------

const leaderboardMeta = {
  title: 'Leaderboards - The New York Times',
  metaTags: [
    {
      property: 'og:title',
      content: 'Leaderboards - The New York Times',
    },
    {
      property: 'og:url',
      content: 'https://www.nytimes.com/puzzles/leaderboards',
    },
    {
      property: 'og:description',
      content: 'See your friends’ Mini puzzle times.',
    },
    {
      property: 'og:image',
      content:
        'https://static01.nyt.com/packages/other/crossword/fb-og/black.png',
    },
  ],
}

const handledErrors = [
  401, // needs to login
  404, // needs to create a profile
  428, // (Precondition Required) indirectly needs a profile (invite page)
]

const leaderboardErrorHandler = mw.renderError(
  'Leaderboard',
  createPageProps({
    metadata: leaderboardMeta,
    entry: 'leaderboard',
    view: 'onboarding',
    data: (req) => ({
      fromAuthPage: !!req.query.auth,
    }),
  }),
  (err) => handledErrors.includes(err.status),
)

router.get(
  '/leaderboards/:manage(confirm|settings)',
  mw.renderView(
    'Leaderboard',
    createPageProps({
      pageName: 'leaderboards',
      metadata: leaderboardMeta,
      entry: 'leaderboard',
      data: svc.getFriends,
      view: (req) =>
        req.params.manage === 'settings'
          ? 'friends'
          : 'friendsConfirm',
    }),
  ),
  leaderboardErrorHandler,
)

router.get(
  '/leaderboards',
  mw.renderView(
    'Leaderboard',
    createPageProps({
      pageName: 'leaderboards',
      metadata: leaderboardMeta,
      entry: 'leaderboard',
      data: svc.getLeaderboard,
      view: 'leaderboards',
    }),
  ),
  leaderboardErrorHandler,
)

router.get(
  '/leaderboards/invite/:code?',
  mw.renderView(
    'Leaderboard',
    createPageProps({
      pageName: 'leaderboards',
      metadata: leaderboardMeta,
      entry: 'leaderboard',
      data: svc.getProfile,
    }),
  ),
  leaderboardErrorHandler,
)

// -----
// STATS
// -----

router.get(
  '/stats',
  mw.injectMetadata({
    trackingPageType: 'statistics',
    title:
      'NYTimes Crosswords - Play Daily and Mini Puzzles - NYTimes.com',
  }),
  mw.renderView(
    'Stats',
    createPageProps({
      pageName: 'stats',
      adUnitPath: 'stats',
      entry: 'stats',
      stats: svc.getStats,
    }),
  ),
)

// ------
// MAINTENANCE
// ------

router.get(
  '/maintenance',
  mw.injectMetadata(maintenance.metaData),
  mw.renderView(
    'MaintenancePage',
    createPageProps(maintenance.pageProps),
  ),
)

// ---------
// CONNECTIONS PREVIEWS
//
// Connections dated preview links are hosted at
// /puzzles/shortz/connections instead of /games, where the
// public-facing game sits. Ads are not shown on the preview
// link pages.
// ---------

router.get(
  '/shortz/connections/:date?',
  mw.restrictAccess((req) => {
    const { date } = req.params
    const validDate = /\d{4}-\d{1,2}-\d{1,2}/.test(`${date}`)
    if (!validDate) {
      return 404
    }
  }),
  mw.renderView(
    'ConnectionsShortz',
    createPageProps({
      pageName: 'connections',
      adUnitPath: 'connections',
      entry: 'connectionsshortz',
      includeScreenIntercept: true,
      data: (req) => {
        const { date } = req.params
        return date
      },
    }),
  ),
)

// ---------
// TESTING
// ---------

router.get(
  '/testing',
  mw.renderView('Testing', createPageProps(testing.pageProps)),
)

// ---------
// NOT FOUND
// ---------

router.get('*', (req, res, next) => {
  next(new StatusError('Not found', 404))
})

module.exports = router
