const moment = require('moment')
const MockDate = require('mockdate')

const getSpellingBee = require('../getSpellingBee')

describe('SERVICE: GET SPELLING BEE', () => {
  let req
  let res
  let examplePuzzles
  let today
  let yesterday
  let expiration
  let apiExpirationNano
  let freeExpirationText

  beforeEach(() => {
    req = { params: {} }
    res = {
      locals: {
        api: {
          get: jest.fn(),
        },
      },
    }

    today = moment('2018-05-20 00:00:00')
    yesterday = today.clone().subtract(1, 'day')
    expiration = today.clone().add(6, 'hours')
    apiExpirationNano = expiration.unix()
    const freeExpiration = expiration.unix() / 1000
    freeExpirationText = moment(apiExpirationNano).format('MMMM Do.')

    examplePuzzles = {
      'svc/spelling-bee/v1/2018-05-03.json': {
        data: {
          id: 3,
          expiration: apiExpirationNano,
          center_letter: 'g',
          outer_letters: 'abcd',
          pangrams: ['e', 'f'],
          answers: ['g', 'h'],
          print_date: '2018-05-03',
          is_free: true,
          free_expiration: freeExpiration,
          editor: 'Sam Ezersky',
        },
      },
      'svc/spelling-bee/v1/2021-08-31.json': {
        data: {
          id: 3,
          expiration: apiExpirationNano,
          center_letter: 't',
          outer_letters: 'tuvw',
          answers: ['u', 'w'],
          print_date: '2021-08-31',
          is_free: true,
          free_expiration: freeExpiration,
          editor: 'Sam Ezersky',
        },
      },
      'svc/spelling-bee/v1/active.json': {
        data: {
          thisWeek: [],
          lastWeek: [],
          puzzles: [
            {
              id: 2,
              expiration: apiExpirationNano,
              center_letter: 'f',
              outer_letters: 'efgh',
              pangrams: ['e', 'f'],
              answers: ['g', 'h'],
              print_date: today.format('YYYY-MM-DD'),
              is_free: true,
              free_expiration: freeExpiration,
              editor: 'Sam Ezersky',
            },
            {
              id: 1,
              expiration: apiExpirationNano,
              center_letter: 'e',
              outer_letters: 'abcd',
              pangrams: ['a', 'b'],
              answers: ['c', 'd'],
              print_date: yesterday.format('YYYY-MM-DD'),
              is_free: true,
              free_expiration: freeExpiration,
              editor: 'Sam Ezersky',
            },
          ],
          today: 0,
          yesterday: 1,
        },
      },
    }

    res.locals.api.get.mockImplementation((file) =>
      Promise.resolve(examplePuzzles[file]),
    )

    MockDate.set(today.clone().add('1 hour'))
  })

  afterEach(() => {
    MockDate.reset()
  })

  it('adapts api data correctly', () => {
    expect.assertions(2)

    return getSpellingBee(req, res).then((v) => {
      expect(v.today).toEqual({
        expiration: apiExpirationNano,
        displayWeekday: today.format('dddd'),
        displayDate: today.format('MMMM D, YYYY'),
        printDate: today.format('YYYY-MM-DD'),
        centerLetter: 'f',
        outerLetters: ['e', 'f', 'g', 'h'],
        validLetters: ['f', 'e', 'f', 'g', 'h'],
        pangrams: ['e', 'f'],
        answers: ['e', 'f', 'g', 'h'],
        id: 2,
        isFree: true,
        freeExpiration: freeExpirationText,
        editor: 'Sam Ezersky',
      })
      expect(v.yesterday).toEqual({
        expiration: apiExpirationNano,
        displayWeekday: yesterday.format('dddd'),
        displayDate: yesterday.format('MMMM D, YYYY'),
        printDate: yesterday.format('YYYY-MM-DD'),
        centerLetter: 'e',
        outerLetters: ['a', 'b', 'c', 'd'],
        validLetters: ['e', 'a', 'b', 'c', 'd'],
        pangrams: ['a', 'b'],
        answers: ['a', 'b', 'c', 'd'],
        id: 1,
        isFree: true,
        freeExpiration: freeExpirationText,
        editor: 'Sam Ezersky',
      })
    })
  })

  it('fetches puzzles by date (shortz)', () => {
    req.originalUrl = '/puzzles/shortz/spelling-bee/2018-05-03'
    req.params.date = '2018-05-03'
    expect.assertions(2)

    return getSpellingBee(req, res).then((v) => {
      expect(v.today).toEqual({
        expiration: apiExpirationNano,
        displayWeekday: 'Thursday',
        displayDate: 'May 3, 2018',
        printDate: req.params.date,
        centerLetter: 'g',
        outerLetters: ['a', 'b', 'c', 'd'],
        validLetters: ['g', 'a', 'b', 'c', 'd'],
        pangrams: ['e', 'f'],
        answers: ['e', 'f', 'g', 'h'],
        id: 3,
        isFree: true,
        freeExpiration: freeExpirationText,
        editor: 'Sam Ezersky',
      })
      expect(v.yesterday).toEqual({
        expiration: apiExpirationNano,
        displayWeekday: 'Thursday',
        displayDate: 'May 3, 2018',
        printDate: req.params.date,
        centerLetter: 'g',
        outerLetters: ['a', 'b', 'c', 'd'],
        validLetters: ['g', 'a', 'b', 'c', 'd'],
        pangrams: ['e', 'f'],
        answers: ['e', 'f', 'g', 'h'],
        id: 3,
        isFree: true,
        freeExpiration: freeExpirationText,
        editor: 'Sam Ezersky',
      })
    })
  })

  it('creates a 404 for bad looking dates', () => {
    expect.assertions(1)

    req.params.date = '20-05-03'

    return getSpellingBee(req, res).catch((err) => {
      expect(err).toMatchObject({
        status: 404,
        message: 'Invalid date (status: 404)',
      })
    })
  })

  it('rejects with any api failures', () => {
    expect.assertions(1)

    const testError = new Error('abc')
    res.locals.api.get.mockReturnValue(Promise.reject(testError))

    return getSpellingBee(req, res).catch((err) => {
      expect(err).toBe(testError)
    })
  })

  it('returns a puzzle with today and yesterday even if pangrams are missing (shortz)', () => {
    // pangrams are missing on this date
    req.originalUrl = '/puzzles/shortz/spelling-bee/2018-05-03'
    req.params.date = '2021-08-31'
    expect.assertions(2)
    // a puzzle object with 'today' and 'yesterday' keys are returned
    return getSpellingBee(req, res).then((v) => {
      expect(v).toHaveProperty('today')
      expect(v).toHaveProperty('yesterday')
    })
  })

  //
  // Tests disabled while subscription checking is handled by phoenix server
  // /src/server/services/getSpellingBee.js:55
  //

  // it('caches for subscribers until expiration', async () => {
  //   await getSpellingBee(req, res)
  //   expect(res.locals.api.get).toHaveBeenCalledTimes(2)

  //   res.locals.userType.hasXwd = false
  //   await getSpellingBee(req, res)
  //   expect(res.locals.api.get).toHaveBeenCalledTimes(4)
  //   res.locals.userType.hasXwd = true

  //   await getSpellingBee(req, res)
  //   expect(res.locals.api.get).toHaveBeenCalledTimes(4)

  //   MockDate.set(expiration.clone().add(1, 'second'))

  //   await getSpellingBee(req, res)
  //   expect(res.locals.api.get).toHaveBeenCalledTimes(6)
  // })

  // it('caches for non-subscribers if during test', async () => {
  //   res.locals.userType.hasXwd = false

  //   await getSpellingBee(req, res)
  //   await getSpellingBee(req, res)
  //   expect(res.locals.api.get).toHaveBeenCalledTimes(2)

  //   MockDate.set(expiration.clone().add(1, 'second'))

  //   await getSpellingBee(req, res)
  //   expect(res.locals.api.get).toHaveBeenCalledTimes(4)
  // })

  // it('caches error for non-subscribers correctly during test', async () => {
  //   // logged out user first
  //   res.locals.userType.hasXwd = false
  //   const testError = new Error('abc')
  //   testError.status = 401
  //   res.locals.api.get.mockReturnValue(Promise.reject(testError))

  //   try {
  //     await getSpellingBee(req, res)
  //   } catch (e) {
  //     expect(res.locals.api.get).toHaveBeenCalledTimes(2)
  //   }

  //   // another logged out user
  //   try {
  //     await getSpellingBee(req, res)
  //   } catch (e) {
  //     expect(res.locals.api.get).toHaveBeenCalledTimes(4)
  //   }

  //   // then a subscriber
  //   res.locals.userType.hasXwd = true
  //   res.locals.api.get.mockReturnValue(Promise.resolve(examplePuzzles['svc/spelling-bee/v1/puzzle.json']))

  //   await getSpellingBee(req, res)

  //   expect(res.locals.api.get).toHaveBeenCalledTimes(6)

  //   // now, the next non sub
  //   res.locals.userType.hasXwd = false
  //   res.locals.api.get.mockReturnValue(Promise.reject(testError))

  //   try {
  //     await getSpellingBee(req, res)
  //   } catch (e) {
  //     expect(res.locals.api.get).toHaveBeenCalledTimes(8)
  //   }

  //   // finally the next non sub is cached (until expired)
  //   try {
  //     await getSpellingBee(req, res)
  //   } catch (e) {
  //     expect(res.locals.api.get).toHaveBeenCalledTimes(8)
  //   }

  //   // then the cache expires
  //   MockDate.set(expiration.clone().add(1, 'second'))

  //   try {
  //     await getSpellingBee(req, res)
  //   } catch (e) {
  //     expect(res.locals.api.get).toHaveBeenCalledTimes(10)
  //   }
  // })
})
