const path = require('path')

const fs = require('fs-extra')
const axios = require('axios')

const getTiles = require('../getTiles')
const tilesConfig = require('../../util/tilesConfig.json')

jest.mock('axios', () => ({
  get: jest.fn(),
}))

jest.mock('../../../constants', () => ({
  RUNTIME_ENV: 'dev',
  ROUTE_ASSETS: 'route-assets',
  GOOGLE_STORAGE_API: 'gcs',
}))

Date.now = jest.fn(() => new Date(Date.UTC(2018, 1, 28))) // 27 Feb 2018

const TILES_ASSETS = path.resolve(
  __dirname,
  '..',
  '..',
  '..',
  'public',
  'assets',
  'tiles-game',
)

const svg = `<svg width="0" height="0" class="hidden">
  <symbol viewBox="0 0 44 44" fill="none" xmlns="http://www.w3.org/2000/svg" id="A1"></symbol>
  <symbol viewBox="0 0 44 44" fill="none" xmlns="http://www.w3.org/2000/svg" id="Z1"></symbol>
  <symbol viewBox="0 0 44 44" fill="none" xmlns="http://www.w3.org/2000/svg" id="Z2"></symbol>
  <symbol viewBox="0 0 125 109" fill="none" xmlns="http://www.w3.org/2000/svg" id="bg-color">
    <path fill="#0D3264" d="M0 0h125v109H0z"></path>
  </symbol>
  <symbol viewBox="0 0 125 109" fill="none" xmlns="http://www.w3.org/2000/svg" id="font-color">
    <path fill="#fff" d="M0 0h125v109H0z"></path>
  </symbol>
  <symbol viewBox="0 0 125 109" fill="none" xmlns="http://www.w3.org/2000/svg" id="selection-color">
    <path fill="#000" d="M0 0h125v109H0z"></path>
  </symbol>
</svg>`

const iconsSVG = `<svg width="0" height="0" class="hidden">
  <symbol viewBox="0 0 44 44" fill="none" xmlns="http://www.w3.org/2000/svg" id="default-palette"></symbol>
  <symbol viewBox="0 0 44 44" fill="none" xmlns="http://www.w3.org/2000/svg" id="test-palette"></symbol>

</svg>`

const exampleGCSCalls = {
  'gcs/v2/assets/tiles-game/palettes/topeka.svg': {
    data: svg,
  },
  'gcs/v2/assets/tiles-game/palettes/lisbon.svg': {
    data: svg,
  },
  'gcs/v2/assets/tiles-game/palettes/icons.svg': {
    data: iconsSVG,
  },
  'gcs/v2/assets/tiles-game/palettes/kuala-lumpur.svg': {
    data: svg,
  },
}

const sampleResponse = {
  filename: 'lisbon',
  displayName: 'Lisbon',
  createdBy: 'Robert Vinluan',
  svg,
  iconsSVG,
  bgColor: '#0D3264',
  fontColor: '#fff',
  selectionColor: '#000',
  zLayer: ['Z1', 'Z2'],
  layers: [['A1']],
  success: true,
  allPalettes: [
    {
      displayName: 'Soho',
      filename: 'soho',
      highlightAsNew: {
        endDate: '2024-02-05',
        startDate: '2024-01-05',
      },
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Lisbon',
      filename: 'lisbon',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Kuala Lumpur',
      filename: 'kuala-lumpur',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Hong Kong',
      filename: 'hong-kong',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Austin',
      filename: 'austin',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'New Haven',
      filename: 'new-haven',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Paris',
      filename: 'paris',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Brighton',
      filename: 'brighton',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Tangier',
      filename: 'tangier',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Holland',
      filename: 'holland',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Los Angeles',
      filename: 'los-angeles',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Utrecht',
      filename: 'utrecht',
      createdBy: 'Andrew Dore',
    },
    {
      displayName: 'Topeka',
      filename: 'topeka',
      createdBy: 'Robert Vinluan',
    },
    {
      displayName: 'Default',
      filename: 'lisbon',
      createdBy: 'Robert Vinluan',
    },
  ],
}

describe('SERVICE: GET TILES', () => {
  test('gets palette from config and returns derived data for valid palette name', () => {
    axios.get.mockImplementation((url) =>
      Promise.resolve(exampleGCSCalls[url]),
    )

    return getTiles('lisbon').then((d) => {
      expect(d).toEqual(sampleResponse)
    })
  })

  test('should handle error fetching file', () => {
    axios.get.mockImplementation(() => Promise.reject(new Error()))

    return getTiles().catch((e) => {
      expect(e.message).toContain('Tiles: Failed to fetch')
    })
  })

  test('should handle error parsing svg', () => {
    const badSVG = `<svg width="0" height="0" class="hidden"></svg>`

    const exampleGCSCallsBadSVG = {
      'gcs/v2/assets/tiles-game/palettes/lisbon.svg': {
        data: badSVG,
      },
      'gcs/v2/assets/tiles-game/palettes/icons.svg': {
        data: iconsSVG,
      },
    }

    axios.get.mockImplementation((url) =>
      Promise.resolve(exampleGCSCallsBadSVG[url]),
    )

    return getTiles('lisbon').catch((e) => {
      expect(e.message).toBe('Tiles: Failed to parse svg')
    })
  })

  test('palettes config', async () => {
    expect(tilesConfig.allPalettes).toBeInstanceOf(Array)
    expect(tilesConfig.allPalettes.length).toBeGreaterThan(0)
    expect(tilesConfig.configs).toBeInstanceOf(Array)
    expect(tilesConfig.allPalettes.length).toBeGreaterThan(0)

    const palettes = new Set([])

    tilesConfig.allPalettes.forEach((palette) => {
      expect(typeof palette.name).toBe('string')
      expect(typeof palette.createdBy).toBe('string')
      palettes.add(palette.name)
    })

    tilesConfig.configs.forEach((c) => {
      expect(c.palettes).toBeInstanceOf(Array)
      expect(c.palettes.length).toBeGreaterThan(0)
      c.palettes.forEach((palette) => {
        expect(typeof palette).toBe('string')
        palettes.add(palette)
      })
      expect(c.startDate).toMatch(/^\d{4}-\d{2}-\d{2}$/)
    })

    try {
      await Promise.all(
        Array.from(palettes).map(async (p) => {
          try {
            const file = path.resolve(
              TILES_ASSETS,
              'palettes',
              `${p}.svg`,
            )
            const buff = await fs.readFile(file)
            const svg = buff.toString()
            getTiles.extractInfoFromSVG(svg) // if this throws the palette is for sure messed up
          } catch (e) {
            e.message = `Invalid palette '${p}'\n${e.message}`
            throw e
          }
        }),
      )
    } catch (e) {
      expect(e.stack).not.toBeDefined()
    }
  })

  test('returns todays palette if invalid palette name is passed in', () => {
    axios.get.mockImplementation((url) =>
      Promise.resolve(exampleGCSCalls[url]),
    )

    return getTiles('nonexistent-palette').then((d) => {
      expect(d.success).toEqual(false)
      expect(d.error).toContain(
        "invalid palette name 'nonexistent-palette'",
      )
    })
  })
})
