const axios = require('axios')
const moment = require('moment-timezone')
const { parse } = require('svg-parser')

const {
  APPLICATION_HOST_SERVER,
  RUNTIME_ENV,
  ROUTE_ASSETS,
  GOOGLE_STORAGE_API,
} = require('../../constants')
const tilesConfig = require('../util/tilesConfig.json')

async function fetchFile(fileUrl) {
  try {
    return await axios.get(fileUrl)
  } catch (e) {
    e.message = `Tiles: Failed to fetch ${fileUrl}`
    throw e
  }
}

const getTodaysConfig = (configs) =>
  configs.find(({ startDate, endDate }) =>
    moment().isBetween(startDate, endDate, 'day', '[]'),
  ) || configs[0]

const getTodaysPalette = ({ startDate, palettes }) => {
  const totalPalettes = palettes.length
  const nthDay = moment().diff(startDate, 'days')

  return nthDay < totalPalettes && nthDay > 0
    ? palettes[nthDay]
    : palettes[Math.abs(nthDay % totalPalettes)]
}

const extractInfoFromSVG = (svg) => {
  try {
    const parsedSVG = parse(svg).children
    const bgColorObj = parsedSVG.find(
      (obj) => obj.attributes.id === 'bg-color',
    )
    const bgColor = bgColorObj.children[0].attributes.fill
    const fontColorObj = parsedSVG.find(
      (obj) => obj.attributes.id === 'font-color',
    )

    const fontColor = fontColorObj.children[0].attributes.fill

    const selectionColorObj = parsedSVG.find(
      (obj) => obj.attributes.id === 'selection-color',
    )

    const selectionColor =
      selectionColorObj === undefined
        ? '#FF0000'
        : selectionColorObj.children[0].attributes.fill

    const layerLetters = [...Array(25).keys()].map((i) =>
      String.fromCharCode(i + 65),
    ) // Alphabet minus Z
    const layers = []

    const getLayerIds = (layer) =>
      parsedSVG
        .filter((symbol) => symbol.attributes.id.includes(layer))
        .map((symbol) => symbol.attributes.id)

    layerLetters.forEach((letter) => {
      const layerIds = getLayerIds(letter)
      if (layerIds.length > 0) layers.push(layerIds)
    })

    return {
      bgColor,
      fontColor,
      selectionColor,
      zLayer: getLayerIds('Z'),
      layers,
    }
  } catch (e) {
    e.message = 'Tiles: Failed to parse svg'
    throw e
  }
}

const morphPaletteName = (name) =>
  name
    .split('-')
    .map((s) => s.charAt(0).toUpperCase() + s.slice(1))
    .join(' ')

const formatPalettesArray = (allPalettes) =>
  allPalettes.map((p) => ({
    displayName: morphPaletteName(p.name),
    filename: p.name,
    createdBy: p.createdBy,
    ...(p.highlightAsNew && { highlightAsNew: p.highlightAsNew }),
  }))

/**
 * Use the Google Storage API except on production
 */
const TILES_ASSET_BASE =
  RUNTIME_ENV === 'prd'
    ? `${APPLICATION_HOST_SERVER}${ROUTE_ASSETS}/assets/tiles-game`
    : `${GOOGLE_STORAGE_API}/v2/assets/tiles-game`

module.exports = async function getTiles(paletteName = null) {
  const { allPalettes, experiments } = tilesConfig

  const todaysConfig = getTodaysConfig(tilesConfig.configs)
  const processedPalettes = allPalettes.filter((palette) =>
    todaysConfig.palettes.includes(palette.name),
  )

  const publicAndExperimentalPalettes = [
    ...allPalettes,
    ...experiments,
  ]

  const validNameList = publicAndExperimentalPalettes.map(
    (el) => el.name,
  )

  const validPaletteName = validNameList.includes(paletteName)
    ? paletteName
    : null

  const palette = validPaletteName || getTodaysPalette(todaysConfig)
  const [{ createdBy }] = publicAndExperimentalPalettes.filter(
    (p) => p.name === palette,
  )
  const paletteUrl = `${TILES_ASSET_BASE}/palettes/${palette}.svg`
  const paletteResponse = await fetchFile(paletteUrl)
  const svg = paletteResponse.data
  const SVGdata = extractInfoFromSVG(svg)

  const iconsUrl = `${TILES_ASSET_BASE}/palettes/icons.svg`
  const iconsResponse = await fetchFile(iconsUrl)
  const iconsSVG = iconsResponse.data

  const data = {
    filename: palette,
    displayName: morphPaletteName(palette),
    createdBy,
    svg,
    iconsSVG,
    ...SVGdata,
    allPalettes: [
      ...formatPalettesArray(processedPalettes),
      { displayName: 'Default', filename: palette, createdBy },
    ],
    success: true,
  }

  if (paletteName && validPaletteName === null) {
    return {
      ...data,
      success: false,
      error: `invalid palette name '${paletteName}', valid experiment names include: ${experiments
        .map((e) => `'${e.name}'`)
        .join(', ')}`,
    }
  }

  return data
}

module.exports.extractInfoFromSVG = extractInfoFromSVG
