const axios = require('axios')
const moment = require('moment')

const StatusError = require('../util/StatusError')
const {
  APPLICATION_HOST_SERVER,
  RUNTIME_ENV,
  ROUTE_ASSETS,
  GOOGLE_STORAGE_API,
} = require('../../constants')

const VERTEX_ASSET_BASE =
  RUNTIME_ENV === 'dev'
    ? `${APPLICATION_HOST_SERVER}${ROUTE_ASSETS}/assets/vertex`
    : `${GOOGLE_STORAGE_API}/v2/assets/vertex`

const TODAY_URL = 'svc/vertex/v1/puzzle/today.json'

const fetchFile = (fileUrl) => axios.get(fileUrl)

const formatConstructors = (constructorsArray = []) => {
  let [constructors] = constructorsArray

  for (let i = 1; i < constructorsArray.length; i += 1) {
    if (i === constructorsArray.length - 1) {
      constructors += ` and ${constructorsArray[i]}`
    } else {
      constructors += `, ${constructorsArray[i]}`
    }
  }

  return constructors
}

const adaptPuzzle = (apiPuzzle) => {
  const puzzle = {
    vertices: {},
  }

  try {
    const date = moment(apiPuzzle.printDate, 'YYYY-M-D')

    Object.keys(apiPuzzle.body.vertices).forEach((vertex) => {
      puzzle.vertices[vertex] = {
        coordinates: apiPuzzle.body.vertices[vertex].coordinates,
        shapes: [],
      }

      Object.keys(apiPuzzle.body.shapes).forEach((shape) => {
        if (
          apiPuzzle.body.shapes[shape].vertices.includes(
            parseInt(vertex),
          )
        ) {
          puzzle.vertices[vertex].shapes.push(shape)
        }
      })
    })

    puzzle.palette = apiPuzzle.body.palettes
    puzzle.shapes = apiPuzzle.body.shapes

    if (apiPuzzle.body.preDrawnLines) {
      puzzle.preDrawnLines = apiPuzzle.body.preDrawnLines.map(
        (line) => line.coordinates,
      )
    }

    return {
      id: apiPuzzle.id,
      displayDate: date.format('dddd, MMMM D'),
      puzzleConstructor: formatConstructors(apiPuzzle.constructors),
      theme: apiPuzzle.theme,
      ...puzzle,
    }
  } catch (e) {
    throw new Error(`failed to adapt ${JSON.stringify(apiPuzzle)}`)
  }
}

const adaptPuzzlePack = (puzzles) => {
  try {
    return Object.values(puzzles).map((puzz) => adaptPuzzle(puzz))
  } catch (e) {
    throw new Error(`failed to adapt tutorial puzzles`)
  }
}

const getAPIPuzzle = (api, url) =>
  api.get(url).then(({ data }) => adaptPuzzle(data))

const getPacks = (url) =>
  fetchFile(url).then(({ data }) => adaptPuzzlePack(data))

const getVertex = (req, res) => {
  const { api } = res.locals
  const { originalUrl } = req
  const isShortzRequest = /shortz/.test(originalUrl)
  let url

  if (req.params.date && isShortzRequest) {
    url = `svc/vertex/v1/puzzle/${req.params.date}.json`
  } else if (req.params.date && !isShortzRequest) {
    return Promise.reject(
      new StatusError(`restricted vertex ${req.params.date}`, 404),
    )
  } else {
    url = TODAY_URL
  }

  return Promise.all([
    getAPIPuzzle(api, url),
    getPacks(`${VERTEX_ASSET_BASE}/tutorial.json`),
    getPacks(`${VERTEX_ASSET_BASE}/starter-pack.json`),
  ])
    .then(
      ([today, adaptedTutorialPuzzles, adaptedStarterPuzzles]) => ({
        ...today,
        tutorial: adaptedTutorialPuzzles,
        starterPuzzles: adaptedStarterPuzzles,
      }),
    )
    .catch((e) => {
      if (e.status === 401 && isShortzRequest) {
        throw new StatusError(`restricted vertex puzzle`, 404)
      }
      e.message = `failed to process puzzle: ${e}`
      throw e
    })
}

module.exports = getVertex
