const StatusError = require('../StatusError')

describe('APP/UTIL: CREATE STATUS ERROR', () => {
  it('supports defaults', () => {
    const err = new StatusError()
    expect(err.message).toEqual('something went wrong (status: 500)')
    expect(err.status).toEqual(500)
    expect(err.data).toEqual({})
  })

  it('supports a string message', () => {
    const err = new StatusError('my error', 404)
    expect(err.message).toEqual('my error (status: 404)')
    expect(err.status).toEqual(404)
    expect(err.data).toEqual({})
  })

  it('supports an error object', () => {
    const originalError = new Error('raw error')
    const err = new StatusError(originalError, 401)
    expect(err.message).toEqual('raw error (status: 401)')
    expect(err.status).toEqual(401)
    expect(err.data).toEqual({})
  })

  it('passes through another status error', () => {
    const err = new StatusError('123', 401)
    const another = new StatusError(err)
    expect(another.message).toEqual('123 (status: 401)')
    expect(another.status).toEqual(401)
    expect(err.data).toEqual({})
  })

  it('allows overwriting status error’s status', () => {
    const err = new StatusError('123', 401)
    const another = new StatusError(err, 404)
    expect(another.message).toEqual('123 (status: 404)')
    expect(another.status).toBe(404)
    expect(err.data).toEqual({})
  })

  it('allows overwriting error data', () => {
    const err = new StatusError('123', 401, { extra: true })
    expect(err.data).toEqual({ extra: true })
  })

  it('supports an axios error in request', () => {
    const requestError = new Error()
    requestError.request = { status: 404 }
    const err = new StatusError(requestError)
    expect(err.message).toEqual(`axios request error (status: 404)`)
    expect(err.data).toEqual({})
  })

  it('supports an axios error in response', () => {
    const responseError = new Error()
    responseError.request = {}
    responseError.response = {
      data: { extra: true },
      config: { url: 'abc.com' },
      statusText: 'bad request',
      status: 404,
      headers: {},
    }

    const err = new StatusError(responseError)
    expect(err.message).toEqual(
      `abc.com => bad request (status: 404)`,
    )
    expect(err.data).toEqual({ extra: true })
  })

  it('creates a helpful stack', () => {
    const myTestFunction = () => {
      throw new StatusError('my super error', 404)
    }

    try {
      myTestFunction()
    } catch (err) {
      expect(err.stack).toEqual(
        expect.stringContaining(`at myTestFunction`),
      )

      // removes stack entry for custom error constructor
      expect(err.stack).not.toEqual(
        expect.stringContaining(`at StatusError`),
      )
    }
  })

  it('creates for wrapped errors', () => {
    const myTestFunction = () => {
      throw new Error('my super error')
    }

    try {
      myTestFunction()
    } catch (err) {
      // NOTE: the stack will not have the status if printed :(
      // but the original stack seems more useful than the poluted one
      const wrapped = new StatusError(err)
      expect(wrapped.stack).toEqual(
        expect.stringContaining(`at myTestFunction`),
      )
    }
  })
})
