const url = require('url')

const express = require('express')
const request = require('supertest')

const createNavLinks = require('../createNavigationLinks')

jest.mock('../../../constants', () => ({
  SUBSCRIPTION_CAMPAIGNS: {
    default: {
      halfPrice:
        'https://www.nytimes.com/subscription/games?campaignId=123',
      fullPrice:
        'https://www.nytimes.com/subscription/games?campaignId=234',
    },
    extra: {
      halfPrice:
        'https://www.nytimes.com/subscription/games?campaignId=123',
      fullPrice:
        'https://www.nytimes.com/subscription/games?campaignId=234',
    },
  },
  SUBSCRIBE_URL: 'https://www.nytimes.com/subscription/games',
  NATIVE_SCHEME_GAMES: 'nytxwd://',
  NATIVE_SCHEME_NEWSREADER: 'nytimes://',
  NATIVE_SUBSCRIBE_PATH: 'products',
  NATIVE_SUBSCRIBE_PATH_ANDROID: 'reader/gamesProducts/',

  NATIVE_LOGIN_PATH: 'login',
  NATIVE_REGI_PATH: 'createAccount',
  ACCOUNT_URL: 'https://myaccount.nytimes.com',
  APPLICATION_HOST_CLIENT: 'http://local.nytimes.com',
}))

describe('APP/UTIL: CREATE NAVIGATION LINKS', () => {
  let app
  let userType
  let deviceType
  let cookies

  beforeEach(() => {
    userType = {}
    deviceType = {}
    cookies = {}
    app = express()
    app.use((req, res) => {
      req.cookies = cookies
      res.locals.userType = userType
      res.locals.deviceType = deviceType
      res.send(createNavLinks(req, res))
    })
  })

  test('should create a web-specific login link with encoded url in query', () => {
    expect.assertions(4)

    return request(app)
      .get('/page?query=abc')
      .then((res) => {
        const parsedUrl = url.parse(res.body.login, true)
        expect(parsedUrl.hostname).toBe('myaccount.nytimes.com')
        expect(parsedUrl.pathname).toBe('/auth/enter-email')
        expect(parsedUrl.search).toContain('query%3Dabc') // make sure it is encoded
        expect(parsedUrl.query).toEqual({
          redirect_uri: 'http://local.nytimes.com/page?query=abc',
          response_type: 'cookie',
          client_id: 'games',
          application: 'crosswords',
        })
      })
  })

  test('should create logout link with encoded url in query', () => {
    expect.assertions(4)

    return request(app)
      .get('/page?query=abc')
      .then((res) => {
        const parsedUrl = url.parse(res.body.logout, true)
        expect(parsedUrl.hostname).toBe('myaccount.nytimes.com')
        expect(parsedUrl.pathname).toBe('/auth/logout')
        expect(parsedUrl.search).toContain('query%3Dabc') // make sure it is encoded
        expect(parsedUrl.query.redirect_uri).toEqual(
          'http://local.nytimes.com/page?query=abc',
        )
      })
  })

  test('should create register link with encoded url in query', () => {
    expect.assertions(4)

    return request(app)
      .get('/page?query=abc')
      .then((res) => {
        const parsedUrl = url.parse(res.body.register, true)
        expect(parsedUrl.hostname).toBe('myaccount.nytimes.com')
        expect(parsedUrl.pathname).toBe('/auth/register')
        expect(parsedUrl.search).toContain('query%3Dabc') // make sure it is encoded
        expect(parsedUrl.query).toEqual({
          redirect_uri: 'http://local.nytimes.com/page?query=abc',
          response_type: 'cookie',
          client_id: 'games',
          application: 'crosswords',
        })
      })
  })

  test('should create campaign links for users', () => {
    return request(app)
      .get('/page?query=abc')
      .then((res) => {
        expect(res.body.subscribe).toEqual({
          default:
            'https://www.nytimes.com/subscription/games?campaignId=234',
          extra:
            'https://www.nytimes.com/subscription/games?campaignId=234',
        })
      })
  })

  test('should create deep links for newsreader users', () => {
    deviceType.newsreaderAppPlatform = 'ios'
    deviceType.newsreaderAppVersion = '9.40.0'

    return request(app)
      .get('/page?query=abc')
      .then((res) => {
        expect(res.body.subscribe).toEqual({
          default: 'nytimes://products?campaignId=234',
          extra: 'nytimes://products?campaignId=234',
        })
        expect(res.body.login).toEqual('nytimes://login')
        expect(res.body.register).toEqual('nytimes://createAccount')
      })
  })

  test('should create deep links for newsreader Android users', () => {
    deviceType.newsreaderAppPlatform = 'android'
    deviceType.newsreaderAppVersion = '9.46'

    return request(app)
      .get('/page?query=abc')
      .then((res) => {
        expect(res.body.subscribe).toEqual({
          default: 'nytimes://reader/gamesProducts/?campaignId=234',
          extra: 'nytimes://reader/gamesProducts/?campaignId=234',
        })
        expect(res.body.login).toEqual('nytimes://login')
        expect(res.body.register).toEqual('nytimes://createAccount')
      })
  })

  test('should create deep links for hybrid users', () => {
    deviceType.gamesAppPlatform = 'android'

    return request(app)
      .get('/page?query=abc')
      .then((res) => {
        expect(res.body.subscribe).toEqual({
          default: 'nytxwd://products?campaignId=234',
          extra: 'nytxwd://products?campaignId=234',
        })
        expect(res.body.login).toEqual('nytxwd://login')
        expect(res.body.register).toEqual('nytxwd://createAccount')
      })
  })
})
