const React = require('react')
const ReactDOMServer = require('react-dom/server')
const request = require('supertest')
const express = require('express')

const createRenderer = require('../createRenderer')
const createPropGetter = require('../createPropGetter')

jest.mock('../createPropGetter', () => jest.fn())

// Middleware Helper Functions
const mw = {}

mw.setShortzMode = (inShortzMode) => (req, res, next) => {
  res.locals = { userType: { inShortzMode } }
  next()
}

mw.setErrorParam = (errorStatus) => (req, res, next) => {
  req.query.error = errorStatus.toString()
  next()
}

describe('APP/UTIL: CREATE RENDERER', () => {
  beforeEach(() => {
    /* eslint-disable-next-line no-unused-vars */
    createPropGetter.mockImplementation((props) => () =>
      Promise.resolve({}),
    )
  })

  it('supports string rendering', () => {
    createPropGetter.mockImplementation((props) => (req, res) =>
      Promise.resolve({
        foo: props.foo * 2,
        url: req.url,
        extra: res.locals.extra,
      }),
    )

    const renderFn = (view, props) => {
      return view === 'my-view'
        ? `foo: ${props.foo} | url: ${props.url} | extra: ${props.extra}`
        : 'other view'
    }

    const app = express()

    app.use((req, res, next) => {
      res.locals.extra = 'yes'
      next()
    })

    app.get(
      '/page',
      createRenderer(renderFn).renderView('my-view', {
        foo: 1,
      }),
    )

    return request(app)
      .get('/page')
      .expect(200)
      .expect('Content-Type', 'text/html; charset=utf-8')
      .expect('<!DOCTYPE html>\nfoo: 2 | url: /page | extra: yes')
  })

  it('should support streaming', () => {
    const renderFn = (view) =>
      ReactDOMServer.renderToStaticNodeStream(
        React.createElement('div', null, view),
      )

    const app = express()
    app.get('/page', createRenderer(renderFn).renderView('my-view'))

    return request(app)
      .get('/page')
      .expect(200)
      .expect('Content-Type', 'text/html; charset=utf-8')
      .expect('<!DOCTYPE html>\n<div>my-view</div>')
  })

  it('should handle errors getting props', () => {
    createPropGetter.mockImplementation(({ hasError }) => () =>
      hasError
        ? Promise.reject(new Error('my bad error'))
        : Promise.resolve({}),
    )

    const renderFn = (view, props) => {
      return view === 'error' ? props.error.message : 'my view'
    }

    const { renderView, renderError } = createRenderer(renderFn)

    const app = express()

    app.use(mw.setShortzMode(false))
    app.get('/page', renderView('view', { hasError: true }))
    app.use(renderError('error'))

    return request(app)
      .get('/page')
      .expect(500)
      .expect('<!DOCTYPE html>\nmy bad error')
  })

  it('should handle errors with a status', () => {
    createPropGetter.mockImplementation(({ hasError }) => () => {
      if (hasError) {
        const e = new Error('my bad error')

        e.status = 404

        return Promise.reject(e)
      }

      return Promise.resolve({})
    })

    const renderFn = (view, props) => {
      return view === 'error' ? props.error.message : 'my view'
    }

    const { renderView, renderError } = createRenderer(renderFn)

    const app = express()

    app.use(mw.setShortzMode(false))
    app.get('/page', renderView('view', { hasError: true }))
    app.use(renderError('error'))

    return request(app)
      .get('/page')
      .expect(404)
      .expect('<!DOCTYPE html>\nmy bad error')
  })

  it('should handle errors rendering', () => {
    const renderFn = (view, props) => {
      if (view === 'error') return props.error.message

      throw new Error('render error')
    }

    const { renderView, renderError } = createRenderer(renderFn)

    const app = express()
    app.use(mw.setShortzMode(false))
    app.get('/page', renderView('view'))
    app.use(renderError('error'))

    return request(app)
      .get('/page')
      .expect(500)
      .expect('<!DOCTYPE html>\nrender error')
  })

  it('should allow error specific handlers', () => {
    const renderFn = (view, props) => {
      if (view === 'error') return props.error.message

      if (view === 'error2') return 'fall through'

      throw new Error('render error')
    }

    const { renderView, renderError } = createRenderer(renderFn)

    const app = express()

    app.use(mw.setShortzMode(false))
    app.get('/page', renderView('view'))
    app.use(renderError('error', {}, () => false))
    app.use(renderError('error2'))

    return request(app)
      .get('/page')
      .expect(500)
      .expect('<!DOCTYPE html>\nfall through')
  })

  it('should handle errors in streaming render', () => {
    expect.assertions(1)
    /* eslint-disable-next-line no-unused-vars */
    const MyComponent = (props) => {
      throw new Error('render error')
    }

    const renderFn = (view, props) => {
      if (view === 'error') {
        expect(props.error.message).toEqual('render error')

        return ReactDOMServer.renderToStaticNodeStream(
          React.createElement('span', null, props.error.message),
        )
      }

      return ReactDOMServer.renderToStaticNodeStream(
        React.createElement(
          'div',
          null,
          React.createElement(MyComponent),
        ),
      )
    }

    const { renderView, renderError } = createRenderer(renderFn)

    const app = express()

    app.use(mw.setShortzMode(false))
    app.get('/page', renderView('view'))
    app.use(renderError('error'))

    return request(app)
      .get('/page')
      .expect(
        '<!DOCTYPE html>\n<!DOCTYPE html>\n<span>render error</span>',
      )
  })
})

it('should handle errors status overrride through query param', () => {
  createPropGetter.mockImplementation(({ hasError }) => () => {
    if (hasError) {
      const e = new Error('my bad error')

      e.status = 404

      return Promise.reject(e)
    }

    return Promise.resolve({})
  })

  const renderFn = (view, props) => {
    return view === 'error' ? props.error.message : 'my view'
  }

  const { renderView, renderError } = createRenderer(renderFn)

  const app = express()

  app.use(mw.setShortzMode(true))
  app.use(mw.setErrorParam(401))
  app.get('/page', renderView('view', { hasError: true }))
  app.use(renderError('error'))

  // Default error status is 404
  // But query param changes it to 401
  return request(app)
    .get('/page')
    .expect(401)
    .expect('<!DOCTYPE html>\nmy bad error')
})
