const {
  getNewsreaderWebView,
  gtSemverCompare,
  isPlayTab,
  getNewsreaderVersion,
} = require('../newsreader')

describe('APP/UTIL: NEWSREADER', () => {
  describe('getNewsreaderWebView()', () => {
    it('returns empty string if provided no arguments', () => {
      expect(getNewsreaderWebView()).toBe('')
    })

    it('returns "ios" if query param "embed" is "ios"', () => {
      expect(getNewsreaderWebView({ embed: 'ios' })).toBe('ios')
    })

    it('returns "ios" if query param "embed" is "ipad"', () => {
      expect(getNewsreaderWebView({ embed: 'ipad' })).toBe('ios')
    })

    it('returns "ios" if query param "embed" is "iphone"', () => {
      expect(getNewsreaderWebView({ embed: 'iphone' })).toBe('ios')
    })

    it('returns empty string if query param "embed" is something else', () => {
      expect(getNewsreaderWebView({ embed: 'notiPhone' })).toBe('')
    })

    it('returns "ios" if UA is from iOS Newsreader', () => {
      expect(getNewsreaderWebView({}, 'nytios/9.28.0')).toBe('ios')
    })

    it('returns "android" if query param "embed" is "android"', () => {
      expect(getNewsreaderWebView({ embed: 'android' })).toBe(
        'android',
      )
    })

    it('returns "android" if query param "embed" is "androidtab"', () => {
      expect(getNewsreaderWebView({ embed: 'androidtab' })).toBe(
        'android',
      )
    })

    it('returns "android" if query param "embed" is "androidphone"', () => {
      expect(getNewsreaderWebView({ embed: 'androidphone' })).toBe(
        'android',
      )
    })

    it('returns empty string if query param "embed" is something else', () => {
      expect(getNewsreaderWebView({ embed: 'notAndroid' })).toBe('')
    })

    it('returns "android" if UA is from Android Newsreader', () => {
      expect(getNewsreaderWebView({}, 'nyt_android/9.13')).toBe(
        'android',
      )
    })

    it('returns "android" if UA is from Android Newsreader Hybrid', () => {
      expect(getNewsreaderWebView({}, 'Reader/9.13')).toBe('android')
    })

    it('returns empty string if UA is not from the Newsreader', () => {
      expect(
        getNewsreaderWebView(
          {},
          'Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19',
        ),
      ).toBe('')
    })
  })

  describe('getNewsreaderVersion()', () => {
    test('should get android version with patch number', () => {
      const ua = 'nyt_android/9.13.1'
      expect(getNewsreaderVersion(ua)).toBe('9.13.1')
    })

    test('should get android version without patch number', () => {
      const ua = 'nyt_android/9.13'
      expect(getNewsreaderVersion(ua)).toBe('9.13')
    })

    test('should get android hybrid SB version with patch number', () => {
      const ua = 'Reader/9.70 (Android; Android 12; Build 1/release)'
      expect(getNewsreaderVersion(ua)).toBe('9.70')
    })

    test('should get android hybrid SB version without patch number', () => {
      const ua =
        'Reader/9.70.0 (Android; Android 12; Build 1/release)'
      expect(getNewsreaderVersion(ua)).toBe('9.70.0')
    })

    test('should get iOS version with patch number', () => {
      const ua =
        'Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.3.0'
      expect(getNewsreaderVersion(ua)).toBe('9.3.0')
    })

    test('should get iOS version without patch number', () => {
      const ua =
        'Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.3'
      expect(getNewsreaderVersion(ua)).toBe('9.3')
    })
  })

  describe('gtSemverCompare()', () => {
    test('return false if not provided a version', () => {
      expect(gtSemverCompare()).toBe(false)
    })

    test('return false if not provided a minVersion', () => {
      expect(gtSemverCompare('1.2.3')).toBe(false)
    })

    test('return true for supported major versions', () => {
      expect(gtSemverCompare('2.0.0', '1.0.0')).toBe(true)
    })

    test('return false for unsupported major versions', () => {
      expect(gtSemverCompare('2.0.0', '3.0.0')).toBe(false)
    })

    test('return true for supported minor versions', () => {
      expect(gtSemverCompare('1.2.0', '1.1.0')).toBe(true)
    })

    test('return false for unsupported minor versions', () => {
      expect(gtSemverCompare('1.2.0', '1.3.0')).toBe(false)
    })

    test('return true for supported patch versions', () => {
      expect(gtSemverCompare('2.0.2', '2.0.1')).toBe(true)
    })

    test('return false for unsupported patch versions', () => {
      expect(gtSemverCompare('2.0.2', '2.0.3')).toBe(false)
    })

    test('return true for current version with patch and min version without', () => {
      expect(gtSemverCompare('4.8.0', '4.7')).toBe(true)
    })

    test('return true for current version without patch and min version with patch', () => {
      expect(gtSemverCompare('4.8', '4.7.0')).toBe(true)
    })

    test('return true for minimum version with greater minor than current version', () => {
      expect(gtSemverCompare('5.7', '4.8')).toBe(true)
    })

    test('return true for minimum version with greater patch than current version', () => {
      expect(gtSemverCompare('4.8.0', '4.7.3')).toBe(true)
    })
  })

  describe('isPlayTab()', () => {
    test('return false if not provided anything', () => {
      expect(isPlayTab()).toBe(false)
    })

    test('return true if provided "entry" query param and UA is from iOS Newsreader', () => {
      expect(isPlayTab({ entry: 'playTab' }, 'nytios/9.28.0')).toBe(
        true,
      )
    })
  })
})
