const querystring = require('querystring')

const {
  ACCOUNT_URL,
  SUBSCRIPTION_CAMPAIGNS,
  SUBSCRIBE_URL,
  NATIVE_SCHEME_GAMES,
  NATIVE_SCHEME_NEWSREADER,
  NATIVE_SUBSCRIBE_PATH,
  NATIVE_SUBSCRIBE_PATH_ANDROID,
  NATIVE_LOGIN_PATH,
  NATIVE_REGI_PATH,
  APPLICATION_HOST_CLIENT,
  XWD_SUBMISSION_URL,
} = require('../../constants')

const {
  gtSemverCompare,
  IOS_VERSION_SUPPORTS_SUB_VIEW,
  ANDROID_VERSION_SUPPORTS_SUB_VIEW,
} = require('./newsreader')

const campaignKeys = Object.keys(SUBSCRIPTION_CAMPAIGNS)

/**
 * Determine if we should render the native link or not. Based on things
 * like feature flag and current device, which is derived from User Agent
 * string parsing.
 *
 * @param {Object} deviceType Information about the current device
 * @returns {Bool} If we should render a native subscription URL or not
 */
const shouldRenderNativeLink = (deviceType) => {
  if (deviceType.gamesAppPlatform) {
    return true
  }

  if (
    deviceType.newsreaderAppPlatform === 'ios' &&
    gtSemverCompare(
      deviceType.newsreaderAppVersion,
      IOS_VERSION_SUPPORTS_SUB_VIEW,
    )
  ) {
    return true
  }

  if (
    deviceType.newsreaderAppPlatform === 'android' &&
    gtSemverCompare(
      deviceType.newsreaderAppVersion,
      ANDROID_VERSION_SUPPORTS_SUB_VIEW,
    )
  ) {
    return true
  }

  return false
}

/**
 * Reduce the campaign URLs based on price key.
 *
 * @param {String} priceKey The pricing key, should match key in constants. Ex: halfPrice|fullPrice
 * @returns {Object} A map of all subscribe URLs, keyed on campaign key.
 */
const makePriceLinks = (priceKey) =>
  campaignKeys.reduce((urls, campaignKey) => {
    return Object.assign(urls, {
      [campaignKey]: SUBSCRIPTION_CAMPAIGNS[campaignKey][priceKey],
    })
  }, {})

/**
 * Determine whether to use nytimes:// (Newsreader) or nytxwd:// (Games)
 * for deep links.
 *
 * @param {Object} deviceType Information about the current device
 * @param {String} path Where we want to go
 * @returns {String} The full deep link, such as 'nytxwd://login'
 */
const getNativeLink = (deviceType, path) => {
  const scheme = deviceType.gamesAppPlatform
    ? NATIVE_SCHEME_GAMES
    : NATIVE_SCHEME_NEWSREADER

  return `${scheme}${path}`
}

/**
 * Create subscribe links based on some current request, device, and user data. Generally, the
 * returned value will be a reduced version of constants.js's SUBSCRIPTION_CAMPAIGNS.
 *
 * But, if we need to render a native link, well swap the URL host strings
 * Ex:
 *   { ..., spellingBeePaywall: 'https://www.nytimes.com/subscription/games?campaignId=7RJKF', ... }
 *   ->
 *   { ..., spellingBeePaywall: 'nytimes://products?campaignId=7RJKF', ... }
 *
 * These links are utilized throughout the server side render, and then provided to the client
 * for any client-side rendering.
 *
 * @param {Object} deviceType Device type info
 * @param {Bool} qualifiesForHalfOff Whether a user qualifies for half off pricing
 * @returns {Object} A map of all subscribe URLs, keyed on campaign key.
 */
const createSubscribeLinks = (deviceType, qualifiesForHalfOff) => {
  const priceLinks = makePriceLinks(
    qualifiesForHalfOff ? 'halfPrice' : 'fullPrice',
  )

  if (shouldRenderNativeLink(deviceType)) {
    return campaignKeys.reduce((replacedUrls, campaignKey) => {
      return Object.assign(replacedUrls, {
        [campaignKey]: priceLinks[campaignKey].replace(
          SUBSCRIBE_URL,
          getNativeLink(
            deviceType,
            deviceType.newsreaderAppPlatform === 'android'
              ? NATIVE_SUBSCRIBE_PATH_ANDROID
              : NATIVE_SUBSCRIBE_PATH,
          ),
        ),
      })
    }, {})
  }

  return priceLinks
}

/**
 * Create a bunch of navigation links based on any request data.
 *
 * These links are utilized throughout the server side render, and then provided to the client
 * for any client-side rendering.
 */
module.exports = function createNavigationLinks(req, res) {
  // We've disabled the half off promotion
  // See https://jira.nyt.net/browse/XWD-14263
  const qualifiesForHalfOff = false
  const fullUrl = APPLICATION_HOST_CLIENT + req.originalUrl

  const boomerang = querystring.stringify({
    redirect_uri: fullUrl, // querystring encodes the
    response_type: 'cookie',
    client_id: 'games',
    application: 'crosswords',
  })

  const CROSSWORDS = '/crosswords'
  const PUZZLES = '/puzzles'
  const GAMES = '/games'
  const HOME = 'https://www.nytimes.com'

  const webLoginUrl = `${ACCOUNT_URL}/auth/enter-email?${boomerang}`
  const webRegiUrl = `${ACCOUNT_URL}/auth/register?${boomerang}`

  const useNativeLink = shouldRenderNativeLink(res.locals.deviceType)
  const loginLink = useNativeLink
    ? getNativeLink(res.locals.deviceType, NATIVE_LOGIN_PATH)
    : webLoginUrl

  const registerLink = useNativeLink
    ? getNativeLink(res.locals.deviceType, NATIVE_REGI_PATH)
    : webRegiUrl

  return {
    // Account:
    subscribe: createSubscribeLinks(
      res.locals.deviceType,
      qualifiesForHalfOff,
    ),
    login: loginLink,
    logout: `${ACCOUNT_URL}/auth/logout?${querystring.stringify({
      redirect_uri: fullUrl,
    })}`,
    register: registerLink,
    account: 'https://account.nytimes.com',
    // Crosswords:
    hub: CROSSWORDS,
    archive: `${CROSSWORDS}/archive`,
    daily: `${CROSSWORDS}/game/daily`,
    mini: `${CROSSWORDS}/game/mini`,
    apps: `${CROSSWORDS}/apps`,
    // Puzzles:
    stats: `${PUZZLES}/stats`,
    leaderboards: `${PUZZLES}/leaderboards`,
    spellingBee: `${PUZZLES}/spelling-bee`,
    spellingBeePuzzleHub: `${PUZZLES}/spelling-bee/hub`,
    wordle: `${GAMES}/wordle/index.html`,
    tiles: `${PUZZLES}/tiles`,
    letterBoxed: `${PUZZLES}/letter-boxed`,
    vertex: `${PUZZLES}/vertex`,
    sudoku: `${PUZZLES}/sudoku`,
    digits: `${GAMES}/digits`,
    connections: `${GAMES}/connections`,
    submission: XWD_SUBMISSION_URL,
    sbForum: `${HOME}/spotlight/spelling-bee-forum`,
    // Internal
    home: HOME,
    wordplay: `${HOME}/column/wordplay`,
    guide: `${HOME}/guides/crosswords/how-to-solve-a-crossword-puzzle`,
    gift: `${HOME}/subscription/games/gift`,
    privacy: `${HOME}/privacy/privacy-policy`,
    faq: 'https://help.nytimes.com/hc/en-us/sections/360011158491-NYT-Games ',
    shop: 'https://store.nytimes.com/collections/games',
    termsOfService:
      'https://help.nytimes.com/hc/en-us/articles/115014893428-Terms-of-service',
    sitemap: 'https://www.nytimes.com/sitemap/',
    // External
    company: 'https://www.nytco.com/',
    feedback: 'mailto:nytgames@nytimes.com',
    twitter: 'https://twitter.com/NYTGames',
  }
}
