const createPropGetter = require('./createPropGetter')

const preamble = '<!DOCTYPE html>\n'

module.exports = function createRenderer(renderFn, errorHandlers) {
  const createRenderMiddleware = (view, props = {}, errorFilter) => {
    /** Make prop getter once up front */
    const getProps = createPropGetter(props)

    const renderView = (req, res, next) => {
      getProps(req, res)
        .then((props) => {
          /** expose error status to view */
          if (res.locals.error) {
            if (!res.headersSent) {
              res.status(res.locals.error.status)
            }
            props.error = res.locals.error
          }

          /** render to string or stream */
          const output = renderFn(view, props)

          if (output.on) {
            /** Pipe stream to browser immediately */
            if (!res.headersSent) {
              res.set('content-type', 'text/html; charset=utf-8')
            }
            res.write('<!DOCTYPE html>\n')
            output.on('error', (err) => {
              next(err)
            })
            output.pipe(res)
          } else {
            /** Send entire response */
            res.send(preamble + output)
          }
        })
        .catch(next)
    }

    if (errorFilter) {
      const renderError = (err, req, res, next) => {
        err.status = err.status || 500

        /** For QA to test different error page statuses */
        const { inShortzMode } = res.locals.userType
        const { error = null } = req.query
        const errorParam = error && parseInt(req.query.error)
        const isValidParam = [401, 404, 500].includes(errorParam)
        if (inShortzMode && isValidParam) err.status = errorParam

        res.locals.error = err
        if (errorFilter(err)) {
          return renderView(req, res, next)
        }
        next(err)
      }

      return errorHandlers
        ? [...errorHandlers, renderError]
        : renderError
    }

    return renderView
  }

  return {
    renderView: createRenderMiddleware,
    renderError: (view, props, errorFilter) =>
      createRenderMiddleware(
        view,
        props,
        errorFilter || (() => true),
      ),
  }
}
