/**
 * The iOS Newsreader version that begins support of the native
 * subscription flow. Used to render a native button in the web view.
 */
const IOS_VERSION_SUPPORTS_SUB_VIEW = '9.40.0'
const ANDROID_VERSION_SUPPORTS_SUB_VIEW = '9.46'

/**
 * Determine if the user agent indicates we're in the Android Newsreader WebView.
 * Special case: for hybrid spelling-bee, the user agent will include 'Reader/'
 *
 * Example UAs:
 * "nyt_android/9.13"
 * "Reader/9.13"
 *
 * @param {String} userAgent
 * @returns {Bool}
 */
function isAndroidNewsreaderWebView(userAgent = '') {
  const androidRegex = /nyt[-_]?android/i
  const sbUserAgent = 'Reader/'
  return (
    userAgent.includes(sbUserAgent) || androidRegex.test(userAgent)
  )
}

/**
 * Determine if the user agent indicates we're in the iOS Newsreader WebView.
 *
 * Example UAs:
 * "Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.33.0"
 * "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.33.0"
 *
 * @param {String} userAgent
 * @returns {Bool}
 */
function isIOSNewsreaderWebView(userAgent = '') {
  const iOSRegex = /nyt[-_]?ios/i
  return iOSRegex.test(userAgent)
}

/**
 * Get the Android Newsreader version number from the user agent.
 * Special case: for hybrid spelling-bee, the user agent will include 'Reader/'
 *
 * Example UA: "nyt_android/9.13"
 * Example SB: "Reader/9.70.0 (Android; Android 12; Build 1/release)"
 *
 * @param {String} userAgent
 * @returns {String}
 */
function getAndroidNewsreaderVersion(userAgent = '') {
  const sbRegex = /Reader\/([0-9]+\.[0-9]+(\.[0-9]+)?)/i
  const sbMatches = userAgent.match(sbRegex) // matches will be ['Reader/9.70.0', '9.70.0']

  if (sbMatches && sbMatches.length >= 2) {
    return sbMatches[1]
  }

  const androidRegex = /nyt[-_]?android\/([0-9]+\.[0-9]+(\.[0-9]+)?)/i
  const matches = userAgent.match(androidRegex) // matches will be ['nyt_android/9.13', '9.13']

  if (matches && matches.length >= 2) {
    return matches[1]
  }

  return ''
}

/**
 * Get the iOS Newsreader version number from the user agent.
 *
 * Example UAs:
 * "Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.33.0"
 * "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) nytios/9.33.0"
 *
 * @param {String} userAgent
 * @returns {String}
 */
function getIOSNewsreaderVersion(userAgent = '') {
  const iOSRegex = /nyt[-_]?ios\/([0-9]+\.[0-9]+(\.[0-9]+)?)/i
  const matches = userAgent.match(iOSRegex) // matches will be ['nytios/9.33.0', '9.33.0']

  if (matches && matches.length >= 2) {
    return matches[1]
  }

  return ''
}

/**
 * Get the newsreader version.
 *
 * @param {String} userAgent
 * @returns {String} The version
 */
const getNewsreaderVersion = (userAgent) =>
  getIOSNewsreaderVersion(userAgent) ||
  getAndroidNewsreaderVersion(userAgent) ||
  ''

/**
 * Determine if we are in a WebView. We'll look at a few different things here
 * and short-circuit evaluate to avoid doing more progressively expensive or
 * sketchy things.
 *
 *
 * Some helpful reading from hybrid folks:
 * https://docs.google.com/document/d/10PNzQxXIHImtuh2c-VLRuH7yUGDgCh7zxqMrYMxg1eQ
 *
 * @param {Object} queryParams
 * @param {String} userAgent
 * @returns {String} The Newsreader platform ios|android
 */
function getNewsreaderWebView(queryParams = {}, userAgent = '') {
  const iosQueryParams = ['ios', 'ipad', 'iphone']
  const androidQueryParams = ['android', 'androidtab', 'androidphone']

  if (
    iosQueryParams.includes(queryParams.embed) ||
    isIOSNewsreaderWebView(userAgent)
  ) {
    return 'ios'
  }

  if (
    androidQueryParams.includes(queryParams.embed) ||
    isAndroidNewsreaderWebView(userAgent)
  ) {
    return 'android'
  }

  return ''
}

/**
 * Compare if the provided version is greater than the provided minVersion.
 *
 * @param {String} version The desired version to compare against the minVersion
 * @param {String} minVersion The minimum version to compare
 * @returns {Bool} True if the version is greater than the minVersion
 */
function gtSemverCompare(version, minVersion) {
  /* eslint-disable */
  if (!version || !minVersion) {
    return false
  }

  const [major, minor, patch = 0] = version.split('.').map(Number)
  const [minMajor, minMinor, minPatch = 0] = minVersion
    .split('.')
    .map(Number)

  // Compare major versions and if neither check returns
  // It means major and minMajor are equal
  if (major > minMajor) return true
  if (major < minMajor) return false

  // Now compare minor versions and if neither check returns
  // It means minor and minMinor are equal
  if (minor > minMinor) return true
  if (minor < minMinor) return false

  // Now compare patch versions and if neither check returns
  // It means patch and minPatch are equal
  if (patch > minPatch) return true
  if (patch < minPatch) return false

  // If we haven't returned yet, that means that version and minVersion are equal
  return true

  /* eslint-enable */
}

/**
 * Determines if a request originated from the iOS Newsreader Play tab.
 *
 * @param {Object} queryParams
 * @param {String} userAgent
 * @returns {Bool}
 */
function isPlayTab(queryParams = {}, userAgent = '') {
  const playTabParam = 'playTab'

  if (
    queryParams.entry === playTabParam &&
    getNewsreaderWebView(queryParams, userAgent) === 'ios'
  ) {
    return true
  }

  return false
}

module.exports = {
  IOS_VERSION_SUPPORTS_SUB_VIEW,
  ANDROID_VERSION_SUPPORTS_SUB_VIEW,
  getNewsreaderWebView,
  getNewsreaderVersion,
  gtSemverCompare,
  isPlayTab,
}
