// Parses the necessary directives from the PURR header
// per the format here: https://purr.tech.nyt.net/purr-cookie-schema/

const PURR_DIRECTIVE_FORMAT = {
  PURR_DataSaleOptOutUI_v2: {
    index: 7,
    values: {
      h: 'hide',
      s: 'show',
      o: 'show-opted-out',
    },
    default: 'hide',
    queryParam: 'opt-out',
  },
  PURR_CaliforniaNoticesUI: {
    index: 8,
    values: {
      h: 'hide',
      s: 'show',
    },
    default: 'hide',
    queryParam: 'california-notices',
  },
  PURR_AdConfiguration_v3: {
    index: 11,
    values: {
      f: 'full',
      r: 'rdp',
      n: 'npa',
      s: 'adluce-socrates',
      l: 'ltd',
    },
    default: 'full',
    queryParam: 'ad-config',
  },
  PURR_LimitSensitivePI: {
    index: 12,
    values: {
      h: 'hide',
      s: 'show',
    },
    default: 'hide',
    queryParam: 'limit-sensitive-information',
  },
  PURR_TOSBlocker_v1: {
    index: 15,
    values: {
      h: 'hide',
      s: 'show',
    },
    default: 'hide',
    queryParam: 'tos-blocker',
  },
  PURR_FidesTCF: {
    index: 16,
    values: {
      h: 'hide',
      s: 'show',
    },
    default: 'hide',
    queryParam: 'fides-tcf',
  },
}

/**
 * Returns an object of directive -> value.
 *
 * @param {string} header PURR header
 * @returns {Object} A mapping of directives to values.  Undefined directive values will remain as such.  A malformed or missing header will produce default values.
 */
const parsePurrDirectiveHeader = (req) => {
  const header = req.headers['x-nyt-purr-directives']
  return Object.keys(PURR_DIRECTIVE_FORMAT).reduce((ds, key) => {
    const directive = PURR_DIRECTIVE_FORMAT[key]
    try {
      const value =
        req.query[directive.queryParam] ||
        directive.values[header[directive.index]]

      return {
        ...ds,
        [key]: value,
      }
    } catch (e) {
      return {
        ...ds,
        [key]: directive.default,
      }
    }
  }, {})
}

module.exports = {
  PURR_DIRECTIVE_FORMAT,
  parsePurrDirectiveHeader,
}
