const moment = require('moment')

const StatusError = require('./StatusError')

const adaptPuzzle = (puzz) => {
  // spelling bee will break without a pangrams key,
  // so an empty array is set as a backup
  const pangrams = puzz.pangrams || []
  try {
    const date = moment(puzz.print_date, 'YYYY-M-D')
    return {
      expiration: puzz.expiration,
      displayWeekday: date.format('dddd'),
      displayDate: date.format('MMMM D, YYYY'),
      printDate: puzz.print_date,
      centerLetter: puzz.center_letter,
      outerLetters: puzz.outer_letters.split(''),
      validLetters: [puzz.center_letter].concat(
        puzz.outer_letters.split(''),
      ),
      pangrams,
      answers: pangrams.concat(puzz.answers),
      id: puzz.id,
      isFree: puzz.is_free,
      freeExpiration: puzz.free_expiration
        ? moment(puzz.free_expiration * 1000).format('MMMM Do.')
        : 0,
      editor: puzz.editor,
    }
  } catch (e) {
    throw new Error(`failed to adapt ${JSON.stringify(puzz)}`)
  }
}

const adaptActivePuzzles = (activeRes) => {
  try {
    const puzzles = activeRes.puzzles.map(adaptPuzzle)

    const getPuzzlesFromIndices = (indices) =>
      indices.map((idx) => puzzles[idx]).filter((puzz) => !!puzz)

    return {
      today: puzzles[activeRes.today],
      yesterday: puzzles[activeRes.yesterday],
      thisWeek: getPuzzlesFromIndices(activeRes.thisWeek),
      lastWeek: getPuzzlesFromIndices(activeRes.lastWeek),
    }
  } catch (e) {
    throw new Error(
      `failed to adapt past puzzles ${JSON.stringify(activeRes)}`,
    )
  }
}

const SVC_ROOT = 'svc/spelling-bee/v1/'
const ACTIVE = `${SVC_ROOT}active.json`

const getActivePuzzles = (api) =>
  api.get(ACTIVE).then(({ data }) => adaptActivePuzzles(data))

const validDate = /\d{4}-\d{2}-\d{2}/

const getShortzPuzzle = async (api, date) => {
  if (!validDate.test(date)) {
    return Promise.reject(new StatusError('Invalid date', 404))
  }

  try {
    const { data } = await api.get(`${SVC_ROOT}${date}.json`)
    const puzz = adaptPuzzle(data, true)
    return {
      today: puzz,
      yesterday: puzz,
      isPastPuzzle: false,
    }
  } catch (e) {
    if (e.status === 401) {
      return Promise.reject(
        new StatusError(`restricted SB ${date}`, 404),
      )
    }

    return Promise.reject(e)
  }
}

const getPuzzleByDate = async (api, date) => {
  if (!validDate.test(date)) {
    return Promise.reject(new StatusError('Invalid date', 404))
  }

  const activePuzzleRes = await getActivePuzzles(api)

  const activePuzzles = [
    ...activePuzzleRes.thisWeek,
    ...activePuzzleRes.lastWeek,
  ]

  const activePuzzleForRoute = activePuzzles.find(
    (p) => p.printDate === date,
  )

  // inside active range
  if (activePuzzleForRoute) {
    return {
      today: activePuzzleForRoute,
      yesterday: null,
      pastPuzzles: activePuzzleRes,
      isPastPuzzle: true,
    }
  }

  return Promise.reject(
    new StatusError(`Date ${date} outside active range`, 404),
  )
}

module.exports = {
  getActivePuzzles,
  getPuzzleByDate,
  getShortzPuzzle,
}
