import React from 'react'
import { getFidesSnippet } from '@nyt/fides'

import { withServerData } from './withServerData'
import { ORG_JSON_LD } from './Constants'

export const CacheSafeHead = withServerData(
  ({ env, metadata, assets, titleOverride }) => (
    <head>
      <title>
        {titleOverride || metadata.title} - The New York Times
      </title>
      <meta
        name="viewport"
        content="width=device-width, initial-scale=1, minimal-ui"
      />
      <meta
        httpEquiv="Content-Type"
        content="text/html"
        charSet="utf-8"
      />
      <meta httpEquiv="X-UA-Compatible" content="IE=edge,chrome=1" />
      <meta name="version" content={env.version} />

      <meta name="CG" content={metadata.contentGroup} />
      <meta name="PT" content={metadata.pageType} />

      {metadata.linkTags?.map((linkProps, idx) => (
        <link key={idx} {...linkProps} />
      ))}

      {metadata.metaTags?.map((tagProps, idx) => (
        <meta key={idx} {...tagProps} />
      ))}

      {metadata.metaTags?.map(
        (metaTag) =>
          metaTag.property &&
          metaTag.property === 'og:url' && (
            <link rel="canonical" href={metaTag.content} />
          ),
      )}

      {metadata?.pageType === 'puzzle hub' ? (
        // for home page, Organization json metadata, see https://schema.org/Organization
        <>
          <script
            type="application/ld+json"
            dangerouslySetInnerHTML={{
              __html: JSON.stringify(ORG_JSON_LD),
            }}
          />
          <script
            type="application/ld+json"
            dangerouslySetInnerHTML={{
              __html: JSON.stringify({
                '@type': 'Game',
                '@context': 'http://schema.org',
                inLanguage: 'en-US',
                description: `${metadata?.title}`,
              }),
            }}
          />
        </>
      ) : (
        // for other game pages, Game json metadata, see https://schema.org/Game
        <script
          type="application/ld+json"
          dangerouslySetInnerHTML={{
            __html: JSON.stringify({
              '@type': 'Game',
              '@context': 'http://schema.org',
              inLanguage: 'en-US',
              description: `${metadata?.title}`,
              alternateName: metadata?.alternateName,
              image: metadata?.image,
              sourceOrganization: {
                '@context': 'http://schema.org',
                '@type': 'Organization',
                name: 'The New York Times',
                url: 'https://www.nytimes.com/',
                '@id': 'https://www.nytimes.com/',
                sameAs: [
                  'https://www.facebook.com/nytimes/',
                  'https://twitter.com/nytimes',
                  'https://www.instagram.com/nytimes/',
                  'https://www.youtube.com/user/TheNewYorkTimes',
                  'https://www.linkedin.com/company/the-new-york-times',
                  'https://www.wikidata.org/wiki/Q9684',
                  'https://en.wikipedia.org/wiki/The_New_York_Times',
                ],
                logo: {
                  '@context': 'http://schema.org',
                  '@type': 'ImageObject',
                  url: 'https://static01.nyt.com/images/misc/NYT_logo_rss_250x40.png',
                  height: 40,
                  width: 250,
                },
              },
            }),
          }}
        />
      )}

      {assets.styles?.map((css, i) => {
        if (css.src) {
          return (
            <link
              key={i}
              rel="stylesheet"
              type="text/css"
              href={css.src}
            />
          )
        }

        if (css.inline) {
          return (
            <style
              key={i}
              id={css.id}
              // eslint-disable-next-line react/no-danger
              dangerouslySetInnerHTML={{
                __html: css.inline,
              }}
            />
          )
        }
        throw new Error(`bad style config ${JSON.stringify(css)}`)
      })}

      {assets.prefetch?.map((asset, i) => {
        if (asset.src) {
          return (
            <link
              key={i}
              rel="preload"
              href={asset.src}
              as={asset.as}
            />
          )
        }
        throw new Error(
          `bad prefetch config ${JSON.stringify(asset)}`,
        )
      })}

      <script
        type="text/javascript"
        dangerouslySetInnerHTML={{
          __html: getFidesSnippet(),
        }}
      />
    </head>
  ),
)

export default CacheSafeHead
