import {
  allAccessSaleStartDate,
  allAccessSaleEndDate,
  urgencyMessages as allAccessMessages,
} from './configs/allAccessSale'
import {
  games_sale_cta_flag,
  games_sale_banner_flag,
  games_sale_active_variant,
  games_sale_nav_flag,
  gamesSaleStartDate,
  gamesSaleEndDate,
  urgencyConfig as gamesSaleConfig,
} from './configs/gamesSale'

/**
 * Used in non-cached Page component. Checks to see if the Abra tests that are currently
 * active contain a test and variant. It does not check if the user meets that variant,
 * just that the test is set to Active in the Abra UI and that a variant exists in that test
 */
export function findInAbraConfig(abraConfig, testName, variantName) {
  const test = abraConfig?.[testName]

  let abtest = []
  test?.if?.forEach((item) => {
    if (item.hasOwnProperty('abtest_partition')) {
      abtest = item.abtest_partition
    }
  })
  return abtest && abtest?.includes(variantName)
}

/**
 * Checks to see if the Games Sale banner test has been set to active in the config and
 * the variant exists in that config. This separation allows us to toggle portions of the
 * games sale on and off if things are buggy.
 */
export function findGamesSaleBannerTest(abraConfig) {
  return findInAbraConfig(
    abraConfig,
    games_sale_banner_flag,
    games_sale_active_variant,
  )
}

/**
 * Checks to see if the Games Sale nav test has been set to active in the config and
 * the variant exists in that config. This separation allows us to toggle portions of the
 * games sale on and off if things are buggy.
 */
export function findGamesSaleNavTest(abraConfig) {
  return findInAbraConfig(
    abraConfig,
    games_sale_nav_flag,
    games_sale_active_variant,
  )
}

/**
 * Checks to see if the Games Sale cta test has been set to active in the config and
 * the variant exists in that config. This separation allows us to toggle portions of the
 * games sale on and off if things are buggy.
 */
export function findGamesSaleCtaTest(abraConfig) {
  return findInAbraConfig(
    abraConfig,
    games_sale_cta_flag,
    games_sale_active_variant,
  )
}

/**
 * Used to determine which CTA messaging to show
 */
export function isSub(entitlement) {
  return entitlement === 'sub' || entitlement === 'cr'
}

/**
 * Allows devs to develop and test the sale code by overriding the current date
 */
export function getDateOverride(url) {
  if (url && url.search) {
    const paramIndex = url.search('date-override')
    const params = url.substring(paramIndex)
    const urlParams = new URLSearchParams(params)
    return urlParams.get('date-override')
  }
}

/**
 * Determines if the date range falls within the AA Sale dates.
 * Checks the url for a date override param.
 */
export const isAASaleActive = (url) => {
  // Checks for `date-override` URL param during testing
  const dateOverride = getDateOverride(url)
  const date =
    new Date(dateOverride).getTime() || new Date().getTime()

  return date >= allAccessSaleStartDate && date < allAccessSaleEndDate
}

/**
 * Determines if the date range falls within the Games Sale dates.
 * Checks the url for a date override param.
 */
export function isGamesSaleActive(url) {
  // Checks for `date-override` URL param during testing
  const dateOverride = getDateOverride(url) || null
  const date =
    new Date(dateOverride).getTime() || new Date().getTime()

  return date >= gamesSaleStartDate && date < gamesSaleEndDate
}

/**
 * Determines urgency messaging for banners using the AA Sale config.
 */
export function getAASaleUrgencyMessage(url) {
  // Checks for `date-override` URL param during testing
  const dateOverride = getDateOverride(url)
  const date =
    new Date(dateOverride).getTime() || new Date().getTime()

  const urgencyMessage = allAccessMessages.find(
    ({ startDate }) =>
      date >= startDate && date < allAccessSaleEndDate,
  )

  return urgencyMessage ? urgencyMessage.copy : ''
}

/**
 * Determines urgency messaging for banners using the Games Sale config. Also determines
 * color changes based on dates.
 */
export function getGamesSaleBannerUrgency(url) {
  // Checks for `date-override` URL param during testing
  const dateOverride = getDateOverride(url)
  const date =
    new Date(dateOverride).getTime() || new Date().getTime()

  const urgency = gamesSaleConfig.urgency.find(
    ({ startDate }) => date >= startDate && date < gamesSaleEndDate,
  )
  const urgencyMessage = urgency?.copy || ''
  const shortUrgencyMessage = urgency?.shortCopy || ''
  const bannerColor = urgency?.color || '#000'

  return {
    urgencyMessage,
    shortUrgencyMessage,
    bannerColor,
  }
}

/**
 * Used in the navigation to show Games Sale acq messaging for anons and regis and
 * upgrade messaging for non-AA subs
 */
export function getButtonText(entitlement) {
  return isSub(entitlement)
    ? gamesSaleConfig.upgradeMessage
    : gamesSaleConfig.acqMessage
}
