import React from 'react'
import pt from 'prop-types'

import {
  findGamesSaleCtaTest,
  findGamesSaleNavTest,
  isGamesSaleActive,
  isAASaleActive,
  isSub,
} from './Sale/functions'
import TopSaleBanner from './Sale/TopSaleBanner'
import { GameSalesBanner } from './Sale/GameSalesBanners'
import { MessagingBanner } from './Messaging/MessagingBanner'
import FreeGameBanner from './Free/FreeGameBanner'

/**
 * Using the URL, device, and user info to determine if the AA Sale Banner should render
 */
export const determineTopSaleBanner = ({
  url,
  userType,
  deviceType,
}) => {
  return (
    // The date range and url matches
    isAASaleActive(url) &&
    // Web only
    deviceType.gamesAppPlatform === '' &&
    deviceType.newsreaderAppPlatform === '' &&
    // Not an ADA subscriber
    !(userType.hasXwd && userType.hasDigi)
  )
}

/**
 * Determine the CSS classes that need to be added to the body tag for the banners
 */
export const getSaleBodyClasses = ({
  url,
  userGeo,
  abraConfig,
  userType,
  deviceType,
}) => {
  const userEntitlement = userType?.entitlement ?? ''
  const isGamesSale =
    isGamesSaleActive(url) && userEntitlement !== 'sub,cr'

  // Is the AA sale banner going to show
  const isTopSaleBanner = determineTopSaleBanner({
    url,
    userType,
    deviceType,
  })

  /**
   * These pages don't have AbraProvider or sometimes UserProvider so we have to manually
   * add the sale classes to the body as opposed to letting our abra library do it
   */
  const pagesWithoutProvider =
    url?.includes('stats') ||
    url?.includes('leaderboards') ||
    url?.includes('crosswords/archive')

  // Is the Games Sale CTA going to show in the nav
  const isGamesSaleAbraNav =
    // The url and date range matches
    isGamesSale &&
    // The user is not in India
    userGeo !== 'IN' &&
    // The abra config contains the test name for the sale
    findGamesSaleNavTest(abraConfig)

  // Is the Games Sale CTA going to show in Stats, Leaderboard, and Xword Archive
  const isGamesSaleAbraCta =
    // The url and date range matches
    isGamesSale &&
    // The user is not in India
    userGeo !== 'IN' &&
    // The abra config contains the test name for the sale
    findGamesSaleCtaTest(abraConfig)

  return {
    'pz-sale-banner': isTopSaleBanner,
    'pz-games-sale-nav': pagesWithoutProvider && isGamesSaleAbraNav,
    'pz-games-sale-cta': pagesWithoutProvider && isGamesSaleAbraCta,
    'pz-games-sale-sub': isSub(userEntitlement),
  }
}

/**
 * When a business sponsors a puzzle to play for free on the hub, the FreeGameBanner will show
 * and these properties will be set on the hub state
 */
export const getFreeGameBannerConfig = ({
  hubPuzzles,
  userType,
  url,
}) => {
  if (
    hubPuzzles &&
    hubPuzzles.free_products &&
    hubPuzzles.free_products.length > 0
  ) {
    const freeProduct = hubPuzzles.free_products[0]
    return {
      isSubscriber: userType?.hasXwd,
      referrer: url,
      game: freeProduct.name,
      expiration: freeProduct.expiration,
    }
  }
}

/**
 * Contains the logic for determining which banners to show at the top of uncached pages
 * - Free Game Banner
 * - AA Sale Banner
 * - Games Sale Banner
 * - Delta Banner
 */
const UniversalTopBanner = ({
  url,
  deviceType,
  userType,
  hubPuzzles,
  messaging,
  subscribeUrl,
  navigationLinks,
}) => {
  const freeGameBannerConfig = getFreeGameBannerConfig({
    hubPuzzles,
    userType,
    url,
  })

  const isTopSaleBanner = determineTopSaleBanner({
    url,
    userType,
    deviceType,
  })

  const isGamesSale = isGamesSaleActive(url)

  // Only show the Delta banner if the AA Sale and Games sale are inactive and the user is anon
  const canShowDeltaBanner =
    !isTopSaleBanner && !isGamesSale && !userType?.isLoggedIn

  return (
    <>
      {canShowDeltaBanner && <div id="delta-banner-portal" />}
      {messaging && <MessagingBanner {...messaging} />}
      {isTopSaleBanner && (
        <TopSaleBanner navLinks={navigationLinks} url={url} />
      )}
      {isGamesSale && (
        <GameSalesBanner
          subscribeUrl={subscribeUrl}
          url={url}
          userType={userType}
        />
      )}
      {freeGameBannerConfig && (
        <FreeGameBanner {...freeGameBannerConfig} />
      )}
    </>
  )
}

UniversalTopBanner.propTypes = {
  userType: pt.shape({
    isLoggedIn: pt.bool,
    hasDigi: pt.bool,
    hasXwd: pt.bool,
  }),
  url: pt.string,
  deviceType: pt.shape({
    gamesAppPlatform: pt.string,
    newsreaderAppPlatform: pt.string,
  }),
  userGeo: pt.string,
  abraConfig: pt.object,
  hubPuzzles: pt.object,
  messaging: pt.object,
  subscribeUrl: pt.string,
  navigationLinks: pt.shape({
    login: pt.string,
  }),
}

export default UniversalTopBanner
