import React from 'react'
import cn from 'classnames'

import {
  THE_CROSSWORD,
  THE_MINI,
  LEADERBOARDS,
  WORDPLAY,
  SPELLING_BEE,
  SB_PUZZLE_HUB,
  LETTER_BOXED,
  TILES,
  SUDOKU,
  VERTEX,
  DIGITS,
  CONNECTIONS,
  ALL_GAMES,
  STATS,
  CROSSWORD_ARCHIVES,
  HOW_TO_SOLVE,
  WORDLE,
} from 'hub/track/constants'
import { appendSupportedParams, jsHook } from 'views/util'

import HybridBackButton from '../../../shared/components/HybridBackButton'

import { withServerData } from './withServerData'
import { NavLogo } from './NavLogo'
import { getButtonText } from './Marketing/Sale/functions'

const CONNECTIONS_ON =
  new Date() >= new Date('6/12/2023 10:00 AM EDT')

export const Nav = withServerData(
  ({
    userType,
    navigationLinks,
    isGamePage,
    isHybridWebView,
    isPlayTab,
    compliance,
    deviceType,
    url,
    isTopSaleBanner,
  }) => {
    const { isLoggedIn } = userType

    const loginUrl = appendSupportedParams(
      navigationLinks.login,
      '&asset=navigation-bar',
    )

    const showCompliance =
      !isHybridWebView &&
      deviceType.isMobile &&
      [
        compliance.PURR_CaliforniaNoticesUI,
        compliance.PURR_DataSaleOptOutUI_v2,
      ].some((c) => c !== 'hide')

    const subscribeButton =
      !userType.hasXwd || isHybridWebView ? (
        <a
          href={navigationLinks.subscribe.default}
          className={`pz-nav__button default ${jsHook(
            'nav-subscribe',
          )}`}
          data-track-label="subscribe-nav"
        >
          Subscribe
        </a>
      ) : null

    // Hamburger CTA should only show on Web
    // Sale subscribe button shows acq messaging to regis and anons
    // and ugprade messaging to non-AA subscribers
    const saleButton = (
      <a
        href={navigationLinks.subscribe.default}
        className={`pz-nav__button subscribe ${jsHook(
          'nav-subscribe js-sale-nav-tracker',
        )} pz-hide-ada`}
        data-track-label="subscribe-nav-header"
      >
        {getButtonText(userType.entitlement)}
      </a>
    )

    // Bar1 CTA should only show on Web
    // Sale subscribe button shows acq messaging to regis and anons
    // and ugprade messaging to non-games and non-AA subscribers
    const bar1SaleButton = !userType.hasXwd ? (
      <a
        href={navigationLinks.subscribe.default}
        className={`pz-nav__button subscribe js-sale-nav-tracker ${jsHook(
          'nav-subscribe',
        )} pz-hide-ada`}
        data-track-label="subscribe-nav-header"
      >
        {getButtonText(userType.entitlement)}
      </a>
    ) : null

    const loginButton =
      !isLoggedIn || isHybridWebView ? (
        <a
          href={loginUrl}
          className={`pz-nav__button white ${jsHook('nav-login')}`}
          data-track-label="log-in-nav"
        >
          Log&nbsp;In
        </a>
      ) : null

    const logoutButton =
      isLoggedIn || isHybridWebView ? (
        <a
          href={`${navigationLinks.logout}&asset=navigation-bar`}
          className={`pz-nav__button gray ${jsHook('nav-logout')}`}
          data-track-label="log-out-nav"
        >
          Log&nbsp;Out
        </a>
      ) : null

    let shouldRenderNavDrawer = true

    let burgButton = (
      <button
        type="button"
        aria-label="Navigation menu button"
        aria-haspopup="true"
        aria-expanded="false"
        className="pz-nav__hamburger-helper pz-nav__hamburger pz-nav__hamburger-squeeze"
        id={jsHook('nav-burger')}
      >
        <span className="pz-nav__hamburger-box">
          <span className="pz-nav__hamburger-inner" />
        </span>
      </button>
    )

    let titleLogo = (
      <a
        id={jsHook('logo-nav')}
        className="pz-nav__title pz-h1"
        href={navigationLinks.hub}
        aria-label="New York Times Games Logo. Click for more puzzles"
      >
        <NavLogo />
      </a>
    )

    const isAndroidNewsreaderHybrid =
      isHybridWebView &&
      deviceType.newsreaderAppPlatform === 'android'

    const isIOSNewsreaderHybrid =
      isHybridWebView && deviceType.newsreaderAppPlatform === 'ios'

    // If we're in the Games App, we replace the nav with the hybrid back button
    // We do the same for the android Newsreader App, in hybrid webviews
    if (
      isGamePage &&
      (deviceType.gamesAppPlatform || isAndroidNewsreaderHybrid)
    ) {
      burgButton = <HybridBackButton id={jsHook('hybrid-back')} />
      shouldRenderNavDrawer = false
    }

    // If we're in the Play Tab, we hide a few global navigational elements
    // We do the same for the iOS Newsreader App, in hybrid webviews, since the app's nav is present
    if (isPlayTab || isIOSNewsreaderHybrid) {
      burgButton = null
      titleLogo = null
      shouldRenderNavDrawer = false
    }

    const urlStartsWith = (h) => url.startsWith(h)
    const urlMatches = (h) => url === h

    const createNavLink = (
      href,
      text,
      trackLabel,
      isSelected = urlStartsWith,
      isNew,
      id = trackLabel,
    ) => (
      <a
        id={id}
        href={href}
        className={cn(
          'js-nav-tracker',
          'pz-nav-drawer__link',
          isSelected(href) && 'selected',
        )}
        data-track-label={trackLabel}
      >
        {text}
        {isNew && <span className="pz-nav-drawer__new">NEW</span>}
      </a>
    )

    return (
      <>
        <div className="pz-nav" id={jsHook('global-nav')}>
          {/* HAMBURGER */}
          {burgButton}

          {/* TITLE */}
          {titleLogo}

          {/* TOOLBAR */}
          <div
            className="pz-nav__toolbar"
            id={jsHook('mobile-toolbar')}
          >
            {/* Tools injected by foundation.mobileNavTools */}
          </div>

          {/* NAV ACTIONS */}
          <div
            className="pz-nav__actions pz-flex-row"
            id={jsHook('nav-actions')}
          >
            {subscribeButton}
            {bar1SaleButton}
            {loginButton}
          </div>
        </div>

        {/* NAV DRAWER */}
        {shouldRenderNavDrawer && (
          <div
            role="menu"
            aria-label="Navigation menu"
            aria-hidden="true"
            className={
              isTopSaleBanner
                ? cn('pz-nav-drawer', 'top-sale-banner-drawer')
                : cn('pz-nav-drawer')
            }
            // className="pz-nav-drawer"
            id={jsHook('nav-drawer')}
          >
            <nav>
              <h4 className="pz-nav-drawer__heading">
                New York Times Games
              </h4>
              <div
                className="pz-nav-drawer__group daily-crossword"
                data-testid="daily-crossword"
              >
                {createNavLink(
                  navigationLinks.daily,
                  THE_CROSSWORD,
                  'daily-page-nav',
                )}
                {createNavLink(
                  navigationLinks.stats,
                  STATS,
                  'stats-nav',
                )}
              </div>
              <div
                className="pz-nav-drawer__group mini-crossword"
                data-testid="mini-crossword"
              >
                {createNavLink(
                  navigationLinks.mini,
                  THE_MINI,
                  'mini-page-nav',
                )}
                {createNavLink(
                  navigationLinks.leaderboards,
                  LEADERBOARDS,
                  'leaderboards-nav',
                )}
              </div>
              <div
                className="pz-nav-drawer__group spelling-bee"
                data-testid="spelling-bee"
              >
                {createNavLink(
                  navigationLinks.spellingBee,
                  SPELLING_BEE,
                  'spelling-bee-nav',
                  (h) =>
                    urlStartsWith(h) &&
                    !urlStartsWith(
                      navigationLinks.spellingBeePuzzleHub,
                    ),
                )}
                {createNavLink(
                  navigationLinks.spellingBeePuzzleHub,
                  SB_PUZZLE_HUB,
                  'spelling-bee-hub-nav',
                  urlStartsWith,
                )}
              </div>
              <div
                className="pz-nav-drawer__group wordle"
                data-testid="wordle"
              >
                {createNavLink(
                  navigationLinks.wordle,
                  WORDLE,
                  'wordle-nav',
                )}
              </div>
              {CONNECTIONS_ON && (
                <div
                  className="pz-nav-drawer__group connections"
                  data-testid="connections"
                >
                  {createNavLink(
                    navigationLinks.connections,
                    CONNECTIONS,
                    'connections-nav',
                    urlStartsWith,
                    false,
                    undefined,
                  )}
                </div>
              )}
              {new Date() < new Date('8/8/2023') && (
                <div
                  className="pz-nav-drawer__group digits"
                  data-testid="digits"
                >
                  {createNavLink(
                    navigationLinks.digits,
                    DIGITS,
                    'digits-nav',
                    urlStartsWith,
                    false,
                    undefined,
                  )}
                </div>
              )}
              <div
                className="pz-nav-drawer__group tiles"
                data-testid="tiles"
              >
                {createNavLink(
                  navigationLinks.tiles,
                  TILES,
                  'tiles-nav',
                )}
              </div>
              <div
                className="pz-nav-drawer__group letter-boxed"
                data-testid="letter-boxed"
              >
                {createNavLink(
                  navigationLinks.letterBoxed,
                  LETTER_BOXED,
                  'letter-boxed-nav',
                )}
              </div>
              <div
                className="pz-nav-drawer__group vertex"
                data-testid="vertex"
              >
                {createNavLink(
                  navigationLinks.vertex,
                  VERTEX,
                  'vertex-nav',
                )}
              </div>
              <div className="pz-nav-drawer__group hub">
                {createNavLink(
                  navigationLinks.hub,
                  ALL_GAMES,
                  'all-games-nav',
                  urlMatches,
                )}
              </div>
              <hr />
              <div className="pz-nav-drawer__group archive">
                {createNavLink(
                  navigationLinks.archive,
                  CROSSWORD_ARCHIVES,
                  'archive-nav',
                )}
              </div>
              <hr />
              <h4 className="pz-nav-drawer__heading">Other Games</h4>
              <div
                className="pz-nav-drawer__group sudoku"
                data-testid="sudoku"
              >
                {createNavLink(
                  navigationLinks.sudoku,
                  SUDOKU,
                  'sudoku-nav',
                )}
              </div>
              <hr />
              <h4 className="pz-nav-drawer__heading">
                Tips and Tricks
              </h4>
              <div className="pz-nav-drawer__group tips-and-tricks">
                {createNavLink(
                  navigationLinks.wordplay,
                  WORDPLAY,
                  'wordplay-nav',
                )}
                {createNavLink(
                  navigationLinks.guide,
                  HOW_TO_SOLVE,
                  'featured-article-nav',
                )}
              </div>
              {/* CCPA Privacy Link */}
              {showCompliance && (
                <>
                  <hr />
                  {compliance.PURR_DataSaleOptOutUI_v2 === 'show' && (
                    <div className="pz-nav-drawer__group ccpa-link ccpa-opt-out">
                      <a
                        href="https://www.nytimes.com/privacy/your-privacy-choices"
                        data-region="menu"
                        data-track="linkOptOut"
                        className="pz-nav-drawer__link ccpa-link"
                      >
                        <div className="ccpa-icon" />
                        Your Privacy Choices
                      </a>
                    </div>
                  )}

                  {/* CALIFORNIA NOTICES  */}
                  {compliance.PURR_CaliforniaNoticesUI === 'show' && (
                    <div className="pz-nav-drawer__group ccpa-link ccpa-california-notice">
                      <a
                        target="_blank"
                        rel="noopener noreferrer"
                        href="https://www.nytimes.com/privacy/privacy-policy#california-notice"
                        data-region="menu"
                        data-track="linkCANotice"
                        className="pz-nav-drawer__link ccpa-link"
                      >
                        California Notices
                      </a>
                    </div>
                  )}
                </>
              )}
            </nav>

            <div
              className="pz-nav-drawer__account"
              id={jsHook('nav-profile')}
            >
              {(isLoggedIn || isHybridWebView) && (
                <>
                  <h4
                    className={`pz-nav-drawer__heading ${jsHook(
                      'nav-account-details',
                    )}`}
                  >
                    Profile
                  </h4>
                  <a
                    href={navigationLinks.account}
                    className={`pz-nav-drawer__link ${jsHook(
                      'nav-account-details',
                    )}`}
                    data-track-label="account-details-nav"
                  >
                    Account details
                  </a>
                </>
              )}
              <div className="pz-nav-drawer__account-actions">
                {logoutButton}
                {subscribeButton}
                {saleButton}
                {loginButton}
              </div>
            </div>
          </div>
        )}
      </>
    )
  },
)

export default Nav
