import React from 'react'
import { shallow } from 'enzyme'

import { Footer } from '../Footer'

const createNavLinks = require('.../../../util/createNavigationLinks')

const navigationLinks = createNavLinks(
  {},
  {
    locals: {
      userType: { hasDigi: true },
      deviceType: {
        newsreaderAppPlatform: '',
        newsreaderAppVersion: '',
      },
    },
  },
)

jest.mock('../withServerData', () => ({
  withServerData: (component) => component,
}))

describe('SERVER COMPONENTS: FOOTER', () => {
  let props

  beforeEach(() => {
    props = {
      compliance: { optOut: false },
      navigationLinks,
      deviceType: { newsreaderAppPlatform: '' },
      userType: { hasXwd: false },
      env: { name: 'stg' },
    }
  })

  test('ccpa - show opt out', () => {
    props.compliance.PURR_DataSaleOptOutUI_v2 = 'show'
    const footer = shallow(<Footer {...props} />)
    const optOut = footer.find('.ccpa-opt-out')
    const link = footer.find('.ccpa-opt-out > a')
    expect(optOut.exists()).toBe(true)
    expect(optOut.text()).toBe('Your Privacy Choices')
    expect(link.prop('data-region')).toBe('footer')
  })

  test('ccpa - hide opt out', () => {
    props.compliance.PURR_DataSaleOptOutUI_v2 = 'hide'
    const footer = shallow(<Footer {...props} />)
    const optOut = footer.find('.ccpa-opt-out')
    expect(optOut.exists()).toBe(false)
  })

  test('ccpa - show california notice', () => {
    props.compliance.PURR_CaliforniaNoticesUI = 'show'
    const footer = shallow(<Footer {...props} />)
    const ccpa = footer.find('.ccpa-california-notice')
    const ccpaHref = ccpa.find('a')
    expect(ccpa.exists()).toBe(true)
    expect(ccpaHref.prop('href')).toBe(
      'https://www.nytimes.com/privacy/privacy-policy#california-notice',
    )
  })

  test('ccpa - hide california notice', () => {
    props.compliance.PURR_CaliforniaNoticesUI = 'hide'
    const footer = shallow(<Footer {...props} />)
    const ccpa = footer.find('.ccpa-california-notice')
    expect(ccpa.exists()).toBe(false)
  })

  test('verify footer links', () => {
    const footer = shallow(<Footer {...props} />)

    // NYT GAMES
    const dailyLink = footer.find(
      `a[href="${navigationLinks.daily}"]`,
    )

    const sbLink = footer.find(
      `a[href="${navigationLinks.spellingBee}"]`,
    )

    const lbLink = footer.find(
      `a[href="${navigationLinks.letterBoxed}"]`,
    )
    const miniLink = footer.find(`a[href="${navigationLinks.mini}"]`)
    const tilesLink = footer.find(
      `a[href="${navigationLinks.tiles}"]`,
    )

    const vertexLink = footer.find(
      `a[href="${navigationLinks.vertex}"]`,
    )

    const sudokuLink = footer.find(
      `a[href="${navigationLinks.sudoku}"]`,
    )

    const allLink = footer
      .find('.pz-footer__link a')
      .filterWhere((n) => n.text() === 'All Games')

    // LEARN MORE
    const feedbackLink = footer.find(
      `a[href="${navigationLinks.feedback}"]`,
    )
    const faqLink = footer.find(`a[href="${navigationLinks.faq}"]`)
    const giftLink = footer.find(`a[href="${navigationLinks.gift}"]`)

    // CROSSWORDS
    const archiveLink = footer.find(
      `a[href="${navigationLinks.archive}"]`,
    )

    const statsLink = footer.find(
      `a[href="${navigationLinks.stats}"]`,
    )

    const lbdLink = footer.find(
      `a[href="${navigationLinks.leaderboards}"]`,
    )
    const appsLink = footer.find(`a[href="${navigationLinks.apps}"]`)
    const submissionLink = footer.find(
      `a[href="${navigationLinks.submission}"]`,
    )

    // COMMUNITY
    const wordplayLink = footer.find(
      `a[href="${navigationLinks.wordplay}"]`,
    )

    const twitterLink = footer.find(
      `a[href="${navigationLinks.twitter}"]`,
    )

    // LEGAL
    const cpyRtYear = new Date().getFullYear()

    const nytimesLink = footer.find(
      `a[href="${navigationLinks.company}"]`,
    )
    const homeLink = footer.find(`a[href="${navigationLinks.home}"]`)
    const mapLink = footer.find(
      `a[href="${navigationLinks.sitemap}"]`,
    )

    const privacyLink = footer.find(
      `a[href="${navigationLinks.privacy}"]`,
    )

    const termsLink = footer.find(
      `a[href="${navigationLinks.termsOfService}"]`,
    )

    expect(dailyLink.text()).toBe('<Icon />The Crossword')
    expect(miniLink.text()).toBe('<Icon />The Mini Crossword')
    expect(sbLink.text()).toBe('<Icon />Spelling Bee')
    expect(lbLink.text()).toBe('<Icon />Letter Boxed')
    expect(vertexLink.text()).toBe('<Icon />Vertex')
    expect(tilesLink.text()).toBe('<Icon />Tiles')
    expect(sudokuLink.text()).toBe('<Icon />Sudoku')
    expect(allLink.props().href).toBe(navigationLinks.hub)

    expect(feedbackLink.text()).toBe('Contact Us')
    expect(faqLink.text()).toBe('FAQs')
    expect(giftLink.text()).toBe('Gift Subscriptions')

    expect(archiveLink.text()).toBe('Crossword Archives')
    expect(statsLink.text()).toBe('Statistics')
    expect(lbdLink.text()).toBe('Leaderboards')
    expect(appsLink.text()).toBe('Download the App')
    expect(submissionLink.text()).toBe('Submit a Crossword')

    expect(wordplayLink.text()).toBe('Gameplay Stories')
    expect(twitterLink.text()).toBe('Games Twitter')

    expect(nytimesLink.text()).toBe(
      `© ${cpyRtYear} The New York Times Company`,
    )
    expect(homeLink.text()).toBe('NYTimes.com')
    expect(mapLink.text()).toBe('Sitemap')
    expect(privacyLink.text()).toBe('Privacy Policy')
    expect(termsLink.text()).toBe('Terms of Service')
  })

  test('verify footer sections', () => {
    const footer = shallow(<Footer {...props} />)
    expect(
      footer.find('.pz-footer__section-header').at(0).text(),
    ).toBe('About New York Times Games')
    expect(
      footer.find('.pz-footer__section-header').at(1).text(),
    ).toBe('New York Times Games')
    expect(
      footer.find('.pz-footer__section-header').at(2).text(),
    ).toBe('Crosswords')
    expect(
      footer.find('.pz-footer__section-header').at(3).text(),
    ).toBe('Community')
    expect(
      footer.find('.pz-footer__section-header').at(4).text(),
    ).toBe('Learn More')
  })

  test('subscribe text should appear for non subs', () => {
    const footer = shallow(<Footer {...props} />)
    expect(
      footer.find('.pz-footer__featured-text').at(1).text(),
    ).toBe('Subscribe now for unlimited access.')
  })

  test('subscribe text should not appear for subs', () => {
    props.userType.hasXwd = true

    const footer = shallow(<Footer {...props} />)
    const subLink = footer.find('.pz-footer__featured-text a')

    expect(subLink.exists()).toBe(false)
  })
})
