import React from 'react'
import { mount, shallow } from 'enzyme'

import { Nav } from '../Nav'

jest.mock('../withServerData', () => ({
  withServerData: (component) => component,
}))

describe('SERVER COMPONENTS: NAV', () => {
  let props

  const dailyLink = '/crosswords/game/daily'
  const miniLink = '/crosswords/game/mini'
  const sbLink = '/puzzles/spelling-bee'
  const puzzleHubLink = '/puzzles/spelling-bee/hub'
  const hubLink = '/crosswords'
  const archiveLink = '/crosswords/archive'
  const accountLink = '/account'
  const loginLink = '/login'
  const logoutLink = '/logout'
  const subscribeLink = '/subscribe/games'

  beforeEach(() => {
    props = {
      compliance: {},
      userType: {
        isLoggedIn: true,
        hasXwd: false,
      },
      deviceType: {
        isMobile: true,
        gamesAppPlatform: '',
      },
      env: {
        name: 'stg',
      },
      isGamePage: false,
      navigationLinks: {
        login: loginLink,
        logout: logoutLink,
        register: 'register',
        account: accountLink,
        daily: dailyLink,
        mini: miniLink,
        spellingBee: sbLink,
        spellingBeePuzzleHub: puzzleHubLink,
        hub: hubLink,
        archive: archiveLink,
        subscribe: {
          default: subscribeLink,
        },
      },
      url: '/foobar',
    }
  })

  describe('mobile drawer', () => {
    it('privacy hub link renders if required', () => {
      props.compliance.PURR_DataSaleOptOutUI_v2 = 'show'
      props.userType.isLoggedIn = false
      const nav = shallow(<Nav {...props} />)
      const link = nav.find('.ccpa-opt-out > a')
      const privacyHub = nav.find('.ccpa-opt-out')
      expect(privacyHub.exists()).toBe(true)
      expect(privacyHub.text()).toBe('Your Privacy Choices')
      expect(link.prop('data-region')).toBe('menu')
    })

    it('ccpa does not render if hidden', () => {
      props.compliance.PURR_DataSaleOptOutUI_v2 = 'hide'
      props.userType.isLoggedIn = false
      const nav = shallow(<Nav {...props} />)
      const optOut = nav.find('.ccpa-opt-out')
      expect(optOut.exists()).toBe(false)
    })

    it('renders the california ccpa notice if required', () => {
      props.compliance.PURR_CaliforniaNoticesUI = 'show'
      const nav = shallow(<Nav {...props} />)
      const ccpa = nav.find('.ccpa-california-notice')
      const ccpaHref = ccpa.find('a')
      expect(ccpa.exists()).toBe(true)
      expect(ccpa.text()).toBe('California Notices')
      expect(ccpaHref.prop('href')).toBe(
        'https://www.nytimes.com/privacy/privacy-policy#california-notice',
      )
    })

    it('hides the california ccpa notice if not required', () => {
      props.compliance.PURR_CaliforniaNoticesUI = 'hide'
      const nav = shallow(<Nav {...props} />)
      const ccpa = nav.find('.ccpa-california-notice')
      expect(ccpa.exists()).toBe(false)
    })

    it('renders links to various games & pages with game-specific classes in the nav drawer', () => {
      const nav = shallow(<Nav {...props} />)
      const navDrawer = nav.find('.pz-nav-drawer')

      const dailyATag = navDrawer.find(`a[href="${dailyLink}"]`)
      expect(dailyATag.exists()).toBeTruthy()
      expect(
        dailyATag.parent().hasClass('daily-crossword'),
      ).toBeTruthy()
      expect(dailyATag.text()).toEqual('The Crossword')

      const miniATag = navDrawer.find(`a[href="${miniLink}"]`)
      expect(miniATag.exists()).toBeTruthy()
      expect(
        miniATag.parent().hasClass('mini-crossword'),
      ).toBeTruthy()
      expect(miniATag.text()).toEqual('The Mini')

      const sbATag = navDrawer.find(`a[href="${sbLink}"]`)
      expect(sbATag.exists()).toBeTruthy()
      expect(sbATag.parent().hasClass('spelling-bee')).toBeTruthy()
      expect(sbATag.text()).toEqual('Spelling Bee')

      const archiveATag = navDrawer.find(`a[href="${archiveLink}"]`)
      expect(archiveATag.exists()).toBeTruthy()
      expect(archiveATag.parent().hasClass('archive')).toBeTruthy()
      expect(archiveATag.text()).toEqual('Crossword Archives')
    })

    it('renders a link to the hub in the nav drawer', () => {
      const nav = shallow(<Nav {...props} />)
      const navDrawer = nav.find('.pz-nav-drawer')

      const hubATag = navDrawer.find(`a[href="${hubLink}"]`)
      expect(hubATag.text()).toEqual('All Games')
      expect(hubATag.exists()).toBeTruthy()
    })

    it('renders a link to login with the URL from navigationLinks', () => {
      props.userType.isLoggedIn = false
      const nav = shallow(<Nav {...props} />)
      const loginButton = nav.find('.pz-nav__actions').childAt(2)
      expect(loginButton.prop('href')).toContain(loginLink)
    })

    it('renders a link to subscribe with the default URL from navigationLinks', () => {
      props.userType.isLoggedIn = false
      const nav = shallow(<Nav {...props} />)
      const subscribeButton = nav.find('.pz-nav__actions').childAt(0)
      expect(subscribeButton.prop('href')).toContain(subscribeLink)
    })

    it('renders subscribe and login actions in header if user is anonymous', () => {
      props.userType.isLoggedIn = false
      const nav = shallow(<Nav {...props} />)
      const navActions = nav.find('.pz-nav__actions')
      const subscribeButton = navActions.childAt(0)
      const loginButton = navActions.childAt(2)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Subscribe')
      expect(loginButton.exists()).toBe(true)
      expect(loginButton.text()).toBe('Log\xa0In')
    })

    it('renders sale subscribe message in header if user is anonymous and games sale is active', () => {
      props.userType.isLoggedIn = false
      const isGamesSale = true
      const nav = shallow(
        <Nav {...props} isGamesSale={isGamesSale} />,
      )
      const navActions = nav.find('.pz-nav__actions')
      const subscribeButton = navActions.childAt(1)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Subscribe for 50% off')
    })

    it('renders subscribe action in header if user is a regi', () => {
      props.userType.isLoggedIn = true
      const nav = shallow(<Nav {...props} />)
      const navActions = nav.find('.pz-nav__actions')
      const subscribeButton = navActions.childAt(0)
      const loginButton = navActions.childAt(2)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toContain('Subscribe')
      expect(loginButton.exists()).toBe(false)
    })

    it('renders sale subscribe message in header if user is regi and games sale is active', () => {
      props.userType.isLoggedIn = true
      const isGamesSale = true
      const nav = shallow(
        <Nav {...props} isGamesSale={isGamesSale} />,
      )
      const navActions = nav.find('.pz-nav__actions')
      const subscribeButton = navActions.childAt(1)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Subscribe for 50% off')
    })

    it('renders subscribe message in header if user is an non-AA subscriber and games sale is active', () => {
      props.userType.isLoggedIn = true
      props.userType.entitlement = 'sub'
      const isGamesSale = true
      const nav = shallow(
        <Nav {...props} isGamesSale={isGamesSale} />,
      )
      const navActions = nav.find('.pz-nav__actions')
      const subscribeButton = navActions.childAt(1)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Upgrade and save')
    })

    it('renders login action in header if user is an anon subscriber', () => {
      props.userType.isLoggedIn = false
      props.userType.hasXwd = true
      const nav = shallow(<Nav {...props} />)
      const navActions = nav.find('.pz-nav__actions')
      const loginButton = navActions.childAt(0)
      expect(loginButton.exists()).toBe(true)
    })

    it('renders subscribe and login actions in drawer if user is anonymous', () => {
      props.userType.isLoggedIn = false
      const nav = shallow(<Nav {...props} />)
      const navDrawerActions = nav.find(
        '.pz-nav-drawer__account-actions',
      )
      const subscribeButton = navDrawerActions.childAt(0)
      const loginButton = navDrawerActions.childAt(2)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Subscribe')
      expect(loginButton.exists()).toBe(true)
      expect(loginButton.text()).toBe('Log\xa0In')
    })

    it('renders sale subscribe messaging in drawer if user is anonymous and games sale is active', () => {
      props.userType.isLoggedIn = false
      props.userType.entitlement = 'anon'
      const isGamesSale = true
      const nav = shallow(
        <Nav {...props} isGamesSale={isGamesSale} />,
      )

      const navDrawerActions = nav.find(
        '.pz-nav-drawer__account-actions',
      )
      const subscribeButton = navDrawerActions.childAt(1)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toContain('50% off')
    })

    it('renders logout and subscribe actions in drawer if user is a regi', () => {
      props.userType.isLoggedIn = true
      const nav = shallow(<Nav {...props} />)
      const profileHeading = nav.find('.pz-nav-drawer__heading')
      const navDrawerActions = nav.find(
        '.pz-nav-drawer__account-actions',
      )
      const accountInfoLink = nav.find(`a[href="${accountLink}"]`)
      const logoutButton = navDrawerActions.childAt(0)
      const subscribeButton = navDrawerActions.childAt(1)
      expect(profileHeading.exists()).toBe(true)
      expect(accountInfoLink.text()).toBe('Account details')
      expect(logoutButton.exists()).toBe(true)
      expect(logoutButton.text()).toBe('Log\xa0Out')
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Subscribe')
    })

    it('renders sale subscribe messaging in drawer if user is regi and games sale is active', () => {
      props.userType.isLoggedIn = true
      props.userType.entitlement = 'regi'
      const isGamesSale = true
      const nav = shallow(
        <Nav {...props} isGamesSale={isGamesSale} />,
      )

      const navDrawerActions = nav.find(
        '.pz-nav-drawer__account-actions',
      )
      const subscribeButton = navDrawerActions.childAt(2)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Subscribe for 50% off')
    })

    it('renders sales subscribe messaging in drawer if user is a non-AA subscriber and games sale is active', () => {
      props.userType.isLoggedIn = true
      props.userType.entitlement = 'sub'
      const isGamesSale = true
      const nav = shallow(
        <Nav {...props} isGamesSale={isGamesSale} />,
      )

      const navDrawerActions = nav.find(
        '.pz-nav-drawer__account-actions',
      )
      const subscribeButton = navDrawerActions.childAt(2)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Upgrade and save')
    })

    it('renders the login action in drawer if user is an anon subscriber', () => {
      props.userType.isLoggedIn = false
      props.userType.hasXwd = true
      const nav = shallow(<Nav {...props} />)
      const profileHeading = nav.find('.pz-nav-drawer__heading')
      const navDrawerActions = nav.find(
        '.pz-nav-drawer__account-actions',
      )
      const accountInfoLink = nav.find(`a[href="${accountLink}"]`)
      const loginButton = navDrawerActions.childAt(1)
      expect(accountInfoLink.exists()).toBe(false)
      expect(profileHeading.exists()).toBe(true)
      expect(loginButton.exists()).toBe(true)
      expect(loginButton.text()).toBe('Log\xa0In')
    })

    it('does not render the back button if not in hybrid Games app', () => {
      props.isGamePage = true
      props.deviceType.gamesAppPlatform = ''
      const nav = mount(<Nav {...props} />)
      const backButton = nav.find('.pz-hybrid-back')
      expect(backButton.exists()).toBe(false)
    })

    it('renders the back button if on game page in hybrid Games app', () => {
      props.isGamePage = true
      props.deviceType.gamesAppPlatform = 'ios'
      const nav = mount(<Nav {...props} />)
      const backButton = nav.find('.pz-hybrid-back')
      expect(backButton.exists()).toBe(true)
    })

    it('hides the hamburger menu and nav drawer if on game page in hybrid Games app', () => {
      props.isGamePage = true
      props.deviceType.gamesAppPlatform = 'ios'
      const nav = shallow(<Nav {...props} />)
      const menu = nav.find('.pz-nav__hamburger')
      const drawer = nav.find('.pz-nav-drawer')
      expect(menu.exists()).toBe(false)
      expect(drawer.exists()).toBe(false)
    })

    it('renders all login, logout, and subscribe buttons if in hybrid', () => {
      props.isHybridWebView = true
      const nav = shallow(<Nav {...props} />)

      const navActions = nav.find('.pz-nav__actions')
      const subscribeButton = navActions.childAt(0)
      const loginButton = navActions.childAt(2)
      expect(subscribeButton.exists()).toBe(true)
      expect(subscribeButton.text()).toBe('Subscribe')
      expect(loginButton.exists()).toBe(true)
      expect(loginButton.text()).toBe('Log\xa0In')

      const navDrawerActions = nav.find(
        '.pz-nav-drawer__account-actions',
      )
      const accountInfoLink = nav.find(`a[href="${accountLink}"]`)
      const logoutButton = navDrawerActions.childAt(0)
      expect(accountInfoLink.exists()).toBe(true)
      expect(logoutButton.exists()).toBe(true)
      expect(logoutButton.text()).toBe('Log\xa0Out')
    })

    it('renders spelling bee link as focused and when page is spelling bee game', () => {
      props.url = '/puzzles/spelling-bee/2023-03-09'
      const nav = shallow(<Nav {...props} />)
      const navDrawer = nav.find('.pz-nav-drawer')

      const sbATag = navDrawer.find(`a[href="${sbLink}"]`)
      expect(sbATag.exists()).toBeTruthy()
      expect(sbATag.parent().hasClass('spelling-bee')).toBeTruthy()
      expect(sbATag.text()).toEqual('Spelling Bee')
      expect(sbATag.hasClass('selected')).toBeTruthy()

      const ppATag = navDrawer.find(`a[href="${puzzleHubLink}"]`)
      expect(ppATag.exists()).toBeTruthy()
      expect(ppATag.parent().hasClass('spelling-bee')).toBeTruthy()
      expect(ppATag.text()).toContain('Past Puzzles')
      expect(ppATag.hasClass('selected')).toBeFalsy()
    })

    it('renders spelling bee past puzzles link as focused when page is puzzle hub', () => {
      props.url = '/puzzles/spelling-bee/hub?login=email'
      const nav = shallow(<Nav {...props} />)
      const navDrawer = nav.find('.pz-nav-drawer')

      const sbATag = navDrawer.find(`a[href="${sbLink}"]`)
      expect(sbATag.exists()).toBeTruthy()
      expect(sbATag.parent().hasClass('spelling-bee')).toBeTruthy()
      expect(sbATag.text()).toEqual('Spelling Bee')
      expect(sbATag.hasClass('selected')).toBeFalsy()

      const ppATag = navDrawer.find(`a[href="${puzzleHubLink}"]`)
      expect(ppATag.exists()).toBeTruthy()
      expect(ppATag.parent().hasClass('spelling-bee')).toBeTruthy()
      expect(ppATag.text()).toContain('Past Puzzles')
      expect(ppATag.hasClass('selected')).toBeTruthy()
    })
  })
})
