/* SERVER-SIDE LOGGING */
const pino = require('pino')
const chalk = require('chalk')

const { RUNTIME_ENV, LOG_LEVEL } = require('./constants')

// mapping between default pino level labels and cloud logging severity values
// see https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry#LogSeverity for more details
const LEVEL_LABEL_TO_SEVERITY = {
  trace: 'DEBUG',
  debug: 'DEBUG',
  info: 'INFO',
  warn: 'WARNING',
  error: 'ERROR',
  fatal: 'CRITICAL',
}

// options for pino logger
const options = {
  base: {
    pid: undefined,
    hostname: undefined,
  },
  // only include timestamp in local environments
  timestamp: RUNTIME_ENV === 'dev',
  // the minimum level to log
  level: LOG_LEVEL,
}

if (RUNTIME_ENV === 'dev') {
  options.transport = {
    target: 'pino-pretty',
  }

  options.colorize = true
  options.levelFirst = true
}

// environment-specific options
if (RUNTIME_ENV !== 'dev') {
  // change the key used for message strings from the default ("msg")
  // both Datadog and Cloud Logging expect the message key to be "message"
  options.messageKey = 'message'
  // change the key used for error defaults from the default ("err")
  // Datadog expects the error key to be "error"
  options.errorKey = 'error'
  // transform the level shape and map to Cloud Logging "severity" values
  // note: when ingested by datadog, the Cloud Logging "severity" will be translated into a Datadog "status"
  // see https://www.datadoghq.com/blog/how-to-categorize-logs/#categorize-by-logs-based-on-standard-severity-levels for more details
  options.formatters = {
    level(label /* , number */) {
      return { severity: LEVEL_LABEL_TO_SEVERITY[label] || 'DEFAULT' }
    },
  }
}

const logger = pino(options)

if (RUNTIME_ENV !== 'dev') {
  // patch the global `console` instance to prevent multiline log entries
  // these are created when 3rd party libs use console[log|info|etc]
  // ignoring (for now) the case of custom instances derived from `console.Console`

  // bind the aliases to the `global.console[level]` equivalent..
  const methodsToOverride = Object.keys(logger.levels.values)
  methodsToOverride.forEach((level) => {
    if (level in global.console) {
      global.console[level] = logger[level].bind(logger)
    }
  })

  // pino instances do not have a `log` method so we'll use `logger.info` for `console.log`
  global.console.log = logger.info.bind(logger)
}

/**
 * Logs color-coded info with a label and message
 * @param {string} label
 * @param {string} msg
 * @param {string} color
 */
logger.labeledInfo = (label, msg, color = 'cyan') => {
  // only colorize when running locally
  if (RUNTIME_ENV !== 'dev') {
    logger.info(`${label} ${msg}`)
    return
  }
  logger.info(`${label} ${chalk[color](msg)}`)
}

/**
 * Logs a header to help distingish a block of log output
 * @param {string} msg
 */
logger.headingInfo = (msg, color = 'white') => {
  // only colorize when running locally
  if (RUNTIME_ENV !== 'dev') {
    logger.info(`[ ${msg.toUpperCase()} ]`)
    return
  }
  logger.info(`[ ${chalk[color](msg.toUpperCase())} ]`)
}

/**
 * Logs a compilation error for scss/js files
 * @param {Object} err
 * @returns {Object}
 */
logger.compilationError = (err) => {
  // handle scss error
  if (err.file) {
    err.message = `.scss compilation failed\n\n> ${err.message}\n> file: ${err.file} [${err.line}:${err.column}]\n`
  }

  // handle buble error
  if (err.plugin === 'buble') {
    const { message, loc } = err
    err.message = `.js transpile failed\n\n> ${message}\n> file: ${loc.file}\n`
  }

  logger.error(err)

  return err
}

/**
 * Starts a timer when called. Returns a function that,
 * when called, logs information on elapsed time.
 * @param {string} label
 * @returns {Function}
 */
logger.checkTime = (label) => {
  const NS_PER_SEC = 1e9
  const startTime = process.hrtime()

  return function logTimeElapsed() {
    const diff = process.hrtime(startTime)
    const duration = (diff[0] * NS_PER_SEC + diff[1]) / 1000000
    logger.labeledInfo(label, `${duration >> 0} ms`)
  }
}

module.exports = logger
