import React from 'react'

import useAbra from './useAbra'

/**
 * For adding features flags via abra to any game
 * Note - this logic was originally used by wordle - wordle/js/FeatureFlags.tsx
 * How to use:
 *  1. Make sure FeatureFlagProvider is in your component tree, under AbraProvider
 *  2. Add test key in gameFeatureFlags array
 *  3. Add testName and active variant for the test key in featureFlagConfig
 *  4. Add <FeatureFlagProvider /> wrapper to game (nested under AbraProvider)
 *  5. should be able to access allocation via `useFeatureFlag` example:
 *    `import useFeatureFlag from '../../../../shared/abra/useFeatureFlag'`
 *    `const newWelcomeMoment = useFeatureFlag('sudokuWelcomeMoment') => returns true or false`
 */

const gameFeatureFlags = [
  'activationRegiModal',
  'activationRegiModalControl',
  'loginOffer',
  'letterboxedWelcome', // for lb modals
  'gamesSaleCTA',
  'authFlowConnections',
  'sbLifetimeStats',
  'sbHubIconsOld',
  'connectionsReact',
] as const

export type GameFeatureFlag = (typeof gameFeatureFlags)[number]

export type FeatureFlagTest = {
  testName: string
  activeVariant: string
}

export type FeatureFlagConfig = Record<
  GameFeatureFlag,
  FeatureFlagTest
>

const featureFlagConfig: FeatureFlagConfig = {
  // NSO-2385
  activationRegiModal: {
    testName: 'ON_wordle_regi_modal',
    activeVariant: '1_regi_modal',
  },
  // NSO-2385
  activationRegiModalControl: {
    testName: 'ON_wordle_regi_modal',
    activeVariant: '0_control',
  },
  loginOffer: {
    testName: 'ON_wordleOnboardingOffer_0822',
    activeVariant: '1_ShowOffer',
  },
  letterboxedWelcome: {
    testName: 'GAMES_letter_boxed_welcome_0328', // inactive as of 12/20/2023
    activeVariant: '1_NewLetterBoxedWelcome',
  },
  gamesSaleCTA: {
    testName: 'MWD_phoenix_games_sale_cta',
    activeVariant: '1_Show',
  },
  authFlowConnections: {
    testName: 'GAMES_connections_auth_web_1016', // inactive as of 12/20/2023
    activeVariant: '1_Auth_Flow',
  },
  sbLifetimeStats: {
    testName: 'GAMES_SBLifetimeStats_1123', // inactive as of 12/20/2023
    activeVariant: '1_LifetimeStats',
  },
  sbHubIconsOld: {
    testName: 'GAMES_spellingBeeHubIcons_0124',
    activeVariant: '0_control',
  },
  connectionsReact: {
    testName: 'GAMES_connectionsRollout_1130',
    activeVariant: '1_ConnectionsV2',
  },
}

export type FeatureFlagContextType = Record<GameFeatureFlag, boolean>

type GetVariant = (variant: string) => string | null

const getFeatureFlagMap = (getVariant?: GetVariant) => {
  return gameFeatureFlags.reduce((acc, flag) => {
    if (getVariant) {
      const { testName, activeVariant } = featureFlagConfig[flag]
      acc[flag] = getVariant(testName) === activeVariant
    } else {
      acc[flag] = false
    }

    return acc
  }, {} as FeatureFlagContextType)
}

/**
 * Returns true if any feature flag value has changed
 */
const detectFlagChange = (
  mapA: FeatureFlagContextType,
  mapB: FeatureFlagContextType,
) => gameFeatureFlags.some((flag) => mapA[flag] !== mapB[flag])

export const FeatureFlagContext =
  React.createContext<FeatureFlagContextType>(getFeatureFlagMap())

export const FeatureFlagProvider = ({
  children,
}: {
  children: React.ReactNode
}) => {
  const { getVariant } = useAbra()
  const flagMap = getFeatureFlagMap(getVariant)
  const [contextValue, setContextValue] = React.useState(flagMap)

  React.useEffect(() => {
    if (detectFlagChange(flagMap, contextValue)) {
      setContextValue(flagMap)
    }
  }, [flagMap])

  return (
    <FeatureFlagContext.Provider value={contextValue}>
      {children}
    </FeatureFlagContext.Provider>
  )
}

/**
 * Convenience hook to report abra exposure, so that you don't need
 * the testName and variant strings outside of this file.
 *
 * @param {GameFeatureFlag} flag
 * @param {boolean} isUxDiverged - exposure should only be reported at the moment UX changes based on the user's allocation
 */
export const useReportExposure = (
  flag: GameFeatureFlag,
  isUxDiverged = true,
) => {
  const abra = useAbra()
  const { testName } = featureFlagConfig[flag]
  const testActive = !!abra.getVariant(testName)

  React.useEffect(() => {
    if (testActive && isUxDiverged) {
      abra.reportExposure(testName)
    }
  }, [testActive, isUxDiverged])
}
