import React from 'react'
import { render, screen, fireEvent } from '@testing-library/react'
import isMonday from 'date-fns/isMonday'
import isSunday from 'date-fns/isSunday'
import format from 'date-fns/format'

import { getDateFromInputString } from '../utils'

import EditorialLinksTool, {
  fourMondaysFromNow,
} from './EditorialLinksTool'

describe('fourMondaysFromNow()', () => {
  test('with Sunday, returns expected date', () => {
    expect(
      format(
        fourMondaysFromNow(new Date('2023-01-01')),
        'yyyy/MM/dd',
      ),
    ).toEqual('2023/01/23')
  })
})

describe('EDITORIAL LINKS TOOL', () => {
  test('renders', () => {
    render(<EditorialLinksTool />)
    expect(
      screen.getByText('Editorial Links Tool'),
    ).toBeInTheDocument()
  })

  test('has default values', () => {
    render(<EditorialLinksTool />)

    const gameSelect = screen.getByLabelText(
      'Game',
    ) as HTMLSelectElement

    const start = screen
      .getByLabelText('Start Date')
      .getAttribute('value') as string

    const end = screen
      .getByLabelText('End Date')
      .getAttribute('value') as string

    const startDate = getDateFromInputString(start)
    const endDate = getDateFromInputString(end)

    expect(gameSelect.value).toBe('daily')
    expect(isMonday(startDate)).toBe(true)
    expect(isSunday(endDate)).toBe(true)
  })

  test('prevents selecting an end date that comes before the start date', () => {
    render(<EditorialLinksTool />)

    const start = screen.getByLabelText('Start Date')
    const end = screen.getByLabelText('End Date')

    expect(end.getAttribute('min')).toEqual(
      start.getAttribute('value'),
    )
  })

  test('renders a list of urls for a valid date range', () => {
    render(<EditorialLinksTool />)

    const list = screen.getByRole('list')
    expect(list.children.length).toEqual(0)

    const submit = screen.getByDisplayValue('Submit')
    fireEvent.click(submit)

    expect(list.children.length).toEqual(7)
  })

  test('renders links in a different format depending on the game', () => {
    render(<EditorialLinksTool />)

    const submit = screen.getByDisplayValue('Submit')
    const start = screen
      .getByLabelText('Start Date')
      .getAttribute('value') as string

    const end = screen
      .getByLabelText('End Date')
      .getAttribute('value') as string

    fireEvent.click(submit)

    expect(
      screen.getAllByRole('link')[0].getAttribute('href'),
    ).toEqual(
      `https://www.nytimes.com/crosswords/game/daily/${start.replace(
        /-/g,
        '/',
      )}`,
    )

    const gameSelect = screen.getByLabelText('Game')

    fireEvent.change(gameSelect, { target: { value: 'wordle' } })
    fireEvent.click(submit)

    expect(
      screen.getAllByRole('link')[0].getAttribute('href'),
    ).toEqual(
      `https://www.nytimes.com/puzzles/shortz/wordle/${start}`,
    )

    expect(
      screen.getAllByRole('link')[6].getAttribute('href'),
    ).toEqual(`https://www.nytimes.com/puzzles/shortz/wordle/${end}`)
  })
})
