import React, { FormEvent, useState } from 'react'
import addWeeks from 'date-fns/addWeeks'
import nextMonday from 'date-fns/nextMonday'
import format from 'date-fns/format'
import nextSunday from 'date-fns/nextSunday'
import eachDayOfInterval from 'date-fns/eachDayOfInterval'

import { getDateFromInputString } from '../utils'

// Urls are intentionally hardcoded to the domain of
// https://www.nytimes.com (rather than matching your environment)
// because they are intended to be reviewed in production.
const DOMAIN = 'https://www.nytimes.com'

const CROSSWORDS_ROUTE = `${DOMAIN}/crosswords/game`
const PUZZLES_ROUTE = `${DOMAIN}/puzzles`

enum GameChoiceEnum {
  connections = 'connections',
  daily = 'daily',
  mini = 'mini',
  'spellingBee' = 'spellingBee',
  wordle = 'wordle',
  vertex = 'vertex',
}

const GameChoices: {
  [key in GameChoiceEnum]: string
} = {
  [GameChoiceEnum.connections]: 'Connections',
  [GameChoiceEnum.daily]: 'Daily',
  [GameChoiceEnum.mini]: 'Mini',
  [GameChoiceEnum.spellingBee]: 'Spelling Bee',
  [GameChoiceEnum.wordle]: 'Wordle',
  [GameChoiceEnum.vertex]: 'Vertex',
}

const urlFormatter: Record<GameChoiceEnum, (date: Date) => string> = {
  [GameChoiceEnum.connections]: (date) =>
    `${PUZZLES_ROUTE}/shortz/connections/${format(
      date,
      'yyyy-MM-dd',
    )}`,
  [GameChoiceEnum.daily]: (date) =>
    `${CROSSWORDS_ROUTE}/daily/${format(date, 'yyyy/MM/dd')}`,
  [GameChoiceEnum.mini]: (date) =>
    `${CROSSWORDS_ROUTE}/mini/${format(date, 'yyyy/MM/dd')}`,
  [GameChoiceEnum.spellingBee]: (date) =>
    `${PUZZLES_ROUTE}/shortz/spelling-bee/${format(
      date,
      'yyyy-MM-dd',
    )}`,
  [GameChoiceEnum.wordle]: (date) =>
    `${PUZZLES_ROUTE}/shortz/wordle/${format(date, 'yyyy-MM-dd')}`,
  [GameChoiceEnum.vertex]: (date) =>
    `${PUZZLES_ROUTE}/shortz/vertex/${format(date, 'yyyy-MM-dd')}`,
}

export const fourMondaysFromNow = (date = new Date()) =>
  addWeeks(nextMonday(date), 3)

const EditorialLinksTool = () => {
  const defaultStartDate = fourMondaysFromNow()
  const formattedStartDate = format(defaultStartDate, 'yyyy-MM-dd')

  const defaultEndDate = nextSunday(defaultStartDate)
  const formattedEndDate = format(defaultEndDate, 'yyyy-MM-dd')

  const [game, setGame] = useState<GameChoiceEnum>(
    GameChoiceEnum.daily,
  )
  const [start, setStart] = useState<string>(formattedStartDate)
  const [end, setEnd] = useState<string>(formattedEndDate)
  const [urls, setUrls] = useState<
    { dayOfTheWeek: string; url: string }[]
  >([])

  const renderLinks = (event: FormEvent<HTMLFormElement>) => {
    event.preventDefault()

    const startDate = getDateFromInputString(start)
    const endDate = getDateFromInputString(end)

    const datesToRender = eachDayOfInterval({
      start: startDate,
      end: endDate,
    })

    const formattedUrls = datesToRender.map((date) => {
      return {
        url: urlFormatter[game](date),
        dayOfTheWeek: format(date, 'EEEE'),
      }
    })

    setUrls(formattedUrls)
  }

  return (
    <div className="editorial-links-page">
      <div className="editorial-links-form-wrapper">
        <h1 className="editorial-links-heading">
          Editorial Links Tool
        </h1>

        <p className="instructions">
          Select a game and a date range to get a list of urls for
          review.
        </p>

        <form
          className="editorial-links-form"
          name="editorial-links-form"
          onSubmit={renderLinks}
        >
          <label htmlFor="game-select">
            <p>Game</p>
            <select
              id="game-select"
              name="game-select"
              onChange={(event) =>
                setGame(event.target.value as GameChoiceEnum)
              }
              value={game}
            >
              {(Object.keys(GameChoices) as GameChoiceEnum[]).map(
                (name) => {
                  return (
                    <option value={name} key={`option-${name}`}>
                      {GameChoices[name]}
                    </option>
                  )
                },
              )}
            </select>
          </label>

          <label htmlFor="start-date-input">
            <p>Start Date</p>
            <input
              id="start-date-input"
              name="start-date-input"
              onChange={(event) => setStart(event.target.value)}
              type="date"
              value={start}
            />
          </label>

          <label htmlFor="end-date-input">
            <p>End Date</p>
            <input
              id="end-date-input"
              min={start}
              name="end-date-input"
              onChange={(event) => setEnd(event.target.value)}
              type="date"
              value={end}
            />
          </label>
          <input type="submit" value="Submit" />
        </form>
      </div>

      <div className="list-container">
        <ol className="url-list">
          {urls.map(({ url, dayOfTheWeek }) => {
            return (
              <li className="list-item" key={url}>
                <a href={url} rel="noreferrer" target="_blank">
                  {url}
                </a>
                <span className="day-of-the-week">
                  ({dayOfTheWeek.toUpperCase()})
                </span>
              </li>
            )
          })}
        </ol>
      </div>
    </div>
  )
}

export default EditorialLinksTool
