import type { UserType } from 'foundation'

import { AbraParams, AbraTests } from '../../window'
import { win } from '../aliases'
import { agentIdCookie, env, nytGeoCookie } from '../util'
import { captureSentryError } from '../sentry'

import { meetsRestrictions, setTestClasses } from './restrictions'
import limitedAudienceTests from './limitedAudienceTests'

let exposed: { [key: string]: boolean } = {}

const getOverrides = () => {
  const searchParams = new URLSearchParams(window.location.search)
  const overrides = searchParams.get('abra-overrides')
  const overridesMap: AbraTests = {}

  if (overrides) {
    const testArray = overrides.split(',')
    testArray.forEach((ovr) => {
      const [key, val] = ovr.split('=')
      overridesMap[key] = val
    })
  }

  return overridesMap
}

const init = (userType?: UserType) => {
  const rules = window?.abra?.config
  if (!rules) return

  const params: AbraParams = {
    agent_id: agentIdCookie,
    regi_id: userType?.regiId,
    geo_country: nytGeoCookie, // nyt-geo is an ISO 3166-1 alpha-2 code, so we must use geo_country, not geo_ip
  }
  const overrides = window.config?.AbraConfig || getOverrides()

  // initialize Abra to get a function to retrieve variants for a specific test
  const getVariant = win.Abra(rules, params, overrides)

  const restrictedGetVariant = (test: string) => {
    // If there are audience restrictions for this test, make sure that criteria is met first
    if (limitedAudienceTests[test]) {
      return meetsRestrictions(limitedAudienceTests[test], userType)
        ? getVariant(test)
        : null
    }

    return getVariant(test)
  }

  // Wrap Abra library getVeriant with user, device, and time restrictions
  win.abra.getVariant = restrictedGetVariant

  // Get and store allocations for every test for this user
  win.abra.tests = {}
  Object.keys({ ...rules, ...getOverrides() }).forEach((test) => {
    const variant = restrictedGetVariant(test)
    const testRestriction = limitedAudienceTests[test]

    win.abra.tests[test] = variant

    if (variant && testRestriction) {
      setTestClasses(variant, testRestriction)
    }
  })
}

const getTests = () => win.abra.tests

const getVariant = (test: string) => win?.abra?.tests[test]

/**
 * This function is necessary because the shape of the ab-expose event data and
 * the event name varies by platform. Incorrectly formatted data will not throw
 * an error to the app or the game. It simply wont show up.
 *
 * To validate your test objects, see:
 * https://et.stg.nyt.net/validator/view?client_library=Native+App+SDKs&subject=ab-expose
 */
export const formatExposureData = (test: string) => {
  // This is the shape of the data ET is expecting for the expose event
  const etData = {
    test,
    variant: getVariant(test) || null,
    config_ver: getVariant('.ver'),
    integration: `games-${env.name === 'prod' ? 'prd' : 'stg'}`,
  }

  if (!etData.config_ver && Math.random() < 0.01) {
    captureSentryError(
      new Error('abra exposure missing config_ver'),
      { test },
    )
  }

  // For web and newsreader, we must wrap the etdata in an abtest object
  const webAndNewsreaderData = {
    abtest: etData,
  }

  const isXWDapp = !!window.gamesAppPlatform
  /**
   * The XWD apps expect the event name to be with an underscore but all
   * other apps expect the event name with a dash.
   */
  const eventName = isXWDapp ? 'ab_expose' : 'ab-expose'
  const data = isXWDapp ? { ...etData } : { ...webAndNewsreaderData }

  return { data, eventName }
}

const reportExposure = (test: string) => {
  if (typeof window === undefined) return // prevent server-side exposes

  if (exposed[test]) return // prevent reporting multiple times for the same test:

  const { data, eventName } = formatExposureData(test)

  if (window.isHybridWebView && window.NativeBridge) {
    window.NativeBridge.sendAnalytic(eventName, data)
  } else {
    win.dataLayer = win.dataLayer || []
    win.dataLayer.push({ event: eventName, ...data })
  }

  exposed[test] = true
}

const reset = () => {
  exposed = {}
}

export const abra = {
  init,
  getTests,
  getVariant,
  reset,
  reportExposure,
  getOverrides,
}
