import { UserType } from 'foundation'

import type {
  AbraTestRestriction,
  AbraUserType,
  AbraPlatformType,
  AbraBodyClasses,
} from './limitedAudienceTests'
import {
  checkActiveDate,
  checkPlatforms,
  checkUsers,
  meetsRestrictions,
  setTestClasses,
} from './restrictions'

const pastDate = new Date('04/20/2023 00:00:000-0400')
const futureDate = new Date(pastDate)
futureDate.setFullYear(3023)

Object.defineProperty(window, 'location', {
  value: {
    search: '',
  },
  writable: true,
})

const testConfig: AbraTestRestriction = {}

const anonUser: Partial<UserType> = {
  isLoggedIn: false,
  hasXwd: false,
  hasDigi: false,
  regiId: null,
}

const regiUser: Partial<UserType> = {
  ...anonUser,
  isLoggedIn: true,
  regiId: '123',
}

const subUser: Partial<UserType> = {
  ...regiUser,
  hasXwd: true,
}

const digiUser: Partial<UserType> = {
  ...regiUser,
  hasDigi: true,
}

const adaUser: Partial<UserType> = {
  ...regiUser,
  hasDigi: true,
  hasXwd: true,
}

describe('Abra: Restricted Tests', () => {
  describe('Time duration Abra Tests', () => {
    it('is always active for tests without dates', () => {
      const activeTestWithoutDates = { ...testConfig }
      expect(checkActiveDate(activeTestWithoutDates)).toBe(true)
    })
    it("is active for tests that have start and end dates and today's date is in this timeframe", () => {
      const activeTestWithDates = {
        ...testConfig,
        startDate: pastDate,
        endDate: futureDate,
      }
      expect(checkActiveDate(activeTestWithDates)).toBe(true)
    })
    it('is active for tests that don`t have a start date and the end date is in the future', () => {
      const activeTestWithoutStart = {
        ...testConfig,
        endDate: futureDate,
      }
      expect(checkActiveDate(activeTestWithoutStart)).toBe(true)
    })
    it('is active for tests that don`t have an end date and the start date is in the past', () => {
      const activeTestWithoutEnd = {
        ...testConfig,
        startDate: pastDate,
      }
      expect(checkActiveDate(activeTestWithoutEnd)).toBe(true)
    })
    it('is not active for tests that have start and ends dates and the timeframe is in the future', () => {
      const inactiveTestWithDates = {
        ...testConfig,
        startDate: futureDate,
        endDate: futureDate,
      }
      expect(checkActiveDate(inactiveTestWithDates)).toBe(false)
    })
    it('is not active for tests that don`t have a start date and the end date is in the past', () => {
      const inactiveTestWithoutStart = {
        ...testConfig,
        endDate: pastDate,
      }
      expect(checkActiveDate(inactiveTestWithoutStart)).toBe(false)
    })
    it('is not active for tests that don`t have an end date and the start date is in the future', () => {
      const inactiveTestWithoutEnd = {
        ...testConfig,
        startDate: futureDate,
      }
      expect(checkActiveDate(inactiveTestWithoutEnd)).toBe(false)
    })
    it('allows a date-override url parameter to help developers test experiments in the future', () => {
      const date = new Date(futureDate)
      date.setDate(30)

      window.location.search = `?date-override=${date.toLocaleDateString()}`

      const futureTest = {
        ...testConfig,
        startDate: futureDate,
      }
      expect(checkActiveDate(futureTest)).toBe(true)
    })
  })
  describe('Audience restrictions', () => {
    describe('User audience', () => {
      it('is always active for tests without user audience', () => {
        expect(checkUsers([], regiUser)).toBe(true)
        expect(checkUsers([], anonUser)).toBe(true)
        expect(checkUsers([], subUser)).toBe(true)
        expect(checkUsers([], digiUser)).toBe(true)
        expect(checkUsers([], adaUser)).toBe(true)
      })

      it('regi tests are active for regi users', () => {
        expect(checkUsers(['regi'], regiUser)).toBe(true)
        expect(checkUsers(['regi'], anonUser)).toBe(false)
        expect(checkUsers(['regi'], subUser)).toBe(false)
        expect(checkUsers(['regi'], digiUser)).toBe(false)
        expect(checkUsers(['regi'], adaUser)).toBe(false)
      })

      it('regi and anon tests are active for regi and anon users', () => {
        expect(checkUsers(['regi', 'anon'], regiUser)).toBe(true)
        expect(checkUsers(['regi', 'anon'], anonUser)).toBe(true)
        expect(checkUsers(['regi', 'anon'], subUser)).toBe(false)
        expect(checkUsers(['regi', 'anon'], digiUser)).toBe(false)
        expect(checkUsers(['regi', 'anon'], adaUser)).toBe(false)
      })

      it('regi, anon, and digi tests are active for regi, anon, and digi users', () => {
        expect(checkUsers(['regi', 'anon', 'digi'], regiUser)).toBe(
          true,
        )
        expect(checkUsers(['regi', 'anon', 'digi'], anonUser)).toBe(
          true,
        )
        expect(checkUsers(['regi', 'anon', 'digi'], subUser)).toBe(
          false,
        )
        expect(checkUsers(['regi', 'anon', 'digi'], digiUser)).toBe(
          true,
        )
        expect(checkUsers(['regi', 'anon', 'digi'], adaUser)).toBe(
          false,
        )
      })

      it('regi, anon, digi, sub tests are active for regi, anon, digi, and sub users', () => {
        expect(
          checkUsers(['regi', 'anon', 'digi', 'sub'], regiUser),
        ).toBe(true)
        expect(
          checkUsers(['regi', 'anon', 'digi', 'sub'], anonUser),
        ).toBe(true)
        expect(
          checkUsers(['regi', 'anon', 'digi', 'sub'], subUser),
        ).toBe(true)
        expect(
          checkUsers(['regi', 'anon', 'digi', 'sub'], digiUser),
        ).toBe(true)
        expect(
          checkUsers(['regi', 'anon', 'digi', 'sub'], adaUser),
        ).toBe(false)
      })

      it('regi, anon, digi, sub, ada tests are active for regi, anon, digi, and sub users', () => {
        expect(
          checkUsers(
            ['regi', 'anon', 'digi', 'sub', 'ada'],
            regiUser,
          ),
        ).toBe(true)
        expect(
          checkUsers(
            ['regi', 'anon', 'digi', 'sub', 'ada'],
            anonUser,
          ),
        ).toBe(true)
        expect(
          checkUsers(['regi', 'anon', 'digi', 'sub', 'ada'], subUser),
        ).toBe(true)
        expect(
          checkUsers(
            ['regi', 'anon', 'digi', 'sub', 'ada'],
            digiUser,
          ),
        ).toBe(true)
        expect(
          checkUsers(['regi', 'anon', 'digi', 'sub', 'ada'], adaUser),
        ).toBe(true)
      })
    })
    describe('Platform restrictions', () => {
      it('is always active for tests without a platform specification', () => {
        // desktop
        window.isHybridWebView = false
        window.newsreaderAppPlatform = ''
        window.gamesAppPlatform = ''
        window.isMobileDevice = false

        expect(checkPlatforms([])).toBe(true)
      })

      it('web tests are active for desktop users', () => {
        // desktop
        window.isHybridWebView = false
        window.newsreaderAppPlatform = ''
        window.gamesAppPlatform = ''
        window.isMobileDevice = false

        expect(checkPlatforms(['web'])).toBe(true)
      })

      it('web tests are active for mobile users', () => {
        // mobile
        window.isHybridWebView = false
        window.newsreaderAppPlatform = ''
        window.gamesAppPlatform = ''
        window.isMobileDevice = true

        expect(checkPlatforms(['web'])).toBe(true)
      })

      it('desktop tests are active for desktop users', () => {
        // mobile
        window.isHybridWebView = false
        window.newsreaderAppPlatform = ''
        window.gamesAppPlatform = ''
        window.isMobileDevice = true

        expect(checkPlatforms(['desktop'])).toBe(false)

        // desktop
        window.isHybridWebView = false
        window.newsreaderAppPlatform = ''
        window.gamesAppPlatform = ''
        window.isMobileDevice = false

        expect(checkPlatforms(['desktop'])).toBe(true)
      })

      it('hybrid and web tests are active for desktop, mobile, hybrid but not newsreader', () => {
        // desktop
        window.isHybridWebView = false
        window.newsreaderAppPlatform = ''
        window.gamesAppPlatform = ''
        window.isMobileDevice = false

        expect(checkPlatforms(['web', 'hybrid'])).toBe(true)

        // games-ios
        window.isHybridWebView = true
        window.newsreaderAppPlatform = ''
        window.gamesAppPlatform = 'ios'
        window.isMobileDevice = false

        expect(checkPlatforms(['web', 'hybrid'])).toBe(true)

        // newsreader-android
        window.isHybridWebView = false
        window.newsreaderAppPlatform = 'android'
        window.gamesAppPlatform = ''
        window.isMobileDevice = false

        expect(checkPlatforms(['web', 'hybrid'])).toBe(false)
      })
    })

    it('is active for desktop anon users when a config targets anon desktop users', () => {
      const users: AbraUserType[] = ['anon']
      const platforms: AbraPlatformType[] = ['desktop']

      const test = {
        ...testConfig,
        audience: { users, platforms },
      }

      // desktop
      window.isHybridWebView = false
      window.newsreaderAppPlatform = ''
      window.gamesAppPlatform = ''
      window.isMobileDevice = false

      expect(meetsRestrictions(test, anonUser)).toBe(true)
    })

    it('is active for a regi user in mobile web', () => {
      const users: AbraUserType[] = ['anon', 'regi']
      const platforms: AbraPlatformType[] = ['web', 'hybrid']

      const test = {
        ...testConfig,
        audience: { users, platforms },
      }

      // mobile
      window.isHybridWebView = false
      window.newsreaderAppPlatform = ''
      window.gamesAppPlatform = ''
      window.isMobileDevice = true

      expect(meetsRestrictions(test, regiUser)).toBe(true)
    })

    it('is active for a regi user in hybrid', () => {
      const users: AbraUserType[] = ['anon', 'regi']
      const platforms: AbraPlatformType[] = ['web', 'hybrid']

      const test = {
        ...testConfig,
        audience: { users, platforms },
      }

      // ios games
      window.isHybridWebView = true
      window.newsreaderAppPlatform = ''
      window.gamesAppPlatform = 'ios'
      window.isMobileDevice = false

      expect(meetsRestrictions(test, regiUser)).toBe(true)
    })

    it('is not active for a sub user in games app', () => {
      const users: AbraUserType[] = ['anon', 'regi']
      const platforms: AbraPlatformType[] = ['web', 'hybrid']

      const test = {
        ...testConfig,
        audience: { users, platforms },
      }

      // ios games
      window.isHybridWebView = true
      window.newsreaderAppPlatform = ''
      window.gamesAppPlatform = 'ios'
      window.isMobileDevice = false

      expect(meetsRestrictions(test, subUser)).toBe(false)
    })

    it('is not active for an ada user on web', () => {
      const users: AbraUserType[] = ['anon', 'regi']
      const platforms: AbraPlatformType[] = ['web', 'hybrid']

      const test = {
        ...testConfig,
        audience: { users, platforms },
      }

      // desktop
      window.isHybridWebView = false
      window.newsreaderAppPlatform = ''
      window.gamesAppPlatform = ''
      window.isMobileDevice = false

      expect(meetsRestrictions(test, adaUser)).toBe(false)
    })

    it('will add classes to the body tag if the user is in the variant', () => {
      const classNames: AbraBodyClasses = {
        '1_variant': 'pz-variant-class',
      }

      const test = {
        ...testConfig,
        classNames,
      }

      setTestClasses('1_variant', test)
      const body = document.querySelector('body')

      expect(body?.classList).toContain('pz-variant-class')
    })
  })
})
