import type { UserType } from 'foundation'

import { setBodyClasses } from '../../../responsive/useBody'

import type {
  AbraTestRestriction,
  AbraUserType,
  AbraPlatformType,
} from './limitedAudienceTests'

const getDate = () => {
  const now = new Date(Date.now())

  if (typeof window === undefined) return now

  const searchParams = new URLSearchParams(window.location.search)
  const override = searchParams.get('date-override')

  if (override) return new Date(override)

  return now
}

export const checkActiveDate = (test: AbraTestRestriction) => {
  const dateNow = getDate()

  // No dates means the test is active until taken down
  if (!test.startDate && !test.endDate) return true

  // If there is a start date, check if it's in the timeframe
  if (test.startDate && test.startDate <= dateNow) {
    // No end date means it's up until taken down
    if (!test.endDate) return true

    return test.endDate >= dateNow
  }

  // No start date but has an end date means we should check the end date
  if (!test.startDate && test.endDate) return test.endDate >= dateNow

  return false
}

export const checkUsers = (
  users: AbraUserType[],
  userType?: Partial<UserType>,
) => {
  // If there are no user restrictions, return early
  if (users.length === 0) return true

  // We have user restrictions but there's no userType
  if (!userType) return false

  const check = (requirement: AbraUserType) => {
    switch (requirement) {
      case 'anon':
        return !userType.isLoggedIn && !userType.hasXwd
      case 'regi':
        return (
          userType.isLoggedIn && !userType.hasDigi && !userType.hasXwd
        )
      case 'sub':
        return userType.hasXwd && !userType.hasDigi
      case 'ada':
        return userType.hasXwd && userType.hasDigi
      case 'digi':
        return !userType.hasXwd && userType.hasDigi
      default:
        return false
    }
  }

  return users
    .map((req) => check(req))
    .some((value) => value === true)
}

export const checkPlatforms = (platforms: AbraPlatformType[]) => {
  if (platforms.length === 0) return true

  if (typeof window === undefined) return false

  const check = (requirement: AbraPlatformType) => {
    switch (requirement) {
      case 'web':
        return (
          !window.isHybridWebView && !window.newsreaderAppPlatform
        )
      case 'mobile-web':
        return (
          !window.isHybridWebView &&
          !window.newsreaderAppPlatform &&
          window.isMobileDevice
        )
      case 'desktop':
        return (
          !window.isHybridWebView &&
          !window.newsreaderAppPlatform &&
          !window.isMobileDevice
        )
      case 'games-ios':
        return window.gamesAppPlatform === 'ios'
      case 'games-android':
        return window.gamesAppPlatform === 'android'
      case 'newsreader-ios':
        return window.newsreaderAppPlatform === 'ios'
      case 'newsreader-android':
        return window.newsreaderAppPlatform === 'android'
      case 'hybrid':
        return window.isHybridWebView
      default:
        return false
    }
  }

  return platforms
    .map((req) => check(req))
    .some((value) => value === true)
}

export const meetsRestrictions = (
  test: AbraTestRestriction,
  userType?: Partial<UserType>,
) => {
  let usersOk = true
  let platformsOk = true
  let datesOk = true

  if (test.startDate || test.endDate) {
    datesOk = checkActiveDate(test)
  }

  if (test.audience) {
    const { users, platforms } = test.audience

    if (users) {
      usersOk = checkUsers(users, userType)
    }

    if (platforms) {
      platformsOk = checkPlatforms(platforms)
    }
  }

  return datesOk && usersOk && platformsOk
}

export const setTestClasses = (
  variant: string | null,
  test?: AbraTestRestriction,
) => {
  if (!test) return

  const { classNames } = test

  if (!classNames || !variant) return

  if (classNames[variant]) setBodyClasses([classNames[variant]])
}
