import type { UserEntitlement } from 'foundation'
import { width } from 'responsive'

import sov from './ads/kvps/sov'
import getVp from './ads/kvps/vp'
import getPlat from './ads/kvps/plat'
import getProp from './ads/kvps/prop'
import { getPageName, getQueryParams } from './util'
import pageViewId from './utils/pageViewId'
import { userType } from './user'
import { addClass, removeClass } from './dom'
import { win, doc } from './aliases'
import getAbraDFPTests from './ads/getAbraDFPTests'
import loadBiddingScripts from './ads/loadBiddingScripts'
import getSizeMappings from './ads/getSizeMappings'
import getActiveBidders from './ads/getActiveBidders'
import { adLifeCycleEventsToTrack } from './ads/adTracking/constants'
import { subscribeCallBack } from './ads/adTracking/adTracking'

const AD_PATH_BASE = '29390238/NYT/crosswords'

/**
 * Recreates server-side logic from Ad.js. Makes an exception for
 * Wordle, where the ad should show on tablet. Wordle handles hiding of
 * the ad on non-tablet mobile and in webviews in shouldShowAd.ts
 */
export const areAdsDisabled = () => {
  return (
    window.gamesAppPlatform ||
    (window.isMobileDevice &&
      !['wordle', 'hub', 'connections', 'mini-page'].includes(
        window.pageName || '',
      )) ||
    window.purrDirectives?.PURR_AdConfiguration_v3 ===
      'adluce-socrates'
  )
}

const getViewportHeight = () => {
  return window && window.innerHeight
}

win.AdSlot4 = win.AdSlot4 || {
  cmd: [],
  init: () => null,
  events: { subscribe: () => ({ remove: () => {} }) },
  refreshAds: () => null,
}
const AdSlot = win.AdSlot4

export const loadBidders = () => {
  return loadBiddingScripts(AdSlot, getPageName(), getActiveBidders())
}

export const getBaseTargeting = () => {
  return {
    plat: getPlat(),
    prop: getProp(),
    typ: 'games',
    sov,
    vp: getVp(width.value),
  }
}

/** initialize all ad related content */
export const initializeAds = (
  adPath: string,
  abraTests: object | undefined,
  retried: number = 0,
) => {
  const MAX_RETRY_TIMES = 10
  /*
   If we're in hybrid, then we're out of luck right now for gaining pageviewId.
   nyt_et seems to have a race with this script's execution. We'll need to ping to check if it's available.
   This only seems to be a small occurrence, and I've found it only repeats once or twice at the most,
   that puts us at 10~20ms range for delay with a max of 50ms which is acceptable.
   */
  if (
    !win.isHybridWebView &&
    !win.nyt_et &&
    retried < MAX_RETRY_TIMES
  ) {
    const WAIT_TIME = 5
    setTimeout(
      initializeAds,
      WAIT_TIME,
      adPath,
      abraTests,
      retried + 1,
    )
    return
  }
  AdSlot.cmd.push(() => {
    if (areAdsDisabled()) return

    adLifeCycleEventsToTrack.forEach((eventName) => {
      AdSlot.events.subscribe({
        name: eventName,
        scope: 'all',
        callback: subscribeCallBack(eventName),
      })
    })

    const adKeywords = getQueryParams('ad-keywords')
    const pvID = pageViewId.current

    const targeting = {
      ...getBaseTargeting(),
      ...(userType?.entitlement
        ? { sub: userType?.entitlement }
        : {}),
      page_view_id: pvID,
      ...(adKeywords ? { adv: adKeywords } : {}),
      abra_dfp: getAbraDFPTests(abraTests),
    }

    AdSlot.init({
      adTargeting: targeting,
      adUnitPath: `${AD_PATH_BASE}/${adPath}`,
      sizeMapping: getSizeMappings(adPath, getViewportHeight()),
    })

    AdSlot.events.subscribe({
      name: 'AdRendered',
      scope: 'all',
      callback: ({ slot, size }) => {
        if (slot) {
          const elm = doc.getElementById(slot.getSlotElementId())
          if (size && size[1] === 0) {
            // an empty size appears to imply fluid
            if (elm) {
              addClass(elm, 'fluid')
            }
          } else if (elm) {
            removeClass(elm, 'fluid')
          }
        }
      },
    })
  })
}

export const refreshAds = () => {
  // TODO: coordinate this with refactored pageViewId handling
  // then we can use it for frontend routing
  AdSlot.cmd.push(() => {
    if (areAdsDisabled()) return
    AdSlot.refreshAds()
  })
}

export const refreshAdsEntitlement = (
  entitlement: UserEntitlement,
) => {
  window?.AdSlot4?.updateAdReq?.({ sub: entitlement })
}
