import setLocalStorageValues from '../setLocalStorageValues'
import {
  BT_KEY,
  MKTG_KEY,
  SUB_KEY,
  TEST_CLIENTSIDE_KEY,
} from '../constants'

describe('setLocalStorageValues', () => {
  it('does not call setItem', async () => {
    const setItemSpy = jest.fn()
    const ls = {
      setItem: setItemSpy,
    }
    await setLocalStorageValues(ls)({
      notBt: 'test',
      notMktg: 'test1',
      not_sub: 'test2',
      not_clientside: 'test3',
    })
    expect(setItemSpy).not.toHaveBeenCalled()
  })
  it('calls setItem when bt key exists', async () => {
    const expectedValue = 'TESTING_VALUE'
    const setItemSpy = jest.fn()
    const ls = {
      setItem: setItemSpy,
    }
    await setLocalStorageValues(ls)({
      [BT_KEY]: expectedValue,
      notMktg: 'test1',
      not_sub: 'test2',
      not_clientside: 'test3',
    })
    expect(setItemSpy).toHaveBeenCalledWith(BT_KEY, expectedValue)
  })
  it('calls setItem when mktg key exists', async () => {
    const expectedValue = 'TESTING_VALUE'
    const setItemSpy = jest.fn()
    const ls = {
      setItem: setItemSpy,
    }
    await setLocalStorageValues(ls)({
      [MKTG_KEY]: expectedValue,
      notBt: 'test1',
      not_sub: 'test2',
      not_clientside: 'test3',
    })
    expect(setItemSpy).toHaveBeenCalledWith(MKTG_KEY, expectedValue)
  })
  it('calls setItem when sub key exists', async () => {
    const expectedValue = 'TESTING_VALUE'
    const setItemSpy = jest.fn()
    const ls = {
      setItem: setItemSpy,
    }
    await setLocalStorageValues(ls)({
      [SUB_KEY]: expectedValue,
      notBt: 'test1',
      notMktg: 'test2',
      not_clientside: 'test3',
    })
    expect(setItemSpy).toHaveBeenCalledWith(SUB_KEY, expectedValue)
  })
  it('calls setItem when als_clientside key exists and prepends with ls-', async () => {
    const expectedValue = 'TESTING_VALUE'
    const expectedModifiedValue = `ls-${expectedValue}`
    const setItemSpy = jest.fn()
    const ls = {
      setItem: setItemSpy,
    }
    await setLocalStorageValues(ls)({
      [TEST_CLIENTSIDE_KEY]: expectedValue,
      notBt: 'test1',
      notMktg: 'test2',
      not_sub: 'test3',
    })
    expect(setItemSpy).toHaveBeenCalledWith(
      TEST_CLIENTSIDE_KEY,
      expectedModifiedValue,
    )
  })
  it('returns back expected data passed in', () => {
    const setItemSpy = jest.fn()
    const ls = {
      setItem: setItemSpy,
    }

    const initialData = {
      notBt: 'test',
      notMktg: 'test1',
      not_sub: 'test2',
      not_clientside: 'test3',
    }

    return expect(
      setLocalStorageValues(ls)(initialData),
    ).resolves.toEqual(initialData)
  })
  it('does not return modified data from data passed in when a valid key exists', () => {
    const setItemSpy = jest.fn()
    const ls = {
      setItem: setItemSpy,
    }

    const initialData = {
      [BT_KEY]: 'test',
      notMktg: 'test1',
      not_sub: 'test2',
      not_clientside: 'test3',
    }

    return expect(
      setLocalStorageValues(ls)(initialData),
    ).resolves.toEqual(initialData)
  })
  it('should reject with expected error if localStorage.setItem throws', () => {
    const expectedError = new Error('Error: FooBar')
    const setItem = () => {
      throw expectedError
    }

    const initialData = {
      [BT_KEY]: 'test',
      notMktg: 'test1',
      not_sub: 'test2',
      not_clientside: 'test3',
    }

    return expect(
      setLocalStorageValues({ setItem })(initialData),
    ).rejects.toEqual(expectedError)
  })
})
