/* eslint-disable global-require */

jest.mock('./inView', () => jest.fn())

jest.mock('./tracking', () => ({
  track: jest.fn(),
}))

jest.mock('./xhr', () => ({
  xhr: {
    get: jest.fn(),
    post: jest.fn(),
  },
}))

let xhr
let ccpa
let track
let inView
let footerLink
let menuLink

const setup = (proptions) => {
  document.body.innerHTML = ''

  jest.resetModules()

  const options = {
    env: 'prod',
    loggedIn: true,
    isEligible: true,
    linkType: 'optOut',
    ...proptions,
  }

  window.env = { name: options.env }

  xhr = require('./xhr').xhr

  track = require('./tracking').track
  inView = require('./inView')

  if (options.isEligible) {
    footerLink = document.createElement('div')
    footerLink.innerHTML =
      '<span data-region="footer">footer link text</span>'
    footerLink.setAttribute('data-region', 'footer')

    menuLink = document.createElement('div')
    menuLink.innerHTML =
      '<span data-region="menu">menu link text</span>'
    menuLink.setAttribute('data-region', 'menu')

    if (options.linkType === 'optOut') {
      footerLink.classList.add('ccpa-link', 'ccpa-opt-out')

      footerLink.setAttribute('data-track', 'linkOptOut')
      menuLink.classList.add('ccpa-link', 'ccpa-opt-out')
      menuLink.setAttribute('data-track', 'linkOptOut')
    }

    document.body.appendChild(menuLink)
    document.body.appendChild(footerLink)
  }

  ccpa = require('./ccpa').default
}

const teardown = () => {
  if (footerLink && footerLink.parentNode) {
    document.body.removeChild(footerLink)
  }

  if (menuLink && menuLink.parentNode) {
    document.body.removeChild(menuLink)
  }

  footerLink = null
  menuLink = null
}

const click = (elm) => {
  const evt = document.createEvent('HTMLEvents')

  evt.initEvent('click', false, true)
  elm.dispatchEvent(evt)
}

const footerInteractionTest = (type, moduleName) => {
  test('clicking footer link tracks interaction', () => {
    setup({ linkType: type })
    ccpa()

    expect(track).not.toHaveBeenCalled()

    click(footerLink)

    expect(track).toHaveBeenCalledWith('moduleInteraction', {
      eventData: {
        pagetype: 'game',
        trigger: 'module',
        type: 'click',
      },
      module: {
        name: moduleName,
        region: 'footer',
        label: 'footer link text',
      },
    })
  })
}

const menuInteractionTest = (type, moduleName) => {
  test('clicking menu link tracks interaction', () => {
    setup({ linkType: type })
    ccpa()

    expect(track).not.toHaveBeenCalled()

    click(menuLink)

    expect(track).toHaveBeenCalledWith('moduleInteraction', {
      eventData: {
        pagetype: 'game',
        trigger: 'module',
        type: 'click',
      },
      module: {
        name: moduleName,
        region: 'menu',
        label: 'menu link text',
      },
    })

    teardown()
  })
}

const nonCcpaUsers = (className) => {
  test('does nothing for non ccpa users', () => {
    setup({ isEligible: false })
    expect(document.getElementsByClassName(className)).toHaveLength(0)
    ccpa()
    teardown()
  })
}

describe('CCPA OPT OUT', () => {
  beforeEach(() => {
    window.ccpa = false
  })

  footerInteractionTest('optOut', 'DNSS opt-out link')
  menuInteractionTest('optOut', 'DNSS opt-out link')
  nonCcpaUsers('ccpa-opt-out')

  test('calls purr cache', () => {
    setup()
    ccpa()
    expect(xhr.get).toHaveBeenCalled()
    teardown()
  })

  test('track footer in view', () => {
    setup()
    ccpa()

    expect(track).not.toHaveBeenCalled()

    const [elm, view] = inView.mock.calls.find(
      ([elm]) => elm.textContent === 'footer link text',
    )

    view({ target: elm })

    expect(track).toHaveBeenCalledWith('impression', {
      module: {
        name: 'DNSS opt-out link',
        label: 'footer link text',
        region: 'footer',
      },
    })

    teardown()
  })

  test('track menu in view', () => {
    setup()
    ccpa()

    expect(track).not.toHaveBeenCalled()

    const [elm, view] = inView.mock.calls.find(
      ([elm]) => elm.textContent === 'menu link text',
    )

    view({ target: elm })

    expect(track).toHaveBeenCalledWith('impression', {
      module: {
        name: 'DNSS opt-out link',
        label: 'menu link text',
        region: 'menu',
      },
    })

    teardown()
  })
})
