import { device } from './device'
import { user, userType } from './user'
import { getPageName, jsHook, createQueryString } from './util'
import pageViewId from './utils/pageViewId'
import { getElementById } from './dom'
import { win, doc } from './aliases'
import { trackClick } from './tracking'
import { parsePurrCookieValue } from './parsePurrCookieValue'

const FEEDBACK_LINK = 'mailto:nytgames@nytimes.com'

const createFeedbackEmailHref = (
  user: { id: number },
  device: { feedback: string },
  subject = 'Web Feedback',
) => {
  const page_view_id = pageViewId.current
  const emailSubject = `${subject} (${page_view_id.slice(-6)})`

  const { screen } = win
  const rootElm = doc.documentElement

  const tz = new Date().getTimezoneOffset()

  const yes = 'Yes'
  const no = 'No'

  // NOTE:
  // the substitution order must match
  // the stored template, verify very carefully if changing
  const substitutions = [
    win.location.pathname,
    screen.width,
    screen.height,
    rootElm.clientWidth,
    rootElm.clientHeight,
    `UTC${tz > 0 ? '' : '+'}${tz / -60}`,
    user.id ? user.id : 'anon',
    userType?.hasDigi ? yes : no,
    userType?.hasXwd ? yes : no,
    page_view_id,
  ]

  let body = device.feedback

  const token = /%s/

  let i = 0

  // Remove the HD property which is set server side
  // See: https://jira.nyt.net/browse/XWD-22673
  body = body.replace('\nHd: %s', '')

  while (i < substitutions.length) {
    body = body.replace(token, substitutions[i].toString())
    i += 1
  }

  return (
    FEEDBACK_LINK + createQueryString({ subject: emailSubject, body })
  )
}

export const getFeedbackLink = (subject?: string) => {
  return Promise.all([user.get(), device.get()])
    .then(([user, device]) => {
      return createFeedbackEmailHref(user, device, subject)
    })
    .catch(() => {
      return FEEDBACK_LINK
    })
}

const addFooterTrackers = () => {
  const footerElements = [
    ...document.querySelectorAll(`.${jsHook('footer-tracking')}`),
  ] as HTMLElement[]

  const name = getPageName()

  footerElements.forEach((elem) => {
    const label = elem?.dataset?.trackLabel || ''
    const medium = elem?.dataset?.medium
    const source = elem?.dataset?.source

    if (!label) {
      return
    }

    elem.addEventListener('click', (e) => {
      trackClick({
        name,
        label,
        useBeacon: true,
        context: null,
        medium,
        source,
      })
    })
  })
}

export const addFidesLink = () => {
  const directives = parsePurrCookieValue()
  const enableFides = directives.PURR_FidesTCF === 'show'

  if (enableFides) {
    const listItem = document.getElementById(
      jsHook('privacy-preferences-link'),
    )

    if (!listItem) return

    listItem.classList.add('fides-tcf-manage')
  }
}

export const initializeFooter = () => {
  const linkNode = getElementById(jsHook('feedback-link'))

  if (!linkNode) return

  getFeedbackLink().then((link) => {
    linkNode.setAttribute('href', link)
  })

  addFooterTrackers()
  addFidesLink()
}
