import initMock, { parseUserMock } from '../initMock'
import {
  COMMAND_AUTHENTICATE_USER,
  COMMAND_GET_USER_DETAILS,
  bridgeCommands,
} from '../commands'

describe('NATIVE BRIDGE: Mock Initialization', () => {
  const win = {
    navigator: {},
    location: {
      search: '',
    },
  }

  initMock(win)

  it('implements the list of bridge commands on the NativeBridge object', () => {
    // eslint-disable-next-line
    bridgeCommands.forEach((command) => {
      expect(win.NativeBridge[command]).toBeDefined()
    })
  })

  describe('Mocking user types', () => {
    const win = {
      location: {
        search: '',
      },
    }

    it('allows devs to mock a subscribed user through the query string', () => {
      win.location.search = 'user=sub'
      const userMock = parseUserMock(win)

      expect(userMock.isSubscribed).toBe(true)
      expect(userMock.isUserLoggedIn).toBe(true)
      expect(userMock.regiID).toBe(131999385)
    })

    it('allows devs to mock a regi user through the query string', () => {
      win.location.search = 'user=regi'
      const userMock = parseUserMock(win)

      expect(userMock.isSubscribed).toBe(false)
      expect(userMock.isUserLoggedIn).toBe(true)
      expect(userMock.regiID).toBe(131999385)
    })

    it('defaults to anon if not specified', () => {
      win.location.search = ''
      const userMock = parseUserMock(win)

      expect(userMock.isSubscribed).toBe(false)
      expect(userMock.isUserLoggedIn).toBe(false)
      expect(userMock.regiID).toBe(0)
    })

    it('should return the correct mock user when the authenticate command is called', async () => {
      win.location.search = 'user=sub'
      win.navigator = {}

      initMock(win)

      const {
        values: { gamesAuthenticateUser },
      } = await win.NativeBridge[COMMAND_AUTHENTICATE_USER]()

      expect(gamesAuthenticateUser.isSubscribed).toBe(true)
      expect(gamesAuthenticateUser.isUserLoggedIn).toBe(true)
      expect(gamesAuthenticateUser.regiID).toBe(131999385)
    })

    it('should return the correct mock user when the getuserdetails command is called', async () => {
      win.location.search = 'user=regi'
      win.navigator = {}

      initMock(win)

      const {
        values: { gamesGetUserDetails },
      } = await win.NativeBridge[COMMAND_GET_USER_DETAILS]()

      expect(gamesGetUserDetails.isSubscribed).toBe(false)
      expect(gamesGetUserDetails.isUserLoggedIn).toBe(true)
      expect(gamesGetUserDetails.regiID).toBe(131999385)
    })

    it('anon users should also be returned from getuserdetails', async () => {
      win.location.search = 'user=anon'
      win.navigator = {}

      initMock(win)

      const {
        values: { gamesGetUserDetails },
      } = await win.NativeBridge[COMMAND_GET_USER_DETAILS]()

      expect(gamesGetUserDetails.isSubscribed).toBe(false)
      expect(gamesGetUserDetails.isUserLoggedIn).toBe(false)
      expect(gamesGetUserDetails.regiID).toBe(0)
    })
  })
})
