/**
 * Defines all expected commands (triggered here) and events (triggered natively)
 */
export const COMMAND_SEND_ANALYTIC = 'sendAnalytic'
export const COMMAND_PTR_ENABLED = 'setPullToRefreshEnabled'
export const COMMAND_AUTHENTICATE_USER = 'gamesAuthenticateUser'
export const COMMAND_BACK_TO_HUB = 'gamesBackToHub'
export const COMMAND_CACHE_REFRESH = 'gamesCacheRefresh'
export const COMMAND_GET_PAGE_VIEW_ID = 'getPageViewID'
export const COMMAND_GET_USER_DETAILS = 'gamesGetUserDetails'
export const COMMAND_INITIALIZE_STATE = 'gamesInitializeState'
export const COMMAND_SEND_EMAIL = 'gamesSendEmail'
export const COMMAND_SHARE = 'share'
export const COMMAND_SET_NATIVE_COLOR_THEME =
  'gamesSetNativeColorTheme'
export const EVENT_ON_NAVIGATE_BACK = 'gamesOnNavigateBack'
export const EVENT_GET_GAME_DATA = 'gamesGetGameData'

/**
 * Defines the allowable bridge command name literals
 */
export type BridgeCommand =
  | typeof COMMAND_SEND_ANALYTIC
  | typeof COMMAND_PTR_ENABLED
  | typeof COMMAND_AUTHENTICATE_USER
  | typeof COMMAND_BACK_TO_HUB
  | typeof COMMAND_CACHE_REFRESH
  | typeof COMMAND_GET_PAGE_VIEW_ID
  | typeof COMMAND_GET_USER_DETAILS
  | typeof COMMAND_INITIALIZE_STATE
  | typeof COMMAND_SEND_EMAIL
  | typeof COMMAND_SHARE
  | typeof COMMAND_SET_NATIVE_COLOR_THEME

/**
 * The list of all the bridge commands to be implemented and enabled
 */
export const bridgeCommands: BridgeCommand[] = [
  COMMAND_SEND_ANALYTIC,
  COMMAND_PTR_ENABLED,
  COMMAND_AUTHENTICATE_USER,
  COMMAND_BACK_TO_HUB,
  COMMAND_CACHE_REFRESH,
  COMMAND_GET_PAGE_VIEW_ID,
  COMMAND_GET_USER_DETAILS,
  COMMAND_INITIALIZE_STATE,
  COMMAND_SEND_EMAIL,
  COMMAND_SHARE,
  COMMAND_SET_NATIVE_COLOR_THEME,
]

/**
 * Defines the allowable bridge event name literals
 */
type BridgeEvent =
  | typeof EVENT_ON_NAVIGATE_BACK
  | typeof EVENT_GET_GAME_DATA

export type NativeBridgeCommandState = {
  success: boolean
  error: string
}

export type NativeBridgeResponse<T> = Promise<T>

export type InitializeStateResponse = NativeBridgeCommandState & {
  values: {
    gamesInitializeState: {
      isReturningFromBackground: boolean
    }
  }
}

type UserDetails = {
  isUserLoggedIn: boolean
  isSubscribed: boolean
  regiID: string
  email: string
  isFreeTrial?: boolean
}

export type UserDetailsResponse = NativeBridgeCommandState & {
  values: {
    gamesGetUserDetails: UserDetails
  }
}

export type AuthenticateUserResponse = NativeBridgeCommandState & {
  values: {
    gamesAuthenticateUser: {
      isUserLoggedIn: boolean
      isSubscribed: boolean
      regiID: number
    }
    aborted: boolean | string // true if the user chose to cancel the auth flow
  }
}

export enum SEND_EMAIL_TYPE {
  // Generic
  SUGGESTION = 'suggest',
  FEEDBACK = 'feedback',
  CUSTOM = 'custom',

  // Games
  LETTERBOXED = 'feedback-letter-boxed',
  SUDOKU = 'feedback-sudoku',
  TILES = 'feedback-tiles',
  WORDLE = 'feedback-wordle',
  // @TODO: make sure android/ios are ready to accept this connections type
  CONNECTIONS = 'feedback-connections',
}

export enum NATIVE_COLOR_THEME_ENUM {
  LIGHT = 'light',
  DARK = 'dark',
}
export type NATIVE_COLOR_THEME_TYPE = `${NATIVE_COLOR_THEME_ENUM}`

/**
 * A type enforcing the Games Native Bridge contract
 */
type GamesNativeBridgeCommands = {
  /**
   * Accepts an event name string and event data
   */
  [COMMAND_SEND_ANALYTIC]: (
    eventName: string,
    options: Record<string, any>,
  ) => NativeBridgeResponse<NativeBridgeCommandState>

  /**
   * Instructs the native app to enable pull to refresh
   */
  [COMMAND_PTR_ENABLED]: (
    value: boolean,
  ) => NativeBridgeResponse<NativeBridgeCommandState>

  /**
   * Requests the native bridge to login or subscribe. Returns user details
   */
  [COMMAND_AUTHENTICATE_USER]: (
    type: 'login' | 'subscribe',
  ) => NativeBridgeResponse<AuthenticateUserResponse>

  /**
   * Navigates to the Hub screen
   */
  [COMMAND_BACK_TO_HUB]: () => NativeBridgeResponse<NativeBridgeCommandState>

  /**
   * Clears the user's local cache
   */
  [COMMAND_CACHE_REFRESH]: () => NativeBridgeResponse<NativeBridgeCommandState>

  /**
   * Get the PVID 
   */
  [COMMAND_GET_PAGE_VIEW_ID]: () => NativeBridgeResponse<String>

  /**
   * Get the user's information from the bridge
   */
  [COMMAND_GET_USER_DETAILS]: () => NativeBridgeResponse<UserDetailsResponse>

  /**
   * TK: What does this do and why?
   */
  [COMMAND_INITIALIZE_STATE]: () => NativeBridgeResponse<InitializeStateResponse>

  /**
   * Opens the send feedback email
   */

  [COMMAND_SEND_EMAIL]: ({
    type,
    debugInfo,
    emailTitle,
    emailDestination,
  }: {
    type: SEND_EMAIL_TYPE
    debugInfo: string
    emailTitle?: string
    emailDestination?: string
  }) => NativeBridgeResponse<NativeBridgeCommandState>

  /**
   * Opens native share
   */
  [COMMAND_SHARE]: (
    url: string,
    text: string,
    title: string,
  ) => NativeBridgeResponse<NativeBridgeCommandState>

  /**
   * Sets native theme
   */
  [COMMAND_SET_NATIVE_COLOR_THEME]: (
    theme: NATIVE_COLOR_THEME_TYPE,
  ) => NativeBridgeResponse<NativeBridgeCommandState>
}

type GameNames = {
  spellingBee?: string
  wordle?: string
} // Expand as we need data from other games
type GameInfo = {
  puzzleID: number
  analyticsPuzzleID: string
} // Expand & split into specific game types as we return more complex data
type BridgeEventPayload =
  | boolean
  | {
      [Property in keyof GameNames]: GameInfo
    }
export type BridgeListenerResponse = Partial<
  Record<BridgeEvent, BridgeEventPayload>
>

export type BridgeListener = (event: {
  respondWith: (res: BridgeListenerResponse) => void
}) => void

type GamesNativeBridgeListeners = {
  addEventListener: (
    eventName: BridgeEvent,
    listener: BridgeListener,
  ) => void
  removeEventListener: (
    eventName: BridgeEvent,
    listener: BridgeListener,
  ) => void
}

export type GamesNativeBridge = GamesNativeBridgeCommands &
  GamesNativeBridgeListeners

export type NativeBridgeCall = (
  command: BridgeCommand,
  ...args: any[]
) => NativeBridgeResponse<any>

/**
 * The formal definition of the Games Native Bridge contract defined above. Returns an object
 * that contains all of the agreed bridge commands, their parameters, and return types.
 */
export const defineContract = (
  callNativeBridgeCommand: NativeBridgeCall,
): GamesNativeBridgeCommands => {
  const bridge: GamesNativeBridgeCommands = {
    [COMMAND_SEND_ANALYTIC]: (
      eventName: string,
      options: Record<string, any>,
    ) =>
      callNativeBridgeCommand(COMMAND_SEND_ANALYTIC, {
        eventName,
        options: options || {},
      }),

    [COMMAND_PTR_ENABLED]: (value: boolean) =>
      callNativeBridgeCommand(COMMAND_PTR_ENABLED, {
        value,
      }),

    [COMMAND_AUTHENTICATE_USER]: (type: 'login' | 'subscribe') =>
      callNativeBridgeCommand(COMMAND_AUTHENTICATE_USER, {
        type,
      }),

    [COMMAND_BACK_TO_HUB]: () =>
      callNativeBridgeCommand(COMMAND_BACK_TO_HUB),

    [COMMAND_CACHE_REFRESH]: () =>
      callNativeBridgeCommand(COMMAND_CACHE_REFRESH),

    [COMMAND_GET_PAGE_VIEW_ID]: () =>
      callNativeBridgeCommand(COMMAND_GET_PAGE_VIEW_ID),

    [COMMAND_GET_USER_DETAILS]: () =>
      callNativeBridgeCommand(COMMAND_GET_USER_DETAILS),

    [COMMAND_INITIALIZE_STATE]: () =>
      callNativeBridgeCommand(COMMAND_INITIALIZE_STATE),

    [COMMAND_SEND_EMAIL]: ({
      type,
      debugInfo,
      emailTitle,
      emailDestination,
    }: {
      type: SEND_EMAIL_TYPE
      debugInfo: string
      emailTitle?: string
      emailDestination?: string
    }) =>
      callNativeBridgeCommand(COMMAND_SEND_EMAIL, {
        type,
        debugInfo,
        emailTitle,
        emailDestination,
      }),

    [COMMAND_SHARE]: (url: string, text: string, title: string) =>
      callNativeBridgeCommand(COMMAND_SHARE, {
        url,
        text,
        title,
      }),

    [COMMAND_SET_NATIVE_COLOR_THEME]: (
      theme: NATIVE_COLOR_THEME_TYPE,
    ) =>
      callNativeBridgeCommand(COMMAND_SET_NATIVE_COLOR_THEME, {
        theme,
      }),
  }

  return bridge
}
