/**
 * This file provides centralized logic for detecting hybrid capabilities.
 * The logic for determining these window properties' values lives in responsive/hybrid.ts
 * so that they can be made available before any React rendering logic runs
 */

/**
 * Checks the window.newsreaderAppPlatform value
 */
export const isAndroidNewsreader = () =>
  window.newsreaderAppPlatform === 'android'

/**
 * Checks the window.gamesAppPlatform value
 */
export const isAndroidGamesApp = () =>
  window.gamesAppPlatform === 'android'

/**
 * Checks the window.newsreaderAppPlatform value
 */
export const isiOSNewsreader = () =>
  window.newsreaderAppPlatform === 'ios'

/**
 * Checks the window.isPlayTab value
 */
export const isPlayTab = () => window.isPlayTab === true

/**
 * Checks the window.gamesAppPlatform value
 */
export const isiOSGamesApp = () => window.gamesAppPlatform === 'ios'

/**
 * Returns true if user is using the Android Newsreader or Android Games app in a Hybrid Webview
 */
export const isAndroidHybridWebview = () =>
  (isAndroidGamesApp() || isAndroidNewsreader()) &&
  window.isHybridWebView

/**
 * Returns true if user is using the iOS Newsreader or iOS Games app in a Hybrid Webview
 */
export const isiOSHybridWebview = () =>
  (isiOSGamesApp() || isiOSNewsreader()) && window.isHybridWebView

/**
 * Returns true if a semantic version meets its minimum requirement
 */
export const meetsMinimumVersion = (
  appVersion: string,
  minVersion: string,
) => {
  if (!appVersion || !minVersion) {
    return false
  }

  const [major, minor = 0, patch = 0] = appVersion
    .split('.')
    .map(Number)

  const [minMajor, minMinor = 0, minPatch = 0] = minVersion
    .split('.')
    .map(Number)

  // Compare major versions and if neither check returns
  // It means major and minMajor are equal
  if (major > minMajor) return true

  if (major < minMajor) return false

  // Now compare minor versions and if neither check returns
  // It means minor and minMinor are equal
  if (minor > minMinor) return true

  if (minor < minMinor) return false

  // Now compare patch versions and if neither check returns
  // It means patch and minPatch are equal
  if (patch > minPatch) return true

  if (patch < minPatch) return false

  // If we haven't returned yet, that means that version and minVersion are equal
  return true
}
