import { initNativeBridge } from '@nyt/native-bridge'

import hybridDebugger from './hybridDebugger'
import {
  BridgeCommand,
  bridgeCommands,
  defineContract,
  GamesNativeBridge,
} from './commands'
import android from './android'
import ios from './ios'
import {
  isAndroidHybridWebview,
  isiOSHybridWebview,
} from './detection'

/**
 * Determines if the criteria has been met for initializing the Native Bridge
 */
export const shouldInitNativeBridge = (w = window) =>
  isAndroidHybridWebview() || isiOSHybridWebview()

/**
 * For some reason we need to do this. I don't know why, and also it still works
 * even if we don't do this, but we need to do this. ::shrug::
 */
export const setupFakeAsset = (w = window) => {
  w.asset = {
    uri: 'fakeAssetUri',
  }
}

/**
 * A requirement for the Native Bridge. The commands must exist on an object
 * window.navigator.native.bridgeCommands with a property "enabled: true"
 */
export const enableListOfCommands = (
  w = window,
  bridgeCommands: BridgeCommand[],
) => {
  w.navigator.native = {
    bridgeCommands: {},
  }

  bridgeCommands.forEach((command) => {
    w.navigator.native.bridgeCommands[command] = {
      enabled: true,
    }
  })
}

/**
 * A requirement for the Native Bridge. The commands must exist on an object
 * window.navigator.native.bridgeCommands with a property "enabled: true"
 */
export const initializeNativeBridge = (w = window) => {
  const NativeBridge = initNativeBridge(w, true)
  const { callNativeBridgeCommand } = NativeBridge

  return { NativeBridge, callNativeBridgeCommand }
}

/**
 * This is the implentation of the Games Native Bridge. The commands defined in
 * commands.ts are enabled on the window.navigator.native object, and the commands
 * themselves are attached to the NativeBridge object.
 */
export const applyContract = (
  w = window,
  bridgeCommands: BridgeCommand[],
): GamesNativeBridge => {
  setupFakeAsset(w)
  enableListOfCommands(w, bridgeCommands)

  const { NativeBridge, callNativeBridgeCommand } =
    initializeNativeBridge(w)

  const bridgeCommandWithDebugger = (
    command: BridgeCommand,
    ...args: any[]
  ) => {
    const response = callNativeBridgeCommand(command, ...args)
    hybridDebugger(
      `Executing native bridge command: ${command}`,
      args,
      response,
    )

    return response
  }

  const contract = defineContract(bridgeCommandWithDebugger)

  bridgeCommands.forEach((command: BridgeCommand) => {
    NativeBridge[command] = contract[command]
  })

  return NativeBridge
}

/**
 * Sets up the Games Native Bridge, and kick off any platform-specific work
 */
export default function (w = window) {
  const NativeBridge = applyContract(w, bridgeCommands)

  if (isAndroidHybridWebview()) {
    android(NativeBridge)
  } else if (isiOSHybridWebview()) {
    ios(NativeBridge)
  } else {
    hybridDebugger('Platform is neither android nor ios', [
      w.newsreaderAppPlatform,
      w.gamesAppPlatform,
    ])
  }

  // Attach native bridge to window object
  w.NativeBridge = NativeBridge
}
