import android from './android'
import {
  BridgeCommand,
  bridgeCommands,
  COMMAND_AUTHENTICATE_USER,
  COMMAND_GET_PAGE_VIEW_ID,
  COMMAND_GET_USER_DETAILS,
  COMMAND_INITIALIZE_STATE,
  defineContract,
} from './commands'
import {
  isAndroidHybridWebview,
  isiOSHybridWebview,
} from './detection'
import ios from './ios'

/**
 * Checks for a bridge=mock parameter in the query string
 */
export function shouldMockNativeBridge(d = document) {
  const qs = d?.location?.search.toLowerCase() || ''

  return qs.indexOf('bridge=mock') !== -1
}

export type MockedUser = {
  isUserLoggedIn: boolean
  isSubscribed: boolean
  regiID: number
  email: string
}

/**
 * A mocked sub user object returned from the bridge
 */
export const subMock: MockedUser = {
  isUserLoggedIn: true,
  isSubscribed: true,
  regiID: 131999385,
  email: 'sub.email@nytimes.com',
}

/**
 * A mocked regi user object returned from the bridge
 */
export const regiMock: MockedUser = {
  isUserLoggedIn: true,
  isSubscribed: false,
  regiID: 131999385,
  email: 'regi.email@nytimes.com',
}

/**
 * A mocked anon user object returned from the bridge
 */
export const anonMock: MockedUser = {
  isUserLoggedIn: false,
  isSubscribed: false,
  regiID: 0,
  email: '',
}

/**
 * Checks the query string for a user paraneter and mocks the user type
 */
export const parseUserMock = (w = window) => {
  const searchParams = new URLSearchParams(w.location.search)
  const userParam = searchParams.get('user')

  switch (userParam) {
    case 'sub':
      return subMock
    case 'regi':
      return regiMock
    default:
      return anonMock
  }
}

/**
 * A mocked command success message
 */
export const commandSuccessMock = { success: true }

/**
 * Returns a mocked response object for the authenticate command
 */
export const authenticateUserRequestMock = (w = window) => {
  const userMock = parseUserMock(w)

  return {
    ...commandSuccessMock,
    values: {
      gamesAuthenticateUser: { ...userMock },
      aborted: false,
    },
  }
}

/** A mocked PVID response */
export const pageViewIDMock = {
  ...commandSuccessMock,
  values: {
    pageViewID: "mock PVID from mock bridge",
  },
}

/**
 * Returns a mocked response object for the user details command
 */
export const userDetailsRequestMock = (w = window) => {
  const userMock = parseUserMock(w)

  return {
    ...commandSuccessMock,
    values: {
      gamesGetUserDetails: { ...userMock },
    },
  }
}

/**
 * A mocked request state object
 */
export const stateRequestMock = {
  ...commandSuccessMock,
  values: {
    gamesInitializeState: { isReturningFromBackground: false },
  },
}

/**
 * Returns an empty object for Native Bridge and a no-op for callNativeBridgeCommand
 */
export const initializeMockBridge = (w = window) => {
  const NativeBridge: any = {}
  const callNativeBridgeCommand = (
    command: string,
    ...args: any[]
  ) => {
    // eslint-disable-next-line no-console
    console.log(
      `[NATIVE BRIDGE MOCK]: ${command} called by web app`,
      args,
    )

    switch (command) {
      case COMMAND_INITIALIZE_STATE:
        console.log(
          `[NATIVE BRIDGE MOCK]: ${command} responded with`,
          stateRequestMock,
        )
        return Promise.resolve(stateRequestMock)
      case COMMAND_GET_PAGE_VIEW_ID:
        console.log(
          `[NATIVE BRIDGE MOCK]: ${command} responded with`,
          pageViewIDMock,
        )
        return Promise.resolve(pageViewIDMock)
      case COMMAND_GET_USER_DETAILS:
        console.log(
          `[NATIVE BRIDGE MOCK]: ${command} responded with`,
          userDetailsRequestMock(w),
        )
        return Promise.resolve(userDetailsRequestMock(w))
      case COMMAND_AUTHENTICATE_USER:
        console.log(
          `[NATIVE BRIDGE MOCK]: ${command} responded with`,
          authenticateUserRequestMock(w),
        )
        return Promise.resolve(authenticateUserRequestMock(w))
      default:
        console.log(
          `[NATIVE BRIDGE MOCK]: ${command} responded with`,
          commandSuccessMock,
        )
        return Promise.resolve(commandSuccessMock)
    }
  }

  return { NativeBridge, callNativeBridgeCommand }
}

/**
 *
 */
export const applyContract = (
  bridgeCommands: BridgeCommand[],
  w = window,
) => {
  const { NativeBridge, callNativeBridgeCommand } =
    initializeMockBridge(w)

  const contract = defineContract(callNativeBridgeCommand)

  bridgeCommands.forEach((command) => {
    NativeBridge[command] = contract[command]
  })

  return NativeBridge
}

/**
 * Assembles a Mock NativeBridge object which implements the contract but
 * with no-op functions that simulate a hybrid call
 */
export default function (w = window) {
  const userMock = parseUserMock(w)
  const NativeBridge = applyContract(bridgeCommands, w)

  // Mock the Hybrid Config
  w.hybridConfig = {
    OS: 'Mock OS',
    AbraConfig: {},
    userInfo: {
      ...userMock,
      agentID: 'mock-agent-id',
    },
  }

  // Provide a mock for the attach/removal of event listeners and mocking event dispatch on the bridge
  let eventTarget: EventTarget | null = null

  NativeBridge.addEventListener = (
    evt: string,
    listener: EventListenerOrEventListenerObject,
    opts: any,
  ) => {
    if (!eventTarget) {
      eventTarget = new EventTarget()
    }

    eventTarget.addEventListener(evt, listener, opts)
  }

  NativeBridge.removeEventListener = (
    evt: string,
    listener: EventListenerOrEventListenerObject,
    opts: any,
  ) => {
    if (eventTarget) {
      eventTarget.removeEventListener(evt, listener, opts)
      eventTarget = null
    }
  }

  NativeBridge.dispatchEvent = (evt: string) => {
    if (eventTarget) {
      const event = new Event(evt)
      // @ts-ignore we're mocking a custom response method used with NativeBridge events
      event.respondWith = (response: any) => {
        console.log(
          `[NATIVE BRIDGE MOCK]: ${evt} responded with`,
          response,
        )
      }
      eventTarget.dispatchEvent(event)
    }
  }

  if (isAndroidHybridWebview()) {
    android(NativeBridge)
  } else if (isiOSHybridWebview()) {
    ios(NativeBridge)
  } else {
    console.log(
      '[NATIVE BRIDGE MOCK]: Platform is neither android nor ios',
      w.newsreaderAppPlatform,
      w.gamesAppPlatform,
    )
  }

  // Attach the mock native bridge to the window object
  w.NativeBridge = NativeBridge
}
