// @ts-ignore
import { trackHubItem } from 'hub/track/trackClick'

import { getPageName, jsHook } from './util'
import {
  mountNodeList,
  createNode,
  addClass,
  removeClass,
  setAttribute,
  getElementById,
  addEventListener,
  appendChild,
  HTMLElementWithEvents,
} from './dom'
import { track, trackClick } from './tracking'
import { UserDetailsResponse } from './hybrid/commands'

// all login, logout, subscribe buttons are initially visible in the navigation
// for hybrid so that we can hide them based on entitlements from the bridge
const addHybridEntitlementsHandler = () => {
  if (window.isHybridWebView && window.NativeBridge) {
    const loginButtons = document.querySelectorAll(
      `.${jsHook('nav-login')}`,
    )

    const logoutButtons = document.querySelectorAll(
      `.${jsHook('nav-logout')}`,
    )

    const subscribeButtons = document.querySelectorAll(
      `.${jsHook('nav-subscribe')}`,
    )

    const accountDetails = document.querySelectorAll(
      `.${jsHook('nav-account-details')}`,
    )

    const hideAllElements = (elements: NodeListOf<Element>) =>
      elements.forEach((el) => {
        addClass(el, 'hybrid-hidden')
      })

    const hideElementsBasedOnEntitlements = (userData: {
      isUserLoggedIn: boolean
      isSubscribed: boolean
    }) => {
      const isLoggedIn = userData?.isUserLoggedIn
      const hasXwd = userData?.isSubscribed

      if (isLoggedIn) {
        hideAllElements(loginButtons)
      } else {
        hideAllElements(logoutButtons)
        hideAllElements(accountDetails)
      }

      if (hasXwd) {
        hideAllElements(subscribeButtons)
      }
    }

    window.NativeBridge.gamesGetUserDetails()
      .then((data: UserDetailsResponse) => {
        if (!data.success) throw new Error(data.error)
        hideElementsBasedOnEntitlements(
          data.values.gamesGetUserDetails,
        )
      })
      .catch((e: Error) => {
        console.error('Getting user details failed', e)
        hideAllElements(logoutButtons)
        hideAllElements(accountDetails)
      })

    addEventListener(
      window,
      'gamesUserCredentialChanged',
      (event) => {
        // handle show/hide buttons in the nav
        hideElementsBasedOnEntitlements(
          // @ts-ignore
          event?.detail?.values?.gamesAuthenticateUser,
        )
      },
    )

    const handleButtonsOnClick = (
      buttons: NodeListOf<Element>,
      authenticationType: 'subscribe' | 'login',
    ) => {
      buttons.forEach((button) => {
        addEventListener(button, 'click', (event) => {
          event.preventDefault()
          window.NativeBridge?.gamesAuthenticateUser(
            authenticationType,
          ).then((data: any) => {
            if (!data.success) throw new Error(data.error)

            // dispatch the event to the game
            window.dispatchEvent(
              new CustomEvent('gamesUserCredentialChanged', {
                detail: data,
              }),
            )
          })
        })
      })
    }

    handleButtonsOnClick(subscribeButtons, 'subscribe')
    handleButtonsOnClick(loginButtons, 'login')
  }
}

const addHybridBackButtonHandler = () => {
  const backButton = getElementById(jsHook('hybrid-back'))

  if (backButton && window.isHybridWebView && window.NativeBridge) {
    addEventListener(backButton, 'click', () => {
      if (getPageName() !== 'sudoku') {
        // see sudoku/Layout for replaced functionality
        window.NativeBridge?.gamesBackToHub()
      }
    })
  }
}

// Add tracking to top sale banner
const trackSaleBanner = () => {
  const navElement = [
    ...document.querySelectorAll('.top-sale-banner'),
  ] as HTMLElement[]

  const urgencyTextElem = document.querySelector(
    '.top-sale-banner__urgency',
  )?.innerHTML

  const urgencyText = urgencyTextElem ?? ''

  const trackingLabel = `${urgencyText}50% off your first year of Games.`

  const region = getPageName()

  if (navElement.length > 0) {
    track('impression', {
      module: {
        name: 'games-sale-banner',
        region,
        label: trackingLabel,
      },
      eventData: {
        page_type: 'games-sale-banner',
      },
    })
  }

  navElement.forEach((elem) => {
    elem.addEventListener(
      'click',
      () => {
        track('moduleInteraction', {
          module: {
            region: '',
            label: trackingLabel,
            element: {
              name: region,
              label: 'games-sale-banner',
            },
          },
          eventData: {
            pageType: 'games-sale-banner',
            type: 'tap',
          },
        })
      },
      true,
    )
  })
}

const addSaleNavTrackers = () => {
  const navElements = [
    ...document.querySelectorAll('.js-sale-nav-tracker'),
  ] as HTMLElement[]

  const name = getPageName()

  navElements.forEach((elem) => {
    const label = elem?.dataset?.trackLabel || ''

    if (!label) {
      return
    }

    elem.addEventListener('click', () => {
      track('moduleInteraction', {
        module: {
          label,
          element: {
            name,
            label: elem.innerText,
            url: 'https://www.nytimes.com/subscription/games?campaignId=4QHQ8',
          },
        },
      })
    })
  })
}

const addNavTrackers = () => {
  const navElements = [
    ...document.querySelectorAll('.js-nav-tracker'),
  ] as HTMLElement[]
  const navProfile = getElementById(jsHook('nav-profile'))
  const navProfileElements: HTMLElement[] = navProfile
    ? [...navProfile.querySelectorAll('a')]
    : []

  const allElements = [...navElements, ...navProfileElements]

  const name = getPageName()

  allElements.forEach((elem) => {
    const label = elem?.dataset?.trackLabel || ''

    if (!label) {
      return
    }

    elem.addEventListener('click', () => {
      trackClick({
        name,
        label,
        useBeacon: true,
        context: null,
      })
    })
  })
}

const addHubTrackers = () => {
  const hubElements = [
    ...document.querySelectorAll('.js-hub-tracker'),
  ] as HTMLElement[]

  hubElements.forEach((elem) => {
    const labelToTrack = elem.dataset.trackHub
    const contextToTrack = elem.dataset.trackHubContext || null

    elem.addEventListener(
      'click',
      () => {
        trackHubItem(labelToTrack, contextToTrack)
      },
      true,
    )
  })
}

export const initializeNav = () => {
  addNavTrackers()
  addSaleNavTrackers()
  addHybridBackButtonHandler()
  addHybridEntitlementsHandler()
  trackSaleBanner()

  /**
   * Note: Some elements - like the Download App button - don't render in time with other elements.
   * This setTimeout is here to make the addHubTrackers method run after those elements are rendered,
   * so that all eventlisteners are attached.
   */
  setTimeout(() => {
    addHubTrackers()
  }, 0)

  // Let's store some state here
  let isNavOpen = false
  let currentFocusIndex = -1

  // Gather all the things up
  const burger = getElementById(jsHook('nav-burger'))
  const drawer = getElementById(jsHook('nav-drawer'))

  // In a hybrid Games webview, there is no hamburger menu, so we can simply return
  if (!burger || !drawer) return

  const navLinks = [
    ...drawer.querySelectorAll('.pz-nav-drawer__link'),
    ...drawer.querySelectorAll('.pz-nav__button'),
  ] as HTMLElement[]

  const navLinkFirstChars = navLinks.map((link) => {
    const text = link.textContent
    return text && text[0] ? text[0].toUpperCase() : ''
  })

  /**
   * Open the navigation by updating state and toggling the UI in
   * src/server/views/components/Nav.js
   */
  function openNav() {
    isNavOpen = true

    if (!burger || !drawer) return

    addClass(drawer, 'open')
    addClass(burger, 'active')

    trackClick({
      name: getPageName(),
      label: 'click-nav',
      context: null,
    })

    setAttribute(burger, 'aria-expanded', isNavOpen.toString())
    setAttribute(drawer, 'aria-hidden', (!isNavOpen).toString())
  }

  /**
   * Close the navigation by updating state and toggling the UI in
   * src/server/views/components/Nav.js
   */
  function closeNav() {
    isNavOpen = false

    if (!burger || !drawer) return

    removeClass(drawer, 'open')
    addClass(drawer, 'closing')
    // Delaying removing this class so the closing animation is visible
    setTimeout(() => {
      removeClass(drawer, 'closing')
    }, 500)
    removeClass(burger, 'active')

    setAttribute(burger, 'aria-expanded', isNavOpen.toString())
    setAttribute(drawer, 'aria-hidden', (!isNavOpen).toString())
  }

  addEventListener(burger, 'click', () => {
    if (!isNavOpen) {
      openNav()
    } else {
      closeNav()
    }

    burger.blur() // Remove focus from button after it's been clicked
  })

  addEventListener(burger, 'keydown', (event) => {
    const { key, keyCode } = event as KeyboardEvent

    if (key === 'Escape' || key === 'Esc') {
      closeNav()
      return
    }

    if (key === 'Enter' || key === 'Space' || keyCode === 32) {
      if (!isNavOpen) {
        openNav()
        currentFocusIndex = 0
        navLinks[currentFocusIndex].focus()
      } else {
        closeNav()
      }

      return
    }

    if (key === 'ArrowDown') {
      event.preventDefault()

      openNav()
      currentFocusIndex = 0
      navLinks[currentFocusIndex].focus()

      return
    }

    if (key === 'ArrowUp') {
      event.preventDefault()

      openNav()
      currentFocusIndex = navLinks.length - 1
      navLinks[currentFocusIndex].focus()

      // eslint-disable-next-line no-useless-return
      return
    }
  })

  addEventListener(drawer, 'keydown', (event) => {
    const { key, keyCode, shiftKey } = event as KeyboardEvent

    if (key === 'Escape' || key === 'Esc') {
      closeNav()
      burger.focus()
      return
    }

    if (
      key === 'ArrowDown' ||
      key === 'ArrowRight' ||
      (key === 'Tab' && !shiftKey)
    ) {
      event.preventDefault()

      if (currentFocusIndex === navLinks.length - 1) {
        currentFocusIndex = 0
      } else {
        currentFocusIndex += 1
      }

      navLinks[currentFocusIndex].focus()

      return
    }

    if (
      key === 'ArrowUp' ||
      key === 'ArrowLeft' ||
      (key === 'Tab' && shiftKey)
    ) {
      event.preventDefault()

      if (currentFocusIndex === 0) {
        currentFocusIndex = navLinks.length - 1
      } else {
        currentFocusIndex -= 1
      }

      navLinks[currentFocusIndex].focus()

      return
    }

    // Handle an alpha keypress and go to the first link that starts
    // with that character that _isn't_ currently focused
    if (keyCode >= 65 && keyCode <= 90) {
      const foundIndex = navLinkFirstChars.findIndex((char, i) => {
        return (
          char === String.fromCharCode(keyCode) &&
          i !== currentFocusIndex
        )
      })

      if (foundIndex >= 0) {
        currentFocusIndex = foundIndex
        navLinks[currentFocusIndex].focus()
      }

      // eslint-disable-next-line no-useless-return
      return
    }
  })

  // EVENTS - CLICK - NAV - Logo
  const logo = getElementById(jsHook('logo-nav')) as HTMLElement

  logo.addEventListener('click', () => {
    const eventData = {
      eventData: {
        pagetype: 'game',
        trigger: 'module',
        type: 'click',
      },
      module: {
        name: 'click',
        context: '',
        element: {
          name: window.location.href,
          label: 'games-logo',
        },
      },
    }

    track('moduleInteraction', eventData)
  })

  // Add tracking for login & subscribe on the nav bar/header/
  const navButtons = document.querySelector(
    '#js-nav-actions',
  ) as HTMLElement

  const navSubscribeButton = navButtons?.querySelectorAll(
    'a.js-nav-subscribe',
  )[0] as HTMLElement

  const navLoginButton = navButtons?.querySelectorAll(
    'a.js-nav-login',
  )[0] as HTMLElement

  navSubscribeButton?.addEventListener('click', () => {
    const gameName = getPageName()

    track('moduleInteraction', {
      eventData: {
        pagetype: 'game',
        trigger: 'module',
        type: 'ob_click',
      },
      module: {
        element: { name: gameName, label: 'subscribe-header' },
      },
    })
  })

  navLoginButton?.addEventListener('click', () => {
    const gameName = getPageName()

    track('moduleInteraction', {
      eventData: {
        pagetype: 'game',
        trigger: 'module',
        type: 'ob_click',
      },
      module: {
        element: { name: gameName, label: 'log-in-header' },
      },
    })
  })
}

/*
  MOBILE TOOLBAR
  This is for displaying a game's toolbar
  alongside the top level navigation
*/
export type MobileNavItem = {
  type: 'text'
  value: string
  action: () => any | null
  icon?: string
  id?: string
}

type DropDownMenu = {
  value: string
  items: DropDownNavItem[]
}

type DropDownLink = {
  type: 'navigation'
  value: string
  url: string
  action?: () => void
  icon?: string
}

type DropDownButton = {
  type: 'button'
  value: string
  action: () => void
  icon?: string
}

export type DropDownNavItem = DropDownLink | DropDownButton

export const globalNavId = jsHook('global-nav')
export const mobileToolbarId = jsHook('mobile-toolbar')
export const showToolbarClass = 'show-mobile-toolbar'
const header = document.querySelector('.pz-header')

export const mobileNavTools = (() => {
  const globalNav = getElementById(globalNavId) as HTMLElement
  const mobileToolbar = getElementById(mobileToolbarId) as HTMLElement
  const toolbarItemClass = 'pz-mobile-nav__toolbar-item'

  const createMobileNavItem = (
    navItem: MobileNavItem | DropDownNavItem,
  ): HTMLElement => {
    let nodeTag: string
    const options: any = {
      class: toolbarItemClass,
    }

    if (navItem.type === 'text' && navItem.id) {
      options.id = navItem.id
    }

    switch (navItem.type) {
      case 'button':
        nodeTag = 'button'
        options.onClick = navItem.action
        break
      case 'navigation':
        nodeTag = 'a'
        options.href = navItem.url
        options.target = window.isHybridWebView ? '_self' : '_blank'
        options.rel = 'noreferrer'
        if (navItem.action) options.onClick = navItem.action
        break
      default:
        nodeTag = 'span'
        options.onClick = navItem.action
    }

    const navItemNode = createNode(nodeTag, options, navItem.value)

    if (navItem.icon) {
      const icon = createNode('i', {
        class: `pz-toolbar-icon ${navItem.icon}`,
      })
      appendChild(navItemNode, icon)
    }

    return navItemNode
  }

  const createMobileNavItems = (
    navItems: Array<MobileNavItem | DropDownNavItem | HTMLElement>,
  ): HTMLElement[] => {
    return navItems.map((navItem) => {
      if (navItem instanceof HTMLElement) return navItem

      return createMobileNavItem(navItem)
    })
  }

  let isActive = false

  return {
    create(
      navItems: (
        | MobileNavItem
        | HTMLElementWithEvents
        | DropDownNavItem
      )[],
    ) {
      mountNodeList(mobileToolbar, createMobileNavItems(navItems))
    },
    activate() {
      if (isActive) return

      isActive = true
      addClass(globalNav, showToolbarClass)
    },
    deactivate() {
      if (!isActive) return
      isActive = false
      removeClass(globalNav, showToolbarClass)
    },
  }
})()
