import type { Integration, Event, Exception } from '@sentry/types'

/**
 * A Sentry custom integration that applies further noise reducing logic. Sometimes other teams need to
 * insert scripts into our page. We don't want to see these errors in Sentry, and it's easy to filter
 * them out by script name or path
 */
export class NoiseFilter implements Integration {
  static id = 'NoiseFilter'

  name = 'NoiseFilter'

  /**
   * Define a regex or string (case sensitive) that will be ran against the error's entire stack trace.
   * You can filter out errors from specific file paths or function names.
   */
  filter = [
    /**
     * Filter out all errors from script with the path /ads. This is where some third-party ad code
     * like prebid get added. Ad blockers will cause them to error and blow up Sentry
     */
    /ads\/.*/,
    // Amazon ad scripts
    /aax2\/apstag/,
  ]

  setupOnce = () => {}

  runThroughStackTraces(ex: Exception) {
    let matchedFilter = false

    ex?.stacktrace?.frames?.forEach((frame) => {
      this.filter.forEach((regxOrString) => {
        if (
          frame.filename?.match(regxOrString) ||
          frame.function?.match(regxOrString)
        ) {
          matchedFilter = true
        }
      })
    })

    return matchedFilter
  }

  /**
   * For each event that is processed, see if any pattern in the filter array gets a match in the
   * event's stack trace. If there's a match, don't send the event
   */
  processEvent(
    event: Event,
  ): Event | null | PromiseLike<Event | null> {
    let shouldFilter = false

    event?.exception?.values?.forEach((ex) => {
      shouldFilter = this.runThroughStackTraces(ex)
    })

    return shouldFilter ? null : event
  }
}
