import * as Sentry from '@sentry/react'
import type { Integration, Event } from '@sentry/types'

/**
 * A Sentry custom integration that applies a different sample rate to certain games.
 * Some games are noisier than others, so this helps us keep spikes to a minimum without
 * turning the volume down on less noisier games.
 */
export class RouteSampler implements Integration {
  static id = 'RouteSampler'

  name = 'RouteSampler'

  sampleRate = 0

  environment = ''

  constructor({
    sampleRate,
    environment,
  }: {
    sampleRate: number
    environment: string
  }) {
    this.sampleRate = sampleRate
    this.environment = environment
  }

  setupOnce = () => {}

  /**
   * Try to get the pathname from the window, transaction, or the event request
   */
  getPathName = (event: Event) => {
    let url = new URL(window.location.href)

    if (url.pathname) {
      return url.pathname
    }

    if (event.transaction) {
      return event.transaction
    }

    if (event.request?.url) {
      url = new URL(event.request?.url)

      if (url.pathname) {
        return url.pathname
      }
    }

    return ''
  }

  /**
   * Defines the reduced sample rate for each game. This sample rate is
   * applied on top of the uniform sample rate defined in the Sentry config.
   */
  getPathSampleRate = (pathname: string) => {
    if (!pathname) return 0.3

    if (pathname.includes('/wordle')) {
      return 0.1
    }

    if (pathname.includes('/spelling-bee')) {
      return 0.25
    }

    if (pathname.includes('/connections')) {
      return 0
    }

    if (pathname.includes('/crosswords/game/daily')) {
      return 0.25
    }

    return 0.3
  }

  /**
   * Given a pathname, returns a route tag for Sentry issues
   */
  getRouteTag = (pathname: string) => {
    switch (true) {
      case /games\/wordle/.test(pathname):
        return 'wordle'
      case /puzzles\/spelling-bee/.test(pathname):
        return 'spelling-bee'
      case /games\/connections/.test(pathname):
        return 'connections'
      case /puzzles\/vertex/.test(pathname):
        return 'vertex'
      case /puzzles\/tiles/.test(pathname):
        return 'tiles'
      case /puzzles\/sudoku/.test(pathname):
        return 'sudoku'
      case /puzzles\/letter-boxed/.test(pathname):
        return 'letter-boxed'
      case /leaderboards/.test(pathname):
        return 'leaderboards'
      case /crosswords\/game\/daily/.test(pathname):
        return 'crossword'
      case /crosswords\/game\/mini/.test(pathname):
        return 'mini'
      case /crosswords\/archive/.test(pathname):
        return 'archive'
      case /crosswords/.test(pathname):
        return 'hub'
      default:
        return 'other'
    }
  }

  /**
   * For each event that is processed, apply the new sample rate for the route.
   */
  processEvent(
    event: Event,
  ): Event | null | PromiseLike<Event | null> {
    // Only sample events on prod where the traffic is high
    if (this.environment !== 'prod') return event

    const pathname = this.getPathName(event)

    // Annotate the event with a route tag
    Sentry.setTag('route', this.getRouteTag(pathname))

    const samplePercent = this.getPathSampleRate(pathname) * 100
    const randomNumber = Math.floor(Math.random() * 100) + 1

    return randomNumber <= samplePercent ? event : null
  }
}
