import * as Sentry from '@sentry/react'

import type { UserType } from 'foundation'

import { agentIdCookie } from '../util'

interface SentryTag {
  [key: string]: string | number | null
}

type SentryData = {
  [key: string]: string | number | null
}

/**
 * Use a user's agentid cookie or a random base64 string to identify them this session
 * A Sentry user can't be blank because we have alerting that counts unique user misery.
 */
const generateSentryUserIdentifier = () =>
  agentIdCookie || window.btoa(Math.random().toString().substring(2))

/**
 * Manually reports a JavaScript error to Sentry. Optionally provide an object
 * containing tags to allow you to query this issue (e.g. { api: moogle, hybrid: true })
 */
export const captureSentryError = (
  e: Error,
  tags: SentryTag = {},
) => {
  // Only do work if e is really an Error
  if (e && e.message) {
    Sentry.withScope((scope) => {
      Object.keys(tags).forEach((key) => {
        scope.setTag(key, tags[key])
      })
      Sentry.captureException(e)
    })
  }
}

/**
 * Log key-value pairs in Sentry. Helpful to do before something happens.
 */
export const setSentryData = (name: string, data: SentryData) => {
  if (name && data) {
    Sentry.setContext(name, data)
  }
}

/**
 * Tags any issue the user will face with their appropriate user type.
 * (e.g. 'anon', 'regi', 'sub', 'anon-sub', 'free-trial')
 */
export const setSentryUser = (user: Partial<UserType>) => {
  let type = 'anon'

  if (user.isLoggedIn) {
    type = 'regi'
  }

  if (user.hasXwd) {
    type = 'sub'

    if (!user.isLoggedIn) {
      type = 'anon-sub'
    }
  } else if (user.isFreeTrial) {
    type = 'free-trial'
  }

  Sentry.setTag('userType', type)

  if (user.regiId) {
    Sentry.setTag('regiId', user.regiId)
  }

  Sentry.setUser({ id: generateSentryUserIdentifier() })
}
