import {
  ReportCallback,
  onCLS,
  onFID,
  onINP,
  onLCP,
} from 'web-vitals'

import { doc, win } from './aliases'
import { querySelector } from './dom'
import {
  createQueryString,
  deltaCookie,
  env,
  makeAsyncValue,
  now,
} from './util'
import pageViewId from './utils/pageViewId'
import { xhr } from './xhr'

const DATA_LAYER = 'dataLayer'
const GTM_APP = 'games-crosswords'

win[DATA_LAYER] = win[DATA_LAYER] || []

if (!window.isHybridWebView) {
  win[DATA_LAYER].push({
    event: 'gtm.js',
    'gtm.start': now(),
  })
}

const canonicalLink = querySelector(
  'link[rel=canonical]',
) as HTMLLinkElement

const assetUrl = canonicalLink
  ? canonicalLink.href
  : doc.location.href

/**
 * Looks for a cookie set by Fastly indicating that the user is visiting from a
 * b2b partnership. If found, fire a page_update event.
 */
export const trackPartnership = (event: string) => {
  if (!deltaCookie) return null

  switch (event) {
    case 'impression':
    case 'moduleInteraction':
      window.nyt_et?.({
        subject: 'page_update',
        fastly_headers: {
          b2b: 'delta',
        },
      })
      break
    default:
  }
}

export const track = (event: string, data: any) => {
  if (typeof window === undefined) {
    return
  }

  if (window.isHybridWebView && window.NativeBridge) {
    // For some reason we normalize moduleInteraction to interaction for hybrid only
    const eventName =
      event === 'moduleInteraction' ? 'interaction' : event

    /**
     * Do not send heartbeat events in hybrid.
     * The sendAnalytic command will return an unhandled rejected promise with
     * the value 'isNil' which typically means the event type is not supported.
     * Both Games Hybrid Webviews and Android appear to support this, but behind the scenes
     * it is failing silently. On Newsreader it fails loudly, raising our Sentry alerts,
     * which is why we are adding this line of code below
     */
    if (eventName === 'heartbeat') return

    window.NativeBridge.sendAnalytic(eventName, data)
  } else {
    win[DATA_LAYER].push({ event, ...data })
    trackPartnership(event)
  }
}

// @ts-ignore TEMP SOLUTION, see XWD-18120
window.foundationTrack = track

export const trackPageUpdate = () => {
  const event = {
    native_app: {
      asset: {
        section: 'spelling-bee',
        view: 'web view - hybrid',
      },
    },
  }
  track('page_update', event)
}

export const trackModuleInteraction = (
  category: string,
  context: number | string | undefined,
  action: string,
  elementLabel: string,
  moduleLabel?: string,
  useBeacon?: boolean,
) => {
  const event = {
    module: {
      name: category,
      context,
      element: {
        name: action || '',
        label:
          typeof elementLabel === 'string'
            ? elementLabel
            : JSON.stringify(elementLabel),
      },
      ...(moduleLabel && { label: moduleLabel }),
    },
    eventData: {
      pageType: 'game',
      type: useBeacon ? 'ob_click' : 'click',
      trigger: 'module',
      // important to leave an empty string for `status`
      // for the event to pass correctly to google analytics
      // consult with someone from tracking team before removing
      // and confirm that the events are still availably in big query
      status: '',
    },
  }
  track('moduleInteraction', event)
}

export const trackImpression = (
  name: string,
  region: string,
  label?: string[] | string | null,
  context?: string,
) => {
  track('impression', {
    module: {
      name,
      context,
      region: region || '',
      label: label || undefined,
    },
  })
}

const tagxUser = makeAsyncValue(() => {
  if (window.isHybridWebView) {
    return null
  }

  const query = createQueryString({
    sourceApp: GTM_APP,
    referrer: doc.referrer,
    assetUrl,
    caller_id: GTM_APP,
  })
  const tagxUrl = `${env.tagx}/svc/nyt/data-layer${query}`
  return xhr.get(tagxUrl)
})

const createHeartbeat = () => {
  const HEARTBEAT_INTERVAL = 5000
  const start = now()

  const heartBeat = () => {
    if (doc.hasFocus()) {
      track('heartbeat', {
        pageview: {
          heartbeat: {
            timeSincePageDataReady: now() - start,
            heartbeatInterval: HEARTBEAT_INTERVAL,
          },
        },
      })
    }
  }

  setInterval(heartBeat, HEARTBEAT_INTERVAL)
}

export const setupTracking = () => {
  tagxUser.initialize()

  const initialTracking = (user?: unknown) => {
    if (user) track('userDataReady', user)

    track('pageDataReady', {
      application: {
        name: GTM_APP,
        environment: env.name,
      },
      asset: {
        url: assetUrl,
      },
      pageview: { id: pageViewId.current },
    })

    createHeartbeat()
  }

  tagxUser
    .get()
    .then(initialTracking)
    .catch(() => initialTracking())
}

type TrackClickArg = {
  name?: string
  label?: string | null
  context: any
  element?: { name: string; label: string } | null
  useBeacon?: boolean
  medium?: string | null
  source?: string | null
  region?: string | null
  moduleName?: string[] | null
  moduleLabel?: string[] | string | null
}

// DIG uses module.element.name and module.element.label
// for all interactions. Module.name/label are reserved
// for extra information, such as what word was guessed.
export const trackClick = ({
  name,
  label,
  context,
  element = null,
  useBeacon = false,
  medium = null,
  source = null,
  region = null,
  moduleName = null,
  moduleLabel = null,
}: TrackClickArg) => {
  if (typeof window === 'undefined') {
    return
  }

  track('moduleInteraction', {
    eventData: {
      pagetype: 'game',
      trigger: 'module',
      type: useBeacon ? 'ob_click' : 'click',
    },
    module: {
      type: 'click',
      element: element || { name, label },
      context,
      label: moduleLabel || label,
      medium,
      source,
      region,
      name: moduleName,
    },
  })
}

type TrackInteractionArg = {
  context: any
  element?: { name: string; label: string } | null
  name?: string | object | null
  label?: string | number | null
  region?: string | null
  url?: string | null
  type?: string
  useBeacon?: boolean
}

export const trackInteraction = ({
  context,
  element,
  name,
  label,
  region,
  url,
  type = 'click',
  useBeacon,
}: TrackInteractionArg) => {
  if (typeof window === 'undefined') {
    return
  }

  track('moduleInteraction', {
    eventData: {
      pagetype: 'game',
      trigger: 'module',
      type: useBeacon ? 'ob_click' : 'click',
    },
    module: {
      type,
      element,
      context,
      name,
      label,
      region,
      url,
    },
  })
}

// https://github.com/GoogleChrome/web-vitals
const webVitalsToDataLayer: ReportCallback = ({
  name,
  delta,
  id,
}) => {
  const event = 'performance'
  const data = {
    eventAction: 'Web Vitals',
    eventLabel: id,
    pageview: {
      performance: {
        [name.toLowerCase()]: Math.round(
          name === 'CLS' ? delta * 1000 : delta,
        ),
      },
    },
  }

  track(event, data)
}

export const reportWebVitals = () => {
  onCLS(webVitalsToDataLayer)
  onFID(webVitalsToDataLayer)
  onLCP(webVitalsToDataLayer)
  onINP(webVitalsToDataLayer)
}
