import { doc, win, encodeComponent, localStore } from './aliases'
/**
 * expose the page name
 */
export const getPageName = () => {
  const path = window?.location?.pathname || ''

  // All /crosswords/game/(:stream) routes are game-page by default
  // so we'll manually swap this out for the mini-page
  if (path.startsWith('/crosswords/game/mini')) {
    win.pageName = 'mini-page'
  }

  return win.pageName || ''
}

/**
 * expose the runtime environment
 */
export const { env } = win

/**
 * expose the feature flags
 */
export const { featureFlags = {} } = win

/**
 * use the version to help
 * decide when we should reinitialize local async values
 */
const { version } = env
const versionKey = 'pz-version'
const storedVersion = localStore.getItem(versionKey)
const versionHasChanged = version !== storedVersion

try {
  localStore.setItem(versionKey, version)
} catch (e) {
  console.error(e.error)
}

// NOTE: must stay in sync with views/util.js:jsHook
/**
 * jshook: formalizes how we identify dom elements that are coupled to js
 */
export const jsHook = (ID_OR_CLASSNAME: string) =>
  `js-${ID_OR_CLASSNAME}`

// NOTE: must stay in sync with views/util.js:URLS_WITHOUT_PARAM_SUPPORT
const URLS_WITHOUT_PARAM_SUPPORT = [
  'nytimes://login',
  'nytimes://createAccount',
]

// NOTE: must stay in sync with views/util.js:appendSupportedParams
/**
 * Add query params to links that have query param support.
 * All web links have query param support, but
 * in the case of ios deeplinks, some urls do not support
 * unhandled params, so we need to use the original navigation
 * deeplinks as-is to prevent errors.
 *
 * @param link navigation link
 * @param params params to add, if supported
 */
export const appendSupportedParams = (
  link: string,
  params: string,
) => {
  const useOriginalLink = URLS_WITHOUT_PARAM_SUPPORT.some((url) =>
    link.startsWith(url),
  )

  return useOriginalLink ? link : `${link}${params}`
}

/**
 * reused util to get current unix time
 */
export const now = () => Date.now()

/**
 * createQueryString: convert an object to a query string
 * NOTE: this doesnt handle arrays yet
 */
export const createQueryString = function (
  obj: Record<string, string>,
) {
  const outStr: string[] = []

  Object.keys(obj).forEach((p) => {
    outStr.push(`${encodeComponent(p)}=${encodeComponent(obj[p])}`)
  })

  return `?${outStr.join('&')}`
}

/**
 * insertScriptTag: append an inline or remote script to the body
 */
export const insertScriptTag = (
  src: string,
  content = '',
  isAsync = true,
) => {
  return new Promise((resolve, reject) => {
    const script = doc.createElement('script')
    script.async = isAsync
    if (src) {
      script.src = src
      script.onload = resolve
      script.onerror = reject
    } else if (content) {
      script.innerHTML = content
      resolve(undefined)
    }
    doc.body.appendChild(script)
  })
}

/**
 * makeAsyncValue: creates an object with an initialize and get method.
 * it is useful for async replacement of placeholder values.
 */
export const makeAsyncValue = <T>(
  initializer: () => void,
  storage?: Storage,
  storageKey?: string,
  check?: string,
) => {
  const pending: Array<[Function, Function]> = []
  const getItem = 'getItem'
  const setItem = 'setItem'
  const removeItem = 'removeItem'
  const checkKey = `${storageKey}-check`

  let value: T
  let ready: boolean
  let fromStorage: string | null

  /**
   * If the software version has changed
   * we shouldn't assume we can parse the stored value
   * so we delete it
   */
  if (storage && storageKey && versionHasChanged) {
    storage[removeItem](checkKey)
    storage[removeItem](storageKey)
  }

  /**
   * `drain` resolves pending tasks if there is no error
   * and resolves them if there is
   */
  const drain = (err?: Error) => {
    while (pending.length) {
      pending.shift()?.[err ? 1 : 0](err || value)
    }
  }

  return {
    get(): Promise<T> {
      return ready
        ? Promise.resolve(value)
        : new Promise((resolve, reject) =>
            pending.push([resolve, reject]),
          )
    },
    initialize() {
      if (
        storage && // are we storing this locally?
        storageKey &&
        storage[getItem](checkKey) === check // is the stored value still good?
      ) {
        fromStorage = storage[getItem](storageKey)
      }

      return Promise.resolve(
        (fromStorage && JSON.parse(fromStorage)) || // prefer local
          initializer(), // otherwise, initialize
      )
        .then((v) => {
          // We need `ready` because if value is falsy
          // we wouldnt be able to test `value` truthiness as a proxy for `ready`
          ready = true
          value = v

          // If we are storing locally
          if (storage && check && storageKey && !fromStorage) {
            try {
              // we attempt to save the value and a validation key
              storage[setItem](checkKey, check)
              storage[setItem](storageKey, JSON.stringify(value))
            } catch (e) {
              console.error(e.error)
            }
          }

          drain() // then we drain all the pending tasks
        })
        .catch(drain)
    },
  }
}

export const agentIdCookie = (/nyt-a=([^;]+)/.exec(
  doc.cookie || '',
) || [])[1]

export const disableAdsCookie = (/NYT-NO-ADS=([^;]+)/.exec(
  doc.cookie || '',
) || [])[1]

export const deltaCookie = (/nyt-delta=([^;]+)/.exec(
  doc.cookie || '',
) || [])[1]

export const nytGeoCookie = (/nyt-geo=([^;]+)/.exec(
  doc.cookie || '',
) || [])[1]

/**
 * getQueryParams.js
 *
 * Returns a value of the query key
 * @param  {string}  key query name to look for
 * @param  {string}  path optional url to query
 */
export const getQueryParams = (key: string, path?: string) => {
  const url = path || win.location.href
  const name = key.replace(/[[\]]/g, '\\$&')
  const results = new RegExp(`[?&]${name}(=([^&#]*)|&|#|$)`).exec(url)

  if (!results) return null

  if (!results[2]) return ''

  return decodeURIComponent(results[2].replace(/\+/g, ''))
}

/**
 * Adapted from nytimes/news
 * https://github.com/nytimes/news/blob/main/projects/vi/src/shared/utils/getDeviceType.js#L36
 */
export const isRobot = () => {
  const ua = navigator?.userAgent?.toLowerCase()
  if (ua) {
    const re =
      /(ads|google|bing|msn|yandex|baidu|ro|career|)(bot|-inspectiontool|other)|(baidu|jike|symantec)spider|scanner|(web)crawler/i

    return re.test(ua)
  }

  return false
}
