const jsonType = 'json'

interface ErrorWithStatus extends Error {
  status?: number
}

const responseError = (res: { status: number }) => {
  if (res.status < 400) return

  const e: ErrorWithStatus = new Error('bad req')
  e.status = res.status

  return e
}
type Headers = Record<string, string>

type Options = {
  withCredentials?: boolean
  withStatus?: boolean
  responseType?: string
  cookie?: string | null
  data?: any
  isSync?: boolean
  headers?: Headers
}

const defaults: Options = {
  withCredentials: true,
  responseType: jsonType,
}

const request = <R>(
  method: string,
  url: string,
  proptions: Options = {},
): Promise<R> => {
  const options = { ...defaults, ...proptions }
  const { isSync = false } = options
  return new Promise((resolve, reject) => {
    // navigator.onLine isn't the most reliable in a hybrid webview--
    // it might think the user is offline when they are actually online.
    // Since we're only showing the webview once the user is online
    // let's skip this check so we have a chance of getting yesterday's progress.
    if (!window?.isHybridWebView && !navigator.onLine) {
      return reject(new Error('No internet'))
    }

    const newX = new XMLHttpRequest()
    // sync request needs false flag
    newX.open(method, url, !isSync)
    newX.withCredentials = !!options.withCredentials

    if (!isSync) {
      newX.timeout = 5000
    }

    newX.ontimeout = () => {
      reject(new Error(`${method} request to ${url} timed out`))
    }

    newX.setRequestHeader(
      'Content-type',
      options.data
        ? 'application/json'
        : 'application/x-www-form-urlencoded',
    )

    if (options.headers) {
      const { headers } = options
      Object.keys(headers).forEach((header) => {
        newX.setRequestHeader(header, headers[header])
      })
    }

    newX.onload = () => {
      try {
        const txt = newX.responseText
        if (txt && options.responseType === jsonType) {
          const res = JSON.parse(txt)
          if (options.withStatus) {
            res.status = newX.status
          }
          resolve(res)
        } else {
          // @ts-ignore
          resolve(txt)
        }
      } catch (e) {
        reject(new Error('Something went wrong'))
      }
    }

    // @ts-ignore
    newX.addEventListener('error', (e) => reject(responseError(e)))

    newX.send(options.data && JSON.stringify(options.data))
  })
}

export const xhr = {
  get: <R>(url: string, options?: Options) =>
    request<R>('GET', url, options),
  post: <R>(url: string, data: any, options?: Options) =>
    request<R>('POST', url, { ...options, data }),
  put: <R>(url: string, data: any, options?: Options) =>
    request<R>('PUT', url, { ...options, data }),
  delete: <R>(url: string, options?: Options) =>
    request<R>('DELETE', url, options),
  request,
}
