import React from 'react'
import pt from 'prop-types'
import { connect } from 'react-redux'

import AlternateCardPhone from 'hub/AlternateCardPhone'
import TopAd from 'hub/TopAd/TopAd'
import Welcome from 'hub/Welcome'
import Progress from 'hub/Progress/Progress'
import GuidePromo from 'hub/GuidePromo'
import PuzzleGroup from 'hub/PuzzleGroup/PuzzleGroup'
import GamesSection from 'hub/GamesSection/GamesSection'
import MobileStatsCard from 'hub/MobileStatsCard/MobileStatsCard'
import Modals from 'hub/Modal/Modals'
import {
  getPuzzlesProgress,
  getStatsAndStreaks,
  recallPrintPrefs,
} from 'hub/store/actionCreators'
import { getIsMoarGamesVariant } from 'hub/store/selectors'
import {
  ALLOW_HUB_VARIANTS,
  HUB_VARIANT_MOAR_GAMES,
  HUB_VARIANT_DEFAULT,
  SET_HUB_VARIANT,
} from 'hub/store/constants'
import AdMobile from 'views/components/AdMobile'

import SponsorshipBanner from '../../../server/views/components/Marketing/Sponsorship/SponsorshipBanner'

/**
 * Throttle function pulled straight outta underscore.js. Instead of loading a
 * new dependency, let's just put this in here and delete it as soon as we're
 * done with the Hub A/B testing.
 *
 * DELETE ME WHEN HUB MORE TEST IS COMPLETE
 *
 * @param {Function} func The function to throttle
 * @param {Number} wait The number of milliseconds to trigger a function call
 * @param {Object} options
 */
/* eslint-disable */
function throttle(func, wait, options) {
  var context, args, result
  var timeout = null
  var previous = 0
  if (!options) options = {}
  var later = function () {
    previous = options.leading === false ? 0 : Date.now()
    timeout = null
    result = func.apply(context, args)
    if (!timeout) context = args = null
  }
  return function () {
    var now = Date.now()
    if (!previous && options.leading === false) previous = now
    var remaining = wait - (now - previous)
    context = this
    args = arguments
    if (remaining <= 0 || remaining > wait) {
      if (timeout) {
        clearTimeout(timeout)
        timeout = null
      }
      previous = now
      result = func.apply(context, args)
      if (!timeout) context = args = null
    } else if (!timeout && options.trailing !== false) {
      timeout = setTimeout(later, remaining)
    }
    return result
  }
}
/* eslint-enable */

/**
 * Determine which Hub variant we will be rendering based on various
 * bits of business logic such as device type (from UA string) or the
 * device viewport width. Can be toggled on/off with ALLOW_HUB_VARIANTS.
 *
 * This function runs on both the client and server.
 *
 * When on the client, we will look at the viewport width.
 * When on the server, we instead have access to a deviceType object, which
 * is determined from the UA string.
 * @see src/server/middleware/injectDeviceType.js
 *
 * These implementations aren't quite at parity since the mobile-detect
 * package used in injectDeviceType.js uses the UA string. But, given the
 * resize event is such an edge case, we can un-complicate what would be
 * required code (updating the deviceType object on the client by async
 * fetching our /device endpoint) by bifurcating our business logic on
 * the client and server.
 *
 * @param {Object} deviceType Optional. Comes from the UA string on the server.
 * @returns {String} An enum describing the hub variant to be rendered
 */
export const getHubVariant = (deviceType) => {
  // On the client, we check the viewport width
  if (typeof window !== 'undefined') {
    const viewportCheck = document.body.clientWidth < 768

    // On the client, we can override the ALLOW_HUB_VARIANTS
    // flag w/ a query parameter
    if (
      viewportCheck &&
      window.location.search &&
      window.location.search.includes('moarGames=1')
    ) {
      return HUB_VARIANT_MOAR_GAMES
    }

    // Force HUB_VARIANT_DEFAULT when variants are disabled
    if (ALLOW_HUB_VARIANTS === false) {
      return HUB_VARIANT_DEFAULT
    }

    if (viewportCheck) {
      return HUB_VARIANT_MOAR_GAMES
    }
  }

  // We are on the server and we have a fresh deviceType from withServerData()
  if (deviceType !== undefined) {
    // Force HUB_VARIANT_DEFAULT when variants are disabled
    if (ALLOW_HUB_VARIANTS === false) {
      return HUB_VARIANT_DEFAULT
    }

    if (deviceType.isTablet) {
      return HUB_VARIANT_DEFAULT
    }

    if (deviceType.isMobile && !deviceType.newsreaderAppPlatform) {
      return HUB_VARIANT_MOAR_GAMES
    }
  }

  return HUB_VARIANT_DEFAULT
}

const Layout = ({
  hubVariant,
  api,
  user,
  fetchStatsAndStreaks,
  fetchAllPuzzleProgress,
  recallPrintPreferences,
  isMoarGamesVariant,
  updateVariant,
  device,
  compliance,
}) => {
  // When we have user id, fetch puzzle progress & stats
  React.useEffect(() => {
    if (!user.id) return

    fetchAllPuzzleProgress()
    fetchStatsAndStreaks()
    recallPrintPreferences()
  }, [user])

  // When a user resizes/rotates their device
  React.useEffect(() => {
    const handleResize = () => {
      const newHubVariant = getHubVariant()
      if (newHubVariant !== hubVariant) {
        updateVariant(newHubVariant)
      }
    }

    const throttledResize = throttle(handleResize, 500)

    window.addEventListener('resize', throttledResize)

    return () => {
      window.removeEventListener('resize', throttledResize)
    }
  }, [hubVariant])

  const xwdLayout = (
    <>
      <SponsorshipBanner />
      <Welcome showStatsAndStreaks />
      {device.isMobile ? (
        <AdMobile
          id="ad-mid1"
          position="mid1"
          deviceType={device}
          compliance={compliance}
        />
      ) : (
        <TopAd />
      )}
      {!isMoarGamesVariant && <GamesSection />}
      {isMoarGamesVariant && <AlternateCardPhone />}
      <Progress api={api} />
      <MobileStatsCard />
      <GuidePromo />
    </>
  )

  const anonOrRegiLayout = (
    <>
      {!device.isMobile ? <TopAd /> : null}
      <SponsorshipBanner />
      <Welcome />
      {device.isMobile ? (
        <AdMobile
          id="ad-mid1"
          position="mid1"
          deviceType={device}
          compliance={compliance}
        />
      ) : null}
      {!isMoarGamesVariant && <GamesSection />}
      <MobileStatsCard />
      <GuidePromo />
      <PuzzleGroup
        track="free-puzzles"
        title="Free From Our Archive"
        puzzleCollection="freePuzzles"
        moreLink="/crosswords/archive"
      />
    </>
  )

  const hdLayout = (
    <>
      <SponsorshipBanner />
      <Welcome />
      {device.isMobile ? (
        <AdMobile
          id="ad-mid1"
          position="mid1"
          deviceType={device}
          compliance={compliance}
        />
      ) : (
        <TopAd />
      )}

      <Progress />
      <GuidePromo />
      {!isMoarGamesVariant && <GamesSection />}
      <MobileStatsCard />
      {isMoarGamesVariant && <AlternateCardPhone />}
      <PuzzleGroup
        track="free-puzzles"
        title="Free From Our Archive"
        puzzleCollection="freePuzzles"
        moreLink="/crosswords/archive"
      />
    </>
  )

  const getLayout = ({ hasXwd, hasHd }) => {
    if (hasXwd) return xwdLayout

    if (hasHd) return hdLayout

    return anonOrRegiLayout
  }

  return (
    <>
      {getLayout(user)}
      <Modals />
      <AdMobile
        id="ad-bottom"
        adUnitPath="crosswords"
        position="bottom"
        deviceType={device}
        compliance={compliance}
      />
    </>
  )
}

Layout.propTypes = {
  api: pt.shape({
    fetchAllStats: pt.func,
    fetchPuzzleProgress: pt.func,
  }),
  fetchAllPuzzleProgress: pt.func.isRequired,
  fetchStatsAndStreaks: pt.func.isRequired,
  recallPrintPreferences: pt.func.isRequired,
  user: pt.shape({
    id: pt.number,
    hasHd: pt.bool,
    hasXwd: pt.bool,
    hasDigi: pt.bool,
    isLoggedIn: pt.bool,
    inShortzMode: pt.bool,
    entitlement: pt.string,
    isErsatzShortz: pt.bool,
  }).isRequired,
  isMoarGamesVariant: pt.bool,
  hubVariant: pt.string,
  updateVariant: pt.func,
  compliance: pt.shape({
    ads: pt.bool,
  }),
  device: pt.shape({
    isMobile: pt.bool,
  }),
}

const mapStateToProps = (state) => ({
  user: state.user,
  isMoarGamesVariant: getIsMoarGamesVariant(state),
  hubVariant: state.hubVariant,
  device: state.device,
  compliance: state.compliance,
})

const mapDispatchToProps = (dispatch, { api }) => ({
  updateVariant: (variant) =>
    dispatch({
      type: SET_HUB_VARIANT,
      payload: variant,
    }),
  fetchStatsAndStreaks: () =>
    dispatch(getStatsAndStreaks(api.fetchAllStats)),
  fetchAllPuzzleProgress: () =>
    dispatch(getPuzzlesProgress(api.fetchPuzzleProgress)),
  recallPrintPreferences: () => {
    dispatch(recallPrintPrefs())
  },
})

export default connect(mapStateToProps, mapDispatchToProps)(Layout)
