import React from 'react'
import pt from 'prop-types'
import { connect } from 'react-redux'
import cn from 'classnames'

import { STATS } from 'hub/track/constants'
import { formatAsDuration } from 'hub/utils/time'
import { calculateBarHeight, getBarHeight } from 'hub/utils/barHeight'

import NoSubsCard from './NoSubsCard'

export const MobileStatsCard = ({
  statsAndStreaks,
  user,
  subscribeLinks,
}) => {
  const { stats, streaks, miniStats } = statsAndStreaks

  const noStats = (
    <div className="hub-mobile-stats__no-stats">No Data</div>
  )

  if (!streaks || !user.hasXwd) {
    return (
      <div className="hub-mobile-stats__container">
        <h2>Statistics</h2>
        <NoSubsCard user={user} subscribeLinks={subscribeLinks} />
      </div>
    )
  }

  const {
    bestTime: bestMiniTime,
    todaysTime: todaysMiniTime,
  } = miniStats
  const today = streaks.dateEnd
  const dayOfWeek = new Date(today.replace(/-/g, '/')).getDay()
  const index = (dayOfWeek + 6) % 7
  const thisDaysStats = stats.statsByDay[index]
  const { thisWeeksTime, avgTime, bestTime } = thisDaysStats
  const largestTimeForThisDay = Math.max(thisWeeksTime, avgTime)

  const bestDailyBarHeight = calculateBarHeight(
    bestTime,
    largestTimeForThisDay,
  )

  const bestMiniBarHeight = calculateBarHeight(
    bestMiniTime,
    todaysMiniTime,
  )

  const todaysDailyBarHeight = calculateBarHeight(
    thisWeeksTime,
    largestTimeForThisDay,
  )

  const todaysMiniBarHeight = calculateBarHeight(
    todaysMiniTime,
    todaysMiniTime,
  )

  // Has Stats Variables
  const hasDailyStats = !!bestTime
  const hasMiniStats = !!bestMiniTime
  const hasNoDailyStats = !thisWeeksTime && !hasDailyStats

  // Chart Bars
  const bestDailyBar = (
    <div
      className="hub-mobile-stats__best daily"
      style={getBarHeight(bestDailyBarHeight)}
    >
      <span className="hub-mobile-stats__time">
        {formatAsDuration(bestTime)}
      </span>
    </div>
  )

  const todaysDailyBar = (
    <div
      className="hub-mobile-stats__today daily"
      style={getBarHeight(todaysDailyBarHeight)}
    >
      <span className="hub-mobile-stats__time">
        {formatAsDuration(thisWeeksTime)}
      </span>
    </div>
  )

  const bestMiniBar = (
    <div
      className="hub-mobile-stats__best mini"
      style={getBarHeight(bestMiniBarHeight)}
    >
      <span className="hub-mobile-stats__time">
        {formatAsDuration(bestMiniTime)}
      </span>
    </div>
  )

  const todaysMiniBar = (
    <div
      className="hub-mobile-stats__today mini"
      style={getBarHeight(todaysMiniBarHeight)}
    >
      <span className="hub-mobile-stats__time">
        {formatAsDuration(todaysMiniTime)}
      </span>
    </div>
  )

  return (
    <div
      data-track-hub={STATS}
      className="hub-mobile-stats__container js-hub-tracker"
    >
      <h2>Statistics</h2>
      <div className="hub-mobile-stats__stats-container">
        <div className="hub-mobile-stats__stats-chart-block">
          <div>
            <div className="hub-mobile-stats__stats-chart-label">
              Mini
            </div>
            <div
              className={cn('hub-mobile-stats__stats-bars-block', {
                'hub-mobile-stats__no-stats-block': !hasMiniStats,
              })}
            >
              {!!todaysMiniTime && todaysMiniBar}
              {hasMiniStats ? bestMiniBar : noStats}
            </div>
          </div>
          <div>
            <div className="hub-mobile-stats__stats-chart-label">
              Daily
            </div>
            <div
              className={cn('hub-mobile-stats__stats-bars-block', {
                'hub-mobile-stats__no-stats-block': hasNoDailyStats,
              })}
            >
              {hasNoDailyStats && noStats}
              {!!thisWeeksTime && todaysDailyBar}
              {hasDailyStats && bestDailyBar}
            </div>
          </div>
        </div>
        <a
          href="/puzzles/stats"
          className="hub-mobile-stats__stats-more"
        >
          More
        </a>
      </div>
    </div>
  )
}

const mapStateToProps = (state) => ({
  user: state.user,
  statsAndStreaks: {
    stats: state.stats.dailyStats,
    streaks: state.stats.streaks,
    miniStats: state.stats.miniStats,
  },
  subscribeLinks: state.subscribeLinks,
})

export default connect(mapStateToProps)(MobileStatsCard)

MobileStatsCard.propTypes = {
  user: pt.shape({
    hasXwd: pt.bool,
  }),
  subscribeLinks: pt.shape({}),
  statsAndStreaks: pt.shape({
    stats: pt.shape({
      statsByDay: pt.arrayOf(
        pt.shape({
          thisWeeksTime: pt.number.isRequired,
          avgTime: pt.number.isRequired,
          bestTime: pt.number.isRequired,
          latestTime: pt.number.isRequired,
          label: pt.string.isRequired,
          latestDate: pt.string.isRequired,
        }),
      ),
    }),
    streaks: pt.shape({
      currentStreak: pt.number.isRequired,
      longestStreak: pt.number.isRequired,
      dateEnd: pt.string.isRequired,
    }),
    miniStats: pt.shape({
      bestDate: pt.string.isRequired,
      bestTime: pt.number.isRequired,
      todaysDate: pt.string.isRequired,
      todaysTime: pt.number.isRequired,
    }),
  }),
}
