import React from 'react'
import PT from 'prop-types'
import { connect } from 'react-redux'
import classNames from 'classnames'

import { formatDate } from '../store/selectors'
import {
  changeBlockOpacity,
  changePrintLayout,
  changePrintOption,
} from '../store/actionCreators'

import * as printOptions from './printOptions'

class PrintModal extends React.Component {
  sendChangeToStore = (event) => {
    const { handlePrintOptionChange } = this.props
    const { target } = event
    const { checked, name, type, value } = target
    const sendValue = type === 'radio'
    handlePrintOptionChange(name, sendValue ? value : checked)
  }

  updateLayout = (event) => {
    const { changeLayout } = this.props
    const newLayout = event.target.value || 'standard'
    changeLayout(newLayout)
  }

  updateOpacity = (event) => {
    const { changeOpacity } = this.props
    const newOpacity = event.target.checked ? 30 : 100
    changeOpacity(newOpacity)
  }

  printPuzzle = () => {
    const { printUrl } = this.props
    window.open(printUrl)
  }

  render() {
    const {
      version,
      layout,
      selectedPuzzle,
      puzzleDate,
      opacity,
      showBlack,
      showSpoiler,
      hasLayoutOptions,
      printLayoutsUnavailable,
      printOpacityUnavailable,
      hasBothVersions,
      solutionIsAvailable,
    } = this.props

    return (
      <>
        <header className="hub-print-modal__title">Print</header>
        <div className="hub-print-modal-content">
          {selectedPuzzle.formatType && (
            <div className="hub-print-modal-puzzle-details">
              <div className="hub-print-modal-puzzle-title">
                {selectedPuzzle.title || 'The Crossword'}
              </div>
              <div className="hub-print-modal-puzzle-date">
                {puzzleDate}
              </div>
            </div>
          )}
          <div className="hub-print-modal-options">
            <div className="hub-print-modal-section">
              <div className="hub-print-modal-left">
                <label htmlFor="puzzle">
                  <input
                    type="radio"
                    id="puzzle"
                    name="version"
                    value="puzzle"
                    checked={version === 'puzzle'}
                    onChange={this.sendChangeToStore}
                    tabIndex={0}
                  />
                  <span>Puzzle</span>
                </label>
                {hasLayoutOptions && (
                  <div
                    className={classNames('hub-print-modal-inset', {
                      'hub-print-modal-disabled':
                        printLayoutsUnavailable,
                    })}
                  >
                    <label htmlFor="standard">
                      <input
                        type="radio"
                        id="standard"
                        name="layout"
                        value="standard"
                        checked={layout === 'standard'}
                        onChange={this.updateLayout}
                        disabled={printLayoutsUnavailable}
                        aria-disabled={printLayoutsUnavailable}
                      />
                      <span>Standard Layout</span>
                    </label>

                    <label htmlFor="large_print">
                      <input
                        type="radio"
                        id="large_print"
                        name="layout"
                        value="large_print"
                        checked={layout === 'large_print'}
                        onChange={this.updateLayout}
                        disabled={printLayoutsUnavailable}
                        aria-disabled={printLayoutsUnavailable}
                      />
                      <span>Large Print</span>
                    </label>

                    <label htmlFor="southpaw">
                      <input
                        type="radio"
                        id="southpaw"
                        name="layout"
                        value="southpaw"
                        checked={layout === 'southpaw'}
                        onChange={this.updateLayout}
                        disabled={printLayoutsUnavailable}
                        aria-disabled={printLayoutsUnavailable}
                      />
                      <span>Left-handed</span>
                    </label>
                  </div>
                )}
                <label htmlFor="solution">
                  <input
                    type="radio"
                    id="solution"
                    name="version"
                    value="solution"
                    checked={version === 'solution'}
                    onChange={this.sendChangeToStore}
                    disabled={!solutionIsAvailable}
                    aria-disabled={!solutionIsAvailable}
                  />
                  <span>Solution</span>
                  {!solutionIsAvailable && (
                    <div className="hub-print-modal-subscript">
                      (available Thursday 10 p.m. ET)
                    </div>
                  )}
                </label>
              </div>
              <div
                className={classNames(
                  'hub-print-modal-cell-darkness',
                  {
                    'hub-print-modal-disabled':
                      printOpacityUnavailable,
                  },
                )}
              >
                <label htmlFor="opacity">
                  <input
                    onChange={this.updateOpacity}
                    onMouseUp={this.updateOpacity}
                    type="checkbox"
                    id="opacity"
                    name="opacity"
                    disabled={printOpacityUnavailable}
                    aria-disabled={printOpacityUnavailable}
                    checked={opacity < 100}
                    // from styles.opacitySlider in games-web, not used
                    className="hub-print-modal-opacity-slider"
                  />
                  <span>Ink Saver</span>
                </label>
                <div>
                  <div className="hub-print-modal-opacity-icon">
                    <div className="hub-print-modal-opacity-reference" />
                    <div
                      className="hub-print-modal-user-opacity"
                      style={{ opacity: opacity / 100 }}
                    />
                  </div>
                </div>
              </div>
            </div>
            {hasBothVersions && (
              <div className="hub-print-modal-section hub-print-modal-left">
                <label htmlFor="newspaper">
                  <input
                    type="radio"
                    id="newspaper"
                    name="version"
                    value="newspaper"
                    checked={version === 'newspaper'}
                    onChange={this.sendChangeToStore}
                  />
                  <span>Newspaper Version</span>
                </label>
              </div>
            )}
            <div className="hub-print-modal__button-wrapper">
              <button
                type="button"
                className="hub-print-modal__button pz-modal__button dark"
                onClick={this.printPuzzle}
              >
                Print
              </button>
            </div>
          </div>
        </div>
      </>
    )
  }
}

PrintModal.propTypes = {
  version: PT.string,
  layout: PT.string,
  handlePrintOptionChange: PT.func,
  changeOpacity: PT.func.isRequired,
  changeLayout: PT.func.isRequired,
  selectedPuzzle: PT.shape({
    formatType: PT.string,
    title: PT.string,
  }),
  printUrl: PT.string,
  puzzleDate: PT.string,
  opacity: PT.number,
  showBlack: PT.bool,
  showSpoiler: PT.bool,
  printLayoutsUnavailable: PT.bool,
  printOpacityUnavailable: PT.bool,
  hasLayoutOptions: PT.bool,
  hasBothVersions: PT.bool,
  solutionIsAvailable: PT.bool,
}

PrintModal.defaultProps = {
  version: '',
  layout: 'standard',
  handlePrintOptionChange: () => {},
  selectedPuzzle: {
    formatType: '',
    title: '',
  },
  printUrl: '',
  puzzleDate: '',
  opacity: undefined,
  showBlack: false,
  showSpoiler: false,
  printLayoutsUnavailable: false,
  printOpacityUnavailable: false,
  hasLayoutOptions: false,
  hasBothVersions: false,
  solutionIsAvailable: false,
}

const mapStateToProps = (state) => ({
  ...printOptions.getPrintOptions(state), // selectedPuzzle and opacity
  printUrl: printOptions.getPrintUrl(state),
  puzzleDate: formatDate(state),
  printLayoutsUnavailable:
    printOptions.printLayoutsUnavailable(state),
  printOpacityUnavailable:
    printOptions.printOpacityUnavailable(state),
  hasLayoutOptions: printOptions.hasLayoutOptions(state),
  hasBothVersions: printOptions.hasBothVersions(state),
  solutionIsAvailable: printOptions.solutionIsAvailable(state),
})

const mapDispatchToProps = (dispatch) => ({
  changeOpacity: (opacity) => dispatch(changeBlockOpacity(opacity)),
  changeLayout: (layout) => dispatch(changePrintLayout(layout)),
  handlePrintOptionChange: (name, value) =>
    dispatch(changePrintOption(name, value)),
})

export default connect(
  mapStateToProps,
  mapDispatchToProps,
)(PrintModal)
