import { createSelector } from 'reselect'
import moment from 'moment'
import 'moment-timezone'

const getMomentNow = () => moment()

export const getPrintOptions = (state) => {
  const { printOptions, printPrefs } = state
  return {
    ...printOptions,
    ...printPrefs,
  }
}
const isGeneratable = (state) => state.printOptions.isGeneratable
export const selectedPuzzle = (state) =>
  state.printOptions.selectedPuzzle
const pdfVersion = (state) => state.printOptions.version

export const hasLayoutOptions = createSelector(
  selectedPuzzle,
  (puzzle) => puzzle.category === 0,
)

export const hasBothVersions = createSelector(
  selectedPuzzle,
  (puzzle) => {
    const { category, publicationDate } = puzzle

    if (category !== 0) return false // Only crosswords have both versions

    const stream = puzzle.stream || puzzle.publishStream // publishStream available only on game pages

    let firstDateOfPdf

    switch (stream) {
      case 'daily':
        firstDateOfPdf = '2011-04-01'
        break
      case 'bonus': {
        return false
      }
      default:
        return false
    }

    return moment(publicationDate).isSameOrAfter(firstDateOfPdf)
  },
)

export const solutionIsAvailable = createSelector(
  selectedPuzzle,
  (puzzle) => {
    const { category, publicationDate } = puzzle

    const thursdayAt10PM = moment(publicationDate)
      .tz('America/New_York')
      .add({ days: 4, hours: 22 })
      .format()

    const isSolutionAvailableNow = getMomentNow()
      .tz('America/New_York')
      .isSameOrAfter(thursdayAt10PM)

    // For PDF-only (category "other") puzzles, solution PDFs are
    // only available at 10pm the Thursday after publish date
    if (category === 8) return isSolutionAvailableNow

    return true
  },
)

export const printLayoutsUnavailable = createSelector(
  isGeneratable,
  pdfVersion,
  (generatable, version) => !generatable || version === 'newspaper',
)

export const printOpacityUnavailable = createSelector(
  isGeneratable,
  pdfVersion,
  (generatable, version) => !generatable || version === 'newspaper',
)

const getGeneratedPdfUrl = createSelector(
  selectedPuzzle,
  pdfVersion,
  (puzzle, version) => {
    const base = '/svc/crosswords/v2/puzzle/'
    const extension = version === 'solution' ? '.ans.pdf' : '.pdf'
    return `${base + puzzle.id + extension}`
  },
)

const getNewspaperPdfUrl = createSelector(
  selectedPuzzle,
  pdfVersion,
  (puzzle, version) => {
    const base = '/svc/crosswords/v2/puzzle/print/'
    const stream = puzzle.stream || puzzle.publishStream // publishStream available only on game pages

    const dateString = moment(puzzle.publicationDate).format(
      'MMMDDYY',
    )
    const extension = version === 'solution' ? '.ans.pdf' : '.pdf'

    switch (stream) {
      case 'daily':
        break
      default:
        return ''
    }

    return `${base + dateString + extension}`
  },
)

const getQueryStrings = createSelector(
  getPrintOptions,
  printOpacityUnavailable,
  (options, inkSaverDisabled) => {
    const { opacity } = options
    const params = {}

    if (!inkSaverDisabled && opacity < 100) {
      params.block_opacity = opacity
    }

    if (options.layout === 'southpaw') {
      params.southpaw = true
    } else if (options.layout === 'large_print') {
      params.large_print = true
    }

    const constructQuery = (parameters) =>
      Object.entries(parameters)
        .map((keyValue) => keyValue.join('='))
        .join('&')

    const hereBeQueries = Object.keys(params).length > 0
    return hereBeQueries ? `?${constructQuery(params)}` : ''
  },
)

export const getPrintUrl = createSelector(
  getPrintOptions,
  isGeneratable,
  getGeneratedPdfUrl,
  getNewspaperPdfUrl,
  getQueryStrings,
  (
    options,
    generatable,
    generatedPdfUrl,
    newspaperPdfUrl,
    queryStrings,
  ) => {
    const pdfUrl = generatable ? generatedPdfUrl : newspaperPdfUrl
    switch (options.version) {
      case 'puzzle':
      case 'solution':
        return `${pdfUrl}${queryStrings}`
      case 'newspaper':
        return `${newspaperPdfUrl}${queryStrings}`
      default:
        return `${generatedPdfUrl}${queryStrings}`
    }
  },
)
