import React, { useEffect, useState } from 'react'
import PT from 'prop-types'
import { connect } from 'react-redux'

import withPuzzleCollection from '../withPuzzleCollection'
import Accordion from '../shared/Accordion/Accordion'
import Section from '../shared/Section/Section'
import TabGroup from '../shared/TabGroup/TabGroup'
import ButtonTab from '../shared/ButtonTab/ButtonTab'
import TrackInView from '../TrackInView/TrackInView'
import { HideOn } from '../shared/Visibility/index'
import PuzzleCard from '../PuzzleCard/PuzzleCard'
import { getRecentPuzzles } from '../store/actionCreators'
import {
  formatConsecutiveDates,
  getStreakBoundaries,
  formatDate,
} from '../utils/time'

const mapStateToProps = ({ user }) => ({ user })

const mapDispatchToProps = (dispatch, { api }) => ({
  fetchRecent: () =>
    dispatch(getRecentPuzzles(api.fetchRecentPuzzles)),
})

const Bonus = withPuzzleCollection(({ puzzles, user }) => (
  <section>
    <h2 className="progress__sectionHeader">Monthly Bonus</h2>
    <TrackInView event="hub-monthly-bonus">
      <PuzzleCard
        puzzle={puzzles[0]}
        user={user}
        type="Island"
        section="monthly-bonus"
      />
    </TrackInView>
  </section>
))

const PlayMoreLink = () => (
  <div className="progress__playMoreLink">
    <a href="/crosswords/archive">PLAY MORE FROM THE ARCHIVE</a>
  </div>
)

const withStreaks = (state) => ({
  streakDates: state.stats.streaks && state.stats.streaks.dates,
})

const DumbCollection = ({
  useStreakLines,
  puzzles,
  timeFormatter,
  streakDates,
  titleFormat = 'ddd',
  children,
  isPending,
  className,
  user,
  section,
  setShowPlayMoreLink,
}) => {
  const puzzleDates = puzzles.map((puzz) => puzz.print_date)
  const formats = timeFormatter ? timeFormatter(puzzleDates) : []
  const streakProps =
    useStreakLines && streakDates
      ? getStreakBoundaries(streakDates, puzzleDates)
      : []

  /**
   * We must be sure that the recent progress has been
   * fetched and it is actually empty before we decide to say
   * the user has no progress
   */
  const displayNoPuzzlesState = !isPending && puzzles.length === 0

  // Hide the "Play More" link when a user has no in-progress puzzles;
  // the link will appear inside the "In Progress" drawer instead
  useEffect(() => {
    if (section === 'last-7-days') return
    setShowPlayMoreLink(!displayNoPuzzlesState)
  }, [section, displayNoPuzzlesState])

  const minHeight = 162

  return displayNoPuzzlesState ? (
    <>
      <div
        className="expandToRow progress__noProgressContainer"
        style={{ minHeight }}
      >
        {children}
      </div>
    </>
  ) : (
    <>
      <div className={className} style={{ minHeight }}>
        {puzzles.slice(0, 7).map((puzz, idx) => (
          <PuzzleCard
            key={puzz.puzzle_id}
            type="Thumbnail"
            dateFormat={formats[idx]}
            title={formatDate(puzz.print_date, titleFormat)}
            puzzle={puzz}
            {...streakProps[idx]}
            user={user}
            section={section}
          />
        ))}
      </div>
      <div className="progress__desktopOnly">
        <PlayMoreLink />
      </div>
    </>
  )
}

DumbCollection.propTypes = {
  puzzles: PT.arrayOf(PT.shape({})),
  timeFormatter: PT.func,
  streakDates: PT.arrayOf(PT.arrayOf(PT.string)),
  titleFormat: PT.string,
  isPending: PT.bool,
  children: PT.oneOfType([PT.element, PT.string]),
  className: PT.string,
  useStreakLines: PT.bool,
  user: PT.shape({}),
  section: PT.string,
  setShowPlayMoreLink: PT.func,
}

const Collection = withPuzzleCollection(
  connect(withStreaks)(DumbCollection),
)

const NoProgress = () => (
  <div className="progress__noProgress">
    <p>Choose from over 20 years of the puzzles in our archive.</p>
    <PlayMoreLink />
  </div>
)

const Progress = ({ fetchRecent, user }) => {
  const [showPlayMoreLink, setShowPlayMoreLink] = useState(true)

  const AccordionGroup = [
    <Accordion key="last-7" title="Last 7 Days" startOpen>
      <Collection
        useStreakLines
        puzzleCollection="weeklyPuzzles"
        timeFormatter={formatConsecutiveDates}
        titleFormat="dddd"
        className="progress__accordionContent"
        user={user}
        section="last-7-days"
        setShowPlayMoreLink={setShowPlayMoreLink}
      />
    </Accordion>,
  ]

  const tabs = [
    {
      Tab: <ButtonTab title="Last 7 Days" />,
      content: (
        <Collection
          useStreakLines
          puzzleCollection="weeklyPuzzles"
          timeFormatter={formatConsecutiveDates}
          className="expandToRow"
          user={user}
          section="last-7-days"
          setShowPlayMoreLink={setShowPlayMoreLink}
        />
      ),
      key: 'last-7',
    },
  ]

  if (user.hasXwd) {
    AccordionGroup.push(
      <Accordion
        key="recent"
        title="In Progress"
        onView={(firstCall) => {
          if (firstCall) fetchRecent()
        }}
      >
        <Collection
          puzzleCollection="recentPuzzles"
          titleFormat="dddd"
          className="progress__accordionContent"
          user={user}
          section="in-progress"
          setShowPlayMoreLink={setShowPlayMoreLink}
        >
          <NoProgress />
        </Collection>
      </Accordion>,
    )
    tabs.push({
      Tab: <ButtonTab title="In Progress" />,
      onView: (firstCall) => {
        if (firstCall) fetchRecent()
      },
      content: (
        <Collection
          puzzleCollection="recentPuzzles"
          className="expandToRow"
          user={user}
          section="in-progress"
          setShowPlayMoreLink={setShowPlayMoreLink}
        >
          <NoProgress />
        </Collection>
      ),
      key: 'in-progress',
    })
  }

  return (
    <Section backgroundColor="white">
      {user.hasXwd && (
        <HideOn phone smallDevice>
          <Bonus puzzleCollection="bonusPuzzle" user={user} />
        </HideOn>
      )}
      <div className="progress__recentlyTabbedModule">
        <div className="progress__mobileOnly">
          {AccordionGroup}
          {showPlayMoreLink && <PlayMoreLink />}
        </div>
        <div className="progress__tabGroup">
          <TabGroup tabs={tabs} />
        </div>
      </div>
    </Section>
  )
}

Progress.propTypes = {
  user: PT.shape({
    hasXwd: PT.bool,
  }),
  fetchRecent: PT.func.isRequired,
}

export default connect(mapStateToProps, mapDispatchToProps)(Progress)
