import React from 'react'
import { connect } from 'react-redux'
import pt from 'prop-types'
import cn from 'classnames'

import { camelizeKeysDeep } from '../utils/miscellany'
import { SELECT_PUZZLE } from '../store/constants'
import { useModalSystem } from '../Modal/ModalContext'

import PuzzleIcon from './PuzzleIcon'
import PuzzleRibbon from './PuzzleRibbon'
import CardContent from './PuzzleCardContent'
import Print from './Print'
import adaptCardProps from './adaptCardProps'
import {
  cardTypes,
  cardTypeClassNames,
  actionTypeClassNames,
  VIEW_NEWSPAPER,
  SUBSCRIBE,
} from './cardConstants'

const subTypeClasses = {
  Primary: 'primary',
  Mobile: 'mobile',
}

const subTypes = Object.keys(subTypeClasses)

/**
 * The Puzzle card
 * takes in the props, simplifies them
 * and turns over the rendering to one of the card types
 */
export const PuzzleCard = (props) => {
  const modalSystem = useModalSystem()
  /**
   * It is easy for a component above
   * to pass an undefined puzzle if
   * depending on an empty list (if api is missing the puzzles)
   * so we make sure the puzzle is real before rendering
   */
  if (!props.puzzle) return null // eslint-disable-line

  const {
    type: cardType,
    subType,
    user,
    selectPrintPuzzle,
    isFake,
    isStatic,
    subscribeLinks,
    title,
    section,
  } = props

  const openPrintModal = (puzzle) => {
    selectPrintPuzzle(puzzle)
    modalSystem.open('print')
  }

  // NOTE: we should camelCase the puzzle keys at the api level
  const puzzle = camelizeKeysDeep(props.puzzle) // eslint-disable-line
  const adaptedProps = adaptCardProps({
    ...props,
    puzzle,
    openPrintModal,
  })

  const {
    link,
    action,
    handlePrintAction,
    shouldDisplayUpsell,
    hasGoldStar,
    isBrandNew,
  } = adaptedProps

  const wrapperClass = cn(
    cardTypeClassNames[cardType],
    actionTypeClassNames[action],
    subTypeClasses[subType] || 'standard',
    'puzzleInfo',
    { hasUpsell: shouldDisplayUpsell },
    { hasGoldStar },
    { isFake },
  )

  // We've disabled the half off promotion
  // See https://jira.nyt.net/browse/XWD-14263
  const qualifiesForHalfOff = false

  const progressIcon = (
    <div className="progressIcon">
      <PuzzleIcon
        {...adaptedProps}
        isBrandNew={isBrandNew}
        isFake={isFake}
        isStatic={isStatic}
      />
      {isFake || (
        <PuzzleRibbon {...adaptedProps} isBrandNew={isBrandNew} />
      )}
    </div>
  )

  const shouldDisplayPrintTools = action !== SUBSCRIBE && !isFake
  const disablePrintModal = action === VIEW_NEWSPAPER

  const cardContent = isFake ? (
    <div className="puzzleAction">
      <CardContent
        type={cardType}
        {...adaptedProps}
        progressIcon={progressIcon}
      />
    </div>
  ) : (
    <a
      className="puzzleAction"
      href={link}
      onClick={handlePrintAction}
    >
      <CardContent
        type={cardType}
        {...adaptedProps}
        progressIcon={progressIcon}
      />
    </a>
  )

  const upsellTracking = () => {
    if (!window.foundationTrack) {
      return
    }

    if (shouldDisplayUpsell) {
      window.foundationTrack('moduleInteraction', {
        module: {
          type: 'click',
          element: {
            name: 'hub',
            label: 'crossword-subscribe',
          },
        },
      })
    }
  }

  return (
    <div
      className={cn(wrapperClass, 'js-hub-tracker')}
      role="button"
      tabIndex="0"
      data-track-hub={title}
      data-track-hub-context={puzzle.puzzleId}
      onClick={() => upsellTracking()}
      onKeyDown={(e) => (e.key === 'Enter' ? upsellTracking() : null)}
    >
      {cardContent}
      {shouldDisplayPrintTools && (
        <Print
          puzzle={puzzle}
          printAction={openPrintModal}
          pdfLinkOnly={disablePrintModal}
          section={section}
        />
      )}
      {shouldDisplayUpsell && (
        <a href={subscribeLinks.daily} className="upsell">
          <span className="copy">
            {'Want to play online? '}
            <b>Subscribe&nbsp;Today!</b>
          </span>
          {qualifiesForHalfOff && <div className="halfOff" />}
        </a>
      )}
    </div>
  )
}

PuzzleCard.propTypes = {
  subType: pt.oneOf(subTypes),
  type: pt.oneOf(cardTypes).isRequired,
  puzzle: pt.shape({}),
  user: pt.shape({
    // TODO: rename this to userType during milestone 3 of client side entitlements epic
    hasDigi: pt.bool,
    hasHd: pt.bool,
  }).isRequired,
  dateFormat: pt.string, // custom date formatting from the card
  title: pt.string, // allows the card to specify the title
  isFree: pt.bool, // declare that the puzzle is free (until api can tell us)
  hasStreakAfter: pt.bool,
  hasStreakBefore: pt.bool,
  selectPrintPuzzle: pt.func.isRequired,
  isFake: pt.bool,
  isBrandNew: pt.bool,
  isStatic: pt.bool,
  subscribeLinks: pt.shape({
    daily: pt.string,
  }),
  section: pt.string,
}

const mapDispatchToProps = (dispatch) => ({
  selectPrintPuzzle: (selectedPuzzle) => {
    dispatch({ type: SELECT_PUZZLE, payload: selectedPuzzle })
  },
})

export default connect(null, mapDispatchToProps)(PuzzleCard)
