import React from 'react'
import { shallow } from 'enzyme'

import adaptCardProps from '../adaptCardProps'
import PuzzleIcon from '../PuzzleIcon'
import PuzzleRibbon from '../PuzzleRibbon'
import PuzzleCardContent from '../PuzzleCardContent'
import Print from '../Print'
import { PuzzleCard } from '../PuzzleCard'

jest.mock('../adaptCardProps', () => jest.fn())

jest.mock('../PuzzleIcon', () => () => <div />)
jest.mock('../PuzzleRibbon', () => () => <div />)
jest.mock('../PuzzleCardContent', () => () => <div />)
jest.mock('../Print', () => () => <div />)

const getWrapperClass = (card) =>
  card.find('.puzzleInfo').props().className

describe('PUZZLE CARD', () => {
  let defaultProps
  let printAction
  let card
  let adaptedProps
  let user

  beforeEach(() => {
    jest.resetAllMocks()

    printAction = jest.fn()

    user = {}

    defaultProps = {
      user,
      subscribeLinks: {
        daily: 'daily-sub-link',
      },
      type: 'Featured',
      puzzle: {
        print_date: '123',
      },
      selectPrintPuzzle: () => {},
    }

    adaptedProps = {
      link: 'link-text',
      action: 'PLAY',
      handlePrintAction: printAction,
      shouldDisplayUpsell: true,
      hasGoldStar: true,
    }

    adaptCardProps.mockReturnValue(adaptedProps)
  })

  test('renders null if puzzle is mizzing', () => {
    expect(
      shallow(
        <PuzzleCard {...defaultProps} puzzle={undefined} />,
      ).html(),
    ).toBe(null)
  })

  test('passes camel cased keys to adapt props', () => {
    shallow(
      <PuzzleCard {...defaultProps} puzzle={{ prop_a: true }} />,
    )
    expect(adaptCardProps).toHaveBeenCalledWith(
      expect.objectContaining({
        ...defaultProps,
        puzzle: { propA: true },
        // openPrintModal isn't exposed by PuzzleCard, so we can't directly match on it
        openPrintModal: expect.any(Function),
      }),
    )
  })

  test('build wrapper classes', () => {
    card = shallow(<PuzzleCard {...defaultProps} />)
    expect(getWrapperClass(card)).toContain('featured') // the type class
    expect(getWrapperClass(card)).toContain('standard') // the subtype class
    expect(getWrapperClass(card)).toContain('play') // the action class
    expect(getWrapperClass(card)).toContain('hasUpsell') // the upsell class
    expect(getWrapperClass(card)).toContain('hasGoldStar') // the gold star class
    expect(getWrapperClass(card)).not.toContain('isFake') // the gold star class

    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      shouldDisplayUpsell: false,
      hasGoldStar: false,
    })

    card = shallow(
      <PuzzleCard
        {...defaultProps}
        type="Island"
        subType="Mobile"
        isFake
        action="PLAY"
      />,
    )
    expect(getWrapperClass(card)).toContain('island') // the type class
    expect(getWrapperClass(card)).toContain('mobile') // the subtype class
    expect(getWrapperClass(card)).toContain('play') // the action class
    expect(getWrapperClass(card)).not.toContain('hasUpsell') // the upsell class
    expect(getWrapperClass(card)).not.toContain('hasGoldStar') // the gold star class
    expect(getWrapperClass(card)).toContain('isFake') // the gold star class
  })

  test('creates the puzzle link and action with content inside', () => {
    card = shallow(<PuzzleCard {...defaultProps} />)
    const actionWrapper = card.find('.puzzleAction')

    expect(actionWrapper.type()).toBe('a')
    expect(actionWrapper.props().href).toBe('link-text')

    const content = actionWrapper.find(PuzzleCardContent)

    expect(content.props()).toMatchObject({
      type: 'Featured',
      ...adaptedProps,
    })

    const icon = shallow(content.props().progressIcon)

    expect(icon.find(PuzzleIcon).props()).toEqual({
      ...adaptedProps,
      isBrandNew: undefined,
    })

    expect(icon.find(PuzzleRibbon).props()).toEqual({
      ...adaptedProps,
      isBrandNew: undefined,
    })
  })

  test('passes brand new prop', () => {
    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      isBrandNew: true,
    })

    card = shallow(<PuzzleCard {...defaultProps} />)
    const content = card.find(PuzzleCardContent)
    const icon = shallow(content.props().progressIcon)

    expect(icon.find(PuzzleIcon).props()).toEqual({
      ...adaptedProps,
      isBrandNew: true,
    })

    expect(icon.find(PuzzleRibbon).props()).toEqual({
      ...adaptedProps,
      isBrandNew: true,
    })
  })

  test('does not render ribbon for fake cards', () => {
    card = shallow(<PuzzleCard {...defaultProps} isFake />)
    const content = card.find(PuzzleCardContent)
    const icon = shallow(content.props().progressIcon)

    expect(icon.find(PuzzleIcon).length).toBe(1)
    expect(icon.find(PuzzleRibbon).length).toBe(0)
  })

  test('does not render print tools for fake cards', () => {
    card = shallow(<PuzzleCard {...defaultProps} isFake />)
    expect(card.find(Print).length).toBe(0)
  })

  test('does not render a link', () => {
    card = shallow(<PuzzleCard {...defaultProps} isFake />)
    expect(card.find('a.puzzleAction').length).toBe(0)
  })

  test('displays print tools when appropriate', () => {
    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      action: 'SUBSCRIBE',
    })

    /** no print and download for non-subs */
    card = shallow(<PuzzleCard {...defaultProps} />)

    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      action: 'PRINT',
    })

    /** otherwise show the component */
    card = shallow(<PuzzleCard {...defaultProps} />)

    expect(card.find(Print).length).toBe(1)
  })

  test('displays upsell for correct users', () => {
    const { foundationTrack } = global.window
    Object.defineProperty(window, 'foundationTrack', {
      writable: true,
      value: jest.fn(),
    })

    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      shouldDisplayUpsell: false,
    })

    card = shallow(<PuzzleCard {...defaultProps} />)
    expect(card.find('a.upsell').length).toBe(0)

    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      shouldDisplayUpsell: true,
    })

    card = shallow(<PuzzleCard {...defaultProps} />)

    const upsell = card.find('a.upsell')
    expect(upsell.props().href).toBe('daily-sub-link')
    expect(upsell.find('.halfOff').length).toBe(0)

    card.find('.hasUpsell').simulate('click')

    const [eventType, eventData] =
      window.foundationTrack.mock.calls[0]

    expect(eventType).toBe('moduleInteraction')
    expect(eventData).toEqual({
      module: {
        type: 'click',
        element: {
          name: 'hub',
          label: 'crossword-subscribe',
        },
      },
    })

    Object.defineProperty(window, 'foundationTrack', {
      writable: true,
      value: foundationTrack,
    })
  })

  test('displays half off for correct users', () => {
    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      shouldDisplayUpsell: true,
    })

    card = shallow(
      <PuzzleCard {...defaultProps} user={{ hasHd: true }} />,
    )

    expect(card.find('a.upsell').find('.halfOff').length).toBe(0)
  })

  test('displays streak lines', () => {
    adaptCardProps.mockReturnValue({
      ...adaptedProps,
      hasStreakBefore: true,
      hasStreakAfter: true,
    })

    card = shallow(<PuzzleCard {...defaultProps} />)
    const content = card.find(PuzzleCardContent)
    const icon = shallow(content.props().progressIcon)

    expect(icon.find(PuzzleIcon).props()).toEqual({
      ...adaptedProps,
      hasStreakBefore: true,
      hasStreakAfter: true,
    })
  })
})
