import {
  PLAY,
  PRINT,
  SUBSCRIBE,
  VIEW_NEWSPAPER,
  ISLAND,
  THUMBNAIL,
  FEATURED,
  CALENDAR,
} from '../cardConstants'
import createCardProps from '../adaptCardProps'

jest.mock('../../utils/time', () => ({
  formatDate: (date, format) => `${date}-${format}`,
}))

jest.mock('../../constants/puzzleTypes', () => ({
  downloadableFormats: ['downloadable-format'],
  playableFormats: ['playable-format'],
}))

describe('PUZZLE CARD (create props)', () => {
  let props
  let type
  let puzzle
  let user
  let collection
  let hasStreakAfter
  let hasStreakBefore
  let dateFormat
  let refreshAllProps
  let title
  let isFree
  let isFake
  let openPrintModal
  let isBrandNew
  let link
  let subscribeLinks

  beforeEach(() => {
    puzzle = {
      publishType: 'Daily',
      printDate: 'today',
    }

    user = {
      hasXwd: true,
      hasHd: true,
    }

    subscribeLinks = {
      default: 'other',
    }

    type = ISLAND
    title = undefined
    isFree = undefined
    isFake = undefined
    link = undefined
    dateFormat = undefined
    collection = undefined
    hasStreakAfter = false
    hasStreakBefore = false
    openPrintModal = jest.fn()

    refreshAllProps = () => {
      props = createCardProps({
        link,
        type,
        title,
        user,
        isFree,
        isFake,
        puzzle,
        dateFormat,
        collection,
        hasStreakAfter,
        hasStreakBefore,
        openPrintModal,
        isBrandNew,
        subscribeLinks,
      })
    }
  })

  describe('get author', () => {
    test('without an author', () => {
      refreshAllProps()
      expect(props.author).toEqual('')
    })
    test('with an author', () => {
      puzzle.author = 'Edward'
      refreshAllProps()
      expect(props.author).toEqual('By EDWARD')
    })
  })

  describe('get editor', () => {
    test('without an editor', () => {
      refreshAllProps()
      expect(props.editor).toEqual('')
    })
    test('with an editor', () => {
      puzzle.editor = 'Susan'
      refreshAllProps()
      expect(props.editor).toEqual('Edited by SUSAN')
    })
  })

  describe('get title', () => {
    test('for a puzzle with title', () => {
      puzzle.title = 'A Great Puzzle'
      refreshAllProps()
      expect(props.title).toEqual('A Great Puzzle')
    })
    test('for a card with title', () => {
      puzzle.title = 'A Great Puzzle'
      title = 'A Card Title' // the card title takes precedence
      refreshAllProps()
      expect(props.title).toEqual('A Card Title')
    })
    test('for a puzzle without a title', () => {
      puzzle.printDate = 'date'
      refreshAllProps()
      expect(props.title).toEqual('date-dddd')
    })
  })

  describe('get icon type', () => {
    test('for a mini puzzle', () => {
      puzzle.publishType = 'Mini'
      refreshAllProps()
      expect(props.iconType).toEqual('mini')
    })
    test('for any other puzzle', () => {
      puzzle.publishType = 'Other'
      refreshAllProps()
      expect(props.iconType).toEqual('puzzle')
    })
  })

  describe('get simple props', () => {
    test('puzzle id', () => {
      puzzle.puzzleId = 123
      refreshAllProps()
      expect(props.puzzleId).toEqual(123)
    })
    test('progress props', () => {
      puzzle.percentFilled = 70
      puzzle.solved = false
      puzzle.star = 'Gold'
      hasStreakAfter = true
      hasStreakBefore = false
      refreshAllProps()
      expect(props).toMatchObject({
        percentFilled: 70,
        isSolved: false,
        hasGoldStar: true,
        hasStreakAfter: true,
        hasStreakBefore: false,
      })
    })
  })

  describe('get brand new', () => {
    test('if not brand new, pass false', () => {
      isBrandNew = false
      puzzle.solved = false
      puzzle.star = ''
      puzzle.percentFilled = 0
      refreshAllProps()
      expect(props.isBrandNew).toEqual(false)
    })
    test('if brand new and nothing else, pass true', () => {
      isBrandNew = true
      puzzle.solved = false
      puzzle.star = ''
      puzzle.percentFilled = 0
      refreshAllProps()
      expect(props.isBrandNew).toEqual(true)
    })
    test('if is brand new and is solved, pass false', () => {
      isBrandNew = true
      puzzle.solved = true
      puzzle.star = ''
      puzzle.percentFilled = 0
      refreshAllProps()
      expect(props.isBrandNew).toEqual(false)
    })
    test('if is brand new and has progress, pass false', () => {
      isBrandNew = true
      puzzle.solved = false
      puzzle.star = ''
      puzzle.percentFilled = 50
      refreshAllProps()
      expect(props.isBrandNew).toEqual(false)
    })
    test('if is brand new and has gold star, pass false', () => {
      isBrandNew = true
      puzzle.solved = false
      puzzle.star = 'Gold'
      puzzle.percentFilled = 0
      refreshAllProps()
      expect(props.isBrandNew).toEqual(false)
    })
  })

  describe('get date', () => {
    test('for bonus puzzles', () => {
      puzzle.publishType = 'Bonus'
      refreshAllProps()
      expect(props.date).toEqual('today-{nyt-m} YYYY')
    })
    test('for island cards', () => {
      type = ISLAND
      refreshAllProps()
      expect(props.date).toEqual('today-{nyt-m} D, YYYY')
    })
    test('for thumbnail cards', () => {
      type = THUMBNAIL
      refreshAllProps()
      expect(props.date).toEqual('today-M/D/YY')
    })
    test('for primary featured cards', () => {
      type = FEATURED
      refreshAllProps()
      expect(props.date).toEqual('today-dddd, {nyt-m}\xa0D,\xa0YYYY')
    })
    test('for calendar cards', () => {
      type = CALENDAR
      refreshAllProps()
      expect(props.date).toEqual('today-D')
    })
    test('for a card with a dateFormat', () => {
      dateFormat = 'xxx'
      refreshAllProps()
      expect(props.date).toEqual('today-xxx')
    })
    test('for anything else', () => {
      type = null
      refreshAllProps()
      expect(props.date).toEqual('today-{nyt-m} D, YYYY')
    })
  })

  describe('get action', () => {
    beforeEach(() => {
      puzzle.formatType = 'playable-format'
      user.hasXwd = false
      user.hasHd = false
      puzzle.isFree = false
      isFree = false
    })
    test('actions are defined', () => {
      ;[PLAY, PRINT, SUBSCRIBE, VIEW_NEWSPAPER].forEach((action) => {
        expect(typeof action).toBe('string')
      })
    })
    test('for playing free puzzles', () => {
      puzzle.isFree = true
      refreshAllProps()
      expect(props.action).toEqual(PLAY)

      puzzle.isFree = false
      isFree = true // decided at the card prop level
      refreshAllProps()
      expect(props.action).toEqual(PLAY)
    })
    test('for paid puzzles (subscriber)', () => {
      user.hasXwd = true
      refreshAllProps()
      expect(props.action).toEqual(PLAY)
    })
    test('for print only puzzles (subscriber)', () => {
      user.hasXwd = true
      puzzle.formatType = 'print-only-format'
      refreshAllProps()
      expect(props.action).toEqual(PRINT)
    })
    test('for paid puzzles (home delivery subscriber)', () => {
      user.hasHd = true
      refreshAllProps()
      expect(props.action).toEqual(VIEW_NEWSPAPER)
    })
    test('for paid puzzles that are not printable (home delivery subscriber)', () => {
      user.hasHd = true
      puzzle.publishType = 'Bonus'
      refreshAllProps()
      expect(props.action).toEqual(SUBSCRIBE)
    })
    test('for paid puzzles (anonymous user)', () => {
      refreshAllProps()
      expect(props.action).toEqual(SUBSCRIBE)
    })
    test('for print only puzzles (anonymous user)', () => {
      puzzle.formatType = 'print-only-format'
      refreshAllProps()
      expect(props.action).toEqual(SUBSCRIBE)
    })
  })

  describe('get link', () => {
    test('for play action', () => {
      puzzle.isFree = true
      puzzle.formatType = 'playable-format'
      refreshAllProps()
      expect(props.action).toEqual(PLAY)
      expect(props.link).toEqual(
        '/crosswords/game/daily/today-YYYY/MM/DD',
      )
    })
    test('for hardcoded links', () => {
      link = 'override-link'
      refreshAllProps()
      expect(props.link).toEqual('override-link')
    })
    test('for print action', () => {
      user.hasXwd = true
      puzzle.formatType = 'print-only-format'
      refreshAllProps()
      expect(props.action).toEqual(PRINT)
      expect(props.link).toEqual(
        '/svc/crosswords/v2/puzzle/print/today-MMMDDYY.pdf',
      )
    })
    test('for view newspaper action', () => {
      user.hasXwd = false
      user.hasHd = true
      refreshAllProps()
      expect(props.action).toEqual(VIEW_NEWSPAPER)
      expect(props.link).toEqual(
        '/svc/crosswords/v2/puzzle/print/today-MMMDDYY.pdf',
      )
    })
    test('for subscribe action', () => {
      user.hasXwd = false
      user.hasHd = false
      refreshAllProps()
      expect(props.action).toEqual(SUBSCRIBE)
      expect(props.link).toEqual('other')
    })
  })

  describe('get handlePrintAction function', () => {
    test('for print action', () => {
      user.hasXwd = true
      puzzle.formatType = 'print-only-format'
      refreshAllProps()
      const event = { preventDefault: jest.fn() }
      props.handlePrintAction(event)
      expect(event.preventDefault).toHaveBeenCalled()
      expect(openPrintModal).toHaveBeenCalledWith(puzzle)
    })
    test('for other actions', () => {
      user.hasXwd = true
      puzzle.formatType = 'playable-format'
      refreshAllProps()
      const event = { preventDefault: jest.fn() }
      props.handlePrintAction(event)
      expect(event.preventDefault).not.toHaveBeenCalled()
      expect(openPrintModal).not.toHaveBeenCalled()
    })
  })

  describe('get if card should display upsell', () => {
    test('if user can play a Featured card', () => {
      user.hasXwd = true
      type = FEATURED
      refreshAllProps()
      expect(props.shouldDisplayUpsell).toBe(false)
    })
    test('if user cannot play a Featured card', () => {
      user.hasXwd = false
      type = FEATURED
      refreshAllProps()
      expect(props.shouldDisplayUpsell).toBe(true)
    })
    test('if user cannot play but the type is not Featured', () => {
      user.hasXwd = false
      type = ISLAND
      refreshAllProps()
      expect(props.shouldDisplayUpsell).toBe(false)
    })
  })
})
