import pt from 'prop-types'

import {
  createPDFUrl,
  createPlayUrl,
  isPrintOnly,
  hasNewspaperVersion,
} from '../utils/createUrls'
import { formatDate } from '../utils/time'

import {
  ISLAND,
  THUMBNAIL,
  FEATURED,
  CALENDAR,
  PLAY,
  PRINT,
  SUBSCRIBE,
  VIEW_NEWSPAPER,
} from './cardConstants'

const NONBREAKING_SPACE = String.fromCharCode(160)

/**
 * CARD PROPERTIES
 */
const getIconType = (puzzle) =>
  puzzle.publishType === 'Mini' ? 'mini' : 'puzzle'

const getAuthorString = ({ author }) =>
  author ? `By ${author.toUpperCase()}` : ''

const getEditorString = ({ editor }) =>
  editor ? `Edited by ${editor.toUpperCase()}` : ''

const getTitle = (puzzle, title) =>
  title || puzzle.title || formatDate(puzzle.printDate, 'dddd')

const getDateFormat = (puzzle, cardType, dateFormat) => {
  switch (true) {
    case !!dateFormat:
      return dateFormat
    case puzzle.publishType === 'Bonus':
      return '{nyt-m} YYYY'
    case cardType === FEATURED:
      return `dddd, {nyt-m}${NONBREAKING_SPACE}D,${NONBREAKING_SPACE}YYYY`
    case cardType === ISLAND:
      return '{nyt-m} D, YYYY'
    case cardType === THUMBNAIL:
      return 'M/D/YY'
    case cardType === CALENDAR:
      return 'D'
    default:
      return '{nyt-m} D, YYYY'
  }
}

const getAction = (puzzle, user, userCanPlay) => {
  const isPlayable = !isPrintOnly(puzzle)

  if (userCanPlay && isPlayable) return PLAY

  if (userCanPlay && !isPlayable) return PRINT

  if (user.hasHd && hasNewspaperVersion(puzzle)) return VIEW_NEWSPAPER

  return SUBSCRIBE
}

const getPrintAction = (action, puzzle, openPrintModal) =>
  action === PRINT
    ? (event) => {
        event.preventDefault()
        openPrintModal(puzzle)
      }
    : () => {}

const getActionLink = {
  [PLAY]: (puzzle) => createPlayUrl(puzzle),
  [PRINT]: (puzzle) => createPDFUrl(puzzle),
  [VIEW_NEWSPAPER]: (puzzle) => createPDFUrl(puzzle),
  [SUBSCRIBE]: (
    puzzle,
    // in the archive PuzzleCard is rendered only when the user is subscribed,
    // so we're adding a default value here to prevent errors
    subscribeLinks = {},
    subscriptionContext = 'default',
  ) => subscribeLinks[subscriptionContext],
}

/**
 * ADAPT CARD PROPS
 *
 * we adapt the puzzle, the user, and the other props
 * into a smaller set of props here in order to
 * centralize the tricky decisions for the card
 */
export default (incomingProps) => {
  const {
    type,
    user,
    puzzle,
    dateFormat,
    isFree,
    title,
    hasStreakAfter,
    hasStreakBefore,
    subscriptionContext,
    openPrintModal,
    isBrandNew,
    link,
    subscribeLinks,
  } = incomingProps

  // the isFree arg is important the api call decides what is free
  const isSolved = puzzle.solved
  const hasGoldStar = !!puzzle.star
  const { percentFilled } = puzzle
  const isFreePuzzle = puzzle.isFree || isFree
  const userCanPlay = isFreePuzzle || user.hasXwd
  const action = getAction(puzzle, user, userCanPlay)
  const isNew =
    !!isBrandNew && !hasGoldStar && !percentFilled && !isSolved

  const adaptedLink =
    link ||
    getActionLink[action](puzzle, subscribeLinks, subscriptionContext)

  const handlePrintAction = getPrintAction(
    action,
    puzzle,
    openPrintModal,
  )
  const finalDateFormat = getDateFormat(puzzle, type, dateFormat)

  const shouldDisplayUpsell = !userCanPlay && type === FEATURED

  return {
    /** action related props */
    link: adaptedLink,
    action,
    handlePrintAction,
    shouldDisplayUpsell,

    /** puzzle meta props */
    date: formatDate(puzzle.printDate, finalDateFormat),
    title: getTitle(puzzle, title),
    author: getAuthorString(puzzle),
    editor: getEditorString(puzzle),
    puzzleId: puzzle.puzzleId,

    /** progress icon props */
    percentFilled,
    isSolved,
    hasGoldStar,
    hasStreakAfter,
    hasStreakBefore,
    isBrandNew: isNew,
    iconType: getIconType(puzzle),
  }
}

/**
 * SHARED PROP TYPES FOR CARDS
 */
export const cardIconPropTypes = {
  action: pt.oneOf([PLAY, PRINT, SUBSCRIBE, VIEW_NEWSPAPER]),
  percentFilled: pt.number,
  isSolved: pt.bool,
  hasGoldStar: pt.bool,
  hasStreakAfter: pt.bool,
  hasStreakBefore: pt.bool,
  iconType: pt.oneOf(['mini', 'puzzle']),
}

export const cardPropTypes = {
  ...cardIconPropTypes,
  dateFormat: pt.string,
  date: pt.string.isRequired,
  title: pt.string,
  author: pt.string.isRequired,
  editor: pt.string,
  link: pt.string,
  puzzleId: pt.number,
  handlePrintAction: pt.func,
}
