import React from 'react'
import PT from 'prop-types'
import { connect } from 'react-redux'

import { formatAsDuration } from 'hub/utils/time'
import { calculateBarHeight, getBarHeight } from 'hub/utils/barHeight'
import TrackInView from 'hub/TrackInView/TrackInView'
import { STATS } from 'hub/track/constants'

export const StatsCard = (props) => {
  // Return placeholder if we have not gotten statsAndStreaks yet
  const { statsAndStreaks } = props

  if (!statsAndStreaks.streaks) {
    return (
      <TrackInView
        event="hub-stats"
        tagName="article"
        className="hub-stats-card__puzzle-info"
      >
        <a
          className="hub-stats-card__puzzle-info-content js-hub-tracker"
          href="/puzzles/stats"
          data-track-hub={STATS}
        >
          <div className="hub-stats-card__title">My Average</div>
          <div className="hub-stats-card__more-stats">More Stats</div>
        </a>
      </TrackInView>
    )
  }

  // Stats Variables
  const { stats, streaks } = statsAndStreaks
  const { currentStreak, longestStreak } = streaks
  const today = streaks.dateEnd // eslint-disable-line
  const dayOfWeek = new Date(today.replace(/-/g, '/')).getDay() // eslint-disable-line
  const index = (dayOfWeek + 6) % 7
  const thisDaysStats = stats.statsByDay[index]
  const {
    thisWeeksTime,
    avgTime,
    bestTime,
    latestDate,
  } = thisDaysStats
  const largestTimeForThisDay = Math.max(thisWeeksTime, avgTime)

  // Chart Bar Heights
  const avgBarHeight = calculateBarHeight(
    avgTime,
    largestTimeForThisDay,
  )

  const bestBarHeight = calculateBarHeight(
    bestTime,
    largestTimeForThisDay,
  )

  const latestBarHeight = calculateBarHeight(
    thisWeeksTime,
    largestTimeForThisDay,
  )

  // Has Stats Variables
  const todaysPuzzleIsSolved = today === latestDate
  const hasStats = !!avgTime
  const hasStreakInfo = !!currentStreak || !!longestStreak

  // Chart Bars
  const avgBar = (
    <div
      className="hub-stats-card__bar avg"
      style={getBarHeight(avgBarHeight)}
    />
  )

  const bestBar = (
    <div
      className="hub-stats-card__bar best"
      style={getBarHeight(bestBarHeight)}
    >
      {todaysPuzzleIsSolved || (
        <span className="hub-stats-card__time">
          {formatAsDuration(bestTime)}
        </span>
      )}
    </div>
  )

  const latestBar = (
    <div
      className="hub-stats-card__bar today"
      style={getBarHeight(latestBarHeight)}
    >
      <span className="hub-stats-card__time">
        {formatAsDuration(thisWeeksTime)}
      </span>
    </div>
  )

  // No Stats Placeholder
  const noStats = (
    <div className="hub-stats-card__no-stats">
      Solve today’s crossword to see some more stats!
    </div>
  )

  // Streak Block
  const streakBlock = (
    <div className="hub-stats-card__streak-block">
      <div className="hub-stats-card__streak-info">
        {currentStreak}
        <div className="hub-stats-card__streak-type">Current</div>
      </div>
      <div className="hub-stats-card__streak-info">
        {longestStreak}
        <div className="hub-stats-card__streak-type">Longest</div>
      </div>
    </div>
  )

  return (
    <article className="hub-stats-card__puzzle-info">
      <a
        className="hub-stats-card__puzzle-info-content js-hub-tracker"
        href="/puzzles/stats"
        data-track-hub={STATS}
      >
        <div className="hub-stats-card__title">My Average</div>
        <div className="hub-stats-card__chart-block">
          <div className="hub-stats-card__bars-block">
            {todaysPuzzleIsSolved && latestBar}
            {hasStats && bestBar}
            {hasStats && avgBar}
            {hasStats || noStats}
          </div>
          <div className="hub-stats-card__day-of-week">
            {hasStats && thisDaysStats.label}
          </div>
        </div>
        <div
          className={
            hasStreakInfo
              ? 'hub-stats-card__streak-title'
              : 'hub-stats-card__title'
          }
        >
          Streaks
        </div>
        {hasStreakInfo ? (
          streakBlock
        ) : (
          <div className="hub-stats-card__no-stats hub-stats-card__no-streaks">
            No current streak
          </div>
        )}
        <div className="hub-stats-card__more-stats">More Stats</div>
      </a>
    </article>
  )
}

StatsCard.propTypes = {
  statsAndStreaks: PT.shape({
    stats: PT.shape({
      statsByDay: PT.arrayOf(
        PT.shape({
          thisWeeksTime: PT.number.isRequired,
          avgTime: PT.number.isRequired,
          bestTime: PT.number.isRequired,
          latestTime: PT.number.isRequired,
          label: PT.string.isRequired,
          latestDate: PT.string.isRequired,
        }),
      ),
    }),
    streaks: PT.shape({
      currentStreak: PT.number.isRequired,
      longestStreak: PT.number.isRequired,
    }),
  }),
}

const mapStateToProps = ({ stats }) => ({
  statsAndStreaks: {
    stats: stats.dailyStats,
    streaks: stats.streaks,
  },
})

export default connect(mapStateToProps)(StatsCard)
