import React, { Component } from 'react'
import propTypes from 'prop-types'

import Tabs from '../Tabs/Tabs'
import { tabsPropTypes } from '../../../sharedPropTypes'

/**
 * USAGE: TabGroup takes an array of tab objects (defined in tabPropTypes)
 *
 * Each tab object contains (at minimum) an identifying key,
 * and the Tab itself (an instance of ButtonTab or LinkTab).
 * This allows us to use TabGroup in multiple places easily, with or without client-side routing.
 *
 * Because of this setup, we only need to import the react-router bundle if we're
 * using TabGroup with LinkTab and client side routing.
 *
 * There are two ways to handle the tab's contents:
 *
 * For simple, non-interactive tab contents, you can simply pass down JSX as a tab prop (tab.content)
 *
 * For more complex, interactive content (to avoid issues with closures and stale props),
 * you can use the onChange function to pass the current tab index back up, and use that
 * to render your own <TabContents />.
 */
class TabGroup extends Component {
  constructor(props) {
    super(props)
    this.state = {
      currentTab: props.initialTab || 0,
      called: [],
    }
    this.updateTab = this.updateTab.bind(this)
  }

  componentDidMount() {
    this.onTabView()
  }

  // make sure that the back button works
  // when TabGroup uses links for tabs
  static getDerivedStateFromProps(props, state) {
    const { param, tabs } = props
    const { currentTab } = state

    if (!param) {
      return null
    }

    if (currentTab.param !== param) {
      const index = tabs.findIndex((tab) => param === tab.param)
      return {
        currentTab: index,
      }
    }

    return null
  }

  shouldComponentUpdate(nextProps, nextState) {
    const { currentTab } = this.state
    return currentTab !== nextState.currentTab
  }

  componentDidUpdate(prevProps, prevState) {
    const { currentTab } = this.state
    if (prevState.currentTab !== currentTab) {
      this.onTabView()
      this.onTabChange()
    }
  }

  onTabView() {
    const { tabs } = this.props
    const { currentTab, called } = this.state
    const handler =
      tabs && tabs[currentTab] && tabs[currentTab].onView

    const isFirstCall = !called.includes(currentTab)

    if (typeof handler === 'function') {
      handler(isFirstCall)

      if (isFirstCall) {
        this.setState((prevState) => ({
          called: [prevState.currentTab, ...prevState.called],
        }))
      }
    }
  }

  onTabChange() {
    const { onChange } = this.props
    const { currentTab } = this.state

    if (typeof onChange === 'function') {
      onChange(currentTab)
    }
  }

  updateTab(newTab) {
    const { currentTab } = this.state

    if (currentTab !== newTab) {
      this.setState({ currentTab: newTab })
    }
  }

  render() {
    const { currentTab } = this.state
    const { tabs } = this.props

    return (
      <Tabs
        currentTab={currentTab}
        updateTab={this.updateTab}
        tabs={tabs}
      />
    )
  }
}

TabGroup.propTypes = {
  onChange: propTypes.func,
  initialTab: propTypes.number,
  tabs: tabsPropTypes.isRequired,
  param: propTypes.string,
}

TabGroup.defaultProps = {
  onChange: () => {},
  initialTab: 0,
  param: '',
}

export default TabGroup
