import localforage from 'localforage'

import logger from '../../../games/shared/utils/clientLogger'
import normalizeCollections from '../utils/normalizeCollections'
import needsProgress from '../utils/needsProgress'

import {
  ANONYMOUS_ID,
  CHANGE_BLOCK_OPACITY,
  CHANGE_PRINT_LAYOUT,
  CHANGE_PRINT_OPTION,
  GET_DEVICE_OS,
  GET_PUZZLE_COLLECTIONS,
  GET_STATS_AND_STREAKS,
  GET_USER_ID,
  LOAD_PRINT_PREFS,
  UPDATE_PUZZLES_PROGRESS,
} from './constants'

export const changePrintOption = (name, value) => ({
  type: CHANGE_PRINT_OPTION,
  payload: { [name]: value },
})

const defaultPrintPrefs = {
  opacity: 100,
  layout: 'standard', // standard | southpaw | large_print
}

export const getLocalUserPreferences = (userId = ANONYMOUS_ID) => {
  return localforage
    .getItem(`${userId}#preferences`)
    .then((preferences) => {
      return preferences.printPrefs ? preferences : defaultPrintPrefs
    })
    .catch(() => {
      // logger.error(err)
      return defaultPrintPrefs
    })
}

export const recallPrintPrefs = () => (dispatch, getState) => {
  const { user } = getState()
  getLocalUserPreferences(user.id).then((preferences) => {
    dispatch({
      type: LOAD_PRINT_PREFS,
      payload: preferences.printPrefs,
    })
  })
}

export const changeBlockOpacity = (opacity) => (dispatch) =>
  dispatch({
    type: CHANGE_BLOCK_OPACITY,
    payload: { opacity },
  })

export const changePrintLayout = (layout) => (dispatch) =>
  dispatch({
    type: CHANGE_PRINT_LAYOUT,
    payload: { layout },
  })

export const getStatsAndStreaks =
  (fetchAllStats) => (dispatch, getState) => {
    return fetchAllStats(getState().user.id)
      .then((statsAndStreaks) => {
        dispatch({
          type: GET_STATS_AND_STREAKS,
          payload: statsAndStreaks,
        })
      })
      .catch((error) => {
        logger.error(`fetch stats and streaks failed: ${error.stack}`)
      })
  }

export const getRecentPuzzles =
  (fetchRecentPuzzles) => (dispatch, getState) => {
    return fetchRecentPuzzles(getState().user.id)
      .then((recentPuzzles) => {
        dispatch({
          type: GET_PUZZLE_COLLECTIONS,
          payload: normalizeCollections({ recentPuzzles }),
        })
      })
      .catch((error) => {
        logger.error(`fetch recent puzzles failed: ${error.stack}`)
      })
  }

export const getPuzzlesProgress =
  (fetchPuzzleProgress) => (dispatch, getState) => {
    const { user, puzzles } = getState()

    /** no need to get progress if user is logged out */
    if (!user.isLoggedIn) return Promise.resolve({})

    const puzzleIds = Object.keys(puzzles).filter(needsProgress)
    return fetchPuzzleProgress(user.id, puzzleIds)
      .then((payload) => {
        dispatch({ type: UPDATE_PUZZLES_PROGRESS, payload })
      })
      .catch((error) => {
        logger.error(
          `fetch hub puzzles progress failed: ${error.stack}`,
        )
      })
  }

export const getUserId = (user) => (dispatch) => {
  user
    .get()
    .then((userObj) => {
      if (!userObj.id) return
      dispatch({
        type: GET_USER_ID,
        payload: userObj.id,
      })
    })
    .catch((err) => console.error('Failed to fetch user', err))
}

export const getDeviceOs = (device) => (dispatch) => {
  device
    .get()
    .then((deviceObj) => {
      if (!deviceObj.os) return
      dispatch({
        type: GET_DEVICE_OS,
        payload: deviceObj.os.family,
      })
    })
    .catch((err) => console.error('Failed to fetch device os', err))
}
