import moment from 'moment'

import {
  formatDate,
  getGoldStarCutoff,
  formatConsecutiveDates,
  getStreakBoundaries,
  getStartOf,
  getEndOf,
} from '../time'

describe('utility: time', () => {
  test('handle normal moment formatting', () => {
    expect(formatDate('2013-10-17', 'M/D/YYYY')).toBe('10/17/2013')
  })

  test('handle new york times month', () => {
    expect(formatDate('2013-09-17', '{nyt-m} YYYY {nyt-m}')).toBe(
      'Sept. 2013 Sept.',
    )
    expect(formatDate('2013-07-17', '{nyt-m} YYYY')).toBe('July 2013')
    expect(formatDate('2013-03-17', '{nyt-m} YYYY')).toBe('Mar. 2013')
  })

  test('throw for bad nyt time format', () => {
    expect(() => {
      formatDate('2013-09-17', '{nyt-m} YYYY {nyt-bad}')
    }).toThrow(new Error('Unrecognized NYT time: "bad"'))
  })

  test('doesnt throw for missing time', () => {
    expect(() => {
      expect(formatDate(undefined, '{nyt-m} YYYY {nyt-bad}')).toBe('')
    }).not.toThrow()
  })

  test('format consecutive dates', () => {
    expect(
      formatConsecutiveDates([
        '2013-09-02',
        '2013-09-01',
        '2013-08-31',
        '2013-08-30',
      ]),
    ).toMatchObject(['{nyt-m} D', 'D', '{nyt-m} D', 'D'])
  })

  test('get gold star cutoff returns PST midnight of publish date', () => {
    const publishDate = '2017-07-19'
    const cutoff = getGoldStarCutoff(publishDate)
    expect(
      moment(cutoff)
        .tz('America/New_York')
        .format('DD h:mma z'),
    ).toBe('20 2:59am EDT')
    expect(
      moment(cutoff)
        .tz('America/Los_Angeles')
        .format('DD h:mma z'),
    ).toBe('19 11:59pm PDT')
  })

  test('get gold star cutoff returns the year 3000 if publish date is bad', () => {
    expect(moment(getGoldStarCutoff(undefined)).year()).toBe(3000)
    expect(moment(getGoldStarCutoff('hello')).year()).toBe(3000)
    expect(moment(getGoldStarCutoff(null)).year()).toBe(3000)
    expect(moment(getGoldStarCutoff({})).year()).toBe(3000)
    expect(moment(getGoldStarCutoff(123)).year()).toBe(3000)
    expect(moment(getGoldStarCutoff('07-23')).year()).toBe(3000)
  })

  test('find streak boundaries', () => {
    const streaks = [
      ['2001-01-01'], // one item streaks should not create false-positives
      ['2002-01-05', '2002-01-10'],
    ]

    const dates = [
      '2002-01-01', // before
      '2002-01-04', // just before
      '2002-01-05', // beginning
      '2002-01-08', // mid-streak
      '2002-01-10', // end
      '2002-01-11', // just after
      '2002-01-12', // after
    ]
    expect(getStreakBoundaries(streaks, dates)).toMatchObject([
      { hasStreakBefore: false, hasStreakAfter: false }, // before
      { hasStreakBefore: false, hasStreakAfter: false }, // just before
      { hasStreakBefore: false, hasStreakAfter: true }, // beginning
      { hasStreakBefore: true, hasStreakAfter: true }, // mid-streak
      { hasStreakBefore: true, hasStreakAfter: false }, // end
      { hasStreakBefore: false, hasStreakAfter: false }, // just after
      { hasStreakBefore: false, hasStreakAfter: false }, // after
    ])
  })

  test('getStartOf - month', () => {
    expect(getStartOf(2013, 0)).toBe('2013-01-01')
  })

  test('getStartOf - year', () => {
    expect(getStartOf(2013)).toBe('2013-01-01')
  })

  test('getEndOf - month', () => {
    expect(getEndOf(2013, 0)).toBe('2013-01-31')
  })

  test('getEndOf - year', () => {
    expect(getEndOf(2013)).toBe('2013-12-31')
  })
})
