import moment from 'moment'

import {
  downloadableFormats,
  playableFormats,
} from '../constants/puzzleTypes'

import { formatDate } from './time'

const pdfGeneratedBase = '/svc/crosswords/v2/puzzle/'
const pdfNewspaperBase = '/svc/crosswords/v2/puzzle/print/'
const pdfSuffixes = { Daily: '' }

/** NOTE: This is technically not enough information to determine this affirmatively,
    e.g. we only started keeping newspaper versions of daily puzzles in April 2001),
    but should be OK for what it's being used for here. But this should be kept front-of-mind
    when we redo the archive pages. */
export const hasNewspaperVersion = ({ publishType }) =>
  typeof pdfSuffixes[publishType] !== 'undefined'

export const isPlayable = ({ formatType }) =>
  playableFormats.includes(formatType)
export const isPrintOnly = (puzzle) => !isPlayable(puzzle)
export const isDownloadable = ({ formatType, publishType }) =>
  downloadableFormats.includes(formatType) && publishType !== 'Mini'

/**
 * create a url for the newspaper version or answers
 */
export const createPDFUrl = (puzzle, forSolution) => {
  const { publishType, printDate } = puzzle

  if (!hasNewspaperVersion(puzzle)) return undefined

  const dateString = formatDate(printDate, 'MMMDDYY')
  const suffix = pdfSuffixes[publishType]
  const extension = forSolution ? '.ans.pdf' : '.pdf'
  return `${pdfNewspaperBase}${dateString}${suffix}${extension}`
}

/**
 * create the url to play the game
 */
export const createPlayUrl = (puzzle) => {
  const { formatType, printDate, publishType } = puzzle

  if (isPrintOnly(puzzle)) return undefined

  const base = '/crosswords/game/'
  const dateFormat =
    publishType === 'Bonus' ? 'YYYY/MM' : 'YYYY/MM/DD'
  const dateString = formatDate(printDate, dateFormat)
  const typeString = publishType.toLowerCase()

  return `${base}${typeString}/${dateString}`
}

export const createGeneratedUrls = (puzzle) => {
  if (!isDownloadable(puzzle)) return {}

  return {
    puzzle: `${pdfGeneratedBase}${puzzle.puzzleId}.pdf`,
    solution: `${pdfGeneratedBase}${puzzle.puzzleId}.ans.pdf`,
  }
}

const solutionHasBeenPublished = ({ printDate }) => {
  const thursdayAt10PM = moment(printDate)
    .tz('America/New_York')
    .add({ days: 4, hours: 22 })
    .format()

  return moment().tz('America/New_York').isSameOrAfter(thursdayAt10PM)
}

const EARLIEST_DAILY_PDF = '2011-04-01'

export const hasNewspaperPuzzle = (puzzle) => {
  const { publishType, printDate } = puzzle
  const date = moment(printDate)
  switch (publishType) {
    default:
    case 'Bonus':
      return false // Bonus is online-only
    case 'Daily':
      return date.isSameOrAfter(EARLIEST_DAILY_PDF)
    case 'Assorted':
      return true
  }
}

export const hasNewspaperSolution = (puzzle) => {
  const { publishType, printDate } = puzzle
  const date = moment(printDate)

  switch (publishType) {
    default:
    case 'Bonus':
      return false // Bonus is online-only
    case 'Daily': // Only Sunday dailies have a separate newspaper solution file
      if (
        date.isBefore(EARLIEST_DAILY_PDF) ||
        date.format('dddd') !== 'Sunday'
      ) {
        return false
      }
      break
    case 'Assorted':
  }

  // Previous week's answers released with latest puzzle
  return solutionHasBeenPublished(puzzle)
}

const buildNewspaperBase = (puzzle) => {
  const { publishType, printDate } = puzzle
  const dateString = formatDate(printDate, 'MMMDDYY')
  const suffix = pdfSuffixes[publishType]
  return `${pdfNewspaperBase}${dateString}${suffix}`
}

export const createNewspaperUrls = (puzzle) => {
  const base = buildNewspaperBase(puzzle)
  return {
    puzzle: hasNewspaperPuzzle(puzzle) ? `${base}.pdf` : undefined,
    solution: hasNewspaperSolution(puzzle)
      ? `${base}.ans.pdf`
      : undefined,
  }
}

const createUrls = (puzzle) => ({
  play: createPlayUrl(puzzle),
  generated: createGeneratedUrls(puzzle),
  newspaper: createNewspaperUrls(puzzle),
})

export default createUrls
