import moment from 'moment'
import 'moment-timezone'

export const formatTime = (time) => {
  const h = Math.floor(time / 3600)
  const m = Math.floor((time % 3600) / 60)
  const mm = m < 10 ? `0${m}` : m
  const s = Math.floor(time % 60)
  const ss = s < 10 ? `0${s}` : s
  if (h > 0) return [h, mm, ss].join(':')

  return `${m}:${ss}`
}

const NYT_TOKEN_RE = /(\w*)\{nyt-(\w+)\}(\w*)/g
const NYT_TIME_FORMATS = {
  m: (timestamp) => {
    const fullMonth = timestamp.format('MMMM')
    const threeLetterMonth =
      fullMonth === 'September' ? 'Sept' : fullMonth.substring(0, 3)

    return fullMonth.length > 4 ? `${threeLetterMonth}.` : fullMonth
  },
}

const getNYTTimeTokens = (timestamp, fmt) =>
  fmt.replace(NYT_TOKEN_RE, (a, b, nytFormat) => {
    const formatter = NYT_TIME_FORMATS[nytFormat]
    if (!formatter) {
      throw new Error(`Unrecognized NYT time: "${nytFormat}"`)
    }

    return `[${formatter(timestamp)}]`
  })

export const formatDate = (date, fmt) => {
  if (!date) return ''
  const timestamp = moment(date)
  const specialFormat = getNYTTimeTokens(timestamp, fmt)
  return timestamp.format(specialFormat)
}

const inDifferentMonths = (curr, prev) =>
  !prev || moment(curr).format('MM') !== moment(prev).format('MM')

export const formatConsecutiveDates = (dates) =>
  dates.map((date, idx) =>
    inDifferentMonths(date, dates[idx - 1]) ? '{nyt-m} D' : 'D',
  )

const hasWithin = (date) => ([start, end]) =>
  date.isBetween(start, end, 'day', '[]')

export const getStreakLines = (streaks, d) => {
  const goodStreaks = streaks.filter((v) => v.length > 1)
  const date = moment(d)
  const dayBefore = date.clone().subtract(1, 'day')
  const dayAfter = date.clone().add(1, 'day')
  const isGold = goodStreaks.some(hasWithin(date))
  return {
    hasStreakBefore: isGold && goodStreaks.some(hasWithin(dayBefore)),
    hasStreakAfter: isGold && goodStreaks.some(hasWithin(dayAfter)),
  }
}

export const getStreakBoundaries = (streaks, dates) =>
  dates.map((d) => getStreakLines(streaks, d))

export const getElapsedTime = (
  totalElapsedTime,
  sessionStartTimestamp,
) => {
  const timeDiff = sessionStartTimestamp
    ? moment().unix() - sessionStartTimestamp
    : 0

  return totalElapsedTime + timeDiff
}

export const formatAsDuration = (totalTime) => {
  // 2 -> 02; 10 -> 10
  const zeroPad = (unpadded) => {
    const pad = unpadded < 10 ? '0' : ''
    return pad + unpadded
  }

  if (!totalTime) return ''
  const hours = Math.floor(totalTime / 3600)
  const minutes = Math.floor((totalTime / 60) % 60)
  const seconds = totalTime % 60
  let timePieces = ''

  if (hours) timePieces = [hours, zeroPad(minutes), zeroPad(seconds)]
  else timePieces = [minutes, zeroPad(seconds)]

  return timePieces.join(':')
}

const VALID_PUBLISH_DATE = /\d{4}-\d{2}-\d{2}/
const isValidPublishDate = (publishDate) =>
  typeof publishDate === 'string' &&
  VALID_PUBLISH_DATE.exec(publishDate)

/*
 * This is mostly broken out so it can be mocked
 * in unit testing without mocking moment entirely
 */
export const getMomentNow = () => moment()

export const getGoldStarCutoff = (publishDate) =>
  isValidPublishDate(publishDate)
    ? moment
        .tz(publishDate, 'America/Los_Angeles')
        .endOf('day')
        .format()
    : moment({ year: 3000 })

export const getStartOf = (year, month) => {
  const useMonth = typeof month !== 'undefined'
  if (useMonth) {
    return formatDate(
      moment()
        .year(year)
        .month(month)
        .startOf('month'),
      'YYYY-MM-DD',
    )
  }

  return formatDate(
    moment()
      .year(year)
      .startOf('year'),
    'YYYY-MM-DD',
  )
}

export const getEndOf = (year, month) => {
  const useMonth = typeof month !== 'undefined'
  if (useMonth) {
    return formatDate(
      moment()
        .year(year)
        .month(month)
        .endOf('month'),
      'YYYY-MM-DD',
    )
  }

  return formatDate(
    moment()
      .year(year)
      .endOf('year'),
    'YYYY-MM-DD',
  )
}
