/**
 * withPuzzleCollection()
 * is a higher order component to help make components that
 * accept a collection key prop and recieve puzzles from redux
 * the reason for the complexity is that using reselect
 * with props, across multiple components is tricky!
 * see here: https://github.com/reactjs/reselect#sharing-selectors-with-props-across-multiple-components
 *
 * Example use:
 * const MyComponent = ({ puzzles }) => <div>{JSON.stringify(puzzles)}</div>
 * const Connected = withPuzzleCollection(MyComponent)
 * <Connected puzzleCollection="freePuzzles" />
 */
import { connect } from 'react-redux'
import { createSelector } from 'reselect'

import { getPuzzles } from './store/selectors'

/** must be generated because it depends on props */
const getCollection = (state, props) =>
  state.collections[props.puzzleCollection]

export default (Component) =>
  connect(() => {
    /** we must make the selector for each instance */
    const getCollectionPuzzles = createSelector(
      [getCollection, getPuzzles],
      (collection, puzzles) =>
        collection && collection.map((id) => puzzles[id]),
    )

    /** we must make the mapper for each instance */
    return (state, props) => {
      const puzzles = getCollectionPuzzles(state, props)
      /**
       * In order to distinguish between a puzzle list that is still
       * being fetched, and one that is actually empty,
       * we pass an 'isPending' boolean
       * this helps with cases like recent progress
       * where we fetch it async, but if it comes back empty
       * we prompt the user to do something.
       */
      return {
        isPending: !puzzles,
        puzzles: puzzles || [],
      }
    }
  })(Component)
