import React from 'react'
import { mount } from 'enzyme'
import ClipboardJS from 'clipboard'

import { trackEvent } from '../track/track'

import CopyToClipboard from './CopyToClipboard'

jest.mock('clipboard', () => jest.fn())

jest.mock('../track/track', () => ({
  trackEvent: jest.fn(),
}))

const mostRecentClipboardCallText = () => {
  const numOfCalls = ClipboardJS.mock.calls.length
  const options = ClipboardJS.mock.calls[numOfCalls - 1][1]
  return options.text()
}

describe('LEADERBOARD: BUTTON / COPY TO CLIPBOARD', () => {
  let onCopy
  let onDestroy

  beforeEach(() => {
    onCopy = jest.fn()
    onDestroy = jest.fn()

    function FakeClipboard() {
      this.on = onCopy
      this.destroy = onDestroy
    }

    ClipboardJS.mockImplementation(FakeClipboard)
  })

  it('implements clipboard correctly', () => {
    const copyButton = mount(
      <CopyToClipboard content="copy this">Copy</CopyToClipboard>,
    )
    const [ref, options] = ClipboardJS.mock.calls[0]

    expect(copyButton.getDOMNode()).toBe(ref)
    expect(options.text()).toBe('copy this')
    expect(onCopy.mock.calls[0][0]).toBe('success')
  })

  it('updates button text on copy', () => {
    const copyButton = mount(
      <CopyToClipboard content="copy this" successText="After Copy">
        Before Copy
      </CopyToClipboard>,
    )
    const callback = onCopy.mock.calls[0][1]
    expect(copyButton.text()).toBe('Before Copy')

    callback()
    expect(trackEvent).toHaveBeenCalledWith('INVITE_COPY')
    expect(copyButton.text()).toBe('After Copy')
  })

  it('resets button if content changes', () => {
    const copyButton = mount(
      <CopyToClipboard content="copy this" successText="After Copy">
        Before Copy
      </CopyToClipboard>,
    )

    const callback = onCopy.mock.calls[0][1]
    expect(copyButton.text()).toBe('Before Copy')

    callback()
    expect(copyButton.text()).toBe('After Copy')

    copyButton.setProps({ content: 'copy this' }) // rerender with same prop
    expect(copyButton.text()).toBe('After Copy')

    copyButton.setProps({ content: 'new stuff' }) // rerender with different prop
    expect(copyButton.text()).toBe('Before Copy')
  })

  it('resets copied text if content changes', () => {
    const copyButton = mount(
      <CopyToClipboard content="copy version 1">
        Copy
      </CopyToClipboard>,
    )
    expect(mostRecentClipboardCallText()).toBe('copy version 1')

    copyButton.setProps({ content: 'copy version 2' })
    expect(mostRecentClipboardCallText()).toBe('copy version 2')
  })

  it('cleans up', () => {
    const copyButton = mount(
      <CopyToClipboard content="copy this" successText="After Copy">
        Before Copy
      </CopyToClipboard>,
    )
    copyButton.unmount()
    expect(onDestroy).toHaveBeenCalled()
  })
})
