import React from 'react'
import { shallow, mount } from 'enzyme'

import APIContext from '../api/APIContext'
import Button from '../Button/Button'
import AsyncButton from '../Button/AsyncButton'
import CopyToClipboard from '../Button/CopyToClipboard'

import InviteContainer, { StepOne } from './StepOne'

jest.mock('../Button/Button', () => () => '')
jest.mock('../Button/CopyToClipboard', () => () => '')
jest.mock('../Button/AsyncButton', () => () => '')
jest.mock('../api/APIContext', () => ({
  Consumer: () => '',
}))

describe('LEADERBOARD: INVITE', () => {
  it('consumes api', () => {
    const container = shallow(<InviteContainer shareURL="share" />)

    const render = container
      .find(APIContext.Consumer)
      .prop('children')
    const apiMethod = jest.fn()
    const rendered = render({ apiMethod })

    expect(rendered.props.apiMethod).toBe(apiMethod)
    expect(rendered.props.shareURL).toBe('share')
  })

  it('should render shareURL', () => {
    const share = 'share-link'
    const invite = shallow(<StepOne shareURL={share} />)
    const link = invite.find('.lbd-invite__link')

    expect(link.prop('value')).toBe(share)
  })

  it('should render email button', () => {
    const share = 'share-link'
    const invite = shallow(<StepOne shareURL={share} />)
    const button = invite.find(Button).at(0)
    const email = button.prop('href')
    expect(email).toMatch(/^mailto:/)
    expect(email).toContain(share)
    expect(button.prop('track')).toBe('INVITE_EMAIL')
    expect(email).toMatchSnapshot()
  })

  it('should copy to clipboard', () => {
    const share = 'share-link'
    const invite = shallow(<StepOne shareURL={share} />)
    const button = invite.find(CopyToClipboard)

    expect(button.prop('content')).toBe(share)
  })

  it('should regenerate share links', () => {
    const regenerateLink = jest.fn()

    const invite = mount(
      <StepOne
        shareURL="share-link"
        regenerateLink={regenerateLink}
      />,
    )
    const modalLink = invite.find('.lbd-regenerate-button')

    modalLink.simulate('click')

    const confirmButton = invite.find('.lbd-modals').find(AsyncButton)

    expect(confirmButton.prop('track')).toBe('INVITE_REGENERATE')

    confirmButton.prop('action')()

    expect(regenerateLink).toHaveBeenCalled()
    expect(invite.find('.lbd-modals').exists()).toBe(true)
    expect(invite.find('.lbd-invite__link').prop('value')).toBe(
      'share-link',
    )

    confirmButton.prop('onThen')('share-link-after')

    invite.update()

    expect(invite.find('.lbd-invite__link').prop('value')).toBe(
      'share-link-after',
    )
  })

  it('should show error modal when regeneration fails', () => {
    const regenerateLink = jest.fn()

    const share = 'share-link'
    const invite = mount(
      <StepOne shareURL={share} regenerateLink={regenerateLink} />,
    )
    const modalLink = invite.find('.lbd-regenerate-button')

    modalLink.simulate('click')

    const confirmButton = invite.find('.lbd-modals').find(AsyncButton)

    confirmButton.prop('onCatch')(new Error('Error time'))

    invite.update()

    expect(
      invite.find('.lbd-modals .lbd-type__heading-sm').text(),
    ).toBe(`We couldn’t create your link`)
  })
})
