import React from 'react'
import { shallow, mount } from 'enzyme'

import APIContext from '../api/APIContext'
import AsyncButton from '../Button/AsyncButton'
import ErrorModal from '../Modals/ErrorModal'

import StepTwoContainer, { StepTwo } from './StepTwo'

/* eslint-disable */
jest.mock('../Button/AsyncButton', () => () => '')
jest.mock('../Modals/ErrorModal', () => ({ children }) => (
  <div>{children}</div>
))
jest.mock('../api/APIContext', () => ({
  Consumer: () => '',
}))
jest.mock('../api/handleAPIError', () => ({
  getModalError: (e) => ['modal error', e.message],
}))
/* eslint-enable */

describe('LEADERBOARD: STEP TWO', () => {
  it('consumes api', () => {
    const container = shallow(
      <StepTwoContainer name="abc" shareCode="share" />,
    )

    const render = container
      .find(APIContext.Consumer)
      .prop('children')
    const apiMethod = jest.fn()
    const rendered = render({ apiMethod })

    expect(rendered.props.apiMethod).toBe(apiMethod)
    expect(rendered.props.shareCode).toBe('share')
    expect(rendered.props.name).toBe('abc')
  })

  it('should show inviter display name', () => {
    const invite = shallow(
      <StepTwo name="user" shareCode="00-00-00" />,
    )
    expect(
      invite.find('.lbd-invite__two .lbd-type__heading-md').text(),
    ).toBe('Add user to your leaderboard.')
    expect(
      invite.find('.lbd-type__heading-md + .lbd-type__text').text(),
    ).toMatch(
      'You’ll see user’s Mini Crossword times on your leaderboard. User will see your times as well.',
    )
    expect(
      invite.find('.lbd-invite__two .lbd-type__blue').text(),
    ).toBe('user')
  })

  it('should request to join', () => {
    expect.assertions(9)

    const requestToJoin = jest.fn()
    const shareCode = '0000000'

    requestToJoin.mockReturnValue(Promise.resolve(''))

    const invite = mount(
      <StepTwo
        name="user"
        shareCode={shareCode}
        requestToJoin={requestToJoin}
      />,
    )
    const button = invite.find(AsyncButton).at(0)

    expect(button.prop('pending')).toBe('Sending...')
    expect(button.prop('children')).toBe('Add')
    expect(button.hasClass('checked')).toEqual(false)
    expect(invite.find('.lbd-invite__redirect').exists()).toBe(false)

    const clickHandler = button.prop('action')
    const p = clickHandler()

    invite.update()

    expect(requestToJoin).toHaveBeenCalledWith(shareCode)

    return p.then(() => {
      button.prop('onThen')()
      invite.update()

      const buttonAfter = invite.find(AsyncButton).at(0)

      expect(buttonAfter.prop('children')).toBe('Added')
      expect(buttonAfter.hasClass('checked')).toEqual(true)
      expect(invite.find('.lbd-invite__redirect').text()).toBe(
        'Check out your friend on your leaderboard!',
      )
      expect(
        invite.find('.lbd-invite__redirect a').prop('href'),
      ).toBe('/puzzles/leaderboards')
    })
  })

  it('should request to join only once', async () => {
    expect.assertions(1)

    const requestToJoin = jest.fn()
    const shareCode = '0000000'

    requestToJoin.mockReturnValue(Promise.resolve(''))

    const invite = mount(
      <StepTwo
        name="user"
        shareCode={shareCode}
        requestToJoin={requestToJoin}
      />,
    )
    const button = invite.find(AsyncButton).at(0)

    const clickHandler = button.prop('action')

    await clickHandler()

    invite.update()
    button.prop('onThen')()
    invite.update()

    const buttonAfter = invite.find(AsyncButton).at(0)
    const clickAfter = buttonAfter.prop('action')

    await clickAfter()

    expect(requestToJoin).toHaveBeenCalledTimes(1)
  })

  it('should open modal on error', () => {
    const testError = new Error('error body')
    const invite = shallow(
      <StepTwo
        name="user"
        shareCode="00-00-00"
        requestToJoin={() => {}}
      />,
    )
    const button = invite.find(AsyncButton).at(0)
    const errorHandler = button.prop('onCatch')

    errorHandler(testError)
    invite.update()

    const modal = invite.find(ErrorModal)

    expect(modal.prop('isOpen')).toBe(true)
    expect(modal.prop('error')).toBe(testError)

    modal.prop('onClose')()
    invite.update()

    const modalAfter = invite.find(ErrorModal)

    expect(modalAfter.prop('isOpen')).toBe(false)
    expect(modalAfter.prop('error')).toBe(null)
  })
})
