import React from 'react'
import { shallow } from 'enzyme'

import LeaderboardItem from '../LeaderboardItem/LeaderboardItem'
import Button from '../Button/Button'
import PendingRequests from '../PendingRequests/PendingRequests'

import Leaderboard from './Leaderboard'

jest.mock(
  '../LeaderboardItem/LeaderboardItem',
  () =>
    function Fake() {
      return <div />
    },
)
jest.mock(
  '../Button/Button',
  () =>
    function Fake() {
      return <button />
    },
)
jest.mock(
  '../PendingRequests/PendingRequests',
  () =>
    function Fake() {
      return <div />
    },
)

describe('LEADERBOARD: BOARD', () => {
  it('renders header', () => {
    const props = {
      displayDate: 'date',
      scoreList: [{}, {}],
    }

    const board = shallow(<Leaderboard {...props} />)
    const header = board.find('.lbd-board__header')
    const subhead = header.find('.lbd-type__subhead')
    const heading = header.find('.lbd-type__heading')
    const date = header.find('.lbd-type__date')

    expect(subhead.text()).not.toBeFalsy()
    expect(heading.text()).not.toBeFalsy()
    expect(date.text()).toBe(props.displayDate)
  })

  it('renders leaderboard items', () => {
    const score0 = { foo: 1, bar: 2 }
    const score1 = { foo: 2, bar: 1 }

    const props = {
      displayDate: 'date',
      puzzleLink: 'link',
      scoreList: [score0, score1],
    }

    const board = shallow(<Leaderboard {...props} />)
    const itemWrapper = board.find('.lbd-board__items')
    const items = itemWrapper.find(LeaderboardItem)

    expect(items.at(0).props()).toEqual({
      ...score0,
      puzzleLink: props.puzzleLink,
    })
    expect(items.at(1).props()).toEqual({
      ...score1,
      puzzleLink: props.puzzleLink,
    })
  })

  it('renders actions', () => {
    const props = {
      displayDate: 'date',
      scoreList: [{}, {}],
    }

    const board = shallow(<Leaderboard {...props} />)
    const actionWrapper = board.find('.lbd-board__actions')
    const actions = actionWrapper.find(Button)
    const manage = actions.at(0)
    const add = actions.at(1)

    expect(manage.prop('children')).toBe('Settings')
    expect(manage.prop('type')).toBe('white')
    expect(manage.prop('href')).toBe('/puzzles/leaderboards/settings')
    expect(manage.prop('track')).toBe('LEADERBOARD_TO_FRIENDS')

    expect(add.prop('children')).toBe('Add Friends')
    expect(add.prop('type')).toBe('black')
    expect(add.prop('href')).toBe('/puzzles/leaderboards/invite')
    expect(add.prop('track')).toBe('LEADERBOARD_TO_INVITE')
  })

  it('renders tip if user has no friends', () => {
    const props = {
      displayDate: 'date',
      scoreList: [{}],
    }

    const board = shallow(<Leaderboard {...props} />)

    expect(board.find('.lbd-board__tip')).toHaveLength(1)
  })

  it('does not render tip if user has friends', () => {
    const props = {
      displayDate: 'date',
      scoreList: [{}, {}],
      pendingRequests: 0,
    }

    const board = shallow(<Leaderboard {...props} />)

    expect(board.find('.lbd-board__tip')).toHaveLength(0)
  })

  it('does not render tip if user no friends, but does have pending connections', () => {
    const props = {
      displayDate: 'date',
      scoreList: [{}],
      pendingRequests: 2,
    }

    const board = shallow(<Leaderboard {...props} />)

    expect(board.find('.lbd-board__tip')).toHaveLength(0)
  })

  it('renders pending requests if there are any', () => {
    const props = {
      displayDate: 'date',
      scoreList: [{}],
      pendingRequests: 2,
    }

    const board = shallow(<Leaderboard {...props} />)

    expect(board.find(PendingRequests)).toHaveLength(1)
    expect(board.find(PendingRequests).prop('pendingRequests')).toBe(
      2,
    )
  })

  it('does not render pending requests if there are none', () => {
    const props = {
      displayDate: 'date',
      scoreList: [{}],
      pendingRequests: 0,
    }

    const board = shallow(<Leaderboard {...props} />)

    expect(board.find(PendingRequests)).toHaveLength(0)
  })
})
