import React from 'react'
import { Provider } from 'react-redux'
import { mount, shallow } from 'enzyme'

import { trackEvent } from '../../../track/track'
import Pending, { PendingFriend } from '../Pending'
import { PENDING_STATES } from '../../../Store/constants'

jest.mock('../../../track/track', () => ({
  trackEvent: jest.fn(),
}))

const renderConnection = (props) => {
  const store = {
    getState: () => {},
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  }

  return mount(
    <Provider store={store}>
      <Pending {...props} />
    </Provider>,
  )
}

describe('LEADERBOARD: FRIENDS LIST / PENDING', () => {
  it('renders a name', () => {
    const connection = renderConnection({
      name: 'foo',
      userID: 1,
    })

    const name = connection.find('.lbd-friend__label').text()

    expect(name).toBe('foo')
  })

  it('shows awaiting message if pending state is awaiting', () => {
    const connection = renderConnection({
      name: 'foo',
      userID: 1,
      pendingState: PENDING_STATES.AWAITING,
    })

    const stateText = connection.find('.lbd-pending__action').text()

    expect(stateText).toBe('(Awaiting their confirmation)')
  })

  it('shows confirming message if pending state is confirming', () => {
    const connection = renderConnection({
      name: 'foo',
      userID: 1,
      pendingState: PENDING_STATES.CONFIRMING,
    })

    const stateText = connection.find('.lbd-pending__action').text()

    expect(stateText).toBe('Confirming...')
  })

  it('shows confirmed message if pending state is confirmed', () => {
    const connection = renderConnection({
      name: 'foo',
      userID: 1,
      pendingState: PENDING_STATES.CONFIRMED,
    })

    const stateText = connection.find('.lbd-pending__action').text()

    expect(stateText).toBe('Confirmed')
  })

  it('shows pending buttons if pending state is pending', () => {
    const connection = renderConnection({
      name: 'foo',
      userID: 1,
      pendingState: PENDING_STATES.PENDING,
    })

    const action = connection.find('.lbd-friend__action')

    expect(action).toHaveLength(2)
    expect(action.first().text()).toBe('Reject')
    expect(action.last().text()).toBe('Confirm')
  })

  it('fires confirm or reject upon pending button click', () => {
    let rejected = false

    const props = {
      name: 'foo',
      userID: 1,
      pendingState: PENDING_STATES.PENDING,
      confirmPending: jest.fn().mockReturnValue(Promise.resolve()),
      blockFriend: jest.fn(),
      confirm: () => {
        rejected = true
      },
    }

    const connection = shallow(<PendingFriend {...props} />)

    const action = connection.find('.lbd-friend__action')

    action.at(0).simulate('click')
    action.at(1).simulate('click')

    expect(trackEvent).toHaveBeenCalledWith('FRIEND_CONFIRM')
    expect(rejected).toBe(true)
    expect(props.confirmPending).toHaveBeenCalledWith(props.userID)
  })
})
