import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import Self from '../Self'

jest.mock('../../../Store/actions', () => ({
  editSelf: jest
    .fn()
    .mockImplementation((newName) =>
      Promise.resolve([newName, 'submit']),
    ),
  editSelfEnter: jest.fn().mockReturnValue('editSelfEnter'),
  editSelfLeave: jest.fn().mockReturnValue('editSelfLeave'),
  editSelfUpdate: jest
    .fn()
    .mockImplementation((newName) => [newName, 'update']),
}))

jest.mock('../../../api/handleAPIError', () => ({
  getNameError: (e) => `getNameError(${e.message})`,
}))

describe('LEADERBOARD: FRIENDS LIST / SELF', () => {
  let dispatch
  let renderSelf
  let state
  let store

  beforeEach(() => {
    renderSelf = (selfState) => {
      dispatch = jest.fn().mockImplementation((_) => _)

      state = {
        connections: {
          self: [selfState],
        },
      }

      store = {
        getState: () => state,
        subscribe: jest.fn(),
        dispatch,
      }

      return mount(
        <Provider store={store}>
          <Self />
        </Provider>,
      )
    }
  })

  it('starts with a name and edit button', () => {
    const self = renderSelf({
      name: 'Greg',
      isEditing: false,
    })

    expect(self.find('.lbd-friend__label').text()).toBe('Greg')
    expect(self.find('.lbd-friend__action').text()).toBe(
      'Edit Display Name',
    )

    self.find('.lbd-friend__action').simulate('click')

    expect(dispatch).toHaveBeenCalledWith('editSelfEnter')
  })

  it('cancels editing', () => {
    const self = renderSelf({
      name: 'Before',
      pendingName: 'Greg',
      isEditing: true,
    })

    const cancel = self.find('.lbd-friend__action').at(0)

    cancel.simulate('click')

    expect(dispatch).toHaveBeenCalledWith('editSelfLeave')
  })

  it('displays a pending name and actions while editing', () => {
    const self = renderSelf({
      name: 'Greg',
      pendingName: 'New Name',
      isEditing: true,
    })

    const input = self.find('input.lbd-friend__label.name-input')
    const buttons = self.find('.lbd-friend__action')

    expect(input.prop('value')).toBe('New Name')
    expect(buttons.at(0).text()).toBe('Cancel')
    expect(buttons.at(1).text()).toBe('Save')

    input.simulate('change', { target: { value: 'abc' } })

    expect(dispatch).toHaveBeenCalledWith(['abc', 'update'])
  })

  it('submits on enterKey', async () => {
    const self = renderSelf({
      name: 'Greg',
      pendingName: 'New Name',
      isEditing: true,
    })

    const input = self.find('input.lbd-friend__label.name-input')

    input.simulate('keyup', { key: 'A' })

    expect(dispatch).not.toHaveBeenCalled()

    input.simulate('keyup', { key: 'Enter' })

    const result = await dispatch.mock.calls[0][0]

    expect(result).toEqual(['New Name', 'submit'])
  })

  it('submits on Save', async () => {
    const self = renderSelf({
      name: 'Greg',
      pendingName: 'New Name',
      isEditing: true,
    })

    const button = self.find('.lbd-friend__action').at(1)

    button.simulate('click')

    const result = await dispatch.mock.calls[0][0]

    expect(result).toEqual(['New Name', 'submit'])
  })

  it('displays error', () => {
    const self = renderSelf({
      name: 'Greg',
      pendingName: 'New Name',
      isEditing: true,
      error: { message: 'invalid name' },
    })

    const error = self.find('.lbd-friend__error')

    expect(error.text()).toBe('Error: getNameError(invalid name)')
  })

  it('renders pending state', () => {
    const self = renderSelf({
      name: 'Greg',
      pendingName: 'New Name',
      isEditing: true,
      isSubmitting: true,
    })

    const input = self.find('input.lbd-friend__label.name-input')
    const buttons = self.find('.lbd-friend__action')
    const pendingMessage = self.find('.lbd-pending__action')

    expect(input.prop('disabled')).toBe(true)
    expect(buttons).toHaveLength(0)
    expect(pendingMessage.text()).toBe('Saving...')
  })
})
