import React from 'react'
import { Provider } from 'react-redux'
import { mount } from 'enzyme'

import { trackEvent } from '../track/track'
import ErrorModal from '../Modals/ErrorModal'
import ActionModal from '../Modals/ActionModal'

import FriendsList from './FriendsList'
import Connection from './Connection/index'

jest.mock('./Connection/index', () => () => '')
jest.mock('../Modals/ErrorModal', () => () => '')
jest.mock('../Modals/ActionModal', () => () => '')
jest.mock('../track/track', () => ({
  trackEvent: jest.fn(),
}))

const renderList = (props, state) => {
  const store = {
    getState: () => state,
    subscribe: jest.fn(),
    dispatch: jest.fn(),
  }

  return mount(
    <Provider store={store}>
      <FriendsList {...props} />
    </Provider>,
  )
}

describe('LEADERBOARD: MANAGE FRIENDS / FRIENDS LIST', () => {
  it('renders a title', () => {
    const state = {
      connections: {
        foo: [{ userID: 1 }, { userID: 2 }, { userID: 3 }],
      },
    }

    const list = renderList(
      {
        type: 'foo',
        title: 'list title',
      },
      state,
    )

    const title = list.find('.lbd-friends__title').text()

    expect(title).toBe('list title')
  })

  it('renders connections by type', () => {
    const state = {
      connections: {
        foo: [{ userID: 1 }, { userID: 2 }, { userID: 3 }],
        bar: [{ userID: 4 }, { userID: 5 }],
      },
    }

    const fooList = renderList({ type: 'foo' }, state)
    const barList = renderList({ type: 'bar' }, state)

    expect(fooList.find(Connection)).toHaveLength(3)
    expect(barList.find(Connection)).toHaveLength(2)
  })

  it('passes correct props to connections', () => {
    const state = {
      connections: {
        foo: [{}, {}, { userID: 30, name: 'Sara', extra: true }],
      },
    }

    const list = renderList({ type: 'foo' }, state)

    const sara = list.find(Connection).at(2)

    expect(sara.props()).toEqual({
      type: 'foo',
      userID: 30,
      name: 'Sara',
      extra: true,
      confirm: expect.any(Function),
      showError: expect.any(Function),
    })
  })

  it('renders an empty state', () => {
    const state = {
      connections: { foo: [], bar: [{ userID: 1 }] },
    }

    const emptyMessageSelector = '.lbd-friend.empty'

    const list = renderList(
      {
        type: 'foo',
        empty: 'Nothing here!',
      },
      state,
    )

    expect(list.find(Connection)).toHaveLength(0)
    expect(list.find(emptyMessageSelector).text()).toBe(
      'Nothing here!',
    )

    const notEmpty = renderList(
      {
        type: 'bar',
        empty: 'Nothing here!',
      },
      state,
    )

    expect(notEmpty.find(emptyMessageSelector).exists()).toBe(false)
  })

  it('supports a toggle', () => {
    const state = {
      connections: { foo: [], bar: [{ userID: 1 }] },
    }

    const toggleSelector = '.lbd-friend__action.toggle'

    const list = renderList(
      {
        type: 'bar',
        toggle: 'Toggled',
      },
      state,
    )

    expect(list.find(Connection)).toHaveLength(0)
    expect(list.find(toggleSelector).text()).toBe('View Toggled')
    expect(list.find(toggleSelector).hasClass('active')).toBe(false)

    list.find(toggleSelector).simulate('click')

    expect(list.find(Connection)).toHaveLength(1)
    expect(list.find(toggleSelector).text()).toBe('Hide Toggled')
    expect(list.find(toggleSelector).hasClass('active')).toBe(true)

    const emptyList = renderList(
      {
        type: 'foo',
        toggle: 'Toggled',
      },
      state,
    )

    expect(emptyList.find(toggleSelector).exists()).toBe(false)
    expect(emptyList.find(Connection)).toHaveLength(0)
  })

  it('supports confirming', async () => {
    const state = {
      connections: { bar: [{ userID: 1 }] },
    }

    const list = renderList({ type: 'bar' }, state)
    const confirm = list
      .find(Connection)
      .at(0)
      .prop('confirm')

    const modalBefore = list.find(ActionModal).props()

    expect(modalBefore.isOpen).toBe(false)

    confirm({
      foo: true,
      track: 'eventName',
    })

    await list.update()

    const modalAfter = list.find(ActionModal).props()

    expect(modalAfter.isOpen).toBe(true)
    expect(modalAfter.foo).toBe(true)
    expect(trackEvent).toHaveBeenCalledWith('eventName')

    modalAfter.onClose()
    await list.update()

    const modalFinal = list.find(ActionModal).props()

    expect(modalFinal.isOpen).toBe(false)
    expect(modalFinal.foo).toBe(undefined)
  })

  it('supports showing an error', async () => {
    const err = new Error('uh oh')

    const state = {
      connections: { bar: [{ userID: 1 }] },
    }

    const list = renderList({ type: 'bar' }, state)
    const confirm = list
      .find(Connection)
      .at(0)
      .prop('confirm')

    confirm({ foo: true })

    const actionModal = list.find(ActionModal).props()

    actionModal.onFail(err)
    await list.update()

    const modalFinal = list.find(ActionModal).props()
    const errorModal = list.find(ErrorModal).props()

    expect(modalFinal.isOpen).toBe(false)
    expect(modalFinal.foo).toBe(undefined)

    expect(errorModal.error).toBe(err)
    expect(errorModal.isOpen).toBe(true)

    errorModal.onClose()
    await list.update()

    const afterErrorModal = list.find(ErrorModal).props()

    expect(afterErrorModal.error).toBe(undefined)
    expect(afterErrorModal.isOpen).toBe(false)
  })
})
