import React from 'react'
import pt from 'prop-types'
import cn from 'classnames'

import Button from '../Button/Button'
import APIContext from '../api/APIContext'
import { getNameError } from '../api/handleAPIError'
import { trackEvent } from '../track/track'

class SetupProfile extends React.Component {
  constructor(props) {
    super(props)

    this.state = {
      name: '',
      disabled: true,
      error: '',
    }

    this.name = React.createRef()
  }

  componentDidMount = () => {
    const { getSuggestedName } = this.props

    getSuggestedName().then((name) => {
      this.setState({
        ...name,
        disabled: false,
      })
    })
  }

  onChange = (e) => {
    this.setState({
      name: e.target.value,
      error: '',
      disabled: !e.target.value,
    })
  }

  inputDelete = () => {
    this.setState({
      name: '',
    })

    this.name.current.focus()
  }

  submit = () => {
    const { name } = this.state
    const { createProfile } = this.props

    if (!name) return

    this.setState({ disabled: true })

    trackEvent('ONBOARDING_NAME')

    return createProfile(name)
      .then(() => {
        if (window.location.pathname.match(/leaderboards[/]?$/)) {
          window.location.assign(
            '/puzzles/leaderboards/invite?onboarding=true',
          )
        } else {
          window.location.reload()
        }
      })
      .catch((error) => {
        this.setState({
          error: getNameError(error),
        })
      })
  }

  submitKey = (e) => {
    if (e.key === 'Enter') {
      this.submit()
    }
  }

  render() {
    const { name, disabled, error } = this.state
    const { titled } = this.props

    return (
      <div
        className={cn('lbd-profile', {
          'has-error': !!error,
        })}
      >
        {titled ? (
          <h2 className="lbd-profile__title lbd-type__heading-md">
            Enter a name:
          </h2>
        ) : (
          <h3 className="lbd-type__text">
            <strong>Enter a name:</strong>
          </h3>
        )}
        <div className="lbd-type__input-wrapper">
          <input
            className="lbd-type__input"
            type="text"
            id="name"
            placeholder="Your name or other handle"
            value={name}
            onChange={this.onChange}
            onKeyUp={this.submitKey}
            autoFocus
            ref={this.name}
          />
          {name.length ? (
            <button
              type="button"
              className="lbd-type__input-delete"
              onClick={this.inputDelete}
            />
          ) : (
            ''
          )}
        </div>
        <p className="lbd-type__error">
          Error:
          {` `}
          {error}
        </p>
        <p className={cn('lbd-type__text', 'lbd-profile__explain')}>
          This is the name your friends will see on the leaderboard.
          You can always change it later.
        </p>
        <div className="lbd-actions__block">
          <Button
            type="black"
            isDisabled={disabled || !name}
            action={this.submit}
          >
            Next
          </Button>
        </div>
      </div>
    )
  }
}

SetupProfile.propTypes = {
  titled: pt.bool,
  createProfile: pt.func,
  getSuggestedName: pt.func,
}

SetupProfile.defaultProps = {
  titled: false,
  createProfile: () => {},
  getSuggestedName: () => {},
}

const ConnectedSetupProfile = (props) => (
  <APIContext.Consumer>
    {(api) => <SetupProfile {...api} {...props} />}
  </APIContext.Consumer>
)

export default ConnectedSetupProfile
