import React from 'react'
import { shallow } from 'enzyme'

import { trackEvent } from '../track/track'
import Button from '../Button/Button'
import APIContext from '../api/APIContext'

import SetupProfile from './SetupProfile'

jest.mock(
  '../Button/Button',
  () =>
    function MockButton() {
      return <div />
    },
)

jest.mock('../api/APIContext', () => ({
  Consumer: function Mock() {
    return <div />
  },
}))

jest.mock('../api/handleAPIError', () => ({
  getNameError: (e) => `getNameError(${e.message})`,
}))

jest.mock('../track/track', () => ({
  trackEvent: jest.fn(),
}))

describe('LEADERBOARD: SETUP PROFILE', () => {
  const { location } = window
  let setup
  let onChange
  let submit
  let submitKey
  let createProfile
  let getSuggestedName

  beforeEach(() => {
    createProfile = jest.fn()
    getSuggestedName = jest.fn()

    delete window.location
    window.location = {
      ...location,
      reload: jest.fn(),
      assign: jest.fn(),
    }

    jest.spyOn(window.location, 'reload').mockImplementation(() => {})
    jest.spyOn(window.location, 'assign').mockImplementation(() => {})

    const profile = shallow(<SetupProfile />)

    const renderFn = profile
      .find(APIContext.Consumer)
      .prop('children')

    getSuggestedName.mockReturnValue(Promise.resolve({}))

    setup = shallow(renderFn({ createProfile, getSuggestedName }))

    onChange = setup.find('input.lbd-type__input').prop('onChange')

    submit = setup.find(Button).prop('action')

    submitKey = setup.find('input.lbd-type__input').prop('onKeyUp')

    setup.debug()
  })

  afterEach(() => {
    window.location = location
  })

  test('next is disabled whenever input is empty', () => {
    expect(setup.find(Button).prop('isDisabled')).toBe(true)
    setup.find(Button).simulate('click')
    expect(window.location.reload).not.toHaveBeenCalled()

    onChange({ target: { value: 'aaaa' } })

    setup.update()

    expect(setup.find(Button).prop('isDisabled')).toBe(false)

    onChange({ target: { value: '' } })

    setup.update()

    expect(setup.find(Button).prop('isDisabled')).toBe(true)
  })

  test('failed input', async () => {
    const fakeName = 'invalid name'
    const fakeErrorMessage = 'fake error'

    createProfile.mockReturnValue(
      Promise.reject(new Error(fakeErrorMessage)),
    )

    onChange({ target: { value: fakeName } })

    setup.update()
    await submit()
    setup.update()

    expect(createProfile).toHaveBeenCalledWith(fakeName)
    expect(setup.find('.lbd-type__error').text()).toBe(
      `Error: getNameError(${fakeErrorMessage})`,
    )
    expect(setup.hasClass('has-error')).toBe(true)
  })

  test('input is disabled while network is pending', async () => {
    createProfile.mockImplementation(() => {
      // 1. disabled before request goes out
      expect(setup.find(Button).prop('isDisabled')).toBe(true)

      return Promise.reject(new Error('err'))
    })

    onChange({ target: { value: 'foo' } })

    setup.update()
    await submit()
    setup.update()

    // 2. disabled when error returns
    expect(setup.find(Button).prop('isDisabled')).toBe(true)

    onChange({ target: { value: 'bar' } })

    setup.update()

    // 3. enabled after input change
    expect(setup.find(Button).prop('isDisabled')).toBe(false)
  })

  test('redirects page when profile is successful', async () => {
    window.history.pushState(
      {},
      'Test Title',
      '/puzzles/leaderboards',
    )
    window.location.pathname = '/puzzles/leaderboards'
    createProfile.mockReturnValue(Promise.resolve({}))

    onChange({ target: { value: 'foo' } })

    setup.update()
    await submit()
    setup.update()

    expect(trackEvent).toHaveBeenCalledWith('ONBOARDING_NAME')
    expect(window.location.assign).toHaveBeenCalled()
  })

  test('does not submit if there is no name', async () => {
    createProfile.mockReturnValue(Promise.resolve({}))

    onChange({ target: { value: '' } })

    setup.update()
    await submit()
    setup.update()

    expect(window.location.reload).not.toHaveBeenCalled()
  })

  test('enter key submits input', async () => {
    window.history.pushState(
      {},
      'Test Title',
      '/puzzles/leaderboards/',
    )
    window.location.pathname = '/puzzles/leaderboards'

    createProfile.mockReturnValue(Promise.resolve({}))

    onChange({ target: { value: 'foo' } })

    setup.update()
    await submitKey({ key: 'Enter' })
    setup.update()

    expect(window.location.assign).toHaveBeenCalledWith(
      '/puzzles/leaderboards/invite?onboarding=true',
    )
  })

  test('submit refreshes on invite pages', async () => {
    window.history.pushState(
      {},
      'Test Title',
      '/puzzles/leaderboards/invite/sadfdfsfd',
    )
    window.location.pathname =
      '/puzzles/leaderboards/invite/sadfdfsfd'

    createProfile.mockReturnValue(Promise.resolve({}))

    onChange({ target: { value: 'foo' } })

    setup.update()
    await submitKey({ key: 'Enter' })
    setup.update()

    expect(window.location.reload).toHaveBeenCalled()
  })

  test('other keys do not submit', async () => {
    createProfile.mockReturnValue(Promise.resolve({}))

    onChange({ target: { value: 'foo' } })

    setup.update()
    await submitKey({ key: 'Other' })
    setup.update()

    expect(window.location.assign).not.toHaveBeenCalled()
    expect(window.location.reload).not.toHaveBeenCalled()
  })
})
