import { createStore, applyMiddleware } from 'redux'

import createConnectionMiddleware from './createConnectionMiddleware'

describe('LEADERBOARDS: STORE / CREATE CONNECTION MIDDLEWARE', () => {
  let store
  let history
  let api
  let err
  let res

  let FOO_ACTION
  let BAR_ACTION

  beforeEach(() => {
    history = []

    api = {
      connection: {
        foo: jest.fn(),
        bar: jest.fn(),
      },
    }

    res = { baz: true }
    err = new Error('fail')

    api.connection.foo.mockImplementation(() => Promise.resolve(res))
    api.connection.bar.mockImplementation(() => Promise.reject(err))

    const trackActions = () => (next) => (action) => {
      history.push(action)
      return next(action)
    }

    store = createStore(
      (_) => _,
      applyMiddleware(createConnectionMiddleware(api), trackActions),
    )

    FOO_ACTION = 'foo'
    BAR_ACTION = 'bar'
  })

  it('handles a successful connection method', () => {
    const p = store.dispatch({
      type: FOO_ACTION,
      payload: { method: 'foo', userID: 123 },
      api: true,
    })

    expect(history).toEqual([
      {
        type: `${FOO_ACTION}/pending`,
        payload: { method: FOO_ACTION, userID: 123 },
      },
    ])

    return p.then(() => {
      expect(history).toEqual([
        {
          type: `${FOO_ACTION}/pending`,
          payload: { method: FOO_ACTION, userID: 123 },
        },
        {
          type: `${FOO_ACTION}/success`,
          payload: {
            request: { method: FOO_ACTION, userID: 123 },
            response: res,
          },
        },
      ])
    })
  })

  it('handles a failed connection method', () => {
    const p = store.dispatch({
      type: BAR_ACTION,
      payload: { method: 'bar', userID: 123 },
      api: true,
    })

    expect(history).toEqual([
      {
        type: `${BAR_ACTION}/pending`,
        payload: { method: BAR_ACTION, userID: 123 },
      },
    ])

    return p.catch((err) => {
      expect(err).toBe(err)
      expect(history).toEqual([
        {
          type: `${BAR_ACTION}/pending`,
          payload: { method: BAR_ACTION, userID: 123 },
        },
        {
          type: `${BAR_ACTION}/error`,
          payload: {
            request: { method: BAR_ACTION, userID: 123 },
            error: err,
          },
        },
      ])
    })
  })

  it('passes through other actions', () => {
    const action = { type: 'other', payload: 123 }
    store.dispatch(action)

    expect(history).toEqual([action])
  })
})
