import Store from './index'

import * as actions from './actions'
import {
  SELF_EDIT,
  SELF_EDIT_ENTER,
  SELF_EDIT_LEAVE,
  SELF_EDIT_UPDATE,
  FRIEND_REMOVE,
  FRIEND_BLOCK,
  PENDING_CONFIRM,
  PENDING_REJECT,
  BLOCKED_UNBLOCK,
  PENDING_STATES,
  PENDING_CONFIRMING,
} from './constants'

const sampleData = {
  connections: {
    self: [{ name: 'Garrett' }],
    friends: [
      { userID: 1, name: 'Alfred' },
      { userID: 2, name: 'Barbara' },
      { userID: 3, name: 'Charlie' },
      { userID: 4, name: 'Denise' },
      {
        userID: 4,
        name: 'Denise',
        pendingState: 'awaiting',
      },
    ],
    pending: [
      { userID: 5, name: 'Evan' },
      { userID: 6, name: 'Frank' },
      { userID: 7, name: 'Georgette' },
    ],
    blocked: [
      { userID: 8, name: 'James' },
      { userID: 9, name: 'Karen' },
    ],
  },
}

describe('LEADERBOARDS: STORE', () => {
  let store
  let res
  let err
  let state
  let initialState

  beforeEach(() => {
    sampleData.api = {
      connection: {
        [SELF_EDIT]: jest.fn(),
        [FRIEND_REMOVE]: jest.fn(),
        [FRIEND_BLOCK]: jest.fn(),
        [PENDING_CONFIRM]: jest.fn(),
        [PENDING_REJECT]: jest.fn(),
        [BLOCKED_UNBLOCK]: jest.fn(),
      },
    }

    res = { message: 'success' }
    err = new Error('fail')

    Object.keys(sampleData.api.connection).forEach((connection) => {
      sampleData.api.connection[connection].mockImplementation(
        ({ userID }) => {
          return userID === undefined || userID < 10
            ? Promise.resolve(res)
            : Promise.reject(err)
        },
      )
    })

    store = Store(sampleData)
    state = () => store.getState().connections
    initialState = sampleData.connections
  })

  it('reads data from setup into the redux store', () => {
    expect(state()).toEqual(initialState)
  })

  it(`${SELF_EDIT_ENTER} / ${SELF_EDIT_UPDATE} / ${SELF_EDIT_LEAVE}`, () => {
    expect(state().self[0].isEditing).toBe(undefined)
    expect(state().self[0].error).toBe(undefined)

    store.dispatch(actions.editSelfEnter())

    expect(state().self[0].isEditing).toBe(true)
    expect(state().self[0].pendingName).toBe('')

    store.dispatch(actions.editSelfUpdate('new'))

    expect(state().self[0].isEditing).toBe(true)
    expect(state().self[0].pendingName).toBe('new')

    store.dispatch(actions.editSelfLeave())

    expect(state().self[0].isEditing).toBe(false)
    expect(state().self[0].error).toBe(null)
    expect(state().self[0].pendingName).toBe('')
  })

  it(`${SELF_EDIT}/success changes the name value`, () => {
    const action = actions.editSelf('Greg')
    const p = store.dispatch(action)

    expect(state().self[0].isSubmitting).toBe(true)

    return p.then(() => {
      expect(state().self[0].name).toBe('Greg')
      expect(state().self[0].isSubmitting).toBe(false)
    })
  })

  it(`${SELF_EDIT}/success changes the name value`, () => {
    const action = actions.editSelf('Greg')
    const p = store.dispatch(action)

    expect(state().self[0].isSubmitting).toBe(true)

    return p.then(() => {
      expect(state().self[0].name).toBe('Greg')
      expect(state().self[0].isSubmitting).toBe(false)
    })
  })

  it(`${SELF_EDIT}/error handles error and clears on change`, () => {
    const err = new Error('bad name')
    sampleData.api.connection[SELF_EDIT].mockReturnValue(
      Promise.reject(err),
    )

    const action = actions.editSelf('Greg')
    const p = store.dispatch(action)

    expect.assertions(5)
    expect(state().self[0].isSubmitting).toBe(true)

    return p.catch((e) => {
      expect(state().self[0].error).toBe(err)
      expect(state().self[0].isSubmitting).toBe(false)

      store.dispatch(actions.editSelfUpdate('blah'))

      expect(state().self[0].error).toBe(null)
      expect(e).toBe(err)
    })
  })

  it(`${FRIEND_BLOCK}/success removes a friend`, () => {
    const target = initialState.friends[0]
    const action = actions.blockFriend(target.userID)
    const p = store.dispatch(action)

    return p.then(() => {
      expect(state().friends).toHaveLength(
        initialState.friends.length - 1,
      )
      expect(state().friends).not.toContainEqual(target)
    })
  })

  it(`${FRIEND_BLOCK}/success removes a pending connection`, () => {
    const target = initialState.pending[0]
    const action = actions.blockFriend(target.userID)
    const p = store.dispatch(action)

    return p.then(() => {
      expect(state().pending).toHaveLength(
        initialState.pending.length - 1,
      )
      expect(state().pending).not.toContainEqual(target)
    })
  })

  it(`${FRIEND_BLOCK}/success adds a friend to the blocked list`, () => {
    const target = initialState.friends[0]
    const action = actions.blockFriend(target.userID)
    const p = store.dispatch(action)

    return p.then(() => {
      expect(state().blocked).toHaveLength(
        initialState.blocked.length + 1,
      )
      expect(state().blocked).toContainEqual(target)
    })
  })

  it(`${FRIEND_REMOVE}/success removes a friend`, () => {
    const target = initialState.friends[0]
    const action = actions.removeFriend(target.userID)
    const p = store.dispatch(action)

    return p.then(() => {
      expect(state().friends).toHaveLength(
        initialState.friends.length - 1,
      )
      expect(state().friends).not.toContainEqual(target)
    })
  })

  it(`${PENDING_CONFIRM}/success sets the pending state to confirmed`, () => {
    const target = initialState.pending[0]
    const action = actions.apiConfirmPending(target.userID)
    const p = store.dispatch(action)

    const stateUser = state().pending.find(
      (user) => user.userID === target.userID,
    )

    return p.then(() => {
      expect(stateUser.pendingState).toBe(PENDING_STATES.CONFIRMED)
    })
  })

  it(`${PENDING_CONFIRM}/success adds target to friends`, () => {
    const target = initialState.pending[0]
    const action = actions.apiConfirmPending(target.userID)
    const p = store.dispatch(action)

    return p.then(() => {
      expect(state().friends).toHaveLength(
        initialState.friends.length + 1,
      )
      expect(state().friends).toContainEqual(target)
    })
  })

  it(`${PENDING_REJECT}/success removes a pending user from the list`, () => {
    const target = initialState.pending[0]
    const action = actions.rejectPending(target.userID)
    const p = store.dispatch(action)

    return p.then(() => {
      expect(state().pending).toHaveLength(
        initialState.pending.length - 1,
      )
      expect(state().pending).not.toContainEqual(target)
    })
  })

  it(`${BLOCKED_UNBLOCK}/success removes a connection from the blocked list`, () => {
    const target = initialState.blocked[0]
    const action = actions.unblock(target.userID)
    const p = store.dispatch(action)

    return p.then(() => {
      expect(state().blocked).toHaveLength(
        initialState.blocked.length - 1,
      )
      expect(state().blocked).not.toContainEqual(target)
    })
  })

  it(`${PENDING_CONFIRMING}/success sets a confirming pending state`, () => {
    const target = initialState.pending[0]
    const action = actions.confirmingPending(target.userID)
    store.dispatch(action)

    expect(target.pendingState).toBe('confirming')
  })

  it(`dispatches 'confirm pending' actions`, () => {
    const fn = actions.confirmPending({ foo: 'bar' })
    expect(typeof fn).toBe('function')
    const dispatch = jest.fn()
    fn(dispatch)
    expect(dispatch).toHaveBeenCalledWith({
      api: true,
      payload: { foo: 'bar' },
      type: 'ManageFriends/PENDING_CONFIRM',
    })
    expect(dispatch).toHaveBeenCalledWith({
      payload: { foo: 'bar' },
      type: 'ManageFriends/PENDING_CONFIRMING',
    })
  })
})
