import { xhr } from 'foundation'

import * as constants from '../Store/constants'

import * as api from './api'
import { handleAPIError } from './handleAPIError'

jest.mock('foundation', () => ({
  env: {
    api: 'api',
  },
  xhr: {
    post: jest.fn(),
    put: jest.fn(),
    get: jest.fn(),
  },
}))

jest.mock('./forcePendingTime', () => (_) => _)
jest.mock('./handleAPIError', () => ({
  handleAPIError: jest.fn(),
}))

describe('LEADERBOARD: API METHODS', () => {
  beforeEach(() => {
    handleAPIError.mockImplementation((_) => _)
  })

  test('createProfile success', async () => {
    xhr.post.mockReturnValue(Promise.resolve(true))
    await api.createProfile('greg')

    expect(xhr.post).toHaveBeenCalledWith(
      'api/svc/crosswords/v6/leaderboard/profile.json',
      {
        name: 'greg',
      },
    )
  })

  test('createProfile failure', async () => {
    expect.assertions(1)

    xhr.post.mockReturnValue(Promise.resolve())
    handleAPIError.mockImplementation(() => {
      throw new Error('handled error')
    })

    try {
      await api.createProfile('greg')
    } catch (e) {
      expect(e.message).toBe('handled error')
    }
  })

  test('regenerateLink success', async () => {
    xhr.get.mockReturnValue(Promise.resolve({ URL: 'foo' }))
    const url = await api.regenerateLink()

    expect(url).toBe('foo')
    expect(xhr.get).toHaveBeenCalledWith(
      'api/svc/crosswords/v6/leaderboard/profile/code.json',
    )
  })

  test('regenerateLink failure', async () => {
    expect.assertions(1)

    xhr.get.mockReturnValue(Promise.resolve())
    handleAPIError.mockImplementation(() => {
      throw new Error('handled error')
    })

    try {
      await api.regenerateLink()
    } catch (e) {
      expect(e.message).toBe('handled error')
    }
  })

  test('requestToJoin success', async () => {
    xhr.put.mockReturnValue(Promise.resolve(true))
    await api.requestToJoin('abc')

    expect(xhr.put).toHaveBeenCalledWith(
      'api/svc/crosswords/v6/leaderboard/connection/request.json',
      {
        code: 'abc',
      },
    )
  })

  test('requestToJoin failure', async () => {
    expect.assertions(1)

    xhr.put.mockReturnValue(Promise.resolve())
    handleAPIError.mockImplementation(() => {
      throw new Error('handled error')
    })

    try {
      await api.requestToJoin()
    } catch (e) {
      expect(e.message).toBe('handled error')
    }
  })

  test('edit name success', async () => {
    xhr.put.mockReturnValue(Promise.resolve(true))
    await api.connection[constants.SELF_EDIT]('abc')

    expect(xhr.put).toHaveBeenCalledWith(
      'api/svc/crosswords/v6/leaderboard/profile.json',
      {
        name: 'abc',
      },
    )
  })

  test('edit name failure', async () => {
    expect.assertions(1)

    xhr.put.mockReturnValue(Promise.resolve())
    handleAPIError.mockImplementation(() => {
      throw new Error('handled error')
    })

    try {
      await api.connection[constants.SELF_EDIT]('abc')
    } catch (e) {
      expect(e.message).toBe('handled error')
    }
  })

  const friendActions = {
    [constants.FRIEND_REMOVE]: 'remove',
    [constants.FRIEND_BLOCK]: 'block',
    [constants.PENDING_CONFIRM]: 'approve',
    [constants.PENDING_REJECT]: 'reject',
    [constants.BLOCKED_UNBLOCK]: 'unblock',
  }

  Object.keys(friendActions).forEach((action) => {
    test(`${friendActions[action]} success`, async () => {
      xhr.put.mockReturnValue(Promise.resolve(true))
      await api.connection[action]('123')

      expect(xhr.put).toHaveBeenCalledWith(
        `api/svc/crosswords/v6/leaderboard/connection/${friendActions[action]}.json`,
        {
          userID: '123',
        },
      )
    })

    test(`${friendActions[action]} failure`, async () => {
      expect.assertions(1)

      xhr.put.mockReturnValue(Promise.resolve())
      handleAPIError.mockImplementation(() => {
        throw new Error('handled error')
      })

      try {
        await api.connection[action]()
      } catch (e) {
        expect(e.message).toBe('handled error')
      }
    })
  })
})
