import React from 'react'
import { createPortal } from 'react-dom'
import { render, screen } from '@testing-library/react'

import { useUser } from 'user'

import useDeltaBanner from './useDeltaBanner'

jest.mock('user', () => ({
  useUser: jest
    .fn()
    .mockReturnValue({ userType: {}, hasLoaded: true }),
}))

jest.mock('shared/foundation/js/util', () => ({
  ...jest.requireActual('shared/foundation/js/util'),
  deltaCookie: 'nyt-delta=1',
}))

type MockedUserHookReturn = {
  userType: {
    regiId: string
    isLoggedIn: boolean
    hasXwd?: boolean
    hasDigi?: boolean
  }
  hasLoaded: boolean
}

const anon: MockedUserHookReturn = {
  userType: {
    regiId: '',
    isLoggedIn: false,
  },
  hasLoaded: true,
}

const regi: MockedUserHookReturn = {
  userType: {
    ...anon.userType,
    regiId: '123',
    isLoggedIn: true,
  },
  hasLoaded: true,
}

const gamesSub: MockedUserHookReturn = {
  userType: {
    ...regi.userType,
    hasXwd: true,
  },
  hasLoaded: true,
}

const newsSub: MockedUserHookReturn = {
  userType: {
    ...regi.userType,
    hasDigi: true,
  },
  hasLoaded: true,
}

const NavigationDrawer = () => <div id="js-nav-drawer" />

const DeltaBannerPortal = () => <div id="delta-banner-portal" />

const MiniPage = () => {
  const { willShowDeltaBanner, portalEl } = useDeltaBanner()

  return (
    <>
      {willShowDeltaBanner &&
        portalEl &&
        createPortal(<div>Delta Banner</div>, portalEl)}
    </>
  )
}

const SpellingBeePage = () => {

  return (
    <>
    </>
  )
}

const renderPage = () => {
  return render(
    <>
      <DeltaBannerPortal />
      <NavigationDrawer />
      <div>container</div>
    </>,
  )
}

const setUser = (user: MockedUserHookReturn) => {
  // @ts-ignore mock with partial user return type
  useUser.mockReturnValue(user)
}

describe('HOOK: useDeltaBanner', () => {
  window.isHybridWebView = false
  window.newsreaderAppPlatform = ''
  window.isMobileDevice = false

  const page = renderPage()
  const container = page.getByText('container')

  describe('when on the delta wifi', () => {
    describe('visiting the mini page', () => {
      it('logged out users should see a banner', () => {
        setUser(anon)
        render(<MiniPage />, { container })
        expect(screen.getByText('Delta Banner')).toBeTruthy()
      })

      it('logged in users should NOT see a banner', () => {
        setUser(regi)
        render(<MiniPage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('games subs should NOT see a banner', () => {
        setUser(gamesSub)
        render(<MiniPage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('news subs should NOT see a banner', () => {
        setUser(newsSub)
        render(<MiniPage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })
    })

    describe('visiting the spelling bee page', () => {
      it('logged out users should NOT see a banner', () => {
        setUser(anon)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('logged in users should NOT see a banner', () => {
        setUser(regi)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('games subs should NOT see a banner', () => {
        setUser(gamesSub)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('news subs should NOT see a banner', () => {
        setUser(newsSub)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })
    })
  })

  describe('when NOT on the delta wifi', () => {
    jest.mock('shared/foundation/js/util', () => ({
      ...jest.requireActual('shared/foundation/js/util'),
      deltaCookie: '',
    }))

    describe('visiting the mini page', () => {
      it('logged out users should NOT see a banner', () => {
        setUser(anon)
        render(<MiniPage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('logged in users should NOT see a banner', () => {
        setUser(regi)
        render(<MiniPage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('games subs should NOT see a banner', () => {
        setUser(gamesSub)
        render(<MiniPage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('news subs should NOT see a banner', () => {
        setUser(newsSub)
        render(<MiniPage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })
    })

    describe('visiting the spelling bee page', () => {
      it('logged out users should NOT see a banner', () => {
        setUser(anon)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('logged in users should NOT see a banner', () => {
        setUser(regi)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('games subs should NOT see a banner', () => {
        setUser(gamesSub)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })

      it('news subs should NOT see a banner', () => {
        setUser(newsSub)
        render(<SpellingBeePage />, { container })
        expect(screen.queryByText('Delta Banner')).toBeNull()
      })
    })
  })
})
