/**
 * An enum containing all of the possible values that the Meter Service will
 * grant access to a user.
 */
export enum grantReasons {
  /**
   * Access is granted because content is not subject to metering (homepage, category pages).
   */
  OPEN = 'OPEN',
  /**
   * Asset is counted and gated based on current meter rules. When this value is encountered
   * it's important to respect the gatewayType to show the appropriate experience (e.g. Paywall or Regiwall).
   */
  METERING = 'METERING',
  /**
   * User has exhausted their available bucket count for a specific custom bucket for the month.
   * When this value is encountered, counted and granted will both be false and hitPaywall will be true.
   */
  BUCKET_LIMIT = 'BUCKET_LIMIT',
  /**
   * User has exhausted their available meter count and has already hit the maximum meter
   * threshold for the month. When this value is encountered, typically counted and granted
   * will also both be false.
   */
  METER_LIMIT = 'METER_LIMIT',
  /**
   * Unsure what this value is used for.
   */
  FREE_CLICK = 'FREE_CLICK',
  /**
   * Asset is counted but not gated based on current meter rules.
   */
  COUNTED = 'COUNTED',
  /**
   * Access is granted because the content is presented with a valid Facebook News token.
   */
  FB_TOP = 'FB_TOP',
  /**
   * Access is granted because content is presented with a valid subscriber link share code.
   */
  UNLOCKED_ARTICLE_CODE = 'UNLOCKED_ARTICLE_CODE',
  /**
   * Access is granted based on the meter rules set for it. It's not counted and not gated.
   */
  WHITELISTED = 'WHITELISTED',
  /**
   * Access is granted due to user's subscription status.
   */
  SUBSCRIBER = 'SUBSCRIBER',
  /**
   * Access is granted due to user accessing from a known licensed IP range (typically B2B Site License).
   */
  IP = 'IP',
  /**
   * Access is granted as user is visiting via a known B2B client website (e.g. hotel portal).
   */
  REFERRER = 'REFERRER',
  /**
   * Access is granted to user who have free minutes remaining, usually given to users from known
   * B2B client websites initially granted with the REFERRER grant reason.
   */
  FREETIME = 'FREETIME',
}

/**
 * Indicates the type of gateway that Meter is recommending to show the user based on their meter.
 */
export type gatewayType = 'REGIWALL' | 'PAYWALL'

/**
 * A subtype of gatewayType specifying the behavior of the type gateway being shown
 */
export type gatewayTypeVariation =
  | 'DISMISSIBLE'
  | 'TRUNCATOR'
  | 'IP'
  | 'DEFAULT'

/**
 * Properties of a Meter response for a given New York Times asset url and optional game level
 * (e.g. Spelling Bee ranks like Solid, Amazing, Genius, etc.)
 */
export type MeterProperties = {
  /**
   * A structure which shows the AB test names and variants to which the current pageview is exposed to.
   */
  ab?: string[]
  /**
   * The custom asset type of the current pageview. Nullable, and defined in meter rules.
   * E.g.: COVID19, features-based, lesson_plan
   */
  assetType?: string
  /**
   * Shows whether the asset is supposed to be metered according to the rules. Ignores the fact
   * whether the asset was actually counted in the current pageview. Can be true even if the
   * user was granted access for the current pageview.
   */
  countable?: boolean
  /**
   * Indicates if the current pageview was metered and did increase the meter counter (v field value)
   */
  counted?: boolean
  /**
   * Indicates if the paywall is globally enabled. When the paywall is globally disabled, no pageviews
   * will trigger paywall regardless of their individual settings.
   */
  gateway?: boolean
  /**
   * Shows whether the gateway can be triggered on the page. Can be true even if the user is
   * granted access to the asset.
   */
  gatewayEligible?: boolean
  /**
   * Indicates that current pageview was not supposed to trigger paywall (or regiwall) because of its
   * individual settings or the current user session state.
   */
  gatewayExempt?: boolean
  /**
   * The type of gateway displayed (paywall, regiwall, etc). Should be used together with the granted field.
   * The client app may use the value to determine which type of gateway it should show.
   */
  gatewayType?: gatewayType
  /**
   * The subtype of gatewayType, contains additional information about gateway.
   */
  gatewayTypeVariation?: gatewayTypeVariation
  /**
   * Shows whenever meter allows access to content. Effectively it will have the reversed
   * value of the hitPaywall field.
   */
  granted: boolean
  /**
   * Explain why the user was granted (or not granted) access to the content. See grantReasons enum defined here.
   */
  grantReason: grantReasons
  /**
   * The integrity checksum of the nyt-m cookie payload.
   */
  hash?: string
  /**
   * Indicates that current pageview triggered paywall. The client app should limit access to the page content and
   * display a non-dismissable paywall asset.
   */
  hitPaywall?: boolean
  /**
   * The current pageview triggered regiwall. The client app should limit access to the page content and display
   * a non-dismissable regiwall asset.
   */
  hitRegiwall?: boolean
  /**
   * @deprecated
   * Will be removed in future releases of Meter.
   */
  hitSoftPaywall?: boolean
  /**
   * @deprecated
   * Will be removed in future releases of Meter.
   */
  hitSoftRegiwall?: boolean
  /**
   * The nyt-m cookie in the request was present and valid.
   */
  isCookieValid?: boolean
  /**
   * Indicates if the last request to meter succeeded or failed. This is a property added onto the response
   * by us (Games team) to assist with fail scenarios
   */
  isOffline: boolean
  /**
   * When presenting Meter with a valid NYT-S cookie, this property will indicate if the user is logged in.
   */
  loggedIn: boolean
  /**
   * Indicates that metering is globally enabled. When the metering is globally disabled, no pageviews
   * will increase the meter counter or trigger paywall regardless of their individual settings.
   */
  meter?: boolean
  /**
   * The basic capability field in NYT-S cookie is expired. The cookie should be refreshed. The client app
   * should take care of refreshing the NYT-S cookie and handling the user state after it.
   */
  nytsNeedsRefresh?: boolean
  /**
   * Indicates that the regiwall is globally enabled. When the regiwall is globally disabled, no pageviews
   * will trigger regiwall regardless of their individual settings.
   */
  regiwall?: boolean
  /**
   * Indicates the version of the Meter Rules that are in effect for this session.
   */
  rulesVersion?: string
  /**
   * The name of the section from the rules file. Each section can have its own set of rules, views and thresholds.
   */
  section?: string
  /**
   * @deprecated
   * Will be removed in future releases of Meter.
   */
  showGrowl?: boolean
  /**
   * The amount of allowed metered pageviews in the current period (month). Also known as meter threshold.
   */
  t?: number
  /**
   * The amount of spent metered pageviews in the current period (month). Also known as meter counter.
   * Increases every time the user visits a metered page without hitting the paywall.
   */
  v?: number
  /**
   * Indicates the current version of the Meter Service for this session.
   */
  version?: string
}

/**
 * An enum indicating the sourceApp values for each platform
 */
export enum sourceApp {
  WEB = 'Games-web',
  ANDROID_NEWS = 'Games-Android-NR',
  ANDROID_XWD = 'Games-Android',
  IOS_NEWS = 'Games-iOS-NR',
  IOS_XWD = 'Games-iOS',
}
