import hybridDebugger from 'shared/foundation/js/hybrid/hybridDebugger'

import { MeterProperties, grantReasons } from './MeterTypes'
import { setCache } from './request'

/**
 * A set of Meter credentials that will open gameplay
 */
export const openMeterCredentials = {
  loggedIn: true,
  grantReason: grantReasons.IP,
  granted: true,
  isOffline: true,
}

type DebugType = {
  isMeterInDebugMode: boolean
  mockResponse: MeterProperties
}

/**
 * A singleton used to debug the Meter by mocking requests and responses
 */
export const debug: DebugType = {
  isMeterInDebugMode: false,
  mockResponse: openMeterCredentials,
}

/**
 * A function mocking a successful Meter request with open-access credentials. This is
 * used temporarily as we ramp traffic to the Meter service
 */
export const mockRetrieveMeterValue = (
  puzzleUrl: string,
  level?: number,
): Promise<MeterProperties> => {
  hybridDebugger(
    'Mock request made to Meter service',
    [puzzleUrl, level],
    debug.mockResponse,
  )
  setCache(debug.mockResponse)
  return Promise.resolve(debug.mockResponse)
}

type MeterDebugResponseParameters = {
  debugMode: true
  meterCredentials: MeterProperties
}

/**
 * A function used by developers to debug the meter. This will allow you to set the meter into debug mode and optionally
 * provide a mock Meter response you'd like to see. If no response is provided, the default mock is failing open
 * with a grantReason of IP. To return meter to normal use, call this function with { debugMode: false }
 */
export const setDebugResponse = ({
  debugMode,
  meterCredentials,
}: MeterDebugResponseParameters) => {
  let msg = ''
  let args: MeterProperties[] = []
  let resp

  if (debugMode) {
    debug.isMeterInDebugMode = true
    debug.mockResponse = { ...meterCredentials }
    msg =
      'Meter debug mode turned on with mock response for Meter service'
    args = [meterCredentials]
    resp = { ...debug.mockResponse }
  } else {
    debug.isMeterInDebugMode = false
    msg =
      'Meter debug mode turned off. Requests to Meter service will resume'
  }

  hybridDebugger(msg, args, resp)
}
