import { xhr } from 'foundation'
import hybridDebugger from 'shared/foundation/js/hybrid/hybridDebugger'

import { grantReasons, MeterProperties } from './MeterTypes'
import { getCache, setCache, generateMeterUrl } from './request'
import { mockRetrieveMeterValue } from './debug'
import { willMakeMeterRequest } from './abra'

/**
 * Criteria to determine if a user's Meter value merits failing open should the Meter Service go down.
 */
export const shouldFailOpen = (meterValue: MeterProperties) => {
  return (
    meterValue.granted &&
    (meterValue.grantReason === grantReasons.IP ||
      meterValue.grantReason === grantReasons.SUBSCRIBER ||
      meterValue.grantReason === grantReasons.OPEN ||
      meterValue.grantReason === grantReasons.REFERRER ||
      meterValue.grantReason === grantReasons.FREETIME)
  )
}

/**
 * Sets a failed state on the meter response to inform consumers that the Meter is not online
 */
export const setFailedState = () => {
  const cache = getCache()
  const response = { ...cache, isOffline: true }

  if (shouldFailOpen(response)) {
    hybridDebugger(
      'Game is instructed to fail open based on previous grantReason',
      [cache.grantReason],
    )
  } else {
    hybridDebugger(
      'Game is instructed not fail open due to previous grantReason',
      [cache.grantReason],
    )
  }

  return response
}

type XHROptions = {
  headers?: {
    [key in string]: string
  }
}

type MeterRequestParameters = {
  puzzleUrl?: string
  level?: number
}

/**
 * Makes an http request to the meter service for a given asset and optional game level.
 */
export const retrieveMeterValue = ({
  puzzleUrl = window.location.href,
  level,
}: MeterRequestParameters): Promise<MeterProperties> => {
  const meterUrl = generateMeterUrl(puzzleUrl)

  let options: XHROptions = {}

  if (typeof level !== 'undefined') {
    options = {
      ...options,
      headers: { 'NYT-M-Level': window.btoa(level.toString()) },
    }
  }

  // Only send real requests to Meter Service if allocated to
  if (!willMakeMeterRequest()) {
    return mockRetrieveMeterValue(puzzleUrl, level)
  }

  return xhr
    .get<MeterProperties>(meterUrl, options)
    .then((response) => {
      hybridDebugger(
        'Meter service request succeeded',
        [meterUrl, puzzleUrl, level],
        response,
      )

      const meterCredentials = { ...response, isOffline: false }
      setCache(meterCredentials)
      return meterCredentials
    })
    .catch((e) => {
      hybridDebugger(
        'Meter service request failed',
        [meterUrl, puzzleUrl, level],
        e,
      )
      return setFailedState()
    })
}

export { setDebugResponse } from './debug'
export { willUseMeterToGate } from './abra'
