import { env } from 'foundation'

import {
  MeterProperties,
  grantReasons,
  sourceApp,
} from './MeterTypes'

const defaultMeter: MeterProperties = {
  granted: true,
  grantReason: grantReasons.METERING,
  loggedIn: false,
  hash: '0',
  isOffline: true,
}

type RequestType = {
  cache: MeterProperties
  url: null | string
}

const request: RequestType = {
  cache: defaultMeter,
  url: null,
}

/**
 * Determines the correct sourceApp parameter to send in the Meter request
 */
export const determineSourceApp = () => {
  const { newsreaderAppPlatform, gamesAppPlatform } = window

  if (newsreaderAppPlatform) {
    return newsreaderAppPlatform === 'ios'
      ? sourceApp.IOS_NEWS
      : sourceApp.ANDROID_NEWS
  }

  if (gamesAppPlatform) {
    return gamesAppPlatform === 'ios'
      ? sourceApp.IOS_XWD
      : sourceApp.ANDROID_XWD
  }

  return sourceApp.WEB
}

/**
 * Generates the Meter url we request for Meter data. This generation happens once each
 * page view and is cached.
 */
export const generateMeterUrl = (puzzleUrl: string) => {
  if (!request.url) {
    const url = new URL(puzzleUrl)
    const sanitized = `${url.origin + url.pathname}`
    const sourceApp = determineSourceApp()
    request.url = `${env.meter}/meter.js?sourceApp=${sourceApp}&url=${sanitized}`
  }

  return request.url
}

/**
 * Stores a successful meter response. This should be called when each request to Meter succeeds.
 */
export const setCache = (response: MeterProperties) => {
  request.cache = { ...response }
}

/**
 * Retrieves the last stored Meter response from the cache. Useful when trying to determine whether
 * to fail open for the user.
 */
export const getCache = () => {
  return { ...request.cache }
}
