import { SpellingBeeData } from '@nyt/games-schema'
import thunk from 'redux-thunk'
import {
  createStore,
  applyMiddleware,
  compose,
  combineReducers,
  AnyAction,
  Reducer,
} from 'redux'

import { MoogleOptions } from './types'
import { moogleReducer } from './moogleReducer'
import { createMoogleMiddleware } from './moogleMiddleware'
import { createMoogleLoader } from './moogleActions'
import { fetchMultiStates, postToMoogle } from './api'
import { MoogleData } from './moogleState'

jest.mock(
  'lodash-es/debounce',
  () =>
    (func: any) =>
    (...args: any) =>
      func(...args),
)

jest.mock('./api', () => ({
  fetchMultiStates: jest.fn(),
  postToMoogle: jest.fn(),
}))

// mock apis for an anon user
const mockFetchFailure = () => {
  // @ts-ignore
  fetchMultiStates.mockImplementation(
    (
      game: string,
      puzzleId: string,
      onSuccess: (data: any) => {},
      onFailure: (message: string) => {},
    ) => {
      onFailure('')
    },
  )
}

// mock apis for a regi user
const createApiMock = () => {
  let coreState: MoogleData | null = null

  const mockPost = () => {
    // @ts-ignore
    postToMoogle.mockImplementation((moogleData: MoogleData) => {
      coreState = moogleData
      return Promise.resolve({ version: `${moogleData.timestamp}` })
    })
  }

  mockPost()

  // @ts-ignore
  fetchMultiStates.mockImplementation(
    (
      game: string,
      puzzleId: string,
      onSuccess: (data: any, regiId?: number) => {},
    ) => {
      if (!coreState) {
        return onSuccess([], 12345)
      }
      onSuccess([coreState], 12345)
    },
  )

  return {
    goOffline: () => {
      // @ts-ignore
      postToMoogle.mockImplementation((moogleData: MoogleData) => {
        return Promise.reject(new Error(''))
      })
    },
    goOnline: () => {
      mockPost()
      window.dispatchEvent(new Event('online'))
    },
  }
}

const getPostedEventsFromMock = () => {
  // @ts-ignore
  return postToMoogle.mock.calls.reduce((acc, call) => {
    const arg = call[0]
    if (arg) {
      return [...acc, arg]
    }

    return acc
  }, [])
}

// a much simpler version of the spelling-bee reducer,
// that only keeps track of which answers the user has found
const testGameReducer: Reducer<{
  answers: string[]
  hasLoaded: boolean
}> = (
  state = { answers: [], hasLoaded: false },
  action: AnyAction,
) => {
  switch (action.type) {
    case 'insert_word':
      return {
        ...state,
        answers: [...(state.answers || []), action.word],
      }
    case 'loaded_progress':
      return { ...state, answers: action.answers, hasLoaded: true }
    default:
      return state
  }
}

const rootReducer = combineReducers({
  game: testGameReducer,
  moogle: moogleReducer,
})

type TestStoreState = ReturnType<typeof rootReducer>

// adapted from the spelling bee moogle options, but made simpler
const options: MoogleOptions<
  TestStoreState,
  AnyAction,
  SpellingBeeData
> = {
  game: 'spelling_bee',
  getLocalStorageKey: (state, regiId) => `test-${regiId || 'ANON'}`,
  selectPuzzleId: () => '1',
  selectMoogleState: (state) => state.moogle,
  shouldSave: (state, action) => action.type === 'insert_word',
  selectPersistedState: (state) => ({
    answers: state.game.answers || [],
    isRevealed: false,
  }),
  reconcileRemoteAndLocalState: (remoteState, localState) => {
    if (!remoteState && !localState) {
      return null
    }

    if (!remoteState) {
      return localState
    }

    if (!localState) {
      return remoteState
    }

    if (remoteState.answers.length >= localState.answers.length) {
      return remoteState
    }

    return localState
  },
  handleMidGameLogin: (anon, regi) => {
    if (!anon && !regi) {
      return null
    }

    if (!anon) {
      return regi
    }

    if (!regi) {
      return anon
    }

    return {
      answers: [...anon.answers, ...regi.answers],
      isRevealed: false,
    }
  },
  insertProgress: (data) => (dispatch) =>
    dispatch({
      type: 'loaded_progress',
      answers: data?.answers || [],
    }),
}

// here are the two functions that we're actually testing
const testSaveMiddleware = createMoogleMiddleware(options)
const loadTestProgress = createMoogleLoader(options)

// set up a test store with the simplified reducer and only the middleware needed for this test
const setUpTestStore = () => {
  const middleware = [applyMiddleware(thunk, testSaveMiddleware)]

  return createStore(rootReducer, compose(...middleware))
}

describe('Moogle client utility', () => {
  afterEach(() => {
    window.localStorage.clear()
    jest.clearAllMocks()
  })

  it('should load nothing if theres nothing to load', async () => {
    mockFetchFailure()
    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    expect(store.getState().game.answers).toEqual([])
    expect(store.getState().game.hasLoaded).toEqual(true)
  })

  it('should save and load from local storage for anonymous user', async () => {
    mockFetchFailure()
    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await store.dispatch({ type: 'insert_word', word: 'hello' })

    const newStore = setUpTestStore()
    expect(newStore.getState().game.answers).toEqual([])
    await newStore.dispatch(loadTestProgress())
    expect(newStore.getState().game.answers).toEqual(['hello'])
  })

  it('should save to backend for a regi user', async () => {
    createApiMock()

    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await store.dispatch({ type: 'insert_word', word: 'hello' })

    // simulate going to a different browser
    window.localStorage.clear()

    const newStore = setUpTestStore()
    await newStore.dispatch(loadTestProgress())
    expect(newStore.getState().game.answers).toEqual(['hello'])
  })

  it('should reconcile with local storage if a save failed', async () => {
    const { goOffline } = createApiMock()

    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await store.dispatch({ type: 'insert_word', word: 'hello' })
    goOffline() // force backend save to fail
    await store.dispatch({ type: 'insert_word', word: 'hey' })

    const newStore = setUpTestStore()
    await newStore.dispatch(loadTestProgress())
    expect(newStore.getState().game.answers).toEqual(['hello', 'hey'])
  })

  it('should retry save when coming back online', async () => {
    const { goOffline, goOnline } = createApiMock()

    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    goOffline() // force backend save to fail
    await store.dispatch({ type: 'insert_word', word: 'hello' })
    goOnline() // should retry save when coming back online

    // simulate going to a different browser
    window.localStorage.clear()

    const newStore = setUpTestStore()
    expect(newStore.getState().game.answers).toEqual([])
    await newStore.dispatch(loadTestProgress())
    expect(newStore.getState().game.answers).toEqual(['hello'])
  })

  it('should handle midgame login scenario', async () => {
    mockFetchFailure() // start playing as anon
    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await store.dispatch({ type: 'insert_word', word: 'hello' })

    createApiMock() // but then log in
    const newStore = setUpTestStore()
    await newStore.dispatch(loadTestProgress())
    expect(newStore.getState().game.answers).toEqual(['hello'])
  })

  it('should save the data that gets loaded, immediately', async () => {
    mockFetchFailure() // start playing as anon
    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await store.dispatch({ type: 'insert_word', word: 'hello' })

    createApiMock() // but then log in
    const newStore = setUpTestStore()
    await newStore.dispatch(loadTestProgress())

    // meanwhile, in another browser...
    window.localStorage.clear()

    const testStore = setUpTestStore()
    await testStore.dispatch(loadTestProgress())
    expect(testStore.getState().game.answers).toEqual(['hello'])
  })

  it('should force a save with most recent progress in the case that post reqs are triggered while another one is in flight', async () => {
    createApiMock()
    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await Promise.all([
      store.dispatch({ type: 'insert_word', word: 'first' }),
      store.dispatch({ type: 'insert_word', word: 'second' }),
      store.dispatch({ type: 'insert_word', word: 'third' }),
    ])

    const postCalls = getPostedEventsFromMock()
    expect(postCalls.length).toBeGreaterThanOrEqual(3)

    // from loadTestProgress
    expect(postCalls[0].game_data.answers).toEqual([])

    // from first insert_word
    expect(postCalls[1].game_data.answers).toEqual(['first'])

    // force save with most recent progress after triggering a post req while another one was in flight
    expect(postCalls[2].game_data.answers).toEqual([
      'first',
      'second',
      'third',
    ])
  })

  it('should force a save after post request returns an error', async () => {
    jest.useFakeTimers()
    jest.spyOn(global, 'setTimeout')
    createApiMock()

    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await store.dispatch({ type: 'insert_word', word: 'first' })

    // @ts-ignore
    postToMoogle.mockImplementationOnce(() => {
      return Promise.reject(new Error('player locked'))
    })

    await store.dispatch({ type: 'insert_word', word: 'second' })

    // Expect retry delay after second insert_word which fails
    jest.runAllTimers()
    expect(setTimeout).toHaveBeenCalledTimes(1)
    expect(setTimeout).toHaveBeenLastCalledWith(
      expect.any(Function),
      5000,
      {
        payload: { puzzleId: '1' },
        type: 'shared/moogle/FORCE_SAVE',
      },
    )

    // simulate going to a different browser to test if expected words were saved by the API
    window.localStorage.clear()

    const newStore = setUpTestStore()
    expect(newStore.getState().game.answers).toEqual([])
    await newStore.dispatch(loadTestProgress())
    expect(newStore.getState().game.answers).toEqual([
      'first',
      'second',
    ])
  })

  it('if retry errors, we should not force another save', async () => {
    jest.useFakeTimers()
    jest.spyOn(global, 'setTimeout')
    createApiMock()

    const store = setUpTestStore()
    await store.dispatch(loadTestProgress())
    await store.dispatch({ type: 'insert_word', word: 'first' })

    // @ts-ignore
    postToMoogle.mockImplementation(() => {
      return Promise.reject(new Error('player locked'))
    })

    await Promise.all([
      store.dispatch({ type: 'insert_word', word: 'second' }),
      store.dispatch({ type: 'insert_word', word: 'third' }),
      store.dispatch({ type: 'insert_word', word: 'fourth' }),
    ])

    // Expect retry delay to only have been called once
    jest.runAllTimers()
    await Promise.resolve()
    expect(setTimeout).toHaveBeenCalledTimes(1)

    // simulate going to a different browser to test if expected words were saved by the API
    window.localStorage.clear()

    const newStore = setUpTestStore()
    expect(newStore.getState().game.answers).toEqual([])
    await newStore.dispatch(loadTestProgress())
    expect(newStore.getState().game.answers).toEqual(['first'])
  })
})
