import type { CoreState, Player } from '@nyt/games-schema'
import type { ThunkAction } from 'redux-thunk'
import type { AnyAction } from 'redux'

import {
  getFromLocalStorage,
  getMoogleData,
  getMoogleRegiId,
} from './utils'
import { fetchMultiStates } from './api'
import { MoogleOptions, Versioned } from './types'
import type { MoogleData } from './moogleState'

export const LOAD_MULTI_STATES = 'shared/moogle/LOAD_MULTI_STATES'
export const MULTI_STATES_SUCCESS =
  'shared/moogle/MULTI_STATES_SUCCESS'
export const MULTI_STATES_ERROR = 'shared/moogle/MULTI_STATES_ERROR'

export const LOAD_MOOGLE_POST = 'shared/moogle/LOAD_MOOGLE_POST'
export const MOOGLE_POST_SUCCESS = 'shared/moogle/MOOGLE_POST_SUCCESS'
export const MOOGLE_POST_ERROR = 'shared/moogle/MOOGLE_POST_ERROR'

export const FORCE_SAVE = 'shared/moogle/FORCE_SAVE'
export const ENQUEUE_PENDING_SAVE =
  'shared/moogle/ENQUEUE_PENDING_SAVE'
export const DEQUEUE_PENDING_SAVE =
  'shared/moogle/DEQUEUE_PENDING_SAVE'
export const SET_ERROR_RETRY = 'shared/moogle/SET_ERROR_RETRY'

type LoadMultiStatesAction = {
  type: typeof LOAD_MULTI_STATES
}

type MultiStatesSuccessAction = {
  type: typeof MULTI_STATES_SUCCESS
  payload: {
    data: MoogleData[]
    regiId?: number
    player?: Player | null
  }
}

type MultiStatesErrorAction = {
  type: typeof MULTI_STATES_ERROR
  payload: {
    message: string
  }
}

type LoadMooglePostAction = {
  type: typeof LOAD_MOOGLE_POST
  payload: {
    puzzleId: string
  }
}

type MooglePostSuccessAction = {
  type: typeof MOOGLE_POST_SUCCESS
  payload: {
    puzzleId: string
    player?: Player | null
  }
}

type MooglePostErrorAction = {
  type: typeof MOOGLE_POST_ERROR
  payload: {
    puzzleId: string
    message: string
  }
}

export type ForceSaveAction = {
  type: typeof FORCE_SAVE
  payload: {
    puzzleId: string
  }
}

type EnqueuePendingSaveAction = {
  type: typeof ENQUEUE_PENDING_SAVE
  payload: {
    puzzleId: string
  }
}

type DequeuePendingSaveAction = {
  type: typeof DEQUEUE_PENDING_SAVE
}

type SetErrorRetryAction = {
  type: typeof SET_ERROR_RETRY
  payload: {
    puzzleId: string
    value: boolean
  }
}

export type MoogleAction =
  | LoadMultiStatesAction
  | MultiStatesSuccessAction
  | MultiStatesErrorAction
  | LoadMooglePostAction
  | MooglePostSuccessAction
  | MooglePostErrorAction
  | ForceSaveAction
  | EnqueuePendingSaveAction
  | DequeuePendingSaveAction
  | SetErrorRetryAction

export const forceSave = (puzzleId: string): ForceSaveAction => ({
  type: FORCE_SAVE,
  payload: { puzzleId },
})

const fetchRemoteState =
  <IStoreState>(
    game: CoreState['game'],
    puzzleIds: string[],
  ): ThunkAction<Promise<void>, IStoreState, unknown, MoogleAction> =>
  async (dispatch) => {
    dispatch({ type: LOAD_MULTI_STATES })
    await fetchMultiStates(
      game,
      puzzleIds,
      (data, regiId, player) =>
        dispatch({
          type: MULTI_STATES_SUCCESS,
          payload: { data, regiId, player },
        }),
      (message) =>
        dispatch({
          type: MULTI_STATES_ERROR,
          payload: { message },
        }),
    )
  }

/**
 * Redux thunk that will insert initial progress into redux store,
 * based on both states from the games-state API and local storage,
 * as configured via the MoogleOptions. This should only be called
 * internally within createMoogleLoader down below, as it assumes
 * state from the API has already been fetched.
 */
const reconcileRemoteAndLocalState =
  <
    IStoreState,
    IAction extends AnyAction,
    IGameData extends CoreState['game_data'],
  >(
    options: MoogleOptions<IStoreState, IAction, IGameData>,
  ): ThunkAction<
    void,
    IStoreState,
    unknown,
    IAction | MoogleAction
  > =>
  (dispatch, getState) => {
    const puzzleId = options.selectPuzzleId(getState())

    if (!puzzleId) {
      return null
    }

    const state = getState()
    const data = getMoogleData(options, state)
    const regiId = getMoogleRegiId(options, state)

    let remoteState: Versioned<IGameData> | null = null
    if (data?.game_data) {
      remoteState = {
        timestamp: data.timestamp,
        schemaVersion: data.schema_version,
        ...(data.game_data as IGameData),
      }
    }

    // retrieve latest state from local storage
    const localStorageKey = options.getLocalStorageKey(state, regiId)
    const localState =
      getFromLocalStorage<Versioned<IGameData>>(localStorageKey)

    // construct initial state based off remote and local progress
    let initialState = options.reconcileRemoteAndLocalState(
      remoteState,
      localState,
    )

    // optionally handle the situation where a user begins a puzzle as anon but logs in later
    if (options.handleMidGameLogin && !!regiId) {
      const anonKey = options.getLocalStorageKey(state)
      const lsAnon =
        getFromLocalStorage<Versioned<IGameData>>(anonKey)
      initialState = options.handleMidGameLogin(initialState, lsAnon)
    }

    // insert initial progress into game's redux store
    dispatch(options.insertProgress(initialState))
    // save immediately after load, so that backend is in sync with what user sees,
    // in case local storage was ahead of backend
    dispatch(forceSave(puzzleId))
  }

/**
 * Thunk generator that loads initial state into the redux store. Takes a list
 * of MoogleOptions objects as arguments and loads states for each puzzle, as configured.
 *
 * NOTE: for now, all options objects passed into this function must have the same "game" field
 */
export const createMoogleLoader =
  <
    IStoreState,
    IAction extends AnyAction,
    IGameData extends CoreState['game_data'],
  >(
    ...optionsList: Array<
      MoogleOptions<IStoreState, IAction, IGameData>
    >
  ) =>
  (): ThunkAction<
    void,
    IStoreState,
    unknown,
    IAction | MoogleAction
  > =>
  async (dispatch, getState) => {
    if (!optionsList.length) {
      return
    }

    // gather up the puzzle ids
    const state = getState()
    const allPuzzleIds = optionsList.reduce<string[]>(
      (acc, options) => {
        return [
          ...acc,
          options.selectPuzzleId(state),
          ...(options.selectAdditionalPuzzleIdsToFetch?.(state) ||
            []),
        ]
      },
      [],
    )

    const puzzleIds = Array.from(new Set(allPuzzleIds))

    // all options must have same "game" field, for now
    await dispatch(fetchRemoteState(optionsList[0].game, puzzleIds))

    optionsList.forEach((options) => {
      dispatch(reconcileRemoteAndLocalState(options))
    })
  }
