import { CoreState } from '@nyt/games-schema'
import debounce from 'lodash-es/debounce'
import type { AnyAction, Middleware } from 'redux'
import type { ThunkAction } from 'redux-thunk'
import type {} from 'redux-thunk/extend-redux'

import {
  getIsErrorRetry,
  getIsPostLoading,
  getIsSavePending,
  getMoogleRegiId,
  getPendingSaves,
  writeToLocalStorage,
} from './utils'
import { postToMoogle } from './api'
import type { Versioned, MoogleOptions } from './types'
import {
  LOAD_MOOGLE_POST,
  MoogleAction,
  MOOGLE_POST_SUCCESS,
  MOOGLE_POST_ERROR,
  SET_ERROR_RETRY,
  ENQUEUE_PENDING_SAVE,
  DEQUEUE_PENDING_SAVE,
  forceSave,
  FORCE_SAVE,
  ForceSaveAction,
} from './moogleActions'

const saveThunk =
  <
    IStoreState,
    IAction extends AnyAction,
    IGameData extends CoreState['game_data'],
  >(
    persist: Versioned<IGameData>,
    options: MoogleOptions<IStoreState, IAction, IGameData>,
  ): ThunkAction<Promise<void>, IStoreState, unknown, MoogleAction> =>
  async (dispatch, getState) => {
    const { schemaVersion, timestamp, ...persistedState } = persist
    const state = getState()
    const regiId = getMoogleRegiId(options, state)
    const puzzleId = options.selectPuzzleId(state)

    // don't bother hitting the API for Anons
    if (!regiId) {
      return Promise.resolve()
    }

    // if there is a post request already in flight, queue up a pending save and return early
    const isLoading = getIsPostLoading(options, getState())
    if (isLoading) {
      dispatch({
        type: ENQUEUE_PENDING_SAVE,
        payload: { puzzleId },
      })
      return
    }

    dispatch({ type: LOAD_MOOGLE_POST, payload: { puzzleId } })

    try {
      const data = await postToMoogle({
        game: options.game,
        game_data: persistedState,
        puzzle_id: puzzleId,
        schema_version: schemaVersion,
        timestamp,
        user_id: regiId,
      })

      if (data.error) {
        throw new Error(data.error)
      }

      if (!data || !data.version) {
        throw new Error('malformed Moogle response data')
      }

      dispatch({
        type: MOOGLE_POST_SUCCESS,
        payload: { puzzleId, player: data.player },
      })
    } catch (err: any) {
      dispatch({
        type: MOOGLE_POST_ERROR,
        payload: { puzzleId, message: err.message },
      })

      /* For the case where the API returns a 'locked' error, which
      means that we have sent a post request that will cause an
      update to be made to the player document while the API
      is already in the middle of modifying the player document
      from a previous post request.
        If there is a locked error,  we have not already retried
      AND a save is not pending, wait five seconds and try again.
      This error retry is still ruled by the one request in flight
      at a time logic, so as long as there is a save pending, we will
      honor that save. If there is no save pending and we receive this
      error we will retry. */
      if (
        err.message === 'player locked' &&
        !getIsErrorRetry(options, getState()) &&
        !getIsSavePending(options, getState())
      ) {
        dispatch({
          type: SET_ERROR_RETRY,
          payload: { puzzleId, value: true },
        })
        setTimeout(dispatch, 5000, forceSave(puzzleId))
        return
      }
    }

    if (getIsSavePending(options, getState())) {
      const nextPuzzleToSave = getPendingSaves(options, getState())[0]
      dispatch({ type: DEQUEUE_PENDING_SAVE })
      dispatch(forceSave(nextPuzzleToSave))
    }

    if (getIsErrorRetry(options, getState())) {
      dispatch({
        type: SET_ERROR_RETRY,
        payload: { puzzleId, value: false },
      })
    }
  }

export const createMoogleMiddleware =
  <
    IStoreState,
    IAction extends AnyAction,
    IGameData extends CoreState['game_data'],
  >(
    options: MoogleOptions<IStoreState, IAction, IGameData>,
  ): Middleware<{}, IStoreState> =>
  (store) => {
    const saveToBackend = (persist: Versioned<IGameData>) => {
      store.dispatch(saveThunk(persist, options))
    }

    // must wait at least 1 second, since we cannot send 2 states w/ the same timestamp
    const debouncedSaveToBackend = debounce(saveToBackend, 1000)

    window.addEventListener('online', () => {
      const moogleState = options.selectMoogleState(store.getState())
      const puzzleId = options.selectPuzzleId(store.getState())
      const hasFailedSave = moogleState.postRequest[puzzleId]?.error

      if (hasFailedSave) {
        store.dispatch(forceSave(puzzleId))
      }
    })

    return (next) => (action: IAction | MoogleAction) => {
      // send action to the reducer before store.getState
      const returnValue = next(action)
      const state = store.getState()
      const timestamp = Math.floor(Date.now() / 1000) // backend expects seconds, not milliseconds
      const regiId = getMoogleRegiId(options, state)
      const puzzleId = options.selectPuzzleId(state)

      const isForceSave =
        action.type === FORCE_SAVE &&
        (action as ForceSaveAction).payload.puzzleId === puzzleId

      // STATE
      if (options.shouldSave(state, action) || isForceSave) {
        const persist: Versioned<IGameData> = {
          timestamp,
          schemaVersion: '0.2.6', // this would ideally be exported by @nyt/games-schema
          ...options.selectPersistedState(state),
        }

        const localStorageKey = options.getLocalStorageKey(
          state,
          regiId,
        )

        writeToLocalStorage(localStorageKey, persist)
        debouncedSaveToBackend(persist)
      }

      return returnValue
    }
  }
