import { Reducer } from 'redux'

import type { MoogleData, MoogleState } from './moogleState'
import {
  LOAD_MULTI_STATES,
  MULTI_STATES_SUCCESS,
  MULTI_STATES_ERROR,
  LOAD_MOOGLE_POST,
  MOOGLE_POST_SUCCESS,
  MOOGLE_POST_ERROR,
  SET_ERROR_RETRY,
  ENQUEUE_PENDING_SAVE,
  DEQUEUE_PENDING_SAVE,
  MoogleAction,
} from './moogleActions'

const getEmptyPostState = () => ({
  isLoading: false,
  error: false,
  errorMessage: null,
  isErrorRetry: false,
})

const initialState: MoogleState = {
  getRequest: {
    isLoading: false,
    error: false,
    errorMessage: null,
    regiId: null,
    statesByPuzzleId: {},
  },
  postRequest: {},
  pendingSaves: [],
  player: null,
}

export const moogleReducer: Reducer<MoogleState, MoogleAction> = (
  state = initialState,
  action,
): MoogleState => {
  switch (action.type) {
    case LOAD_MULTI_STATES: {
      return {
        ...state,
        getRequest: {
          ...state.getRequest,
          isLoading: true,
        },
      }
    }

    case MULTI_STATES_SUCCESS: {
      const { data, regiId = null, player = null } = action.payload

      const statesToAdd = data.reduce<{
        [puzzleId: string]: MoogleData
      }>((acc, moogle) => {
        acc[moogle.puzzle_id] = moogle
        return acc
      }, {})

      return {
        ...state,
        getRequest: {
          ...state.getRequest,
          isLoading: false,
          error: false,
          regiId,
          statesByPuzzleId: {
            ...state.getRequest.statesByPuzzleId,
            ...statesToAdd,
          },
        },
        player,
      }
    }

    case MULTI_STATES_ERROR: {
      return {
        ...state,
        getRequest: {
          ...state.getRequest,
          isLoading: false,
          error: true,
          errorMessage: action.payload.message,
        },
      }
    }

    case LOAD_MOOGLE_POST: {
      const { puzzleId } = action.payload
      const puzzleState =
        state.postRequest[puzzleId] || getEmptyPostState()

      return {
        ...state,
        postRequest: {
          ...state.postRequest,
          [puzzleId]: {
            ...puzzleState,
            isLoading: true,
          },
        },
      }
    }

    case MOOGLE_POST_SUCCESS: {
      const { puzzleId, player = null } = action.payload
      const puzzleState =
        state.postRequest[puzzleId] || getEmptyPostState()

      return {
        ...state,
        postRequest: {
          ...state.postRequest,
          [puzzleId]: {
            ...puzzleState,
            isLoading: false,
            error: false,
          },
        },
        player,
      }
    }

    case MOOGLE_POST_ERROR: {
      const { puzzleId, message } = action.payload
      const puzzleState =
        state.postRequest[puzzleId] || getEmptyPostState()

      return {
        ...state,
        postRequest: {
          ...state.postRequest,
          [puzzleId]: {
            ...puzzleState,
            isLoading: false,
            error: true,
            errorMessage: message,
          },
        },
      }
    }

    case ENQUEUE_PENDING_SAVE: {
      return {
        ...state,
        pendingSaves: [
          ...state.pendingSaves,
          action.payload.puzzleId,
        ],
      }
    }

    case DEQUEUE_PENDING_SAVE: {
      return {
        ...state,
        pendingSaves: state.pendingSaves.slice(1),
      }
    }

    case SET_ERROR_RETRY: {
      const { puzzleId, value: isErrorRetry } = action.payload
      const puzzleState =
        state.postRequest[puzzleId] || getEmptyPostState()

      return {
        ...state,
        postRequest: {
          ...state.postRequest,
          [puzzleId]: {
            ...puzzleState,
            isErrorRetry,
          },
        },
      }
    }

    default:
      return state
  }
}
