import { setHybridWindowProperties } from './hybrid'

/**
 * Responsive creates a hook into our responsive css media queries by listening to css
 * transitionEnds on two dom elements, which allows us to respond to width and ratio
 * the same way in css and game js
 *
 * Inspired by: https://paulrhayes.com/use-css-transitions-to-link-media-queries-and-javascript/
 * Related files: styles/responsive, views/partials/responsiveHook
 */

// cheap transitionEnd polyfill
const transitionEnds = [
  'transitionend',
  'webkitTransitionEnd',
  'oTransitionEnd',
  'msTransitionEnd',
]

const transitionEnd = transitionEnds.find(
  (endType: string) => `on${endType.toLowerCase()}` in window,
) as string

type Listener = (width: number) => void
type Subscriber = (cb: Listener) => () => void
type Axis = {
  value: number
  subscribe: Subscriber
}

/**
 * Creates an object with the current responsive axes and a subscription method
 */
function createAxis(id: string) {
  const listeners: Listener[] = []
  const elm = document.getElementById(id) as HTMLElement

  const axis: Axis = {
    value: elm?.offsetWidth,
    subscribe(fn) {
      listeners.push(fn)
      return function unsubscribe() {
        listeners.splice(listeners.indexOf(fn), 1)
      }
    },
  }

  const update = (v: number) => {
    axis.value = v
  }

  /* Transition ends, media query matched */
  elm?.addEventListener(
    transitionEnd,
    function handleRatio() {
      const value = elm?.offsetWidth
      update(value)

      if (!listeners.length) return

      let i = listeners.length

      while (i > 0) {
        i -= 1
        listeners[i](value)
      }
    },
    false,
  )

  return axis
}

/**
 * Adds hybrid and device classnames to the body tag to inform stylesheets
 * and js hooks. Updates meta tags where necessary.
 */
const setHybridDOMProperties = () => {
  /**
   * Only add body classes for pages that are cache safe
   * Unsafe pages will include these server side
   */
  if (!window.isCacheSafe) return

  const body = document.querySelector('body')
  const viewport = document.querySelector('[name="viewport"]')
  const classes: string[] = []

  if (window.isMobileDevice) {
    classes.push('pz-mobile')
    if (window.isIOSDevice) {
      classes.push(`pz-mobile-ios`)
    }

    if (window.isAndroidDevice) {
      classes.push(`pz-mobile-android`)
    }
  } else {
    classes.push('pz-desktop')
  }

  const isHybridWebView = !!(
    window !== undefined && window.isHybridWebView
  )

  const isInNewsreader = !!(
    window !== undefined && window.newsreaderAppPlatform
  )

  if (isHybridWebView) {
    classes.push('pz-hybrid')
    // Disallow pinch-to-zoom on hybrid
    viewport?.setAttribute(
      'content',
      'width=device-width, initial-scale=1, minimal-ui, maximum-scale=1, user-scalable=no',
    )
  }

  if (window.gamesAppPlatform) {
    classes.push('pz-games-app')
    classes.push(`pz-games-app-${window.gamesAppPlatform}`)
  } else if (isInNewsreader) {
    classes.push('pz-newsreader')
    classes.push(`pz-${window.newsreaderAppPlatform}`)

    if (window.isPlayTab) {
      classes.push('pz-play-tab')
    }

    if (isHybridWebView) {
      classes.push(
        `pz-newsreader-${window.newsreaderAppPlatform}-hybrid`,
      )
    }
  } else {
    classes.push('pz-web')
  }

  body?.classList.add(...classes)
}

// If not set by the server, ultimately sets isHybridWebView
setHybridWindowProperties()
setHybridDOMProperties()

/**
 * Exported isMobile constant for games to import.
 *
 * NOTE: Do not conflate with server-side 'isMobile' value. They are
 * related but different. Server-side, 'injectDeviceType' middleware sets
 * 'isMobile', which is used to apply the pz-mobile class in 'Page.js'.
 * Client-side, we also check user agent and re-write the class
 * (here in 'setHybridDOMProperties') to be sure the value is cache-safe.
 */
export const isMobile = document.body.classList.contains('pz-mobile')
/**
 * Global responsive object for ratio.
 */
export const ratio = createAxis('ratio-hook')
/**
 * Global responsive object for width.
 */
export const width = createAxis('width-hook')

export * from './hybrid'
