import React from 'react'
import cn from 'classnames'

import { Day } from './WeeklyStats'

export const formatTime = (time: number) => {
  const h = Math.floor(time / 3600)
  const m = Math.floor((time % 3600) / 60)
  const mm = m < 10 ? `0${m}` : m
  const s = Math.floor(time % 60)
  const ss = s < 10 ? `0${s}` : s
  if (h > 0) return [h, mm, ss].join(':')

  return `${m}:${ss}`
}

const MIN_HEIGHT = 70
const MAX_HEIGHT = 275
const MIN_WIDTH = 80
const MAX_WIDTH = 320
const HEIGHT_DIFF = MAX_HEIGHT - MIN_HEIGHT
const WIDTH_DIFF = MAX_WIDTH - MIN_WIDTH

const calculateBarHeight = (
  time: number,
  shortest: number,
  longest: number,
) => {
  const solveTimeRange = longest - shortest
  const percentOfRange = (time - shortest) / solveTimeRange
  return {
    height: MIN_HEIGHT + HEIGHT_DIFF * percentOfRange,
    width: MIN_WIDTH + WIDTH_DIFF * percentOfRange,
  }
}

type SingleDayStatsProps = Day

const SingleDayStats = (props: SingleDayStatsProps) => {
  const {
    label,
    isToday,
    averageTime,
    latestTime,
    bestTime,
    bestDate,
    fastestTime,
    slowestTime,
  } = props

  const latestBarSize = calculateBarHeight(
    latestTime,
    fastestTime,
    slowestTime,
  )

  const bestBarSize = calculateBarHeight(
    bestTime,
    fastestTime,
    slowestTime,
  )

  const averageBarSize = calculateBarHeight(
    averageTime,
    fastestTime,
    slowestTime,
  )
  const dayOrWeek = isToday ? 'Today' : 'This Week'
  const bestBottom = bestBarSize.height + 8
  const bestLeft = bestBarSize.width + 40
  const latestTimeFormatted =
    latestTime !== 0 ? formatTime(latestTime) : '--:--'
  const bestTimeFormatted = formatTime(bestTime)
  const averageTimeFormatted = formatTime(averageTime)

  return (
    <div
      className={cn('single-day', { active: isToday })}
      data-testid={`single-day-${label.toLowerCase()}`}
    >
      <p className="day-of-week">
        <span className="day-first">{label.substring(0, 1)}</span>
        <span className="day-tail">{label.substring(1)}</span>
      </p>
      {averageTime === 0 ? (
        <>
          <div className="time-bar today no-stats" />
          <div className="time-bar best no-stats" />
          <div className="time-bar average no-stats" />
        </>
      ) : (
        <>
          <div
            className="time-bar today"
            style={{
              height: latestBarSize.height,
              width: latestBarSize.width,
            }}
          >
            <p className="time">{latestTimeFormatted}</p>
            <p>{dayOrWeek}</p>
          </div>
          <p
            className="best-date"
            style={{ bottom: bestBottom, left: bestLeft }}
          >
            {bestDate}
          </p>
          <div
            className="time-bar best"
            style={{
              height: bestBarSize.height,
              width: bestBarSize.width,
            }}
          >
            <p className="time">{bestTimeFormatted}</p>
            <p>Best</p>
          </div>
          <div
            className="time-bar average"
            style={{
              height: averageBarSize.height,
              width: averageBarSize.width,
            }}
          >
            <div className="condensed">
              <p>
                T <span className="time">{latestTimeFormatted}</span>
              </p>
              <p>
                B <span className="time">{bestTimeFormatted}</span>
              </p>
              <p>
                A <span className="time">{averageTimeFormatted}</span>
              </p>
            </div>
            <div className="expanded">
              <p className="time">{averageTimeFormatted}</p>
              <p>Average</p>
            </div>
          </div>
        </>
      )}
    </div>
  )
}

export default SingleDayStats
