import React, { useState, useEffect } from 'react'

import {
  UserType,
  captureSentryError,
  setSentryUser,
  xhr,
} from 'foundation'

import { refreshAdsEntitlement } from '../foundation/js/ads'
import { handleUserInformationUpdate } from '../../games/helpers/hybridLinkHelpers'
import hybridDebugger from '../foundation/js/hybrid/hybridDebugger'
import useBody from '../responsive/useBody'

import UserContext from './UserContext'
import {
  createUserInfoObject,
  defaultUserType,
  fetchShortzAccess,
  fetchEntitlementsFromNativeBridge,
  createAnonSubObject,
  createDefaultUser,
  shouldAttemptAnonFetch,
  fetchAnonEntitlements,
  fetchRegiEntitlements,
} from './userHelpers'

type UserProviderProps = {
  children: React.ReactNode
  disableShortz?: boolean
}

const UserProvider = ({
  children,
  disableShortz,
}: UserProviderProps) => {
  const [userType, setUserType] = useState<UserType>(defaultUserType)
  const bodyTag = useBody()

  const updateUserType = (newUserType: Partial<UserType>) =>
    setUserType((u) => {
      const updatedUser = {
        ...u,
        ...newUserType,
      }
      bodyTag.setUserType(updatedUser)

      // To help debug entitlements issues
      setSentryUser(updatedUser)
      hybridDebugger('User data updated', [updatedUser])

      return updatedUser
    })

  const [hasLoaded, setHasLoaded] = useState(false)
  const [shortzHasLoaded, setShortzHasLoaded] = useState(false)

  useEffect(() => {
    if (hasLoaded === true) return

    if (window.isHybridWebView && window.NativeBridge) {
      // get user entitlements from the bridge in hybrid
      fetchEntitlementsFromNativeBridge().then((data) => {
        const userData = data?.values.gamesGetUserDetails

        if (userData) {
          handleUserInformationUpdate(userData, updateUserType)
        }
        setHasLoaded(true)
      })
    } else if (
      window.newsreaderAppPlatform === 'ios' &&
      window.config?.userInfo.entitlements?.includes('XWD')
    ) {
      // user is a games subscriber. now we need to figure out if the
      // user is a regi or not
      if (window.config.userInfo.regiId) {
        updateUserType(
          createUserInfoObject({
            user: {
              userInfo: {
                regiId: window.config.userInfo.regiId,
                subscriptions: window.config.userInfo.entitlements,
              },
            },
          }),
        )
      } else {
        // creating a stub for the anon games subscriber because
        // userInfo.entitlements gives us 'XWD' but not a subscription number
        updateUserType(
          createAnonSubObject({
            products: {
              XWD: 1,
            },
          }),
        )
      }

      setHasLoaded(true)
    } else if (shouldAttemptAnonFetch()) {
      // get user entitlements from Samizdat or Games Ecomm
      Promise.allSettled([
        fetchAnonEntitlements(),
        fetchRegiEntitlements(),
      ]).then(([anonResult, regiResult]) => {
        const anonValue =
          anonResult.status === 'fulfilled' ? anonResult.value : null

        const regiValue =
          regiResult.status === 'fulfilled' ? regiResult.value : null

        if (anonValue?.products?.XWD) {
          updateUserType(createAnonSubObject(anonValue))
        } else if (regiValue?.data?.user?.userInfo?.regiId) {
          updateUserType(createUserInfoObject(regiValue.data))
        } else {
          if (anonResult.status === 'rejected') {
            console.warn(
              `Unable to determine the user's entitlements. There was an error in the anon fetch: ${anonResult.reason}`,
            ) // eslint-disable-line no-console
          }

          if (regiResult.status === 'rejected') {
            console.warn(
              `Unable to determine the user's entitlements. There was an error in the samizdat fetch: ${regiResult.reason}`,
            ) // eslint-disable-line no-console
          }

          if (
            anonResult.status === 'rejected' &&
            regiResult.status === 'rejected'
          ) {
            captureSentryError(
              new Error(
                `Could not determine the user's entitlements anonResult: ${anonResult.reason}, regiResult: ${regiResult.reason}`,
              ),
            )
          }

          updateUserType(createDefaultUser())
        }

        setHasLoaded(true)
      })
    } else {
      // get user entitlements from Samizdat
      fetchRegiEntitlements()
        .then((result) => {
          if (result?.data?.user?.userInfo?.regiId) {
            updateUserType(createUserInfoObject(result.data))
          } else {
            updateUserType(createDefaultUser())
          }

          setHasLoaded(true)
        })
        .catch((err) => {
          captureSentryError(
            new Error(
              `Could not determine the user's entitlements: ${err}`,
            ),
          )

          console.warn(
            `Unable to determine the user's entitlements. There was an error in the samizdat fetch: ${err}`,
          ) // eslint-disable-line no-console

          updateUserType(createDefaultUser())
          setHasLoaded(true)
        })
    }
  }, [hasLoaded])

  useEffect(() => {
    if (userType.regiId && !disableShortz) {
      fetchShortzAccess(userType.regiId).then((inShortzMode) => {
        updateUserType({
          inShortzMode: !!inShortzMode,
        })
        setShortzHasLoaded(true)
      })
    }
  }, [userType.regiId, disableShortz])

  useEffect(() => {
    refreshAdsEntitlement(userType.entitlement)
  }, [userType.entitlement])

  return (
    <UserContext.Provider
      value={{
        userType,
        updateUserType,
        hasLoaded,
        setHasLoaded,
        shortzHasLoaded,
      }}
    >
      {children}
    </UserContext.Provider>
  )
}

export default UserProvider
