/* eslint-disable require-await */
import React from 'react'
import { render, screen, waitFor } from '@testing-library/react'

import { refreshAdsEntitlement } from '../../foundation/js/ads'
import UserProvider from '../UserProvider'
import UserContext from '../UserContext'

jest.mock('foundation', () => ({
  xhr: {
    get: jest.fn(() => Promise.resolve(true)),
  },
  setSentryUser: jest.fn(),
}))

jest.mock('../../foundation/js/parsePurrCookieValue', () => ({
  parsePurrCookieValue: () => ({
    PURR_DataSaleOptOutUI_v2: 'hide',
    PURR_CaliforniaNoticesUI: 'show',
    PURR_AdConfiguration_v3: 'full',
  }),
}))

jest.mock('../../foundation/js/ads', () => ({
  refreshAdsEntitlement: jest.fn(),
}))

jest.mock('../userHelpers', () => ({
  shouldAttemptAnonFetch:
    jest.requireActual('../userHelpers').shouldAttemptAnonFetch,

  fetchRegiEntitlements: jest.fn(() =>
    Promise.resolve({
      data: {
        user: {
          userInfo: {
            regiId: '123456',
            subscriptions: [],
          },
        },
      },
    }),
  ),

  fetchAnonEntitlements: jest.fn(() =>
    Promise.resolve({
      products: {
        XWD: 1234567890,
      },
    }),
  ),

  fetchEntitlementsFromNativeBridge: jest.fn(() =>
    Promise.resolve({
      error: false,
      values: {
        gamesGetUserDetails: {
          isUserLoggedIn: true,
          isSubscribed: true,
          regiID: '0',
        },
      },
    }),
  ),

  fetchShortzAccess: jest.fn(() => Promise.resolve(true)),

  createUserInfoObject: jest.fn(() => ({
    regiId: '123456',
    isLoggedIn: true,
    hasXwd: true,
    hasDigi: true,
    hasHd: true,
    entitlement: 'sub,cr',
    inShortzMode: false,
  })),

  createAnonSubObject: jest.fn(() => ({
    regiId: '',
    isLoggedIn: false,
    hasXwd: true,
    hasDigi: false,
    hasHd: false,
    entitlement: 'anon,sub',
    inShortzMode: false,
  })),

  defaultUserType: {
    regiId: '',
    isLoggedIn: false,
    hasXwd: false,
    hasDigi: false,
    hasHd: false,
    entitlement: 'anon',
    inShortzMode: false,
  },
}))

describe('getUserEntitlement()', () => {
  const Foo = () => (
    <UserContext.Consumer>
      {({ userType: { isLoggedIn, inShortzMode, entitlement } }) =>
        isLoggedIn && inShortzMode ? (
          <div
            role="main"
            data-isloggedin="yes"
            data-inshortzmode="yes"
            data-entitlement={entitlement}
          />
        ) : (
          <div
            role="main"
            data-isloggedin="no"
            data-inshortzmode="no"
            data-entitlement={entitlement}
          />
        )
      }
    </UserContext.Consumer>
  )

  it('gets user entitlements from hybrid bridge', async () => {
    const data = {
      error: false,
      values: {
        gamesGetUserDetails: {
          isUserLoggedIn: true,
          isSubscribed: true,
          regiID: '0',
        },
      },
    }

    const gamesGetUserDetails = jest
      .fn()
      .mockImplementation(() => Promise.resolve(data))
    window.isHybridWebView = true
    window.NativeBridge = { gamesGetUserDetails }

    render(
      <UserProvider>
        <Foo />
      </UserProvider>,
    )

    const div = await screen.findByRole('main')

    expect(div).toHaveAttribute('data-isloggedin', 'yes')
    expect(div).toHaveAttribute('data-inshortzmode', 'yes')
  })

  it('gets user entitlements from samizdat', async () => {
    window.isHybridWebView = false
    window.NativeBridge = false

    render(
      <UserProvider>
        <Foo />
      </UserProvider>,
    )

    const div = await screen.findByRole('main')

    expect(div).toHaveAttribute('data-isloggedin', 'yes')
    expect(div).toHaveAttribute('data-inshortzmode', 'yes')
  })

  it('gets user entitlements from ecomm', async () => {
    window.isHybridWebView = false
    window.NativeBridge = false
    window.gamesAppPlatform = 'ios'

    render(
      <UserProvider>
        <Foo />
      </UserProvider>,
    )

    const div = await screen.findByRole('main')

    expect(div).toHaveAttribute('data-entitlement', 'anon,sub')
  })

  it('refreshes ad slots with updated entitlements', async () => {
    window.isHybridWebView = false
    window.NativeBridge = false
    window.gamesAppPlatform = ''

    render(
      <UserProvider>
        <Foo />
      </UserProvider>,
    )

    await waitFor(() => {
      expect(refreshAdsEntitlement).toHaveBeenLastCalledWith('sub,cr')
    })
  })
})
