import {
  UserType,
  UserEntitlement,
  xhr,
  env,
  captureSentryError,
} from 'foundation'

export type UserQueryResponseData = {
  data?: {
    user?: {
      userInfo?: {
        regiId: string
        subscriptions: string[]
      }
    }
  }
}

export type AnonEntitlementsResponseData = {
  products?: {
    XWD: number
  }
}

export function getUserEntitlement(
  isLoggedIn: boolean,
  hasDigi: boolean,
  hasXwd: boolean,
): UserEntitlement {
  if (!isLoggedIn) return 'anon'

  if (hasDigi && hasXwd) return 'sub,cr'

  if (hasDigi) return 'sub'

  return hasXwd ? 'cr' : 'reg'
}

export function createUserInfoObject(
  userQueryResponseData?: UserQueryResponseData['data'],
): Partial<UserType> {
  const regiId = userQueryResponseData?.user?.userInfo?.regiId

  const isLoggedIn = !!regiId

  let subscriptions =
    userQueryResponseData?.user?.userInfo?.subscriptions || []
  subscriptions = subscriptions.map((e) => e.toLowerCase())

  // NOTE: hasDigi indicates a digital news (basic) subscriber WITHOUT Games access, not an all digital access subscriber
  const hasDigi = !!subscriptions.find(
    (s) => s === 'mm' || s === 'msd' || s === 'mtd',
  )
  const hasXwd = !!subscriptions.find((s) => s === 'xwd')

  const hasHd = !!subscriptions.find((s) => s === 'hd')

  const entitlement = getUserEntitlement(isLoggedIn, hasDigi, hasXwd)

  const inShortzMode = false

  return {
    entitlement,
    hasDigi,
    hasHd,
    hasXwd,
    inShortzMode,
    isLoggedIn,
    regiId,
  }
}

/**
 * Create a default User object, assuming we know nothing about the user.
 */
export function createDefaultUser(): Partial<UserType> {
  return {
    regiId: '',
    entitlement: 'anon',
    isLoggedIn: false,
    hasXwd: false,
    hasDigi: false,
    hasHd: false,
    inShortzMode: false,
  }
}

export function createAnonSubObject(
  anonData: AnonEntitlementsResponseData,
): Partial<UserType> {
  const defaultUserInfo = createDefaultUser()

  if (anonData.products?.XWD) {
    return {
      ...defaultUserInfo,
      entitlement: 'anon,sub',
      hasXwd: true,
    }
  }

  return defaultUserInfo
}

/**
 * These will be determined in the inline scripts
 */
export const shouldAttemptAnonFetch = () => {
  const isHybrid = window.isHybridWebView && window.NativeBridge
  const isInApp =
    window.gamesAppPlatform || window.newsreaderAppPlatform

  return Boolean(isInApp && !isHybrid)
}

export const fetchAnonEntitlements = () => {
  return xhr.get<AnonEntitlementsResponseData>(
    `${env.api}/svc/crosswords/v6/anon/entitlements.json`,
  )
}

export const fetchRegiEntitlements = () => {
  const userQuery = `
    query UserQuery {
      user {
        userInfo {
          regiId
          subscriptions
        }
      }
    }
  `

  const body = {
    operationName: 'UserQuery',
    query: userQuery.replace(/\n/g, ' '),
  }

  return xhr.post<UserQueryResponseData>(
    `${env.samizdat.host}/graphql/v2`,
    body,
    {
      headers: {
        'Content-Type': 'application/json',
        'nyt-app-type': env.samizdat.appType,
        'nyt-app-version': env.samizdat.appVersion,
        'nyt-token': env.samizdat.token,
      },
    },
  )
}

export const fetchShortzAccess = (regiId: string) =>
  xhr
    .get<boolean>(
      `${env?.hostname}/puzzles/shortz/${regiId}`,
      // this option is necessary to avoid CORS violations in hybrid
      { withCredentials: false },
    )
    .catch((err) => {
      console.error('Getting shortz mode details failed', err)
    })

export const fetchEntitlementsFromNativeBridge = () =>
  window
    .NativeBridge!.gamesGetUserDetails()
    .then((data) => {
      if (data.error) throw new Error(data.error)

      return data
    })
    .catch((e) => {
      console.error(
        'Failed to fetch entitlements from Native Bridge',
        e,
      )
      captureSentryError(
        new Error('Failed to fetch entitlements from Native Bridge'),
        {
          api: 'native-bridge',
          category: 'entitlements',
        },
      )
    })

export const defaultUserType: UserType = {
  isLoggedIn: false,
  hasXwd: false,
  hasDigi: false,
  hasHd: false,
  entitlement: 'anon',
  regiId: '',
  inShortzMode: false,
  isFreeTrial: false,
}
