const fs = require('fs')
const path = require('path')

const globModule = require('glob')
const chalk = require('chalk')
const promisify = require('es6-promisify') // eslint-disable-line import/order

const glob = promisify(globModule)
const readFile = promisify(fs.readFile)

const gzipSize = require('gzip-size')
const filesize = require('filesize')

const logger = require('../src/serverLogger')
const { DIST_ROOT } = require('../src/constants')

const filePattern = `${DIST_ROOT}/**/*.*(*.js|*.css|*.jpg|*.svg|*.png)`

const LIMITS = {
  '.js': { warn: 50 * 1000, err: 130 * 1000 },
  '.css': { warn: 50 * 1000, err: 130 * 1000 },
  default: { warn: 500 * 1000, err: 1000 * 1000 },
}

const getLimit = (ext) => LIMITS[ext] || LIMITS.default

const getFileSizes = async (filepath) => {
  const ext = path.extname(filepath)
  const displayName = path
    .relative(DIST_ROOT, filepath)
    .replace(/\..*$/, ext)
  const contents = await readFile(filepath)
  const rawSize = contents.byteLength
  const gzippedSize = gzipSize.sync(contents)
  const savings = rawSize
    ? Math.floor((gzippedSize / rawSize) * 100)
    : 0

  const uncompressed = `raw: ${filesize(rawSize)}`
  const gzipPercent = `gzip: ${savings}%`
  const compressed = `- ${filesize(gzippedSize)}`

  let color = 'gray'
  let marker = ''

  if (rawSize > getLimit(ext).warn) {
    color = 'yellow'
    marker = '?'
  }

  if (rawSize > getLimit(ext).err) {
    color = 'red'
    marker = '!!'
  }

  return {
    uncompressed,
    gzipPercent,
    compressed,
    color,
    marker,
    displayName,
    gzippedSize,
  }
}

const getTotalCompressedSizeForEntries = async () => {
  const manifestRaw = await readFile(`${DIST_ROOT}/manifest.json`)
  const manifest = JSON.parse(manifestRaw)
  const entries = Object.keys(manifest)

  return Promise.all(
    entries.map((entry) => {
      const { scripts, styles } = manifest[entry]
      const files = [...scripts, ...styles].map(
        (name) => `${DIST_ROOT}/${name}`,
      )

      return Promise.all(files.map(getFileSizes))
        .then((sizeObjs) =>
          sizeObjs.reduce(
            (acc, sizeObj) => acc + sizeObj.gzippedSize,
            0,
          ),
        )
        .then((totalCompressedSize) => ({
          entry,
          totalCompressedSizeInKb: Math.round(
            totalCompressedSize / 1000,
          ),
        }))
    }),
  ).then(console.info)
}

const logFileSize = async (filepath) => {
  const {
    uncompressed,
    gzipPercent,
    compressed,
    color,
    marker,
    displayName,
  } = await getFileSizes(filepath)

  logger.info(
    chalk[color](
      marker.padEnd(2),
      uncompressed.padEnd(16),
      gzipPercent.padEnd(9),
      compressed.padEnd(12),
      displayName,
    ),
  )
}

module.exports = async () => {
  logger.headingInfo('Checking File Size')
  const files = await glob(filePattern)
  return Promise.all(files.map(logFileSize)).then(
    getTotalCompressedSizeForEntries,
  )
}
