const path = require('path')

const fs = require('fs-extra')
const express = require('express')
const tinylr = require('tiny-lr')
const connectLiveReload = require('connect-livereload')

const logger = require('../src/serverLogger')
const {
  PORT,
  RUNTIME_ENV,
  USE_LIVE_RELOAD,
  LIVE_RELOAD_PORT,
  USE_HTTPS,
  SSL_CERT_FILE_PATH,
  SSL_KEY_FILE_PATH,
} = require('../src/constants')

/**
 * Setup a live reload server
 */
let liveReloadServer
let options

if (USE_HTTPS) {
  options = {
    key: fs.readFileSync(SSL_KEY_FILE_PATH),
    cert: fs.readFileSync(SSL_CERT_FILE_PATH),
  }
}

if (USE_LIVE_RELOAD) {
  liveReloadServer = tinylr(options)

  liveReloadServer.forceUpdate = () => {
    // force a refresh by telling the browser the index changed
    logger.info('forced live reload')
    liveReloadServer.changed({
      body: { files: ['index.html'] },
    })
  }
}

/**
 * When the app needs to restart
 * what require modules need to be busted
 */
const bustRegex = new RegExp(
  `(${[
    path.resolve(__dirname, '../src'),
    path.resolve(__dirname, '../dist'),
  ].join('|')}).*`,
)

/**
 * prevent cache busting on these files
 * for example, refreshing the logger creates a memory leak
 * which will likely be resolved with the new middleware structure
 */
const skipCacheBust = [
  path.resolve(__dirname, '../src/serverLogger.js'),
]

/**
 * Store the refreshed server here
 */
let server
let hadError

const serveError = (e) => {
  server = (req, res) => {
    res.send(`
      <!DOCTYPE html>
      <html>
        <head>
          <title>Error!</title>
        </head>
        <body>
          <h2>${e.message}</h2><pre>${e.stack}</pre>
        </body>
      </html>
    `)
  }
}

const rebootServer = (isFirst) => {
  try {
    server = require('../src/server')
  } catch (e) {
    if (isFirst) {
      console.error(e)
      process.exit()
    } else {
      serveError(e)
      throw e
    }
  }

  if (!isFirst) logger.headingInfo('reboot server')
}

exports.create = () => {
  const logCreateTime = logger.checkTime('created server in')

  /**
   * Get the initial version of the app
   */
  rebootServer(true)

  /**
   * Create the dev appliction
   */
  const app = express()

  if (liveReloadServer) {
    liveReloadServer.listen(LIVE_RELOAD_PORT, () => {
      logger.info(`Live Reload Listening on ${LIVE_RELOAD_PORT}`)
    })

    app.use(
      connectLiveReload({
        port: LIVE_RELOAD_PORT,
        disableCompression: true,
      }),
    )
  }

  /**
   * defer all app handling to local var `server`
   * so that when the server is re-required the request handling will stay fresh
   */
  app.use((req, res, next) => server(req, res, next))

  /**
   * Start the app
   */
  app.listen(PORT, () => {
    logger.info(
      `[${RUNTIME_ENV}] Dev server listening on port ${PORT}`,
    )
    logCreateTime()
  })

  logger.headingInfo('create server')
}

exports.restart = (error, changes) => {
  /** start timing the restart * */
  const logRestartTime = logger.checkTime('refreshed server in')

  /**
   * Only handle changes if there is a live reload server
   */
  if (liveReloadServer) {
    /**
     * Handle compilation errors
     */
    if (error) {
      hadError = true
      serveError(error)
      liveReloadServer.forceUpdate()
      return
    }

    /**
     * Handle asset and other changes
     */
    if (hadError) {
      liveReloadServer.forceUpdate()
    } else if (changes && changes.length) {
      logger.labeledInfo(
        'livereload',
        changes.map((f) => `\n     - ${f}`).join(''),
      )
      liveReloadServer.changed({ body: { files: changes } })
    } else {
      liveReloadServer.forceUpdate()
    }

    if (!error) {
      hadError = false
    }
  }

  /**
   * Bust the require cache for all app files and dist files
   */
  Object.keys(require.cache).forEach((key) => {
    if (bustRegex.test(key) && !skipCacheBust.includes(key)) {
      delete require.cache[key]
    }
  })

  /**
   * update the server by re-requiring it
   */
  rebootServer()

  logger.headingInfo('refresh server')
  logRestartTime()
}
