const path = require('path')

const autoprefixer = require('autoprefixer')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const MomentLocalesPlugin = require('moment-locales-webpack-plugin')
const webpack = require('webpack')
const { WebpackManifestPlugin } = require('webpack-manifest-plugin')
const { merge } = require('webpack-merge')
const nodeExternals = require('webpack-node-externals')
const { sentryWebpackPlugin } = require('@sentry/webpack-plugin')

const aliases = require('./tools/createAliases')
const correctURLs = require('./tools/pluginPostcssAssetUrls')
const clientEntry = require('./tools/webpackClientEntry')
const { JS_STATICS } = require('./src/constants')

const commonConfig = {
  mode: process.env.IS_DRONE_BUILD ? 'production' : 'development',
  plugins: [
    new MiniCssExtractPlugin({
      filename: process.env.IS_DRONE_BUILD
        ? '[name].[contenthash].css'
        : '[name].css',
    }),
  ],
  module: {
    rules: [
      {
        // needed for sentry source maps
        // TODO: does this work on a production build?
        test: /\.js$/,
        enforce: 'pre',
        use: ['source-map-loader'],
      },
      {
        test: /\.(js|ts|tsx)$/,
        exclude: /node_modules/,
        // see .babelrc for babel config
        use: ['babel-loader'],
      },
      {
        test: /\.s[ac]ss$/i,
        use: [
          {
            loader: MiniCssExtractPlugin.loader,
          },
          {
            loader: 'css-loader',
            options: {
              url: false,
              modules: {
                // only load as a CSS module if file name contains .module
                auto: true,
                // determines the classname used for CSS module classes
                localIdentName: '[name]_[local]__[hash:base64:5]',
              },
            },
          },
          {
            loader: 'postcss-loader',
            options: {
              postcssOptions: {
                plugins: [autoprefixer, correctURLs],
              },
            },
          },
          'sass-loader',
        ],
      },
      {
        test: /\.html$/i,
        loader: 'html-loader',
      },
    ],
  },
  resolve: {
    extensions: ['*', '.ts', '.tsx', '.js'],
    alias: Object.keys(aliases).reduce((acc, key) => {
      acc[key] = path.resolve(__dirname, aliases[key])
      return acc
    }, {}),
  },
}

const serverViewsConfig = {
  entry: {
    serverViews: './src/server/views/index.js',
  },
  output: {
    path: path.resolve(__dirname, 'dist'),
    filename: '[name].js',
    globalObject: 'this',
    library: {
      name: 'serverViews',
      type: 'umd',
    },
  },
  externalsPresets: { node: true }, // in order to ignore built-in modules like path, fs, etc.
  externals: [nodeExternals()], // in order to ignore all modules in node_modules folder
}

const clientConfig = {
  entry: clientEntry,
  output: {
    path: path.resolve(__dirname, 'dist'),
    filename: process.env.IS_DRONE_BUILD
      ? '[name].[contenthash].js'
      : '[name].js',
  },
  devtool: process.env.IS_DRONE_BUILD
    ? 'source-map'
    : 'inline-cheap-source-map',
  optimization: {
    splitChunks: {
      chunks: 'all',
    },
  },
  plugins: [
    /**
     * This plugin generates a manifest.json in the dist folder,
     * mapping each entry point to the scripts and styles it needs.
     * This manifest is then used in createAssets.js, determining which
     * files to include in the html at runtime
     */
    new WebpackManifestPlugin({
      generate: (seed, files, entries) => {
        return Object.keys(entries).reduce((acc, entry) => {
          const files = entries[entry]
          const scripts = files.filter((name) => name.endsWith('.js'))
          const styles = files.filter((name) => name.endsWith('.css'))
          acc[entry] = { scripts, styles }
          return acc
        }, {})
      },
    }),
    new webpack.DefinePlugin(JS_STATICS),
    new MiniCssExtractPlugin({
      filename: process.env.IS_DRONE_BUILD
        ? '[name].[contenthash].css'
        : '[name].css',
    }),
    new webpack.ProgressPlugin(),
    new MomentLocalesPlugin(),
    sentryWebpackPlugin({
      authToken: process.env.SENTRY_AUTH_TOKEN,
      org: 'nytimes',
      project: 'games-phoenix',
    }),
  ],
}

module.exports = [
  merge(commonConfig, serverViewsConfig),
  merge(commonConfig, clientConfig),
]
